<?php

namespace App\Console\Commands;

use App\Models\Broker;
use App\Models\Company;
use Carbon\Carbon;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;

class CreateOrUpdateBrokersTrimblePlaceId extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'create-or-update-brokers-trimble-place-id:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        Log::info('Cron Start : Create Or Update Brokers Trimble PlaceId ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
        $companyTrimbleApiKeys = Company::pluck('trimble_api_key', 'id')->toArray();

        $companyAuthTokens = [];
        foreach ($companyTrimbleApiKeys as $companyId => $companyTrimbleApiKey){
            $companyAuthTokens[$companyId] = $this->getTrimblePlacesAuthenticationToken($companyTrimbleApiKey);
        }

        $brokers = Broker::whereNull('geofence_trimble_place_id')
                    ->where('address', 'NOT LIKE', 'PO%')->where('address', 'NOT LIKE', 'P.O%')
                    ->where('address', 'NOT LIKE', 'P O%')->where('address', 'NOT LIKE', 'P. O%')
                    ->orderBy('id')->get(['id', 'company_id', 'address', 'address_2', 'city', 'state', 'zip']);

        $brokerApiKeyNotFound = [];
        foreach ($brokers as $key => $broker){
            $companyId = $broker->company_id;

            if($companyId != ''){
                $dbLocation = [];
                if($broker->address != ''){
                    $dbLocation[] = $broker->address;
                }
                if($broker->address_2 != ''){
                    $dbLocation[] = $broker->address_2;
                }
                if($broker->city != ''){
                    $dbLocation[] = $broker->city;
                }
                if($broker->state != ''){
                    $dbLocation[] = $broker->state;
                }
                if($broker->zip != ''){
                    $dbLocation[] = $broker->zip;
                }
                $dbLocation = implode(', ', $dbLocation);

                if($companyTrimbleApiKeys[$companyId] != ''){
                    if($dbLocation != ''){
                        $trimblePlaceDetails = $this->getTrimblePlaceDetailsByAddress($companyTrimbleApiKeys[$companyId], $dbLocation);

                        if(count($trimblePlaceDetails->Locations) > 0){
                            $isGetTrimblePlaceId = false;
                            foreach ($trimblePlaceDetails->Locations as $LocationDetails){
                                if(isset($LocationDetails->TrimblePlaceId) && $LocationDetails->TrimblePlaceId != ''){
                                    $timeZoneDetails = $this->getTimeZoneDetails($companyTrimbleApiKeys[$companyId], $trimblePlaceDetails->Locations[0]->Address->StreetAddress, $trimblePlaceDetails->Locations[0]->Address->City, $trimblePlaceDetails->Locations[0]->Address->State, substr($trimblePlaceDetails->Locations[0]->Address->Zip, 0, 5));

                                    $isGetTrimblePlaceId = true;
                                    $broker->address = $LocationDetails->Address->StreetAddress ?? '';
                                    $broker->address_2 = $LocationDetails->Address->LocalArea ?? '';
                                    $broker->city = $LocationDetails->Address->City ?? '';
                                    $broker->state = $LocationDetails->Address->State ?? '';
                                    $broker->zip = substr($LocationDetails->Address->Zip, 0, 5) ?? '';
                                    $broker->latitude = $LocationDetails->Coords->Lat ?? '';
                                    $broker->longitude = $LocationDetails->Coords->Lon ?? '';
                                    $broker->timezone = $timeZoneDetails['timezone'] ?? '';
                                    $broker->timezone_offset = $timeZoneDetails['timezone_offset'] ?? '';
                                    $broker->timezone_abbr = $timeZoneDetails['timezone_abbr'] ?? '';
                                    $broker->address_verified = $timeZoneDetails['address_verified'] ?? 0;
                                    $broker->geofence_trimble_place_id = $LocationDetails->TrimblePlaceId ?? '';
                                    $broker->geofence_updated_at = Carbon::now()->format('m/d/Y') ?? '';
                                    $broker->save();
                                    break;
                                }
                            }

                            if($isGetTrimblePlaceId == false){
                                if($companyAuthTokens[$companyId] != ''){
                                    if(isset($trimblePlaceDetails->Locations[0])){
                                        $trimbleLocation = [];
                                        if($trimblePlaceDetails->Locations[0]->Address->StreetAddress != ''){
                                            $trimbleLocation[] = $trimblePlaceDetails->Locations[0]->Address->StreetAddress;
                                        }
                                        if($trimblePlaceDetails->Locations[0]->Address->LocalArea != ''){
                                            $trimbleLocation[] = $trimblePlaceDetails->Locations[0]->Address->LocalArea;
                                        }
                                        if($trimblePlaceDetails->Locations[0]->Address->City != ''){
                                            $trimbleLocation[] = $trimblePlaceDetails->Locations[0]->Address->City;
                                        }
                                        if($trimblePlaceDetails->Locations[0]->Address->State != ''){
                                            $trimbleLocation[] = $trimblePlaceDetails->Locations[0]->Address->State;
                                        }
                                        if($trimblePlaceDetails->Locations[0]->Address->Zip != ''){
                                            $trimbleLocation[] = $trimblePlaceDetails->Locations[0]->Address->Zip;
                                        }
                                        $trimbleLocation = implode(', ', $trimbleLocation);

                                        $trimbleCreatedPlaceDetails = $this->createTrimblePlaceByAddress($companyAuthTokens[$companyId], $trimbleLocation, $trimblePlaceDetails->Locations[0]);

                                        if(isset($trimbleCreatedPlaceDetails->trimblePlaceId) && $trimbleCreatedPlaceDetails->trimblePlaceId != ''){
                                            $timeZoneDetails = $this->getTimeZoneDetails($companyTrimbleApiKeys[$companyId], $trimblePlaceDetails->Locations[0]->Address->StreetAddress, $trimblePlaceDetails->Locations[0]->Address->City, $trimblePlaceDetails->Locations[0]->Address->State, substr($trimblePlaceDetails->Locations[0]->Address->Zip, 0, 5));
                                            $broker->address = $trimblePlaceDetails->Locations[0]->Address->StreetAddress ?? '';
                                            $broker->address_2 = $trimblePlaceDetails->Locations[0]->Address->LocalArea ?? '';
                                            $broker->city = $trimblePlaceDetails->Locations[0]->Address->City ?? '';
                                            $broker->state = $trimblePlaceDetails->Locations[0]->Address->State ?? '';
                                            $broker->zip = substr($trimblePlaceDetails->Locations[0]->Address->Zip, 0, 5) ?? '';
                                            $broker->latitude = $trimblePlaceDetails->Locations[0]->Coords->Lat ?? '';
                                            $broker->longitude = $trimblePlaceDetails->Locations[0]->Coords->Lon ?? '';
                                            $broker->timezone = $timeZoneDetails['timezone'] ?? '';
                                            $broker->timezone_offset = $timeZoneDetails['timezone_offset'] ?? '';
                                            $broker->timezone_abbr = $timeZoneDetails['timezone_abbr'] ?? '';
                                            $broker->address_verified = $timeZoneDetails['address_verified'] ?? 0;
                                            $broker->geofence_trimble_place_id = $trimbleCreatedPlaceDetails->trimblePlaceId ?? '';
                                            $broker->geofence_updated_at = Carbon::now()->format('m/d/Y') ?? '';
                                            $broker->save();
                                        }
                                    }
                                }
                            }
                        }
                    }
                }else{
                    $brokerApiKeyNotFound[] = $broker->id;
                }
            }
            echo "Progress: ".round((($key + 1) / count($brokers)) * 100)."% (".($key + 1)."/".count($brokers)."), done \r";
        }

        if(count($brokerApiKeyNotFound) > 0){
            Log::info('Trimble maps API key not found for brokers: '.implode(', ', $brokerApiKeyNotFound));
        }

        Log::info('Cron End : Create Or Update Brokers Trimble PlaceId ( '.Carbon::now()->format('Y-m-d H:i:s').' )');

        $this->info('Broker Trimble PlaceId Updated Successfully.');
    }

    public function getTrimblePlacesAuthenticationToken($apiKey = null) {
        $authToken = null;

        if(is_null($apiKey)){
            $apiKey = getTrimbleApiKey();
        }
        if(!is_null($apiKey)){
            try {
                $client = new \GuzzleHttp\Client(['verify' => false]);
                $response = $client->request('POST', 'https://api.trimblemaps.com/places/v1/authenticate', [
                    'headers' => [
                        'Content-Type' => 'application/json'
                    ],
                    'body' => json_encode([
                        'apiKey' => $apiKey
                    ])
                ]);
                // Check if response status code is OK (200)
                if ($response->getStatusCode() == 200) {
                    $body = json_decode($response->getBody());
                    $authToken = $body->token;
                }
            } catch (\GuzzleHttp\Exception\ClientException $e) {
                // Handle client-side errors (e.g. 4xx status codes)
                $response = $e->getResponse();
                if ($response) {
                    $statusCode = $response->getStatusCode();
                    $body = $response->getBody()->getContents();
                    Log::error($body);
                } else {
                    Log::error($e->getMessage());
                }
            } catch (\GuzzleHttp\Exception\ServerException $e) {
                // Handle server-side errors (e.g. 5xx status codes)
                $response = $e->getResponse();
                if ($response) {
                    $statusCode = $response->getStatusCode();
                    $body = $response->getBody()->getContents();
                    Log::error($body);
                } else {
                    Log::error($e->getMessage());
                }
            } catch (\Exception $e) {
                // Handle other exceptions
                Log::error($e->getMessage());
            }
        }

        return $authToken;
    }

    public function getTrimblePlaceDetailsByAddress($apiKey = null, $location) {
        $data = null;

        if(is_null($apiKey)){
            $apiKey = getTrimbleApiKey();
        }
        if($apiKey != '' && $location != ''){
            try {
                $client = new \GuzzleHttp\Client(['verify' => false]);
                $response = $client->request('GET', 'https://singlesearch.alk.com/NA/api/search?query='.$location.'&includeTrimblePlaceIds=true', [
                    'headers' => [
                        'Accept'        => 'application/json',
                        'Content-Type'  => 'application/json',
                        'Authorization' => $apiKey
                    ]
                ]);
                // Check if response status code is OK (200)
                if ($response->getStatusCode() == 200) {
                    $data = json_decode($response->getBody());
                }
            } catch (\GuzzleHttp\Exception\ClientException $e) {
                // Handle client-side errors (e.g. 4xx status codes)
                $response = $e->getResponse();
                if ($response) {
                    $statusCode = $response->getStatusCode();
                    $body = $response->getBody()->getContents();
                    Log::error($body);
                } else {
                    Log::error($e->getMessage());
                }
            } catch (\GuzzleHttp\Exception\ServerException $e) {
                // Handle server-side errors (e.g. 5xx status codes)
                $response = $e->getResponse();
                if ($response) {
                    $statusCode = $response->getStatusCode();
                    $body = $response->getBody()->getContents();
                    Log::error($body);
                } else {
                    Log::error($e->getMessage());
                }
            } catch (\Exception $e) {
                // Handle other exceptions
                Log::error($e->getMessage());
            }
        }

        return $data;
    }

    public function createTrimblePlaceByAddress($authTokens = null, $location, $trimblePlaceDetails){
        $data = null;

        if($authTokens != ''){
            try {
                $client = new \GuzzleHttp\Client(['verify' => false]);
                $response = $client->request('POST', 'https://api.trimblemaps.com/places/v1/place', [
                    'headers' => [
                        'Content-Type' => 'application/json',
                        'Authorization' => 'bearer '. $authTokens
                    ],
                    'body' => json_encode([
                        'name' => $location,
                        'setId' => env('TRIMBLE_PLACE_SET_ID'),
                        'address' => array(
                            'streetAddress' => $trimblePlaceDetails->Address->StreetAddress ?? '',
                            'extendedAddress' => $trimblePlaceDetails->Address->LocalArea ?? '',
                            'streetNumber' => '',
                            'city' => $trimblePlaceDetails->Address->City ?? '',
                            'county' => $trimblePlaceDetails->Address->County ?? '',
                            'state' => array(
                                'name' => $trimblePlaceDetails->Address->StateName ?? '',
                                'abbrv' => $trimblePlaceDetails->Address->State ?? ''
                            ),
                            'country' => array(
                                'name' => $trimblePlaceDetails->Address->CountryFullName ?? '',
                                'abbrv' => $trimblePlaceDetails->Address->Country ?? ''
                            ),
                            'postalCode' => $trimblePlaceDetails->Address->Zip ?? '',
                        ),
                        'formattedAddress' => $location,
                        'geometry' => array(
                            'type' => 'Point',
                            'coordinates' => [
                                $trimblePlaceDetails->Coords->Lon ?? '',
                                $trimblePlaceDetails->Coords->Lat ?? '',
                            ],
                        )
                    ])
                ]);

                // Check if response status code is Created (201)
                if ($response->getStatusCode() == 201) {
                    $data = json_decode($response->getBody());
                }
            } catch (\GuzzleHttp\Exception\ClientException $e) {
                // Handle client-side errors (e.g. 4xx status codes)
                $response = $e->getResponse();
                if ($response) {
                    $statusCode = $response->getStatusCode();
                    $body = $response->getBody()->getContents();
                    Log::error($body);
                } else {
                    Log::error($e->getMessage());
                }
            } catch (\GuzzleHttp\Exception\ServerException $e) {
                // Handle server-side errors (e.g. 5xx status codes)
                $response = $e->getResponse();
                if ($response) {
                    $statusCode = $response->getStatusCode();
                    $body = $response->getBody()->getContents();
                    Log::error($body);
                } else {
                    Log::error($e->getMessage());
                }
            } catch (\Exception $e) {
                // Handle other exceptions
                Log::error($e->getMessage());
            }
        }

        return $data;
    }

    public function getTimeZoneDetails($apiKey = null, $address, $city, $state, $zip) {
        $data = [];
        if($apiKey != '' && $address != ''&& $city != ''&& $state != ''&& $zip != ''){
            try {
                $client = new \GuzzleHttp\Client(['verify' => false]);
                $response = $client->request('GET', 'https://pcmiler.alk.com/apis/rest/v1.0/Service.svc/locations?street='.$address.'&city='.$city.'&state='.$state.'&postcode='.$zip.'&region=NA&dataset=Current', [
                    'headers' => [
                        'Accept'        => 'application/json',
                        'Content-Type'  => 'application/json',
                        'Authorization' => $apiKey
                    ]
                ]);
                // Check if response status code is OK (200)
                if ($response->getStatusCode() == 200) {
                    $res = json_decode($response->getBody());
                    if(count($res) > 0){
                        $data['timezone_abbr'] = $res[0]->TimeZoneAbbreviation;
                        if(isset($data['timezone_abbr'])){
                            $data['timezone'] = timezone_name_from_abbr($data['timezone_abbr']);
                            $data['timezone_offset'] = Carbon::createFromTimestamp(time(), $data['timezone'])->getOffsetString();
                            $data['address_verified'] = '1';
                        }
                    }
                }
            } catch (\GuzzleHttp\Exception\ClientException $e) {
                // Handle client-side errors (e.g. 4xx status codes)
                $response = $e->getResponse();
                if ($response) {
                    $statusCode = $response->getStatusCode();
                    $body = $response->getBody()->getContents();
                    Log::error($body);
                } else {
                    Log::error($e->getMessage());
                }
            } catch (\GuzzleHttp\Exception\ServerException $e) {
                // Handle server-side errors (e.g. 5xx status codes)
                $response = $e->getResponse();
                if ($response) {
                    $statusCode = $response->getStatusCode();
                    $body = $response->getBody()->getContents();
                    Log::error($body);
                } else {
                    Log::error($e->getMessage());
                }
            } catch (\Exception $e) {
                // Handle other exceptions
                Log::error($e->getMessage());
            }
        }
        return $data;
    }
}
