<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;

use App\Models\Driver;
use App\Models\CompanyAdmin;

use DB;
use Carbon\Carbon;

class DriverMissingLoadCron extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature    = 'drivermissingload:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description  = 'Driver Missing Load';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        Log::info('Cron Start : Driver Missing Load ( '.Carbon::now()->format('Y-m-d').' )');
        $now                = Carbon::now(); //Carbon::parse('2022-05-01');
        $currentDate        = $now->format('Y-m-d');
        $previous           = Carbon::yesterday(); //$now->subDays();
        $previousDate       = $previous->format('Y-m-d');

        $month              = $previous->format('m');
        $monthName          = $previous->format('F');
        $year               = $previous->format('Y');

        $arry               = [];
        $query              = Driver::select(
                'drivers.id',
                'drivers.manager_id',
                'drivers.team_id',
                'drivers.first_name',
                'drivers.last_name',
                'drivers.middle_name',
                DB::raw('CONCAT(drivers.last_name, " ", drivers.first_name) AS dname'),
                'drivers.start_date'
            )
            ->join('load_details','load_details.driver_id','=','drivers.id')
            ->join('driver_informations','driver_informations.driver_id','=','drivers.id')
            ->whereDate('drivers.start_date', '<=', $previousDate)
            ->where('load_details.is_disable',0)
            ->where('load_details.load_status', '!=', 9)
            ->where(function($q) {
                $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
            })
            ->where(function($q) {
                $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
            })
            ->groupBy('drivers.id')
            ->orderByRaw('drivers.last_name ASC, drivers.first_name ASC');
        $drivers        = $query->get();
        if($drivers->isNotEmpty()){
            foreach ($drivers as $key => $driver) {
                $getLoadByDateMonth = getLoadByDateMonth($previousDate,$driver->id);
                if(count($getLoadByDateMonth)<=0){
                    if(isset($driver->manager_id) && $driver->manager_id!=''){
                        $arry[$driver->manager_id][] = $driver->dname; //$arry[$driver->team->head_id][] = $driver->dname;
                    }
                }
            }
        }
        if(!empty($arry)){
            foreach ($arry as $dispatcherId => $names) {
                $dispatcher     = CompanyAdmin::whereId($dispatcherId)->first();
                if($dispatcher){
                    // 5 = Liner Fleet Calendar
                    $properties = [
                        'user_id'       => $dispatcher->id,
                        'user_type'     => $dispatcher->type,
                        'previous_date' => $previousDate
                    ];
                    createNotification(
                        6, // source
                        'Attention!', // name
                        $dispatcher->id, // user_id
                        $dispatcher->type, // user_type
                        2, // type
                        null, // action_id
                        'Driver Missing Load - '.implode(', ', $names), // description
                        'Alert', // module
                        $properties, // properties
                        $dispatcher->company_id // company_id
                    );
                }
            }
        }
        Log::info('Cron End : Driver Missing Load ( '.Carbon::now()->format('Y-m-d').' )');
        Log::info("Cron is working fine!");
    }
}
