<?php

namespace App\Console\Commands;

use App\Models\DropTrailer;
use App\Models\LoadDetail;
use Carbon\Carbon;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;

class DropTrailerLoadDroppedOrOnHoldNotifyCron extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'dropTrailerLoadDroppedOrOnHoldNotify:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'This Cron Send Notification base on Set Retrieval Notification date on Dropped Trailers';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        Log::info('Cron Start : Dropped Trailers Load Dropped Or OnHold Notify ( '.Carbon::now()->format('Y-m-d H:i:s').' )');

        $records = DropTrailer::where('retrieval_notification', '1')->where('type', '1')
                    ->where('notify_date', '>=', now()->subMinutes(10)->toDateTimeString())
                    ->where('notify_date', '<=', now()->toDateTimeString())
                    ->get(['load_id', 'drop_or_keep_driver', 'notify_users']);

        foreach ($records as $record){
            if($record->notify_users != ""){
                $loadDetails = LoadDetail::where('load_id', $record->load_id)->first();
                $notifyUsers = json_decode($record->notify_users);

                if($record->drop_or_keep_driver == '1'){
                    sendDroppedDropTrailerNotification($notifyUsers, ['load_number' => $loadDetails->load_number ?? '']);
                }elseif ($record->drop_or_keep_driver == '2') {
                    sendOnHoldDropTrailerNotification($notifyUsers, ['load_number' => $loadDetails->load_number ?? '']);
                }
            }
        }

        Log::info('Cron End : Dropped Trailers Load Dropped Or OnHold Notify ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
    }
}
