<?php

namespace App\Console\Commands;

use App\Mail\CompanyAdmin\SendLoadBillingReminder;
use App\Models\CompanyAdmin;
use App\Models\LoadBillingReminderHistory;
use App\Models\LoadDetail;
use Carbon\Carbon;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;

class LoadBillingReminder extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'load-billing-reminder:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        Log::info('Cron Start : Billing Reminder ( '.Carbon::now()->format('Y-m-d H:i:s').' )');

        try {
            $currentDateTime = Carbon::now();

            $billingReminders = LoadBillingReminderHistory::where('reminder_status', 1)->get();

            foreach ($billingReminders as $billingReminder){
                if($billingReminder->next_reminder_repeat_at != ''){
                    $reminderDateTime = Carbon::parse($billingReminder->next_reminder_repeat_at);

                    if($reminderDateTime->lte($currentDateTime)){
                        $contacts = [];

                        $loadDetail = LoadDetail::findOrFail($billingReminder->load_detail_id);
                        $brokerDetail = $loadDetail->broker;

                        if(!empty($brokerDetail)){
                            $contacts['name'] = $brokerDetail->bill_to_name ?? $brokerDetail->name; // Broker Name
                            $contacts['email'] = $brokerDetail->billing_email ?? ''; // Broker Email
                            $contacts['phone'] = $brokerDetail->billing_phone ?? ''; // Broker Phone

                            if(!empty($billingReminder->cc)){
                                foreach ($billingReminder->cc as $cc){
                                    $companyAdminDetail = CompanyAdmin::findOrFail($cc);

                                    $contacts['cc'][] = array(
                                        'name' => $companyAdminDetail->full_name ?? '', // Company Admin Name
                                        'email' => str_replace(' ', '', $companyAdminDetail->email) ?? '', // Company Admin Email
                                        'phone' => $companyAdminDetail->cell_phone ?? '', // Company Admin Phone
                                    );
                                }
                            }

                            if(isset($contacts['email']) && $contacts['email'] != ''){
                                $mailData = [];
                                $mailData['company_id'] = $brokerDetail->company_id;
                                $mailData['load_number'] = $loadDetail->load_number;
                                $mailData['name'] = $contacts['name'];
                                $mailData['instruction'] = $billingReminder->instruction;

                                // Replace variable $contactsUpdated for Bug Fix Multiple CC
                                $contactsUpdated = $contacts;
                                $contactsUpdated['cc'] = array_column($contactsUpdated['cc'], 'email');

                                sendEmail($contacts, new SendLoadBillingReminder($mailData));
                            }

                            if(isset($contacts['phone']) && $contacts['phone'] != ''){
                                $getCompanyTwilioData = getCompanyTwilioData($brokerDetail->company_id);

                                if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                                    $messageBody = 'Billing Reminder for load number: '.$loadDetail->load_number.'\n\n'.$billingReminder->instruction;

                                    sendSMS($contacts['phone'], $messageBody, $getCompanyTwilioData);

                                    foreach ($contacts['cc'] as $cc){
                                        if($cc['phone'] != ''){
                                            sendSMS($contacts['phone'], $messageBody, $getCompanyTwilioData);
                                        }
                                    }
                                }
                            }

                            $reminderHistory = $billingReminder->history ?? [];

                            $reminderHistory[] = array(
                                'date' => Carbon::parse($reminderDateTime)->format('m/d/Y'),
                                'time' => Carbon::parse($reminderDateTime)->format(getTimeLayout('0')),
                                'created_by' => $billingReminder->created_by,
                                'repeat' => $billingReminder->repeat,
                                'reminder' => ($billingReminder->repeat == '1') ? reminderRepeatFrequency()[$billingReminder->reminder_repeat_every] : '',
                                'cc' => (count($contacts['cc']) > 0) ? array_column($contacts['cc'], 'name') : null,
                                'instruction' => $billingReminder->instruction,
                            );

                            if($billingReminder->repeat == '1'){
                                if(!is_null($billingReminder->reminder_repeat_every)){
                                    if($billingReminder->reminder_repeat_every == '1_day'){
                                        $billingReminder->next_reminder_repeat_at = Carbon::parse($billingReminder->next_reminder_repeat_at)->addDay(1)->format('Y-m-d H:i:s');
                                    }else if($billingReminder->reminder_repeat_every == '1_week'){
                                        $billingReminder->next_reminder_repeat_at = Carbon::parse($billingReminder->next_reminder_repeat_at)->addWeek(1)->format('Y-m-d H:i:s');
                                    }else if($billingReminder->reminder_repeat_every == '15_day'){
                                        $billingReminder->next_reminder_repeat_at = Carbon::parse($billingReminder->next_reminder_repeat_at)->addDay(15)->format('Y-m-d H:i:s');
                                    }else if($billingReminder->reminder_repeat_every == '1_month'){
                                        $billingReminder->next_reminder_repeat_at = Carbon::parse($billingReminder->next_reminder_repeat_at)->addMonth(1)->format('Y-m-d H:i:s');
                                    }else if($billingReminder->reminder_repeat_every == '2_month'){
                                        $billingReminder->next_reminder_repeat_at = Carbon::parse($billingReminder->next_reminder_repeat_at)->addMonth(2)->format('Y-m-d H:i:s');
                                    }else if($billingReminder->reminder_repeat_every == '3_month'){
                                        $billingReminder->next_reminder_repeat_at = Carbon::parse($billingReminder->next_reminder_repeat_at)->addMonth(3)->format('Y-m-d H:i:s');
                                    }else if($billingReminder->reminder_repeat_every == '4_month'){
                                        $billingReminder->next_reminder_repeat_at = Carbon::parse($billingReminder->next_reminder_repeat_at)->addMonth(4)->format('Y-m-d H:i:s');
                                    }else if($billingReminder->reminder_repeat_every == '5_month'){
                                        $billingReminder->next_reminder_repeat_at = Carbon::parse($billingReminder->next_reminder_repeat_at)->addMonth(5)->format('Y-m-d H:i:s');
                                    }else if($billingReminder->reminder_repeat_every == '6_month'){
                                        $billingReminder->next_reminder_repeat_at = Carbon::parse($billingReminder->next_reminder_repeat_at)->addMonth(6)->format('Y-m-d H:i:s');
                                    }else{
                                        $billingReminder->next_reminder_repeat_at = null;
                                    }

                                    $billingReminder->history = $reminderHistory ?? [];
                                    $billingReminder->save();
                                }
                            }else{
                                $billingReminder->history = $reminderHistory ?? [];
                                $billingReminder->reminder_status = 0;
                                $billingReminder->save();
                            }
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            Log::error('Billing Reminder Error: '.$e->getMessage());
        }

        Log::info('Cron End : Billing Reminder ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
    }
}
