<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;

use App\Models\LoadDetail;

use DB;
use Carbon\Carbon;

class LoadNotPickedUponTimeCron extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature    = 'loadnotpickedupontime:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description  = 'Load not picked up on time';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        Log::info('Cron Start : Load not picked up on time ( '.Carbon::now()->toDateTimeString().' )');
        Log::info('Older 30 min date & time ( '.Carbon::now()->subMinutes(30)->toDateTimeString().' )');
        $loads = LoadDetail::select('id','load_id','load_number','driver_id','booked_by','submitted_by','load_type','load_status','shipper_date_time','shipper_date_time_end','load_not_picked_up_on_time','self_dispatch','is_on_hold')
            ->where('is_disable', 0)
            ->whereIn('load_type', [0])
            ->where('load_status', 3)
            ->where(function($q) {
                $q->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0]);
            })
            ->where(function($q) {
                $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
            })
            ->whereRaw('IFNULL(shipper_date_time_end, shipper_date_time) <= ?', [Carbon::now()->subMinutes(30)->toDateTimeString()])
            ->whereColumn(DB::raw('IFNULL(shipper_date_time_end, shipper_date_time)'),'!=',DB::raw('IFNULL(load_not_picked_up_on_time, "")'))
            ->orderByRaw('IFNULL(shipper_date_time_end, shipper_date_time) asc')
            ->get();
        if($loads->isNotEmpty()){
            Log::info('Total Load not picked up on time : ( '.$loads->count().' )');
            foreach ($loads as $key => $load) {
                $load_not_picked_up_on_time         = $load->getRawOriginal('shipper_date_time');
                if(!is_null($load->getRawOriginal('shipper_date_time_end'))){
                    $load_not_picked_up_on_time     = $load->getRawOriginal('shipper_date_time_end');
                }
                $load->load_not_picked_up_on_time   = $load_not_picked_up_on_time;
                $load->updated_at                   = Carbon::now();
                $load->save();
                if(isset($load->self_dispatch) && $load->self_dispatch==1){
                    if((isset($load->driver) && !is_null($load->driver)) && (isset($load->driver->dispatcher) && !is_null($load->driver->dispatcher))){
                        createNotification(
                            1, // source
                            'Attention!', // name
                            $load->driver->dispatcher->id, // user_id
                            $load->driver->dispatcher->type, // user_type
                            2, // type
                            $load->load_id, // action_id
                            'Load not picked up on time - '.$load->load_number, // description
                            'Alert', // module
                            [], // properties
                            $load->driver->dispatcher->company_id // company_id
                        );
                    }
                }elseif (isset($load->dispatcher) && !is_null($load->dispatcher)) {
                    createNotification(
                        1, // source
                        'Attention!', // name
                        $load->dispatcher->id, // user_id
                        $load->dispatcher->type, // user_type
                        2, // type
                        $load->load_id, // action_id
                        'Load not picked up on time - '.$load->load_number, // description
                        'Alert', // module
                        [], // properties
                        $load->dispatcher->company_id // company_id
                    );
                }else{
                    Log::info('Driver manager record not found : ( '.$load->load_number.' )');
                }
            }
        }else{
            Log::info('No record found : Load not picked up on time');
        }
        Log::info('Cron End : Load not picked up on time ( '.Carbon::now()->toDateTimeString().' )');
        Log::info("Cron is working fine!");
    }
}
