<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;

use App\Models\LoadDetail;
use App\Models\IntegrationSetting;

use App\Traits\TrimbleTrait;
use App\Traits\SamsaraTrait;

use Carbon\Carbon;

class TripUpdateGPSPositions extends Command
{

    use TrimbleTrait, SamsaraTrait;

    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature    = 'trip:updategpspositions';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description  = 'Trip Update GPS Positions';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        Log::info('Cron Start : Update the position of driver on Trip management Trip View map ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
        $query      = LoadDetail::select('id', 'load_id', 'load_number', 'load_type', 'intransit_date_time', 'dispatched_date_time', 'is_on_hold', 'load_status', 'driver_id', 'truck_number', 'load_trip_id');
        $query->where(function($q) {
            $q->where('is_disable', 0)
            ->whereIn('load_type', [0, 6])
            ->where('load_status', 4)
            ->where('is_on_hold', '!=', 2)
            ->where(function($q) {
                $q->whereNotNull('driver_id')
                    ->where('driver_id', '!=', 0);
            })
            ->whereNotNull('truck_number')
            ->whereNotNull('dispatched_date_time')
            ->whereNotNull('load_trip_id');
        });
        $results    = $query->get();
        if($results->isNotEmpty()){
            Log::info('Total Trip ( '.$results->count().' )');
            foreach( $results as $key => $load_detail ) {
                $trimble_api_key                                    = $load_detail->loads->company->trimble_api_key ?? null;
                if(!is_null($trimble_api_key)){
                    $truck                                          = $load_detail->truck ?? null;
                    if(!is_null($truck)){
                        $vehicleIds                                 = $truck->samsara_id ?? null; // DTL88 = 278018081719814
                        if(!is_null($vehicleIds)){
                            $startTime                              = $load_detail->getRawOriginal('dispatched_date_time'); // '2023-09-28 09:30:00';
                            $getGPSCoords                           = $this->getGPSCoords($load_detail->load_trip_id, $trimble_api_key);
                            if(!empty($getGPSCoords)){
                                $gpsLastValue                       = end($getGPSCoords);    // Get the last element
                                if ($gpsLastValue !== false) {
                                    // The array is not empty, and $gpsLastValue contains the last element.
                                    // Log::info(json_encode($gpsLastValue));
                                    $startTime                      = Carbon::parse($gpsLastValue->deviceDT)->addSeconds(1)->format('Y-m-d H:i:s');
                                }
                            }
                            // Parse the date in 'Y-m-d H:i:s' format using Carbon
                            $startTime                              = Carbon::parse($startTime);
                            // Format the date in RFC 3339 format ('Y-m-d\TH:i:s\Z')
                            $startTime                              = $startTime->format('Y-m-d\TH:i:s\Z');

                            $currentTime                            = Carbon::now(); // Carbon::parse('2023-09-28 23:00:00');
                            // Format the current time in RFC 3339 format ('Y-m-d\TH:i:s\Z')
                            $currentTime                            = $currentTime->format('Y-m-d\TH:i:s\Z');

                            //
                            $getAPIKey                              = IntegrationSetting::where('company_id', $truck->company_id)->where('carrier_id', $truck->carrier_id)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_vehicles', '1')->first();
                            if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                                $params                             = [
                                    'startTime'                     => $startTime,
                                    'endTime'                       => $currentTime,
                                    'vehicleIds'                    => $vehicleIds,
                                    'types'                         => 'gps'
                                ];

                                $historicalStats                    = $this->historicalStats($getAPIKey->api_key, $params);
                                if(!empty($historicalStats)){
                                    Log::info('Vehicle Id : ( '.$truck->unit.' = '.$vehicleIds.' )');
                                    $firstElement                   = reset($historicalStats); // Get the first element
                                    if ($firstElement !== false) {
                                        Log::info('Start time : ( '.Carbon::parse($startTime)->format('Y-m-d H:i:s').' = '.$startTime.' )');
                                        // Log::info(json_encode($firstElement));
                                    }
                                    $lastElement                    = end($historicalStats);    // Get the last element
                                    if ($lastElement !== false) {
                                        Log::info('Current time : ( '.Carbon::parse($currentTime)->format('Y-m-d H:i:s').' = '.$currentTime.' )');
                                        // Log::info(json_encode($lastElement));
                                    }
                                    // $historicalStats             = array_slice($historicalStats, 0, 10); // Get the first 10 elements
                                    // Log::info(json_encode($historicalStats));

                                    // Trip Management’s Update GPS Positions
                                    $array                          = [];
                                    $array["tripId"]                = $load_detail->load_trip_id;

                                    // Create a new array with the fixed values using array_map
                                    $array["locations"]             = array_map(function ($item) {
                                        return [
                                            "coords"                => [
                                                "lat"               => $item->latitude,
                                                "lon"               => $item->longitude
                                            ],
                                            "deviceDT"              => Carbon::parse($item->time)->toIso8601String()

                                        ];
                                    }, $historicalStats);

                                    // Update GPS Positions Api
                                    $gpsPosition                    = $this->gpsPosition($array, $trimble_api_key);
                                    if(isset($gpsPosition->success) && $gpsPosition->success == true){
                                        Log::info('Update GPS Position : ( Load Number => '.$load_detail->load_number.', Trip Id => '.$load_detail->load_trip_id.' )');
                                    }
                                    // Trip Management’s Update GPS Positions
                                }
                            }
                            //
                        }
                    }
                }
            }
        }
        Log::info('Cron End : Update the position of driver on Trip management Trip View map ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
    }
}
