<?php

namespace App\Console\Commands;

use App\Models\TruckIntegration;
use App\Models\TruckTransponderProviderHistory;
use App\Models\TruckTransponderProviderRemoveAndReassign;
use Carbon\Carbon;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;

class TruckTransponderSync extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'truck-transponder-sync:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        Log::info('Cron Start : Truck Transponder Sync ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
        $startDateTime      = Carbon::now()->subMinutes(10);
        $currentDateTime    = Carbon::now();

        $transponderProviders = TruckTransponderProviderRemoveAndReassign::where('action_date', $currentDateTime->format('Y-m-d'))
            ->where('action_time', '>=', $startDateTime->format('H:i'))
            ->where('action_time', '<=', $currentDateTime->format('H:i'))
            ->orderBy('id')->get();

        foreach ($transponderProviders as $transponderProvider){
            if($transponderProvider->action_type == 'remove'){
                if(!is_null($transponderProvider->transponder_id)){
                    $truckIntegrationExists = TruckIntegration::find($transponderProvider->transponder_id);
                    if($truckIntegrationExists){
                        TruckTransponderProviderHistory::create([
                            'truck_id' => $transponderProvider->truck_id,
                            'action' => 'Removed',
                            'truck_number' => $transponderProvider->truck_number,
                            'provider' => $transponderProvider->provider,
                            'provider_id' => $transponderProvider->provider_id,
                            'action_date_time' => Carbon::now(),
                        ]);
                        $truckIntegrationExists->delete();
                        $transponderProvider->delete();
                    }
                }
            }else if($transponderProvider->action_type == 'add'){
                if(!is_null($transponderProvider->truck_id)){
                    $truckIntegrationExists = TruckIntegration::where('truck_id', $transponderProvider->truck_id)->where('integration_type', 'transponder-provider')->first();
                    if($truckIntegrationExists){
                        TruckTransponderProviderHistory::create([
                            'truck_id' => $transponderProvider->truck_id,
                            'action' => 'Removed',
                            'truck_number' => $transponderProvider->truck_number,
                            'provider' => $transponderProvider->provider,
                            'provider_id' => $transponderProvider->provider_id,
                            'action_date_time' => Carbon::now(),
                        ]);
                        $truckIntegrationExists->delete();
                    }

                    $createTransponder = TruckIntegration::create([
                        'company_id' => $transponderProvider->truck->company_id ?? '',
                        'truck_id' => $transponderProvider->truck_id,
                        'integration_type' => 'transponder-provider',
                        'provider' => $transponderProvider->provider ?? '',
                        'provider_id' => $transponderProvider->provider_id ?? '',
                        'connected_status' => 1,
                        'extra_params' => array(
                            'truck_number' => $transponderProvider->truck_number ?? '',
                            'added_date' => $transponderProvider->action_date ?? '',
                            'added_time' => $transponderProvider->action_time ?? '',
                            'remove_date' => '',
                            'remove_time' => '',
                            'replace_date' => '',
                            'replace_time' => '',
                            'replace_provider' => '',
                            'replace_provider_id' => '',
                        ),
                    ]);

                    if($createTransponder) {
                        TruckTransponderProviderHistory::create([
                            'truck_id' => $createTransponder->truck_id,
                            'action' => 'Added',
                            'truck_number' => $createTransponder->extra_params['truck_number'],
                            'provider' => $createTransponder->provider,
                            'provider_id' => $createTransponder->provider_id,
                            'action_date_time' => Carbon::now(),
                        ]);
                        $transponderProvider->delete();
                    }
                }
            }
        }
        Log::info('Cron End : Truck Transponder Sync ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
    }
}
