<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;

use App\Models\LoadDetail;
use App\Models\FuelSurchargeTemplate;

use Carbon\Carbon;

class UpdateLoadFuelSurcharge extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature    = 'update-load-fuel-surcharge:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description  = 'Update Load fuel surcharge';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {   
        Log::info('Cron Start : Update Load Fuel Surcharge for Diesel & Gas ( '.Carbon::now()->format('Y-m-d H:i:s').' )');

        $startDateTime                                                  = Carbon::now();
        $endDateTime                                                    = Carbon::now()->addMinutes(90);

        $query                                                          = LoadDetail::query()
            ->where('is_disable', 0)
            ->whereIn('load_type', [0, 6]) // 0 = OCCUPIED, 6 = TONU
            ->whereIn('load_status',[0, 2, 3, 4, 11]) // 0 = Pending,  2 = Preplanned, 3 = Confirmed, 4 = In Transit, 11 = Send To Driver
            ->where(function($q) {
                $q->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0, 1]);
            })
            ->whereBetween('shipper_date_time', [$startDateTime, $endDateTime])
            ->where('fuel_surcharge_type', 1) // 1 = Automatic
            ->whereHas('truck', function ($query) {
                $query->whereIn('fuel_type', [0, 1]);
            });
        $load_details                                                   = $query->get();
        if($load_details->isNotEmpty()){
            Log::info('Total Load : ( '.$load_details->count().' )');

            foreach ($load_details as $key => $load_detail) {
                $oldData                                                = $load_detail->toArray();

                $company_id                                             = $load_detail->loads->company_id;
                $load_number                                            = $load_detail->load_number ?? '';
                $customer_name                                          = $load_detail->broker->name ?? '';
                $truck_fuel_type                                        = $load_detail->truck->fuel_type == 0 ? 'Desiel' : 'Regular Gas';

                $fuel_surcharge_type                                    = $load_detail->fuel_surcharge_type;
                $loaded_miles                                           = $load_detail->loaded_miles ?? '';
                $agreed_upon_rate                                       = $load_detail->agreed_upon_rate ?? '';
                $fuel_surcharge_cost_per_mile                           = $load_detail->fuel_surcharge_cost_per_mile ?? '';
                $fuel_surcharge                                         = $load_detail->fuel_surcharge ?? '';
                $line_haul                                              = $load_detail->line_haul ?? '';

                // Get the original value of a specific attribute without mutators
                $shipper_date_time                                      = $load_detail->getRawOriginal('shipper_date_time');
                $date                                                   = Carbon::parse($shipper_date_time);
                $shipper_date                                           = $date->format('Y-m-d');
                $startOfWeek                                            = $date->startOfWeek(); // Shipper start of week

                Log::info('Load Number = '.$load_detail->load_number.', Customer Name = '.$customer_name.', Truck Fuel Type = '.$truck_fuel_type.', Fuel Surcharge Type = Automatic, Shipper Date = '.$shipper_date.', Loaded Miles = '.$loaded_miles.', Agreed Upon Rate = '.$agreed_upon_rate.', Fuel Surcharge Cost Per Mile = '.$fuel_surcharge_cost_per_mile.', Fuel Surcharge = '.$fuel_surcharge.', Line Haul = '.$line_haul);

                $template_fuel_surcharge                                = null;
                $template_fuel_surcharge_cost                           = null;

                if($load_detail->truck->fuel_type == 0){
                    // Desiel
                    $templateQuery                                      = FuelSurchargeTemplate::query()
                        ->where('company_id', $company_id)
                        ->where('customer', $load_detail->brokerage_firm)
                        ->where('fuel_type', 'EPD2D') // EPD2D = U.S. On-Highway Diesel, EPMR = U.S. Regular Gasoline Price
                        // ->where(function ($q) use ($shipper_date) {
                        //     $q->whereDate('start_date', '<=', $shipper_date)
                        //         ->whereDate('end_date', '>=', $shipper_date);
                        // })
                        ->where('set_as_active', '1'); 
                    $template                                           = $templateQuery->first();
                    if(!is_null($template)){
                        // Format the start of the week date as you need
                        $startOfWeekFormattedDate                       = $startOfWeek->format('Y-m-d');
                        // Retrive fuel surcharge data
                        $getFuelSurchargeArray                          = getTemplateFuelSurchargeArray($template, $startOfWeekFormattedDate);
                        if(!empty($getFuelSurchargeArray) && isset($getFuelSurchargeArray['fuel_surcharge']) && $getFuelSurchargeArray['fuel_surcharge']!=''){
                            $template_fuel_surcharge                    = is_numeric($getFuelSurchargeArray['fuel_surcharge']) ? round($getFuelSurchargeArray['fuel_surcharge'], 2) : null;
                            Log::info('Customer '.$load_detail->brokerage_firm.' => EPD2D = U.S. On-Highway Diesel '.$template_fuel_surcharge);
                        }
                    }else{
                        $templateQuery                                  = FuelSurchargeTemplate::query()
                            ->where('company_id', $company_id)
                            ->where('customer', 'FSD')
                            ->where('fuel_type', 'EPD2D') // EPD2D = U.S. On-Highway Diesel, EPMR = U.S. Regular Gasoline Price
                            ->where('set_as_active', '1'); 
                        $template                                       = $templateQuery->first();
                        if(!is_null($template)){
                            // Retrive fuel surcharge data
                            $getFuelSurchargeArray                      = getTemplateFuelSurchargeArray($template);
                            if(!empty($getFuelSurchargeArray) && isset($getFuelSurchargeArray['fuel_surcharge']) && $getFuelSurchargeArray['fuel_surcharge']!=''){
                                $template_fuel_surcharge                = is_numeric($getFuelSurchargeArray['fuel_surcharge']) ? round($getFuelSurchargeArray['fuel_surcharge'], 2) : null;
                                Log::info('Default EPD2D = U.S. On-Highway Diesel '.$template_fuel_surcharge);
                            }
                        }
                    }
                } elseif ($load_detail->truck->fuel_type == 1) {
                    // Regular Gas
                    $templateQuery                                      = FuelSurchargeTemplate::query()
                        ->where('company_id', $company_id)
                        ->where('customer', $load_detail->brokerage_firm)
                        ->where('fuel_type', 'EPMR') // EPD2D = U.S. On-Highway Diesel, EPMR = U.S. Regular Gasoline Price
                        // ->where(function ($q) use ($shipper_date) {
                        //     $q->whereDate('start_date', '<=', $shipper_date)
                        //         ->whereDate('end_date', '>=', $shipper_date);
                        // })
                        ->where('set_as_active', '1'); 
                    $template                                           = $templateQuery->first();
                    if(!is_null($template)){
                        // Format the start of the week date as you need
                        $startOfWeekFormattedDate                       = $startOfWeek->format('Y-m-d');
                        // Retrive fuel surcharge data
                        $getFuelSurchargeArray                          = getTemplateFuelSurchargeArray($template, $startOfWeekFormattedDate);
                        if(!empty($getFuelSurchargeArray) && isset($getFuelSurchargeArray['fuel_surcharge']) && $getFuelSurchargeArray['fuel_surcharge']!=''){
                            $template_fuel_surcharge                    = is_numeric($getFuelSurchargeArray['fuel_surcharge']) ? round($getFuelSurchargeArray['fuel_surcharge'], 2) : null;
                            Log::info('Customer '.$load_detail->brokerage_firm.' => EPD2D = U.S. On-Highway Diesel '.$template_fuel_surcharge);
                        }
                    }else{
                        $templateQuery                                  = FuelSurchargeTemplate::query()
                            ->where('company_id', $company_id)
                            ->where('customer', 'FSG')
                            ->where('fuel_type', 'EPMR') // EPD2D = U.S. On-Highway Diesel, EPMR = U.S. Regular Gasoline Price
                            ->where('set_as_active', '1'); 
                        $template                                       = $templateQuery->first();
                        if(!is_null($template)){
                            // Retrive fuel surcharge data
                            $getFuelSurchargeArray                      = getTemplateFuelSurchargeArray($template);
                            if(!empty($getFuelSurchargeArray) && isset($getFuelSurchargeArray['fuel_surcharge']) && $getFuelSurchargeArray['fuel_surcharge']!=''){
                                $template_fuel_surcharge                = is_numeric($getFuelSurchargeArray['fuel_surcharge']) ? round($getFuelSurchargeArray['fuel_surcharge'], 2) : null;
                                Log::info('Default EPD2D = U.S. On-Highway Diesel '.$template_fuel_surcharge);
                            }
                        }
                    }
                }

                if(isset($template)){
                    if($template->rate_type == '1'){
                        // 0: Per Mile, 1: Percentage, 2: Both
                        if($agreed_upon_rate !='' && isset($template_fuel_surcharge) && $template_fuel_surcharge != ''){
                            Log::info('Percentage = '.$template_fuel_surcharge);

                            // Calculate the fuel surcharge
                            $template_fuel_surcharge_cost               = $agreed_upon_rate * ($template_fuel_surcharge / 100);
                            $template_fuel_surcharge_cost               = (float) number_format((float)$template_fuel_surcharge_cost, 2, '.', ''); // floor($template_fuel_surcharge_cost * 100) / 100;
                            $load_detail->fuel_surcharge_rate_type      = 1;
                        }
                    }else{
                        // 0: Per Mile, 1: Percentage, 2: Both
                        if($loaded_miles !='' && isset($template_fuel_surcharge) && $template_fuel_surcharge != ''){
                            Log::info('Per Mile = '.$template_fuel_surcharge);

                            // Calculate the fuel surcharge
                            $template_fuel_surcharge_cost               = $loaded_miles * $template_fuel_surcharge; // 541.668545;
                            $template_fuel_surcharge_cost               = (float) number_format((float)$template_fuel_surcharge_cost, 2, '.', ''); // floor($template_fuel_surcharge_cost * 100) / 100;
                            $load_detail->fuel_surcharge_rate_type      = 0;
                        }  
                    }
                }

                $load_detail->fuel_surcharge_cost_per_mile              = isset($template_fuel_surcharge) ? $template_fuel_surcharge : null;
                $load_detail->fuel_surcharge                            = isset($template_fuel_surcharge_cost) ? $template_fuel_surcharge_cost : null;
                if(isset($agreed_upon_rate) && isset($load_detail->fuel_surcharge)){
                    $load_detail->line_haul                             = $agreed_upon_rate - $load_detail->fuel_surcharge;
                }else{
                    $load_detail->line_haul                             = null;
                }
                if($load_detail->save()){
                    Log::info('Fuel surcharge cost updated');    
                }

                Log::info('=======');
            }
        }else{
            Log::info('No record found');
        }

        Log::info('Cron End : Update Load Fuel Surcharge for Diesel & Gas ( '.Carbon::now()->format('Y-m-d H:i:s').' )');

        $this->info('Upldate Load fuel surcharge');
    }
}
