<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;

use App\Models\Truck;
use App\Models\IntegrationSetting;
use App\Models\VehicleHistoricalStats;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ServerException;

use Carbon\Carbon;

class VehicleHistoricalStatsInsert extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature    = 'vehiclehistoricalstats:insert';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description  = 'Vehicle stats insert 5 min';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        Log::info('Cron Start : Vehicle stats insert ( '.Carbon::now()->format('Y-m-d H:i:s').' )');

        VehicleHistoricalStats::withTrashed()->where('created_at', '<', Carbon::now()->subHours(1))->forceDelete();

        $query      = Truck::whereNotNull('samsara_id');
        $results    = $query->get();
        if($results->isNotEmpty()){
            foreach( $results as $key => $truck ) {
                $getAPIKey      = IntegrationSetting::where('company_id', $truck->company_id)->where('carrier_id', $truck->carrier_id)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_vehicles', '1')->first();
                if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                    try {
                        $url                    = 'https://api.samsara.com/fleet/vehicles/stats?vehicleIds='.$truck->samsara_id.'&types=gps,obdOdometerMeters';
                        $client                 = new \GuzzleHttp\Client();
                        $response               = $client->request('GET', $url, [
                            'headers'           => [
                                'Accept'        => 'application/json',
                                'Authorization' => 'Bearer '.$getAPIKey->api_key,
                            ],
                        ]);
                        // Check if response status code is OK (200)
                        if ($response->getStatusCode() == 200) {
                            $body               = json_decode($response->getBody());
                            // Do something with the response body
                            if(isset($body)){
                                $data           = isset($body->data[0])?$body->data[0]:null;
                                if(isset($data)){
                                    $item       = VehicleHistoricalStats::create([
                                        'truck_id'              => $truck->id,
                                        'obd_odometer_meters'   => $data->obdOdometerMeters->value ?? null,
                                        'obd_odometer_miles'    => $data->obdOdometerMeters->value ? round($data->obdOdometerMeters->value * 0.000621371) : null,
                                        'speed_mph'             => $data->gps->speedMilesPerHour ?? null,
                                        'latitude'              => $data->gps->latitude ?? null,
                                        'longitude'             => $data->gps->longitude ?? null,
                                        'location'              => $data->gps->reverseGeo->formattedLocation ?? null,
                                        'last_updated_at'       => $data->gps->time ? Carbon::parse($data->gps->time)->format('Y-m-d H:i:s') : null,
                                        'data'                  => $data
                                    ]);
                                }else{
                                    Log::error('Truck vin number ('.$truck->vin_number.') : Samsara response data not set.');
                                }
                            }
                        }
                    } catch (\GuzzleHttp\Exception\ClientException $e) {
                        // Handle client-side errors (e.g. 4xx status codes)
                        $response               = $e->getResponse();
                        if ($response) {
                            $statusCode         = $response->getStatusCode();
                            $body               = $response->getBody()->getContents();
                            Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : client exception '.$body);
                        } else {
                            $errorMessage       = $e->getMessage();
                            Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : client exception '.$errorMessage);
                        }
                    } catch (\GuzzleHttp\Exception\ServerException $e) {
                        // Handle server-side errors (e.g. 5xx status codes)
                        $response               = $e->getResponse();
                        if ($response) {
                            $statusCode         = $response->getStatusCode();
                            $body               = $response->getBody()->getContents();
                            Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : server exception '.$body);
                        } else {
                            $errorMessage       = $e->getMessage();
                            Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : server exception '.$errorMessage);
                        }
                    } catch (\Exception $e) {
                        // Handle other exceptions
                        $errorMessage           = $e->getMessage();
                        Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : other exception '.$errorMessage);
                    }
                }
            }
        }
        Log::info('Cron end : Vehicle stats insert ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
    }
}
