<?php

namespace App\Console;

use Illuminate\Console\Scheduling\Schedule;
use Illuminate\Foundation\Console\Kernel as ConsoleKernel;

class Kernel extends ConsoleKernel
{
    /**
     * The Artisan commands provided by your application.
     *
     * @var array
     */
    protected $commands = [
        Commands\OnHoldSendNotification1Hours::class,
        Commands\OnHoldSendNotification4Hours::class,
        Commands\OnHoldSendNotification8Hours::class,
        Commands\OnHoldSendNotification24Hours::class,

        Commands\NotifySendNotification5Minutes::class,
        Commands\NotifySendNotification15Minutes::class,
        Commands\NotifySendNotification30Minutes::class,
        Commands\NotifySendNotification60Minutes::class,

        Commands\OfferExpireCron::class,
        Commands\ReportDriversDataCron::class,

        Commands\DriverMissingLoadCron::class,
        Commands\LoadNotPickedUponTimeCron::class,
        Commands\LoadNotDeliveredOnTimeCron::class,
        Commands\LoadAccessorialTempsDestroyCron::class,

        Commands\RealTimeTruckLocationUpdate::class,
        Commands\VehicleHistoricalStatsInsert::class,
        Commands\AppointmentReminderCron::class,
        Commands\DocumentRequestReminderCron::class,
        Commands\CreateChatLoads::class,
        Commands\TripUpdateGPSPositions::class,
        Commands\DropTrailerLoadDroppedOrOnHoldNotifyCron::class,
        Commands\SyncTruckOdometer::class,
        Commands\RemoveEmailLogs::class,
        Commands\DropHookRetrievalReminder::class,
        Commands\UpdateDefaultFuelSurcharge::class,
        Commands\UpdateLoadFuelSurcharge::class,

        Commands\SmsLimitNotification::class,
        Commands\StatusLog::class,
        Commands\LoadBillingReminder::class,
        Commands\GalleryRemoveDummyRecords::class,
        Commands\TruckTransponderSync::class,
        Commands\CreateOrUpdateBrokersTrimblePlaceId::class,
    ];

    /**
     * Define the application's command schedule.
     *
     * @param  \Illuminate\Console\Scheduling\Schedule  $schedule
     * @return void
     */
    protected function schedule(Schedule $schedule)
    {
        $schedule->command('onhold:1hours')->hourly();
        $schedule->command('onhold:4hours')->cron('0 */4 * * *');
        $schedule->command('onhold:8hours')->cron('0 */8 * * *');
        $schedule->command('onhold:24hours')->dailyAt('09:00');

        $schedule->command('notify:5minutes')->cron('5 * * * *');
        $schedule->command('notify:15minutes')->cron('15 * * * *');
        $schedule->command('notify:30minutes')->cron('30 * * * *');
        $schedule->command('notify:60minutes')->hourly();

        $schedule->command('offerexpire:cron')->dailyAt('00:01');
        $schedule->command('reportdriversdata:cron')->everyMinute();

        $schedule->command('drivermissingload:cron')->daily();
        $schedule->command('loadnotpickedupontime:cron')->everyMinute();
        $schedule->command('loadnotdeliveredontime:cron')->everyMinute();
        $schedule->command('loadaccessorialtempsdestroy:cron')->everyTenMinutes();

        $schedule->command('trucklocation:update')->everyFifteenMinutes();
        $schedule->command('vehiclehistoricalstats:insert')->everyFiveMinutes();
        $schedule->command('appointmentreminder:cron')->everyTenMinutes();
        $schedule->command('documentRequestReminder:cron')->everyTenMinutes();
        $schedule->command('createchatloads:cron')->twiceDaily(1);
        $schedule->command('trip:updategpspositions')->everyFiveMinutes();

        $schedule->command('dropTrailerLoadDroppedOrOnHoldNotify:cron')->everyTenMinutes();
        $schedule->command('syncTruckOdometer:cron')->everyThirtyMinutes();
        $schedule->command('emailLogs:remove')->dailyAt('00:00');
        $schedule->command('drophookretrievalreminder:cron')->everyMinute();
        $schedule->command('update-default-fuel-surcharge:cron')->everyTwoHours();
        $schedule->command('update-load-fuel-surcharge:cron')->cron('*/90 * * * *');

        $schedule->command('smslimitnotification')->daily();
        $schedule->command('statuslog')->monthly();
        $schedule->command('load-billing-reminder:cron')->everyTenMinutes();
        $schedule->command('galleryRemoveDummyRecords:cron')->daily();
        $schedule->command('truck-transponder-sync:cron')->everyTenMinutes();
        $schedule->command('create-or-update-brokers-trimble-place-id:cron')->daily();
    }

    /**
     * Register the commands for the application.
     *
     * @return void
     */
    protected function commands()
    {
        $this->load(__DIR__.'/Commands');

        require base_path('routes/console.php');
    }
}

