<?php

namespace App\Exports;

use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithMapping;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;
use PhpOffice\PhpSpreadsheet\Style\Border;

class LoadsExport implements FromCollection, WithMapping, WithHeadings, WithEvents
{
    protected $data;

    public function __construct(Collection $data)
    {
        $this->data     = $data;
    }

    public function collection()
    {
        return $this->data;
    }

    public function map($row): array
    {
        // Use a closure for custom mapping
        $customMap      = function ($row) {
            $load                                       = [];

            // If the driver's name matches with the driver's relation name
            if (isset($row->driver)) {
                $row->driver_name                       = $row->driver;
                unset($row->driver);
            }

            $load_occupied                              = false;
            // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 4 = NO DISPATCH, 5 = TIME OFF
            if(in_array($row->load_type, ['0', '6'])){
                // 0 = OCCUPIED, 6 = TONU
                $load_occupied                          = true;
            }

            $load['Load Number']                        = $load_occupied ? $row->load_number : getLoadTypeList()[$row->load_type];

            $load_status                                = getLoadStatusList()[$row->load_status] ?? '';
            $is_on_hold                                 = $row->is_on_hold ?? 0;
            if($is_on_hold == '3'){
                $load_status                            = 'Dropped';
            }elseif($is_on_hold == '2'){
                $load_status                            = 'Cancel';
            }elseif($is_on_hold == '1'){
                $load_status                            = 'On Hold';
            }
            $load['Status']                             = $load_status;

            $load['Billing Status']                     = ''; // Pending

            $load['Customer Name']                      = $load_occupied ? ($row->broker->name ?? '') : '';
            $load['Customer MC']                        = $load_occupied ? ($row->broker->mc_number ?? '') : '';
            $load['Customer Phone']                     = $load_occupied ? ($row->phone ?? '') : '';
            $load['Extension']                          = $load_occupied ? ($row->ext ?? '') : '';
            $load['Customer Representative']            = $load_occupied ? ($row->rep ?? '') : '';
            $load['Customer Email']                     = $load_occupied ? ($row->broker_email ?? '') : '';

            $load['Credit Score']                       = $load_occupied ? (isset($row->is_cod) && $row->is_cod == '1' ? '' : $row->score_ratting ?? '') : '';

            $load['Carrier']                            = $row->carrier->name ?? '';
            $load['Terminal']                           = $row->terminal->name ?? '';
            $load['Team']                               = $row->driver->team->name ?? '';

            $dispatcher_name                            = ''; // $row->driver->dispatcher->full_name ?? '';
            if(isset($row->self_dispatch) && $row->self_dispatch == 1){
                $dispatcher_name                        = 'Self-Dispatch';
            }else{
                $dispatcher_name                        = $row->dispatcher->full_name ?? '';
            }
            $load['Dispatcher Name']                    = $dispatcher_name;

            $load['Driver Name']                        = $row->driver_name ?? '';
            $load['Co Driver']                          = $row->co_driver->full_name ?? '';
            $load['Truck Number']                       = $row->truck_number ?? '';

            if(isset($row->loads->load_trailers) && $row->loads->load_trailers->isNotEmpty()){
                $trailer_number                         = '';
                if(isset($row->trailer_number)){
                    $trailer_number                     = $row->trailer_number;
                }
                $trailer_type                           = 'Not specified';
                if (isset($row->trailer_type) && $row->trailer_type != '' && isset(getTrailerTypeList()[$row->trailer_type])) {
                    $trailer_type                       = getTrailerTypeList()[$row->trailer_type];
                }

                $load['Trailer Type - 1']               = $trailer_type;
                $load['Trailer Number - 1']             = $trailer_number;

                $tn                                     = 2;
                foreach ($row->loads->load_trailers as $key => $load_trailer) {
                    // Break the loop when $tn is 7
                    if ($tn === 7) {
                        break;
                    }

                    $trailer_number                     = '';
                    if(isset($load_trailer->trailer_number)){
                        $trailer_number                 = $load_trailer->trailer_number;
                    }
                    $trailer_type                       = 'Not specified';
                    if (isset($load_trailer->trailer_type) && $load_trailer->trailer_type != '' && isset(getTrailerTypeList()[$load_trailer->trailer_type])) {
                        $trailer_type                   = getTrailerTypeList()[$load_trailer->trailer_type];
                    }

                    $load['Trailer Type - '.$tn]        = $trailer_type;
                    $load['Trailer Number - '.$tn]      = $trailer_number;

                    ++$tn;
                }

                for ($ti = $tn; $ti <= 6; ++$ti) {
                    $load['Trailer Type - '.$ti]        = '';
                    $load['Trailer Number - '.$ti]      = '';
                }
            }else{
                $trailer_number                         = '';
                if(isset($row->trailer_number)){
                    $trailer_number                     = $row->trailer_number;
                }
                $trailer_type                           = 'Not specified';
                if (isset($row->trailer_type) && $row->trailer_type != '' && isset(getTrailerTypeList()[$row->trailer_type])) {
                    $trailer_type                       = getTrailerTypeList()[$row->trailer_type];
                }

                $load['Trailer Type - 1']               = $trailer_type;
                $load['Trailer Number - 1']             = $trailer_number;

                $load['Trailer Type - 2']               = '';
                $load['Trailer Number - 2']             = '';
                $load['Trailer Type - 3']               = '';
                $load['Trailer Number - 3']             = '';
                $load['Trailer Type - 4']               = '';
                $load['Trailer Number - 4']             = '';
                $load['Trailer Type - 5']               = '';
                $load['Trailer Number - 5']             = '';
                $load['Trailer Type - 6']               = '';
                $load['Trailer Number - 6']             = '';
            }

            $total_stops                                = 0;
            if(isset($row->loads->load_stops) && $row->loads->load_stops->isNotEmpty()){
                $total_stops                            += $row->loads->load_stops->count();
            }

            $load['Extra Stops']                        = $total_stops;

            $load['Deadhead Miles']                     = $load_occupied ? (isset($row->empty_miles) ? formatNumberWithoutZeros($row->empty_miles, ['removeCommas' => true]) : '') : '';
            $load['Loaded Miles']                       = $load_occupied ? (isset($row->loaded_miles) ? formatNumberWithoutZeros($row->loaded_miles, ['removeCommas' => true]) : '') : '';
            $load['Total Miles']                        = $load_occupied ? (isset($row->total_miles) ? formatNumberWithoutZeros($row->total_miles, ['removeCommas' => true]) : '') : '';

            $load['Rate']                               = ''; // Pending

            // Calculation
            $total_pay                                  = 0;
            if(isset($row->agreed_upon_rate) && $row->agreed_upon_rate != ''){
                $total_pay                              = (float)$row->agreed_upon_rate;
            }
            $detention_due                              = 0;
            if(isset($row->detentions) && $row->detentions->isNotEmpty()){
                foreach ($row->detentions as $key => $load_detention) {
                    if(in_array($load_detention->detention_status,[2,3]) && $load_detention->review_status!='Denied'){
                        $detention_due                  += (float)$load_detention->detention_due;
                    }
                }
                $total_pay                              += $detention_due;
            }
            $other_charges                              = 0;
            if(isset($row->load_accessorials) && $row->load_accessorials->isNotEmpty()){
                $mileage_accessorial_array              = [];
                foreach ($row->load_accessorials as $key => $load_accessorial) {
                    $mileage_accessorial_array[]        = [
                        'company_accessorial_id'        => $load_accessorial->company_accessorial_id,
                        'company_accessorial_name'      => $load_accessorial->company_accessorial->name ?? '',
                        'charge_deduct'                 => $load_accessorial->charge_deduct,
                        'qty'                           => $load_accessorial->qty,
                        'rate'                          => $load_accessorial->rate,
                        'total'                         => $load_accessorial->total,
                    ];
                }
                $mileage_accessorial_array              = array_reduce($mileage_accessorial_array, function ($carry, $item) {
                    $key           = $item['company_accessorial_name'];
                    $carry[$key][] = $item;
                    return $carry;
                }, []);
                if(!empty($mileage_accessorial_array)){
                    foreach ($mileage_accessorial_array as $company_accessorial_name => $accessorials_array) {
                        $atotal                         = 0; // Accessorial total
                        foreach ($accessorials_array as $key => $accessorial_array) {
                            if($accessorial_array['charge_deduct']==1){
                                $atotal                 += $accessorial_array['total'];
                            }else if($accessorial_array['charge_deduct']==2){
                                $atotal                 -= $accessorial_array['total'];
                            }
                        }
                        if($atotal >= 0){
                            $other_charges              += $atotal;
                            $total_pay                  += $atotal;
                        }else{
                            $other_charges              -= abs($atotal);
                            $total_pay                  -= abs($atotal);
                        }
                    }
                }
            }
            // Calculation

            $load['Pay']                                = $load_occupied ? ($total_pay != 0 ? $total_pay : '') : '';

            $load['Shipper']                            = $load_occupied ? ($row->shipper->name ?? '') : '';
            $load['Shipper Appointment Type']           = $load_occupied ? ($row->shipping_time_range == 0 ? 'Appointment' : 'Range') : '';
            $load['Shipper Date']                       = $load_occupied ? (isset($row->shipper_date_time) ? date('n/j/Y', strtotime($row->getRawOriginal('shipper_date_time'))) : '') : '';
            $load['Shipper Time']                       = $load_occupied ? (isset($row->shipper_date_time) ? date('H:i', strtotime($row->getRawOriginal('shipper_date_time'))) : '') : '';
            $load['Shipper Address']                    = $load_occupied ? ($row->shipper_address_one ?? '') : '';
            $load['Shipper Address 2']                  = $load_occupied ? ($row->shipper_address_two ?? '') : '';
            $load['Shipper City']                       = $load_occupied ? ($row->shipper_city ?? '') : '';
            $load['Shipper State']                      = $load_occupied ? ($row->shipper_state ?? '') : '';
            $load['Shipper Zip Code']                   = $load_occupied ? ($row->shipper_zip ?? '') : '';
            $load['Shipper Phone']                      = $load_occupied ? ($row->shipper->phone ?? '') : '';
            $load['Shipper Contact']                    = $load_occupied ? ($row->shipper->dispatch_contact ?? '') : '';
            $load['Shipper Email']                      = $load_occupied ? ($row->shipper->email ?? '') : '';

            $load['Consignee']                          = $load_occupied ? ($row->receiver->name ?? '') : '';
            $load['Consignee Appointment Type']         = $load_occupied ? ($row->delivery_time_range == 0 ? 'Appointment' : 'Range') : '';
            $load['Consignee Date']                     = $load_occupied ? (isset($row->delivery_date_time) ? date('n/j/Y', strtotime($row->getRawOriginal('delivery_date_time'))) : '') : '';
            $load['Consignee Time']                     = $load_occupied ? (isset($row->delivery_date_time) ? date('H:i', strtotime($row->getRawOriginal('delivery_date_time'))) : '') : '';
            $load['Consignee Address']                  = $load_occupied ? ($row->delivery_address_one ?? '') : '';
            $load['Consignee Address 2']                = $load_occupied ? ($row->delivery_address_two ?? '') : '';
            $load['Consignee City']                     = $load_occupied ? ($row->delivery_city ?? '') : '';
            $load['Consignee State']                    = $load_occupied ? ($row->delivery_state ?? '') : '';
            $load['Consignee Zip Code']                 = $load_occupied ? ($row->delivery_zip ?? '') : '';
            $load['Consignee Phone']                    = $load_occupied ? ($row->receiver->phone ?? '') : '';
            $load['Consignee Contact']                  = $load_occupied ? ($row->receiver->dispatch_contact ?? '') : '';
            $load['Consignee Email']                    = $load_occupied ? ($row->receiver->email ?? '') : '';

            $load['Agreed Upon Rate']                   = $load_occupied ? (isset($row->agreed_upon_rate) ? formatNumberWithoutZeros($row->agreed_upon_rate, ['removeCommas' => true]) : '') : '';
            $load['Line Haul']                          = $load_occupied ? (isset($row->line_haul) ? formatNumberWithoutZeros($row->line_haul, ['removeCommas' => true]) : '') : '';
            $load['Fuel Surcharge']                     = $load_occupied ? (isset($row->fuel_surcharge) ? formatNumberWithoutZeros($row->fuel_surcharge, ['removeCommas' => true]) : '') : '';

            $load['Detention']                          = $load_occupied ? ($detention_due != 0 ? $detention_due : '') : '';
            $load['Other Charges']                      = $load_occupied ? ($other_charges != 0 ? $other_charges : '') : '';

            // Calculation
            $driver_pay_type                            = 0;
            if(isset($row->load_drivers) && $row->load_drivers->isNotEmpty()){
                $firstMatchingRecord                    = $row->load_drivers
                        ->where('driver_id', '=', $row->driver_id)
                        ->sortByDesc('created_at')
                        ->first();
                if(isset($firstMatchingRecord->pay_type) && $firstMatchingRecord->pay_type != ''){
                    $driver_pay_type                    = (float)$firstMatchingRecord->pay_type;
                }
            }
            $driver_pay_amount                          = 0;
            $driver_detention_due                       = 0;
            if(isset($row->detentions) && $row->detentions->isNotEmpty()){
                $load_detentions                        = $row->detentions()
                        ->where('driver_id', '=', $row->driver_id)
                        ->get();
                foreach ($load_detentions as $key => $load_detention) {
                    if(in_array($load_detention->detention_status,[2,3]) && $load_detention->review_status!='Denied'){
                        $driver_detention_due           += (float)$load_detention->detention_due;
                    }
                }
                $driver_pay_amount                      += $driver_detention_due;
            }
            $driver_other_charges                       = 0;
            if(isset($row->load_accessorials) && $row->load_accessorials->isNotEmpty()){
                $load_accessorials                      = $row->load_accessorials()
                        ->where('driver_liable', '=', 1)
                        ->where('driver_id', '=', $row->driver_id)
                        ->get();
                if($load_accessorials->isNotEmpty()){
                    $driver_accessorial_array           = [];
                    foreach ($load_accessorials as $key => $load_accessorial) {
                        $driver_accessorial_array[]     = [
                            'company_accessorial_id'    => $load_accessorial->company_accessorial_id,
                            'company_accessorial_name'  => $load_accessorial->company_accessorial->name ?? '',
                            'doc_required'              => $load_accessorial->doc_required,
                            'is_received'               => $load_accessorial->is_received,
                            'pay_deduct'                => $load_accessorial->pay_deduct,
                            'is_deduction'              => $load_accessorial->is_deduction,
                            'driver_qty'                => $load_accessorial->driver_qty,
                            'driver_rate'               => $load_accessorial->driver_rate,
                            'driver_total'              => $load_accessorial->driver_total
                        ];
                    }
                    $driver_accessorial_array           = array_reduce($driver_accessorial_array, function ($carry, $item) {
                        $key           = $item['company_accessorial_name'];
                        $carry[$key][] = $item;
                        return $carry;
                    }, []);
                    if(!empty($driver_accessorial_array)){
                        foreach ($driver_accessorial_array as $company_accessorial_name => $accessorials_array) {
                            $atotal                     = 0;
                            foreach ($accessorials_array as $key => $accessorial_array) {
                                if($accessorial_array['pay_deduct']==1){
                                    $atotal             += $accessorial_array['driver_total'];
                                }else if($accessorial_array['pay_deduct']==2){
                                    if($accessorial_array['doc_required']==0){
                                        $atotal         -= $accessorial_array['driver_total'];
                                    }else{
                                        if($accessorial_array['is_received']==0 || $accessorial_array['is_deduction']==0){
                                            $atotal     -= $accessorial_array['driver_total'];
                                        }
                                    }
                                    
                                }
                            }
                            if($atotal >= 0){
                                $driver_other_charges   += $atotal;
                                $driver_pay_amount      += $atotal;
                            }else{
                                $driver_other_charges   -= abs($atotal);
                                $driver_pay_amount      -= abs($atotal);
                            }
                        }
                    }
                }
            }
            // Calculation

            $load['Driver Pay Type']                    = $load_occupied ? ($driver_pay_type != 0 ? $driver_pay_type : '') : '';
            $load['Driver Pay Amount']                  = $load_occupied ? ($driver_pay_amount != 0 ? $driver_pay_amount : '') : '';

            // Calculation
            $co_driver_pay_type                         = 0;
            if(isset($row->load_drivers) && $row->load_drivers->isNotEmpty()){
                $firstMatchingRecord                    = $row->load_drivers
                        ->where('driver_id', '=', $row->co_driver_id)
                        ->sortByDesc('created_at')
                        ->first();
                if(isset($firstMatchingRecord->pay_type) && $firstMatchingRecord->pay_type != ''){
                    $co_driver_pay_type                 = (float)$firstMatchingRecord->pay_type;
                }
            }
            $co_driver_pay_amount                       = 0;
            $co_driver_detention_due                    = 0;
            if(isset($row->detentions) && $row->detentions->isNotEmpty()){
                $load_detentions                        = $row->detentions()
                        ->where('driver_id', '=', $row->co_driver_id)
                        ->get();
                foreach ($load_detentions as $key => $load_detention) {
                    if(in_array($load_detention->detention_status,[2,3]) && $load_detention->review_status!='Denied'){
                        $co_driver_detention_due        += (float)$load_detention->detention_due;
                    }
                }
                $co_driver_pay_amount                   += $co_driver_detention_due;
            }
            $co_driver_other_charges                    = 0;
            if(isset($row->load_accessorials) && $row->load_accessorials->isNotEmpty()){
                $load_accessorials                      = $row->load_accessorials()
                        ->where('driver_liable', '=', 1)
                        ->where('driver_id', '=', $row->co_driver_id)
                        ->get();
                if($load_accessorials->isNotEmpty()){
                    $co_driver_accessorial_array        = [];
                    foreach ($load_accessorials as $key => $load_accessorial) {
                        $co_driver_accessorial_array[]  = [
                            'company_accessorial_id'    => $load_accessorial->company_accessorial_id,
                            'company_accessorial_name'  => $load_accessorial->company_accessorial->name ?? '',
                            'doc_required'              => $load_accessorial->doc_required,
                            'is_received'               => $load_accessorial->is_received,
                            'pay_deduct'                => $load_accessorial->pay_deduct,
                            'is_deduction'              => $load_accessorial->is_deduction,
                            'driver_qty'                => $load_accessorial->driver_qty,
                            'driver_rate'               => $load_accessorial->driver_rate,
                            'driver_total'              => $load_accessorial->driver_total
                        ];
                    }
                    $co_driver_accessorial_array        = array_reduce($co_driver_accessorial_array, function ($carry, $item) {
                        $key           = $item['company_accessorial_name'];
                        $carry[$key][] = $item;
                        return $carry;
                    }, []);
                    if(!empty($co_driver_accessorial_array)){
                        foreach ($co_driver_accessorial_array as $company_accessorial_name => $accessorials_array) {
                            $atotal                     = 0;
                            foreach ($accessorials_array as $key => $accessorial_array) {
                                if($accessorial_array['pay_deduct']==1){
                                    $atotal             += $accessorial_array['driver_total'];
                                }else if($accessorial_array['pay_deduct']==2){
                                    if($accessorial_array['doc_required']==0){
                                        $atotal         -= $accessorial_array['driver_total'];
                                    }else{
                                        if($accessorial_array['is_received']==0 || $accessorial_array['is_deduction']==0){
                                            $atotal     -= $accessorial_array['driver_total'];
                                        }
                                    }
                                    
                                }
                            }
                            if($atotal >= 0){
                                $co_driver_other_charges+= $atotal;
                                $co_driver_pay_amount   += $atotal;
                            }else{
                                $co_driver_other_charges-= abs($atotal);
                                $co_driver_pay_amount   -= abs($atotal);
                            }
                        }
                    }
                }
            }
            // Calculation

            $load['Co Driver Pay Type']                 = $load_occupied ? ($co_driver_pay_type != 0 ? $co_driver_pay_type : '') : '';
            $load['Co Driver Pay Amount']               = $load_occupied ? ($co_driver_detention_due != 0 ? $co_driver_detention_due : '') : '';

            $paperwork_received                         = 'No';
            $paperwork_received_date_time               = '';

            $audit_complete                             = 'No';
            $audit_complete_date_time                   = '';

            $invoiced                                   = 'No';
            $invoiced_date_time                         = '';

            $paid_invoice                               = 'No';
            $paid_invoice_date_time                     = '';

            $days_to_pay                                = 0;
            $days_to_bill                               = 0;

            $billing_activity                           = $row->load_billing_activities ?? null;
            if(!is_null($billing_activity)){
                $paperwork_received                     = $billing_activity->paperwork_received == '1' ? 'Yes' : 'No';
                $paperwork_received_date_time           = $paperwork_received == 'Yes' ? (isset($billing_activity->paperwork_received_date_time) ? date('n/j/Y H:i', strtotime($billing_activity->paperwork_received_date_time)) : '') : '';

                $audit_complete                         = $billing_activity->audit_complete == '1' ? 'Yes' : 'No';
                $audit_complete_date_time               = $audit_complete == 'Yes' ? (isset($billing_activity->audit_complete_date_time) ? date('n/j/Y H:i', strtotime($billing_activity->audit_complete_date_time)) : '') : '';

                $invoiced                               = $billing_activity->invoiced == '1' ? 'Yes' : 'No';
                $invoiced_date_time                     = $invoiced == 'Yes' ? (isset($billing_activity->invoiced_date_time) ? date('n/j/Y H:i', strtotime($billing_activity->invoiced_date_time)) : '') : '';
                if($invoiced_date_time != ''){
                    $invoiceDateTime                    = \Carbon\Carbon::parse($billing_activity->invoiced_date_time);
                    $now                                = \Carbon\Carbon::now();
                    $days_to_bill                       = $invoiceDateTime->diffInDays($now) ?? 0;
                }

                $paid_invoice                           = $billing_activity->paid_invoice == '1' ? 'Yes' : 'No';
                $paid_invoice_date_time                 = $paid_invoice == 'Yes' ? (isset($billing_activity->paid_invoice_date_time) ? date('n/j/Y H:i', strtotime($billing_activity->paid_invoice_date_time)) : '') : '';
                if($paid_invoice_date_time != ''){
                    $paidInvoiceDateTime                = \Carbon\Carbon::parse($billing_activity->paid_invoice_date_time);
                    $now                                = \Carbon\Carbon::now();
                    $days_to_pay                        = $paidInvoiceDateTime->diffInDays($now) ?? 0;
                }
            }

            $amount_paid                                = 0;
            $pay_invoices                               = $row->load_billing_pay_invoices ?? null;
            if(!is_null($pay_invoices)){
                $paidInvoiceTotal                       = $pay_invoices->sum('total_payment') ?? 0;
                $chargebackAgainstBalance               = $pay_invoices->pluck('chargeback_against_balance');
                $chargebackAmountsData                  = $pay_invoices->pluck('chargeback_amount');
                $chargebackAmounts                      = array_sum($chargebackAmountsData->map(function ($values, $key) use ($chargebackAgainstBalance) {
                    if (count($values) > 0) {
                        $amounts                        = [];
                        foreach ($values as $k => $v) {
                            if (isset($chargebackAgainstBalance[$key][$k]) && $chargebackAgainstBalance[$key][$k] == '1') {
                                $amounts[]              = $v;
                            }
                        }
                        return array_sum($amounts);
                    }
                    return 0;
                })->toArray());

                $amount_paid                            = round(($paidInvoiceTotal + $chargebackAmounts), 2);
            }

            $load['Required Paperwork Received']        = $paperwork_received;

            $load['Time Date Stamp Paperwork Received'] = $paperwork_received_date_time;
            $load['Time Date Paperwork Audited']        = $audit_complete_date_time;
            $load['Time Date Invoice Paid']             = $paid_invoice_date_time;

            $load['Days to Pay']                        = $days_to_pay;
            $load['Days to Bill']                       = $days_to_bill;

            $load['Amount Paid']                        = $amount_paid;
            $load['Payment Type']                       = ''; // Query

            $load['Dwell Time Shipper']                 = ''; // Pending
            $load['Dwell Time Receiver']                = ''; // Pending

            return $load;
        };

        return $customMap($row);
    }

    public function headings(): array
    {
        return [
            'Load Number',
            'Status',
            'Billing Status',
            'Customer Name',
            'Customer MC',
            'Customer Phone',
            'Extension',
            'Customer Representative',
            'Customer Email',
            'Credit Score',
            'Carrier',
            'Terminal',
            'Team',
            'Dispatcher Name',
            'Driver Name',
            'Co Driver',
            'Truck Number',
            'Trailer Type - 1',
            'Trailer Number - 1',
            'Trailer Type - 2',
            'Trailer Number - 2',
            'Trailer Type - 3',
            'Trailer Number - 3',
            'Trailer Type - 4',
            'Trailer Number - 4',
            'Trailer Type - 5',
            'Trailer Number - 5',
            'Trailer Type - 6',
            'Trailer Number - 6',
            'Extra Stops',
            'Deadhead Miles',
            'Loaded Miles',
            'Total Miles',
            'Rate',
            'Pay',
            'Shipper',
            'Shipper Appointment Type',
            'Shipper Date',
            'Shipper Time',
            'Shipper Address',
            'Shipper Address 2',
            'Shipper City',
            'Shipper State',
            'Shipper Zip Code',
            'Shipper Phone',
            'Shipper Contact',
            'Shipper Email',
            'Consignee', 
            'Consignee Appointment Type', 
            'Consignee Date', 
            'Consignee Time', 
            'Consignee Address', 
            'Consignee Address 2', 
            'Consignee City', 
            'Consignee State', 
            'Consignee Zip Code', 
            'Consignee Phone', 
            'Consignee Contact', 
            'Consignee Email', 
            'Agreed Upon Rate', 
            'Line Haul', 
            'Fuel Surcharge', 
            'Detention', 
            'Other Charges', 
            'Driver Pay Type', 
            'Driver Pay Amount', 
            'Co Driver Pay Type', 
            'Co Driver Pay Amount', 
            'Required Paperwork Received', 
            'Time Date Stamp Paperwork Received', 
            'Time Date Paperwork Audited', 
            'Time Date Invoice Paid', 
            'Days to Pay', 
            'Days to Bill', 
            'Amount Paid', 
            'Payment Type', 
            'Dwell Time Shipper', 
            'Dwell Time Receiver'
        ];
    }

    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {
                $startColumnIndex   = 1;  // 1 corresponds to column 'A'
                $endColumnIndex     = 78;  // 78 corresponds to column 'BZ'

                for ($columnIndex   = $startColumnIndex; $columnIndex <= $endColumnIndex; $columnIndex++) {
                    $column         = Coordinate::stringFromColumnIndex($columnIndex);
                    $event->sheet->getColumnDimension($column)->setWidth(18);
                }

                // You can also set other styling options here if needed
                $event->sheet->getStyle("A1:BZ1")->applyFromArray([
                    'font' => [
                        'bold'  => true
                    ],
                    'borders' => [
                        'outline' => [
                            'borderStyle' => Border::BORDER_THIN,
                            'color' => ['rgb' => '000000'], // Black border color
                        ],
                    ],
                ]);
            },
        ];
    }
}
