<?php

namespace App\Exports;

use App\Models\LoadDetail;
use Illuminate\Http\Request;
use Maatwebsite\Excel\Concerns\FromQuery;
use Maatwebsite\Excel\Concerns\WithMapping;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;
use PhpOffice\PhpSpreadsheet\Style\Border;

class LoadsExportTest implements FromQuery, WithMapping, WithHeadings, WithEvents
{
    protected $request;

    public function __construct(Request $request)
    {
        $this->request  = $request;
    }

    public function query()
    {
        $query          = LoadDetail::query();
        if($this->request->filled('id')){
            $query->whereIn('id', explode(',', $this->request->id));
        }
        return $query;
    }

    public function map($row): array
    {
        // Use a closure for custom mapping
        $customMap      = function ($row) {
            $load                                       = [];
            $load['Load Number']                        = $row->load_number;

            $load_status                                = getLoadStatusList()[$row->load_status] ?? '';
            $is_on_hold                                 = $row->is_on_hold ?? 0;
            if($is_on_hold == '3'){
                $load_status                            = 'Dropped';
            }elseif($is_on_hold == '2'){
                $load_status                            = 'Cancel';
            }elseif($is_on_hold == '1'){
                $load_status                            = 'On Hold';
            }
            $load['Status']                             = $load_status;

            $load['Billing Status']                     = '';

            $load['Customer Name']                      = $row->broker->name ?? '';
            $load['Customer MC']                        = $row->broker->mc_number ?? '';
            $load['Customer Phone']                     = $row->phone ?? '';
            $load['Extension']                          = $row->ext ?? '';
            $load['Customer Representative']            = $row->rep ?? '';
            $load['Customer Email']                     = $row->broker_email ?? '';

            $load['Credit Score']                       = isset($row->is_cod) && $row->is_cod == '1' ? '' : $row->score_ratting ?? '';

            $load['Carrier']                            = $row->carrier->name ?? '';
            $load['Terminal']                           = $row->terminal->name ?? '';
            $load['Team']                               = $row->driver->team->name ?? '';

            $dispatcher_name                            = ''; // $row->driver->dispatcher->full_name ?? '';
            if(isset($row->self_dispatch) && $row->self_dispatch == 1){
                $dispatcher_name                        = 'Self-Dispatch';
            }else{
                $dispatcher_name                        = $row->dispatcher->full_name ?? '';
            }
            $load['Dispatcher Name']                    = $dispatcher_name;

            $load['Driver Name']                        = $row->driver->full_name ?? '';
            $load['Co Driver']                          = $row->co_driver->full_name ?? '';
            $load['Truck Number']                       = $row->truck_number ?? '';

            if(isset($row->loads->load_trailers) && $row->loads->load_trailers->isNotEmpty()){
                $trailer_number                         = '';
                if(isset($row->trailer_number)){
                    $trailer_number                     = $row->trailer_number;
                }
                $trailer_type                           = 'Not specified';
                if (isset($row->trailer_type) && $row->trailer_type != '' && isset(getTrailerTypeList()[$row->trailer_type])) {
                    $trailer_type                       = getTrailerTypeList()[$row->trailer_type];
                }

                $load['Trailer Type - 1']               = $trailer_type;
                $load['Trailer Number - 1']             = $trailer_number;

                $tn                                     = 2;
                foreach ($row->loads->load_trailers as $key => $load_trailer) {
                    // Break the loop when $tn is 7
                    if ($tn === 7) {
                        break;
                    }

                    $trailer_number                     = '';
                    if(isset($load_trailer->trailer_number)){
                        $trailer_number                 = $load_trailer->trailer_number;
                    }
                    $trailer_type                       = 'Not specified';
                    if (isset($load_trailer->trailer_type) && $load_trailer->trailer_type != '' && isset(getTrailerTypeList()[$load_trailer->trailer_type])) {
                        $trailer_type                   = getTrailerTypeList()[$load_trailer->trailer_type];
                    }

                    $load['Trailer Type - '.$tn]        = $trailer_type;
                    $load['Trailer Number - '.$tn]      = $trailer_number;

                    ++$tn;
                }

                for ($ti = $tn; $ti <= 6; ++$ti) {
                    $load['Trailer Type - '.$ti]        = '';
                    $load['Trailer Number - '.$ti]      = '';
                }
            }else{
                $trailer_number                         = '';
                if(isset($row->trailer_number)){
                    $trailer_number                     = $row->trailer_number;
                }
                $trailer_type                           = 'Not specified';
                if (isset($row->trailer_type) && $row->trailer_type != '' && isset(getTrailerTypeList()[$row->trailer_type])) {
                    $trailer_type                       = getTrailerTypeList()[$row->trailer_type];
                }

                $load['Trailer Type - 1']               = $trailer_type;
                $load['Trailer Number - 1']             = $trailer_number;

                $load['Trailer Type - 2']               = '';
                $load['Trailer Number - 2']             = '';
                $load['Trailer Type - 3']               = '';
                $load['Trailer Number - 3']             = '';
                $load['Trailer Type - 4']               = '';
                $load['Trailer Number - 4']             = '';
                $load['Trailer Type - 5']               = '';
                $load['Trailer Number - 5']             = '';
                $load['Trailer Type - 6']               = '';
                $load['Trailer Number - 6']             = '';
            }

            $total_stops                                = 0;
            if(isset($row->loads->load_stops) && $row->loads->load_stops->isNotEmpty()){
                $total_stops                            += $row->loads->load_stops->count();
            }

            $load['Extra Stops']                        = $total_stops;

            $load['Deadhead Miles']                     = isset($row->empty_miles) ? formatNumberWithoutZeros($row->empty_miles, ['removeCommas' => true]) : '';
            $load['Loaded Miles']                       = isset($row->loaded_miles) ? formatNumberWithoutZeros($row->loaded_miles, ['removeCommas' => true]) : '';
            $load['Total Miles']                        = isset($row->total_miles) ? formatNumberWithoutZeros($row->total_miles, ['removeCommas' => true]) : '';

            $load['Rate']                               = '';
            $load['Pay']                                = '';

            $load['Shipper']                            = $row->shipper->name ?? '';
            $load['Shipper Appointment Type']           = $row->shipping_time_range == 0 ? 'Appointment' : 'Range';
            $load['Shipper Date']                       = ''; // Query
            $load['Shipper Time']                       = ''; // Query
            $load['Shipper Address']                    = $row->shipper_address_one ?? '';
            $load['Shipper Address 2']                  = $row->shipper_address_two ?? '';
            $load['Shipper City']                       = $row->shipper_city ?? '';
            $load['Shipper State']                      = $row->shipper_state ?? '';
            $load['Shipper Zip Code']                   = $row->shipper_zip ?? '';
            $load['Shipper Phone']                      = $row->shipper->phone ?? '';
            $load['Shipper Contact']                    = $row->shipper->dispatch_contact ?? '';
            $load['Shipper Email']                      = $row->shipper->email ?? '';

            $load['Consignee']                          = $row->receiver->name ?? '';
            $load['Consignee Appointment Type']         = $row->delivery_time_range == 0 ? 'Appointment' : 'Range';
            $load['Consignee Date']                     = ''; // Query
            $load['Consignee Time']                     = ''; // Query
            $load['Consignee Address']                  = $row->delivery_address_one ?? '';
            $load['Consignee Address 2']                = $row->delivery_address_two ?? '';
            $load['Consignee City']                     = $row->delivery_city ?? '';
            $load['Consignee State']                    = $row->delivery_state ?? '';
            $load['Consignee Zip Code']                 = $row->delivery_zip ?? '';
            $load['Consignee Phone']                    = $row->receiver->phone ?? '';
            $load['Consignee Contact']                  = $row->receiver->dispatch_contact ?? '';
            $load['Consignee Email']                    = $row->receiver->email ?? '';

            $load['Agreed Upon Rate']                   = isset($row->agreed_upon_rate) ? formatNumberWithoutZeros($row->agreed_upon_rate, ['removeCommas' => true]) : '';
            $load['Line Haul']                          = isset($row->line_haul) ? formatNumberWithoutZeros($row->line_haul, ['removeCommas' => true]) : '';
            $load['Fuel Surcharge']                     = isset($row->fuel_surcharge) ? formatNumberWithoutZeros($row->fuel_surcharge, ['removeCommas' => true]) : '';

            $load['Detention']                          = ''; // Query
            $load['Other Charges']                      = ''; // Query

            $load['Driver Pay Type']                    = ''; // Query
            $load['Driver Pay Amount']                  = ''; // Query

            $load['Co Driver Pay Type']                 = ''; // Query
            $load['Co Driver Pay Amount']               = ''; // Query

            $load['Required Paperwork Received']        = ''; // Query

            $load['Time Date Stamp Paperwork Received'] = ''; // Query
            $load['Time Date Paperwork Audited']        = ''; // Query
            $load['Time Date Invoice Paid']             = ''; // Query

            $load['Days to Pay']                        = ''; // Query
            $load['Days to Bill']                       = ''; // Query

            $load['Amount Paid']                        = ''; // Query
            $load['Payment Type']                       = ''; // Query

            $load['Dwell Time Shipper']                 = '';
            $load['Dwell Time Receiver']                = '';

            return $load;
        };

        return $customMap($row);
    }

    public function headings(): array
    {
        return [
            'Load Number',
            'Status',
            'Billing Status',
            'Customer Name',
            'Customer MC',
            'Customer Phone',
            'Extension',
            'Customer Representative',
            'Customer Email',
            'Credit Score',
            'Carrier',
            'Terminal',
            'Team',
            'Dispatcher Name',
            'Driver Name',
            'Co Driver',
            'Truck Number',
            'Trailer Type - 1',
            'Trailer Number - 1',
            'Trailer Type - 2',
            'Trailer Number - 2',
            'Trailer Type - 3',
            'Trailer Number - 3',
            'Trailer Type - 4',
            'Trailer Number - 4',
            'Trailer Type - 5',
            'Trailer Number - 5',
            'Trailer Type - 6',
            'Trailer Number - 6',
            'Extra Stops',
            'Deadhead Miles',
            'Loaded Miles',
            'Total Miles',
            'Rate',
            'Pay',
            'Shipper',
            'Shipper Appointment Type',
            'Shipper Date',
            'Shipper Time',
            'Shipper Address',
            'Shipper Address 2',
            'Shipper City',
            'Shipper State',
            'Shipper Zip Code',
            'Shipper Phone',
            'Shipper Contact',
            'Shipper Email',
            'Consignee', 
            'Consignee Appointment Type', 
            'Consignee Date', 
            'Consignee Time', 
            'Consignee Address', 
            'Consignee Address 2', 
            'Consignee City', 
            'Consignee State', 
            'Consignee Zip Code', 
            'Consignee Phone', 
            'Consignee Contact', 
            'Consignee Email', 
            'Agreed Upon Rate', 
            'Line Haul', 
            'Fuel Surcharge', 
            'Detention', 
            'Other Charges', 
            'Driver Pay Type', 
            'Driver Pay Amount', 
            'Co Driver Pay Type', 
            'Co Driver Pay Amount', 
            'Required Paperwork Received', 
            'Time Date Stamp Paperwork Received', 
            'Time Date Paperwork Audited', 
            'Time Date Invoice Paid', 
            'Days to Pay', 
            'Days to Bill', 
            'Amount Paid', 
            'Payment Type', 
            'Dwell Time Shipper', 
            'Dwell Time Receiver'
        ];
    }

    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {
                $startColumnIndex   = 1;  // 1 corresponds to column 'A'
                $endColumnIndex     = 78;  // 78 corresponds to column 'BZ'

                for ($columnIndex   = $startColumnIndex; $columnIndex <= $endColumnIndex; $columnIndex++) {
                    $column         = Coordinate::stringFromColumnIndex($columnIndex);
                    $event->sheet->getColumnDimension($column)->setWidth(18);
                }

                // You can also set other styling options here if needed
                $event->sheet->getStyle("A1:BZ1")->applyFromArray([
                    'font' => [
                        'bold'  => true
                    ],
                    'borders' => [
                        'outline' => [
                            'borderStyle' => Border::BORDER_THIN,
                            'color' => ['rgb' => '000000'], // Black border color
                        ],
                    ],
                ]);
            },
        ];
    }
}
