<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Admin\AdminController;
use Illuminate\Http\Request;
use Illuminate\Support\Str;

use App\Http\Requests\Admin\AccountStoreRequest;
use App\Http\Requests\Admin\AccountUpdateRequest;

use App\Models\Admin;

use App\Mail\Admin\WelcomeMailToAdmin;

use Mail;
use Hash;
use DataTables;

class AccountController extends AdminController{

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $totalAdmins    = Admin::admin()->count();
        $totalSubAdmins = Admin::subAdmin()->count();
        $totalSupports  = Admin::support()->count();
        if ($request->ajax()) {
            $data       = Admin::select('*');
            return Datatables::of($data)
                ->addIndexColumn()
                ->editColumn('type', function($row){
                    if ($row->type == 0) {
                        $type = '<h5><label class="badge badge-success">Admin</label></h5>';
                    } elseif ($row->type == 1) {
                        $type = '<h5><label class="badge badge-warning">Sub Admin</label></h5>';
                    } elseif ($row->type == 2) {
                        $type = '<h5><label class="badge badge-info">Support</label></h5>';
                    }
                    return $type;
                })
                ->filterColumn('type', function ($query, $keyword) {
                    if (Str::contains('admin' , Str::lower($keyword))) {
                        $query = $query->where('type', 0);
                    }
                    if (Str::contains('sub admin' , Str::lower($keyword))) {
                        $query = $query->orWhere('type', 1);
                    }
                    if (Str::contains('support' , Str::lower($keyword))) {
                        $query = $query->orWhere('type', 2);
                    }
                    return $query;
                })
                ->addColumn('action', function($row){
                    $btn = '<a href="'.route('admin.accounts.show', $row->id).'"><img src="'.asset('theme/img/big-eye.png').'" alt=""></a>';
                    $btn = $btn.' <a href="'.route('admin.accounts.edit', $row->id).'"><img src="'.asset('theme/img/big-pen.png').'" alt=""></a>';
                    return $btn;
                })
                ->rawColumns(['action', 'type'])
                ->make(true);
        }
        return view('admin.account.index', compact('totalAdmins', 'totalSubAdmins', 'totalSupports'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return view('admin.account.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(AccountStoreRequest $request){
        $input                  = $request->all();

        $password               = Str::random(8);
        $input['password']      = Hash::make($password);
        Admin::create($input);

        $mailData               = [];
        $mailData['name']       = $input['first_name'].' '.$input['last_name'];
        $mailData['email']      = $input['email'];
        $mailData['password']   = $password;
        Mail::to($input['email'])->send(new WelcomeMailToAdmin($mailData));

        notification('success', $this->crudMessage('create', 'Account'));

        return redirect()->route('admin.accounts.index');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Admin $account){
        return view('admin.account.show', compact('account'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Admin $account){
        return view('admin.account.edit', compact('account'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(AccountUpdateRequest $request, Admin $account){
        $input = $request->all();

        $account->update($input);

        notification('success', $this->crudMessage('update', 'Account'));

        return redirect()->route('admin.accounts.index');
    }
}
