<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Admin\AdminController;

use App\Models\AnnouncementClearByUser;
use Illuminate\Http\Request;
use Illuminate\Support\Str;

use App\Models\Driver;
use App\Models\CompanyAdmin;

use App\Models\Announcement;
use App\Http\Requests\Admin\AnnouncementStoreRequest;
use App\Http\Requests\Admin\AnnouncementUpdateRequest;

use DB;
use DataTables;
use Carbon\Carbon;

class AnnouncementController extends AdminController{

    /**
     * Create a new controller instance.
     *
     * @return void
     */

    public $timeZone;
    public $timeZoneId;
    public $offset;

    public function __construct(){
        parent::__construct();

        $this->middleware(function ($request, $next) {
            $this->timeZone     = getTimeZone();
            $this->timeZoneId   = getTimeZoneId();
            $this->offset       = getTimeZoneOffset();

            return $next($request);
        });
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        if ($request->ajax()) {
            $offset     = $this->offset;
            $search     = $request->search['value'] ?? '';
            $data       = Announcement::query()
                ->select(
                    '*',
                    DB::raw('CONVERT_TZ(announcements.created_at,"+00:00","'.$offset.'") as created_at')
                )
                ->where('module','Compel')
                ->where('created_by',auth()->guard('admin')->user()->id);
            return Datatables::of($data)
                    ->addIndexColumn()
                    ->editColumn('description', function ($data) {
                        return Str::limit(strip_tags($data->description), 50);
                    })
                    ->editColumn('created_at', function($data){
                        if(auth()->guard('admin')->user()->time_format=='12'){
                            return !is_null($data->created_at) ? Carbon::parse($data->created_at)->format(config('settings.date_time_format.12_hour','m/d/Y h:i A'))." ".$this->timeZoneId : '';
                        }else{
                            return !is_null($data->created_at) ? Carbon::parse($data->created_at)->format(config('settings.date_time_format.24_hour','m/d/Y H:i'))." ".$this->timeZoneId : '';
                        }
                    })
                    ->addColumn('action', function($data){
                        $action     = '';
                        $action     .= '<a href="'.route('admin.announcement.show', $data->id).'"><img src="'.asset('theme/img/eye.png').'" alt=""></a>&nbsp;&nbsp;&nbsp;';
                        $action     .= '<a href="'.route('admin.announcement.edit', $data->id).'"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>&nbsp;&nbsp;&nbsp;';
                        $action     .= '<a href="javascript:void(0);" class="destroy" data-action="'.route('admin.announcement.destroy',$data->id).'" data-id="'.$data->id.'"><img src="'.asset('theme/img/delete-icon.png').'" alt=""></a>';
                        return $action;
                    })
                    ->filterColumn('created_at', function ($query, $keyword) use ($search, $offset) {
                        if(auth()->guard('admin')->user()->time_format=='12'){
                            $query->whereRaw("CONCAT(DATE_FORMAT(CONVERT_TZ(announcements.created_at,'+00:00','".$offset."'),'".config('settings.database_date_time_format.12_hour','%m/%d/%Y %h:%i %A')."'),' ','".$this->timeZoneId."') like ?", ["%$search%"]);
                        }else{
                            $query->whereRaw("CONCAT(DATE_FORMAT(CONVERT_TZ(announcements.created_at,'+00:00','".$offset."'),'".config('settings.database_date_time_format.24_hour','%m/%d/%Y %H:%i')."'),' ','".$this->timeZoneId."') like ?", ["%$search%"]);
                        }
                    })
                    ->rawColumns(['action'])
                    ->make(true);
        }
        return view('admin.announcements.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        $getCompanyList = getCompanyList();
        return view('admin.announcements.create',compact('getCompanyList'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(AnnouncementStoreRequest $request){
        $input                          = $request->all();

        $company_id                     = null;
        if (isset($input['show_company']) && !empty($input['company_id'])){
            $company_id                 = implode(',', $input['company_id']);
        }

        $showArry                       = [];
        if (isset($input['show_manager'])){
            $showArry[]                 = $input['show_manager'];
        }
        if (isset($input['show_dispatcher'])){
            $showArry[]                 = $input['show_dispatcher'];
        }
        if (isset($input['show_driver'])){
            $showArry[]                 = $input['show_driver'];
        }
        $show                           = null;
        if(!empty($showArry)){
            $show                       = implode(',', $showArry);
        }

        $notifyArry                     = [];
        $notifyUsers                    = [];
        if (isset($input['notify_manager'])){
            $notifyArry[]               = $input['notify_manager'];

            if (isset($input['show_company']) && !empty($input['company_id'])){
                $notifyManagers         = CompanyAdmin::manager()->whereIn('company_id',$input['company_id'])->get();
            }else{
               $notifyManagers          = CompanyAdmin::manager()->get();
            }
            $notifyUsers['manager']     = $notifyManagers;
        }
        if (isset($input['notify_dispatcher'])){
            $notifyArry[]               = $input['notify_dispatcher'];

            if (isset($input['show_company']) && !empty($input['company_id'])){
                $notifyDispatchers      = CompanyAdmin::dispatcher()->whereIn('company_id',$input['company_id'])->get();
            }else{
               $notifyDispatchers       = CompanyAdmin::dispatcher()->get();
            }
            $notifyUsers['dispatcher']  = $notifyDispatchers;
        }
        if (isset($input['notify_driver'])){
            $notifyArry[]               = $input['notify_driver'];

            if (isset($input['show_company']) && !empty($input['company_id'])){
                $notifyDrivers          = Driver::whereIn('company_id',$input['company_id'])->get();
            }else{
               $notifyDrivers           = Driver::all();
            }
            $notifyUsers['driver']      = $notifyDrivers;
        }
        $notify                         = null;
        if(!empty($notifyArry)){
            $notify                     = implode(',', $notifyArry);
        }

        $announcement                   = Announcement::create([
            'name'                      => $input['name'],
            'description'               => $input['description'],
            'company_id'                => $company_id,
            'show_up'                   => $show,
            'notify'                    => $notify,
            'module'                    => 'Compel',
            'show_on_top'               => $input['show_on_top'],
            'bg_color'                  => $input['bg_color'],
            'sticky'                    => $input['sticky'],
            'start_view'                => $input['start_view'],
            'end_view'                  => ($input['sticky'] == '1' && $input['end_view'] != '') ? $input['end_view'] : null,
            'created_at'                => Carbon::now(),
            'created_by'                => auth()->guard('admin')->user()->id
        ]);

        if($announcement){
            if (!empty($notifyUsers)) {
                foreach ($notifyUsers as $key => $value) {
                    //7 = dispatch-pro-announcement
                    if($key=='manager'){
                        $users              = $notifyUsers[$key];
                        if($users->isNotEmpty()){
                            foreach ($users as $key => $user) {
                                createNotification(
                                    9, // source
                                    $announcement->name, // name
                                    $user->id, // user_id
                                    $user->type, // user_type
                                    1, // type
                                    $announcement->id, // action_id
                                    $announcement->description, // description
                                    'Notification', // module
                                    [], // properties
                                    $user->company_id // company_id
                                );
                                $checkNotify = isUnreadDelay($user->id, $user->type, 9);
                                if(isset($checkNotify)){
                                    if(isset($checkNotify->email) && $checkNotify->email==1 && isset($user->email)) {
                                        $details                    = [];
                                        $details['company_id']      = $user->company_id;
                                        $details['email']           = $user->email;
                                        $details['name']            = $user->full_name;
                                        $details['announcement']    = $announcement;
                                        //dispatch(new App\Jobs\AnnouncementEmailJob($details));
                                        \Mail::to($user->email)->send(new \App\Mail\AnnouncementEmail($details));
                                    }
                                    if (isset($checkNotify->sms) && $checkNotify->sms==1 && isset($user->cell_phone)) {
                                        // Twilio's credentials are fetched from env File
                                       sendSMS($user->cell_phone,$announcement->name);
                                    }
                                }
                            }
                        }
                    }elseif ($key=='dispatcher') {
                        $users              = $notifyUsers[$key];
                        if($users->isNotEmpty()){
                            foreach ($users as $key => $user) {
                                createNotification(
                                    9, // source
                                    $announcement->name, // name
                                    $user->id, // user_id
                                    $user->type, // user_type
                                    1, // type
                                    $announcement->id, // action_id
                                    $announcement->description, // description
                                    'Notification', // module
                                    [], // properties
                                    $user->company_id // company_id
                                );
                                $checkNotify = isUnreadDelay($user->id, $user->type, 9);
                                if(isset($checkNotify)){
                                    if(isset($checkNotify->email) && $checkNotify->email==1 && isset($user->email)) {
                                        $details                    = [];
                                        $details['company_id']      = $user->company_id;
                                        $details['email']           = $user->email;
                                        $details['name']            = $user->full_name;
                                        $details['announcement']    = $announcement;
                                        //dispatch(new App\Jobs\AnnouncementEmailJob($details));
                                        \Mail::to($user->email)->send(new \App\Mail\AnnouncementEmail($details));
                                    }
                                    if (isset($checkNotify->sms) && $checkNotify->sms==1 && isset($user->cell_phone)) {
                                        // Twilio's credentials are fetched from env File
                                        sendSMS($user->cell_phone,$announcement->name);
                                    }
                                }
                            }
                        }
                    }elseif ($key=='driver') {
                        $users              = $notifyUsers[$key];
                        if($users->isNotEmpty()){
                            foreach ($users as $key => $user) {
                                createNotification(
                                    9, // source
                                    $announcement->name, // name
                                    $user->id, // user_id
                                    3, // user_type
                                    1, // type
                                    $announcement->id, // action_id
                                    $announcement->description, // description
                                    'Notification', // module
                                    [], // properties
                                    $user->company_id // company_id
                                );
                                $checkNotify = isUnreadDelay($user->id, 3, 9);
                                if(isset($checkNotify)){
                                    if(isset($checkNotify->email) && $checkNotify->email==1 && isset($user->email)) {
                                        $details                    = [];
                                        $details['company_id']      = $user->company_id;
                                        $details['email']           = $user->email;
                                        $details['name']            = $user->full_name;
                                        $details['announcement']    = $announcement;
                                        //dispatch(new App\Jobs\AnnouncementEmailJob($details));
                                        \Mail::to($user->email)->send(new \App\Mail\AnnouncementEmail($details));
                                    }
                                    if (isset($checkNotify->sms) && $checkNotify->sms==1 && isset($user->cell_phone)) {
                                        // Twilio's credentials are fetched from env File
                                        sendSMS($user->cell_phone,$announcement->name);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        notification('success', $this->crudMessage('create', 'Announcement'));

        return redirect()->route('admin.announcement.index');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Announcement $announcement){
        $getCompanyList = getCompanyList();
        $getTimeZone    = $this->timeZone;
        $getTimeZoneId  = $this->timeZoneId;
        return view('admin.announcements.show', compact('announcement', 'getCompanyList', 'getTimeZone', 'getTimeZoneId'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Announcement $announcement){
        $getCompanyList = getCompanyList();
        return view('admin.announcements.edit', compact('announcement', 'getCompanyList'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(AnnouncementUpdateRequest $request, Announcement $announcement){
        $input                          = $request->all();

        $company_id                     = null;
        if (isset($input['show_company']) && !empty($input['company_id'])){
            $company_id                 = implode(',', $input['company_id']);
        }

        $showArry                       = [];
        if (isset($input['show_manager'])){
            $showArry[]                 = $input['show_manager'];
        }
        if (isset($input['show_dispatcher'])){
            $showArry[]                 = $input['show_dispatcher'];
        }
        if (isset($input['show_driver'])){
            $showArry[]                 = $input['show_driver'];
        }
        $show                           = null;
        if(!empty($showArry)){
            $show                       = implode(',', $showArry);
        }

        $notifyArry                     = [];
        $notifyUsers                    = [];
        if (isset($input['notify_manager'])){
            $notifyArry[]               = $input['notify_manager'];

            if (isset($input['show_company']) && !empty($input['company_id'])){
                $notifyManagers         = CompanyAdmin::manager()->whereIn('company_id',$input['company_id'])->get();
            }else{
               $notifyManagers          = CompanyAdmin::manager()->get();
            }
            $notifyUsers['manager']     = $notifyManagers;
        }
        if (isset($input['notify_dispatcher'])){
            $notifyArry[]               = $input['notify_dispatcher'];

            if (isset($input['show_company']) && !empty($input['company_id'])){
                $notifyDispatchers      = CompanyAdmin::dispatcher()->whereIn('company_id',$input['company_id'])->get();
            }else{
               $notifyDispatchers       = CompanyAdmin::dispatcher()->get();
            }
            $notifyUsers['dispatcher']  = $notifyDispatchers;
        }
        if (isset($input['notify_driver'])){
            $notifyArry[]               = $input['notify_driver'];

            if (isset($input['show_company']) && !empty($input['company_id'])){
                $notifyDrivers          = Driver::whereIn('company_id',$input['company_id'])->get();
            }else{
               $notifyDrivers           = Driver::all();
            }
            $notifyUsers['driver']      = $notifyDrivers;
        }
        $notify                         = null;
        if(!empty($notifyArry)){
            $notify                     = implode(',', $notifyArry);
        }

        $announcement->name             = $input['name'];
        $announcement->description      = $input['description'];
        $announcement->company_id       = $company_id;
        $announcement->show_up          = $show;
        $announcement->notify           = $notify;
        $announcement->module           = 'Compel';
        $announcement->show_on_top      = $input['show_on_top'];
        $announcement->bg_color         = $input['bg_color'];
        $announcement->sticky           = $input['sticky'];
        $announcement->start_view       = $input['start_view'];
        $announcement->end_view         = ($input['sticky'] == '1' && $input['end_view'] != '') ? $input['end_view'] : null;
        $announcement->updated_at       = Carbon::now();
        $announcement->updated_by       = auth()->guard('admin')->user()->id;
        if($announcement->save()){
            if(count($announcement->getChanges()) > 1){
                AnnouncementClearByUser::where('announcement_id', $announcement->id)->delete();
            }

            if (!empty($notifyUsers)) {
                foreach ($notifyUsers as $key => $value) {
                    if($key=='manager'){
                        $users              = $notifyUsers[$key];
                        if($users->isNotEmpty()){
                            foreach ($users as $key => $user) {
                                createNotification(
                                    9, // source
                                    $announcement->name, // name
                                    $user->id, // user_id
                                    $user->type, // user_type
                                    1, // type
                                    $announcement->id, // action_id
                                    $announcement->description, // description
                                    'Notification', // module
                                    [], // properties
                                    $user->company_id // company_id
                                );
                                $checkNotify = isUnreadDelay($user->id, $user->type, 9);
                                if(isset($checkNotify)){
                                    if(isset($checkNotify->email) && $checkNotify->email==1 && isset($user->email)) {
                                        $details                    = [];
                                        $details['company_id']      = $user->company_id;
                                        $details['email']           = $user->email;
                                        $details['name']            = $user->full_name;
                                        $details['announcement']    = $announcement;
                                        //dispatch(new App\Jobs\AnnouncementEmailJob($details));
                                        \Mail::to($user->email)->send(new \App\Mail\AnnouncementEmail($details));
                                    }
                                    if (isset($checkNotify->sms) && $checkNotify->sms==1 && isset($user->cell_phone)) {
                                        // Twilio's credentials are fetched from env File
                                        sendSMS($user->cell_phone,$announcement->name);
                                    }
                                }
                            }
                        }
                    }elseif ($key=='dispatcher') {
                        $users              = $notifyUsers[$key];
                        if($users->isNotEmpty()){
                            foreach ($users as $key => $user) {
                                createNotification(
                                    9, // source
                                    $announcement->name, // name
                                    $user->id, // user_id
                                    $user->type, // user_type
                                    1, // type
                                    $announcement->id, // action_id
                                    $announcement->description, // description
                                    'Notification', // module
                                    [], // properties
                                    $user->company_id // company_id
                                );
                                $checkNotify = isUnreadDelay($user->id, $user->type, 9);
                                if(isset($checkNotify)){
                                    if(isset($checkNotify->email) && $checkNotify->email==1 && isset($user->email)) {
                                        $details                    = [];
                                        $details['company_id']      = $user->company_id;
                                        $details['email']           = $user->email;
                                        $details['name']            = $user->full_name;
                                        $details['announcement']    = $announcement;
                                        //dispatch(new App\Jobs\AnnouncementEmailJob($details));
                                        \Mail::to($user->email)->send(new \App\Mail\AnnouncementEmail($details));
                                    }
                                    if (isset($checkNotify->sms) && $checkNotify->sms==1 && isset($user->cell_phone)) {
                                        // Twilio's credentials are fetched from env File
                                        sendSMS($user->cell_phone,$announcement->name);
                                    }
                                }
                            }
                        }
                    }elseif ($key=='driver') {
                        $users              = $notifyUsers[$key];
                        if($users->isNotEmpty()){
                            foreach ($users as $key => $user) {
                                createNotification(
                                    9, // source
                                    $announcement->name, // name
                                    $user->id, // user_id
                                    3, // user_type
                                    1, // type
                                    $announcement->id, // action_id
                                    $announcement->description, // description
                                    'Notification', // module
                                    [], // properties
                                    $user->company_id // company_id
                                );
                                $checkNotify = isUnreadDelay($user->id, 3, 9);
                                if(isset($checkNotify)){
                                    if(isset($checkNotify->email) && $checkNotify->email==1 && isset($user->email)) {
                                        $details                    = [];
                                        $details['company_id']      = $user->company_id;
                                        $details['email']           = $user->email;
                                        $details['name']            = $user->full_name;
                                        $details['announcement']    = $announcement;
                                        //dispatch(new App\Jobs\AnnouncementEmailJob($details));
                                        \Mail::to($user->email)->send(new \App\Mail\AnnouncementEmail($details));
                                    }
                                    if (isset($checkNotify->sms) && $checkNotify->sms==1 && isset($user->cell_phone)) {
                                        // Twilio's credentials are fetched from env File
                                        sendSMS($user->cell_phone,$announcement->name);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        notification('success', $this->crudMessage('update', 'Announcement'));

        return redirect()->route('admin.announcement.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Announcement $announcement){
        AnnouncementClearByUser::where('announcement_id', $announcement->id)->delete();

        $announcement->delete();
        notification('success', $this->crudMessage('delete', 'Announcement'));
        return redirect()->route('admin.announcement.index');
    }
}
