<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Admin\AdminController;
use Illuminate\Http\Request;

use App\Http\Requests\Admin\AppCenterStoreRequest;
use App\Http\Requests\Admin\AppCenterUpdateRequest;

use App\Models\AppCenter;

use File;
use DataTables;

class AppCenterController extends AdminController{

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        if ($request->ajax()) {
            $data = AppCenter::select('*');
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('logo', function($row){
                    $html       = '<img src="'.asset('storage/companyAdmin/app_logo/').'/'.$row->logo.'" width="80" />';
                    return $html;
                })
                ->addColumn('status', function($row){
                    if($row->status == '1'){
                        $html   = '<span class="text-green">Active</span>';
                    }else{
                        $html   = '<span class="text-red">Inactive</span>';
                    }
                    return $html;
                })
                ->addColumn('action', function($row){
                    $btn        = ' <a href="'.route('admin.app-center.edit', $row->id).'" class="mr-4"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';
                    //$btn      = $btn.' <a href="javascript:" class="destroy" data-action="'.route('admin.app-center.destroy',$row->id).'" data-id="'.$row->id.'"><img src="'.asset('theme/img/delete-icon.png').'" alt=""></a>';
                    return $btn;
                })
                ->rawColumns(['action','logo','status'])
                ->make(true);
        }
        return view('admin.appCenter.index');

    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        notification('error', 'Access permission denied.');
        return redirect()->route('admin.app-center.index');
        //return view('admin.appCenter.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(AppCenterStoreRequest $request)
    {
        notification('error', 'Access permission denied.');
        return redirect()->route('admin.app-center.index');
        /*
        $input = $request->all();
        if($request->profile_picture_hidden){
            $input['logo'] = $request->profile_picture_hidden;
        }
        if($request->status == 1){
            $input['status'] = '1';
        }else{
            $input['status'] = '0';
        }

        AppCenter::create($input);
        notification('success', $this->crudMessage('create', 'AppCenter'));
        return redirect()->route('admin.app-center.index');
        */
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(AppCenter $app_center){
       return redirect()->route('admin.app-center.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(AppCenter $app_center){
        return view('admin.appCenter.edit', compact('app_center'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(AppCenterUpdateRequest $request, AppCenter $app_center){
        $input                  = $request->all();

        if($request->profile_picture_hidden){
            $input['logo']      = $request->profile_picture_hidden;
        }

        if($request->status == 1){
            $input['status']    = '1';
        }else{
            $input['status']    = '0';
        }

        $app_center->update($input);
        notification('success', $this->crudMessage('update', 'app_center'));
        return redirect()->route('admin.app-center.index');
    }

    public function destroy(AppCenter $app_center){
        notification('error', 'Access permission denied.');
        return redirect()->route('admin.app-center.index');
        /*
        $app_center->delete();

        notification('success', $this->crudMessage('delete', 'app_center'));

        return redirect()->route('admin.app-center.index');
        */
    }


    public function cropPicture(Request $request){
        $path           = "storage/companyAdmin/app_logo/";
        if(File::ensureDirectoryExists($path)){
            File::makeDirectory($path);
        }
        $data           = $request->image;
        $image_array_1  = explode(";", $data);
        $image_array_2  = explode(",", $image_array_1[1]);
        $data           = base64_decode($image_array_2[1]);
        $fileName       = "logo_".time().".". 'png';
        $imageName      = $path.$fileName;
        file_put_contents($imageName, $data);
        echo $fileName;
   }

}
