<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Admin\AdminController;
use App\Models\ChartOfAccount;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Yajra\DataTables\Facades\DataTables;

class ChartOfAccountController extends AdminController
{
    private $parents = array(); // This variabel use for findParents() function

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    function findParents($input, $id) {
        if(is_array($input)) {
            foreach($input as $k => $val) {
                if($val['id'] == $id  && $val['parent_account'] != null) {
                    array_push($this->parents, $val['parent_account']);
                    $this->findParents($input, $val['parent_account']);
                }
            }
        }
    }

    public function findChildrens($elements, $parentId, $parentFound = false, $cats = array())
    {
        foreach($elements as $element) {
            if(!is_null($element['parent_account'])){
                if((!$parentFound && $element['id'] == $parentId) || $element['parent_account'] == $parentId) {
                    $rowData = array();
                    foreach($element as $k => $v) {
                        $rowData[$k] = $v;
                    }
                    $cats[] = $rowData;
                    if($element['parent_account'] == $parentId) {
                        $cats = array_merge($cats, $this->findChildrens($elements, $element['id'], true));
                    }
                }
            }
        }
        return $cats;
    }

        /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $isOrderAccountNumber = (isset($request->order[0]) && $request->order[0]['column'] == '0') ? true : false;

            if($isOrderAccountNumber){
                $data = ChartOfAccount::orderBy('account_type', 'ASC')->get();
                $accountNameArr = $data->pluck('account_name', 'id')->toArray();

                $data = $data->toArray();
            }else{
                /* Default sort by Name with Parent alphabetically and child sort ny account number - START */
                $data = array();

                $allRecords = ChartOfAccount::orderBy('account_type', 'ASC')->get()->toArray();

                $allParents = ChartOfAccount::whereNull('parent_account')->orderBy('account_type', 'ASC')->orderBy('account_name', 'ASC')->get()->toArray();
                //array_multisort(array_column($allParents, 'account_name'), SORT_ASC, $allParents); // Parents sorting by account_name

                foreach ($allParents as $parent){
                    $data[] = $parent;
                    $children = $this->findChildrens($allRecords, $parent['id']);
                    if(count($children) > 0){
                        array_multisort(array_column($children, 'account_number'), SORT_ASC, $children); // Children sorting by account_number
                        foreach ($children as $child){
                            $data[] = $child;
                        }
                    }
                }
                /* Default sort by Name with Parent alphabetically and child sort ny account number - END */
            }

            foreach ($data as $key => $value){
                $this->parents = [];
                $this->findParents($data, $value['id']);
                $data[$key]['index'] = count($this->parents);

                if($isOrderAccountNumber){
                    $breadcrumbs = array();
                    foreach ($this->parents as $parent){
                        $breadcrumbs[] = $accountNameArr[$parent];
                    }
                    $data[$key]['breadcrumbs'] = (count($breadcrumbs) > 0) ? 'Sub-account of: '.implode(' : ', array_reverse($breadcrumbs)) : "";
                }
            }

            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $btn = '<a href="#" class="edit-btn mr-4" data-id="'.$row['id'].'" data-toggle="modal" data-target="#editModal"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';

                    $btn = $btn.' <a href="javascript:" class="destroy" data-action="'.route('admin.chart-of-accounts.destroy', $row['id']).'" data-id="'.$row['id'].'"><img src="'.asset('theme/img/delete-icon.png').'" alt=""></a>';

                    return $btn;
                })
                ->editColumn('account_name', function ($row) use ($isOrderAccountNumber){
                    if($isOrderAccountNumber){
                        return $row['account_name'].'<br/>'.$row['breadcrumbs'];
                    }else{
                        return str_repeat("&nbsp;", ($row['index'] * 6)).$row['account_name'];
                    }
                })
                ->editColumn('account_type', function ($row){
                    return getChartOfAccountsTypes($row['account_type']);
                })
                ->rawColumns(['account_type', 'account_name', 'action'])
                ->make(true);
        }
        return view('admin.chartOfAccounts.index');
    }

    public function parentList(Request $request)
    {
        $dropdownHtml = "<option value=''>Select Parent</option>";
        if($request->accountType != ""){
            $parents = ChartOfAccount::select('id', 'account_number', 'account_name')->where('account_type', $request->accountType)->orderBy('account_number')->get()->toArray();

            foreach ($parents as $key => $value){
                $dropdownHtml .= "<option value='".$value['id']."'>".$value['account_number']." - ".$value['account_name']."</option>";
            }
        }
        return response()->json(['dropDownHtml' => $dropdownHtml]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $input      = $request->all();
        $validator  = Validator::make($input,[
            "account_type"  => "required",
            "account_name"  => "required",
            "account_number"  => "required|numeric",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        ChartOfAccount::create($input);

        notification('success', $this->crudMessage('create', 'Account Information'));
        return response()->json(['success' => "Account Information created successfully"]);
    }

    public function editFormView(Request $request)
    {
        $chartOfAccount = ChartOfAccount::find($request->id);
        $parents = array();
        if($chartOfAccount){
            $parentsArr = ChartOfAccount::select('id', 'account_number', 'account_name')->where('account_type', $chartOfAccount->account_type)->where('id', '!=', $chartOfAccount->id)->orderBy('account_number')->get()->toArray();
            foreach ($parentsArr as $parent){
                $parents[$parent['id']] = $parent['account_number'].' - '.$parent['account_name'];
            }
        }
        $view = view('admin.chartOfAccounts.editFormView', compact(['chartOfAccount', 'parents']))->render();

        return response()->json(['view' => $view, 'data' => $chartOfAccount]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, ChartOfAccount $chartOfAccount)
    {
        $input = $request->all();

        $validator  = Validator::make($input,[
            "account_type"  => "required",
            "account_name"  => "required",
            "account_number"  => "required|numeric",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $chartOfAccount->update($input);

        notification('success', $this->crudMessage('update', 'Account Information'));

        return response()->json(['success' => "Account Information updated successfully"]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(ChartOfAccount $chartOfAccount)
    {
        $chartOfAccount->delete();

        notification('success', $this->crudMessage('delete', 'Account Information'));

        return redirect()->route('admin.chart-of-accounts.index');
    }
}
