<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Admin\AdminController;
use Illuminate\Http\Request;

use App\Models\Company;

use App\Models\EmailTemplate;
use App\Models\CompanyEmailTemplate;

use Validator;
use DataTables;

class EmailTemplateController extends AdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        if ($request->ajax()) {
            $data = EmailTemplate::select('*')->orderBy('title', 'ASC');
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $btn = ' <a href="'.route('admin.email-templates.edit', $row->id).'" class="mr-4"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';
                    $btn = $btn.' <a href="javascript:" class="destroy" data-action="'.route('admin.email-templates.destroy', $row->id).'" data-id="'.$row->id.'"><img src="'.asset('theme/img/delete-icon.png').'" alt=""></a>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        return view('admin.emailTemplate.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return view('admin.emailTemplate.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request){
        $input              = $request->all();
        $validator          = Validator::make($input, [
            'title'         => 'required|string|max:255|unique:email_templates,title,NULL,id,deleted_at,NULL',
            'content'       => 'required|string'
        ]);
        if ($validator->fails()) {
            return redirect()->back()->withErrors($validator)->withInput();
        }
        $emailTemplate      = EmailTemplate::create($input);
        if($emailTemplate){
            $getCompanies   = Company::pluck('id');
            if($getCompanies->isNotEmpty()){
                foreach ($getCompanies as $companyId) {
                    CompanyEmailTemplate::create([
                        'company_id'    => $companyId,
                        'title'         => $emailTemplate->title,
                        'subject'       => $emailTemplate->subject,
                        'content'       => $emailTemplate->content,
                        'tags'          => $emailTemplate->tags,
                        'description'   => $emailTemplate->description,
                    ]);
                }
            }
        }
        notification('success', $this->crudMessage('create', 'Email Template'));
        return redirect()->route('admin.email-templates.index');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id){
        return redirect()->route('admin.email-templates.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(EmailTemplate $emailTemplate){
        return view('admin.emailTemplate.edit', compact('emailTemplate'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id){
        $input          = $request->all();
        $validator      = Validator::make($input, [
            'title'     => 'required|string|max:255|unique:email_templates,title,' . $id . ',id,deleted_at,NULL',
            'content'   => 'required|string'
        ]);
        if ($validator->fails()) {
            return redirect()->back()->withErrors($validator)->withInput();
        }
        $emailTemplate  = EmailTemplate::findOrFail($id);
        $emailTemplate->update($input);
        notification('success', $this->crudMessage('update', 'Email Template'));
        return redirect()->route('admin.email-templates.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(EmailTemplate $emailTemplate){
        $emailTemplate->delete();
        notification('success', $this->crudMessage('delete', 'Email Template'));
        return redirect()->route('admin.email-templates.index');
    }
}
