<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Admin\AdminController;
use Illuminate\Http\Request;

use App\Http\Requests\Admin\EventTypeStoreRequest;
use App\Http\Requests\Admin\EventTypeUpdateRequest;

use App\Models\EventType;

use DataTables;

class EventTypeController extends AdminController{

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getEventType       = getEventTypes();
        foreach($getEventType as $key => $getEve){
            $checkEventType = EventType::where('name',$key)->first();
            if(empty($checkEventType->id)){
                $eveType            = [];
                $eveType['name']    = $key;
                $eveType['color']   = '#C1BFBF';
                EventType::create($eveType);
            }
        }

        if ($request->ajax()) {
            $data           = EventType::query();
            return Datatables::of($data)
                ->addIndexColumn()
                ->editColumn('color', function($row){
                    return '<div class="bgcolro" style="background-color:'.$row->color.';">'.$row->color.'</div>';
                })
                ->editColumn('name', function($row){
                    return getEventTypes()[$row->name];
                })
                ->filterColumn('name', function ($query, $keyword) {
                    $results = array();
                    foreach (getEventTypes() as $tid => $types) {
                       if (strstr(strtolower($types), strtolower($keyword)) !== false) { $results[] = $tid; }
                    }
                    if(!empty($results)){
                        $query->whereIn('name',$results);
                    }
                })
                ->addColumn('action', function($row){
                    $btn = ' <a href="'.route('admin.event-type.edit', $row->id).'" class="mr-4"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';
                    return $btn;
                })
                ->rawColumns(['action','color'])
                ->make(true);
        }
        return view('admin.eventType.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return Redirect::back();
        //return view('admin.eventType.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(EventTypeStoreRequest $request){
        $input = $request->all();
        EventType::create($input);

        notification('success', $this->crudMessage('create', 'EventType'));
        return redirect()->route('admin.event-type.index');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(EventType $event_type){
        return redirect()->route('admin.event-type.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(EventType $event_type){
        return view('admin.eventType.edit', compact('event_type'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(EventTypeUpdateRequest $request, EventType $event_type){
        $input = $request->all();
        $event_type->update($input);
        notification('success', $this->crudMessage('update', 'EventType'));
        return redirect()->route('admin.event-type.index');
    }

    public function destroy(EventType $event_type){
        $event_type->delete();

        notification('success', $this->crudMessage('delete', 'EventType'));

        return redirect()->route('admin.event-type.index');
    }

}
