<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Admin\AdminController;
use Illuminate\Http\Request;

use App\Http\Requests\Admin\HolidayStoreRequest;
use App\Http\Requests\Admin\HolidayUpdateRequest;

use App\Models\Holiday;

use DataTables;

class HolidayController extends AdminController{

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        if ($request->ajax()) {
            $data = Holiday::select('*');
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $btn = ' <a href="'.route('admin.holidays.edit', $row->id).'" class="mr-4"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';
                    $btn = $btn.' <a href="javascript:" class="destroy" data-action="'.route('admin.holidays.destroy',$row->id).'" data-id="'.$row->id.'"><img src="'.asset('theme/img/delete-icon.png').'" alt=""></a>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
        }

        return view('admin.holiday.index');

    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return view('admin.holiday.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(HolidayStoreRequest $request){
        $input = $request->all();
        Holiday::create($input);

        notification('success', $this->crudMessage('create', 'Holiday'));
        return redirect()->route('admin.holidays.index');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Holiday $holiday){
       return redirect()->route('admin.holiday.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Holiday $holiday){
        return view('admin.holiday.edit', compact('holiday'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(HolidayUpdateRequest $request, Holiday $holiday){
        $input = $request->all();
        $holiday->update($input);
        notification('success', $this->crudMessage('update', 'Holiday'));
        return redirect()->route('admin.holidays.index');
    }

    public function destroy(Holiday $holiday){
        $holiday->delete();

        notification('success', $this->crudMessage('delete', 'Holiday'));

        return redirect()->route('admin.holidays.index');
    }

}
