<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Admin\AdminController;
use Illuminate\Http\Request;

use App\Models\Company;

use App\Models\PdfTemplate;
use App\Models\CompanyPdfTemplate;

use Validator;
use DataTables;

class PdfTemplateController extends AdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        if ($request->ajax()) {
            $data = PdfTemplate::query();
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $btn = ' <a href="'.route('admin.pdf-templates.edit', $row->id).'" class="mr-4">
                        <img src="'.asset('theme/img/edit-icon.png').'" alt="Edit Pdf Template">
                    </a>';
                    $btn = $btn.' <a href="javascript:void(0);" class="destroy" data-action="'.route('admin.pdf-templates.destroy', $row->id).'" data-id="'.$row->id.'">
                        <img src="'.asset('theme/img/delete-icon.png').'" alt="Delete Pdf Template">
                    </a>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        return view('admin.pdfTemplate.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return view('admin.pdfTemplate.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request){
        $input              = $request->all();
        $validator          = Validator::make($input, [
            'name'          => 'required|string|max:255|unique:pdf_templates,name,NULL,id,deleted_at,NULL',
            'content'       => 'required|string' // Validation for CKEditor content
        ]);
        if ($validator->fails()) {
            return redirect()->back()->withErrors($validator)->withInput();
        }
        $pdfTemplate        = PdfTemplate::create($input);
        if($pdfTemplate){
            $getCompanies   = Company::pluck('id');
            if($getCompanies->isNotEmpty()){
                foreach ($getCompanies as $companyId) {
                    CompanyPdfTemplate::create([
                        'company_id'    => $companyId,
                        'name'          => $pdfTemplate->name,
                        'content'       => $pdfTemplate->content,
                        'tags'          => $pdfTemplate->tags
                    ]);
                }
            }
        }
        notification('success', $this->crudMessage('create', 'Pdf Template'));
        return redirect()->route('admin.pdf-templates.index');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(PdfTemplate $pdf_template){
        return redirect()->route('admin.pdf-templates.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(PdfTemplate $pdf_template){
        return view('admin.pdfTemplate.create', compact('pdf_template'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, PdfTemplate $pdf_template){
        $input          = $request->all();
        $validator      = Validator::make($input, [
            'name'      => 'required|string|max:255|unique:pdf_templates,name,' . $pdf_template->id . ',id,deleted_at,NULL',
            'content'   => 'required|string' // Validation for CKEditor content
        ]);
        if ($validator->fails()) {
            return redirect()->back()->withErrors($validator)->withInput();
        }
        $pdf_template->update($input);
        notification('success', $this->crudMessage('update', 'Pdf Template'));
        return redirect()->route('admin.pdf-templates.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(PdfTemplate $pdf_template){
        $pdf_template->delete();
        notification('success', $this->crudMessage('delete', 'Pdf Template'));
        return redirect()->route('admin.pdf-templates.index');
    }
}
