<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Admin\AdminController;
use App\Models\Company;
use App\Models\CompanyPmWarrantyProvider;
use App\Models\PmWarrantyProvider;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Yajra\DataTables\Facades\DataTables;

class PmWarrantyProviderController extends AdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $data = PmWarrantyProvider::get();
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $btn = '<a href="#" class="edit-btn mr-4" data-id="'.$row->id.'" data-toggle="modal" data-target="#editModal"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';

                    $btn = $btn.' <a href="javascript:" class="destroy" data-action="'.route('admin.pm-warranty-providers.destroy', $row->id).'" data-id="'.$row->id.'"><img src="'.asset('theme/img/delete-icon.png').'" alt=""></a>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        return view('admin.pmWarrantyProviders.index');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $input      = $request->all();
        $validator  = Validator::make($input,[
            "name"  => "required",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $data = PmWarrantyProvider::create($input);

        $getCompnay = Company::get();
        foreach ($getCompnay as $compnay){
            CompanyPmWarrantyProvider::create(['company_id' => $compnay->id, 'name' => $data->name]);
        }

        notification('success', $this->crudMessage('create', 'PmWarrantyProvider'));
        return response()->json(['success' => "Pm Warranty Provider created successfully"]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(PmWarrantyProvider $pmWarrantyProvider)
    {
        return view('admin.pmWarrantyProviders.edit', compact('pmWarrantyProvider'));
    }

    public function editFormView(Request $request)
    {
        $pmWarrantyProvider = PmWarrantyProvider::find($request->id);

        $view = view('admin.pmWarrantyProviders.editFormView', compact('pmWarrantyProvider'))->render();

        return response()->json(['view' => $view, 'data' => $pmWarrantyProvider]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, PmWarrantyProvider $pmWarrantyProvider)
    {
        $input = $request->all();

        $validator  = Validator::make($input,[
            "name"  => "required",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }
        $pmWarrantyProvider->update($input);

        notification('success', $this->crudMessage('update', 'PmWarrantyProvider'));

        return response()->json(['success' => "Pm Warranty Provider updated successfully"]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(PmWarrantyProvider $pmWarrantyProvider)
    {
        CompanyPmWarrantyProvider::where('name', $pmWarrantyProvider->name)->where('company_id', 0)->delete();
        $pmWarrantyProvider->delete();

        notification('success', $this->crudMessage('delete', 'PmWarrantyProvider'));

        return redirect()->route('admin.pm-warranty-providers.index');
    }
}
