<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Admin\AdminController;
use App\Models\Company;
use App\Models\CompanySafetyCompliance;
use App\Models\SafetyCompliance;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Yajra\DataTables\Facades\DataTables;

class SafetyComplianceController extends AdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $data = SafetyCompliance::get();
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $btn = '<a href="#" class="edit-btn mr-4" data-id="'.$row->id.'" data-toggle="modal" data-target="#editModal"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';

                    $btn = $btn.' <a href="javascript:" class="destroy" data-action="'.route('admin.safety-and-compliance-documents.destroy', $row->id).'" data-id="'.$row->id.'"><img src="'.asset('theme/img/delete-icon.png').'" alt=""></a>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        return view('admin.safetyCompliance.index');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $input      = $request->all();
        $validator  = Validator::make($input,[
            "name"  => "required",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $data = SafetyCompliance::create($input);

        $getCompnay = Company::get();
        foreach ($getCompnay as $compnay){
            CompanySafetyCompliance::create(['company_id' => $compnay->id, 'name' => $data->name]);
        }

        notification('success', $this->crudMessage('create', 'SafetyAndComplianceDocument'));
        return response()->json(['success' => "Safety and Compliance Document created successfully"]);
    }

    public function editFormView(Request $request)
    {
        $safetyAndComplianceDocument = SafetyCompliance::find($request->id);

        $view = view('admin.safetyCompliance.editFormView', compact('safetyAndComplianceDocument'))->render();

        return response()->json(['view' => $view, 'data' => $safetyAndComplianceDocument]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, SafetyCompliance $safetyAndComplianceDocument)
    {
        $input = $request->all();

        $validator  = Validator::make($input,[
            "name"  => "required",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }
        $safetyAndComplianceDocument->update($input);

        notification('success', $this->crudMessage('update', 'SafetyAndComplianceDocument'));

        return response()->json(['success' => "Safety and Compliance Document updated successfully"]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(SafetyCompliance $safetyAndComplianceDocument)
    {
        CompanySafetyCompliance::where('name', $safetyAndComplianceDocument->name)->where('company_id', 0)->delete();
        $safetyAndComplianceDocument->delete();

        notification('success', $this->crudMessage('delete', 'SafetyAndComplianceDocument'));

        return redirect()->route('admin.safety-and-compliance-documents.index');
    }
}
