<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Admin\AdminController;
use Illuminate\Http\Request;

use App\Models\MasterMaintenanceTicketType;

use Validator;
use DataTables;

class TicketTypeController extends AdminController{

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        if ($request->ajax()) {
            $data = MasterMaintenanceTicketType::get();
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $btn = '<a href="#" class="edit-btn mr-4" data-id="'.$row->id.'" data-toggle="modal" data-target="#editModal"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';

                    $btn = $btn.' <a href="javascript:" class="destroy" data-action="'.route('admin.ticket-type.destroy',$row->id).'" data-id="'.$row->id.'"><img src="'.asset('theme/img/delete-icon.png').'" alt=""></a>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        return view('admin.ticketType.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return Redirect::back();
        //return view('admin.eventType.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request){
        $input      = $request->all();
        $validator  = Validator::make($input,[
            "name"  => "required",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        MasterMaintenanceTicketType::create($input);

        notification('success', $this->crudMessage('create', 'TicketType'));
        return response()->json(['success' => "Ticket Type created successfully"]);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(EventType $ticket_type){
       return redirect()->route('admin.ticket-type.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(MasterMaintenanceTicketType $ticket_type){
        return view('admin.ticketType.edit', compact('appointment_type'));
    }

    public function editFormView(Request $request){
        $ticketType = MasterMaintenanceTicketType::find($request->id);

        $view       = view('admin.ticketType.editFormView', compact('ticketType'))->render();

        return response()->json(['view' => $view, 'data' => $ticketType]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, MasterMaintenanceTicketType $ticket_type){
        $input      = $request->all();

        $validator  = Validator::make($input,[
            "name"  => "required",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $ticket_type->update($input);

        notification('success', $this->crudMessage('update', 'TicketType'));

        return response()->json(['success' => "TicketType Type updated successfully"]);
    }

    public function destroy(MasterMaintenanceTicketType $ticket_type){
        $ticket_type->delete();

        notification('success', $this->crudMessage('delete', 'TicketType'));

        return redirect()->route('admin.ticket-type.index');
    }

}
