<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Admin\AdminController;
use Illuminate\Http\Request;

use App\Models\Company;

use App\Models\TrailerGroup;
use App\Models\CompanyTrailerGroup;

use Validator;
use DataTables;

class TrailerGroupController extends AdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        if ($request->ajax()) {
            $data = TrailerGroup::get();
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $btn = '<a href="javascript:void(0);" class="edit-btn mr-4" data-id="'.$row->id.'" data-toggle="modal" data-target="#editModal"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';
                    $btn = $btn.' <a href="javascript:void(0);" class="destroy" data-action="'.route('admin.trailer-groups.destroy', $row->id).'" data-id="'.$row->id.'"><img src="'.asset('theme/img/delete-icon.png').'" alt=""></a>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        return view('admin.trailerGroups.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return redirect()->route('admin.trailer-groups.index');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request){
        $input                      = $request->all();
        $validator                  = Validator::make($input,[
            "name"                  => "required",
        ]);
        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }
        $data                       = TrailerGroup::create($input);
        $getCompnay                 = Company::get();
        foreach ($getCompnay as $compnay){
            CompanyTrailerGroup::create(['company_id' => $compnay->id, 'name' => $data->name]);
        }
        notification('success', $this->crudMessage('create', 'Trailer group'));
        return response()->json(['success' => "Trailer group created successfully"]);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id){
        return redirect()->route('admin.trailer-groups.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id){
        return redirect()->route('admin.trailer-groups.index');
    }

    public function editFormView(Request $request){
        $trailer_group              = TrailerGroup::find($request->id);
        $view                       = view('admin.trailerGroups.editFormView', compact('trailer_group'))->render();
        return response()->json(['view' => $view, 'data' => $trailer_group]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, TrailerGroup $trailer_group){
        $input                      = $request->all();
        $validator                  = Validator::make($input,[
            "name"                  => "required",
        ]);
        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }
        $trailer_group->update($input);
        notification('success', $this->crudMessage('update', 'Trailer group'));
        return response()->json(['success' => "Trailer group updated successfully"]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(TrailerGroup $trailer_group){
        $trailer_group->delete();
        notification('success', $this->crudMessage('delete', 'Trailer group'));
        return redirect()->route('admin.trailer-groups.index');
    }
}
