<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;

use App\Models\Admin;
use App\Models\TwoFactorSetting;

class TwoFactorAuthenticationController extends Controller
{
    public function index()
    {
        if(Session::has('admin-2fa')){
            return redirect()->route('admin.home');
        }

        $twoFactorSettings  = TwoFactorSetting::IsAdmin()->first();
        return view('auth.two-factor.admin.index', compact('twoFactorSettings'));
    }

    public function verifyCode(Request $request)
    {
        $request->validate(
            ['two_factor_secret' => 'required'],
            ['two_factor_secret.required' => 'Please enter 2-factor verification code.']
        );

        $user               = auth()->guard('admin')->user();
        $userDetails        = Admin::Admin()->where('id', $user->id)->where('two_factor_secret', $request->two_factor_secret)->first();
        if (!is_null($userDetails)) {
            Session::put('admin-2fa', $user->id);
            return redirect()->route('admin.home');
        }
        return back()->with('error', 'You entered wrong code.');
    }

    public function resendCode(Request $request)
    {
        auth()->guard('admin')->user()->generateTwoFactorCode();
        return back()->with('success', 'We sent you code on your email.');
    }
}
