<?php

namespace App\Http\Controllers\Auth\CompanyAdmin;

use App\Http\Controllers\Controller;
use App\Providers\RouteServiceProvider;
use Illuminate\Foundation\Auth\AuthenticatesUsers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Session;

use App\Models\TwoFactorSetting;

class LoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */

    use AuthenticatesUsers;

    /**
     * Where to redirect users after login.
     *
     * @var string
     */
    protected $redirectTo = '/company-admin/home';

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest')->except('logout');
    }

    /**
     * Display the form of company admin login.
     *
     * @return \Illuminate\Http\Response
    */
    public function showLoginForm()
    {
        if (Auth::guard('company-admin')->check()) {
            return redirect()->route('company.admin.home');
        } else {
            return view('auth.companyAdminLogin');
        }
    }

    /**
     * Login to company admin.
     *
     * @return \Illuminate\Http\Response
    */
    public function login(Request $request)
    {
        $this->validate($request,['email' => 'required|email', 'password' => 'required']);

        if (Auth::guard('company-admin')->attempt(['email' => $request->email, 'password' => $request->password])) {
            getTwilioChatToken();

            /* If Enable 2-Factor Authentication - START */
            $twoFactorSettings  = TwoFactorSetting::IsCompanyAdmin()->first();

            if(!is_null($twoFactorSettings) && $twoFactorSettings->enable_two_factor == '1'){
                auth()->guard('company-admin')->user()->generateTwoFactorCode();
                return redirect()->route('company.admin.two-factor-authentication.index');
            }
            /* If Enable 2-Factor Authentication - END */

            // Check if the intended URL is set in the session
            $intendedUrl        = Session::has('company.url.intended') ? Session::get('company.url.intended', $this->redirectPath()) : '/company-admin/home';
            Session::forget('company.url.intended'); // Clear the stored URL
            return redirect()->intended($intendedUrl); // return redirect()->route('company.admin.home');
        } else {
            return redirect()->back()->with('error' , 'Email or password are wrong.');
        }

        return redirect()->back();
    }

    /**
     * Logout from company admin.
     *
     *
     * @return \Illuminate\Http\Response
    */
    public function logout()
    {
        Auth::guard('company-admin')->logout();
        setcookie("twilio_token", '', time() - 3600, "/"); // set cookie to expired
        // Session::flush();
        Session::forget('companyAdmin-2fa');
        return redirect()->route('company.admin.login');
    }
}