<?php

namespace App\Http\Controllers\Auth\DriverAdmin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use Illuminate\Foundation\Auth\SendsPasswordResetEmails;
use Illuminate\Support\Facades\Password;
use Illuminate\Support\Str;

use App\Models\PasswordReset;
use App\Mail\DriverAdmin\ForgotPassword;

use Mail;
use Validator;
use Carbon\Carbon;

class ForgotPasswordController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Password Reset Controller
    |--------------------------------------------------------------------------
    |
    | This controller is responsible for handling password reset emails and
    | includes a trait which assists in sending these notifications from
    | your application to your users. Feel free to explore this trait.
    |
    */

    use SendsPasswordResetEmails;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->passwordReset = new PasswordReset;
    }

    /** 
     * Display the form to request a password reset link.
     *
     * @return \Illuminate\Http\Response
    */
    public function showLinkRequestForm()
    {   
        return view('auth.passwords.driverAdmin.email');
    }

    /** 
     * Send password reset link email.
     *
     * @return \Illuminate\Http\Response
    */
    public function sendResetLinkEmail(Request $request)
    {
        $input      = $request->all();

        $validator  = Validator::make($input, [
            'email' =>'required|email|exists:drivers,email'
        ]);        

        if ($validator->passes()) {
            $input['token']         = Str::random(100);
            $input['created_at']    = Carbon::now();

            $this->passwordReset->store($input); 

            $mailData               = [];
            $mailData['email']      = $input['email'];
            $mailData['token']      = $input['token'];
            Mail::to($input['email'])->send(new ForgotPassword($mailData));

            return redirect()->back()->with('success','Reset password link sent on your email');
        }
        return redirect()->back()->withErrors($validator)->withInput();
    }
}
