<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

use App\Models\Company; // Added at 10-04-2023
use App\Models\CompanyAdmin; // Added at 10-04-2023

use App\Models\Driver; // Added at 11-04-2023
use App\Models\Broker; // Added at 11-04-2023
use App\Models\Customer; // Added at 11-04-2023
use App\Models\BrokerContact; // Added at 11-04-2023

use App\Models\LoadFile;
use App\Models\LoadStop;
use App\Models\LoadNote;
use App\Models\LoadDetail;

use App\Models\SmsChatHistory; // Added at 10-04-2023
use App\Models\ChatLoadInternal;

use App\Models\Gallery;
use App\Models\IntegrationSetting;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ServerException;

use DB; // Added at 14-04-2023
use Carbon\Carbon;

class ChatController extends Controller
{
    public $timeZone;
    public $timeZoneId;
    public $offset;
    public $logged_in_guard;
    public $guard_table;

    public function __construct(){
        if( request()->route()->getPrefix() == 'company-admin' ) {
            $this->logged_in_guard  = 'company-admin';
            $this->guard_table      = 'company_admins';
            view()->share('extendTheme','companyAdminTheme.default');
        }

        if( request()->route()->getPrefix() == 'driver-admin' ) {
            $this->logged_in_guard  = 'driver-admin';
            $this->guard_table      = 'drivers';
            view()->share('extendTheme','driverAdminTheme.default');
        }

        $this->middleware(function ($request, $next) {
            $this->timeZone         = getTimeZone();
            $this->timeZoneId       = getTimeZoneId();
            $this->offset           = getTimeZoneOffset();

            return $next($request);
        });
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index() {
        $mailchaticn                = asset("theme/images/mail-chat.png"); // Added at 11-04-2023
        $smschaticn                 = asset("theme/images/sms.png"); // Added at 11-04-2023

        $logged_in_guard            = $this->logged_in_guard;
        $company_id                 = auth()->guard($logged_in_guard)->user()->company_id; // Added at 10-04-2023
        $company                    = Company::where('id', $company_id)->first(); // Added at 10-04-2023
        $phoneNumber                = $company->twilio_active_number; // Added at 10-04-2023
        return view('chat.index', compact('logged_in_guard', 'phoneNumber', 'mailchaticn','smschaticn'));
    }

    public function getChatHistoryOld( Request $request ){
        try {
            $messages                                   = getFilterSMSHistory();
            $phone                                      = $request->tophone;
            $company_num                                = $request->fromphone;

            $fromto                                     = getFilterSMSHistory( $phone );
            $tofrom                                     = getFilterSMSHistory( '', $phone );

            // dd( $fromto, $tofrom );

            foreach( $messages as $key => $value ) {
                if( $value['direction'] == 'inbound' ) {
                    $messages[$key]['msg_align_class']  = 'chat-mianLeft';
                } else {
                    $messages[$key]['msg_align_class']  = 'chat-mianRight';
                }
                $messages[$key]['time']                 = date("h:i A", strtotime( $value['date'] ));
            }

            $response['success']                        = true;
            $response['data']                           = $messages;
            return response()->json($response);
        } catch( \Exception $e ) {
            \Log::emergency( "File: ".$e->getFile() );
            \Log::emergency( "Line: ".$e->getLine() );
            \Log::emergency( "Message: ".$e->getMessage() );

            $response['success']                        = false;
            $response['data']                           = [];
            return response()->json($response);
        }
    }

    // Added at 10-04-2023
    public function getChatHistory( Request $request ){
        try {
            $logged_in_guard                            = $this->logged_in_guard;
            $company_id                                 = auth()->guard($logged_in_guard)->user()->company_id;
            $company                                    = Company::where('id', $company_id)->first();

            // $guard                                   = $request->guard;
            $tophone                                    = $request->tophone;
            // $to_id                                   = $request->to_id;
            // $from_id                                 = $request->from_id;

            $skip                                       = 0;
            $take                                       = $request->take;

            $totalSms                                   = SmsChatHistory::where( 'company_id', $company_id )
                ->where(function ($query) use ($tophone ) {
                    $query->where('from_number','LIKE','%'.$tophone.'%')
                    ->orWhere('to_number', 'LIKE','%'. $tophone.'%');
                })
                ->count();

            // dd($totalSms);
            // $totalSms                                = getTotalSmsCountCompany( $company_id );
            // $messages                                = getFilterSMSHistoryByCompany( $company_id, $skip, $take, $logged_in_guard );
            $messages                                   = getFilterSMSHistoryByCompany( $company_id, $skip, $take, $logged_in_guard, '' ,$tophone );
            // $users                                   = $this->getUserPhones();

            $response['success']                        = true;
            $response['data']                           = $messages;
            // $response['users']                       = $users;
            $response['total']                          = $totalSms;
            return response()->json($response);
        } catch( \Exception $e ) {
            \Log::emergency( "File: ".$e->getFile() );
            \Log::emergency( "Line: ".$e->getLine() );
            \Log::emergency( "Message: ".$e->getMessage() );

            $response['success']                        = false;
            $response['data']                           = [];
            return response()->json($response);
        }
    }

    // Added at 11-04-2023
    public function loadMoreSmsHistory( Request $request ){
        try {
            $logged_in_guard                            = $this->logged_in_guard;
            $company_id                                 = auth()->guard($logged_in_guard)->user()->company_id;
            $company                                    = Company::where('id', $company_id)->first();

            $skip                                       = 0;
            $take                                       = $request->take;
            $tophone                                    = $request->tophone;

            $totalSms                                   = SmsChatHistory::where( 'company_id', $company_id )
                ->where(function ($query) use ($tophone ) {
                    $query->where('from_number','LIKE','%'.$tophone.'%')
                    ->orWhere('to_number', 'LIKE','%'. $tophone.'%');
                })
                ->count();

            // $totalSms                                = getTotalSmsCountCompany( $company_id );

            $messages                                   = getFilterSMSHistoryByCompany( $company_id, $skip, $take, $logged_in_guard ,'',$tophone);

            $response['success']                        = true;
            $response['data']                           = $messages;
            $response['total']                          = $totalSms;
            return response()->json($response);
        } catch( \Exception $e ) {
            \Log::emergency( "File: ".$e->getFile() );
            \Log::emergency( "Line: ".$e->getLine() );
            \Log::emergency( "Message: ".$e->getMessage() );

            $response['success']                        = false;
            $response['data']                           = [];
            return response()->json($response);
        }
    }

    // Added at 11-04-2023
    public function searchSmsHistory( Request $request ){
        try {
            $logged_in_guard                            = $this->logged_in_guard;
            $company_id                                 = auth()->guard($logged_in_guard)->user()->company_id;
            $company                                    = Company::where('id', $company_id)->first();
            $tophone                                    = $request->tophone;

            $totalSms                                   = SmsChatHistory::where( 'company_id', $company_id )
                ->where(function ($query) use ($tophone ) {
                    $query->where('from_number','LIKE','%'.$tophone.'%')
                    ->orWhere('to_number', 'LIKE','%'. $tophone.'%');
                })
                ->count();

            // $totalSms                                = getTotalSmsCountCompany( $company_id );

            $skip                                       = 0;
            $take                                       = $totalSms;

            $messages                                   = getFilterSMSHistoryByCompany( $company_id, $skip, $take, $logged_in_guard, $request->search_txt,$tophone );

            $response['success']                        = true;
            $response['data']                           = $messages;
            $response['total']                          = $totalSms;
            return response()->json($response);
        } catch( \Exception $e ) {
            \Log::emergency( "File: ".$e->getFile() );
            \Log::emergency( "Line: ".$e->getLine() );
            \Log::emergency( "Message: ".$e->getMessage() );

            $response['success']                        = false;
            $response['data']                           = [];
            return response()->json($response);
        }
    }

    // Developed at 10-04-2023
    public function sendSmsTwilio( Request $request ){
        try {
            $logged_in_guard                            = $this->logged_in_guard;
            $guard_table                                = $this->guard_table;

            $user                                       = auth()->guard($logged_in_guard)->user();
            $company_id                                 = $user->company_id;

            // Twilio's credentials are fetched from company table
            $sendSms                                    = sendSMSByCompany($user->company_id, $request->to_number, $request->from_number, $request->content);

            $data                                       = array(
                'company_id'                            => $company_id,
                'unique_id_sender'                      => $logged_in_guard.'_'.$user->id,
                'table_name'                            => $guard_table,
                'msg_id'                                => $sendSms,
                'type'                                  => 'sent',
                'from_number'                           => $request->from_number,
                'to_number'                             => $request->to_number,
                'msg_body'                              => $request->content,
                'sms_date'                              => now(),
                'created_at'                            => now(),
                'updated_at'                            => now()
            );

            $storeSms                                   = SmsChatHistory::create( $data );

            // Sms Chat History
            $skip                                       = 0;
            $take                                       = $request->take;

            $totalSms                                   = getTotalSmsCountCompany( $company_id );

            $messages                                   = getFilterSMSHistoryByCompany( $company_id, $skip, $take, $logged_in_guard );

            $response['msg']                            = 'Message sent';
            $response['success']                        = true;
            $response['data']                           = $messages;
            $response['total']                          = $totalSms;
            return response()->json($response);
        } catch( \Exception $e ) {
            \Log::emergency( "File: ".$e->getFile() );
            \Log::emergency( "Line: ".$e->getLine() );
            \Log::emergency( "Message: ".$e->getMessage() );

            $response['success']                        = false;
            $response['data']                           = [];
            return response()->json($response);
        }
    }

    // Added at - 10-04-2023
    // public function getUserPhones()
    // {
    //     $data = array();

    //     if( auth()->guard('driver-admin')->check() ) {
    //         $driver_company_id = auth()->guard('driver-admin')->user()->company_id;

    //         $companyDispatchers = CompanyAdmin::select('id', 'first_name', 'last_name', 'profile_image', 'type', 'company_id', 'cell_phone')->where('company_id', $driver_company_id)->whereNotNull('cell_phone')->get();

    //         foreach ($companyDispatchers as $companyDispatcher){
    //             $type = $companyDispatcher->type == 0 ? 'company_admin_' : ( $companyDispatcher->type == 1 ? 'manager_' : 'dispatcher_');
    //             $data[] = [
    //                 'id' => $companyDispatcher->id,
    //                 'uniqueID' => $type.$companyDispatcher->id,
    //                 'friendlyName' => $companyDispatcher->first_name.' '.$companyDispatcher->last_name,
    //                 'cellPhone' => str_replace(array('(',')','-',' '),array('','','',''),$companyDispatcher->cell_phone)
    //             ];
    //         }
    //     } elseif( auth()->guard('company-admin')->check() ) {
    //         if( auth()->guard('company-admin')->user()->role_id == 0){
    //             $companyDispatchers = CompanyAdmin::company()->dispatcher()->whereNotNull('cell_phone')->get();

    //             $data = array();
    //             foreach ($companyDispatchers as $companyDispatcher){
    //                 $data[] = [
    //                     'uniqueID' => 'dispatcher_'.$companyDispatcher->id,
    //                     'id' => $companyDispatcher->id,
    //                     'friendlyName' => $companyDispatcher->first_name.' '.$companyDispatcher->last_name,
    //                     'cellPhone' => str_replace(array('(',')','-',' '),array('','','',''),$companyDispatcher->cell_phone)
    //                 ];
    //             }
    //         }else if( auth()->guard('company-admin')->user()->role_id == 2){
    //             $companyDispatchers = CompanyAdmin::company()->dispatcher()->where('id', '!=', auth()->guard('company-admin')->user()->id)->whereNotNull('cell_phone')->get();

    //             $data = array();
    //             foreach ($companyDispatchers as $companyDispatcher){
    //                 $data[] = [
    //                     'uniqueID' => 'dispatcher_'.$companyDispatcher->id,
    //                     'id' => $companyDispatcher->id,
    //                     'friendlyName' => $companyDispatcher->first_name.' '.$companyDispatcher->last_name,
    //                     'cellPhone' => str_replace(array('(',')','-',' '),array('','','',''),$companyDispatcher->cell_phone)
    //                 ];
    //             }
    //         }
    //     }

    //     return $data;
    // }

    // Added at - 26-04-2023
    public function getUserPhones(Request $request ){
        $data = array();

        $selectedPhones = [];

        if( isset( $request->phone ) ) {
            foreach( $request->phone as $key => $value){
                $value = substr($value, -10);
                array_push($selectedPhones,preg_replace('/(\d{3})(\d{3})(\d{4})/', '($1) $2-$3', $value));
            }
        }
        if( auth()->guard('driver-admin')->check() ) {
            $driver_company_id = auth()->guard('driver-admin')->user()->company_id;

            $companyDispatchers = CompanyAdmin::select('id', 'first_name', 'last_name', 'profile_image', 'type', 'company_id', 'cell_phone')->where('company_id', $driver_company_id)->whereNotNull('cell_phone')->whereNotIn('cell_phone', $selectedPhones)->get();

            foreach ($companyDispatchers as $companyDispatcher){
                $type = $companyDispatcher->type == 0 ? 'company_admin_' : ( $companyDispatcher->type == 1 ? 'manager_' : 'dispatcher_');
                $data[] = [
                    'id' => $companyDispatcher->id,
                    'uniqueID' => $type.$companyDispatcher->id,
                    'friendlyName' => $companyDispatcher->first_name.' '.$companyDispatcher->last_name,
                    'cellPhone' => str_replace(array('(',')','-',' '),array('','','',''),$companyDispatcher->cell_phone)
                ];
            }
        } elseif( auth()->guard('company-admin')->check() ) {
            // if( auth()->guard('company-admin')->user()->role_id == 0){
                // $companyDispatchers = CompanyAdmin::company()->dispatcher()->whereNotNull('cell_phone')->whereNotIn('cell_phone', $selectedPhones)->get();
                $data = array();
                $companyDispatchers = CompanyAdmin::company()->where('id', '!=', auth()->guard('company-admin')->user()->id)->where( 'company_id', auth()->guard('company-admin')->user()->company_id )->whereNotNull('cell_phone')->whereNotIn('cell_phone', $selectedPhones)->get();
                $companyDrivers = Driver::where( 'company_id', auth()->guard('company-admin')->user()->company_id )->whereNotNull('cell_phone')->whereNotIn('cell_phone', $selectedPhones)->get();

                $companyBrokers = Broker::where( 'company_id', auth()->guard('company-admin')->user()->company_id )->where('id', '!=', auth()->guard('company-admin')->user()->id)->whereNotNull('phone_number')->whereNotIn('phone_number', $selectedPhones)->get();
                foreach ($companyBrokers as $companyBroker){
                    $data[] = [
                        'uniqueID' => 'broker_'.$companyBroker->id,
                        'id' => $companyBroker->id,
                        'friendlyName' => $companyBroker->name,
                        'cellPhone' => str_replace(array('(',')','-',' '),array('','','',''),$companyBroker->phone_number)
                    ];
                    $companyBrokerContacts = $companyBroker->broker_contact()->where('id', '!=', auth()->guard('company-admin')->user()->id)->whereNotNull('phone_number')->whereNotIn('phone_number', $selectedPhones)->get();
                    if($companyBrokerContacts->isNotEmpty()){
                        foreach ($companyBrokerContacts as $key => $companyBrokerContact) {
                            $data[] = [
                                'uniqueID' => 'broker_contact_'.$companyBrokerContact->id,
                                'id' => $companyBrokerContact->id,
                                'friendlyName' => $companyBrokerContact->name,
                                'cellPhone' => str_replace(array('(',')','-',' '),array('','','',''),$companyBrokerContact->phone_number)
                            ];
                        }
                    }
                }

                 $companyCustomers = Customer::where( 'company_id', auth()->guard('company-admin')->user()->company_id )->where('id', '!=', auth()->guard('company-admin')->user()->id)->whereNotNull('phone')->whereNotIn('phone', $selectedPhones)->get();
                    foreach ($companyCustomers as $companyCustomer){
                        $data[] = [
                                'uniqueID' => 'shipper_consignee_'.$companyCustomer->id,
                                'id' => $companyCustomer->id,
                                'friendlyName' => $companyCustomer->name,
                                'cellPhone' => str_replace(array('(',')','-',' '),array('','','',''),$companyCustomer->phone)
                            ];
                    }
                // dd("data1",$data);

                foreach ($companyDispatchers as $companyDispatcher){
                    $type = $companyDispatcher->type == 0 ? 'company_admin' : ( $companyDispatcher->type == 1 ? 'manager' : 'dispatcher');
                    $data[] = [
                        'uniqueID' => $type.'_'.$companyDispatcher->id,
                        'id' => $companyDispatcher->id,
                        'friendlyName' => $companyDispatcher->first_name.' '.$companyDispatcher->last_name,
                        'cellPhone' => str_replace(array('(',')','-',' '),array('','','',''),$companyDispatcher->cell_phone)
                    ];
                }

                foreach ($companyDrivers as $companyDriver){
                    $data[] = [
                        'uniqueID' => 'driver_'.$companyDriver->id,
                        'id' => $companyDriver->id,
                        'friendlyName' => $companyDriver->first_name.' '.$companyDriver->last_name,
                        'cellPhone' => str_replace(array('(',')','-',' '),array('','','',''),$companyDriver->cell_phone)
                    ];
                }
            // }else if( auth()->guard('company-admin')->user()->role_id == 2){
            //     $companyDispatchers = CompanyAdmin::company()->dispatcher()->where('id', '!=', auth()->guard('company-admin')->user()->id)->whereNotNull('cell_phone')->whereNotIn('cell_phone', $selectedPhones)->get();

            //     $companyDrivers = Driver::where( 'company_id', auth()->guard('company-admin')->user()->company_id )->whereNotNull('cell_phone')->whereNotIn('cell_phone', $selectedPhones)->get();

            //     $data = array();
            //     foreach ($companyDispatchers as $companyDispatcher){
            //         $data[] = [
            //             'uniqueID' => 'dispatcher_'.$companyDispatcher->id,
            //             'id' => $companyDispatcher->id,
            //             'friendlyName' => $companyDispatcher->first_name.' '.$companyDispatcher->last_name,
            //             'cellPhone' => str_replace(array('(',')','-',' '),array('','','',''),$companyDispatcher->cell_phone)
            //         ];
            //     }

            //     foreach ($companyDrivers as $companyDriver){
            //         $data[] = [
            //             'uniqueID' => 'dispatcher_'.$companyDriver->id,
            //             'id' => $companyDriver->id,
            //             'friendlyName' => $companyDriver->first_name.' '.$companyDriver->last_name,
            //             'cellPhone' => str_replace(array('(',')','-',' '),array('','','',''),$companyDriver->cell_phone)
            //         ];
            //     }
            // }
        }

        return $data;
    }

    // Added at 11-04-2023
    public function getEmailForExternalUse(Request $request){
        try {
            $selectedEmails = [];

            if( isset( $request->emails ) ) {
                $selectedEmails = $request->emails;
            }

            if( auth()->guard('driver-admin')->check() ) {
                $driver_company_id = auth()->guard('driver-admin')->user()->company_id;

                $companyDispatchers = CompanyAdmin::select('id', 'email', 'profile_image','first_name', 'last_name')->where('company_id', $driver_company_id)->whereNotIn('email', $selectedEmails)->get();

                $data = array();

                foreach ($companyDispatchers as $companyDispatcher){
                    $type = $companyDispatcher->type == 0 ? 'company_admin' : ( $companyDispatcher->type == 1 ? 'manager' : 'dispatcher');
                    $data[] = [
                        'toid' => $companyDispatcher->id,
                        // 'uniqueID' => $type.$companyDispatcher->id,
                        'uniqueID' => $type.'_'.$companyDispatcher->id,
                        'email' => $companyDispatcher->email,
                        'profilePic' => $companyDispatcher->profile_image,
                        'user_type' => $type,
                        'guard' => 'company-admin',
                        'toname' => ucwords($companyDispatcher->first_name.' '.$companyDispatcher->last_name)
                    ];
                }

                $response['success'] = true;
                $response['data'] = $data;
                return response()->json($response);

                /*} elseif( auth()->guard('company-admin')->check() ) {
                if( auth()->guard('company-admin')->user()->role_id == 0){
                    $companyDispatchers = CompanyAdmin::company()->dispatcher()->whereNotIn('email', $selectedEmails)->get();
                    $companyDrivers = Driver::where( 'company_id', auth()->guard('company-admin')->user()->company_id )->whereNotIn('email', $selectedEmails)->get();

                    $data = array();
                    foreach ($companyDispatchers as $companyDispatcher){
                        $data[] = [
                            'toid' => $companyDispatcher->id,
                            'uniqueID' => 'dispatcher_'.$companyDispatcher->id,
                            'email' => $companyDispatcher->email,
                            'profilePic' => $companyDispatcher->profile_image,
                            'user_type' => 'dispatcher',
                            'guard' => 'company-admin',
                            'toname' => ucwords($companyDispatcher->first_name.' '.$companyDispatcher->last_name)
                        ];
                    }

                    foreach( $companyDrivers as $driver ) {
                        $data[] = [
                            'toid' => $driver->id,
                            'uniqueID' => 'driver_'.$driver->id,
                            'email' => $driver->email,
                            'profilePic' => $driver->profile_image != '' ? $driver->profile_image : '',
                            'eld_id' => $driver->samsara_id != '' ? $driver->samsara_id : '',
                            'user_type' => 'driver',
                            'guard' => 'driver-admin',
                            'toname' => ucwords($driver->first_name.' '.$driver->last_name)
                        ];
                    }

                    $response['success'] = true;
                    $response['data'] = $data;
                    return response()->json($response);*/
                   //** }else if( auth()->guard('company-admin')->user()->role_id == 2){
            } else if( auth()->guard('company-admin')->check()){
                    $companyDispatchers = CompanyAdmin::company()->where('id', '!=', auth()->guard('company-admin')->user()->id)->whereNotIn('email', $selectedEmails)->get();
                    $companyDrivers = Driver::where( 'company_id', auth()->guard('company-admin')->user()->company_id )->whereNotIn('email', $selectedEmails)->get();

                    $companyBrokers = Broker::where( 'company_id', auth()->guard('company-admin')->user()->company_id )->whereNotNull('email')->whereNotIn('email', $selectedEmails)->get();
                    foreach ($companyBrokers as $companyBroker){
                        $data[] = [
                            'toid' => $companyBroker->id,
                            'uniqueID' => 'broker_'.$companyBroker->id,
                            'email' => $companyBroker->email,
                            'profilePic' => null,
                            'user_type' => 'broker',
                            'guard' => 'company-admin',
                            'toname' => ucwords($companyBroker->name)
                        ];

                        $companyBrokerContacts = $companyBroker->broker_contact()->whereNotIn('email', $selectedEmails)->whereNotNull('email')->get();
                        if($companyBrokerContacts->isNotEmpty()){
                            foreach ($companyBrokerContacts as $key => $companyBrokerContact) {
                                $data[] = [
                                    'toid' => $companyBrokerContact->id,
                                    'uniqueID' => 'broker_contact_'.$companyBrokerContact->id,
                                    'email' => $companyBrokerContact->email,
                                    'profilePic' => null,
                                    'user_type' => 'broker_contact',
                                    'guard' => 'company-admin',
                                    'toname' => ucwords($companyBrokerContact->name)
                                ];
                            }
                        }
                    }

                    $companyCustomers = Customer::where( 'company_id', auth()->guard('company-admin')->user()->company_id )->whereNotIn('email', $selectedEmails)->whereNotNull('email')->get();
                    foreach ($companyCustomers as $companyCustomer){
                        $data[] = [
                            'toid' => $companyCustomer->id,
                            'uniqueID' => 'shipper_consignee_'.$companyCustomer->id,
                            'email' => $companyCustomer->email,
                            'profilePic' => null,
                            'user_type' => 'shipper_consignee',
                            'guard' => 'company-admin',
                            'toname' => ucwords($companyCustomer->name)
                        ];
                    }


                    // dd("Hello",$data);
                    //shippers-and-consignees
                    //whereNotNull('trailer_number')
                    //name
                    //phone_number
                    //customer => phone

                    // $data = array();
                    foreach ($companyDispatchers as $companyDispatcher){
                        $type = $companyDispatcher->type == 0 ? 'company_admin' : ( $companyDispatcher->type == 1 ? 'manager' : 'dispatcher');
                        $data[] = [
                            'toid' => $companyDispatcher->id,
                            // 'uniqueID' => 'dispatcher_'.$companyDispatcher->id,
                            'uniqueID' => $type.'_'.$companyDispatcher->id,
                            'email' => $companyDispatcher->email,
                            'profilePic' => $companyDispatcher->profile_image,
                            // 'user_type' => 'dispatcher',
                            'user_type' => $type,
                            'guard' => 'company-admin',
                            // 'toname' => ucwords($companyDispatcher->first_name.' '.$companyDispatcher->last_name)
                            'toname' => ucwords($companyDispatcher->first_name.' '.$companyDispatcher->last_name)
                        ];
                    }

                    foreach( $companyDrivers as $driver ) {
                        $data[] = [
                            'toid' => $driver->id,
                            'uniqueID' => 'driver_'.$driver->id,
                            'email' => $driver->email,
                            'profilePic' => $driver->profile_image != '' ? $driver->profile_image : '',
                            'eld_id' => $driver->samsara_id != '' ? $driver->samsara_id : '',
                            'user_type' => 'driver',
                            'guard' => 'driver-admin',
                            'toname' => ucwords($driver->first_name.' '.$driver->last_name)
                        ];
                    }
                    // dd("Hello",$data);
                    $response['success'] = true;
                    $response['data'] = $data;
                    return response()->json($response);
                //** }
            } else {
                $response['success'] = false;
                $response['data'] = [];
                $response['msg'] = "Unauthenticated";
                return response()->json($response);
            }
        } catch( \Exception $e ) {
            \Log::info( "File: ".$e->getFile() );
            \Log::info( "Line: ".$e->getLine() );
            \Log::info( "Message: ".$e->getMessage() );
        }
    }

    public function loadDetail(Request $request){
        $response['success']                                    = false;
        $response['message']                                    = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                if($request->has('load_number')){
                    $item                                       = LoadDetail::with(['driver','broker'])
                    ->where('load_number',$request->load_number)->first();
                    if(!is_null($item)){
                        $query                                  = LoadNote::where('load_id',$item->load_id);
                        $getNote                                = $query->orderBy('created_at','asc')->get(['note','document']);
                        // dd( $getNote);
                        $mapOption                              = [
                            'tollRoadsOption'                   => $item->map_toll_roads ?? null,
                            'elevLimit'                         => $item->map_max_elevation ?? null,
                            'avoidCountry'                      => $item->map_close_borders ?? null
                        ];
                        $routeLngLat                            = [];
                        if($item->shipper_address_verified==1){
                            $routeLngLat['pickup']              = array(
                                'longitude'                     => $item->shipper_longitude,
                                'latitude'                      => $item->shipper_latitude
                            );
                        }
                        if(isset($item->loads)){
                            if($item->loads->load_stops->isNotEmpty()){
                                foreach ($item->loads->load_stops as $key => $stop) {
                                    if($stop->address_verified==1){
                                        $routeLngLat['stop_'.++$key] = array(
                                            'longitude'         => $stop->longitude,
                                            'latitude'          => $stop->latitude
                                        );
                                    }
                                }
                            }
                        }
                        if($item->delivery_address_verified==1){
                            $routeLngLat['delivery']            = array(
                                'longitude'                     => $item->delivery_longitude,
                                'latitude'                      => $item->delivery_latitude
                            );
                        }
                        $item->load_status                      = getLoadStatus($item->load_status,$item->is_on_hold);
                        $response['item']                       = $item;
                        if(isset($item->broker->id)){
                            $response['brokerLink']             = route('company.admin.brokers.edit', base64Encode($item->broker->id));
                        }
                        if(isset($item->truck->id)){
                            $response['truckLink']              = route('company.admin.trucks.show', base64Encode($item->truck->id));
                        }
                        if(isset($item->trailer->id)){
                            $response['trailerLink']            = route('company.admin.trailers.show', base64Encode($item->trailer->id));
                        }
                        $response['gtypeLabel']                 = 'Not specified';
                        if (isset($item->trailer_type) && $item->trailer_type!='' && isset(getTrailerTypeList()[$item->trailer_type])) {
                            $response['gtypeLabel']             = getTrailerTypeList()[$item->trailer_type];
                        }
                        $response['mapOption']                  = $mapOption;
                        $response['routeLngLat']                = $routeLngLat;
                        $response['truckLngLat']                = $this->loadCurrentLocation($item);
                        $response['stopView']                   = $this->loadStopView($item);
                        $response['fileView']                   = $this->loadFileView($item);
                        $noteAttachments                        = [];
                        // $notes                               = [];
                        $notesDetails                           = [];
                        foreach($getNote as $key => $value){
                            // $notes                           = $value->note;
                            if($value->document != ""){
                                $string                         = $value->document;
                                $start                          = ':"';
                                $end                            = '";';
                                $occurrences                    = [];

                                while (($posStart = strpos($string, $start)) !== false && ($posEnd = strpos($string, $end, $posStart + strlen($start))) !== false) {
                                    $occurrence                 = substr($string, $posStart + strlen($start), $posEnd - $posStart - strlen($start));
                                    $occurrences[]              = $occurrence;
                                    $string                     = substr($string, $posEnd + strlen($end));
                                }
                                $noteAttachments[$key]          = $occurrences;
                            }
                            else{
                                $noteAttachments[$key]          = [];
                            }
                            $notesDetails[$key]['note']         = $value->note;
                            $notesDetails[$key]['documents']    = $noteAttachments[$key];
                        }
                        $response['notes']                      = view('companyAdmin.chat.load.note.view', compact('notesDetails'))->render();
                        $response['success']                    = true;
                        $response['message']                    = "Load detail retrieved successfully.";
                    }
                }
            }catch (\Exception $e) {
                $response['success']                            = false;
                $response['message']                            = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function loadInternalDetail(Request $request){
        $response['success']                        = false;
        $response['message']                        = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                if($request->has('load_number')){
                    $item                           = LoadDetail::where('load_number',$request->load_number)->first(['load_id','actual_delivery_date']);
                    if(!is_null($item)){
                        $getLoadInternal            = ChatLoadInternal::where('unique_id',"load_".$item->load_id)->first(['sid','friendly_name','status']);
                        // dd($item->load_id,$getLoadInternal);
                        // dd( $getNote);
                        if(!is_null($getLoadInternal)){
                            $response['load_id']    = $item->load_id;
                            $response['internal']   = $getLoadInternal;
                            $response['success']    = true;
                            $response['message']    = "Load detail internal fetched successfully.";
                        }
                        else {
                            $attr = $item->getAttributes();
                            // dd($attr);
                            $pickupDate             = Carbon::parse(date('Y-m-d', strtotime($attr['actual_delivery_date'])));
                            $fromDate               = Carbon::now()->format('Y-m-d');
                            $fromDate               = Carbon::parse($fromDate);
                            // $toDate              = $pickupDate->addDays(2);
                            $toDate                 = $pickupDate->addDays(4);
                            $diffInDays             = $fromDate->diffInDays($toDate);
                            $response['load_id']    = $item->load_id;
                            $response['internal']   = [];
                            $response['ttl']        = $diffInDays;
                            $response['company_id'] = $item->loads->company_id;
                            $response['success']    = true;
                            $response['message']    = "Load detail internal do not exists.";
                        }
                    }
                    else {
                        $response['success']        = false;
                        $response['message']        = "Load details does not exists";
                    }
                }
            }catch (\Exception $e) {
                $response['success']                = false;
                $response['message']                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function loadTruckLocationUpdate(Request $request){
        $response['success']                        = false;
        $response['message']                        = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                if($request->has('load_number')){
                    $item                           = LoadDetail::with(['driver','broker'])
                        ->where('load_number',$request->load_number)->first();
                    if(!is_null($item)){
                        $mapOption                  = [
                            'tollRoadsOption'       => $item->map_toll_roads ?? null,
                            'elevLimit'             => $item->map_max_elevation ?? null,
                            'avoidCountry'          => $item->map_close_borders ?? null
                        ];
                        $routeLngLat                = [];
                        if($item->shipper_address_verified==1){
                            $routeLngLat['pickup']  = array(
                                'longitude'         => $item->shipper_longitude,
                                'latitude'          => $item->shipper_latitude

                            );
                        }
                        if(isset($item->loads)){
                            if($item->loads->load_stops->isNotEmpty()){
                                foreach ($item->loads->load_stops as $key => $stop) {
                                    if($stop->address_verified==1){
                                        $routeLngLat['stop_'.++$key] = array(
                                            'longitude'         => $stop->longitude,
                                            'latitude'          => $stop->latitude

                                        );
                                    }
                                }
                            }
                        }
                        if($item->delivery_address_verified==1){
                            $routeLngLat['delivery']= array(
                                'longitude'         => $item->delivery_longitude,
                                'latitude'          => $item->delivery_latitude

                            );
                        }
                        $response['mapOption']      = $mapOption;
                        $response['routeLngLat']    = $routeLngLat;
                        $response['truckLngLat']    = $this->loadCurrentLocation($item);
                        $response['success']        = true;
                        $response['message']        = "Load truck retrieved successfully.";
                    }
                }
            }catch (\Exception $e) {
                $response['success']                = false;
                $response['message']                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function loadCurrentLocation($loadDetail){
        $array                              = [];
        if(isset($loadDetail->truck)){
            // eld to get truck location
            if(isset($loadDetail->truck->samsara_id)){
                $getAPIKey                  = IntegrationSetting::where('company_id', $loadDetail->truck->company_id)->where('carrier_id', $loadDetail->truck->carrier_id)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_vehicles', '1')->first();
                if(!is_null($getAPIKey)){
                    try {
                        $url                = 'https://api.samsara.com/fleet/vehicles/locations?vehicleIds='.$loadDetail->truck->samsara_id;
                        $client             = new Client();
                        $api_response       = $client->request('GET', $url, [
                            'headers'                   => [
                                'Accept'                => 'application/json',
                                'Authorization'         => 'Bearer '.$getAPIKey->api_key,
                            ],
                        ]);
                        // Check if response status code is OK (200)
                        if ($api_response->getStatusCode() == 200) {
                            $body           = json_decode($api_response->getBody());
                            // Do something with the response body
                            if(isset($body)){
                                $data       = isset($body->data[0])?$body->data[0]:null;
                                if(isset($data)){
                                    if(isset($data->location)){
                                        $location       = $data->location;
                                        $array          = array(
                                            'longitude'         => $location->longitude,
                                            'latitude'          => $location->latitude,
                                            'address'           => $location->reverseGeo->formattedLocation,
                                            'last_updated'      => Carbon::parse($location->time)->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId
                                        );
                                    }
                                }
                            }
                        }
                    } catch (ClientException $e) {
                        // Handle client-side errors (e.g. 4xx status codes)
                        $api_response       = $e->getResponse();
                        if ($api_response) {
                            $statusCode     = $api_response->getStatusCode();
                            $body           = $api_response->getBody()->getContents();
                            // Log::error($body);
                        } else {
                            // Log::error($e->getMessage());
                        }
                    } catch (ServerException $e) {
                        // Handle server-side errors (e.g. 5xx status codes)
                        $api_response       = $e->getResponse();
                        if ($response) {
                            $statusCode     = $api_response->getStatusCode();
                            $body           = $api_response->getBody()->getContents();
                            // Log::error($body);
                        } else {
                            // Log::error($e->getMessage());
                        }
                    } catch (\Exception $e) {
                        // Handle other exceptions
                        // Log::error($e->getMessage());
                    }
                }
            }
            //
            if(empty($array)){
                if($loadDetail->shipper_address_verified==1 && $loadDetail->shipper_is_arrived==1){
                    $array                  = array(
                        'longitude'         => $loadDetail->shipper_longitude,
                        'latitude'          => $loadDetail->shipper_latitude,
                    );
                    $address                = [];
                    if(isset($loadDetail->shipper_address_one) && $loadDetail->shipper_address_one!==''){
                        $address[]          = $loadDetail->shipper_address_one;
                    }
                    if(isset($loadDetail->shipper_address_two) && $loadDetail->shipper_address_two!==''){
                        $address[]          = $loadDetail->shipper_address_two;
                    }
                    if(isset($loadDetail->shipper_city) && $loadDetail->shipper_city!==''){
                        $address[]          = $loadDetail->shipper_city;
                    }
                    if(isset($loadDetail->shipper_state) && $loadDetail->shipper_state!==''){
                        $address[]          = $loadDetail->shipper_state;
                    }
                    if(isset($loadDetail->shipper_zip) && $loadDetail->shipper_zip!==''){
                        $address[]          = $loadDetail->shipper_zip;
                    }
                    $array['address']       = implode(', ', $address);
                    if($loadDetail->shipper_is_leaves==1){
                        $last_updated       = isset($loadDetail->shipper_leaves_at)?convertTimeToUSERzone([
                            'str'           => $loadDetail->shipper_leaves_at,
                            'toFormat'      => getTimeLayout(),
                            'fromFormat'    => 'Y-m-d H:i:s',
                            'userTimezone'  => $this->timeZone
                        ]).' '.$this->timeZoneId:'';
                    }else{
                        $last_updated       = isset($loadDetail->shipper_arrived_at)?convertTimeToUSERzone([
                            'str'           => $loadDetail->shipper_arrived_at,
                            'toFormat'      => getTimeLayout(),
                            'fromFormat'    => 'Y-m-d H:i:s',
                            'userTimezone'  => $this->timeZone
                        ]).' '.$this->timeZoneId:'';
                    }
                    $array['last_updated']  = $last_updated;
                }
                if(isset($loadDetail->loads)){
                    if($loadDetail->loads->load_stops->isNotEmpty()){
                        foreach ($loadDetail->loads->load_stops as $key => $stop) {
                            if($stop->address_verified==1 && $stop->is_arrived==1){
                                $array                  = array(
                                    'longitude'         => $stop->longitude,
                                    'latitude'          => $stop->latitude,
                                );
                                $address                = [];
                                if(isset($stop->address_one) && $stop->address_one!==''){
                                    $address[]          = $stop->address_one;
                                }
                                if(isset($stop->address_two) && $stop->address_two!==''){
                                    $address[]          = $stop->address_two;
                                }
                                if(isset($stop->city) && $stop->city!==''){
                                    $address[]          = $stop->city;
                                }
                                if(isset($stop->state) && $stop->state!==''){
                                    $address[]          = $stop->state;
                                }
                                if(isset($stop->zip) && $stop->zip!==''){
                                    $address[]          = $stop->zip;
                                }
                                $array['address']       = implode(', ', $address);
                                if($stop->is_leaves==1){
                                    $last_updated       = isset($stop->leaves_at)?convertTimeToUSERzone([
                                        'str'           => $stop->leaves_at,
                                        'toFormat'      => getTimeLayout(),
                                        'fromFormat'    => 'Y-m-d H:i:s',
                                        'userTimezone'  => $this->timeZone
                                    ]).' '.$this->timeZoneId:'';
                                }else{
                                    $last_updated       = isset($stop->arrived_at)?convertTimeToUSERzone([
                                        'str'           => $stop->arrived_at,
                                        'toFormat'      => getTimeLayout(),
                                        'fromFormat'    => 'Y-m-d H:i:s',
                                        'userTimezone'  => $this->timeZone
                                    ]).' '.$this->timeZoneId:'';
                                }
                                $array['last_updated']  = $last_updated;
                            }
                        }
                    }
                }
                if($loadDetail->delivery_address_verified==1 && $loadDetail->delivery_is_arrived==1){
                    $array['load']          = array(
                        'longitude'         => $loadDetail->delivery_longitude,
                        'latitude'          => $loadDetail->delivery_latitude,
                    );
                    $address                = [];
                    if(isset($loadDetail->delivery_address_one) && $loadDetail->delivery_address_one!==''){
                        $address[]          = $loadDetail->delivery_address_one;
                    }
                    if(isset($loadDetail->delivery_address_two) && $loadDetail->delivery_address_two!==''){
                        $address[]          = $loadDetail->delivery_address_two;
                    }
                    if(isset($loadDetail->delivery_city) && $loadDetail->delivery_city!==''){
                        $address[]          = $loadDetail->delivery_city;
                    }
                    if(isset($loadDetail->delivery_state) && $loadDetail->delivery_state!==''){
                        $address[]          = $loadDetail->delivery_state;
                    }
                    if(isset($loadDetail->delivery_zip) && $loadDetail->delivery_zip!==''){
                        $address[]          = $loadDetail->delivery_zip;
                    }
                    $array['address']       = implode(', ', $address);
                    if($loadDetail->delivery_is_leaves==1){
                        $last_updated       = isset($loadDetail->delivery_leaves_at)?convertTimeToUSERzone([
                            'str'           => $loadDetail->delivery_leaves_at,
                            'toFormat'      => getTimeLayout(),
                            'fromFormat'    => 'Y-m-d H:i:s',
                            'userTimezone'  => $this->timeZone
                        ]).' '.$this->timeZoneId:'';
                    }else{
                        $last_updated       = isset($loadDetail->delivery_arrived_at)?convertTimeToUSERzone([
                            'str'           => $loadDetail->delivery_arrived_at,
                            'toFormat'      => getTimeLayout(),
                            'fromFormat'    => 'Y-m-d H:i:s',
                            'userTimezone'  => $this->timeZone
                        ]).' '.$this->timeZoneId:'';
                    }
                    $array['last_updated']  = $last_updated;
                }
            }
            if(!empty($array)){
                $array['truck_number']      = $loadDetail->truck_number;
            }
        }
        return $array;
    }

    public function loadStopView($loadDetail){
        $getTimeZoneId                      = $this->timeZoneId;
        return view('companyAdmin.chat.load.stop.view', compact('loadDetail','getTimeZoneId'))->render();
    }

    public function loadFileView($loadDetail){
        $getTimeLayout                                              = getTimeLayout();

        $loadStop                                                   = LoadStop::where('load_id',$loadDetail->load_id)->orderBy('stop_no','ASC')->orderBy('date_time','ASC')->get();
        $static_files                                               = [];
        if(isset($loadDetail->invoice_files) && is_serialized($loadDetail->invoice_files) && !empty(unserialize($loadDetail->invoice_files))){
            $value                                                  = unserialize($loadDetail->invoice_files);
            $array                                                  = [];
            $array['id']                                            = $loadDetail->id;
            $array['source']                                        = 'load_details';
            $array['field_name']                                    = 'invoice_files';

            $array['name']                                          = 'Load';
            $array['filename']                                      = 'Invoice';
            $array['include_w_billing']                             = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
            $array['hide_from_driver']                              = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

            $status                                                 = 'Pending';
            $array['file']                                          = $value['file'] ?? null;
            if(isset($value['file']) && !empty($value['file'])){
                $status                                             = 'Complete';
                $array['upload_user_type']                          = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                $array['uploaded_by']                               = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                $array['uploaded_at']                               = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
            }
            $array['status']                                        = $status;

            $static_files[]                                         = $array;
        }else{
            $array                                                  = [];
            $array['id']                                            = $loadDetail->id;
            $array['source']                                        = 'load_details';
            $array['field_name']                                    = 'invoice_files';

            $array['name']                                          = 'Load';
            $array['filename']                                      = 'Invoice';
            $array['include_w_billing']                             = 'no';
            $array['hide_from_driver']                              = 'no';

            $array['file']                                          = null;
            $array['status']                                        = 'Pending';

            $static_files[]                                         = $array;
        }

        if(isset($loadDetail->rate_confirmation_files) && is_serialized($loadDetail->rate_confirmation_files) && !empty(unserialize($loadDetail->rate_confirmation_files))){
            $value                                                  = unserialize($loadDetail->rate_confirmation_files);
            $array                                                  = [];
            $array['id']                                            = $loadDetail->id;
            $array['source']                                        = 'load_details';
            $array['field_name']                                    = 'rate_confirmation_files';

            $array['name']                                          = 'Load';
            $array['filename']                                      = 'Rate Confirmation';
            $array['include_w_billing']                             = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
            $array['hide_from_driver']                              = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

            $status                                                 = 'Pending';
            $array['file']                                          = $value['file'] ?? null;
            if(isset($value['file']) && !empty($value['file'])){
                $status                                             = 'Complete';
                $array['upload_user_type']                          = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                $array['uploaded_by']                               = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                $array['uploaded_at']                               = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
            }
            $array['status']                                        = $status;

            $static_files[]                                         = $array;
        }else{
            $array                                                  = [];
            $array['id']                                            = $loadDetail->id;
            $array['source']                                        = 'load_details';
            $array['field_name']                                    = 'rate_confirmation_files';

            $array['name']                                          = 'Load';
            $array['filename']                                      = 'Rate Confirmation';
            $array['include_w_billing']                             = 'no';
            $array['hide_from_driver']                              = 'no';

            $array['file']                                          = null;
            $array['status']                                        = 'Pending';

            $static_files[]                                         = $array;
        }

        $required_files                                             = [];
        $requested_files                                            = [];
        //
        if(isset($loadDetail->shipper_files_upload) && is_serialized($loadDetail->shipper_files_upload) && !empty(unserialize($loadDetail->shipper_files_upload))){
            $uploads                                                = unserialize($loadDetail->shipper_files_upload);
            $uploads                                                = array_filter($uploads, function ($var) {
                return (isset($var['name_id']) && $var['name_id'] !== null && $var['type'] == 1);
            });
            foreach ($uploads as $key => $value) {
                $array                                              = [];
                $array['id']                                        = $loadDetail->id;
                $array['source']                                    = 'load_details';
                $array['field_name']                                = 'shipper_files_upload';

                $array['name']                                      = 1;
                $array['filename']                                  = $value['name'] ?? '';
                $array['filename_id']                               = $value['name_id'] ?? '';
                $array['include_w_billing']                         = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
                $array['hide_from_driver']                          = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

                $status                                             = 'Pending';
                $array['file']                                      = $value['file'] ?? null;
                if(isset($value['file']) && !empty($value['file'])){
                    $status                                         = 'Complete';
                    $array['upload_user_type']                      = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                    $array['uploaded_by']                           = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                    $array['uploaded_at']                           = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                }else{
                    $file_not_provided                              = (isset($value['file_not_provided']) && $value['file_not_provided']=="on")? "on" : "off";
                    if($file_not_provided == 'on'){
                        $status                                     = 'Not provided';
                    }
                }
                $array['status']                                    = $status;

                $required_files[]                                   = $array;
            }
        }
        if(isset($loadDetail->shipper_load_document_request_items) && $loadDetail->shipper_load_document_request_items->isNotEmpty()){
            foreach ($loadDetail->shipper_load_document_request_items as $key => $item) {
                $value                                              = [];
                if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                    $value                                          = unserialize($item->files);
                }
                if(!empty($value)){
                    if(
                        isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1])
                    ){
                        $array                                      = [];
                        $array['id']                                = $item->id;
                        $array['source']                            = 'load_document_request_items';
                        $array['field_name']                        = 'files';

                        $array['name']                              = 1;
                        $array['filename']                          = $value['name'] ?? '';
                        $array['filename_id']                       = $value['name_id'] ?? '';
                        $array['include_w_billing']                 = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
                        $array['hide_from_driver']                  = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

                        $status                                     = 'Pending';
                        $array['file']                              = $value['file'] ?? null;
                        if(isset($value['file']) && !empty($value['file'])){
                            $status                                 = 'Complete';
                            $array['upload_user_type']              = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                            $array['uploaded_by']                   = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                            $array['uploaded_at']                   = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                        }else{
                            $file_not_provided                      = (isset($value['file_not_provided']) && $value['file_not_provided']=="on")? "on" : "off";
                            if($file_not_provided == 'on'){
                                $status                             = 'Not provided';
                            }
                        }
                        $array['status']                            = $status;

                        $requested_files[]                          = $array;
                    }
                }
            }
        }

        $stop_no                                                    = 2;
        foreach ($loadStop as $key => $stop) {
            //
            $customer_type                                          = $stop->customer_type;
            $field_name                                             = $customer_type=='1'?'stop_shipper_files_upload':'stop_consignee_files_upload';
            if(isset($stop->$field_name) && is_serialized($stop->$field_name) && !empty(unserialize($stop->$field_name))){
                $uploads                                            = unserialize($stop->$field_name);
                $uploads                                            = array_filter($uploads, function ($var) {
                    return (isset($var['name_id']) && $var['name_id'] !== null && $var['type'] == 1);
                });
                foreach ($uploads as $key => $value) {
                    $array                                          = [];
                    $array['id']                                    = $stop->id;
                    $array['source']                                = 'load_stops';
                    $array['field_name']                            = $field_name;

                    $array['name']                                  = $stop_no;
                    $array['filename']                              = $value['name'];
                    $array['filename_id']                           = $value['name_id'] ?? '';
                    $array['include_w_billing']                     = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
                    $array['hide_from_driver']                      = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

                    $status                                         = 'Pending';
                    $array['file']                                  = $value['file'] ?? null;
                    if(isset($value['file']) && !empty($value['file'])){
                        $status                                     = 'Complete';
                        $array['upload_user_type']                  = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                        $array['uploaded_by']                       = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                        $array['uploaded_at']                       = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                    }else{
                        $file_not_provided                          = (isset($value['file_not_provided']) && $value['file_not_provided']=="on")? "on" : "off";
                        if($file_not_provided == 'on'){
                            $status                                 = 'Not provided';
                        }
                    }
                    $array['status']                                = $status;

                    $required_files[]                               = $array;
                }
            }
            if(isset($stop->load_document_request_items) && $stop->load_document_request_items->isNotEmpty()){
                foreach ($stop->load_document_request_items as $key => $item) {
                    $value                                          = [];
                    if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                        $value                                      = unserialize($item->files);
                    }
                    if(!empty($value)){
                        if(
                            isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1])
                        ){
                            $array                                  = [];
                            $array['id']                            = $item->id;
                            $array['source']                        = 'load_document_request_items';
                            $array['field_name']                    = 'files';

                            $array['name']                          = $stop_no;
                            $array['filename']                      = $value['name'] ?? '';
                            $array['filename_id']                   = $value['name_id'] ?? '';
                            $array['include_w_billing']             = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
                            $array['hide_from_driver']              = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

                            $status                                 = 'Pending';
                            $array['file']                          = $value['file'] ?? null;
                            if(isset($value['file']) && !empty($value['file'])){
                                $status                             = 'Complete';
                                $array['upload_user_type']          = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                                $array['uploaded_by']               = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                                $array['uploaded_at']               = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                            }else{
                                $file_not_provided                  = (isset($value['file_not_provided']) && $value['file_not_provided']=="on")? "on" : "off";
                                if($file_not_provided == 'on'){
                                    $status                         = 'Not provided';
                                }
                            }
                            $array['status']                        = $status;

                            $requested_files[]                      = $array;
                        }
                    }
                }
            }
            //
            ++$stop_no;
        }

        //
        if(isset($loadDetail->consignee_files_upload) && is_serialized($loadDetail->consignee_files_upload) && !empty(unserialize($loadDetail->consignee_files_upload))){
            $uploads                                                = unserialize($loadDetail->consignee_files_upload);
            $uploads                                                = array_filter($uploads, function ($var) {
                return (isset($var['name_id']) && $var['name_id'] !== null && $var['type'] == 1);
            });
            foreach ($uploads as $key => $value) {
                $array                                              = [];
                $array['id']                                        = $loadDetail->id;
                $array['source']                                    = 'load_details';
                $array['field_name']                                = 'consignee_files_upload';

                $array['name']                                      = 99;
                $array['filename']                                  = $value['name'];
                $array['filename_id']                               = $value['name_id'] ?? '';
                $array['include_w_billing']                         = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
                $array['hide_from_driver']                          = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

                $status                                             = 'Pending';
                $array['file']                                      = $value['file'] ?? null;
                if(isset($value['file']) && !empty($value['file'])){
                    $status                                         = 'Complete';
                    $array['upload_user_type']                      = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                    $array['uploaded_by']                           = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                    $array['uploaded_at']                           = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                }else{
                    $file_not_provided                              = (isset($value['file_not_provided']) && $value['file_not_provided']=="on")? "on" : "off";
                    if($file_not_provided == 'on'){
                        $status                                     = 'Not provided';
                    }
                }
                $array['status']                                    = $status;

                $required_files[]                                   = $array;
            }
        }
        if(isset($loadDetail->receiver_load_document_request_items) && $loadDetail->receiver_load_document_request_items->isNotEmpty()){
            foreach ($loadDetail->receiver_load_document_request_items as $key => $item) {
                $value                                              = [];
                if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                    $value                                          = unserialize($item->files);
                }
                if(!empty($value)){
                    if(
                        isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1])
                    ){
                        $array                                      = [];
                        $array['id']                                = $item->id;
                        $array['source']                            = 'load_document_request_items';
                        $array['field_name']                        = 'files';

                        $array['name']                              = 99;
                        $array['filename']                          = $value['name'] ?? '';
                        $array['filename_id']                       = $value['name_id'] ?? '';
                        $array['include_w_billing']                 = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
                        $array['hide_from_driver']                  = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

                        $status                                     = 'Pending';
                        $array['file']                              = $value['file'] ?? null;
                        if(isset($value['file']) && !empty($value['file'])){
                            $status                                 = 'Complete';
                            $array['upload_user_type']              = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                            $array['uploaded_by']                   = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                            $array['uploaded_at']                   = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                        }else{
                            $file_not_provided                      = (isset($value['file_not_provided']) && $value['file_not_provided']=="on")? "on" : "off";
                            if($file_not_provided == 'on'){
                                $status                             = 'Not provided';
                            }
                        }
                        $array['status']                            = $status;

                        $requested_files[]                          = $array;
                    }
                }
            }
        }
        //

        $accessorial_files                                          = [];
        $shipper_accessorials                                       = $loadDetail->shipper_accessorials()->where('doc_required', '!=', 0)->get();
        if($shipper_accessorials->isNotEmpty()){
            foreach ($shipper_accessorials as $key => $load_accessorial) {
                $gallery                                            = Gallery::where('company_id', auth()->guard($this->logged_in_guard)->user()->company_id)
                    ->where('action_id', $load_accessorial->id)
                    ->where('source', 'load_accessorials')
                    ->where('field_name', 'accessorial_files')
                    ->first();
                $filename                                           = $load_accessorial->company_accessorial->name;
                if(!is_null($gallery)){
                    $filename                                      .= ' - '.$gallery->file_name;
                }
                $array                                              = [];
                $array['id']                                        = $load_accessorial->id;
                $array['source']                                    = 'load_accessorials';
                $array['field_name']                                = 'image';

                $array['name']                                      = 1;
                $array['filename']                                  = $filename;
                $array['include_w_billing']                         = isset($load_accessorial->include_w_billing) && $load_accessorial->include_w_billing=='yes' ? 'yes' : 'no';
                $array['hide_from_driver']                          = isset($load_accessorial->hide_from_driver) && $load_accessorial->hide_from_driver=='yes' ? 'yes' : 'no';

                $array['file']                                      = $gallery->image ?? null;
                //status
                if(isset($load_accessorial->temp_details) && is_serialized($load_accessorial->temp_details) && !empty(unserialize($load_accessorial->temp_details))){
                    $temp_details                                   = unserialize($load_accessorial->temp_details);
                    if(isset($temp_details['status'])){
                        $load_accessorial->status                   = $temp_details['status'];
                    }
                }
                $status                                             = 'Pending';
                if($load_accessorial->status=='1'){
                    $status                                         = 'Complete';
                }elseif ($load_accessorial->status=='2') {
                    $status                                         = 'Driver Bypass';
                }
                //

                if(isset($gallery->image) && !empty($gallery->image)){
                    $array['upload_user_type']                      = isset($gallery->created_by_type)?$gallery->created_by_type:null;
                    $array['uploaded_by']                           = isset($gallery->created_by)?getUserByType($gallery->created_by,$gallery->created_by_type):null;
                    $array['uploaded_at']                           = isset($gallery->created_at)?Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                }
                $array['status']                                    = $status;

                $accessorial_files[]                                = $array;
            }
        }

        $accessorial_no                                             = 2;
        foreach ($loadStop as $key => $stop) {
            $stop_accessorials                                      = $stop->accessorials()->where('doc_required', '!=', 0)->get();
            if($stop_accessorials->isNotEmpty()){
                foreach ($stop_accessorials as $key => $load_accessorial) {
                    $gallery                                        = Gallery::where('company_id', auth()->guard($this->logged_in_guard)->user()->company_id)
                        ->where('action_id', $load_accessorial->id)
                        ->where('source', 'load_accessorials')
                        ->where('field_name', 'accessorial_files')
                        ->first();
                    $filename                                       = $load_accessorial->company_accessorial->name;
                    if(!is_null($gallery)){
                        $filename                                  .= ' - '.$gallery->file_name;
                    }
                    $array                                          = [];
                    $array['id']                                    = $load_accessorial->id;
                    $array['source']                                = 'load_accessorials';
                    $array['field_name']                            = 'image';

                    $array['name']                                  = $accessorial_no;
                    $array['filename']                              = $filename;
                    $array['include_w_billing']                     = isset($load_accessorial->include_w_billing) && $load_accessorial->include_w_billing=='yes' ? 'yes' : 'no';
                    $array['hide_from_driver']                      = isset($load_accessorial->hide_from_driver) && $load_accessorial->hide_from_driver=='yes' ? 'yes' : 'no';

                    $array['file']                                  = $gallery->image ?? null;
                    //status
                    if(isset($load_accessorial->temp_details) && is_serialized($load_accessorial->temp_details) && !empty(unserialize($load_accessorial->temp_details))){
                        $temp_details                               = unserialize($load_accessorial->temp_details);
                        if(isset($temp_details['status'])){
                            $load_accessorial->status               = $temp_details['status'];
                        }
                    }
                    $status                                         = 'Pending';
                    if($load_accessorial->status=='1'){
                        $status                                     = 'Complete';
                    }elseif ($load_accessorial->status=='2') {
                        $status                                     = 'Driver Bypass';
                    }
                    //

                    if(isset($gallery->image) && !empty($gallery->image)){
                        $array['upload_user_type']                  = isset($gallery->created_by_type)?$gallery->created_by_type:null;
                        $array['uploaded_by']                       = isset($gallery->created_by)?getUserByType($gallery->created_by,$gallery->created_by_type):null;
                        $array['uploaded_at']                       = isset($gallery->created_at)?Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                    }
                    $array['status']                                = $status;

                    $accessorial_files[]                            = $array;
                }
            }
            ++$accessorial_no;
        }

        $receiver_accessorials                                      = $loadDetail->receiver_accessorials()->where('doc_required', '!=', 0)->get();
        if($receiver_accessorials->isNotEmpty()){
            foreach ($receiver_accessorials as $key => $load_accessorial) {
                $gallery                                            = Gallery::where('company_id', auth()->guard($this->logged_in_guard)->user()->company_id)
                    ->where('action_id', $load_accessorial->id)
                    ->where('source', 'load_accessorials')
                    ->where('field_name', 'accessorial_files')
                    ->first();
                $filename                                           = $load_accessorial->company_accessorial->name;
                if(!is_null($gallery)){
                    $filename                                      .= ' - '.$gallery->file_name;
                }
                $array                                              = [];
                $array['id']                                        = $load_accessorial->id;
                $array['source']                                    = 'load_accessorials';
                $array['field_name']                                = 'image';

                $array['name']                                      = 99;
                $array['filename']                                  = $filename;
                $array['include_w_billing']                         = isset($load_accessorial->include_w_billing) && $load_accessorial->include_w_billing=='yes' ? 'yes' : 'no';
                $array['hide_from_driver']                          = isset($load_accessorial->hide_from_driver) && $load_accessorial->hide_from_driver=='yes' ? 'yes' : 'no';

                $array['file']                                      = $gallery->image ?? null;
                //status
                if(isset($load_accessorial->temp_details) && is_serialized($load_accessorial->temp_details) && !empty(unserialize($load_accessorial->temp_details))){
                    $temp_details                                   = unserialize($load_accessorial->temp_details);
                    if(isset($temp_details['status'])){
                        $load_accessorial->status                   = $temp_details['status'];
                    }
                }
                $status                                             = 'Pending';
                if($load_accessorial->status=='1'){
                    $status                                         = 'Complete';
                }elseif ($load_accessorial->status=='2') {
                    $status                                         = 'Driver Bypass';
                }
                //

                if(isset($gallery->image) && !empty($gallery->image)){
                    $array['upload_user_type']                      = isset($gallery->created_by_type)?$gallery->created_by_type:null;
                    $array['uploaded_by']                           = isset($gallery->created_by)?getUserByType($gallery->created_by,$gallery->created_by_type):null;
                    $array['uploaded_at']                           = isset($gallery->created_at)?Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                }
                $array['status']                                    = $status;

                $accessorial_files[]                                = $array;
            }
        }

        $billing_accessorials                                       = $loadDetail->billing_accessorials()->where('doc_required', '!=', 0)->get();
        if($billing_accessorials->isNotEmpty()){
            foreach ($billing_accessorials as $key => $load_accessorial) {
                $gallery                                            = Gallery::where('company_id', auth()->guard($this->logged_in_guard)->user()->company_id)
                    ->where('action_id', $load_accessorial->id)
                    ->where('source', 'load_accessorials')
                    ->where('field_name', 'accessorial_files')
                    ->first();
                $filename                                           = $load_accessorial->company_accessorial->name;
                if(!is_null($gallery)){
                    $filename                                      .= ' - '.$gallery->file_name;
                }
                $array                                              = [];
                $array['id']                                        = $load_accessorial->id;
                $array['source']                                    = 'load_accessorials';
                $array['field_name']                                = 'image';

                $array['name']                                      = 'Load';
                $array['filename']                                  = $filename;
                $array['include_w_billing']                         = isset($load_accessorial->include_w_billing) && $load_accessorial->include_w_billing=='yes' ? 'yes' : 'no';
                $array['hide_from_driver']                          = isset($load_accessorial->hide_from_driver) && $load_accessorial->hide_from_driver=='yes' ? 'yes' : 'no';

                $array['file']                                      = $gallery->image ?? null;
                //status
                if(isset($load_accessorial->temp_details) && is_serialized($load_accessorial->temp_details) && !empty(unserialize($load_accessorial->temp_details))){
                    $temp_details                                   = unserialize($load_accessorial->temp_details);
                    if(isset($temp_details['status'])){
                        $load_accessorial->status                   = $temp_details['status'];
                    }
                }
                $status                                             = 'Pending';
                if($load_accessorial->status=='1'){
                    $status                                         = 'Complete';
                }elseif ($load_accessorial->status=='2') {
                    $status                                         = 'Driver Bypass';
                }
                //

                if(isset($gallery->image) && !empty($gallery->image)){
                    $array['upload_user_type']                      = isset($gallery->created_by_type)?$gallery->created_by_type:null;
                    $array['uploaded_by']                           = isset($gallery->created_by)?getUserByType($gallery->created_by,$gallery->created_by_type):null;
                    $array['uploaded_at']                           = isset($gallery->created_at)?Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                }
                $array['status']                                    = $status;

                $accessorial_files[]                                = $array;
            }
        }

        $cod_files                                                  = [];
        $get_cod_files                                              = Gallery::where('company_id', auth()->guard($this->logged_in_guard)->user()->company_id)
            ->where('action_id', $loadDetail->load_id)
            ->where('source', 'load')
            ->whereIn('field_name', ['up_front_file', 'at_pickup_file', 'at_delivery_file'])
            ->get();
        if($get_cod_files->isNotEmpty()){
            foreach ($get_cod_files as $key => $cod_file) {
                if(isset($cod_file->image) && $cod_file->image!=''){
                    $array                                          = [];
                    $array['id']                                    = $cod_file->id;
                    $array['source']                                = 'load_cod_galleries';
                    $array['field_name']                            = 'image';

                    $array['name']                                  = 'COD';
                    $array['filename']                              = $cod_file->file_name;
                    $array['include_w_billing']                     = isset($cod_file->include_w_billing) && $cod_file->include_w_billing=='yes' ? 'yes' : 'no';
                    $array['hide_from_driver']                      = isset($cod_file->hide_from_driver) && $cod_file->hide_from_driver=='yes' ? 'yes' : 'no';

                    $array['file']                                  = $cod_file->image ?? null;
                    if(isset($cod_file->image) && !empty($cod_file->image)){
                        $array['upload_user_type']                  = isset($cod_file->created_by_type)?$cod_file->created_by_type:null;
                        $array['uploaded_by']                       = isset($cod_file->created_by)?getUserByType($cod_file->created_by,$cod_file->created_by_type):null;
                        $array['uploaded_at']                       = isset($cod_file->created_at)?Carbon::createFromFormat('Y-m-d H:i:s', $cod_file->created_at)->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                    }
                    $array['status']                                = 'Complete';

                    $cod_files[]                                    = $array;
                }
            }
        }

        $getFile                                                    = LoadFile::where('load_id', $loadDetail->load_id)->orderBy('id','desc')->get();
        return view('companyAdmin.chat.load.file.view', compact('static_files', 'required_files', 'requested_files', 'accessorial_files', 'cod_files', 'getFile'))->render();
    }

    // Added at at 18-04-2023
    public function getUnreadMessageDetails( Request $request ){
        try {
            $logged_in_guard                = $this->logged_in_guard;
            $company_id                     = auth()->guard($logged_in_guard)->user()->company_id;
            $company                        = Company::where('id', $company_id)->first();

            $skip                           = 0;
            $take                           = 1;
            $tophone                        = $request->tophone;

            $totalSms                       = SmsChatHistory::where( 'company_id', $company_id )
                ->where(function ($query) use ($tophone ) {
                    $query->where('from_number','LIKE','%'.$tophone.'%')
                    ->orWhere('to_number', 'LIKE','%'. $tophone.'%');
                })
                // ->where('type', 'received')
                // ->whereNotIn('from_number', $selectedPhones)
                ->where('is_sms_read', 0)
                ->orderBy('id','DESC')
                ->count();

            $totalSms                       = getTotalSmsCountCompany( $company_id );

            $messages                       = getunReadSMSByCompany( $company_id, $skip, $take, $logged_in_guard , $tophone);

            $response['success']            = true;
            $response['data']               = $messages;
            $response['total']              = $totalSms;
            return response()->json($response);
        } catch( \Exception $e ) {
            \Log::emergency( "File: ".$e->getFile() );
            \Log::emergency( "Line: ".$e->getLine() );
            \Log::emergency( "Message: ".$e->getMessage() );

            $response['success']            = false;
            $response['data']               = [];
            return response()->json($response);
        }
    }

    // Added at 19-04-2023
    public function getUnreadMessageCount( Request $request ){
        try {
            $logged_in_guard                = $this->logged_in_guard;
            $company_id                     = auth()->guard($logged_in_guard)->user()->company_id;
            $company                        = Company::where('id', $company_id)->first();

            $totalSms                       = SmsChatHistory::where( 'company_id', $company_id )->where('type', 'received')->where('is_sms_read', 0)->count();

            $response['success']            = true;
            $response['total']              = $totalSms;
            return response()->json($response);
        } catch( \Exception $e ) {
            \Log::emergency( "File: ".$e->getFile() );
            \Log::emergency( "Line: ".$e->getLine() );
            \Log::emergency( "Message: ".$e->getMessage() );

            $response['success']            = false;
            $response['data']               = [];
            return response()->json($response);
        }
    }

    /* Added at 14-04-2023 */
    public function getUnreadMessage(Request $request){
        try {
            $company_id                     = $request->company_id;

            $selectedPhones                 = [];

            if( isset( $request->phoneExt ) ) {
                $selectedPhones             = $request->phoneExt;
            }

            $unreadSms                      = SmsChatHistory::select('from_number', DB::raw('count(*) as total'))->where( 'company_id', $company_id )
                ->groupBy('from_number')
                ->where('type', 'received')
                // ->whereNotIn('from_number', $selectedPhones)
                ->where('is_sms_read', 0)
                ->orderBy('id','DESC')
                ->take(5)
                ->get();

            $data                           = [];
            $phones                         = [];

            $smschaticn                     = asset("theme/images/sms.png");
            foreach( $unreadSms as $key => $value ) {
                // $sender_id               = $value->from_number;
                $sender_id                  = substr($value->from_number, -10);
                $cell_phone                 = preg_replace('/(\d{3})(\d{3})(\d{4})/', '($1) $2-$3', $sender_id);
                // str_replace(array('(',')','-',' '),array('','','',''),$companyDispatcher->cell_phone)
                // Search above phone in company admin table
                $user                       = CompanyAdmin::where("cell_phone", $cell_phone)->first();
                $guard                      = '';
                $id                         = '';
                $name                       = $sender_id;
                $tophone                    = $sender_id;

                if( $user ) {
                    $guard                  = 'company-admin';
                    $id                     = $user->id;
                    $name                   = ucwords($user->first_name.' '.$user->last_name);
                } else {
                    $user                   = Driver::where("cell_phone", $cell_phone)->first();

                    if( $user ) {
                        $guard              = 'driver-admin';
                        $id                 = $user->id;
                        $name               = ucwords($user->first_name.' '.$user->last_name);
                    }
                    else{
                        $user               = Broker::where("phone_number", $sender_id)->first();
                         if( $user ) {
                            $guard          = 'company-admin';
                            $id             = $user->id;
                            $name           = ucwords($user->name);
                        }
                        else{
                            $user           = BrokerContact::where("phone_number", $sender_id)->first();
                            if( $user ) {
                                $guard      = 'company-admin';
                                $id         = $user->id;
                                $name       = ucwords($user->name);
                            }
                            else{
                                $user       = Customer::where("phone", $sender_id)->first();
                                if( $user ) {
                                    $guard  = 'company-admin';
                                    $id     = $user->id;
                                    $name   = ucwords($user->name);
                                }
                            }
                        }
                    }
                }

                $getBeforeEmailPart         = explode('@', $tophone)[0];
                $li_id                      = base64_encode($getBeforeEmailPart);
                $datakey                    = preg_replace('/[^a-zA-Z0-9]/', '', $li_id);

                $phones[]                   = array(
                    'id'                    => $id,
                    'guard'                 => $guard,
                    'toname'                => $name,
                    'liid'                  => $datakey,
                    'tophone'               => $tophone,
                    'smschaticn'            => $smschaticn,
                    'total'                 => $value->total
                );

                $data[]                     = '<li id="'.$datakey.'"><a href="javascript:void(0)" data-guard="'. $guard .'" data-id="'. $id .'" data-toname="'. $name .'" data-key="'.$datakey.'" data-tophone="'. $tophone .'" class="receiveSms"><span class="chatMenu-img" style="background-image:url('. $smschaticn .');"></span><span>'.$name.'(+1'.$tophone.')</span><div id="urdSmsCnt_'.$datakey.'" title="Unread SMS" class="badge badge-info position-absolute" style="right: 0%;">'.$value->total.'</div></a></li><input type="hidden" name="selectedPhone[]" class="selected_emails" value="'. $tophone .'">';
            }

            $response['success']            = true;
            $response['data']               = $data;
            $response['phones']             = $phones;
            return response()->json( $response );
        } catch( \Exception $e ) {
            \Log::emergency( "File: ".$e->getFile() );
            \Log::emergency( "Line: ".$e->getLine() );
            \Log::emergency( "Message: ".$e->getMessage() );

            $response['success']            = false;
            $response['msg']                = $e->getMessage();
            return response()->json( $response );
        }
    }
}
