<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use App\Models\BrokerAdditionalRequiredDocument;
use App\Models\BrokerBondInformation;
use App\Models\BrokerCarrierInsurance;
use App\Models\BrokerCompanyImage;
use App\Models\BrokerContract;
use App\Models\BrokerCustomerInsurance;
use App\Models\BrokerPricing;
use App\Models\BrokerRequiredDocument;
use App\Models\BrokerRequiredDocumentProfile;
use App\Models\CustomerFiles;
use App\Models\CustomerNote;
use App\Models\DefaultFuelSurcharge;
use App\Models\FuelSurchargeRegion;
use App\Models\FuelSurchargeTemplate;
use App\Models\Gallery;
use App\Models\InternalNote;
use App\Models\LoadDetail;
use Illuminate\Http\Request;

use App\Http\Requests\CompanyAdmin\BrokerStoreRequest;
use App\Http\Requests\CompanyAdmin\BrokerUpdateRequest;

use App\Models\Broker;
use App\Models\BrokerOffice;
use App\Models\BrokerContact;
use App\Models\BrokerAccessorial;

use App\Models\Bookmark;
use App\Models\FileUpload;
use App\Models\CompanyAccessorial;
use App\Models\CompanyLoadDocument;

use File;
use Yajra\DataTables\Facades\DataTables;
use Carbon\Carbon;
use App\Helpers\Common;
use Illuminate\Support\Facades\DB;
use Illuminate\Routing\UrlGenerator;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Str;

class BrokerController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */

    public $timeZone;
    public $timeZoneId;
    public $offset;
    protected $url;

    public function __construct(UrlGenerator $url){
        parent::__construct();

        $this->middleware(function ($request, $next) {
            $this->timeZone     = getTimeZone();
            $this->timeZoneId   = getTimeZoneId();
            $this->offset       = getTimeZoneOffset();

            return $next($request);
        });

        $this->url = $url;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getCompanyAdmin            = auth()->guard('company-admin')->user();
        $getField                   = '';
        $isCheck                    = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','brokers')->first();
        if(!empty($isCheck->id)){
            //$getField               = unserialize($isCheck->config);
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_brokers_first'));
        }else{
            $getField               = config('settings.table_brokers_first');
        }
        if ($request->ajax()) {
            $data                   = Broker::isCompany();

            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq       = 'search_string_condition_'.$key;
                $sortReq            = 'search_string_sort_'.$key;

                if( isset( request()->$conditionReq ) ) {
                    $typeReq        = 'search_string_type_'.$key;
                    $type           = request()->$typeReq;

                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }

                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                if( isset( request()->$sortReq ) ) {
                    $nameReq        = 'search_string_name_'.$key;
                    $sortReq        = 'search_string_sort_'.$key;
                    $typeReq        = 'search_string_type_'.$key;
                    $tableNameReq   = 'search_table_name_'.$key;
                    $colAliasReq    = 'search_string_col_alias_'.$key;

                    $origData       = request()->$nameReq;
                    $condition_sort = request()->$sortReq;
                    $type           = request()->$typeReq;
                    $tableName      = isset( request()->$tableNameReq ) ? request()->$tableNameReq : '';
                    $colAlias       = isset( request()->$colAliasReq ) ? request()->$colAliasReq : '';

                    if( $type == 'string' ) {
                        if( isset( $condition_sort ) ) {
                            if( $colAlias != '' ) {
                                $data->orderBy( $tableName.'.'.$colAlias, $condition_sort );
                            } else {
                                if( $tableName != '' ) {
                                    $data->orderBy( $tableName.'.'.$origData, $condition_sort );
                                } else {
                                    $data->orderBy( $origData, $condition_sort );
                                }
                            }
                        }
                    }
                }
            }
            $data->select('*');
            /// end

            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $action = '<a href="'.route('company.admin.brokers.show', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                    $action .= '<a href="'.route('company.admin.brokers.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                    $action .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash destroy" data-action="'.route('company.admin.brokers.destroy',$row->id).'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';

                    return $action;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        $tableName = 'settings.table_brokers';
        return view('companyAdmin.broker.index', compact('getField','tableName'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        $broker                                 = new Broker;
        $load_documents                         = getCompanyLoadDocument();
        $shipper_files                          = [];
        $fixed_shipper_files                    = [];
        $consignee_files                        = [];
        $fixed_consignee_files                  = [];
        foreach ($load_documents as $key => $value) {
            $arry                               = $value->toArray();
            if($value->for_whom == 'shipper'){
                if($value->fixed == 'no'){
                    $shipper_files[]            = $arry;
                }else{
                    $fixed_shipper_files[]      = $arry;
                }
            }else if($value->for_whom == 'consignee'){
                if($value->fixed == 'no'){
                    $consignee_files[]          = $arry;
                }else{
                    $fixed_consignee_files[]    = $arry;
                }
            }else {
                if($value->fixed == 'no'){
                    $shipper_files[]            = $arry;
                    $consignee_files[]          = $arry;
                }else{
                    $fixed_shipper_files[]      = $arry;
                    $fixed_consignee_files[]    = $arry;
                }
            }
        }

        $parentCompany = Broker::IsCompany()->select('id', DB::raw('CONCAT(name, " - ", (select name from companies where id = company_id), " - ", city) as parent'))->where('office_type', '1')->pluck('parent', 'id')->toArray();

        if(request()->has('page') && request()->page == 'old'){
            return view('companyAdmin.broker.create_18122023', compact('shipper_files', 'fixed_shipper_files', 'consignee_files', 'fixed_consignee_files'));
        }else {
            return view('companyAdmin.broker.create', compact('parentCompany'));
        }
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(BrokerStoreRequest $request){
        // dd('Store Broker', $request->all());

        $input                                                          = $request->all();

        if($request->address_verified==1){
            $timezone                                                   = null;
            $timezone_offset                                            = null;
            if(isset($input['timezone_abbr'])){
                $timezone                                               = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset                                        = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']                                          = $timezone;
            $input['timezone_offset']                                   = $timezone_offset;
        }
        if($request->billing_address_verified==1){
            $billing_timezone                                           = null;
            $billing_timezone_offset                                    = null;
            if(isset($input['billing_timezone_abbr'])){
                $billing_timezone                                       = timezone_name_from_abbr($input['billing_timezone_abbr']);
                $billing_timezone_offset                                = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['billing_timezone']                                  = $billing_timezone;
            $input['billing_timezone_offset']                           = $billing_timezone_offset;
        }

        $input['company_id']                                            = auth()->guard('company-admin')->user()->company_id;
        if (isset($input['logo'])) {
            $input['logo']                                              = FileUpload::upload('companyAdmin/brokers', $input['logo']);
        }

        $shipper_files_array                                            = [];
        $shipper_i                                                      = 0;
        if(isset($request->shipper_files) && !empty($request->shipper_files)){
            foreach($request->shipper_files['name_id'] as $key => $sefdata){
                $shipper_files_array[$shipper_i]['name_id']             = isset($request->shipper_files['name_id'][$key]) ? $request->shipper_files['name_id'][$key] : '';
                $shipper_files_array[$shipper_i]['name']                = isset($request->shipper_files['name'][$key]) ? $request->shipper_files['name'][$key] : '';
                $shipper_files_array[$shipper_i]['is_required']         = isset($request->shipper_files['is_required'][$key]) ? $request->shipper_files['is_required'][$key] : 'off';
                $shipper_files_array[$shipper_i]['type']                = isset($request->shipper_files['type'][$key]) ? $request->shipper_files['type'][$key] : '';
                $shipper_files_array[$shipper_i]['fixed']               = isset($request->shipper_files['fixed'][$key]) ? $request->shipper_files['fixed'][$key] : '';
                $shipper_files_array[$shipper_i]['include_w_billing']   = isset($request->shipper_files['include_w_billing'][$key]) && $request->shipper_files['include_w_billing'][$key]=='yes' ? 'yes' : 'no';
                $shipper_i++;
            }
        }

        $consignee_files_array                                          = [];
        $consignee_i                                                    = 0;
        if(isset($request->consignee_files) && !empty($request->consignee_files)){
            foreach($request->consignee_files['name_id'] as $key => $sefdata){
                $consignee_files_array[$consignee_i]['name_id']         = isset($request->consignee_files['name_id'][$key]) ? $request->consignee_files['name_id'][$key] : '';
                $consignee_files_array[$consignee_i]['name']            = isset($request->consignee_files['name'][$key]) ? $request->consignee_files['name'][$key] : '';
                $consignee_files_array[$consignee_i]['is_required']     = isset($request->consignee_files['is_required'][$key]) ? $request->consignee_files['is_required'][$key] : 'off';
                $consignee_files_array[$consignee_i]['type']            = isset($request->consignee_files['type'][$key]) ? $request->consignee_files['type'][$key] : '';
                $consignee_files_array[$consignee_i]['fixed']           = isset($request->consignee_files['fixed'][$key]) ? $request->consignee_files['fixed'][$key] : '';
                $consignee_files_array[$consignee_i]['include_w_billing']= isset($request->consignee_files['include_w_billing'][$key]) && $request->consignee_files['include_w_billing'][$key]=='yes' ? 'yes' : 'no';
                $consignee_i++;
            }
        }

        $input['shipper_files']                                         = !empty($shipper_files_array) ? serialize($shipper_files_array) : null;
        $input['consignee_files']                                       = !empty($consignee_files_array) ? serialize($consignee_files_array) : null;
        if($input['account_number']!=""){
            $input['account_number']                                    = base64_encode($input['account_number']);
        }
        $broker                                                         = Broker::create($input);

        $offices                                                        = $request->offices ?? [];
        // $offices                                                     = array_filter($offices, function($var) {
        //     return isset($var['name']) && $var['name'] !== null;
        // });
        if(!empty($offices)){
            foreach ($offices as $key => $office) {
                $broker_offices                                         = [];
                $broker_offices['broker_id']                            = $broker->id;
                $broker_offices['name']                                 = $office['name'];
                $broker_offices['address_one']                          = $office['address'];
                $broker_offices['state']                                = $office['state'];
                $broker_offices['city']                                 = $office['city'];
                $broker_offices['zip']                                  = $office['zip'];
                if($office['address_verified']==1){
                    $timezone                                           = null;
                    $timezone_offset                                    = null;
                    if(isset($office['timezone_abbr'])){
                        $timezone                                       = timezone_name_from_abbr($office['timezone_abbr']);
                        $timezone_offset                                = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
                    }
                    $broker_offices['latitude']                         = $office['latitude'];
                    $broker_offices['longitude']                        = $office['longitude'];
                    $broker_offices['timezone']                         = $timezone;
                    $broker_offices['timezone_offset']                  = $timezone_offset;
                    $broker_offices['timezone_abbr']                    = $office['timezone_abbr'];
                }
                $broker_offices['address_verified']                     = $office['address_verified'];
                BrokerOffice::create($broker_offices);
            }
        }

        $additional_contacts                                            = $request->additional_contacts ?? [];
        if(!empty($additional_contacts)){
            foreach ($additional_contacts as $key => $contact) {
                $cat                                                    = [];
                $cat['source']                                          = 'broker';
                $cat['action_id']                                       = $broker->id;
                $cat['department']                                      = $contact['department'];

                $name                                                   = null;
                if(!is_null($contact['first_name'])){
                    $name                                               = $contact['first_name'];
                }
                if(!is_null($contact['last_name'])){
                    if(!is_null($name)){
                        $name                                           .= ' '.$contact['last_name'];
                    }else{
                        $name                                           = $contact['last_name'];
                    }
                }
                $cat['name']                                            = $name;
                $cat['first_name']                                      = $contact['first_name'];
                $cat['last_name']                                       = $contact['last_name'];

                $cat['phone_number']                                    = $contact['phone_number'];
                $cat['cell_number']                                     = $contact['cell_number'];
                $cat['ext']                                             = $contact['ext'];
                $cat['email']                                           = $contact['email'];

                $preferred_connections                                  = null;
                if(isset($contact['preferred_connections']) && !empty($contact['preferred_connections'])){
                    $preferred_connections                              = implode(',', $contact['preferred_connections']);
                }
                $cat['preferred_connections']                           = $preferred_connections;

                $primary_contact                                        = isset($contact['primary_contact'])?$contact['primary_contact']:0;
                $cat['primary_contact']                                 = $primary_contact;
                if($primary_contact==1){
                    $primary_contact_id                                 = null;
                    if(isset($contact['dispatcher_id']) && !empty($contact['dispatcher_id'])){
                        $primary_contact_id                             = implode(',', $contact['dispatcher_id']);
                    }
                    $cat['primary_contact_id']                          = $primary_contact_id;
                }else{
                    $cat['primary_contact_id']                          = null;
                }
                BrokerContact::create($cat);
            }
        }

        notification('success', $this->crudMessage('create', 'Broker'));
        return redirect()->route('company.admin.brokers.index');
    }

    /**
     * Display the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id){
        $id = base64Decode($id);
        $broker = Broker::findOrFail($id);
        $broker->rep_details = $broker->broker_contact->where('is_representative', '1')->first() ?? null;

        $currentDate = Carbon::now()->timezone($this->timeZone);

        $brokerFiles = []; $hoursOfOperations = [];

        if($broker){
            $brokerFiles = $broker->galleries->pluck('image', 'field_name')->toArray();
            $hoursOfOperations = $this->getHoursOfOperation($broker);
        }

        /*************** Fuel Surcharge - START ***************/
        $fileHistory = FuelSurchargeTemplate::IsCompany()->where('customer', $id)->orderBy('id', 'DESC')->get(['customer_file_name', 'start_date', 'end_date', 'created_by', 'set_as_active']);

        $regions = FuelSurchargeRegion::get(['id', 'name as text', 'code', 'parent'])->toArray();
        $regionOptions = FuelSurchargesController::parentChildTree($regions);

        $regionEPMRHtml = ''; $regionEPD2DHtml = '';
        $regionEPMRHtml .= '<option value="">Select Region</option>';
        $regionEPD2DHtml .= '<option value="">Select Region</option>';

        foreach($regionOptions as $key => $values){
            if($key == 0){
                foreach($values['children'] as $childs){
                    $regionEPMRHtml .= '<optgroup class="select2-result-selectable" label="'.$childs['text'].'">';
                    foreach($childs['children'] as $child){
                        if($child['code'] != ""){
                            $regionEPMRHtml .= '<option value="'.$child['text'].' ('.$child['code'].')">'.$child['text'].' ('.$child['code'].')</option>';
                        }else{
                            $regionEPMRHtml .= '<option value="'.$child['text'].'">'.$child['text'].'</option>';
                        }
                    }
                    $regionEPMRHtml .= '</optgroup>';
                }
            }else{
                foreach($values['children'] as $childs){
                    $regionEPD2DHtml .= '<optgroup class="select2-result-selectable" label="'.$childs['text'].'">';
                    foreach($childs['children'] as $child){
                        if($child['code'] != ""){
                            $regionEPD2DHtml .= '<option value="'.$child['text'].' ('.$child['code'].')">'.$child['text'].' ('.$child['code'].')</option>';
                        }else{
                            $regionEPD2DHtml .= '<option value="'.$child['text'].'">'.$child['text'].'</option>';
                        }
                    }
                    $regionEPD2DHtml .= '</optgroup>';
                }
            }
        }
        /*************** Fuel Surcharge - END ***************/

        /*************** Loads - START ***************/
        $getCompanyAdmin = auth()->guard('company-admin')->user();
        $getField = '';
        $isCheck = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','customer_loads')->first();
        if(!empty($isCheck->id)){
            $getField = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_customer_loads_first'));
        }else{
            $getField = config('settings.table_customer_loads_first');
        }
        /*************** Loads - END ***************/

        /*************** Billing - START ***************/
        $getCompanyAdmin = auth()->guard('company-admin')->user();
        $getField2 = '';
        $isCheck = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','customer_billing_loads')->first();
        if(!empty($isCheck->id)){
            $getField2 = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_customer_billing_loads_first'));
        }else{
            $getField2 = config('settings.table_customer_billing_loads_first');
        }
        /*************** Billing - END ***************/

        $isMulitpleTable        = 1;

        $tables                 = array(
            0 => array(
                "name"          => 'table1',
                "id"            => 'customer-load-data-table',
                "class"         => 'customer-load-data-table',
                "table"         => 'settings.table_customer_loads',
                "route"         => route('company.admin.broker.loads', ['customer' => $id]),
                "getField"      => $getField,
            ),
            1 => array(
                "name"          => 'table2',
                "id"            => 'all-loads-data-table',
                "class"         => 'all-loads-data-table',
                "table"         => 'settings.table_customer_billing_loads',
                "route"         => route('company.admin.billing.loads.index'),
                "getField"      => $getField2,
            ),
        );

        return view('companyAdmin.broker.show', compact('broker', 'currentDate', 'brokerFiles', 'hoursOfOperations', 'regionEPMRHtml', 'regionEPD2DHtml', 'fileHistory', 'getField', 'getField2', 'isMulitpleTable', 'tables'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit_backup($id){
        $id                                     = base64Decode($id);
        $broker                                 = Broker::findOrFail($id);
        if($broker->account_number!=""){
            $broker->account_number             = base64_decode($broker->account_number);
        }

        $accessorialList                        = CompanyAccessorial::isCompany()->get(['id', 'name', 'default_amount', 'gl_code']);
        $accessorialDropDown                    = "";
        if($accessorialList->count() > 0){
            $accessorialDropDown                .= '<option value="">Select accessorial</option>';
            foreach ($accessorialList as $accessorial){
                $accessorialDropDown            .= '<option value="'.$accessorial->id.'" data-default-amount="'.$accessorial->default_amount.'">'.$accessorial->name.'</option>';
            }
        }else{
            $accessorialDropDown                .= '<option value="" data-default-amount="">Select accessorial</option>';
        }

        $broker_shipper_files                   = [];
        if(isset($broker->shipper_files) && is_serialized($broker->shipper_files) && !empty(unserialize($broker->shipper_files))){
            $broker_shipper_files               = unserialize($broker->shipper_files);
            $broker_shipper_files               = array_filter($broker_shipper_files, function($var) {
                return isset($var['name_id']) && $var['name_id'] !== null;
            });
        }

        $broker_consignee_files                 = [];
        if(isset($broker->consignee_files) && is_serialized($broker->consignee_files) && !empty(unserialize($broker->consignee_files))){
            $broker_consignee_files             = unserialize($broker->consignee_files);
            $broker_consignee_files             = array_filter($broker_consignee_files, function($var) {
                return isset($var['name_id']) && $var['name_id'] !== null;
            });
        }

        $load_documents                         = getCompanyLoadDocument();
        $shipper_files                          = [];
        $fixed_shipper_files                    = [];
        $consignee_files                        = [];
        $fixed_consignee_files                  = [];
        foreach ($load_documents as $key => $value) {
            $arry                               = $value->toArray();
            if($value->for_whom == 'shipper'){
                if($value->fixed == 'no'){
                    $shipper_files[]            = $arry;
                }else{
                    $fixed_shipper_files[]      = $arry;
                }
            }else if($value->for_whom == 'consignee'){
                if($value->fixed == 'no'){
                    $consignee_files[]          = $arry;
                }else{
                    $fixed_consignee_files[]    = $arry;
                }
            }else {
                if($value->fixed == 'no'){
                    $shipper_files[]            = $arry;
                    $consignee_files[]          = $arry;
                }else{
                    $fixed_shipper_files[]      = $arry;
                    $fixed_consignee_files[]    = $arry;
                }
            }
        }

        return view('companyAdmin.broker.edit', compact(
                'broker',
                'accessorialDropDown',
                'broker_shipper_files',
                'broker_consignee_files',
                'shipper_files',
                'fixed_shipper_files',
                'consignee_files',
                'fixed_consignee_files'
            )
        );
    }

    public function edit($id)
    {
        $id = base64Decode($id);
        $broker = Broker::findOrFail($id);
        $broker->customer_development_representative = (!is_null($broker->customer_development_representative)) ? explode(',', $broker->customer_development_representative) : [];

        if(isset($broker->broker_carrier_insurances)){
            $broker->carrier_insurance_name = (isset($broker->broker_carrier_insurances->carrier_insurance_name) && array_filter(unserialize($broker->broker_carrier_insurances->carrier_insurance_name))) ? unserialize($broker->broker_carrier_insurances->carrier_insurance_name) : null;
            $broker->carrier_insurance_type = (isset($broker->broker_carrier_insurances->carrier_insurance_type) && array_filter(unserialize($broker->broker_carrier_insurances->carrier_insurance_type))) ? unserialize($broker->broker_carrier_insurances->carrier_insurance_type) : null;
            $broker->carrier_policy_number = (isset($broker->broker_carrier_insurances->carrier_policy_number) && array_filter(unserialize($broker->broker_carrier_insurances->carrier_policy_number))) ? unserialize($broker->broker_carrier_insurances->carrier_policy_number) : null;
            $broker->carrier_limit = (isset($broker->broker_carrier_insurances->carrier_limit) && array_filter(unserialize($broker->broker_carrier_insurances->carrier_limit))) ? unserialize($broker->broker_carrier_insurances->carrier_limit) : null;
            $broker->carrier_deductible = (isset($broker->broker_carrier_insurances->carrier_deductible) && array_filter(unserialize($broker->broker_carrier_insurances->carrier_deductible))) ? unserialize($broker->broker_carrier_insurances->carrier_deductible) : null;
            $broker->carrier_expiration_date = (isset($broker->broker_carrier_insurances->carrier_expiration_date) && array_filter(unserialize($broker->broker_carrier_insurances->carrier_expiration_date))) ? unserialize($broker->broker_carrier_insurances->carrier_expiration_date) : null;
        }
        if(isset($broker->broker_customer_insurances)){
            $broker->customer_insurance_name = (isset($broker->broker_customer_insurances->customer_insurance_name) && array_filter(unserialize($broker->broker_customer_insurances->customer_insurance_name))) ? unserialize($broker->broker_customer_insurances->customer_insurance_name) : null;
            $broker->customer_insurance_type = (isset($broker->broker_customer_insurances->customer_insurance_type) && array_filter(unserialize($broker->broker_customer_insurances->customer_insurance_type))) ? unserialize($broker->broker_customer_insurances->customer_insurance_type) : null;
            $broker->customer_policy_number = (isset($broker->broker_customer_insurances->customer_policy_number) && array_filter(unserialize($broker->broker_customer_insurances->customer_policy_number))) ? unserialize($broker->broker_customer_insurances->customer_policy_number) : null;
            $broker->customer_limit = (isset($broker->broker_customer_insurances->customer_limit) && array_filter(unserialize($broker->broker_customer_insurances->customer_limit))) ? unserialize($broker->broker_customer_insurances->customer_limit) : null;
            $broker->customer_deductible = (isset($broker->broker_customer_insurances->customer_deductible) && array_filter(unserialize($broker->broker_customer_insurances->customer_deductible))) ? unserialize($broker->broker_customer_insurances->customer_deductible) : null;
            $broker->customer_expiration_date = (isset($broker->broker_customer_insurances->customer_expiration_date) && array_filter(unserialize($broker->broker_customer_insurances->customer_expiration_date))) ? unserialize($broker->broker_customer_insurances->customer_expiration_date) : null;
        }
        if(isset($broker->broker_bond_information)){
            $broker->surety_bond_required = (isset($broker->broker_bond_information->surety_bond_required) && array_filter(unserialize($broker->broker_bond_information->surety_bond_required))) ? unserialize($broker->broker_bond_information->surety_bond_required) : null;
            $broker->surety_bond_company = (isset($broker->broker_bond_information->surety_bond_company) && array_filter(unserialize($broker->broker_bond_information->surety_bond_company))) ? unserialize($broker->broker_bond_information->surety_bond_company) : null;
            $broker->surety_bond_expiration = (isset($broker->broker_bond_information->surety_bond_expiration)) ? unserialize($broker->broker_bond_information->surety_bond_expiration) : null;
            $broker->surety_bond_amount = (isset($broker->broker_bond_information->surety_bond_amount) && array_filter(unserialize($broker->broker_bond_information->surety_bond_amount))) ? unserialize($broker->broker_bond_information->surety_bond_amount) : null;
            $broker->surety_bond_email = (isset($broker->broker_bond_information->surety_bond_email) && array_filter(unserialize($broker->broker_bond_information->surety_bond_email))) ? unserialize($broker->broker_bond_information->surety_bond_email) : null;
            $broker->bond_contact_name = (isset($broker->broker_bond_information->bond_contact_name) && array_filter(unserialize($broker->broker_bond_information->bond_contact_name))) ? unserialize($broker->broker_bond_information->bond_contact_name) : null;
            $broker->bond_phone_number = (isset($broker->broker_bond_information->bond_phone_number) && array_filter(unserialize($broker->broker_bond_information->bond_phone_number))) ? unserialize($broker->broker_bond_information->bond_phone_number) : null;
        }

        $format = (getTimeFormat() == '12') ? config('settings.time_format.12_hour') : config('settings.time_format.24_hour');
        $broker->ho_sun_open = ($broker->ho_sun_open != '') ? Carbon::parse($broker->ho_sun_open)->format($format) : '';
        $broker->ho_sun_close = ($broker->ho_sun_close != '') ?  Carbon::parse($broker->ho_sun_close)->format($format) : '';
        $broker->ho_mon_open = ($broker->ho_mon_open != '') ?  Carbon::parse($broker->ho_mon_open)->format($format) : '';
        $broker->ho_mon_close = ($broker->ho_mon_close != '') ?  Carbon::parse($broker->ho_mon_close)->format($format) : '';
        $broker->ho_tue_open = ($broker->ho_tue_open != '') ?  Carbon::parse($broker->ho_tue_open)->format($format) : '';
        $broker->ho_tue_close = ($broker->ho_tue_close != '') ?  Carbon::parse($broker->ho_tue_close)->format($format) : '';
        $broker->ho_wed_open = ($broker->ho_wed_open != '') ?  Carbon::parse($broker->ho_wed_open)->format($format) : '';
        $broker->ho_wed_close = ($broker->ho_wed_close != '') ?  Carbon::parse($broker->ho_wed_close)->format($format) : '';
        $broker->ho_thu_open = ($broker->ho_thu_open != '') ?  Carbon::parse($broker->ho_thu_open)->format($format) : '';
        $broker->ho_thu_close = ($broker->ho_thu_close != '') ?  Carbon::parse($broker->ho_thu_close)->format($format) : '';
        $broker->ho_fri_open = ($broker->ho_fri_open != '') ?  Carbon::parse($broker->ho_fri_open)->format($format) : '';
        $broker->ho_fri_close = ($broker->ho_fri_close != '') ?  Carbon::parse($broker->ho_fri_close)->format($format) : '';
        $broker->ho_sat_open = ($broker->ho_sat_open != '') ?  Carbon::parse($broker->ho_sat_open)->format($format) : '';
        $broker->ho_sat_close = ($broker->ho_sat_close != '') ?  Carbon::parse($broker->ho_sat_close)->format($format) : '';

        $parentCompany = Broker::IsCompany()->select('id', DB::raw('CONCAT(name, " - ", (select name from companies where id = company_id), " - ", city) as parent'))->where('office_type', '1')->where('id', '!=', $id)->pluck('parent', 'id')->toArray();

        return view('companyAdmin.broker.create', compact('broker', 'parentCompany'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(BrokerUpdateRequest $request, Broker $broker){
        // dd('Update Broker', $request->all());

        $input                                                              = $request->all();

        if($request->address_verified==1){
            $timezone                                                       = null;
            $timezone_offset                                                = null;
            if(isset($input['timezone_abbr'])){
                $timezone                                                   = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset                                            = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']                                              = $timezone;
            $input['timezone_offset']                                       = $timezone_offset;
        }else{
            $input['latitude']                                              = null;
            $input['longitude']                                             = null;
            $input['timezone']                                              = null;
            $input['timezone_offset']                                       = null;
            $input['timezone_abbr']                                         = null;
        }
        if($request->billing_address_verified==1){
            $billing_timezone                                               = null;
            $billing_timezone_offset                                        = null;
            if(isset($input['billing_timezone_abbr'])){
                $billing_timezone                                           = timezone_name_from_abbr($input['billing_timezone_abbr']);
                $billing_timezone_offset                                    = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['billing_timezone']                                      = $billing_timezone;
            $input['billing_timezone_offset']                               = $billing_timezone_offset;
        }else{
            $input['billing_latitude']                                      = null;
            $input['billing_longitude']                                     = null;
            $input['billing_timezone']                                      = null;
            $input['billing_timezone_offset']                               = null;
            $input['billing_timezone_abbr']                                 = null;
        }

        if (isset($input['logo'])) {
            $input['logo']                                                  = FileUpload::upload('companyAdmin/brokers', $input['logo']);
        }else{
            if(!empty($input['remove_image'])){
                $input['logo']                                              = null;
            }
        }

        $shipper_files_array                                                = [];
        $shipper_i                                                          = 0;
        if(isset($request->shipper_files) && !empty($request->shipper_files)){
            foreach($request->shipper_files['name_id'] as $key => $sefdata){
                $shipper_files_array[$shipper_i]['name_id']                 = isset($request->shipper_files['name_id'][$key]) ? $request->shipper_files['name_id'][$key] : '';
                $shipper_files_array[$shipper_i]['name']                    = isset($request->shipper_files['name'][$key]) ? $request->shipper_files['name'][$key] : '';
                $shipper_files_array[$shipper_i]['is_required']             = isset($request->shipper_files['is_required'][$key]) ? $request->shipper_files['is_required'][$key] : 'off';
                $shipper_files_array[$shipper_i]['type']                    = isset($request->shipper_files['type'][$key]) ? $request->shipper_files['type'][$key] : '';
                $shipper_files_array[$shipper_i]['fixed']                   = isset($request->shipper_files['fixed'][$key]) ? $request->shipper_files['fixed'][$key] : '';
                $shipper_files_array[$shipper_i]['include_w_billing']       = isset($request->shipper_files['include_w_billing'][$key]) && $request->shipper_files['include_w_billing'][$key]=='yes' ? 'yes' : 'no';
                $shipper_i++;
            }
        }

        $consignee_files_array                                              = [];
        $consignee_i                                                        = 0;
        if(isset($request->consignee_files) && !empty($request->consignee_files)){
            foreach($request->consignee_files['name_id'] as $key => $sefdata){
                $consignee_files_array[$consignee_i]['name_id']             = isset($request->consignee_files['name_id'][$key]) ? $request->consignee_files['name_id'][$key] : '';
                $consignee_files_array[$consignee_i]['name']                = isset($request->consignee_files['name'][$key]) ? $request->consignee_files['name'][$key] : '';
                $consignee_files_array[$consignee_i]['is_required']         = isset($request->consignee_files['is_required'][$key]) ? $request->consignee_files['is_required'][$key] : 'off';
                $consignee_files_array[$consignee_i]['type']                = isset($request->consignee_files['type'][$key]) ? $request->consignee_files['type'][$key] : '';
                $consignee_files_array[$consignee_i]['fixed']               = isset($request->consignee_files['fixed'][$key]) ? $request->consignee_files['fixed'][$key] : '';
                $consignee_files_array[$consignee_i]['include_w_billing']   = isset($request->consignee_files['include_w_billing'][$key]) && $request->consignee_files['include_w_billing'][$key]=='yes' ? 'yes' : 'no';
                $consignee_i++;
            }
        }

        $input['shipper_files']                                             = !empty($shipper_files_array) ? serialize($shipper_files_array) : null;
        $input['consignee_files']                                           = !empty($consignee_files_array) ? serialize($consignee_files_array) : null;
        if($input['account_number']!=""){
            $input['account_number']                                        = base64_encode($input['account_number']);
        }
        $broker->update($input);

        // office
        $offices                                                            = $request->offices ?? [];
        // $offices                                                         = array_filter($offices, function($var) {
        //     return isset($var['name']) && $var['name'] !== null;
        // });

        $offices_old_id                                                     = array_column($offices, 'old_id');
        $offices_remove_query                                               = BrokerOffice::where('broker_id', $broker->id);
        if(!empty($offices_old_id)){
            $offices_remove_query->whereNotIn('id', $offices_old_id);
        }
        $offices_remove_query_result                                        = $offices_remove_query->get();

        if(!empty($offices)){
            foreach ($offices as $key => $office) {
                if(isset($office['old_id']) && $office['old_id']!=''){
                    $item                                                   = BrokerOffice::where('id', '=', $office['old_id'])->first();
                    if($item){
                        $item->name                                         = $office['name'];
                        $item->address_one                                  = $office['address'];
                        $item->state                                        = $office['state'];
                        $item->city                                         = $office['city'];
                        $item->zip                                          = $office['zip'];
                        if($office['address_verified']==1){
                            $timezone                                       = null;
                            $timezone_offset                                = null;
                            if(isset($office['timezone_abbr'])){
                                $timezone                                   = timezone_name_from_abbr($office['timezone_abbr']);
                                $timezone_offset                            = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
                            }
                            $item->latitude                                 = $office['latitude'];
                            $item->longitude                                = $office['longitude'];
                            $item->timezone                                 = $timezone;
                            $item->timezone_offset                          = $timezone_offset;
                            $item->timezone_abbr                            = $office['timezone_abbr'];
                        }else{
                            $item->latitude                                 = null;
                            $item->longitude                                = null;
                            $item->timezone                                 = null;
                            $item->timezone_offset                          = null;
                            $item->timezone_abbr                            = null;
                        }
                        $item->address_verified                             = $office['address_verified'];
                        $item->save();
                    }
                }else{
                    $item                                                   = new BrokerOffice();
                    $item->broker_id                                        = $broker->id;
                    $item->name                                             = $office['name'];
                    $item->address_one                                      = $office['address'];
                    $item->state                                            = $office['state'];
                    $item->city                                             = $office['city'];
                    $item->zip                                              = $office['zip'];
                    if($office['address_verified']==1){
                        $timezone                                           = null;
                        $timezone_offset                                    = null;
                        if(isset($office['timezone_abbr'])){
                            $timezone                                       = timezone_name_from_abbr($office['timezone_abbr']);
                            $timezone_offset                                = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
                        }
                        $item->latitude                                     = $office['latitude'];
                        $item->longitude                                    = $office['longitude'];
                        $item->timezone                                     = $timezone;
                        $item->timezone_offset                              = $timezone_offset;
                        $item->timezone_abbr                                = $office['timezone_abbr'];
                    }
                    $item->address_verified                                 = $office['address_verified'];
                    $item->save();
                }
            }
        }

        if($offices_remove_query_result->count() > 0){
            foreach ($offices_remove_query_result as $remove) {
                $remove->delete();
            }
        }

        // additional_contacts
        $additional_contacts                                                = $request->additional_contacts ?? [];

        $contacts_old_id                                                    = array_column($additional_contacts, 'old_id');
        $contacts_remove_query                                              = BrokerContact::where('action_id', $broker->id)->where('source', 'broker');
        if(!empty($contacts_old_id)){
            $contacts_remove_query->whereNotIn('id', $contacts_old_id);
        }
        $contacts_remove_query_result                                       = $contacts_remove_query->get();

        if(!empty($additional_contacts)){
            foreach ($additional_contacts as $key => $contact) {
                if(isset($contact['old_id']) && $contact['old_id']!=''){
                    $item                                                   = BrokerContact::where('id', '=', $contact['old_id'])->first();
                    if($item){
                        $item->office_id                                    = $contact['office_id'];
                        $item->department                                   = $contact['department'];

                        $name                                               = null;
                        if(!is_null($contact['first_name'])){
                            $name                                           = $contact['first_name'];
                        }
                        if(!is_null($contact['last_name'])){
                            if(!is_null($name)){
                                $name                  .= ' '.$contact['last_name'];
                            }else{
                                $name                                       = $contact['last_name'];
                            }
                        }

                        $item->name                                         = $name;
                        $item->first_name                                   = $contact['first_name'];
                        $item->last_name                                    = $contact['last_name'];

                        $item->phone_number                                 = $contact['phone_number'];
                        $item->cell_number                                  = $contact['cell_number'];
                        $item->ext                                          = $contact['ext'];
                        $item->email                                        = $contact['email'];

                        $preferred_connections                              = null;
                        if(isset($contact['preferred_connections']) && !empty($contact['preferred_connections'])){
                            $preferred_connections                          = implode(',', $contact['preferred_connections']);
                        }
                        $item->preferred_connections                        = $preferred_connections;

                        $primary_contact                                    = isset($contact['primary_contact'])?$contact['primary_contact']:0;
                        $item->primary_contact                              = $primary_contact;
                        if($primary_contact==1){
                            $primary_contact_id                             = null;
                            if(isset($contact['dispatcher_id']) && !empty($contact['dispatcher_id'])){
                                $primary_contact_id                         = implode(',', $contact['dispatcher_id']);
                            }
                            $item->primary_contact_id                       = $primary_contact_id;
                        }else{
                            $item->primary_contact_id                       = null;
                        }
                        $item->save();
                    }
                }else{
                    $cat                                                    = [];
                    $cat['source']                                          = 'broker';
                    $cat['action_id']                                       = $broker->id;
                    $cat['office_id']                                       = $contact['office_id'];
                    $cat['department']                                      = $contact['department'];

                    $name                                                   = null;
                    if(!is_null($contact['first_name'])){
                        $name                                               = $contact['first_name'];
                    }
                    if(!is_null($contact['last_name'])){
                        if(!is_null($name)){
                            $name                      .= ' '.$contact['last_name'];
                        }else{
                            $name                                           = $contact['last_name'];
                        }
                    }
                    $cat['name']                                            = $name;
                    $cat['first_name']                                      = $contact['first_name'];
                    $cat['last_name']                                       = $contact['last_name'];

                    $cat['phone_number']                                    = $contact['phone_number'];
                    $cat['cell_number']                                     = $contact['cell_number'];
                    $cat['ext']                                             = $contact['ext'];
                    $cat['email']                                           = $contact['email'];

                    $preferred_connections                                  = null;
                    if(isset($contact['preferred_connections']) && !empty($contact['preferred_connections'])){
                        $preferred_connections                              = implode(',', $contact['preferred_connections']);
                    }
                    $cat['preferred_connections']                           = $preferred_connections;

                    $primary_contact                                        = isset($contact['primary_contact'])?$contact['primary_contact']:0;
                    $cat['primary_contact']                                 = $primary_contact;
                    if($primary_contact==1){
                        $primary_contact_id                                 = null;
                        if(isset($contact['dispatcher_id']) && !empty($contact['dispatcher_id'])){
                            $primary_contact_id                             = implode(',', $contact['dispatcher_id']);
                        }
                        $cat['primary_contact_id']                          = $primary_contact_id;
                    }else{
                        $cat['primary_contact_id']                          = null;
                    }
                    BrokerContact::create($cat);
                }
            }
        }

        if($contacts_remove_query_result->count() > 0){
            foreach ($contacts_remove_query_result as $remove) {
                $remove->delete();
            }
        }
        //
        notification('success', $this->crudMessage('update', 'Broker'));
        return redirect()->route('company.admin.brokers.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Broker $broker){
        $broker->delete();
        notification('success', $this->crudMessage('delete', 'Broker'));
        return redirect()->route('company.admin.brokers.index');
    }

    public function addContact(Request $request){
        $input                          = $request->all();

        $cat                            = [];
        $cat['source']                  = 'broker';
        $cat['action_id']               = $input['action_id'];
        $cat['department']              = $input['department'];

        $name                           = null;
        if(!is_null($input['first_name'])){
            $name                       = $input['first_name'];
        }
        if(!is_null($input['last_name'])){
            if(!is_null($name)){
                $name                  .= ' '.$input['last_name'];
            }else{
                $name                   = $input['last_name'];
            }
        }
        $cat['name']                    = $name;
        $cat['first_name']              = $input['first_name'];
        $cat['last_name']               = $input['last_name'];

        $cat['phone_number']            = $input['phone_number'];
        $cat['cell_number']             = $input['cell_number'];
        $cat['ext']                     = $input['ext'];
        $cat['email']                   = $input['email'];

        $preferred_connections          = null;
        if(isset($input['preferred_connections']) && !empty($input['preferred_connections'])){
            $preferred_connections      = implode(',', $input['preferred_connections']);
        }
        $cat['preferred_connections']   = $preferred_connections;

        $primary_contact                = isset($input['primary_contact'])?$input['primary_contact']:0;
        $cat['primary_contact']         = $primary_contact;
        if($primary_contact==1){
            $primary_contact_id         = null;
            if(isset($input['dispatcher_id']) && !empty($input['dispatcher_id'])){
                $primary_contact_id     = implode(',', $input['dispatcher_id']);
            }
            $cat['primary_contact_id']  = $primary_contact_id;
        }else{
            $cat['primary_contact_id']  = null;
        }
        if(isset($input['include_address']) && $input['include_address'] == 1){
            $cat['include_department_address']  = 1;
        }
        if(isset($input['invite']) && $input['invite'] == 1){
            $cat['is_invite']           = 1;
        }
        $cat['created_by']              = auth()->guard('company-admin')->user()->id;
        $getBroker                      = BrokerContact::create($cat);
        return response()->json(['success' => true, 'id' => $getBroker->name]);
    }

    public function accessorialSave(Request $request){
        $response['success']                            = false;
        $response['message']                            = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                if($request->has('id') && !is_null($request->id)){
                    $item = BrokerAccessorial::where('id', $request->id)
                        ->update([
                            'company_accessorial_id'    => $request->company_accessorial_id,
                            'rate'                      => $request->rate,
                            'note'                      => $request->note
                        ]);
                    if($item){
                        $response['success']            = true;
                        $response['message']            = "Save successfully";
                    }else{
                        $response['success']            = false;
                        $response['message']            = "Oops! Something went wrong..";
                    }
                }else{
                    $item = BrokerAccessorial::create([
                        'broker_id'                     => $request->broker_id,
                        'company_accessorial_id'        => $request->company_accessorial_id,
                        'rate'                          => $request->rate,
                        'note'                          => $request->note
                    ]);
                    if($item){
                        $response['success']            = true;
                        $response['message']            = "Save successfully";
                    }else{
                        $response['success']            = false;
                        $response['message']            = "Oops! Something went wrong..";
                    }
                }
            }catch (\Exception $e) {
                $response['success']                    = false;
                $response['message']                    = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function accessorialDestroy(Request $request){
        $response['success']                            = false;
        $response['message']                            = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $item                                   = BrokerAccessorial::where('id',$request->id)->first();
                if($item){
                    $item->delete();
                    $response['success']                = true;
                    $response['message']                = "Deleted successfully";
                }else{
                    $response['success']                = false;
                    $response['message']                = "Record not found";
                }
            }catch (\Exception $e) {
                $response['success']                    = false;
                $response['message']                    = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function accessorialView(Request $request){
        $response['success']                            = false;
        $response['message']                            = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $item                                   = Broker::where('id',$request->id)->first();
                if(!is_null($item)){
                    $response['html']                   = view('companyAdmin.broker.ajax.accessorial', compact('item'))->render();
                    $response['success']                = true;
                    $response['message']                = "Retrieved successfully";
                }
            }catch (\Exception $e) {
                $response['success']                    = false;
                $response['message']                    = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function saveGeneralInfo(Request $request)
    {
        $input = $request->all();

        $input['company_id'] = auth()->guard('company-admin')->user()->company_id;

        if($request->address_verified == 1){
            $timezone                                                   = null;
            $timezone_offset                                            = null;
            if(isset($input['timezone_abbr'])){
                $timezone                                               = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset                                        = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']                                          = $timezone;
            $input['timezone_offset']                                   = $timezone_offset;
        }

        if(!is_null($request->broker_id)){
            if(!is_null($input['customer_logo'])){
                $input['customer_logo'] = FileUpload::upload('companyAdmin/broker/customer_logo/', $input['customer_logo']);
            }else{
                unset($input['customer_logo']);
            }

            $broker = Broker::IsCompany()->where('id', $request->broker_id)->first();

            if(is_null($broker->customer_unique_id)){
                $input['customer_unique_id'] = getCompIdentifier().$broker->id;
            }

            $broker->update($input);
        }else{
            if(!is_null($input['customer_logo'])){
                $input['customer_logo'] = FileUpload::upload('companyAdmin/broker/customer_logo/', $input['customer_logo']);
            }else{
                $input['customer_logo'] = null;
            }

            $broker = Broker::create($input);

            if(isset($input['dummy_id']) && !is_null($input['dummy_id']) && $input['dummy_id'] != $broker->id){
                $dummyloadid = $input['dummy_id'];
                Gallery::where('action_id', $dummyloadid)->update(['action_id' => $broker->id]);
            }

            $broker->customer_unique_id = getCompIdentifier().$broker->id;
            $broker->save();
        }

        if($broker){
            return response()->json(['success' => true, 'msg' => 'Customer General Info saved successfully', 'brokerId' => $broker->id]);
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Customer General Info']);
        }
    }

    public function refreshFuelSurchargeOptions(Request $request)
    {
        $optionsHtml = '';
        $optionsHtml .= '<option value="">Select File</option>';

        if($request->type != '' && $request->broker_id != ''){
            $optionsArr = getCustomerFuelSurchargeTemplates($request->type, $request->broker_id);
            if(count($optionsArr) > 0){
                foreach ($optionsArr as $key => $value){
                    $optionsHtml .= '<option value="'.$key.'">'.$value.'</option>';
                }
            }
        }

        return response()->json(['optionsHtml' => $optionsHtml]);
    }

    public function saveBillingInsurance(Request $request)
    {
        $input = $request->all();

        if($request->broker_id != ''){
            $input['customer_development_representative'] = (isset($input['customer_development_representative']) && count($input['customer_development_representative']) > 0) ? implode(',', $input['customer_development_representative']) : null;

            $broker = Broker::updateOrCreate(['id' => $request->broker_id, 'company_id' => auth()->guard('company-admin')->user()->company_id], $input);

            if($broker){
                if(isset($input['carrier_insurance_name']) && isset($input['carrier_insurance_type']) && isset($input['carrier_policy_number']) && isset($input['carrier_limit']) && isset($input['carrier_deductible']) && isset($input['carrier_expiration_date'])){
                    $input['carrier_insurance_name'] = serialize($input['carrier_insurance_name']);
                    $input['carrier_insurance_type'] = serialize($input['carrier_insurance_type']);
                    $input['carrier_policy_number'] = serialize($input['carrier_policy_number']);
                    $input['carrier_limit'] = serialize($input['carrier_limit']);
                    $input['carrier_deductible'] = serialize($input['carrier_deductible']);
                    $input['carrier_expiration_date'] = serialize($input['carrier_expiration_date']);
                    BrokerCarrierInsurance::updateOrCreate(['company_id' => auth()->guard('company-admin')->user()->company_id, 'broker_id' => $broker->id], $input);
                }

                if(isset($input['customer_insurance_name']) && isset($input['customer_insurance_type']) && isset($input['customer_policy_number']) && isset($input['customer_limit']) && isset($input['customer_deductible']) && isset($input['customer_expiration_date'])){
                    $input['customer_insurance_name'] = serialize($input['customer_insurance_name']);
                    $input['customer_insurance_type'] = serialize($input['customer_insurance_type']);
                    $input['customer_policy_number'] = serialize($input['customer_policy_number']);
                    $input['customer_limit'] = serialize($input['customer_limit']);
                    $input['customer_deductible'] = serialize($input['customer_deductible']);
                    $input['customer_expiration_date'] = serialize($input['customer_expiration_date']);
                    BrokerCustomerInsurance::updateOrCreate(['company_id' => auth()->guard('company-admin')->user()->company_id, 'broker_id' => $broker->id], $input);
                }

                if(isset($input['surety_bond_required']) && isset($input['surety_bond_company']) && isset($input['surety_bond_expiration']) && isset($input['surety_bond_amount']) && isset($input['surety_bond_email']) && isset($input['bond_contact_name']) && isset($input['bond_phone_number'])){
                    $input['surety_bond_required'] = serialize($input['surety_bond_required']);
                    $input['surety_bond_company'] = serialize($input['surety_bond_company']);
                    $input['surety_bond_expiration'] = serialize($input['surety_bond_expiration']);
                    $input['surety_bond_amount'] = serialize($input['surety_bond_amount']);
                    $input['surety_bond_email'] = serialize($input['surety_bond_email']);
                    $input['bond_contact_name'] = serialize($input['bond_contact_name']);
                    $input['bond_phone_number'] = serialize($input['bond_phone_number']);
                    BrokerBondInformation::updateOrCreate(['company_id' => auth()->guard('company-admin')->user()->company_id, 'broker_id' => $broker->id], $input);
                }

                if(isset($input['dummy_id']) && !is_null($input['dummy_id']) && $input['dummy_id'] != $broker->id){
                    $dummyloadid = $input['dummy_id'];
                    Gallery::where('action_id', $dummyloadid)->update(['action_id' => $broker->id]);
                }

                return response()->json(['success' => true, 'msg' => 'Billing Insurance saved successfully', 'brokerId' => $broker->id]);
            }else{
                return response()->json(['success' => false, 'msg' => 'Error, on save Billing Insurance']);
            }
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Billing Insurance']);
        }
    }

    public function saveLanesRates(Request $request)
    {
        $input = $request->all();

        if($input['broker_id'] != ''){
            BrokerAccessorial::where('broker_id', $request->broker_id)->forceDelete();

            if(count(array_filter($input['company_accessorial_id'])) > 0){
                foreach ($request->company_accessorial_id as $key => $value){
                    BrokerAccessorial::create([
                        'broker_id' => $input['broker_id'],
                        'company_accessorial_id' => $input['company_accessorial_id'][$key],
                        'rate' => $input['rate'][$key] ?? null,
                    ]);
                }
            }

            if($input['broker_id']){
                return response()->json(['success' => true, 'msg' => 'Lanes & Rates saved successfully', 'brokerId' => $input['broker_id']]);
            }else{
                return response()->json(['success' => false, 'msg' => 'Error, on save Lanes & Rates']);
            }
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Lanes & Rates']);
        }
    }

    public function requiredDocumentProfile(Request $request)
    {
        if(isset($request->documentProfile)){
            $documentProfile = $request->documentProfile;
            $shipperDocuments = unserialize(base64Decode($request->shipperDocuments));
            $shipperDocTypes = unserialize(base64Decode($request->shipperDocTypes));
            $shipperIncludeWBilling = unserialize(base64Decode($request->shipperIncludeWBilling));
            $shipperHideFromDriver = unserialize(base64Decode($request->shipperHideFromDriver));
            $receiverDocuments = unserialize(base64Decode($request->receiverDocuments));
            $receiverDocTypes = unserialize(base64Decode($request->receiverDocTypes));
            $receiverIncludeWBilling = unserialize(base64Decode($request->receiverIncludeWBilling));
            $receiverHideFromDriver = unserialize(base64Decode($request->receiverHideFromDriver));

            $view = view('companyAdmin.broker.ajax.required-document-profile', compact('documentProfile', 'shipperDocuments', 'shipperDocTypes', 'shipperIncludeWBilling', 'shipperHideFromDriver', 'receiverDocuments', 'receiverDocTypes', 'receiverIncludeWBilling', 'receiverHideFromDriver'))->render();

            return response()->json(['success' => true, 'preViewHtml' => $view]);
        }else{
            $view = view('companyAdmin.broker.ajax.required-document-profile')->render();

            return response()->json(['success' => true, 'view' => $view]);
        }
    }

    public function addRequiredDocumentProfile(Request $request)
    {
        $createRequiredDocumentProfile = BrokerRequiredDocumentProfile::create([
            'company_id' => auth()->guard('company-admin')->user()->company_id,
            'profile_name' => $request->profile_name,
            'shipper_documents' => $request->shipper_documents,
            'shipper_doc_types' => $request->shipper_doc_types,
            'shipper_include_w_billing' => $request->shipper_include_w_billing,
            'shipper_hide_from_driver' => $request->shipper_hide_from_driver,
            'receiver_documents' => $request->receiver_documents,
            'receiver_doc_types' => $request->receiver_doc_types,
            'receiver_include_w_billing' => $request->receiver_include_w_billing,
            'receiver_hide_from_driver' => $request->receiver_hide_from_driver,
        ]);

        if($createRequiredDocumentProfile){
            $documentProfileHtml = '<option value="" data-shipper-documents="" data-shipper-doc-types="" data-shipper-include-w-billing="" data-shipper-hide-from-driver="" data-receiver-documents="" data-receiver-doc-types="" data-receiver-include-w-billing="" data-receiver-hide-from-driver="">Select Required Document Profile</option>';
            foreach(getBrokerRequiredDocumentProfiles() as $requiredDocumentProfile){
                $documentProfileHtml .= '<option value="'.$requiredDocumentProfile->id.'" data-shipper-documents="'.base64Encode(serialize($requiredDocumentProfile->shipper_documents)).'" data-shipper-doc-types="'.base64Encode(serialize($requiredDocumentProfile->shipper_doc_types)).'" data-shipper-include-w-billing="'.base64Encode(serialize($requiredDocumentProfile->shipper_include_w_billing)).'" data-shipper-hide-from-driver="'.base64Encode(serialize($requiredDocumentProfile->shipper_hide_from_driver)).'" data-receiver-documents="'.base64Encode(serialize($requiredDocumentProfile->receiver_documents)).'" data-receiver-doc-types="'.base64Encode(serialize($requiredDocumentProfile->receiver_doc_types)).'" data-receiver-include-w-billing="'.base64Encode(serialize($requiredDocumentProfile->receiver_include_w_billing)).'" data-receiver-hide-from-driver="'.base64Encode(serialize($requiredDocumentProfile->receiver_hide_from_driver)).'">'.$requiredDocumentProfile->profile_name.'</option>';
            }

            return response()->json(['success' => true, 'msg' => 'Required Document Profile saved successfully', 'documentProfileHtml' => $documentProfileHtml, 'new_profile_id' => $createRequiredDocumentProfile->id]);
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Required Document Profile.']);
        }
    }

    public function saveRequiredDocuments(Request $request)
    {
        $input = $request->all();
        $section = $request->section;
        $brokerId = $request->broker_id;

        if($brokerId != ""){
            $broker = Broker::find($brokerId);
            $brokerCompanyId = $broker->company_id;

            if($section == '1'){
                $isExists = BrokerRequiredDocument::where('company_id', $brokerCompanyId)->where('broker_id', $brokerId)->count();

                if($isExists > 0){
                    BrokerRequiredDocument::where('company_id', $brokerCompanyId)->where('broker_id', $brokerId)->delete();
                }

                $requiredDocumentProfile = $input['required_document_profile'] ?? null;

                if(!is_null($requiredDocumentProfile)){
                    BrokerRequiredDocument::create([
                        'company_id' => $brokerCompanyId,
                        'broker_id' => $brokerId,
                        'required_document_profile' => $requiredDocumentProfile,
                        'shipper_documents' => $input['shipper_documents'] ?? null,
                        'shipper_doc_types' => $input['shipper_doc_types'] ?? null,
                        'shipper_include_w_billing' => $input['shipper_include_w_billing'] ?? null,
                        'shipper_hide_from_driver' => $input['shipper_hide_from_driver'] ?? null,
                        'receiver_documents' => $input['receiver_documents'] ?? null,
                        'receiver_doc_types' => $input['receiver_doc_types'] ?? null,
                        'receiver_include_w_billing' => $input['receiver_include_w_billing'] ?? null,
                        'receiver_hide_from_driver' => $input['receiver_hide_from_driver'] ?? null,
                    ]);
                }
            }else if($section == '2'){
                $isExists = BrokerAdditionalRequiredDocument::where('company_id', $brokerCompanyId)->where('broker_id', $brokerId)->count();

                if($isExists > 0){
                    BrokerAdditionalRequiredDocument::where('company_id', $brokerCompanyId)->where('broker_id', $brokerId)->delete();
                }

                if(isset($input['documents']) && count(array_filter($input['documents'])) > 0){
                    BrokerAdditionalRequiredDocument::create([
                        'company_id' => $brokerCompanyId,
                        'broker_id' => $brokerId,
                        'documents' => $input['documents'] ?? null,
                        'doc_types' => $input['doc_types'] ?? null,
                        'include_w_billing' => $input['include_w_billing'] ?? null,
                        'hide_from_driver' => $input['hide_from_driver'] ?? null,
                    ]);
                }
            }

            return response()->json(['success' => true, 'brokerId' => $brokerId]);
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Required Documents']);
        }
    }

    public function saveContacts(Request $request)
    {
        $input = $request->all();

        if($input['broker_id'] != ''){
            $contacts = $request->contacts ?? [];

            if(!empty($contacts)){
                if($input['is_create_rep'] == '0'){
                    BrokerContact::where('source', 'broker')->where('action_id', $input['broker_id'])->whereNotIn('id', array_column($contacts, 'id'))->forceDelete();
                }

                foreach ($contacts as $key => $contact) {
                    if(!empty($contact['first_name']) && !empty($contact['last_name'])){
                        $contact['name'] = $contact['first_name'].' '.$contact['last_name'];;

                        if(isset($contact['id']) && $contact['id'] != ''){
                            if(!is_null($contact['representative_profile_img'])){
                                $contact['representative_profile_img'] = FileUpload::upload('companyAdmin/broker/contact_profile/', $contact['representative_profile_img']);
                            }else{
                                unset($contact['representative_profile_img']);
                            }

                            $contact['department'] = $contact['department'] ?? null;
                            $contact['is_invite'] = $contact['is_invite'] ?? '0';

                            BrokerContact::updateOrCreate(['id' => $contact['id'], 'source' => 'broker', 'action_id' => $input['broker_id']], $contact);
                        }else{
                            if(!is_null($contact['representative_profile_img'])){
                                $contact['representative_profile_img'] = FileUpload::upload('companyAdmin/broker/contact_profile/', $contact['representative_profile_img']);
                            }else{
                                $contact['representative_profile_img'] = null;
                            }

                            $contact['source'] = 'broker';
                            $contact['action_id'] = $input['broker_id'];
                            $contact['department'] = $contact['department'] ?? null;
                            $contact['is_invite'] = $contact['is_invite'] ?? '0';

                            BrokerContact::create($contact);
                        }
                    }
                }

                /*
                if(isset($input['dummy_id']) && !is_null($input['dummy_id']) && $input['dummy_id'] != $input['broker_id']){
                    $dummyloadid = $input['dummy_id'];
                    Gallery::where('action_id', $dummyloadid)->update(['action_id' => $input['broker_id']]);
                }
                */
            }

            return response()->json(['success' => true, 'msg' => 'Contacts saved successfully', 'brokerId' => $input['broker_id']]);
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Contacts']);
        }
    }

    public function getContacts(Request $request)
    {
        $options = '';
        if(isset($request->id))
        {
            $brokerContacts = getBrokerContactsOptions('broker', $request->id);

            foreach ($brokerContacts as $key => $value){
                $options .= '<option value="'.$key.'">'.$value.'</option>';
            }

            return response()->json(['success' => 'true', 'options' => $options]);
        }else{
            return response()->json(['success' => 'false', 'options' => '']);
        }
    }

    public function saveNotificationCenter(Request $request)
    {
        $input = $request->all();

        if($input['broker_id'] != ''){
            $input['standard_notification'] = $input['standard_notification'] ?? [];
            $input['geofense_notification'] = $input['geofense_notification'] ?? [];
            $input['edi_notification'] = $input['edi_notification'] ?? [];
            $input['integration_notification'] = $input['integration_notification'] ?? [];

            $broker = Broker::updateOrCreate(['id' => $input['broker_id'], 'company_id' => auth()->guard('company-admin')->user()->company_id], $input);

            if($broker){
                return response()->json(['success' => true, 'msg' => 'Notification Center saved successfully', 'brokerId' => $broker->id]);
            }else{
                return response()->json(['success' => false, 'msg' => 'Error, on save Notification Center']);
            }
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Notification Center']);
        }
    }

    public function saveCustomerNotes(Request $request)
    {
        $input = $request->all();

        if($input['broker_id'] != ''){
            $isExists = CustomerNote::IsCompany()->where('broker_id', $input['broker_id'])->count();

            if($isExists > 0){
                CustomerNote::IsCompany()->where('broker_id', $input['broker_id'])->delete();
            }

            foreach ($input['customer-notes'] as $customerNote){
                if(isset($customerNote['add_to']) && $customerNote['add_to'] != ''){
                    CustomerNote::create([
                        'company_id' => auth()->guard('company-admin')->user()->company_id,
                        'broker_id' => $input['broker_id'],
                        'add_to' => $customerNote['add_to'],
                        'notes' => $customerNote['notes'],
                    ]);
                }
            }

            return response()->json(['success' => true, 'msg' => 'Customer Notes saved successfully', 'brokerId' => $input['broker_id']]);
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Customer Notes']);
        }
    }

    public function saveCustomerFiles(Request $request){
        $input = $request->all();

        if($input['broker_id'] != ''){
            CustomerFiles::IsCompany()->where('broker_id', $input['broker_id'])->where('section', '1')->delete();
            if(isset($input['contracts']) && count($input['contracts']) > 0){
                foreach ($input['contracts'] as $contract){
                    if($contract['name'] != ''){
                        CustomerFiles::create([
                            'company_id' => auth()->guard('company-admin')->user()->company_id,
                            'broker_id' => $input['broker_id'],
                            'section' => '1', // Contracts
                            'name' => $contract['name'],
                            'status' => $contract['status'],
                            'start_date' => $contract['start_date'],
                            'end_date' => $contract['end_date'],
                            'submitted_to' => $contract['submitted_to'] ?? null,
                            'submission_date' => $contract['submission_date'],
                            'submitted_by' => $contract['submitted_by'],
                        ]);
                    }
                }
            }

            CustomerFiles::IsCompany()->where('broker_id', $input['broker_id'])->where('section', '2')->delete();
            if(isset($input['pricing']) && count($input['pricing']) > 0){
                foreach ($input['pricing'] as $pricing){
                    if($pricing['name'] != ''){
                        CustomerFiles::create([
                            'company_id' => auth()->guard('company-admin')->user()->company_id,
                            'broker_id' => $input['broker_id'],
                            'section' => '2', // Pricing
                            'name' => $pricing['name'],
                            'status' => $pricing['status'],
                            'start_date' => $pricing['start_date'],
                            'end_date' => $pricing['end_date'],
                            'submitted_to' => $pricing['submitted_to'] ?? null,
                            'submission_date' => $pricing['submission_date'],
                            'submitted_by' => $pricing['submitted_by'],
                        ]);
                    }
                }
            }

            CustomerFiles::IsCompany()->where('broker_id', $input['broker_id'])->where('section', '3')->delete();
            if(isset($input['fuelTables']) && count($input['fuelTables']) > 0){
                foreach ($input['fuelTables'] as $fuelTables){
                    if($fuelTables['name'] != ''){
                        CustomerFiles::create([
                            'company_id' => auth()->guard('company-admin')->user()->company_id,
                            'broker_id' => $input['broker_id'],
                            'section' => '3', // Fuel Tables
                            'name' => $fuelTables['name'],
                            'status' => $fuelTables['status'],
                            'start_date' => $fuelTables['start_date'],
                            'end_date' => $fuelTables['end_date'],
                            'submitted_to' => $fuelTables['submitted_to'] ?? null,
                            'submission_date' => $fuelTables['submission_date'],
                            'submitted_by' => $fuelTables['submitted_by'],
                        ]);
                    }
                }
            }

            CustomerFiles::IsCompany()->where('broker_id', $input['broker_id'])->where('section', '4')->delete();
            if(isset($input['quickQuotes']) && count($input['quickQuotes']) > 0){
                foreach ($input['quickQuotes'] as $quickQuotes){
                    if($quickQuotes['name'] != ''){
                        CustomerFiles::create([
                            'company_id' => auth()->guard('company-admin')->user()->company_id,
                            'broker_id' => $input['broker_id'],
                            'section' => '4', // Quick Quotes
                            'name' => $quickQuotes['name'],
                            'status' => $quickQuotes['status'],
                            'start_date' => $quickQuotes['start_date'],
                            'end_date' => $quickQuotes['end_date'],
                            'submitted_to' => $quickQuotes['submitted_to'] ?? null,
                            'submission_date' => $quickQuotes['submission_date'],
                            'submitted_by' => $quickQuotes['submitted_by'],
                        ]);
                    }
                }
            }

            CustomerFiles::IsCompany()->where('broker_id', $input['broker_id'])->where('section', '5')->delete();
            if(isset($input['performanceReviews']) && count($input['performanceReviews']) > 0){
                foreach ($input['performanceReviews'] as $performanceReviews){
                    if($performanceReviews['name'] != ''){
                        CustomerFiles::create([
                            'company_id' => auth()->guard('company-admin')->user()->company_id,
                            'broker_id' => $input['broker_id'],
                            'section' => '5', // Performance Reviews
                            'name' => $performanceReviews['name'],
                            'status' => $performanceReviews['status'],
                            'start_date' => $performanceReviews['start_date'],
                            'end_date' => $performanceReviews['end_date'],
                            'submitted_to' => $performanceReviews['submitted_to'] ?? null,
                            'submission_date' => $performanceReviews['submission_date'],
                            'submitted_by' => $performanceReviews['submitted_by'],
                        ]);
                    }
                }
            }

            CustomerFiles::IsCompany()->where('broker_id', $input['broker_id'])->where('section', '6')->delete();
            if(isset($input['insuranceRequirements']) && count($input['insuranceRequirements']) > 0){
                foreach ($input['insuranceRequirements'] as $insuranceRequirements){
                    if($insuranceRequirements['name'] != ''){
                        CustomerFiles::create([
                            'company_id' => auth()->guard('company-admin')->user()->company_id,
                            'broker_id' => $input['broker_id'],
                            'section' => '6', // Insurance Requirements
                            'name' => $insuranceRequirements['name'],
                            'status' => $insuranceRequirements['status'],
                            'start_date' => $insuranceRequirements['start_date'],
                            'end_date' => $insuranceRequirements['end_date'],
                            'submitted_to' => $insuranceRequirements['submitted_to'] ?? null,
                            'submission_date' => $insuranceRequirements['submission_date'],
                            'submitted_by' => $insuranceRequirements['submitted_by'],
                        ]);
                    }
                }
            }

            if(!empty($input['dummy_id']) && $input['dummy_id'] != $input['broker_id']){
                $dummyloadid = $input['dummy_id'];
                Gallery::where('action_id', $dummyloadid)->update(['action_id' => $input['broker_id']]);
            }

            return response()->json(['success' => true, 'msg' => 'Customer Files saved successfully', 'brokerId' => $input['broker_id']]);
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Customer Files']);
        }
    }

    public function saveInternalNotes(Request $request)
    {
        $input = $request->all();

        if($input['broker_id'] != ''){
            $isExists = InternalNote::IsCompany()->where('broker_id', $input['broker_id'])->count();

            if($isExists > 0){
                InternalNote::IsCompany()->where('broker_id', $input['broker_id'])->delete();
            }

            foreach ($input['internal-notes'] as $internalNote){
                if(isset($internalNote['add_to']) && $internalNote['add_to'] != ''){
                    InternalNote::create([
                        'company_id' => auth()->guard('company-admin')->user()->company_id,
                        'broker_id' => $input['broker_id'],
                        'add_to' => $internalNote['add_to'],
                        'notes' => $internalNote['notes'],
                    ]);
                }
            }

            return response()->json(['success' => true, 'msg' => 'Internal Notes saved successfully', 'brokerId' => $input['broker_id']]);
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Internal Notes']);
        }
    }

    public function saveGeofence(Request $request)
    {
        $input = $request->all();

        if($input['broker_id'] != ''){
            $input['geofence_status'] = $input['geofence_status'] ?? '0';

            $broker = Broker::updateOrCreate(['id' => $input['broker_id'], 'company_id' => auth()->guard('company-admin')->user()->company_id], $input);

            if($broker){
                return response()->json(['success' => true, 'msg' => 'Customer details saved successfully']);
            }else{
                return response()->json(['success' => false, 'msg' => 'Error, on save Geofence']);
            }
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Geofence']);
        }
    }

    public function savePlace(Request $request){
        $input = $request->all();

        if($input['broker_id'] != ''){
            $broker = Broker::updateOrCreate(['id' => $input['broker_id'], 'company_id' => auth()->guard('company-admin')->user()->company_id], ['geofence_trimble_place_id' => $input['placeId'], 'geofence_updated_at' => Carbon::now()->format('m/d/Y')]);

            if($broker){
                return response()->json(['success' => true]);
            }else{
                return response()->json(['success' => false]);
            }
        }else{
            return response()->json(['success' => false]);
        }
    }

    public function updateBrokerStatus(Request $request)
    {
        $broker_id = $request->broker_id;
        $status = $request->status;

        $broker = Broker::find($broker_id);
        $broker->status = $status;
        $broker->save();

        if($broker){
            return response()->json(['success' => true, 'message' => 'Broker Status updated successfully.']);
        }else{
            return response()->json(['success' => false, 'message' => 'Oops! Something went wrong..']);
        }
    }

    public function getHoursOfOperation($broker) {
        $currentDate = Carbon::now()->timezone($this->timeZone);
        $format = (getTimeFormat() == '12') ? config('settings.time_format.12_hour') : config('settings.time_format.24_hour');

        $hoursOfOperations =  array(
            'sunday' => ($broker->ho_sun_open != '' && $broker->ho_sun_close != '') ? array(
                'open' => Carbon::parse($broker->ho_sun_open)->format($format),
                'close' => Carbon::parse($broker->ho_sun_close)->format($format)
            ) : 'Closed',
            'monday' => ($broker->ho_mon_open != '' && $broker->ho_mon_close != '') ? array(
                'open' => Carbon::parse($broker->ho_mon_open)->format($format),
                'close' => Carbon::parse($broker->ho_mon_close)->format($format)
            ) : 'Closed',
            'tuesday' => ($broker->ho_tue_open != '' && $broker->ho_tue_close != '') ? array(
                'open' => Carbon::parse($broker->ho_tue_open)->format($format),
                'close' => Carbon::parse($broker->ho_tue_close)->format($format)
            ) : 'Closed',
            'wednesday' => ($broker->ho_wed_open != '' && $broker->ho_wed_close != '') ? array(
                'open' => Carbon::parse($broker->ho_wed_open)->format($format),
                'close' => Carbon::parse($broker->ho_wed_close)->format($format)
            ) : 'Closed',
            'thursday' => ($broker->ho_thu_open != '' && $broker->ho_thu_close != '') ? array(
                'open' => Carbon::parse($broker->ho_thu_open)->format($format),
                'close' => Carbon::parse($broker->ho_thu_close)->format($format)
            ) : 'Closed',
            'friday' => ($broker->ho_fri_open != '' && $broker->ho_fri_close != '') ? array(
                'open' => Carbon::parse($broker->ho_fri_open)->format($format),
                'close' => Carbon::parse($broker->ho_fri_close)->format($format)
            ) : 'Closed',
            'saturday' => ($broker->ho_sat_open != '' && $broker->ho_sat_close != '') ? array(
                'open' => Carbon::parse($broker->ho_sat_open)->format($format),
                'close' => Carbon::parse($broker->ho_sat_close)->format($format)
            ) : 'Closed',
        );

        $status = '';
        if(is_array($hoursOfOperations[strtolower($currentDate->format('l'))])){
            if($currentDate->gt($hoursOfOperations[strtolower($currentDate->format('l'))]['open']) && $currentDate->lt($hoursOfOperations[strtolower($currentDate->format('l'))]['close'])){
                $status = '<span style="color: #2F9840;">Open</span> - Closes '.$hoursOfOperations[strtolower($currentDate->format('l'))]['close'];
            }else{
                $status = '<span style="color: #dc3545;">Closed</span> - Opens '.$hoursOfOperations[strtolower($currentDate->format('l'))]['open'];
            }
        }else{
            if(is_array($hoursOfOperations[strtolower($currentDate->addDay(1)->format('l'))])){
                $status = '<span style="color: #dc3545;">Closed</span> - Opens '.$hoursOfOperations[strtolower($currentDate->format('l'))]['open'].' '.$currentDate->format('D');
            }else{
                $status = '<span style="color: #dc3545;">Closed</span>';
            }
        }
        $hoursOfOperations['status'] = $status;

        return $hoursOfOperations;
    }

    public function updateRepresentative(Request $request)
    {
        $broker_id = $request->broker_id;
        $representative = $request->representative;

        $brokerHasRepresentative = BrokerContact::where('source', 'broker')->where('action_id', $broker_id)->where('is_representative', '1')->count();
        if($brokerHasRepresentative > 0){
            BrokerContact::where('source', 'broker')->where('action_id', $broker_id)->update(['is_representative' => '0']);
        }

        $brokerContact = BrokerContact::find($representative);
        $brokerContact->is_representative = '1';
        $brokerContact->save();

        if($brokerContact){
            return response()->json(['success' => true]);
        }else{
            return response()->json(['success' => false, 'message' => 'Oops! Something went wrong..']);
        }
    }

    public function createRep(Request $request)
    {
        return view('companyAdmin.broker.create-rep');
    }

    public function getCustomerFiles(Request $request)
    {
        $sections = customerFileTypes();
        $sectionsFileKeys = customerFileKeys();
        $brokerId = request()->broker_id;

        $data = CustomerFiles::IsCompany()->where('broker_id', $brokerId)->select('broker_id', 'section', 'name', 'submitted_by', 'submission_date', DB::raw('ROW_NUMBER() OVER (PARTITION BY section ORDER BY section, id) AS file_index'))->orderBy('section','ASC')->orderBy('id','ASC')->get();

        return Datatables::of($data)
            ->editColumn('section', function($row) use ($sections){
                return $sections[$row->section] ?? '-';
            })
            ->editColumn('name', function($row) use($sectionsFileKeys) {
                if($row->name != ''){
                    if(isset($sectionsFileKeys[$row->section])){
                        return '<a href="javascript:void(0);" class="customer-file-name gallary-edit" data-id="'.$row->broker_id.'" data-source="brokers" data-field="'.$sectionsFileKeys[$row->section].($row->file_index - 1).'" data-toggle="modal" data-target="#galleryModal">'.$row->name.'</a>';
                    }
                }
                return '-';
            })
            ->addColumn('description', function($row){
                return '-';
            })
            ->rawColumns(['section', 'name', 'description'])
            ->make(true);
    }

    public function saveFilesAndImages(Request $request)
    {
        $input = $request->all();
        $broker = Broker::IsCompany()->findOrFail($input['customer']);

        if($input['file_type'] === '1'){
            $customerFileKey = customerFileKeys()[$input['category']].$broker->customer_files($input['category'])->count();
            $broker->galleries()->where('field_name', $customerFileKey)->delete();

            $customerFiles = CustomerFiles::create([
                'company_id' => auth()->guard('company-admin')->user()->company_id,
                'broker_id' => $input['customer'],
                'section' => $input['category'],
                'name' => $input['file_name'],
                'submission_date' => $input['date_entered'],
                'submitted_by' => auth()->guard('company-admin')->user()->full_name,
            ]);

            if($customerFiles){
                $customerGalleryFiles = array(
                    'company_id' => auth()->guard('company-admin')->user()->company_id,
                    'action_id' => $input['customer'],
                    'source' => 'brokers',
                    'field_name' => $customerFileKey,
                    'file_name' => $input['file_name'],
                    'uploaded_date' => Carbon::parse($input['date_entered'])->format('m/d/Y H:i'),
                    'created_by' => auth()->guard('company-admin')->user()->id,
                    'created_by_type' => auth()->guard('company-admin')->user()->type,
                    'status' => 'approved',
                    'file_type' => getRequestDocumentsList(Str::beforeLast($customerFileKey, '_'),'field_name')['file_type'],
                );

                foreach($input['uploaded_files'] as $image){
                    if(isset($image)) {
                        $customerGalleryFiles['image'] = FileUpload::upload('companyAdmin/gallery', $image);
                    }
                    Gallery::create($customerGalleryFiles);
                }

                return response()->json(['success' => true, 'message' => 'Files added successfully.']);
            }else{
                return response()->json(['success' => false, 'message' => 'Oops! Something went wrong..']);
            }
        }else if($input['file_type'] === '2'){
            $brokerCompanyImage = array(
                'company_id' => auth()->guard('company-admin')->user()->company_id,
                'broker_id' => $input['customer'],
                'uploaded_date' => Carbon::parse($input['date_entered'])->format('m/d/Y'),
                'file_name' => $input['file_name'],
            );

            foreach($input['uploaded_files'] as $image){
                if(isset($image)) {
                    $type = $image->getClientOriginalExtension();
                    if($type == 'png' || $type == 'jpeg' || $type == 'jpg' || $type == 'gif'){
                        $brokerCompanyImage['image'] = FileUpload::upload('companyAdmin/broker/company_images/', $image);
                    }
                }
                BrokerCompanyImage::create($brokerCompanyImage);
            }

            return response()->json(['success' => true, 'message' => 'Images added successfully.']);
        }
    }

    public function searchImages(Request $request)
    {
        $broker_id = $request->broker_id;
        $search = $request->search;

        $brokerCompanyImages = BrokerCompanyImage::IsCompany()->where('broker_id', $broker_id)->where('file_name', 'like', '%'.$search.'%')->get();

        $imageView = '';
        foreach ($brokerCompanyImages as $brokerCompanyImage){
            $imageView .= '<div class="col-xl-2 col-sm-4 col-6">';
            $imageView .= '<div class="company-image-box text-center">';
            $imageView .= '<div class="overflow-hidden mb-1">';
            $imageView .= '<a data-fslightbox="gallery" href="'.asset('storage/companyAdmin/broker/company_images/'.$brokerCompanyImage->image).'">';
            $imageView .= '<img src="'.asset('storage/companyAdmin/broker/company_images/'.$brokerCompanyImage->image).'" alt="" class="img-fluid">';
            $imageView .= '</a>';
            $imageView .= '</div>';
            $imageView .= '<a href="javascript:void(0);" class="text-theme-color mt-2 text-underline d-block" >'.$brokerCompanyImage->file_name.'</a>';
            $imageView .= '</div>';
            $imageView .= '</div>';
        }

        return response()->json(['success' => true, 'imageView' => $imageView]);
    }

    public function childContentArray($rowData){
        $childDataArr = array(
            'load_id' => $rowData->load_id,
            'load_detail_id' => $rowData->id,
        );

        return json_encode($childDataArr);
    }

    public function billLoadsIndex(Request $request)
    {
        try {
            $getCompanyAdmin = auth()->guard('company-admin')->user();

            $getField2 = '';
            $isCheck = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','customer_billing_loads')->first();
            if(!empty($isCheck->id)){
                $getField2 = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_customer_billing_loads_first'));
            }else{
                $getField2 = config('settings.table_customer_billing_loads_first');
            }

            if ($request->ajax()) {
                /*
                $data = LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
                        ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
                        ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
                        ->where('load_details.is_disable', 0)
                        ->where(function($query) {
                            $query->where('load_details.load_status', 6)->where(function($q) {
                                    $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                                });
                        })
                        ->where('load_details.load_status', 6)
                        ->where(function($q) {
                            $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                        })
                        ->whereHas('loads', function($q){
                            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                        });
                */
                $brokerId = base64Decode($request->broker_id);

                $data = LoadDetail::select('load_details.*', 'brokers.factor_invoices as broker_factor_invoices', 'load_billing_invoice.id as invoice_id', 'load_billing_invoice.invoice_number')
                    ->join('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
                    ->leftJoin('load_billing_invoice', 'load_billing_invoice.load_detail_id', '=', 'load_details.id')
                    ->leftJoin('load_billing_activities', 'load_billing_activities.load_detail_id', '=', 'load_details.id')
                    ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
                    ->where('load_details.brokerage_firm', $brokerId)
                    ->where(function($query) {
                        $query->where('load_details.load_status', 5) // DELIVERED: without Document
                        ->orWhere(function($que) {
                            $que->whereIn('load_details.load_status', [5, 6]) // DELIVERED & READY TO BILL: With Document Missing Or Uploaded
                            ->where('load_details.is_disable', 0)
                                ->where(function($q) {
                                    $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                                })
                                ->whereIn('load_billing_activities.paperwork_received', ['0', '1'])
                                ->whereNotNull('load_details.driver_id')
                                ->where('load_details.driver_id', '!=', 0);
                        })->orWhere(function($que) {
                            $que->where(function($q) {
                                $q->where('load_details.load_status', 7) // ON HOLD
                                ->orWhere('load_details.is_on_hold', 1);
                            })->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                        })->orWhere(function($que) {
                            $que->whereIn('load_details.load_status', [8, 10]) // INVOICED & PAID
                            ->where(function ($q){
                                $q->where('load_billing_activities.invoiced', '1')->orWhere('load_billing_activities.paid_invoice', '1');
                            });
                        });
                    })
                    ->whereHas('loads', function($q){
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    });

                // General filter code - start
                foreach( $getField2 as $key => $value ) {
                    $conditionReq       = 'search_string_condition_'.$key;
                    $sortReq            = 'search_string_sort_'.$key;
                    $conditionDateReq   = 'search_date_condition_'.$key;
                    $sortDateReq        = 'search_date_sort_'.$key;
                    $conditionStatusReq = 'search_status_condition_'.$key;
                    $sortStatusReq      = 'search_status_sort_'.$key;

                    // String filter
                    if( isset( request()->$conditionReq ) ) {
                        $typeReq        = 'search_string_type_'.$key;
                        $type           = request()->$typeReq;

                        if( $type == 'string' ) {
                            Common::getStringFilterResult( $data, $key );
                        }
                    }

                    // Status filter
                    if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                        Common::getStatusFilterResult( $data, $key );
                    }

                    // Date filter
                    if( isset( request()->$conditionDateReq ) ) {
                        $typeReq        = 'search_date_type_'.$key;
                        $type           = request()->$typeReq;

                        if( $type == 'date' ) {
                            Common::getDateFilterResult( $data, $key );
                        }
                    }

                    // String sort only
                    if( isset( request()->$sortReq ) ) {
                        Common::sortStringFilter( $data, $key );
                    }

                    // Date sort only
                    if( isset( request()->$sortDateReq ) ) {
                        Common::sortDateFilter( $data, $key );
                    }

                    // Status sort only
                    if( isset( request()->$sortStatusReq ) ) {
                        Common::sortStatusFilter( $data, $key );
                    }
                }
                // General filter code - end

                return \Yajra\DataTables\DataTables::of($data)
                    ->addIndexColumn()
                    ->setRowClass(function($row){
                        $class          = "";
                        if($row->billing_load_is_audit == '1' && $row->billing_load_audit_by_id != auth()->guard('company-admin')->user()->id){
                            $class = 'other-user-audit-lock';
                        }
                        return $class;
                    })
                    ->addColumn('audit', function($row){
                        if($row->billing_load_is_audit == '1' && $row->billing_load_audit_by_id != auth()->guard('company-admin')->user()->id){
                            $auditHtml ='<a href="javascript:void(0);" class="text-theme-color audit-lock" data-load="'.$row->load_id.'" data-load-details="'.$row->id.'" title="locked by '.$row->billing_load_audit_by_name.'"><i class="fa fa-lock" aria-hidden="true"></i></a>';
                            $auditHtml .='<a href="javascript:void(0);" class="text-theme-color text-underline d-none audit-load" data-load="'.$row->load_id.'" data-load-details="'.$row->id.'">Audit</a>';
                        }else{
                            $auditHtml ='<a href="javascript:void(0);" class="text-theme-color d-none audit-lock" data-load="'.$row->load_id.'" data-load-details="'.$row->id.'" title="locked"><i class="fa fa-lock" aria-hidden="true"></i></a>';
                            $auditHtml .='<a href="javascript:void(0);" class="text-theme-color text-underline audit-load" data-load="'.$row->load_id.'" data-load-details="'.$row->id.'">Audit</a>';
                        }

                        return $auditHtml;
                    })
                    ->editColumn('load_number', function($row){
                        $html = '<div class="status-block">';
                        $html .= '<div class="round-icon details-control"><i class="fa fa-caret-down" aria-hidden="true"></i></div>';

                        if ($row->is_disable == 0) {
                            if(in_array($row->load_type, ['0', '6'])) {
                                if ($row->load_type == 0) {
                                    $html .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                                } else {
                                    $html .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                                }
                            } else {
                                $html .= '<a href="'.route('company.admin.loads.edit', base64Encode($row->load_id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        } else {
                            if ($row->load_type == 0) {
                                $html .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                            } else {
                                $html .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        }

                        return $html;
                    })
                    ->addColumn('invoice_number', function($row){
                        if(isset($row->invoice_number)){
                            return '<a href="'.route('company.admin.billing.invoice.show', ['id' => base64Encode($row->invoice_id)]).'" target="_blank">'.$row->invoice_number.'</a>';
                        }
                        return '';
                    })
                    ->editColumn('shipper_location', function($row){
                        $html = [];
                        if($row->shipper_address_one){
                            $html[0]    = $row->shipper_address_one;
                        }
                        if($row->shipper_address_two){
                            $html[1]    = $row->shipper_address_two;
                        }
                        if($row->shipper_city){
                            $html[2]    = $row->shipper_city;
                        }
                        if($row->shipper_state){
                            $html[3]    = $row->shipper_state;
                        }
                        if($row->shipper_zip){
                            $html[4]    = $row->shipper_zip;
                        }
                        if(count($html) == 0){
                            return $row->shipper_location;
                        }else{
                            return implode(', ', $html);
                        }
                    })
                    ->editColumn('actual_shipper_date', function($row){
                        return !is_null($row->actual_shipper_date) ? $row->actual_shipper_date." ".$row->shipper_timezone_abbr : '';
                    })
                    ->editColumn('delivery_location', function($row){
                        $html = [];
                        if($row->delivery_address_one){
                            $html[0]    = $row->delivery_address_one;
                        }
                        if($row->delivery_address_two){
                            $html[1]    = $row->delivery_address_two;
                        }
                        if($row->delivery_city){
                            $html[2]    = $row->delivery_city;
                        }
                        if($row->delivery_state){
                            $html[3]    = $row->delivery_state;
                        }
                        if($row->delivery_zip){
                            $html[4]    = $row->delivery_zip;
                        }
                        if(count($html) == 0){
                            return $row->delivery_location;
                        }else{
                            return implode(', ', $html);
                        }
                    })
                    ->editColumn('actual_delivery_date', function($row){
                        return !is_null($row->actual_delivery_date) ? $row->actual_delivery_date." ".$row->delivery_timezone_abbr : '';
                    })
                    ->addColumn('status', function($row){
                        return getLoadStatus($row->load_status,$row->is_on_hold);
                    })
                    ->filterColumn('status', function ($query, $keyword) {
                        if (Str::contains('pending' , Str::lower($keyword))) {
                            $query = $query->where('load_status', 0);
                        }
                        if (Str::contains('accepted' , Str::lower($keyword))) {
                            $query = $query->where('load_status', 1);
                        }
                        if (Str::contains('preplaned' , Str::lower($keyword))) {
                            $query = $query->where('load_status', 2);
                        }
                        if (Str::contains('confirmed' , Str::lower($keyword))) {
                            $query = $query->where('load_status', 3);
                        }
                        if (Str::contains('in transit' , Str::lower($keyword))) {
                            $query = $query->where('load_status', 4);
                        }
                        if (Str::contains('delivered' , Str::lower($keyword))) {
                            $query = $query->where('load_status', 5);
                        }
                        if (Str::contains('ready to bill' , Str::lower($keyword))) {
                            $query = $query->where('load_status', 6);
                        }
                        if (Str::contains('on hold' , Str::lower($keyword))) {
                            $query = $query->where('load_status', 7);
                        }
                        if (Str::contains('complete' , Str::lower($keyword))) {
                            $query = $query->where('load_status', 8);
                        }
                        if (Str::contains('cancelled' , Str::lower($keyword))) {
                            $query = $query->where('load_status', 9);
                        }
                        return $query;
                    })
                    ->editColumn('total_bill', function($row){
                        return getPrice($row->total_bill);
                    })
                    ->addColumn('factor_invoices', function($row){
                        return (!is_null($row->broker_factor_invoices) && $row->broker_factor_invoices == '1') ? 'Factored' : 'Direct Bill';
                    })
                    ->addColumn('documents', function($row){
                        if(getLoadDocumentsCount($row) == 0){
                            return '<i class="fa fa-file-text-o document-complete document-icon" aria-hidden="true"></i>';
                        }else{
                            if(!is_null($row->load_billing_activities) && $row->load_billing_activities->paperwork_received == '1'){
                                return '<i class="fa fa-file-text-o document-complete document-icon" aria-hidden="true"></i>';
                            }
                            return '<i class="fa fa-file-text-o document-in-review document-icon" aria-hidden="true"></i>';
                        }
                    })
                    ->addColumn('review', function($row){
                        if(!is_null($row->load_billing_activities) && $row->load_billing_activities->audit_complete == '1'){
                            return 'Complete';
                        }
                        return 'In Review';
                    })
                    ->addColumn('invoice_now', function($row){
                        /*
                        if(!is_null($row->load_billing_activities) && $row->load_billing_activities->paperwork_received == '1' && $row->load_billing_activities->audit_complete == '1'){
                            if ($row->load_billing_activities->paid_invoice == '1'){
                                return '<span class="text-green">Paid</span>';
                            }else if($row->load_billing_activities->invoiced == '1'){
                                return '<span class="text-green">Invoiced <i class="fa fa-reply" aria-hidden="true"></i></span>';
                            }
                            return '<a href="'.route('company.admin.billing.invoice.create', ['id' => base64Encode($row->id)]).'" class="btn btn-success">Invoice Now</a>';
                        }
                        return '<a href="javascript:void(0);" class="btn btn-liteGray">Invoice Now</a>';
                        */
                        if(!is_null($row->load_billing_activities)){
                            if($row->load_billing_activities->paperwork_received == '1' && $row->load_billing_activities->audit_complete == '1'){
                                if ($row->load_billing_activities->paid_invoice == '1'){
                                    return '<span class="text-green">Paid</span>';
                                }else if($row->load_billing_activities->invoiced == '1'){
                                    return '<span class="text-green">Invoiced <i class="fa fa-reply" aria-hidden="true"></i></span>';
                                }else{
                                    return '<a href="'.route('company.admin.billing.invoice.create', ['id' => base64Encode($row->id)]).'" class="btn btn-success">Invoice Now</a>';
                                }
                            }else{
                                if(count(getLoadDocuments($row)['merged_documents']) > 0){
                                    return '<a href="javascript:void(0);" class="btn btn-liteGray">Invoice Now</a>';
                                }else{
                                    return '<a href="'.route('company.admin.billing.invoice.create', ['id' => base64Encode($row->id)]).'" class="btn btn-success">Invoice Now</a>';
                                }
                            }
                        }else{
                            return '<a href="'.route('company.admin.billing.invoice.create', ['id' => base64Encode($row->id)]).'" class="btn btn-success">Invoice Now</a>';
                        }
                    })
                    ->addColumn('child_content', function($row){
                        return $this->childContentArray($row);
                    })
                    ->rawColumns(['audit', 'load_number', 'invoice_number', 'type', 'documents', 'review', 'invoice_now', 'child_content'])
                    ->make(true);
            }
        } catch( \Exception $e ) {
            \Log::emergency( "File: ".$e->getFile() );
            \Log::emergency( "Line: ".$e->getLine() );
            \Log::emergency( "Message: ".$e->getMessage() );
        }
    }

    public function getLoads(Request $request)
    {
        $getCompanyAdmin = auth()->guard('company-admin')->user();

        $getField = '';
        $isCheck = Bookmark::where('company_id', getUserCompnayId())->where('user_id', $getCompanyAdmin->id)->where('user_type', $getCompanyAdmin->type)->where('table_id', 'customer_loads')->first();
        if (!empty($isCheck->id)) {
            $getField = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_customer_loads_first'));
        } else {
            $getField = config('settings.table_customer_loads_first');
        }

        if ($request->ajax()) {
            $brokerId = base64Decode($request->customer);
            $startDate = $request->start_date;
            $endDate = $request->end_date;

            $data = LoadDetail::select('load_details.*', DB::raw('CONCAT(drivers.first_name," ",drivers.last_name) as driver'), 'drivers.cell_phone as phone_number')
                ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
                ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
                ->where('load_details.brokerage_firm', $brokerId)
                ->whereNull('load_details.deleted_at')
                ->where(function ($query) use ($endDate, $startDate) {
                    $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                        ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                        ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                        ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
                })
                ->whereHas('loads', function ($q) {
                    $q->where('loads.company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
                });

            // General filter code - start
            foreach ($getField as $key => $value) {
                $conditionReq       = 'search_string_condition_'.$key;
                $sortReq            = 'search_string_sort_'.$key;
                $conditionDateReq   = 'search_date_condition_'.$key;
                $sortDateReq        = 'search_date_sort_'.$key;
                $conditionStatusReq = 'search_status_condition_'.$key;
                $sortStatusReq      = 'search_status_sort_'.$key;

                // String filter
                if( isset( request()->$conditionReq ) ) {
                    $typeReq        = 'search_string_type_'.$key;
                    $type           = request()->$typeReq;

                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }
                }

                // Status filter
                if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                    Common::getStatusFilterResult( $data, $key );
                }

                // Date filter
                if( isset( request()->$conditionDateReq ) ) {
                    $typeReq        = 'search_date_type_'.$key;
                    $type           = request()->$typeReq;

                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                // String sort only
                if( isset( request()->$sortReq ) ) {
                    Common::sortStringFilter( $data, $key );
                }

                // Date sort only
                if( isset( request()->$sortDateReq ) ) {
                    Common::sortDateFilter( $data, $key );
                }

                // Status sort only
                if( isset( request()->$sortStatusReq ) ) {
                    Common::sortStatusFilter( $data, $key );
                }
            }

            $data->orderBy('load_id', 'DESC')->orderBy('id', 'ASC')->get();

            return Datatables::of($data)
                ->addIndexColumn()
                ->editColumn('load_number', function ($row) {
                    if ($row->is_disable == 0) {
                        if (in_array($row->load_type, ['0', '6'])) {
                            if ($row->load_type == 0) {
                                return '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link">' . $row->load_number . '</a>';
                            } else {
                                return '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link">' . getLoadTypeList()[$row->load_type] . '</a>';
                            }
                        } else {
                            return '<a href="' . route('company.admin.loads.edit', base64Encode($row->load_id)) . '" class="mr-1 load-edit-link">' . getLoadTypeList()[$row->load_type] . '</a>';
                        }
                    } else {
                        if ($row->load_type == 0) {
                            return '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link">' . $row->load_number . '</a>';
                        } else {
                            return '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link">' . getLoadTypeList()[$row->load_type] . '</a>';
                        }
                    }
                })
                ->filterColumn('driver', function ($query, $keyword) {
                    $query->whereHas('driver', function ($q) use ($keyword) {
                        $q->where(DB::raw("CONCAT(drivers.first_name, ' ', drivers.last_name)"), 'LIKE', "%" . $keyword . "%");
                    });
                })
                ->filterColumn('truck_number', function ($query, $keyword) {
                    $query->whereHas('truck', function ($q) use ($keyword) {
                        $q->where("truck_number", 'LIKE', "%" . $keyword . "%");
                    });
                })
                ->filterColumn('phone_number', function ($query, $keyword) {
                    $query->whereHas('driver', function ($q) use ($keyword) {
                        $q->where("drivers.cell_phone", 'LIKE', "%" . $keyword . "%");
                    });
                })
                ->addColumn('dispatcher', function ($row) {
                    return getLoadDispatcherName($row->booked_by);
                })
                ->filterColumn('dispatcher', function ($query, $keyword) {
                    $query->whereHas('dispatcher', function ($q) use ($keyword) {
                        $q->where(DB::raw("CONCAT(company_admins.first_name, ' ', company_admins.last_name)"), 'LIKE', "%" . $keyword . "%");
                    });
                })
                ->editColumn('shipper_location', function ($row) {
                    $html = [];
                    if ($row->shipper_address_one) {
                        $html[0] = $row->shipper_address_one;
                    }
                    if ($row->shipper_address_two) {
                        $html[1] = $row->shipper_address_two;
                    }
                    if ($row->shipper_city) {
                        $html[2] = $row->shipper_city;
                    }
                    if ($row->shipper_state) {
                        $html[3] = $row->shipper_state;
                    }
                    if ($row->shipper_zip) {
                        $html[4] = $row->shipper_zip;
                    }
                    if (count($html) == 0) {
                        return $row->shipper_location;
                    } else {
                        return implode(', ', $html);
                    }
                })
                ->editColumn('delivery_location', function ($row) {
                    $html = [];
                    if ($row->delivery_address_one) {
                        $html[0] = $row->delivery_address_one;
                    }
                    if ($row->delivery_address_two) {
                        $html[1] = $row->delivery_address_two;
                    }
                    if ($row->delivery_city) {
                        $html[2] = $row->delivery_city;
                    }
                    if ($row->delivery_state) {
                        $html[3] = $row->delivery_state;
                    }
                    if ($row->delivery_zip) {
                        $html[4] = $row->delivery_zip;
                    }
                    if (count($html) == 0) {
                        return $row->delivery_location;
                    } else {
                        return implode(', ', $html);
                    }
                })
                ->addColumn('status', function ($row) {
                    return getLoadStatus($row->load_status, $row->is_on_hold);
                })
                ->filterColumn('status', function ($query, $keyword) {
                    if (Str::contains('pending', Str::lower($keyword))) {
                        $query = $query->where('load_status', 0);
                    }
                    if (Str::contains('accepted', Str::lower($keyword))) {
                        $query = $query->where('load_status', 1);
                    }
                    if (Str::contains('preplaned', Str::lower($keyword))) {
                        $query = $query->where('load_status', 2);
                    }
                    if (Str::contains('confirmed', Str::lower($keyword))) {
                        $query = $query->where('load_status', 3);
                    }
                    if (Str::contains('in transit', Str::lower($keyword))) {
                        $query = $query->where('load_status', 4);
                    }
                    if (Str::contains('delivered', Str::lower($keyword))) {
                        $query = $query->where('load_status', 5);
                    }
                    if (Str::contains('ready to bill', Str::lower($keyword))) {
                        $query = $query->where('load_status', 6);
                    }
                    if (Str::contains('on hold', Str::lower($keyword))) {
                        $query = $query->where('load_status', 7);
                    }
                    if (Str::contains('complete', Str::lower($keyword))) {
                        $query = $query->where('load_status', 8);
                    }
                    if (Str::contains('cancelled', Str::lower($keyword))) {
                        $query = $query->where('load_status', 9);
                    }
                    return $query;
                })
                ->editColumn('total_bill', function ($row) {
                    return getPrice($row->total_bill);
                })
                ->editColumn('booked_by', function ($row) {
                    if ($row->booked_by == 0) {
                        return 'Self Dispatch';
                    } else {
                        return $row->dispatcher->full_name;
                    }
                })
                ->filterColumn('booked_by', function ($query, $keyword) {
                    $query->whereHas('dispatcher', function ($q) use ($keyword) {
                        $q->where(DB::raw("CONCAT(company_admins.first_name, ' ', company_admins.last_name)"), 'LIKE', "%" . $keyword . "%");
                    });
                })
                ->editColumn('trailer_type', function ($row) {
                    return isset(getTrailerTypeList()[$row->trailer_type]) ? getTrailerTypeList()[$row->trailer_type] : '';
                })
                ->rawColumns(['load_number'])
                ->make(true);
        }
    }

    public function calculatePercentage($preVal, $currVal)
    {
        if($preVal > 0 && $currVal > 0){
            if($currVal > $preVal){
                //$percentage = (abs($currVal - $preVal) / $preVal) * 100;
                $percentage = (abs($currVal - $preVal) / $currVal) * 100;
                return '<span class="blue"><img src="'.asset('theme/img/broker/top-arrow.svg').'" alt=""> '.round($percentage).'%</span>';
            }else if($currVal < $preVal){
                $percentage = (abs($preVal - $currVal) / $preVal) * 100;
                return '<span class="orange"><img src="'.asset('theme/img/broker/down-arrow.svg').'" alt=""> '.round($percentage).'%</span>';
            }else{
                return '<span>'.round(0).'%</span>';
            }
        }else{
            return '<span>'.round(0).'%</span>';
        }
    }

    public function loadsTabReports(Request $request)
    {
        $input = $request->all();

        $broker = Broker::findOrFail($input['broker_id']);
        $startDate = Carbon::parse($input['start_date'])->format('Y-m-d');
        $endDate = Carbon::parse($input['end_date'])->format('Y-m-d');

        $reportActiveLoad = '00'; $compareActiveLoad = ''; $reportReadyToBill = '00';  $compareReadyToBill = '';
        $reportDelivered = '00'; $compareDelivered = ''; $reportComplete = '00'; $compareComplete = '';
        $reportOnHold = '00'; $compareOnHold = ''; $reportCancelled = '00'; $compareCancelled = '';
        $reportAllLoads = '00'; $compareAllLoads = ''; $reportCargoClaims = '00'; $compareCargoClaims = '';

        $reportActiveLoad = brokerTotalActiveLoads($startDate, $endDate, $broker->id);
        $reportReadyToBill = brokerTotalReadyToBillLoads($startDate, $endDate, $broker->id);
        $reportDelivered = brokerTotalDeliveredLoads($startDate, $endDate, $broker->id);
        $reportComplete = brokerTotalCompletedLoads($startDate, $endDate, $broker->id);
        $reportOnHold = brokerTotalOnHoldLoads($startDate, $endDate, $broker->id);
        $reportCancelled = brokerTotalCancelledLoads($startDate, $endDate, $broker->id);
        $reportAllLoads = brokerTotalLoads($startDate, $endDate, $broker->id);
        $reportCargoClaims = brokerCargoClaimsLoads($startDate, $endDate, $broker->id);

        if($input['compare'] != ""){
            $compareStartDate = ""; $compareEndDate = "";

            /*
             * 1: Previous Period
             * 2: Previous Period Last Year
            */
            if($input['compare'] == '1'){
                $start = Carbon::parse($startDate);
                $end = Carbon::parse($endDate);
                $days = $start->diffInDays($end) + 1;

                $compareStartDate = Carbon::parse($input['start_date'])->subDays($days)->format('Y-m-d');
                $compareEndDate = Carbon::parse($input['end_date'])->subDays($days)->format('Y-m-d');
            }else if($input['compare'] == '2'){
                $start = Carbon::parse($startDate);
                $end = Carbon::parse($endDate);
                $days = $start->diffInDays($end) + 1;

                $compareStartDate = Carbon::parse($input['start_date'])->subYear(1)->format('Y-m-d');
                $compareEndDate = Carbon::parse($input['end_date'])->subYear(1)->format('Y-m-d');
            }

            $periodActiveLoad = brokerTotalActiveLoads($compareStartDate, $compareEndDate, $broker->id);
            $periodReadyToBill = brokerTotalReadyToBillLoads($compareStartDate, $compareEndDate, $broker->id);
            $periodDelivered = brokerTotalDeliveredLoads($compareStartDate, $compareEndDate, $broker->id);
            $periodComplete = brokerTotalCompletedLoads($compareStartDate, $compareEndDate, $broker->id);
            $periodOnHold = brokerTotalOnHoldLoads($compareStartDate, $compareEndDate, $broker->id);
            $periodCancelled = brokerTotalCancelledLoads($compareStartDate, $compareEndDate, $broker->id);
            $periodAllLoads = brokerTotalLoads($compareStartDate, $compareEndDate, $broker->id);
            $periodCargoClaims = brokerCargoClaimsLoads($compareStartDate, $compareEndDate, $broker->id);

            $compareActiveLoad = $this->calculatePercentage($periodActiveLoad, $reportActiveLoad);
            $compareReadyToBill = $this->calculatePercentage($periodReadyToBill, $reportReadyToBill);
            $compareDelivered = $this->calculatePercentage($periodDelivered, $reportDelivered);
            $compareComplete = $this->calculatePercentage($periodComplete, $reportComplete);
            $compareOnHold = $this->calculatePercentage($periodOnHold, $reportOnHold);
            $compareCancelled = $this->calculatePercentage($periodCancelled, $reportCancelled);
            $compareAllLoads = $this->calculatePercentage($periodAllLoads, $reportAllLoads);
            $compareCargoClaims = $this->calculatePercentage($periodCargoClaims, $reportCargoClaims);
        }

        return response()->json([
            'reportActiveLoad' => $reportActiveLoad, 'compareActiveLoad' => $compareActiveLoad,
            'reportReadyToBill' => $reportReadyToBill, 'compareReadyToBill' => $compareReadyToBill,
            'reportDelivered' => $reportDelivered, 'compareDelivered' => $compareDelivered,
            'reportComplete' => $reportComplete, 'compareComplete' => $compareComplete,
            'reportOnHold' => $reportOnHold, 'compareOnHold' => $compareOnHold,
            'reportCancelled' => $reportCancelled, 'compareCancelled' => $compareCancelled,
            'reportAllLoads' => $reportAllLoads, 'compareAllLoads' => $compareAllLoads,
            'reportCargoClaims' => $reportCargoClaims, 'compareCargoClaims' => $compareCargoClaims,
        ]);
    }

    public function updateRepresentativeRating(Request $request) 
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $input = $request->all();
                $input['representativeId'] = base64Decode($input['representativeId']);
                $input['rating'] = (int)$input['rating'];

                BrokerContact::where('source', 'broker')->where('id', $input['representativeId'])->where('contact_status', '1')
                ->update([
                    'rating' => $input['rating']
                ]);

                $response['success'] = true;
                $response['message'] = "Thank you for your rating.";
            }catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }
        return response()->json($response);
    }
}
