<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;

use App\Models\Carrier;
use App\Models\CarrierContact;
use App\Models\Bookmark;

use Arr;
use Validator;
use DataTables;
use Carbon\Carbon;
use App\Helpers\Common;


class CarrierController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getCompanyAdmin    = auth()->guard('company-admin')->user();
        $getField           = '';
        $isCheck            = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','carriers')->first();
        if(!empty($isCheck->id)){
            //$getField       = unserialize($isCheck->config);
            $getField       = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_carriers_first'));
        }else{
            $getField       = config('settings.table_carriers_first');
        }
        if ($request->ajax()) {
            $data  = Carrier::isCompany();

            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq = 'search_string_condition_'.$key;
                $sortReq = 'search_string_sort_'.$key;

                if( isset( request()->$conditionReq ) ) {
                    $typeReq = 'search_string_type_'.$key;
                    $type = request()->$typeReq;

                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }

                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                if( isset( request()->$sortReq ) ) {
                    $nameReq = 'search_string_name_'.$key;
                    $sortReq = 'search_string_sort_'.$key;
                    $typeReq = 'search_string_type_'.$key;
                    $tableNameReq = 'search_table_name_'.$key;
                    $colAliasReq = 'search_string_col_alias_'.$key;

                    $origData = request()->$nameReq;
                    $condition_sort = request()->$sortReq;
                    $type = request()->$typeReq;
                    $tableName = isset( request()->$tableNameReq ) ? request()->$tableNameReq : '';
                    $colAlias = isset( request()->$colAliasReq ) ? request()->$colAliasReq : '';

                    if( $type == 'string' ) {
                        if( isset( $condition_sort ) ) {
                            if( $colAlias != '' ) {
                                $data->orderBy( $tableName.'.'.$colAlias, $condition_sort );
                            } else {
                                if( $tableName != '' ) {
                                    $data->orderBy( $tableName.'.'.$origData, $condition_sort );
                                } else {
                                    $data->orderBy( $origData, $condition_sort );
                                }
                            }
                        }
                    }
                }
            }
            $data->select('*');
            /// end

            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $action = '<a href="'.route('company.admin.carriers.show', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                    $action .= '<a href="'.route('company.admin.carriers.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                    $action .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash destroy" data-action="'.route('company.admin.carriers.destroy',$row->id).'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';

                    return $action;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        $tableName = 'settings.table_carriers';
        return view('companyAdmin.carrier.index', compact('getField','tableName'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return view('companyAdmin.carrier.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request){
        $input                  = $request->all();

        $input['desired_load_number'] = ($request->desired_option == "0") ? $request->text_desired_load_number : null;

        $input['contacts']      = Arr::except($input['contacts'],['replace_key']);
        $validator              = Validator::make($input,[
            'name'                              => 'required',
            'mc_number'                         => 'required|unique:carriers,mc_number,NULL,id,company_id,'.auth()->guard('company-admin')->user()->company_id.',deleted_at,NULL',
            'contacts.*.name'                   => 'required',
            //'contacts.*.phone_number'         => 'required|size:10',
            'contacts.*.email'                  => 'nullable|email',
            'identifier'                        => 'required|min:3|unique:carriers,identifier,NULL,id,company_id,'.auth()->guard('company-admin')->user()->company_id.',deleted_at,NULL',
        ],[
            'contacts.*.name.required'          => 'The name field is required.',
            'contacts.*.phone_number.required'  => 'The phone number field is required.',
            //'contacts.*.phone_number.size'    => 'The phone number must be 10 digits.',
            'contacts.*.email.email'            => 'The email must be a valid email address.',
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $input['company_id']            = auth()->guard('company-admin')->user()->company_id;
        if($request->address_verified==1){
            $timezone                   = null;
            $timezone_offset            = null;
            if(isset($input['timezone_abbr'])){
                $timezone               = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset        = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']          = $timezone;
            $input['timezone_offset']   = $timezone_offset;
        }

        $carrier                        = Carrier::create($input);

        if (!empty($input['contacts'])) {
            foreach ($input['contacts'] as $key => $contact) {
                $data                   = [];
                $data['carrier_id']     = $carrier->id;
                $data['department']     = $contact['department'];
                $data['name']           = $contact['name'];
                $data['email']          = $contact['email'];
                $data['phone_number']   = $contact['phone_number'];
                $data['ext']            = $contact['ext'];
                CarrierContact::create($data);
            }
        }

        notification('success', $this->crudMessage('create', 'Carrier'));

        return response()->json(['success' => "Carrier created successfully"]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id){
        $id = base64Decode($id);
        $carrier = Carrier::findOrFail($id);
        return view('companyAdmin.carrier.edit', compact('carrier'));
    }

    /**
     * Display the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id){
        $id = base64Decode($id);
        $carrier = Carrier::findOrFail($id);
        return view('companyAdmin.carrier.show', compact('carrier'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Carrier $carrier){
        $input                          = $request->all();

        if(isset($request->desired_option)){
            $input['desired_load_number'] = ($request->desired_option == "0") ? $request->text_desired_load_number : null;
        }

        $input['contacts']              = Arr::except($input['contacts'],['replace_key']);
        $validator                      = Validator::make($input,[
            'name'                              => 'required',
            'mc_number'                         => 'nullable|unique:carriers,mc_number,'.$carrier->id.',id,company_id,'.auth()->guard('company-admin')->user()->company_id.',deleted_at,NULL',
            'contacts.*.name'                   => 'required',
            //'contacts.*.phone_number'         => 'required|size:10',
            'contacts.*.email'                  => 'nullable|email',
            'identifier'                        => 'required|min:3|unique:carriers,identifier,'.$carrier->id.',id,company_id,'.auth()->guard('company-admin')->user()->company_id.',deleted_at,NULL',
        ],[
            'contacts.*.name.required'          => 'The name field is required.',
            'contacts.*.phone_number.required'  => 'The phone number field is required.',
            //'contacts.*.phone_number.size'    => 'The phone number must be 10 digits.',
            'contacts.*.email.email'            => 'The email must be a valid email address.',
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }
        if($request->address_verified==1){
            $timezone                   = null;
            $timezone_offset            = null;
            if(isset($input['timezone_abbr'])){
                $timezone               = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset        = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']          = $timezone;
            $input['timezone_offset']   = $timezone_offset;
        }else{
            $input['latitude']          = null;
            $input['longitude']         = null;
            $input['timezone']          = null;
            $input['timezone_offset']   = null;
            $input['timezone_abbr']     = null;
        }
        $carrier->update($input);

        if (!empty($input['contacts'])) {
            $ids            = [];
            foreach ($input['contacts'] as $contact) {
                if (isset($contact['id'])) {
                    $ids[]  = $contact['id'];
                }
            }

            CarrierContact::where('carrier_id', $carrier->id)->whereNotIn('id',$ids)->delete();

            foreach ($input['contacts'] as $contact) {
                $data                   = [];
                $data['carrier_id']     = $carrier->id;
                $data['department']     = $contact['department'];
                $data['name']           = $contact['name'];
                $data['email']          = $contact['email'];
                $data['phone_number']   = $contact['phone_number'];
                $data['ext']            = $contact['ext'];

                if (isset($contact['id'])) {
                    $carrierContact     = CarrierContact::find($contact['id']);
                    $carrierContact->update($data);
                } else {
                    CarrierContact::create($data);
                }
            }
        }

        notification('success', $this->crudMessage('update', 'Carrier'));

        return response()->json(['success' => "Carrier updated successfully"]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Carrier $carrier){
        $carrier->delete();

        notification('success', $this->crudMessage('delete', 'Carrier'));

        return redirect()->route('company.admin.carriers.index');
    }
}
