<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Helpers\Common;
use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use App\Models\Bookmark;
use App\Models\ChartOfAccount;
use App\Models\CompanyChartOfAccount;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Shuchkin\SimpleXLSX;
use Yajra\DataTables\Facades\DataTables;

class ChartOfAccountController extends CompanyAdminController
{
    private $parents = array(); // This variabel use for findParents() function

    public function __construct()
    {
        parent::__construct();
    }

    function findParents($input, $id) {
        if(is_array($input)) {
            foreach($input as $k => $val) {
                if($val['id'] == $id  && $val['parent_account'] != null) {
                    array_push($this->parents, $val['parent_account']);
                    $this->findParents($input, $val['parent_account']);
                }
            }
        }
    }

    public function findChildrens($elements, $parentId, $parentFound = false, $cats = array())
    {
        foreach($elements as $element) {
            if(!is_null($element['parent_account'])){
                if((!$parentFound && $element['id'] == $parentId) || $element['parent_account'] == $parentId) {
                    $rowData = array();
                    foreach($element as $k => $v) {
                        $rowData[$k] = $v;
                    }
                    $cats[] = $rowData;
                    if($element['parent_account'] == $parentId) {
                        $cats = array_merge($cats, $this->findChildrens($elements, $element['id'], true));
                    }
                }
            }
        }
        return $cats;
    }

    public function index(Request $request)
    {
        $getField = '';
        $getCompanyAdmin = auth()->guard('company-admin')->user();
        $isCheck = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id', 'chart_of_accounts')->first();

        if(!empty($isCheck->id)){
            //$getField = unserialize($isCheck->config);
            $getField = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_chart_of_accounts_first'));
        }else{
            $getField = config('settings.table_chart_of_accounts_first');
        }

        if ($request->ajax()) {
            $displayInactiveRows = request()->display_inactive_rows;

            $data = CompanyChartOfAccount::IsCompany()->where(function ($query) use ($displayInactiveRows){
                if($displayInactiveRows == '0'){
                    $query->where('active_status', '0');
                }
            });
            $accountNameArr = $data->pluck('account_name', 'id')->toArray();

            // General filter code - start
            $isOrder = false; $isOrderAccountNumber = false;
            foreach( $getField as $key => $value ) {
                $conditionReq = 'search_string_condition_'.$key;
                $sortReq = 'search_string_sort_'.$key;

                if( isset( request()->$conditionReq ) ) {
                    $typeReq = 'search_string_type_'.$key;
                    $type = request()->$typeReq;

                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }

                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }

                    $isOrder = true;
                }

                if( isset( request()->$sortReq ) ) {
                    $nameReq = 'search_string_name_'.$key;
                    $sortReq = 'search_string_sort_'.$key;
                    $typeReq = 'search_string_type_'.$key;
                    $tableNameReq = 'search_table_name_'.$key;
                    $colAliasReq = 'search_string_col_alias_'.$key;

                    $origData = request()->$nameReq;
                    $condition_sort = request()->$sortReq;
                    $type = request()->$typeReq;
                    $tableName = isset( request()->$tableNameReq ) ? request()->$tableNameReq : '';
                    $colAlias = isset( request()->$colAliasReq ) ? request()->$colAliasReq : '';

                    if( $type == 'string' ) {
                        if( isset( $condition_sort ) ) {
                            if( $colAlias != '' ) {
                                $data->orderBy( $tableName.'.'.$colAlias, $condition_sort );
                            } else {
                                if( $tableName != '' ) {
                                    $data->orderBy( $tableName.'.'.$origData, $condition_sort );
                                } else {
                                    $data->orderBy( $origData, $condition_sort );
                                }
                            }
                        }
                    }

                    $isOrder = true;
                }

                if(request()->search_string_sort_account_number != "" || request()->search_string_condition_account_number != "" || request()->search_date_condition_account_number != ""){
                    $isOrderAccountNumber = true;
                }
            }
            // General filter code - end

            if($isOrder){
                $data = $data->orderBy('account_type', 'ASC')->get()->toArray();
            }else{
                /* Default sort by Name with Parent alphabetically and child sort ny account number - START */
                $dataUpdated = array();

                $allRecords = $data->orderBy('account_type', 'ASC')->get()->toArray();

                $allParents = $data->whereNull('parent_account')->orderBy('account_type', 'ASC')->orderBy('account_name', 'ASC')->get()->toArray();
                //array_multisort(array_column($allParents, 'account_name'), SORT_ASC, $allParents); // Parents sorting by account_name

                foreach ($allParents as $parent){
                    $dataUpdated[] = $parent;
                    $children = $this->findChildrens($allRecords, $parent['id']);
                    if(count($children) > 0){
                        array_multisort(array_column($children, 'account_number'), SORT_ASC, $children); // Children sorting by account_number
                        foreach ($children as $child){
                            $dataUpdated[] = $child;
                        }
                    }
                }
                $data = $dataUpdated;
                /* Default sort by Name with Parent alphabetically and child sort ny account number - END */
            }

            foreach ($data as $key => $value){
                $this->parents = [];
                $this->findParents($data, $value['id']);
                $data[$key]['index'] = count($this->parents);

                if($isOrderAccountNumber){
                    $breadcrumbs = array();
                    foreach ($this->parents as $parent){
                        $breadcrumbs[] = $accountNameArr[$parent];
                    }
                    $data[$key]['breadcrumbs'] = (count($breadcrumbs) > 0) ? 'Sub-account of: '.implode(' : ', array_reverse($breadcrumbs)) : "";
                }
            }

            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $btn = '<a href="#" class="edit-btn mr-4" data-id="'.$row['id'].'" data-toggle="modal" data-target="#editModal" style="color: black; font-weight: 600;">Edit</a>';

                    if($row['active_status'] == '1'){
                        $btn = $btn.' <a href="javascript:void(0);" class="updateStatus" data-action="'.route('company.admin.chart-of-accounts.updateAccountStatus').'" data-id="'.$row['id'].'" style="color: #d50a0a; font-weight: 600;">Make Active</a>';
                    }else{
                        $btn = $btn.' <a href="javascript:void(0);" class="updateStatus" data-action="'.route('company.admin.chart-of-accounts.updateAccountStatus').'" data-id="'.$row['id'].'" style="color: black; font-weight: 600;">Make Inactive</a>';
                    }

                    return $btn;
                })
                ->editColumn('account_name', function ($row) use ($isOrderAccountNumber){
                    if($isOrderAccountNumber){
                        return $row['account_name'].'<br/>'.$row['breadcrumbs'];
                    }else{
                        return str_repeat("&nbsp;", ($row['index'] * 6)).$row['account_name'];
                    }
                })
                ->editColumn('account_type', function ($row){
                    return getChartOfAccountsTypes($row['account_type']);
                })
                ->rawColumns(['account_type', 'account_name', 'action'])
                ->make(true);
        }
        $tableName = 'settings.table_chart_of_accounts';

        $isEmptyTable = CompanyChartOfAccount::IsCompany()->count();

        return view('companyAdmin.chartOfAccounts.index', compact('getField','tableName', 'isEmptyTable'));
    }

    public function makeParentChildTree($arr, $parent = null)
    {
        $pages = Array();
        foreach($arr as $page) {
            if($page['parent_account'] == $parent) {
                $page['sub'] = isset($page['sub']) ? $page['sub'] : $this->makeParentChildTree($arr, $page['id']);
                $pages[] = $page;
            }
        }
        return $pages;
    }

    public function makePreviewTreeHtml($parentChildData)
    {
        $html = '';
        $html .= '<ul>';
        foreach($parentChildData as $page) {
            $html .= '<li id="parent_'.$page['id'].'">'.$page['account_number'].' - '.$page['account_name'].'</li>';

            if(count($page['sub']) > 0){
                $html .= $this->makePreviewTreeHtml($page['sub']);
            }
        }
        $html .= '</ul>';
        return $html;
    }

    public function parentList(Request $request)
    {
        $displayInactiveParent = $request->displayInactiveParent;
        $editId = $request->editId;

        $dropdownHtml = "<option value=''>Select Parent</option>";
        $data = CompanyChartOfAccount::IsCompany()->where(function ($query) use ($displayInactiveParent){
                    if($displayInactiveParent == '0'){
                        $query->where('active_status', '0');
                    }
                })->where('account_type', $request->accountType);

        $dataUpdated = array();
        $allRecords = $data->orderBy('account_type', 'ASC')->get()->toArray();
        $allParents = $data->whereNull('parent_account')->orderBy('account_type', 'ASC')->orderBy('account_name', 'ASC')->get()->toArray();
        foreach ($allParents as $parent){
            $dataUpdated[] = $parent;
            $children = $this->findChildrens($allRecords, $parent['id']);
            if(count($children) > 0){
                array_multisort(array_column($children, 'account_number'), SORT_ASC, $children); // Children sorting by account_number
                foreach ($children as $child){
                    $dataUpdated[] = $child;
                }
            }
        }
        $parentsChildArr = $dataUpdated;

        $parentChildTree = $this->makePreviewTreeHtml($this->makeParentChildTree($parentsChildArr));

        if($editId != ""){
            foreach ($parentsChildArr as $key => $value){
                if($editId != $value['id']){
                    $dropdownHtml .= "<option value='".$value['id']."'>".$value['account_number']." - ".$value['account_name']."</option>";
                }
            }
        }else{
            foreach ($parentsChildArr as $key => $value){
                $dropdownHtml .= "<option value='".$value['id']."'>".$value['account_number']." - ".$value['account_name']."</option>";
            }
        }

        return response()->json(['dropDownHtml' => $dropdownHtml, 'parentChildTree' => $parentChildTree]);
    }

    public function store(Request $request)
    {
        $input = $request->all();
        $validator  = Validator::make($input, [
            "account_type"  => "required",
            "account_name"  => "required",
            "account_number"  => "required|numeric|unique:company_chart_of_accounts,account_number,NULL,id,company_id,".auth()->guard('company-admin')->user()->company_id,
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $input['company_id'] = auth()->guard('company-admin')->user()->company_id;

        CompanyChartOfAccount::create($input);

        notification('success', $this->crudMessage('create', 'Account Information'));

        return response()->json(['success' => "Account Information created successfully"]);
    }

    public function editFormView(Request $request)
    {
        $displayInactiveParent = $request->displayInactiveParent;
        $companyChartOfAccount = CompanyChartOfAccount::IsCompany()->find($request->id);

        $parents = array();
        if($companyChartOfAccount){
            $data = CompanyChartOfAccount::IsCompany()->where(function ($query) use ($displayInactiveParent){
                if($displayInactiveParent == '0'){
                    $query->where('active_status', '0');
                }
            })->where('account_type', $companyChartOfAccount->account_type);

            $dataUpdated = array();
            $allRecords = $data->orderBy('account_type', 'ASC')->get()->toArray();
            $allParents = $data->whereNull('parent_account')->orderBy('account_type', 'ASC')->orderBy('account_name', 'ASC')->get()->toArray();

            foreach ($allParents as $parent){
                $dataUpdated[] = $parent;
                $children = $this->findChildrens($allRecords, $parent['id']);
                if(count($children) > 0){
                    array_multisort(array_column($children, 'account_number'), SORT_ASC, $children); // Children sorting by account_number
                    foreach ($children as $child){
                        $dataUpdated[] = $child;
                    }
                }
            }
            $parentsChildArr = $dataUpdated;
            $parentChildTree = $this->makePreviewTreeHtml($this->makeParentChildTree($parentsChildArr));

            foreach ($parentsChildArr as $parent){
                if($request->id != $parent['id']){
                    $parents[$parent['id']] = $parent['account_number'].' - '.$parent['account_name'];
                }
            }


            $headerPreview = getChartOfAccountsTypes($companyChartOfAccount->account_type);
        }

        $view = view('companyAdmin.chartOfAccounts.editFormView', compact(['companyChartOfAccount', 'parents']))->render();

        return response()->json(['view' => $view, 'data' => $companyChartOfAccount, 'parentChildTree' => $parentChildTree, 'headerPreview' => $headerPreview]);
    }

    public function update(Request $request, CompanyChartOfAccount $chartOfAccount)
    {
        $input = $request->all();

        $validator  = Validator::make($input,[
            "account_type"  => "required",
            "account_name"  => "required",
            "account_number"  => "required|numeric|unique:company_chart_of_accounts,account_number,".$chartOfAccount->id.",id,company_id,".auth()->guard('company-admin')->user()->company_id,
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $input['company_id'] = auth()->guard('company-admin')->user()->company_id;

        if($input['parent_account'] != $chartOfAccount->parent_account){
            CompanyChartOfAccount::where('parent_account', $chartOfAccount->id)->update(['parent_account' => null]);
        }

        $chartOfAccount->update($input);

        notification('success', $this->crudMessage('update', 'Account Information'));

        return response()->json(['success' => "Account Information updated successfully"]);
    }

    public function updateAccountStatus(Request $request)
    {
        $companyChartOfAccount = CompanyChartOfAccount::IsCompany()->find($request->id);

        if($companyChartOfAccount){
            if($companyChartOfAccount->active_status == "0"){
                $companyChartOfAccount->active_status = "1";
                $companyChartOfAccount->save();
            }elsE{
                $companyChartOfAccount->active_status = "0";
                $companyChartOfAccount->save();
            }

            return redirect()->route('company.admin.chart-of-accounts.index');
        }else{
            return redirect()->route('company.admin.chart-of-accounts.index');
        }
    }

    public function insertIfChildExist($lastParentInsertId, $parentId)
    {
        $children =  ChartOfAccount::where('parent_account', $parentId)->get()->toArray();

        if(count($children) > 0){
            foreach ($children as $child){
                $insertCompanyChartOfAccountChild = new CompanyChartOfAccount();
                $insertCompanyChartOfAccountChild->company_id = auth()->guard('company-admin')->user()->company_id;
                $insertCompanyChartOfAccountChild->account_type = $child['account_type'];
                $insertCompanyChartOfAccountChild->parent_account = $lastParentInsertId; // Inserted Parent id in company table
                $insertCompanyChartOfAccountChild->account_name = $child['account_name'];
                $insertCompanyChartOfAccountChild->account_number = $child['account_number'];
                $insertCompanyChartOfAccountChild->save();

                $moreChildren = ChartOfAccount::where('parent_account', $child['id'])->count();
                if($moreChildren > 0){
                    $this->insertIfChildExist($insertCompanyChartOfAccountChild->id, $child['id']);
                }
            }
        }else{
            return 1;
        }
    }

    public function importDefault()
    {
        $chartOfAccountsParents = ChartOfAccount::whereNull('parent_account')->orderBy('id')->get()->toArray();

        foreach ($chartOfAccountsParents as $chartOfAccountsParent){
            $insertCompanyChartOfAccountParent = new CompanyChartOfAccount();
            $insertCompanyChartOfAccountParent->company_id = auth()->guard('company-admin')->user()->company_id;
            $insertCompanyChartOfAccountParent->account_type = $chartOfAccountsParent['account_type'];
            $insertCompanyChartOfAccountParent->parent_account = $chartOfAccountsParent['parent_account'];
            $insertCompanyChartOfAccountParent->account_name = $chartOfAccountsParent['account_name'];
            $insertCompanyChartOfAccountParent->account_number = $chartOfAccountsParent['account_number'];
            $insertCompanyChartOfAccountParent->save();

            if($insertCompanyChartOfAccountParent){
                $children = ChartOfAccount::where('parent_account', $chartOfAccountsParent['id'])->count();
                if($children > 0){
                    $this->insertIfChildExist($insertCompanyChartOfAccountParent->id, $chartOfAccountsParent['id']);
                }
            }
        }
    }

    public function exportList(Request $request)
    {
        $fileName = 'chart-of-accounts-list.csv';
        $data = CompanyChartOfAccount::IsCompany()->where('active_status', '0');

        $dataUpdated = array();

        $allRecords = $data->orderBy('account_type', 'ASC')->get()->toArray();

        $allParents = $data->whereNull('parent_account')->orderBy('account_type', 'ASC')->orderBy('account_name', 'ASC')->get()->toArray();

        if(count($allParents) > 0){
            foreach ($allParents as $parent){
                $dataUpdated[] = $parent;
                $children = $this->findChildrens($allRecords, $parent['id']);
                if(count($children) > 0){
                    array_multisort(array_column($children, 'account_number'), SORT_ASC, $children); // Children sorting by account_number
                    foreach ($children as $child){
                        $dataUpdated[] = $child;
                    }
                }
            }

            $data = $dataUpdated;

            foreach ($data as $key => $value){
                $this->parents = [];
                $this->findParents($data, $value['id']);
                $data[$key]['index'] = count($this->parents);
            }

            $printArr = array();
            foreach ($data as $key => $values){
                $printArr[$key] = array(
                    'account_number' => $values['account_number'],
                    'account_name' => str_repeat(" ", ($values['index'] * 6)).$values['account_name'],
                    'account_type' => getChartOfAccountsTypes($values['account_type']),
                );
            }

            $headers = array(
                "Content-type"        => "text/csv",
                "Content-Disposition" => "attachment; filename=$fileName",
                "Pragma"              => "no-cache",
                "Cache-Control"       => "must-revalidate, post-check=0, pre-check=0",
                "Expires"             => "0"
            );

            $columns = array(
                'Account Number', 'Account Name', 'Account Type'
            );

            $callback = function () use ($printArr, $columns) {
                $file = fopen('php://output', 'w');
                fputcsv($file, $columns);

                foreach ($printArr as $val) {
                    $row['Account Number']  = $val['account_number'];
                    $row['Account Name']  = $val['account_name'];
                    $row['Account Type']  = $val['account_type'];

                    fputcsv($file, array(
                        $row['Account Number'],
                        $row['Account Name'],
                        $row['Account Type']
                    ));
                }
                fclose($file);
            };
            return response()->stream($callback, 200, $headers);
        }else{
            return redirect()->route('company.admin.chart-of-accounts.index');
        }
    }

    public function importAccounts()
    {
        return view('companyAdmin.chartOfAccounts.import');
    }

    public function importAccountsStepOne(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'import_file_name' => 'required',
            'import_file' => 'required'
        ], [
            'import_file_name.required' => 'Please upload CSV or Excel file to continue',
            'import_file.required' => 'Please upload CSV or Excel file to continue',
        ]);

        if($validate->fails()) {
            $response['status'] = "error";
            $response['swal_icon'] = "warning";
            $response['msg'] = $validate->errors()->first();
            $response['data'] = array();
            $response['errors'] = $validate->errors()->first();
            return response()->json($response);
        }

        $fileExt = $request->file('import_file')->getClientOriginalExtension();
        $path = $request->file('import_file')->getRealPath();

        $data = array();
        if($fileExt == "csv"){
            $data = csvToArray($path);
            $fileType = "csv";
        }else{
            $data = xlsxToArray($path);
            $fileType = "excel";
        }

        if(count($data) == 0) {
            $response['status'] = "error";
            $response['swal_icon'] = "error";
            $response['msg'] = 'You are importing file is empty!';
            $response['data'] =  [];
            return response()->json($response);
        }else if(count($data) <= 1) {
            $response['status'] = "error";
            $response['swal_icon'] = "error";
            $response['msg'] = 'Please add at least one record for importing!';
            $response['data'] =  [];
            return response()->json($response);
        }else{
            $headers = array_keys($data[0]);

            $table_fields = array(
                0 => array(
                    'key' => 'account_number',
                    'value' => 'Account Number'
                ),
                1 => array(
                    'key' => 'account_name',
                    'value' => 'Account Name'
                ),
                2 => array(
                    'key' => 'account_type',
                    'value' => 'Type'
                )
            );

            $file = $request->import_file;
            $uniqueId = '';

            if( $file != 'undefined' ) {
                $uniqueId = "chart_of_accounts_".$fileType."_" . uniqId() . ".".$fileExt;
                $destinationPath = public_path('storage/companyAdmin/chart-of-account-files');
                $file->move($destinationPath,$uniqueId);
            }

            $tableHtml = "";
            foreach ($table_fields as $key => $val){
                $tableHtml .= '<tr>';
                $tableHtml .= '<td class="font-16">'.$val['value'].'</td>';
                $tableHtml .= '<td>';
                $tableHtml .= '<select class="form-control select2-select imAccount-select" name="field_value['.$val['key'].']" id="field_id_'.$key.'" required>';
                $tableHtml .= '<option value="">Select Your Field</option>';
                foreach($headers as $ke => $vl) {
                    $tableHtml .= '<option value="'.$vl.'" '.(($val['value'] == $vl) ? 'selected' : '').'>'.$vl.'</option>';
                }
                $tableHtml .= '</select>';
                $tableHtml .= '</td>';
                $tableHtml .= '<td><span class="text-success acN0 '.((!in_array($val['value'], $headers)) ? 'd-none' : '').'" id="check_id_'.$key.'"><i class="fa fa-check" aria-hidden="true"></i></span></td>';
                $tableHtml .= ' </tr>';
            }

            $response['status'] = "success";
            $response['swal_icon'] = "success";
            $response['file_path'] = $uniqueId;
            $response['tableHtml'] = $tableHtml;

            return response()->json($response);
        }
    }

    public function importAccountsStepTwo(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'file' => 'required',
            'field_value' => 'required'
        ], [
            'file.required' => 'Uploaded CSV or Excel file to not found',
            'field_value.required' => 'Uploaded CSV or Excel columns not found',
        ]);

        if($validate->fails()) {
            $response['status'] = "error";
            $response['swal_icon'] = "error";
            $response['msg'] = $validate->errors()->first();
            $response['data'] = array();
            $response['errors'] = $validate->errors()->first();
            return response()->json($response);
        }

        $assignedFields = $request->field_value;
        $fileName = $request->file;
        $filePath = public_path('storage/companyAdmin/chart-of-account-files/'.$fileName);

        if(file_exists($filePath)){
            $fileExt = pathinfo($fileName, PATHINFO_EXTENSION);

            $data = array();
            if($fileExt == "csv"){
                $data = csvToArray($filePath);
            }else{
                $data = xlsxToArray($filePath);
            }

            if(count($data) == 0) {
                $response['status'] = "error";
                $response['swal_icon'] = "error";
                $response['msg'] = 'Uploaded file is empty!';
                $response['data'] =  [];
                return response()->json($response);
            }else if(count($data) <= 1) {
                $response['status'] = "error";
                $response['swal_icon'] = "error";
                $response['msg'] = 'Please add at least one record for importing!';
                $response['data'] =  [];
                return response()->json($response);
            }else{
                $existingAccountNumberArr = CompanyChartOfAccount::IsCompany()->pluck('account_number')->toArray();

                $tableHtml = '<tr>';
                $tableHtml .= '<th>';
                $tableHtml .= '<div class="custom-control custom-checkbox">';
                $tableHtml .= '<input type="checkbox" class="custom-control-input" id="selectAll" checked>';
                $tableHtml .= '<label class="custom-control-label" for="selectAll"></label>';
                $tableHtml .= '</div>';
                $tableHtml .= '</th>';
                foreach ($assignedFields as $field){
                    $tableHtml .= '<th>'.$field.'</th>';
                }
                $tableHtml .= '</tr>';

                foreach ($data as $key => $value){
                    $tableHtml .= '<tr>';

                    $tableHtml .= '<td>';
                    $tableHtml .= '<div class="custom-control custom-checkbox">';
                    $tableHtml .= '<input type="checkbox" class="custom-control-input" id="import_'.$key.'" name="import_chk[]" value="'.$key.'" checked>';
                    $tableHtml .= '<label class="custom-control-label" for="import_'.$key.'"></label>';
                    $tableHtml .= '</div>';
                    $tableHtml .= '</td>';

                    $tableHtml .= '<td>';
                    $tableHtml .= '<input type="number" class="from-control number" name="account_number['.$key.']" value="'.trim($value[$assignedFields['account_number']]).'">';
                    $tableHtml .= '</td>';

                    $tableHtml .= '<td>';
                    $tableHtml .= '<input type="text" class="from-control" name="account_name['.$key.']" value="'.trim($value[$assignedFields['account_name']]).'">';
                    $tableHtml .= '</td>';

                    $tableHtml .= '<td>';
                    $tableHtml .= '<select class="form-control select2-select imAccount-select" name="account_type['.$key.']">';
                    $tableHtml .= '<option value="">Select Type</option>';
                    foreach (getChartOfAccountsTypes('', 'listing') as $k => $v){
                        $tableHtml .= '<option value="'.$k.'" '.(($value[$assignedFields['account_type']] == $v) ? "selected" : "").'>'.$v.'</option>';
                    }
                    $tableHtml .= '</select>';
                    $tableHtml .= '</td>';

                    $tableHtml .= '</tr>';
                }

                $response['status'] = "success";
                $response['swal_icon'] = "success";
                $response['tableHtml'] = $tableHtml;
                $response['fileName'] = $fileName;
                $response['thirdStepHeader'] = count($data)." accounts are ready to be import";

                return response()->json($response);
            }
        }else{
            $response['status'] = "error";
            $response['swal_icon'] = "error";
            $response['msg'] = "Uploaded file not found!";
            $response['data'] = [];
            $response['errors'] = "Uploaded file not found!";

            return response()->json($response);
        }
    }

    public function importAccountsStepDone(Request $request)
    {
        $importChkKeyArr = $request->import_chk;
        $accountNumberArr = $request->account_number;
        $accountNameArr = $request->account_name;
        $accountTypeArr = $request->account_type;
        $fileName = $request->fileName;

        $importData =  array();
        $accountNumberIsEmptyArr = array();
        $accountNameIsEmptyArr = array();
        $accountTypeIsEmptyArr = array();
        $duplicateAccountNumberArr = array();

        $rowNumber = 1;
        $existingAccountNumberArr = CompanyChartOfAccount::IsCompany()->pluck('account_number')->toArray();
        foreach ($importChkKeyArr as $key){
            if($accountNumberArr[$key] != "" && $accountNameArr[$key] != "" && $accountTypeArr[$key] != "" ){
                if(in_array($accountNumberArr[$key], $existingAccountNumberArr)){
                    $duplicateAccountNumberArr[] = $rowNumber;
                }else{
                    $importData[] = array(
                        'company_id' => auth()->guard('company-admin')->user()->company_id,
                        'account_type' => $accountTypeArr[$key],
                        'account_name' => $accountNameArr[$key],
                        'account_number' => $accountNumberArr[$key],
                        'created_at' => now(),
                        'updated_at' => now(),
                    );
                }
            }else{
                if($accountNumberArr[$key] == ""){
                    $accountNumberIsEmptyArr[] = $rowNumber;
                }
                if($accountNameArr[$key] == ""){
                    $accountNameIsEmptyArr[] = $rowNumber;
                }
                if($accountTypeArr[$key] == ""){
                    $accountTypeIsEmptyArr[] = $rowNumber;
                }
            }
            $rowNumber++;
        }

        if(count($accountNumberIsEmptyArr) > 0){
            $response['status'] = "error";
            $response['swal_icon'] = "error";
            $response['msg'] = "The <b>Account Number</b> cannot be empty.<br>Please add data for following :<br/><b>Rows # ". implode(', ', $accountNumberIsEmptyArr)."</b>";

            return response()->json($response);
        }else if(count($accountNameIsEmptyArr) > 0){
            $response['status'] = "error";
            $response['swal_icon'] = "error";
            $response['msg'] = "The <b>Account Name</b> cannot be empty.<br>Please add data for following :<br/><b>Rows # ". implode(', ', $accountNameIsEmptyArr)."</b>";

            return response()->json($response);
        }else if(count($accountTypeIsEmptyArr) > 0){
            $response['status'] = "error";
            $response['swal_icon'] = "error";
            $response['msg'] = "The <b>Account Type</b> cannot be empty.<br>Please select data for following :<br/><b>Rows # ". implode(', ', $accountTypeIsEmptyArr)."</b>";

            return response()->json($response);
        }else if(count($duplicateAccountNumberArr) > 0){
            $response['status'] = "error";
            $response['swal_icon'] = "error";
            $response['msg'] = "The <b>Account Number</b> has already been taken.<br>Please check data for following :<br/><b>Rows # ". implode(', ', $duplicateAccountNumberArr)."</b><br><br><p style='font-size: 13px;font-weight: bold;'>Note: If you do not import following rows please uncheck checkbox.</p>";

            return response()->json($response);
        }else{
            CompanyChartOfAccount::insert($importData); // Import data

            unlink(public_path('storage/companyAdmin/chart-of-account-files/'.$fileName)); //Remove file when done import functionality

            $response['status'] = "success";

            return response()->json($response);
        }
    }
}
