<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

use App\Models\LoadFile;
use App\Models\LoadStop;
use App\Models\LoadDetail;

use App\Models\Gallery;
use App\Models\IntegrationSetting;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ServerException;

use Carbon\Carbon;

class ChatController extends CompanyAdminController
{
    public $timeZone;
    public $timeZoneId;
    public $offset;

    public function __construct(){
        parent::__construct();

        $this->middleware(function ($request, $next) {
            $this->timeZone     = getTimeZone();
            $this->timeZoneId   = getTimeZoneId();
            $this->offset       = getTimeZoneOffset();

            return $next($request);
        });
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(){
        return view('companyAdmin.chat.index');
    }

    public function loadDetail(Request $request){
        $response['success']                        = false;
        $response['message']                        = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                if($request->has('load_number')){
                    $item                           = LoadDetail::with(['driver','broker'])
                        ->where('load_number',$request->load_number)->first();
                    if(!is_null($item)){
                        $mapOption                  = [
                            'tollRoadsOption'       => $item->map_toll_roads ?? null,
                            'elevLimit'             => $item->map_max_elevation ?? null,
                            'avoidCountry'          => $item->map_close_borders ?? null
                        ];
                        $routeLngLat                = [];
                        if($item->shipper_address_verified==1){
                            $routeLngLat['pickup']  = array(
                                'longitude'         => $item->shipper_longitude,
                                'latitude'          => $item->shipper_latitude

                            );
                        }
                        if(isset($item->loads)){
                            if($item->loads->load_stops->isNotEmpty()){
                                foreach ($item->loads->load_stops as $key => $stop) {
                                    if($stop->address_verified==1){
                                        $routeLngLat['stop_'.++$key] = array(
                                            'longitude'         => $stop->longitude,
                                            'latitude'          => $stop->latitude

                                        );
                                    }
                                }
                            }
                        }
                        if($item->delivery_address_verified==1){
                            $routeLngLat['delivery']= array(
                                'longitude'         => $item->delivery_longitude,
                                'latitude'          => $item->delivery_latitude

                            );
                        }
                        $item->load_status          = getLoadStatus($item->load_status,$item->is_on_hold);
                        $response['item']           = $item;
                        if(isset($item->broker->id)){
                            $response['brokerLink'] = route('company.admin.brokers.edit', base64Encode($item->broker->id));
                        }
                        if(isset($item->truck->id)){
                            $response['truckLink']  = route('company.admin.trucks.show', base64Encode($item->truck->id));
                        }
                        if(isset($item->trailer->id)){
                            $response['trailerLink']= route('company.admin.trailers.show', base64Encode($item->trailer->id));
                        }
                        $response['gtypeLabel']     = 'Not specified';
                        if (isset($item->trailer_type) && $item->trailer_type!='' && isset(getTrailerTypeList()[$item->trailer_type])) {
                            $response['gtypeLabel'] = getTrailerTypeList()[$item->trailer_type];
                        }
                        $response['mapOption']      = $mapOption;
                        $response['routeLngLat']    = $routeLngLat;
                        $response['truckLngLat']    = $this->loadCurrentLocation($item);
                        $response['stopView']       = $this->loadStopView($item);
                        $response['fileView']       = $this->loadFileView($item);
                        $response['success']        = true;
                        $response['message']        = "Load detail retrieved successfully.";
                    }
                }
            }catch (\Exception $e) {
                $response['success']                = false;
                $response['message']                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function loadTruckLocationUpdate(Request $request){
        $response['success']                        = false;
        $response['message']                        = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                if($request->has('load_number')){
                    $item                           = LoadDetail::with(['driver','broker'])
                        ->where('load_number',$request->load_number)->first();
                    if(!is_null($item)){
                        $mapOption                  = [
                            'tollRoadsOption'       => $item->map_toll_roads ?? null,
                            'elevLimit'             => $item->map_max_elevation ?? null,
                            'avoidCountry'          => $item->map_close_borders ?? null
                        ];
                        $routeLngLat                = [];
                        if($item->shipper_address_verified==1){
                            $routeLngLat['pickup']  = array(
                                'longitude'         => $item->shipper_longitude,
                                'latitude'          => $item->shipper_latitude

                            );
                        }
                        if(isset($item->loads)){
                            if($item->loads->load_stops->isNotEmpty()){
                                foreach ($item->loads->load_stops as $key => $stop) {
                                    if($stop->address_verified==1){
                                        $routeLngLat['stop_'.++$key] = array(
                                            'longitude'         => $stop->longitude,
                                            'latitude'          => $stop->latitude

                                        );
                                    }
                                }
                            }
                        }
                        if($item->delivery_address_verified==1){
                            $routeLngLat['delivery']= array(
                                'longitude'         => $item->delivery_longitude,
                                'latitude'          => $item->delivery_latitude

                            );
                        }
                        $response['mapOption']      = $mapOption;
                        $response['routeLngLat']    = $routeLngLat;
                        $response['truckLngLat']    = $this->loadCurrentLocation($item);
                        $response['success']        = true;
                        $response['message']        = "Load truck retrieved successfully.";
                    }
                }
            }catch (\Exception $e) {
                $response['success']                = false;
                $response['message']                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function loadCurrentLocation($loadDetail){
        $array                              = [];
        if(isset($loadDetail->truck)){
            // eld to get truck location
            if(isset($loadDetail->truck->samsara_id)){
                $getAPIKey                  = IntegrationSetting::where('company_id', $loadDetail->truck->company_id)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_vehicles', '1')->first();
                if(!is_null($getAPIKey)){
                    try {
                        $url                = 'https://api.samsara.com/fleet/vehicles/locations?vehicleIds='.$loadDetail->truck->samsara_id;
                        $client             = new Client();
                        $api_response       = $client->request('GET', $url, [
                            'headers'                   => [
                                'Accept'                => 'application/json',
                                'Authorization'         => 'Bearer '.$getAPIKey->api_key,
                            ],
                        ]);
                        // Check if response status code is OK (200)
                        if ($api_response->getStatusCode() == 200) {
                            $body           = json_decode($api_response->getBody());
                            // Do something with the response body
                            if(isset($body)){
                                $data       = isset($body->data[0])?$body->data[0]:null;
                                if(isset($data)){
                                    if(isset($data->location)){
                                        $location       = $data->location;
                                        $array          = array(
                                            'longitude'         => $location->longitude,
                                            'latitude'          => $location->latitude,
                                            'address'           => $location->reverseGeo->formattedLocation,
                                            'last_updated'      => Carbon::parse($location->time)->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId
                                        );
                                    }
                                }
                            }
                        }
                    } catch (ClientException $e) {
                        // Handle client-side errors (e.g. 4xx status codes)
                        $api_response       = $e->getResponse();
                        if ($api_response) {
                            $statusCode     = $api_response->getStatusCode();
                            $body           = $api_response->getBody()->getContents();
                            // Log::error($body);
                        } else {
                            // Log::error($e->getMessage());
                        }
                    } catch (ServerException $e) {
                        // Handle server-side errors (e.g. 5xx status codes)
                        $api_response       = $e->getResponse();
                        if ($response) {
                            $statusCode     = $api_response->getStatusCode();
                            $body           = $api_response->getBody()->getContents();
                            // Log::error($body);
                        } else {
                            // Log::error($e->getMessage());
                        }
                    } catch (\Exception $e) {
                        // Handle other exceptions
                        // Log::error($e->getMessage());
                    }
                }
            }
            //
            if(empty($array)){
                if($loadDetail->shipper_address_verified==1 && $loadDetail->shipper_is_arrived==1){
                    $array                  = array(
                        'longitude'         => $loadDetail->shipper_longitude,
                        'latitude'          => $loadDetail->shipper_latitude,
                    );
                    $address                = [];
                    if(isset($loadDetail->shipper_address_one) && $loadDetail->shipper_address_one!==''){
                        $address[]          = $loadDetail->shipper_address_one;
                    }
                    if(isset($loadDetail->shipper_address_two) && $loadDetail->shipper_address_two!==''){
                        $address[]          = $loadDetail->shipper_address_two;
                    }
                    if(isset($loadDetail->shipper_city) && $loadDetail->shipper_city!==''){
                        $address[]          = $loadDetail->shipper_city;
                    }
                    if(isset($loadDetail->shipper_state) && $loadDetail->shipper_state!==''){
                        $address[]          = $loadDetail->shipper_state;
                    }
                    if(isset($loadDetail->shipper_zip) && $loadDetail->shipper_zip!==''){
                        $address[]          = $loadDetail->shipper_zip;
                    }
                    $array['address']       = implode(', ', $address);
                    if($loadDetail->shipper_is_leaves==1){
                        $last_updated       = isset($loadDetail->shipper_leaves_at)?convertTimeToUSERzone([
                            'str'           => $loadDetail->shipper_leaves_at,
                            'toFormat'      => getTimeLayout(),
                            'fromFormat'    => 'Y-m-d H:i:s',
                            'userTimezone'  => $this->timeZone
                        ]).' '.$this->timeZoneId:'';
                    }else{
                        $last_updated       = isset($loadDetail->shipper_arrived_at)?convertTimeToUSERzone([
                            'str'           => $loadDetail->shipper_arrived_at,
                            'toFormat'      => getTimeLayout(),
                            'fromFormat'    => 'Y-m-d H:i:s',
                            'userTimezone'  => $this->timeZone
                        ]).' '.$this->timeZoneId:'';
                    }
                    $array['last_updated']  = $last_updated;
                }
                if(isset($loadDetail->loads)){
                    if($loadDetail->loads->load_stops->isNotEmpty()){
                        foreach ($loadDetail->loads->load_stops as $key => $stop) {
                            if($stop->address_verified==1 && $stop->is_arrived==1){
                                $array                  = array(
                                    'longitude'         => $stop->longitude,
                                    'latitude'          => $stop->latitude,
                                );
                                $address                = [];
                                if(isset($stop->address_one) && $stop->address_one!==''){
                                    $address[]          = $stop->address_one;
                                }
                                if(isset($stop->address_two) && $stop->address_two!==''){
                                    $address[]          = $stop->address_two;
                                }
                                if(isset($stop->city) && $stop->city!==''){
                                    $address[]          = $stop->city;
                                }
                                if(isset($stop->state) && $stop->state!==''){
                                    $address[]          = $stop->state;
                                }
                                if(isset($stop->zip) && $stop->zip!==''){
                                    $address[]          = $stop->zip;
                                }
                                $array['address']       = implode(', ', $address);
                                if($stop->is_leaves==1){
                                    $last_updated       = isset($stop->leaves_at)?convertTimeToUSERzone([
                                        'str'           => $stop->leaves_at,
                                        'toFormat'      => getTimeLayout(),
                                        'fromFormat'    => 'Y-m-d H:i:s',
                                        'userTimezone'  => $this->timeZone
                                    ]).' '.$this->timeZoneId:'';
                                }else{
                                    $last_updated       = isset($stop->arrived_at)?convertTimeToUSERzone([
                                        'str'           => $stop->arrived_at,
                                        'toFormat'      => getTimeLayout(),
                                        'fromFormat'    => 'Y-m-d H:i:s',
                                        'userTimezone'  => $this->timeZone
                                    ]).' '.$this->timeZoneId:'';
                                }
                                $array['last_updated']  = $last_updated;
                            }
                        }
                    }
                }
                if($loadDetail->delivery_address_verified==1 && $loadDetail->delivery_is_arrived==1){
                    $array['load']          = array(
                        'longitude'         => $loadDetail->delivery_longitude,
                        'latitude'          => $loadDetail->delivery_latitude,
                    );
                    $address                = [];
                    if(isset($loadDetail->delivery_address_one) && $loadDetail->delivery_address_one!==''){
                        $address[]          = $loadDetail->delivery_address_one;
                    }
                    if(isset($loadDetail->delivery_address_two) && $loadDetail->delivery_address_two!==''){
                        $address[]          = $loadDetail->delivery_address_two;
                    }
                    if(isset($loadDetail->delivery_city) && $loadDetail->delivery_city!==''){
                        $address[]          = $loadDetail->delivery_city;
                    }
                    if(isset($loadDetail->delivery_state) && $loadDetail->delivery_state!==''){
                        $address[]          = $loadDetail->delivery_state;
                    }
                    if(isset($loadDetail->delivery_zip) && $loadDetail->delivery_zip!==''){
                        $address[]          = $loadDetail->delivery_zip;
                    }
                    $array['address']       = implode(', ', $address);
                    if($loadDetail->delivery_is_leaves==1){
                        $last_updated       = isset($loadDetail->delivery_leaves_at)?convertTimeToUSERzone([
                            'str'           => $loadDetail->delivery_leaves_at,
                            'toFormat'      => getTimeLayout(),
                            'fromFormat'    => 'Y-m-d H:i:s',
                            'userTimezone'  => $this->timeZone
                        ]).' '.$this->timeZoneId:'';
                    }else{
                        $last_updated       = isset($loadDetail->delivery_arrived_at)?convertTimeToUSERzone([
                            'str'           => $loadDetail->delivery_arrived_at,
                            'toFormat'      => getTimeLayout(),
                            'fromFormat'    => 'Y-m-d H:i:s',
                            'userTimezone'  => $this->timeZone
                        ]).' '.$this->timeZoneId:'';
                    }
                    $array['last_updated']  = $last_updated;
                }
            }
            if(!empty($array)){
                $array['truck_number']      = $loadDetail->truck_number;
            }
        }
        return $array;
    }

    public function loadStopView($loadDetail){
        $getTimeZoneId                                              = $this->timeZoneId;
        return view('companyAdmin.chat.load.stop.view', compact('loadDetail','getTimeZoneId'))->render();
    }

    public function loadFileView($loadDetail){
        $getTimeLayout                                              = getTimeLayout();

        $loadStop                                                   = LoadStop::where('load_id',$loadDetail->load_id)->orderBy('stop_no','ASC')->orderBy('date_time','ASC')->get();
        $static_files                                               = [];
        if(isset($loadDetail->invoice_files) && is_serialized($loadDetail->invoice_files) && !empty(unserialize($loadDetail->invoice_files))){
            $value                                                  = unserialize($loadDetail->invoice_files);
            $array                                                  = [];
            $array['id']                                            = $loadDetail->id;
            $array['source']                                        = 'load_details';
            $array['field_name']                                    = 'invoice_files';

            $array['name']                                          = 'Load';
            $array['filename']                                      = 'Invoice';
            $array['include_w_billing']                             = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
            $array['hide_from_driver']                              = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

            $status                                                 = 'Pending';
            $array['file']                                          = $value['file'] ?? null;
            if(isset($value['file']) && !empty($value['file'])){
                $status                                             = 'Complete';
                $array['upload_user_type']                          = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                $array['uploaded_by']                               = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                $array['uploaded_at']                               = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
            }
            $array['status']                                        = $status;

            $static_files[]                                         = $array;
        }else{
            $array                                                  = [];
            $array['id']                                            = $loadDetail->id;
            $array['source']                                        = 'load_details';
            $array['field_name']                                    = 'invoice_files';

            $array['name']                                          = 'Load';
            $array['filename']                                      = 'Invoice';
            $array['include_w_billing']                             = 'no';
            $array['hide_from_driver']                              = 'no';

            $array['file']                                          = null;
            $array['status']                                        = 'Pending';

            $static_files[]                                         = $array;
        }

        if(isset($loadDetail->rate_confirmation_files) && is_serialized($loadDetail->rate_confirmation_files) && !empty(unserialize($loadDetail->rate_confirmation_files))){
            $value                                                  = unserialize($loadDetail->rate_confirmation_files);
            $array                                                  = [];
            $array['id']                                            = $loadDetail->id;
            $array['source']                                        = 'load_details';
            $array['field_name']                                    = 'rate_confirmation_files';

            $array['name']                                          = 'Load';
            $array['filename']                                      = 'Rate Confirmation';
            $array['include_w_billing']                             = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
            $array['hide_from_driver']                              = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

            $status                                                 = 'Pending';
            $array['file']                                          = $value['file'] ?? null;
            if(isset($value['file']) && !empty($value['file'])){
                $status                                             = 'Complete';
                $array['upload_user_type']                          = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                $array['uploaded_by']                               = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                $array['uploaded_at']                               = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
            }
            $array['status']                                        = $status;

            $static_files[]                                         = $array;
        }else{
            $array                                                  = [];
            $array['id']                                            = $loadDetail->id;
            $array['source']                                        = 'load_details';
            $array['field_name']                                    = 'rate_confirmation_files';

            $array['name']                                          = 'Load';
            $array['filename']                                      = 'Rate Confirmation';
            $array['include_w_billing']                             = 'no';
            $array['hide_from_driver']                              = 'no';

            $array['file']                                          = null;
            $array['status']                                        = 'Pending';

            $static_files[]                                         = $array;
        }

        $required_files                                             = [];
        $requested_files                                            = [];
        //
        if(isset($loadDetail->shipper_files_upload) && is_serialized($loadDetail->shipper_files_upload) && !empty(unserialize($loadDetail->shipper_files_upload))){
            $uploads                                                = unserialize($loadDetail->shipper_files_upload);
            $uploads                                                = array_filter($uploads, function ($var) {
                return (isset($var['name_id']) && $var['name_id'] !== null && $var['type'] == 1);
            });
            foreach ($uploads as $key => $value) {
                $array                                              = [];
                $array['id']                                        = $loadDetail->id;
                $array['source']                                    = 'load_details';
                $array['field_name']                                = 'shipper_files_upload';

                $array['name']                                      = 1;
                $array['filename']                                  = $value['name'] ?? '';
                $array['filename_id']                               = $value['name_id'] ?? '';
                $array['include_w_billing']                         = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
                $array['hide_from_driver']                          = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

                $status                                             = 'Pending';
                $array['file']                                      = $value['file'] ?? null;
                if(isset($value['file']) && !empty($value['file'])){
                    $status                                         = 'Complete';
                    $array['upload_user_type']                      = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                    $array['uploaded_by']                           = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                    $array['uploaded_at']                           = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                }else{
                    $file_not_provided                              = (isset($value['file_not_provided']) && $value['file_not_provided']=="on")? "on" : "off";
                    if($file_not_provided == 'on'){
                        $status                                     = 'Not provided';
                    }
                }
                $array['status']                                    = $status;

                $required_files[]                                   = $array;
            }
        }
        if(isset($loadDetail->shipper_load_document_request_items) && $loadDetail->shipper_load_document_request_items->isNotEmpty()){
            foreach ($loadDetail->shipper_load_document_request_items as $key => $item) {
                $value                                              = [];
                if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                    $value                                          = unserialize($item->files);
                }
                if(!empty($value)){
                    if(
                        isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1])
                    ){
                        $array                                      = [];
                        $array['id']                                = $item->id;
                        $array['source']                            = 'load_document_request_items';
                        $array['field_name']                        = 'files';

                        $array['name']                              = 1;
                        $array['filename']                          = $value['name'] ?? '';
                        $array['filename_id']                       = $value['name_id'] ?? '';
                        $array['include_w_billing']                 = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
                        $array['hide_from_driver']                  = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

                        $status                                     = 'Pending';
                        $array['file']                              = $value['file'] ?? null;
                        if(isset($value['file']) && !empty($value['file'])){
                            $status                                 = 'Complete';
                            $array['upload_user_type']              = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                            $array['uploaded_by']                   = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                            $array['uploaded_at']                   = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                        }else{
                            $file_not_provided                      = (isset($value['file_not_provided']) && $value['file_not_provided']=="on")? "on" : "off";
                            if($file_not_provided == 'on'){
                                $status                             = 'Not provided';
                            }
                        }
                        $array['status']                            = $status;

                        $requested_files[]                          = $array;
                    }
                }
            }
        }

        $stop_no                                                    = 2;
        foreach ($loadStop as $key => $stop) {
            //
            $customer_type                                          = $stop->customer_type;
            $field_name                                             = $customer_type=='1'?'stop_shipper_files_upload':'stop_consignee_files_upload';
            if(isset($stop->$field_name) && is_serialized($stop->$field_name) && !empty(unserialize($stop->$field_name))){
                $uploads                                            = unserialize($stop->$field_name);
                $uploads                                            = array_filter($uploads, function ($var) {
                    return (isset($var['name_id']) && $var['name_id'] !== null && $var['type'] == 1);
                });
                foreach ($uploads as $key => $value) {
                    $array                                          = [];
                    $array['id']                                    = $stop->id;
                    $array['source']                                = 'load_stops';
                    $array['field_name']                            = $field_name;

                    $array['name']                                  = $stop_no;
                    $array['filename']                              = $value['name'];
                    $array['filename_id']                           = $value['name_id'] ?? '';
                    $array['include_w_billing']                     = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
                    $array['hide_from_driver']                      = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

                    $status                                         = 'Pending';
                    $array['file']                                  = $value['file'] ?? null;
                    if(isset($value['file']) && !empty($value['file'])){
                        $status                                     = 'Complete';
                        $array['upload_user_type']                  = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                        $array['uploaded_by']                       = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                        $array['uploaded_at']                       = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                    }else{
                        $file_not_provided                          = (isset($value['file_not_provided']) && $value['file_not_provided']=="on")? "on" : "off";
                        if($file_not_provided == 'on'){
                            $status                                 = 'Not provided';
                        }
                    }
                    $array['status']                                = $status;

                    $required_files[]                               = $array;
                }
            }
            if(isset($stop->load_document_request_items) && $stop->load_document_request_items->isNotEmpty()){
                foreach ($stop->load_document_request_items as $key => $item) {
                    $value                                          = [];
                    if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                        $value                                      = unserialize($item->files);
                    }
                    if(!empty($value)){
                        if(
                            isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1])
                        ){
                            $array                                  = [];
                            $array['id']                            = $item->id;
                            $array['source']                        = 'load_document_request_items';
                            $array['field_name']                    = 'files';

                            $array['name']                          = $stop_no;
                            $array['filename']                      = $value['name'] ?? '';
                            $array['filename_id']                   = $value['name_id'] ?? '';
                            $array['include_w_billing']             = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
                            $array['hide_from_driver']              = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

                            $status                                 = 'Pending';
                            $array['file']                          = $value['file'] ?? null;
                            if(isset($value['file']) && !empty($value['file'])){
                                $status                             = 'Complete';
                                $array['upload_user_type']          = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                                $array['uploaded_by']               = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                                $array['uploaded_at']               = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                            }else{
                                $file_not_provided                  = (isset($value['file_not_provided']) && $value['file_not_provided']=="on")? "on" : "off";
                                if($file_not_provided == 'on'){
                                    $status                         = 'Not provided';
                                }
                            }
                            $array['status']                        = $status;

                            $requested_files[]                      = $array;
                        }
                    }
                }
            }
            //
            ++$stop_no;
        }

        //
        if(isset($loadDetail->consignee_files_upload) && is_serialized($loadDetail->consignee_files_upload) && !empty(unserialize($loadDetail->consignee_files_upload))){
            $uploads                                                = unserialize($loadDetail->consignee_files_upload);
            $uploads                                                = array_filter($uploads, function ($var) {
                return (isset($var['name_id']) && $var['name_id'] !== null && $var['type'] == 1);
            });
            foreach ($uploads as $key => $value) {
                $array                                              = [];
                $array['id']                                        = $loadDetail->id;
                $array['source']                                    = 'load_details';
                $array['field_name']                                = 'consignee_files_upload';

                $array['name']                                      = 99;
                $array['filename']                                  = $value['name'];
                $array['filename_id']                               = $value['name_id'] ?? '';
                $array['include_w_billing']                         = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
                $array['hide_from_driver']                          = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

                $status                                             = 'Pending';
                $array['file']                                      = $value['file'] ?? null;
                if(isset($value['file']) && !empty($value['file'])){
                    $status                                         = 'Complete';
                    $array['upload_user_type']                      = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                    $array['uploaded_by']                           = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                    $array['uploaded_at']                           = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                }else{
                    $file_not_provided                              = (isset($value['file_not_provided']) && $value['file_not_provided']=="on")? "on" : "off";
                    if($file_not_provided == 'on'){
                        $status                                     = 'Not provided';
                    }
                }
                $array['status']                                    = $status;

                $required_files[]                                   = $array;
            }
        }
        if(isset($loadDetail->receiver_load_document_request_items) && $loadDetail->receiver_load_document_request_items->isNotEmpty()){
            foreach ($loadDetail->receiver_load_document_request_items as $key => $item) {
                $value                                              = [];
                if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                    $value                                          = unserialize($item->files);
                }
                if(!empty($value)){
                    if(
                        isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1])
                    ){
                        $array                                      = [];
                        $array['id']                                = $item->id;
                        $array['source']                            = 'load_document_request_items';
                        $array['field_name']                        = 'files';

                        $array['name']                              = 99;
                        $array['filename']                          = $value['name'] ?? '';
                        $array['filename_id']                       = $value['name_id'] ?? '';
                        $array['include_w_billing']                 = isset($value['include_w_billing']) && $value['include_w_billing']=='yes' ? 'yes' : 'no';
                        $array['hide_from_driver']                  = isset($value['hide_from_driver']) && $value['hide_from_driver']=='yes' ? 'yes' : 'no';

                        $status                                     = 'Pending';
                        $array['file']                              = $value['file'] ?? null;
                        if(isset($value['file']) && !empty($value['file'])){
                            $status                                 = 'Complete';
                            $array['upload_user_type']              = isset($value['upload_user_type'])?$value['upload_user_type']:null;
                            $array['uploaded_by']                   = isset($value['upload_by'])?getUserByType($value['upload_by'],$value['upload_user_type']):null;
                            $array['uploaded_at']                   = isset($value['uploaded_at'])?Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                        }else{
                            $file_not_provided                      = (isset($value['file_not_provided']) && $value['file_not_provided']=="on")? "on" : "off";
                            if($file_not_provided == 'on'){
                                $status                             = 'Not provided';
                            }
                        }
                        $array['status']                            = $status;

                        $requested_files[]                          = $array;
                    }
                }
            }
        }
        //

        $accessorial_files                                          = [];
        $shipper_accessorials                                       = $loadDetail->shipper_accessorials()->where('doc_required', '!=', 0)->get();
        if($shipper_accessorials->isNotEmpty()){
            foreach ($shipper_accessorials as $key => $load_accessorial) {
                $gallery                                            = Gallery::where('company_id', auth()->guard($this->logged_in_guard)->user()->company_id)
                    ->where('action_id', $load_accessorial->id)
                    ->where('source', 'load_accessorials')
                    ->where('field_name', 'accessorial_files')
                    ->first();
                $filename                                           = $load_accessorial->company_accessorial->name;
                if(!is_null($gallery)){
                    $filename                                      .= ' - '.$gallery->file_name;
                }
                $array                                              = [];
                $array['id']                                        = $load_accessorial->id;
                $array['source']                                    = 'load_accessorials';
                $array['field_name']                                = 'image';

                $array['name']                                      = 1;
                $array['filename']                                  = $filename;
                $array['include_w_billing']                         = isset($load_accessorial->include_w_billing) && $load_accessorial->include_w_billing=='yes' ? 'yes' : 'no';
                $array['hide_from_driver']                          = isset($load_accessorial->hide_from_driver) && $load_accessorial->hide_from_driver=='yes' ? 'yes' : 'no';

                $array['file']                                      = $gallery->image ?? null;
                //status
                if(isset($load_accessorial->temp_details) && is_serialized($load_accessorial->temp_details) && !empty(unserialize($load_accessorial->temp_details))){
                    $temp_details                                   = unserialize($load_accessorial->temp_details);
                    if(isset($temp_details['status'])){
                        $load_accessorial->status                   = $temp_details['status'];
                    }
                }
                $status                                             = 'Pending';
                if($load_accessorial->status=='1'){
                    $status                                         = 'Complete';
                }elseif ($load_accessorial->status=='2') {
                    $status                                         = 'Driver Bypass';
                }
                //

                if(isset($gallery->image) && !empty($gallery->image)){
                    $array['upload_user_type']                      = isset($gallery->created_by_type)?$gallery->created_by_type:null;
                    $array['uploaded_by']                           = isset($gallery->created_by)?getUserByType($gallery->created_by,$gallery->created_by_type):null;
                    $array['uploaded_at']                           = isset($gallery->created_at)?Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                }
                $array['status']                                    = $status;

                $accessorial_files[]                                = $array;
            }
        }

        $accessorial_no                                             = 2;
        foreach ($loadStop as $key => $stop) {
            $stop_accessorials                                      = $stop->accessorials()->where('doc_required', '!=', 0)->get();
            if($stop_accessorials->isNotEmpty()){
                foreach ($stop_accessorials as $key => $load_accessorial) {
                    $gallery                                        = Gallery::where('company_id', auth()->guard($this->logged_in_guard)->user()->company_id)
                        ->where('action_id', $load_accessorial->id)
                        ->where('source', 'load_accessorials')
                        ->where('field_name', 'accessorial_files')
                        ->first();
                    $filename                                       = $load_accessorial->company_accessorial->name;
                    if(!is_null($gallery)){
                        $filename                                  .= ' - '.$gallery->file_name;
                    }
                    $array                                          = [];
                    $array['id']                                    = $load_accessorial->id;
                    $array['source']                                = 'load_accessorials';
                    $array['field_name']                            = 'image';

                    $array['name']                                  = $accessorial_no;
                    $array['filename']                              = $filename;
                    $array['include_w_billing']                     = isset($load_accessorial->include_w_billing) && $load_accessorial->include_w_billing=='yes' ? 'yes' : 'no';
                    $array['hide_from_driver']                      = isset($load_accessorial->hide_from_driver) && $load_accessorial->hide_from_driver=='yes' ? 'yes' : 'no';

                    $array['file']                                  = $gallery->image ?? null;
                    //status
                    if(isset($load_accessorial->temp_details) && is_serialized($load_accessorial->temp_details) && !empty(unserialize($load_accessorial->temp_details))){
                        $temp_details                               = unserialize($load_accessorial->temp_details);
                        if(isset($temp_details['status'])){
                            $load_accessorial->status               = $temp_details['status'];
                        }
                    }
                    $status                                         = 'Pending';
                    if($load_accessorial->status=='1'){
                        $status                                     = 'Complete';
                    }elseif ($load_accessorial->status=='2') {
                        $status                                     = 'Driver Bypass';
                    }
                    //

                    if(isset($gallery->image) && !empty($gallery->image)){
                        $array['upload_user_type']                  = isset($gallery->created_by_type)?$gallery->created_by_type:null;
                        $array['uploaded_by']                       = isset($gallery->created_by)?getUserByType($gallery->created_by,$gallery->created_by_type):null;
                        $array['uploaded_at']                       = isset($gallery->created_at)?Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                    }
                    $array['status']                                = $status;

                    $accessorial_files[]                            = $array;
                }
            }
            ++$accessorial_no;
        }

        $receiver_accessorials                                      = $loadDetail->receiver_accessorials()->where('doc_required', '!=', 0)->get();
        if($receiver_accessorials->isNotEmpty()){
            foreach ($receiver_accessorials as $key => $load_accessorial) {
                $gallery                                            = Gallery::where('company_id', auth()->guard($this->logged_in_guard)->user()->company_id)
                    ->where('action_id', $load_accessorial->id)
                    ->where('source', 'load_accessorials')
                    ->where('field_name', 'accessorial_files')
                    ->first();
                $filename                                           = $load_accessorial->company_accessorial->name;
                if(!is_null($gallery)){
                    $filename                                      .= ' - '.$gallery->file_name;
                }
                $array                                              = [];
                $array['id']                                        = $load_accessorial->id;
                $array['source']                                    = 'load_accessorials';
                $array['field_name']                                = 'image';

                $array['name']                                      = 99;
                $array['filename']                                  = $filename;
                $array['include_w_billing']                         = isset($load_accessorial->include_w_billing) && $load_accessorial->include_w_billing=='yes' ? 'yes' : 'no';
                $array['hide_from_driver']                          = isset($load_accessorial->hide_from_driver) && $load_accessorial->hide_from_driver=='yes' ? 'yes' : 'no';

                $array['file']                                      = $gallery->image ?? null;
                //status
                if(isset($load_accessorial->temp_details) && is_serialized($load_accessorial->temp_details) && !empty(unserialize($load_accessorial->temp_details))){
                    $temp_details                                   = unserialize($load_accessorial->temp_details);
                    if(isset($temp_details['status'])){
                        $load_accessorial->status                   = $temp_details['status'];
                    }
                }
                $status                                             = 'Pending';
                if($load_accessorial->status=='1'){
                    $status                                         = 'Complete';
                }elseif ($load_accessorial->status=='2') {
                    $status                                         = 'Driver Bypass';
                }
                //

                if(isset($gallery->image) && !empty($gallery->image)){
                    $array['upload_user_type']                      = isset($gallery->created_by_type)?$gallery->created_by_type:null;
                    $array['uploaded_by']                           = isset($gallery->created_by)?getUserByType($gallery->created_by,$gallery->created_by_type):null;
                    $array['uploaded_at']                           = isset($gallery->created_at)?Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                }
                $array['status']                                    = $status;

                $accessorial_files[]                                = $array;
            }
        }

        $billing_accessorials                                       = $loadDetail->billing_accessorials()->where('doc_required', '!=', 0)->get();
        if($billing_accessorials->isNotEmpty()){
            foreach ($billing_accessorials as $key => $load_accessorial) {
                $gallery                                            = Gallery::where('company_id', auth()->guard($this->logged_in_guard)->user()->company_id)
                    ->where('action_id', $load_accessorial->id)
                    ->where('source', 'load_accessorials')
                    ->where('field_name', 'accessorial_files')
                    ->first();
                $filename                                           = $load_accessorial->company_accessorial->name;
                if(!is_null($gallery)){
                    $filename                                      .= ' - '.$gallery->file_name;
                }
                $array                                              = [];
                $array['id']                                        = $load_accessorial->id;
                $array['source']                                    = 'load_accessorials';
                $array['field_name']                                = 'image';

                $array['name']                                      = 'Load';
                $array['filename']                                  = $filename;
                $array['include_w_billing']                         = isset($load_accessorial->include_w_billing) && $load_accessorial->include_w_billing=='yes' ? 'yes' : 'no';
                $array['hide_from_driver']                          = isset($load_accessorial->hide_from_driver) && $load_accessorial->hide_from_driver=='yes' ? 'yes' : 'no';

                $array['file']                                      = $gallery->image ?? null;
                //status
                if(isset($load_accessorial->temp_details) && is_serialized($load_accessorial->temp_details) && !empty(unserialize($load_accessorial->temp_details))){
                    $temp_details                                   = unserialize($load_accessorial->temp_details);
                    if(isset($temp_details['status'])){
                        $load_accessorial->status                   = $temp_details['status'];
                    }
                }
                $status                                             = 'Pending';
                if($load_accessorial->status=='1'){
                    $status                                         = 'Complete';
                }elseif ($load_accessorial->status=='2') {
                    $status                                         = 'Driver Bypass';
                }
                //

                if(isset($gallery->image) && !empty($gallery->image)){
                    $array['upload_user_type']                      = isset($gallery->created_by_type)?$gallery->created_by_type:null;
                    $array['uploaded_by']                           = isset($gallery->created_by)?getUserByType($gallery->created_by,$gallery->created_by_type):null;
                    $array['uploaded_at']                           = isset($gallery->created_at)?Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                }
                $array['status']                                    = $status;

                $accessorial_files[]                                = $array;
            }
        }

        $cod_files                                                  = [];
        $get_cod_files                                              = Gallery::where('company_id', auth()->guard($this->logged_in_guard)->user()->company_id)
            ->where('action_id', $loadDetail->load_id)
            ->where('source', 'load')
            ->whereIn('field_name', ['up_front_file', 'at_pickup_file', 'at_delivery_file'])
            ->get();
        if($get_cod_files->isNotEmpty()){
            foreach ($get_cod_files as $key => $cod_file) {
                if(isset($cod_file->image) && $cod_file->image!=''){
                    $array                                          = [];
                    $array['id']                                    = $cod_file->id;
                    $array['source']                                = 'load_cod_galleries';
                    $array['field_name']                            = 'image';

                    $array['name']                                  = 'COD';
                    $array['filename']                              = $cod_file->file_name;
                    $array['include_w_billing']                     = isset($cod_file->include_w_billing) && $cod_file->include_w_billing=='yes' ? 'yes' : 'no';
                    $array['hide_from_driver']                      = isset($cod_file->hide_from_driver) && $cod_file->hide_from_driver=='yes' ? 'yes' : 'no';

                    $array['file']                                  = $cod_file->image ?? null;
                    if(isset($cod_file->image) && !empty($cod_file->image)){
                        $array['upload_user_type']                  = isset($cod_file->created_by_type)?$cod_file->created_by_type:null;
                        $array['uploaded_by']                       = isset($cod_file->created_by)?getUserByType($cod_file->created_by,$cod_file->created_by_type):null;
                        $array['uploaded_at']                       = isset($cod_file->created_at)?Carbon::createFromFormat('Y-m-d H:i:s', $cod_file->created_at)->setTimezone($this->timeZone)->format($getTimeLayout)." ".$this->timeZoneId:null;
                    }
                    $array['status']                                = 'Complete';

                    $cod_files[]                                    = $array;
                }
            }
        }

        $getFile                                                    = LoadFile::where('load_id', $loadDetail->load_id)->orderBy('id','desc')->get();
        return view('companyAdmin.chat.load.file.view', compact('static_files', 'required_files', 'requested_files', 'accessorial_files', 'cod_files', 'getFile'))->render();
    }
}
