<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use App\Models\Bookmark;
use App\Models\CompanyCommodity;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Yajra\DataTables\Facades\DataTables;
use App\Helpers\Common;

class CommodityController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $getField = '';
        $getCompanyAdmin = auth()->guard('company-admin')->user();
        $isCheck = Bookmark::where('company_id', getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type', $getCompanyAdmin->type)->where('table_id', 'commodity')->first();

        if(!empty($isCheck->id)){
            //$getField = unserialize($isCheck->config);
            $getField = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_commodity_first'));
        }else{
            $getField = config('settings.table_commodity_first');
        }

        if ($request->ajax()) {
            $data = CompanyCommodity::IsCompany();
            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq = 'search_string_condition_'.$key;
                $sortReq = 'search_string_sort_'.$key;

                if( isset( request()->$conditionReq ) ) {
                    $typeReq = 'search_string_type_'.$key;
                    $type = request()->$typeReq;

                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }

                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                if( isset( request()->$sortReq ) ) {
                    $nameReq = 'search_string_name_'.$key;
                    $sortReq = 'search_string_sort_'.$key;
                    $typeReq = 'search_string_type_'.$key;
                    $tableNameReq = 'search_table_name_'.$key;
                    $colAliasReq = 'search_string_col_alias_'.$key;

                    $origData = request()->$nameReq;
                    $condition_sort = request()->$sortReq;
                    $type = request()->$typeReq;
                    $tableName = isset( request()->$tableNameReq ) ? request()->$tableNameReq : '';
                    $colAlias = isset( request()->$colAliasReq ) ? request()->$colAliasReq : '';

                    if( $type == 'string' ) {
                        if( isset( $condition_sort ) ) {
                            if( $colAlias != '' ) {
                                $data->orderBy( $tableName.'.'.$colAlias, $condition_sort );
                            } else {
                                if( $tableName != '' ) {
                                    $data->orderBy( $tableName.'.'.$origData, $condition_sort );
                                } else {
                                    $data->orderBy( $origData, $condition_sort );
                                }
                            }
                        }
                    }
                }
            }
            $data->get();
            // General filter code - end
            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $action = '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-action edit-btn" data-id="'.$row->id.'" data-toggle="modal" data-target="#editModal"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                    $action .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash destroy" data-action="'.route('company.admin.commodity.destroy', $row->id).'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';

                    return $action;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        $tableName = 'settings.table_commodity';
        return view('companyAdmin.commodity.index', compact('getField','tableName'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $input      = $request->all();
        $validator  = Validator::make($input,[
            "name"  => "required",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }
        $input['company_id'] = auth()->guard('company-admin')->user()->company_id;
        CompanyCommodity::create($input);

        notification('success', $this->crudMessage('create', 'Commodity'));

        return response()->json(['success' => "Commodity created successfully"]);
    }

    public function editFormView(Request $request)
    {
        $commodity = CompanyCommodity::find($request->id);

        $view = view('companyAdmin.commodity.editFormView', compact('commodity'))->render();

        return response()->json(['view' => $view, 'data' => $commodity]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, CompanyCommodity $commodity)
    {
        $input = $request->all();

        $validator  = Validator::make($input,[
            "name"  => "required",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $commodity->update($input);

        notification('success', $this->crudMessage('update', 'Commodity'));

        return response()->json(['success' => "Commodity updated successfully"]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(CompanyCommodity $commodity)
    {
        $commodity->delete();

        notification('success', $this->crudMessage('delete', 'Commodity'));

        return redirect()->route('company.admin.commodity.index');
    }
}
