<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;
use App\Http\Requests\CompanyAdmin\CompanyHolidayStoreRequest;
use App\Http\Requests\CompanyAdmin\CompanyHolidayUpdateRequest;
use App\Models\CompanyHoliday;
use App\Models\Bookmark;
use Hash;
use DataTables;
use Carbon\Carbon;
use Illuminate\Support\Str;
use App\Helpers\Common;

class CompanyHolidayController extends CompanyAdminController
{
    /**

     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        try {
            $getField = '';

            $getCompanyAdmin = auth()->guard('company-admin')->user();

            $isCheck = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','company_holiday')->first();

            if( !empty( $isCheck->id ) ) {
                //$getField = unserialize($isCheck->config);
                $getField = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_company_holiday_first'));
            } else {
                $getField = config('settings.table_company_holiday_first');
            }

            if( $request->ajax() ) {
                $data = CompanyHoliday::isCompany();

                // General filter code - start
                foreach( $getField as $key => $value ) {
                    $conditionReq = 'search_string_condition_'.$key;
                    $sortReq = 'search_string_sort_'.$key;

                    $conditionDateReq = 'search_date_condition_'.$key;
                    $sortDateReq = 'search_date_sort_'.$key;

                    // String filter
                    if( isset( request()->$conditionReq ) ) {
                        $typeReq = 'search_string_type_'.$key;
                        $type = request()->$typeReq;

                        if( $type == 'string' ) {
                            Common::getStringFilterResult( $data, $key );
                        }
                    }

                    // Date filter
                    if( isset( request()->$conditionDateReq ) ) {
                        $typeReq = 'search_date_type_'.$key;
                        $type = request()->$typeReq;

                        if( $type == 'date' ) {
                            Common::getDateFilterResult( $data, $key );
                        }
                    }

                    // String sort only
                    if( isset( request()->$sortReq ) ) {
                        Common::sortStringFilter( $data, $key );
                    }

                    // Date sort only
                    if( isset( request()->$sortDateReq ) ) {
                        Common::sortDateFilter( $data, $key );
                    }
                }
                // General filter code - end

                $data->select('*');

                return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row) {
                    $action = '<a href="'.route('company.admin.company-holiday.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                    $action .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash destroy" data-action="'.route('company.admin.company-holiday.destroy',$row->id).'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';

                    return $action;
                })
                ->filterColumn("from_date", function ($query, $keyword) {
                    $query->whereRaw(
                        "DATE_FORMAT(from_date,'%m/%d/%Y') LIKE ?",
                        ["%$keyword%"]
                    );
                })
                ->filterColumn("to_date", function ($query, $keyword) {
                    $query->whereRaw(
                        "DATE_FORMAT(to_date,'%m/%d/%Y') LIKE ?",
                        ["%$keyword%"]
                    );
                })
                ->rawColumns(['action'])
                ->make(true);
            }

            $tableName = 'settings.table_company_holiday';

            // return view('companyAdmin.companyHoliday.index', compact('getField'));
            return view('companyAdmin.companyHoliday.temp-jp', compact('getField','tableName'));
        } catch( \Exception $e ) {
            \Log::emergency( "File: ".$e->getFile() );
            \Log::emergency( "Line: ".$e->getLine() );
            \Log::emergency( "Message: ".$e->getMessage() );
        }
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('companyAdmin.companyHoliday.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(CompanyHolidayStoreRequest $request)
    {
        $input = $request->all();
        $input['company_id'] = auth()->guard('company-admin')->user()->company_id;
        CompanyHoliday::create($input);

        notification('success', $this->crudMessage('create', 'Company holiday'));
        return redirect()->route('company.admin.company-holiday.index');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $id = base64Decode($id);
        $companyHoliday = CompanyHoliday::findOrFail($id);
       return redirect()->route('company.admin.company-holiday.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $id = base64Decode($id);
        $companyHoliday = CompanyHoliday::findOrFail($id);
        return view('companyAdmin.companyHoliday.edit', compact('companyHoliday'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(CompanyHolidayUpdateRequest $request, CompanyHoliday $companyHoliday)
    {
        $input = $request->all();
        $companyHoliday->update($input);
        notification('success', $this->crudMessage('update', 'Holiday'));
        return redirect()->route('company.admin.company-holiday.index');
    }

    public function destroy(CompanyHoliday $companyHoliday)
    {
        $companyHoliday->delete();

        notification('success', $this->crudMessage('delete', 'Holiday'));

        return redirect()->route('company.admin.company-holiday.index');
    }

}
