<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\Controller;

use Illuminate\Support\Facades\Log;
use Illuminate\Http\Request;

use App\Models\Driver;
use App\Models\CompanyAdmin;

use App\Models\Offer;
use App\Models\Gallery;

use App\Models\LoadStop;
use App\Models\LoadDetail;
use App\Models\LoadAppointment;
use App\Models\LoadAppointmentLog;
use App\Models\LoadAccessorialTemp;

use App\Models\Truck;
use App\Models\IntegrationSetting;
use App\Models\VehicleHistoricalStats;

use App\Models\FuelSurchargeTemplate;

use App\Traits\TrimbleTrait;
use App\Traits\SamsaraTrait;

use DB;
use File;
use Carbon\Carbon;
use Carbon\CarbonPeriod;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ServerException;

class CronJobController extends Controller
{

    use TrimbleTrait, SamsaraTrait;

    public function driverMissingLoad(Request $request)
    {
        Log::info('Cron Start : Driver Missing Load ( '.Carbon::now()->format('Y-m-d').' )');
        $now                = Carbon::now(); //Carbon::parse('2022-05-01');
        $currentDate        = $now->format('Y-m-d');
        $previous           = Carbon::yesterday(); //$now->subDays();
        $previousDate       = $previous->format('Y-m-d');

        $month              = $previous->format('m');
        $monthName          = $previous->format('F');
        $year               = $previous->format('Y');

        $arry               = [];
        $query              = Driver::select(
                'drivers.id',
                'drivers.manager_id',
                'drivers.team_id',
                'drivers.first_name',
                'drivers.last_name',
                'drivers.middle_name',
                DB::raw('CONCAT(drivers.last_name, " ", drivers.first_name) AS dname'),
                'drivers.start_date'
            )
            ->join('load_details','load_details.driver_id','=','drivers.id')
            ->join('driver_informations','driver_informations.driver_id','=','drivers.id')
            ->whereDate('drivers.start_date', '<=', $previousDate)
            ->where('load_details.is_disable',0)
            ->where('load_details.load_status', '!=', 9)
            ->where(function($q) {
                $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
            })
            ->where(function($q) {
                $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
            })
            ->groupBy('drivers.id')
            ->orderByRaw('drivers.last_name ASC, drivers.first_name ASC');
        $drivers        = $query->get();
        if($drivers->isNotEmpty()){
            foreach ($drivers as $key => $driver) {
                $getLoadByDateMonth = getLoadByDateMonth($previousDate,$driver->id);
                if(count($getLoadByDateMonth)<=0){
                    if(isset($driver->manager_id) && $driver->manager_id!=''){
                        $arry[$driver->manager_id][] = $driver->dname; //$arry[$driver->team->head_id][] = $driver->dname;
                    }
                }
            }
        }
        if(!empty($arry)){
            foreach ($arry as $dispatcherId => $names) {
                $dispatcher     = CompanyAdmin::whereId($dispatcherId)->first();
                if($dispatcher){
                    // 5 = Liner Fleet Calendar
                    $properties = [
                        'user_id'       => $dispatcher->id,
                        'user_type'     => $dispatcher->type,
                        'previous_date' => $previousDate
                    ];
                    createNotification(
                        6, // source
                        'Attention!', // name
                        $dispatcher->id, // user_id
                        $dispatcher->type, // user_type
                        2, // type
                        null, // action_id
                        'Driver Missing Load - '.implode(', ', $names), // description
                        'Alert', // module
                        $properties, // properties
                        $dispatcher->company_id // company_id
                    );
                }
            }
        }
        Log::info('Cron End : Driver Missing Load ( '.Carbon::now()->format('Y-m-d').' )');
    }

    public function loadNotPickedUponTime(Request $request)
    {
        Log::info('Cron Start : Load not picked up on time ( '.Carbon::now()->toDateTimeString().' )');
        Log::info('Older 30 min date & time ( '.Carbon::now()->subMinutes(30)->toDateTimeString().' )');
        $loads = LoadDetail::select('id','load_id','load_number','driver_id','booked_by','submitted_by','load_type','load_status','shipper_date_time','shipper_date_time_end','load_not_picked_up_on_time','self_dispatch','is_on_hold')
            ->where('is_disable', 0)
            ->whereIn('load_type', [0])
            ->where('load_status', 3)
            ->where(function($q) {
                $q->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0]);
            })
            ->where(function($q) {
                $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
            })
            ->whereRaw('IFNULL(shipper_date_time_end, shipper_date_time) <= ?', [Carbon::now()->subMinutes(30)->toDateTimeString()])
            ->whereColumn(DB::raw('IFNULL(shipper_date_time_end, shipper_date_time)'),'!=',DB::raw('IFNULL(load_not_picked_up_on_time, "")'))
            ->orderByRaw('IFNULL(shipper_date_time_end, shipper_date_time) asc')
            ->get();
        if($loads->isNotEmpty()){
            Log::info('Total Load not picked up on time : ( '.$loads->count().' )');
            foreach ($loads as $key => $load) {
                $load_not_picked_up_on_time         = $load->getRawOriginal('shipper_date_time');
                if(!is_null($load->getRawOriginal('shipper_date_time_end'))){
                    $load_not_picked_up_on_time     = $load->getRawOriginal('shipper_date_time_end');
                }
                $load->load_not_picked_up_on_time   = $load_not_picked_up_on_time;
                $load->updated_at                   = Carbon::now();
                $load->save();
                if(isset($load->self_dispatch) && $load->self_dispatch==1){
                    if((isset($load->driver) && !is_null($load->driver)) && (isset($load->driver->dispatcher) && !is_null($load->driver->dispatcher))){
                        createNotification(
                            1, // source
                            'Attention!', // name
                            $load->driver->dispatcher->id, // user_id
                            $load->driver->dispatcher->type, // user_type
                            2, // type
                            $load->load_id, // action_id
                            'Load not picked up on time - '.$load->load_number, // description
                            'Alert', // module
                            [], // properties
                            $load->driver->dispatcher->company_id // company_id
                        );
                    }
                }elseif (isset($load->dispatcher) && !is_null($load->dispatcher)) {
                    createNotification(
                        1, // source
                        'Attention!', // name
                        $load->dispatcher->id, // user_id
                        $load->dispatcher->type, // user_type
                        2, // type
                        $load->load_id, // action_id
                        'Load not picked up on time - '.$load->load_number, // description
                        'Alert', // module
                        [], // properties
                        $load->dispatcher->company_id // company_id
                    );
                }else{
                    Log::info('Driver manager record not found : ( '.$load->load_number.' )');
                }
            }
        }else{
            Log::info('No record found : Load not picked up on time');
        }
        Log::info('Cron End : Load not picked up on time ( '.Carbon::now()->toDateTimeString().' )');
    }

    public function loadNotDeliveredOnTime(Request $request)
    {
        Log::info('Cron Start : Load not delivered on time ( '.Carbon::now()->toDateTimeString().' )');
        Log::info('Older 30 min date & time ( '.Carbon::now()->subMinutes(30)->toDateTimeString().' )');
        $loads = LoadDetail::select('id','load_id','load_number','driver_id','booked_by','submitted_by','load_type','load_status','delivery_date_time','delivery_date_time_end','load_not_delivered_on_time','self_dispatch','is_on_hold')
            ->where('is_disable', 0)
            ->whereIn('load_type', [0])
            ->where('load_status', 4)
            ->where(function($q) {
                $q->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0]);
            })
            ->where(function($q) {
                $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
            })
            ->whereRaw('IFNULL(delivery_date_time_end, delivery_date_time) <= ?', [Carbon::now()->subMinutes(30)->toDateTimeString()])
            ->whereColumn(DB::raw('IFNULL(delivery_date_time_end, delivery_date_time)'),'!=',DB::raw('IFNULL(load_not_delivered_on_time, "")'))
            ->orderByRaw('IFNULL(delivery_date_time_end, delivery_date_time) asc')
            ->get();
        if($loads->isNotEmpty()){
            Log::info('Total Load not delivered on time : ( '.$loads->count().' )');
            foreach ($loads as $key => $load) {
                $load_not_delivered_on_time         = $load->getRawOriginal('delivery_date_time');
                if(!is_null($load->getRawOriginal('delivery_date_time_end'))){
                    $load_not_delivered_on_time     = $load->getRawOriginal('delivery_date_time_end');
                }
                $load->load_not_delivered_on_time   = $load_not_delivered_on_time;
                $load->updated_at                   = Carbon::now();
                $load->save();
                if(isset($load->self_dispatch) && $load->self_dispatch==1){
                    if((isset($load->driver) && !is_null($load->driver)) && (isset($load->driver->dispatcher) && !is_null($load->driver->dispatcher))){
                        createNotification(
                            2, // source
                            'Attention!', // name
                            $load->driver->dispatcher->id, // user_id
                            $load->driver->dispatcher->type, // user_type
                            2, // type
                            $load->load_id, // action_id
                            'Load not delivered on time - '.$load->load_number, // description
                            'Alert', // module
                            [], // properties
                            $load->driver->dispatcher->company_id // company_id
                        );
                    }
                }elseif (isset($load->dispatcher) && !is_null($load->dispatcher)) {
                    createNotification(
                        2, // source
                        'Attention!', // name
                        $load->dispatcher->id, // user_id
                        $load->dispatcher->type, // user_type
                        2, // type
                        $load->load_id, // action_id
                        'Load not delivered on time - '.$load->load_number, // description
                        'Alert', // module
                        [], // properties
                        $load->dispatcher->company_id // company_id
                    );
                }else{
                    Log::info('Driver manager record not found : ( '.$load->load_number.' )');
                }
            }
        }else{
            Log::info('No record found : Load not delivered on time');
        }
        Log::info('Cron End : Load not delivered on time ( '.Carbon::now()->toDateTimeString().' )');
    }

    public function loadAccessorialTempsDestroy(Request $request)
    {
        Log::info('Cron Start : Load accessorial temps destroy ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
        $records = LoadAccessorialTemp::where('created_at', '<=', now()->subMinutes(15)->toDateTimeString())->get();
        if($records->isNotEmpty()){
            Log::info('Total Load accessorial temps destroy : ( '.$records->count().' )');
            foreach ($records as $key => $record) {
                $galleries = Gallery::where('action_id', $record->id)
                    ->where('source', 'load_accessorial_temps')
                    ->get();
                if($galleries->isNotEmpty()){
                    foreach ($galleries as $key => $gallery) {
                        $path       = 'storage/companyAdmin/gallery/'.$gallery->image;
                        if (File::exists($path)) {
                            File::delete($path);
                        }
                        $gallery->delete();
                    }
                }
                $record->forceDelete();
            }
        }else{
            Log::info('No record found : Load accessorial temps destroy');
        }
        Log::info('Cron End : Load accessorial temps destroy ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
    }

    public function offerExpire(Request $request)
    {
        Log::info('Cron Start : Offer expire ( '.Carbon::now()->format('Y-m-d').' )');
        $offers     = Offer::whereDate('created_at', Carbon::now()->subDays(14))->where('favorite', '!=', 'No')->get();
        if($offers->isNotEmpty()){
            Log::info('Total Offer expire : ( '.$offers->count().' )');
            foreach ($offers as $key => $offer) {
                $offers->status     = 4;
                $offers->status_at  = Carbon::now();
                $offers->save();
            }
        }else{
            Log::info('No record found : Offer expire');
        }
        Log::info('Cron End : Offer expire ( '.Carbon::now()->format('Y-m-d').' )');
    }

    public function realTimeTruckLocationUpdate(Request $request)
    {
        Log::info('Cron Start : Real time truck location update ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
        $query      = Truck::query();
        $query->where(function ($q) {
            $q->whereNotNull('samsara_id')->whereNotNull('vin_number');
        });
        // $query->skip(15)->take(10);
        $results    = $query->get();
        if($results->isNotEmpty()){
            foreach( $results as $key => $truck ) {
                $getAPIKey                          = IntegrationSetting::where('company_id', $truck->company_id)->where('carrier_id', $truck->carrier_id)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_vehicles', '1')->first();
                if(!is_null($getAPIKey)){
                    try {
                        $url                        = 'https://api.samsara.com/fleet/vehicles/locations?vehicleIds='.$truck->samsara_id;
                        $client                     = new Client();
                        $response                   = $client->request('GET', $url, [
                            'headers'               => [
                                'Accept'            => 'application/json',
                                'Authorization'     => 'Bearer '.$getAPIKey->api_key,
                            ],
                        ]);
                        // Check if response status code is OK (200)
                        if ($response->getStatusCode() == 200) {
                            $body                   = json_decode($response->getBody());
                            // Do something with the response body
                            if(isset($body)){
                                $data               = isset($body->data[0])?$body->data[0]:null;
                                if(isset($data)){
                                    if(isset($data->location)){
                                        try {
                                            $location                   = $data->location;

                                            $array                      = [
                                                'Coords'                => $location->longitude.','.$location->latitude,
                                                'matchNamedRoadsOnly'   => 'true',
                                                'maxCleanupMiles'       => '20',
                                                'region'                => 'NA',
                                                'dataset'               => 'Current',
                                                'vehicle'               => 'Truck',
                                                'timestamp'             => Carbon::now(),
                                                'includeTrimblePlaceIds'=> 'true'
                                            ];
                                            $param                      = array();
                                            foreach($array as $k => $v){
                                                $param[]                = $k . '=' . urlencode($v);
                                            }
                                            $param                      = implode('&', $param);

                                            $trimble_url                = 'https://pcmiler.alk.com/apis/rest/v1.0/Service.svc/locations/reverse?'.$param;
                                            $trimble_response           = $client->request('GET', $trimble_url, [
                                                'headers'               => [
                                                    'Accept'            => 'application/json',
                                                    'Content-Type'      => 'application/json',
                                                    'Authorization'     => $truck->getCompany->trimble_api_key, //'83FF948F08EB3C4BB97E3AED42BEF772',
                                                ],
                                            ]);
                                            // Check if response status code is OK (200)
                                            if ($trimble_response->getStatusCode() == 200) {
                                                $trimble_body           = json_decode($trimble_response->getBody());
                                                // Do something with the response body
                                                if(isset($trimble_body->Address)){
                                                    $truck->last_location_name          = null; // $location->reverseGeo->formattedLocation;
                                                    $truck->last_location_address       = $trimble_body->Address->StreetAddress;
                                                    $truck->last_location_address_2     = null;
                                                    $truck->last_location_city          = $trimble_body->Address->City;
                                                    $truck->last_location_state         = $trimble_body->Address->State;
                                                    $truck->last_location_zip           = $trimble_body->Address->Zip;
                                                }
                                                if(isset($trimble_body->Coords)){
                                                    $truck->last_location_latitude      = $trimble_body->Coords->Lat;
                                                    $truck->last_location_longitude     = $trimble_body->Coords->Lon;
                                                }
                                                $last_location_timezone                 = null;
                                                $last_location_timezone_offset          = null;
                                                if(isset($trimble_body->TimeZone)){
                                                    $last_location_timezone             = timezone_name_from_abbr($trimble_body->TimeZone);
                                                    $last_location_timezone_offset      = Carbon::createFromTimestamp(time(), $last_location_timezone)->getOffsetString();
                                                    $truck->last_location_timezone_abbr = $trimble_body->TimeZone;
                                                }
                                                $truck->last_location_timezone          = $last_location_timezone;
                                                $truck->last_location_timezone_offset   = $last_location_timezone_offset;
                                                $truck->last_location_address_verified  = 1;
                                                $truck->last_location_notes             = null;
                                                $truck->last_location_load_status       = '0';
                                                $truck->updated_at                      = Carbon::now();
                                                if($truck->save()){
                                                    Log::info('Truck vin number ('.$truck->vin_number.') : success');
                                                }
                                            }
                                        } catch (ClientException $e) {
                                            // Handle client-side errors (e.g. 4xx status codes)
                                            $response                   = $e->getResponse();
                                            if ($response) {
                                                $statusCode             = $response->getStatusCode();
                                                $body                   = $response->getBody()->getContents();
                                                Log::error('Truck vin number ('.$truck->vin_number.') : Trimble response '.$body);
                                            } else {
                                                $errorMessage           = $e->getMessage();
                                                Log::error('Truck vin number ('.$truck->vin_number.') : Trimble response '.$errorMessage);
                                            }
                                        } catch (ServerException $e) {
                                            // Handle server-side errors (e.g. 5xx status codes)
                                            $response                   = $e->getResponse();
                                            if ($response) {
                                                $statusCode             = $response->getStatusCode();
                                                $body                   = $response->getBody()->getContents();
                                                Log::error('Truck vin number ('.$truck->vin_number.') : Trimble response '.$body);
                                            } else {
                                                $errorMessage           = $e->getMessage();
                                                Log::error('Truck vin number ('.$truck->vin_number.') : Trimble response '.$errorMessage);
                                            }
                                        } catch (\Exception $e) {
                                            // Handle other exceptions
                                            $errorMessage               = $e->getMessage();
                                            Log::error('Truck vin number ('.$truck->vin_number.') : Trimble response '.$errorMessage);
                                        }
                                    }else{
                                        Log::error('Truck vin number ('.$truck->vin_number.') : Samsara response location not set.');
                                    }
                                }else{
                                    Log::error('Truck vin number ('.$truck->vin_number.') : Samsara response data not set.');
                                }
                            }else{
                                Log::error('Truck vin number ('.$truck->vin_number.') : Samsara response body not set.');
                            }
                        }
                    } catch (ClientException $e) {
                        // Handle client-side errors (e.g. 4xx status codes)
                        $response               = $e->getResponse();
                        if ($response) {
                            $statusCode         = $response->getStatusCode();
                            $body               = $response->getBody()->getContents();
                            Log::error('Truck vin number ('.$truck->vin_number.') : Samsara response '.$body);
                        } else {
                            $errorMessage       = $e->getMessage();
                            Log::error('Truck vin number ('.$truck->vin_number.') : Samsara response '.$errorMessage);
                        }
                    } catch (ServerException $e) {
                        // Handle server-side errors (e.g. 5xx status codes)
                        $response               = $e->getResponse();
                        if ($response) {
                            $statusCode         = $response->getStatusCode();
                            $body               = $response->getBody()->getContents();
                            Log::error('Truck vin number ('.$truck->vin_number.') : Samsara response '.$body);
                        } else {
                            $errorMessage       = $e->getMessage();
                            Log::error('Truck vin number ('.$truck->vin_number.') : Samsara response '.$errorMessage);
                        }
                    } catch (\Exception $e) {
                        // Handle other exceptions
                        $errorMessage           = $e->getMessage();
                        Log::error('Truck vin number ('.$truck->vin_number.') : Samsara response '.$errorMessage);
                    }
                }
            }
        }

        $query      = Truck::query();
        $query->where(function ($q) {
            $q->whereNull('samsara_id')->whereNotNull('vin_number')->whereNotNull('unit');
        });
        $results    = $query->get();
        if($results->isNotEmpty()){
            foreach( $results as $key => $truck ) {
                $load_detail = LoadDetail::select(
                    'id','load_id','load_number','truck_number',
                    'occupied_pickup_day',
                    'shipping_time_range','shipper_date_time','shipper_date_time_end','actual_shipper_date',
                    'shipper_location','shipper_address_one','shipper_address_two','shipper_state','shipper_city','shipper_zip','shipper_note','shipper_latitude','shipper_longitude','shipper_timezone','shipper_timezone_offset','shipper_timezone_abbr','shipper_address_verified',
                    'shipper_is_arrived','shipper_arrived_at',
                    'shipper_is_leaves','shipper_leaves_at',
                    'delivery_time_range','delivery_date_time','delivery_date_time_end','actual_delivery_date',
                    'delivery_location','delivery_address_one','delivery_address_two','delivery_state','delivery_city','delivery_zip','delivery_note','delivery_latitude','delivery_longitude','delivery_timezone','delivery_timezone_offset','delivery_timezone_abbr','delivery_address_verified',
                    'delivery_is_arrived','delivery_arrived_at','delivery_is_leaves','delivery_leaves_at'
                );
                $load_detail->where(function($query) use ($truck){
                    $query->where('truck_number',$truck->unit)
                        ->where('shipper_is_arrived', 1);
                });
                $load_detail->orderBy('actual_shipper_date', 'DESC')
                    ->orderBy('actual_delivery_date', 'DESC');
                $load_detail        = $load_detail->first();
                if(!is_null($load_detail)){
                    if($load_detail->delivery_is_arrived==1){
                        $truck->last_location_name                  = $load_detail->delivery_location;
                        $truck->last_location_address               = $load_detail->delivery_address_one;
                        $truck->last_location_address_2             = $load_detail->delivery_address_two;
                        $truck->last_location_city                  = $load_detail->delivery_city;
                        $truck->last_location_state                 = $load_detail->delivery_state;
                        $truck->last_location_zip                   = $load_detail->delivery_zip;

                        $truck->last_location_latitude              = $load_detail->delivery_latitude;
                        $truck->last_location_longitude             = $load_detail->delivery_longitude;

                        $truck->last_location_timezone              = $load_detail->delivery_timezone;
                        $truck->last_location_timezone_offset       = $load_detail->delivery_timezone_offset;
                        $truck->last_location_timezone_abbr         = $load_detail->delivery_timezone_abbr;
                        $truck->last_location_address_verified      = $load_detail->delivery_address_verified;

                        $truck->last_location_notes                 = $load_detail->delivery_note;

                        $truck->last_location_load_status           = '0';
                        $truck->updated_at                          = Carbon::now();
                        if($truck->save()){
                            Log::info('Truck vin number ('.$truck->vin_number.') : success');
                        }
                    }else{
                        $load_stop  = LoadStop::where('load_id', $load_detail->load_id)->where('is_arrived', 1)->orderBy('date_time', 'DESC')->first();
                        if(!is_null($load_stop)){
                            $truck->last_location_name              = $load_stop->location;
                            $truck->last_location_address           = $load_stop->address_one;
                            $truck->last_location_address_2         = $load_stop->address_two;
                            $truck->last_location_city              = $load_stop->city;
                            $truck->last_location_state             = $load_stop->state;
                            $truck->last_location_zip               = $load_stop->zip;

                            $truck->last_location_latitude          = $load_stop->latitude;
                            $truck->last_location_longitude         = $load_stop->longitude;

                            $truck->last_location_timezone          = $load_stop->timezone;
                            $truck->last_location_timezone_offset   = $load_stop->timezone_offset;
                            $truck->last_location_timezone_abbr     = $load_stop->timezone_abbr;
                            $truck->last_location_address_verified  = $load_stop->address_verified;

                            $truck->last_location_notes             = $load_stop->stop_note;

                            $truck->last_location_load_status       = '0';
                            $truck->updated_at                      = Carbon::now();
                            if($truck->save()){
                                Log::info('Truck vin number ('.$truck->vin_number.') : success');
                            }
                        }else{
                            $truck->last_location_name              = $load_detail->shipper_location;
                            $truck->last_location_address           = $load_detail->shipper_address_one;
                            $truck->last_location_address_2         = $load_detail->shipper_address_two;
                            $truck->last_location_city              = $load_detail->shipper_city;
                            $truck->last_location_state             = $load_detail->shipper_state;
                            $truck->last_location_zip               = $load_detail->shipper_zip;

                            $truck->last_location_latitude          = $load_detail->shipper_latitude;
                            $truck->last_location_longitude         = $load_detail->shipper_longitude;

                            $truck->last_location_timezone          = $load_detail->shipper_timezone;
                            $truck->last_location_timezone_offset   = $load_detail->shipper_timezone_offset;
                            $truck->last_location_timezone_abbr     = $load_detail->shipper_timezone_abbr;
                            $truck->last_location_address_verified  = $load_detail->shipper_address_verified;

                            $truck->last_location_notes             = $load_detail->shipper_note;

                            $truck->last_location_load_status       = '0';
                            $truck->updated_at                      = Carbon::now();
                            if($truck->save()){
                                Log::info('Truck vin number ('.$truck->vin_number.') : success');
                            }
                        }
                    }
                }
            }
        }
        Log::info('Cron End : Real time truck location update ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
    }

    public function appointmentReminder(Request $request)
    {
        Log::info('Cron Start : Appointment reminder ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
        $currentDateTime            = Carbon::now(); // 2023-06-15 14:36:00
        $startDateTime              = Carbon::now()->addHours(4); // 2023-06-15 18:36:00
        $endDateTime                = Carbon::now()->addHours(4)->addMinutes(10); // 2023-06-15 18:46:00

        $loadAppointments           = LoadAppointment::query()
            ->where('reminder_type', '4 Hours')
            ->where('reminder_status', 1)
            ->where('status', '!=', 1)
            ->whereBetween(DB::raw("CONCAT(request_date, ' ', request_time)"), [$startDateTime, $endDateTime])
            ->whereDoesntHave('logs', function ($query) {
                $query->whereIn('review_user_type', ['Broker', 'Broker Contact']);
                // $query->orWhere('review_status', 1);
            })
            ->get();
        if($loadAppointments->isNotEmpty()){
            foreach ($loadAppointments as $key => $loadAppointment) {
                Log::info('4 Hours Appointment '.$loadAppointment->id.' : Started at ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
                appointmentRequestSend($loadAppointment);
                Log::info('4 Hours Appointment '.$loadAppointment->id.' : Ended at ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
            }
        }

        $currentDateTime            = Carbon::now(); // 2023-06-15 09:26:24
        $startDateTime              = Carbon::now()->addDay(); // 2023-06-16 09:26:24
        $endDateTime                = Carbon::now()->addDay()->addMinutes(10); // 2023-06-16 09:36:24

        $loadAppointments           = LoadAppointment::query()
            ->where('reminder_type', '24 Hours')
            ->where('reminder_status', 1)
            ->where('status', '!=', 1)
            ->whereBetween(DB::raw("CONCAT(request_date, ' ', request_time)"), [$startDateTime, $endDateTime])
            ->whereDoesntHave('logs', function ($query) {
                $query->whereIn('review_user_type', ['Broker', 'Broker Contact']);
                // $query->orWhere('review_status', 1);
            })
            ->get();
        if($loadAppointments->isNotEmpty()){
            foreach ($loadAppointments as $key => $loadAppointment) {
                Log::info('24 Hours Appointment '.$loadAppointment->id.' : Started at ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
                appointmentRequestSend($loadAppointment);
                Log::info('24 Hours Appointment '.$loadAppointment->id.' : Ended at ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
            }
        }

        for ($i = 1; $i <= 24; $i++) {
            // Your logic here
            // Use $i as the current iteration value
            $currentDateTime        = Carbon::now(); // 2023-06-15 14:36:00
            $startDateTime          = Carbon::now()->addHours($i); // 2023-06-15 18:36:00
            $endDateTime            = Carbon::now()->addHours($i)->addMinutes(10); // 2023-06-15 18:46:00

            $loadAppointments       = LoadAppointment::query()
                ->where('reminder_type', 'Custom')
                ->where('reminder_repeat_every', 0)
                ->where('reminder_custom_hours', $i)
                ->where('reminder_status', 1)
                ->where('status', '!=', 1)
                ->whereBetween(DB::raw("CONCAT(request_date, ' ', request_time)"), [$startDateTime, $endDateTime])
                ->whereDoesntHave('logs', function ($query) {
                    $query->whereIn('review_user_type', ['Broker', 'Broker Contact']);
                    // $query->orWhere('review_status', 1);
                })
                ->get();
            if($loadAppointments->isNotEmpty()){
                foreach ($loadAppointments as $key => $loadAppointment) {
                    Log::info('Custom Appointment '.$loadAppointment->id.' : Started at ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
                    appointmentRequestSend($loadAppointment);
                    Log::info('Custom Appointment '.$loadAppointment->id.' : Ended at ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
                }
            }

            $currentDateTime        = Carbon::now(); // 2023-06-15 11:38:52
            $startDateTime          = Carbon::now()->subHours($i); // 2023-06-15 10:38:52
            $endDateTime            = Carbon::now()->subHours($i)->addMinutes(10); // 2023-06-15 10:48:52

            $loadAppointments       = LoadAppointment::query()
                ->where('reminder_type', 'Custom')
                ->where('reminder_repeat_every', 1)
                ->where('reminder_custom_hours', $i)
                ->where(function ($query) use ($startDateTime, $endDateTime) {
                    $query->whereBetween('reminder_repeat_date_time', [$startDateTime, $endDateTime])
                        ->orWhereNull('reminder_repeat_date_time');
                })
                ->where('reminder_status', 1)
                ->where('status', '!=', 1)
                ->whereDoesntHave('logs', function ($query) {
                    $query->whereIn('review_user_type', ['Broker', 'Broker Contact']);
                    // $query->orWhere('review_status', 1);
                })
                ->get();
            if($loadAppointments->isNotEmpty()){
                foreach ($loadAppointments as $key => $loadAppointment) {
                    Log::info('Repeat Every Appointment '.$loadAppointment->id.' : Started at ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
                    if(appointmentRequestSend($loadAppointment)){
                        $loadAppointment->reminder_repeat_date_time = $currentDateTime;
                        $loadAppointment->save();
                    }
                    Log::info('Repeat Every Appointment '.$loadAppointment->id.' : Ended at ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
                }
            }
        }
        Log::info('Cron End : Appointment reminder ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
    }

    public function vehicleStats(Request $request)
    {
        Log::info('Cron Start : Vehicle stats insert ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
        $query      = Truck::whereNotNull('samsara_id');
        $results    = $query->get();
        if($results->isNotEmpty()){
            foreach( $results as $key => $truck ) {
                $getAPIKey      = IntegrationSetting::where('company_id', $truck->company_id)->where('carrier_id', $truck->carrier_id)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_vehicles', '1')->first();
                if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                    try {
                        $url                    = 'https://api.samsara.com/fleet/vehicles/stats?vehicleIds='.$truck->samsara_id.'&types=gps,obdOdometerMeters';
                        $client                 = new \GuzzleHttp\Client();
                        $response               = $client->request('GET', $url, [
                            'headers'           => [
                                'Accept'        => 'application/json',
                                'Authorization' => 'Bearer '.$getAPIKey->api_key,
                            ],
                        ]);
                        // Check if response status code is OK (200)
                        if ($response->getStatusCode() == 200) {
                            $body               = json_decode($response->getBody());
                            // Do something with the response body
                            if(isset($body)){
                                $data           = isset($body->data[0])?$body->data[0]:null;
                                if(isset($data)){
                                    $item       = VehicleHistoricalStats::create([
                                        'truck_id'              => $truck->id,
                                        'obd_odometer_meters'   => $data->obdOdometerMeters->value ?? null,
                                        'obd_odometer_miles'    => $data->obdOdometerMeters->value ? round($data->obdOdometerMeters->value * 0.000621371) : null,
                                        'speed_mph'             => $data->gps->speedMilesPerHour ?? null,
                                        'latitude'              => $data->gps->latitude ?? null,
                                        'longitude'             => $data->gps->longitude ?? null,
                                        'location'              => $data->gps->reverseGeo->formattedLocation ?? null,
                                        'last_updated_at'       => $data->gps->time ? Carbon::parse($data->gps->time)->format('Y-m-d H:i:s') : null,
                                        'data'                  => $data
                                    ]);
                                }else{
                                    Log::error('Truck vin number ('.$truck->vin_number.') : Samsara response data not set.');
                                }
                            }
                        }
                    } catch (\GuzzleHttp\Exception\ClientException $e) {
                        // Handle client-side errors (e.g. 4xx status codes)
                        $response               = $e->getResponse();
                        if ($response) {
                            $statusCode         = $response->getStatusCode();
                            $body               = $response->getBody()->getContents();
                            Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : client exception '.$body);
                        } else {
                            $errorMessage       = $e->getMessage();
                            Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : client exception '.$errorMessage);
                        }
                    } catch (\GuzzleHttp\Exception\ServerException $e) {
                        // Handle server-side errors (e.g. 5xx status codes)
                        $response               = $e->getResponse();
                        if ($response) {
                            $statusCode         = $response->getStatusCode();
                            $body               = $response->getBody()->getContents();
                            Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : server exception '.$body);
                        } else {
                            $errorMessage       = $e->getMessage();
                            Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : server exception '.$errorMessage);
                        }
                    } catch (\Exception $e) {
                        // Handle other exceptions
                        $errorMessage           = $e->getMessage();
                        Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : other exception '.$errorMessage);
                    }
                }
            }
        }
        Log::info('Cron end : Vehicle stats insert ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
    }

    public function updatePositions($load_number='DTL-1002', $startTime='2023-09-28 09:30:00', $currentTime='2023-09-28 23:00:00')
    {
        Log::info('Cron Start : Update the position of driver on Trip management Trip View map ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
        $query      = LoadDetail::select('id', 'load_id', 'load_number', 'load_type', 'intransit_date_time', 'is_on_hold', 'load_status', 'driver_id', 'truck_number', 'load_trip_id');
        $query->where('load_number', $load_number);
        $query->where(function($q) {
            $q->where('is_disable', 0)
            ->whereIn('load_type', [0, 6])
            ->where('load_status', 4)
            ->where('is_on_hold', '!=', 2)
            ->where(function($q) {
                $q->whereNotNull('driver_id')
                    ->where('driver_id', '!=', 0);
            })
            ->whereNotNull('truck_number')
            ->whereNotNull('intransit_date_time')
            ->whereNotNull('load_trip_id');
        });
        $results    = $query->get();
        if($results->isNotEmpty()){
            foreach( $results as $key => $load_detail ) {
                $trimble_api_key                                    = $load_detail->loads->company->trimble_api_key ?? null;
                if(!is_null($trimble_api_key)){
                    $truck                                          = $load_detail->truck ?? null;
                    if(!is_null($truck)){
                        $vehicleIds                                 = $truck->samsara_id ?? null;
                        if(!is_null($vehicleIds)){
                            $getGPSCoords                           = $this->getGPSCoords($load_detail->load_trip_id, $trimble_api_key);
                            if(!empty($getGPSCoords)){
                                $gpsLastValue                       = end($getGPSCoords);    // Get the last element
                                if ($gpsLastValue !== false) {
                                    // The array is not empty, and $gpsLastValue contains the last element.
                                    // Log::info(json_encode($gpsLastValue));
                                    $startTime                      = Carbon::parse($gpsLastValue->deviceDT)->addSeconds(1)->format('Y-m-d H:i:s');
                                }
                            }
                            // Parse the date in 'Y-m-d H:i:s' format using Carbon
                            $startTime                              = Carbon::parse($startTime);
                            // Format the date in RFC 3339 format ('Y-m-d\TH:i:s\Z')
                            $startTime                              = $startTime->format('Y-m-d\TH:i:s\Z');

                            $currentTime                            = Carbon::parse($currentTime);
                            // Format the current time in RFC 3339 format ('Y-m-d\TH:i:s\Z')
                            $currentTime                            = $currentTime->format('Y-m-d\TH:i:s\Z');

                            //
                            $getAPIKey                              = IntegrationSetting::where('company_id', $truck->company_id)->where('carrier_id', $truck->carrier_id)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_vehicles', '1')->first();
                            if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                                $params                             = [
                                    'startTime'                     => $startTime,
                                    'endTime'                       => $currentTime,
                                    'vehicleIds'                    => $vehicleIds,
                                    'types'                         => 'gps'
                                ];

                                $historicalStats                    = $this->historicalStats($getAPIKey->api_key, $params);
                                if(!empty($historicalStats)){
                                    Log::info('Vehicle Id : ( '.$truck->unit.' = '.$vehicleIds.' )');
                                    $firstElement                   = reset($historicalStats); // Get the first element
                                    if ($firstElement !== false) {
                                        Log::info('Start time : ( '.Carbon::parse($startTime)->format('Y-m-d H:i:s').' = '.$startTime.' )');
                                        // Log::info(json_encode($firstElement));
                                    }
                                    $lastElement                    = end($historicalStats);    // Get the last element
                                    if ($lastElement !== false) {
                                        Log::info('Current time : ( '.Carbon::parse($currentTime)->format('Y-m-d H:i:s').' = '.$currentTime.' )');
                                        // Log::info(json_encode($lastElement));
                                    }
                                    // $historicalStats             = array_slice($historicalStats, 0, 2); // Get the first 2 elements
                                    // Log::info(json_encode($historicalStats));

                                    // Trip Management’s Update GPS Positions
                                    $array                          = [];
                                    $array["tripId"]                = $load_detail->load_trip_id;

                                    // Create a new array with the fixed values using array_map
                                    $array["locations"]             = array_map(function ($item) {
                                        return [
                                            "coords"                => [
                                                "lat"               => $item->latitude,
                                                "lon"               => $item->longitude
                                            ],
                                            "deviceDT"              => Carbon::parse($item->time)->toIso8601String()

                                        ];
                                    }, $historicalStats);

                                    // Update GPS Positions Api
                                    $gpsPosition                    = $this->gpsPosition($array, $trimble_api_key);
                                    if(isset($gpsPosition->success) && $gpsPosition->success == true){
                                        Log::info('Update GPS Position : ( Load Number => '.$load_detail->load_number.', Trip Id => '.$load_detail->load_trip_id.' )');
                                    }
                                    // Trip Management’s Update GPS Positions
                                }
                            }
                            //
                        }
                    }
                }
            }
        }
        Log::info('Cron End : Update the position of driver on Trip management Trip View map ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
    }

    public function updateDefaultFuelSurcharge(Request $request)
    {
        Log::info('Manual Cron Start : Update Default Fuel Surcharge for Diesel & Gas ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
        $templates      = FuelSurchargeTemplate::whereIn('customer', ['FSG', 'FSD'])->orderBy('id')->get();
        foreach ($templates as $template){
            $data       = saveDefaultFuelSurcharges($template);
            if(!empty($data)){
                Log::info($template->customer_file_name.': '.$data['value'].' (Last Updated: '.$data['last_updated'].')');
            }
        }
        Log::info('Manual Cron End : Update Default Fuel Surcharge for Diesel & Gas ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
    }

    public function updateLoadFuelSurcharge(Request $request)
    {
        // Log::info('Cron Start : Update Load Fuel Surcharge for Diesel & Gas ( '.Carbon::now()->format('Y-m-d H:i:s').' )');

        $startDateTime                                                  = Carbon::now();
        $endDateTime                                                    = Carbon::now()->addMinutes(90);

        $query                                                          = LoadDetail::query()
            ->where('is_disable', 0)
            ->whereIn('load_type', [0, 6]) // 0 = OCCUPIED, 6 = TONU
            ->whereIn('load_status',[0, 2, 3, 4, 11]) // 0 = Pending,  2 = Preplanned, 3 = Confirmed, 4 = In Transit, 11 = Send To Driver
            ->where(function($q) {
                $q->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0, 1]);
            })
            // ->whereBetween('shipper_date_time', [$startDateTime, $endDateTime])
            // ->where('fuel_surcharge_type', 1) // 1 = Automatic
            ->whereHas('truck', function ($query) {
                $query->whereIn('fuel_type', [0, 1]);
            });
        $load_details                                                   = $query->get();
        if($load_details->isNotEmpty()){
            // Log::info('Total Load : ( '.$load_details->count().' )');

            foreach ($load_details as $key => $load_detail) {
                $oldData                                                = $load_detail->toArray();

                $company_id                                             = $load_detail->loads->company_id;
                $load_number                                            = $load_detail->load_number ?? '';
                $customer_name                                          = $load_detail->broker->name ?? '';
                $truck_fuel_type                                        = $load_detail->truck->fuel_type == 0 ? 'Desiel' : 'Regular Gas';

                $fuel_surcharge_type                                    = $load_detail->fuel_surcharge_type;
                $loaded_miles                                           = $load_detail->loaded_miles ?? '';
                $agreed_upon_rate                                       = $load_detail->agreed_upon_rate ?? '';
                $fuel_surcharge_cost_per_mile                           = $load_detail->fuel_surcharge_cost_per_mile ?? '';
                $fuel_surcharge                                         = $load_detail->fuel_surcharge ?? '';
                $line_haul                                              = $load_detail->line_haul ?? '';

                // Get the original value of a specific attribute without mutators
                $shipper_date_time                                      = $load_detail->getRawOriginal('shipper_date_time');
                $date                                                   = Carbon::parse($shipper_date_time);
                $shipper_date                                           = $date->format('Y-m-d');
                $startOfWeek                                            = $date->startOfWeek(); // Shipper start of week

                // Log::info('Load Number = '.$load_detail->load_number.', Customer Name = '.$customer_name.', Truck Fuel Type = '.$truck_fuel_type.', Fuel Surcharge Type = Automatic, Shipper Date = '.$shipper_date.', Loaded Miles = '.$loaded_miles.', Agreed Upon Rate = '.$agreed_upon_rate.', Fuel Surcharge Cost Per Mile = '.$fuel_surcharge_cost_per_mile.', Fuel Surcharge = '.$fuel_surcharge.', Line Haul = '.$line_haul);

                $template_fuel_surcharge                                = null;
                $template_fuel_surcharge_cost                           = null;

                if($load_detail->truck->fuel_type == 0){
                    // Desiel
                    $templateQuery                                      = FuelSurchargeTemplate::query()
                        ->where('company_id', $company_id)
                        ->where('customer', $load_detail->brokerage_firm)
                        ->where('fuel_type', 'EPD2D') // EPD2D = U.S. On-Highway Diesel, EPMR = U.S. Regular Gasoline Price
                        ->where('set_as_active', '1')
                        ->where(function ($q) use ($shipper_date) {
                            $q->whereDate('start_date', '<=', $shipper_date)
                                ->whereDate('end_date', '>=', $shipper_date);
                        }); 
                    $template                                           = $templateQuery->first();
                    if(!is_null($template)){
                        // Format the start of the week date as you need
                        $startOfWeekFormattedDate                       = $startOfWeek->format('Y-m-d');
                        // Retrive fuel surcharge data
                        $getFuelSurchargeArray                          = getTemplateFuelSurchargeArray($template, $startOfWeekFormattedDate);
                        if(!empty($getFuelSurchargeArray) && isset($getFuelSurchargeArray['fuel_surcharge']) && $getFuelSurchargeArray['fuel_surcharge']!=''){
                            $template_fuel_surcharge                    = is_numeric($getFuelSurchargeArray['fuel_surcharge']) ? (float)$getFuelSurchargeArray['fuel_surcharge'] : null;
                            // Log::info('Customer '.$load_detail->brokerage_firm.' => EPD2D = U.S. On-Highway Diesel '.$template_fuel_surcharge);
                        }
                    }else{
                        $templateQuery                                  = FuelSurchargeTemplate::query()
                            ->where('company_id', $company_id)
                            ->where('customer', 'FSD')
                            ->where('fuel_type', 'EPD2D') // EPD2D = U.S. On-Highway Diesel, EPMR = U.S. Regular Gasoline Price
                            ->where('set_as_active', '1'); 
                        $template                                       = $templateQuery->first();
                        if(!is_null($template)){
                            // Retrive fuel surcharge data
                            $getFuelSurchargeArray                      = getTemplateFuelSurchargeArray($template);
                            if(!empty($getFuelSurchargeArray) && isset($getFuelSurchargeArray['fuel_surcharge']) && $getFuelSurchargeArray['fuel_surcharge']!=''){
                                $template_fuel_surcharge                = is_numeric($getFuelSurchargeArray['fuel_surcharge']) ? (float)$getFuelSurchargeArray['fuel_surcharge'] : null;
                                // Log::info('Default EPD2D = U.S. On-Highway Diesel '.$template_fuel_surcharge);
                            }
                        }
                    }
                } elseif ($load_detail->truck->fuel_type == 1) {
                    // Regular Gas
                    $templateQuery                                      = FuelSurchargeTemplate::query()
                        ->where('company_id', $company_id)
                        ->where('customer', $load_detail->brokerage_firm)
                        ->where('fuel_type', 'EPMR') // EPD2D = U.S. On-Highway Diesel, EPMR = U.S. Regular Gasoline Price
                        ->where('set_as_active', '1')
                        ->where(function ($q) use ($shipper_date) {
                            $q->whereDate('start_date', '<=', $shipper_date)
                                ->whereDate('end_date', '>=', $shipper_date);
                        }); 
                    $template                                           = $templateQuery->first();
                    if(!is_null($template)){
                        // Format the start of the week date as you need
                        $startOfWeekFormattedDate                       = $startOfWeek->format('Y-m-d');
                        // Retrive fuel surcharge data
                        $getFuelSurchargeArray                          = getTemplateFuelSurchargeArray($template, $startOfWeekFormattedDate);
                        if(!empty($getFuelSurchargeArray) && isset($getFuelSurchargeArray['fuel_surcharge']) && $getFuelSurchargeArray['fuel_surcharge']!=''){
                            $template_fuel_surcharge                    = is_numeric($getFuelSurchargeArray['fuel_surcharge']) ? (float)$getFuelSurchargeArray['fuel_surcharge'] : null;
                            // Log::info('Customer '.$load_detail->brokerage_firm.' => EPD2D = U.S. On-Highway Diesel '.$template_fuel_surcharge);
                        }
                    }else{
                        $templateQuery                                  = FuelSurchargeTemplate::query()
                            ->where('company_id', $company_id)
                            ->where('customer', 'FSG')
                            ->where('fuel_type', 'EPMR') // EPD2D = U.S. On-Highway Diesel, EPMR = U.S. Regular Gasoline Price
                            ->where('set_as_active', '1'); 
                        $template                                       = $templateQuery->first();
                        if(!is_null($template)){
                            // Retrive fuel surcharge data
                            $getFuelSurchargeArray                      = getTemplateFuelSurchargeArray($template);
                            if(!empty($getFuelSurchargeArray) && isset($getFuelSurchargeArray['fuel_surcharge']) && $getFuelSurchargeArray['fuel_surcharge']!=''){
                                $template_fuel_surcharge                = is_numeric($getFuelSurchargeArray['fuel_surcharge']) ? (float)$getFuelSurchargeArray['fuel_surcharge'] : null;
                                // Log::info('Default EPD2D = U.S. On-Highway Diesel '.$template_fuel_surcharge);
                            }
                        }
                    }
                }

                if(isset($template)){
                    if($template->rate_type == '1'){
                        // 0: Per Mile, 1: Percentage, 2: Both
                        if($agreed_upon_rate !='' && isset($template_fuel_surcharge) && $template_fuel_surcharge != ''){
                            // Calculate the fuel surcharge
                            $template_fuel_surcharge_cost               = $agreed_upon_rate * ($template_fuel_surcharge / 100);
                            $template_fuel_surcharge_cost               = (float) number_format((float)$template_fuel_surcharge_cost, 2, '.', ''); // floor($template_fuel_surcharge_cost * 100) / 100;
                            $load_detail->fuel_surcharge_rate_type      = 1;
                        }
                    }else{
                        // 0: Per Mile, 1: Percentage, 2: Both
                        if($loaded_miles !='' && isset($template_fuel_surcharge) && $template_fuel_surcharge != ''){
                            // Calculate the fuel surcharge
                            $template_fuel_surcharge_cost               = $loaded_miles * $template_fuel_surcharge; // 541.668545;
                            $template_fuel_surcharge_cost               = (float) number_format((float)$template_fuel_surcharge_cost, 2, '.', ''); // floor($template_fuel_surcharge_cost * 100) / 100;
                            $load_detail->fuel_surcharge_rate_type      = 0;
                        }  
                    }
                }

                $load_detail->fuel_surcharge_cost_per_mile              = isset($template_fuel_surcharge) ? $template_fuel_surcharge : null;
                $load_detail->fuel_surcharge                            = isset($template_fuel_surcharge_cost) ? $template_fuel_surcharge_cost : null;
                if(isset($agreed_upon_rate) && isset($load_detail->fuel_surcharge)){
                    $load_detail->line_haul                             = $agreed_upon_rate - $load_detail->fuel_surcharge;
                }else{
                    $load_detail->line_haul                             = null;
                }
                if($load_detail->save()){
                    // Log::info('Fuel surcharge cost per mile updated');    
                }
            }
        }else{
            // Log::info('No record found');
        }

        // Log::info('Cron End : Update Load Fuel Surcharge for Diesel & Gas ( '.Carbon::now()->format('Y-m-d H:i:s').' )');
    }
}
