<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;

use App\Http\Requests\CompanyAdmin\CustomerStoreRequest;
use App\Http\Requests\CompanyAdmin\CustomerUpdateRequest;

use App\Models\Customer;
use App\Models\Bookmark;

use Validator;
use DataTables;
use Carbon\Carbon;
use App\Helpers\Common;

class CustomerController extends CompanyAdminController{

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getCompanyAdmin    = auth()->guard('company-admin')->user();
        $getField           = '';

        $isCheck            = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','customers')->first();
        if(!empty($isCheck->id)){
            //$getField       = unserialize($isCheck->config);
            $getField       = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_customers_first'));
        }else{
            $getField       = config('settings.table_customers_first');
        }

        if ($request->ajax()) {
            $data           = Customer::isCompany();

            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq = 'search_string_condition_'.$key;
                $sortReq = 'search_string_sort_'.$key;

                if( isset( request()->$conditionReq ) ) {
                    $typeReq = 'search_string_type_'.$key;
                    $type = request()->$typeReq;

                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }

                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                if( isset( request()->$sortReq ) ) {
                    $nameReq = 'search_string_name_'.$key;
                    $sortReq = 'search_string_sort_'.$key;
                    $typeReq = 'search_string_type_'.$key;
                    $tableNameReq = 'search_table_name_'.$key;
                    $colAliasReq = 'search_string_col_alias_'.$key;

                    $origData = request()->$nameReq;
                    $condition_sort = request()->$sortReq;
                    $type = request()->$typeReq;
                    $tableName = isset( request()->$tableNameReq ) ? request()->$tableNameReq : '';
                    $colAlias = isset( request()->$colAliasReq ) ? request()->$colAliasReq : '';

                    if( $type == 'string' ) {
                        if( isset( $condition_sort ) ) {
                            if( $colAlias != '' ) {
                                $data->orderBy( $tableName.'.'.$colAlias, $condition_sort );
                            } else {
                                if( $tableName != '' ) {
                                    $data->orderBy( $tableName.'.'.$origData, $condition_sort );
                                } else {
                                    $data->orderBy( $origData, $condition_sort );
                                }
                            }
                        }
                    }
                }
            }
            $data->select('*');
            /// end
            return Datatables::of($data)
            ->addIndexColumn()
            ->addColumn('action', function($row){
                //$action = '<a href="'.route('company.admin.customer.show', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                $action = '<a href="'.route('company.admin.customer.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                $action .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash destroy" data-action="'.route('company.admin.customer.destroy',$row->id).'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';

                return $action;
            })
            ->rawColumns(['action'])
            ->make(true);
        }
        $tableName = 'settings.table_customers';
        return view('companyAdmin.customer.index', compact('getField','tableName'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return view('companyAdmin.customer.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(CustomerStoreRequest $request){
        $input                          = $request->all();

        $input['company_id']            = auth()->guard('company-admin')->user()->company_id;
        if($request->address_verified==1){
            $timezone                   = null;
            $timezone_offset            = null;
            if(isset($input['timezone_abbr'])){
                $timezone               = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset        = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']          = $timezone;
            $input['timezone_offset']   = $timezone_offset;
        }

        $shipper_files_array            = [];
        $shipper_i                      = 0;
        foreach($request->shipper_files['is_required'] as $key => $sefdata){
            $shipper_files_array[$shipper_i]['is_required']     = $request->shipper_files['is_required'][$key];
            $shipper_files_array[$shipper_i]['name']            = $request->shipper_files['name'][$key];
            $shipper_files_array[$shipper_i]['type']            = $request->shipper_files['type'][$key];
            $shipper_files_array[$shipper_i]['fixed']           = $request->shipper_files['fixed'][$key];
            $shipper_i++;
        }

        $consignee_files_array          = [];
        $consignee_i                    = 0;
        foreach($request->consignee_files['is_required'] as $key => $sefdata){
            $consignee_files_array[$consignee_i]['is_required'] = $request->consignee_files['is_required'][$key];
            $consignee_files_array[$consignee_i]['name']        = $request->consignee_files['name'][$key];
            $consignee_files_array[$consignee_i]['type']        = $request->consignee_files['type'][$key];
            $consignee_files_array[$consignee_i]['fixed']       = $request->consignee_files['fixed'][$key];
            $consignee_i++;
        }

        $input['shipper_files']         = serialize($shipper_files_array);
        $input['consignee_files']       = serialize($consignee_files_array);

        if(isset($input['business_account_no']) &&  $input['business_account_no']  !=""){
            $input['business_account_no'] = base64_encode( $input['business_account_no']);
        }

        Customer::create($input);

        notification('success', $this->crudMessage('create', 'Customer'));

        return redirect()->route('company.admin.customer.index');
    }

    public function storeAjax(Request $request){
        $input      = $request->except('stop_index');
        $validator  = Validator::make($input,[
            "name"          => "required",
            "address"       => "required",
            //'mc_number'   => 'required|unique:customers,mc_number',
            'mc_number'     => [
                function ($attribute, $value, $fail) use ($request) {
                    if(!empty($request->mc_number)){
                        $company_id = auth()->guard('company-admin')->user()->company_id;
                        $customer   = Customer::where('company_id', $company_id)
                            ->where('mc_number', $request->mc_number)
                            ->count();
                        if ($customer > 0) {
                            $fail('The MC Number is already taken.');
                        }
                    }
                }
            ]
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $input['company_id']            = auth()->guard('company-admin')->user()->company_id;
        if($request->address_verified==1){
            $timezone                   = null;
            $timezone_offset            = null;
            if(isset($input['timezone_abbr'])){
                $timezone               = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset        = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']          = $timezone;
            $input['timezone_offset']   = $timezone_offset;
        }
        $customer = Customer::create($input);

        notification('success', $this->crudMessage('create', 'Customer'));
        return response()->json(['success' => "Customer created successfully",'id' => $customer->id]);
    }

    /**
     * Display the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $id         = base64Decode($id);
        $customer   = Customer::findOrFail($id);
        return view('companyAdmin.customer.show', compact('customer'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $id         = base64Decode($id);
        $customer   = Customer::findOrFail($id);
        if(isset( $customer) &&  $customer->business_account_no  !=""){
            $customer->business_account_no = base64_decode( $customer->business_account_no);
        }
        return view('companyAdmin.customer.edit', compact('customer'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(CustomerUpdateRequest $request, Customer $customer){
        $input                          = $request->all();

        if($request->address_verified==1){
            $timezone                   = null;
            $timezone_offset            = null;
            if(isset($input['timezone_abbr'])){
                $timezone               = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset        = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']          = $timezone;
            $input['timezone_offset']   = $timezone_offset;
        }else{
            $input['latitude']          = null;
            $input['longitude']         = null;
            $input['timezone']          = null;
            $input['timezone_offset']   = null;
            $input['timezone_abbr']     = null;
        }

        $shipper_files_array            = [];
        $shipper_i                      = 0;
        foreach($request->shipper_files['is_required'] as $key => $sefdata){
            $shipper_files_array[$shipper_i]['is_required']     = $request->shipper_files['is_required'][$key];
            $shipper_files_array[$shipper_i]['name']            = $request->shipper_files['name'][$key];
            $shipper_files_array[$shipper_i]['type']            = $request->shipper_files['type'][$key];
            $shipper_files_array[$shipper_i]['fixed']           = $request->shipper_files['fixed'][$key];
            $shipper_i++;
        }

        $consignee_files_array          = [];
        $consignee_i                    = 0;
        foreach($request->consignee_files['is_required'] as $key => $sefdata){
            $consignee_files_array[$consignee_i]['is_required'] = $request->consignee_files['is_required'][$key];
            $consignee_files_array[$consignee_i]['name']        = $request->consignee_files['name'][$key];
            $consignee_files_array[$consignee_i]['type']        = $request->consignee_files['type'][$key];
            $consignee_files_array[$consignee_i]['fixed']       = $request->consignee_files['fixed'][$key];
            $consignee_i++;
        }

        $input['shipper_files']         = serialize($shipper_files_array);
        $input['consignee_files']       = serialize($consignee_files_array);

        if(isset($input['business_account_no']) &&  $input['business_account_no']  !=""){
            $input['business_account_no'] = base64_encode( $input['business_account_no']);
        }

        $customer->update($input);

        notification('success', $this->crudMessage('update', 'Customer'));

        return redirect()->route('company.admin.customer.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Customer $customer){
        $customer->delete();

        notification('success', $this->crudMessage('delete', 'Customer'));

        return redirect()->route('company.admin.customer.index');
    }

    public function getStoreList(Request $request){
        $name       = $request->get('data');
        $getList    = getCustomerList();

        $html       = '<option value="" >Select '.$name.'</option>';
        foreach($getList as $key => $list){
            $html  .= '<option value="'.$key.'" >'.$list.'</option>';
        }
        return $html;
    }

    public function getCustomerAddressById(Request $request){
        $data = Customer::find($request->customer_id);
        return $data;
    }

}
