<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Support\Facades\Hash;
use Illuminate\Http\Request;

use App\Http\Requests\CompanyAdmin\DispatcherStoreRequest;
use App\Http\Requests\CompanyAdmin\DispatcherUpdateRequest;
use App\Mail\CompanyAdmin\WelcomeMailToDispatcher;
use App\Mail\CompanyAdmin\NewPasswordMailToDispatcher;

use App\Models\LeaveUserType;
use App\Models\LoadDetail;
use App\Models\CompanyAdmin;
use App\Models\FileUpload;
use App\Models\Bookmark;
use App\Models\Event;
use App\Models\Team;
use App\Models\Driver;
use App\Models\ReportDriver;
use App\Models\CompanyAdminStatusLog;


use Mail;
use Session;
use DataTables;
use Carbon\Carbon;
use Twilio\Rest\Client;
use DB;
use App\Helpers\Common;


use Carbon\CarbonPeriod;

class DispatcherController extends CompanyAdminController{

    /**
     * Create a new controller instance.
     *
     * @return void
     */

    public $timeZone;
    public $timeZoneId;
    public $offset;

    public function __construct(){
        parent::__construct();

        $this->middleware(function ($request, $next) {
            $this->timeZone     = getTimeZone();
            $this->timeZoneId   = getTimeZoneId();
            $this->offset       = getTimeZoneOffset();

            return $next($request);
        });
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getCompanyAdmin    = auth()->guard('company-admin')->user();
        $getField           = '';
        $isCheck            = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchers')->first();
        if(!empty($isCheck->id)){
            //$getField       = unserialize($isCheck->config);
            $getField       = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_dispatchers_first'));
        }else{
            $getField       = config('settings.table_dispatchers_first');
        }

        if ($request->ajax()) {
            $getFilter      = $request->session()->get('cutFilter');
            if($getFilter == '1' || empty($getFilter)){
                $data       = CompanyAdmin::company()->dispatcher();
            }elseif($getFilter == '2'){
                $data       = CompanyAdmin::company()->dispatcher()->onlyTrashed();
            }elseif($getFilter == '3'){
                $data       = CompanyAdmin::company()->dispatcher()->withTrashed();
            }

            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq = 'search_string_condition_'.$key;
                $sortReq = 'search_string_sort_'.$key;

                if( isset( request()->$conditionReq ) ) {
                    $typeReq = 'search_string_type_'.$key;
                    $type = request()->$typeReq;

                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }

                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                if( isset( request()->$sortReq ) ) {
                    $nameReq = 'search_string_name_'.$key;
                    $sortReq = 'search_string_sort_'.$key;
                    $typeReq = 'search_string_type_'.$key;
                    $tableNameReq = 'search_table_name_'.$key;
                    $colAliasReq = 'search_string_col_alias_'.$key;

                    $origData = request()->$nameReq;
                    $condition_sort = request()->$sortReq;
                    $type = request()->$typeReq;
                    $tableName = isset( request()->$tableNameReq ) ? request()->$tableNameReq : '';
                    $colAlias = isset( request()->$colAliasReq ) ? request()->$colAliasReq : '';

                    if( $type == 'string' ) {
                        if( isset( $condition_sort ) ) {
                            if( $colAlias != '' ) {
                                $data->orderBy( $tableName.'.'.$colAlias, $condition_sort );
                            } else {
                                if( $tableName != '' ) {
                                    $data->orderBy( $tableName.'.'.$origData, $condition_sort );
                                } else {
                                    $data->orderBy( $origData, $condition_sort );
                                }
                            }
                        }
                    }
                }
            }
            $data->select('*');
            // General filter code - end

            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $action = '<a href="'.route('company.admin.dispatchers.show', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                    $action .= '<a href="'.route('company.admin.dispatchers.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                    $action .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash termination" data-action="'.route('company.admin.dispatchers.destroy',$row->id).'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';

                    if(auth()->guard('admin')->check()){
                        $action .= '<a href="'.route('company.admin.login.to.dispatcher', [base64Encode($row->id),base64Encode(auth()->guard('company-admin')->user()->id)]).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-random" aria-hidden="true"></i></a>';
                    }

                    return $action;
                })
                ->rawColumns(['action'])
                ->make(true);
        }else{
            $request->session()->put('cutFilter',request()->get('f'));
        }

        $tableName = 'settings.table_dispatchers';
        return view('companyAdmin.dispatcher.index', compact('getField','tableName'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return view('companyAdmin.dispatcher.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(DispatcherStoreRequest $request){
        $input                          = $request->all();
        if($request->address_verified==1){
            $timezone                   = null;
            $timezone_offset            = null;
            if(isset($input['timezone_abbr'])){
                $timezone               = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset        = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']          = $timezone;
            $input['timezone_offset']   = $timezone_offset;
        }

        $input['company_id']            = auth()->guard('company-admin')->user()->company_id;
        $password                       = $input['password'];
        $input['password']              = Hash::make($password);
        $input['type']                  = 2;
        if (isset($input['profile_image'])) {
            $input['profile_image']     = FileUpload::upload('companyAdmin/profile', $input['profile_image']);
        }

        $companyAdmin = CompanyAdmin::create($input);

        $logdata = [];
        $logdata['company_id'] = auth()->guard('company-admin')->user()->company_id;
        $logdata['company_admin_id'] = $companyAdmin->id;
        $logdata['type'] = $companyAdmin->type;
        $logdata['status_id'] = $companyAdmin->status;
        CompanyAdminStatusLog::create($logdata);

        $mailData                       = [];
        $mailData['company_id']         = auth()->guard('company-admin')->user()->company_id;
        $mailData['name']               = $input['first_name'].' '.$input['last_name'];
        $mailData['email']              = $input['email'];
        $mailData['password']           = $password;
        //Mail::to($input['email'])->send(new WelcomeMailToDispatcher($mailData));
        sendEmail($input['email'], new WelcomeMailToDispatcher($mailData));

        // send sms
        if (isset($input['sms']) && isset($input['cell_phone'])) {
            // Twilio's credentials are fetched from company table
            $getCompanyTwilioData       = getCompanyTwilioData();
            if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                $message                = "Your account has been created successfully on NGD. Click here to login: ".route('company.admin.login');
                sendSMS($input['cell_phone'], $message, $getCompanyTwilioData);
            }
        }

        notification('success', $this->crudMessage('create', 'Dispatcher'));

        return redirect()->route('company.admin.dispatchers.index');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id){
        $id = base64Decode($id);
        $companyAdmin           = CompanyAdmin::findOrFail($id);

        $getLeaveUserType       = LeaveUserType::where('company_admins_id',$id)->where('year',date('Y'))->get();

        $now                    = Carbon::now()->timezone($this->timeZone);
        $month_name             = $now->format('F');
        $month                  = $now->format('m');
        $year                   = $now->format('Y');
        return view('companyAdmin.dispatcher.show', compact('companyAdmin','getLeaveUserType','month_name','month','year'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id){
        $id = base64Decode($id);
        $companyAdmin = CompanyAdmin::findOrFail($id);

        $drivers = Driver::where('manager_id', $companyAdmin->id)->get(['id', 'manager_id']);

        return view('companyAdmin.dispatcher.edit', compact('companyAdmin', 'drivers'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(DispatcherUpdateRequest $request, $id){
        $input                          = $request->all();

        if($request->address_verified==1){
            $timezone                   = null;
            $timezone_offset            = null;
            if(isset($input['timezone_abbr'])){
                $timezone               = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset        = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']          = $timezone;
            $input['timezone_offset']   = $timezone_offset;
        }else{
            $input['latitude']          = null;
            $input['longitude']         = null;
            $input['timezone']          = null;
            $input['timezone_offset']   = null;
            $input['timezone_abbr']     = null;
        }

        if ($input['password']) {
            $password                   = $input['password'];
            $input['password']          = Hash::make($password);
        } else {
            unset($input['password']);
        }
        if (isset($input['profile_image'])) {
            $input['profile_image']     = FileUpload::upload('companyAdmin/profile', $input['profile_image']);
        }

        $input['sms']                   = $input['sms'] ?? 0;
        $input['allow_login']           = $input['allow_login'] ?? 0;
        $companyAdmin = CompanyAdmin::find($id);
        $companyAdmin->update($input);

        if($companyAdmin->wasChanged()){
            if(isset($companyAdmin->getChanges()['status'])){
                $logdata = [];
                $logdata['company_id'] = auth()->guard('company-admin')->user()->company_id;
                $logdata['company_admin_id'] = $companyAdmin->id;
                $logdata['type'] = $companyAdmin->type;
                $logdata['status_id'] = $companyAdmin->status;
                CompanyAdminStatusLog::create($logdata);
            }
        }

        // send new pwd mail
        if (isset($input['password'])) {
            $mailData                   = [];
            $mailData['company_id']  = auth()->guard('company-admin')->user()->company_id;
            $mailData['name']           = $input['first_name'].' '.$input['last_name'];
            $mailData['email']          = $input['email'];
            $mailData['password']       = $password;
            //Mail::to($input['email'])->send(new NewPasswordMailToDispatcher($mailData));
            sendEmail($input['email'], new NewPasswordMailToDispatcher($mailData));
        }

        notification('success', $this->crudMessage('update', 'Dispatcher'));

        return redirect()->route('company.admin.dispatchers.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request){
        $input                              = $request->except(['_token','button']);

        $Company_admin                      = CompanyAdmin::withTrashed()->where('id',$input['id'])->first();

        if($request->button == 'delete'){
            $input['termination_date']      = Carbon::createFromFormat('m/d/Y',$request->termination_date)->format('Y-m-d');
            $Company_admin->update($input);
            $Company_admin->delete();
        }elseif($request->button == 'forcedelete'){
            $Company_admin->restore();
            $Company_admin->forceDelete();
        }elseif($request->button == 'restore'){
            $input['termination_date']      = Null;
            $input['termination_reason']    = Null;
            $input['termination_rehire']    = Null;
            $input['termination_equipment'] = Null;
            $input['termination_none']      = Null;
            $Company_admin->update($input);
            $Company_admin->restore();
        }

        if ($request->ajax()) {
            return response()->json(['success' => "Termination successfully"]);
        }

        notification('success', $this->crudMessage('delete', 'Dispatcher'));

        return redirect()->route('company.admin.dispatchers.index');
    }

    public function getTermiForm(Request $request){
        $Company_admin                          = CompanyAdmin::withTrashed()->where('id',$request->id)->first();
        if(!empty($Company_admin->termination_date)){
            $Company_admin->termination_date    = Carbon::createFromFormat('Y-m-d',$Company_admin->termination_date)->format('m/d/Y');
        }

        $is_disabled                            = '0';
        if(!empty($Company_admin->deleted_at)){
            $is_disabled                        = '1';
        }

        $view                                   = view('companyAdmin.dispatcher.view.terminateform', compact('Company_admin','is_disabled'))->render();
        return response()->json(['view' => $view]);

    }

    public function getEvents(Request $request){
        $input                  = $request->all();

        $driver_id              = isset($input['driver_id']) ? $input['driver_id'] : Null;
        $companyAdminId         = isset($input['companyAdminId']) ? $input['companyAdminId'] : Null;
        $calendarEvent          = isset($input['calendarEvent']) ? $input['calendarEvent'] : Null;
        $birthdays              = isset($input['birthdays']) ? $input['birthdays'] : Null;
        $expirations            = isset($input['expirations']) ? $input['expirations'] : Null;
        $maintenanceTrucks      = isset($input['maintenanceTrucks']) ? $input['maintenanceTrucks'] : Null;
        $maintenanceTrailers    = isset($input['maintenanceTrailers']) ? $input['maintenanceTrailers'] : Null;
        $selectCompnay          = isset($input['selectCompnay']) ? $input['selectCompnay'] : Null;
        $selectTeam             = isset($input['selectTeam']) ? $input['selectTeam'] : Null;
        $hidePersonalEvents     = isset($input['hidePersonalEvents']) ? $input['hidePersonalEvents'] : Null;
        $driverleaves           = isset($input['driverleaves']) ? $input['driverleaves'] : Null;
        $myLeaves               = isset($input['myLeaves']) ? $input['myLeaves'] : Null;
        $dispatcherLeaves       = isset($input['dispatcherLeaves']) ? $input['dispatcherLeaves'] : Null;
        $maintenanceTicket      = isset($input['maintenanceTicket']) ? $input['maintenanceTicket'] : Null;

        // $getEvent               = getCompanyAdminEventFullCalendar($companyAdminId,$calendarEvent,$birthdays,$expirations,$maintenanceTrucks,$maintenanceTrailers,$selectCompnay,$selectTeam,$hidePersonalEvents,$driverleaves,$myLeaves,null,$maintenanceTicket );
        $getEvent               = getCompanyAdminEventFullCalendar($companyAdminId,$calendarEvent,$birthdays,$expirations,$maintenanceTrucks,$maintenanceTrailers,$selectCompnay,$selectTeam,$hidePersonalEvents,$driverleaves,$myLeaves,$dispatcherLeaves,$maintenanceTicket);
        return response()->json(['view' => $getEvent]);
        //return $getEvent;
    }

    //---------------------------

    public function removeEdit(Request $request)
    {
        $eventData  = Event::where('id',$request->event_id)->first();
        $view       = view('companyAdmin.dispatcher.editEvent-PopUp',compact('eventData'))->render();
        return response()->json(['status' => 1, 'html' => $view, 'message', '']);

    }

    public function getEventsUpdates(Request $request)
    {
        $from_date              = dateTimeFormatter($request->from_date,'Y-m-d H:i:s',getTimeLayout());
        $to_date                = dateTimeFormatter($request->to_date,'Y-m-d H:i:s',getTimeLayout());
        Event::where('id',$request->id)->update([
            'name'      => $request->name,
            'address'   => $request->address,
            'type'      => $request->type,
            'from_date' => $from_date,
            'to_date'   => $to_date,
            'is_repeat' => $request->is_repeat,
            'notes'     => $request->notes
        ]);
        return redirect()->back();
    }

    public function checkTeamHead(Request $request, $id){
        if ($request->ajax()) {
            try {
                $team                               = Team::select('head_id')->whereId($request->team_id)->first();
                if(!is_null($team)){
                    $response['isTeamHead']         = ($team->head_id==$id)?true:false;
                    $response['success']            = true;
                    $response['message']            = "Widget retrieved successfully.";
                }else{
                    $response['success']            = false;
                    $response['message']            = "Team record not found.";
                }
            } catch (\Exception $e) {
                $response['success']                = false;
                $response['message']                = $e->getMessage();
            }
            return response()->json($response);
        }else{
            return abort(404);
        }
    }

    /**
     * Login to company dispatcher.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function login($id,$previousUser=null){
        $id = base64Decode($id);
        $previousUser = base64Decode($previousUser);
        // $last_login_user = ['id'=>auth()->guard('company-admin')->user()->id,
        //     'name'=> auth()->guard('company-admin')->user()->first_name.' '.auth()->guard('company-admin')->user()->last_name,
        //     'user_type' => 'company-admin',
        //     'last_user_guard_type' => auth()->guard('company-admin')->user()->type
        // ];
        // $last_login_user=json_encode($last_login_user);
        $last_user      = Session::get('last_user');

        $companyAdmin   = CompanyAdmin::where('id', $id)->first();
        Session::put('impersonate_user', $companyAdmin);

        auth()->guard('company-admin')->loginUsingId($companyAdmin->id);

        getTwilioChatToken();

        // return redirect()->route('company.admin.home');
        return redirect()->route('company.admin.home');
    }

    public function dispatcherReport(Request $request){
        $getCompanyAdmin        = auth()->guard('company-admin')->user();
        $getField               = '';
        $isCheck                = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatcher_report')->first();
        if(!empty($isCheck->id)){
            //$getField           = unserialize($isCheck->config);
            $getField           = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_dispatcher_report_first'));
        }else{
            $getField           = config('settings.table_dispatcher_report_first');
        }



        return view('companyAdmin.dispatcher.report',compact('getField'));
    }

    public function dispatcherReportData(Request $request){
        $input                  = $request->all();

        $startDate              = Carbon::parse($request->start_date)->format('Y-m-d');
        $endDate                = Carbon::parse($request->end_date)->format('Y-m-d');
        $companyId              = auth()->guard('company-admin')->user()->company_id;

        $dispatchers            = CompanyAdmin::query()
            ->withCount(['dispatcher_active_drivers','dispatcher_inactive_drivers'])
            ->where('company_admins.company_id',auth()->guard('company-admin')->user()->company_id)
            ->where('company_admins.type',2)
            ->get();



        $dispatchers_arr        = json_decode(json_encode($dispatchers), true);

        $self_dispatch_loads    = LoadDetail::where('self_dispatch',1)
            ->whereHas('loads', function($q){
                    $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                })
            ->where(function($q) {
                $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
            })
            ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate]);

        $dispatcher_id_arr      = $dispatchers->pluck('id')->toArray();
        $loads_drivers          = $self_dispatch_loads->pluck('driver_id')->toArray();

        //$totalDays=$this->getLoadOccupancyDaysCountArrayDriverID($startDate, $endDate , $loads_drivers);
        //$totalDay= $totalDays['totalDays'];

        $totalDay = ReportDriver::whereIn('driver_id', $loads_drivers)
                        ->where('date', '>=' ,$startDate)
                        ->where('date', '<=' ,$endDate)
                        ->sum('actual_totaldays');

        // $self_dispatch_load_built            = $self_dispatch_loads->whereIn('submitted_by',$dispatcher_id_arr)->count();
        $self_dispatch_load_count               = $self_dispatch_loads->count();
        $self_dispatch_gross_rev                = $self_dispatch_loads->sum('driver_revenue');
        $self_dispatch_total_miles              = $self_dispatch_loads->sum('total_miles');
        $self_dispatch_unoccupied_loads         = $self_dispatch_loads->whereIn('load_type', [1,2,3,5])->count();
        $self_dispatch_occupied_loads           = $totalDay;
        $self_dispatch_unauthorized_loads       = $self_dispatch_loads->where('load_type',2)->count();
        $self_dispatch_no_dispatch_loads        = $self_dispatch_loads->where('load_type',4)->count();
        $self_dispatch_maintenance_loads        = $self_dispatch_loads->where('load_type',1)->count();
        $self_dispatch_time_off_loads           = $self_dispatch_loads->where('load_type',5)->count();
        $self_dispatch_hos_loads                = $self_dispatch_loads->where('load_type',3)->count();
        $self_dispatch_active_drivers           = Driver::whereIn('id',$loads_drivers)->count();
        $self_dispatch_inactive_drivers         = Driver::onlyTrashed()->whereIn('id',$loads_drivers)->count();

        $dispatchers1 = array(array(
            'id'                                => '0',
            'full_name'                         => 'self dispatch',
            'dispatcher_active_drivers_count'   => $self_dispatch_active_drivers,
            'dispatcher_inactive_drivers_count' => $self_dispatch_inactive_drivers,
            'loads'                             => $self_dispatch_load_count,
            'gross_revenue'                     => $self_dispatch_gross_rev,
            'avg_revenue'                       => '0',
            'total_miles'                       => $self_dispatch_total_miles,
            'avg_miles'                         => '0',
            'avg_cpm'                           => '0',
            'unoccupied_loads'                  => $self_dispatch_unoccupied_loads,
            'occupied_loads'                    => $totalDay,
            'maintenance'                       => $self_dispatch_maintenance_loads,
            'time_off'                          => $self_dispatch_time_off_loads,
            'hos'                               => $self_dispatch_hos_loads,
            'unauthorized'                      => $self_dispatch_unauthorized_loads,
            'no_dispatch'                       => $self_dispatch_no_dispatch_loads,
            'loads_built'                       => 'NULL'));

        $dispatchers                            = array_merge($dispatchers1, $dispatchers_arr);

        foreach($dispatchers as $key => $dispatcher){

            if($dispatcher['id'] != '0'){
                // \DB::enableQueryLog();
                $dispatchers[$key]['loads']             = LoadDetail::where('booked_by',$dispatcher['id'])
                    ->whereHas('loads', function($q){
                            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                        })
                    ->where(function($q) {
                        $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                    })
                    ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                    ->count();
                    // dd(\DB::getQueryLog());

                $dispatchers[$key]['gross_revenue']     = LoadDetail::where('booked_by',$dispatcher['id'])
                    ->whereHas('loads', function($q){
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    })
                    ->where(function($q) {
                        $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                    })
                    ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                    ->sum('driver_revenue');
                    // DB::enableQueryLog();
                $dispatchers[$key]['total_miles']       = LoadDetail::where('booked_by',$dispatcher['id'])
                    ->whereHas('loads', function($q){
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    })
                    ->where(function($q) {
                        $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                    })
                    ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                    ->sum('total_miles');
                    //  dd(\DB::getQueryLog());

                $dispatchers[$key]['unoccupied_loads']  = LoadDetail::where('booked_by',$dispatcher['id'])->whereIn('load_type', [1,2,3,5])
                    ->whereHas('loads', function($q){
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    })
                    ->where(function($q) {
                        $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                    })
                    ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                    ->count();

                $dispatchers[$key]['occupied_loads']  = $totalDay;


                $dispatchers[$key]['unauthorized']      = LoadDetail::where('booked_by',$dispatcher['id'])->where('load_type',2)
                    ->whereHas('loads', function($q){
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    })
                    ->where(function($q) {
                        $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                    })
                    ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                    ->count();

                $dispatchers[$key]['no_dispatch']       = LoadDetail::where('booked_by',$dispatcher['id'])->where('load_type',4)
                    ->whereHas('loads', function($q){
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    })
                    ->where(function($q) {
                        $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                    })
                    ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                    ->count();

                $dispatchers[$key]['maintenance']       = LoadDetail::where('booked_by',$dispatcher['id'])->where('load_type',1)
                    ->whereHas('loads', function($q){
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    })
                    ->where(function($q) {
                        $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                    })
                    ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                    ->count();

                $dispatchers[$key]['time_off']          = LoadDetail::where('booked_by',$dispatcher['id'])->where('load_type',5)
                    ->whereHas('loads', function($q){
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    })
                    ->where(function($q) {
                        $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                    })
                    ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                    ->count();

                $dispatchers[$key]['hos']               = LoadDetail::where('booked_by',$dispatcher['id'])->where('load_type',3)
                    ->whereHas('loads', function($q){
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    })
                    ->where(function($q) {
                        $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                    })
                    ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                    ->count();

                //total load built - submit by dispatcher
                $dispatchers[$key]['loads_built']       = LoadDetail::where('submitted_by',$dispatcher['id'])
                    ->whereHas('loads', function($q){
                            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                        })
                    ->where(function($q) {
                        $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                    })
                    ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                    ->count();
            }
        }

        return Datatables::of($dispatchers)
            ->addIndexColumn()
            ->editColumn('dispatcher_name', function($row){
                // return (isset($row->full_name)) ? $row->full_name : $row['full_name'];
                return (isset($row['full_name'])) ? $row['full_name'] : '-';
            })
            ->addColumn('active_drivers', function($row){
                // if($row['id'] == 0){
                //     return 0;
                // }else{
                //     return ($row['dispatcher_active_drivers_count']) ? $row['dispatcher_active_drivers_count'] : '0';
                // }
                return ($row['dispatcher_active_drivers_count']) ? $row['dispatcher_active_drivers_count'] : '0';
            })
            ->addColumn('inactive_drivers', function($row){
                // if($row['id'] == 0){
                //     return 0;
                // }else{
                //     return ($row['dispatcher_inactive_drivers_count']) ? $row['dispatcher_inactive_drivers_count'] : '0';
                // }
                return ($row['dispatcher_inactive_drivers_count']) ? $row['dispatcher_inactive_drivers_count'] : '0';
            })
            ->addColumn('loads', function($row){
                // if($row['id'] == 0){
                //     return $row['loads'];
                // }else{
                //     return ($row['loads']) ? number_format($row['loads']) : '0';
                // }
                return ($row['loads']) ? number_format($row['loads'],2, '.', '') : '0';

            })

            ->addColumn('gross', function($row){
                return ($row['gross_revenue']) ? '$'.number_format($row['gross_revenue'],2, '.', '') : '$0.00';
                // if($row['id'] == 0){
                //     return 0;
                // }else{
                //     return ($row['gross_revenue']) ? '$'.number_format($row['gross_revenue']) : '0';
                // }
            })
            ->addColumn('avg_revenue', function($row){
                // if($row['id'] == 0){
                //     return 0;
                // }else{
                //     \Log::info($row['dispatcher_active_drivers_count']);
                //     $driver_count = $row['dispatcher_active_drivers_count'];
                //     return $driver_count == 0 ? 0 : '$'.number_format($row['gross_revenue'] / $driver_count,2);
                // }

                // $driver_count = $row['dispatcher_active_drivers_count'];
                // $driver_count = $row['occupied_loads'];
                // return $driver_count == 0 ? 0 : '$'.number_format($row['gross_revenue'] / $driver_count,2, '.', '');
                $driver_count = $row['dispatcher_active_drivers_count'];
                return $driver_count == 0 ? 0 : '$'.number_format($row['gross_revenue'] / $driver_count,2, '.', '');
            })
            ->addColumn('total_miles', function($row){
                // if($row['id'] == 0){
                //     return 0;
                // }else{
                //     return number_format($row['total_miles']);
                // }
                // return number_format($row['total_miles']);

                // $driver_count = $row['occupied_loads'];
                // return $driver_count == 0 ? 0 : '$'.number_format($row['total_miles'] / $driver_count,2);
                return str_replace(',', '', number_format($row['total_miles']));
            })
            ->addColumn('avg_miles', function($row){
                // if($row['id'] == 0){
                //     return 0;
                // }else{
                //     $driver_count = $row['dispatcher_active_drivers_count'];
                //     return $driver_count == 0 ? 0 : number_format($row['total_miles'] / $driver_count);
                // }
                $driver_count = $row['dispatcher_active_drivers_count'];
                return $driver_count == 0 ? 0 : number_format($row['total_miles'] / $driver_count);
            })
            ->addColumn('avg_cpm', function($row){
                // if($row['id'] == 0){
                //     return 0;
                // }else{
                //     if($row['gross_revenue'] == 0 || $row['total_miles'] == 0)
                //         return '0';
                //     else
                //         return number_format($row['gross_revenue'] / $row['total_miles'],2);
                // }
                // if($row['gross_revenue'] == 0 || $row['total_miles'] == 0 || $row['occupied_loads']==0)
                //     return '0';
                // else
                //    $avg_cpm=number_format($row['gross_revenue'] / $row['total_miles'],2);
                //    $data=number_format( $avg_cpm / $row['occupied_loads'],2);
                //    return $data;
                if($row['gross_revenue'] == 0 || $row['total_miles'] == 0)
                return '0';
            else
                return number_format($row['gross_revenue'] / $row['total_miles'],2, '.', '');
            })
            ->addColumn('action', function($row){
                return '<a href="javascript:void(0)" class="notexport"><img height="25" width="25" src="'.asset('theme/img/pdf.svg').'" alt=""></a>';
            })
            ->addColumn('unoccupied_loads',function($row){
                return ($row['unoccupied_loads']) ? number_format($row['unoccupied_loads']) : '0';
            })
            ->addColumn('maintenance',function($row){
                // if($row['id'] == 0){
                //     return 0;
                // }else{
                //     return ($row['maintenance']) ? number_format($row['maintenance']) : '0';
                // }
                return ($row['maintenance']) ? number_format($row['maintenance']) : '0';
            })
            ->addColumn('time_off',function($row){
                // if($row['id'] == 0){
                //     return 0;
                // }else{
                //     return ($row['time_off']) ? number_format($row['time_off']) : '0';
                // }
                return ($row['time_off']) ? number_format($row['time_off']) : '0';
            })
            ->addColumn('hos',function($row){
                // if($row['id'] == 0){
                //     return 0;
                // }else{
                //     return ($row['hos']) ? number_format($row['hos']) : '0';
                // }
                return ($row['hos']) ? number_format($row['hos']) : '0';
            })
            ->addColumn('unauthorized',function($row){
                return ($row['unauthorized']) ? number_format($row['unauthorized']) : '0';
            })
            ->addColumn('no_dispatch',function($row){
                if($row['id'] == 0){
                    return 0;
                }else{
                    return ($row['no_dispatch']) ? number_format($row['no_dispatch']) : '0';
                }
            })
            ->addColumn('loads_built',function($row){
                if($row['id'] == 0){
                    return 'NULL';
                }else{
                    return ($row['loads_built']) ? number_format($row['loads_built'],2, '.', '') : '0';
                }
                // return ($row['loads_built']) ? number_format($row['loads_built']) : '0';
            })
            ->rawColumns(['action'])
            ->make(true);
    }

    public function dispatcherReportChartData(Request $request){
        $input                  = $request->all();

        $startDate              = Carbon::parse($request->start_date)->format('Y-m-d');
        $endDate                = Carbon::parse($request->end_date)->format('Y-m-d');

        $dispatchers            = CompanyAdmin::company()->dispatcher()->pluck('id');

        //total drivers by dispatcher
        $total_drivers_count    = Driver::whereIn('manager_id',$dispatchers)->count();

        $drivers                = Driver::selectRaw('count(id) as total, manager_id')
            ->isCompany()
            ->whereNotNull('manager_id')
            ->groupBy('manager_id')
            ->get();

        if($drivers->isNotEmpty()){
            foreach ($drivers as $key => $value) {
                $driversArray[] = [
                    "name"      => $value->dispatcher->full_name,
                    "y"         => floatval($value->total)
                ];
            }
        }

        $driversData            = json_encode($driversArray);

        //total loads by dipatcher
        $total_load_count       = LoadDetail::whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->where(function($q) {
                $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
            })
            ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
            ->whereIn('booked_by',$dispatchers)
            ->count();

        $loads                  = LoadDetail::selectRaw('count(id) as total, booked_by')
            ->whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->where(function($q) {
                $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
            })
            ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
            ->whereIn('booked_by',$dispatchers)
            ->whereNotNull('booked_by')
            ->groupBy('booked_by')
            ->get();

        // dd($loads);
        $loadschartArray        = array();
        if($loads->isNotEmpty()){
            foreach ($loads as $key => $value) {
                $loadschartArray[] = ([
                    "name"      => $value->dispatcher->full_name,
                    "y"         => floatval($value->total)
                ]);
            }
        }
        $loadsData              = json_encode($loadschartArray);

        // total miles by dispatcher
        $total_miles            = LoadDetail::whereIn('booked_by',$dispatchers)
            ->whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->where(function($q) {
                $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
            })
            ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
            ->sum('total_miles');

        $miles                  = LoadDetail::selectRaw('sum(total_miles) as total, booked_by')
            ->whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->where(function($q) {
                $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
            })
            ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
            ->whereIn('booked_by',$dispatchers)
            ->whereNotNull('booked_by')
            ->groupBy('booked_by')
            ->get();

        $mileschartArray        = array();
        if($miles->isNotEmpty()){
            foreach ($miles as $key => $value) {
                $mileschartArray[] = ([
                    "name"      => $value->dispatcher->full_name,
                    "y"         => floatval($value->total)
                ]);
            }
        }
        $milesData              = json_encode($mileschartArray);


        // total Revenue by dispatcher
        $total_revenue          = LoadDetail::whereIn('booked_by',$dispatchers)
            ->whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->where(function($q) {
                $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
            })
            ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
            ->sum('driver_revenue');

        $revenue                = LoadDetail::selectRaw('sum(driver_revenue) as total, booked_by')
            ->whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->where(function($q) {
                $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
            })
            ->whereBetween(\DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
            ->whereIn('booked_by',$dispatchers)
            ->whereNotNull('booked_by')
            ->groupBy('booked_by')
            ->get();

        $revenuechartArray      = array();
        if($revenue->isNotEmpty()){
            foreach ($revenue as $key => $value) {
                $revenuechartArray[] = ([
                    "name"      => $value->dispatcher->full_name,
                    "y"         => floatval($value->total)
                ]);
            }
        }
        $revenueData            = json_encode($revenuechartArray);
        //dd($total_drivers_count, $driversData, $total_load_count, $loadsData, $total_miles, $milesData, $total_revenue, $revenueData);
        $view                   = view('companyAdmin.dispatcher.view.dispatcherReport', compact('total_drivers_count','driversData','total_load_count','loadsData','total_miles','milesData','total_revenue','revenueData'))->render();

        return response()->json([
            'view'              => $view,
            'driver_chart'      => [
                'total_drivers_count'   => $total_drivers_count,
                'driversData'           => $driversData,
            ],
            'load_chart'        => [
                'total_load_count'      => $total_load_count,
                'loadsData'             => $loadsData,
            ],
            'miles_chart'       => [
                'total_miles'           => $total_miles,
                'milesData'             => $milesData,
            ],
            'revenue_chart'     => [
                'total_revenue'         => $total_revenue,
                'revenueData'           => $revenueData,
            ],
        ]);
    }

    public function changeTeam(Request $request)
    {
        $input = $request->except(['_token']);

        /* Update Current Dispatcher New Team */
        $currentDispatcher = CompanyAdmin::find(base64Decode($input['current_dispatcher']));
        $currentDispatcher->team_id = $input['team'] ?? null;
        $currentDispatcher->save();
        /* Update Current Dispatcher New Team */

        if(!empty($currentDispatcher)){
            /* This is change all Current Dispatcher >> drivers.team_id (New Team) */
            Driver::where('manager_id', $currentDispatcher->id)->withTrashed()->update(['team_id' => $currentDispatcher->team_id]);
            /* This is change all Current Dispatcher >> drivers.team_id (New Team) */

            $drivers = $input['drivers'] ?? [];
            $dispatchers = $input['dispatchers'] ?? [];
            foreach ($drivers as $key => $driver) {
                /* If any driver assign new dispatcher the we update manager_id & team_id for that driver base on new assigned dispatcher */
                if(isset($dispatchers[$key]) && $currentDispatcher->id != $dispatchers[$key]){
                    $newAssignedDispatcher = CompanyAdmin::find($dispatchers[$key]);
                    if($newAssignedDispatcher){
                        $driverDetail = Driver::findOrFail($driver);
                        $driverDetail->manager_id = $newAssignedDispatcher->id ?? null;
                        $driverDetail->team_id = $newAssignedDispatcher->team_id ?? null;
                        $driverDetail->save();
                    }
                }
                /* If any driver assign new dispatcher the we update manager_id & team_id for that driver base on new assigned dispatcher */
            }
            notification('success', 'Team changed successfully.');
        }else{
            notification('error', 'Error in Change Team.');
        }

        return redirect()->route('company.admin.dispatchers.edit', $input['current_dispatcher']);
    }
}
