<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;
use App\Models\DriverCitation;
use App\Models\FileUpload;
use DataTables;
use Validator;

class DriverCitationController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request, $id)
    {
        if ($request->ajax()) {
            $data = DriverCitation::select('*');
            return Datatables::of($data)
                    ->addIndexColumn()
                    ->filterColumn("date", function ($query, $keyword) {
                        $query->whereRaw(
                            "DATE_FORMAT(date,'%m/%d/%Y') LIKE ?",
                            ["%$keyword%"]
                        );
                    })
                    ->addColumn('action', function($row){
                        $btn = '<a href="#" data-id="'.$row->id.'" data-toggle="modal" data-target="#showCitationModal" class="show-citation-btn mr-4"><img src="'.asset('theme/img/eye.png').'" alt=""></a>';
                        $btn = $btn.' <a href="#" data-id="'.$row->id.'" data-toggle="modal" data-target="#editCitationModal" class="edit-citation-btn mr-4"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';
                        $btn = $btn.' <a href="javascript:" class="destroy" data-action="'.route('company.admin.citation.destroy',$row->id).'" data-id="'.$row->id.'"><img src="'.asset('theme/img/delete-icon.png').'" alt=""></a>';
                        return $btn;
                    })
                    ->rawColumns(['action'])
                    ->make(true);
        }

        return view('companyAdmin.driver.citation',compact('id'));
    }

    /**
     * Render the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function createFormView(Request $request)
    {
        $driverId = $request->driver_id;

        $view = view('companyAdmin.driver.view.citationCreateFormView', compact('driverId'))->render();

        return response()->json(['view' => $view]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $input = $request->all();

        $validator = Validator::make($input,[
            "title" => "required",
            "date" => "required",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        if (isset($input['image'])) {
            $input['image'] = FileUpload::upload('companyAdmin/driver', $input['image']);
        }

        DriverCitation::create($input);

        notification('success', $this->crudMessage('create', 'Citation'));

        return response()->json(['success' => "Citation created successfully"]);
    }

    /**
     * Render the form for showing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function showView(Request $request)
    {
        $driverCitation = DriverCitation::find($request->id);

        $view = view('companyAdmin.driver.view.citationShowView', compact('driverCitation'))->render();

        return response()->json(['view' => $view]); 
    }

    /**
     * Render the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function editFormView(Request $request)
    {
        $driverCitation = DriverCitation::find($request->id);

        $view = view('companyAdmin.driver.view.citationEditFormView', compact('driverCitation'))->render();

        return response()->json(['view' => $view]); 
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $input = $request->all();

        $validator = Validator::make($input,[
            "title" => "required",
            "date" => "required",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        if (!is_null($input['image'])) {
            $input['image'] = FileUpload::upload('companyAdmin/driver', $input['image']);
        } else {
            unset($input['image']);
        }
        
        DriverCitation::find($request->id)->update($input);

        notification('success', $this->crudMessage('update', 'Citation'));

        return response()->json(['success' => "Citation updated successfully"]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        DriverCitation::find($id)->delete();

        notification('success', $this->crudMessage('delete', 'Citation'));

        return redirect()->back();
    }
}
