<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;

use Illuminate\Http\Request;
use Illuminate\Support\Str;

use App\Http\Requests\CompanyAdmin\DriverStoreRequest;
use App\Http\Requests\CompanyAdmin\DriverUpdateRequest;

use App\Mail\CompanyAdmin\WelcomeMailToDriver;
use App\Mail\CompanyAdmin\NewPasswordMailToDriver;
use App\Mail\CompanyAdmin\UnreadNotificationMailToUser;

use App\Models\Driver;
use App\Models\DriverOnboarding;
use App\Models\DriverInformation;

use App\Models\DriverFinancial;
use App\Models\DriverAvoidState;
use App\Models\DriverRestrictedState;

use App\Models\DriverCitation;
use App\Models\DriverMechanicalIssue;
use App\Models\DriverComplianceSafety;

use App\Models\DriverHosViolation;
use App\Models\DriverHosViolationsData;

use App\Models\DriverNonViolation;
use App\Models\DriverNonViolationsData;

use App\Models\LoadDetail;
use App\Models\LoadDriver;

use App\Models\LoadFile;
use App\Models\LoadFileType;

use App\Models\Team;
use App\Models\Truck;
use App\Models\Trailer;
use App\Models\Carrier;
use App\Models\Terminal;
use App\Models\CompanyAdmin;

use App\Models\State;
use App\Models\Event;

use App\Models\Gallery;
use App\Models\FileUpload;

use App\Models\Bookmark;
use App\Models\ListWidget;
use App\Models\IntegrationSetting;

use App\Models\LeaveUserType;
use App\Models\InsuranceProvider;
use App\Models\CompanyDeductionType;
use App\Models\DriverStatusLog;

use DB;
use Auth;
use Hash;
use Mail;
use DataTables;
use Carbon\Carbon;
use Carbon\CarbonPeriod;

use App\Helpers\Common;

use Twilio\Rest\Client;
use Illuminate\Support\Js;
use Eastwest\Json\Facades\Json;
use Illuminate\Support\Facades\Session;

class DriverController extends CompanyAdminController{

    public $timeZone;
    public $timeZoneId;
    public $offset;

    public function __construct(){
        parent::__construct();

        $this->middleware(function ($request, $next) {
            $this->timeZone     = getTimeZone();
            $this->timeZoneId   = getTimeZoneId();
            $this->offset       = getTimeZoneOffset();

            return $next($request);
        });
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index_backup(Request $request){
        $getCompanyAdmin            = auth()->guard('company-admin')->user();

        $getField                   = '';
        $isCheck                    = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','drivers')->first();
        if(!empty($isCheck->id)){
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_drivers_first'));
        }else{
            $getField               = config('settings.table_drivers_first');
        }
        if ($request->ajax()) {
            $getFilter              = $request->session()->get('cutFilter');

            if ($getFilter == '1' || empty($getFilter)) {
                $data               = Driver::isCompany()->leftJoin('driver_informations', 'drivers.id', '=', 'driver_informations.driver_id')->whereIn('drivers.status', ['0', '1', '3']);
            } elseif ($getFilter == '2') {
                $data               = Driver::isCompany()->leftJoin('driver_informations', 'drivers.id', '=', 'driver_informations.driver_id')->where('drivers.status', '2')->withTrashed();
            } elseif ($getFilter == '3') {
                $data               = Driver::isCompany()->leftJoin('driver_informations', 'drivers.id', '=', 'driver_informations.driver_id')->whereIn('status', ['0', '1', '2', '3'])->withTrashed();
            }

            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq       = 'search_string_condition_'.$key;
                $sortReq            = 'search_string_sort_'.$key;
                $conditionDateReq   = 'search_date_condition_'.$key;
                $sortDateReq        = 'search_date_sort_'.$key;
                $conditionStatusReq = 'search_status_condition_'.$key;
                $sortStatusReq      = 'search_status_sort_'.$key;

                // String filter
                if( isset( request()->$conditionReq ) ) {
                    $typeReq        = 'search_string_type_'.$key;
                    $type           = request()->$typeReq;

                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }
                }

                // Status filter
                if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                    Common::getStatusFilterResult( $data, $key );
                }

                // Date filter
                if( isset( request()->$conditionDateReq ) ) {
                    $typeReq        = 'search_date_type_'.$key;
                    $type           = request()->$typeReq;

                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                // String sort only
                if( isset( request()->$sortReq ) ) {
                    Common::sortStringFilter( $data, $key );
                }

                // Date sort only
                if( isset( request()->$sortDateReq ) ) {
                    Common::sortDateFilter( $data, $key );
                }

                // Status sort only
                if( isset( request()->$sortStatusReq ) ) {
                    Common::sortStatusFilter( $data, $key );
                }
            }
            // General filter code - end

            $data->select('drivers.*', 'driver_informations.truck_number');

            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('name', function ($row) {
                    return $row->last_name . ', ' . $row->first_name;
                })
                ->filterColumn('name', function ($query, $keyword) {
                    $query->where(DB::raw("CONCAT(last_name, ', ', first_name)"), 'LIKE', "%" . $keyword . "%");
                })
                ->orderColumn("name", function ($query, $order) {
                    $query->orderByRaw(
                        "last_name $order,first_name $order"
                    );
                })
                ->addColumn('email', function ($row) {
                    return '<a href="mailto:'.$row->email.'" class="text-lowercase">'.$row->email.'</a>';
                })
                ->filterColumn('email', function ($query, $keyword) {
                    $query->where(DB::raw("email"), 'LIKE', "%" . $keyword . "%");
                })
                ->addColumn('cell_phone', function ($row) {
                    return '<a href="tel:'.$row->cell_phone.'" >'.$row->cell_phone.'</a>';
                })
                ->filterColumn('cell_phone', function ($query, $keyword) {
                    $query->where(DB::raw("cell_phone"), 'LIKE', "%" . $keyword . "%");
                })
                ->addColumn('truck_number', function ($row) {
                    return $row->truck_number;
                })
                ->editColumn('manager_id', function ($row) {
                    if($row->manager_id != ''){
                        return $row->dispatcher->full_name;
                    }
                    return '';
                })
                ->addColumn('deleted_at', function ($row) {
                    if (empty($row->deleted_at)) {
                        return '<label class="active">Active</label>';
                    } else {
                        return '<label class="inactive">Terminated</label>';
                    }
                })
                ->orderColumn("deleted_at", function ($query, $order) {
                    $query->orderByRaw(
                        "deleted_at $order"
                    );
                })
                ->addColumn('action', function ($row) use ($getFilter) {
                    if (!empty($row->deleted_at)) {
                        $deleted_at = 1;
                    } else {
                        $deleted_at = 0;
                    }

                    $action         = '';

                    if($row->status == '2'){
                        if(hasPermission(config('settings.company_admin_modules.drivers_terminated'),config('settings.permissions.view'))){
                            $action = '<a href="'.route('company.admin.drivers.show', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                        }
                        if(hasPermission(config('settings.company_admin_modules.drivers_terminated'),config('settings.permissions.edit'))){
                            $action.= '<a href="'.route('company.admin.drivers.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                        }

                        /* DP-209 Remove Delete action on listing page
                        if(hasPermission(config('settings.company_admin_modules.drivers_terminated'),config('settings.permissions.delete'))){
                            $action .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash termination" data-action="'.route('company.admin.drivers.destroy', $row->id).'" data-deleted="'.$deleted_at.'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';
                        }
                        */
                    }else{
                        if(hasPermission(config('settings.company_admin_modules.drivers'),config('settings.permissions.view'))){
                            $action = '<a href="'.route('company.admin.drivers.show', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                        }
                        if(hasPermission(config('settings.company_admin_modules.drivers'),config('settings.permissions.edit'))){
                            $action.= '<a href="'.route('company.admin.drivers.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                        }

                        /* DP-209 Remove Delete action on listing page
                        if(hasPermission(config('settings.company_admin_modules.drivers'),config('settings.permissions.delete'))){
                            $action.= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash termination" data-action="'.route('company.admin.drivers.destroy', $row->id).'" data-deleted="'.$deleted_at.'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';
                        }
                        */
                    }

                    if(auth()->guard('admin')->check()){
                        $action    .= '<a href="'.route('company.admin.drivers.login', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-random" aria-hidden="true"></i></a>';
                    }
                    return $action;
                })
                ->rawColumns(['action', 'deleted_at','email','cell_phone'])
                ->make(true);
        } else {
            $request->session()->put('cutFilter', request()->get('f'));
        }

        $tableName                  = 'settings.table_drivers';
        return view('companyAdmin.driver.index-bk-20052024', compact('getField','tableName')); // return view('companyAdmin.driver.index', compact('getField','tableName'));
    }

    public function index(Request $request){
        $getCompanyAdmin            = auth()->guard('company-admin')->user();

        $getField                   = '';
        $isCheck                    = Bookmark::where('company_id', $getCompanyAdmin->company_id)
            ->where('user_id', $getCompanyAdmin->id)
            ->where('user_type', $getCompanyAdmin->type)
            ->where('table_id', 'lists_drivers')
            ->first();
        if( !empty( $isCheck->id ) ) {
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_lists_drivers_first'));
        } else {
            $getField               = config('settings.table_lists_drivers_first');
        }

        $isMulitpleTable            = 1;

        $tables                     = array(
            array(
                'name'              => 'table1',
                'id'                => 'drivers-all-data-table',
                'class'             => 'drivers-all-data-table',
                'route'             => route('company.admin.lists.drivers.all')
            ),
            array(
                'name'              => 'table2',
                'id'                => 'drivers-company-data-table',
                'class'             => 'drivers-company-data-table',
                'route'             => route('company.admin.lists.drivers.all')
            ),
            array(
                'name'              => 'table3',
                'id'                => 'drivers-owner-operators-data-table',
                'class'             => 'drivers-owner-operators-data-table',
                'route'             => route('company.admin.lists.drivers.all')
            )
        );

        $tableName                  = 'settings.table_lists_drivers';

        // Widget
        $driverWidgetActive         = "1";
        $driverWidget               = ListWidget::IsCompanyAdmin()->where('tab', 'driver')->latest()->first();
        if(!is_null($driverWidget)){
            $driverWidgetActive     = $driverWidget->isActive;
        }

        // Driver Count
        $totalActiveDrivers         = Driver::isCompany()->where('status', '0')->count();
        $totalOosDrivers            = Driver::isCompany()->where('status', '1')->count();
        $totalOwnerOperators        = Driver::isCompany()->where('is_company_driver', '0')->withTrashed()->count(); // 0 = Owner Operator, 1 = Company Driver, 2 = Subcontractor
        $totalCompanyDrivers        = Driver::isCompany()->where('is_company_driver', '1')->withTrashed()->count();
        $totalDrivers               = Driver::isCompany()->withTrashed()->count();
        return view('companyAdmin.driver.index', compact(
                'getField',
                'isMulitpleTable',
                'tables',
                'tableName',

                'driverWidgetActive',

                'totalActiveDrivers',
                'totalOosDrivers',
                'totalOwnerOperators',
                'totalCompanyDrivers',
                'totalDrivers'
            )
        );
    }

    public function TargetRevenue(Request $request){
        if(isset($request->driver_id) && !empty($request->driver_id)){
            isReportDriversZero($request->driver_id);
        }

        $input                  = $request->except('_method', '_token');
        $input['start_date']    = Carbon::createFromFormat('m/d/Y', $input['start_date'])->format('Y-m-d');
        $input['end_date']      = Carbon::createFromFormat('m/d/Y', $input['end_date'])->format('Y-m-d');
        if (isset($request->dataid)) {
            $dfinancial         = DriverFinancial::find($request->dataid);
            $dfinancial->update($input);
            return '1';
        } else {
            $is_check           = DriverFinancial::where('driver_id', $input['driver_id'])->where('year', $input['year'])->where('quarter', $input['quarter'])->count();
            if ($is_check == 0) {
                DriverFinancial::create($input);
                return '1';
            }
            return '0';
        }
        return '0';
    }

    public function TargetRevenueView(Request $request){
        $input                  = $request->except('_method', '_token');
        $getData                = DriverFinancial::where('driver_id', $input['driver_id'])->orderBy('year', 'DESC')->latest()->take($input['take'])->get();
        $view                   = view('companyAdmin.driver.view.FinanceView', compact('getData'))->render();
        return response()->json(['view' => $view]);
    }

    public function TargetRevenueDestroy($id){
        $getData                = DriverFinancial::find($id);
        $getData->delete();
        return 1;
    }

    public function getTargetRevenue($id){
        $getData                = DriverFinancial::find($id);
        $getData->start_date    = Carbon::createFromFormat('Y-m-d', $getData->start_date)->format('m/d/Y');
        $getData->end_date      = Carbon::createFromFormat('Y-m-d', $getData->end_date)->format('m/d/Y');
        return response()->json(['data' => $getData]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create_backup() {
        return view('companyAdmin.driver.create-bk-30062023');
    }

    public function create(){
        $driverOnboarding                       = '{[]}';
        $filtered_collection                    = null;

        $usedTruckNumbers                       = Truck::IsCompany()->whereNotNull('driver')->whereNotNull('co_driver')->pluck('unit')->all();
        $usedTrailerNumbers                     = Trailer::IsCompany()->whereNotNull('current_assigned_driver')->pluck('trailer_id')->all();

        $getTrailerNumberListByCompany          = Trailer::isCompany()->get();
        return view('companyAdmin.driver.create', compact('driverOnboarding', 'filtered_collection', 'usedTruckNumbers', 'getTrailerNumberListByCompany', 'usedTrailerNumbers'));
    }

    public function edit($id){
        $id                                     = base64Decode($id);
        $driver                                 = Driver::withTrashed()->findOrFail($id);

        if($driver->status == "2"){
            if(!hasPermission(config('settings.company_admin_modules.drivers_terminated'),config('settings.permissions.edit'))){
                notification('error', 'You do not have permission to access this page.');
                return redirect()->route('company.admin.home');
            }
        }

        if(isset($driver->trailer_experience) &&  $driver->trailer_experience != ""){
            $driver->trailer_experience         = explode(',', $driver->trailer_experience);
        }
        if(isset($driver->truck_experience) &&  $driver->truck_experience != ""){
            $driver->truck_experience           = explode(',', $driver->truck_experience);
        }
        if(isset($driver->ssn) &&  $driver->ssn != ""){
            $driver->ssn                        = base64_decode( $driver->ssn);
        }
        if(isset($driver->ein) &&  $driver->ein != ""){
            $driver->ein                        = base64_decode( $driver->ein);
        }

        if($driver->ssn_or_ein == 0){
            $federalOrSocialId                  = $driver->ssn;
        }else if($driver->ssn_or_ein == 1){
            $federalOrSocialId                  = $driver->ein;
        }else{
            $federalOrSocialId                  = null;
        }

        if(isset($driver->license_number) && $driver->license_number != ""){
            $driver->license_number             = base64_decode($driver->license_number);
        }
        if(isset($driver->passport_number) &&  $driver->passport_number != ""){
            $driver->passport_number            = base64_decode( $driver->passport_number);
        }
        if(isset($driver->license_endorsements) &&  $driver->license_endorsements != ""){
            $driver->license_endorsements       = explode(',', $driver->license_endorsements);
        }

        if(isset($driver->insurance_type) && $driver->insurance_type != ""){
            $driver->insurance_type             = (is_serialized($driver->insurance_type)) ? unserialize($driver->insurance_type) : null;
        }
        if(isset($driver->insurance_provider) && $driver->insurance_provider != ""){
            $driver->insurance_provider         = (is_serialized($driver->insurance_provider)) ? unserialize($driver->insurance_provider) : null;
        }
        if(isset($driver->policy_number) && $driver->policy_number != ""){
            $driver->policy_number              = (is_serialized($driver->policy_number)) ? unserialize($driver->policy_number) : null;
        }
        if(isset($driver->insurance_expiration) && $driver->insurance_expiration != ""){
            $driver->insurance_expiration       = (is_serialized($driver->insurance_expiration)) ? unserialize($driver->insurance_expiration) : null;
        }

        $driver->restricted_states              = json_encode($driver->driver_restricted_states()->pluck('state_id')->toArray());
        $driver->avoid_states                   = json_encode($driver->driver_avoid_states()->pluck('state_id')->toArray());

        if(isset($driver->emergency_contact_name) && $driver->emergency_contact_name != ""){
            $driver->emergency_contact_name     = (is_serialized($driver->emergency_contact_name)) ? unserialize($driver->emergency_contact_name) : null;
        }
        if(isset($driver->emergency_address) && $driver->emergency_address != ""){
            $driver->emergency_address          = (is_serialized($driver->emergency_address)) ? unserialize($driver->emergency_address) : null;
        }
        if(isset($driver->emergency_contact_number) && $driver->emergency_contact_number != ""){
            $driver->emergency_contact_number   = (is_serialized($driver->emergency_contact_number)) ? unserialize($driver->emergency_contact_number) : null;
        }
        if(isset($driver->emergency_home_phone) && $driver->emergency_home_phone != ""){
            $driver->emergency_home_phone       = (is_serialized($driver->emergency_home_phone)) ? unserialize($driver->emergency_home_phone) : null;
        }
        if(isset($driver->emergency_contact_relationship_to_driver) && $driver->emergency_contact_relationship_to_driver != ""){
            $driver->emergency_contact_relationship_to_driver = (is_serialized($driver->emergency_contact_relationship_to_driver)) ? unserialize($driver->emergency_contact_relationship_to_driver) : null;
        }

        if(isset($driver->deduction_name) && $driver->deduction_name != ""){
            $driver->deduction_name             = (is_serialized($driver->deduction_name)) ? unserialize($driver->deduction_name) : null;
        }
        if(isset($driver->deduction_type) && $driver->deduction_type != ""){
            $driver->deduction_type             = (is_serialized($driver->deduction_type)) ? unserialize($driver->deduction_type) : null;
        }
        if(isset($driver->deduction_amount) && $driver->deduction_amount != ""){
            $driver->deduction_amount           = (is_serialized($driver->deduction_amount)) ? unserialize($driver->deduction_amount) : null;
        }
        if(isset($driver->deduction_unit) && $driver->deduction_unit != ""){
            $driver->deduction_unit             = (is_serialized($driver->deduction_unit)) ? unserialize($driver->deduction_unit) : null;
        }
        if(isset($driver->starting_or_cap_amount) && $driver->starting_or_cap_amount != ""){
            $driver->starting_or_cap_amount     = (is_serialized($driver->starting_or_cap_amount)) ? unserialize($driver->starting_or_cap_amount) : null;
        }

        if(isset($driver->days_per_year) && $driver->days_per_year != ""){
            $driver->days_per_year              = (is_serialized($driver->days_per_year)) ? unserialize($driver->days_per_year) : null;
        }
        if(isset($driver->max_consecutive_days) && $driver->max_consecutive_days != ""){
            $driver->max_consecutive_days       = (is_serialized($driver->max_consecutive_days)) ? unserialize($driver->max_consecutive_days) : null;
        }
        if(isset($driver->leave_type) && $driver->leave_type != ""){
            $driver->leave_type                 = (is_serialized($driver->leave_type)) ? unserialize($driver->leave_type) : null;
        }
        if(isset($driver->hire_date) && $driver->hire_date != ""){
            $driver->hire_date                  = (is_serialized($driver->hire_date)) ? unserialize($driver->hire_date) : null;
        }
        if(isset($driver->reset_date) && $driver->reset_date != ""){
            $driver->reset_date                 = (is_serialized($driver->reset_date)) ? unserialize($driver->reset_date) : null;
        }
        if(isset($driver->paid_type) && $driver->paid_type != ""){
            $driver->paid_type                  = (is_serialized($driver->paid_type)) ? unserialize($driver->paid_type) : null;
        }
        if(isset($driver->pto_rollover) && $driver->pto_rollover != ""){
            $driver->pto_rollover               = (is_serialized($driver->pto_rollover)) ? unserialize($driver->pto_rollover) : null;
        }
        if(isset($driver->max_accumulated_pto) && $driver->max_accumulated_pto != ""){
            $driver->max_accumulated_pto        = (is_serialized($driver->max_accumulated_pto)) ? unserialize($driver->max_accumulated_pto) : null;
        }
        if(isset($driver->allow_unpaid_leave) && $driver->allow_unpaid_leave != ""){
            $driver->allow_unpaid_leave         = (is_serialized($driver->allow_unpaid_leave)) ? unserialize($driver->allow_unpaid_leave) : null;
        }
        if(isset($driver->allow_half_days) && $driver->allow_half_days != ""){
            $driver->allow_half_days            = (is_serialized($driver->allow_half_days)) ? unserialize($driver->allow_half_days) : null;
        }

        if(isset($driver->passenger_onboard) && $driver->passenger_onboard != ""){
            $driver->passenger_onboard          = (is_serialized($driver->passenger_onboard)) ? unserialize($driver->passenger_onboard) : null;
        }
        if(isset($driver->passenger_agreement_On_file) && $driver->passenger_agreement_On_file != ""){
            $driver->passenger_agreement_On_file = (is_serialized($driver->passenger_agreement_On_file)) ? unserialize($driver->passenger_agreement_On_file) : null;
        }
        if(isset($driver->passenger_expiration) && $driver->passenger_expiration != ""){
            $driver->passenger_expiration       = (is_serialized($driver->passenger_expiration)) ? unserialize($driver->passenger_expiration) : null;
        }
        if(isset($driver->passenger_full_name) && $driver->passenger_full_name != ""){
            $driver->passenger_full_name        = (is_serialized($driver->passenger_full_name)) ? unserialize($driver->passenger_full_name) : null;
        }
        if(isset($driver->passenger_age) && $driver->passenger_age != ""){
            $driver->passenger_age              = (is_serialized($driver->passenger_age)) ? unserialize($driver->passenger_age) : null;
        }
        if(isset($driver->passenger_phone_number) && $driver->passenger_phone_number != ""){
            $driver->passenger_phone_number     = (is_serialized($driver->passenger_phone_number)) ? unserialize($driver->passenger_phone_number) : null;
        }
        if(isset($driver->passenger_relationship) && $driver->passenger_relationship != ""){
            $driver->passenger_relationship     = (is_serialized($driver->passenger_relationship)) ? unserialize($driver->passenger_relationship) : null;
        }
        if(isset($driver->passenger_gender) && $driver->passenger_gender != ""){
            $driver->passenger_gender           = (is_serialized($driver->passenger_gender)) ? unserialize($driver->passenger_gender) : null;
        }
        if(isset($driver->passenger_allergies) && $driver->passenger_allergies != ""){
            $driver->passenger_allergies        = (is_serialized($driver->passenger_allergies)) ? unserialize($driver->passenger_allergies) : null;
        }

        if(isset($driver->pet_onboard) && $driver->pet_onboard != ""){
            $driver->pet_onboard                = (is_serialized($driver->pet_onboard)) ? unserialize($driver->pet_onboard) : null;
        }
        if(isset($driver->pet_type) && $driver->pet_type != ""){
            $driver->pet_type                   = (is_serialized($driver->pet_type)) ? unserialize($driver->pet_type) : null;
        }
        if(isset($driver->pet_name) && $driver->pet_name != ""){
            $driver->pet_name                   = (is_serialized($driver->pet_name)) ? unserialize($driver->pet_name) : null;
        }
        if(isset($driver->pet_deposit_amount) && $driver->pet_deposit_amount != ""){
            $driver->pet_deposit_amount         = (is_serialized($driver->pet_deposit_amount)) ? unserialize($driver->pet_deposit_amount) : null;
        }

        if(isset($driver->medical_condition) && $driver->medical_condition != ""){
            $driver->medical_condition          = (is_serialized($driver->medical_condition)) ? unserialize($driver->medical_condition) : null;
        }
        if(isset($driver->driving_restrictions) && $driver->driving_restrictions != ""){
            $driver->driving_restrictions       = (is_serialized($driver->driving_restrictions)) ? unserialize($driver->driving_restrictions) : null;
        }
        if(isset($driver->next_doctor_visit) && $driver->next_doctor_visit != ""){
            $driver->next_doctor_visit          = (is_serialized($driver->next_doctor_visit)) ? unserialize($driver->next_doctor_visit) : null;
        }

        $driverInformation                      = $driver->driver_information;
        $carrier                                = Carrier::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('id')->all();
        $terminal                               = Terminal::whereIn('carrier_id',$carrier)->get();

        $driverOnboarding                       = !empty($driver->driver_onboarding) ? $driver->driver_onboarding : '{[]}';
        $filtered_collection                    = null;
        if (!is_null($driverOnboarding) && !empty($driverOnboarding->detail_json)) {
            $t                                  = collect($driverOnboarding->detail_json);
            $filtered_collection                = $t->filter(function ($item) {
                return $item['yes_no'] == 1 && $item['is_complete'] == 1;
            })->last();
        }

        if(is_null($driver->eobr_provider) || is_null($driver->eobr_id)){
            $samsaraId                          = null;
            $carrierId                          = (isset($driver->driver_information->carrier_id)) ? $driver->driver_information->carrier_id : null;

            $getAPIKey                          = IntegrationSetting::where('company_id', $driver->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->first();
            if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                $client                         = new \GuzzleHttp\Client(['verify' => false]);
                $response                       = $client->request('GET', 'https://api.samsara.com/fleet/drivers', [
                    'headers'                   => [
                        'Accept'                => 'application/json',
                        'Authorization'         => 'Bearer '.$getAPIKey->api_key,
                    ],
                ]);
                $data                           = json_decode($response->getBody());
                $data                           = $data->data;
                foreach($data as $driverData){
                    if(isset($driverData->licenseNumber) && $driverData->licenseNumber == $driver->license_number){
                        $samsaraId              = $driverData->id;
                        break;
                    }
                }
                if(!is_null($samsaraId)){
                    $driver->eobr_provider      = '0';
                    $driver->eobr_id            = $samsaraId;
                }
            }
        }

        $usedTruckNumbers                       = Truck::IsCompany()->whereNotNull('driver')->whereNotNull('co_driver')->pluck('unit')->all();
        $usedTrailerNumbers                     = Trailer::IsCompany()->whereNotNull('current_assigned_driver')->pluck('trailer_id')->all();

        $getTrailerNumberListByCompany          = Trailer::isCompany()->get();

        return view('companyAdmin.driver.create', compact('driver', 'driverInformation', 'terminal', 'federalOrSocialId', 'driverOnboarding', 'filtered_collection', 'usedTruckNumbers', 'getTrailerNumberListByCompany', 'usedTrailerNumbers'));
    }

    public function saveGeneralInfo(Request $request){
        $input                                          = $request->all();

        $input['company_id']                            = auth()->guard('company-admin')->user()->company_id;
        $input['is_company_driver']                     = (isset($request->driver_type) && !is_null($request->driver_type)) ? $request->driver_type : 0;
        $input['birthday']                              = $request->date_of_birth;
        $input['carrier_id']                            = $request->carrier;
        $input['terminal_id']                           = $request->terminal;
        $input['manager_id']                            = $request->dispatcher;
        $input['business_na']                           = (isset($request->business_na) && $request->business_na == '1') ? '1' : '0';
        $input['allow_login']                           = (isset($request->allow_login) && $request->allow_login == '1') ? 1 : 0;
        $input['allow_see_invoices']                    = (isset($request->allow_see_invoices) && $request->allow_see_invoices == '1') ? 1 : 0;
        $input['allow_see_rate_confirmations']          = (isset($request->allow_see_rate_confirmations) && $request->allow_see_rate_confirmations == '1') ? 1 : 0;
        $input['dispatch_type']                         = (isset($request->dispatch_type) && $request->dispatch_type == '1') ? 1 : 0;
        $input['sms']                                   = (isset($request->allow_sms) && $request->allow_sms == '1') ? 1 : 0;
        $input['trailer_experience']                    = (!is_null($request->trailer_experience) && count($request->trailer_experience) > 0) ? implode(',', $request->trailer_experience) : null;
        $input['truck_experience']                      = (!is_null($request->truck_experience) && count($request->truck_experience) > 0) ? implode(',', $request->truck_experience) : null;

        if(isset($input['status'])){
            if($input['status'] == '2'){
                $input['termination_date']              = Carbon::parse($input['termination_date'])->format('Y-m-d H:i:s') ?? Carbon::now()->format('Y-m-d H:i:s');
                $input['termination_reason']            = 'Updated Status';
                $input['termination_rehire']            = '1';
                $input['termination_equipment']         = '0';
                $input['termination_none']              = null;
                $input['deleted_at']                    = Carbon::now()->format('Y-m-d H:i:s');
            }else{
                $input['termination_date']              = null;
                $input['termination_reason']            = null;
                $input['termination_rehire']            = null;
                $input['termination_equipment']         = null;
                $input['termination_none']              = null;
                $input['deleted_at']                    = null;
            }
        }

        if(isset($input['status'])){
            if($input['status'] == '1'){
                $input['oos_date']                      = Carbon::now()->format('Y-m-d H:i:s');
            }else{
                $input['oos_date']                      = null;
            }
        }

        if(isset($input['social_security']) &&  $input['social_security'] == "1"){
            $input['ssn_or_ein']                        = 0;
            $input['ssn']                               = base64_encode($input['federal_or_Social_id']);
        }

        if(isset($input['federal_id']) && $input['federal_id'] == "1"){
            $input['ssn_or_ein']                        = 1;
            $input['ein']                               = base64_encode($input['federal_or_Social_id']);
        }

        if(isset($input['same_as_dispatcher']) && $input['same_as_dispatcher'] == "1"){
            $input['load_planner']                      = $request->dispatcher;
        }

        if($request->address_verified == 1){
            $timezone                                   = null;
            $timezone_offset                            = null;
            if(isset($input['timezone_abbr'])){
                $timezone                               = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset                        = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']                          = $timezone;
            $input['timezone_offset']                   = $timezone_offset;
        }

        $driverManager                                  = CompanyAdmin::where('id', $request->dispatcher)->first();
        $team_id                                        = null;
        if(!is_null($driverManager) && isset($driverManager->team)){
            $team_id                                    = $driverManager->team->id;
        }
        $input['team_id']                               = $team_id;

        $renderDriverOnboardingView                     = '';

        if(!is_null($request->driver_id)){
            if ($input['password']) {
                $password                               = $input['password'];
                $input['password']                      = Hash::make($password);
            } else {
                unset($input['password']);
            }

            $driver                                     = Driver::withTrashed()->IsCompany()->where('id', $request->driver_id)->first();

            if($input['allow_login'] == 1 && isset($input['password'])){
                if($driver->allow_login == 0 && $input['allow_login'] == 1){
                    // Send Email
                    $mailData                           = [];
                    $mailData['company_id']             = auth()->guard('company-admin')->user()->company_id;
                    $mailData['name']                   = $input['first_name'].' '.$input['last_name'];
                    $mailData['email']                  = $input['email'];
                    $mailData['password']               = $password;
                    //Mail::to($input['email'])->send(new WelcomeMailToDriver($mailData));
                    sendEmail($input['email'], new WelcomeMailToDriver($mailData));

                    // Send SMS
                    if (isset($input['sms']) && $input['sms'] == 1 && isset($input['cell_phone'])) {
                        // Twilio's credentials are fetched from company table
                        $getCompanyTwilioData           = getCompanyTwilioData();
                        if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                            $message                    = "Your account has been created successfully on NGD. Click here to login: ".route('driver.admin.login');
                            sendSMS($input['cell_phone'], $message, $getCompanyTwilioData);
                        }
                    }
                }else{
                    if(!Hash::check($password, $driver->password)){
                        // Send Email If password change
                        $mailData                       = [];
                        $mailData['company_id']         = auth()->guard('company-admin')->user()->company_id;
                        $mailData['name']               = $input['first_name'] . ' ' . $input['last_name'];
                        $mailData['email']              = $input['email'];
                        $mailData['password']           = $password;
                        //Mail::to($input['email'])->send(new NewPasswordMailToDriver($mailData));
                        sendEmail($input['email'], new NewPasswordMailToDriver($mailData));

                        // Send Sms If password change
                        if (isset($input['sms']) && $input['sms'] == 1 && isset($input['cell_phone'])) {
                            // Twilio's credentials are fetched from company table
                            $getCompanyTwilioData       = getCompanyTwilioData();
                            if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                                $message                = "Your account password has been updated successfully on NGD. Click here to login: ".route('driver.admin.login');
                                sendSMS($input['cell_phone'], $message, $getCompanyTwilioData);
                            }
                        }
                    }
                }
            }

            if(is_null($driver->driver_unique_id)){
                $input['driver_unique_id']              = getCompIdentifier().$driver->id;
            }

            $driver->update($input);

            if($driver->wasChanged()){
                if(isset($driver->getChanges()['status'])){
                    $logdata                            = [];
                    $logdata['company_id']              = auth()->guard('company-admin')->user()->company_id;
                    $logdata['driver_id']               = $driver->id;
                    $logdata['status_id']               = $request->status;
                    DriverStatusLog::create($logdata);
                }
            }

            if(isset($driver->driver_information) && $driver->driver_information->truck_number != ""){
                $truck                                  = Truck::IsCompany()->where('unit', $driver->driver_information->truck_number)->first();

                if($truck->driver == $driver->id){
                    $truck->driver                      = null;
                    $truck->driver_assign_date          = null;
                    $truck->driver_assign_by            = null;
                    $truck->save();
                }else if($truck->co_driver == $driver->id){
                    $truck->co_driver                   = null;
                    $truck->co_driver_assign_date       = null;
                    $truck->co_driver_assign_by         = null;
                    $truck->save();
                }
            }else if(!empty($driver->driver_information()->withTrashed()) && isset($driver->driver_information()->withTrashed()->first()->truck_number) && $driver->driver_information()->withTrashed()->first()->truck_number != ""){
                $truck                                  = Truck::IsCompany()->where('unit', $driver->driver_information()->withTrashed()->first()->truck_number)->first();

                if($truck->driver == $driver->id){
                    $truck->driver                      = null;
                    $truck->driver_assign_date          = null;
                    $truck->driver_assign_by            = null;
                    $truck->save();
                }else if($truck->co_driver == $driver->id){
                    $truck->co_driver                   = null;
                    $truck->co_driver_assign_date       = null;
                    $truck->co_driver_assign_by         = null;
                    $truck->save();
                }
            }

            if($driver->status != '2' && $input['truck_number'] != ""){
                $truck                                  = Truck::IsCompany()->where('unit', $input['truck_number'])->first();

                if($truck->driver == ""){
                    $truck->driver                      = $driver->id;
                    $truck->driver_assign_date          = Carbon::now()->format('m/d/Y');
                    $truck->driver_assign_by            = auth()->guard('company-admin')->user()->id;
                    $truck->save();
                }else if($truck->co_driver == ""){
                    $truck->co_driver                   = $driver->id;
                    $truck->co_driver_assign_date       = Carbon::now()->format('m/d/Y');
                    $truck->co_driver_assign_by         = auth()->guard('company-admin')->user()->id;
                    $truck->save();
                }
            }else{
                $input['truck_number']                  = null;

                if(isset($driver->driver_information)){
                    $driver->driver_information->update(['truck_number' => null]);
                }else{
                    if(!empty($driver->driver_information()->withTrashed())){
                        $driver->driver_information()->withTrashed()->update(['truck_number' => null]);
                    }
                }
            }

            if($request->has('trailer_number')){
                if(isset($driver->driver_information->trailer_number) && $driver->driver_information->trailer_number != $input['trailer_number']){
                    if(isset($driver->driver_information->trailer)){
                        $driver->driver_information->trailer->update(['current_assigned_driver' => null]);
                    }
                }
                if(isset($input['trailer_number'])){
                    Trailer::where('trailer_id', $input['trailer_number'])->update(['current_assigned_driver' => $driver->id]);
                }
            }

            $driverInformation                          = DriverInformation::withTrashed()->updateOrCreate(['driver_id' => $driver->id], $input);
            if($driver->status == '2'){
                $driverInformation->delete();
            }else{
                if(!empty($driver->driver_information()->withTrashed())){
                    $driver->driver_information()->withTrashed()->restore();
                }
            }
        }else{
            $password                                   = $input['password'];
            $input['password']                          = Hash::make($password);

            $driver                                     = Driver::create($input);
            if($driver->wasChanged()){
                if(isset($driver->getChanges()['status'])){
                    $logdata                            = [];
                    $logdata['company_id']              = auth()->guard('company-admin')->user()->company_id;
                    $logdata['driver_id']               = $driver->id;
                    $logdata['status_id']               = $request->status;
                    DriverStatusLog::create($logdata);
                }
            }

            DriverOnboarding::create(['driver_id' => $driver->id]);
            DriverFinancial::where('driver_id', ip2long(\Request::ip()))->update(['driver_id' => $driver->id]);

            $driver->driver_unique_id                   = getCompIdentifier().$driver->id;
            $driver->save();

            if($driver->wasChanged()){
                if(isset($driver->getChanges()['status'])){
                    $logdata                            = [];
                    $logdata['company_id']              = auth()->guard('company-admin')->user()->company_id;
                    $logdata['driver_id']               = $driver->id;
                    $logdata['status_id']               = $request->status;
                    DriverStatusLog::create($logdata);
                }
            }

            $input['driver_id']                         = $driver->id;

            $driverInformation                          = DriverInformation::create($input);

            if($driver->status != '2' && $driverInformation->truck_number != "") {
                $truck                                  = Truck::IsCompany()->where('unit', $driverInformation->truck_number)->first();

                if ($truck->driver == "") {
                    $truck->driver                      = $driver->id;
                    $truck->driver_assign_date          = Carbon::now()->format('m/d/Y');
                    $truck->driver_assign_by            = auth()->guard('company-admin')->user()->id;
                    $truck->save();
                } else if ($truck->co_driver == "") {
                    $truck->co_driver                   = $driver->id;
                    $truck->co_driver_assign_date       = Carbon::now()->format('m/d/Y');
                    $truck->co_driver_assign_by         = auth()->guard('company-admin')->user()->id;
                    $truck->save();
                }
            }

            if(isset($input['trailer_number'])){
                Trailer::where('trailer_id', $input['trailer_number'])->update(['current_assigned_driver' => $driver->id]);
            }

            if($driverManager){
                createNotification(
                    5, // source
                    'Attention!', // name
                    $driverManager->id, // user_id
                    $driverManager->type, // user_type
                    2, // type
                    $driver->id, // action_id
                    'New driver added', // description
                    'Alert', // module
                    [], // properties
                    $driverManager->company_id // company_id
                );
            }

            $checkNotify                                = isUnreadDelay($driverManager->id, $driverManager->type, 5);
            if(isset($checkNotify)){
                $getUser                                = CompanyAdmin::find($driverManager->id);
                if ($checkNotify->email == 1 && isset($driver->email)) {
                    $mailData                           = [];
                    $mailData['company_id']              = auth()->guard('company-admin')->user()->company_id;
                    $mailData['email']                  = $getUser->email;
                    $mailData['name']                   = $getUser->full_name;
                    $mailData['phone']                  = $getUser->cell_phone;
                    $mailData['msg']                    = 'New driver added - '.$driver->full_name;
                    $mailData['url']                    = route('company.admin.notifications.index');
                    //Mail::to($input['email'])->send(new UnreadNotificationMailToUser($mailData));
                    sendEmail($input['email'], new UnreadNotificationMailToUser($mailData));
                }
                if ($checkNotify->sms == 1 && isset($userdata->cell_phone)) {
                    // Twilio's credentials are fetched from company table
                    $getCompanyTwilioData               = getCompanyTwilioData();
                    if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                        $message                        = "New driver added - ".$driver->full_name;
                        sendSMS($input['cell_phone'], $message, $getCompanyTwilioData);
                    }
                }
            }

            if ($input['allow_login'] == 1) {
                // Send Email
                $mailData                               = [];
                $mailData['company_id']                 = auth()->guard('company-admin')->user()->company_id;
                $mailData['name']                       = $input['first_name'].' '.$input['last_name'];
                $mailData['email']                      = $input['email'];
                $mailData['password']                   = $password;
                //Mail::to($input['email'])->send(new WelcomeMailToDriver($mailData));
                sendEmail($input['email'], new WelcomeMailToDriver($mailData));

                // Send SMS
                if (isset($input['sms']) && $input['sms'] == 1 && isset($input['cell_phone'])) {
                    // Twilio's credentials are fetched from company table
                    $getCompanyTwilioData               = getCompanyTwilioData();
                    if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                        $message                        = "Your account has been created successfully on NGD. Click here to login: ".route('driver.admin.login');
                        sendSMS($input['cell_phone'], $message, $getCompanyTwilioData);
                    }
                }
            }

            $driverOnboarding                           = '{[]}';
            $filtered_collection                        = null;

            $renderDriverOnboardingView                 = view('companyAdmin.driver.tab.driverOnboarding', compact('driverOnboarding', 'filtered_collection', 'driver'))->render();
        }

        if($driver){
            return response()->json(['success' => true, 'msg' => 'Driver General Info saved successfully', 'driverId' => $driver->id, 'renderDriverOnboardingView' => $renderDriverOnboardingView]);
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Driver General Info']);
        }
    }

    public function saveCompliance(Request $request){
        $input                                          = $request->all();

        if($request->driver_id != ''){
            $input['license_exp']                       = $request->license_expiration;
            $input['mvr_date']                          = $request->last_mvr_date;
            $input['anniversary']                       = $request->anniversary_date;
            $input['medical_cert']                      = $request->medical_certificate_expiration;
            $input['license_endorsements']              = (!is_null($request->license_endorsements) && count($request->license_endorsements) > 0) ? implode(',', $request->license_endorsements) : null;

            if(isset($input['status'])){
                if($input['status'] == '2'){
                    $input['termination_date']          = Carbon::parse($input['termination_date'])->format('Y-m-d H:i:s') ?? Carbon::now()->format('Y-m-d H:i:s');
                    $input['termination_reason']        = 'Updated Status';
                    $input['termination_rehire']        = '1';
                    $input['termination_equipment']     = '0';
                    $input['termination_none']          = null;
                    $input['deleted_at']                = Carbon::now()->format('Y-m-d H:i:s');
                }else{
                    $input['termination_date']          = null;
                    $input['termination_reason']        = null;
                    $input['termination_rehire']        = null;
                    $input['termination_equipment']     = null;
                    $input['termination_none']          = null;
                    $input['deleted_at']                = null;
                }
            }

            if(isset($input['status'])){
                if($input['status'] == '1'){
                    $input['oos_date']                  = Carbon::now()->format('Y-m-d H:i:s');
                }else{
                    $input['oos_date']                  = null;
                }
            }

            if(isset( $input['license_number']) &&  $input['license_number']!=""){
                $input['license_number']                = base64_encode($input['license_number']);
            }

            if(isset( $input['passport_number']) &&  $input['passport_number']!=""){
                $input['passport_number']               = base64_encode($input['passport_number']);
            }

            $driver                                     = Driver::updateOrCreate(['id' => $request->driver_id, 'company_id' => auth()->guard('company-admin')->user()->company_id], $input);

            if($driver->wasChanged()){
                if(isset($driver->getChanges()['status'])){
                    $logdata                            = [];
                    $logdata['company_id']              = auth()->guard('company-admin')->user()->company_id;
                    $logdata['driver_id']               = $driver->id;
                    $logdata['status_id']               = $request->status;
                    DriverStatusLog::create($logdata);
                }
            }

            if(!is_null($input['dummy_id']) && isset($input['dummy_id']) && $input['dummy_id'] != $driver->id){
                $dummyloadid                            = $input['dummy_id'];
                Gallery::where('action_id', $dummyloadid)->update(['action_id' => $driver->id]);
            }

            if($driver){
                if($driver->status == '2'){
                    if(isset($driver->driver_information) && $driver->driver_information->truck_number != ""){
                        $truck                          = Truck::IsCompany()->where('unit', $driver->driver_information->truck_number)->first();

                        if($truck->driver == $driver->id){
                            $truck->driver              = null;
                            $truck->driver_assign_date  = null;
                            $truck->driver_assign_by    = null;
                            $truck->save();
                        }else if($truck->co_driver == $driver->id){
                            $truck->co_driver           = null;
                            $truck->co_driver_assign_date = null;
                            $truck->co_driver_assign_by = null;
                            $truck->save();
                        }

                        $driver->driver_information->update(['truck_number' => null]);
                    }else if(!empty($driver->driver_information()->withTrashed()) && isset($driver->driver_information()->withTrashed()->first()->truck_number) && $driver->driver_information()->withTrashed()->first()->truck_number != ""){
                        $truck                          = Truck::IsCompany()->where('unit', $driver->driver_information()->withTrashed()->first()->truck_number)->first();

                        if($truck->driver == $driver->id){
                            $truck->driver              = null;
                            $truck->driver_assign_date  = null;
                            $truck->driver_assign_by    = null;
                            $truck->save();
                        }else if($truck->co_driver == $driver->id){
                            $truck->co_driver           = null;
                            $truck->co_driver_assign_date = null;
                            $truck->co_driver_assign_by = null;
                            $truck->save();
                        }

                        $driver->driver_information()->withTrashed()->update(['truck_number' => null]);
                    }
                }
                return response()->json(['success' => true, 'msg' => 'Driver Compliance saved successfully', 'driverId' => $driver->id]);
            }else{
                return response()->json(['success' => false, 'msg' => 'Error, on save Driver Compliance']);
            }
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Driver Compliance']);
        }
    }

    public function saveInsurance(Request $request){
        $input                                          = $request->all();

        if($request->driver_id != ''){
            $input['insurance_type']                    = serialize($request->insurance_type);
            $input['insurance_provider']                = serialize($request->insurance_provider);
            $input['policy_number']                     = serialize($request->policy_number);
            $input['insurance_expiration']              = serialize($request->insurance_expiration);

            if(isset($input['status'])){
                if($input['status'] == '2'){
                    $input['termination_date']          = Carbon::parse($input['termination_date'])->format('Y-m-d H:i:s') ?? Carbon::now()->format('Y-m-d H:i:s');
                    $input['termination_reason']        = 'Updated Status';
                    $input['termination_rehire']        = '1';
                    $input['termination_equipment']     = '0';
                    $input['termination_none']          = null;
                    $input['deleted_at']                = Carbon::now()->format('Y-m-d H:i:s');
                }else{
                    $input['termination_date']          = null;
                    $input['termination_reason']        = null;
                    $input['termination_rehire']        = null;
                    $input['termination_equipment']     = null;
                    $input['termination_none']          = null;
                    $input['deleted_at']                = null;
                }
            }

            if(isset($input['status'])){
                if($input['status'] == '1'){
                    $input['oos_date']                  = Carbon::now()->format('Y-m-d H:i:s');
                }else{
                    $input['oos_date']                  = null;
                }
            }

            /*
            $insuranceFile                              = [];
            foreach ($input['insurance_file'] as $file){
                if($file != "undefined"){
                    $insuranceFile[]                    = FileUpload::upload('companyAdmin/driver', $file);
                }else{
                    $insuranceFile[]                    = null;
                }
            }
            $input['insurance_file']                    = (count($insuranceFile) > 0) ? serialize($insuranceFile) : null;
            */

            $driver                                     = Driver::updateOrCreate(['id' => $request->driver_id, 'company_id' => auth()->guard('company-admin')->user()->company_id], $input);

            if($driver->wasChanged()){
                if(isset($driver->getChanges()['status'])){
                    $logdata                            = [];
                    $logdata['company_id']              = auth()->guard('company-admin')->user()->company_id;
                    $logdata['driver_id']               = $driver->id;
                    $logdata['status_id']               = $request->status;
                    DriverStatusLog::create($logdata);
                }
            }

            if(!is_null($input['dummy_id']) && isset($input['dummy_id']) && $input['dummy_id'] != $driver->id){
                $dummyloadid                            = $input['dummy_id'];
                Gallery::where('action_id', $dummyloadid)->update(['action_id' => $driver->id]);
            }

            if($driver){
                if($driver->status == '2'){
                    if(isset($driver->driver_information) && $driver->driver_information->truck_number != ""){
                        $truck                          = Truck::IsCompany()->where('unit', $driver->driver_information->truck_number)->first();

                        if($truck->driver == $driver->id){
                            $truck->driver              = null;
                            $truck->driver_assign_date  = null;
                            $truck->driver_assign_by    = null;
                            $truck->save();
                        }else if($truck->co_driver == $driver->id){
                            $truck->co_driver           = null;
                            $truck->co_driver_assign_date = null;
                            $truck->co_driver_assign_by = null;
                            $truck->save();
                        }

                        $driver->driver_information->update(['truck_number' => null]);
                    }else if(!empty($driver->driver_information()->withTrashed()) && isset($driver->driver_information()->withTrashed()->first()->truck_number) && $driver->driver_information()->withTrashed()->first()->truck_number != ""){
                        $truck                          = Truck::IsCompany()->where('unit', $driver->driver_information()->withTrashed()->first()->truck_number)->first();

                        if($truck->driver == $driver->id){
                            $truck->driver              = null;
                            $truck->driver_assign_date  = null;
                            $truck->driver_assign_by    = null;
                            $truck->save();
                        }else if($truck->co_driver == $driver->id){
                            $truck->co_driver           = null;
                            $truck->co_driver_assign_date = null;
                            $truck->co_driver_assign_by = null;
                            $truck->save();
                        }

                        $driver->driver_information()->withTrashed()->update(['truck_number' => null]);
                    }
                }

                return response()->json(['success' => true, 'msg' => 'Driver Insurance saved successfully', 'driverId' => $driver->id]);
            }else{
                return response()->json(['success' => false, 'msg' => 'Error, on save Driver Insurance']);
            }
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Driver Insurance']);
        }
    }

    public function saveStateMap(Request $request){
        $input                                          = $request->all();

        if($request->driver_id != ''){
            $input['restricted_states']                 = [];
            $input['states_to_avoid']                   = [];

            foreach (json_decode($input['dataArr']) as $data){
                if ($data->value === ''){
                    $input['restricted_states'][]       = $data->region;
                }else if($data->value === '1'){
                    $input['states_to_avoid'][]         = $data->region;
                }
            }

            if(isset($input['status'])){
                if($input['status'] == '2'){
                    $input['termination_date']          = Carbon::parse($input['termination_date'])->format('Y-m-d H:i:s') ?? Carbon::now()->format('Y-m-d H:i:s');
                    $input['termination_reason']        = 'Updated Status';
                    $input['termination_rehire']        = '1';
                    $input['termination_equipment']     = '0';
                    $input['termination_none']          = null;
                    $input['deleted_at']                = Carbon::now()->format('Y-m-d H:i:s');
                }else{
                    $input['termination_date']          = null;
                    $input['termination_reason']        = null;
                    $input['termination_rehire']        = null;
                    $input['termination_equipment']     = null;
                    $input['termination_none']          = null;
                    $input['deleted_at']                = null;
                }
            }

            if(isset($input['status'])){
                if($input['status'] == '1'){
                    $input['oos_date']                  = Carbon::now()->format('Y-m-d H:i:s');
                }else{
                    $input['oos_date']                  = null;
                }
            }

            $driver                                     = Driver::updateOrCreate(['id' => $request->driver_id, 'company_id' => auth()->guard('company-admin')->user()->company_id], $input);

            if($driver->wasChanged()){
                if(isset($driver->getChanges()['status'])){
                    $logdata                            = [];
                    $logdata['company_id']              = auth()->guard('company-admin')->user()->company_id;
                    $logdata['driver_id']               = $driver->id;
                    $logdata['status_id']               = $request->status;
                    DriverStatusLog::create($logdata);
                }
            }

            if (!empty($input['restricted_states'])) {
                foreach ($input['restricted_states'] as $restrictedState) {
                    $rsIds[]                            = $restrictedState;
                    $rsExist                            = DriverRestrictedState::where('driver_id', $driver->id)->where('state_id', $restrictedState)->first();
                    if (is_null($rsExist)) {
                        DriverRestrictedState::create([
                            'driver_id'                 => $driver->id,
                            'state_id'                  => $restrictedState
                        ]);
                    }
                }
                DriverRestrictedState::where('driver_id', $driver->id)->whereNotIn('state_id', $rsIds)->delete();
            } else {
                DriverRestrictedState::where('driver_id', $driver->id)->delete();
            }

            if (!empty($input['states_to_avoid'])) {
                foreach ($input['states_to_avoid'] as $avoidState) {
                    $asIds[]                            = $avoidState;
                    $asExist                            = DriverAvoidState::where('driver_id', $driver->id)->where('state_id', $avoidState)->first();
                    if (is_null($asExist)) {
                        DriverAvoidState::create([
                            'driver_id'                 => $driver->id,
                            'state_id'                  => $avoidState
                        ]);
                    }
                }
                DriverAvoidState::where('driver_id', $driver->id)->whereNotIn('state_id', $asIds)->delete();
            } else {
                DriverAvoidState::where('driver_id', $driver->id)->delete();
            }

            if($driver){
                if($driver->status == '2'){
                    if(isset($driver->driver_information) && $driver->driver_information->truck_number != ""){
                        $truck                          = Truck::IsCompany()->where('unit', $driver->driver_information->truck_number)->first();

                        if($truck->driver == $driver->id){
                            $truck->driver              = null;
                            $truck->driver_assign_date  = null;
                            $truck->driver_assign_by    = null;
                            $truck->save();
                        }else if($truck->co_driver == $driver->id){
                            $truck->co_driver           = null;
                            $truck->co_driver_assign_date = null;
                            $truck->co_driver_assign_by = null;
                            $truck->save();
                        }

                        $driver->driver_information->update(['truck_number' => null]);
                    }else if(!empty($driver->driver_information()->withTrashed()) && isset($driver->driver_information()->withTrashed()->first()->truck_number) && $driver->driver_information()->withTrashed()->first()->truck_number != ""){
                        $truck                          = Truck::IsCompany()->where('unit', $driver->driver_information()->withTrashed()->first()->truck_number)->first();

                        if($truck->driver == $driver->id){
                            $truck->driver              = null;
                            $truck->driver_assign_date  = null;
                            $truck->driver_assign_by    = null;
                            $truck->save();
                        }else if($truck->co_driver == $driver->id){
                            $truck->co_driver           = null;
                            $truck->co_driver_assign_date = null;
                            $truck->co_driver_assign_by = null;
                            $truck->save();
                        }

                        $driver->driver_information()->withTrashed()->update(['truck_number' => null]);
                    }
                }

                return response()->json(['success' => true, 'msg' => 'Driver State Map saved successfully', 'driverId' => $driver->id]);
            }else{
                return response()->json(['success' => false, 'msg' => 'Error, on save Driver State Map']);
            }
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Driver State Map']);
        }
    }

    public function saveDeductions(Request $request){
        $input                                          = $request->all();

        if($request->driver_id != ''){
            $input['deduction_name']                    = serialize($request->deduction_name);
            $input['deduction_type']                    = serialize($request->deduction_type);

            $deductionAmount                            = [];
            foreach ($request->deduction_type as $key => $deductionType){
                if($deductionType == '0'){
                    $deductionAmount[]                  = $request->deduction_amount_per[$key];
                }else{
                    $deductionAmount[]                  = $request->deduction_amount_doller[$key];
                }
            }

            $input['deduction_amount']                  = serialize($deductionAmount);
            $input['deduction_unit']                    = serialize($request->deduction_unit);

            $startingORCapAmount                        = [];
            foreach ($request->deduction_unit as $key => $deductionUnit){
                if($deductionUnit == '0' || $deductionUnit == '1'){
                    $startingORCapAmount[]              =  $request->starting_balance[$key];
                }else if($deductionUnit == '2'){
                    $startingORCapAmount[]              =  $request->cap_amount[$key];
                }else{
                    $startingORCapAmount[]              = null;
                }
            }
            $input['starting_or_cap_amount']            = serialize($startingORCapAmount);

            if(isset($input['status'])){
                if($input['status'] == '2'){
                    $input['termination_date']          = Carbon::parse($input['termination_date'])->format('Y-m-d H:i:s') ?? Carbon::now()->format('Y-m-d H:i:s');
                    $input['termination_reason']        = 'Updated Status';
                    $input['termination_rehire']        = '1';
                    $input['termination_equipment']     = '0';
                    $input['termination_none']          = null;
                    $input['deleted_at']                = Carbon::now()->format('Y-m-d H:i:s');
                }else{
                    $input['termination_date']          = null;
                    $input['termination_reason']        = null;
                    $input['termination_rehire']        = null;
                    $input['termination_equipment']     = null;
                    $input['termination_none']          = null;
                    $input['deleted_at']                = null;
                }
            }

            if(isset($input['status'])){
                if($input['status'] == '1'){
                    $input['oos_date']                  = Carbon::now()->format('Y-m-d H:i:s');
                }else{
                    $input['oos_date']                  = null;
                }
            }

            $driver                                     = Driver::updateOrCreate(['id' => $request->driver_id, 'company_id' => auth()->guard('company-admin')->user()->company_id], $input);

            if($driver->wasChanged()){
                if(isset($driver->getChanges()['status'])){
                    $logdata                            = [];
                    $logdata['company_id']              = auth()->guard('company-admin')->user()->company_id;
                    $logdata['driver_id']               = $driver->id;
                    $logdata['status_id']               = $request->status;
                    DriverStatusLog::create($logdata);
                }
            }

            if(!is_null($input['dummy_id']) && isset($input['dummy_id']) && $input['dummy_id'] != $driver->id){
                $dummyloadid                            = $input['dummy_id'];
                Gallery::where('action_id', $dummyloadid)->update(['action_id' => $driver->id]);
            }

            if($driver){
                if($driver->status == '2'){
                    if(isset($driver->driver_information) && $driver->driver_information->truck_number != ""){
                        $truck                          = Truck::IsCompany()->where('unit', $driver->driver_information->truck_number)->first();

                        if($truck->driver == $driver->id){
                            $truck->driver              = null;
                            $truck->driver_assign_date  = null;
                            $truck->driver_assign_by    = null;
                            $truck->save();
                        }else if($truck->co_driver == $driver->id){
                            $truck->co_driver           = null;
                            $truck->co_driver_assign_date = null;
                            $truck->co_driver_assign_by = null;
                            $truck->save();
                        }

                        $driver->driver_information->update(['truck_number' => null]);
                    }else if(!empty($driver->driver_information()->withTrashed()) && isset($driver->driver_information()->withTrashed()->first()->truck_number) && $driver->driver_information()->withTrashed()->first()->truck_number != ""){
                        $truck                          = Truck::IsCompany()->where('unit', $driver->driver_information()->withTrashed()->first()->truck_number)->first();

                        if($truck->driver == $driver->id){
                            $truck->driver              = null;
                            $truck->driver_assign_date  = null;
                            $truck->driver_assign_by    = null;
                            $truck->save();
                        }else if($truck->co_driver == $driver->id){
                            $truck->co_driver           = null;
                            $truck->co_driver_assign_date = null;
                            $truck->co_driver_assign_by = null;
                            $truck->save();
                        }

                        $driver->driver_information()->withTrashed()->update(['truck_number' => null]);
                    }
                }

                return response()->json(['success' => true, 'msg' => 'Deductions saved successfully', 'driverId' => $driver->id]);
            }else{
                return response()->json(['success' => false, 'msg' => 'Error, on save Deductions']);
            }
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Deductions']);
        }
    }

    public function saveHr(Request $request){
        $input                                          = $request->all();

        if($request->driver_id != ''){
            $input['days_per_year']                     = serialize($request->days_per_year);
            $input['max_consecutive_days']              = serialize($request->max_consecutive_days);
            $input['leave_type']                        = serialize($request->leave_type);
            $input['hire_date']                         = serialize($request->hire_date);
            $input['reset_date']                        = serialize($request->reset_date);
            $input['paid_type']                         = serialize($request->paid_type);
            $input['pto_rollover']                      = serialize($request->pto_rollover);
            $input['max_accumulated_pto']               = serialize($request->max_accumulated_pto);
            $input['allow_unpaid_leave']                = serialize($request->allow_unpaid_leave);
            $input['allow_half_days']                   = serialize($request->allow_half_days);

            $input['passenger_onboard']                 = serialize($request->passenger_onboard);
            $input['passenger_agreement_On_file']       = serialize($request->passenger_agreement_On_file);
            $input['passenger_expiration']              = serialize($request->passenger_expiration);
            $input['passenger_full_name']               = serialize($request->passenger_full_name);
            $input['passenger_age']                     = serialize($request->passenger_age);
            $input['passenger_phone_number']            = serialize($request->passenger_phone_number);
            $input['passenger_relationship']            = serialize($request->passenger_relationship);
            $input['passenger_gender']                  = serialize($request->passenger_gender);
            $input['passenger_allergies']               = serialize($request->passenger_allergies);

            $input['pet_onboard']                       = serialize($request->pet_onboard);
            $input['pet_type']                          = serialize($request->pet_type);
            $input['pet_name']                          = serialize($request->pet_name);
            $input['pet_deposit_amount']                = serialize($request->pet_deposit_amount);

            $input['medical_condition']                 = serialize($request->medical_condition);
            $input['driving_restrictions']              = serialize($request->driving_restrictions);
            $input['next_doctor_visit']                 = serialize($request->next_doctor_visit);

            if(isset($input['status'])){
                if($input['status'] == '2'){
                    $input['termination_date']          = Carbon::parse($input['termination_date'])->format('Y-m-d H:i:s') ?? Carbon::now()->format('Y-m-d H:i:s');
                    $input['termination_reason']        = 'Updated Status';
                    $input['termination_rehire']        = '1';
                    $input['termination_equipment']     = '0';
                    $input['termination_none']          = null;
                    $input['deleted_at']                = Carbon::now()->format('Y-m-d H:i:s');
                }else{
                    $input['termination_date']          = null;
                    $input['termination_reason']        = null;
                    $input['termination_rehire']        = null;
                    $input['termination_equipment']     = null;
                    $input['termination_none']          = null;
                    $input['deleted_at']                = null;
                }
            }

            if(isset($input['status'])){
                if($input['status'] == '1'){
                    $input['oos_date']                  = Carbon::now()->format('Y-m-d H:i:s');
                }else{
                    $input['oos_date']                  = null;
                }
            }

            $driver                                     = Driver::updateOrCreate(['id' => $request->driver_id, 'company_id' => auth()->guard('company-admin')->user()->company_id], $input);

            if($driver->wasChanged()){
                if(isset($driver->getChanges()['status'])){
                    $logdata                            = [];
                    $logdata['company_id']              = auth()->guard('company-admin')->user()->company_id;
                    $logdata['driver_id']               = $driver->id;
                    $logdata['status_id']               = $request->status;
                    DriverStatusLog::create($logdata);
                }
            }

            if(!is_null($input['dummy_id']) && isset($input['dummy_id']) && $input['dummy_id'] != $driver->id){
                $dummyloadid                            = $input['dummy_id'];
                Gallery::where('action_id', $dummyloadid)->update(['action_id' => $driver->id]);
            }

            if($driver){
                if($driver->status == '2'){
                    if(isset($driver->driver_information) && $driver->driver_information->truck_number != ""){
                        $truck                          = Truck::IsCompany()->where('unit', $driver->driver_information->truck_number)->first();

                        if($truck->driver == $driver->id){
                            $truck->driver              = null;
                            $truck->driver_assign_date  = null;
                            $truck->driver_assign_by    = null;
                            $truck->save();
                        }else if($truck->co_driver == $driver->id){
                            $truck->co_driver           = null;
                            $truck->co_driver_assign_date = null;
                            $truck->co_driver_assign_by = null;
                            $truck->save();
                        }

                        $driver->driver_information->update(['truck_number' => null]);
                    }else if(!empty($driver->driver_information()->withTrashed()) && isset($driver->driver_information()->withTrashed()->first()->truck_number) && $driver->driver_information()->withTrashed()->first()->truck_number != ""){
                        $truck                          = Truck::IsCompany()->where('unit', $driver->driver_information()->withTrashed()->first()->truck_number)->first();

                        if($truck->driver == $driver->id){
                            $truck->driver              = null;
                            $truck->driver_assign_date  = null;
                            $truck->driver_assign_by    = null;
                            $truck->save();
                        }else if($truck->co_driver == $driver->id){
                            $truck->co_driver           = null;
                            $truck->co_driver_assign_date = null;
                            $truck->co_driver_assign_by = null;
                            $truck->save();
                        }

                        $driver->driver_information()->withTrashed()->update(['truck_number' => null]);
                    }
                }

                return response()->json(['success' => true, 'msg' => 'Hr saved successfully', 'driverId' => $driver->id]);
            }else{
                return response()->json(['success' => false, 'msg' => 'Error, on save Hr']);
            }
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Hr']);
        }
    }

    public function saveEmergencyContacts(Request $request){
        $input                                                  = $request->all();

        if($request->driver_id != ''){
            $input['emergency_contact_name']                    = serialize($request->emergency_contact_name);
            $input['emergency_address']                         = serialize($request->emergency_address);
            $input['emergency_contact_number']                  = serialize($request->emergency_cell_phone);
            $input['emergency_home_phone']                      = serialize($request->emergency_home_phone);
            $input['emergency_contact_relationship_to_driver']  = serialize($request->emergency_relationship);

            if(isset($input['status'])){
                if($input['status'] == '2'){
                    $input['termination_date']                  = Carbon::parse($input['termination_date'])->format('Y-m-d H:i:s') ?? Carbon::now()->format('Y-m-d H:i:s');
                    $input['termination_reason']                = 'Updated Status';
                    $input['termination_rehire']                = '1';
                    $input['termination_equipment']             = '0';
                    $input['termination_none']                  = null;
                    $input['deleted_at']                        = Carbon::now()->format('Y-m-d H:i:s');
                }else{
                    $input['termination_date']                  = null;
                    $input['termination_reason']                = null;
                    $input['termination_rehire']                = null;
                    $input['termination_equipment']             = null;
                    $input['termination_none']                  = null;
                    $input['deleted_at']                        = null;
                }
            }

            if(isset($input['status'])){
                if($input['status'] == '1'){
                    $input['oos_date']                          = Carbon::now()->format('Y-m-d H:i:s');
                }else{
                    $input['oos_date']                          = null;
                }
            }

            $driver                                             = Driver::updateOrCreate(['id' => $request->driver_id, 'company_id' => auth()->guard('company-admin')->user()->company_id], $input);

            if($driver->wasChanged()){
                if(isset($driver->getChanges()['status'])){
                    $logdata                                    = [];
                    $logdata['company_id']                      = auth()->guard('company-admin')->user()->company_id;
                    $logdata['driver_id']                       = $driver->id;
                    $logdata['status_id']                       = $request->status;
                    DriverStatusLog::create($logdata);
                }
            }

            if($driver){
                if($driver->status == '2'){
                    if(isset($driver->driver_information) && $driver->driver_information->truck_number != ""){
                        $truck                                  = Truck::IsCompany()->where('unit', $driver->driver_information->truck_number)->first();

                        if($truck->driver == $driver->id){
                            $truck->driver                      = null;
                            $truck->driver_assign_date          = null;
                            $truck->driver_assign_by            = null;
                            $truck->save();
                        }else if($truck->co_driver == $driver->id){
                            $truck->co_driver                   = null;
                            $truck->co_driver_assign_date       = null;
                            $truck->co_driver_assign_by         = null;
                            $truck->save();
                        }

                        $driver->driver_information->update(['truck_number' => null]);
                    }else if(!empty($driver->driver_information()->withTrashed()) && isset($driver->driver_information()->withTrashed()->first()->truck_number) && $driver->driver_information()->withTrashed()->first()->truck_number != ""){
                        $truck                                  = Truck::IsCompany()->where('unit', $driver->driver_information()->withTrashed()->first()->truck_number)->first();

                        if($truck->driver == $driver->id){
                            $truck->driver                      = null;
                            $truck->driver_assign_date          = null;
                            $truck->driver_assign_by            = null;
                            $truck->save();
                        }else if($truck->co_driver == $driver->id){
                            $truck->co_driver                   = null;
                            $truck->co_driver_assign_date       = null;
                            $truck->co_driver_assign_by         = null;
                            $truck->save();
                        }

                        $driver->driver_information()->withTrashed()->update(['truck_number' => null]);
                    }
                }

                return response()->json(['success' => true, 'msg' => 'Driver  Emergency Contacts successfully', 'driverId' => $driver->id]);
            }else{
                return response()->json(['success' => false, 'msg' => 'Error, on save Driver Emergency Contacts']);
            }
        }else{
            return response()->json(['success' => false, 'msg' => 'Error, on save Driver Emergency Contacts']);
        }
    }

    public function deleteGalleryFiles(Request $request){
        return Gallery::where('action_id', $request->dataId)->where('source', $request->source)->where('field_name', $request->field)->delete();
    }

    public function stateListForMapNew(Request $request){
        $states         = State::get();
        $data           = [];
        foreach ($states as $key => $state) {
            if (!is_null($request->restricted_states) && in_array($state->id, $request->restricted_states)) {
                $data[] = [$state->code, 2, $state->name, 'Block'];
            } elseif (!is_null($request->avoid_states) && in_array($state->id, $request->avoid_states)) {
                $data[] = [$state->code, 1, $state->name, 'Preferred'];
            } else {
                $data[] = [$state->code, 0, $state->name, 'Open'];
            }
        }
        return response()->json(['data' => $data]);
    }

    public function mileageRateStateListForMap_backup(Request $request){
        $states         = State::pluck('name', 'code')->toArray();

        $data           = [];
        foreach ($states as $statesCode => $stateName) {
            if(in_array($statesCode, ['CA', 'IL', 'OH', 'PA', 'VA', 'NC'])){
                $data[] = [$statesCode, 1, $stateName, '']; // #174D99
            }else if(in_array($statesCode, ['OR', 'ID', 'NV', 'UT', 'NE', 'KS', 'ME'])){
                $data[] = [$statesCode, 2, $stateName, '']; // #4DA5FF
            }else if(in_array($statesCode, ['WA', 'TX', 'MN', 'WI', 'MI', 'IN', 'KY', 'TN', 'GA', 'SC', 'LA', 'NH'])){
                $data[] = [$statesCode, 3, $stateName, '']; // #2266CC
            }else if(in_array($statesCode, ['MT', 'WY', 'NM'])){
                $data[] = [$statesCode, 4, $stateName, '']; // #B2D9FF
            }else if(in_array($statesCode, ['AZ', 'CO', 'OK', 'IA', 'MO', 'AR', 'MS', 'AL', 'WV', 'VT'])){
                $data[] = [$statesCode, 5, $stateName, '']; // #2C80FF
            }else if(in_array($statesCode, ['ND', 'SD'])){
                $data[] = [$statesCode, 6, $stateName, '']; // #8CC6FF
            }else if(in_array($statesCode, ['FL', 'NY', 'MD', 'DE', 'CT'])){
                $data[] = [$statesCode, 7, $stateName, '']; // #0C3366
            }else if(in_array($statesCode, ['MA', 'NJ'])){
                $data[] = [$statesCode, 8, $stateName, '']; // #031A33
            }else {
                $data[] = [$statesCode, 0, $stateName, '']; // #EAEAEA
            }
        }
        return response()->json(['data' => $data]);
    }

    public function mileageRateStateListForMap(Request $request){
        $states         = State::pluck('name', 'code')->toArray();

        $data           = [];
        foreach ($states as $statesCode => $stateName) {
            if(in_array($statesCode, ['CA', 'IL', 'OH', 'PA', 'VA', 'NC', 'OR', 'ID', 'NV', 'UT', 'NE', 'KS', 'ME', 'WA', 'TX', 'MN', 'WI', 'MI', 'IN', 'KY', 'TN', 'GA', 'SC', 'LA', 'NH', 'MT', 'WY', 'NM', 'AZ', 'CO', 'OK', 'IA', 'MO', 'AR', 'MS', 'AL', 'WV', 'VT', 'ND', 'SD', 'FL', 'NY', 'MD', 'DE', 'CT', 'MA', 'NJ'])){
                $data[] = [$statesCode, rand(0, 100000)];
            }else {
                $data[] = [$statesCode, 0];
            }
        }
        return response()->json(['data' => $data]);
    }

    public function addDeductionName(Request $request){
        $addDeductionName       = CompanyDeductionType::create(['company_id' => auth()->guard('company-admin')->user()->company_id, 'name' => $request->deduct_name]);

        if($addDeductionName){
            $dropdownHtml       = '<option value="">Select Name</option>';
            foreach (getDeductionNameList() as $id => $name){
                $dropdownHtml  .= '<option value="'.$id.'">'.$name.'</option>';
            }

            return response()->json(['success' => true, 'msg' => 'Deduction Name added successfully', 'dropdownHtml' => $dropdownHtml]);
        }else{
            return response()->json(['data' => false, 'msg' => 'Error on added Deduction Name']);
        }
    }

    public function updateDriverStatus(Request $request){
        $driver_id                          = $request->driver_id;
        $status                             = $request->status;

        $driver                             = Driver::withTrashed()->find($driver_id);
        $driver->status                     = $status;

        if($status == '2'){
            $driver->termination_date       = Carbon::now()->format('Y-m-d H:i:s');
            $driver->termination_reason     = 'Updated Status';
            $driver->termination_rehire     = '1';
            $driver->termination_equipment  = '0';
            $driver->termination_none       = null;
            $driver->deleted_at             = Carbon::now()->format('Y-m-d H:i:s');
        }else{
            $driver->termination_date       = null;
            $driver->termination_reason     = null;
            $driver->termination_rehire     = null;
            $driver->termination_equipment  = null;
            $driver->termination_none       = null;
            $driver->deleted_at             = null;
        }

        if($status == '1'){
            $driver->oos_date               = Carbon::now()->format('Y-m-d H:i:s');
        }else{
            $driver->oos_date               = null;
        }

        $driver->save();

        if($driver->wasChanged()){
            if(isset($driver->getChanges()['status'])){
                $logdata                    = [];
                $logdata['company_id']      = auth()->guard('company-admin')->user()->company_id;
                $logdata['driver_id']       = $driver->id;
                $logdata['status_id']       = $request->status;
                DriverStatusLog::create($logdata);
            }
        }

        if($driver){
            return response()->json(['success' => true, 'message' => 'Driver Status updated successfully.']);
        }else{
            return response()->json(['success' => false, 'message' => 'Oops! Something went wrong..']);
        }
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(DriverStoreRequest $request){
        $input                              = $request->all();

        if($request->address_verified==1){
            $timezone                       = null;
            $timezone_offset                = null;
            if(isset($input['timezone_abbr'])){
                $timezone                   = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset            = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']              = $timezone;
            $input['timezone_offset']       = $timezone_offset;
        }

        $input['company_id']                = auth()->guard('company-admin')->user()->company_id;
        $password                           = $input['password'];
        $input['password']                  = Hash::make($password);
        $input['weekly']                    = isset($input['joining_trailer_program']) ? $input['weekly'] : NULL;
        if (isset($input['profile_image'])) {
            $input['profile_image']         = FileUpload::upload('companyAdmin/driver', $input['profile_image']);
        }

        if (isset($input['license_exp_file'])) {
            $input['license_exp_file']      = FileUpload::upload('companyAdmin/driver', $input['license_exp_file']);
        }
        $driverManager                      = CompanyAdmin::where('id', $request->manager_id)->first();
        $team_id                            = null;
        if(!is_null($driverManager) && isset($driverManager->team)){
            $team_id                        = $driverManager->team->id;
        }
        $input['team_id']                   = $team_id;

        $input['mark_as_completed']         = isset($input['mark_as_completed']) ? 1 : 0;
        if(isset($input['is_company_driver']) && $input['is_company_driver']){
            $input['is_company_driver']     = 1;
        }else{
            $input['is_company_driver']     = 0;
        }

        if(isset( $input['license_number']) &&  $input['license_number']!=""){
            $input['license_number']        = base64_encode($input['license_number']);
        }
        if(isset( $input['passport_number']) &&  $input['passport_number']!=""){
            $input['passport_number']       = base64_encode($input['passport_number']);
        }
        if(isset( $input['ssn']) &&  $input['ssn']!=""){
            $input['ssn']                   = base64_encode($input['ssn']);
        }
        if(isset( $input['ein']) &&  $input['ein']!=""){
            $input['ein']                   = base64_encode($input['ein']);
        }

        $driver                             = Driver::create($input);
        if($driver->wasChanged()){
            if(isset($driver->getChanges()['status'])){
                $logdata = [];
                $logdata['company_id'] = auth()->guard('company-admin')->user()->company_id;
                $logdata['driver_id'] = $driver->id;
                $logdata['status_id'] = $request->status;
                DriverStatusLog::create($logdata);
            }
        }

        if(!is_null($input['dummy_id']) && isset($input['dummy_id'])){
            $dummyloadid                    = $input['dummy_id'];
            Gallery::where('action_id', $dummyloadid)->update(['action_id' => $driver->id]);
        }

        if (!empty($request->start_date)) {
            $driver_onboar                  = [];
            $driver_onboar['start_date']    = $request->start_date;
            $driver_onboar['status']        = 0;
            $driver_onboar['driver_id']     = $driver->id;
            $exist                          = DriverOnboarding::where("driver_id", $driver->id)->first();
            if (is_null($exist)) {
                $exist                      = DriverOnboarding::create($driver_onboar);
            } else {
                $exist->update($driver_onboar);
            }
        }

        DriverFinancial::where('driver_id', ip2long(\Request::ip()))->update(['driver_id' => $driver->id]);

        $input['driver_id']                 = $driver->id;
        DriverInformation::create($input);

        if (!empty($input['restricted_states'])) {
            foreach ($input['restricted_states'] as $restrictedState) {
                DriverRestrictedState::create([
                    'driver_id'             => $driver->id,
                    'state_id'              => $restrictedState
                ]);
            }
        }

        if (!empty($input['states_to_avoid'])) {
            foreach ($input['states_to_avoid'] as $avoidState) {
                DriverAvoidState::create([
                    'driver_id'             => $driver->id,
                    'state_id'              => $avoidState
                ]);
            }
        }

        // add alert
        if($driverManager){
            createNotification(
                5, // source
                'Attention!', // name
                $driverManager->id, // user_id
                $driverManager->type, // user_type
                2, // type
                $driver->id, // action_id
                'New driver added', // description
                'Alert', // module
                [], // properties
                $driverManager->company_id // company_id
            );
        }

        $checkNotify                        = isUnreadDelay($driverManager->id, $driverManager->type, 5);

        if(isset($checkNotify)){
            $getUser = $getUser = CompanyAdmin::find($driverManager->id);

            if ($checkNotify->email == 1 && isset($driver->email)) {
                $mailData                   = [];
                $mailData['company_id']     = auth()->guard('company-admin')->user()->company_id;
                $mailData['email']          = $getUser->email;
                $mailData['name']           = $getUser->full_name;
                $mailData['phone']          = $getUser->cell_phone;
                $mailData['msg']            = 'New driver added - '.$driver->full_name;
                $mailData['url']            = route('company.admin.notifications.index');
                //Mail::to($input['email'])->send(new UnreadNotificationMailToUser($mailData));
                sendEmail($input['email'], new UnreadNotificationMailToUser($mailData));
            }
            if ($checkNotify->sms == 1 && isset($userdata->cell_phone)) {
                // Twilio's credentials are fetched from company table
                $getCompanyTwilioData       = getCompanyTwilioData();
                if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                    $message                = "New driver added - ".$driver->full_name;
                    sendSMS($input['cell_phone'], $message, $getCompanyTwilioData);
                }
            }
        }

        $mailData                           = [];
        $mailData['company_id']             = auth()->guard('company-admin')->user()->company_id;
        $mailData['name']                   = $input['first_name'].' '.$input['last_name'];
        $mailData['email']                  = $input['email'];
        $mailData['password']               = $password;
        //Mail::to($input['email'])->send(new WelcomeMailToDriver($mailData));
        sendEmail($input['email'], new WelcomeMailToDriver($mailData));

        // send sms
        if (isset($input['sms']) && isset($input['cell_phone'])) {
            // Twilio's credentials are fetched from company table
            $getCompanyTwilioData           = getCompanyTwilioData();
            if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                $message                    = "Your account has been created successfully on NGD. Click here to login: ".route('driver.admin.login');
                sendSMS($input['cell_phone'], $message, $getCompanyTwilioData);
            }
        }

        $carrierId = (isset($driver->driver_information->carrier_id)) ? $driver->driver_information->carrier_id : null;
        $getAPIKey = IntegrationSetting::where('company_id', $driver->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->first();

        if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
            $samsara_api_key                = $getAPIKey->api_key;
            $endCursor                      = 0;
            return redirect()->route('company.admin.driver.sync.getsyncdriver', ['samsara_api_key' => $samsara_api_key, 'endCursor' => $endCursor]);
        }else{
            notification('success', $this->crudMessage('create', 'Driver'));
            return redirect()->route('company.admin.drivers.index');
        }
    }

    /**
     * Display the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show_backup(Request $request,$id){
        $id                                     = base64Decode($id);
        $driver                                 = Driver::withTrashed()->where('id',$id)->first();

        if($driver->license_number!=""){
            $driver->license_number             = base64_decode($driver->license_number);
        }


        $getLeaveUserType                       = LeaveUserType::IsCompany()->where('driver_id',$id)->where('year',date('Y'))->get();

        $cycleStartedAtTime                     = '00:00';
        $samsara_data                           = '';
        $vehical_data                           = 'None';
        if(!empty($driver->samsara_id)){
            $carrierId                          = (isset($driver->driver_information->carrier_id)) ? $driver->driver_information->carrier_id : null;
            $getAPIKey                          = IntegrationSetting::where('company_id', $driver->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->first();
            if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                $client                         = new \GuzzleHttp\Client();
                $api_url                        = 'https://api.samsara.com/fleet/hos/clocks?driverIds='.$driver->samsara_id;
                $response                       = $client->request('GET', $api_url, [
                    'headers'                   => [
                        'Accept'                => 'application/json',
                        'Authorization'         => 'Bearer '.$getAPIKey->api_key,
                    ],
                ]);

                $data                           = json_decode($response->getBody());
                $samsara_data                   = $data->data[0];

                if(isset($data->data[0])){
                    $dattime                    = $samsara_data->clocks->cycle->cycleStartedAtTime;
                }

                /*
                $explodeT                       = explode('T',$dattime);
                $explodeDot                     = explode('.',$explodeT[1]);
                $cycleStartedAtTime             = $explodeT[0].' '.$explodeDot[0];
                $cycleStartedAtTime             = getDiffInMinutes($cycleStartedAtTime);

                $cycleStartedAtTime             = $cycleStartedAtTime * 60 * 1000;
                $cycleStartedAtTime             = formatSeconds($cycleStartedAtTime);
                */


                $api_url                        = 'https://api.samsara.com/fleet/drivers/'.$driver->samsara_id;
                $response                       = $client->request('GET', $api_url, [
                    'headers'                   => [
                        'Accept'                => 'application/json',
                        'Authorization'         => 'Bearer '.$getAPIKey->api_key,
                    ],
                ]);

                $vdata                          = json_decode($response->getBody());
                if(isset($vdata->data->staticAssignedVehicle->name)){
                    $vehical_data               = $vdata->data->staticAssignedVehicle->name;
                }
            }
        }

        $driverInformation                      = $driver->driver_information;
        $driverHosViolations                    = $driver->driver_hos_violations()->latest()->take(3)->get();
        $driverNonViolations                    = $driver->driver_non_violations()->latest()->take(3)->get();
        $driverCitations                        = $driver->driver_citations()->latest()->take(3)->get();
        $driverAccidents                        = $driver->driver_accidents()->latest()->take(3)->get();
        $driverMechanicalIssues                 = $driver->driver_mechanical_issues()->latest()->take(3)->get();
        $complianceSafety                       = $driver->driver_compliance_safety;
        $driverOnboarding                       = !empty($driver->driver_onboarding) ? $driver->driver_onboarding : '{[]}';

        $driverHosViolationsCheckHOS            = DriverHosViolation::join('driver_hos_violations_datas as dhvd', function ($join) use ($driver) {
            $join->on('driver_hos_violations.id', '=', 'dhvd.driver_hos_violation_id')
                ->where('dhvd.unit_type', 1);
                //->where('driver_financials.quarter', $get_quarter);
            })
            ->where('driver_hos_violations.driver_id', $driver->id)
            ->groupBy('driver_hos_violations.id')
            ->select('driver_hos_violations.*')
            ->get();

        $driverHosViolationsCheckCitations      = DriverHosViolation::join('driver_hos_violations_datas as dhvd', function ($join) use ($driver) {
            $join->on('driver_hos_violations.id', '=', 'dhvd.driver_hos_violation_id')
                ->whereIn('dhvd.unit_type', [2,3]);
                //->where('driver_financials.quarter', $get_quarter);
            })
            ->where('driver_hos_violations.driver_id', $driver->id)
            ->groupBy('driver_hos_violations.id')
            ->select('driver_hos_violations.*')
            ->get();


        $driverNonViolationsCheckHOS            = DriverNonViolation::join('driver_non_violations_datas as dhvd', function ($join) use ($driver) {
            $join->on('driver_non_violations.id', '=', 'dhvd.driver_non_violation_id');
            })
            ->where('driver_non_violations.driver_id', $driver->id)
            ->groupBy('driver_non_violations.id')
            ->select('driver_non_violations.*')
            ->get();

        $filtered_collection                    = null;
        if (!is_null($driverOnboarding) && !empty($driverOnboarding->detail_json)) {
            $t                                  = collect($driverOnboarding->detail_json);
            $filtered_collection                = $t->filter(function ($item) {
                return $item['yes_no'] == 1 && $item['is_complete'] == 1;
            })->last();
        }

        $occupancyCalendar                      = [];

        $getData                                = Driver::select("load_details.*")
            ->join("load_details", "load_details.driver_id", "=", "drivers.id")
            ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
            ->whereNull('load_details.deleted_at')
            ->where('drivers.id', $driver->id)
            ->where('load_drivers.driver_id', $driver->id)
            ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id);

        $totalOccupancy                         = Driver::select("load_details.*")
            ->join("load_details", "load_details.driver_id", "=", "drivers.id")
            ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
            ->whereNull('load_details.deleted_at')
            ->where('drivers.id', $driver->id)
            ->where('load_drivers.driver_id', $driver->id)
            ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
            ->where('load_details.load_type', 0)
            ->count();

        $totalMaintenance                       = Driver::select("load_details.*")
            ->join("load_details", "load_details.driver_id", "=", "drivers.id")
            ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
            ->whereNull('load_details.deleted_at')
            ->where('drivers.id', $driver->id)
            ->where('load_drivers.driver_id', $driver->id)
            ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
            ->where('load_details.load_type', 1)
            ->count();

        $totalUnauthorized                      = Driver::select("load_details.*")
            ->join("load_details", "load_details.driver_id", "=", "drivers.id")
            ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
            ->whereNull('load_details.deleted_at')
            ->where('drivers.id', $driver->id)
            ->where('load_drivers.driver_id', $driver->id)
            ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
            ->where('load_details.load_type', 2)
            ->count();

        $totalHos                               = Driver::select("load_details.*")
            ->join("load_details", "load_details.driver_id", "=", "drivers.id")
            ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
            ->whereNull('load_details.deleted_at')
            ->where('drivers.id', $driver->id)
            ->where('load_drivers.driver_id', $driver->id)
            ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
            ->where('load_details.load_type', 3)
            ->count();

        $totalTimeOff                           = Driver::select("load_details.*")
            ->join("load_details", "load_details.driver_id", "=", "drivers.id")
            ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
            ->whereNull('load_details.deleted_at')
            ->where('drivers.id', $driver->id)
            ->where('load_drivers.driver_id', $driver->id)
            ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
            ->where('load_details.load_type', 5)
            ->count();

        $getData                                = $getData->get();
        $totalData                              = $getData->count();

        $occupancyCalendar['occupancy']         = 0;
        $occupancyCalendar['maintenance']       = 0;
        $occupancyCalendar['unauthorized']      = 0;
        $occupancyCalendar['hos']               = 0;
        $occupancyCalendar['timeoff']           = 0;

        if($totalData > 0){
            $occupancyCalendar['occupancy']     = round(($totalOccupancy * 100) / $totalData, 2);
            $occupancyCalendar['maintenance']   = round(($totalMaintenance * 100) / $totalData, 2);
            $occupancyCalendar['unauthorized']  = round(($totalUnauthorized * 100) / $totalData, 2);
            $occupancyCalendar['hos']           = round(($totalHos * 100) / $totalData, 2);
            $occupancyCalendar['timeoff']       = round(($totalTimeOff * 100) / $totalData, 2);
        }

        $calendarEvent                          = 1;
        $birthdays                              = 'on';
        $expirations                            = 'on';
        $maintenanceTrucks                      = 'on';
        $maintenanceTrailers                    = 'on';
        $hidePersonalEvents                     = 'on';
        $IsArray                                = 1;


        $getEvent                               = getDriverEventFullCalendar($driver->id,$calendarEvent,$birthdays,$expirations,$maintenanceTrucks,$maintenanceTrailers,$hidePersonalEvents,$IsArray);
        $current                                = Carbon::now()->timezone($this->timeZone);
        $currentDate                            = $current->format('Y-m-d');
        $currentDateEnd                         = $current->addDays(31)->format('Y-m-d');

        $period                                 = CarbonPeriod::create($currentDate, $currentDateEnd);
        $getEventByDays                         = [];
        foreach ($getEvent as $edata) {
            foreach ($period as $date) {
                if (@$date->between($edata['start'], @$edata['end'])) {
                    $getEventByDays[]           = $edata;
                }
            }
        }

        if (count($getEventByDays) > 0) {
            $getEventByDays                     = array_unique($getEventByDays, SORT_REGULAR);
            foreach ($getEventByDays as $key => $part) {
                $sort[$key]                     = strtotime($part['start']);
            }
            array_multisort($sort, SORT_DESC, $getEventByDays);
        }

        $now                                    = Carbon::now()->timezone($this->timeZone);
        $month_name                             = $now->format('F');
        $month                                  = $now->format('m');
        $year                                   = $now->format('Y');

        return view('companyAdmin.driver.show-bk-25072023', compact('driver', 'driverInformation', 'driverHosViolations','driverNonViolations', 'driverCitations', 'driverAccidents', 'driverMechanicalIssues', 'complianceSafety', 'driverOnboarding', 'filtered_collection', 'occupancyCalendar', 'getEventByDays','samsara_data','vehical_data','cycleStartedAtTime', 'getLeaveUserType','month_name','month','year','driverHosViolationsCheckHOS','driverHosViolationsCheckCitations','driverNonViolationsCheckHOS'));
    }

    public function getDriverSamsaraInfo($driver){
        $data                                           = [];
        $samsara_id                                     = (isset($driver->samsara_id) && $driver->samsara_id!='')?$driver->samsara_id:'';//4644765
        if($samsara_id!=''){
            $carrierId = (isset($driver->driver_information->carrier_id)) ? $driver->driver_information->carrier_id : null;
            $getAPIKey                                  = IntegrationSetting::where('company_id', $driver->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->first();
            if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                $client                                 = new \GuzzleHttp\Client(['verify' => false]);
                $api_url                                = 'https://api.samsara.com/fleet/hos/clocks?driverIds='.$samsara_id;
                $api_response                           = $client->request('GET', $api_url, [
                    'headers'                           => [
                        'Accept'                        => 'application/json',
                        'Authorization'                 => 'Bearer '.$getAPIKey->api_key,
                    ],
                ]);

                $api_data                               = json_decode($api_response->getBody());
                if(!empty($api_data)){
                    $appData                            = $api_data->data[0];
                    if(!empty($appData)){
                        $data['drive']                  = formatSeconds($appData->clocks->drive->driveRemainingDurationMs);
                        $data['shift']                  = formatSeconds($appData->clocks->shift->shiftRemainingDurationMs);
                        $data['cycle']                  = formatSeconds($appData->clocks->cycle->cycleRemainingDurationMs);
                        $data['cycle_tom']              = formatSeconds($appData->clocks->cycle->cycleTomorrowDurationMs);
                        $data['duty_status']            = $appData->currentDutyStatus->hosStatusType;
                        $data['time_in_current_status'] = '';
                        $data['time_in_unit_break']     = formatSeconds($appData->clocks->break->timeUntilBreakDurationMs);
                    }
                }
            }
        }
        return $data;
    }

    public function getDriverAccountStatistics($driver){
        $currentMonthStartDate                                      = Carbon::now()->timezone($this->timeZone)->startOfMonth()->format('Y-m-d');
        $currentMonthEndDate                                        = Carbon::now()->timezone($this->timeZone)->endOfMonth()->format('Y-m-d');
        $previousMonthStartDate                                     = Carbon::now()->timezone($this->timeZone)->subMonth()->startOfMonth()->format('Y-m-d');
        $previousMonthEndDate                                       = Carbon::now()->timezone($this->timeZone)->subMonth()->endOfMonth()->format('Y-m-d');
        $currentYearStartDate                                       = Carbon::now()->timezone($this->timeZone)->startOfYear()->format('Y-m-d');
        $currentYearEndDate                                         = Carbon::now()->timezone($this->timeZone)->endOfYear()->format('Y-m-d');
        $previousYearStartDate                                      = Carbon::now()->timezone($this->timeZone)->subYear()->startOfYear()->format('Y-m-d');
        $previousYearEndDate                                        = Carbon::now()->timezone($this->timeZone)->subYear()->endOfYear()->format('Y-m-d');

        $currentMonthDHMiles                                        = getLoadDeadheadMileage($currentMonthStartDate, $currentMonthEndDate, $driver->id);
        $currentMonthLoadedMiles                                    = getLoadLoadedMileage($currentMonthStartDate, $currentMonthEndDate, $driver->id);
        $currentMonthTotalMiles                                     = getLoadTotalMileage($currentMonthStartDate, $currentMonthEndDate, $driver->id);
        $currentMonthFuelSurcharge                                  = getLoadFuelSurcharge($currentMonthStartDate, $currentMonthEndDate, $driver->id);
        $currentMonthLinehaul                                       = getLoadLinehaul($currentMonthStartDate, $currentMonthEndDate, $driver->id);
        $currentMonthTotalGross                                     = getLoadGrossRevenue($currentMonthStartDate, $currentMonthEndDate, $driver->id);
        $currentMonthCPM                                            = ($currentMonthTotalGross > 0 && $currentMonthTotalMiles > 0) ? round($currentMonthTotalGross/$currentMonthTotalMiles, 2) : 0.00;
        $currentMonthDHPercentage                                   = ($currentMonthDHMiles > 0 && $currentMonthTotalMiles > 0) ? round(($currentMonthDHMiles/$currentMonthTotalMiles) * 100) : 0;

        $previousMonthDHMiles                                       = getLoadDeadheadMileage($previousMonthStartDate, $previousMonthEndDate, $driver->id);
        $previousMonthLoadedMiles                                   = getLoadLoadedMileage($previousMonthStartDate, $previousMonthEndDate, $driver->id);
        $previousMonthTotalMiles                                    = getLoadTotalMileage($previousMonthStartDate, $previousMonthEndDate, $driver->id);
        $previousMonthFuelSurcharge                                 = getLoadFuelSurcharge($previousMonthStartDate, $previousMonthStartDate, $driver->id);
        $previousMonthLinehaul                                      = getLoadLinehaul($previousMonthStartDate, $previousMonthStartDate, $driver->id);
        $previousMonthTotalGross                                    = getLoadGrossRevenue($previousMonthStartDate, $previousMonthEndDate, $driver->id);
        $previousMonthCPM                                           = ($previousMonthTotalGross > 0 && $previousMonthTotalMiles > 0) ? round($previousMonthTotalGross/$previousMonthTotalMiles, 2) : 0.00;
        $previousMonthDHPercentage                                  = ($previousMonthDHMiles > 0 && $previousMonthTotalMiles > 0) ? round(($previousMonthDHMiles/$previousMonthTotalMiles) * 100) : 0;

        $currentYearDHMiles                                         = getLoadDeadheadMileage($currentYearStartDate, $currentYearEndDate, $driver->id);
        $currentYearLoadedMiles                                     = getLoadLoadedMileage($currentYearStartDate, $currentYearEndDate, $driver->id);
        $currentYearTotalMiles                                      = getLoadTotalMileage($currentYearStartDate, $currentYearEndDate, $driver->id);
        $currentYearFuelSurcharge                                   = getLoadFuelSurcharge($currentYearStartDate, $currentYearEndDate, $driver->id);
        $currentYearLinehaul                                        = getLoadLinehaul($currentYearStartDate, $currentYearEndDate, $driver->id);
        $currentYearTotalGross                                      = getLoadGrossRevenue($currentYearStartDate, $currentYearEndDate, $driver->id);
        $currentYearCPM                                             = ($currentYearTotalGross > 0 && $currentYearTotalMiles > 0) ? round($currentYearTotalGross/$currentYearTotalMiles, 2) : 0.00;
        $currentYearDHPercentage                                    = ($currentYearDHMiles > 0 && $currentYearTotalMiles > 0) ? round(($currentYearDHMiles/$currentYearTotalMiles) * 100) : 0;

        $previousYearDHMiles                                        = getLoadDeadheadMileage($previousYearStartDate, $previousYearEndDate, $driver->id);
        $previousYearLoadedMiles                                    = getLoadLoadedMileage($previousYearStartDate, $previousYearEndDate, $driver->id);
        $previousYearTotalMiles                                     = getLoadTotalMileage($previousYearStartDate, $previousYearEndDate, $driver->id);
        $previousYearFuelSurcharge                                  = getLoadFuelSurcharge($previousYearStartDate, $previousYearEndDate, $driver->id);
        $previousYearLinehaul                                       = getLoadLinehaul($previousYearStartDate, $previousYearEndDate, $driver->id);
        $previousYearTotalGross                                     = getLoadGrossRevenue($previousYearStartDate, $previousYearEndDate, $driver->id);
        $previousYearCPM                                            = ($previousYearTotalGross > 0 && $previousYearTotalMiles > 0) ? round($previousYearTotalGross/$previousYearTotalMiles, 2) : 0.00;
        $previousYearDHPercentage                                   = ($previousYearDHMiles > 0 && $previousYearTotalMiles > 0) ? round(($previousYearDHMiles/$previousYearTotalMiles) * 100) : 0;

        $statistics                                                 = [];

        $statistics['current_month']['DHPercentage']                = $currentMonthDHPercentage.'%';
        $statistics['current_month']['DHMiles']                     = number_format($currentMonthDHMiles);
        $statistics['current_month']['LoadedMiles']                 = number_format($currentMonthLoadedMiles);
        $statistics['current_month']['TotalMiles']                  = number_format($currentMonthTotalMiles);
        $statistics['current_month']['FuelSurcharge']               = '$'.number_format($currentMonthFuelSurcharge, '2', '.', ',');
        $statistics['current_month']['Linehaul']                    = '$'.number_format($currentMonthLinehaul, '2', '.', ',');
        $statistics['current_month']['TotalGross']                  = '$'.number_format($currentMonthTotalGross, '2', '.', ',');
        $statistics['current_month']['CPM']                         = '$'.number_format($currentMonthCPM, '2', '.', ',').'/mi';

        $statistics['previous_month']['DHPercentage']               = $previousMonthDHPercentage.'%';
        $statistics['previous_month']['DHMiles']                    = number_format($previousMonthDHMiles);
        $statistics['previous_month']['LoadedMiles']                = number_format($previousMonthLoadedMiles);
        $statistics['previous_month']['TotalMiles']                 = number_format($previousMonthTotalMiles);
        $statistics['previous_month']['FuelSurcharge']              = '$'.number_format($previousMonthFuelSurcharge, '2', '.', ',');
        $statistics['previous_month']['Linehaul']                   = '$'.number_format($currentMonthLinehaul, '2', '.', ',');
        $statistics['previous_month']['TotalGross']                 = '$'.number_format($previousMonthTotalGross, '2', '.', ',');
        $statistics['previous_month']['CPM']                        = '$'.number_format($previousMonthCPM, '2', '.', ',').'/mi';

        $statistics['current_year']['DHPercentage']                 = $currentYearDHPercentage.'%';
        $statistics['current_year']['DHMiles']                      =  number_format($currentYearDHMiles);
        $statistics['current_year']['LoadedMiles']                  = number_format($currentYearLoadedMiles);
        $statistics['current_year']['TotalMiles']                   = number_format($currentYearTotalMiles);
        $statistics['current_year']['FuelSurcharge']                = '$'.number_format($currentYearFuelSurcharge, '2', '.', ',');
        $statistics['current_year']['Linehaul']                     = '$'.number_format($currentYearLinehaul, '2', '.', ',');
        $statistics['current_year']['TotalGross']                   = '$'.number_format($currentYearTotalGross, '2', '.', ',');
        $statistics['current_year']['CPM']                          = '$'.number_format($currentYearCPM, '2', '.', ',').'/mi';

        $statistics['previous_year']['DHPercentage']                = $previousYearDHPercentage.'%';
        $statistics['previous_year']['DHMiles']                     = number_format($previousYearDHMiles);
        $statistics['previous_year']['LoadedMiles']                 = number_format($previousYearLoadedMiles);
        $statistics['previous_year']['TotalMiles']                  = number_format($previousYearTotalMiles);
        $statistics['previous_year']['FuelSurcharge']               = '$'.number_format($previousYearFuelSurcharge, '2', '.', ',');
        $statistics['previous_year']['Linehaul']                    = '$'.number_format($previousYearLinehaul, '2', '.', ',');
        $statistics['previous_year']['TotalGross']                  = '$'.number_format($previousYearTotalGross, '2', '.', ',');
        $statistics['previous_year']['CPM']                         = '$'.number_format($previousYearCPM, '2', '.', ',').'/mi';

        return $statistics;
    }

    public function getDriverUpcomingEvent(Request $request){
        $driverId                   = $request->driver_id;
        $startDate                  = Carbon::parse($request->start_date);
        $endDate                    = Carbon::parse($request->end_date);

        $period                     = CarbonPeriod::create($startDate, $endDate);

        $getEvent                   = getDriverEventFullCalendar($driverId, '1', 'on', 'on', 'on', 'on', 'on', '1');

        $events                     = [];
        foreach ($getEvent as $edata) {
            foreach ($period as $date) {
                if (@$date->between($edata['start'], @$edata['end'])) {
                    $events[]       = $edata;
                }
            }
        }

        if(count($events) > 0) {
            $events                 = array_unique($events, SORT_REGULAR);
            foreach ($events as $key => $part) {
                $sort[$key]         = strtotime($part['start']);
            }
            array_multisort($sort, SORT_DESC, $events);
        }

        $view                       = view('companyAdmin.driver.view.calendarEventsView', compact('events'))->render();
        return response()->json(['calendarEvents' => $view]);
    }

    public function show(Request $request, $id){
        $getCompanyAdmin                            = auth()->guard('company-admin')->user();

        $id                                         = base64Decode($id);
        $driver                                     = Driver::withTrashed()->where('id', $id)->first();

        if($driver->status == "2"){
            if(!hasPermission(config('settings.company_admin_modules.drivers_terminated'),config('settings.permissions.view'))){
                notification('error', 'You do not have permission to access this page.');
                return redirect()->route('company.admin.home');
            }
        }

        $samsaraInfo                                = $this->getDriverSamsaraInfo($driver);
        $driverLastLocation                         = DispatchHubController::getDriverLastLocation($driver);

        $progress                                   = [];

        $drive_percentage = 0; $shift_percentage = 0; $cycle_percentage = 0; $unit_break_percentage = 0;
        if(!empty($samsaraInfo)){
            $drive                                  = isset($samsaraInfo['drive'])?explode(':', $samsaraInfo['drive']):explode(':', '00:00');
            $drive_hours                            = '00';
            $drive_minutes                          = '00';
            if(!empty($drive)){
                $drive_hours                        = $drive[0];
                $drive_minutes                      = $drive[1];
            }
            $drive_seconds                          = ((int)$drive_hours * 3600) + ((int)$drive_minutes * 60);
            $drive_max_seconds                      = (11 * 3600);
            if($drive_seconds >= $drive_max_seconds){
                $drive_percentage                   = 100;
            }else{
                $drive_percentage                   = intval(($drive_seconds/$drive_max_seconds)*100);
            }

            $shift                                  = isset($samsaraInfo['shift'])?explode(':', $samsaraInfo['shift']):explode(':', '00:00');
            $shift_hours                            = '00';
            $shift_minutes                          = '00';
            if(!empty($shift)){
                $shift_hours                        = $shift[0];
                $shift_minutes                      = $shift[1];
            }
            $shift_seconds                          = ((int)$shift_hours * 3600) + ((int)$shift_minutes * 60);
            $shift_max_seconds                      = (14 * 3600);
            if($shift_seconds >= $shift_max_seconds){
                $shift_percentage                   = 100;
            }else{
                $shift_percentage                   = intval(($shift_seconds/$shift_max_seconds)*100);
            }

            $cycle                                  = isset($samsaraInfo['cycle'])?explode(':', $samsaraInfo['cycle']):explode(':', '00:00');
            $cycle_hours                            = '00';
            $cycle_minutes                          = '00';
            if(!empty($cycle)){
                $cycle_hours                        = $cycle[0];
                $cycle_minutes                      = $cycle[1];
            }
            $cycle_seconds                          = ((int)$cycle_hours * 3600) + ((int)$cycle_minutes * 60);
            $cycle_max_seconds                      = (70 * 3600);
            if($cycle_seconds >= $cycle_max_seconds){
                $cycle_percentage                   = 100;
            }else{
                $cycle_percentage                   = intval(($cycle_seconds/$cycle_max_seconds)*100);
            }

            $unit_break                             = isset($samsaraInfo['time_in_unit_break'])?explode(':', $samsaraInfo['time_in_unit_break']):explode(':', '00:00');
            $unit_break_hours                       = '00';
            $unit_break_minutes                     = '00';
            if(!empty($unit_break)){
                $unit_break_hours                   = $unit_break[0];
                $unit_break_minutes                 = $unit_break[1];
            }
            $unit_break_seconds                     = ((int)$unit_break_hours * 3600) + ((int)$unit_break_minutes * 60);
            $unit_break_max_seconds                 = (8 * 3600);
            if($unit_break_seconds >= $unit_break_max_seconds){
                $unit_break_percentage              = 100;
            }else{
                $unit_break_percentage              = intval(($unit_break_seconds/$unit_break_max_seconds)*100);
            }
        }else{
            $samsaraInfo['drive']                   = "00:00";
            $samsaraInfo['shift']                   = "00:00";
            $samsaraInfo['cycle']                   = "00:00";
            $samsaraInfo['cycle_tom']               = "00:00";
            $samsaraInfo['duty_status']             = "";
            $samsaraInfo['time_in_current_status']  = "";
            $samsaraInfo['time_in_unit_break']      = "00:00";
        }
        $progress['drive_percentage']               = $drive_percentage;
        $progress['shift_percentage']               = $shift_percentage;
        $progress['cycle_percentage']               = $cycle_percentage;
        $progress['unit_break_percentage']          = $unit_break_percentage;

        $statistics                                 = $this->getDriverAccountStatistics($driver);

        $getField                                   = '';
        $isCheck                                    = Bookmark::where('company_id',getUserCompnayId())->where('user_id', $getCompanyAdmin->id)->where('user_type', $getCompanyAdmin->type)->where('table_id','driver_view_loads')->first();
        if(!empty($isCheck->id)) {
            //$getField                               = unserialize($isCheck->config);
            $getField                               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_driver_view_loads_first'));
        } else {
            $getField                               = config('settings.table_driver_view_loads_first');
        }
        $tableName                                  = 'settings.table_driver_view_loads';

        $driverHosViolations                        = $driver->driver_hos_violations()->latest()->take(3)->get();
        $driverNonViolations                        = $driver->driver_non_violations()->latest()->take(3)->get();
        $driverCitations                            = $driver->driver_citations()->latest()->take(3)->get();
        $driverAccidents                            = $driver->driver_accidents()->latest()->take(3)->get();

        $driverHosViolationsCheckCitations          = DriverHosViolation::join('driver_hos_violations_datas as dhvd', function ($join) use ($driver) {
            $join->on('driver_hos_violations.id', '=', 'dhvd.driver_hos_violation_id')
                ->whereIn('dhvd.unit_type', [2,3]);
        })->where('driver_hos_violations.driver_id', $driver->id)->groupBy('driver_hos_violations.id')
        ->select('driver_hos_violations.*')->get();

        $driverOnboarding                           = !empty($driver->driver_onboarding) ? $driver->driver_onboarding : '{[]}';
        $filtered_collection                        = null;
        if (!is_null($driverOnboarding) && !empty($driverOnboarding->detail_json)) {
            $t                                      = collect($driverOnboarding->detail_json);
            $filtered_collection                    = $t->filter(function ($item) {
                return $item['yes_no'] == 1 && $item['is_complete'] == 1;
            })->last();
        }

        $driver->restricted_states                  = json_encode($driver->driver_restricted_states()->pluck('state_id')->toArray());
        $driver->avoid_states                       = json_encode($driver->driver_avoid_states()->pluck('state_id')->toArray());

        $getLeaveUserType                           = LeaveUserType::IsCompany()->where('driver_id',$id)->where('year',date('Y'))->get();
        $now                                        = Carbon::now()->timezone($this->timeZone);
        $month_name                                 = $now->format('F');
        $month                                      = $now->format('m');
        $year                                       = $now->format('Y');

        return view('companyAdmin.driver.show', compact('driver', 'samsaraInfo', 'progress', 'driverLastLocation', 'statistics', 'getField', 'tableName', 'driverHosViolations', 'driverNonViolations', 'driverAccidents', 'driverHosViolationsCheckCitations', 'driverOnboarding', 'filtered_collection', 'getLeaveUserType', 'month_name', 'month', 'year'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit_backup($id){
        $id                             = base64Decode($id);
        $driver                         = Driver::findOrFail($id);

        if(isset($driver->license_number) && $driver->license_number!=""){
            $driver->license_number     = base64_decode($driver->license_number);
        }
        if(isset( $driver->passport_number) &&  $driver->passport_number!=""){
            $driver->passport_number    = base64_decode( $driver->passport_number);
        }
        if(isset( $driver->ssn) &&  $driver->ssn!=""){
            $driver->ssn                = base64_decode( $driver->ssn);
        }
        if(isset( $driver->ein) &&  $driver->ein!=""){
            $driver->ein                = base64_decode( $driver->ein);
        }

        $driverInformation              = $driver->driver_information;
        $carrier                        = Carrier::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('id')->all();
        $terminal                       = Terminal::whereIn('carrier_id',$carrier)->get();
        return view('companyAdmin.driver.edit', compact('driver', 'driverInformation','terminal'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(DriverUpdateRequest $request, Driver $driver){
        $input                              = $request->all();

        if($request->address_verified==1){
            $timezone                       = null;
            $timezone_offset                = null;
            if(isset($input['timezone_abbr'])){
                $timezone                   = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset            = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']              = $timezone;
            $input['timezone_offset']       = $timezone_offset;
        }else{
            $input['latitude']              = null;
            $input['longitude']             = null;
            $input['timezone']              = null;
            $input['timezone_offset']       = null;
            $input['timezone_abbr']         = null;
        }

        if(isset($input['is_company_driver']) && $input['is_company_driver']){
            $input['is_company_driver']     = 1;
        }else{
            $input['is_company_driver']     = 0;
        }


        $driverManager                      = CompanyAdmin::where('id', $request->manager_id)->first();
        $team_id                            = null;
        if(!is_null($driverManager) && isset($driverManager->team)){
            $team_id                        = $driverManager->team->id;
        }
        $input['team_id']                   = $team_id;
        if ($input['password']) {
            $password                       = $input['password'];
            $input['password']              = Hash::make($password);
        } else {
            unset($input['password']);
        }
        if (isset($input['profile_image'])) {
            $input['profile_image']         = FileUpload::upload('companyAdmin/driver', $input['profile_image']);
        }
        $input['sms']                       = $input['sms'] ?? 0;
        $input['send_welcome_email']        = $input['send_welcome_email'] ?? 0;
        $input['allow_login']               = $input['allow_login'] ?? 0;
        $input['weekly']                    = isset($input['joining_trailer_program']) ? $input['weekly'] : NULL;
        if (isset($input['license_exp_file'])) {
            $isbnList                       = [];
            $isbnList[]                     = FileUpload::upload('companyAdmin/driver', $input['license_exp_file']);
            if(!is_null($driver->license_exp_file)){
                $license_exp_files          = explode(',', $driver->license_exp_file);
                foreach ($license_exp_files as $key => $license_exp_file) {
                    $isbnList[]             = $license_exp_file;
                }
            }
            $input['license_exp_file']      = implode(',', $isbnList);
        }

        if(isset( $input['license_number']) &&  $input['license_number']!=""){
            $input['license_number']        = base64_encode($input['license_number']);
        }
        if(isset( $input['passport_number']) &&  $input['passport_number']!=""){
            $input['passport_number']       = base64_encode($input['passport_number']);
        }
        if(isset( $input['ssn']) &&  $input['ssn']!=""){
            $input['ssn']                   = base64_encode($input['ssn']);
        }
        if(isset( $input['ein']) &&  $input['ein']!=""){
            $input['ein']                   = base64_encode($input['ein']);
        }
        $driver->update($input);
        if($driver->wasChanged()){
            if(isset($driver->getChanges()['status'])){
                $logdata                    = [];
                $logdata['company_id']      = auth()->guard('company-admin')->user()->company_id;
                $logdata['driver_id']       = $driver->id;
                $logdata['status_id']       = $request->status;
                DriverStatusLog::create($logdata);
            }
        }

        if (!empty($request->start_date)) {
            $driver_onboar                  = [];
            $driver_onboar['start_date']    = $request->start_date;
            $exist                          = DriverOnboarding::where("driver_id", $driver->id)->first();
            if (is_null($exist)) {
                $exist                      = DriverOnboarding::create($driver_onboar);
            } else {
                $exist->update($driver_onboar);
            }
        }

        $input['joining_trailer_program']   = $input['joining_trailer_program'] ?? 0;
        $input['pets']                      = $input['pets'] ?? 0;
        $input['passenger']                 = $input['passenger'] ?? 0;
        $input['smoke']                     = $input['smoke'] ?? 0;
        $input['twic']                      = $input['twic'] ?? 0;
        $input['tanker']                    = $input['tanker'] ?? 0;
        $driver->driver_information->update($input);

        if (!empty($input['restricted_states'])) {
            foreach ($input['restricted_states'] as $restrictedState) {
                $rsIds[]                    = $restrictedState;
                $rsExist                    = DriverRestrictedState::where('driver_id', $driver->id)->where('state_id', $restrictedState)->first();
                if (is_null($rsExist)) {
                    DriverRestrictedState::create([
                        'driver_id'         => $driver->id,
                        'state_id'          => $restrictedState
                    ]);
                }
            }
            DriverRestrictedState::where('driver_id', $driver->id)->whereNotIn('state_id', $rsIds)->delete();
        } else {
            DriverRestrictedState::where('driver_id', $driver->id)->delete();
        }

        if (!empty($input['states_to_avoid'])) {
            foreach ($input['states_to_avoid'] as $avoidState) {
                $asIds[]                    = $avoidState;
                $asExist                    = DriverAvoidState::where('driver_id', $driver->id)->where('state_id', $avoidState)->first();
                if (is_null($asExist)) {
                    DriverAvoidState::create([
                        'driver_id'         => $driver->id,
                        'state_id'          => $avoidState
                    ]);
                }
            }
            DriverAvoidState::where('driver_id', $driver->id)->whereNotIn('state_id', $asIds)->delete();
        } else {
            DriverAvoidState::where('driver_id', $driver->id)->delete();
        }

        // send new pwd mail
        if (isset($input['password'])) {
            $mailData                       = [];
            $mailData['company_id']         = auth()->guard('company-admin')->user()->company_id;
            $mailData['name']               = $input['first_name'] . ' ' . $input['last_name'];
            $mailData['email']              = $input['email'];
            $mailData['password']           = $password;
            //Mail::to($input['email'])->send(new NewPasswordMailToDriver($mailData));
            sendEmail($input['email'], new NewPasswordMailToDriver($mailData));
        }

        notification('success', $this->crudMessage('update', 'Driver'));
        return redirect()->route('company.admin.drivers.index');
    }


    /**
     * Fetch listing of states for map.
     *
     * @return \Illuminate\Http\Response
     */
    public function stateListForMap(Request $request){
        $states         = State::get();
        $data           = [];
        foreach ($states as $key => $state) {
            if (!is_null($request->restricted_states) && in_array($state->id, $request->restricted_states)) {
                $data[] = [$state->code, 2, $state->name, 'Restricted'];
            } elseif (!is_null($request->avoid_states) && in_array($state->id, $request->avoid_states)) {
                $data[] = [$state->code, 1, $state->name, 'Avoided'];
            } else {
                $data[] = [$state->code, 0, $state->name, 'Allowed'];
            }
        }
        return response()->json(['data' => $data]);
    }

    /**
     * Send new password mail.
     *
     * @return \Illuminate\Http\Response
     */
    public function sendNewPasswordMail($id){
        $driver                 = Driver::find($id);

        $password               = Str::random(8);
        $driver->update(['password' => Hash::make($password)]);

        $mailData               = [];
        $mailData['company_id'] = auth()->guard('company-admin')->user()->company_id;
        $mailData['name']       = $driver->first_name . ' ' . $driver->last_name;
        $mailData['email']      = $driver->email;
        $mailData['password']   = $password;
        //Mail::to($mailData['email'])->send(new NewPasswordMailToDriver($mailData));
        sendEmail($mailData['email'], new NewPasswordMailToDriver($mailData));
        return redirect()->back();
    }

    /**
     * Fetch team of head.
     *
     * @return \Illuminate\Http\Response
     */
    public function teamOfHead(Request $request){
        $team                   = CompanyAdmin::where('id', $request->head_id)->first();
        $teamData               = '';
        if(!is_null($team) && isset($team->team)){
            $teamData           = $team->team;
        }
        return response()->json(['team' => $teamData]);
    }

    /**
     * Fetch head of team.
     *
     * @return \Illuminate\Http\Response
     */
    public function headOfTeam(Request $request){
        $head                   = Team::find($request->id)->dispatcher;
        return response()->json(['head' => $head]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request){
        $input                              = $request->except(['_token', 'button']);
        $driver                             = Driver::withTrashed()->where('id', $input['id'])->first();

        if($driver->status == "2"){
            if(!hasPermission(config('settings.company_admin_modules.drivers_terminated'),config('settings.permissions.delete'))){
                notification('error', 'You do not have permission to access this page.');
                return redirect()->route('company.admin.home');
            }
        }

        if ($request->button == 'delete') {
            $input['status']                = '2';
            $input['termination_date']      = Carbon::createFromFormat('m/d/Y H:i', $request->termination_date)->format('Y-m-d H:i:s');
            $driver->update($input);
            if($driver->wasChanged()){
                if(isset($driver->getChanges()['status'])){
                    $logdata                = [];
                    $logdata['company_id']  = auth()->guard('company-admin')->user()->company_id;
                    $logdata['driver_id']   = $driver->id;
                    $logdata['status_id']   = $driver->status;
                    DriverStatusLog::create($logdata);
                }
            }
            $driver->delete();
        } elseif ($request->button == 'forcedelete') {
            $driver->restore();
            $driver->driver_information()->withTrashed()->restore();
            $driver->forceDelete();
            $driver->driver_information()->forceDelete();
        } elseif ($request->button == 'restore') {
            $input['status']                = '3';
            $input['termination_date']      = Null;
            $input['termination_reason']    = Null;
            $input['termination_rehire']    = Null;
            $input['termination_equipment'] = Null;
            $input['termination_none']      = Null;
            $driver->update($input);
            $driver->restore();
            $driver->driver_information()->withTrashed()->restore();
        }

        if ($request->ajax()) {
            return response()->json(['success' => "Termination successfully"]);
        }

        notification('success', $this->crudMessage('delete', 'Driver'));
        return redirect()->route('company.admin.drivers.index');
    }

    public function getTermiForm(Request $request){
        $driver                             = Driver::withTrashed()->where('id', $request->id)->first();
        if (!empty($driver->termination_date)) {
            $driver->termination_date       = Carbon::createFromFormat('Y-m-d H:i:s', $driver->termination_date)->format('m/d/Y');
        }

        $is_disabled                        = '0';
        if (!empty($driver->deleted_at)) {
            $is_disabled                    = '1';
        }

        $view                               = view('companyAdmin.driver.view.terminateform', compact('driver', 'is_disabled'))->render();
        return response()->json(['view' => $view]);
    }

    public function getLoads(Request $request){
        $getCompanyAdmin                = auth()->guard('company-admin')->user();

        $getField                       = '';
        $isCheck                        = Bookmark::where('company_id',getUserCompnayId())->where('user_id', $getCompanyAdmin->id)->where('user_type', $getCompanyAdmin->type)->where('table_id','driver_view_loads')->first();
        if(!empty($isCheck->id)) {
            //$getField                   = unserialize($isCheck->config);
            $getField                   = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_driver_view_loads_first'));
        } else {
            $getField                   = config('settings.table_driver_view_loads_first');
        }

        if ($request->ajax()) {
            $data                       = LoadDetail::select('*')
                ->where('driver_id', $request->driver_id)
                ->whereNull('deleted_at')
                ->whereHas('loads', function ($q) {
                    $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                });

                // General filter code - start
                foreach($getField as $key => $value ) {
                    $conditionReq       = 'search_string_condition_'.$key;
                    $sortReq            = 'search_string_sort_'.$key;
                    $conditionDateReq   = 'search_date_condition_'.$key;
                    $sortDateReq        = 'search_date_sort_'.$key;
                    $conditionStatusReq = 'search_status_condition_'.$key;
                    $sortStatusReq      = 'search_status_sort_'.$key;

                    // String filter
                    if( isset( request()->$conditionReq ) ) {
                        $typeReq        = 'search_string_type_'.$key;
                        $type           = request()->$typeReq;
                        if( $type == 'string' ) {
                            Common::getStringFilterResult( $data, $key );
                        }
                    }
                    // String sort only
                    if( isset( request()->$sortReq ) ) {
                        Common::sortStringFilter( $data, $key );
                    }

                    // Status filter
                    if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                        Common::getStatusFilterResult( $data, $key );
                    }
                    // Status sort only
                    if( isset( request()->$sortStatusReq ) ) {
                        Common::sortStatusFilter( $data, $key );
                    }
                }

                $data->orderBy('load_id', 'DESC')->orderBy('id', 'ASC')->get();

            return Datatables::of($data)
                ->addIndexColumn()
                ->editColumn('load_number', function ($row) {
                    if ($row->is_disable == 0) {
                        if(in_array($row->load_type, ['0', '6'])) {
                            if ($row->load_type == 0) {
                                return '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link">'.$row->load_number.'</a>';
                            } else {
                                return '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        } else {
                            return '<a href="'.route('company.admin.loads.edit', base64Encode($row->load_id)).'" class="mr-1 load-edit-link">'.getLoadTypeList()[$row->load_type].'</a>';
                        }
                    } else {
                        if ($row->load_type == 0) {
                            return '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link">'.$row->load_number.'</a>';
                        } else {
                            return '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link">'.getLoadTypeList()[$row->load_type].'</a>';
                        }
                    }
                })
                ->addColumn('dispatcher', function ($row) {
                    return getLoadDispatcherName($row->booked_by);
                })
                ->filterColumn('dispatcher', function ($query, $keyword) {
                    $query->whereHas('dispatcher', function ($q) use ($keyword) {
                        $q->where(DB::raw("CONCAT(company_admins.first_name, ' ', company_admins.last_name)"), 'LIKE', "%" . $keyword . "%");
                    });
                })
                ->addColumn('status', function ($row) {
                    return getLoadStatus($row->load_status,$row->is_on_hold);
                })
                ->filterColumn('status', function ($query, $keyword) {
                    if (Str::contains('pending', Str::lower($keyword))) {
                        $query      = $query->where('load_status', 0);
                    }
                    if (Str::contains('accepted', Str::lower($keyword))) {
                        $query      = $query->where('load_status', 1);
                    }
                    if (Str::contains('preplaned', Str::lower($keyword))) {
                        $query      = $query->where('load_status', 2);
                    }
                    if (Str::contains('confirmed', Str::lower($keyword))) {
                        $query      = $query->where('load_status', 3);
                    }
                    if (Str::contains('in transit', Str::lower($keyword))) {
                        $query      = $query->where('load_status', 4);
                    }
                    if (Str::contains('delivered', Str::lower($keyword))) {
                        $query      = $query->where('load_status', 5);
                    }
                    if (Str::contains('ready to bill', Str::lower($keyword))) {
                        $query      = $query->where('load_status', 6);
                    }
                    if (Str::contains('on hold', Str::lower($keyword))) {
                        $query      = $query->where('load_status', 7);
                    }
                    if (Str::contains('complete', Str::lower($keyword))) {
                        $query      = $query->where('load_status', 8);
                    }
                    if (Str::contains('cancelled', Str::lower($keyword))) {
                        $query      = $query->where('load_status', 9);
                    }
                    return $query;
                })
                ->addColumn('last_location', function ($row) {
                    //return 'Seattle, WA, USA';
                    return getAddressCityState($row->delivery_city, $row->delivery_state);
                })
                ->editColumn('total_bill', function ($row) {
                    return getPrice($row->total_bill);
                })
                ->editColumn('shipper_location', function ($row) {
                    $html           = [];
                    if ($row->shipper_address_one) {
                        $html[0]    = $row->shipper_address_one;
                    }
                    if ($row->shipper_address_two) {
                        $html[1]    = $row->shipper_address_two;
                    }
                    if ($row->shipper_city) {
                        $html[2]    = $row->shipper_city;
                    }
                    if ($row->shipper_state) {
                        $html[3]    = $row->shipper_state;
                    }
                    if ($row->shipper_zip) {
                        $html[4]    = $row->shipper_zip;
                    }
                    if (count($html) == 0) {
                        return $row->shipper_location;
                    } else {
                        return implode(', ', $html);
                    }
                })
                ->editColumn('delivery_location', function ($row) {
                    $html           = [];
                    if ($row->delivery_address_one) {
                        $html[0]    = $row->delivery_address_one;
                    }
                    if ($row->delivery_address_two) {
                        $html[1]    = $row->delivery_address_two;
                    }
                    if ($row->delivery_city) {
                        $html[2]    = $row->delivery_city;
                    }
                    if ($row->delivery_state) {
                        $html[3]    = $row->delivery_state;
                    }
                    if ($row->delivery_zip) {
                        $html[4]    = $row->delivery_zip;
                    }
                    if (count($html) == 0) {
                        return $row->delivery_location;
                    } else {
                        return implode(', ', $html);
                    }
                })
                ->rawColumns(['load_number'])
                ->make(true);
        }
        return view('companyAdmin.driver.tab.loads');
    }

    public function OccupancyCalanderView(Request $request){
        $showtype                           = $request->showtype;
        $driver_id                          = $request->driver_id;
        $year                               = '';
        $month                              = '';
        $month_name                         = '';
        $week                               = '';
        $getDriverTargetRevenue             = '';
        $getDriverTargetOccupancy           = '';
        $last_month_name                    = '';
        $verageRevenuePerDay                = 0;
        $AverageMilePerDay                  = 0;
        $CentPerMile                        = 0;

        if ($request->showtype == 'year') {
            if ($request->get_data == 'current') {
                $year                       = Carbon::now()->timezone($this->timeZone)->year;
            } elseif ($request->get_data == 'next') {
                $year                       = $request->cyear + 1;
            } elseif ($request->get_data == 'previous') {
                $year                       = $request->cyear - 1;
            } else {
                $year                       = Carbon::now()->timezone($this->timeZone)->year;
            }
            $startDate                      = $year . '-01-01';
            $endDate                        = $year . '-12-31';
        }

        if ($request->showtype == 'month') {
            if ($request->get_data == 'current') {
                $now                        = Carbon::now()->timezone($this->timeZone);
            } elseif ($request->get_data == 'next') {
                $now                        = Carbon::createFromDate($request->cyear, $request->cmonth, '01',$this->timeZone)->addMonth();
            } elseif ($request->get_data == 'previous') {
                $now                        = Carbon::createFromDate($request->cyear, $request->cmonth, '01',$this->timeZone)->subMonth();
            } else {
                $now                        = Carbon::now()->timezone($this->timeZone);
            }

            $month                          = $now->format('m');
            $month_name                     = $now->format('F');
            $year                           = $now->format('Y');
            $startDate                      = $now->startOfMonth()->format('Y-m-d');
            $endDate                        = $now->endOfMonth()->format('Y-m-d');
        }

        if ($request->showtype == 'quarterly') {
            if ($request->get_data == 'current') {
                $now                        = Carbon::now()->timezone($this->timeZone);
            } elseif ($request->get_data == 'next') {
                $now                        = Carbon::createFromDate($request->cyear, $request->cmonth, '01',$this->timeZone)->addMonth(3);
            } elseif ($request->get_data == 'previous') {
                $now                        = Carbon::createFromDate($request->cyear, $request->cmonth, '01',$this->timeZone)->subMonth(3);
            } else {
                $now                        = Carbon::now()->timezone($this->timeZone);
            }

            $month                          = $now->firstOfQuarter()->format('m');
            $month_name                     = $now->firstOfQuarter()->format('F');
            $year                           = $now->firstOfQuarter()->format('Y');
            $startDate                      = $now->firstOfQuarter()->format('Y-m-d');
            $endDate                        = $now->lastOfQuarter()->format('Y-m-d');
            $last_month_name                = $now->lastOfQuarter()->format('F');
        }

        if ($request->showtype == 'weekly') {
            if ($request->get_data == 'current') {
                $weeklyDay                  = 0;
                $currentStartDate           = Carbon::now()->timezone($this->timeZone)->startOfWeek($weeklyDay)->format('Y-m-d');
                $currentEndDate             = Carbon::parse($currentStartDate)->addDays(6)->format('Y-m-d');
                $startDate                  = Carbon::parse($currentStartDate)->format('Y-m-d');
                $endDate                    = Carbon::parse($currentEndDate)->format('Y-m-d');
            } elseif ($request->get_data == 'next') {
                $startDate                  = Carbon::parse($request->start_date)->addWeek()->format('Y-m-d');
                $endDate                    = Carbon::parse($request->end_date)->addWeek()->format('Y-m-d');
            } elseif ($request->get_data == 'previous') {
                $startDate                  = Carbon::parse($request->start_date)->subWeek()->format('Y-m-d');
                $endDate                    = Carbon::parse($request->end_date)->subWeek()->format('Y-m-d');
            }
            $year                           = Carbon::parse($startDate)->year;
            $week                           = Carbon::parse($startDate)->weekOfYear;
        }
        // dd($startDate, $endDate);

        /*
        if($request->showtype == 'linear'){
            if($request->get_data == 'current'){
                $now                        = Carbon::now();
            }elseif($request->get_data == 'next'){
                $now                        = Carbon::createFromDate($request->cyear, $request->cmonth)->addMonth();
            }elseif($request->get_data == 'previous'){
                $now                        = Carbon::createFromDate($request->cyear, $request->cmonth)->subMonth();
            }else{
                $now                        = Carbon::now();
            }

            $month                          = $now->format('m');
            $month_name                     = $now->format('F');
            $year                           = $now->format('Y');
            $startDate                      = $now->startOfMonth()->format('Y-m-d');
            $endDate = $now->endOfMonth()->format('Y-m-d');

            $month_period                   = CarbonPeriod::create($startDate, $endDate);

            $driverlist                     = Driver::select(DB::raw("CONCAT(last_name,', ',first_name) AS name"),'id')
            ->where('company_id',auth()->guard('company-admin')->user()->company_id)
            ->orderByRaw('drivers.last_name ASC, drivers.first_name ASC')
            ->get();
        }
        */

        $view                               = view('companyAdmin.driver.view.OccupancyCalendarView', compact('week', 'year', 'month', 'showtype', 'driver_id', 'month_name', 'last_month_name', 'startDate', 'endDate'))->render();
        $driver                             = Driver::find($driver_id);
        $driver_name                        = $driver->last_name . ', ' . $driver->first_name;
        return response()->json(['view' => $view, 'driver_name' => $driver_name]);
    }

    public function OccupancyCalanderReportView(Request $request){
        $showtype                           = $request->showtype;
        $driver_id                          = $request->driver_id;
        $year                               = '';
        $month                              = '';
        $month_name                         = '';
        $week                               = '';
        $getDriverTargetRevenue             = '';
        $getDriverTargetOccupancy           = '';
        $last_month_name                    = '';
        $verageRevenuePerDay                = 0;

        // $AverageMilePerDay               = 0;
        $AverageDeadheadMilePerDay          = 0;
        $AverageLoadedMilePerDay            = 0;
        $AverageTotalMilePerDay             = 0;

        // $CentPerMile                     = 0;
        $CentLoadedPerMile                  = 0;
        $CentTotalPerMile                   = 0;

        if ($request->showtype == 'year') {
            if ($request->get_data == 'current') {
                $year                       = Carbon::now()->timezone($this->timeZone)->year;
            } elseif ($request->get_data == 'next') {
                $year                       = $request->cyear + 1;
            } elseif ($request->get_data == 'previous') {
                $year                       = $request->cyear - 1;
            } else {
                $year                       = Carbon::now()->timezone($this->timeZone)->year;
            }
            $startDate                      = $year . '-01-01';
            $endDate                        = $year . '-12-31';
        }

        if ($request->showtype == 'month') {
            if ($request->get_data == 'current') {
                $now                        = Carbon::now()->timezone($this->timeZone);
            } elseif ($request->get_data == 'next') {
                $now                        = Carbon::createFromDate($request->cyear, $request->cmonth, '01',$this->timeZone)->addMonth();
            } elseif ($request->get_data == 'previous') {
                $now                        = Carbon::createFromDate($request->cyear, $request->cmonth, '01',$this->timeZone)->subMonth();
            } else {
                $now                        = Carbon::now()->timezone($this->timeZone);
            }

            $month                          = $now->format('m');
            $month_name                     = $now->format('F');
            $year                           = $now->format('Y');
            $startDate                      = $now->startOfMonth()->format('Y-m-d');
            $endDate                        = $now->endOfMonth()->format('Y-m-d');
        }

        if ($request->showtype == 'quarterly') {
            if ($request->get_data == 'current') {
                $now                        = Carbon::now()->timezone($this->timeZone);
            } elseif ($request->get_data == 'next') {
                $now                        = Carbon::createFromDate($request->cyear, $request->cmonth, '01',$this->timeZone)->addMonth(3);
            } elseif ($request->get_data == 'previous') {
                $now                        = Carbon::createFromDate($request->cyear, $request->cmonth, '01',$this->timeZone)->subMonth(3);
            } else {
                $now                        = Carbon::now()->timezone($this->timeZone);
            }

            $month                          = $now->firstOfQuarter()->format('m');
            $month_name                     = $now->firstOfQuarter()->format('F');
            $year                           = $now->firstOfQuarter()->format('Y');
            $startDate                      = $now->firstOfQuarter()->format('Y-m-d');
            $endDate                        = $now->lastOfQuarter()->format('Y-m-d');
            $last_month_name                = $now->lastOfQuarter()->format('F');
        }

        if ($request->showtype == 'weekly') {
            if ($request->get_data == 'current') {
                $weeklyDay                  = 0;
                $currentStartDate           = Carbon::now()->timezone($this->timeZone)->startOfWeek($weeklyDay)->format('Y-m-d');
                $currentEndDate             = Carbon::parse($currentStartDate)->addDays(6)->format('Y-m-d');
                $startDate                  = Carbon::parse($currentStartDate)->format('Y-m-d');
                $endDate                    = Carbon::parse($currentEndDate)->format('Y-m-d');
            } elseif ($request->get_data == 'next') {
                $startDate                  = Carbon::parse($request->start_date)->addWeek()->format('Y-m-d');
                $endDate                    = Carbon::parse($request->end_date)->addWeek()->format('Y-m-d');
            } elseif ($request->get_data == 'previous') {
                $startDate                  = Carbon::parse($request->start_date)->subWeek()->format('Y-m-d');
                $endDate                    = Carbon::parse($request->end_date)->subWeek()->format('Y-m-d');
            }
            $year                           = Carbon::parse($startDate)->year;
            $week                           = Carbon::parse($startDate)->weekOfYear;
        }
        // dd($startDate, $endDate);

        /*
        if($request->showtype == 'linear'){
            if($request->get_data == 'current'){
                $now                        = Carbon::now();
            }elseif($request->get_data == 'next'){
                $now                        = Carbon::createFromDate($request->cyear, $request->cmonth)->addMonth();
            }elseif($request->get_data == 'previous'){
                $now                        = Carbon::createFromDate($request->cyear, $request->cmonth)->subMonth();
            }else{
                $now                        = Carbon::now();
            }

            $month                          = $now->format('m');
            $month_name                     = $now->format('F');
            $year                           = $now->format('Y');
            $startDate                      = $now->startOfMonth()->format('Y-m-d');
            $endDate = $now->endOfMonth()->format('Y-m-d');

            $month_period                   = CarbonPeriod::create($startDate, $endDate);

            $driverlist                     = Driver::select(DB::raw("CONCAT(last_name,', ',first_name) AS name"),'id')
            ->where('company_id',auth()->guard('company-admin')->user()->company_id)
            ->orderByRaw('drivers.last_name ASC, drivers.first_name ASC')
            ->get();
        }
        */

        $occupied                           = getloadCount($startDate, $endDate, 0, $driver_id);
        $maintenance                        = getloadCount($startDate, $endDate, 1, $driver_id);
        $unauthorized                       = getloadCount($startDate, $endDate, 2, $driver_id);
        $hos                                = getloadCount($startDate, $endDate, 3, $driver_id);
        $timeOff                            = getloadCount($startDate, $endDate, 5, $driver_id);

        $tot                                = $occupied + $maintenance + $unauthorized + $hos + $timeOff;

        if ($tot > 0) {
            $occupied                       = round(($occupied * 100) / $tot, 2);
            $maintenance                    = round(($maintenance * 100) / $tot, 2);
            $unauthorized                   = round(($unauthorized * 100) / $tot, 2);
            $hos                            = round(($hos * 100) / $tot, 2);
            $timeOff                        = round(($timeOff * 100) / $tot, 2);
        }

        $occupanytype                       = view('companyAdmin.driver.view.OccupancyCalendarOccupanytypeView', compact('occupied', 'maintenance', 'unauthorized', 'hos', 'timeOff'))->render();

        $showtype                           = $request->showtype;
        $getLoadGrossRevenue                = getLoadGrossRevenue($startDate, $endDate, $driver_id);
        $getLoadLinehaul                    = getLoadLinehaul($startDate, $endDate, $driver_id);
        $getLoadFuelSurcharge               = getLoadFuelSurcharge($startDate, $endDate, $driver_id);
        $getLoadAccessorials                = getLoadAccessorials($startDate, $endDate, $driver_id);
        // $getLoadMileage                  = getLoadMileage($startDate, $endDate, $driver_id);
        $getLoadDeadheadMileage             = getLoadDeadheadMileage($startDate,$endDate,$driver_id);
        $getLoadLoadedMileage               = getLoadLoadedMileage($startDate,$endDate,$driver_id);
        $getLoadTotalMileage                = getLoadTotalMileage($startDate,$endDate,$driver_id);
        $getLoadOccupancy                   = $request->bggreen; //round(getLoadTotalDays($startDate, $endDate, $driver_id));

        if ($getLoadOccupancy > 0) {
            $verageRevenuePerDay            = round($getLoadGrossRevenue / $getLoadOccupancy, 0);
            // $AverageMilePerDay           = round($getLoadMileage / $getLoadOccupancy, 2);
            $AverageDeadheadMilePerDay      = round($getLoadDeadheadMileage / $getLoadOccupancy, 0);
            $AverageLoadedMilePerDay        = round($getLoadLoadedMileage / $getLoadOccupancy, 0);
            $AverageTotalMilePerDay         = round($getLoadTotalMileage / $getLoadOccupancy, 0);
        }

        if ($getLoadTotalMileage > 0) {
            //$CentPerMile                  = round($getLoadGrossRevenue / $getLoadMileage, 2);
            $CentLoadedPerMile              = round($getLoadGrossRevenue / $getLoadLoadedMileage, 2);
            $CentTotalPerMile               = round($getLoadGrossRevenue / $getLoadTotalMileage, 2);
        }

        $getDriverTargetRevenue             = getDriverTargetRevenue($startDate, $endDate, $driver_id);
        if ($getDriverTargetRevenue > 0) {
            $getDriverTargetRevenue         = round(($getLoadGrossRevenue * 100) / $getDriverTargetRevenue, 2);
        }

        $getDriverTargetOccupancy           = getDriverTargetOccupancy($startDate, $endDate, $driver_id);

        // $performanceSummary              = view('companyAdmin.driver.view.OccupancyCalendarPerformanceSummary', compact('showtype', 'getLoadGrossRevenue', 'getLoadMileage', 'verageRevenuePerDay', 'AverageMilePerDay', 'CentPerMile', 'getDriverTargetRevenue', 'getDriverTargetOccupancy'))->render();

        $performanceSummary                 = view('companyAdmin.driver.view.OccupancyCalendarPerformanceSummary', compact('showtype', 'getLoadGrossRevenue', 'getLoadLinehaul', 'getLoadFuelSurcharge', 'getLoadAccessorials', 'getLoadDeadheadMileage', 'getLoadLoadedMileage', 'getLoadTotalMileage', 'verageRevenuePerDay', 'AverageDeadheadMilePerDay', 'AverageLoadedMilePerDay', 'AverageTotalMilePerDay', 'CentLoadedPerMile', 'CentTotalPerMile', 'getDriverTargetRevenue','getDriverTargetOccupancy'))->render();

        $driver                             = Driver::find($driver_id);
        $driver_name                        = $driver->last_name . ', ' . $driver->first_name;
        return response()->json(['occupanytype' => $occupanytype, 'performanceSummary' => $performanceSummary, 'driver_name' => $driver_name]);
    }

    public function viewLoadDetailsByDate(Request $request){
        $loadIds                            = explode(',', $request->load);

        $loadDetailsBody                    = '';
        $loadDetailsFooter                  = '';
        $totalMileage                       = [];
        $totalRevenue                       = [];
        $totalCpm                           = 0.00;
        foreach ($loadIds as $loadId){
            $loadDetails                    = LoadDetail::where('id', $loadId)->first();

            if(!is_null($loadDetails)){
                $totalRevenue[]             = $loadDetails->driver_revenue;
                $totalMileage[]             = $loadDetails->total_miles;

                $load_number                = $loadDetails->load_number;
                $stop_01                    = $loadDetails->shipper_city.', '.$loadDetails->shipper_state;
                $stop_99                    = $loadDetails->delivery_city.', '.$loadDetails->delivery_state;
                $deadhead_mileage           = number_format($loadDetails->empty_miles);
                $loaded_mileage             = number_format($loadDetails->loaded_miles);
                $total_mileage              = number_format($loadDetails->total_miles);
                $revenue                    = number_format($loadDetails->driver_revenue, '2', '.', ',');
                $cpm                        = ($loadDetails->driver_revenue > 0 && $loadDetails->total_miles > 0) ? round($loadDetails->driver_revenue / $loadDetails->total_miles, 2) : 0.00;

                $loadDetailsBody            .= '<div class="px-2">';
                $loadDetailsBody            .= '<div class="col-md-12 content-header">';
                $loadDetailsBody            .= '<label><a href="'.route('company.admin.loads.show', base64Encode($loadId)).'" target="_blank">'. $load_number.' ('.getLoadTypeList()[$loadDetails->load_type].')</a></label>';
                $loadDetailsBody            .= '</div>';
                $loadDetailsBody            .= '<div class="col-md-12 content-body">';
                $loadDetailsBody            .= '<p class="mb-1">Stop 01 : '.$stop_01.'</p>';
                $loadDetailsBody            .= '<p class="mb-1">Stop 99 : '.$stop_99.'</p>';
                $loadDetailsBody            .= '<p class="mb-1">Mileage : '.$deadhead_mileage.' DH | '.$loaded_mileage.' LD | '.$total_mileage.' Total</p>';
                $loadDetailsBody            .= '<p class="mb-1">Revenue : $'.$revenue.'</p>';
                $loadDetailsBody            .= '<p class="mb-1">CPM : $'.$cpm.'</p>';
                $loadDetailsBody            .= '</div>';
                $loadDetailsBody            .= '</div>';

                if(count($loadIds) > 1 && $loadId != end($loadIds)){
                    $loadDetailsBody        .= '<hr class="mt-2 mb-2" />';
                }
            }
        }

        if(count($totalRevenue) > 0 && count($totalMileage) > 0){
            if(array_sum($totalRevenue) > 0 && array_sum($totalMileage) > 0){
                $totalCpm                   = round(array_sum($totalRevenue) / array_sum($totalMileage), 2);
            }

            $loadDetailsFooter              .= '<div class="col-md-4 py-3 mr-0 text-center">';
            $loadDetailsFooter              .= '<label>'.number_format(array_sum($totalMileage)).'</label>';
            $loadDetailsFooter              .= '<p>Total Mileage</p>';
            $loadDetailsFooter              .= '</div>';
            $loadDetailsFooter              .= '<div class="col-md-4 py-3 ml-0 mr-0 text-center">';
            $loadDetailsFooter              .= '<label>$'.number_format(array_sum($totalRevenue), '2', '.', ',').'</label>';
            $loadDetailsFooter              .= '<p>Total Revenue</p>';
            $loadDetailsFooter              .= '</div>';
            $loadDetailsFooter              .= '<div class="col-md-4 py-3 ml-0 text-center">';
            $loadDetailsFooter              .= '<label>$'.$totalCpm.'</label>';
            $loadDetailsFooter              .= '<p>Total CPM</p>';
            $loadDetailsFooter              .= '</div>';
        }

        return response()->json(['loadDetailsBody' => $loadDetailsBody, 'loadDetailsFooter' => $loadDetailsFooter]);
    }

    public function addEvent(Request $request){
        $input                  = $request->all();
        $input['company_id']    = auth()->guard('company-admin')->user()->company_id;
        $input['fa_icon']       = null;
        if (!empty($request->addshoicon)) {
            $input['fa_icon']   = $request->fa_icon;
        }
        if (!empty($request->setdriver)) {
            $input['driver_id'] = $request->setdriver;
        }
        if (!empty($request->setdispatcher)) {
            $input['driver_id'] = $request->setdispatcher;
        }
        Event::create($input);
        notification('success', $this->crudMessage('create', 'Event'));
        return redirect()->back();
    }

    public function removeEvent(Request $request){
        $input = $request->all();
        $event = Event::find($request->event_id);
        if (isset($event->id)) {
            $event->delete($request->event_id);
        }
        return 1;
    }

    public function getEvents(Request $request){
        $input                  = $request->all();

        $driverId               = isset($input['driver_id']) ? $input['driver_id'] : Null;
        $calendarEvent          = isset($input['calendarEvent']) ? $input['calendarEvent'] : Null;
        $birthdays              = isset($input['birthdays']) ? $input['birthdays'] : Null;
        $expirations            = isset($input['expirations']) ? $input['expirations'] : Null;
        $maintenanceTrucks      = isset($input['maintenanceTrucks']) ? $input['maintenanceTrucks'] : Null;
        $maintenanceTrailers    = isset($input['maintenanceTrailers']) ? $input['maintenanceTrailers'] : Null;
        $hidePersonalEvents     = isset($input['hidePersonalEvents']) ? $input['hidePersonalEvents'] : Null;
        $myLeaves               = isset($input['myLeaves']) ? $input['myLeaves'] : Null;
        $myViolations           = isset($input['myViolations']) ? $input['myViolations'] : Null;
        $maintenanceTicket      = isset($input['maintenanceTicket']) ? $input['maintenanceTicket'] : Null;
        $startDate              = isset($input['startDate']) ? $input['startDate'] : Null;
        $endDate                = isset($input['endDate']) ? $input['endDate'] : Null;

        $getEvent               = getDriverEventFullCalendar($driverId,$calendarEvent,$birthdays,$expirations,$maintenanceTrucks,$maintenanceTrailers,$hidePersonalEvents,null,$myLeaves,$myViolations,$maintenanceTicket, $startDate, $endDate);
        return response()->json(['view' => $getEvent]);
    }

    public function AccountTabChart(Request $request){
        $input                      = $request->all();

        $getDriverTargetRevenue     = '';
        $getDriverTargetOccupancy   = '';
        $verageRevenuePerDay        = 0;
        $AverageMilePerDay          = 0;
        $CentPerMile                = 0;

        $startDate                  = Carbon::parse($request->start_date)->format('Y-m-d');
        $endDate                    = Carbon::parse($request->end_date)->format('Y-m-d');
        $driver_id                  = $request->driver_id;

        $occupied                   = getloadCount($startDate, $endDate, 0, $driver_id);
        $maintenance                = getloadCount($startDate, $endDate, 1, $driver_id);
        $unauthorized               = getloadCount($startDate, $endDate, 2, $driver_id);
        $hos                        = getloadCount($startDate, $endDate, 3, $driver_id);
        $timeOff                    = getloadCount($startDate, $endDate, 5, $driver_id);

        $tot                        = $occupied + $maintenance + $unauthorized + $hos + $timeOff;

        if ($tot > 0) {
            $occupied               = round(($occupied * 100) / $tot, 2);
            $maintenance            = round(($maintenance * 100) / $tot, 2);
            $unauthorized           = round(($unauthorized * 100) / $tot, 2);
            $hos                    = round(($hos * 100) / $tot, 2);
            $timeOff                = round(($timeOff * 100) / $tot, 2);
        }
        $getOccupancy               = getLoadOccupancyCalendarMonth($startDate, $endDate, $driver_id);

        $view                       = view('companyAdmin.driver.view.accountTab', compact('occupied', 'maintenance', 'unauthorized', 'hos', 'timeOff', 'getOccupancy'))->render();

        $getLoadGrossRevenue        = getLoadGrossRevenue($startDate, $endDate, $driver_id);
        $getTargetRevenueByDate     = getTargetRevenueByDate($startDate, $endDate, $driver_id);
        $getRevenueeDifferent       = $getLoadGrossRevenue - $getTargetRevenueByDate;

        $getLoadMileage             = getLoadTotalMileage($startDate, $endDate, $driver_id);
        $getTargetMilesByDate       = getDrivertarget_milestrackerByDate($startDate, $endDate, $driver_id);
        $getMilesDifferent          = $getLoadMileage - $getTargetMilesByDate;


        $getDriverCpmTarget         = getDrivercpm_targettracker($startDate, $endDate, $driver_id);
        if ($getLoadMileage > 0) {
            $CentPerMile            = round($getLoadGrossRevenue / $getLoadMileage, 2);
        }

        $getCpmDifferent            = $CentPerMile - $getDriverCpmTarget;

        $getLoadOccupancy           = $getOccupancy;
        $getDriverTargetOccupancy   = getDriverTargetOccupancy($startDate, $endDate, $driver_id);
        $getOccupancyDifferent      = $getLoadOccupancy - $getDriverTargetOccupancy;
        $goalsView                  = view('companyAdmin.driver.view.accountGoalsView', compact('getLoadGrossRevenue', 'getTargetRevenueByDate', 'getRevenueeDifferent', 'getLoadMileage', 'getTargetMilesByDate', 'getMilesDifferent', 'getDriverCpmTarget', 'CentPerMile', 'getCpmDifferent', 'getLoadOccupancy', 'getDriverTargetOccupancy', 'getOccupancyDifferent'))->render();

        $topfiler1                  = getloadCountByDriverId($startDate, $endDate, $driver_id);
        $topfiler2                  = $getLoadOccupancy . '%';
        $topfiler3                  = getPrice($getLoadGrossRevenue);
        $topfiler4                  = $getLoadMileage;


        $topfiler1                  = $getLoadOccupancy . '%'; // Occupancy
        $topfiler2                  = getloadCountByDriverId($startDate, $endDate, $driver_id); // Loads
        $topfiler3                  = '0'; // Miles Per Gallon
        $topfiler4                  = '$0.00'; // Average Fuel Discount
        $topfiler5                  = '$0.00'; // Average Fuel Cost (PPG)
        $topfiler6                  = number_format($getLoadMileage); // Total Miles
        $topfiler7                  = '$'.number_format($CentPerMile, 2, '.', ','); // Total CPM
        $topfiler8                  = getPrice($getLoadGrossRevenue); // Total Revenue
        $topfiler9                  = '$0.00'; // Earnings

        return response()->json(['view' => $view, 'goalsview' => $goalsView, 'topfiler1' => $topfiler1, 'topfiler2' => $topfiler2, 'topfiler3' => $topfiler3, 'topfiler4' => $topfiler4, 'topfiler5' => $topfiler5, 'topfiler6' => $topfiler6, 'topfiler7' => $topfiler7, 'topfiler8' => $topfiler8, 'topfiler9' => $topfiler9]);
    }

    public function removeEdit(Request $request){
        $eventData  = Event::where('id',$request->event_id)->first();
        $view       = view('companyAdmin.driver.modal.editEvent-PopUp',compact('eventData'))->render();
        return response()->json(['status' => 1, 'html' => $view, 'message', '']);
    }

    public function getEventsUpdates(Request $request){
        $from_date              = dateTimeFormatter($request->from_date,'Y-m-d H:i:s',getTimeLayout());
        $to_date                = dateTimeFormatter($request->to_date,'Y-m-d H:i:s',getTimeLayout());
        Event::where('id',$request->id)->update([
           'name'               =>  $request->name,
           'address'            =>  $request->address,
           'type'               =>  $request->type,
           'from_date'          =>  $from_date,
           'to_date'            =>  $to_date,
            'is_repeat'         =>  $request->is_repeat,
            'notes'             =>  $request->notes
        ]);
        return redirect()->back();
    }

    public function syncDriver(Request $request){
        $eobrProvider                   = $request->eobrProvider;
        $driver                         = Driver::find($request->driverId);

        if($eobrProvider == '0'){
            $carrierId                  = (isset($driver->driver_information->carrier_id)) ? $driver->driver_information->carrier_id : null;
            $getAPIKey                  = IntegrationSetting::where('company_id', $driver->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->first();

            if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                $client                 = new \GuzzleHttp\Client(['verify' => false]);
                $response               = $client->request('GET', 'https://api.samsara.com/fleet/drivers', [
                    'headers' => [
                        'Accept'        => 'application/json',
                        'Authorization' => 'Bearer '.$getAPIKey->api_key,
                    ],
                ]);
                $data                   = json_decode($response->getBody());
                $data                   = $data->data;

                $samsaraId              = null;
                foreach($data as $driverData){
                    if(isset($driverData->licenseNumber) && $driverData->licenseNumber == base64_decode($driver->license_number)){
                        $samsaraId      = $driverData->id;
                        break;
                    }
                }
                if($samsaraId){
                    return response()->json(['type' => 'success', 'msg' => 'Driver synced successfully', 'eobrId' => $samsaraId]);
                }
            }else{
                return response()->json(['type' => 'error', 'msg' => 'EOBR Provider API key not found.']);
            }
        }
        return response()->json(['type' => 'error', 'msg' => 'Something went wrong!']);
    }

    public function getSyncDriver($samsara_api_key,$endCursor){
        $data                           = '';
        $company_id                     = auth()->guard('company-admin')->user()->company_id;
        if(!empty($samsara_api_key)){
            $client                     = new \GuzzleHttp\Client();
            if(empty($endCursor)){
                $api_url                = 'https://api.samsara.com/fleet/drivers?limit=100';
            }else{
                $api_url                = 'https://api.samsara.com/fleet/drivers?limit=100&after='.$endCursor;
            }

            $response                   = $client->request('GET', $api_url, [
                'headers' => [
                    'Accept'            => 'application/json',
                    'Authorization'     => 'Bearer '.$samsara_api_key,
                ],
            ]);
            $data                       = json_decode($response->getBody());

            $endCursor                  = $data->pagination->endCursor;
            $data                       = $data->data;

            foreach($data as $driver){
                if(isset($driver->licenseNumber)){
                    $getDriver          = Driver::where('license_number', $driver->licenseNumber)->where('company_id',$company_id)->first();
                    if(!is_null($getDriver)){
                        $driver_data    = [];
                        $driver_data['samsara_id'] = $driver->id;
                        $getDriver->update($driver_data);
                        $mest           = $driver->licenseNumber.'___'.$driver->id.'___'.$endCursor;
                    }
                }
            }
            if(!empty($endCursor)){
                return redirect()->route('company.admin.driver.sync.getsyncdriver', ['samsara_api_key' => $samsara_api_key, 'endCursor' => $endCursor]);
            }else{
                notification('success', $this->crudMessage('create', 'Driver'));
                return redirect()->route('company.admin.drivers.index');
            }
        }
    }

    public function getRequestDocuments(Request $request){
        if ($request->ajax()) {
            $data = Gallery::isCompany()->where('source','drivers')->where('action_id',$request->id)->whereNotNull('requested_by')->orderBy('id', 'desc');
            return Datatables::of($data)
                ->editColumn('field_name', function($row){
                    return getRequestDocumentsList($row->field_name, 'field_name')['name'];
                })
                ->editColumn('approved_by', function($row){
                    return empty($row->approved_by) ? 'None' : getFullNameByid($row->approved_by, $row->approved_by_type) ;
                })
                ->editColumn('created_by', function($row){
                    return empty($row->created_by) ? 'None' : getFullNameByid($row->created_by, $row->created_by_type) ;
                })
                ->editColumn('status', function($row){
                    if($row->status == 'pending'){
                        return '<span class="status-pending">Pending</span>';
                    }
                    if($row->status == 'approved'){
                        return '<span class="status-approved">Approved</span>';
                    }
                    if($row->status == 'review'){
                        return '<span class="status-review">Review</span>';
                    }
                })
                ->editColumn('access', function($row){
                    if($row->access == 1){
                        return 'Managers';
                    }
                    if($row->access == 3){
                        return 'Dispatchers';
                    }
                    if($row->access == 3){
                        return 'Drivers';
                    }
                    return 'All User';
                })
                ->editColumn('requested_by', function($row){
                    return empty($row->requested_by) ? 'None' : getFullNameByid($row->requested_by, $row->requested_by_type) ;
                })
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $btn        = '<a href="#"  class="gallary-edit mr-2" data-id="'.$row->action_id.'" data-source="drivers" data-field="'.$row->field_name.'" data-toggle="modal" data-target="#galleryModal"><i class="fa fa-eye theme-icon" aria-hidden="true"></i></a>';
                    //$btn      = '<a href="'.route('company.admin.request-documents.show', $row->id).'" class="mr-2"><img src="'.asset('theme/img/eye.png').'" alt=""></a>';
                    //$btn      = $btn.' <a href="'.route('company.admin.request-documents.edit', $row->id).'" class="mr-4"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';
                    if(!is_null(checkeGalleryFileUpload($row->action_id, $row->field_name))){
                        $btn    = $btn.'<a href="'.route('company.admin.gallery.download.file',$row->id).'" class="mr-2"><i class="fa fa-download theme-icon" aria-hidden="true"></i></a>';
                    }
                    $btn        = $btn.' <a href="javascript:" class="destroy" data-action="'.route('company.admin.request-documents.destroy',$row->id).'" data-id="'.$row->id.'"><i class="fa fa-trash theme-danger-icon" aria-hidden="true"></i></a>';
                    return $btn;
                })
                ->rawColumns(['action','field_name','approved_by','created_by','requested_by','status','access'])
                ->make(true);
        }
    }

    /**
     * Login to company admin.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function login($id){
        $id                 = base64Decode($id);
        // $last_login_user = [
        //      'id'                    => auth()->guard('company-admin')->user()->id,
        //     'name'                   => auth()->guard('company-admin')->user()->first_name.' '.auth()->guard('company-admin')->user()->last_name,
        //     'user_type'              => 'company-admin',
        //     'last_user_guard_type'   => auth()->guard('company-admin')->user()->type
        // ];
        // $last_login_user = json_encode($last_login_user);
        $last_user          = Session::get('last_user');

        $driverAdmin        = Driver::where('id', $id)->first();
        Session::put('impersonate_user', $driverAdmin);
        Session::put('driverAdmin-2fa', $id);

        auth()->guard('driver-admin')->loginUsingId($driverAdmin->id);

        getTwilioDriverChatToken();

        // return redirect()->route('driver.admin.home');
        return redirect()->route('driver.admin.home');
    }

    // get terminal list by carrier id
    public function getTerminals(Request $request){
        if(!empty($request->carrier_id)){
            $data['terminals']      = Terminal::where("carrier_id", $request->carrier_id)->get(["name", "id"]);
            return response()->json($data);
        }else{
            $data['terminals']      = '';
        }
    }

    public function getDriversList(Request $request){
        $search                     = ($request->has('search') && $request->search!='')?$request->search:'';
        $query                      = Driver::query();
        if($search!=''){
            $query->where(DB::raw("CONCAT(last_name, ', ', first_name)"), 'LIKE', "%" . $search . "%");
        }
        $records                    = $query->limit(150)->get();
        $data                       = [];
        foreach ($records as $key => $row) {
            $data[]                 = array('id'=>$row->id,'text'=>$row->full_name);
        }
        return response()->json($data);
    }

    public function updateProfileImage(Request $request){
        $input                      = $request->all();
        if (isset($input['profile_image'])) {
            $driver                 = Driver::findOrFail($input['driver_id']);
            $driver->profile_image  = FileUpload::upload('companyAdmin/driver', $input['profile_image']);
            $driver->save();
            if($driver){
                return response()->json(['type' => 'success', 'msg' => 'Profile Image updated successfully', 'url' => asset('storage/companyAdmin/driver/'.$driver->profile_image)]);
            }else{
                return response()->json(['type' => 'error', 'msg' => 'Error on update Profile Image']);
            }
        }else{
            return response()->json(['type' => 'error', 'msg' => 'Something went wrong!']);
        }
    }

    public function checkActiveLoads(Request $request, $id){
        $response['success']                        = false;
        $response['message']                        = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $driver                             = Driver::find(base64Decode($id));

                if ($driver) {
                    $activeLoads                    = $driver->loads->whereIn('load_status', ['0', '1', '2', '3', '4', '11'])->pluck('load_number', 'load_id')->toarray() ?? [];

                    if($activeLoads > 0){
                        $activeLoadsHtml            = '<table class="table table-bordered w-100">';
                        $activeLoadsHtml           .= '<tr style="background: transparent linear-gradient(180deg, #417ccb 0%, #294e7c 100%) 0% 0% no-repeat padding-box;color: #fff;">';
                            $activeLoadsHtml       .= '<th class="text-center" width="10%">No.</th>';
                            $activeLoadsHtml       .= '<th class="text-left" width="90%">Load Number</th>';
                        $activeLoadsHtml           .= '</tr>';
                        $index = 1;
                        foreach ($activeLoads as $loadId => $loadNumber){
                            $activeLoadsHtml       .= '<tr>';
                                $activeLoadsHtml   .= '<td class="text-center">'.$index.'</td>';
                                $activeLoadsHtml   .= '<td class="text-left"><a href="'.route('company.admin.loads.edit', base64Encode($loadId)).'" style="text-decoration: underline;" target="_blank">'.$loadNumber.'</a></td>';
                            $activeLoadsHtml       .= '</tr>';
                            $index++;
                        }
                        $activeLoadsHtml           .= '</table>';
                    }
                    $response['active_loads_html']  = $activeLoadsHtml ?? '';
                    $response['active_loads']       = count($activeLoads);
                    $response['success']            = true;
                    $response['message']            = "Active loads fetch successfully.";
                } else {
                    $response['success']            = false;
                    $response['message']            = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success']                = false;
                $response['message']                = $e->getMessage();
            }
        }

        return response()->json($response);
    }
}
