<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use App\Models\DriverOnboarding;
use App\Models\DriverOnboardingSetting;
use App\Models\DriverOnboardingSubStage;
use Illuminate\Http\Request;
use App\Models\Setting;
use App\Models\SubStage;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Session;
use Validator;
use Arr;

class DriverOnboardingSettingController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //$data = DriverOnboardingSetting::isCompany()->orderBy('order')->get();
        $data = driverOnboardingSetting(base64Decode(request()->driver_id));

        return view('companyAdmin.driverOnboardingSetting.index',compact('data'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('companyAdmin.driverOnboardingSetting.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $input = $request->all();
        $driverId = base64Decode($input['driver_id']);
        $input['sub_stage'] = Arr::except($input['sub_stage'],['replace_key']);

        $validator = Validator::make($input,[
            'title' => 'unique:driver_onboarding_settings,title,NULL,id,company_id,'.auth()->guard('company-admin')->user()->company_id.',deleted_at,NULL',
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $input['company_id'] = auth()->guard('company-admin')->user()->company_id;
        $input['order'] = DriverOnboardingSetting::isCompany()->count() + 1;
        $input['apply_to_profile'] = ($input['apply_to'] == 0) ? $driverId : null;
        $input['hired_before_stage_added_date'] = ($input['apply_to'] == 1) ? Carbon::now()->format('Y-m-d') : null;
        $input['mark_as_completed'] = $input['mark_as_completed'] ?? 0;

        $Setting = DriverOnboardingSetting::create($input);

        $subStageArray = array_filter(array_map('array_filter', $input['sub_stage']));
        if (!empty($subStageArray)) {
            foreach ($input['sub_stage'] as $key => $subStage) {
                $data = [];
                $data['driver_onboarding_setting_id'] = $Setting->id;
                $data['title'] = $subStage['title'];
                $data['file'] = isset($subStage['file']) ? 1 : 0;
                $data['date'] = isset($subStage['date']) ? 1 : 0;
                // if(isset($subStage['mark_as_completed'])){
                //     $data['file'] = 0;
                //     $data['date'] = 0;
                // }else{
                //     $data['file'] = isset($subStage['file']) ? 1 : 0;
                //     $data['date'] = isset($subStage['date']) ? 1 : 0;
                // }
                // $data['mark_as_completed'] = isset($subStage['mark_as_completed']) ? 1 : 0;
                $data['mark_as_completed'] = 0;
                DriverOnboardingSubStage::create($data);
            }
        }

        /********** Apply to this profile only - START **********/
        if(!is_null($Setting->apply_to_profile) && is_null($Setting->hired_before_stage_added_date)){
            $driverOnboarding = DriverOnboarding::where('driver_id', $Setting->apply_to_profile)->first();
            if($driverOnboarding){
                if(!is_null($driverOnboarding->detail)){
                    $detailsArr = json_decode($driverOnboarding->detail, true);

                    if(!array_key_exists($Setting->stageName, $detailsArr)){
                        $detailsArr = array_merge($detailsArr, $this->getDriverUpdateStage($Setting, 0));
                    }

                    $driverOnboarding->detail = json_encode($detailsArr);
                    $driverOnboarding->save();
                }
            }
        }
        /********** Apply to this profile only - END **********/

        /********** Apply to all profiles - START **********/
        if(!is_null($Setting->hired_before_stage_added_date) && is_null($Setting->apply_to_profile)){
            if($Setting->update_profile == 1){
                /********** Mark complete for currently previously drivers - START **********/
                $onboardingDrivers = DriverOnboarding::where('status', 2)->where('application_date', '<', $Setting->hired_before_stage_added_date)->whereNotNull('detail')->get();

                foreach ($onboardingDrivers as $onboardingDriver){
                    $detailsArr = json_decode($onboardingDriver->detail, true);

                    if(!array_key_exists($Setting->stageName, $detailsArr)){
                        $detailsArr = array_merge($detailsArr, $this->getDriverUpdateStage($Setting, 1));
                    }

                    $onboardingDriver->detail = json_encode($detailsArr);
                    $onboardingDriver->save();
                }
                /********** Mark complete for currently previously drivers - END **********/
            }else{
                /********** Do not mark complete for previously hired drivers - START **********/
                $onboardingDrivers = DriverOnboarding::where('status', 2)->where('application_date', '<', $Setting->hired_before_stage_added_date)->whereNotNull('detail')->get();

                foreach ($onboardingDrivers as $onboardingDriver){
                    $detailsArr = json_decode($onboardingDriver->detail, true);

                    if(!array_key_exists($Setting->stageName, $detailsArr)){
                        $detailsArr = array_merge($detailsArr, $this->getDriverUpdateStage($Setting, 0));
                    }

                    $onboardingDriver->detail = json_encode($detailsArr);
                    $onboardingDriver->save();
                }
                /********** Do not mark complete for previously hired drivers - END **********/
            }
        }
        /********** Apply to all profiles - END **********/

        notification('success', $this->crudMessage('create', 'Driver Onboarding Setting'));

        return response()->json(['success' => "Driver Onboarding Setting created successfully"]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($driverId, $id)
    {
        $data = DriverOnboardingSetting::find($id);

        return view('companyAdmin.driverOnboardingSetting.edit', compact('data'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $driverId, $id)
    {
        $input = $request->all();
        $driverId = base64Decode($input['driver_id']);
        $input['sub_stage'] = Arr::except($input['sub_stage'],['replace_key']);

        $validator = Validator::make($input,[
            'title' => 'unique:driver_onboarding_settings,title,'.$id.',id,company_id,'.auth()->guard('company-admin')->user()->company_id.',deleted_at,NULL',
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $input['date'] = $input['date'] ?? 0;
        $input['file'] = $input['file'] ?? 0;
        $input['apply_to_profile'] = ($input['apply_to'] == 0) ? $driverId : null;
        $input['hired_before_stage_added_date'] = ($input['apply_to'] == 1) ? Carbon::now()->format('Y-m-d') : null;
        $input['mark_as_completed'] = $input['mark_as_completed'] ?? 0;

        $Setting = DriverOnboardingSetting::find($id);

        $Setting->update($input);

        $subStageArray = array_filter(array_map('array_filter', $input['sub_stage']));
        if (!empty($subStageArray)) {
            $ids = [];

            foreach ($subStageArray as $subStage) {
                if (isset($subStage['id'])) {
                    $ids[] = $subStage['id'];
                }
            }

            DriverOnboardingSubStage::where('driver_onboarding_setting_id', $Setting->id)->whereNotIn('id',$ids)->delete();

            foreach ($subStageArray as $subStage) {
                $data = [];
                $data['driver_onboarding_setting_id'] = $Setting->id;
                $data['title'] = $subStage['title'];
                $data['file'] = isset($subStage['file']) ? 1 : 0;
                $data['date'] = isset($subStage['date']) ? 1 : 0;
                $data['mark_as_completed'] = 0;
                // if(isset($subStage['mark_as_completed'])){
                //     $data['file'] = 0;
                //     $data['date'] = 0;
                // }else{
                //     $data['file'] = isset($subStage['file']) ? 1 : 0;
                //     $data['date'] = isset($subStage['date']) ? 1 : 0;
                // }
                // $data['mark_as_completed'] = isset($subStage['mark_as_completed']) ? 1 : 0;
                if (isset($subStage['id'])) {
                    $driverOnboardingSubStage = DriverOnboardingSubStage::find($subStage['id']);
                    $driverOnboardingSubStage->update($data);
                } else {
                    DriverOnboardingSubStage::create($data);
                }
            }
        }

        /********** Apply to this profile only - START **********/
        if(!is_null($Setting->apply_to_profile) && is_null($Setting->hired_before_stage_added_date)){

            $driverOnboarding = DriverOnboarding::where('driver_id', $Setting->apply_to_profile)->first();
            if($driverOnboarding){
                if(!is_null($driverOnboarding->detail)){
                    $detailsArr = json_decode($driverOnboarding->detail, true);

                    if(!array_key_exists($Setting->stageName, $detailsArr)){
                        $detailsArr = array_merge($detailsArr, $this->getDriverUpdateStage($Setting, 0));
                    }

                    $driverOnboarding->detail = json_encode($detailsArr);
                    $driverOnboarding->save();
                }
            }
        }
        /********** Apply to this profile only - END **********/

        /********** Apply to all profiles - START **********/
        if(!is_null($Setting->hired_before_stage_added_date) && is_null($Setting->apply_to_profile)){
            if($Setting->update_profile == 1){
                /********** Mark complete for currently previously drivers - START **********/
                $onboardingDrivers = DriverOnboarding::where('status', 2)->where('application_date', '<', $Setting->hired_before_stage_added_date)->whereNotNull('detail')->get();

                foreach ($onboardingDrivers as $onboardingDriver){
                    $detailsArr = json_decode($onboardingDriver->detail, true);

                    if(!array_key_exists($Setting->stageName, $detailsArr)){
                        $detailsArr = array_merge($detailsArr, $this->getDriverUpdateStage($Setting, 1));
                    }

                    $onboardingDriver->detail = json_encode($detailsArr);
                    $onboardingDriver->save();
                }
                /********** Mark complete for currently previously drivers - END **********/
            }else{
                /********** Do not mark complete for previously hired drivers - START **********/
                $onboardingDrivers = DriverOnboarding::where('status', 2)->where('application_date', '<', $Setting->hired_before_stage_added_date)->whereNotNull('detail')->get();

                foreach ($onboardingDrivers as $onboardingDriver){
                    $detailsArr = json_decode($onboardingDriver->detail, true);

                    if(!array_key_exists($Setting->stageName, $detailsArr)){
                        $detailsArr = array_merge($detailsArr, $this->getDriverUpdateStage($Setting, 0));
                    }

                    $onboardingDriver->detail = json_encode($detailsArr);
                    $onboardingDriver->save();
                }
                /********** Do not mark complete for previously hired drivers - END **********/
            }
        }
        /********** Apply to all profiles - END **********/

        notification('success', $this->crudMessage('update', 'Driver Onboarding Setting'));

        return response()->json(['success' => "Driver Onboarding Setting updated successfully"]);
    }
    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($driverId, $id)
    {
        DriverOnboardingSetting::find($id)->delete();

        notification('success', $this->crudMessage('delete', 'Driver Onboarding Setting'));

       // return redirect()->route('company.admin.driver.onboarding.driverOnboardingSetting.index');
        return redirect()->route('company.admin.driver.onboarding.setting.index', ['driver_id' => $driverId]);
    }

    public function sort(Request $request)
    {
        foreach ($request->sort as $key => $value) {
            driverOnboardingSetting::find($value['id'])->update(['order' => $value['display_order']]);
        }

        return response()->json(['success' => "Order updated successfully"]);
    }

    public function getDriverUpdateStage($Setting, $completedStatus)
    {
        $driverStageCompletedArr = [];
        $driverStageArr[$Setting->stageName]['stage_no'] = (string)$Setting->order;
        if($Setting->date == 1){
            $driverStageArr[$Setting->stageName]['date'] = null;
        }
        if($Setting->file == 1){
            $driverStageArr[$Setting->stageName]['file'] = null;
        }
        if(count($Setting->sub_stages) > 0){
            foreach ($Setting->sub_stages as $sub_stage){
                $subStageName = str_replace(' ', '_', strtolower($sub_stage->title));
                if($sub_stage->date == 1){
                    $driverStageArr[$Setting->stageName][$subStageName]['date'] = null;
                }
                if($sub_stage->file == 1){
                    $driverStageArr[$Setting->stageName][$subStageName]['file'] = null;
                }
                $driverStageArr[$Setting->stageName][$subStageName]['yes_no'] = $completedStatus;
                $driverStageArr[$Setting->stageName][$subStageName]['is_complete'] = $completedStatus;
            }
        }
        $driverStageArr[$Setting->stageName]['yes_no'] = $completedStatus;
        $driverStageArr[$Setting->stageName]['is_complete'] = $completedStatus;

        return $driverStageArr;
    }
}
