<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;

use Illuminate\Support\Arr;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;


use App\Models\Driver;
use App\Models\DriverFinancial;
use App\Models\DriverOnboarding;
use App\Models\DriverOnboardingHistory;
use App\Models\DriverFinancialQuaterlyGlobal;

use App\Models\FileUpload;

use Carbon\Carbon;
use DataTables;

class DriverQuaterlyLoadController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    public function index(Request $request)
    {
        return view('companyAdmin.driverQuaterlyLoad.index');
    }

    public function ajaxData(Request $request){
        $start_date = date('Y-m-d H:i:s',strtotime($request->fistartdate));
        $end_date = date('Y-m-d H:i:s',strtotime($request->fienddate));

        $data       = '';
        $data       = Driver::isCompany()->select(
            'drivers.*',
            'driver_financials.*',
            'driver_financials.start_date as fin_start_date',
            'drivers.id as driverId')
            ->where('drivers.start_date','<=',$end_date)
            ->leftJoin('driver_informations', function ($join) {
                $join->on('drivers.id', '=', 'driver_informations.driver_id');
            })
            ->leftJoin('driver_financials', function ($join) use ($request) {
                $join->on('drivers.id', '=', 'driver_financials.driver_id')
                    ->where('driver_financials.year', $request->year)
                    ->where('driver_financials.quarter', $request->quarter);
            })
            ->groupBy('drivers.id');

        if($request->team_id){
            $data->where('drivers.team_id',$request->team_id);
        }
        if($request->manager_id){
            $data->where('drivers.manager_id',$request->manager_id);
        }
        if($request->carrier){
            $data->where('driver_informations.carrier_id',$request->carrier);
        }
        $data       = $data->get();
        return DataTables::of($data)
            ->addIndexColumn()
            ->editColumn('driver_name', function ($row) {
                return ($row->first_name && $row->last_name) ? $row->last_name.',<br>'.$row->first_name : '';
            })
            ->editColumn('year', function ($row) use ($request) {
                $options            = "";
                for ($x = 2020; $x <= 2030; $x++){
                    $selected = '';
                    if($row->year){
                        $selected   = ($row->year == $x) ? 'selected' : '';
                    }
                    if(!empty($request->year)){
                        $selected   = $request->year == $x ? 'selected' : '';
                    }
                    // $selected = ($row->year == $x) ? 'selected' : '';
                    $options       .= '<option value="'.$x.'" '.$selected.'>'.$x.'</option>';
                }
                $options            = ($row->year) ? $row->year : $request->year;
                // return '<select name="year" class="form-control" required autofocus readonly><option value="">Select Year</option>' . $options . '</select>';
                return '<input type="text" name="year" id="year" class="form-control" readonly value="'.$options.'">';
            })
            ->editColumn('quarter', function ($row)  use ($request){
                    $oneselected    = '';
                    $twoselected    = '';
                    $threeselected  = '';
                    $fourselected   = '';
                if($row->quarter){
                    $oneselected    = $row->quarter == 1 ? 'selected' : '';
                    $twoselected    = $row->quarter == 2 ? 'selected' : '';
                    $threeselected  = $row->quarter == 3 ? 'selected' : '';
                    $fourselected   = $row->quarter == 4 ? 'selected' : '';
                }
                if(!empty($request->quarter)){
                    $oneselected    = $request->quarter == 1 ? 'selected' : '';
                    $twoselected    = $request->quarter == 2 ? 'selected' : '';
                    $threeselected  = $request->quarter == 3 ? 'selected' : '';
                    $fourselected   = $request->quarter == 4 ? 'selected' : '';
                }
                return '<select class="form-control" id="quarter" name="quarter" required disabled>
                    <option  value="">Select Quarter</option>
                    <option  value="1" '.$oneselected.'>Quarter 1</option>
                    <option  value="2" '.$twoselected.'>Quarter 2</option>
                    <option  value="3" '.$threeselected.'>Quarter 3</option>
                    <option  value="4" '.$fourselected.'>Quarter 4</option>
                </select>';
            })
            ->editColumn('occupancy', function ($row) {
                return '<input type="number" class="form-control" name="fin_occupancy" value="'.$row->occupancy.'"/>';
            })
            ->addColumn('FinWeeklyTargetMiles',function ($row) use ($request){
                $quarter            = '';
                $quarter_day        = 90;
                if($row->quarter){
                    $quarter        = $row->quarter;
                }else{
                    $quarter        = $request->quarter;
                }
                if($quarter == '1'){  $quarter_day = 90; }
                if($quarter == '2'){  $quarter_day = 91; }
                if($quarter == '3'){  $quarter_day = 92; }
                if($quarter == '4'){  $quarter_day = 92; }
                $weekly_reve        = ((int)$row->target_miles / $quarter_day) * 7;
                $weekly_reve        = round($weekly_reve);
                return '<input type="number" class="form-control dFinWeeklyTargetMiles" id="" name="FinWeeklyTargetMiles" value="'.$weekly_reve.'">';
            })
            ->addColumn('FinWeeklyGross',function($row) use ($request){
                $quarter            = ($row->quarter) ? $row->quarter : $request->quarter;
                $quarter_day        = 90;
                if($quarter == '1'){
                    $quarter_day    = 90;
                }
                else if($quarter == '2'){
                  $quarter_day      = 91;
                }
                else if($quarter == '3'){
                    $quarter_day    = 92;
                }
                else if($quarter == '4'){
                  $quarter_day      = 92;
                }
                $weekly_miles       = ((int)$row->revenue_target / $quarter_day) * 7;
                $weekly_miles       = round($weekly_miles);
                return '<input type="number" class="form-control dFinWeeklyGross" id="" name="FinWeeklyGross" value="'.$weekly_miles.'">';
            })
            ->editColumn('cpm_target', function ($row) {
                return '<input type="number" class="form-control" name="fin_cpm_target" value="'.$row->cpm_target.'"/>';
            })
            ->editColumn('revenue_target', function ($row) {
                return '<input type="number" class="form-control" name="fin_revenue_target" value="'.$row->revenue_target.'" readonly/>';
            })
            ->editColumn('target_miles', function ($row) {
                return '<input type="number" class="form-control" name="fin_target_miles" value="'.$row->target_miles.'" readonly/>';
            })
            ->editColumn('fin_start_date', function ($row) use ($request) {
                $val = ($row->fin_start_date) ? $row->fin_start_date : date('Y-m-d',strtotime($request->fistartdate));
                return '<input type="text" class="form-control date fistartdate" id="fistartdate" name="start_date" value="'.$val.'" readonly/>';
            })
            ->editColumn('end_date', function ($row) use ($request) {
                $val = ($row->end_date) ? $row->end_date : date('Y-m-d',strtotime($request->fienddate));
                return '<input type="text" class="form-control date fienddate" id="fienddate" name="end_date" value="'.$val.'" readonly/>';
            })
            ->addColumn('action', function ($row) {
                $btn = ' <a href="javascript:void(0)" class="finacsubmit mr-4" data-id="'.$row->id.'" data-driver="'.$row->driverId.'">Save</a>';
                return $btn;
            })
            ->editColumn('checkbox', static function ($row) {
                return '<input type="checkbox" class="checkallDriver" name="driver_id[]" value="'.$row->driverId.'"/>';
            })
            ->rawColumns(['action','checkbox','occupancy','cpm_target','fin_start_date','end_date','FinWeeklyTargetMiles','FinWeeklyGross','revenue_target','target_miles'])
            ->escapeColumns([])
            ->make(true);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function TargetRevenue(Request $request){
        $input                  = $request->except('_method', '_token','driver_id');

        $input['start_date']    = Carbon::createFromFormat('m/d/Y', $input['start_date'])->format('Y-m-d');
        $input['end_date']      = Carbon::createFromFormat('m/d/Y', $input['end_date'])->format('Y-m-d');
        if($request->driver_id){
            foreach ($request->driver_id as $key => $driver) {
                $is_check       = DriverFinancial::where('driver_id', $driver)
                    ->where('year', $input['year'])
                    ->where('quarter', $input['quarter'])
                    ->count();
                if ($is_check == 0) {
                    $input['driver_id'] = $driver;
                    DriverFinancial::create($input);
                }else{
                    $dfinancial         = DriverFinancial::where('driver_id',$driver)->first();
                    $dfinancial->update($input);
                }
            }
        }
        return 1;
    }
}
