<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Helpers\Common;
use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use App\Http\Requests\CompanyAdmin\CompanyDropLocationStoreRequest;
use App\Http\Requests\CompanyAdmin\CompanyDropLocationUpdateRequest;
use App\Models\Bookmark;
use App\Models\CompanyDropLocation;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Facades\DataTables;

class DropLocationController extends CompanyAdminController
{
    /**

     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        try {
            $getField = '';

            $getCompanyAdmin = auth()->guard('company-admin')->user();

            $isCheck = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','company_drop_locations')->first();

            if( !empty( $isCheck->id ) ) {
                //$getField = unserialize($isCheck->config);
                $getField = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_company_drop_locations_first'));
            } else {
                $getField = config('settings.table_company_drop_locations_first');
            }

            if( $request->ajax() ) {
                $data = CompanyDropLocation::IsCompany();

                // General filter code - start
                foreach( $getField as $key => $value ) {
                    $conditionReq = 'search_string_condition_'.$key;
                    $sortReq = 'search_string_sort_'.$key;

                    $conditionDateReq = 'search_date_condition_'.$key;
                    $sortDateReq = 'search_date_sort_'.$key;

                    // String filter
                    if( isset( request()->$conditionReq ) ) {
                        $typeReq = 'search_string_type_'.$key;
                        $type = request()->$typeReq;

                        if( $type == 'string' ) {
                            Common::getStringFilterResult( $data, $key );
                        }
                    }

                    // Date filter
                    if( isset( request()->$conditionDateReq ) ) {
                        $typeReq = 'search_date_type_'.$key;
                        $type = request()->$typeReq;

                        if( $type == 'date' ) {
                            Common::getDateFilterResult( $data, $key );
                        }
                    }

                    // String sort only
                    if( isset( request()->$sortReq ) ) {
                        Common::sortStringFilter( $data, $key );
                    }

                    // Date sort only
                    if( isset( request()->$sortDateReq ) ) {
                        Common::sortDateFilter( $data, $key );
                    }
                }
                // General filter code - end

                $data->select('*', DB::raw('IF(address_verified = 1, "Verified", "Not Verified") as address_verified'));

                return Datatables::of($data)
                    ->addIndexColumn()
                    ->addColumn('action', function($row) {
                        $action = '<a href="'.route('company.admin.drop-locations.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                        $action .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash destroy" data-action="'.route('company.admin.drop-locations.destroy',$row->id).'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';

                        return $action;
                    })
                    ->rawColumns(['action'])
                    ->make(true);
            }

            $tableName = 'settings.table_company_drop_locations';

            return view('companyAdmin.dropLocations.index', compact('getField','tableName'));
        } catch( \Exception $e ) {
            \Log::emergency( "File: ".$e->getFile() );
            \Log::emergency( "Line: ".$e->getLine() );
            \Log::emergency( "Message: ".$e->getMessage() );
        }
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('companyAdmin.dropLocations.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(CompanyDropLocationStoreRequest $request)
    {
        $input = $request->all();

        if($request->address_verified==1){
            $timezone                           = null;
            $timezone_offset                    = null;
            if(isset($input['timezone_abbr'])){
                $timezone                       = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset                = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']                  = $timezone;
            $input['timezone_offset']           = $timezone_offset;
        }

        $input['company_id'] = auth()->guard('company-admin')->user()->company_id;
        CompanyDropLocation::create($input);

        notification('success', $this->crudMessage('create', 'Drop Location'));
        return redirect()->route('company.admin.drop-locations.index');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $id = base64Decode($id);
        $dropLocations = CompanyDropLocation::findOrFail($id);
        return redirect()->route('company.admin.drop-locations.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $id = base64Decode($id);
        $dropLocation = CompanyDropLocation::findOrFail($id);
        return view('companyAdmin.dropLocations.edit', compact('dropLocation'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(CompanyDropLocationUpdateRequest $request, CompanyDropLocation $dropLocation)
    {
        $input = $request->all();

        if($request->address_verified==1){
            $timezone                               = null;
            $timezone_offset                        = null;
            if(isset($input['timezone_abbr'])){
                $timezone                           = timezone_name_from_abbr($input['timezone_abbr']);
                $timezone_offset                    = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
            }
            $input['timezone']                      = $timezone;
            $input['timezone_offset']               = $timezone_offset;
        }else{
            $input['latitude']                      = null;
            $input['longitude']                     = null;
            $input['timezone']                      = null;
            $input['timezone_offset']               = null;
            $input['timezone_abbr']                 = null;
        }

        $dropLocation->update($input);
        notification('success', $this->crudMessage('update', 'Drop Location'));
        return redirect()->route('company.admin.drop-locations.index');
    }

    public function destroy(CompanyDropLocation $dropLocation)
    {
        $dropLocation->delete();

        notification('success', $this->crudMessage('delete', 'Drop Location'));

        return redirect()->route('company.admin.drop-locations.index');
    }

}
