<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;

use App\Models\Bookmark;
use App\Models\CompanyEmailTemplate;
use App\Models\EmailTemplate;

use App\Models\CompanyEmailWrapperTemplate;

use App\Helpers\Common;

use DataTables;

class EmailTemplateController extends CompanyAdminController
{
    /**

     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getField                   = '';
        $getCompanyAdmin            = auth()->guard('company-admin')->user();
        $isCheck                    = Bookmark::where('company_id',$getCompanyAdmin->company_id)
            ->where('user_id',$getCompanyAdmin->id)
            ->where('user_type',$getCompanyAdmin->type)
            ->where('table_id','email_templates')
            ->first();

        if(!empty($isCheck->id)){
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_email_templates_first'));
        }else{
            $getField               = config('settings.table_email_templates_first');
        }

        if ($request->ajax()) {
            $data                   = CompanyEmailTemplate::IsCompany()->select('*')->orderBy('title', 'ASC');
            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq       = 'search_string_condition_'.$key;
                $sortReq            = 'search_string_sort_'.$key;

                if( isset( request()->$conditionReq ) ) {
                    $typeReq        = 'search_string_type_'.$key;
                    $type           = request()->$typeReq;
                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }
                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                if( isset( request()->$sortReq ) ) {
                    $nameReq        = 'search_string_name_'.$key;
                    $sortReq        = 'search_string_sort_'.$key;
                    $typeReq        = 'search_string_type_'.$key;
                    $tableNameReq   = 'search_table_name_'.$key;
                    $colAliasReq    = 'search_string_col_alias_'.$key;

                    $origData       = request()->$nameReq;
                    $condition_sort = request()->$sortReq;
                    $type           = request()->$typeReq;
                    $tableName      = isset( request()->$tableNameReq ) ? request()->$tableNameReq : '';
                    $colAlias       = isset( request()->$colAliasReq ) ? request()->$colAliasReq : '';

                    if( $type == 'string' ) {
                        if( isset( $condition_sort ) ) {
                            if( $colAlias != '' ) {
                                $data->orderBy( $tableName.'.'.$colAlias, $condition_sort );
                            } else {
                                if( $tableName != '' ) {
                                    $data->orderBy( $tableName.'.'.$origData, $condition_sort );
                                } else {
                                    $data->orderBy( $origData, $condition_sort );
                                }
                            }
                        }
                    }
                }
            }
            $data->get();
            /// end
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $action         = '';
                    if(hasPermission(config('settings.company_admin_modules.email_templates'),config('settings.permissions.edit'))){
                        $action    .= '<a href="'.route('company.admin.email-templates.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                        $action    .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-action revert-templates" data-id="'.$row->id.'"><i class="fa fa-undo" aria-hidden="true"></i></a>';
                    }
                    if(hasPermission(config('settings.company_admin_modules.email_templates'),config('settings.permissions.view'))){
                        $action    .= '<a href="'.route('company.admin.email-templates.show', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action" target="_blank"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                    }
                    return $action;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        $tableName                  = 'settings.table_email_templates';
        $emailWrapper               = CompanyEmailWrapperTemplate::IsCompany()->first();
        if(!is_null($emailWrapper)){
            $emailWrapper           = $emailWrapper->content;
        }else{
            $emailWrapper           = view('companyAdmin.emailTemplate.email-wrapper')->render();
        }
        return view('companyAdmin.emailTemplate.index', compact('getField','tableName', 'emailWrapper'));
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id){
        $id                             = base64Decode($id);
        $companyEmailTemplateDetails    = CompanyEmailTemplate::findOrFail($id);
        if($companyEmailTemplateDetails){
            $emailTemplate              = getCompanyEmailTemplate($companyEmailTemplateDetails);
            return view('companyAdmin.emailTemplate.preview', compact('emailTemplate'));
        }else{
            return redirect()->route('company.admin.email-templates.index');
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id){
        $id                             = base64Decode($id);
        $companyEmailTemplate           = CompanyEmailTemplate::findOrFail($id);
        $companyEmailTemplate->tags     = explode(',', $companyEmailTemplate->tags);
        return view('companyAdmin.emailTemplate.edit', compact('companyEmailTemplate'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id){
        $input                          = $request->all();
        $companyEmailTemplate           = CompanyEmailTemplate::IsCompany()->findOrFail($id);
        $companyEmailTemplate->subject  = $input['subject'];
        $companyEmailTemplate->content  = $input['content'];
        $companyEmailTemplate->update($input);
        notification('success', $this->crudMessage('update', 'Email Template'));
        return redirect()->route('company.admin.email-templates.index');
    }

    public function revert(Request $request){
        $response['success']                        = false;
        $response['message']                        = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                if ($request->has('id') && $request->filled('id')) {
                    $companyEmailTemplate           = CompanyEmailTemplate::whereId($request->id)->first();
                    if(!is_null($companyEmailTemplate)){
                        $title                      = $companyEmailTemplate->title;
                        $emailTemplate              = EmailTemplate::select('title', 'subject', 'content', 'tags', 'description')->where('title', $title)->first();
                        if(!is_null($emailTemplate)){
                            $companyEmailTemplate->forceDelete();

                            $companyId              = auth()->guard('company-admin')->user()->company_id;
                            $emailTemplate          = array_merge(array('company_id' => $companyId), $emailTemplate->toArray());
                            CompanyEmailTemplate::create($emailTemplate);

                            $response['success']    = true;
                            $response['message']    = "Email Template Revert Successfully.";
                        }
                    }
                } else {
                    $companyId                      = auth()->guard('company-admin')->user()->company_id;
                    CompanyEmailTemplate::where('company_id', $companyId)->forceDelete();

                    // Pdf Template
                    $emailTemplates                 = EmailTemplate::get(['title', 'subject', 'content', 'tags', 'description'])->toArray();
                    foreach ($emailTemplates as $emailTemplate){
                        $emailTemplate              = array_merge(array('company_id' => $companyId), $emailTemplate);
                        CompanyEmailTemplate::create($emailTemplate);
                    }

                    $response['success']            = true;
                    $response['message']            = "All Email Templates Revert Successfully.";
                }
            }catch (\Exception $e) {
                $response['success']                = false;
                $response['message']                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function emailWrapperTemplate(Request $request){
        $input = $request->except(['_token']);
        CompanyEmailWrapperTemplate::updateOrCreate(['company_id' => $input['company_id']], $input);
        notification('success', $this->crudMessage('update', 'Email Wrapper Template'));
        return redirect()->route('company.admin.email-templates.index');
    }

    public function emailWrapperTemplatesRevert(){
        try {
            $emailWrapper = view('companyAdmin.emailTemplate.email-wrapper')->render();
            return response()->json(['success' => true, 'msg' => 'HTML Of Email Wrapper Template Revert Successfully.', 'emailWrapper' => $emailWrapper]);
        }catch (\Exception $e) {
            return response()->json(['success' => false, 'msg' => $e->getMessage()]);
        }
    }
}
