<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;
use App\Models\GalleryDocument;
use App\Models\GalleryDocumentNme;
use App\Models\Bookmark;

use DataTables;
use Validator;
use Arr;

class GalleryDocumentController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $getField = '';

        $getCompanyAdmin = auth()->guard('company-admin')->user();
        $isCheck = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','gallery_document')->first();

        if(!empty($isCheck->id)){
            //$getField       = unserialize($isCheck->config);
            $getField       = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_gallery_document_first'));
        }else{
            $getField = config('settings.table_gallery_document_first');
        }
        if ($request->ajax()) {
            $data = GalleryDocument::isCompany()->select('*');
            return Datatables::of($data)
                    ->addIndexColumn()
                    ->addColumn('action', function($row){
                        $btn = '<a href="'.route('company.admin.gallery-documents.show', $row->id).'" class="mr-4"><img src="'.asset('theme/img/eye.png').'" alt=""></a>';
                        $btn = $btn.' <a href="'.route('company.admin.gallery-documents.edit', $row->id).'" class="mr-4"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';
                        $btn = $btn.' <a href="javascript:" class="destroy" data-action="'.route('company.admin.gallery-documents.destroy',$row->id).'" data-id="'.$row->id.'"><img src="'.asset('theme/img/delete-icon.png').'" alt=""></a>';
                        return $btn;
                    })
                    ->rawColumns(['action'])
                    ->make(true);
        }

        return view('companyAdmin.galleryDocument.index', compact('getField'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('companyAdmin.galleryDocument.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $input = $request->all();

        $input['contacts'] = Arr::except($input['contacts'],['replace_key']);

        $validator = Validator::make($input,[
            'name' => 'required',
            'mc_number' => 'required|unique:gallery-documents,mc_number,NULL,id,company_id,'.auth()->guard('company-admin')->user()->company_id.',deleted_at,NULL',
            'contacts.*.name' => 'required',
            //'contacts.*.phone_number' => 'required|size:10',
            'contacts.*.email' => 'nullable|email',
        ],
        [
            'contacts.*.name.required' => 'The name field is required.',
            'contacts.*.phone_number.required' => 'The phone number field is required.',
            //'contacts.*.phone_number.size' => 'The phone number must be 10 digits.',
            'contacts.*.email.email' => 'The email must be a valid email address.',
        ]
        );

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $input['company_id'] = auth()->guard('company-admin')->user()->company_id;

        GalleryDocument::create($input);

        notification('success', $this->crudMessage('create', 'GalleryDocument'));

        return response()->json(['success' => "GalleryDocument created successfully"]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(GalleryDocument $galleryDocument)
    {
        return view('companyAdmin.galleryDocument.edit', compact('galleryDocument'));
    }

    /**
     * Display the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(GalleryDocument $galleryDocument)
    {
        return view('companyAdmin.galleryDocument.show', compact('galleryDocument'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, GalleryDocument $galleryDocument)
    {
        $input = $request->all();

        $galleryDocument->update($input);

        notification('success', $this->crudMessage('update', 'GalleryDocument'));

        return response()->json(['success' => "GalleryDocument updated successfully"]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(GalleryDocument $galleryDocument)
    {
        $galleryDocument->delete();

        notification('success', $this->crudMessage('delete', 'GalleryDocument'));

        return redirect()->route('company.admin.gallery-documents.index');
    }
}
