<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;
use Illuminate\Support\Str;

use App\Http\Requests\CompanyAdmin\LeaveStoreRequest;
use App\Http\Requests\CompanyAdmin\LeaveUpdateRequest;

use App\Mail\CompanyAdmin\CreateLeave;
use App\Mail\CompanyAdmin\ApprovedLeave;

use App\Models\Leave;
use App\Models\LeaveUserType;

use App\Models\Driver;
use App\Models\CompanyAdmin;

use DB;
use Mail;
use Hash;
use DataTables;
use Carbon\Carbon;
use Carbon\CarbonPeriod;

class LeaveController extends CompanyAdminController{

    /**
     * Create a new controller instance.
     *
     * @return void
     */

    public $timeZone;

    public function __construct(){
        parent::__construct();

        $this->middleware(function ($request, $next) {
            $this->timeZone     = getTimeZone();

            return $next($request);
        });
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        if ($request->ajax()) {
            $startDate      = Carbon::parse($request->start_date)->format('Y-m-d');
            $endDate        = Carbon::parse($request->end_date)->format('Y-m-d');

            $getLeave       = Leave::IsCompany()
                ->where('status',0)
                ->whereBetween('from_date', [$startDate, $endDate])
                ->get();

            $getLeave2      = Leave::IsCompany()
                ->where('status',0)
                ->whereBetween('to_date', [$startDate, $endDate])
                ->get();

            if (count($getLeave2) > 0) {
                $getLeave   = $getLeave2->merge($getLeave);
            }

            $view = view('companyAdmin.leave.view.listLeave', compact('getLeave'))->render();
            return response()->json(['view' => $view]);
        }
        return view('companyAdmin.leave.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return view('companyAdmin.leave.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(LeaveStoreRequest $request){
        $input                                                  = $request->except('_token');

        $leaveUserType                                          = [];
        $to                                                     = Carbon::createFromFormat('m/d/Y', $request->to_date);
        $from                                                   = Carbon::createFromFormat('m/d/Y', $request->from_date);
        $diff_in_days                                           = $to->diffInDays($from) + 1;
        $input['day_count']                                     = $diff_in_days;

        if(isset($request->driver_id)){
            $input['driver_id']                                 = $request->driver_id;

            $getLeaveUserType                                   = LeaveUserType::where('driver_id', $request->driver_id)->where('leave_type_id', $request->leave_type_id)->where('year', date('Y'))->first();
            $leaveUserType['days_per_year_for_driver_used']     = $getLeaveUserType->days_per_year_for_driver_used + $diff_in_days;
            $getLeaveUserType->update($leaveUserType);

            $input['company_id']                                = getUserCompnayId();
            $leave                                              = Leave::create($input);

            $getDriver                                          = Driver::find($request->driver_id);
            $description                                        = 'Apply leave by '.$getDriver->full_name;

            // Twilio's credentials are fetched from company table
            $getCompanyTwilioData                               = getCompanyTwilioData($getDriver->company_id);
            $getCompanyAdminByRole                              = getCompanyAdminByRole(config('settings.company_admin_modules.leaves'),config('settings.permissions.approve'));
            foreach($getCompanyAdminByRole as $userdata){
                createNotification(
                    7, // source
                    'Leave Request', // name
                    $userdata->id, // user_id
                    $userdata->type, // user_type
                    1, // type
                    $leave->id, // action_id
                    $description // description
                );
                $checkNotify                                    = isUnreadDelay($userdata->id, $userdata->type, 7);
                if(isset($checkNotify)){
                    if ($checkNotify->email == 1 && isset($userdata->email)) {
                        $mailData                               = [];
                        $mailData['company_id']                 = auth()->guard('company-admin')->user()->company_id;
                        $mailData['emailto']                    = $userdata->email;
                        $mailData['hiname']                     = $userdata->full_name;
                        $mailData['name']                       = $getDriver->full_name;
                        $mailData['email']                      = $getDriver->email;
                        $mailData['phone']                      = $getDriver->cell_phone;
                        $mailData['leave_date']                 = $leave->from_date.' to '.$leave->to_date;
                        $mailData['day_count']                  = $leave->day_count;
                        $mailData['comment']                    = $leave->comment;
                        //Mail::to($mailData['emailto'])->send(new CreateLeave($mailData));
                        sendEmail($mailData['emailto'], new CreateLeave($mailData));
                    }

                    if ($checkNotify->sms == 1 && isset($userdata->cell_phone)) {
                        if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                            $message                            = "Leave Request by ".$getDriver->full_name." from ".$leave->from_date." to ".$leave->to_date;
                            sendSMS($userdata->cell_phone, $message, $getCompanyTwilioData);
                        }
                    }
                }
            }
        }

        if(isset($request->dispatcher_id)){
            $input['company_admins_id']                         = $request->dispatcher_id;

            $getLeaveUserType                                   = LeaveUserType::where('company_admins_id', $request->dispatcher_id)->where('leave_type_id', $request->leave_type_id)->where('year', date('Y'))->first();
            $leaveUserType['days_per_year_for_dispatcher_used'] = $getLeaveUserType->days_per_year_for_dispatcher_used + $diff_in_days;
            $getLeaveUserType->update($leaveUserType);

            $input['company_id']                                = getUserCompnayId();
            $leave                                              = Leave::create($input);

            $getCompanyAdmin                                    = CompanyAdmin::find($request->dispatcher_id);
            $description                                        = 'Apply leave by '.$getCompanyAdmin->full_name;

            // Twilio's credentials are fetched from company table
            $getCompanyTwilioData                               = getCompanyTwilioData($getCompanyAdmin->company_id);
            $getCompanyAdminByRole                              = getCompanyAdminByRole(config('settings.company_admin_modules.leaves'),config('settings.permissions.approve'));
            foreach($getCompanyAdminByRole as $userdata){
                createNotification(
                    7, // source
                    'Leave Request', // name
                    $userdata->id, // user_id
                    $userdata->type, // user_type
                    1, // type
                    $leave->id, // action_id
                    $description // description
                );
                $checkNotify                                    = isUnreadDelay($userdata->id, $userdata->type, 7);
                if(isset($checkNotify)){
                    if ($checkNotify->email == 1 && isset($userdata->email)) {
                        $mailData                               = [];
                        $mailData['company_id']                 = auth()->guard('company-admin')->user()->company_id;
                        $mailData['emailto']                    = $userdata->email;
                        $mailData['hiname']                     = $userdata->full_name;
                        $mailData['name']                       = $getCompanyAdmin->full_name;
                        $mailData['email']                      = $getCompanyAdmin->email;
                        $mailData['phone']                      = $getCompanyAdmin->cell_phone;
                        $mailData['leave_date']                 = $leave->from_date.' to '.$leave->to_date;
                        $mailData['day_count']                  = $leave->day_count;
                        $mailData['comment']                    = $leave->comment;
                        //Mail::to($mailData['emailto'])->send(new CreateLeave($mailData));
                        sendEmail($mailData['emailto'], new CreateLeave($mailData));
                    }

                    if ($checkNotify->sms == 1 && isset($userdata->cell_phone)) {
                        if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                            $message                            = "Leave Request by ".$getCompanyAdmin->full_name." from ".$leave->from_date." to ".$leave->to_date;
                            sendSMS($userdata->cell_phone, $message, $getCompanyTwilioData);
                        }
                    }
                }
            }
        }

        if(isset($request->manager_id)){
            $input['company_admins_id']                         = $request->manager_id;

            $getLeaveUserType                                   = LeaveUserType::where('company_admins_id', $request->manager_id)->where('leave_type_id', $request->leave_type_id)->where('year', date('Y'))->first();
            $leaveUserType['days_per_year_for_manager_used']    = $getLeaveUserType->days_per_year_for_manager_used + $diff_in_days;
            $getLeaveUserType->update($leaveUserType);

            $input['company_id']                                = getUserCompnayId();
            $leave                                              = Leave::create($input);

            $getCompanyAdmin                                    = CompanyAdmin::find($request->manager_id);
            $description                                        = 'Apply leave by '.$getCompanyAdmin->full_name;

            // Twilio's credentials are fetched from company table
            $getCompanyTwilioData                               = getCompanyTwilioData($getCompanyAdmin->company_id);
            $getCompanyAdminByRole                              = getCompanyAdminByRole(config('settings.company_admin_modules.leaves'),config('settings.permissions.approve'));
            foreach($getCompanyAdminByRole as $userdata){
                createNotification(
                    7, // source
                    'Leave Request', // name
                    $userdata->id, // user_id
                    $userdata->type, // user_type
                    1, // type
                    $leave->id, // action_id
                    $description // description
                );
                $checkNotify                                    = isUnreadDelay($userdata->id, $userdata->type, 7);
                if(isset($checkNotify)){
                    if ($checkNotify->email == 1 && isset($userdata->email)) {
                        $mailData                               = [];
                        $mailData['company_id']                 = auth()->guard('company-admin')->user()->company_id;
                        $mailData['emailto']                    = $userdata->email;
                        $mailData['hiname']                     = $userdata->full_name;
                        $mailData['name']                       = $getCompanyAdmin->full_name;
                        $mailData['email']                      = $getCompanyAdmin->email;
                        $mailData['phone']                      = $getCompanyAdmin->cell_phone;
                        $mailData['leave_date']                 = $leave->from_date.' to '.$leave->to_date;
                        $mailData['day_count']                  = $leave->day_count;
                        $mailData['comment']                    = $leave->comment;
                        //Mail::to($mailData['emailto'])->send(new CreateLeave($mailData));
                        sendEmail($mailData['emailto'], new CreateLeave($mailData));
                    }

                    if ($checkNotify->sms == 1 && isset($userdata->cell_phone)) {
                        if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                            $message                            = "Leave Request by ".$getCompanyAdmin->full_name." from ".$leave->from_date." to ".$leave->to_date;
                            sendSMS($userdata->cell_phone, $message, $getCompanyTwilioData);
                        }
                    }
                }
            }
        }

        notification('success', $this->crudMessage('create', 'Leave'));
        return redirect()->back();
    }

    public function leaveView(Request $request){
        $getLeave   = Leave::find($request->leave_id);
        $view       = view('companyAdmin.leave.view.leaveDetails', compact('getLeave'))->render();
        return response()->json(['view' => $view]);
    }

    public function leaveCalanderView(Request $request){
        $year               = '';
        $month              = '';
        $month_name         = '';
        $data_type          = '';
        $data_type_id       = '';

        if($request->get_data == 'current'){
            $now            = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone);
        }elseif($request->get_data == 'next'){
            $now            = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone)->addMonth();
        }elseif($request->get_data == 'previous'){
            $now            = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone)->subMonth();
        }else{
            $now            = Carbon::now()->timezone($this->timeZone);
        }

        $month              = $now->format('m');
        $month_name         = $now->format('F');
        $year               = $now->format('Y');
        $startDate          = $now->startOfMonth()->format('Y-m-d');
        $endDate            = $now->endOfMonth()->format('Y-m-d');

        if(isset($request->driver_id) && !empty($request->driver_id)){
            $data_type      = 'driver';
            $data_type_id   = $request->driver_id;
        }

        if(isset($request->dispatcher_id) && !empty($request->dispatcher_id)){
            $data_type      = 'dispatchers';
            $data_type_id   = $request->dispatcher_id;
        }

        if(isset($request->manager_id) && !empty($request->manager_id)){
            $data_type      = 'managers';
            $data_type_id   = $request->manager_id;
        }

        $getWeekRange       = getWeekRange($year,$month);

        $dateRange          = CarbonPeriod::create($startDate, $endDate);

        $monthdata          = [];
        $k=0;
        foreach($dateRange as $drange){
            $monthdata[$k]  = ['date' => $drange->format('Y-m-d'),'dayname' => substr($drange->format('l'), 0, 3),'daynumber' => $drange->format('d')];
            $k++;
        }

        $view               = view('companyAdmin.leave.view.leaveCalendar', compact('year','month','month_name','startDate','endDate','monthdata','getWeekRange','data_type','data_type_id'))->render();
        return response()->json(['view' => $view,'month' => $month,'year' => $year,'month_name' => $month_name]);
    }

    public function getTakeDecision(Request $request){
        $getLeave           = Leave::findOrFail($request->id);
        $view               = view('companyAdmin.leave.view.getTakeDecision', compact('getLeave'))->render();
        return response()->json(['view' => $view]);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Leave $leave){
       return redirect()->route('company.admin.leaves.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Leave $leave){
        return view('companyAdmin.leave.edit', compact('leave'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(LeaveUpdateRequest $request, $id){
        $input                                          = $request->all();
        if(!empty(auth()->guard('company-admin')->user()->id)){
            $leave                                      = Leave::find($id);
            $input['approved_by_company_admins_id']     = auth()->guard('company-admin')->user()->id;
            $leave->update($input);

            if($request->status == "1"){
                $description                            = 'Your leave is Approved';
                $type                                   = 1;
            }else{
                $description                            = 'Your leave is Declined';
                $type                                   = 0;
            }
            if(!empty($leave->driver_id)){
                $getDriver                              = Driver::findOrFail($leave->driver_id);

                createNotification(
                    8, // source
                    'Leave', // name
                    $leave->driver_id, // user_id
                    3, // user_type
                    $type, // type
                    $leave->id, // action_id
                    $description // description
                );
                $checkNotify                            = isUnreadDelay($leave->driver_id, 3, 8);
                if(isset($checkNotify)){
                    if ($checkNotify->email == 1 && isset($getDriver->email)) {
                        // send mail
                        $mailData                       = [];
                        $mailData['subject']            = $description;
                        $mailData['company_id']         = auth()->guard('company-admin')->user()->company_id;
                        $mailData['emailto']            = $getDriver->email;
                        $mailData['hiname']             = $getDriver->full_name;
                        $mailData['name']               = $getDriver->full_name;
                        $mailData['email']              = $getDriver->email;
                        $mailData['phone']              = $getDriver->cell_phone;
                        $mailData['leave_date']         = $leave->from_date.' to '.$leave->to_date;
                        $mailData['day_count']          = $leave->day_count;
                        $mailData['comment']            = $leave->comment;
                        //Mail::to($mailData['emailto'])->send(new ApprovedLeave($mailData));
                        sendEmail($mailData['emailto'], new ApprovedLeave($mailData));
                    }

                    if ($checkNotify->sms == 1 && isset($getDriver->cell_phone)) {
                        // Twilio's credentials are fetched from company table
                        $getCompanyTwilioData           = getCompanyTwilioData($getDriver->company_id);
                        if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                            $message                    = $description." - ".$getDriver->full_name." from ".$leave->from_date." to ".$leave->to_date;
                            sendSMS($getDriver->cell_phone, $message, $getCompanyTwilioData);
                        }
                    }
                }

            }
            if(!empty($leave->company_admins_id)){
                $getCompanyAdmin                        = CompanyAdmin::findOrFail($leave->company_admins_id);

                createNotification(
                    7, // source
                    'Leave', // name
                    $getCompanyAdmin->id, // user_id
                    $getCompanyAdmin->type, // user_type
                    $type, // type
                    $leave->id, // action_id
                    $description // description
                );
                $checkNotify                            = isUnreadDelay($getCompanyAdmin->id, $getCompanyAdmin->type, 7);
                if(isset($checkNotify)){
                    if ($checkNotify->email == 1 && isset($getCompanyAdmin->email)) {
                        $mailData                       = [];
                        $mailData['subject']            = $description;
                        $mailData['company_id']         = auth()->guard('company-admin')->user()->company_id;
                        $mailData['emailto']            = $getCompanyAdmin->email;
                        $mailData['hiname']             = $getCompanyAdmin->email;
                        $mailData['name']               = $getCompanyAdmin->full_name;
                        $mailData['email']              = $getCompanyAdmin->email;
                        $mailData['phone']              = $getCompanyAdmin->cell_phone;
                        $mailData['leave_date']         = $leave->from_date.' to '.$leave->to_date;
                        $mailData['day_count']          = $leave->day_count;
                        $mailData['comment']            = $leave->comment;
                        //Mail::to($mailData['emailto'])->send(new CreateLeave($mailData));
                        sendEmail($mailData['emailto'], new CreateLeave($mailData));
                    }

                    if ($checkNotify->sms == 1 && isset($getCompanyAdmin->cell_phone)) {
                        // Twilio's credentials are fetched from company table
                        $getCompanyTwilioData           = getCompanyTwilioData($getCompanyAdmin->company_id);
                        if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                            $message                    = $description." - ".$getCompanyAdmin->full_name." from ".$leave->from_date." to ".$leave->to_date;
                            sendSMS($getCompanyAdmin->cell_phone, $message, $getCompanyTwilioData);
                        }
                    }
                }
            }
            notification('success', $this->crudMessage('update', 'Leave'));
            return redirect()->route('company.admin.leaves.index');
        }
        notification('error', 'Admin ID not found.');
        return redirect()->route('company.admin.leaves.index');
    }

    public function destroy(Leave $leave){
        $leave->delete();
        notification('success', $this->crudMessage('delete', 'Leave'));
        return redirect()->route('company.admin.leaves.index');
    }
}
