<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;

use App\Models\Driver;

use Carbon\Carbon;
use Carbon\CarbonPeriod;

use DB;

class LinerFleetCalendarController extends CompanyAdminController{

    /**
     * Create a new controller instance.
     *
     * @return void
     */

    public $timeZone;

    public function __construct(){
        parent::__construct();

        $this->middleware(function ($request, $next) {
            $this->timeZone     = getTimeZone();

            return $next($request);
        });
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $year           = ($request->has('year') && $request->year!='')?$request->year:'';
        $month          = ($request->has('month') && $request->month!='')?$request->month:'';

        if(!empty($request->shipper_date)){
            $year       = Carbon::createFromFormat('m/d/Y H:i',$request->shipper_date,$this->timeZone)->format('Y');
            $month      = Carbon::createFromFormat('m/d/Y H:i',$request->shipper_date,$this->timeZone)->format('m');
        }

        $manager        = null;
        if($year!='' && $month!=''){
            $manager    = ($request->has('manager') && $request->manager!='')?$request->manager:null;
            $now        = Carbon::create($year, $month)->startOfMonth();
            $month_name = $now->format('F');
            $month      = $now->format('m');
            $year       = $now->format('Y');
        }else{
            $now        = Carbon::now()->timezone($this->timeZone);
            $month_name = $now->format('F');
            $month      = $now->format('m');
            $year       = $now->format('Y');
        }
        return view('companyAdmin.linerFleetCalendar.index',compact('manager','month_name','month','year'));
    }


    public function LinerFleetCalanderView(Request $request){
        $year           = '';
        $month          = '';
        $month_name     = '';

        if($request->get_data == 'current'){
            $now        = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone);
        }elseif($request->get_data == 'next'){
            $now        = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone)->addMonth();
        }elseif($request->get_data == 'previous'){
            $now        = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone)->subMonth();
        }else{
            $now        = Carbon::now()->timezone($this->timeZone);
        }

        $month          = $now->format('m');
        $month_name     = $now->format('F');
        $year           = $now->format('Y');
        $startDate      = $now->startOfMonth()->format('Y-m-d');
        $endDate        = $now->endOfMonth()->format('Y-m-d');

        $data           = Driver::select("drivers.id","drivers.first_name","drivers.last_name","drivers.middle_name")
            //->join("load_details","load_details.driver_id","=","drivers.id")
            //->join("driver_informations","driver_informations.driver_id","=","drivers.id")
            ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
            ->where(function ($query) use ($startDate){
                $query->where('drivers.status', '!=', '2')->orWhere(function ($que) use ($startDate){
                    $que->where('drivers.status', '2')->where(DB::raw('DATE_FORMAT(drivers.termination_date, "%Y-%m-%d")'), '>=', $startDate);
                });
            });
            // ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'),[$startDate, $endDate])
            //->groupBy('drivers.id')
            //->orderByRaw('drivers.last_name ASC, drivers.first_name ASC')
            //->where('load_details.is_disable',0)
            //->where('load_details.load_status', '!=', 9);

            if (!is_null($request->team)) {
                $data   = $data->where('drivers.team_id',$request->team);
            }
            if (!is_null($request->manager)) {
                $data   = $data->where('drivers.manager_id',$request->manager);
            }
//            if (!is_null($request->carrier)) {
//                $data   = $data->where('driver_informations.carrier_id',$request->carrier);
//            }
        $driver_list    = $data->limit(30)->get();

        $getWeekRange   = getWeekRange($year,$month);
        $dateRange      = CarbonPeriod::create($startDate, $endDate);
        $monthdata      = [];
        $k              = 0;
        foreach($dateRange as $drange){
            $monthdata[$k] = ['date' => $drange->format('Y-m-d'),'dayname' => substr($drange->format('l'), 0, 3),'daynumber' => $drange->format('d')];
            $k++;
        }
        $view           = view('companyAdmin.linerFleetCalendar.view.linerFleetCalendar', compact('year','month','month_name','startDate','endDate','driver_list','monthdata','getWeekRange'))->render();
        return response()->json(['view' => $view,'month' => $month,'year' => $year,'month_name' => $month_name]);
    }

    public function LinerFleetCalanderTablebody(Request $request){
        $year           = '';
        $month          = '';
        $month_name     = '';

        if($request->get_data == 'current'){
            $now        = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone);
        }elseif($request->get_data == 'next'){
            $now        = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone)->addMonth();
        }elseif($request->get_data == 'previous'){
            $now        = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone)->subMonth();
        }else{
            $now        = Carbon::now()->timezone($this->timeZone);
        }

        $month          = $now->format('m');
        $month_name     = $now->format('F');
        $year           = $now->format('Y');
        $startDate      = $now->startOfMonth()->format('Y-m-d');
        $endDate        = $now->endOfMonth()->format('Y-m-d');

        $data           = Driver::select("drivers.id","drivers.first_name","drivers.last_name","drivers.middle_name")
            ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id);

        if (!is_null($request->team)) {
            $data       = $data->where('drivers.team_id',$request->team);
        }
        if (!is_null($request->manager)) {
            $data       = $data->where('drivers.manager_id',$request->manager);
        }
        if (!is_null($request->carrier)) {
            $data       = $data->where('driver_informations.carrier_id',$request->carrier);
        }

        $driver_list    = $data->limit($request->take)->skip($request->skip)->get();
        $driver_count   = count($driver_list);
        $getWeekRange   = getWeekRange($year,$month);
        $dateRange      = CarbonPeriod::create($startDate, $endDate);
        $monthdata      = [];
        $k              = 0;
        foreach($dateRange as $drange){
            $monthdata[$k] = ['date' => $drange->format('Y-m-d'),'dayname' => substr($drange->format('l'), 0, 3),'daynumber' => $drange->format('d')];
            $k++;
        }

        $view           = view('companyAdmin.linerFleetCalendar.view.tablebody', compact('year','month','month_name','startDate','endDate','driver_list','monthdata','getWeekRange'))->render();
        return response()->json(['view' => $view,'month' => $month,'year' => $year,'month_name' => $month_name,'driver_count' => $driver_count]);
    }
}
