<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;

use App\Models\Truck;
use App\Models\Driver;
use App\Models\Trailer;

use App\Models\Carrier;
use App\Models\Bookmark;
use App\Models\ListWidget;

use App\Helpers\Common;

use DB;
use DataTables;
use Carbon\Carbon;

class ListController extends CompanyAdminController{

    public $timeZone;
    public $timeZoneId;
    public $offset;

    public function __construct(){
        parent::__construct();

        $this->middleware(function ($request, $next) {
            $this->timeZone         = getTimeZone();
            $this->timeZoneId       = getTimeZoneId();
            $this->offset           = getTimeZoneOffset();

            return $next($request);
        });
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getCompanyAdmin            = auth()->guard('company-admin')->user();

        $getField2                  = '';
        $isCheck                    = Bookmark::where('company_id', $getCompanyAdmin->company_id)
            ->where('user_id', $getCompanyAdmin->id)
            ->where('user_type', $getCompanyAdmin->type)
            ->where('table_id', 'lists_drivers')
            ->first();
        if(!empty($isCheck->id)){
            $getField2              = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_lists_drivers_first'));
        }else{
            $getField2              = config('settings.table_lists_drivers_first');
        }

        $getField3                  = '';
        $isCheck                    = Bookmark::where('company_id', $getCompanyAdmin->company_id)
            ->where('user_id', $getCompanyAdmin->id)
            ->where('user_type', $getCompanyAdmin->type)
            ->where('table_id', 'lists_trucks')
            ->first();
        if(!empty($isCheck->id)){
            $getField3              = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_lists_trucks_first'));
        }else{
            $getField3              = config('settings.table_lists_trucks_first');
        }

        $getField4                  = '';
        $isCheck                    = Bookmark::where('company_id', $getCompanyAdmin->company_id)
            ->where('user_id', $getCompanyAdmin->id)
            ->where('user_type', $getCompanyAdmin->type)
            ->where('table_id', 'lists_trailers')
            ->first();
        if(!empty($isCheck->id)){
            $getField4              = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_lists_trailers_first'));
        }else{
            $getField4              = config('settings.table_lists_trailers_first');
        }

        $isMulitpleTable            = 1;

        $tables                     = array(
            array(
                'name'              => 'table1',
                'id'                => 'drivers-all-data-table',
                'class'             => 'drivers-all-data-table',
                'table'             => 'settings.table_lists_drivers',
                'route'             => route('company.admin.lists.drivers.all'),
                'getField'          => $getField2
            ),
            array(
                'name'              => 'table2',
                'id'                => 'drivers-company-data-table',
                'class'             => 'drivers-company-data-table',
                'table'             => 'settings.table_lists_drivers',
                'route'             => route('company.admin.lists.drivers.all'),
                'getField'          => $getField2
            ),
            array(
                'name'              => 'table3',
                'id'                => 'drivers-owner-operators-data-table',
                'class'             => 'drivers-owner-operators-data-table',
                'table'             => 'settings.table_lists_drivers',
                'route'             => route('company.admin.lists.drivers.all'),
                'getField'          => $getField2
            ),
            array(
                'name'              => 'table4',
                'id'                => 'trucks-all-data-table',
                'class'             => 'trucks-all-data-table',
                'table'             => 'settings.table_lists_trucks',
                'route'             => route('company.admin.lists.trucks.all'),
                'getField'          => $getField3
            ),
            array(
                'name'              => 'table5',
                'id'                => 'trucks-company-data-table',
                'class'             => 'trucks-company-data-table',
                'table'             => 'settings.table_lists_trucks',
                'route'             => route('company.admin.lists.trucks.all'),
                'getField'          => $getField3
            ),
            array(
                'name'              => 'table6',
                'id'                => 'trucks-others-data-table',
                'class'             => 'trucks-others-data-table',
                'table'             => 'settings.table_lists_trucks',
                'route'             => route('company.admin.lists.trucks.all'),
                'getField'          => $getField3
            ),
            array(
                'name'              => 'table7',
                'id'                => 'trailers-all-data-table',
                'class'             => 'trailers-all-data-table',
                'table'             => 'settings.table_lists_trailers',
                'route'             => route('company.admin.lists.trailers.all'),
                'getField'          => $getField4
            ),
            array(
                'name'              => 'table8',
                'id'                => 'trailers-company-data-table',
                'class'             => 'trailers-company-data-table',
                'table'             => 'settings.table_lists_trailers',
                'route'             => route('company.admin.lists.trailers.all'),
                'getField'          => $getField4
            ),
            array(
                'name'              => 'table9',
                'id'                => 'trailers-others-data-table',
                'class'             => 'trailers-others-data-table',
                'table'             => 'settings.table_lists_trailers',
                'route'             => route('company.admin.lists.trailers.all'),
                'getField'          => $getField4
            )
        );

        // Widget
        $driverWidgetActive         = "1";
        $driverWidget               = ListWidget::IsCompanyAdmin()->where('tab', 'driver')->latest()->first();
        if(!is_null($driverWidget)){
            $driverWidgetActive     = $driverWidget->isActive;
        }

        $truckWidgetActive          = "1";
        $truckWidget                = ListWidget::IsCompanyAdmin()->where('tab', 'truck')->latest()->first();
        if(!is_null($truckWidget)){
            $truckWidgetActive      = $truckWidget->isActive;
        }

        $trailerWidgetActive        = "1";
        $trailerWidget              = ListWidget::IsCompanyAdmin()->where('tab', 'trailer')->latest()->first();
        if(!is_null($trailerWidget)){
            $trailerWidgetActive    = $trailerWidget->isActive;
        }

        // Driver Count
        $totalActiveDrivers         = Driver::isCompany()->where('status', '0')->count();
        $totalOosDrivers            = Driver::isCompany()->where('status', '1')->count();
        $totalOwnerOperators        = Driver::isCompany()->where('is_company_driver', '0')->withTrashed()->count(); // 0 = Owner Operator, 1 = Company Driver, 2 = Subcontractor
        $totalCompanyDrivers        = Driver::isCompany()->where('is_company_driver', '1')->withTrashed()->count();
        $totalDrivers               = Driver::isCompany()->withTrashed()->count();

        // Truck Count
        $totalTrucks                = Truck::isCompany()->withTrashed()->count();
        $totalActiveTrucks          = Truck::isCompany()->where(function($q) {
            $q->whereNull('status')->orWhere('status', '0');
        })->count();
        $totalInactiveTrucks        = Truck::isCompany()->where(function($q) {
            $q->whereNotNull('deleted_at')->orWhere('status', '2');
        })
        ->withTrashed() // Include soft deleted records
        ->count();

        // Trailer Count
        $totalTrailers              = Trailer::isCompany()->withTrashed()->count();
        $totalActiveTrailers        = Trailer::isCompany()->where(function($q) {
            $q->whereNull('status')->orWhere('status', 0);
        })->count();
        $totalInactiveTrailers      = Trailer::isCompany()->where(function($q) {
            $q->whereNotNull('deleted_at')->orWhere('status', 2);
        })
        ->withTrashed() // Include soft deleted records
        ->count();
        return view('companyAdmin.list.index', compact(
                'getField2',
                'getField3',
                'getField4',
                'isMulitpleTable',
                'tables',

                'driverWidgetActive',
                'truckWidgetActive',
                'trailerWidgetActive',

                'totalActiveDrivers',
                'totalOosDrivers',
                'totalOwnerOperators',
                'totalCompanyDrivers',
                'totalDrivers',

                'totalTrucks',
                'totalActiveTrucks',
                'totalInactiveTrucks',

                'totalTrailers',
                'totalActiveTrailers',
                'totalInactiveTrailers'
            )
        );
    }

    public function driversAll(Request $request){
        $getCompanyAdmin            = auth()->guard('company-admin')->user();

        $getField                   = '';
        $isCheck                    = Bookmark::where('company_id', $getCompanyAdmin->company_id)
            ->where('user_id', $getCompanyAdmin->id)
            ->where('user_type', $getCompanyAdmin->type)
            ->where('table_id','lists_drivers')
            ->first();
        if(!empty($isCheck->id)) {
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_lists_drivers_first'));
        } else {
            $getField               = config('settings.table_lists_drivers_first');
        }

        $search                     = $request->search['value'] ?? ''; // $request->input('search.value');

        $data                       = Driver::selectRaw('
                drivers.*,
                CASE 
                    WHEN drivers.status IS NULL THEN "Active"
                    WHEN drivers.status = "0" THEN "Active"
                    WHEN drivers.status = "1" THEN "Out Of Service"
                    WHEN drivers.status = "2" THEN "Terminate"
                    WHEN drivers.status = "3" THEN "Rehire"
                    ELSE ""
                END as status_name,
                CASE 
                    WHEN drivers.is_company_driver = 0 THEN "Owner Operator" 
                    WHEN drivers.is_company_driver = 1 THEN "Company Driver" 
                    WHEN drivers.is_company_driver = 2 THEN "Subcontractor" 
                    ELSE "" 
                END as driver_type,
                CASE drivers.suffix
                    WHEN "0" THEN "Mr."
                    WHEN "1" THEN "Mrs."
                    WHEN "2" THEN "Ms."
                    WHEN "3" THEN "Miss"
                    WHEN "4" THEN "Dr."
                    ELSE ""
                END as suffix_name,
                CASE drivers.is_1099
                    WHEN "0" THEN "No"
                    WHEN "1" THEN "Yes"
                    ELSE ""
                END as is_1099_name,
                CASE drivers.eobr_provider
                    WHEN "0" THEN "Samsara"
                    WHEN "1" THEN "GeoTab"
                    WHEN "2" THEN "Omnitracs"
                    ELSE ""
                END as eobr_provider_name,
                CASE drivers.twic_certified
                    WHEN "0" THEN "No"
                    WHEN "1" THEN "Yes"
                    ELSE ""
                END as twic_certified_name,
                CASE drivers.tsa_certified
                    WHEN "0" THEN "No"
                    WHEN "1" THEN "Yes"
                    ELSE ""
                END as tsa_certified_name,
                CASE drivers.tanker_endorsement
                    WHEN "0" THEN "No"
                    WHEN "1" THEN "Yes"
                    ELSE ""
                END as tanker_endorsement_name,
                CASE drivers.license_class
                    WHEN "0" THEN "Class A"
                    WHEN "1" THEN "Class B"
                    WHEN "2" THEN "Class C"
                    WHEN "3" THEN "Class D"
                    WHEN "4" THEN "Class M"
                    ELSE ""
                END as license_class_name,
                CASE drivers.fuel_card_type
                    WHEN "0" THEN "WEX"
                    WHEN "1" THEN "EFS"
                    WHEN "2" THEN "Comdata"
                    ELSE ""
                END as fuel_card_type_name,
                CASE drivers.smoker
                    WHEN "0" THEN "No"
                    WHEN "1" THEN "Yes"
                    ELSE ""
                END as smoker_name,
                CASE drivers.gender
                    WHEN "0" THEN "Male"
                    WHEN "1" THEN "Female"
                    ELSE ""
                END as gender_name,
                driver_informations.truck_number as truck_number,
                driver_informations.trailer_number as trailer_number,
                trailer_types.name as trailer_type_name,
                carriers.name as carrier_name,
                terminals.name as terminal_name,
                teams.name as team_name,
                CONCAT(dc.first_name, " ", dc.last_name) as manager_name,
                CONCAT(lp.first_name, " ", lp.last_name) as load_planner_name,
                CONCAT(rc.first_name, " ", rc.last_name) as recruiter_name,
                company_recruiting_sources.name as recruiting_source_name
            ')
            ->leftJoin('driver_informations as driver_informations', function($join) {
                $join->on('drivers.id', '=', 'driver_informations.driver_id');
            })
            ->leftJoin('trailers as trailers', function($join) use ($getCompanyAdmin) {
                $join->on('driver_informations.trailer_number', '=', 'trailers.trailer_id')
                    ->where('trailers.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('trailer_types as trailer_types', function($join) use ($getCompanyAdmin) {
                $join->on('trailers.type', '=', 'trailer_types.id')
                    ->where('trailer_types.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('carriers as carriers', function($join) use ($getCompanyAdmin) {
                $join->on('driver_informations.carrier_id', '=', 'carriers.id')
                    ->where('carriers.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('terminals as terminals', function($join) use ($getCompanyAdmin) {
                $join->on('driver_informations.terminal_id', '=', 'terminals.id')
                    ->where('terminals.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('teams as teams', function($join) use ($getCompanyAdmin) {
                $join->on('drivers.team_id', '=', 'teams.id')
                    ->where('teams.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('company_admins as dc', function($join) use ($getCompanyAdmin) {
                $join->on('drivers.manager_id', '=', 'dc.id')
                    ->where('dc.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('company_admins as lp', function($join) use ($getCompanyAdmin) {
                $join->on('drivers.load_planner', '=', 'lp.id')
                    ->where('lp.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('company_admins as rc', function($join) use ($getCompanyAdmin) {
                $join->on('drivers.recruiter', '=', 'rc.id')
                    ->where('rc.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('company_recruiting_sources as company_recruiting_sources', function($join) use ($getCompanyAdmin) {
                $join->on('drivers.recruiter_source', '=', 'company_recruiting_sources.id')
                    ->where('company_recruiting_sources.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->where('drivers.company_id', $getCompanyAdmin->company_id);
            if ($request->has('extra_is_company_driver') && $request->filled('extra_is_company_driver')) {
                $data->where('drivers.is_company_driver', $request->extra_is_company_driver);
            }
            if ($request->has('cdstatus') && $request->filled('cdstatus')) {
                if($request->cdstatus == '0'){
                    // Active
                    $data->where(function($q) {
                        $q->whereNull('drivers.status')->orWhere('drivers.status', '0');
                    })
                    ->whereNull('drivers.deleted_at');
                }elseif ($request->cdstatus == '1') {
                    // Out Of Service
                    $data->where(function($q) {
                        $q->where('drivers.status', '1')->whereNull('drivers.deleted_at');
                    });
                }elseif ($request->cdstatus == '2') {
                    // Terminate
                    $data->where(function($q) {
                        $q->where('drivers.status', '2')->orWhereNotNull('drivers.deleted_at');
                    });
                }elseif ($request->cdstatus == '3') {
                    // Rehire
                    $data->where(function($q) {
                        $q->where('drivers.status', '3')->whereNull('drivers.deleted_at');
                    });
                }
            }
            $data->withTrashed();

        // General filter code - start
        foreach( $getField as $key => $value ) {
            $conditionReq           = 'search_string_condition_'.$key;
            $sortReq                = 'search_string_sort_'.$key;
            $conditionDateReq       = 'search_date_condition_'.$key;
            $sortDateReq            = 'search_date_sort_'.$key;
            $conditionStatusReq     = 'search_status_condition_'.$key;
            $sortStatusReq          = 'search_status_sort_'.$key;

            // String filter
            if( isset( request()->$conditionReq ) ) {
                $typeReq            = 'search_string_type_'.$key;
                $type               = request()->$typeReq;

                if( $type == 'string' ) {
                    Common::getStringFilterResult( $data, $key );
                }
            }

            // Status filter
            if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                Common::getStatusFilterResult( $data, $key );
            }

            // Date filter
            if( isset( request()->$conditionDateReq ) ) {
                $typeReq            = 'search_date_type_'.$key;
                $type               = request()->$typeReq;

                if( $type == 'date' ) {
                    Common::getDateFilterResult( $data, $key );
                }
            }

            // String sort only
            if( isset( request()->$sortReq ) ) {
                Common::sortStringFilter( $data, $key );
            }

            // Date sort only
            if( isset( request()->$sortDateReq ) ) {
                Common::sortDateFilter( $data, $key );
            }

            // Status sort only
            if( isset( request()->$sortStatusReq ) ) {
                Common::sortStatusFilter( $data, $key );
            }
        }
        $data->get();
        // General filter code - end

        return Datatables::of($data)
            ->addIndexColumn()
            ->editColumn('status_name', function ($row) {
                $status_name            = '';
                if(isset($row->status_name)){
                    if($row->status_name == 'Active'){
                        $status_name    = '<div class="d-flex align-items-center"> 
                            <i class="fa fa-check-circle text-success fs-16 mr-2" aria-hidden="true"></i> Active
                        </div>';
                    } else if ($row->status_name == 'Out Of Service') {
                        $status_name    = '<div class="d-flex align-items-center"> 
                            <i class="fa fa-times-circle text-danger fs-16 mr-2" aria-hidden="true"></i> Out Of Service
                        </div>';
                    } else if ($row->status_name == 'Terminate') {
                        $status_name    = '<div class="d-flex align-items-center"> 
                            <i class="fa fa-times-circle text-danger fs-16 mr-2" aria-hidden="true"></i> Terminate
                        </div>';
                    } else if ($row->status_name == 'Rehire') {
                        $status_name    = '<div class="d-flex align-items-center"> 
                            <i class="fa fa-exclamation-triangle text-warning fs-16 mr-2" aria-hidden="true"></i> Rehire
                        </div>';
                    }
                }
                return $status_name;
            })
             ->filterColumn('status_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE
                    WHEN drivers.status IS NULL THEN "Active" 
                    WHEN drivers.status = "0" THEN "Active"
                    WHEN drivers.status = "1" THEN "Out Of Service"
                    WHEN drivers.status = "2" THEN "Terminate"
                    WHEN drivers.status = "3" THEN "Rehire"
                    ELSE "" 
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('driver_type', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN drivers.is_company_driver = 0 THEN "Owner Operator" 
                    WHEN drivers.is_company_driver = 1 THEN "Company Driver" 
                    WHEN drivers.is_company_driver = 2 THEN "Subcontractor" 
                    ELSE "" 
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('suffix_name', function ($query, $keyword) use ($search) {
               $query->whereRaw('CASE 
                   WHEN drivers.suffix = "0" THEN "Mr."
                   WHEN drivers.suffix = "1" THEN "Mrs."
                   WHEN drivers.suffix = "2" THEN "Ms."
                   WHEN drivers.suffix = "3" THEN "Miss"
                   WHEN drivers.suffix = "4" THEN "Dr."
                   ELSE "" 
               END LIKE ?', ['%' . $search . '%']);
           })
            ->editColumn('cell_phone', function ($row) {
                if(isset($row->cell_phone) && $row->cell_phone != ''){
                    return '<a href="tel:'.$row->cell_phone.'" >'.$row->cell_phone.'</a>';
                }else{
                    return '';
                }
            })
            ->editColumn('home_phone', function ($row) {
                if(isset($row->home_phone) && $row->home_phone != ''){
                    return '<a href="tel:'.$row->home_phone.'" >'.$row->home_phone.'</a>';
                }else{
                    return '';
                }
            })
            ->editColumn('email', function ($row) {
                if(isset($row->email) && $row->email !=''){
                    return '<a href="mailto:'.$row->email.'" class="text-lowercase">'.$row->email.'</a>';
                }else{
                    return '';
                }
            })
            ->filterColumn('birthday', function ($query, $keyword) use ($search) {
                $date_format    = config('settings.database_date_format', '%m/%d/%Y');
                $query->whereRaw('DATE_FORMAT(drivers.birthday, "' . $date_format . '") LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('is_1099_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN drivers.is_1099 = "0" THEN "No"
                    WHEN drivers.is_1099 = "1" THEN "Yes"
                    ELSE "" 
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('truck_number', function ($query, $keyword) use ($search) {
                $query->where('driver_informations.truck_number', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('trailer_number', function ($query, $keyword) use ($search) {
                $query->where('driver_informations.trailer_number', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('trailer_type_name', function ($query, $keyword) use ($search) {
                $query->where('trailer_types.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('carrier_name', function ($query, $keyword) use ($search) {
                $query->where('carriers.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('terminal_name', function ($query, $keyword) use ($search) {
                $query->where('terminals.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('team_name', function ($query, $keyword) use ($search) {
                $query->where('teams.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('manager_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CONCAT(dc.first_name, " ", dc.last_name) LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('load_planner_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CONCAT(lp.first_name, " ", lp.last_name) LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('recruiter_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CONCAT(rc.first_name, " ", rc.last_name) LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('recruiting_source_name', function ($query, $keyword) use ($search) {
                $query->where('company_recruiting_sources.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('eobr_provider_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN drivers.eobr_provider = "0" THEN "Samsara"
                    WHEN drivers.eobr_provider = "1" THEN "GeoTab"
                    WHEN drivers.eobr_provider = "2" THEN "Omnitracs"
                    ELSE "" 
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('twic_certified_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN drivers.twic_certified = "0" THEN "No"
                    WHEN drivers.twic_certified = "1" THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('tsa_certified_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN drivers.tsa_certified = "0" THEN "No"
                    WHEN drivers.tsa_certified = "1" THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('tanker_endorsement_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN drivers.tanker_endorsement = "0" THEN "No"
                    WHEN drivers.tanker_endorsement = "1" THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('passport_expiration', function ($query, $keyword) use ($search) {
                $date_format    = config('settings.database_date_format', '%m/%d/%Y');
                $query->whereRaw('DATE_FORMAT(drivers.passport_expiration, "' . $date_format . '") LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('medical_cert', function ($query, $keyword) use ($search) {
                $date_format    = config('settings.database_date_format', '%m/%d/%Y');
                $query->whereRaw('DATE_FORMAT(drivers.medical_cert, "' . $date_format . '") LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('license_class_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN drivers.license_class = "0" THEN "Class A"
                    WHEN drivers.license_class = "1" THEN "Class B"
                    WHEN drivers.license_class = "2" THEN "Class C"
                    WHEN drivers.license_class = "3" THEN "Class D"
                    WHEN drivers.license_class = "4" THEN "Class M"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('license_exp', function ($query, $keyword) use ($search) {
                $date_format    = config('settings.database_date_format', '%m/%d/%Y');
                $query->whereRaw('DATE_FORMAT(drivers.license_exp, "' . $date_format . '") LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('anniversary', function ($query, $keyword) use ($search) {
                $date_format    = config('settings.database_date_format', '%m/%d/%Y');
                $query->whereRaw('DATE_FORMAT(drivers.anniversary, "' . $date_format . '") LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('mvr_date', function ($query, $keyword) use ($search) {
                $date_format    = config('settings.database_date_format', '%m/%d/%Y');
                $query->whereRaw('DATE_FORMAT(drivers.mvr_date, "' . $date_format . '") LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('next_review', function ($query, $keyword) use ($search) {
                $date_format    = config('settings.database_date_format', '%m/%d/%Y');
                $query->whereRaw('DATE_FORMAT(drivers.next_review, "' . $date_format . '") LIKE ?', ['%' . $search . '%']);
            })
            ->editColumn('termination_date', function($row){
                return !is_null($row->termination_date) ?  Carbon::parse($row->getRawOriginal('termination_date'))->format(config('settings.date_format','m/d/Y')) : '';
            })
            ->filterColumn('termination_date', function ($query, $keyword) use ($search) {
                $date_format    = config('settings.database_date_format', '%m/%d/%Y');
                $query->whereRaw('DATE_FORMAT(drivers.termination_date, "' . $date_format . '") LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('fuel_card_type_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN drivers.fuel_card_type = "0" THEN "WEX"
                    WHEN drivers.fuel_card_type = "1" THEN "EFS"
                    WHEN drivers.fuel_card_type = "2" THEN "Comdata"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('smoker_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN drivers.smoker = "0" THEN "No"
                    WHEN drivers.smoker = "1" THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('gender_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN drivers.gender = "0" THEN "Male"
                    WHEN drivers.gender = "1" THEN "Female"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->addColumn('action', function($row){
                $action         = '';
                if($row->status == '2'){
                    if(hasPermission(config('settings.company_admin_modules.drivers_terminated'),config('settings.permissions.view'))){
                        $action .= '<a href="'.route('company.admin.drivers.show', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                    }
                    if(hasPermission(config('settings.company_admin_modules.drivers_terminated'),config('settings.permissions.edit'))){
                        $action .= '<a href="'.route('company.admin.drivers.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                    }
                }else{
                    if(hasPermission(config('settings.company_admin_modules.drivers'),config('settings.permissions.view'))){
                        $action .= '<a href="'.route('company.admin.drivers.show', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                    }
                    if(hasPermission(config('settings.company_admin_modules.drivers'),config('settings.permissions.edit'))){
                        $action .= '<a href="'.route('company.admin.drivers.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                    }
                }

                if(auth()->guard('admin')->check()){
                    $action     .= '<a href="'.route('company.admin.drivers.login', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-random" aria-hidden="true"></i></a>';
                }
                return $action;
            })
            ->rawColumns(['action','status_name','email','cell_phone','home_phone'])
            ->make(true);
    }

    public function trucksAll(Request $request){
        $getCompanyAdmin            = auth()->guard('company-admin')->user();

        $getField                   = '';
        $isCheck                    = Bookmark::where('company_id', $getCompanyAdmin->company_id)
            ->where('user_id', $getCompanyAdmin->id)
            ->where('user_type', $getCompanyAdmin->type)
            ->where('table_id','lists_trucks')
            ->first();
        if(!empty($isCheck->id)) {
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_lists_trucks_first'));
        } else {
            $getField               = config('settings.table_lists_trucks_first');
        }

        $search                     = $request->search['value'] ?? ''; // $request->input('search.value');

        $data                       = Truck::selectRaw('
                trucks.*,
                CASE
                    WHEN trucks.deleted_at IS NOT NULL THEN "Terminate" 
                    WHEN trucks.status IS NULL THEN "Active"
                    WHEN trucks.status = "0" THEN "Active"
                    WHEN trucks.status = "1" THEN "Out Of Service"
                    WHEN trucks.status = "2" THEN "Terminate"
                    WHEN trucks.status = "3" THEN "Rehire"
                    ELSE ""
                END as status_name,
                IF(trucks.driver IS NOT NULL AND trucks.driver != "", "Assigned", "Available") as assign_status,
                CASE trucks.ownership
                    WHEN "0" THEN "Owned By Carrier"
                    WHEN "1" THEN "Leased By Carrier"
                    WHEN "2" THEN "Rented By Carrier"
                    WHEN "3" THEN "Owned By Other"
                    WHEN "4" THEN "Leased By Other"
                    WHEN "5" THEN "Rented By Other"
                    ELSE ""
                END as ownership_name,
                CASE trucks.ownership
                    WHEN "0" THEN ownership_carriers.name
                    WHEN "3" THEN CONCAT(ownership_drivers.last_name, ", ", ownership_drivers.first_name)
                    ELSE ""
                END as company_owner_name,
                CASE trucks.ownership
                    WHEN "1" THEN ownership_vendors.name
                    WHEN "4" THEN ownership_vendors.name
                    ELSE ""
                END as company_lease_name,
                CASE trucks.required_license
                    WHEN "CDL_CA" THEN "CDL Class A"
                    WHEN "CDL_CB" THEN "CDL Class B"
                    WHEN "CDL_CC" THEN "CDL Class C"
                    WHEN "CDL_CD" THEN "CDL Class D"
                    WHEN "CDL_CM" THEN "CDL Class M"
                    ELSE ""
                END as license_class_name,
                CASE trucks.carb_compliant
                    WHEN "0" THEN "No"
                    WHEN "1" THEN "Yes"
                    ELSE ""
                END as carb_compliant_name,
                (
                    SELECT 
                        CASE 
                            WHEN truck_integrations.provider = "geo-tab" THEN "GeoTab"
                            WHEN truck_integrations.provider = "omni-tracs" THEN "OmniTracs"
                            WHEN truck_integrations.provider = "samsara" THEN "Samsara"
                            WHEN truck_integrations.provider = "trans-flo" THEN "TransFlo"
                            ELSE ""
                        END
                    FROM truck_integrations 
                    WHERE truck_integrations.truck_id = trucks.id AND truck_integrations.integration_type = "eld-provider" LIMIT 1
                ) AS eld_provider,
                (CASE WHEN EXISTS (SELECT 1 FROM galleries WHERE galleries.action_id = trucks.id AND galleries.source = "truck" AND galleries.field_name = "tag_photo" AND galleries.image IS NOT NULL AND galleries.is_dummy = 0) THEN "Yes" ELSE "No" END) AS tag_photo_exists,
                CASE trucks.tag
                    WHEN "1" THEN "Interstate"
                    WHEN "2" THEN "Intrastate"
                    ELSE ""
                END as tag_type_name,
                CASE trucks.fuel_type
                    WHEN "0" THEN "Desiel"
                    WHEN "1" THEN "Regular Gas"
                    WHEN "2" THEN "Electric"
                    ELSE ""
                END as fuel_type_name,
                CASE trucks.automatic_transmission
                    WHEN "0" THEN "No"
                    WHEN "1" THEN "Yes"
                    ELSE ""
                END as automatic_transmission_name,
                CASE trucks.high_cube
                    WHEN "0" THEN "No"
                    WHEN "1" THEN "Yes"
                    ELSE ""
                END as high_cube_name,
                CASE trucks.vented
                    WHEN 0 THEN "No"
                    WHEN 1 THEN "Yes"
                    ELSE ""
                END as vented_name,
                CASE trucks.liftgate
                    WHEN "0" THEN "No"
                    WHEN "1" THEN "Yes"
                    ELSE ""
                END as liftgate_name,
                CASE trucks.pallet_jack
                    WHEN "0" THEN "No"
                    WHEN "1" THEN "Yes"
                    ELSE ""
                END as pallet_jack_name,
                CASE trucks.reefer_equipped
                    WHEN "0" THEN "No"
                    WHEN "1" THEN "Yes"
                    ELSE ""
                END as reefer_equipped_name,
                CASE trucks.apu_equipped
                    WHEN "0" THEN "No"
                    WHEN "1" THEN "Yes"
                    ELSE ""
                END as apu_equipped_name,
                states.name as tag_state_name,
                CONCAT(md.first_name, " ", md.last_name) as driver_name,
                CONCAT(cd.first_name, " ", cd.last_name) as co_driver_name,
                company_truck_types.name as truck_type_name,
                carriers.name as carrier_name,
                terminals.name as terminal_name,
                teams.name as team_name,
                CONCAT(dc.first_name, " ", dc.last_name) as manager_name,
                company_truck_groups.name as truck_group_name
            ')
            ->leftJoin('states as states', function($join) use ($getCompanyAdmin) {
                $join->on('trucks.tag_state', '=', 'states.id');
            })
            ->leftJoin('drivers as md', function($join) use ($getCompanyAdmin) {
                $join->on('trucks.driver', '=', 'md.id')
                    ->where('md.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('teams as teams', function($join) use ($getCompanyAdmin) {
                $join->on('md.team_id', '=', 'teams.id')
                    ->where('teams.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('company_admins as dc', function($join) use ($getCompanyAdmin) {
                $join->on('md.manager_id', '=', 'dc.id')
                    ->where('dc.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('drivers as cd', function($join) use ($getCompanyAdmin) {
                $join->on('trucks.co_driver', '=', 'cd.id')
                    ->where('cd.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('company_truck_types as company_truck_types', function($join) use ($getCompanyAdmin) {
                $join->on('trucks.truck_type', '=', 'company_truck_types.id')
                    ->where('company_truck_types.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('carriers as carriers', function($join) use ($getCompanyAdmin) {
                $join->on('trucks.carrier_id', '=', 'carriers.id')
                    ->where('carriers.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('terminals as terminals', function($join) use ($getCompanyAdmin) {
                $join->on('trucks.terminal_id', '=', 'terminals.id')
                    ->where('terminals.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('carriers as ownership_carriers', function($join) use ($getCompanyAdmin) {
                $join->on('trucks.ownership_id', '=', 'ownership_carriers.id')
                    ->where('ownership_carriers.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('drivers as ownership_drivers', function($join) use ($getCompanyAdmin) {
                $join->on('trucks.ownership_id', '=', 'ownership_drivers.id')
                    ->where('ownership_drivers.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('vendors as ownership_vendors', function($join) use ($getCompanyAdmin) {
                $join->on('trucks.leasing_company', '=', 'ownership_vendors.id')
                    ->where('ownership_vendors.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('company_truck_groups as company_truck_groups', function($join) use ($getCompanyAdmin) {
                $join->on('trucks.truck_group', '=', 'company_truck_groups.id')
                    ->where('company_truck_groups.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->where('trucks.company_id', $getCompanyAdmin->company_id);
            if ($request->has('cdstatus') && $request->filled('cdstatus')) {
                if($request->cdstatus == '0'){
                    // Active
                    $data->where(function($q) {
                        $q->whereNull('trucks.status')->orWhere('trucks.status', '0');
                    })
                    ->whereNull('trucks.deleted_at');
                }elseif ($request->cdstatus == '1') {
                    // Out Of Service
                    $data->where(function($q) {
                        $q->where('trucks.status', '1')->whereNull('trucks.deleted_at');
                    });
                }elseif ($request->cdstatus == '2') {
                    // Terminate
                    $data->where(function($q) {
                        $q->where('trucks.status', '2')->orWhereNotNull('trucks.deleted_at');
                    });
                }elseif ($request->cdstatus == '3') {
                    // Rehire
                    $data->where(function($q) {
                        $q->where('trucks.status', '3')->whereNull('trucks.deleted_at');
                    });
                }
            }
            $data->withTrashed();

        // General filter code - start
        foreach( $getField as $key => $value ) {
            $conditionReq           = 'search_string_condition_'.$key;
            $sortReq                = 'search_string_sort_'.$key;
            $conditionDateReq       = 'search_date_condition_'.$key;
            $sortDateReq            = 'search_date_sort_'.$key;
            $conditionStatusReq     = 'search_status_condition_'.$key;
            $sortStatusReq          = 'search_status_sort_'.$key;

            // String filter
            if( isset( request()->$conditionReq ) ) {
                $typeReq            = 'search_string_type_'.$key;
                $type               = request()->$typeReq;

                if( $type == 'string' ) {
                    Common::getStringFilterResult( $data, $key );
                }
            }

            // Status filter
            if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                Common::getStatusFilterResult( $data, $key );
            }

            // Date filter
            if( isset( request()->$conditionDateReq ) ) {
                $typeReq            = 'search_date_type_'.$key;
                $type               = request()->$typeReq;

                if( $type == 'date' ) {
                    Common::getDateFilterResult( $data, $key );
                }
            }

            // String sort only
            if( isset( request()->$sortReq ) ) {
                Common::sortStringFilter( $data, $key );
            }

            // Date sort only
            if( isset( request()->$sortDateReq ) ) {
                Common::sortDateFilter( $data, $key );
            }

            // Status sort only
            if( isset( request()->$sortStatusReq ) ) {
                Common::sortStatusFilter( $data, $key );
            }
        }
        $data->get();
        // General filter code - end

        return Datatables::of($data)
            ->addIndexColumn()
            ->editColumn('status_name', function ($row) {
                $status_name            = '';
                if(isset($row->status_name)){
                    if($row->status_name == 'Active'){
                        $status_name    = '<div class="d-flex align-items-center"> 
                            <i class="fa fa-check-circle text-success fs-16 mr-2" aria-hidden="true"></i> Active
                        </div>';
                    } else if ($row->status_name == 'Out Of Service') {
                        $status_name    = '<div class="d-flex align-items-center"> 
                            <i class="fa fa-times-circle text-danger fs-16 mr-2" aria-hidden="true"></i> Out Of Service
                        </div>';
                    } else if ($row->status_name == 'Terminate') {
                        $status_name    = '<div class="d-flex align-items-center"> 
                            <i class="fa fa-times-circle text-danger fs-16 mr-2" aria-hidden="true"></i> Terminate
                        </div>';
                    } else if ($row->status_name == 'Rehire') {
                        $status_name    = '<div class="d-flex align-items-center"> 
                            <i class="fa fa-exclamation-triangle text-warning fs-16 mr-2" aria-hidden="true"></i> Rehire
                        </div>';
                    }
                }
                return $status_name;
            })
            ->filterColumn('status_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE
                    WHEN trucks.deleted_at IS NOT NULL THEN "Terminate" 
                    WHEN trucks.status IS NULL THEN "Active" 
                    WHEN trucks.status = "0" THEN "Active"
                    WHEN trucks.status = "1" THEN "Out Of Service"
                    WHEN trucks.status = "2" THEN "Terminate"
                    WHEN trucks.status = "3" THEN "Rehire"
                    ELSE "" 
                END LIKE ?', ['%' . $search . '%']);
            })
            ->editColumn('assign_status', function ($row) {
                $assign_status          = '';
                if(isset($row->assign_status)){
                    if($row->assign_status == 'Assigned'){
                        $assign_status  = '<span>Assigned</span>';
                    } else if ($row->assign_status == 'Available') {
                        $assign_status  = '<span class="text-success">Available</span>';
                    }
                }
                return $assign_status;
            })
            ->filterColumn('assign_status', function ($query, $keyword) use ($search) {
                $query->whereRaw('IF(trucks.driver IS NOT NULL AND trucks.driver != "", "Assigned", "Available") LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('ownership_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trucks.ownership = "0" THEN "Owned By Carrier"
                    WHEN trucks.ownership = "1" THEN "Leased By Carrier"
                    WHEN trucks.ownership = "2" THEN "Rented By Carrier"
                    WHEN trucks.ownership = "3" THEN "Owned By Other"
                    WHEN trucks.ownership = "4" THEN "Leased By Other"
                    WHEN trucks.ownership = "5" THEN "Rented By Other"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('company_owner_name', function ($query, $keyword) use ($search) {
                $query->where(function($query) use ($search) {
                    $query->where('ownership_carriers.name', 'LIKE', '%' . $search . '%')
                        ->orWhereRaw('CONCAT(ownership_drivers.last_name, ", ", ownership_drivers.first_name) LIKE ?', ['%' . $search . '%']);
                });
            })
            ->filterColumn('company_lease_name', function ($query, $keyword) use ($search) {
                $query->where('ownership_vendors.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('license_class_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trucks.required_license = "CDL_CA" THEN "CDL Class A"
                    WHEN trucks.required_license = "CDL_CB" THEN "CDL Class B"
                    WHEN trucks.required_license = "CDL_CC" THEN "CDL Class C"
                    WHEN trucks.required_license = "CDL_CD" THEN "CDL Class D"
                    WHEN trucks.required_license = "CDL_CM" THEN "CDL Class M"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('carb_compliant_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trucks.carb_compliant = "0" THEN "No"
                    WHEN trucks.carb_compliant = "1" THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('tag_type_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trucks.tag = "1" THEN "Interstate"
                    WHEN trucks.tag = "2" THEN "Intrastate"
                    ELSE "" 
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('fuel_type_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trucks.fuel_type = "0" THEN "Desiel"
                    WHEN trucks.fuel_type = "1" THEN "Regular Gas"
                    WHEN trucks.fuel_type = "2" THEN "Electric"
                    ELSE "" 
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('automatic_transmission_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trucks.automatic_transmission = "0" THEN "No"
                    WHEN trucks.automatic_transmission = "1" THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('high_cube_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trucks.high_cube = "0" THEN "No"
                    WHEN trucks.high_cube = "1" THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('vented_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trucks.vented = 0 THEN "No"
                    WHEN trucks.vented = 1 THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('liftgate_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trucks.liftgate = "0" THEN "No"
                    WHEN trucks.liftgate = "1" THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('pallet_jack_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trucks.pallet_jack = "0" THEN "No"
                    WHEN trucks.pallet_jack = "1" THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('reefer_equipped_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trucks.reefer_equipped = "0" THEN "No"
                    WHEN trucks.reefer_equipped = "1" THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('apu_equipped_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trucks.apu_equipped = "0" THEN "No"
                    WHEN trucks.apu_equipped = "1" THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('tag_state_name', function ($query, $keyword) use ($search) {
                $query->where('states.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('tag_exp', function ($query, $keyword) use ($search) {
                $date_format    = config('settings.database_date_format', '%m/%d/%Y');
                $query->whereRaw('DATE_FORMAT(trucks.tag_exp, "' . $date_format . '") LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('driver_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CONCAT(md.first_name, " ", md.last_name) LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('co_driver_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CONCAT(cd.first_name, " ", cd.last_name) LIKE ?', ['%' . $search . '%']);
            })
            ->editColumn('trailer', function ($row) {
                $trailers       = isset($row->trailer) && $row->trailer != '' ? array_filter(explode(',', $row->trailer)) : [];
                if(!empty($trailers)){
                    if(count($trailers) > 1){
                        $html   = '<ul>';
                        foreach ($trailers as $key => $value) {
                            $html .= '<li>'.$value.'</li>';
                        }
                        $html  .= '</ul>';
                        return '<a href="javascript:void(0);" class="text-underline multiple-trailers">'.count($trailers).' Trailers</a>
                            <div class="trailer-tooltip d-none">'.$html.'</div>';
                    }else{
                        return $row->trailer;
                    }
                }
                return '';
            })
            ->filterColumn('trailer', function ($query, $keyword) use ($search) {
                $query->whereRaw('FIND_IN_SET(?, trucks.trailer)', [$search]);
            })
            ->filterColumn('truck_type_name', function ($query, $keyword) use ($search) {
                $query->where('company_truck_types.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('carrier_name', function ($query, $keyword) use ($search) {
                $query->where('carriers.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('terminal_name', function ($query, $keyword) use ($search) {
                $query->where('terminals.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('team_name', function ($query, $keyword) use ($search) {
                $query->where('teams.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('manager_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CONCAT(dc.first_name, " ", dc.last_name) LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('truck_group_name', function ($query, $keyword) use ($search) {
                $query->where('company_truck_groups.name', 'LIKE', '%' . $search . '%');
            })
            ->addColumn('action', function($row){
                $action     = '<a href="'.route('company.admin.trucks.show', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                $action    .= '<a href="'.route('company.admin.trucks.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                return $action;
            })
            ->rawColumns(['action','status_name','assign_status','trailer'])
            ->make(true);
    }

    public function trailersAll(Request $request){
        $getCompanyAdmin            = auth()->guard('company-admin')->user();

        $getField                   = '';
        $isCheck                    = Bookmark::where('company_id', $getCompanyAdmin->company_id)
            ->where('user_id', $getCompanyAdmin->id)
            ->where('user_type', $getCompanyAdmin->type)
            ->where('table_id','lists_trailers')
            ->first();
        if(!empty($isCheck->id)) {
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_lists_trailers_first'));
        } else {
            $getField               = config('settings.table_lists_trailers_first');
        }

        $search                     = $request->search['value'] ?? ''; // $request->input('search.value');

        $data                       = Trailer::selectRaw('
                trailers.*,
                CASE 
                    WHEN trailers.deleted_at IS NOT NULL THEN "Terminate"
                    WHEN trailers.status IS NULL THEN "Active"
                    WHEN trailers.status = 0 THEN "Active"
                    WHEN trailers.status = 1 THEN "Out Of Service"
                    WHEN trailers.status = 2 THEN "Terminate"
                    WHEN trailers.status = 3 THEN "Rehire"
                    ELSE ""
                END as status_name,
                IF(trailers.current_assigned_truck IS NOT NULL AND trailers.current_assigned_truck != "", "Assigned", "Available") as assign_status,
                CASE trailers.carb_compliant
                    WHEN 0 THEN "No"
                    WHEN 1 THEN "Yes"
                    ELSE ""
                END as carb_compliant_name,
                (CASE WHEN EXISTS (SELECT 1 FROM galleries WHERE galleries.action_id = trailers.id AND galleries.source = "trailers" AND galleries.field_name = "tag_upload_image" AND galleries.image IS NOT NULL AND galleries.is_dummy = 0) THEN "Yes" ELSE "No" END) AS tag_photo_exists,
                CASE trailers.plated_trailer
                    WHEN 0 THEN "No"
                    WHEN 1 THEN "Yes"
                    ELSE ""
                END as plated_trailer_name,
                CASE trailers.suspension_type
                    WHEN "1" THEN "Air Ride"
                    WHEN "2" THEN "Spring Ride"
                    ELSE ""
                END as suspension_type_name,
                CASE trailers.roof_type
                    WHEN "1" THEN "Metal"
                    WHEN "2" THEN "Translucent"
                    WHEN "3" THEN "Mesh"
                    WHEN "4" THEN "Open"
                    ELSE ""
                END as roof_type_name,
                CASE trailers.floor_type
                    WHEN "1" THEN "Wood"
                    WHEN "2" THEN "Aluminum"
                    WHEN "3" THEN "Wood w/Composite"
                    ELSE ""
                END as floor_type_name,
                CASE trailers.doors_type
                    WHEN 1 THEN "Roll Up"
                    WHEN 2 THEN "Swing"
                    ELSE ""
                END as doors_type_name,
                carriers.name as carrier_name,
                states.name as tag_state_name,
                terminals.name as terminal_name,
                trucks.unit as truck_unit,
                CONCAT(md.first_name, " ", md.last_name) as driver_name,
                teams.name as team_name,
                CONCAT(dc.first_name, " ", dc.last_name) as manager_name,
                trailer_types.name as trailer_type_name
            ')
            ->leftJoin('carriers as carriers', function($join) use ($getCompanyAdmin) {
                $join->on('trailers.carrier', '=', 'carriers.id')
                    ->where('carriers.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('states as states', function($join) use ($getCompanyAdmin) {
                $join->on('trailers.tag_state', '=', 'states.id');
            })
            ->leftJoin('terminals as terminals', function($join) use ($getCompanyAdmin) {
                $join->on('trailers.terminal', '=', 'terminals.id')
                    ->where('terminals.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('trucks as trucks', function($join) use ($getCompanyAdmin) {
                $join->on('trailers.current_assigned_truck', '=', 'trucks.id')
                    ->where('trucks.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('drivers as md', function($join) use ($getCompanyAdmin) {
                $join->on('trucks.driver', '=', 'md.id')
                    ->where('md.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('teams as teams', function($join) use ($getCompanyAdmin) {
                $join->on('md.team_id', '=', 'teams.id')
                    ->where('teams.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('company_admins as dc', function($join) use ($getCompanyAdmin) {
                $join->on('md.manager_id', '=', 'dc.id')
                    ->where('dc.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->leftJoin('trailer_types as trailer_types', function($join) use ($getCompanyAdmin) {
                $join->on('trailers.type', '=', 'trailer_types.id')
                    ->where('trailer_types.company_id', '=', $getCompanyAdmin->company_id);
            })
            ->where('trailers.company_id', $getCompanyAdmin->company_id);
            if ($request->has('cdstatus') && $request->filled('cdstatus')) {
                if($request->cdstatus == '0'){
                    // Active
                    $data->where(function($q) {
                        $q->whereNull('trailers.status')->orWhere('trailers.status', '0');
                    })
                    ->whereNull('trailers.deleted_at');
                }elseif ($request->cdstatus == '1') {
                    // Out Of Service
                    $data->where(function($q) {
                        $q->where('trailers.status', '1')->whereNull('trailers.deleted_at');
                    });
                }elseif ($request->cdstatus == '2') {
                    // Terminate
                    $data->where(function($q) {
                        $q->where('trailers.status', '2')->orWhereNotNull('trailers.deleted_at');
                    });
                }elseif ($request->cdstatus == '3') {
                    // Rehire
                    $data->where(function($q) {
                        $q->where('trailers.status', '3')->whereNull('trailers.deleted_at');
                    });
                }
            }
            $data->withTrashed();

        // General filter code - start
        foreach( $getField as $key => $value ) {
            $conditionReq           = 'search_string_condition_'.$key;
            $sortReq                = 'search_string_sort_'.$key;
            $conditionDateReq       = 'search_date_condition_'.$key;
            $sortDateReq            = 'search_date_sort_'.$key;
            $conditionStatusReq     = 'search_status_condition_'.$key;
            $sortStatusReq          = 'search_status_sort_'.$key;

            // String filter
            if( isset( request()->$conditionReq ) ) {
                $typeReq            = 'search_string_type_'.$key;
                $type               = request()->$typeReq;

                if( $type == 'string' ) {
                    Common::getStringFilterResult( $data, $key );
                }
            }

            // Status filter
            if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                Common::getStatusFilterResult( $data, $key );
            }

            // Date filter
            if( isset( request()->$conditionDateReq ) ) {
                $typeReq            = 'search_date_type_'.$key;
                $type               = request()->$typeReq;

                if( $type == 'date' ) {
                    Common::getDateFilterResult( $data, $key );
                }
            }

            // String sort only
            if( isset( request()->$sortReq ) ) {
                Common::sortStringFilter( $data, $key );
            }

            // Date sort only
            if( isset( request()->$sortDateReq ) ) {
                Common::sortDateFilter( $data, $key );
            }

            // Status sort only
            if( isset( request()->$sortStatusReq ) ) {
                Common::sortStatusFilter( $data, $key );
            }
        }
        $data->get();
        // General filter code - end

        return Datatables::of($data)
            ->addIndexColumn()
            ->editColumn('status_name', function ($row) {
                $status_name            = '';
                if(isset($row->status_name)){
                    if($row->status_name == 'Active'){
                        $status_name    = '<div class="d-flex align-items-center"> 
                            <i class="fa fa-check-circle text-success fs-16 mr-2" aria-hidden="true"></i> Active
                        </div>';
                    } else if ($row->status_name == 'Out Of Service') {
                        $status_name    = '<div class="d-flex align-items-center"> 
                            <i class="fa fa-times-circle text-danger fs-16 mr-2" aria-hidden="true"></i> Out Of Service
                        </div>';
                    } else if ($row->status_name == 'Terminate') {
                        $status_name    = '<div class="d-flex align-items-center"> 
                            <i class="fa fa-times-circle text-danger fs-16 mr-2" aria-hidden="true"></i> Terminate
                        </div>';
                    } else if ($row->status_name == 'Rehire') {
                        $status_name    = '<div class="d-flex align-items-center"> 
                            <i class="fa fa-exclamation-triangle text-warning fs-16 mr-2" aria-hidden="true"></i> Rehire
                        </div>';
                    }
                }
                return $status_name;
            })
            ->filterColumn('status_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE
                    WHEN trailers.deleted_at IS NOT NULL THEN "Terminate"
                    WHEN trailers.status IS NULL THEN "Active" 
                    WHEN trailers.status = 0 THEN "Active"
                    WHEN trailers.status = 1 THEN "Out Of Service"
                    WHEN trailers.status = 2 THEN "Terminate"
                    WHEN trailers.status = 3 THEN "Rehire"
                    ELSE "" 
                END LIKE ?', ['%' . $search . '%']);
            })
            ->editColumn('assign_status', function ($row) {
                $assign_status          = '';
                if(isset($row->assign_status)){
                    if($row->assign_status == 'Assigned'){
                        $assign_status  = '<span>Assigned</span>';
                    } else if ($row->assign_status == 'Available') {
                        $assign_status  = '<span class="text-success">Available</span>';
                    }
                }
                return $assign_status;
            })
            ->filterColumn('assign_status', function ($query, $keyword) use ($search) {
                $query->whereRaw('IF(trailers.current_assigned_truck IS NOT NULL AND trailers.current_assigned_truck != "", "Assigned", "Available") LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('carrier_name', function ($query, $keyword) use ($search) {
                $query->where('carriers.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('tag_state_name', function ($query, $keyword) use ($search) {
                $query->where('states.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('tag_expiration', function ($query, $keyword) use ($search) {
                $date_format    = config('settings.database_date_format', '%m/%d/%Y');
                $query->whereRaw('DATE_FORMAT(trailers.tag_expiration, "' . $date_format . '") LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('terminal_name', function ($query, $keyword) use ($search) {
                $query->where('terminals.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('truck_unit', function ($query, $keyword) use ($search) {
                $query->where('trucks.unit', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('driver_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CONCAT(md.first_name, " ", md.last_name) LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('team_name', function ($query, $keyword) use ($search) {
                $query->where('teams.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('manager_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CONCAT(dc.first_name, " ", dc.last_name) LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('carb_compliant_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trailers.carb_compliant = 0 THEN "No"
                    WHEN trailers.carb_compliant = 1 THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('trailer_type_name', function ($query, $keyword) use ($search) {
                $query->where('trailer_types.name', 'LIKE', '%' . $search . '%');
            })
            ->filterColumn('plated_trailer_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trailers.plated_trailer = 0 THEN "No"
                    WHEN trailers.plated_trailer = 1 THEN "Yes"
                    ELSE ""
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('suspension_type_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trailers.suspension_type = "1" THEN "Air Ride"
                    WHEN trailers.suspension_type = "2" THEN "Spring Ride"
                    ELSE "" 
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('roof_type_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trailers.roof_type = "1" THEN "Metal"
                    WHEN trailers.roof_type = "2" THEN "Translucent"
                    WHEN trailers.roof_type = "3" THEN "Mesh"
                    WHEN trailers.roof_type = "4" THEN "Open"
                    ELSE "" 
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('floor_type_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trailers.floor_type = "1" THEN "Wood"
                    WHEN trailers.floor_type = "2" THEN "Aluminum"
                    WHEN trailers.floor_type = "3" THEN "Wood w/Composite"
                    ELSE "" 
                END LIKE ?', ['%' . $search . '%']);
            })
            ->filterColumn('doors_type_name', function ($query, $keyword) use ($search) {
                $query->whereRaw('CASE 
                    WHEN trailers.doors_type = 1 THEN "Roll Up"
                    WHEN trailers.doors_type = 2 THEN "Swing"
                    ELSE "" 
                END LIKE ?', ['%' . $search . '%']);
            })
            ->addColumn('action', function($row){
                $action     = '<a href="'.route('company.admin.trailers.show', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                $action     .= '<a href="'.route('company.admin.trailers.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                return $action;
            })
            ->rawColumns(['action','status_name','assign_status'])
            ->make(true);
    }

    public function widgetsSave(Request $request){
        $response['success']                    = false;
        $response['message']                    = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $item                           = ListWidget::updateOrCreate([
                    'company_admins_id'         => auth()->guard('company-admin')->id(),
                    'tab'                       => $request->tab
                ], [
                    'company_admins_id'         => auth()->guard('company-admin')->id(),
                    'tab'                       => $request->tab,
                    'isActive'                  => $request->isActive
                ]);
                if($item){
                    $response['success']        = true;
                    $response['message']        = "Save successfully";
                }
            }catch (\Exception $e) {
                $response['success']            = false;
                $response['message']            = $e->getMessage();
            }
        }
        return response()->json($response);
    }
}
