<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;

use App\Models\Broker;
use App\Models\Bookmark;
use App\Models\CompanyLoadDocument;

use App\Helpers\Common;

use Validator;
use DataTables;
use Carbon\Carbon;

class LoadDocumentController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getField                   = '';
        $getCompanyAdmin            = auth()->guard('company-admin')->user();
        $isCheck                    = Bookmark::where('company_id',getUserCompnayId())
            ->where('user_id',$getCompanyAdmin->id)
            ->where('user_type',$getCompanyAdmin->type)
            ->where('table_id', 'load_document')
            ->first();

        if(!empty($isCheck->id)){
            //$getField               = unserialize($isCheck->config);
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_load_document_first'));
        }else{
            $getField               = config('settings.table_load_document_first');
        }

        if ($request->ajax()) {
            $data                   = CompanyLoadDocument::IsCompany();
            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq       = 'search_string_condition_'.$key;
                $sortReq            = 'search_string_sort_'.$key;

                if( isset( request()->$conditionReq ) ) {
                    $typeReq        = 'search_string_type_'.$key;
                    $type           = request()->$typeReq;
                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }
                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                if( isset( request()->$sortReq ) ) {
                    $nameReq        = 'search_string_name_'.$key;
                    $sortReq        = 'search_string_sort_'.$key;
                    $typeReq        = 'search_string_type_'.$key;
                    $tableNameReq   = 'search_table_name_'.$key;
                    $colAliasReq    = 'search_string_col_alias_'.$key;

                    $origData       = request()->$nameReq;
                    $condition_sort = request()->$sortReq;
                    $type           = request()->$typeReq;
                    $tableName      = isset( request()->$tableNameReq ) ? request()->$tableNameReq : '';
                    $colAlias       = isset( request()->$colAliasReq ) ? request()->$colAliasReq : '';

                    if( $type == 'string' ) {
                        if( isset( $condition_sort ) ) {
                            if( $colAlias != '' ) {
                                $data->orderBy( $tableName.'.'.$colAlias, $condition_sort );
                            } else {
                                if( $tableName != '' ) {
                                    $data->orderBy( $tableName.'.'.$origData, $condition_sort );
                                } else {
                                    $data->orderBy( $origData, $condition_sort );
                                }
                            }
                        }
                    }
                }
            }
            $data->get();
            // General filter code - end

            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $action = '';
                    if(hasPermission(config('settings.company_admin_modules.load_document'),config('settings.permissions.edit'))){
                        $action .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-action edit-btn" data-id="'.$row->id.'" data-toggle="modal" data-target="#editModal"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                    }
                    if(hasPermission(config('settings.company_admin_modules.load_document'),config('settings.permissions.delete'))){
                        $action .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash destroy" data-action="'.route('company.admin.load-document.destroy', $row->id).'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';
                    }

                    return $action;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        $tableName                  = 'settings.table_load_document';
        return view('companyAdmin.loadDocument.index', compact('getField','tableName'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request){
        $input                      = $request->all();
        $validator                  = Validator::make($input,[
            "name"                  => "required",
        ]);
        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }
        $input['company_id']        = auth()->guard('company-admin')->user()->company_id;
        $input['include_w_billing'] = $request->include_w_billing ?? 'no';
        $input['fixed']             = $request->fixed ?? 'no';
        $input['for_whom']          = $request->for_whom ?? 'both';
        CompanyLoadDocument::create($input);
        notification('success', $this->crudMessage('create', 'Load document'));
        return response()->json(['success' => "Load document created successfully"]);
    }

    public function editFormView(Request $request){
        $load_document              = CompanyLoadDocument::find($request->id);
        $view                       = view('companyAdmin.loadDocument.editFormView', compact('load_document'))->render();
        return response()->json(['view' => $view, 'data' => $load_document]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, CompanyLoadDocument $load_document){
        $input                      = $request->all();
        $validator                  = Validator::make($input,[
            "name"                  => "required",
        ]);
        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }
        $input['include_w_billing'] = $request->include_w_billing ?? 'no';
        $input['fixed']             = $request->fixed ?? 'no';
        $input['for_whom']          = $request->for_whom ?? 'both';
        $load_document->update($input);

        // Broker Document Update
        $brokers                                = Broker::select('id','company_id', 'shipper_files', 'consignee_files')
            ->where('company_id', $load_document->company_id)
            ->where(function ($query) {
                $query->whereNotNull('shipper_files')
                    ->orWhereNotNull('consignee_files');
            })
            ->where(function ($query) use ($load_document) {
                $query->whereRaw("shipper_files LIKE '%\"name_id\";s:%" . strlen($load_document->id) . ":\"$load_document->id\"%'")
                      ->orWhereRaw("consignee_files LIKE '%\"name_id\";s:%" . strlen($load_document->id) . ":\"$load_document->id\"%'");
            })
            ->get();
        if($brokers->isNotEmpty()){
            foreach ($brokers as $key => $broker) {
                // Shipper
                $broker_shipper_files           = [];
                if(isset($broker->shipper_files) && is_serialized($broker->shipper_files) && !empty(unserialize($broker->shipper_files))){
                    $broker_shipper_files       = unserialize($broker->shipper_files);
                }

                if(!empty($broker_shipper_files)){
                    $searchkey                  = array_search($load_document->id, array_column($broker_shipper_files, 'name_id'));
                    if($searchkey!==false){
                        $broker_shipper_array   = $broker_shipper_files[$searchkey];
                        if(in_array($load_document->for_whom, ['shipper', 'both'])){
                            if(isset($broker_shipper_array['name']) && ($broker_shipper_array['name'] !== $load_document->name)){
                                if (isset($broker_shipper_files[$searchkey]['name'])) {
                                    $broker_shipper_files[$searchkey]['name']                   = $load_document->name;
                                }
                            }
                            if(isset($broker_shipper_array['fixed']) && ($broker_shipper_array['fixed'] !== $load_document->fixed)){
                                if (isset($broker_shipper_files[$searchkey]['fixed'])) {
                                    $broker_shipper_files[$searchkey]['fixed']                  = $load_document->fixed;
                                }
                            }
                            if(isset($broker_shipper_array['include_w_billing']) && ($broker_shipper_array['include_w_billing'] !== $load_document->include_w_billing)){
                                if (isset($broker_shipper_files[$searchkey]['include_w_billing'])) {
                                    $broker_shipper_files[$searchkey]['include_w_billing']      = $load_document->include_w_billing;
                                }
                            }
                        }else{
                            unset($broker_shipper_files[$searchkey]);
                        }
                    }
                }

                $broker_shipper_files           = array_values($broker_shipper_files);
                if(!empty($broker_shipper_files)){
                    $broker->shipper_files      = serialize($broker_shipper_files);
                }else{
                    $broker->shipper_files      = null;
                }
                // Shipper

                // Consignee
                $broker_consignee_files         = [];
                if(isset($broker->consignee_files) && is_serialized($broker->consignee_files) && !empty(unserialize($broker->consignee_files))){
                    $broker_consignee_files     = unserialize($broker->consignee_files);
                }

                if(!empty($broker_consignee_files)){
                    $searchkey                  = array_search($load_document->id, array_column($broker_consignee_files, 'name_id'));
                    if($searchkey!==false){
                        $broker_consignee_array = $broker_consignee_files[$searchkey];
                        if(in_array($load_document->for_whom, ['consignee', 'both'])){
                            if(isset($broker_consignee_array['name']) && ($broker_consignee_array['name'] !== $load_document->name)){
                                if (isset($broker_consignee_files[$searchkey]['name'])) {
                                    $broker_consignee_files[$searchkey]['name']                 = $load_document->name;
                                }
                            }
                            if(isset($broker_consignee_array['fixed']) && ($broker_consignee_array['fixed'] !== $load_document->fixed)){
                                if (isset($broker_consignee_files[$searchkey]['fixed'])) {
                                    $broker_consignee_files[$searchkey]['fixed']                = $load_document->fixed;
                                }
                            }
                            if(isset($broker_consignee_array['include_w_billing']) && ($broker_consignee_array['include_w_billing'] !== $load_document->include_w_billing)){
                                if (isset($broker_consignee_files[$searchkey]['include_w_billing'])) {
                                    $broker_consignee_files[$searchkey]['include_w_billing']    = $load_document->include_w_billing;
                                }
                            }
                        }else{
                            unset($broker_consignee_files[$searchkey]);
                        }
                    }
                }

                $broker_consignee_files         = array_values($broker_consignee_files);
                if(!empty($broker_consignee_files)){
                    $broker->consignee_files    = serialize($broker_consignee_files);
                }else{
                    $broker->consignee_files    = null;
                }
                // Consignee

                $broker->save();
            }
        }
        // Broker Document Update

        notification('success', $this->crudMessage('update', 'Load document'));
        return response()->json(['success' => "Load document updated successfully"]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(CompanyLoadDocument $load_document){
        // Broker Document Update
        $brokers                                = Broker::select('id','company_id', 'shipper_files', 'consignee_files')
            ->where('company_id', $load_document->company_id)
            ->where(function ($query) {
                $query->whereNotNull('shipper_files')
                    ->orWhereNotNull('consignee_files');
            })
            ->where(function ($query) use ($load_document) {
                $query->whereRaw("shipper_files LIKE '%\"name_id\";s:%" . strlen($load_document->id) . ":\"$load_document->id\"%'")
                      ->orWhereRaw("consignee_files LIKE '%\"name_id\";s:%" . strlen($load_document->id) . ":\"$load_document->id\"%'");
            })
            ->get();
        if($brokers->isNotEmpty()){
            foreach ($brokers as $key => $broker) {
                // Shipper
                $broker_shipper_files           = [];
                if(isset($broker->shipper_files) && is_serialized($broker->shipper_files) && !empty(unserialize($broker->shipper_files))){
                    $broker_shipper_files       = unserialize($broker->shipper_files);
                }

                if(!empty($broker_shipper_files)){
                    $searchkey                  = array_search($load_document->id, array_column($broker_shipper_files, 'name_id'));
                    if($searchkey!==false){
                        unset($broker_shipper_files[$searchkey]);
                    }
                }

                $broker_shipper_files           = array_values($broker_shipper_files);
                if(!empty($broker_shipper_files)){
                    $broker->shipper_files      = serialize($broker_shipper_files);
                }else{
                    $broker->shipper_files      = null;
                }
                // Shipper

                // Consignee
                $broker_consignee_files         = [];
                if(isset($broker->consignee_files) && is_serialized($broker->consignee_files) && !empty(unserialize($broker->consignee_files))){
                    $broker_consignee_files     = unserialize($broker->consignee_files);
                }

                if(!empty($broker_consignee_files)){
                    $searchkey                  = array_search($load_document->id, array_column($broker_consignee_files, 'name_id'));
                    if($searchkey!==false){
                        unset($broker_consignee_files[$searchkey]);
                    }
                }

                $broker_consignee_files         = array_values($broker_consignee_files);
                if(!empty($broker_consignee_files)){
                    $broker->consignee_files    = serialize($broker_consignee_files);
                }else{
                    $broker->consignee_files    = null;
                }
                // Consignee

                $broker->save();
            }
        }
        // Broker Document Update

        $load_document->delete();
        notification('success', $this->crudMessage('delete', 'Load document'));
        return redirect()->route('company.admin.load-document.index');
    }
}
