<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;
use App\Models\TrailerType;
use App\Models\Bookmark;
use App\Models\CompanyLoadRequirement;

use DataTables;
use Validator;
use Carbon\Carbon;

class LoadReqirementController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        Carbon::setWeekStartsAt(Carbon::SUNDAY);
        Carbon::setWeekEndsAt(Carbon::SATURDAY);
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {

        $getField = '';

        $getCompanyAdmin = auth()->guard('company-admin')->user();
        $isCheck = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','trailer_type')->first();

        if(!empty($isCheck->id)){
            //$getField = unserialize($isCheck->config);
            $getField = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_load_requirement_first'));
        }else{
            $getField = config('settings.table_load_requirement_first');
        }

        if ($request->ajax()) {
            $data = CompanyLoadRequirement::isCompany()->select('*');
            return Datatables::of($data)
                    ->addIndexColumn()
                    ->addColumn('action', function($row){
                        $btn = '<a href="#" class="edit-btn mr-4" data-id="'.$row->id.'" data-toggle="modal" data-target="#editModal"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';
                        $btn = $btn.' <a href="#" class="destroy" data-action="'.route('company.admin.load-requirement.destroy',$row->id).'" data-id="'.$row->id.'"><img src="'.asset('theme/img/delete-icon.png').'" alt=""></a>';
                        return $btn;
                    })
                    ->rawColumns(['action'])
                    ->make(true);
        }

        return view('companyAdmin.load-requirement.index', compact('getField'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $input = $request->all();

        $validator = Validator::make($input,[
            "name" => "required",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $input['company_id'] = auth()->guard('company-admin')->user()->company_id;

        CompanyLoadRequirement::create($input);

        notification('success', $this->crudMessage('create', 'Load Requirement'));

        return response()->json(['success' => "Load Requirement created successfully"]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function editFormView(Request $request)
    {
        $loadRequirement = CompanyLoadRequirement::find($request->id);

        $view = view('companyAdmin.load-requirement.editFormView', compact('loadRequirement'))->render();

        return response()->json(['view' => $view, 'data' => $loadRequirement]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $input = $request->all();
        $validator = Validator::make($input,[
            "name" => "required",
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }
        CompanyLoadRequirement::find($id)->update($input);

        notification('success', $this->crudMessage('update', 'Load Requirement'));

        return response()->json(['success' => "Load Requirement updated successfully"]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(CompanyLoadRequirement $loadRequirement)
    {
        $loadRequirement->delete();

        notification('success', $this->crudMessage('delete', 'Load Requirement'));

        return redirect()->back();
    }
}
