<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;

use App\Models\Truck;

class MapController extends CompanyAdminController{

    public function index(Request $request){
        if( $request->ajax() ) {
            try {
                $query                                  = $this->getTruckQuery();
                $trucks                                 = $query->get();
                $coordinates                            = [];
                if($trucks->isNotEmpty()){
                    foreach ($trucks as $latkey => $row) {
                        $array                          = [];
                        $array['id']                    = $row->id;
                        $array['text']                  = $row->unit;
                        $array['lat']                   = '';
                        $array['lng']                   = '';
                        $array['category']              = 'Inactive Truck';
                        if(isset($row->previousLoadDetail)){
                            if(
                                (isset($row->previousLoadDetail->delivery_latitude) && $row->previousLoadDetail->delivery_latitude != '') && 
                                (isset($row->previousLoadDetail->delivery_longitude) && $row->previousLoadDetail->delivery_longitude != '')
                            ){
                                $array['lat']           = $row->previousLoadDetail->delivery_latitude;
                                $array['lng']           = $row->previousLoadDetail->delivery_longitude;
                            }
                        }
                        if(
                            (isset($row->last_location_latitude) && $row->last_location_latitude != '') && 
                            (isset($row->last_location_longitude) && $row->last_location_longitude != '')
                        ){
                            $array['lat']               = $row->last_location_latitude;
                            $array['lng']               = $row->last_location_longitude;
                        }
                        if(
                            (isset($row->latestVehicleHistoricalStat->latitude) && $row->latestVehicleHistoricalStat->latitude != '') && 
                            (isset($row->latestVehicleHistoricalStat->longitude) && $row->latestVehicleHistoricalStat->longitude != '')
                        ){
                            $array['lat']               = $row->latestVehicleHistoricalStat->latitude;
                            $array['lng']               = $row->latestVehicleHistoricalStat->longitude;
                        }
                        if(isset($row->currentLoadDetail->id) && $row->currentLoadDetail->id != ''){
                            $array['category']          = 'Active Truck';

                            $array['load_number']       = getLoadTypeList()[$row->currentLoadDetail->load_type] ?? '';
                            if(isset($row->currentLoadDetail->load_number) && $row->currentLoadDetail->load_number != ''){
                                $array['load_number']   = $row->currentLoadDetail->load_number;
                            }
                            $array['url']               = route('company.admin.loads.show', base64Encode($row->currentLoadDetail->id));
                        }
                        $coordinates[]                  = $array;
                    }
                }
                $response['data']                       = $coordinates;
                $response['success']                    = true;
                $response['message']                    = 'Retrieved successfully';
            } catch( \Exception $e ) {
                $response['success']                    = false;
                $response['message']                    = $e->getMessage();
            }
            return response()->json($response);
        } else {
            try {
                return view('companyAdmin.map.cluster.index');
            } catch( \Exception $e ) {
                return abort(404);
            }
        }
    }

    public function filterList(Request $request){
        $search                         = ($request->has('search') && $request->search!='') ? $request->search : '';

        $perPage                        = $request->input('perPage', 10); // Default per page
        $page                           = $request->input('page', 1);
        $offset                         = ($page - 1) * $perPage;

        $query                          = $this->getTruckQuery();
        if($search!=''){
            $query->where('unit','LIKE','%'.$search.'%');
        }
        if($request->has('rtoggle') && in_array($request->rtoggle, ['Active Truck', 'Inactive Truck'])){
            if($request->rtoggle == 'Active Truck'){
                $query->whereHas('currentLoadDetail');
            }else{
                $query->whereDoesntHave('currentLoadDetail');
            }
        }
        // Total Count
        $total_count                    = $query->count();

        $query->offset($offset);
        $query->limit($perPage);
        $records                        = $query->get();

        $data                           = [];
        foreach ($records as $key => $row) {
            $array                      = [];
            $array['id']                = $row->id;
            $array['text']              = $row->unit;
            $array['lat']               = '';
            $array['lng']               = '';
            if(isset($row->previousLoadDetail)){
                if(
                    (isset($row->previousLoadDetail->delivery_latitude) && $row->previousLoadDetail->delivery_latitude != '') && 
                    (isset($row->previousLoadDetail->delivery_longitude) && $row->previousLoadDetail->delivery_longitude != '')
                ){
                    $array['lat']       = $row->previousLoadDetail->delivery_latitude;
                    $array['lng']       = $row->previousLoadDetail->delivery_longitude;
                }
            }
            if(
                (isset($row->last_location_latitude) && $row->last_location_latitude != '') && 
                (isset($row->last_location_longitude) && $row->last_location_longitude != '')
            ){
                $array['lat']           = $row->last_location_latitude;
                $array['lng']           = $row->last_location_longitude;
            }
            if(
                (isset($row->latestVehicleHistoricalStat->latitude) && $row->latestVehicleHistoricalStat->latitude != '') && 
                (isset($row->latestVehicleHistoricalStat->longitude) && $row->latestVehicleHistoricalStat->longitude != '')
            ){
                $array['lat']           = $row->latestVehicleHistoricalStat->latitude;
                $array['lng']           = $row->latestVehicleHistoricalStat->longitude;
            }
            $array['category']          = 'Inactive Truck';
            if(isset($row->currentLoadDetail->id) && $row->currentLoadDetail->id != ''){
                $array['category']      = 'Active Truck';
            }
            $data[]                     = $array;
        }
        return response()->json([
            'items'                     => $data,
            'total_count'               => $total_count
        ]);
    }

    public function getTruckQuery(){
        $query                          = Truck::select(
                'trucks.id', 
                'trucks.company_id',
                'trucks.samsara_id',
                'trucks.eorb_provider',
                'trucks.eorb_id', 
                'trucks.unit',
                'trucks.vin_number', 
                'trucks.make', 
                'trucks.year', 
                'trucks.model',
                'trucks.last_location_name',
                'trucks.last_location_address',
                'trucks.last_location_address_2',
                'trucks.last_location_city',
                'trucks.last_location_state',
                'trucks.last_location_zip',
                'trucks.last_location_latitude',
                'trucks.last_location_longitude',
                'trucks.last_location_timezone',
                'trucks.last_location_timezone_offset',
                'trucks.last_location_timezone_abbr',
                'trucks.last_location_address_verified',
                'trucks.last_location_notes',
                'trucks.last_location_load_status'
            )
            ->with([
                'latestVehicleHistoricalStat' => function ($query) {
                    $query->select('id', 'truck_id', 'location', 'latitude', 'longitude', 'data', 'last_updated_at', 'created_at')
                          ->where('last_updated_at', '>=', now()->subMinutes(10));
                },
                'currentLoadDetail' => function ($query) {
                    $query->select('id', 'truck_number', 'load_number', 'load_id', 'load_type', 'load_status');
                },
                'previousLoadDetail' => function ($query) {
                    $query->select('id', 'truck_number', 'load_number', 'load_id', 'load_type', 'load_status', 'delivery_address_one', 'delivery_address_two', 'delivery_city', 'delivery_state', 'delivery_zip', 'delivery_latitude', 'delivery_longitude', 'delivery_timezone', 'delivery_timezone_offset', 'delivery_timezone_abbr', 'delivery_address_verified', 'actual_delivery_date');
                }
            ])
            ->where('company_id', auth()->guard('company-admin')->user()->company_id);
        // $trucks = $query->get();
        return $query;
    }
}
