<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;

use App\Models\Note;
use App\Models\Team;
use App\Models\Broker;
use App\Models\Driver;
use App\Models\Vendor;
use App\Models\FileUpload;
use App\Models\CompanyAdmin;

use App\Models\Trailer;
use App\Models\Truck;
use App\Models\Maintenance;
use App\Models\DriverInformation;

use App\Mail\CompanyAdmin\NoteCreate;

use DataTables;
use File;
use Mail;

class NoteController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */

    public $timeZone;
    public $timeZoneId;
    public $offset;

    public function __construct(){
        parent::__construct();

        $this->middleware(function ($request, $next) {
            $this->timeZone     = getTimeZone();
            $this->timeZoneId   = getTimeZoneId();
            $this->offset       = getTimeZoneOffset();

            return $next($request);
        });
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getTimeZone    = $this->timeZone;
        $getTimeZoneId  = $this->timeZoneId;
        $getNote        = Note::where('action_id',$request->action_id)->where('action_type',$request->action_type)->orderBy('created_at','asc')->get();
        $view           = view('companyAdmin.notes.index', compact('getNote','getTimeZone','getTimeZoneId'))->render();
        return response()->json(['view' => $view]);
    }


    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request){
        $input                                              = $request->except(['_token', 'submit']);
        if(!empty($request->action_id) && !empty($request->note)){
            if ($request->notify_dispatcher == 1) {
                $input['notify_dispatcher']                 = 1;
            }else{
                $input['notify_dispatcher']                 = 0;
            }
            if ($request->notify_dirver == 1) {
                $input['notify_dirver']                     = 1;
            }else{
                $input['notify_dirver']                     = 0;
            }
            if ($request->notify_my == 1) {
                $input['notify_my']                         = 1;
            }else{
                $input['notify_my']                         = 0;
            }
            if ($request->notify_private == 1) {
                $input['notify_private']                    = 1;
            }else{
                $input['notify_private']                    = 0;
            }
            if(empty($request->request_type)){
                $input['request_type']                      = 0;
            }
            if(empty($request->type)){
                $input['type']                              = 0;
            }

            if (isset($request->notify_private_company_admin_id) && count($request->notify_private_company_admin_id) > 0) {
                $getID                                      = implode(',', $request->notify_private_company_admin_id);
                $input['notify_private_company_admin_id']   = $getID;
            }


            if (isset($request->note_file) && !empty($request->note_file[0])) {
                $file_temp                                  = [];
                $image_array                                = [];
                foreach($request->note_file as $image){
                    $seme                                   = FileUpload::upload('companyAdmin/temp', $image);
                    $stopath                                = public_path('storage/companyAdmin/temp/'.$seme);
                    $data                                   = file_get_contents($stopath);
                    $type                                   = pathinfo($stopath, PATHINFO_EXTENSION);
                    if($type == 'png' || $type == 'jpeg' || $type == 'jpg' || $type == 'gif'){
                        $base64                             = 'data:image/' . $type . ';base64,' . base64_encode($data);
                        $file_temp[]                        = $base64;
                    }else{
                        $image_array[] = $seme = FileUpload::upload('companyAdmin/notes', $image);
                    }
                }

                if(count($file_temp) > 0){
                    $fileName                               = $request->action_type.'_'.$request->action_id;
                    $image_array[]                          = FileUpload::PDFupload('companyAdmin/notes', ['data' => $file_temp], $fileName);
                }

                $input['document']                          = serialize($image_array);
                //$input['document']                        = FileUpload::PDFupload('companyAdmin/notes', $request->note_file);
            }

            $getNote                                        = Note::create($input);

            if($getNote->notify_my == 1){
                $userdata                                   = auth()->guard('company-admin')->user();
                $SourceId                                   = getNotiSourceList($request->action_type, 'slug')['id'];
                if($SourceId == 11){
                    $getUser                                = Broker::find($request->action_id);
                    $action_id                              = $getUser->id;
                    $message                                = " You have a new note on Broker - : ".$getUser->name;
                }
                if($SourceId == 12){
                    $getUser                                = Vendor::find($request->action_id);
                    $action_id                              = $getUser->id;
                    $message                                = " You have a new note on Vendor - : ".$getUser->name;
                }

                // send notify
                createNotification(
                    $SourceId, // source
                    'Added Note - '.$getUser->name, // name
                    $userdata->id, // user_id
                    $userdata->type, // user_type
                    1, // type
                    $action_id, // action_id
                    $getNote->note // description
                );
                $checkNotify                                = isUnreadDelay($userdata->id, $userdata->type, $SourceId);
                if(isset($checkNotify)){
                    if ($checkNotify->email == 1 && isset($userdata->email)) {
                        // send mail
                        $mailData                           = [];
                        $mailData['company_id']             = auth()->guard('company-admin')->user()->company_id;
                        $mailData['emailto']                = $userdata->email;
                        $mailData['hiname']                 = $userdata->full_name;
                        $mailData['phone']                  = $userdata->cell_phone;
                        $mailData['type']                   = getNoteType()[$getNote->type];
                        $mailData['request_type']           = getNoteRequestType()[$getNote->request_type];
                        $mailData['note']                   = $getNote->note;
                        $mailData['document']               = $getNote->note;
                        $mailData['source_type']            = getNotiSourceList($SourceId, 'id')['slug'];
                        $mailData['action_id']              = $action_id;
                        $mailData['full_name']              = $getUser->name;
                        //Mail::to($mailData['emailto'])->send(new NoteCreate($mailData));
                        sendEmail($mailData['emailto'], new NoteCreate($mailData));
                    }

                    if ($checkNotify->sms == 1 && isset($userdata->cell_phone)) {
                        // Twilio's credentials are fetched from company table
                        $getCompanyTwilioData               = getCompanyTwilioData();
                        if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                            sendSMS($userdata->cell_phone, $message, $getCompanyTwilioData);
                        }
                    }
                }
            }

            if($getNote->notify_private == 1){
                $getIds                                     = explode(',',$getNote->notify_private_company_admin_id);
                if(count($getIds) > 0){
                    foreach($getIds as $getId){
                        $userdata                           = CompanyAdmin::find($getId);
                        //$SourceId                         = array_keys(getNotiSourceList(),$request->action_type)[0];
                        $SourceId                           = getNotiSourceList($request->action_type, 'slug')['id'];
                        if($SourceId == 11){
                            $getUser                        = Broker::find($request->action_id);
                            $action_id                      = $getUser->id;
                            $message                        = " You have a new note on Broker - : ".$getUser->name;
                        }
                        if($SourceId == 12){
                            $getUser                        = Vendor::find($request->action_id);
                            $action_id                      = $getUser->id;
                            $message                        = " You have a new note on Vendor - : ".$getUser->name;
                        }

                        // send notify
                        createNotification(
                            $SourceId, // source
                            'Added Note - '.$getUser->name, // name
                            $userdata->id, // user_id
                            $userdata->type, // user_type
                            1, // type
                            $action_id, // action_id
                            $getNote->note // description
                        );
                        $checkNotify                        = isUnreadDelay($userdata->id, $userdata->type, $SourceId);
                        if(isset($checkNotify)){
                            if ($checkNotify->email == 1 && isset($userdata->email)) {
                                // send mail
                                $mailData                   = [];
                                $mailData['company_id']     = auth()->guard('company-admin')->user()->company_id;
                                $mailData['emailto']        = $userdata->email;
                                $mailData['hiname']         = $userdata->full_name;
                                $mailData['phone']          = $userdata->cell_phone;
                                $mailData['type']           = getNoteType()[$getNote->type];
                                $mailData['request_type']   = getNoteRequestType()[$getNote->request_type];
                                $mailData['note']           = $getNote->note;
                                $mailData['document']       = $getNote->note;
                                $mailData['source_type']    = getNotiSourceList($SourceId, 'id')['slug'];
                                $mailData['action_id']      = $action_id;
                                $mailData['full_name']      = $getUser->name;
                                //Mail::to($mailData['emailto'])->send(new NoteCreate($mailData));
                                sendEmail($mailData['emailto'], new NoteCreate($mailData));
                            }

                            if ($checkNotify->sms == 1 && isset($userdata->cell_phone)) {
                                // Twilio's credentials are fetched from company table
                                $getCompanyTwilioData       = getCompanyTwilioData();
                                if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                                    sendSMS($userdata->cell_phone, $message, $getCompanyTwilioData);
                                }
                            }
                        }
                    }
                }
            }

            if($getNote->notify_dispatcher == 1){
                $userdata                                   = auth()->guard('company-admin')->user();
                $SourceId                                   = getNotiSourceList($request->action_type, 'slug')['id'];

                if($SourceId == 16){
                    $maintenance                            = Maintenance::findOrFail($request->action_id);
                    $action_id                              = $maintenance->id;
                    if($maintenance->equipment_type == 1){
                        $getTruck                           = Truck::find($maintenance->truck_namber);
                        $gerDriverInformation   = DriverInformation::where('truck_number', $getTruck->unit)->first();
                        $gerDriver                          = Driver::find($gerDriverInformation->driver_id);
                    }

                    if($maintenance->equipment_type == 2){
                        $getLoadDetail                      = LoadDetail::where('trailer_number', $maintenance->trailer_namber)->first();
                        $gerDriver                          = Driver::find($getLoadDetail->driver_id);
                    }
                }

                if(isset($maintenance->id) && !empty($maintenance->id)){
                    $getTeam                                = Team::find($gerDriver->team_id);
                    $getUser                                = CompanyAdmin::where('team_id', $getTeam->id)->where('type',2)->first();

                    // send notify
                    createNotification(
                        $SourceId, // source
                        'Added Note in Maintenance - '.$maintenance->ticket_number, // name
                        $getUser->id, // user_id
                        $getUser->type, // user_type
                        1, // type
                        $action_id, // action_id
                        $getNote->note // description
                    );
                    $checkNotify                            = isUnreadDelay($getUser->id, $getUser->type, $SourceId);
                    if(isset($checkNotify)){
                        if ($checkNotify->email == 1 && isset($getUser->email)) {
                            // send mail
                            $mailData                       = [];
                            $mailData['company_id']         = auth()->guard('company-admin')->user()->company_id;
                            $mailData['emailto']            = $getUser->email;
                            $mailData['hiname']             = $getUser->full_name;
                            $mailData['phone']              = $getUser->cell_phone;
                            $mailData['type']               = getNoteType()[$getNote->type];
                            $mailData['request_type']       = getNoteRequestType()[$getNote->request_type];
                            $mailData['note']               = $getNote->note;
                            $mailData['document']           = $getNote->note;
                            $mailData['source_type']        = getNotiSourceList($SourceId, 'id')['slug'];
                            $mailData['action_id']          = $action_id;
                            $mailData['full_name']          = $getUser->name;
                            //Mail::to($mailData['emailto'])->send(new NoteCreate($mailData));
                            sendEmail($mailData['emailto'], new NoteCreate($mailData));
                        }

                        if ($checkNotify->sms == 1 && isset($getUser->cell_phone)) {
                            // Twilio's credentials are fetched from company table
                            $getCompanyTwilioData           = getCompanyTwilioData();
                            if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                                $message                    = "Added new maintenance - : ".$maintenance->ticket_number;
                                sendSMS($getUser->cell_phone, $message, $getCompanyTwilioData);
                            }
                        }
                    }
                }
            }

            if($request->notify_driver == 1){
                $userdata                                   = auth()->guard('company-admin')->user();
                $SourceId                                   = getNotiSourceList($request->action_type, 'slug')['id'];
                if($SourceId == 16){
                    $maintenance                            = Maintenance::findOrFail($request->action_id);
                    $action_id                              = $maintenance->id;
                    if($maintenance->equipment_type == 1){
                        $getTruck                           = Truck::find($maintenance->truck_namber);
                        $gerDriverInformation = DriverInformation::where('truck_number', $getTruck->unit)->first();
                        $getUser                            = Driver::find($gerDriverInformation->driver_id);
                    }
                    if($maintenance->equipment_type == 2){
                        $getLoadDetail                      = LoadDetail::where('trailer_number', $maintenance->trailer_namber)->first();
                        $gerDriver                          = Driver::find($getLoadDetail->driver_id);
                    }
                }

                if(isset($maintenance->id) && !empty($maintenance->id)){
                    // send notify
                    createNotification(
                        $SourceId, // source
                        'Added Note in Maintenance - '.$maintenance->ticket_number, // name
                        $getUser->id, // user_id
                        3, // user_type
                        1, // type
                        $action_id, // action_id
                        $getNote->note // description
                    );
                    $checkNotify                            = isUnreadDelay($getUser->id, 3, $SourceId);
                    if(isset($checkNotify)){
                        if ($checkNotify->email == 1 && isset($getUser->email)) {
                            // send mail
                            $mailData                       = [];
                            $mailData['company_id']         = auth()->guard('company-admin')->user()->company_id;
                            $mailData['emailto']            = $getUser->email;
                            $mailData['hiname']             = $getUser->full_name;
                            $mailData['phone']              = $getUser->cell_phone;
                            $mailData['type']               = getNoteType()[$getNote->type];
                            $mailData['request_type']       = getNoteRequestType()[$getNote->request_type];
                            $mailData['note']               = $getNote->note;
                            $mailData['document']           = $getNote->note;
                            $mailData['source_type']        = getNotiSourceList($SourceId, 'id')['slug'];
                            $mailData['action_id']          = $action_id;
                            $mailData['full_name']          = $getUser->name;

                            //Mail::to($mailData['emailto'])->send(new NoteCreate($mailData));
                            sendEmail($mailData['emailto'], new NoteCreate($mailData));
                        }

                        if ($checkNotify->sms == 1 && isset($getUser->cell_phone)) {
                            // Twilio's credentials are fetched from company table
                            $getCompanyTwilioData           = getCompanyTwilioData();
                            if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                                $message                    = "Added new maintenance - : ".$maintenance->ticket_number;
                                sendSMS($getUser->cell_phone, $message, $getCompanyTwilioData);
                            }
                        }
                    }
                }
            }
            return 1;
        }
        return 0;
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id){
        $getLoadNote = Note::find($id);
        File::delete('storage/companyAdmin/notes/'.$getLoadNote->document);
        $getLoadNote->delete();
        return 1;
    }
}
