<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;
use Illuminate\Support\Str;

use App\Http\Requests\CompanyAdmin\DriverStoreRequest;
use App\Http\Requests\CompanyAdmin\DriverUpdateRequest;

use App\Mail\CompanyAdmin\NewPasswordMailToDriver;
use App\Mail\CompanyAdmin\WelcomeMailToDriver;

use App\Models\Team;
use App\Models\State;
use App\Models\CompanyHoliday;

use App\Models\LoadDetail;
use App\Models\LoadDriver;

use App\Models\LoadFile;
use App\Models\LoadFileType;

use App\Models\Driver;
use App\Models\DriverFinancial;
use App\Models\DriverInformation;

use App\Models\DriverAvoidState;
use App\Models\DriverRestrictedState;

use App\Models\DriverCitation;
use App\Models\DriverHosViolation;
use App\Models\DriverMechanicalIssue;
use App\Models\DriverComplianceSafety;

use App\Models\FileUpload;

use DB;
use Hash;
use Mail;
use DataTables;
use Carbon\Carbon;
use Carbon\CarbonPeriod;
use Twilio\Rest\Client;

class OccupancyCalendarController extends CompanyAdminController{

    /**
     * Create a new controller instance.
     *
     * @return void
     */

    public $timeZone;

    public function __construct(){
        parent::__construct();

        $this->middleware(function ($request, $next) {
            $this->timeZone     = getTimeZone();

            return $next($request);
        });
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(){
        return view('companyAdmin.occupancyCalendar.index');
    }

    public function OccupancyCalanderView(Request $request){
        $showtype                   = $request->showtype;
        $driver_id                  = $request->driver_id;
        $year                       = '';
        $month                      = '';
        $month_name                 = '';
        $week                       = '';
        $getDriverTargetRevenue     = '';
        $getDriverTargetOccupancy   = '';
        $last_month_name            = '';
        $verageRevenuePerDay        = 0;
        $AverageMilePerDay          = 0;
        $CentPerMile                = 0;

        if($request->showtype == 'year'){
            if($request->get_data == 'current'){
                $year               = Carbon::now()->timezone($this->timeZone)->year;
            }elseif($request->get_data == 'next'){
                $year               = $request->cyear + 1;
            }elseif($request->get_data == 'previous'){
                $year               = $request->cyear - 1;
            }else{
                $year               = Carbon::now()->timezone($this->timeZone)->year;
            }

            $startDate              = $year.'-01-01';
            $endDate                = $year.'-12-31';
        }

        if($request->showtype == 'month'){
            if($request->get_data == 'current'){
                $now                = Carbon::now()->timezone($this->timeZone);
            }elseif($request->get_data == 'next'){
                $now                = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone)->addMonth();
            }elseif($request->get_data == 'previous'){
                $now                = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone)->subMonth();
            }else{
                $now                = Carbon::now()->timezone($this->timeZone);
            }

            $month                  = $now->format('m');
            $month_name             = $now->format('F');
            $year                   = $now->format('Y');
            $startDate              = $now->startOfMonth()->format('Y-m-d');
            $endDate                = $now->endOfMonth()->format('Y-m-d');
        }

        if($request->showtype == 'quarterly'){
            if($request->get_data == 'current'){
                $now                = Carbon::now()->timezone($this->timeZone);
            }elseif($request->get_data == 'next'){
                $now                = Carbon::createFromDate($request->cyear, $request->cmonth,'01',$this->timeZone)->addMonth(3);
            }elseif($request->get_data == 'previous'){
                $now                = Carbon::createFromDate($request->cyear, $request->cmonth,'01',$this->timeZone)->subMonth(3);
            }else{
                $now                = Carbon::now()->timezone($this->timeZone);
            }

            $month                  = $now->firstOfQuarter()->format('m');
            $month_name             = $now->firstOfQuarter()->format('F');
            $year                   = $now->firstOfQuarter()->format('Y');
            $startDate              = $now->firstOfQuarter()->format('Y-m-d');
            $endDate                = $now->lastOfQuarter()->format('Y-m-d');
            $last_month_name        = $now->lastOfQuarter()->format('F');
        }

        if($request->showtype == 'weekly'){
            if ($request->get_data == 'current') {
                $weeklyDay          = 0;
                $currentStartDate   = Carbon::now()->timezone($this->timeZone)->startOfWeek($weeklyDay)->format('Y-m-d');
                $currentEndDate     = Carbon::parse($currentStartDate)->addDays(6)->format('Y-m-d');
                $startDate          = Carbon::parse($currentStartDate)->format('Y-m-d');
                $endDate            = Carbon::parse($currentEndDate)->format('Y-m-d');
            } elseif ($request->get_data == 'next') {
                $startDate          = Carbon::parse($request->start_date)->addWeek()->format('Y-m-d');
                $endDate            = Carbon::parse($request->end_date)->addWeek()->format('Y-m-d');
            } elseif ($request->get_data == 'previous') {
                $startDate          = Carbon::parse($request->start_date)->subWeek()->format('Y-m-d');
                $endDate            = Carbon::parse($request->end_date)->subWeek()->format('Y-m-d');
            }

            $year                   = Carbon::parse($startDate)->year;
            $week                   = Carbon::parse($startDate)->weekOfYear;
        }

        if($request->showtype == 'linear'){
            if($request->get_data == 'current'){
                $now                = Carbon::now()->timezone($this->timeZone);
            }elseif($request->get_data == 'next'){
                $now                = Carbon::createFromDate($request->cyear, $request->cmonth,'01',$this->timeZone)->addMonth();
            }elseif($request->get_data == 'previous'){
                $now                = Carbon::createFromDate($request->cyear, $request->cmonth,'01',$this->timeZone)->subMonth();
            }else{
                $now                = Carbon::now()->timezone($this->timeZone);
            }

            $month                  = $now->format('m');
            $month_name             = $now->format('F');
            $year                   = $now->format('Y');
            $startDate              = $now->startOfMonth()->format('Y-m-d');
            $endDate                = $now->endOfMonth()->format('Y-m-d');

            $month_period           = CarbonPeriod::create($startDate, $endDate);

            $driverlist             = Driver::select(DB::raw("CONCAT(last_name,', ',first_name) AS name"),'id')
                ->where('company_id',auth()->guard('company-admin')->user()->company_id)
                ->orderByRaw('drivers.last_name ASC, drivers.first_name ASC')
                ->get();
        }

        
        $view                       = view('companyAdmin.occupancyCalendar.view.OccupancyCalendarView', compact('week','year','month','showtype','driver_id','month_name','last_month_name','startDate','endDate'))->render();
        $driver                     = Driver::find($driver_id);
        $driver_name                = $driver->last_name.', '.$driver->first_name;
        return response()->json(['view' => $view, 'driver_name' => $driver_name]);
    }

    public function OccupancyCalanderReportView(Request $request){
        $showtype                   = $request->showtype;
        $driver_id                  = $request->driver_id;
        $year                       = '';
        $month                      = '';
        $month_name                 = '';
        $week                       = '';
        $getDriverTargetRevenue     = '';
        $getDriverTargetOccupancy   = '';
        $last_month_name            = '';
        $verageRevenuePerDay        = 0;

        //$AverageMilePerDay        = 0;
        $AverageDeadheadMilePerDay  = 0;
        $AverageLoadedMilePerDay    = 0;
        $AverageTotalMilePerDay     = 0;

        //$CentPerMile              = 0;
        $CentLoadedPerMile          = 0;
        $CentTotalPerMile           = 0;

        if($request->showtype == 'year'){
            if($request->get_data == 'current'){
                $year               = Carbon::now()->timezone($this->timeZone)->year;
            }elseif($request->get_data == 'next'){
                $year               = $request->cyear + 1;
            }elseif($request->get_data == 'previous'){
                $year               = $request->cyear - 1;
            }else{
                $year               = Carbon::now()->timezone($this->timeZone)->year;
            }

            $startDate              = $year.'-01-01';
            $endDate                = $year.'-12-31';
        }

        if($request->showtype == 'month'){
            if($request->get_data == 'current'){
                $now                = Carbon::now()->timezone($this->timeZone);
            }elseif($request->get_data == 'next'){
                $now                = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone)->addMonth();
            }elseif($request->get_data == 'previous'){
                $now                = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone)->subMonth();
            }else{
                $now                = Carbon::now()->timezone($this->timeZone);
            }

            $month                  = $now->format('m');
            $month_name             = $now->format('F');
            $year                   = $now->format('Y');
            $startDate              = $now->startOfMonth()->format('Y-m-d');
            $endDate                = $now->endOfMonth()->format('Y-m-d');
        }

        if($request->showtype == 'quarterly'){
            if($request->get_data == 'current'){
                $now                = Carbon::now()->timezone($this->timeZone);
            }elseif($request->get_data == 'next'){
                $now                = Carbon::createFromDate($request->cyear, $request->cmonth,'01',$this->timeZone)->addMonth(3);
            }elseif($request->get_data == 'previous'){
                $now                = Carbon::createFromDate($request->cyear, $request->cmonth,'01',$this->timeZone)->subMonth(3);
            }else{
                $now                = Carbon::now()->timezone($this->timeZone);
            }

            $month                  = $now->firstOfQuarter()->format('m');
            $month_name             = $now->firstOfQuarter()->format('F');
            $year                   = $now->firstOfQuarter()->format('Y');
            $startDate              = $now->firstOfQuarter()->format('Y-m-d');
            $endDate                = $now->lastOfQuarter()->format('Y-m-d');
            $last_month_name        = $now->lastOfQuarter()->format('F');
        }

        if($request->showtype == 'weekly'){
            if ($request->get_data == 'current') {
                $weeklyDay          = 0;
                $currentStartDate   = Carbon::now()->timezone($this->timeZone)->startOfWeek($weeklyDay)->format('Y-m-d');
                $currentEndDate     = Carbon::parse($currentStartDate)->addDays(6)->format('Y-m-d');
                $startDate          = Carbon::parse($currentStartDate)->format('Y-m-d');
                $endDate            = Carbon::parse($currentEndDate)->format('Y-m-d');
            } elseif ($request->get_data == 'next') {
                $startDate          = Carbon::parse($request->start_date)->addWeek()->format('Y-m-d');
                $endDate            = Carbon::parse($request->end_date)->addWeek()->format('Y-m-d');
            } elseif ($request->get_data == 'previous') {
                $startDate          = Carbon::parse($request->start_date)->subWeek()->format('Y-m-d');
                $endDate            = Carbon::parse($request->end_date)->subWeek()->format('Y-m-d');
            }

            $year                   = Carbon::parse($startDate)->year;
            $week                   = Carbon::parse($startDate)->weekOfYear;
        }

        /*
        $occupied                   = getloadCount($startDate, $endDate, 0, $driver_id);
        $maintenance                = getloadCount($startDate, $endDate, 1, $driver_id);
        $unauthorized               = getloadCount($startDate, $endDate, 2, $driver_id);
        $hos                        = getloadCount($startDate, $endDate, 3, $driver_id);
        $timeOff                    = getloadCount($startDate, $endDate, 5, $driver_id);

        $tot                        = $occupied + $maintenance + $unauthorized + $hos + $timeOff;

        if ($tot > 0) {
            $occupied               = round(($occupied * 100) / $tot, 2);
            $maintenance            = round(($maintenance * 100) / $tot, 2);
            $unauthorized           = round(($unauthorized * 100) / $tot, 2);
            $hos                    = round(($hos * 100) / $tot, 2);
            $timeOff                = round(($timeOff * 100) / $tot, 2);
        }
        */

        $otypearray                         = [];

        $dateRange                          = CarbonPeriod::create($startDate, $endDate);
        $monthdata                          = [];
        $k                                  = 0;
        foreach($dateRange as $drange){
            $monthdata[$k]                  = ['date' => $drange->format('Y-m-d'),'dayname' => substr($drange->format('l'), 0, 3),'daynumber' => $drange->format('d')];
            $k++;
        }

        $loadDetails                        = LoadDetail::select('id','load_id','load_number','split_date','driver_id','truck_number','co_driver_id','booked_by','self_dispatch','load_type','load_status','is_on_hold','start_date','end_date','shipping_time_range','occupied_pickup_day','shipper_date_time','shipper_date_time_end','actual_shipper_date','delivery_time_range','delivery_date_time','delivery_date_time_end','actual_delivery_date','submitted_by')
            ->where('driver_id', $driver_id)
            ->where(function ($query) use ($endDate, $startDate) {
                $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
            })
            ->where('is_disable', 0)
            ->where('load_status', '!=', 9)
            ->where(function($q) {
                $q->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0,1]);
            })
            ->get();
        if($loadDetails->isNotEmpty()){
            $dayloads                       = [];
            foreach ($monthdata as $mdata){
                $daynumber                  = $mdata['daynumber'];
                $date                       = $mdata['date'];
                $laod_array                 = [];
                foreach ($loadDetails as $key => $load) {
                    $actual_shipper_date    = Carbon::createFromFormat(getTimeLayout(),$load->actual_shipper_date)->format('Y-m-d');
                    $actual_delivery_date   = Carbon::createFromFormat(getTimeLayout(),$load->actual_delivery_date)->format('Y-m-d');

                    if(in_array($load->load_type, [0, 4, 6]) && $load->occupied_pickup_day == 0 && $actual_shipper_date < $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                        // 0 = OCCUPIED, 4 = NO DISPATCH, 6 = TONU
                        $laod_array[$load->load_id]     = $load->load_type;
                    }
                    if(in_array($load->load_type, [0, 4, 6]) && $load->occupied_pickup_day == 1 && $actual_shipper_date <= $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                        // 0 = OCCUPIED, 4 = NO DISPATCH, 6 = TONU
                        $laod_array[$load->load_id]     = $load->load_type;
                    }
                    if(in_array($load->load_type, [1, 2, 3, 5]) && $actual_shipper_date <= $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                        // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
                        $laod_array[$load->load_id]     = $load->load_type;
                    }
                }
                if(!empty($laod_array)){
                    $dayloads[$date]                    = $laod_array;
                }
            }
            if(!empty($dayloads)){
                foreach ($dayloads as $day => $laod_array) {
                    if(!empty($laod_array)){
                        if(count($laod_array) > 1){
                            $arry                       = [];
                            foreach($laod_array as $load_id => $ltype){
                                $arry[]                 = $ltype;
                            }
                            $arry                       = array_values(array_unique($arry));
                            if(count($arry) === 1){
                                $otypearray[$arry[0]][] = 1;
                            }else{
                                // Order 1.) Unauthorized 2.) Hours of Service 3.) Maintenance 4.) Time-off 5.) No Dispatch 6.) TONU 7.) Occupied
                                // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH, 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
                                if (in_array(2, $arry)) {
                                    // 2 = UNAUTHORIZED
                                    $otypearray[2][]    = 1;
                                } elseif (in_array(3, $arry)) {
                                    // 3 = HOS
                                    $otypearray[3][]    = 1;
                                } elseif (in_array(1, $arry)) {
                                    // 1 = MAINTENANCE
                                    $otypearray[1][]    = 1;
                                } elseif (in_array(5, $arry)) {
                                    // 5 = TIME OFF
                                    $otypearray[5][]    = 1;
                                } elseif (in_array(4, $arry)) {
                                    // 4 = NO DISPATCH
                                    $otypearray[4][]    = 1;
                                } elseif (in_array(6, $arry)) {
                                    // 6 = TONU
                                    $otypearray[6][]    = 1;
                                }else{
                                    // 0 = OCCUPIED
                                    $otypearray[0][]    = 1;
                                }
                            }
                        }else{
                            foreach($laod_array as $load_id => $ltype){
                                $otypearray[$ltype][]   = 1;
                            }
                        }
                    }
                }
            }
        }

        $tot                                = 0;
        $occupied                           = 0;
        $tonu                               = 0;
        $nodispatch                         = 0;
        $maintenance                        = 0;
        $unauthorized                       = 0;
        $hos                                = 0;
        $timeOff                            = 0;
        if(isset($dayloads) && !empty($dayloads)){
            $tot                            = count($dayloads);
            $occupied                       = isset($otypearray[0]) && !empty($otypearray[0]) ? count($otypearray[0]) : 0;
            $tonu                           = isset($otypearray[6]) && !empty($otypearray[6]) ? count($otypearray[6]) : 0;
            $nodispatch                     = isset($otypearray[4]) && !empty($otypearray[4]) ? count($otypearray[4]) : 0;
            $maintenance                    = isset($otypearray[1]) && !empty($otypearray[1]) ? count($otypearray[1]) : 0;
            $unauthorized                   = isset($otypearray[2]) && !empty($otypearray[2]) ? count($otypearray[2]) : 0;
            $hos                            = isset($otypearray[3]) && !empty($otypearray[3]) ? count($otypearray[3]) : 0;
            $timeOff                        = isset($otypearray[5]) && !empty($otypearray[5]) ? count($otypearray[5]) : 0;
        }

        if ($tot > 0) {
            $occupied                       = round(($occupied * 100) / $tot);
            $tonu                           = round(($tonu * 100) / $tot);
            $nodispatch                     = round(($nodispatch * 100) / $tot);
            $maintenance                    = round(($maintenance * 100) / $tot);
            $unauthorized                   = round(($unauthorized * 100) / $tot);
            $hos                            = round(($hos * 100) / $tot);
            $timeOff                        = round(($timeOff * 100) / $tot);
        }

        $occupanytype                       = view('companyAdmin.occupancyCalendar.view.OccupancyCalendarOccupanytypeView', compact('occupied','tonu','nodispatch','maintenance','unauthorized','hos','timeOff'))->render();


        $showtype                           = $request->showtype;
        $getLoadGrossRevenue                = getLoadGrossRevenue($startDate,$endDate,$driver_id);
        //$getLoadMileage                   = getLoadMileage($startDate,$endDate,$driver_id);
        $getLoadDeadheadMileage             = getLoadDeadheadMileage($startDate,$endDate,$driver_id);
        $getLoadLoadedMileage               = getLoadLoadedMileage($startDate,$endDate,$driver_id);
        $getLoadTotalMileage                = getLoadTotalMileage($startDate,$endDate,$driver_id);
        $getLoadOccupancy                   = $request->bggreen;
        //$getLoadOccupancy                 = round(getLoadTotalDays($startDate,$endDate,$driver_id));

        if($getLoadOccupancy > 0){
            $verageRevenuePerDay            = round($getLoadGrossRevenue / $getLoadOccupancy, 0);
            //$AverageMilePerDay            = round($getLoadMileage / $getLoadOccupancy, 2);
            $AverageDeadheadMilePerDay      = round($getLoadDeadheadMileage / $getLoadOccupancy, 0);
            $AverageLoadedMilePerDay        = round($getLoadLoadedMileage / $getLoadOccupancy, 0);
            $AverageTotalMilePerDay         = round($getLoadTotalMileage / $getLoadOccupancy, 0);
        }

        if($getLoadTotalMileage > 0){
            //$CentPerMile                  = round($getLoadGrossRevenue / $getLoadTotalMileage, 2);
            $CentLoadedPerMile              = round($getLoadGrossRevenue / $getLoadLoadedMileage, 2);
            $CentTotalPerMile               = round($getLoadGrossRevenue / $getLoadTotalMileage, 2);
        }

        $getDriverTargetRevenue             = getDriverTargetRevenue($startDate,$endDate,$driver_id);
        if($getDriverTargetRevenue > 0){
            $getDriverTargetRevenue         = round(($getLoadGrossRevenue * 100) / $getDriverTargetRevenue, 2);
        }

        $getDriverTargetOccupancy           = getDriverTargetOccupancy($startDate,$endDate,$driver_id);

        //$performanceSummary               = view('companyAdmin.occupancyCalendar.view.OccupancyCalendarPerformanceSummary', compact('showtype', 'getLoadGrossRevenue','getLoadMileage','verageRevenuePerDay','AverageMilePerDay','CentPerMile','getDriverTargetRevenue','getDriverTargetOccupancy'))->render();
        $performanceSummary                 = view('companyAdmin.occupancyCalendar.view.OccupancyCalendarPerformanceSummary', compact('showtype', 'getLoadGrossRevenue', 'getLoadDeadheadMileage', 'getLoadLoadedMileage', 'getLoadTotalMileage', 'verageRevenuePerDay', 'AverageDeadheadMilePerDay', 'AverageLoadedMilePerDay', 'AverageTotalMilePerDay', 'CentLoadedPerMile', 'CentTotalPerMile', 'getDriverTargetRevenue','getDriverTargetOccupancy'))->render();

        $driver         = Driver::find($driver_id);
        $driver_name    = $driver->last_name.', '.$driver->first_name;
        return response()->json(['performanceSummary' => $performanceSummary, 'driver_name' => $driver_name, 'occupanytype' => $occupanytype,]);
    }

    /*
    public function OccupancyCalanderReportView_backup(Request $request){
        $showtype                   = $request->showtype;
        $driver_id                  = $request->driver_id;
        $year                       = '';
        $month                      = '';
        $month_name                 = '';
        $week                       = '';
        $getDriverTargetRevenue     = '';
        $getDriverTargetOccupancy   = '';
        $last_month_name            = '';
        $verageRevenuePerDay        = 0;
        $AverageMilePerDay          = 0;
        $CentPerMile                = 0;

        if($request->showtype == 'year'){
            if($request->get_data == 'current'){
                $year               = Carbon::now()->timezone($this->timeZone)->year;
            }elseif($request->get_data == 'next'){
                $year               = $request->cyear + 1;
            }elseif($request->get_data == 'previous'){
                $year               = $request->cyear - 1;
            }else{
                $year               = Carbon::now()->timezone($this->timeZone)->year;
            }

            $startDate              = $year.'-01-01';
            $endDate                = $year.'-12-31';
        }

        if($request->showtype == 'month'){
            if($request->get_data == 'current'){
                $now                = Carbon::now()->timezone($this->timeZone);
            }elseif($request->get_data == 'next'){
                $now                = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone)->addMonth();
            }elseif($request->get_data == 'previous'){
                $now                = Carbon::createFromDate($request->cyear,$request->cmonth,'01',$this->timeZone)->subMonth();
            }else{
                $now                = Carbon::now()->timezone($this->timeZone);
            }

            $month                  = $now->format('m');
            $month_name             = $now->format('F');
            $year                   = $now->format('Y');
            $startDate              = $now->startOfMonth()->format('Y-m-d');
            $endDate                = $now->endOfMonth()->format('Y-m-d');
        }

        if($request->showtype == 'quarterly'){
            if($request->get_data == 'current'){
                $now                = Carbon::now()->timezone($this->timeZone);
            }elseif($request->get_data == 'next'){
                $now                = Carbon::createFromDate($request->cyear, $request->cmonth,'01',$this->timeZone)->addMonth(3);
            }elseif($request->get_data == 'previous'){
                $now                = Carbon::createFromDate($request->cyear, $request->cmonth,'01',$this->timeZone)->subMonth(3);
            }else{
                $now                = Carbon::now()->timezone($this->timeZone);
            }

            $month                  = $now->firstOfQuarter()->format('m');
            $month_name             = $now->firstOfQuarter()->format('F');
            $year                   = $now->firstOfQuarter()->format('Y');
            $startDate              = $now->firstOfQuarter()->format('Y-m-d');
            $endDate                = $now->lastOfQuarter()->format('Y-m-d');
            $last_month_name        = $now->lastOfQuarter()->format('F');
        }

        if($request->showtype == 'weekly'){
            if ($request->get_data == 'current') {
                $weeklyDay          = 0;
                $currentStartDate   = Carbon::now()->timezone($this->timeZone)->startOfWeek($weeklyDay)->format('Y-m-d');
                $currentEndDate     = Carbon::parse($currentStartDate)->addDays(6)->format('Y-m-d');
                $startDate          = Carbon::parse($currentStartDate)->format('Y-m-d');
                $endDate            = Carbon::parse($currentEndDate)->format('Y-m-d');
            } elseif ($request->get_data == 'next') {
                $startDate          = Carbon::parse($request->start_date)->addWeek()->format('Y-m-d');
                $endDate            = Carbon::parse($request->end_date)->addWeek()->format('Y-m-d');
            } elseif ($request->get_data == 'previous') {
                $startDate          = Carbon::parse($request->start_date)->subWeek()->format('Y-m-d');
                $endDate            = Carbon::parse($request->end_date)->subWeek()->format('Y-m-d');
            }

            $year                   = Carbon::parse($startDate)->year;
            $week                   = Carbon::parse($startDate)->weekOfYear;
        }

        if($request->showtype == 'linear'){
            if($request->get_data == 'current'){
                $now                = Carbon::now()->timezone($this->timeZone);
            }elseif($request->get_data == 'next'){
                $now                = Carbon::createFromDate($request->cyear, $request->cmonth,'01',$this->timeZone)->addMonth();
            }elseif($request->get_data == 'previous'){
                $now                = Carbon::createFromDate($request->cyear, $request->cmonth,'01',$this->timeZone)->subMonth();
            }else{
                $now                = Carbon::now()->timezone($this->timeZone);
            }

            $month                  = $now->format('m');
            $month_name             = $now->format('F');
            $year                   = $now->format('Y');
            $startDate              = $now->startOfMonth()->format('Y-m-d');
            $endDate                = $now->endOfMonth()->format('Y-m-d');

            $month_period           = CarbonPeriod::create($startDate, $endDate);

            $driverlist             = Driver::select(DB::raw("CONCAT(last_name,', ',first_name) AS name"),'id')
                ->where('company_id',auth()->guard('company-admin')->user()->company_id)
                ->orderByRaw('drivers.last_name ASC, drivers.first_name ASC')
                ->get();
        }

        $loadDetails                = LoadDetail::where('load_details.driver_id', $driver_id)
            ->join("load_drivers as ld", "ld.load_detail_id", "=", "load_details.id")
            ->where(function ($query) use ($endDate, $startDate) {
                $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
            })
            ->where('load_details.load_status', '!=', 9)
            ->where(function($q) {
                $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
            })
            ->whereNull('load_details.deleted_at')
            ->select('load_details.id','load_details.driver_id','load_details.actual_shipper_date','load_details.actual_delivery_date','load_details.load_status','load_details.load_type','load_details.occupied_pickup_day','load_details.driver_revenue','ld.miles')
            ->get();

        $dayCount                   = 0;
        $arryType                   = [];
        if(count($loadDetails) > 0){
            $load_type              = [0,1,2,3,5];
            foreach($load_type as $loadType){
                $dayCount           = 0;
                foreach($loadDetails as $load){
                    if($load->load_type == $loadType){
                       $startDateLoad       = Carbon::parse($load->actual_shipper_date)->format('Y-m-d');
                        $endDateLoad        = Carbon::parse($load->actual_delivery_date)->format('Y-m-d');
                        if($startDateLoad <= $startDate){
                            $startDateLoad  = $startDate;
                        }
                        if($endDateLoad >= $endDate){
                            $endDateLoad    = $endDate;
                        }
                        $period             = CarbonPeriod::create($startDateLoad, $endDateLoad);
                        $dcount             = 0;
                        $ky                 = 0;
                        foreach ($period as $date) {
                            $load_type      = array(0, 4, 6);
                            if (in_array($load->load_type, $load_type)) {
                                if($ky != 0){
                                    $dcount += 1;
                                }
                            }

                            $load_type      = array(1, 2, 3, 5);
                            if (in_array($load->load_type, $load_type)) {
                                $dcount     += 1;
                            }
                            $ky++;
                        }
                        if($load->occupied_pickup_day == 1){
                            $dcount         += 1;
                        }

                        $dayCount           += $dcount;
                    }
                    $arryType[$loadType]    = $dayCount;
                }
            }
        }
        $occupied                           = isset($arryType[0]) ? $arryType[0] : 0;
        $maintenance                        = isset($arryType[1]) ? $arryType[1] : 0;
        $unauthorized                       = isset($arryType[2]) ? $arryType[2] : 0;
        $hos                                = isset($arryType[3]) ? $arryType[3] : 0;
        $timeOff                            = isset($arryType[5]) ? $arryType[5] : 0;
        $tot                                = $occupied + $maintenance + $unauthorized + $hos + $timeOff;

        if($tot > 0){
            $occupied                       = round(($occupied * 100) / $tot, 2);
            $maintenance                    = round(($maintenance * 100) / $tot, 2);
            $unauthorized                   = round(($unauthorized * 100) / $tot, 2);
            $hos                            = round(($hos * 100) / $tot, 2);
            $timeOff                        = round(($timeOff * 100) / $tot, 2);
        }


        $occupanytype                       = view('companyAdmin.occupancyCalendar.view.OccupancyCalendarOccupanytypeView', compact('occupied','maintenance','unauthorized','hos','timeOff'))->render();


        $showtype                           = $request->showtype;
        $getLoadGrossRevenue                = getLoadGrossRevenue($startDate,$endDate,$driver_id);
        $getLoadMileage                     = getLoadMileage($startDate,$endDate,$driver_id);
        $getLoadOccupancy                   = $request->bggreen;
        //$getLoadOccupancy = round(getLoadTotalDays($startDate,$endDate,$driver_id));

        if($getLoadOccupancy > 0){
            $verageRevenuePerDay            = round($getLoadGrossRevenue / $getLoadOccupancy, 0);
            $AverageMilePerDay              = round($getLoadMileage / $getLoadOccupancy, 2);
        }

        if($getLoadMileage > 0){
            $CentPerMile                    = round($getLoadGrossRevenue / $getLoadMileage, 2);
        }

        $getDriverTargetRevenue             = getDriverTargetRevenue($startDate,$endDate,$driver_id);
        if($getDriverTargetRevenue > 0){
            $getDriverTargetRevenue         = round(($getLoadGrossRevenue * 100) / $getDriverTargetRevenue, 2);
        }

        $getDriverTargetOccupancy           = getDriverTargetOccupancy($startDate,$endDate,$driver_id);

        $performanceSummary                 = view('companyAdmin.occupancyCalendar.view.OccupancyCalendarPerformanceSummary', compact('showtype', 'getLoadGrossRevenue','getLoadMileage','verageRevenuePerDay','AverageMilePerDay','CentPerMile','getDriverTargetRevenue','getDriverTargetOccupancy'))->render();

        $driver         = Driver::find($driver_id);
        $driver_name    = $driver->last_name.', '.$driver->first_name;
        return response()->json(['performanceSummary' => $performanceSummary, 'driver_name' => $driver_name, 'occupanytype' => $occupanytype,]);
    }
    */
}
