<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;

use App\Models\Bookmark;
use App\Models\CompanyDocumentProfile;

use App\Helpers\Common;

use DataTables;

class OnBoardDocumentProfileController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getField                   = '';
        $getCompanyAdmin            = auth()->guard('company-admin')->user();
        $isCheck                    = Bookmark::where('company_id',$getCompanyAdmin->company_id)
            ->where('user_id',$getCompanyAdmin->id)
            ->where('user_type',$getCompanyAdmin->type)
            ->where('table_id','company_on_board_document_profiles')
            ->first();

        if(!empty($isCheck->id)){
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_company_on_board_document_profiles_first'));
        }else{
            $getField               = config('settings.table_company_on_board_document_profiles_first');
        }

        if ($request->ajax()) {
            $data                   = CompanyDocumentProfile::IsCompany();
            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq       = 'search_string_condition_'.$key;
                $sortReq            = 'search_string_sort_'.$key;

                if( isset( request()->$conditionReq ) ) {
                    $typeReq        = 'search_string_type_'.$key;
                    $type           = request()->$typeReq;
                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }
                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                if( isset( request()->$sortReq ) ) {
                    $nameReq        = 'search_string_name_'.$key;
                    $sortReq        = 'search_string_sort_'.$key;
                    $typeReq        = 'search_string_type_'.$key;
                    $tableNameReq   = 'search_table_name_'.$key;
                    $colAliasReq    = 'search_string_col_alias_'.$key;

                    $origData       = request()->$nameReq;
                    $condition_sort = request()->$sortReq;
                    $type           = request()->$typeReq;
                    $tableName      = isset( request()->$tableNameReq ) ? request()->$tableNameReq : '';
                    $colAlias       = isset( request()->$colAliasReq ) ? request()->$colAliasReq : '';

                    if( $type == 'string' ) {
                        if( isset( $condition_sort ) ) {
                            if( $colAlias != '' ) {
                                $data->orderBy( $tableName.'.'.$colAlias, $condition_sort );
                            } else {
                                if( $tableName != '' ) {
                                    $data->orderBy( $tableName.'.'.$origData, $condition_sort );
                                } else {
                                    $data->orderBy( $origData, $condition_sort );
                                }
                            }
                        }
                    }
                }
            }
            $data->get();
            // General filter code - end

            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $action         = '';
                    if(hasPermission(config('settings.company_admin_modules.on_board_document_profiles'),config('settings.permissions.edit'))){
                        $action    .= '<a href="'.route('company.admin.truck-on-board-document-profiles.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                    }
                    if(hasPermission(config('settings.company_admin_modules.on_board_document_profiles'),config('settings.permissions.delete'))){
                        $action    .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash destroy" data-action="'.route('company.admin.truck-on-board-document-profiles.destroy',$row->id).'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';
                    }
                    return $action;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        $tableName                  = 'settings.table_company_on_board_document_profiles';
        return view('companyAdmin.onBoardDocumentProfile.index', compact('getField','tableName'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return redirect()->route('company.admin.truck-on-board-document-profiles.index');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request){
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id){
        return redirect()->route('company.admin.truck-on-board-document-profiles.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id){
        $profile = CompanyDocumentProfile::findOrFail(base64Decode($id));
        return view('companyAdmin.onBoardDocumentProfile.edit', compact('profile'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id){
        $input                                              = $request->except(['_token','_method']);

        // Custom callback function to filter out null, blank, and empty values
        $filtered_titles                                    = array_filter($input['titles'], function($value) {
            // Filter out null, blank, and empty strings
            return $value !== null && $value !== "" && !empty($value);
        });
        // Reset array keys if needed
        $filtered_titles                                    = array_values($filtered_titles);

        $titles                                             = (!empty($filtered_titles)) ? implode(',', $filtered_titles) : null;

        $input['titles']                                    = $titles;
        $profile                                            = CompanyDocumentProfile::findOrFail($id);
        $profile->update($input);
        notification('success', $this->crudMessage('update', 'Document Profile'));
        return redirect()->route('company.admin.truck-on-board-document-profiles.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(CompanyDocumentProfile $profile){
        $profile->delete();
        notification('success', $this->crudMessage('delete', 'Document Profile'));
        return redirect()->back();
    }
}
