<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Storage;

use App\Models\Bookmark;
use App\Models\CompanyPdfTemplate;
use App\Models\PdfTemplate;

use App\Helpers\Common;

use PDF;
use DataTables;

class PdfTemplateController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getField                   = '';
        $getCompanyAdmin            = auth()->guard('company-admin')->user();
        $isCheck                    = Bookmark::where('company_id',$getCompanyAdmin->company_id)
            ->where('user_id',$getCompanyAdmin->id)
            ->where('user_type',$getCompanyAdmin->type)
            ->where('table_id','pdf_templates')
            ->first();

        if(!empty($isCheck->id)){
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_pdf_templates_first'));
        }else{
            $getField               = config('settings.table_pdf_templates_first');
        }

        if ($request->ajax()) {
            $data                   = CompanyPdfTemplate::IsCompany()->select('*')->orderBy('name', 'ASC');
            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq       = 'search_string_condition_'.$key;
                $sortReq            = 'search_string_sort_'.$key;

                if( isset( request()->$conditionReq ) ) {
                    $typeReq        = 'search_string_type_'.$key;
                    $type           = request()->$typeReq;
                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }
                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                if( isset( request()->$sortReq ) ) {
                    $nameReq        = 'search_string_name_'.$key;
                    $sortReq        = 'search_string_sort_'.$key;
                    $typeReq        = 'search_string_type_'.$key;
                    $tableNameReq   = 'search_table_name_'.$key;
                    $colAliasReq    = 'search_string_col_alias_'.$key;

                    $origData       = request()->$nameReq;
                    $condition_sort = request()->$sortReq;
                    $type           = request()->$typeReq;
                    $tableName      = isset( request()->$tableNameReq ) ? request()->$tableNameReq : '';
                    $colAlias       = isset( request()->$colAliasReq ) ? request()->$colAliasReq : '';

                    if( $type == 'string' ) {
                        if( isset( $condition_sort ) ) {
                            if( $colAlias != '' ) {
                                $data->orderBy( $tableName.'.'.$colAlias, $condition_sort );
                            } else {
                                if( $tableName != '' ) {
                                    $data->orderBy( $tableName.'.'.$origData, $condition_sort );
                                } else {
                                    $data->orderBy( $origData, $condition_sort );
                                }
                            }
                        }
                    }
                }
            }
            $data->get();
            // General filter code - end

            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $action         = '';
                    if(hasPermission(config('settings.company_admin_modules.pdf_templates'),config('settings.permissions.edit'))){
                        $action    .= '<a href="'.route('company.admin.pdf-templates.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                        $action    .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-action revert-templates" data-id="'.$row->id.'"><i class="fa fa-undo" aria-hidden="true"></i></a>';
                    }
                    if(hasPermission(config('settings.company_admin_modules.pdf_templates'),config('settings.permissions.view'))){
                        $action    .= '<a href="'.route('company.admin.pdf-templates.show', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action" target="_blank"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                    }
                    return $action;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        $tableName                  = 'settings.table_pdf_templates';
        return view('companyAdmin.pdfTemplate.index', compact('getField','tableName'));
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id){
        // Template
        $pdf_template                                   = CompanyPdfTemplate::findOrFail(base64Decode($id));

        // Replace the placeholder with the desired value
        $content                                        = $pdf_template->content;

        // Inject CSS for margins and padding directly into the HTML content
        $css                                            = '<style>
            @page { margin: 5mm; }
        </style>';

        $content                                        = $css . '<div class="pdf-container">' . $content . '</div>';

        // Render Laravel view to HTML
        // $html = view('pdf_template', ['content' => $content])->render();

        // Echo the HTML for inspection
        // echo $html;

        // Stop further execution
        // die();

        $pdf                                            = PDF::loadHTML($content)->setPaper('a4');

        // Set options for the PDF (optional)
        // $pdf->setOptions(['dpi' => 150]);

        return $pdf->stream($pdf_template->name.'.pdf');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id){
        $pdf_template               = CompanyPdfTemplate::findOrFail(base64Decode($id));
        return view('companyAdmin.pdfTemplate.edit', compact('pdf_template'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id){
        $input                      = $request->except(['_token','_method']);
        $pdf_template               = CompanyPdfTemplate::findOrFail($id);
        $pdf_template->update($input);
        notification('success', $this->crudMessage('update', 'Pdf Template'));
        return redirect()->route('company.admin.pdf-templates.index');
    }

    public function revert(Request $request){
        $response['success']                        = false;
        $response['message']                        = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                if ($request->has('id') && $request->filled('id')) {
                    $companyPdfTemplate             = CompanyPdfTemplate::whereId($request->id)->first();
                    if(!is_null($companyPdfTemplate)){
                        $name                       = $companyPdfTemplate->name;
                        $pdfTemplate                = PdfTemplate::select('name', 'content', 'tags')->where('name', $name)->first();
                        if(!is_null($pdfTemplate)){
                            $companyPdfTemplate->forceDelete();

                            $companyId              = auth()->guard('company-admin')->user()->company_id;
                            $pdfTemplate            = array_merge(array('company_id' => $companyId), $pdfTemplate->toArray());
                            CompanyPdfTemplate::create($pdfTemplate);

                            $response['success']    = true;
                            $response['message']    = "Pdf Template Revert Successfully.";
                        }
                    }
                } else {
                    $companyId                      = auth()->guard('company-admin')->user()->company_id;
                    CompanyPdfTemplate::where('company_id', $companyId)->forceDelete();

                    // Pdf Template
                    $pdfTemplates                   = PdfTemplate::get(['name', 'content', 'tags'])->toArray();
                    foreach ($pdfTemplates as $pdfTemplate){
                        $pdfTemplate                = array_merge(array('company_id' => $companyId), $pdfTemplate);
                        CompanyPdfTemplate::create($pdfTemplate);
                    }

                    $response['success']            = true;
                    $response['message']            = "All Pdf Templates Revert Successfully.";
                }
            }catch (\Exception $e) {
                $response['success']                = false;
                $response['message']                = $e->getMessage();
            }
        }
        return response()->json($response);
    }
}
