<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;
use App\Http\Requests\CompanyAdmin\ReceiverStoreRequest;
use App\Http\Requests\CompanyAdmin\ReceiverUpdateRequest;
use App\Models\Receiver;
use DataTables;
use Validator;

class ReceiverController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $data = Receiver::isCompany()->select('*');
            return Datatables::of($data)
                    ->addIndexColumn()
                    ->addColumn('action', function($row){
                        //$btn = '<a href="'.route('company.admin.receiver.show', $row->id).'" class="mr-4"><img src="'.asset('theme/img/eye.png').'" alt=""></a>';
                        $btn = ' <a href="'.route('company.admin.receiver.edit', $row->id).'" class="mr-4"><img src="'.asset('theme/img/edit-icon.png').'" alt=""></a>';
                        $btn = $btn.' <a href="javascript:" class="destroy" data-action="'.route('company.admin.receiver.destroy',$row->id).'" data-id="'.$row->id.'"><img src="'.asset('theme/img/delete-icon.png').'" alt=""></a>';
                        return $btn;
                    })
                    ->rawColumns(['action'])
                    ->make(true);
        }

        return view('companyAdmin.receiver.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('companyAdmin.receiver.create');   
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(ReceiverStoreRequest $request)
    {
        $input = $request->all();

        $input['company_id'] = auth()->guard('company-admin')->user()->company_id;

        Receiver::create($input);

        notification('success', $this->crudMessage('create', 'Receiver'));

        return redirect()->route('company.admin.receivers.index');
    }

    public function storeAjax(Request $request)
    {
        $input = $request->all();

        $validator = Validator::make($input,[
            "name" => "required",
            "address" => "required",
            "phone_number" => "required",
            'mc_number' => 'required|unique:shippers,mc_number'
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }

        $input['company_id'] = auth()->guard('company-admin')->user()->company_id;
        Receiver::create($input);

        notification('success', $this->crudMessage('create', 'Receiver'));

        return response()->json(['success' => "Receiver created successfully"]);
    }

    /**
     * Display the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Receiver $receiver)
    {
        return view('companyAdmin.receiver.show', compact('receiver'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Receiver $receiver)
    {
        return view('companyAdmin.receiver.edit', compact('receiver'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(ReceiverUpdateRequest $request, Receiver $receiver)
    {
        $input = $request->all();

        $receiver->update($input);

        notification('success', $this->crudMessage('update', 'Receiver'));

        return redirect()->route('company.admin.receiver.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Receiver $receiver)
    {
        $receiver->delete();

        notification('success', $this->crudMessage('delete', 'Receiver'));

        return redirect()->route('company.admin.receiver.index');
    }

    public function getReceiverList()
    {
        $getList = getReceiverList();

        $html = '<option value="" >Select Receiver</option>';
        foreach($getList as $key => $list){
            $html .= '<option value="'.$key.'" >"'.$list.'"</option>';
        }

        return $html;

    }

    
}
