<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Validation\Validator;
use Illuminate\Validation\ValidationException;

use App\Models\CompanyAdminRole;
use App\Models\CompanyAdminModule;
use App\Models\CompanyAdminPermission;
use App\Models\CompanyAdminModulePermission;
use App\Models\CompanyAdminRoleModulePermission;

use Twilio\Rest\Client;

class RolePermissionController extends CompanyAdminController{

    /***
    *
    *   Description: Display list roles page
    ***/
    public function index(){
        $role = new CompanyAdminRole;
        return view('companyAdmin.roles.index',compact('role'));
    }

   /***
    *
    *   Description: Validate create role request
    ***/
    public function validator($request){
        return $this->validate($request, [
         'name' => 'required|max:50',
      ]);
    }


    /***
    *
    *   Description: Get role list in datatable format to display in list page
    ***/
    public function roleList(){
        $role = CompanyAdminRole::where('company_id',auth()->guard('company-admin')->user()->company_id)->get();
        return datatables()->of($role)
            ->editColumn('status', function ($role) {
                $inactive_url   = url('company-admin/roles/inactive', $role->id);
                $active_url     = url('company-admin/roles/active', $role->id);
                if($role->status=="active"){
                    return '<a class="active" href="'.$inactive_url.'" title="Click here to deactivate" >Active</a>';
                }else{
                    return '<a class="inactive" href="'.$active_url.'" title="Click here to activate" >Inactive</a>';
                }
            })
            ->escapeColumns([])
            ->addColumn('action', function ($role) {
                $action         = '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-action" title="Edit" onClick="editRole('.$role->id.',\''.$role->name.'\',\''.$role->guard_name.'\')"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                $action        .= '<a href="'.url('company-admin/roles/edit-permissions', base64Encode($role->id)).'" class="btn btn-sm btn-outline-black-action" title="Set permissions"><i class="fa fa-check-circle-o" aria-hidden="true"></i></a>';
                $action        .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash destroy" data-action="'.url('company-admin/roles/destroy', $role->id).'" data-id="'.$role->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';
                return $action;
          })
          ->make(true);
    }

    /***
    *
    *   Description: Save role in database
    ***/
    public function storeRole(Request $request){
        $input = $request->except(['_token']);
        if ($this->validator($request)) {
            $input['company_id']    = auth()->guard('company-admin')->user()->company_id;
            $role                   = CompanyAdminRole::create($input);
            notification('success', $this->crudMessage('create', 'Role'));
            return back();
        }
    }


    /***
    *
    *   Description: Save updated role details in database
    ***/
    public function roleUpdate(Request $request){
        if ($this->validator($request)) {
            $input                  = $request->except(['_token']);
            $id                     = $input['roleId'];
            $role                   = CompanyAdminRole::findOrFail($id);
            $input['company_id']    = auth()->guard('company-admin')->user()->company_id;
            $role->update($input);
            notification('success', $this->crudMessage('update', 'Role'));
            return back();
        }
    }

    /***
    *
    *   Description: Delete role
    ***/
    public function roleDestroy($id){
        $role                       = CompanyAdminRole::findOrFail($id);
        $role->delete();
        $notification               = array(
          'message'                 =>  __('messages.roleDeleteSuccess'),
          'alert-type'              => 'success'
        );
        notification('success', $this->crudMessage('delete', 'Role'));
        return back();
    }

    /***
    *
    *   Description: Deactivate role
    ***/
    public function roleInactive($id){
        $role                       = CompanyAdminRole::findOrFail($id);
        $role->status               = 'inactive';
        $role->save();

        $notification               = array(
          'message'                 =>  __('messages.roleDeactivateSuccess'),
          'alert-type'              => 'success'
        );
        notification('success', $this->crudMessage('inactve', 'Role'));
        return back();
    }

    /***
    *
    *   Description: Activate role
    ***/
    public function roleActive($id){
        $role                       = CompanyAdminRole::findOrFail($id);
        $role->status               = 'active';
        $role->save();
        notification('success', $this->crudMessage('active', 'Role'));
        return back();
    }

    /***
    *
    *   Description: Edit permissions for role by id
    ***/
    public function editRolePermissions($id){
        $id                                 = base64Decode($id);
        $usertype                           = auth()->guard('company-admin')->user()->type;
        $this->updatemodule();
        $role                               = CompanyAdminRole::find($id);
        $permissions                        = CompanyAdminPermission::orderBy('order','asc')->get();
        $modules                            = CompanyAdminModule::with(['module_permissions'])->orderBy('id')->get();
        foreach ($modules as $key => $module) {
            if($usertype!=0 && trim($module->name)=='Role And Permission'){ //if(($usertype!=0 && $usertype!=1) && trim($module->name)=='Role And Permission'){
                $modules->forget($key);
            }
            $modulePermissions[$module->id] = $module->module_permissions->pluck('permission_id')->toArray();
          // $modulePermissionIds[$module->id] = $module->module_permissions()->with(['permission'],function($q){
          //   $q->orderBy('order');
          // })->get()->pluck('id')->toArray();
            $modulePermissionIds[$module->id] = CompanyAdminModulePermission::join('company_admin_permissions', function($join) {
                $join->on('company_admin_module_permissions.permission_id', '=', 'company_admin_permissions.id');
            })
            ->where('company_admin_module_permissions.module_id',$module->id)
            ->orderBy('company_admin_permissions.order')->pluck('company_admin_module_permissions.id')->toArray();
        }
        $rolePermissions                    = CompanyAdminRoleModulePermission::where('role_id', $id)->pluck('module_permission_id')->toArray();
        return view('companyAdmin.roles.edit-permissions')->with(compact('role','permissions','modules','rolePermissions','modulePermissionIds','modulePermissions'));
    }

    /***
    *
    *   Description: Edit permissions for role by id
    ***/
    public function updateRolePermissions(Request $request, $id){
        CompanyAdminRoleModulePermission::where('role_id', $id)->delete();
        if(isset($request->module_permissions)){
            foreach($request->module_permissions as $permissionId){
                $permissionData['role_id'] = $id;
                $permissionData['module_permission_id'] = $permissionId;
                CompanyAdminRoleModulePermission::create($permissionData);
            }
        }
        $role = CompanyAdminRole::find($id);
        notification('success', $this->crudMessage('update', 'Role'));
        return back();
    }

    public function updatemodule(){
        $getdata                    = CompanyAdminModule::whereDate('created_at', '=', now())->first();
        $getdata                    = '';
        if(empty($getdata->id)){
            CompanyAdminModule::truncate();
            CompanyAdminPermission::truncate();
            CompanyAdminModulePermission::truncate();
            $camodule_array         = [
                'Dispatch Hub',
                'Loads',
                'Teams',
                'Weekly Fuel Surcharges',
                'Brokers',
                'Managers',
                'Dispatchers',
                'Drivers (Active, OOS, Rehire)',
                'Role And Permission',
                'Customers',
                'Carriers',
                'Trailers',
                'Trucks',
                'Manage Type',
                'Leave Type',
                'Company Holiday',
                'Leaves',
                'Leave User Types',
                'Vendor',
                'Notifications Status',
                'Terminals',
                'Company Dashboard',
                'Request Document',
                'Violation Codes',
                'Maintenance',
                'Maintenance Ticket Types',
                'Driver Quarterly Goals',
                'Load Reservations',
                'Dispatch Pro Announcement',
                'Announcement',
                'Detention Request',
                'Occupancy Reports',
                'Currently Onboarding',
                'Trailer Types',
                'Accessorial',
                'Commodity',
                'Terms',
                'Load Requirements Items',
                'Dispatcher- Revenue Tracker',
                'Chart Of Accounts',
                'Compliance',
                'ProRoutes',
                'Truck Owners',
                'Fuel Surcharges',
                'Truck Types',
                'Work Order Documents',
                'Safety And Compliance Documents',
                'Onboarding Documents',
                'Deduction Names',
                'Load Document',
                'Recruiting Source',
                'Insurance Types',
                'Truck Insurance Profiles',
                'Truck PM Maintenance & Service Profiles',
                'PM Types',
                'PM Service Types',
                'PM Warranty Providers',
                'Equipment Titles',
                'Truck Equipment Profiles',
                'Document Titles',
                'Truck On Board Document Profiles',
                'Drop Locations',
                'Usage',
                'Email Templates',
                'Default Fuel Surcharge Diesel & Gas',
                'Drivers (Terminated)',
                'Billing Dashboard',
                'Trailers Groups',
                'Truck Groups',
                'Trailer Insurance Profiles',
                'Trailer PM Maintenance & Service Profiles',
                'Trailer Equipment Profiles',
                'Trailer On Board Document Profiles',
                'Trailer Warranty Profiles',
                'Truck Warranty Profiles',
                'Pdf Templates',
            ];
            foreach($camodule_array as $camodulearray){
                $camodule['name']   = $camodulearray;
                CompanyAdminModule::create($camodule);
            }

            $capermission_array     = [
                ['name' => 'Create','order' => 2],
                ['name' => 'Edit','order' => 3],
                ['name' => 'Delete','order' => 4],
                ['name' => 'View','order' => 1],
                ['name' => 'Approve','order' => 5],
                ['name' => 'Verify','order' => 6],
            ];

            foreach($capermission_array as $capermissionarray){
                $capermission['name']   = $capermissionarray['name'];
                $capermission['order']  = $capermissionarray['order'];
                CompanyAdminPermission::create($capermission);
            }

            $camodule_permissions_array = [
                ['module_id' => 1,  'permission_id' => 4], // Dispatch Hub ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)

                ['module_id' => 2,  'permission_id' => 1], // Loads ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 2,  'permission_id' => 2], // Loads
                ['module_id' => 2,  'permission_id' => 3], // Loads
                ['module_id' => 2,  'permission_id' => 4], // Loads

                ['module_id' => 3,  'permission_id' => 1], // Teams ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 3,  'permission_id' => 2], // Teams
                ['module_id' => 3,  'permission_id' => 3], // Teams
                ['module_id' => 3,  'permission_id' => 4], // Teams

                ['module_id' => 4,  'permission_id' => 1], // Weekly Fuel Surcharges ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 4,  'permission_id' => 2], // Weekly Fuel Surcharges
                ['module_id' => 4,  'permission_id' => 3], // Weekly Fuel Surcharges
                ['module_id' => 4,  'permission_id' => 4], // Weekly Fuel Surcharges

                ['module_id' => 5,  'permission_id' => 1], // Brokers ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 5,  'permission_id' => 2], // Brokers
                ['module_id' => 5,  'permission_id' => 3], // Brokers
                ['module_id' => 5,  'permission_id' => 4], // Brokers

                ['module_id' => 6,  'permission_id' => 1], // Managers ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 6,  'permission_id' => 2], // Managers
                ['module_id' => 6,  'permission_id' => 3], // Managers
                ['module_id' => 6,  'permission_id' => 4], // Managers

                ['module_id' => 7,  'permission_id' => 1], // Dispatchers ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 7,  'permission_id' => 2], // Dispatchers
                ['module_id' => 7,  'permission_id' => 3], // Dispatchers
                ['module_id' => 7,  'permission_id' => 4], // Dispatchers

                ['module_id' => 8,  'permission_id' => 1], // Drivers (Active, OOS, Rehire) ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 8,  'permission_id' => 2], // Drivers (Active, OOS, Rehire)
                ['module_id' => 8,  'permission_id' => 3], // Drivers (Active, OOS, Rehire)
                ['module_id' => 8,  'permission_id' => 4], // Drivers (Active, OOS, Rehire)
                ['module_id' => 8,  'permission_id' => 6], // Drivers (Active, OOS, Rehire)

                ['module_id' => 9,  'permission_id' => 1], // Role And Permission ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 9,  'permission_id' => 2], // Role And Permission
                ['module_id' => 9,  'permission_id' => 3], // Role And Permission
                ['module_id' => 9,  'permission_id' => 4], // Role And Permission

                ['module_id' => 10, 'permission_id' => 1], // Customers ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 10, 'permission_id' => 2], // Customers
                ['module_id' => 10, 'permission_id' => 3], // Customers
                ['module_id' => 10, 'permission_id' => 4], // Customers

                ['module_id' => 11, 'permission_id' => 1], // Carriers ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 11, 'permission_id' => 2], // Carriers
                ['module_id' => 11, 'permission_id' => 3], // Carriers
                ['module_id' => 11, 'permission_id' => 4], // Carriers

                ['module_id' => 12, 'permission_id' => 1], // Trailers ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 12, 'permission_id' => 2], // Trailers
                ['module_id' => 12, 'permission_id' => 3], // Trailers
                ['module_id' => 12, 'permission_id' => 4], // Trailers
                ['module_id' => 12, 'permission_id' => 6], // Trailers

                ['module_id' => 13, 'permission_id' => 1], // Trucks ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 13, 'permission_id' => 2], // Trucks
                ['module_id' => 13, 'permission_id' => 3], // Trucks
                ['module_id' => 13, 'permission_id' => 4], // Trucks
                ['module_id' => 13, 'permission_id' => 6], // Trucks

                ['module_id' => 14, 'permission_id' => 1], // Manage Type ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 14, 'permission_id' => 2], // Manage Type
                ['module_id' => 14, 'permission_id' => 3], // Manage Type
                ['module_id' => 14, 'permission_id' => 4], // Manage Type

                ['module_id' => 15, 'permission_id' => 1], // Leave Type ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 15, 'permission_id' => 2], // Leave Type
                ['module_id' => 15, 'permission_id' => 3], // Leave Type
                ['module_id' => 15, 'permission_id' => 4], // Leave Type

                ['module_id' => 16, 'permission_id' => 1], // Company Holiday ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 16, 'permission_id' => 2], // Company Holiday
                ['module_id' => 16, 'permission_id' => 3], // Company Holiday
                ['module_id' => 16, 'permission_id' => 4], // Company Holiday

                ['module_id' => 17, 'permission_id' => 1], // Leaves ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 17, 'permission_id' => 3], // Leaves
                ['module_id' => 17, 'permission_id' => 4], // Leaves
                ['module_id' => 17, 'permission_id' => 5], // Leaves

                ['module_id' => 18, 'permission_id' => 2], // Leave User Types ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)

                ['module_id' => 19, 'permission_id' => 1], // Vendor ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 19, 'permission_id' => 2], // Vendor
                ['module_id' => 19, 'permission_id' => 3], // Vendor
                ['module_id' => 19, 'permission_id' => 4], // Vendor

                ['module_id' => 20, 'permission_id' => 4], // Notifications Status ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 20, 'permission_id' => 2], // Notifications Status

                ['module_id' => 21, 'permission_id' => 1], // Terminals ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 21, 'permission_id' => 2], // Terminals
                ['module_id' => 21, 'permission_id' => 3], // Terminals
                ['module_id' => 21, 'permission_id' => 4], // Terminals

                ['module_id' => 22, 'permission_id' => 4], // Company Dashboard ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)

                ['module_id' => 23, 'permission_id' => 1], // Request Document ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 23, 'permission_id' => 2], // Request Document
                ['module_id' => 23, 'permission_id' => 3], // Request Document
                ['module_id' => 23, 'permission_id' => 4], // Request Document

                ['module_id' => 24, 'permission_id' => 1], // Violation Codes ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 24, 'permission_id' => 2], // Violation Codes
                ['module_id' => 24, 'permission_id' => 3], // Violation Codes
                ['module_id' => 24, 'permission_id' => 4], // Violation Codes

                ['module_id' => 25, 'permission_id' => 1], // Maintenance ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 25, 'permission_id' => 2], // Maintenance
                ['module_id' => 25, 'permission_id' => 3], // Maintenance
                ['module_id' => 25, 'permission_id' => 4], // Maintenance

                ['module_id' => 26, 'permission_id' => 1], // Maintenance Ticket Types ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 26, 'permission_id' => 2], // Maintenance Ticket Types
                ['module_id' => 26, 'permission_id' => 3], // Maintenance Ticket Types
                ['module_id' => 26, 'permission_id' => 4], // Maintenance Ticket Types

                ['module_id' => 27, 'permission_id' => 1], // Driver Quarterly Goals ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 27, 'permission_id' => 2], // Driver Quarterly Goals
                ['module_id' => 27, 'permission_id' => 3], // Driver Quarterly Goals
                ['module_id' => 27, 'permission_id' => 4], // Driver Quarterly Goals

                ['module_id' => 28, 'permission_id' => 1], // Load Reservations ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 28, 'permission_id' => 2], // Load Reservations
                ['module_id' => 28, 'permission_id' => 3], // Load Reservations
                ['module_id' => 28, 'permission_id' => 4], // Load Reservations

                ['module_id' => 29, 'permission_id' => 1], // Dispatch Pro Announcement ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 29, 'permission_id' => 2], // Dispatch Pro Announcement
                ['module_id' => 29, 'permission_id' => 3], // Dispatch Pro Announcement
                ['module_id' => 29, 'permission_id' => 4], // Dispatch Pro Announcement

                ['module_id' => 30, 'permission_id' => 1], // Announcement ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 30, 'permission_id' => 2], // Announcement
                ['module_id' => 30, 'permission_id' => 3], // Announcement
                ['module_id' => 30, 'permission_id' => 4], // Announcement

                ['module_id' => 31, 'permission_id' => 1], // Detention Request ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 31, 'permission_id' => 2], // Detention Request
                ['module_id' => 31, 'permission_id' => 3], // Detention Request
                ['module_id' => 31, 'permission_id' => 4], // Detention Request

                ['module_id' => 32, 'permission_id' => 4], // Occupancy Reports ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)

                ['module_id' => 33, 'permission_id' => 1], // Currently Onboarding ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 33, 'permission_id' => 2], // Currently Onboarding
                ['module_id' => 33, 'permission_id' => 3], // Currently Onboarding
                ['module_id' => 33, 'permission_id' => 4], // Currently Onboarding

                ['module_id' => 34, 'permission_id' => 1], // Trailer Types ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 34, 'permission_id' => 2], // Trailer Types
                ['module_id' => 34, 'permission_id' => 3], // Trailer Types
                ['module_id' => 34, 'permission_id' => 4], // Trailer Types

                ['module_id' => 35, 'permission_id' => 1], // Accessorial ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 35, 'permission_id' => 2], // Accessorial
                ['module_id' => 35, 'permission_id' => 3], // Accessorial
                ['module_id' => 35, 'permission_id' => 4], // Accessorial

                ['module_id' => 36, 'permission_id' => 1], // Commodity ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 36, 'permission_id' => 2], // Commodity
                ['module_id' => 36, 'permission_id' => 3], // Commodity
                ['module_id' => 36, 'permission_id' => 4], // Commodity

                ['module_id' => 37, 'permission_id' => 1], // Terms ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 37, 'permission_id' => 2], // Terms
                ['module_id' => 37, 'permission_id' => 3], // Terms
                ['module_id' => 37, 'permission_id' => 4], // Terms

                ['module_id' => 38, 'permission_id' => 1], // Load Requirements Items ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 38, 'permission_id' => 2], // Load Requirements Items
                ['module_id' => 38, 'permission_id' => 3], // Load Requirements Items
                ['module_id' => 38, 'permission_id' => 4], // Load Requirements Items

                ['module_id' => 39, 'permission_id' => 4], // Dispatcher- Revenue Tracker
                ['module_id' => 39, 'permission_id' => 4], // Dispatcher- Revenue Tracker

                ['module_id' => 40, 'permission_id' => 1], // Chart Of Account ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 40, 'permission_id' => 2], // Chart Of Account
                ['module_id' => 40, 'permission_id' => 3], // Chart Of Account
                ['module_id' => 40, 'permission_id' => 4], // Chart Of Account

                ['module_id' => 41, 'permission_id' => 4], // Compliance ('view' => 4)

                ['module_id' => 42, 'permission_id' => 4], // ProRoutes ('view' => 4)

                ['module_id' => 43, 'permission_id' => 1], // Truck owners ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 43, 'permission_id' => 2], // Truck owners
                ['module_id' => 43, 'permission_id' => 3], // Truck owners
                ['module_id' => 43, 'permission_id' => 4], // Truck owners

                ['module_id' => 44, 'permission_id' => 4], // Fuel Surcharges ('view' => 4)

                ['module_id' => 45, 'permission_id' => 1], // Truck Types ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 45, 'permission_id' => 2], // Truck Types
                ['module_id' => 45, 'permission_id' => 3], // Truck Types
                ['module_id' => 45, 'permission_id' => 4], // Truck Types

                ['module_id' => 46, 'permission_id' => 1], // Work Order Documents ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 46, 'permission_id' => 2], // Work Order Documents
                ['module_id' => 46, 'permission_id' => 3], // Work Order Documents
                ['module_id' => 46, 'permission_id' => 4], // Work Order Documents

                ['module_id' => 47, 'permission_id' => 1], // Safety And Compliance Documents ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 47, 'permission_id' => 2], // Safety And Compliance Documents
                ['module_id' => 47, 'permission_id' => 3], // Safety And Compliance Documents
                ['module_id' => 47, 'permission_id' => 4], // Safety And Compliance Documents

                ['module_id' => 48, 'permission_id' => 1], // Onboarding Documents ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 48, 'permission_id' => 2], // Onboarding Documents
                ['module_id' => 48, 'permission_id' => 3], // Onboarding Documents
                ['module_id' => 48, 'permission_id' => 4], // Onboarding Documents

                ['module_id' => 49, 'permission_id' => 1], // Deduction Names ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 49, 'permission_id' => 2], // Deduction Names
                ['module_id' => 49, 'permission_id' => 3], // Deduction Names
                ['module_id' => 49, 'permission_id' => 4], // Deduction Names

                ['module_id' => 50, 'permission_id' => 1], // Load document ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4,'approve' => 5, 'verify' => 6)
                ['module_id' => 50, 'permission_id' => 2], // Load document
                ['module_id' => 50, 'permission_id' => 3], // Load document
                ['module_id' => 50, 'permission_id' => 4], // Load document

                ['module_id' => 51, 'permission_id' => 1], // Recruiting Source ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 51, 'permission_id' => 2], // Recruiting Source
                ['module_id' => 51, 'permission_id' => 3], // Recruiting Source
                ['module_id' => 51, 'permission_id' => 4], // Recruiting Source

                ['module_id' => 52, 'permission_id' => 1], // Insurance Types ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 52, 'permission_id' => 2], // Insurance Types
                ['module_id' => 52, 'permission_id' => 3], // Insurance Types
                ['module_id' => 52, 'permission_id' => 4], // Insurance Types

                ['module_id' => 53, 'permission_id' => 1], // Truck Insurance Profiles ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 53, 'permission_id' => 2], // Truck Insurance Profiles
                ['module_id' => 53, 'permission_id' => 3], // Truck Insurance Profiles
                ['module_id' => 53, 'permission_id' => 4], // Truck Insurance Profiles

                ['module_id' => 54, 'permission_id' => 1], // Truck PM Maintenance & Service Profiles ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 54, 'permission_id' => 2], // Truck PM Maintenance & Service Profiles
                ['module_id' => 54, 'permission_id' => 3], // Truck PM Maintenance & Service Profiles
                ['module_id' => 54, 'permission_id' => 4], // Truck PM Maintenance & Service Profiles

                ['module_id' => 55, 'permission_id' => 1], // PM Types ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 55, 'permission_id' => 2], // PM Types
                ['module_id' => 55, 'permission_id' => 3], // PM Types
                ['module_id' => 55, 'permission_id' => 4], // PM Types

                ['module_id' => 56, 'permission_id' => 1], // PM Service Types ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 56, 'permission_id' => 2], // PM Service Types
                ['module_id' => 56, 'permission_id' => 3], // PM Service Types
                ['module_id' => 56, 'permission_id' => 4], // PM Service Types

                ['module_id' => 57, 'permission_id' => 1], // PM Warranty Providers ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 57, 'permission_id' => 2], // PM Warranty Providers
                ['module_id' => 57, 'permission_id' => 3], // PM Warranty Providers
                ['module_id' => 57, 'permission_id' => 4], // PM Warranty Providers

                ['module_id' => 58, 'permission_id' => 1], // Equipment Titles ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 58, 'permission_id' => 2], // Equipment Titles
                ['module_id' => 58, 'permission_id' => 3], // Equipment Titles
                ['module_id' => 58, 'permission_id' => 4], // Equipment Titles

                ['module_id' => 59, 'permission_id' => 1], // Truck Equipment Profiles ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 59, 'permission_id' => 2], // Truck Equipment Profiles
                ['module_id' => 59, 'permission_id' => 3], // Truck Equipment Profiles
                ['module_id' => 59, 'permission_id' => 4], // Truck Equipment Profiles

                ['module_id' => 60, 'permission_id' => 1], // Document Titles ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 60, 'permission_id' => 2], // Document Titles
                ['module_id' => 60, 'permission_id' => 3], // Document Titles
                ['module_id' => 60, 'permission_id' => 4], // Document Titles

                ['module_id' => 61, 'permission_id' => 1], // Truck On Board Document Profiles ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 61, 'permission_id' => 2], // Truck On Board Document Profiles
                ['module_id' => 61, 'permission_id' => 3], // Truck On Board Document Profiles
                ['module_id' => 61, 'permission_id' => 4], // Truck On Board Document Profiles

                ['module_id' => 62, 'permission_id' => 1], // Drop Locations ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 62, 'permission_id' => 2], // Drop Locations
                ['module_id' => 62, 'permission_id' => 3], // Drop Locations
                ['module_id' => 62, 'permission_id' => 4], // Drop Locations

                ['module_id' => 63,  'permission_id' => 4], // Usage Hub ('view' => 4)

                ['module_id' => 64,  'permission_id' => 2], // Email Templates ('edit' => 2, 'view' => 4)
                ['module_id' => 64,  'permission_id' => 4], // Email Templates

                ['module_id' => 65,  'permission_id' => 1], // Default Fuel Surcharge Diesel & Gas ('create' => 1, 'edit' => 2)
                ['module_id' => 65,  'permission_id' => 2], // Default Fuel Surcharge Diesel & Gas

                ['module_id' => 66,  'permission_id' => 2], // Drivers (Terminated) ('edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 66,  'permission_id' => 3], // Drivers (Terminated)
                ['module_id' => 66,  'permission_id' => 4], // Drivers (Terminated)

                ['module_id' => 67,  'permission_id' => 4], // Billing Dashboard ('view' => 4)

                ['module_id' => 68, 'permission_id' => 1], // Trailers Groups ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 68, 'permission_id' => 2], // Trailers Groups
                ['module_id' => 68, 'permission_id' => 3], // Trailers Groups
                ['module_id' => 68, 'permission_id' => 4], // Trailers Groups

                ['module_id' => 69, 'permission_id' => 1], // Truck Groups ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 69, 'permission_id' => 2], // Truck Groups
                ['module_id' => 69, 'permission_id' => 3], // Truck Groups
                ['module_id' => 69, 'permission_id' => 4], // Truck Groups

                ['module_id' => 70, 'permission_id' => 1], // Trailer Insurance Profiles ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 70, 'permission_id' => 2], // Trailer Insurance Profiles
                ['module_id' => 70, 'permission_id' => 3], // Trailer Insurance Profiles
                ['module_id' => 70, 'permission_id' => 4], // Trailer Insurance Profiles

                ['module_id' => 71, 'permission_id' => 1], // Trailer PM Maintenance & Service Profiles ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 71, 'permission_id' => 2], // Trailer PM Maintenance & Service Profiles
                ['module_id' => 71, 'permission_id' => 3], // Trailer PM Maintenance & Service Profiles
                ['module_id' => 71, 'permission_id' => 4], // Trailer PM Maintenance & Service Profiles

                ['module_id' => 72, 'permission_id' => 1], // Trailer Equipment Profiles ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 72, 'permission_id' => 2], // Trailer Equipment Profiles
                ['module_id' => 72, 'permission_id' => 3], // Trailer Equipment Profiles
                ['module_id' => 72, 'permission_id' => 4], // Trailer Equipment Profiles

                ['module_id' => 73, 'permission_id' => 1], // Trailer On Board Document Profiles ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 73, 'permission_id' => 2], // Trailer On Board Document Profiles
                ['module_id' => 73, 'permission_id' => 3], // Trailer On Board Document Profiles
                ['module_id' => 73, 'permission_id' => 4], // Trailer On Board Document Profiles

                ['module_id' => 74, 'permission_id' => 1], // Trailer Warranty Profiles ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 74, 'permission_id' => 2], // Trailer Warranty Profiles
                ['module_id' => 74, 'permission_id' => 3], // Trailer Warranty Profiles
                ['module_id' => 74, 'permission_id' => 4], // Trailer Warranty Profiles

                ['module_id' => 75, 'permission_id' => 1], // Truck Warranty Profiles ('create' => 1, 'edit' => 2, 'delete' => 3, 'view' => 4)
                ['module_id' => 75, 'permission_id' => 2], // Truck Warranty Profiles
                ['module_id' => 75, 'permission_id' => 3], // Truck Warranty Profiles
                ['module_id' => 75, 'permission_id' => 4], // Truck Warranty Profiles

                ['module_id' => 76,  'permission_id' => 2], // Pdf Templates ('edit' => 2, 'view' => 4)
                ['module_id' => 76,  'permission_id' => 4], // Pdf Templates
            ];

            foreach($camodule_permissions_array as $camodulepermissionsarray){
                $camodulepermission['module_id']        = $camodulepermissionsarray['module_id'];
                $camodulepermission['permission_id']    = $camodulepermissionsarray['permission_id'];
                CompanyAdminModulePermission::create($camodulepermission);
            }
        }
    }
}
