<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use App\Models\Company;
use App\Models\IntegrationSetting;
use App\Models\AppCenter;
use App\Models\Driver;
use App\Models\Truck;
use App\Models\Trailer;
use Illuminate\Http\Request;
use File;
use Illuminate\Support\Facades\Log;

class SamsaraController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    public function getSyncDriver($samsara_api_key, $endCursor)
    {
        $data = '';
        $company_id = auth()->guard('company-admin')->user()->company_id;

        if(!empty($samsara_api_key)){
            $client = new \GuzzleHttp\Client(['verify' => false]);
            if(empty($endCursor)){
                $api_url = 'https://api.samsara.com/fleet/drivers?limit=100';
            }else{
                $api_url = 'https://api.samsara.com/fleet/drivers?limit=100&after='.$endCursor;
            }

            $response = $client->request('GET', $api_url, [
                'headers' => [
                    'Accept' => 'application/json',
                    'Authorization' => 'Bearer '.$samsara_api_key,
                ],
            ]);
            $data = json_decode($response->getBody());

            $endCursor = $data->pagination->endCursor;
            $data = $data->data;

            foreach($data as $driver){
                if(isset($driver->licenseNumber)){
                    $getDriver = Driver::where('license_number', base64_encode($driver->licenseNumber))->where('company_id',$company_id)->first();
                    if(!is_null($getDriver)){
                        $driver_data = [];
                        $driver_data['samsara_id'] = $driver->id;
                        $getDriver->update($driver_data);
                        $mest = $driver->licenseNumber.'___'.$driver->id.'___'.$endCursor;
                    }
                }
            }

            if(!empty($endCursor)){
                return redirect()->route('company.admin.samsara.sync.getsyncdriver', ['samsara_api_key' => $samsara_api_key, 'endCursor' => $endCursor]);
            }else{
                notification('success', 'Drivers synchronize successfully.');
                return redirect()->back();
            }
        }
    }

    public function getSyncVehicles($samsara_api_key, $endCursor)
    {
        $data = '';
        $company_id = auth()->guard('company-admin')->user()->company_id;

        if(!empty($samsara_api_key)){
            $client = new \GuzzleHttp\Client(['verify' => false]);
            if(empty($endCursor)){
                $api_url = 'https://api.samsara.com/fleet/vehicles?limit=100';
            }else{
                $api_url = 'https://api.samsara.com/fleet/vehicles?limit=100&after='.$endCursor;
            }


            $response = $client->request('GET', $api_url, [
                'headers' => [
                    'Accept' => 'application/json',
                    'Authorization' => 'Bearer '.$samsara_api_key,
                ],
            ]);
            $data = json_decode($response->getBody());


            $endCursor = $data->pagination->endCursor;
            $data = $data->data;

            foreach($data as $vehicle){
                if(isset($vehicle->vin)){
                    $getVehicle = Truck::where('vin_number', $vehicle->vin)->where('company_id',$company_id)->first();
                    if(!empty($getVehicle->id)){
                        $vehicle_data = [];
                        $vehicle_data['samsara_id'] = $vehicle->id;
                        $getVehicle->update($vehicle_data);

                    }
                }
            }

            if(!empty($endCursor)){
                return redirect()->route('company.admin.samsara.sync.getsyncvehicles', ['samsara_api_key' => $samsara_api_key, 'endCursor' => $endCursor]);
            }else{
                notification('success', 'Vehicles synchronize successfully.');
                return redirect()->back();
            }
        }
    }

    public function getSyncTrailers($samsara_api_key,$endCursor)
    {
        $data = '';
        $company_id = auth()->guard('company-admin')->user()->company_id;

        if(!empty($samsara_api_key)){
            $client = new \GuzzleHttp\Client(['verify' => false]);
            if(empty($endCursor)){
                $api_url = 'https://api.samsara.com/fleet/trailers?limit=100';
            }else{
                $api_url = 'https://api.samsara.com/fleet/trailers?limit=100&after='.$endCursor;
            }

            $response = $client->request('GET', $api_url, [
                'headers' => [
                    'Accept' => 'application/json',
                    'Authorization' => 'Bearer '.$samsara_api_key,
                ],
            ]);
            $data = json_decode($response->getBody());

            $endCursor = $data->pagination->endCursor;
            $data = $data->data;

            foreach($data as $trailer){
                if(isset($trailer->name)){
                    $getTrailer = Trailer::where('trailer_id', $trailer->name)->where('company_id',$company_id)->first();
                    if(!empty($getTrailer->id)){
                        $trailer_data = [];
                        $trailer_data['samsara_id'] = $trailer->id;
                        $getTrailer->update($trailer_data);
                    }
                }
            }

            if(!empty($endCursor)){
                return redirect()->route('company.admin.samsara.sync.getsynctrailers', ['samsara_api_key' => $samsara_api_key, 'endCursor' => $endCursor]);
            }else{
                notification('success', 'Trailers synchronize successfully.');
                return redirect()->back();
            }
        }
    }

    public function syncDriverByAjax($samsara_api_key, $endCursor)
    {
        $data = '';
        $company_id = auth()->guard('company-admin')->user()->company_id;

        if(!empty($samsara_api_key)){
            $client = new \GuzzleHttp\Client(['verify' => false]);
            if(empty($endCursor)){
                $api_url = 'https://api.samsara.com/fleet/drivers?limit=100';
            }else{
                $api_url = 'https://api.samsara.com/fleet/drivers?limit=100&after='.$endCursor;
            }

            $response = $client->request('GET', $api_url, [
                'headers' => [
                    'Accept' => 'application/json',
                    'Authorization' => 'Bearer '.$samsara_api_key,
                ],
            ]);
            $data = json_decode($response->getBody());

            $endCursor = $data->pagination->endCursor;
            $data = $data->data;

            foreach($data as $driver){
                if(isset($driver->licenseNumber)){
                    $getDriver = Driver::where('license_number', base64_encode($driver->licenseNumber))->where('company_id',$company_id)->first();
                    if(!is_null($getDriver)){
                        $driver_data = [];
                        $driver_data['samsara_id'] = $driver->id;
                        $getDriver->update($driver_data);
                        $mest = $driver->licenseNumber.'___'.$driver->id.'___'.$endCursor;
                    }
                }
            }

            if(!empty($endCursor)){
                return $this->syncDriverByAjax($samsara_api_key, $endCursor);
            }else{
                return response()->json(['success' => 'true']);
            }
        }
    }

    public function syncVehiclesByAjax($samsara_api_key, $endCursor)
    {
        $data = '';
        $company_id = auth()->guard('company-admin')->user()->company_id;

        if(!empty($samsara_api_key)){
            $client = new \GuzzleHttp\Client(['verify' => false]);
            if(empty($endCursor)){
                $api_url = 'https://api.samsara.com/fleet/vehicles?limit=100';
            }else{
                $api_url = 'https://api.samsara.com/fleet/vehicles?limit=100&after='.$endCursor;
            }

            $response = $client->request('GET', $api_url, [
                'headers' => [
                    'Accept' => 'application/json',
                    'Authorization' => 'Bearer '.$samsara_api_key,
                ],
            ]);
            $data = json_decode($response->getBody());

            $endCursor = $data->pagination->endCursor;
            $data = $data->data;

            foreach($data as $vehicle){
                if(isset($vehicle->vin)){
                    $getVehicle = Truck::where('vin_number', $vehicle->vin)->where('company_id',$company_id)->first();
                    if(!empty($getVehicle->id)){
                        $vehicle_data = [];
                        $vehicle_data['samsara_id'] = $vehicle->id;
                        $getVehicle->update($vehicle_data);

                    }
                }
            }

            if(!empty($endCursor)){
                return $this->syncVehiclesByAjax($samsara_api_key, $endCursor);
            }else{
                return response()->json(['success' => 'true']);
            }
        }
    }

    public function syncTrailersByAjax($samsara_api_key, $endCursor)
    {
        $data = '';
        $company_id = auth()->guard('company-admin')->user()->company_id;

        if(!empty($samsara_api_key)){
            $client = new \GuzzleHttp\Client(['verify' => false]);
            if(empty($endCursor)){
                $api_url = 'https://api.samsara.com/fleet/trailers?limit=100';
            }else{
                $api_url = 'https://api.samsara.com/fleet/trailers?limit=100&after='.$endCursor;
            }

            $response = $client->request('GET', $api_url, [
                'headers' => [
                    'Accept' => 'application/json',
                    'Authorization' => 'Bearer '.$samsara_api_key,
                ],
            ]);
            $data = json_decode($response->getBody());

            $endCursor = $data->pagination->endCursor;
            $data = $data->data;

            foreach($data as $trailer){
                if(isset($trailer->name)){
                    $getTrailer = Trailer::where('trailer_id', $trailer->name)->where('company_id',$company_id)->first();
                    if(!empty($getTrailer->id)){
                        $trailer_data = [];
                        $trailer_data['samsara_id'] = $trailer->id;
                        $getTrailer->update($trailer_data);
                    }
                }
            }

            if(!empty($endCursor)){
                return $this->syncTrailersByAjax($samsara_api_key, $endCursor);
            }else{
                return response()->json(['success' => 'true']);
            }
        }
    }

    public function syncDriver(Request $request)
    {
        $input = $request->all();
        if(!empty($request->samsara_api_key)){
            $client = new \GuzzleHttp\Client(['verify' => false]);

            $api_url = 'https://api.samsara.com/fleet/drivers?limit=20&after='.$request->endCursor;

            $response = $client->request('GET', $api_url, [
              'headers' => [
                'Accept' => 'application/json',
                'Authorization' => 'Bearer '.$request->samsara_api_key,
              ],
            ]);
            $data = json_decode($response->getBody());

            $endCursor = $data->pagination->endCursor;
            $data = $data->data;
        }

        $view = view('companyAdmin.integrationSettings.ajax.syncDriver', compact('data'))->render();

        return response()->json(['view' => $view,'endCursor' => $endCursor]);

    }

    public function syncVehicles(Request $request)
    {
        $input = $request->all();

        if(!empty($request->samsara_api_key)){
            $client = new \GuzzleHttp\Client(['verify' => false]);

            $api_url = 'https://api.samsara.com/fleet/vehicles?limit=20&after='.$request->endCursorVehicles;

            $response = $client->request('GET', $api_url, [
              'headers' => [
                'Accept' => 'application/json',
                'Authorization' => 'Bearer '.$request->samsara_api_key,
              ],
            ]);

            $data = json_decode($response->getBody());
            $endCursorVehicles = $data->pagination->endCursor;
            $data = $data->data;
        }

        $view = view('companyAdmin.integrationSettings.ajax.syncVehicles', compact('data'))->render();
        return response()->json(['view' => $view,'endCursorVehicles' => $endCursorVehicles]);
    }

    public function syncTrailers(Request $request)
    {
        $input = $request->all();

        if(!empty($request->samsara_api_key)){
            $client = new \GuzzleHttp\Client(['verify' => false]);

            $api_url = 'https://api.samsara.com/fleet/trailers?limit=20&after='.$request->endCursorTrailers;

            $response = $client->request('GET', $api_url, [
              'headers' => [
                'Accept' => 'application/json',
                'Authorization' => 'Bearer '.$request->samsara_api_key,
              ],
            ]);

            $data = json_decode($response->getBody());
            $endCursorTrailers = $data->pagination->endCursor;
            $data = $data->data;
        }

        $view = view('companyAdmin.integrationSettings.ajax.syncTrailers', compact('data'))->render();
        return response()->json(['view' => $view,'endCursorTrailers' => $endCursorTrailers]);
    }

    public function getSyncChecking(){

        $getCompany = Company::get();

        foreach($getCompany as $company){
            $integration = IntegrationSetting::where('app_center_id', config('settings.app_id.samsara'))->where('company_id',$company->id)->whereNotNull('api_key')->first();
            if(!empty($integration->api_key)){
                Log::notice('test');
                $getDriver = Driver::where('company_id',$company->id)->whereNotNull('samsara_id')->get();
                foreach ($getDriver as $driver) {

                    $api_url = 'https://api.samsara.com/fleet/drivers/'.$driver->samsara_id;

                    $client = new \GuzzleHttp\Client(['verify' => false]);
                    $response = $client->request('GET', $api_url, [
                      'headers' => [
                        'Accept' => 'application/json',
                        'Authorization' => 'Bearer '.$integration->api_key,
                      ],
                    ]);

                    $data = json_decode($response->getBody());
                    $getStatus = $data->data->driverActivationStatus;
                    if($getStatus == 'active'){
                        $getEditDriver = Driver::find($driver->id);
                        $edit_array = [];
                        $edit_array['samsara_id'] = null;
                        $getEditDriver->update($edit_array);
                    }

                }
            }

        }


    }
}
