<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;

use App\Models\Bookmark;
use App\Models\CompanyTrailerGroup;

use App\Helpers\Common;

use Validator;
use DataTables;

class TrailerGroupController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getField                   = '';
        $getCompanyAdmin            = auth()->guard('company-admin')->user();
        $isCheck                    = Bookmark::where('company_id',$getCompanyAdmin->company_id)
            ->where('user_id',$getCompanyAdmin->id)
            ->where('user_type',$getCompanyAdmin->type)
            ->where('table_id','company_trailer_groups')
            ->first();

        if(!empty($isCheck->id)){
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_company_trailer_groups_first'));
        }else{
            $getField               = config('settings.table_company_trailer_groups_first');
        }

        if ($request->ajax()) {
            $data                   = CompanyTrailerGroup::isCompany()->select('id', 'company_id', 'name');
            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq       = 'search_string_condition_'.$key;
                $sortReq            = 'search_string_sort_'.$key;

                if( isset( request()->$conditionReq ) ) {
                    $typeReq        = 'search_string_type_'.$key;
                    $type           = request()->$typeReq;
                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }
                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                if( isset( request()->$sortReq ) ) {
                    $nameReq        = 'search_string_name_'.$key;
                    $sortReq        = 'search_string_sort_'.$key;
                    $typeReq        = 'search_string_type_'.$key;
                    $tableNameReq   = 'search_table_name_'.$key;
                    $colAliasReq    = 'search_string_col_alias_'.$key;

                    $origData       = request()->$nameReq;
                    $condition_sort = request()->$sortReq;
                    $type           = request()->$typeReq;
                    $tableName      = isset( request()->$tableNameReq ) ? request()->$tableNameReq : '';
                    $colAlias       = isset( request()->$colAliasReq ) ? request()->$colAliasReq : '';

                    if( $type == 'string' ) {
                        if( isset( $condition_sort ) ) {
                            if( $colAlias != '' ) {
                                $data->orderBy( $tableName.'.'.$colAlias, $condition_sort );
                            } else {
                                if( $tableName != '' ) {
                                    $data->orderBy( $tableName.'.'.$origData, $condition_sort );
                                } else {
                                    $data->orderBy( $origData, $condition_sort );
                                }
                            }
                        }
                    }
                }
            }
            $data->get();
            // General filter code - end

            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $action         = '';
                    if(hasPermission(config('settings.company_admin_modules.trailer_groups'),config('settings.permissions.edit'))){
                        $action    .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-action edit-btn" data-id="'.$row->id.'" data-toggle="modal" data-target="#editModal"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                    }
                    if(hasPermission(config('settings.company_admin_modules.trailer_groups'),config('settings.permissions.delete'))){
                        $action    .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash destroy" data-action="'.route('company.admin.trailer-groups.destroy', $row->id).'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';
                    }
                    return $action;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        $tableName                  = 'settings.table_company_trailer_groups';
        return view('companyAdmin.trailerGroups.index', compact('getField','tableName'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return redirect()->route('company.admin.trailer-groups.index');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request){
        $input                      = $request->all();
        $validator                  = Validator::make($input,[
            "name"                  => "required",
        ]);
        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }
        $input['company_id']        = auth()->guard('company-admin')->user()->company_id;
        CompanyTrailerGroup::create($input);
        notification('success', $this->crudMessage('create', 'Trailer group'));
        return response()->json(['success' => "Trailer group created successfully"]);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id){
        return redirect()->route('company.admin.trailer-groups.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id){
        return redirect()->route('company.admin.trailer-groups.index');
    }

    public function editFormView(Request $request){
        $trailer_group              = CompanyTrailerGroup::find($request->id);
        $view                       = view('companyAdmin.trailerGroups.editFormView', compact('trailer_group'))->render();
        return response()->json(['view' => $view, 'data' => $trailer_group]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, CompanyTrailerGroup $trailer_group){
        $input                      = $request->all();
        $validator                  = Validator::make($input,[
            "name"                  => "required",
        ]);
        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()]);
        }
        $trailer_group->update($input);
        notification('success', $this->crudMessage('update', 'Trailer group'));
        return response()->json(['success' => "Trailer group updated successfully"]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(CompanyTrailerGroup $trailer_group){
        $trailer_group->delete();
        notification('success', $this->crudMessage('delete', 'Trailer group'));
        return redirect()->back();
    }
}
