<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;

use App\Models\Bookmark;
use App\Models\TrailerCompanyPmServiceProfile;

use App\Helpers\Common;

use DataTables;

class TrailerPmServiceProfileController extends CompanyAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getField                   = '';
        $getCompanyAdmin            = auth()->guard('company-admin')->user();
        $isCheck                    = Bookmark::where('company_id',$getCompanyAdmin->company_id)
            ->where('user_id',$getCompanyAdmin->id)
            ->where('user_type',$getCompanyAdmin->type)
            ->where('table_id','trailer_company_pm_service_profiles')
            ->first();

        if(!empty($isCheck->id)){
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_trailer_company_pm_service_profiles_first'));
        }else{
            $getField               = config('settings.table_trailer_company_pm_service_profiles_first');
        }
        
        if ($request->ajax()) {
            $data                   = TrailerCompanyPmServiceProfile::IsCompany();
            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq       = 'search_string_condition_'.$key;
                $sortReq            = 'search_string_sort_'.$key;

                if( isset( request()->$conditionReq ) ) {
                    $typeReq        = 'search_string_type_'.$key;
                    $type           = request()->$typeReq;
                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }
                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                if( isset( request()->$sortReq ) ) {
                    $nameReq        = 'search_string_name_'.$key;
                    $sortReq        = 'search_string_sort_'.$key;
                    $typeReq        = 'search_string_type_'.$key;
                    $tableNameReq   = 'search_table_name_'.$key;
                    $colAliasReq    = 'search_string_col_alias_'.$key;

                    $origData       = request()->$nameReq;
                    $condition_sort = request()->$sortReq;
                    $type           = request()->$typeReq;
                    $tableName      = isset( request()->$tableNameReq ) ? request()->$tableNameReq : '';
                    $colAlias       = isset( request()->$colAliasReq ) ? request()->$colAliasReq : '';

                    if( $type == 'string' ) {
                        if( isset( $condition_sort ) ) {
                            if( $colAlias != '' ) {
                                $data->orderBy( $tableName.'.'.$colAlias, $condition_sort );
                            } else {
                                if( $tableName != '' ) {
                                    $data->orderBy( $tableName.'.'.$origData, $condition_sort );
                                } else {
                                    $data->orderBy( $origData, $condition_sort );
                                }
                            }
                        }
                    }
                }
            }
            $data->get();
            // General filter code - end

            return DataTables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $action         = '';
                    if(hasPermission(config('settings.company_admin_modules.trailer_pm_service_profiles'),config('settings.permissions.edit'))){
                        $action    .= '<a href="'.route('company.admin.trailer-pm-service-profiles.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                    }
                    if(hasPermission(config('settings.company_admin_modules.trailer_pm_service_profiles'),config('settings.permissions.delete'))){
                        $action    .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash destroy" data-action="'.route('company.admin.trailer-pm-service-profiles.destroy',$row->id).'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';
                    }
                    return $action;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        $tableName                  = 'settings.table_trailer_company_pm_service_profiles';
        return view('companyAdmin.trailer.profiles.pm_service.index', compact('getField','tableName'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return redirect()->route('company.admin.trailer-pm-service-profiles.index');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request){
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id){
        return redirect()->route('company.admin.trailer-pm-service-profiles.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id){
        $profile = TrailerCompanyPmServiceProfile::findOrFail(base64Decode($id));
        return view('companyAdmin.trailer.profiles.pm_service.edit', compact('profile'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id){
        $input                                              = $request->except(['_token','_method']);
        $input['types']                                     = serialize($request->types);
        $input['service_types']                             = serialize($request->service_type);
        $input['interval_types']                            = serialize($request->interval_type);
        $input['schedules']                                 = serialize($request->schedule);
        $profile                                            = TrailerCompanyPmServiceProfile::findOrFail($id);
        $profile->update($input);
        notification('success', $this->crudMessage('update', 'PM/Service Profile'));
        return redirect()->route('company.admin.trailer-pm-service-profiles.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(TrailerCompanyPmServiceProfile $profile){
        $profile->delete();
        notification('success', $this->crudMessage('delete', 'PM/Service Profile'));
        return redirect()->back();
    }
}
