<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;

use App\Models\TruckOwner;
use App\Models\Bookmark;

use App\Helpers\Common;

use Validator;
use DataTables;
use Carbon\Carbon;

class TruckOwnerController extends CompanyAdminController{

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(){
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getCompanyAdmin            = auth()->guard('company-admin')->user();

        $getField                   = '';
        $isCheck                    = Bookmark::where('company_id', $getCompanyAdmin->company_id)
            ->where('user_id', $getCompanyAdmin->id)
            ->where('user_type', $getCompanyAdmin->type)
            ->where('table_id', 'truck_owners')
            ->first();
        if(!empty($isCheck->id)){
            //$getField               = unserialize($isCheck->config);
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_truck_owners_first'));
        }else{
            $getField               = config('settings.table_truck_owners_first');
        }
        if ($request->ajax()) {
            $data                   = TruckOwner::select('*')->where('company_id', $getCompanyAdmin->company_id);
            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq       = 'search_string_condition_'.$key;
                $sortReq            = 'search_string_sort_'.$key;

                if( isset( request()->$conditionReq ) ) {
                    $typeReq        = 'search_string_type_'.$key;
                    $type           = request()->$typeReq;

                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }

                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                if( isset( request()->$sortReq ) ) {
                    $nameReq        = 'search_string_name_'.$key;
                    $sortReq        = 'search_string_sort_'.$key;
                    $typeReq        = 'search_string_type_'.$key;
                    $tableNameReq   = 'search_table_name_'.$key;
                    $colAliasReq    = 'search_string_col_alias_'.$key;

                    $origData       = request()->$nameReq;
                    $condition_sort = request()->$sortReq;
                    $type           = request()->$typeReq;
                    $tableName      = isset( request()->$tableNameReq ) ? request()->$tableNameReq : '';
                    $colAlias       = isset( request()->$colAliasReq ) ? request()->$colAliasReq : '';

                    if( $type == 'string' ) {
                        if( isset( $condition_sort ) ) {
                            if( $colAlias != '' ) {
                                $data->orderBy( $tableName.'.'.$colAlias, $condition_sort );
                            } else {
                                if( $tableName != '' ) {
                                    $data->orderBy( $tableName.'.'.$origData, $condition_sort );
                                } else {
                                    $data->orderBy( $origData, $condition_sort );
                                }
                            }
                        }
                    }
                }
            }
            /// end
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function($row){
                    $action = '<a href="'.route('company.admin.truck-owners.show', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                    $action .= '<a href="'.route('company.admin.truck-owners.edit', base64Encode($row->id)).'" class="btn btn-sm btn-outline-black-action"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                    $action .= '<a href="javascript:void(0);" class="btn btn-sm btn-outline-black-trash destroy" data-action="'.route('company.admin.truck-owners.destroy',$row->id).'" data-id="'.$row->id.'"><i class="fa fa-trash" aria-hidden="true"></i></a>';

                    return $action;
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        $tableName                  = 'settings.table_truck_owners';
        return view('companyAdmin.truckOwners.index', compact('getField','tableName'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(){
        return view('companyAdmin.truckOwners.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request){
        $validatedData = $request->validate([
            'name'              => 'required',
        ]);

        $input                  = $request->except(['_token']);
        $input['company_id']    = auth()->guard('company-admin')->user()->company_id;
        $input['owner_id']      = auth()->guard('company-admin')->user()->getCompany->unique_id.'_TruckOwner_'.getNextAutoID('truck_owners');
        TruckOwner::create($input);

        notification('success', $this->crudMessage('create', 'Truck owner'));

        return redirect()->route('company.admin.truck-owners.index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id){
        $id                     = base64Decode($id);
        $truckOwner             = TruckOwner::findOrFail($id);
        return view('companyAdmin.truckOwners.edit', compact('truckOwner'));
    }

    /**
     * Display the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id){
        $id                     = base64Decode($id);
        $truckOwner             = TruckOwner::findOrFail($id);
        return view('companyAdmin.truckOwners.show', compact('truckOwner'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, TruckOwner $truckOwner){
        $validatedData = $request->validate([
            'name'              => 'required',
        ]);

        $input                  = $request->except(['_token','_method']);

        $truckOwner->update($input);

        notification('success', $this->crudMessage('update', 'Truck owner'));

        return redirect()->route('company.admin.truck-owners.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(TruckOwner $truckOwner){
        $truckOwner->delete();

        notification('success', $this->crudMessage('delete', 'Truck owner'));

        return redirect()->route('company.admin.truck-owners.index');
    }
}
