<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Helpers\Common;
use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use App\Models\Bookmark;
use App\Models\EmailLog;
use App\Models\SmsLog;
use App\Models\Company;

use Illuminate\Http\Request;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Facades\DataTables;

class UsageController extends CompanyAdminController
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function indexEmail(Request $request)
    {
        $getField                   = '';
        $getCompanyAdmin            = auth()->guard('company-admin')->user();
        $isCheck                    = Bookmark::where('company_id', getUserCompnayId())->where('user_id', $getCompanyAdmin->id)->where('user_type', $getCompanyAdmin->type)->where('table_id', 'email_logs')->first();
        if(!empty($isCheck->id)){
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_email_logs_first'));
        }else{
            $getField               = config('settings.table_email_logs_first');
        }

        if ($request->ajax()) {
            $data = EmailLog::IsCompany()->select('user_id', 'from', 'to', 'subject', 'content', DB::raw('DATE_FORMAT(sent_date, "%m/%d/%Y %H:%i:%S") as sent_date'), DB::raw('IF(status="1", "Sent", "Failed") as status'));

            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq       = 'search_string_condition_'.$key;
                $sortReq            = 'search_string_sort_'.$key;

                $conditionDateReq   = 'search_date_condition_'.$key;
                $sortDateReq        = 'search_date_sort_'.$key;

                // String filter
                if( isset( request()->$conditionReq ) ) {
                    $typeReq        = 'search_string_type_'.$key;
                    $type           = request()->$typeReq;

                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }
                }

                // Date filter
                if( isset( request()->$conditionDateReq ) ) {
                    $typeReq        = 'search_date_type_'.$key;
                    $type           = request()->$typeReq;

                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                // String sort only
                if( isset( request()->$sortReq ) ) {
                    Common::sortStringFilter( $data, $key );
                }

                // Date sort only
                if( isset( request()->$sortDateReq ) ) {
                    Common::sortDateFilter( $data, $key );
                }
            }
            // General filter code - end

            $data = $data->orderBy('id', 'DESC')->get();

            return DataTables::of($data)
                    ->addIndexColumn()
                    ->editColumn('user_id', function($row){
                        return $row->company_admin->full_name ?? '';
                    })
                    ->addColumn('content_html', function($row){
                        return $row->content ?? '';
                    })
                    ->editColumn('content', function($row){
                        if(!is_null($row->content)){
                            $html       = '<div class="status-block">';
                            $html      .= '<div class="round-icon details-control"><i class="fa fa-caret-down" aria-hidden="true"></i></div>';
                            $html      .= '</div>';
                            return $html;
                        }
                        return '';
                    })
                    ->editColumn('status', function($row){
                        if($row->status == 'Sent'){
                            return '<span class="font-weight-bold text-green">'.$row->status.'</span>';
                        }else{
                            return '<span class="font-weight-bold text-danger">'.$row->status.'</span>';
                        }
                    })
                    ->rawColumns(['content', 'status'])
                    ->make(true);
        }

        $tableName = 'settings.table_email_logs';
        $startDate = Carbon::now()->startOfMonth();
        $endDate = Carbon::now();

        $logCount = EmailLog::IsCompany()->where(DB::raw('DATE_FORMAT(sent_date, "%Y-%m-%d")'), '>=', $startDate)->where(DB::raw('DATE_FORMAT(sent_date, "%Y-%m-%d")'), '<=', $endDate)->where('status', '1')->count();

        return view('companyAdmin.usage.email_index', compact(['getField', 'tableName', 'logCount']));
    }

    public function indexSms(Request $request)
    {

        $getField                   = '';
        $getCompanyAdmin            = auth()->guard('company-admin')->user();
        $isCheck                    = Bookmark::where('company_id', getUserCompnayId())->where('user_id', $getCompanyAdmin->id)->where('user_type', $getCompanyAdmin->type)->where('table_id', 'sms_logs')->first();
        if(!empty($isCheck->id)){
            $getField               = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_sms_logs_first'));
        }else{
            $getField               = config('settings.table_sms_logs_first');
        }

        if ($request->ajax()) {
            $data = SmsLog::IsCompany()->select('user_id', 'to_name', 'from', 'to', 'content', DB::raw('DATE_FORMAT(sent_date, "%m/%d/%Y %H:%i:%S") as sent_date'), DB::raw('IF(status="1", "Sent", "Failed") as status'));

            // General filter code - start
            foreach( $getField as $key => $value ) {
                $conditionReq       = 'search_string_condition_'.$key;
                $sortReq            = 'search_string_sort_'.$key;

                $conditionDateReq   = 'search_date_condition_'.$key;
                $sortDateReq        = 'search_date_sort_'.$key;

                // String filter
                if( isset( request()->$conditionReq ) ) {
                    $typeReq        = 'search_string_type_'.$key;
                    $type           = request()->$typeReq;

                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }
                }

                // Date filter
                if( isset( request()->$conditionDateReq ) ) {
                    $typeReq        = 'search_date_type_'.$key;
                    $type           = request()->$typeReq;

                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                // String sort only
                if( isset( request()->$sortReq ) ) {
                    Common::sortStringFilter( $data, $key );
                }

                // Date sort only
                if( isset( request()->$sortDateReq ) ) {
                    Common::sortDateFilter( $data, $key );
                }
            }
            // General filter code - end

            $data = $data->orderBy('id', 'DESC')->get();

            return DataTables::of($data)
                ->addIndexColumn()
                ->editColumn('user_id', function($row){
                    return !empty($row->to_name) ? $row->to_name : $row->company_admin->full_name;
                })
                ->addColumn('content_html', function($row){
                    return $row->content ?? '';
                })
                ->editColumn('content', function($row){
                    if(!is_null($row->content)){
                        $html       = '<div class="status-block">';
                        $html      .= '<div class="round-icon details-control"><i class="fa fa-caret-down" aria-hidden="true"></i></div>';
                        $html      .= '</div>';
                        return $html;
                    }
                    return '';
                })
                ->editColumn('status', function($row){
                    if($row->status == 'Sent'){
                        return '<span class="font-weight-bold text-green">'.$row->status.'</span>';
                    }else{
                        return '<span class="font-weight-bold text-danger">'.$row->status.'</span>';
                    }
                })
                ->rawColumns(['content', 'status'])
                ->make(true);
        }

        $tableName = 'settings.table_sms_logs';
        $startDate = Carbon::now()->startOfMonth()->format('Y-m-d');
        $endDate = Carbon::now();
        
        $logCount = SmsLog::IsCompany()->where(DB::raw('DATE_FORMAT(sent_date, "%Y-%m-%d")'), '>=', $startDate)->where(DB::raw('DATE_FORMAT(sent_date, "%Y-%m-%d")'), '<=', $endDate)->where('status', '1')->count();

        return view('companyAdmin.usage.sms_index', compact(['getField', 'tableName', 'logCount']));
    }
}
