<?php

namespace App\Http\Controllers\DriverAdmin;

use App\Http\Controllers\DriverAdmin\DriverAdminController;

use App\Models\AnnouncementClearByUser;
use Illuminate\Http\Request;
use Illuminate\Support\Str;

use App\Models\Announcement;

use DB;
use DataTables;
use Carbon\Carbon;

class CompelAnnouncementController extends DriverAdminController{

    /**
     * Create a new controller instance.
     *
     * @return void
     */

    public $timeZone;
    public $timeZoneId;
    public $offset;

    public function __construct(){
        parent::__construct();

        $this->middleware(function ($request, $next) {
            $this->timeZone     = getTimeZone();
            $this->timeZoneId   = getTimeZoneId();
            $this->offset       = getTimeZoneOffset();

            return $next($request);
        });
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        if ($request->ajax()) {
            $offset     = $this->offset;
            $search     = $request->search['value'] ?? '';
            $data       = Announcement::query()
                ->select(
                    '*',
                    DB::raw('CONVERT_TZ(announcements.created_at,"+00:00","'.$offset.'") as created_at')
                )
                ->where('module','Compel')
                ->where(function($query) {
                    $query->where(function($q) {
                        $q->whereNull('company_id')->whereRaw('FIND_IN_SET(3,show_up)');
                    })->orWhere(function($q) {
                        $q->whereNotNUll('company_id')
                            ->whereRaw('FIND_IN_SET(3,show_up)')
                            ->whereRaw('FIND_IN_SET("'.auth()->guard('driver-admin')->user()->company_id.'",company_id)');
                    });
                });
            return Datatables::of($data)
                    ->addIndexColumn()
                    ->editColumn('description', function ($data) {
                        return Str::limit(strip_tags($data->description), 50);
                    })
                    ->editColumn('created_at', function($data){
                        if(auth()->guard('driver-admin')->user()->time_format=='12'){
                            return !is_null($data->created_at) ? Carbon::parse($data->created_at)->format(config('settings.date_time_format.12_hour','m/d/Y h:i A'))." ".$this->timeZoneId : '';
                        }else{
                            return !is_null($data->created_at) ? Carbon::parse($data->created_at)->format(config('settings.date_time_format.24_hour','m/d/Y H:i'))." ".$this->timeZoneId : '';
                        }
                    })
                    ->addColumn('action', function($data){
                        $action         = '';
                        $action        .= '<a href="'.route('driver.admin.compel.announcement.show', base64Encode($data->id)).'"><img src="'.asset('theme/img/eye.png').'" alt=""></a>&nbsp;&nbsp;&nbsp;';
                        return $action;
                    })
                    ->filterColumn('created_at', function ($query, $keyword) use ($search, $offset) {
                        if(auth()->guard('driver-admin')->user()->time_format=='12'){
                            $query->whereRaw("CONCAT(DATE_FORMAT(CONVERT_TZ(announcements.created_at,'+00:00','".$offset."'),'".config('settings.database_date_time_format.12_hour','%m/%d/%Y %h:%i %A')."'),' ','".$this->timeZoneId."') like ?", ["%$search%"]);
                        }else{
                            $query->whereRaw("CONCAT(DATE_FORMAT(CONVERT_TZ(announcements.created_at,'+00:00','".$offset."'),'".config('settings.database_date_time_format.24_hour','%m/%d/%Y %H:%i')."'),' ','".$this->timeZoneId."') like ?", ["%$search%"]);
                        }
                    })
                    ->rawColumns(['action'])
                    ->make(true);
        }
        return view('driverAdmin.announcements.compel.index');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id){
        $compel_announcement    = Announcement::find(base64Decode($id));
        $announcement           = $compel_announcement;
        $getTimeZone            = $this->timeZone;
        $getTimeZoneId          = $this->timeZoneId;
        return view('driverAdmin.announcements.compel.show', compact('announcement', 'getTimeZone', 'getTimeZoneId'));
    }

    public function announcementClear(Request $request){
        $companyId              = auth()->guard('company-admin')->user()->company_id;
        $announcementId         = $request->dataId;
        $userType               = '3';
        $userId                 = auth()->guard('driver-admin')->user()->id;

        AnnouncementClearByUser::create([
            'company_id'        => $companyId,
            'announcement_id'   => $announcementId,
            'user_type'         => $userType,
            'user_id'           => $userId,
        ]);
        return true;
    }
}
