<?php

namespace App\Http\Controllers\DriverAdmin;

use App\Http\Controllers\DriverAdmin\DriverAdminController;
use Illuminate\Http\Request;

use Illuminate\Support\Facades\DB;
use Illuminate\Support\Arr;

use App\Models\Widget;

use App\Models\Leave;
use App\Models\Event;
use App\Models\Announcement;

use App\Models\Truck;
use App\Models\TruckMaintenanceLog;

use App\Models\Trailer;
use App\Models\TrailerMaintenanceLog;

use App\Models\LoadDetail;

use App\Models\Driver;
use App\Models\DriverFinancial;
use App\Models\DriverInformation;

use Carbon\Carbon;
use Carbon\CarbonPeriod;

use Session;
use DateTime;
use DataTables;

class HomeController extends DriverAdminController
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */

    public $driver_admin;
    public $timeZone;
    public $timeZoneId;
    //public $offset;

    public function __construct(){
        parent::__construct();

        $this->middleware(function ($request, $next) {
            $this->driver_admin     = auth()->guard('driver-admin')->user();
            $this->timeZone         = getTimeZone();
            $this->timeZoneId       = getTimeZoneId();
            //$this->offset         = getTimeZoneOffset();

            return $next($request);
        });

        ini_set('max_execution_time', 600);
    }

    function getWeek($week, $year) {
        $dto                    = new DateTime("now", new \DateTimeZone( $this->timeZone ) ); //new DateTime();
        $dto->setISODate($year, $week, $this->driver_admin->weekly_strat_day ?? 1);
        $result['startDate']    = $dto->format('Y-m-d');
        $dto->modify('+6 days');
        $result['endDate']      = $dto->format('Y-m-d');
        return $result;
    }

    /**
     * Show the driver admin dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function home(){
        $widgetData             = $this->getWidget();
        return view('driverAdmin.home',compact('widgetData'));
    }

    public function getWidget(){
        $user                   = $this->driver_admin;

        $param                  = [];
        $param['type']          = 3;
        $param['search']        = '';
        $param['column']        = 'id';

        $widgetData             = [];
        $widgetArray            = widgetArray($param);
        usort($widgetArray, function($a, $b) { return $a['display_no'] <=> $b['display_no'];});

        $widgetArray             = array_values(array_filter($widgetArray));
        if(!empty($widgetArray)){
            foreach ($widgetArray as $key => $widget) {
                $arry           = $widget;
                $getWidget      = Widget::isDriverAdmin()
                    ->where('guard_name', 'driver-admin')
                    ->where('widget_id', $widget['id'])
                    ->where('tab', 'own')
                    ->withTrashed()
                    ->first();
                if(!is_null($getWidget)){
                    //$arry['getWidget']                    = $getWidget->toArray();
                    if(!is_null($getWidget->properties)){
                        $arry['properties']                 = $getWidget->properties;
                    }
                    if(!is_null($getWidget->deleted_at)){ // Is delete
                        $arry['is_delete']                  = 1;
                    }
                }else{
                    $insertArry                             = [];
                    if($widget['slug']=='occupancy'){
                        $insertArry['range']                = 'y';
                    }elseif ($widget['slug']=='revenue') {
                        $insertArry['range']                = 'y';
                    }elseif ($widget['slug']=='cpm') {
                        $insertArry['range']                = 'y';
                    }elseif ($widget['slug']=='revenue_by_trailer_type') {
                        $insertArry['range']                = 'y';
                    }
                    $insertArry['company_id']               = $user->company_id;
                    $insertArry['guard_name']               = 'driver-admin';
                    $insertArry['driver_id']                = $user->id;
                    $insertArry['widget_id']                = $widget['id'];
                    $insertArry['properties']               = $widget['properties'];
                    $insertArry['tab']                      = 'own';
                    $insertArry['created_at']               = Carbon::now();
                    $getWidget                              = Widget::create($insertArry);
                    // $arry['getWidget']                   = $getWidget->toArray();
                }
                $widgetData[$widget['slug']]                = $arry;
            }
        }
        foreach ($widgetData as $key => &$widget) {
            if(!isset($widget['is_delete'])){
                $properties                                 = json_decode($widget['properties'],true);
                $content                                    = view('driverAdmin.home.ajax_contents',compact('widget'))->render();
                $properties['content']                      = $content;
                $widget['properties']                       = json_encode($properties);
            }
        }
        return $widgetData;
    }

    public function getOccupancy($param=[]){
        $user                                   = $this->driver_admin;

        $range                                  = isset($param['range'])?$param['range']:'y';
        if($range=='w'){
            $weekly_strat_day                   = $user->weekly_strat_day ?? 1;
            $getWeekDate                        = getWeekDate($weekly_strat_day);
            $startDate                          = $getWeekDate['startDate'];
            $endDate                            = $getWeekDate['endDate'];
        }elseif($range=='m') {
            $startDate                          = Carbon::now()->timezone($this->timeZone)->startOfMonth()->format('Y-m-d');
            $endDate                            = Carbon::now()->timezone($this->timeZone)->endOfMonth()->format('Y-m-d');
        }else{
            $startDate                          = Carbon::now()->timezone($this->timeZone)->startOfYear()->format('Y-m-d');
            $endDate                            = Carbon::now()->timezone($this->timeZone)->endOfYear()->format('Y-m-d');
        }

        $arry                                   = [];
        $arry['startDate']                      = date('M d, Y', strtotime($startDate));
        $arry['endDate']                        = date('M d, Y', strtotime($endDate));

        $occupancyGoal                          = 0;
        $fleetOccupancy                         = 0;

        $getOccupancyGoal                       = 0;
        $getFleetOccupancy                      = 0;
        $getActiveDriverCount                   = 0;

        $getLoadDriverIds                       = LoadDetail::select('driver_id')
            ->where('driver_id', $user->id)
            ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
            ->where('load_type','!=',5)
            ->groupBy('driver_id')
            ->pluck('driver_id')
            ->toArray();
        if(!empty($getLoadDriverIds)){
            $getData                            = $this->getLoadOccupancyCalendarMonth2($startDate, $endDate, $getLoadDriverIds);
            $getOccupancy                       = $getData['getOccupancy'];

            $getTargetRevenueMiles              = $this->getTargetRevenueMilesByDate($startDate, $endDate, $getLoadDriverIds);
            $getOccupancyGoal                   = $getTargetRevenueMiles['occupancy'];

            $getFleetOccupancy                  = $getOccupancy;

            $getActiveDriverCount               = count($getLoadDriverIds);
            if($getActiveDriverCount > 0){
                if ($getOccupancyGoal > 0) {
                    $occupancyGoal              = round($getOccupancyGoal / $getActiveDriverCount);
                }
                if ($getFleetOccupancy > 0) {
                    $fleetOccupancy             = round($getFleetOccupancy / $getActiveDriverCount);
                }
            }
            $occupancyGoal                      = (int)min(100, $occupancyGoal);
            $fleetOccupancy                     = (int)min(100, $fleetOccupancy);
        }

        $arry['occupancyGoal']                  = $occupancyGoal;
        $arry['fleetOccupancy']                 = $fleetOccupancy;
        return $arry;
    }

    public function getRevenue($param=[]){
        $user                                   = $this->driver_admin;

        $range                                  = isset($param['range'])?$param['range']:'y';
        if($range=='w'){
            $weekly_strat_day                   = $user->weekly_strat_day ?? 1;
            $getWeekDate                        = getWeekDate($weekly_strat_day);
            $startDate                          = $getWeekDate['startDate'];
            $endDate                            = $getWeekDate['endDate'];
        }elseif($range=='m') {
            $startDate                          = Carbon::now()->timezone($this->timeZone)->startOfMonth()->format('Y-m-d');
            $endDate                            = Carbon::now()->timezone($this->timeZone)->endOfMonth()->format('Y-m-d');
        }else{
            $startDate                          = Carbon::now()->timezone($this->timeZone)->startOfYear()->format('Y-m-d');
            $endDate                            = Carbon::now()->timezone($this->timeZone)->endOfYear()->format('Y-m-d');
        }

        $arry                                   = [];
        $arry['startDate']                      = date('M d, Y', strtotime($startDate));
        $arry['endDate']                        = date('M d, Y', strtotime($endDate));

        $getActualRevenue                       = 0;
        $getTargetRevenue                       = 0;
        $getActiveDriverCount                   = 0;

        $getLoadDriverIds                       = LoadDetail::select('driver_id')
            ->where('driver_id', $user->id)
            ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
            ->where('load_type','!=',5)
            ->groupBy('driver_id')
            ->pluck('driver_id')
            ->toArray();
        if(!empty($getLoadDriverIds)){
            $getLoadGrossRevenue                = LoadDetail::select('driver_revenue')
                ->whereHas('driver', function($q) use($getLoadDriverIds){
                    $q->whereIn('id', $getLoadDriverIds);
                })
                ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                ->whereIn('load_details.is_disable', [0,1])
                ->where('load_details.load_status', '!=', 9)
                ->where(function($q) {
                    $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
                })
                ->whereNotIn('load_details.load_type', [1, 2, 3, 5]);
            $getActualRevenue                   = $getLoadGrossRevenue->sum('driver_revenue');

            $getTargetRevenueMiles              = $this->getTargetRevenueMilesByDate($startDate, $endDate, $getLoadDriverIds);
            $getTargetRevenue                   = $getTargetRevenueMiles['revenue'];

            $getActiveDriverCount               = count($getLoadDriverIds);
        }

        $arry['actualRevenue']                  = number_format((float)$getActualRevenue, 2, '.', '');
        $arry['targetRevenue']                  = number_format((float)$getTargetRevenue, 2, '.', '');
        return $arry;
    }

    public function getCpm($param=[]){
        $user                                   = $this->driver_admin;

        $range                                  = isset($param['range'])?$param['range']:'y';
        if($range=='w'){
            $weekly_strat_day                   = $user->weekly_strat_day ?? 1;
            $getWeekDate                        = getWeekDate($weekly_strat_day);
            $startDate                          = $getWeekDate['startDate'];
            $endDate                            = $getWeekDate['endDate'];
        }elseif($range=='m') {
            $startDate                          = Carbon::now()->timezone($this->timeZone)->startOfMonth()->format('Y-m-d');
            $endDate                            = Carbon::now()->timezone($this->timeZone)->endOfMonth()->format('Y-m-d');
        }else{
            $startDate                          = Carbon::now()->timezone($this->timeZone)->startOfYear()->format('Y-m-d');
            $endDate                            = Carbon::now()->timezone($this->timeZone)->endOfYear()->format('Y-m-d');
        }

        $arry                                   = [];
        $arry['startDate']                      = date('M d, Y', strtotime($startDate));
        $arry['endDate']                        = date('M d, Y', strtotime($endDate));

        $targetCPM                              = 0;
        $actualCPM                              = 0;

        $getLoadDriverIds                       = LoadDetail::select('driver_id')
            ->where('driver_id', $user->id)
            ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
            ->where('load_type','!=',5)
            ->groupBy('driver_id')
            ->pluck('driver_id')
            ->toArray();
        if(!empty($getLoadDriverIds)){
            $activeDrivers                      = 1;

            $getLoadGrossRevenue                = LoadDetail::select(
                    DB::raw('SUM(load_details.driver_revenue) as total'),
                    'load_details.driver_id'
                )
                ->join('drivers', 'drivers.id', '=', 'load_details.driver_id')
                ->whereIn('load_details.driver_id',$getLoadDriverIds)
                ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                ->whereIn('load_details.is_disable', [0,1])
                ->where('load_details.load_status', '!=', 9)
                ->where(function($q) {
                    $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
                })
                ->whereNotIn('load_details.load_type', [1, 2, 3, 5])
                ->whereNull('drivers.deleted_at')
                ->groupBy('load_details.driver_id')
                ->pluck('total', 'load_details.driver_id')->toArray();
            $getLoadGrossRevenue                = array_sum($getLoadGrossRevenue);

            $getLoadMileage                     = LoadDetail::select(
                    DB::raw('SUM(load_drivers.miles) as total'),
                    'load_details.driver_id'
                )
                ->join('drivers', 'drivers.id', '=', 'load_details.driver_id')
                ->join('load_drivers', 'load_drivers.load_detail_id', '=', 'load_details.id')
                ->whereIn('load_details.driver_id',$getLoadDriverIds)
                ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                ->whereIn('load_details.is_disable', [0,1])
                ->where('load_details.load_status', '!=', 9)
                ->where(function($q) {
                    $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
                })
                ->whereNotIn('load_details.load_type', [1, 2, 3, 5])
                ->whereNull('drivers.deleted_at')
                ->whereNull('load_drivers.deleted_at')
                ->groupBy('load_details.driver_id')
                ->pluck('total', 'load_details.driver_id')->toArray();
            $getLoadMileage                     = array_sum($getLoadMileage);

            if($getLoadGrossRevenue > 0 && $getLoadMileage > 0){
                $actualCPM                      = (($getLoadGrossRevenue / $getLoadMileage) / $activeDrivers);
            }

            $getTargetRevenueMiles              = $this->getTargetRevenueMilesByDate($startDate, $endDate, $getLoadDriverIds);
            if(isset($getTargetRevenueMiles['cpm']) && $getTargetRevenueMiles['cpm'] > 0){
                $targetCPM                      = $getTargetRevenueMiles['cpm'] / $activeDrivers;
            }
        }

        //$targetCPM                            = min(100, $targetCPM);
        $arry['targetCPM']                      = number_format((float)$targetCPM, 2, '.', '');

        //$actualCPM                            = min(100, $actualCPM);
        $arry['actualCPM']                      = number_format((float)$actualCPM, 2, '.', '');
        return $arry;
    }

    public function getTrailerType(){
        $user                                   = $this->driver_admin;

        $driverQuery                            = Driver::with(['driver_information'])->has('team');
        $driverQuery->where('id', $user->id);
        $driverQuery->whereHas('driver_information', function($q){
            $q->whereNotNull('trailer_number');
        });
        $trailer_ids                            = $driverQuery->get()->pluck('driver_information.trailer_number')->toArray();

        $arry                                   = [];
        $trailerTypeData                        = [];

        $trailerTypeQuery                       = Trailer::selectRaw('count(*) as total, (CASE WHEN type IS NULL THEN "Not specified" ELSE type END) AS type');
        $trailerTypeQuery->whereIn('trailer_id', $trailer_ids);

        $getTrailerType                         = $trailerTypeQuery->groupBy('type')
            ->pluck('total', 'type')
            ->toArray();

        if(!empty($getTrailerType)){
            foreach ($getTrailerType as $gtype => $gtotal) {
                $gtypeLabel                     = 'Not specified';
                if (isset(getTrailerTypeList()[$gtype])) {
                    $gtypeLabel                 = isset(getTrailerTypeList()[$gtype]) ? getTrailerTypeList()[$gtype] : '';
                }
                $trailerTypeData[$gtypeLabel]   = $gtotal;
            }
        }

        $arry['trailerType']                    =  array_sum($trailerTypeData);
        $arry['trailerTypeData']                =  json_encode($trailerTypeData);
        return $arry;
    }

    public function getTargetRevenue(){
        $user                                   = $this->driver_admin;

        $data       = DriverFinancial::where('driver_id', $user->id)
            ->orderBy('quarter', 'ASC')
            ->get();
        $array      = [];
        foreach ($data as $key => $item) {
            $to     = Carbon::createFromFormat('Y-m-d', $item->start_date); //Carbon::createFromFormat('Y-m-d', $item->start_date)->setTimezone($this->timeZone);
            $from   = Carbon::createFromFormat('Y-m-d', $item->end_date); //Carbon::createFromFormat('Y-m-d', $item->end_date)->setTimezone($this->timeZone);
            $quarterdays    = $to->diffInDays($from) + 1;
            $perdayRevenue  = $item->revenue_target / $quarterdays;
            $dateWiseArray  = [];
            for($i  = $to; $i <= $from; $i->modify('+1 day')){
                $dateWiseArray[strtotime($i->format("Y-m-d"))* 1000]    = $perdayRevenue;
            }
            $array[]        = $dateWiseArray;
        }
        ksort($array);

        $dataArray  = [];
        foreach ($array as $akey => $value) {
            foreach ($value as $vkey => $val) {
                if (!isset($dataArray[$vkey])) {
                    $dataArray[$vkey] = [$vkey, (float)$val];
                }
                else {
                    $dataArray[$vkey][1] += (float)$val;
                }
            }
        }
        $dataArray  = array_values($dataArray);
        return json_encode($dataArray);
    }

    public function getActualRevenue(){
        $user                                   = $this->driver_admin;

        $revenueQuery                           = LoadDetail::selectRaw('SUM(load_details.driver_revenue) as total, DATE(load_details.actual_delivery_date) as date')
            ->join('drivers', 'drivers.id', '=', 'load_details.driver_id')
            ->whereNotNull('load_details.actual_delivery_date')
            ->where('load_details.load_status', '!=', 9)
            ->where(function($q) {
                $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
            })
            ->whereNotIn('load_details.load_type', [1, 2, 3, 5])
            ->whereNull('drivers.deleted_at')
            ->where('drivers.id', $user->id);

        $revenues   = $revenueQuery->groupBy('date')
            ->orderBy('date','asc')
            ->pluck('total', 'date');
        $dataArray  = [];
        foreach ($revenues as $key => $item) {
            //$dateInMills  = strtotime(date('Y-m-d', strtotime($key))) * 1000;
            $dataArray[]    = [(Carbon::parse($key)->timestamp)*1000, (float)$item];
        }
        return json_encode($dataArray);
    }

    public function inTransitLoads(Request $request){
        $user                                   = $this->driver_admin;

        $inTransitLoadsQuery                    = LoadDetail::query()
            ->where('load_details.load_status', 4)
            ->where(function($q) {
                $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
            })
            ->where('load_details.driver_id', $user->id);
            $data                               = $inTransitLoadsQuery->get();
            return Datatables::of($data)
                ->addColumn('dispatcher', function($row){
                    return getLoadDispatcherName($row->booked_by);
                })
                ->addColumn('origin', function($row){
                    return getAddressCityState($row->shipper_city, $row->shipper_state);
                })
                ->addColumn('destination', function($row){
                    return getAddressCityState($row->delivery_city, $row->delivery_state);
                })
                ->addColumn('status', function($row){
                    return getLoadStatus($row->load_status,$row->is_on_hold);
                })
                ->addColumn('load_date', function($row){
                    return !is_null($row->actual_shipper_date) ? $row->actual_shipper_date." ".$row->shipper_timezone_abbr : '';
                })
                ->addColumn('delivery_date', function($row){
                    return !is_null($row->actual_delivery_date) ? $row->actual_delivery_date." ".$row->delivery_timezone_abbr : '';
                })
                ->rawColumns(['load_date','delivery_date'])
                ->make(true);
    }

    public function getRevenueByTrailerType($param=[]){
        $user                                   = $this->driver_admin;

        $range                                  = isset($param['range'])?$param['range']:'y';
        if($range=='w'){
            $weekly_strat_day                   = $user->weekly_strat_day ?? 1;
            $getWeekDate                        = getWeekDate($weekly_strat_day);
            $startDate                          = $getWeekDate['startDate'];
            $endDate                            = $getWeekDate['endDate'];
        }elseif($range=='m') {
            $startDate                          = Carbon::now()->timezone($this->timeZone)->startOfMonth()->format('Y-m-d');
            $endDate                            = Carbon::now()->timezone($this->timeZone)->endOfMonth()->format('Y-m-d');
        }else{
            $startDate                          = Carbon::now()->timezone($this->timeZone)->startOfYear()->format('Y-m-d');
            $endDate                            = Carbon::now()->timezone($this->timeZone)->endOfYear()->format('Y-m-d');
        }

        $arry                                   = [];
        $arry['startDate']                      = date('M d, Y', strtotime($startDate));
        $arry['endDate']                        = date('M d, Y', strtotime($endDate));

        $revenueByTrailerTypeData               = [];
        $revenueByTrailerTypeQuery              = LoadDetail::selectRaw('SUM(load_details.driver_revenue) as total,load_details.trailer_type')
            ->join('drivers', 'drivers.id', '=', 'load_details.driver_id')
            ->join('teams', 'teams.id', '=', 'drivers.team_id')
            ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
            ->whereIn('load_details.is_disable', [0,1])
            ->where('load_details.load_status', '!=', 9)
            ->where(function($q) {
                $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
            })
            ->whereNotIn('load_details.load_type', [1, 2, 3, 5])
            ->whereNull('drivers.deleted_at')
            ->where('load_details.driver_id', $user->id);
        $getTrailerLoadRevenue                  = $revenueByTrailerTypeQuery->groupBy('load_details.trailer_type')
            ->pluck('total','trailer_type')
            ->toArray();
        if(!empty($getTrailerLoadRevenue)){
            foreach ($getTrailerLoadRevenue as $gtype => $gtotal) {
                $gtypeLabel                     = 'Not specified';
                if (isset(getTrailerTypeList()[$gtype])) {
                    $gtypeLabel                 = isset(getTrailerTypeList()[$gtype]) ? getTrailerTypeList()[$gtype] : '';
                }
                $revenueByTrailerTypeData[$gtypeLabel] = number_format((float)$gtotal, 2, '.', '');
            }
        }
        $revenueByTrailerType                   = array_sum($revenueByTrailerTypeData);

        $arry['revenueByTrailerType']           = $revenueByTrailerType;
        $arry['revenueByTrailerTypeData']       = json_encode($revenueByTrailerTypeData);
        return $arry;
    }

    public function getLoadsStats(){
        $user                                   = $this->driver_admin;

        $arry                                   = [];

        $loadsStatsQuery                        = LoadDetail::where('driver_id', $user->id);
        $loadsStatsQuery->where('is_disable', 0);
        $arry['loadsStats']                     = $loadsStatsQuery->count();

        $loadsStatsOnHoldQuery                  = LoadDetail::where('driver_id', $user->id);
        $loadsStatsOnHoldQuery->where(function($q) {
            $q->where('load_status', 7)->orWhere('is_on_hold', 1);
        });
        $arry['loadsStatsOnHold']               = $loadsStatsOnHoldQuery->count();

        $loadsStatsPlannedQuery                 = LoadDetail::where('driver_id', $user->id);
        $loadsStatsPlannedQuery->whereIn("load_status",[0,2,11,3])->where(function($q) {
            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
        });
        $arry['loadsStatsPlanned']              = $loadsStatsPlannedQuery->count();

        $loadsStatsInTransitQuery               = LoadDetail::where('driver_id', $user->id);
        $loadsStatsInTransitQuery->where("load_status",4)->where(function($q) {
            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
        });
        $arry['loadsStatsInTransit']            = $loadsStatsInTransitQuery->count();

        return $arry;
    }

    public function getRevenueOccupancy(){
        $user                                       = $this->driver_admin;

        $year                                       = date('Y');
        $currentweek                                = date("W");
        $totalweek                                  = date('W', strtotime($year.'-12-31'));

        $arry                                       = [];
        $actualRevenueArry                          = [];
        $targetRevenueArry                          = [];
        $occupancyGoalArry                          = [];
        $fleetOccupancyArry                         = [];

        $weeks                                      = [];
        for($i=1;$i<=$currentweek;$i++) {
            $result                                 = $this->getWeek($i,$year);

            $currentDate                            = date('Y-m-d');
            $startDate                              = $result['startDate'];
            $endDate                                = $result['endDate'];
            if(strtotime($currentDate) < strtotime($startDate)){
                break;
            }
            //echo "Week:".$i." Start date:".$result['start']." End date:".$result['end']."<br>";

            $week                                   = [];
            $week['startDate']                      = $startDate;
            $week['endDate']                        = $endDate;
            // if ($i == 1) {
            //     $week['startDate']               = "2022-01-01";
            // } else if ($i == 52) {
            //     $week['endDate']                 = "2022-12-31";
            // }
            $weeks["Week ".$i]                      = $week;

            // week calculation
            $occupancyGoal                          = 0;
            $fleetOccupancy                         = 0;

            $getActualRevenue                       = 0;
            $getTargetRevenue                       = 0;
            $getOccupancyGoal                       = 0;
            $getFleetOccupancy                      = 0;
            $getActiveDriverCount                   = 0;

            $getLoadDriverIds                       = LoadDetail::select('driver_id')
                ->where('driver_id', $user->id)
                ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                ->where('load_type','!=',5)
                ->groupBy('driver_id')
                ->pluck('driver_id')
                ->toArray();
            if(!empty($getLoadDriverIds)){
                $getData                            = $this->getLoadOccupancyCalendarMonth2($startDate, $endDate, $getLoadDriverIds);
                $getOccupancy                       = $getData['getOccupancy'];

                $getLoadGrossRevenue                = LoadDetail::select('driver_revenue')
                    ->whereHas('driver', function($q) use($getLoadDriverIds){
                        $q->whereIn('id', $getLoadDriverIds);
                    })
                    ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                    ->whereIn('load_details.is_disable', [0,1])
                    ->where('load_details.load_status', '!=', 9)
                    ->where(function($q) {
                        $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
                    })
                    ->whereNotIn('load_details.load_type', [1, 2, 3, 5]);
                $getActualRevenue                   = $getLoadGrossRevenue->sum('driver_revenue');

                $getTargetRevenueMiles              = $this->getTargetRevenueMilesByDate($startDate, $endDate, $getLoadDriverIds);
                $getTargetRevenue                   = $getTargetRevenueMiles['revenue'];
                $getOccupancyGoal                   = $getTargetRevenueMiles['occupancy'];

                $getFleetOccupancy                  = $getOccupancy;

                $getActiveDriverCount               = count($getLoadDriverIds);
                if($getActiveDriverCount > 0){
                    if ($getOccupancyGoal > 0) {
                        $occupancyGoal              = round($getOccupancyGoal / $getActiveDriverCount);
                    }
                    if ($getFleetOccupancy > 0) {
                        $fleetOccupancy             = round($getFleetOccupancy / $getActiveDriverCount);
                    }
                }
                $occupancyGoal                      = (int)min(100, $occupancyGoal);
                $fleetOccupancy                     = (int)min(100, $fleetOccupancy);
            }

            $actualRevenueArry[]                    = $getActualRevenue;
            $targetRevenueArry[]                    = $getTargetRevenue;
            $occupancyGoalArry[]                    = $occupancyGoal;
            $fleetOccupancyArry[]                   = $fleetOccupancy;
        }

        $arry['actualRevenue']                      = json_encode($actualRevenueArry);
        $arry['targetRevenue']                      = json_encode($targetRevenueArry);
        $arry['occupancyGoal']                      = json_encode($occupancyGoalArry);
        $arry['fleetOccupancy']                     = json_encode($fleetOccupancyArry);
        $arry['weeks']                              = json_encode(array_keys($weeks));
        return $arry;
    }

    public function getRevenueActualVsGoal(){
        $user                                       = $this->driver_admin;

        $year                                       = date('Y');
        $currentweek                                = date("W");
        $totalweek                                  = date('W', strtotime($year.'-12-31'));

        $arry                                       = [];
        $actualRevenueArry                          = [];
        $targetRevenueArry                          = [];

        $weeks                                      = [];
        for($i=1;$i<=$currentweek;$i++) {
            $result                                 = $this->getWeek($i,$year);

            $currentDate                            = date('Y-m-d');
            $startDate                              = $result['startDate'];
            $endDate                                = $result['endDate'];
            if(strtotime($currentDate) < strtotime($startDate)){
                break;
            }
            //echo "Week:".$i." Start date:".$result['start']." End date:".$result['end']."<br>";

            $week                                   = [];
            $week['startDate']                      = $startDate;
            $week['endDate']                        = $endDate;
            // if ($i == 1) {
            //     $week['startDate']               = "2022-01-01";
            // } else if ($i == 52) {
            //     $week['endDate']                 = "2022-12-31";
            // }
            $weeks["Week ".$i]                      = $week;

            // week calculation
            $getActualRevenue                       = 0;
            $getTargetRevenue                       = 0;
            $getActiveDriverCount                   = 0;

            $getLoadDriverIds                       = LoadDetail::select('driver_id')
                ->where('driver_id', $user->id)
                ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                ->where('load_type','!=',5)
                ->groupBy('driver_id')
                ->pluck('driver_id')
                ->toArray();
            if(!empty($getLoadDriverIds)){
                $getLoadGrossRevenue                = LoadDetail::select('driver_revenue')
                    ->whereHas('driver', function($q) use($getLoadDriverIds){
                        $q->whereIn('id', $getLoadDriverIds);
                    })
                    ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                    ->whereIn('load_details.is_disable', [0,1])
                    ->where('load_details.load_status', '!=', 9)
                    ->where(function($q) {
                        $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
                    })
                    ->whereNotIn('load_details.load_type', [1, 2, 3, 5]);
                $getActualRevenue                   = $getLoadGrossRevenue->sum('driver_revenue');

                $getTargetRevenueMiles              = $this->getTargetRevenueMilesByDate($startDate, $endDate, $getLoadDriverIds);
                $getTargetRevenue                   = $getTargetRevenueMiles['revenue'];

                $getActiveDriverCount               = count($getLoadDriverIds);
            }

            $actualRevenueArry[]                    = $getActualRevenue;
            $targetRevenueArry[]                    = $getTargetRevenue;
        }

        $arry['actualRevenue']                      = json_encode($actualRevenueArry);
        $arry['targetRevenue']                      = json_encode($targetRevenueArry);
        $arry['weeks']                              = json_encode(array_keys($weeks));
        return $arry;
    }

    public function getRevenueOccupancyVsGoal(){
        $user                                       = $this->driver_admin;

        $year                                       = date('Y');
        $currentweek                                = date("W");
        $totalweek                                  = date('W', strtotime($year.'-12-31'));

        $arry                                       = [];
        $occupancyGoalArry                          = [];
        $fleetOccupancyArry                         = [];

        $weeks                                      = [];
        for($i=1;$i<=$currentweek;$i++) {
            $result                                 = $this->getWeek($i,$year);

            $currentDate                            = date('Y-m-d');
            $startDate                              = $result['startDate'];
            $endDate                                = $result['endDate'];
            if(strtotime($currentDate) < strtotime($startDate)){
                break;
            }
            //echo "Week:".$i." Start date:".$result['start']." End date:".$result['end']."<br>";

            $week                                   = [];
            $week['startDate']                      = $startDate;
            $week['endDate']                        = $endDate;
            // if ($i == 1) {
            //     $week['startDate']               = "2022-01-01";
            // } else if ($i == 52) {
            //     $week['endDate']                 = "2022-12-31";
            // }
            $weeks["Week ".$i]                      = $week;

            // week calculation
            $occupancyGoal                          = 0;
            $fleetOccupancy                         = 0;

            $getOccupancyGoal                       = 0;
            $getFleetOccupancy                      = 0;
            $getActiveDriverCount                   = 0;

            $getLoadDriverIds                       = LoadDetail::select('driver_id')
                ->where('driver_id', $user->id)
                ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
                ->where('load_type','!=',5)
                ->groupBy('driver_id')
                ->pluck('driver_id')
                ->toArray();
            if(!empty($getLoadDriverIds)){
                $getData                            = $this->getLoadOccupancyCalendarMonth2($startDate, $endDate, $getLoadDriverIds);
                $getOccupancy                       = $getData['getOccupancy'];

                $getTargetRevenueMiles              = $this->getTargetRevenueMilesByDate($startDate, $endDate, $getLoadDriverIds);
                $getOccupancyGoal                   = $getTargetRevenueMiles['occupancy'];

                $getFleetOccupancy                  = $getOccupancy;

                $getActiveDriverCount               = count($getLoadDriverIds);
                if($getActiveDriverCount > 0){
                    if ($getOccupancyGoal > 0) {
                        $occupancyGoal              = round($getOccupancyGoal / $getActiveDriverCount);
                    }
                    if ($getFleetOccupancy > 0) {
                        $fleetOccupancy             = round($getFleetOccupancy / $getActiveDriverCount);
                    }
                }
                $occupancyGoal                      = (int)min(100, $occupancyGoal);
                $fleetOccupancy                     = (int)min(100, $fleetOccupancy);
            }

            $occupancyGoalArry[]                    = $occupancyGoal;
            $fleetOccupancyArry[]                   = $fleetOccupancy;
        }

        $arry['occupancyGoal']                      = json_encode($occupancyGoalArry);
        $arry['fleetOccupancy']                     = json_encode($fleetOccupancyArry);
        $arry['weeks']                              = json_encode(array_keys($weeks));
        return $arry;
    }

    public function getEvents(){
        $user                               = $this->driver_admin;
        $userId                             = $user->id;

        $eventArry                          = [];

        $current                            = Carbon::now()->timezone($this->timeZone);
        $currentDate                        = $current->format('Y-m-d');

        $privateEventQuery                  = Event::where('driver_id', $userId)->where('share',0);

        $publicEventQuery                   = Event::where('share',1);

        $driverEventQuery                   = Event::where('driver_id', $userId)->where('share',2);

        $getEvent                           = $privateEventQuery->union($publicEventQuery)->union($driverEventQuery)->get();
        if($getEvent->isNotEmpty()){
            foreach($getEvent as $event) {
                $Checktime                  = Carbon::parse($event->to_date)->format('H:i');
                if($Checktime == "00:00"){
                    $from_date              = Carbon::parse($event->from_date)->format('Y-m-d');
                    $to_date                = Carbon::parse($event->to_date)->format('Y-m-d');
                }else{
                    $from_date              = Carbon::parse($event->from_date)->format('Y-m-d H:i');
                    $to_date                = Carbon::parse($event->to_date)->format('Y-m-d H:i');
                }

                $eventArry[]                = [
                    'title'                 => $event->name,
                    'start'                 => $from_date,
                    'end'                   => $to_date,
                    'icon'                  => $event->fa_icon
                ];

                // Is repeat
                if(isset($event->is_repeat) && $event->is_repeat != 1){
                    //Weeks
                    if($event->is_repeat == 2){
                        for ($x = 1; $x <= 24; $x++) {
                            if($Checktime == "00:00"){
                                $from_date  = Carbon::parse($event->from_date)->addWeeks($x)->format('Y-m-d');
                                $to_date    = Carbon::parse($event->to_date)->addWeeks($x)->format('Y-m-d');
                            }else{
                                $from_date  = Carbon::parse($event->from_date)->addWeeks($x)->format('Y-m-d H:i');
                                $to_date    = Carbon::parse($event->to_date)->addWeeks($x)->format('Y-m-d H:i');
                            }
                            $eventArry[]    = [
                                'title'     => $event->name,
                                'start'     => $from_date,
                                'end'       => $to_date,
                                'icon'      => $event->fa_icon
                            ];
                        }
                    }
                    //Month
                    if($event->is_repeat == 3){
                        for ($x = 1; $x <= 24; $x++) {
                            if($Checktime == "00:00"){
                                $from_date  = Carbon::parse($event->from_date)->addMonths($x)->format('Y-m-d');
                                $to_date    = Carbon::parse($event->to_date)->addMonths($x)->format('Y-m-d');
                            }else{
                                $from_date  = Carbon::parse($event->from_date)->addMonths($x)->format('Y-m-d H:i');
                                $to_date    = Carbon::parse($event->to_date)->addMonths($x)->format('Y-m-d H:i');
                            }
                            $eventArry[]    = [
                                'title'     => $event->name,
                                'start'     => $from_date,
                                'end'       => $to_date,
                                'icon'      => $event->fa_icon
                            ];
                        }
                    }
                    //Six Month
                    if($event->is_repeat == 4){
                        $six = [6,12,18,24,30,36];
                        foreach($six as $sixa) {
                            if($Checktime == "00:00"){
                                $from_date  = Carbon::parse($event->from_date)->addMonths($sixa)->format('Y-m-d');
                                $to_date    = Carbon::parse($event->to_date)->addMonths($sixa)->format('Y-m-d');
                            }else{
                                $from_date  = Carbon::parse($event->from_date)->addMonths($sixa)->format('Y-m-d H:i');
                                $to_date    = Carbon::parse($event->to_date)->addMonths($sixa)->format('Y-m-d H:i');
                            }
                            $eventArry[]    = [
                                'title'     => $event->name,
                                'start'     => $from_date,
                                'end'       => $to_date,
                                'icon'      => $event->fa_icon
                            ];
                        }
                    }
                    //Years
                    if($event->is_repeat == 5){
                        for ($x = 1; $x <= 6; $x++) {
                            if($Checktime == "00:00"){
                                $from_date  = Carbon::parse($event->from_date)->addYear($x)->format('Y-m-d');
                                $to_date    = Carbon::parse($event->to_date)->addYear($x)->format('Y-m-d');
                            }else{
                                $from_date  = Carbon::parse($event->from_date)->addYear($x)->format('Y-m-d H:i');
                                $to_date    = Carbon::parse($event->to_date)->addYear($x)->format('Y-m-d H:i');
                            }
                            $eventArry[]    = [
                                'title'     => $event->name,
                                'start'     => $from_date,
                                'end'       => $to_date,
                                'icon'      => $event->fa_icon
                            ];
                        }
                    }
                }
            }
        }

        //myLeaves
        $myLeaves                           = Leave::where('driver_id', $userId)
            ->whereDate('from_date', '>', $currentDate)
            ->get();
        if($myLeaves->isNotEmpty()){
            foreach($myLeaves as $leave){
                $from_date                  = Carbon::parse($leave->from_date)->format('Y-m-d');
                $to_date                    = Carbon::parse($leave->to_date)->format('Y-m-d');
                $eventArry[]                = [
                    'title'                 => 'Leave - '.$leave->leave_type->name,
                    'start'                 => $from_date,
                    'end'                   => $to_date,
                    'icon'                  => 'fa fa-universal-access'
                ];
            }
        }

        //company event
        $getCompanyDriver                   = Driver::whereId($userId)->first();
        if(!is_null($getCompanyDriver)){
            if(isset($getCompanyDriver->birthday) && $getCompanyDriver->birthday!=''){
                $birthday                   = $getCompanyDriver->birthday;
                $age                        = Carbon::parse($birthday)->age + 1;
                $from_date                  = Carbon::parse($birthday)->addYear($age)->format('Y-m-d');
                $to_date                    = Carbon::parse($birthday)->addYear($age)->format('Y-m-d');
                $eventArry[]                = [
                    'title'                 => $getCompanyDriver->full_name,
                    'start'                 => $from_date,
                    'end'                   => $to_date,
                    'icon'                  => 'fa fa-birthday-cake'
                ];
            }
            if(isset($getCompanyDriver->start_date) && $getCompanyDriver->start_date!=''){
                $start_date                 = $getCompanyDriver->getRawOriginal('start_date');
                $start_date                 = Carbon::parse($start_date)->format('Y-m-d');
                $age                        = Carbon::parse($start_date)->age + 1;
                $from_date                  = Carbon::parse($start_date)->addYear($age)->format('Y-m-d');
                $to_date                    = Carbon::parse($start_date)->addYear($age)->format('Y-m-d');
                $eventArry[]                = [
                    'title'                 => 'Work Anniversary: '.$getCompanyDriver->full_name,
                    'start'                 => $from_date,
                    'end'                   => $to_date,
                    'icon'                  => 'fa fa-birthday-cake'
                ];
            }
            if(isset($getCompanyDriver->driver_compliance_safety->license_exp) && $getCompanyDriver->driver_compliance_safety->license_exp!=''){
                $from_date                  = Carbon::parse($getCompanyDriver->driver_compliance_safety->license_exp)->format('Y-m-d');
                $to_date                    = Carbon::parse($getCompanyDriver->driver_compliance_safety->license_exp)->format('Y-m-d');
                if($from_date > $currentDate){
                    $eventArry[]            = [
                        'title'             => 'Drivers License Exp: '.$getCompanyDriver->full_name,
                        'start'             => $from_date,
                        'end'               => $to_date,
                        'icon'              => 'fa fa-id-card'
                    ];
                }
            }
            if(isset($getCompanyDriver->driver_compliance_safety->tag_exp) && $getCompanyDriver->driver_compliance_safety->tag_exp!=''){
                $from_date                  = Carbon::parse($getCompanyDriver->driver_compliance_safety->tag_exp)->format('Y-m-d');
                $to_date                    = Carbon::parse($getCompanyDriver->driver_compliance_safety->tag_exp)->format('Y-m-d');
                if($from_date > $currentDate){
                    $eventArry[]            = [
                        'title'             => 'Tag Exp: '.$getCompanyDriver->full_name,
                        'start'             => $from_date,
                        'end'               => $to_date,
                        'icon'              => 'fa fa-tag'
                    ];
                }
            }
            if(isset($getCompanyDriver->driver_compliance_safety->passport_expiration) && $getCompanyDriver->driver_compliance_safety->passport_expiration!=''){
                $from_date                  = Carbon::parse($getCompanyDriver->driver_compliance_safety->passport_expiration)->format('Y-m-d');
                $to_date                    = Carbon::parse($getCompanyDriver->driver_compliance_safety->passport_expiration)->format('Y-m-d');
                if($from_date > $currentDate){
                    $eventArry[]            = [
                        'title'             => 'Passport Exp: '.$getCompanyDriver->full_name,
                        'start'             => $from_date,
                        'end'               => $to_date,
                        'icon'              => 'fa fa-id-card'
                    ];
                }
            }
            $getDriverInformation           = DriverInformation::where('driver_id',$getCompanyDriver->id)->get();
            if(!is_null($getDriverInformation)){
                foreach($getDriverInformation as $driverInformation){
                    $getTruck               = Truck::where('unit',$driverInformation->truck_number)->first();
                    if(!is_null($getTruck)){
                        $getTrucklog        = TruckMaintenanceLog::where('truck_id',$getTruck->id)->where('is_completed',0)->get();
                        if($getTrucklog->isNotEmpty()){
                            foreach($getTrucklog as $trucklog){
                                if(isset($trucklog->due_dat) && $trucklog->due_dat!=''){
                                    $from_date      = Carbon::parse($trucklog->due_date)->format('Y-m-d');
                                    $to_date        = Carbon::parse($trucklog->due_date)->format('Y-m-d');
                                    $eventArry[]    = [
                                        'title'     => 'Truck Maintenance: '.$driverInformation->truck_number,
                                        'start'     => $from_date,
                                        'end'       => $to_date,
                                        'icon'      => 'fa fa-truck'
                                    ];
                                }
                            }
                        }
                    }
                    $getTrailer             = Trailer::where('trailer_id',$driverInformation->trailer_number)->first();
                    if(!is_null($getTrailer)){
                        $getTrailerlog      = TrailerMaintenanceLog::where('trailer_id',$getTrailer->id)->where('is_completed',0)->get();
                        if($getTrailerlog->isNotEmpty()){
                            foreach($getTrailerlog as $trailerlog){
                                if(isset($trailerlog->due_dat) && $trailerlog->due_dat!=''){
                                    $from_date      = Carbon::parse($trailerlog->due_date)->format('Y-m-d');
                                    $to_date        = Carbon::parse($trailerlog->due_date)->format('Y-m-d');
                                    $eventArry[]    = [
                                        'title'     => 'Trailer Maintenance: '.$driverInformation->trailer_number,
                                        'start'     => $from_date,
                                        'end'       => $to_date,
                                        'icon'      => 'fa fa-truck'
                                    ];
                                }
                            }
                        }
                    }
                }
            }
        }

        $eventArryFiltered                  = array_filter($eventArry, function ($v) use($currentDate){
            return $v['start'] > $currentDate;
        });
        array_multisort(array_column($eventArryFiltered, "start"), SORT_ASC, $eventArryFiltered);
        $eventArryFiltered                  = array_slice($eventArryFiltered, 0, 5, true);

        $arry                               = array();
        foreach ($eventArryFiltered as $value){
            $arry[date('Y-m-d', strtotime($value['start']))][] = $value;
        }
        return ['events' => $arry];
    }

    public function getCompelAnnouncements(){
        $user                           = $this->driver_admin;

        $announcementArry               = [];
        $data                           = Announcement::where('module','Compel')
            ->where(function($query) use($user) {
                $query->where(function($q) {
                    $q->whereNull('company_id')->whereRaw('FIND_IN_SET(3,show_up)');
                })->orWhere(function($q) use($user) {
                    $q->whereNotNUll('company_id')
                        ->whereRaw('FIND_IN_SET(3,show_up)')
                        ->whereRaw('FIND_IN_SET("'.$user->company_id.'",company_id)');
                });
            });
        $getAnnouncements               = $data->orderBy('id', 'desc')->take(5)->get();
        if($getAnnouncements->isNotEmpty()){
            foreach($getAnnouncements as $announcement) {
                $announcementArry[]     = [
                    'id'                => $announcement->id,
                    'name'              => $announcement->name,
                    'description'       => $announcement->description,
                    'created_at'        => \Carbon\Carbon::createFromFormat('Y-m-d H:i:s', $announcement->created_at)->setTimezone($this->timeZone)->format('Y-m-d H:i:s')
                ];
            }
        }
        array_multisort(array_column($announcementArry, "id"), SORT_ASC, $announcementArry);
        $arry                           = array();
        foreach ($announcementArry as $value){
            $arry[date('Y-m-d', strtotime($value['created_at']))][] = $value;
        }
        return ['announcements' => $arry];
    }

    public function getCompanyAnnouncements(){
        $user                           = $this->driver_admin;

        $announcementArry               = [];
        $data                           = Announcement::where('module','Company')
            ->where(function($query) use($user) {
                $query->where(function($q) use($user) {
                    $q->whereRaw('FIND_IN_SET(3,show_up)')
                        ->whereRaw('FIND_IN_SET("'.$user->company_id.'",company_id)');
                });
            });
        $getAnnouncements               = $data->orderBy('id', 'desc')->take(5)->get();
        if($getAnnouncements->isNotEmpty()){
            foreach($getAnnouncements as $announcement) {
                $announcementArry[]     = [
                    'id'                => $announcement->id,
                    'name'              => $announcement->name,
                    'description'       => $announcement->description,
                    'created_at'        => \Carbon\Carbon::createFromFormat('Y-m-d H:i:s', $announcement->created_at)->setTimezone($this->timeZone)->format('Y-m-d H:i:s')
                ];
            }
        }
        array_multisort(array_column($announcementArry, "id"), SORT_ASC, $announcementArry);
        $arry                           = array();
        foreach ($announcementArry as $value){
            $arry[date('Y-m-d', strtotime($value['created_at']))][] = $value;
        }
        return ['announcements' => $arry];
    }

    public function chartData(Request $request){
        if($request->ajax()){
            try {
                $user                           = $this->driver_admin;

                $formdata                       = $request->all();
                $data                           = [];
                foreach ($formdata['obj'] as $key => $value) {
                    $param                      = [];

                    $widget                     = null;
                    $widgetId                   = $value['id'];
                    $widgetSlug                 = $value['slug'];

                    $data['id']                 = $widgetId;
                    $data['slug']               = $widgetSlug;
                    //chartRange
                    $chartRange                 = isset($value['range'])?$value['range']:null;
                    if(isset($value['load']) && $value['load']==1){
                        $widget                 = Widget::isDriverAdmin()
                            ->where('guard_name', 'driver-admin')
                            ->where('widget_id',$widgetId)
                            ->where('tab', 'own')
                            ->first();
                        if($widget){
                            $chartRange         = $widget->range;
                        }
                    }
                    Widget::isDriverAdmin()
                        ->where('guard_name', 'driver-admin')
                        ->where('widget_id', $widgetId)
                        ->where('tab', 'own')
                        ->update(['range' => $chartRange,'updated_at' => Carbon::now()]);
                    if(in_array($widgetId, [1,2,3,9,10]) && is_null($chartRange)){
                        $chartRange             = 'y';
                        Widget::isDriverAdmin()
                            ->where('guard_name', 'driver-admin')
                            ->where('widget_id', $widgetId)
                            ->where('tab', 'own')
                            ->update(['range' => $chartRange,'updated_at' => Carbon::now()]);
                    }
                    //chartRange

                    if($widgetSlug=='occupancy'){
                        $param['range']                         = $chartRange;
                        $data['chartData']                      = $this->getOccupancy($param);
                    }elseif($widgetSlug=='revenue'){
                        $param['range']                         = $chartRange;
                        $data['chartData']                      = $this->getRevenue($param);
                    }elseif($widgetSlug=='cpm'){
                        $param['range']                         = $chartRange;
                        $data['chartData']                      = $this->getCpm($param);
                    }elseif($widgetSlug=='trailer_type') {
                        $data['chartData']                      = $this->getTrailerType();
                    }elseif($widgetSlug=='target_actual_revenue'){
                        $data['chartData']['isActiveTargetActualRevenue']  = 1;
                    }elseif($widgetSlug=='loads_in_transit'){
                        $totalInTransitLoads                    = LoadDetail::query()->where('driver_id', $user->id)->where('load_status', 4)->where(function($q) {
                            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
                        })->count();
                        $html                                   = view('driverAdmin.home.ajax_loads_in_transit',compact('totalInTransitLoads'))->render();
                        $data['chartData']                      = $html;
                    }elseif($widgetSlug=='revenue_by_trailer_type') {
                        $param['range']                         = $chartRange;
                        $data['chartData']                      = $this->getRevenueByTrailerType($param);
                    }elseif($widgetSlug=='loads_stats'){
                        $getLoadsStats                          = $this->getLoadsStats();
                        $html                                   = view('driverAdmin.home.ajax_loads_stats',compact('getLoadsStats'))->render();
                        $data['chartData']                      = $html;
                    }elseif($widgetSlug=='revenue_occupancy') {
                        $data['chartData']                      = $this->getRevenueOccupancy();
                    }elseif($widgetSlug=='revenue_actual_vs_goal') {
                        $data['chartData']                      = $this->getRevenueActualVsGoal();
                    }elseif($widgetSlug=='revenue_occupancy_vs_goal') {
                        $data['chartData']                      = $this->getRevenueOccupancyVsGoal();
                    }elseif($widgetSlug=='events') {
                        $html                                   = '';
                        $getEventsData                          = $this->getEvents();
                        if(isset($getEventsData['events']) && !empty($getEventsData['events'])){
                            $events                             = $getEventsData['events'];
                            $getTimeZoneId                      = '';//$this->timeZoneId;
                            $html                               = view('driverAdmin.home.ajax_events',compact('events','getTimeZoneId'))->render();
                        }
                        $data['chartData']                      = $html;
                    }elseif($widgetSlug=='compel_announcements') {
                        $html                                   = '';
                        $getCompelAnnouncementsData             = $this->getCompelAnnouncements();
                        if(isset($getCompelAnnouncementsData['announcements']) && !empty($getCompelAnnouncementsData['announcements'])){
                            $announcements                      = $getCompelAnnouncementsData['announcements'];
                            $getTimeZoneId                      = $this->timeZoneId;
                            $html                               = view('driverAdmin.home.ajax_compel_announcements',compact('announcements','getTimeZoneId'))->render();
                        }
                        $data['chartData']                      = $html;
                    }elseif($widgetSlug=='company_announcements') {
                        $html                                   = '';
                        $getCompanyAnnouncementsData            = $this->getCompanyAnnouncements();
                        if(isset($getCompanyAnnouncementsData['announcements']) && !empty($getCompanyAnnouncementsData['announcements'])){
                            $announcements                      = $getCompanyAnnouncementsData['announcements'];
                            $getTimeZoneId                      = $this->timeZoneId;
                            $html                               = view('driverAdmin.home.ajax_company_announcements',compact('announcements','getTimeZoneId'))->render();
                        }
                        $data['chartData']                      = $html;
                    }elseif($widgetSlug=='notes') {
                        $html                                   = view('driverAdmin.home.ajax_notes',compact('widget','param'))->render();
                        $data['chartData']                      = $html;
                    }
                }
                if(!empty($data)){
                    $response['success']                        = true;
                    $response['message']                        = "Chart data retrieved successfully.";
                    $response['data']                           = $data;
                }else{
                    $response['success']                        = false;
                    $response['message']                        = "Oops! Something went wrong..";
                }
            }catch (\Exception $e) {
                $response['success']                            = false;
                $response['message']                            = $e->getMessage();
            }
        }else{
            $response['success']                                = false;
            $response['message']                                = "Oops! Something went wrong..";
        }
        return response()->json($response);
    }

    public function widgetData(Request $request){
        if ($request->ajax()) {
            try {
                $widgetData                         = $this->getWidget();
                if(!empty($widgetData)){
                    $response['widgetData']         = $widgetData;
                    $response['success']            = true;
                    $response['message']            = "Widget retrieved successfully.";
                }else{
                    $response['success']            = false;
                    $response['message']            = "Widget record not found.";
                }
            } catch (\Exception $e) {
                $response['success']                = false;
                $response['message']                = $e->getMessage();
            }
            return response()->json($response);
        }else{
            return abort(404);
        }
    }

    public function widgetDestroy(Request $request){
        if ($request->ajax()) {
            try {
                $widgetData         = [];
                foreach ($request->widgets as $key => $value) {
                    $getWidget      = Widget::isDriverAdmin()
                        ->where('guard_name', 'driver-admin')
                        ->where('widget_id',$value['id'])
                        ->where('tab', 'own')
                        ->first();
                    if($getWidget->delete()){
                        $param                      = [];
                        $param['search']            = $getWidget->widget_id;
                        $param['column']            = 'id';
                        $arry                       = widgetArray($param);
                        if(!is_null($getWidget->properties)){
                            $arry['properties']     = $getWidget->properties;
                        }
                        if(!is_null($getWidget->deleted_at)){ // Is delete
                            $arry['is_delete']      = 1;
                        }
                       $widgetData[$arry['slug']]   = $arry;
                    }
                }
                if(!empty($widgetData)){
                    $response['widgetData']         = $widgetData;
                    $response['success']            = true;
                    $response['message']            = "Widget deleted successfully.";
                }else{
                    $response['success']            = false;
                    $response['message']            = "Widget record not found.";
                }
            } catch (\Exception $e) {
                $response['success']                = false;
                $response['message']                = $e->getMessage();
            }
            return response()->json($response);
        }else{
            return abort(404);
        }
    }

    public function widgetRestore(Request $request){
        if ($request->ajax()) {
            try {
                $getWidget      = Widget::isDriverAdmin()
                    ->where('guard_name', 'driver-admin')
                    ->where('widget_id',$request->id)
                    ->where('tab', 'own')
                    ->withTrashed()
                    ->first();
                if(!is_null($getWidget)){
                    if($getWidget->restore()){
                        $param                                                  = [];
                        $param['search']                                        = $getWidget->widget_id;
                        $param['column']                                        = 'id';
                        $widget                                                 = widgetArray($param);
                        if(!is_null($getWidget->properties)){
                            $widget['properties']                               = $getWidget->properties;
                        }
                        $properties                                             = json_decode($widget['properties'],true);
                        $content                                                = view('driverAdmin.home.ajax_contents',compact('widget'))->render();
                        $properties['content']                                  = $content;
                        $widget['properties']                                   = json_encode($properties);

                        $response['success']                                    = true;
                        $response['message']                                    = "Widget restore successfully.";
                        $response['widget']                                     = $widget;
                    }else{
                        $response['success']                                    = false;
                        $response['message']                                    = "Widget restore unsuccessfully.";
                    }
                }else{
                    $response['success']                                        = false;
                    $response['message']                                        = "Widget record not found.";
                }
            } catch (\Exception $e) {
                $response['success']                                            = false;
                $response['message']                                            = $e->getMessage();
            }
            return response()->json($response);
        }else{
            return abort(404);
        }
    }

    public function widgetNotesStore(Request $request){
        Widget::isDriverAdmin()
            ->where('guard_name', 'driver-admin')
            ->where('widget_id', 16)
            ->where('tab', 'own')
            ->update(['notes' => $request->notes,'updated_at' => Carbon::now()]);
        $response['success'] = true;
        $response['message'] = "Note store successfully.";
        return response()->json($response);
    }

    public function scrollDataInsert(Request $request){
        if ($request->ajax()) {
            try {
                foreach ($request->widgets as $key => $value) {
                    Widget::isDriverAdmin()
                        ->where('guard_name', 'driver-admin')
                        ->where('widget_id', $value['id'])
                        ->where('tab', 'own')
                        ->update(['properties' => json_encode(['x' => $value['x'], 'y' => $value['y'], 'w' => $value['w'], 'h' => $value['h']]),'updated_at' => Carbon::now()]);
                }
                $response['success']                = true;
                $response['message']                = "Widget updated successfully.";
            } catch (\Exception $e) {
                $response['success']                = false;
                $response['message']                = $e->getMessage();
            }
            return response()->json($response);
        }else{
            return abort(404);
        }
    }

    public function getLoadOccupancyCalendarMonth2($startDate, $endDate, $driverIds=[]){
        $loadDetails                            = LoadDetail::whereIn('driver_id', $driverIds)
            ->where(function ($query) use ($endDate, $startDate) {
                $query->where(DB::raw('DATE_FORMAT(load_details.actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                    ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate]);
            })
            ->where('load_details.load_status', '!=', 9)
            ->where(function($q) {
                $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
            })
            ->orderBy('driver_id', 'ASC')
            ->get()->toArray();
        $arry                                   = [];
        if(!empty($loadDetails)){
            $driverWiseLoadDetails              = [];
            foreach ($loadDetails as $lkey => $value) {
                $driverWiseLoadDetails[$value['driver_id']][] = $value;
            }
            //
            foreach ($driverWiseLoadDetails as $dwldkey => $driverLoadDetails) {
                $daycount                       = 0;
                $totalDays                      = 0;

                $dateYMD                        = [];
                $OccupieddateYMD                = [];
                $UnOccupieddateYMD              = [];

                $bcoutn                         = 0;

                foreach ($driverLoadDetails as $dldkey => $load) {
                    $startDateLoad              = Carbon::createFromFormat(getTimeLayout(), $load['actual_shipper_date'])->format('Y-m-d');
                    $endDateLoad                = Carbon::createFromFormat(getTimeLayout(), $load['actual_delivery_date'])->format('Y-m-d');
                    if($startDateLoad <= $startDate){
                        $startDateLoad          = $startDate;
                    }
                    if($endDateLoad >= $endDate){
                        $endDateLoad            = $endDate;
                    }
                    $period                     = CarbonPeriod::create($startDateLoad, $endDateLoad);

                    $dateae                     = 0;
                    foreach ($period as $date) {
                        if($dateae == 0){
                            if($load['occupied_pickup_day'] == 1 || in_array($load['load_type'], [1, 2, 3, 5])){
                                if(!in_array($date->format('m/d/Y'), $dateYMD)){
                                    $dateYMD[]                  = $date->format('m/d/Y');
                                }
                            }
                        }else{
                            if(!in_array($date->format('m/d/Y'), $dateYMD)){
                                $dateYMD[]                      = $date->format('m/d/Y');
                            }
                        }
                        $dateae++;
                    }

                    $cheie                      = 0;
                    if (in_array($load['load_type'], [0, 4, 6])) {
                        foreach ($period as $date) {
                            if($cheie == 0){
                                if($load['occupied_pickup_day'] == 1){
                                    if(!in_array($date->format('m/d/Y'), $OccupieddateYMD)){
                                        $OccupieddateYMD[]      = $date->format('m/d/Y');
                                    }
                                }
                            }else{
                                if(!in_array($date->format('m/d/Y'), $OccupieddateYMD)){
                                    $OccupieddateYMD[]          = $date->format('m/d/Y');
                                }
                            }
                            $cheie++;
                        }
                    }

                    if (in_array($load['load_type'], [1, 2, 3, 5])) {
                        foreach ($period as $date) {
                            if(!in_array($date->format('m/d/Y'), $UnOccupieddateYMD)){
                                $UnOccupieddateYMD[]            = $date->format('m/d/Y');
                            }
                        }
                    }
                }

                foreach($UnOccupieddateYMD as $unOcuDate){
                    if(in_array($unOcuDate, $OccupieddateYMD)){
                        $bcoutn++;
                    }
                }

                $daycount                       = count($dateYMD);
                $totalDays                      = count($OccupieddateYMD);
                $totalDays                      = $totalDays - $bcoutn;

                $mapeval                        = 0;
                if($daycount > 0){
                    $mapeval                    = max(number_format(100 / $daycount * $totalDays, 2), 0);
                }
                $arry[$dwldkey]                 = ['getOccupancy' => $mapeval, 'getLoadGrossRevenue' => $mapeval];
            }
        }
        return [
            'getOccupancy'                      => array_sum(array_column($arry, 'getOccupancy')),
            'getLoadGrossRevenue'               => array_sum(array_column($arry, 'getLoadGrossRevenue'))
        ];
    }

    public function getTargetRevenueMilesByDate($startDate, $endDate, $driverIds=[]){
        $startQuarterDate                       = Carbon::parse($startDate)->startOfQuarter()->format('Y-m-d');
        $endQuarterDate                         = Carbon::parse($endDate)->endOfQuarter()->format('Y-m-d');
        $getDriverFinancials                    = DriverFinancial::select('id','driver_id','target_miles','revenue_target','cpm_target','occupancy','year','quarter','start_date','end_date')
            ->whereIn('driver_id', $driverIds)
            ->whereDate('start_date', '>=', $startQuarterDate)
            ->whereDate('end_date', '<=', $endQuarterDate)
            ->orderBy('year', 'ASC')
            ->orderBy('quarter', 'ASC')
            ->get()->toArray();
        $arry                                   = [];
        if(!empty($getDriverFinancials)){
            $driverWiseFinancials               = [];
            $loadDriverWiseFinancials           = [];
            foreach ($getDriverFinancials as $dfkey => $value) {
                $driverWiseFinancials[$value['driver_id']][] = $value;
            }

            //
            $to_d                               = Carbon::createFromFormat('Y-m-d', $startDate);
            $from_d                             = Carbon::createFromFormat('Y-m-d', $endDate);
            foreach ($driverWiseFinancials as $dwfdkey => $driverFinancials) {
                $loads                          = LoadDetail::selectRaw(
                        'YEAR(actual_shipper_date) AS actual_shipper_date_year,
                        QUARTER(DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")) AS actual_shipper_date_quarter,
                        YEAR(actual_delivery_date) AS actual_delivery_date_year,
                        QUARTER(DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")) AS actual_delivery_date_quarter
                        '
                    )
                    ->where('driver_id', $dwfdkey)
                    ->where(function ($query) use ($driverFinancials) {
                        foreach ($driverFinancials as $key => $value) {
                            if($key==0){
                                $query->where(function ($q) use ($value) {
                                    $q->where(DB::raw('YEAR(actual_shipper_date)'), $value['year'])
                                        ->where(DB::raw('QUARTER(DATE_FORMAT(actual_shipper_date, "%Y-%m-%d"))'), $value['quarter'])
                                        ->where(DB::raw('YEAR(actual_delivery_date)'), $value['year'])
                                        ->where(DB::raw('QUARTER(DATE_FORMAT(actual_delivery_date, "%Y-%m-%d"))'), $value['quarter']);
                                });
                            }else{
                                $query->orWhere(function ($q) use ($value) {
                                    $q->where(DB::raw('YEAR(actual_shipper_date)'), $value['year'])
                                        ->where(DB::raw('QUARTER(DATE_FORMAT(actual_shipper_date, "%Y-%m-%d"))'), $value['quarter'])
                                        ->where(DB::raw('YEAR(actual_delivery_date)'), $value['year'])
                                        ->where(DB::raw('QUARTER(DATE_FORMAT(actual_delivery_date, "%Y-%m-%d"))'), $value['quarter']);
                                });
                            }
                        }
                    })
                    ->where('load_details.load_status', '!=', 9)
                    ->where(function($q) {
                        $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
                    })
                    ->groupBy(['actual_shipper_date_year', 'actual_shipper_date_quarter'])
                    ->groupBy(['actual_delivery_date_year', 'actual_delivery_date_quarter'])
                    ->get()
                    ->toArray();
                $flattened                      = Arr::dot($loads);

                $trevemiles                     = [];
                $treverevenue                   = [];
                foreach ($driverFinancials as $dfdkey => $driverFinancial) {
                    if(in_array($driverFinancial['year'], $flattened) && in_array($driverFinancial['quarter'], $flattened)){
                        $loadDriverWiseFinancials[$dwfdkey][] = $driverFinancial;
                    }

                    $to                         = Carbon::createFromFormat('Y-m-d', $driverFinancial['start_date']);
                    $from                       = Carbon::createFromFormat('Y-m-d', $driverFinancial['end_date']);
                    $quarterdays                = $to->diffInDays($from) + 1;

                    $perdayMiles                = $driverFinancial['target_miles'] / $quarterdays;
                    $perdayRevenue              = $driverFinancial['revenue_target'] / $quarterdays;

                    $period                     = CarbonPeriod::create($driverFinancial['start_date'], $driverFinancial['end_date']);
                    $daycount                   = 1;
                    foreach ($period as $date) {
                        if ($date->between($to_d, $from_d)) {
                            $daycount++;
                        }
                    }

                    if ($daycount > 91) {
                        $daycount = 91;
                    }

                    if ($daycount > 0) {
                        $trevemiles[]           = $perdayMiles * $daycount;
                        $treverevenue[]         = $perdayRevenue * $daycount;
                    }
                }
                $gettotRevenue                  = round(array_sum($treverevenue));
                $gettotMiles                    = round(array_sum($trevemiles));
                $arry[$dwfdkey]['revenue']      = $gettotRevenue;
                $arry[$dwfdkey]['miles']        = $gettotMiles;
            }

            foreach ($loadDriverWiseFinancials as $ldwfdkey => $loadDriverFinancials) {
                $driverTotalQuarter             = count($loadDriverFinancials);
                $driverCpmTarget                = 0;
                $driverOccupancyTarget          = 0;
                foreach ($loadDriverFinancials as $dfdkey => $loadDriverFinancial) {
                    $driverCpmTarget            += $loadDriverFinancial['cpm_target'];
                    $driverOccupancyTarget      += $loadDriverFinancial['occupancy'];
                }
                $driverGoalCpmTarget            = $driverCpmTarget/$driverTotalQuarter;
                $driverGoalOccupancyTarget      = $driverOccupancyTarget/$driverTotalQuarter;

                $arry[$ldwfdkey]['cpm']         = $driverGoalCpmTarget;
                $arry[$ldwfdkey]['occupancy']   = $driverGoalOccupancyTarget;
            }
        }
        return [
            'drivers_wise_target'           => $arry,
            'revenue'                       => array_sum(array_column($arry, 'revenue')),
            'miles'                         => array_sum(array_column($arry, 'miles')),
            'cpm'                           => array_sum(array_column($arry, 'cpm')),
            'occupancy'                     => array_sum(array_column($arry, 'occupancy')),
        ];
    }
}
