<?php

namespace App\Http\Requests\CompanyAdmin;

use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Http\Request;

class DispatcherUpdateRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules(Request $request)
    {
        return [
            'role_id' => 'required',
            'first_name' => 'required',
            'last_name' => 'required',
            'address' => 'required',
            'start_date' => 'required',
            'email' => 'required|email|unique:company_admins,email,'.$request->segment(3).',id,deleted_at,NULL',
            'cell_phone' => 'nullable|required_if:sms,1|size:14',
            'home_phone' => 'nullable|size:14',
            'text_number' => 'nullable|size:14',
        ];
    }

    /**
     * Custom message for validation
     *
     * @return array
     */
    public function messages()
    {
        return [
            'cell_phone.required_if' => 'The cell phone is required when sms is active.',
            'cell_phone.size' => 'The cell phone must be 10 digits.',
            'home_phone.size' => 'The home phone must be 10 digits.',
            'text_number.size' => 'The text number must be 10 digits.',
        ];
    }
}
