<?php

namespace App\Http\Requests\CompanyAdmin;

use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use App\Models\LoadDetail;

class LoadStoreRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules(Request $request)
    {
        return [
            'driver_id'                     => 'required',
            'reason_on_hold'                => 'required_if:load_status,7',
            'shipper_date_time'             => 'required',
            'shipper_date_time_end'         => 'required_if:shipping_time_range,1',
            'delivery_date_time'            => 'required',
            'delivery_date_time_end'        => 'required_if:delivery_time_range,1',
            'brokerage_firm'                => 'required_if:load_type,0',
            'brokerage_mc'                  => 'required_if:load_type,0',
            'load_number'                   => ['required_if:load_type,0',
                function ($attribute, $value, $fail) use ($request) {
                    $loadNumberExist        = LoadDetail::where('load_number', $value)
                        ->where('load_type', 0)
                        ->whereHas('loads', function ($q) {
                            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                        })
                        ->first();
                    if (!is_null($loadNumberExist) && $request->load_type == 0) {
                        $fail('The load number is already taken.');
                    }
                }
            ],
            'weight'                        => 'required_if:load_type,0',
            //'temp'                        => 'required_if:trailer_type,1',
            'trailer_type'                  => 'required_if:load_type,0',
            'truck_number'                  => 'required_if:load_type,0',
            'trailer_number'                => 'required_if:load_type,0',
            'empty_miles'                   => 'required_if:load_type,0',
            'loaded_miles'                  => 'required_if:load_type,0',
            'agreed_upon_rate'              => 'required_if:load_type,0',
            'fuel_surcharge_cost_per_mile'  => 'required_if:load_type,0'
        ];
    }

    /**
     * Custom message for validation
     *
     * @return array
     */
    public function messages()
    {
        return [
            'driver_id.required'                        => 'The driver field is required.',
            'reason_on_hold.required_if'                => 'The reason field is required.',
            'shipper_date_time_end.required_if'         => 'The shipper date time end field is required.',
            'delivery_date_time_end.required_if'        => 'The delivery date time end field is required.',
            'brokerage_firm.required_if'                => 'The brokerage firm field is required.',
            'brokerage_mc.required_if'                  => 'The brokerage mc field is required.',
            'rep.required_if'                           => 'The rep field is required.',
            'phone.required_if'                         => 'The phone field is required.',
            'ext.required_if'                           => 'The ext field is required.',
            'load_number.required_if'                   => 'The load number field is required.',
            'weight.required_if'                        => 'The weight field is required.',
            'temp.required_if'                          => 'The temp field is required.',
            'trailer_type.required_if'                  => 'The trailer type field is required.',
            'truck_number.required_if'                  => 'The truck number field is required.',
            'trailer_number.required_if'                => 'The trailer number field is required.',
            'empty_miles.required_if'                   => 'The empty miles field is required.',
            'loaded_miles.required_if'                  => 'The loaded miles field is required.',
            'agreed_upon_rate.required_if'              => 'The agreed upon rate field is required.',
            'fuel_surcharge_cost_per_mile.required_if'  => 'The fuel surcharge cost per mile field is required.',
            'detention.required_if'                     => 'The detention field is required.',
            'accessorial.required_if'                   => 'The accessorial field is required.',
            'lumper.required_if'                        => 'The lumper field is required.',
        ];
    }
}
