<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;

use Illuminate\Support\Facades\Log;

use App\Models\LoadDetail;
use App\Traits\AuditLoagTrait;

class DeadHeadUpdateJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    use AuditLoagTrait;

    protected $data;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct(array $data)
    {
        $this->data = $data;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        // Access the name from the data passed to the job
        $id                 = $this->data['id'];
        $company_id         = $this->data['company_id'];
        $user_id            = $this->data['user_id'];
        $user_type          = $this->data['user_type'];

        $extraData          = [
            'company_id'    => $company_id,
            'user_id'       => $user_id,
            'user_type'     => $user_type
        ];

        Log::info('Dead head update location job - start - '.$id);

        Log::info(json_encode($extraData));

        $crt_load_detail    = LoadDetail::whereId($id)
            ->whereIn('load_type', [0, 6])
            ->where(function($sub) {
                $sub->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0, 1]);
            })
            ->where(function($sub) {
                $sub->whereNotNull('deadhead_mileage_options')->whereJsonContains('deadhead_mileage_options->type', 1);
            })
            ->where('is_disable', 0)
            ->first();
        if(!is_null($crt_load_detail)){
            if(!is_null($crt_load_detail->driver_id) && $crt_load_detail->driver_id!=0){
                if(!is_null($crt_load_detail->deadhead_mileage_options)){
                    $crt_deadhead_mileage_options                   = json_decode($crt_load_detail->deadhead_mileage_options, true);
                    if(!empty($crt_deadhead_mileage_options) && isset($crt_deadhead_mileage_options['type']) && $crt_deadhead_mileage_options['type']==1){
                        $query                                      = LoadDetail::query();
                        $query->where('driver_id', $crt_load_detail->driver_id)
                            ->whereIn('load_type', [0, 6])
                            ->where(function($sub) {
                                $sub->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0, 1]);
                            })
                            ->where('is_disable', 0)
                            ->whereDate('actual_delivery_date', '>', $crt_load_detail->getRawOriginal('actual_delivery_date'))
                            ->whereDate('delivery_date_time', '>=', $crt_load_detail->getRawOriginal('shipper_date_time'))
                            ->where(function($sub) use ($crt_deadhead_mileage_options) {
                                $sub->whereNotNull('deadhead_mileage_options')
                                    // $condition                   = '{"type":1,"city":"Dallas","state":"TX","zip":"75201"}';
                                    // ->whereJsonContains('deadhead_mileage_options', json_decode($condition, true))
                                    ->whereJsonContains('deadhead_mileage_options->city', $crt_deadhead_mileage_options['city'])
                                    ->whereJsonContains('deadhead_mileage_options->state', $crt_deadhead_mileage_options['state'])
                                    ->whereJsonContains('deadhead_mileage_options->zip', $crt_deadhead_mileage_options['zip'])
                                    ->whereJsonContains('deadhead_mileage_options->type', $crt_deadhead_mileage_options['type']);
                            });
                        $nxt_load_detail                            = $query->first();
                        if(!is_null($nxt_load_detail)){
                            $update_deadhead_mileage_options        = [
                                'type'                              => 1,
                                'abbr'                              => $crt_load_detail->delivery_timezone_abbr,
                                'latitude'                          => $crt_load_detail->delivery_latitude,
                                'longitude'                         => $crt_load_detail->delivery_longitude,
                                'verified'                          => 1,
                                'city'                              => $crt_load_detail->delivery_city,
                                'state'                             => $crt_load_detail->delivery_state,
                                'zip'                               => $crt_load_detail->delivery_zip
                            ];

                            Log::info($crt_load_detail->load_number.' => '.$nxt_load_detail->load_number);

                            $trimble_api_key                        = $crt_load_detail->loads->company->trimble_api_key ?? null;
                            if(!is_null($trimble_api_key)){
                                $stops                              = array();
                                if($update_deadhead_mileage_options['latitude']!='' && $update_deadhead_mileage_options['longitude']!=''){
                                    $stops[]                        = array(
                                        'type'                      => 'Deadhead',
                                        'is_stop'                   => 0,
                                        'latitude'                  => $update_deadhead_mileage_options['latitude'],
                                        'longitude'                 => $update_deadhead_mileage_options['longitude']
                                    );
                                }
                                if($nxt_load_detail->shipper_latitude!='' && $nxt_load_detail->shipper_longitude!=''){
                                    $stops[]                        = array(
                                        'type'                      => 'Pickup',
                                        'is_stop'                   => 0,
                                        'latitude'                  => $nxt_load_detail->shipper_latitude,
                                        'longitude'                 => $nxt_load_detail->shipper_longitude
                                    );
                                }
                                if(isset($nxt_load_detail->loads->load_stops) && $nxt_load_detail->loads->load_stops->isNotEmpty()){
                                    foreach ($nxt_load_detail->loads->load_stops as $key => $load_stop) {
                                        if($load_stop->latitude!='' && $load_stop->longitude!=''){
                                            $stops[]                = array(
                                                'is_stop'           => 1,
                                                'type'              => $load_stop->customer_type == '1' ? 'Pickup' : 'Delivery',
                                                'latitude'          => $load_stop->latitude,
                                                'longitude'         => $load_stop->longitude
                                            );
                                        }
                                    }
                                }
                                if($nxt_load_detail->delivery_latitude!='' && $nxt_load_detail->delivery_longitude!=''){
                                    $stops[]                        = array(
                                        'type'                      => 'Delivery',
                                        'is_stop'                   => 0,
                                        'latitude'                  => $nxt_load_detail->delivery_latitude,
                                        'longitude'                 => $nxt_load_detail->delivery_longitude
                                    );
                                }

                                $keys_values_to_check               = array(
                                    array(
                                        'type'                      => 'Deadhead',
                                        'is_stop'                   => 0
                                    ),
                                    array(
                                        'type'                      => 'Pickup',
                                        'is_stop'                   => 0
                                    ),
                                    array(
                                        'type'                      => 'Delivery',
                                        'is_stop'                   => 0
                                    )
                                );

                                $allExist                           = true;
                                foreach ($keys_values_to_check as $keys_values) {
                                    $found                          = false;
                                    foreach ($stops as $item) {
                                        if (count(array_intersect_assoc($item, $keys_values)) === count($keys_values)) {
                                            $found                  = true;
                                            break;
                                        }
                                    }
                                    if (!$found) {
                                        $allExist                   = false;
                                        break;
                                    }
                                }

                                if ($allExist) {
                                    // "All combinations exist in the array.";

                                    Log::info('Draw map allExist- '.$nxt_load_detail->load_number);

                                    $deadheadKey                    = array_search('Deadhead', array_column($stops, 'type'));

                                    // Get Empty Miles
                                    try {
                                        $coordinatesString          = implode(';', array_map(function($coordinates) {
                                            return $coordinates['longitude'] . ',' . $coordinates['latitude'];
                                        }, $stops));

                                        $tollRoads                  = 2;
                                        if(!is_null($nxt_load_detail->map_toll_roads)){
                                            $tollRoads              = (int) $nxt_load_detail->map_toll_roads;
                                        }

                                        $elevLimit                  = 0;
                                        if(!is_null($nxt_load_detail->map_max_elevation)){
                                            $elevLimit              = (int) $nxt_load_detail->map_max_elevation;
                                        }

                                        $avoidCountry               = '';
                                        if($nxt_load_detail->map_close_borders==1){
                                            $avoidCountry           = 'CA,MX';
                                        }

                                        $hazMat                     = 0;
                                        if(!is_null($nxt_load_detail->hazardous_load)==1){
                                            $hazMat                 = $nxt_load_detail->hazardous_options ?? 0;
                                        }

                                        $maparray                   = [
                                            'stops'                 => $coordinatesString,
                                            'reports'               => 'Mileage',
                                            'tollRoads'             => $tollRoads,
                                            'elevLimit'             => $elevLimit,
                                            'avoidCountry'          => $avoidCountry,
                                            'hazMat'                => $hazMat, // HazMatType, HazMatTypes
                                        ];

                                        $mapparam                   = array();
                                        foreach($maparray as $k => $v){
                                            $mapparam[]             = $k . '=' . urlencode($v);
                                        }
                                        $mapparam                   = implode('&', $mapparam);

                                        $url                        = 'https://pcmiler.alk.com/apis/rest/v1.0/Service.svc/route/routeReports?'.$mapparam;
                                        $client                     = new \GuzzleHttp\Client();
                                        $api_response               = $client->request('GET', $url, [
                                            'headers'               => [
                                                'Accept'            => 'application/json',
                                                'Content-Type'      => 'application/json',
                                                'Authorization'     => $trimble_api_key
                                            ],
                                        ]);

                                        // Check if response status code is OK (200)
                                        if ($api_response->getStatusCode() == 200) {
                                            $body                   = json_decode($api_response->getBody());
                                            if(isset($body)){
                                                if(isset($body[0]->ReportLines[1]->TMiles) && $body[0]->ReportLines[1]->TMiles!=''){
                                                    $empty_miles    = round($body[0]->ReportLines[1]->TMiles); // (float) number_format($body[0]->ReportLines[1]->TMiles, 2, '.', '');

                                                    Log::info('Empty miles retrieved successfully - '.$empty_miles.' ( '.$body[0]->ReportLines[1]->TMiles.' )');
                                                }
                                            }
                                        }
                                    } catch (\GuzzleHttp\Exception\ClientException $e) {
                                        // Handle client-side errors (e.g. 4xx status codes)
                                        $api_response               = $e->getResponse();
                                        if ($api_response) {
                                            $statusCode             = $api_response->getStatusCode();
                                            $body                   = $api_response->getBody()->getContents();
                                            Log::error($body);
                                        } else {
                                            Log::error($e->getMessage());
                                        }
                                    } catch (\GuzzleHttp\Exception\ServerException $e) {
                                        // Handle server-side errors (e.g. 5xx status codes)
                                        $api_response               = $e->getResponse();
                                        if ($api_response) {
                                            $statusCode             = $api_response->getStatusCode();
                                            $body                   = $api_response->getBody()->getContents();
                                            Log::error($body);
                                        } else {
                                            Log::error($e->getMessage());
                                        }
                                    } catch (\Exception $e) {
                                        // Handle other exceptions
                                        Log::error($e->getMessage());
                                    }
                                    //

                                    // Get Trip Details
                                    if(isset($nxt_load_detail->load_trip_id)){
                                        try {
                                            $url                    = 'https://tripmanagement.alk.com/api/trip?tripId='.$nxt_load_detail->load_trip_id;
                                            $client                 = new \GuzzleHttp\Client();
                                            $api_response           = $client->request('GET', $url, [
                                                'headers'           => [
                                                    'Accept'        => 'application/json',
                                                    'Content-Type'  => 'application/json',
                                                    'Authorization' => $trimble_api_key
                                                ],
                                            ]);
                                            // Check if response status code is OK (200)
                                            if ($api_response->getStatusCode() == 200) {
                                                $body               = json_decode($api_response->getBody());
                                                // Do something with the response body
                                                if(isset($body)){
                                                    $tripDetails    = $body;
                                                    if(isset($tripDetails)){
                                                        Log::info('Trip details retrieved successfully.');

                                                        // $timestamp                       = "2023-07-31T07:17:10+00:00";
                                                        // $dateTime                        = Carbon::createFromFormat('Y-m-d\TH:i:sP', $timestamp)->format('Y-m-d H:i:s');
                                                        // dd($dateTime);

                                                        // Update Trip Details
                                                        if(isset($tripDetails->stops) && !empty($tripDetails->stops)){
                                                            $tripstops                      = [];

                                                            foreach ($tripDetails->stops as $key => $tripstop) {
                                                                // Convert object to array
                                                                // $location                = json_decode(json_encode($tripstop->location), true);
                                                                $arry                       = [];
                                                                if($key==0){
                                                                    $location               = [
                                                                        'Coords'            => [
                                                                            'lat'           => $stops[0]['latitude'],
                                                                            'lon'           => $stops[0]['longitude']
                                                                        ],
                                                                        'Label'             => $tripstop->location->label
                                                                    ];
                                                                }else{
                                                                    $location               = [
                                                                        'Coords'            => [
                                                                            'lat'           => $tripstop->location->coords->lat,
                                                                            'lon'           => $tripstop->location->coords->lon,
                                                                        ],
                                                                        'Label'             => $tripstop->location->label
                                                                    ];
                                                                }
                                                                $arry['Location']           = $location;
                                                                $arry['stopType']           = $tripstop->stopType;
                                                                $arry['stopSequence']       = $tripstop->stopSequence;
                                                                $arry['earliestArrivalTime']= $tripstop->earliestArrivalTime;
                                                                $arry['latestArrivalTime']  = $tripstop->latestArrivalTime;
                                                                $tripstops[]                = $arry;
                                                            }

                                                            Log::info($stops[0]['latitude'].', '.$stops[0]['longitude']);

                                                            $triparray                      = [
                                                                'tripId'                    => $nxt_load_detail->load_trip_id,
                                                                // 'plannedStartTime'       => $tripDetails->plannedStartTime,
                                                                'plannedStartLocation'      => [
                                                                    'Coords'                => [
                                                                        'Lat'               => $stops[0]['latitude'],
                                                                        'Lon'               => $stops[0]['longitude']
                                                                    ]
                                                                ],
                                                                // 'externalOrderIds'       => [],
                                                                'stops'                     => $tripstops,
                                                                'routingProfile'            => [
                                                                    'tollDiscouraged'       => $tripDetails->routingProfile->tollDiscouraged,
                                                                    'elevationLimit'        => $tripDetails->routingProfile->elevationLimit,
                                                                    'bordersOpen'           => $tripDetails->routingProfile->bordersOpen,
                                                                    'hazmatType'            => $tripDetails->routingProfile->hazmatType,
                                                                    'truckDimensions'       => $tripDetails->routingProfile->truckDimensions,
                                                                    'totalLength'           => $tripDetails->routingProfile->totalLength,
                                                                    'maxWidth'              => $tripDetails->routingProfile->maxWidth,
                                                                    'maxHeight'             => $tripDetails->routingProfile->maxHeight,
                                                                    'totalWeight'           => $tripDetails->routingProfile->totalWeight
                                                                ]
                                                            ];

                                                            try {
                                                                $url                        = 'https://tripmanagement.alk.com/api/trip/modify';
                                                                $client                     = new \GuzzleHttp\Client();
                                                                $api_response               = $client->request('PUT', $url, [
                                                                    'headers'               => [
                                                                        'Accept'            => 'application/json',
                                                                        'Content-Type'      => 'application/json',
                                                                        'Authorization'     => $trimble_api_key
                                                                    ],
                                                                    'json'                  => $triparray
                                                                ]);
                                                                // Check if response status code is OK (200)
                                                                if ($api_response->getStatusCode() == 200) {
                                                                    $body               = json_decode($api_response->getBody());
                                                                    if(isset($body)){
                                                                        Log::info('Trip updated successfully.');
                                                                    }
                                                                }
                                                            } catch (\Exception $e) {
                                                                // Handle other exceptions
                                                                Log::error($e->getMessage());
                                                            }
                                                        }
                                                        //
                                                    }
                                                }
                                            }
                                        } catch (\GuzzleHttp\Exception\ClientException $e) {
                                            // Handle client-side errors (e.g. 4xx status codes)
                                            $api_response           = $e->getResponse();
                                            if ($api_response) {
                                                $statusCode         = $api_response->getStatusCode();
                                                $body               = $api_response->getBody()->getContents();
                                                Log::error($body);
                                            } else {
                                                Log::error($e->getMessage());
                                            }
                                        } catch (\GuzzleHttp\Exception\ServerException $e) {
                                            // Handle server-side errors (e.g. 5xx status codes)
                                            $api_response           = $e->getResponse();
                                            if ($api_response) {
                                                $statusCode         = $api_response->getStatusCode();
                                                $body               = $api_response->getBody()->getContents();
                                                Log::error($body);
                                            } else {
                                                Log::error($e->getMessage());
                                            }
                                        } catch (\Exception $e) {
                                            // Handle other exceptions
                                            Log::error($e->getMessage());
                                        }
                                    }
                                    //
                                }
                            }

                            //
                            $oldData                                = $nxt_load_detail->toArray();

                            $empty_miles                            = $empty_miles ?? 0;
                            $nxt_load_detail->deadhead_mileage_options = json_encode($update_deadhead_mileage_options); // $crt_load_detail->deadhead_mileage_options;
                            $nxt_load_detail->empty_miles           = $empty_miles;
                            $nxt_load_detail->total_miles           = $empty_miles+$nxt_load_detail->loaded_miles;
                            if($nxt_load_detail->save()){
                                Log::info('Load detail updated');
                                $this->setLoadAudit($nxt_load_detail->load_id, 'update', 'Load detail updated', $nxt_load_detail->getChanges(), $oldData, $extraData);
                            }

                            $load_drivers                           = $nxt_load_detail->load_drivers()->where('driver_id', $nxt_load_detail->driver_id)->withoutTrashed()->get();
                            if($load_drivers->isNotEmpty()){
                                foreach ($load_drivers as $key => $load_driver) {
                                    $oldData                        = $load_driver->toArray();
                                    $load_driver->empty_miles       = $empty_miles;
                                    $load_driver->total_miles       = $empty_miles+$load_driver->loaded_miles;
                                    if($load_driver->save()){
                                        Log::info('Load driver updated');
                                        $this->setLoadAudit($load_driver->load_id, 'update', 'Load driver updated', $load_driver->getChanges(), $oldData, $extraData);
                                    }
                                }
                            }

                            $load_truck_owners                      = $nxt_load_detail->load_truck_owners()->where('driver_id', $nxt_load_detail->driver_id)->withoutTrashed()->get();
                            if($load_truck_owners->isNotEmpty()){
                                foreach ($load_truck_owners as $key => $load_truck_owner) {
                                    $oldData                        = $load_truck_owner->toArray();
                                    $load_truck_owner->empty_miles  = $empty_miles;
                                    $load_truck_owner->total_miles  = $empty_miles+$load_driver->loaded_miles;
                                    if($load_truck_owner->save()){
                                        Log::info('Load truck owner updated');
                                        $this->setLoadAudit($load_truck_owner->load_id, 'update', 'Load truck owner updated', $load_truck_owner->getChanges(), $oldData, $extraData);
                                    }
                                }
                            }
                            //

                            Log::info($crt_load_detail->deadhead_mileage_options.' => '.json_encode($update_deadhead_mileage_options));
                        }
                    }
                }
            }
        }

        Log::info('Dead head update location job - stop - '.$id);
    }
}
