<?php

namespace App\Mail;

use App\Models\CompanyEmailTemplate;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Mail\Mailable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;

class AppointmentRequestBroker extends Mailable
{
    use Queueable, SerializesModels;

    public $details;

    /**
     * Create a new message instance.
     *
     * @return void
     */
    public function __construct($details)
    {
        $this->details      = $details;
        $this->details['app_name'] = config('app.name');
    }

    /**
     * Build the message.
     *
     * @return $this
     */
    public function build()
    {
        /*
        $load_number        = $this->details['appointment']->load_detail->load_number ?? '#########';
        return $this->subject('Dispatch Pro - Appointment Request for load number: '.$load_number)
            ->markdown('emails.appointment.request_broker')
            ->with([
                'details'   => $this->details,
                'link'      => route('company.admin.appointment.request.redirect', [
                    'code'  => $this->details['short_code']
                ])
            ]);
        */

        $companyId = $this->details['appointment']->load_detail->loads->company_id ?? '';

        if($companyId != ''){
            try {
                $getTemplate = CompanyEmailTemplate::where('company_id', $companyId)->where('title', 'Appointment Request Broker')->first();

                if(!is_null($getTemplate)){
                    $load_number = $this->details['appointment']->load_detail->load_number ?? '#########';
                    $link = route('company.admin.appointment.request.redirect', ['code'  => $this->details['short_code']]);

                    $stop_name = '';
                    if($this->details['appointment']->is_stop == 1){
                        $stop_no = $this->details['appointment']->stop->stop_no;
                        if($this->details['appointment']->type == 1){
                            $stop_name = 'Pickup - Stop '.($stop_no+1);
                        }else{
                            $stop_name = 'Delivery - Stop '.($stop_no+1);
                        }
                    }else{
                        if($this->details['appointment']->type == 1){
                            $stop_name = 'Pickup - Stop 1';
                        }else{
                            $stop_name = 'Delivery - Stop 99';
                        }
                    }

                    $subject = str_replace('{%load_number%}', $load_number, $getTemplate->subject);
                    $emailTemplate = getCompanyEmailTemplate($getTemplate);
                    $emailTemplate = str_replace(
                        [
                            '{%name%}',
                            '{%stop_name%}',
                            '{%link%}',
                            '{%app_name%}'
                        ],
                        [
                            $this->details['name'] ?? '',
                            $stop_name,
                            $link ?? '',
                            $this->details['app_name'] ?? ''
                        ],
                        $emailTemplate
                    );

                    return $this->subject($subject)->html($emailTemplate);
                }
            } catch (\Exception $e) {
                // Handle other exceptions
                Log::error($e->getMessage());

                return $e->getMessage();
            }
        }
    }
}
