<?php

namespace App\Mail\CompanyAdmin;

use App\Models\CompanyEmailTemplate;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Mail\Mailable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;

class SendLoadInvoice extends Mailable
{
    use Queueable, SerializesModels;

    public $data;

    /**
     * Create a new message instance.
     *
     * @return void
     */
    public function __construct($data)
    {
        $this->data = $data;
        $this->data['app_name'] = config('app.name');
    }

    /**
     * Build the message.
     *
     * @return $this
     */
    public function build()
    {
        try {
            $invoiceLink        = route('company.admin.broker.invoice.view', [
                    'type'      => 1, // Type => // 0 = None, 1 = Email, 2 = SMS, 3 = Scan QR, 4 = Shared Link
                    'id'        => base64Encode($this->data['invoice_id']),
                    'format'    => 0 // Format => // 0 = Stream, 1 = Save means Download, 2 = Print
                ]
            );

            $getTemplate        = CompanyEmailTemplate::where('company_id', $this->data['company_id'])->where('title', 'Send Load Invoice Broker')->first();

            if(!is_null($getTemplate)){
                $subject        = str_replace(
                    [
                        '{%invoice_number%}',
                        '{%load_number%}'
                    ],
                    [
                        $this->data['invoice_number'] ?? '',
                        $this->data['load_number']
                    ],
                    $getTemplate->subject
                );
                $emailTemplate  = getCompanyEmailTemplate($getTemplate);
                $emailTemplate  = str_replace(
                    [
                        '{%name%}',
                        '{%invoice_number%}',
                        '{%invoice_due_date%}',
                        '{%invoice_amount%}',
                        '{%link%}',
                        '{%app_name%}'
                    ],
                    [
                        $this->data['name'] ?? '',
                        $this->data['invoice_number'] ?? '',
                        $this->data['invoice_due_date'] ?? '',
                        $this->data['invoice_amount'] ?? '',
                        $invoiceLink ?? '',
                        $this->data['app_name'] ?? ''
                    ],
                    $emailTemplate
                );

                return $this->subject($subject)->html($emailTemplate);
            }
        } catch (\Exception $e) {
            // Handle other exceptions
            Log::error($e->getMessage());
            return $e->getMessage();
        }
    }
}
