<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Illuminate\Database\Eloquent\SoftDeletes;
use Carbon\Carbon;

class Broker extends Model
{
    use HasFactory, SoftDeletes;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'company_id',
        'name',
        'mc_number',
        'rep_name',
        'phone_number',
        'ext',
        'eight_hundred_number',
        'ap_contact_contact_ext',
        'ap_contact_phone',
        'ap_contact_email',
        'ap_contact_name',
        'dispatch_contact_contact_ext',
        'dispatch_contact_phone',
        'dispatch_contact_email',
        'dispatch_contact_name',
        'lumper_receipts',
        'purchase_order',
        'scale_ticket',
        'delivery_receipt',
        'packing_slip',
        'bill_of_lading',
        'factored',
        'billing_terms',

        'bill_to_name',

        'bill_address_search',
        'billing_address',
        'billing_address_2',
        'billing_city',
        'billing_state',
        'billing_zip',
        'billing_latitude',
        'billing_longitude',
        'billing_timezone',
        'billing_timezone_offset',
        'billing_timezone_abbr',
        'billing_address_verified',

        'billing_phone',
        'billing_extention',
        'billing_email',
        'billing_fax',
        'billing_method',

        'logo',
        'account_type',
        'account_number',
        'scac',
        'usdot',
        'status',
        'federal_id',
        'fax',
        'email',
        'phone',

        'address_search',
        'address',
        'address_2',
        'city',
        'state',
        'zip',
        'latitude',
        'longitude',
        'timezone',
        'timezone_offset',
        'timezone_abbr',
        'address_verified',

        'shipper_files',
        'consignee_files',
        'detention_price',
        'detention_time',

        'customer_unique_id',
        'customer_type',
        'customer_dba',
        'duns_number',
        'duns_expiration_date',
        'c_tpat_certified',
        'c_tpat_svi_number',
        'broker_loads',
        'sourcing',
        'performance_review_frequency',
        'website',
        'afterhours',
        'office_type',
        'parent_company',

        'fuel_surcharge_diesel',
        'fuel_surcharge_diesel_file',
        'fuel_surcharge_gasoline',
        'fuel_surcharge_gasoline_file',
        'ho_sun_open',
        'ho_sun_close',
        'ho_mon_open',
        'ho_mon_close',
        'ho_tue_open',
        'ho_tue_close',
        'ho_wed_open',
        'ho_wed_close',
        'ho_thu_open',
        'ho_thu_close',
        'ho_fri_open',
        'ho_fri_close',
        'ho_sat_open',
        'ho_sat_close',
        'fmcsa_unsafe',
        'fmcsa_crash',
        'fmcsa_hos',
        'fmcsa_maintenance',
        'fmcsa_substances',
        'fmcsa_hazardous',
        'fmcsa_driver_fitness',
        'fmcsa_insurance',

        'billing_id',
        'payment_method',
        'billing_afterhours',
        'customer_assignment',
        'terminal_assignment',
        'customer_service_representative',
        'customer_development_representative',
        'customer_sales_representative',
        'billing_representative',
        'claims_representative',
        'collection_representative',
        'factor_invoices',
        'factor_company',
        'factor_charge_type',
        'factor_rate',
        'minimum_charge',
        'term_start_date',
        'terms',
        'freight_bill_format',
        'first_credit_check',
        'last_credit_check',
        'credit_score',
        'credit_limit',
        'credit_warning',
        'credit_balance',
        'credit_app_completed',
        'credit_app_expiration',
        'required_safer_status',
        'required_authority_tenure',
        'required_fleet_size',
        'required_loads_last_30_days',
        'required_loads_per_month',

        'standard_notification',
        'geofense_notification',
        'edi_notification',
        'integration_notification',

        'geofence_status',
        'geofence_trimble_place_id',
        'geofence_updated_at',

        'customer_logo',
    ];

    /*
     * Scopes
     */
    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    public function setDetentionTimeAttribute($value): ?string
    {
        return $this->attributes['detention_time'] = !is_null($value) ? Carbon::createFromFormat('H:i',$value)->format('H:i:s') : NULL;
    }

    public function getDetentionTimeAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format('H:i') : NULL;
    }

    public function company(){
        return $this->belongsTo(Company::class);
    }

    public function broker_accessorials()
    {
        return $this->hasMany(BrokerAccessorial::class);
    }

    public function broker_offices()
    {
        return $this->hasMany(BrokerOffice::class);
    }

    public function broker_contact()
    {
        return $this->hasMany(BrokerContact::class, 'action_id', 'id')->where('source', 'broker');
    }

    public function setDunsExpirationDateAttribute($value): ?string {
        return $this->attributes['duns_expiration_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getDunsExpirationDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function setFirstCreditCheckAttribute($value): ?string {
        return $this->attributes['first_credit_check'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getFirstCreditCheckAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function setLastCreditCheckAttribute($value): ?string {
        return $this->attributes['last_credit_check'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getLastCreditCheckAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function setCreditAppCompletedAttribute($value): ?string {
        return $this->attributes['credit_app_completed'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getCreditAppCompletedAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function setCreditAppExpirationAttribute($value): ?string {
        return $this->attributes['credit_app_expiration'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getCreditAppExpirationAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function broker_carrier_insurances()
    {
        return $this->hasOne(BrokerCarrierInsurance::class);
    }

    public function broker_customer_insurances()
    {
        return $this->hasOne(BrokerCustomerInsurance::class);
    }

    public function broker_bond_information()
    {
        return $this->hasOne(BrokerBondInformation::class);
    }

    public function broker_required_documents()
    {
        return $this->hasOne(BrokerRequiredDocument::class);
    }

    public function broker_additional_required_documents()
    {
        return $this->hasOne(BrokerAdditionalRequiredDocument::class);
    }

    public function setStandardNotificationAttribute($value): ?string {
        return $this->attributes['standard_notification'] = (!empty($value) && is_array($value)) ? serialize($value) : NULL;
    }

    public function getStandardNotificationAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : [];
    }

    public function setGeofenseNotificationAttribute($value): ?string {
        return $this->attributes['geofense_notification'] = (!empty($value) && is_array($value)) ? serialize($value) : NULL;
    }

    public function getGeofenseNotificationAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : [];
    }

    public function setEdiNotificationAttribute($value): ?string {
        return $this->attributes['edi_notification'] = (!empty($value) && is_array($value)) ? serialize($value) : NULL;
    }

    public function getEdiNotificationAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : [];
    }

    public function setIntegrationNotificationAttribute($value): ?string {
        return $this->attributes['integration_notification'] = (!empty($value) && is_array($value)) ? serialize($value) : NULL;
    }

    public function getIntegrationNotificationAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : [];
    }

    public function customer_notes()
    {
        return $this->hasMany(CustomerNote::class)->orderBy('id', 'ASC');
    }

    public function internal_notes()
    {
        return $this->hasMany(InternalNote::class)->orderBy('id', 'ASC');
    }

    public function customer_files($section)
    {
        return $this->hasMany(CustomerFiles::class)->where('section', $section)->orderBy('id', 'ASC');
    }

    public function setGeofenceUpdatedAtAttribute($value): ?string {
        return $this->attributes['geofence_updated_at'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getGeofenceUpdatedAtAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function galleries(){
        return $this->hasMany(Gallery::class,'action_id','id')->where('source', 'brokers');
    }

    public function broker_company_images()
    {
        return $this->hasMany(BrokerCompanyImage::class);
    }

    public function billingFullAddress() {
        $billingFullAddress = '';
        if($this->attributes['billing_address'] != ''){
            $billingFullAddress .= $this->attributes['billing_address'].',<br>';
        }
        if($this->attributes['billing_address_2'] != ''){
            $billingFullAddress .= $this->attributes['billing_address_2'].', ';
        }
        if($this->attributes['billing_city'] != ''){
            $billingFullAddress .= $this->attributes['billing_city'].', ';
        }
        if($this->attributes['billing_state'] != ''){
            $billingFullAddress .= $this->attributes['billing_state'].', ';
        }
        if($this->attributes['billing_zip'] != ''){
            $billingFullAddress .= $this->attributes['billing_zip'].', ';
        }
        return $billingFullAddress;
    }
}
