<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Illuminate\Database\Eloquent\SoftDeletes;

class BrokerContact extends Model
{
    use HasFactory, SoftDeletes;

    protected $table    = 'broker_contacts';
    protected $fillable = [
        'source',
        'action_id',
        'office_id',
        'department',
        'name',
        'first_name',
        'middle_name',
        'last_name',
        'phone_number',
        'cell_number',
        'ext',
        'fax',
        'email',
        'preferred_connections',
        'date_of_birth',
        'start_date_work_anniversary',
        'gender',
        'suffix',
        'primary_contact',
        'primary_contact_id',
        'include_department_address',
        'is_invite',
        'pro_chat_id',
        'contact_status',
        'representative_profile_img',
        'is_representative',
        'rating',
        'created_at',
        'created_by',
        'updted_at',
        'updaated_by',
        'deleted_at',
    ];

    protected $dates    = ['deleted_at'];

    public function broker(){
        return $this->hasOne(Broker::class, 'id', 'action_id')->where('source', 'broker')->withTrashed();
    }

    public function setDateOfBirthAttribute($value): ?string {
        return $this->attributes['date_of_birth'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getDateOfBirthAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function setStartDateWorkAnniversaryAttribute($value): ?string {
        return $this->attributes['start_date_work_anniversary'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getStartDateWorkAnniversaryAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function setDepartmentAttribute($value): ?string {
        if($this->attributes['source'] == 'terminal'){
            return $this->attributes['department'] = $value ?? NULL;
        }else{
            if(!is_null($value)){
                $value = array_map('strtolower', $value);
            }
            return $this->attributes['department'] = (is_array($value)) ? implode(',', $value) : NULL;
        }
    }

    public function getDepartmentAttribute($value): ?array {
        return !is_null($value) ? explode(',', strtolower($value)) : [];
    }
}
