<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class BrokerRequiredDocument extends Model
{
    use HasFactory;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'company_id', 'broker_id', 'required_document_profile', 'shipper_documents', 'shipper_doc_types', 'shipper_include_w_billing', 'shipper_hide_from_driver', 'receiver_documents', 'receiver_doc_types', 'receiver_include_w_billing', 'receiver_hide_from_driver'
    ];

    /*
     * Scopes
     */
    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    public function setShipperDocumentsAttribute($value): ?string{
        return $this->attributes['shipper_documents'] = is_array($value) ? serialize($value) : NULL;
    }

    public function getShipperDocumentsAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : NULL;
    }

    public function setShipperDocTypesAttribute($value): ?string{
        return $this->attributes['shipper_doc_types'] = is_array($value) ? serialize($value) : NULL;
    }

    public function getShipperDocTypesAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : NULL;
    }

    public function setShipperIncludeWBillingAttribute($value): ?string{
        return $this->attributes['shipper_include_w_billing'] = is_array($value) ? serialize($value) : NULL;
    }

    public function getShipperIncludeWBillingAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : NULL;
    }

    public function setShipperHideFromDriverAttribute($value): ?string{
        return $this->attributes['shipper_hide_from_driver'] = is_array($value) ? serialize($value) : NULL;
    }

    public function getShipperHideFromDriverAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : NULL;
    }

    public function setReceiverDocumentsAttribute($value): ?string{
        return $this->attributes['receiver_documents'] = is_array($value) ? serialize($value) : NULL;
    }

    public function getReceiverDocumentsAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : NULL;
    }

    public function setReceiverDocTypesAttribute($value): ?string{
        return $this->attributes['receiver_doc_types'] = is_array($value) ? serialize($value) : NULL;
    }

    public function getReceiverDocTypesAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : NULL;
    }

    public function setReceiverIncludeWBillingAttribute($value): ?string{
        return $this->attributes['receiver_include_w_billing'] = is_array($value) ? serialize($value) : NULL;
    }

    public function getReceiverIncludeWBillingAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : NULL;
    }

    public function setReceiverHideFromDriverAttribute($value): ?string{
        return $this->attributes['receiver_hide_from_driver'] = is_array($value) ? serialize($value) : NULL;
    }

    public function getReceiverHideFromDriverAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : NULL;
    }
}
