<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Illuminate\Database\Eloquent\SoftDeletes;
use App\Models\CarrierContact;

class Carrier extends Model
{
    use HasFactory, SoftDeletes;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'company_id',
        'name',
        'mc_number',
        'address',
        'address_2',
        'city',
        'state',
        'zip',
        'latitude',
        'longitude',
        'timezone',
        'timezone_offset',
        'timezone_abbr',
        'address_verified',
        'identifier',
        'desired_option',
        'desired_load_number'
    ];

    public static function boot()
    {
        parent::boot();

        static::deleting(function($item) {
            $item->carrier_contacts()->delete();
        });
    }

    /*
     * Scopes
     */
    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    /*
     * Relationships
     */
    public function carrier_contacts()
    {
        return $this->hasMany(CarrierContact::class);
    }

    public function terminals()
    {
        return $this->hasMany(Terminal::class);
    }

}
