<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Carbon\Carbon;

class CompanyHoliday extends Model
{
    use HasFactory;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'company_id', 'name', 'from_date', 'to_date'
    ];

    /*
     * Scopes
     */
    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    public function setFromDateAttribute($value): ?string
    {
        return $this->attributes['from_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setToDateAttribute($value): ?string
    {
        return $this->attributes['to_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getFromDateAttribute($value): ?string
    {
        return Carbon::parse($value)->format(config('settings.date_format','m/d/Y'));
    }

    public function getToDateAttribute($value): ?string
    {
        return Carbon::parse($value)->format(config('settings.date_format','m/d/Y'));
    }


}
