<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class CustomerFiles extends Model
{
    use HasFactory;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'company_id', 'broker_id', 'section', 'name', 'status', 'start_date', 'end_date', 'submitted_to', 'submission_date', 'submitted_by'
    ];

    /*
     * Scopes
     */
    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    public function setStartDateAttribute($value): ?string {
        return $this->attributes['start_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getStartDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function setEndDateAttribute($value): ?string {
        return $this->attributes['end_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getEndDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function setSubmissionDateAttribute($value): ?string {
        return $this->attributes['submission_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getSubmissionDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function setSubmittedToAttribute($value): ?string {
        return $this->attributes['submitted_to'] = is_array($value) ? implode(',', $value) : NULL;
    }

    public function getSubmittedToAttribute($value): ?array {
        return !is_null($value) ? explode(',', $value) : [];
    }
}
