<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Carbon\Carbon;

class DriverComplianceSafety extends Model
{
    use HasFactory, SoftDeletes;

    protected $table = 'driver_compliance_safety';

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'driver_id', 'license_exp', 'license_exp_file', 'mvr_date', 'anniversary', 'medical_cert', 'schedule_orientation', 'birthday', 'drug_alcohol', 'home_time_in', 'home_time_out', 'next_review', 'truck_service_mileage', 'truck_service_date', 'fai_due_date', 'due', 'kyu_field', 'tag_exp', 'license_plate', 'license_plate_file', 'nyhut', 'passport_number', 'passport_expiration', 'date_of_birth', 'emergency_contact_name_and_phone_number'
    ];

    /*
     * Mutators
     */
    public function setLicenseExpAttribute($value): ?string
    {
        return $this->attributes['license_exp'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setMvrDateAttribute($value): ?string
    {
        return $this->attributes['mvr_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setAnniversaryAttribute($value): ?string
    {
        return $this->attributes['anniversary'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setMedicalCertAttribute($value): ?string
    {
        return $this->attributes['medical_cert'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setScheduleOrientationAttribute($value): ?string
    {
        return $this->attributes['schedule_orientation'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setBirthdayAttribute($value): ?string
    {
        return $this->attributes['birthday'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setDrugAlcoholAttribute($value): ?string
    {
        return $this->attributes['drug_alcohol'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setHomeTimeInAttribute($value): ?string
    {
        return $this->attributes['home_time_in'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setHomeTimeOutAttribute($value): ?string
    {
        return $this->attributes['home_time_out'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setNextReviewAttribute($value): ?string
    {
        return $this->attributes['next_review'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setTruckServiceMileageAttribute($value): ?string
    {
        return $this->attributes['truck_service_mileage'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setTruckServiceDateAttribute($value): ?string
    {
        return $this->attributes['truck_service_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setFaiDueDateAttribute($value): ?string
    {
        return $this->attributes['fai_due_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setDueAttribute($value): ?string
    {
        return $this->attributes['due'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setKyuFieldAttribute($value): ?string
    {
        return $this->attributes['kyu_field'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setTagExpAttribute($value): ?string
    {
        return $this->attributes['tag_exp'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setNyhutAttribute($value): ?string
    {
        return $this->attributes['nyhut'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setPassportExpirationAttribute($value): ?string
    {
        return $this->attributes['passport_expiration'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setDateOfBirthAttribute($value): ?string
    {
        return $this->attributes['date_of_birth'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }


    public function driverData()
    {
        //return $this->belongsTo(Driver::class,'first_name','driver_id');

         return $this->hasOne( 'App\Models\Driver', 'id', 'driver_id');
    }

    /*
     * Accessors
     */
    public function getLicenseExpAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getMvrDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getAnniversaryAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getMedicalCertAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getScheduleOrientationAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getBirthdayAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getDrugAlcoholAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getHomeTimeInAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getHomeTimeOutAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getNextReviewAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getTruckServiceMileageAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getTruckServiceDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getFaiDueDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getDueAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getKyuFieldAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getTagExpAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getNyhutAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getPassportExpirationAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getDateOfBirthAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }



}
