<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Carbon\Carbon;

class DriverHosViolation extends Model
{
    use HasFactory, SoftDeletes;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'driver_id','short_description','title', 'date', 'type', 'load_number', 'state', 'note', 'image','inspection_level','starttime','endtime','motor_carrier','unit1','unit2','hazmat_inspection','disciplinary_action','certified_returned','dataq','dot_crash','towed','injury','citation_image','repair_image','disciplinary_image','dataq_image','time_weight','severity_weight','total_weight','status','event_calendar_date','short_description'
    ];

    /*
     * Mutators
     */
    public function setDateAttribute($value): ?string
    {
        return $this->attributes['date']    = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d H:i:s') : NULL;
    }

    public function setStartTimeAttribute($value): ?string
    {
        return $this->attributes['starttime']    = !is_null($value) ? Carbon::createFromFormat(getTimeLayout(0),$value)->format('H:i') : NULL;
    }

    public function setEndTimeAttribute($value): ?string
    {
        return $this->attributes['endtime']     = !is_null($value) ? Carbon::createFromFormat(getTimeLayout(0),$value)->format('H:i') : NULL;
    }


    /*
     * Accessors
     */
    public function getDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getStartTimeAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(getTimeLayout(0)) : NULL;
    }

    public function getEndTimeAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(getTimeLayout(0)) : NULL;
    }

    public function DriverHosViolationsData()
    {
        return $this->hasMany(DriverHosViolationsData::class, 'driver_hos_violation_id', 'id');
    }

    public function galleries()
    {
        return $this->hasMany(Gallery::class, 'action_id', 'id')->where('source', 'driver_hos_violations');
    }
}
