<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Carbon\Carbon;

class DriverOnboardingHistory extends Model
{
    use HasFactory, SoftDeletes;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'driver_onboarding_id', 'driver_id', 'application_date', 'start_date', 'status', 'days_to_onboard', 'detail', 'notes'
    ];

    /*
     * Mutators
     */
    public function setApplicationDateAttribute($value): ?string
    {
        return $this->attributes['application_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setStartDateAttribute($value): ?string
    {
        return $this->attributes['start_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    /*
     * Accessors
     */
    public function getApplicationDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getStartDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getDetailJsonAttribute($value): ?array
    {
        return !is_null($this->attributes['detail']) ? json_decode($this->attributes['detail'],true) : NULL;
    }
}
