<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class DropTrailer extends Model
{
    use HasFactory;

    protected $table = 'drop_trailers';

    protected $fillable = [
        'load_id',
        'stop_number',
        'type',
        'driver_id',
        'truck_number',
        'trailer_number',
        'empty_loaded',
        'drop_at_stop',
        'keep_assigned',
        'drop_date',
        'user_type',
        'loc',
        'add1',
        'add2',
        'city',
        'state',
        'zip',
        'latitude',
        'longitude',
        'timezone',
        'timezone_offset',
        'timezone_abbr',
        'address_verified',
        'notes',
        'drop_or_keep_driver',
        'retrieval_notification',
        'notify_date',
        'notify_users',
        'created_by',
    ];

    public function getDropDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y H:i') : NULL;
    }

    public function setDropDateAttribute($value): ?string
    {
        return $this->attributes['drop_date'] = !is_null($value) ? convertTimeToUTCzone(['str' => $value,'toFormat' => 'Y-m-d H:i:s','fromFormat' => getTimeLayout()]) : NULL;
    }

    public function getNotifyDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y H:i') : NULL;
    }

    public function setNotifyDateAttribute($value): ?string
    {
        return $this->attributes['notify_date'] = !is_null($value) ? convertTimeToUTCzone(['str' => $value,'toFormat' => 'Y-m-d H:i:s','fromFormat' => getTimeLayout()]) : NULL;
    }

    public function getNotifyUsersAttribute($value): ?string{
        return !is_null($value) ? json_encode(explode(',', $value)) : NULL;
    }
}
