<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Carbon\Carbon;

class Event extends Model
{
    use HasFactory;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'company_id', 'company_admin_id', 'driver_id', 'name', 'from_date', 'to_date', 'type','share','fa_icon','is_repeat','address','notes'
    ];

    /*
     * Scopes
     */
    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    public function setFromDateAttribute($value): ?string
    {
        return $this->attributes['from_date'] = !is_null($value) ? Carbon::createFromFormat(getTimeLayout(),$value)->format('Y-m-d H:i:s') : NULL;
    }

    public function setToDateAttribute($value): ?string
    {
        return $this->attributes['to_date'] = !is_null($value) ? Carbon::createFromFormat(getTimeLayout(),$value)->format('Y-m-d H:i:s') : NULL;
    }

    public function getFromDateAttribute($value): ?string
    {
        return Carbon::parse($value)->format(getTimeLayout());
    }

    public function getToDateAttribute($value): ?string
    {
        return Carbon::parse($value)->format(getTimeLayout());
    }

}
