<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class FuelSurchargeTemplate extends Model
{
    use HasFactory;

    protected $fillable = [
        'company_id', 'fuel_type', 'region', 'rate_type', 'start_price', 'ending_price', 'price_change', 'base_percentage', 'percentage_change', 'per_mile_start', 'per_mile_change', 'customer', 'customer_file_name', 'start_date', 'end_date', 'set_as_active', 'created_by'
    ];

    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    public function setStartDateAttribute($value): ?string {
        return $this->attributes['start_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getStartDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }

    public function setEndDateAttribute($value): ?string {
        return $this->attributes['end_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getEndDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->timezone(getTimeZone())->format('m/d/Y') : NULL;
    }
}
