<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Carbon\Carbon;

class Gallery extends Model
{
    use HasFactory;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $table = 'galleries';

    protected $fillable = ['company_id','action_id','source','field_name','image', 'is_dummy','created_by','created_by_type','requested_by','requested_by_type','approved_by','approved_by_type','file_name','file_type','requires_approval','notes','date','include_w_billing', 'hide_from_driver','access','status','assess_level','uploaded_date','reminder_on', 'reminder_type', 'reminder_repeat', 'reminder_repeat_date_time','reminder_custom_hours'];

    // this is a recommended way to declare event handlers
    public static function boot() {
        parent::boot();

        static::created(function($model){
            if($model->isDirty('source') && $model->source == 'load_accessorials' && $model->field_name == 'accessorial_files') {
                $loadDetailId = $model->load_accessorial->load_detail_id ?? '';
                if($loadDetailId != ''){
                    updateLoadBillingActivity($loadDetailId);
                }
            }
        });

        static::updated(function($model){
            if($model->isDirty('source') && $model->source == 'load_accessorials' && $model->field_name == 'accessorial_files') {
                $loadDetailId = $model->load_accessorial->load_detail_id ?? '';
                if($loadDetailId != ''){
                    updateLoadBillingActivity($loadDetailId);
                }
            }
        });
    }

    /*
     * Scopes
     */
    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    public function setUploadedDateAttribute($value): ?string
    {
        return $this->attributes['uploaded_date'] = Carbon::createFromFormat(config('settings.date_time_format.24_hour','m/d/Y H:i'),$value)->format('Y-m-d H:i:s');
    }

    public function getUploadedDateAttribute($value): ?string
    {
        return !is_null($value) ? convertTimeToUSERzone(['str' => $value,'toFormat' => getTimeLayout(),'fromFormat' => 'Y-m-d H:i:s']) : NULL;
    }

    public function setDateAttribute($value): ?string
    {
        return $this->attributes['date'] = Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d');
    }

    public function getDateAttribute($value): ?string
    {
        return $value ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : '';
    }

    public function load_accessorial(){
        return $this->hasOne(LoadAccessorial::class,'id','action_id');
    }
}
