<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\SoftDeletes;

class Leave extends Model
{
    use HasFactory, SoftDeletes;

    protected $table = 'leaves';
    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'company_id', 'leave_type_id', 'driver_id', 'company_admins_id', 'from_date', 'to_date', 'day_count', 'paid_type', 'duration', 'comment', 'approved_by_company_admins_id', 'status', 'notes'
    ];

    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    public function leave_type()
    {
        return $this->hasOne(LeaveType::class, 'id', 'leave_type_id')->withTrashed();
    }

    public function driver()
    {
        return $this->hasOne(Driver::class, 'id', 'driver_id')->withTrashed();
    }

    public function companyAdmin()
    {
        return $this->hasOne(CompanyAdmin::class, 'id', 'company_admins_id')->withTrashed();
    }



    public function setFromDateAttribute($value): ?string
    {
        return $this->attributes['from_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setToDateAttribute($value): ?string
    {
        return $this->attributes['to_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getFromDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function getToDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }


}
