<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Illuminate\Database\Eloquent\SoftDeletes;

class LoadAppointmentLog extends Model
{
    use HasFactory, SoftDeletes;

    protected $table        = 'load_appointment_logs';
    protected $fillable     = [
    	'load_appointment_id',
        'date',
        'time',
        'note',

        'sender_id',
        'sent_at',

        'source',
        'reply_from',
        'reply_at',
        'ip_address',

        'review_user_type',
        'review_by',
        'review_status',
        'review_at',

        'created_at',
        'updated_at',
        'deleted_at'
    ];

    protected $dates   	    = ['deleted_at'];
    // public $timestamps   = false;

    /*
     * Relationships
     */
    public function load_appointment(){
        return $this->belongsTo(LoadAppointment::class, 'load_appointment_id', 'id');
    }

    public function reviewer(){
        // 'Company Admin','Broker','Broker Contact','Customer'
        if ($this->review_user_type === 'Broker') {
            return $this->hasOne(Broker::class,'id','review_by')->withTrashed();
        } elseif ($this->review_user_type === 'Broker Contact') {
            return $this->hasOne(BrokerContact::class,'id','review_by')->withTrashed();
        } elseif ($this->review_user_type === 'Customer') {
            return $this->hasOne(Customer::class,'id','review_by')->withTrashed();
        } else {
            return $this->hasOne(CompanyAdmin::class,'id','review_by')->withTrashed();
        }
    }

    public function review_comapny_admin(){
        return $this->hasOne(CompanyAdmin::class,'id','review_by')->withTrashed();
    }

    public function review_broker(){
        return $this->hasOne(Broker::class,'id','review_by')->withTrashed();
    }

    public function review_broker_contact(){
        return $this->hasOne(BrokerContact::class,'id','review_by')->withTrashed();
    }

    public function review_customer(){
        return $this->hasOne(Customer::class,'id','review_by')->withTrashed();
    }
}
