<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class LoadBillingPayInvoice extends Model
{
    use HasFactory;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'load_detail_id',
        'payment_date',
        'paid_by',
        'factor',
        'payment_type',
        'additional_info',
        'check_number',
        'account_number',
        'total_payment',
        'load_pay',
        'load_gl_account',
        'factor_fee',
        'factor_gl_account',
        'chargeback',
        'chargeback_type',
        'chargeback_user',
        'chargeback_amount',
        'chargeback_against_balance',
        'notes',
        'total_balance',
    ];

    public function setPaymentDateAttribute($value): ?string {
        return $this->attributes['payment_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d H:i:s') : NULL;
    }

    public function getPaymentDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format('m/d/Y H:i:s') : NULL;
    }

    public function setChargebackTypeAttribute($value): ?string {
        return $this->attributes['chargeback_type'] = (is_array($value)) ? serialize($value) : NULL;
    }

    public function getChargebackTypeAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : [];
    }

    public function setChargebackUserAttribute($value): ?string {
        return $this->attributes['chargeback_user'] = (is_array($value)) ? serialize($value) : NULL;
    }

    public function getChargebackUserAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : [];
    }

    public function setChargebackAmountAttribute($value): ?string {
        return $this->attributes['chargeback_amount'] = (is_array($value)) ? serialize($value) : NULL;
    }

    public function getChargebackAmountAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : [];
    }

    public function setChargebackAgainstBalanceAttribute($value): ?string {
        return $this->attributes['chargeback_against_balance'] = (is_array($value)) ? serialize($value) : NULL;
    }

    public function getChargebackAgainstBalanceAttribute($value): ?array{
        return !is_null($value) ? unserialize($value) : [];
    }

    public function load_detail(){
        return $this->hasOne(LoadDetail::class,'id','load_detail_id');
    }
}
