<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Illuminate\Database\Eloquent\SoftDeletes;

class LoadDetention extends Model
{
    use HasFactory, SoftDeletes;

    protected $table        = 'load_detentions';
    protected $fillable     = [
        'driver_id',
        'broker_id',

        'load_detail_id',
        'load_stop_id',
        'type',
        'is_stop',
        'gps_location',

        'detention_status',
        'detention_arrival_time',
        'detention_start_time',
        'detention_end_time',

        'free_time',
        'billed_time',
        'adjusted_billed_time',
        'rate',
        'adjusted_rate',
        'detention_due',
        'adjusted_detention_due',

        'driver_comment_show_to_broker',
        'driver_comment',
        'reviewer_comment_show_to_broker',
        'reviewer_comment',
        'billing_comment_show_to_broker',
        'comment',

        'sent_acknowledgement_to_broker',
        'sent_acknowledgement_to_broker_at',
        'acknowledgement_emails',
        'acknowledged_user_type',
        'acknowledged_at',
        'acknowledged_by',
        'broker_viewed_acknowledgement_at',
        'broker_acknowledgement_ip',

        'stopped_user_type',
        'stopped_at',
        'stopped_by',

        'submitted_user_type',
        'submitted_at',
        'submitted_by',

        'sent_to_broker',
        'sent_to_broker_at',
        'sent_to',

        'approved_user_type',
        'approved_at',
        'approved_by',
        'approved_ip',

        'denied_user_type',
        'denied_at',
        'denied_by',
        'denied_ip',

        'amended_at',
        'amended_by',
        'amended_ip',
        'broker_denied_at',
        'broker_denied_by',
        'broker_denied_ip',

        'is_send_to_broker',
        'is_send_to_broker_at',
        'broker_emails',
        'broker_view_at',
        'broker_view_ip',

        'signature',
        'signature_name',
        'signature_datetime',

        'broker_review_status_ip',

        'review_user_type',
        'review_status',
        'review_at',
        'review_by',

        'created_at',
        'updated_at',
        'deleted_at'
    ];

    protected $dates   	    = ['deleted_at'];
    // public $timestamps   = false;

    /*
     * Mutators
     */
    public function setRateAttribute($value){
        $valueWithoutCommas                         = null;
        if(!is_null($value)){
            // Remove commas from the value
            $valueWithoutCommas                     = str_replace(',', '', $value);
            $valueWithoutCommas                     = (float) $valueWithoutCommas;
        }
        $this->attributes['rate']                   = $valueWithoutCommas;
    }

    public function setAdjustedRateAttribute($value){
        $valueWithoutCommas                         = null;
        if(!is_null($value)){
            // Remove commas from the value
            $valueWithoutCommas                     = str_replace(',', '', $value);
            $valueWithoutCommas                     = (float) $valueWithoutCommas;
        }
        $this->attributes['adjusted_rate']          = $valueWithoutCommas;
    }

    public function setDetentionDueAttribute($value){
        $valueWithoutCommas                         = null;
        if(!is_null($value)){
            // Remove commas from the value
            $valueWithoutCommas                     = str_replace(',', '', $value);
            $valueWithoutCommas                     = (float) $valueWithoutCommas;
        }
        $this->attributes['detention_due']          = $valueWithoutCommas;
    }

    public function setAdjustedDetentionDueAttribute($value){
        $valueWithoutCommas                         = null;
        if(!is_null($value)){
            // Remove commas from the value
            $valueWithoutCommas                     = str_replace(',', '', $value);
            $valueWithoutCommas                     = (float) $valueWithoutCommas;
        }
        $this->attributes['adjusted_detention_due'] = $valueWithoutCommas;
    }

    /*
     * Relationships
     */
    public function driver(){
        return $this->belongsTo(Driver::class)->withTrashed();
    }

    public function broker(){
        return $this->hasOne(Broker::class,'id','broker_id')->withTrashed();
    }

    public function load_detail(){
        return $this->hasOne(LoadDetail::class,'id','load_detail_id');
    }

    public function stop(){
        return $this->hasOne(LoadStop::class,'id','load_stop_id');
    }

    public function acknowledgedor(){
        // Company Admin, Driver
        if ($this->acknowledged_user_type === 'Driver') {
            return $this->hasOne(Driver::class,'id','acknowledged_by')->withTrashed();
        } else {
            return $this->hasOne(CompanyAdmin::class, 'id', 'acknowledged_by')->withTrashed();
        }
    }

    public function stoppedor(){
        // Company Admin, Driver
        if ($this->stopped_user_type === 'Driver') {
            return $this->hasOne(Driver::class,'id','stopped_by')->withTrashed();
        } else {
            return $this->hasOne(CompanyAdmin::class, 'id', 'stopped_by')->withTrashed();
        }
    }

    public function submittedor(){
        // Company Admin, Driver
        if ($this->submitted_user_type === 'Driver') {
            return $this->hasOne(Driver::class,'id','submitted_by')->withTrashed();
        } else {
            return $this->hasOne(CompanyAdmin::class, 'id', 'submitted_by')->withTrashed();
        }
    }

    public function approvedor(){
        // Company Admin, Broker
        if ($this->approved_user_type === 'Broker') {
            return $this->hasOne(Broker::class,'id','approved_by')->withTrashed();
        } else {
            return $this->hasOne(CompanyAdmin::class, 'id', 'approved_by')->withTrashed();
        }
    }

    public function deniedor(){
        // Company Admin, Broker
        if ($this->denied_user_type === 'Broker') {
            return $this->hasOne(Broker::class,'id','denied_by')->withTrashed();
        } else {
            return $this->hasOne(CompanyAdmin::class, 'id', 'denied_by')->withTrashed();
        }
    }

    public function reviewedor(){
        // Company Admin, Broker, Driver
        if ($this->review_user_type === 'Broker') {
            return $this->hasOne(Broker::class,'id','review_by')->withTrashed();
        } else if ($this->review_user_type === 'Driver') {
            return $this->belongsTo(Driver::class,'review_by')->withTrashed();
        } else {
            return $this->hasOne(CompanyAdmin::class, 'id', 'review_by')->withTrashed();
        }
    }

    public function review_driver_admin(){
        return $this->belongsTo(Driver::class,'review_by')->withTrashed();
    }

    public function review_comapny_admin(){
        return $this->belongsTo(CompanyAdmin::class,'review_by')->withTrashed();
    }

    public function review_broker(){
        return $this->belongsTo(Broker::class,'review_by')->withTrashed();
    }

    public function amended_broker(){
        return $this->belongsTo(Broker::class,'amended_by')->withTrashed();
    }

    public function denied_broker_request(){
        return $this->belongsTo(Broker::class,'broker_denied_by')->withTrashed();
    }
}
