<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Illuminate\Database\Eloquent\SoftDeletes;

class LoadStopReview extends Model
{
    use HasFactory, SoftDeletes;

    protected $table        = 'load_stop_reviews';
    protected $fillable     = [
    	'customer_id',
        'load_detail_id',
        'load_stop_id',
        'type',
        'is_stop',
        'rating',
        'review',

        'created_at',
        'created_by',
        'created_by_type',

        'updated_at',
        'updated_by',
        'updated_by_type',

        'deleted_at',
        'deleted_by',
        'deleted_by_type'
    ];

    protected $dates   	    = ['deleted_at'];
    public $timestamps   	= false;

    public function customer(){
        return $this->hasOne(Customer::class,'id','customer_id')->withTrashed();
    }

    public function load_detail(){
        return $this->hasOne(LoadDetail::class,'id','load_detail_id');
    }

    public function stop(){
        return $this->hasOne(LoadStop::class,'id','load_stop_id');
    }

    public function creator(){
    	// '0' => 'company_admin', '1' => 'managers', '2' => 'dispatchers', '3' => 'drivers'
        if ($this->created_by_type === 3) {
            return $this->hasOne(Driver::class,'id','created_by')->withTrashed();
        } else {
            return $this->hasOne(CompanyAdmin::class, 'id', 'created_by')->withTrashed();
        }
    }

    public function editor(){
    	// '0' => 'company_admin', '1' => 'managers', '2' => 'dispatchers', '3' => 'drivers'
    	if ($this->updated_by_type === 3) {
    	    return $this->hasOne(Driver::class,'id','updated_by')->withTrashed();
    	} else {
    	    return $this->hasOne(CompanyAdmin::class, 'id', 'updated_by')->withTrashed();
    	}
    }

    public function deletor(){
    	// '0' => 'company_admin', '1' => 'managers', '2' => 'dispatchers', '3' => 'drivers'
    	if ($this->deleted_by_type === 3) {
    	    return $this->hasOne(Driver::class,'id','deleted_by')->withTrashed();
    	} else {
    	    return $this->hasOne(CompanyAdmin::class, 'id', 'deleted_by')->withTrashed();
    	}
    }
}
