<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Illuminate\Database\Eloquent\SoftDeletes;

class LoadTruckOwner extends Model
{
    use HasFactory, SoftDeletes;

    protected $table        = 'load_truck_owners';
    protected $fillable     = [
    	'load_id',
        'load_detail_id',
        'driver_id',
        'owner_id',
        'owner_type',

        'empty_miles',
        'loaded_miles',
        'total_miles',

        'pay_type',
        'gross_wage',
        'total_bill',

        'pay_percentage',
        'mileage_percentage',
        'pay',
        'miles',

        'created_at',
        'updated_at',
        'deleted_at'
    ];

    protected $dates        = ['deleted_at'];
    // public $timestamps   = false;

    /*
     * Mutators
     */
    private function removeCommasAndConvertToFloat($value){
        return is_null($value) ? null : (float) str_replace(',', '', $value);
    }

    public function setEmptyMilesAttribute($value){
        $valueWithoutCommas                 = $this->removeCommasAndConvertToFloat($value);
        $this->attributes['empty_miles']    = $valueWithoutCommas;
    }

    public function setLoadedMilesAttribute($value){
        $valueWithoutCommas                 = $this->removeCommasAndConvertToFloat($value);
        $this->attributes['loaded_miles']   = $valueWithoutCommas;
    }

    public function setTotalMilesAttribute($value){
        $emptyMiles                         = $this->removeCommasAndConvertToFloat($this->attributes['empty_miles']);
        $loadedMiles                        = $this->removeCommasAndConvertToFloat($this->attributes['loaded_miles']);

        $totalMiles                         = $emptyMiles + $loadedMiles;
        $this->attributes['total_miles']    = $totalMiles;
    }

    public function setPayTypeAttribute($value){
        $valueWithoutCommas                 = $this->removeCommasAndConvertToFloat($value);
        $this->attributes['pay_type']       = $valueWithoutCommas;
    }

    public function setGrossWageAttribute($value){
        $valueWithoutCommas                 = $this->removeCommasAndConvertToFloat($value);
        $this->attributes['gross_wage']     = $valueWithoutCommas;
    }

    public function setTotalBillAttribute($value){
        $valueWithoutCommas                 = $this->removeCommasAndConvertToFloat($value);
        $this->attributes['total_bill']     = $valueWithoutCommas;
    }

    public function setPayAttribute($value){
        $valueWithoutCommas                 = $this->removeCommasAndConvertToFloat($value);
        $this->attributes['pay']            = $valueWithoutCommas;
    }

    public function setMilesAttribute($value){
        $valueWithoutCommas                 = $this->removeCommasAndConvertToFloat($value);
        $this->attributes['miles']          = $valueWithoutCommas;
    }

    /*
     * Relationships
     */
    public function driver(){
        return $this->hasOne(Driver::class,'id','driver_id')->withTrashed();
    }

    public function load_detail(){
        return $this->hasOne(LoadDetail::class,'id','load_detail_id');
    }
}
