<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Carbon\Carbon;

class Maintenance extends Model
{
    use HasFactory;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'ticket_number','company_id','name','status','complete_on','equipment_type','truck_namber','trailer_namber','type','notify_dispatcher','notify_driver','notify_company','notify_other','vendor_id','estimated_cost','final_cost','estimated_file','due_on','due_date','due_miles','display_on_calendar_dispatcher','display_on_calendar_driver','display_on_calendar_company','display_on_calendar_other','repeats','how_often','repeat_due_miles','created_by','odometer_reading_at_completion','warranty','warranty_date','warranty_date_file','warranty_mileage','warranty_mileage_file','custom_repeat_every_number','custom_repeat_every_type','custom_repeat_on','custom_ends','custom_ends_date','custom_ends_occurrences_number','custom_ends_occurrences_type'
    ];

    /*
     * Scopes
     */
    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    public function truck()
    {
        return $this->hasOne(Truck::class,'id','truck_namber')->withTrashed();
    }

    public function trailer()
    {
        return $this->hasOne(Trailer::class,'id','trailer_namber')->withTrashed();
    }

    public function vendor()
    {
        return $this->hasOne(Vendor::class,'id','vendor_id')->withTrashed();
    }

    public function companyAdmin()
    {
        return $this->hasOne(CompanyAdmin::class,'id','created_by')->withTrashed();
    }
    
    public function setDueDateAttribute($value): ?string
    {
        return $this->attributes['due_date'] = !is_null($value) ? convertTimeToUTCzone(['str' => $value,'toFormat' => 'Y-m-d H:i:s','fromFormat' => getTimeLayout()]) : NULL;
    }

    public function getDueDateAttribute($value): ?string
    {
        return !is_null($value) ? convertTimeToUSERzone(['str' => $value,'toFormat' => getTimeLayout(),'fromFormat' => 'Y-m-d H:i:s']) : NULL;
    }

    public function setCustomEndsDateAttribute($value): ?string
    {
        return $this->attributes['custom_ends_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getCustomEndsDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setCompleteOnAttribute($value): ?string
    {
        return $this->attributes['complete_on'] = !is_null($value) ? convertTimeToUTCzone(['str' => $value,'toFormat' => 'Y-m-d H:i:s','fromFormat' => getTimeLayout()]) : NULL;
    }

    public function getCompleteOnAttribute($value): ?string
    {
        return !is_null($value) ? convertTimeToUSERzone(['str' => $value,'toFormat' => getTimeLayout(),'fromFormat' => 'Y-m-d H:i:s']) : NULL;
    }

    public function setWarrantyDateAttribute($value): ?string
    {
        return $this->attributes['warranty_date'] = !is_null($value) ? convertTimeToUTCzone(['str' => $value,'toFormat' => 'Y-m-d H:i:s','fromFormat' => getTimeLayout()]) : NULL;
    }

    public function getWarrantyDateAttribute($value): ?string
    {
        return !is_null($value) ? convertTimeToUSERzone(['str' => $value,'toFormat' => getTimeLayout(),'fromFormat' => 'Y-m-d H:i:s']) : NULL;
    }

}
