<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Carbon\Carbon;

class MaintenanceAppointment extends Model
{
    use HasFactory;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'company_id','maintenance_id','summary','description','type','location','address','address_2','city','state','zip','latitude','longitude','timezone','timezone_offset','timezone_abbr','address_verified','reminders_schedule','all_days','start_time','end_time','owner','attendees','do_not_email_invitation'
    ];

    /*
     * Scopes
     */
    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }
    
    public function setStartTimeAttribute($value): ?string
    {
        return $this->attributes['start_time'] = !is_null($value) ? convertTimeToUTCzone(['str' => $value,'toFormat' => 'Y-m-d H:i:s','fromFormat' => getTimeLayout()]) : NULL;
    }

    public function getStartTimeAttribute($value): ?string
    {
        return !is_null($value) ? convertTimeToUSERzone(['str' => $value,'toFormat' => getTimeLayout(),'fromFormat' => 'Y-m-d H:i:s']) : NULL;
    }

    public function setEndTimeAttribute($value): ?string
    {
        return $this->attributes['end_time'] = !is_null($value) ? convertTimeToUTCzone(['str' => $value,'toFormat' => 'Y-m-d H:i:s','fromFormat' => getTimeLayout()]) : NULL;
    }

    public function getEndTimeAttribute($value): ?string
    {
        return !is_null($value) ? convertTimeToUSERzone(['str' => $value,'toFormat' => getTimeLayout(),'fromFormat' => 'Y-m-d H:i:s']) : NULL;
    }

    
    

}
