<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Carbon\Carbon;

class Trailer extends Model
{
    use HasFactory, SoftDeletes;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'trailer_unique_id',
        'company_id',
        'samsara_id',
        'trailer_id',
        'vin',
        'truck_type',
        'type',
        'year',
        'make',
        'model',
        'color',
        'required_license',
        'current_assigned_truck',
        'current_assigned_driver',
        'trailer_group',
        'assigned_customer',
        'axles',
        'tire_size',
        'tire_qty',
        'suspension_type',
        'spare_tire_rank',
        'roof_type',
        'floor_type',
        'doors_type',
        'vented',
        'plated_trailer',
        'inside_width',
        'paper_spec',
        'pallet_jack',
        'liftgate',
        'high_cube',
        'spread_axie',
        'securement',
        'tire_auto_inflation',
        'tracker_provider',
        'tracker_id',
        'air_scale_provider',
        'air_scale_id',
        'tire_sensor_provider',
        'tire_sensor_id',
        'tag_state',
        'tag_number',
        'tag_expiration',
        'tag_upload_image',
        'carrier',
        'terminal',
        'ownership',
        'owner_id',
        'owner_name',
        'owner_address',
        'purchased_new',
        'owner_purchase_price',
        'owner_purchase_date',
        'owner_documents',
        'leasing_co',
        'lease_expiration_date',
        'lease_documents',
        'lease_agreement_title',
        'fai_due_date',
        'fai_attachment',
        'trailer_interchange',
        'trailer_interchange_image',
        'depreciated_value',
        'onboard_equipment',
        'equipment_comments',
        'in_service_date',
        'federal_inspection_type',
        'federal_inspection_interval',
        'last_inspection',
        'last_server_date',
        'last_inspection_location',
        'next_inspection',
        'next_service_date',
        'next_service',
        'maintenance_date',
        'lease_exp_date',   
        'insurance_carrier',
        'insurance_exp',
        'insurance_value',
        'leasing_company',
        'sliding',
        'trailer_service_date',
        'reefer_service_notes',
        'reefer_model_number',
        'reefer_service_interval_hours',
        'reefer_next_pm_service',
        'reefer_last_pm_service',
        'reefer_service_due',
        'reefer_current_hours',
        'etrack',
        'plated_interior',
        'drop_trailer_pool',
        'logistics_post',
        'insured_value',
        'phys_dam_attachment',
        'physical_dam_exp_date',
        'phys_dam_start_date',
        'phys_dam_ins_carrier',
        'carb_id',
        'carb_expiration',
        'carb_compliant',
        'deck_height',
        'hubometer',
        'unladen_weight',
        'max_load_weight',
        'gross_weight',
        'length',
        'width',
        'height',
        'last_location_name',
        'last_location_load_status',
        'last_location_address',
        'last_location_address_2',
        'last_location_city',
        'last_location_state',
        'last_location_zip',
        'last_location_latitude',
        'last_location_longitude',
        'last_location_timezone',
        'last_location_timezone_offset',
        'last_location_timezone_abbr',
        'last_location_address_verified',
        'last_location_notes',
        'status',
        'created_at',
        'updated_at',
        'deleted_at'
    ];

    /*
     * Scopes
     */
    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    /*
     * Mutators
     */
    public function setTagExpirationAttribute($value): ?string
    {
        return $this->attributes['tag_expiration']          = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d H:i:s') : null;
    }

    public function setCarbExpirationAttribute($value): ?string
    {
        return $this->attributes['carb_expiration']         = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : null;
    }

    public function setTrailerServiceDateAttribute($value): ?string
    {
        return $this->attributes['trailer_service_date']    = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : null;
    }

    public function setNextServiceAttribute($value): ?string
    {
        return $this->attributes['next_service']            = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d H:i:s') : null;
    }

    public function setLastInspectionAttribute($value): ?string
    {
        return $this->attributes['last_inspection']         = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d H:i:s') : null;
    }

    public function setNextInspectionAttribute($value): ?string
    {
        return $this->attributes['next_inspection']         = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d H:i:s') : null;
    }

    public function setMaintenanceDateAttribute($value): ?string
    {
        return $this->attributes['maintenance_date']        = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d H:i:s') : null;
    }

    public function setLeaseExpDateAttribute($value): ?string
    {
        return $this->attributes['lease_exp_date']          = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d H:i:s') : null;
    }

    public function setPhysDamStartDateAttribute($value): ?string
    {
        return $this->attributes['phys_dam_start_date']     = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : null;
    }

    public function setPhysicalDamExpDateAttribute($value): ?string
    {
        return $this->attributes['physical_dam_exp_date']   = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : null;
    }

    public function setOwnerPurchaseDateAttribute($value): ?string
    {
        return $this->attributes['owner_purchase_date']     = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : null;
    }

    public function setLeaseExpirationDateAttribute($value): ?string
    {
        return $this->attributes['lease_expiration_date']   = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : null;
    }

    public function setFaiDueDateAttribute($value): ?string
    {
        return $this->attributes['fai_due_date']            = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : null;
    }

    public function setLastServerDateAttribute($value): ?string
    {
        return $this->attributes['last_server_date']        = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : null;
    }

    public function setNextServiceDateAttribute($value): ?string
    {
        return $this->attributes['next_service_date']       = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : null;
    }

    public function setInServiceDateAttribute($value): ?string
    {
        return $this->attributes['in_service_date']         = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : null;
    }


    /*
     * Accessors
     */
    public function getTagExpirationAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getCarbExpirationAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getTrailerServiceDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getNextServiceAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getLastInspectionAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getNextInspectionAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getMaintenanceDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getLeaseExpDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getPhysDamStartDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getPhysicalDamExpDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getOwnerPurchaseDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getLeaseExpirationDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getFaiDueDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getLastServerDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getNextServiceDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getInServiceDateAttribute($value): ?string
    {
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    /*
     * Relationships
     */
    public function company(){
        return $this->belongsTo(Company::class);
    }
    
    public function load_details(){
        return $this->hasMany(LoadDetail::class,'trailer_number','trailer_id')->select('id', 'load_number');
    }

    public function trailer_type(){
        return $this->hasOne(TrailerType::class,'id','type')->withTrashed();
    }

    public function truck(){
        return $this->hasOne(Truck::class,'id','current_assigned_truck')->withTrashed();
    }

    public function driver(){
        return $this->hasOne(Driver::class,'id','current_assigned_driver')->withTrashed();
    }

    public function insurance_profile_items(){
        return $this->hasMany(TrailerInsuranceProfileItem::class,'trailer_id','id');
    }

    public function insurance_additionals(){
        return $this->hasMany(TrailerInsuranceAdditional::class,'trailer_id','id');
    }

    public function pm_service_profile_items(){
        return $this->hasMany(TrailerPmServiceProfileItem::class,'trailer_id','id');
    }

    public function pm_service_additionals(){
        return $this->hasMany(TrailerPmServiceAdditional::class,'trailer_id','id');
    }

    public function pm_service_warranty_profile_items(){
        return $this->hasMany(TrailerPmServiceWarrantyProfileItem::class,'trailer_id','id');
    }

    public function equipment_profile_items() {
        return $this->hasMany(TrailerEquipmentProfileItem::class,'trailer_id','id');
    }

    public function equipment_additionals() {
        return $this->hasMany(TrailerEquipmentAdditional::class,'trailer_id','id');
    }

    public function document_profile_items() {
        return $this->hasMany(TrailerDocumentProfileItem::class,'trailer_id','id');
    }

    public function document_additionals() {
        return $this->hasMany(TrailerDocumentAdditional::class,'trailer_id','id');
    }

    public function get_carrier(){
        return $this->hasOne(Carrier::class,'id','carrier')->withTrashed();
    }
}
