<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Illuminate\Database\Eloquent\SoftDeletes;

use Carbon\Carbon;

class TrailerDocumentAdditional extends Model
{
    use HasFactory, SoftDeletes;

    protected $table    = 'trailer_document_additionals';
    protected $fillable = [
        'trailer_id', 'company_document_title_id', 'dot_required', 'exp_date', 'doc_type', 'created_at', 'updated_at', 'deleted_at'
    ];
    public $timestamps  = true;
    protected $dates    = ['deleted_at'];

    /*
     * Mutators
     */
    public function setExpDateAttribute($value): ?string{
        return $this->attributes['exp_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    /*
     * Accessors
     */
    public function getExpDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    /*
     * Relationships
     */
    public function trailer(){
        return $this->belongsTo(Trailer::class,'trailer_id','id');
    }

    public function company_document_title(){
        return $this->hasOne(CompanyPmType::class,'id','company_document_title_id')->withTrashed();
    }
}
