<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Illuminate\Database\Eloquent\SoftDeletes;

use Carbon\Carbon;

class TrailerEquipmentAdditional extends Model
{
    use HasFactory, SoftDeletes;

    protected $table    = 'trailer_equipment_additionals';
    protected $fillable = [
        'trailer_id', 'company_equipment_title_id', 'qty', 'value', 'description', 'entered_by', 'date_entered', 'created_at', 'updated_at', 'deleted_at'
    ];
    public $timestamps  = true;
    protected $dates    = ['deleted_at'];

    /*
     * Mutators
     */
    public function setDateEnteredAttribute($value): ?string{
        return $this->attributes['date_entered'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    /*
     * Accessors
     */
    public function getDateEnteredAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    /*
     * Relationships
     */
    public function trailer(){
        return $this->belongsTo(Trailer::class,'trailer_id','id');
    }

    public function company_equipment_title(){
        return $this->hasOne(CompanyEquipmentTitle::class,'id','company_equipment_title_id')->withTrashed();
    }
}
