<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

use Illuminate\Database\Eloquent\SoftDeletes;

use Carbon\Carbon;

class TrailerInsuranceProfileItem extends Model
{
    use HasFactory, SoftDeletes;

    protected $table    = 'trailer_insurance_profile_items';
    protected $fillable = [
        'trailer_id', 'trailer_company_insurance_profile_id', 'company_insurance_type_id', 'vendor_id', 'policy_number', 'value', 'date_added', 'expiration_date', 'created_at', 'updated_at', 'deleted_at'
    ];
    public $timestamps  = true;
    protected $dates    = ['deleted_at'];

    /*
     * Mutators
     */
    public function setDateAddedAttribute($value): ?string{
        return $this->attributes['date_added']      = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : null;
    }

    public function setExpirationDateAttribute($value): ?string{
        return $this->attributes['expiration_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : null;
    }

    /*
     * Accessors
     */
    public function getDateAddedAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    public function getExpirationDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : null;
    }

    /*
     * Relationships
     */
    public function trailer(){
        return $this->belongsTo(Trailer::class,'trailer_id','id');
    }

    public function trailer_company_insurance_profile(){
        return $this->belongsTo(TrailerCompanyInsuranceProfile::class, 'trailer_company_insurance_profile_id', 'id');
    }

    public function company_insurance_type(){
        return $this->hasOne(CompanyInsuranceType::class,'id','company_insurance_type_id')->withTrashed();
    }

    public function vendor(){
        return $this->hasOne(Vendor::class,'id','vendor_id')->withTrashed();
    }
}
