<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use App\Models\Carrier;
use Carbon\Carbon;

class Truck extends Model
{
    use HasFactory, SoftDeletes;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'company_id',
        'carrier_id',
        'unit',
        'year',
        'make',
        'vin_number',
        'tag',
        'nyhut',
        'vented',
        'e_track',
        'fai_due_date',
        'due',
        'license_plate',
        'license_plate_file',
        'truck_service_mileage',
        'truck_service_date',
        'kyu_field',
        'tag_exp',
        'nyhut_date',
        'samsara_id',
        'is_service_date',
        'color',
        'tag_state',
        'tag_number',
        'ez_pass_id',
        'prepass_id',
        'eorb_provider',
        'eorb_id',
        'camera_provider',
        'camera_id',
        'phys_dam_ins_carrier',
        'phys_dam_start_date',
        'phys_dam_exp_date',
        'phys_dam_insured_value',
        'bobtail_carrier',
        'bobtail_carrier_start_date',
        'bobtail_carrier_exp_date',
        'bobtail_carrier_insured_value',
        'terminal_id',
        'owner_id',
        'owner_type',
        'ownership',
        'ownership_id',
        'leasing_co',
        'lease_expiration_date',
        'lease_agreement_title',
        'owner_name',
        'owner_purchase_price',
        'owner_purchase_date',
        'owner_agreement_title',
        'kyu_due_date',
        '2290_due_date',
        'carb_id',
        'carb_id_date',
        'carb_compliant',
        'weight',
        'max_load_weight',
        'fuel_capacity',
        'axles',
        'tire_size',
        'wheel_size',
        'fuel_type',
        'tuel_capacity',
        'odometer',
        'engine_hours',
        'status',
        'tag_photo',
        'phys_dam_file',
        'bobtail_carrier_file',
        'lease_documents',
        'owner_documents',
        'fai_file',
        'kyu_file',
        '2290_file',
        'nyhut_file',
        'carb_id_file',
        'truck_image',
        'model',
        'length',
        'width',
        'height',
        'last_location_name',
        'last_location_address',
        'last_location_address_2',
        'last_location_city',
        'last_location_state',
        'last_location_zip',
        'last_location_latitude',
        'last_location_longitude',
        'last_location_timezone',
        'last_location_timezone_offset',
        'last_location_timezone_abbr',
        'last_location_address_verified',
        'last_location_notes',
        'last_location_load_status',

        'truck_unique_id',
        'truck_type',
        'required_license',
        'class',
        'driver',
        'co_driver',
        'trailer',
        'trailer_assign_date',
        'trailer_assign_by',
        'tire_qty',
        'gross_weight',
        'engine',
        'transmission',
        'automatic_transmission',
        'high_cube',
        'vented',
        'liftgate',
        'pallet_jack',
        'reefer_equipped',
        'apu_equipped',
        'onboarding_odometer',
        'current_odometer',
        'truck_group',
        'assigned_customer',

        'down_payment',
        'balance_due',
        'purchase_agreement_on_file',
        'leasing_company',
        'lease_point_of_contact',
        'lease_start',
        'lease_end',
        'lease_agreement_on_file',
        'rental_company',
        'rental_point_of_contact',
        'rental_start',
        'rental_end',
        'rental_agreement_on_file',
        'billing_type',
        'payment_type',
        'payment_amount',
        'security_deposit',

        'oregon_permit_number',
        'oregon_axles',
        'oregon_date',
        'oregon_weight',
        'conf_number_2290',
        'filed_date_2290',
        'exp_date_2290',
        'nyhut_unladen',
        'kyu_permit_number',
        'apply_ifta',
        'apply_nyhut',
        'apply_kyu',
        'apply_new_mexico',
        'apply_oregon',

        'profile_image',
        'profile_image_upload_by',
        'profile_image_update_date',
    ];

    /*
     * Scopes
     */
    public function scopeIsCompany($query){
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    /*
     * Mutators
     */
    public function setFaiDueDateAttribute($value): ?string{
        return $this->attributes['fai_due_date']                    = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getFaiDueDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setDueAttribute($value): ?string{
        return $this->attributes['due']                             = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getDueAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setNyhutDateAttribute($value): ?string{
        return $this->attributes['nyhut_date']                      = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getNyhutDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setTruckServiceDateAttribute($value): ?string{
        return $this->attributes['truck_service_date']              = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getTruckServiceDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setKyuFieldAttribute($value): ?string{
        return $this->attributes['kyu_field']                       = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getKyuFieldAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setTagExpAttribute($value): ?string{
        return $this->attributes['tag_exp']                         = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getTagExpAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setIsServiceDateAttribute($value): ?string{
        return $this->attributes['is_service_date']                 = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getIsServiceDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setPhysDamStartDateAttribute($value): ?string{
        return $this->attributes['phys_dam_start_date']             = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getPhysDamStartDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setPhysDamExpDateAttribute($value): ?string{
        return $this->attributes['phys_dam_exp_date']               = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getPhysDamExpDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setBobtailCarrierStartDateAttribute($value): ?string{
        return $this->attributes['bobtail_carrier_start_date']      = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getBobtailCarrierStartDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setBobtailCarrierExpDateAttribute($value): ?string{
        return $this->attributes['bobtail_carrier_exp_date']        = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getBobtailCarrierExpDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setLeaseExpirationDateAttribute($value): ?string{
        return $this->attributes['lease_expiration_date']           = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getLeaseExpirationDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setOwnerPurchaseDateAttribute($value): ?string{
        return $this->attributes['owner_purchase_date']             = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getOwnerPurchaseDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setKyuDueDateAttribute($value): ?string{
        return $this->attributes['kyu_due_date']                    = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getKyuDueDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function set2290DueDateAttribute($value): ?string{
        return $this->attributes['2290_due_date']                   = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function get2290DueDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setCarbIdDateAttribute($value): ?string{
        return $this->attributes['carb_id_date']                    = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getCarbIdDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setPhysDamInsuredValueAttribute($value): ?string{
        return $this->attributes['phys_dam_insured_value']          = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getPhysDamInsuredValueAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setBobtailCarrierInsuredValueAttribute($value): ?string{
        return $this->attributes['bobtail_carrier_insured_value']   = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getBobtailCarrierInsuredValueAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setLeaseStartAttribute($value): ?string{
        return $this->attributes['lease_start']                     = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getLeaseStartAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setLeaseEndAttribute($value): ?string{
        return $this->attributes['lease_end']                       = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getLeaseEndAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setRentalStartAttribute($value): ?string{
        return $this->attributes['rental_start']                    = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getRentalStartAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setRentalEndAttribute($value): ?string{
        return $this->attributes['rental_end']                      = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getRentalEndAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setOregonDateAttribute($value): ?string{
        return $this->attributes['oregon_date']                     = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getOregonDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setFiledDate2290Attribute($value): ?string{
        return $this->attributes['filed_date_2290']                 = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getFiledDate2290Attribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setExpDate2290Attribute($value): ?string{
        return $this->attributes['exp_date_2290']                   = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getExpDate2290Attribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setTrailerAssignDateAttribute($value): ?string{
        return $this->attributes['trailer_assign_date']             = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getTrailerAssignDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setDriverAssignDateAttribute($value): ?string{
        return $this->attributes['driver_assign_date']              = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getDriverAssignDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setCoDriverAssignDateAttribute($value): ?string{
        return $this->attributes['co_driver_assign_date']           = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getCoDriverAssignDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setProfileImageUpdateDateAttribute($value): ?string{
        return $this->attributes['profile_image_update_date']       = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getProfileImageUpdateDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    /*
     * Relationships
     */
    public function getCompany(){
        return $this->belongsTo(Company::class,'company_id','id');
    }

    public function carrier(){
        return $this->belongsTo(Carrier::class);
    }

    public function driver_details() {
        return $this->hasOne(Driver::class, 'id', 'driver');
    }

    public function co_driver_details() {
        return $this->hasOne(Driver::class, 'id', 'co_driver');
    }

    public function truck_restricted_states(){
        return $this->hasMany(TruckRestrictedState::class);
    }

    public function truck_avoid_states(){
        return $this->hasMany(TruckAvoidState::class);
    }

    public function truck_integrations(){
        return $this->hasMany(TruckIntegration::class);
    }

    public function truck_profile_insurances() {
        return $this->hasMany(TruckProfileInsurance::class)->orderBy('id', 'ASC');
    }

    public function truck_additional_insurances() {
        return $this->hasMany(TruckAdditionalInsurance::class)->orderBy('id', 'ASC');
    }

    public function truck_profile_pm_services() {
        return $this->hasMany(TruckProfilePmService::class)->orderBy('id', 'ASC');
    }

    public function truck_additional_pm_services() {
        return $this->hasMany(TruckAdditionalPmService::class)->orderBy('id', 'ASC');
    }

    public function truck_warranty_pm_services() {
        return $this->hasMany(TruckWarrantyPmService::class)->orderBy('id', 'ASC');
    }

    public function truck_profile_equipments() {
        return $this->hasMany(TruckProfileEquipment::class)->orderBy('id', 'ASC');
    }

    public function truck_additional_equipments() {
        return $this->hasMany(TruckAdditionalEquipment::class)->orderBy('id', 'ASC');
    }

    public function truck_profile_documents() {
        return $this->hasMany(TruckProfileDocument::class)->orderBy('id', 'ASC');
    }

    public function truck_additional_documents() {
        return $this->hasMany(TruckAdditionalDocument::class)->orderBy('id', 'ASC');
    }

    public function galleries() {
        return $this->hasMany(Gallery::class, 'action_id', 'id')->where('source', 'truck');
    }

    public function load_details(){
        return $this->hasMany(LoadDetail::class, 'truck_number', 'unit')->select('id', 'load_number');
    }

    public function currentLoadDetail(){
        return $this->hasOne(LoadDetail::class, 'truck_number', 'unit')
            ->where('load_status', 4)
            ->where(function ($query) {
                $query->whereNull('is_on_hold')
                      ->orWhereIn('is_on_hold', [0, 1]);
            })
            ->where('load_type', [0, 6])
            // ->where('dispatched', 1)
            // ->whereNotNull('dispatched_date_time')
            ->where('is_disable', 0)
            // ->whereNull('deleted_at')
            ->latest('actual_shipper_date');
    }

    public function previousLoadDetail(){
        return $this->hasOne(LoadDetail::class, 'truck_number', 'unit')
            ->whereIn('load_status', [5, 6, 10, 8])
            ->where(function ($query) {
                $query->whereNull('is_on_hold')
                      ->orWhereIn('is_on_hold', [0, 1]);
            })
            ->whereIn('load_type', [0, 6])
            // ->where('dispatched', 1)
            // ->whereNotNull('dispatched_date_time')
            ->where('is_disable', 0)
            // ->whereNull('deleted_at')
            ->latest('actual_delivery_date');
    }

    public function getTruckMaintenanceLog(){
        return $this->hasMany(TruckMaintenanceLog::class, 'truck_id', 'id');
    }

    public function vehicleHistoricalStats(){
        return $this->hasMany(VehicleHistoricalStats::class, 'truck_id', 'id')->orderByDesc('last_updated_at');
    }

    public function latestVehicleHistoricalStat(){
        return $this->hasOne(VehicleHistoricalStats::class, 'truck_id', 'id')->orderByDesc('last_updated_at');
    }
}
