<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class TruckProfileInsurance extends Model
{
    use HasFactory;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'company_id', 'truck_id', 'insurance_profile', 'insurance_type', 'insurance_vendor', 'policy_number', 'value', 'date_added', 'expiration_date'
    ];

    /*
     * Scopes
     */
    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }

    public function setDateAddedAttribute($value): ?string{
        return $this->attributes['date_added'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getDateAddedAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }

    public function setExpirationDateAttribute($value): ?string{
        return $this->attributes['expiration_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getExpirationDateAttribute($value): ?string{
        return !is_null($value) ? Carbon::parse($value)->format(config('settings.date_format','m/d/Y')) : NULL;
    }
}
