<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Carbon\Carbon;

class WeeklyFuelSurcharge extends Model
{
    use HasFactory, SoftDeletes;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'company_id', 'fuel_surcharge', 'date', 'start_date', 'end_date', 'week'
    ];

    /*
     * Scopes
     */
    public function scopeIsCompany($query)
    {
        return $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
    }


    /*
     * Mutators
     */
    public function setDateAttribute($value): ?string
    {
        return $this->attributes['date'] = Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d H:i:s');
    }

    public function getDateAttribute($value): ?string
    {
        return Carbon::parse($value)->format(config('settings.date_format','m/d/Y'));
    }



    public function setStartDateAttribute($value): ?string
    {
        return $this->attributes['start_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function setEndDateAttribute($value): ?string
    {
        return $this->attributes['end_date'] = !is_null($value) ? Carbon::createFromFormat(config('settings.date_format','m/d/Y'),$value)->format('Y-m-d') : NULL;
    }

    public function getStartDateAttribute($value): ?string
    {
        return Carbon::parse($value)->format(config('settings.date_format','m/d/Y'));
    }

    public function getEndDateAttribute($value): ?string
    {
        return Carbon::parse($value)->format(config('settings.date_format','m/d/Y'));
    }

}
