<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class CreateOffersTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('offers', function (Blueprint $table) {
            $table->id();
            $table->unsignedBigInteger('driver_id');
            $table->unsignedBigInteger('submitted_by');
            $table->tinyInteger('bound_call')->nullable()->comment('1=Inbound call,0=Outbound call');

            $table->unsignedBigInteger('broker_id')->nullable()->comment('Brokerage Firm');
            $table->string('brokerage_mc')->nullable();
            $table->string('point_of_contact')->nullable();
            $table->string('phone')->nullable();
            $table->string('ext')->nullable();
            $table->string('email')->nullable();
            $table->string('credit')->nullable();

            $table->datetime('shipper_date_time')->nullable();
            $table->string('shipper_location')->nullable();
            $table->datetime('delivery_date_time')->nullable();
            $table->string('delivery_location')->nullable();

            $table->float('empty_miles')->nullable();
            $table->float('loaded_miles')->nullable();
            $table->float('total_miles')->nullable();

            $table->float('rate_offered')->nullable();
            $table->float('fuel_surcharge_cost_per_mile')->nullable();
            $table->float('agreed_upon_rate')->nullable();

            $table->text('notes')->nullable();
            $table->tinyInteger('is_add_note_to_load')->default(0)->comment('1=Yes,0=No');

            $table->enum('favorite',['Yes', 'No'])->default('No');
            $table->tinyInteger('status')->default(0)->comment('0=Pending,1=New,2=Reserved,3=Converted to Load,4=Expired');
            $table->timestamp('status_at')->nullable();
            
            $table->timestamps();
            $table->softDeletes();
            $table->unsignedBigInteger('created_by')->nullable();
            $table->unsignedBigInteger('updated_by')->nullable();
            $table->unsignedBigInteger('deleted_by')->nullable();
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('offers');
    }
}
