<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class AddColumnsForHrPaidTimeOffInDriversTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::table('drivers', function (Blueprint $table) {
            $table->text('days_per_year')->nullable()->comment('0: 00, 1: 01, 2: 02, 3: 03, 4: 04, 5: 05')->after('starting_or_cap_amount');
            $table->text('max_consecutive_days')->nullable()->comment('0: 00 Days, 1: 01 Days, 2: 02 Days, 3: 03 Days, 4: 04 Days, 5: 05 Days')->after('days_per_year');
            $table->text('leave_type')->nullable()->comment('0: Holidays, 1: Vacation, 2: Family Leave, 3: Sick Leave, 4: Bereavement Leave, 5: Personal Leave, 6: Jury Duty, 7: Parental Leave, 8: Voting Leave, 9: Military Leave, 10: Comp. Time Leave')->after('max_consecutive_days');
            $table->text('hire_date')->nullable()->after('leave_type');
            $table->text('reset_date')->nullable()->after('hire_date');
            $table->text('paid_type')->nullable()->comment('0: Unpaid, 1: Paid')->after('reset_date');
            $table->text('pto_rollover')->nullable()->comment('0: No, 1: Yes')->after('paid_type');
            $table->text('max_accumulated_pto')->comment('0: 00 Days, 1: 01 Days, 2: 02 Days, 3: 03 Days, 4: 04 Days, 5: 05 Days, 6: 06 Days, 7: 07 Days, 8: 08 Days')->nullable()->after('pto_rollover');
            $table->text('allow_unpaid_leave')->nullable()->after('max_accumulated_pto');
            $table->text('allow_half_days')->nullable()->after('allow_unpaid_leave');

            $table->text('passenger_onboard')->nullable()->after('allow_half_days');
            $table->text('passenger_agreement_On_file')->nullable()->after('passenger_onboard');
            $table->text('passenger_expiration')->nullable()->after('passenger_agreement_On_file');
            $table->text('passenger_full_name')->nullable()->after('passenger_expiration');
            $table->text('passenger_age')->nullable()->after('passenger_full_name');
            $table->text('passenger_phone_number')->nullable()->after('passenger_age');
            $table->text('passenger_relationship')->nullable()->after('passenger_phone_number');
            $table->text('passenger_gender')->nullable()->comment('0: Male, 1: Female')->after('passenger_relationship');
            $table->text('passenger_allergies')->nullable()->after('passenger_gender');

            $table->text('pet_onboard')->nullable()->after('passenger_allergies');
            $table->text('pet_type')->nullable()->after('pet_onboard');
            $table->text('pet_name')->nullable()->after('pet_type');
            $table->text('pet_deposit_amount')->nullable()->after('pet_name');

            $table->text('medical_condition')->nullable()->comment('0: Corrective Lenses, 1: Blood Pressure, 2: Diabetes, 3: Hearing Aid, 4: Sleep Apnea, 5: Short Physical')->after('pet_deposit_amount');
            $table->text('driving_restrictions')->nullable()->comment('0: No, 1: Yes')->after('medical_condition');
            $table->text('next_doctor_visit')->nullable()->after('driving_restrictions');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::table('drivers', function (Blueprint $table) {
            $table->dropColumn(['days_per_year', 'max_consecutive_days', 'leave_type', 'hire_date', 'reset_date', 'paid_type', 'pto_rollover', 'max_accumulated_pto', 'allow_unpaid_leave', 'allow_half_days']);
        });
    }
}
