<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class AddColumnsToTrucksTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::table('trucks', function (Blueprint $table) {
            $table->text('owner_id')->nullable()->change();
            $table->date('owner_purchase_date')->nullable()->change();

            $table->string('down_payment', 50)->nullable()->after('high_cube');
            $table->string('balance_due', 50)->nullable()->after('down_payment');
            $table->enum('purchase_agreement_on_file', ['0', '1'])->comment('0: No, 1: Yes')->nullable()->after('balance_due');

            $table->text('leasing_company')->nullable()->after('purchase_agreement_on_file');
            $table->text('lease_point_of_contact')->nullable()->after('leasing_company');
            $table->date('lease_start')->nullable()->after('lease_point_of_contact');
            $table->date('lease_end')->nullable()->after('lease_start');
            $table->enum('lease_agreement_on_file', ['0', '1'])->comment('0: No, 1: Yes')->nullable()->after('lease_end');

            $table->text('rental_company')->nullable()->after('lease_agreement_on_file');
            $table->text('rental_point_of_contact')->nullable()->after('rental_company');
            $table->date('rental_start')->nullable()->after('rental_point_of_contact');
            $table->date('rental_end')->nullable()->after('rental_start');
            $table->enum('rental_agreement_on_file', ['0', '1'])->comment('0: No, 1: Yes')->nullable()->after('rental_end');

            $table->enum('billing_type', ['0', '1', '2', '3', '4'])->comment('0: ACH, 1: Cheque, 2: Credit Card, 3: Cash, 4: Invoiced')->nullable()->after('rental_agreement_on_file');
            $table->enum('payment_type', ['0', '1', '2', '3', '4'])->comment('0: ACH, 1: Cheque, 2: Credit Card, 3: Cash, 4: Invoiced')->nullable()->after('billing_type');
            $table->string('payment_amount', 50)->nullable()->after('payment_type');
            $table->string('security_deposit', 50)->nullable()->after('payment_amount');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::table('trucks', function (Blueprint $table) {
            $table->bigInteger('owner_id')->unsigned()->nullable()->change();
            $table->string('owner_purchase_date')->nullable()->change();

            $table->dropColumn(['down_payment', 'balance_due', 'purchase_agreement_on_file', 'leasing_company', 'lease_point_of_contact', 'lease_start', 'lease_end', 'lease_agreement_on_file', 'rental_company', 'rental_point_of_contact', 'rental_start', 'rental_end', 'rental_agreement_on_file', 'billing_type', 'payment_type', 'payment_amount', 'security_deposit']);
        });
    }
}
