<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class AddColumnsForGeneralInfoAt20122023OnBrokersTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::table('brokers', function (Blueprint $table) {
            $table->string('billing_id')->nullable()->after('logo');
            $table->enum('payment_method', ['1', '2', '3', '4', '5'])->comment('1: ACH/Wire, 2: Cash, 3: Certified Check, 4: Credit/Debit Card, 5: Standard Check')->nullable()->after('billing_method');
            $table->string('billing_afterhours', 20)->nullable()->after('payment_method');

            $table->enum('customer_assignment', ['1', '2'])->comment('1: Company, 2: Agent')->nullable()->after('fmcsa_insurance');
            $table->integer('terminal_assignment')->unsigned()->nullable()->after('customer_assignment');
            $table->integer('customer_service_representative')->unsigned()->nullable()->after('terminal_assignment');
            $table->integer('customer_development_representative')->unsigned()->nullable()->after('customer_service_representative');
            $table->integer('customer_sales_representative')->unsigned()->nullable()->after('customer_development_representative');
            $table->integer('billing_representative')->unsigned()->nullable()->after('customer_sales_representative');
            $table->integer('claims_representative')->unsigned()->nullable()->after('billing_representative');
            $table->integer('collection_representative')->unsigned()->nullable()->after('claims_representative');

            $table->enum('factor_invoices', ['0', '1'])->comment('0: No, 1: Yes')->nullable()->after('collection_representative');
            $table->enum('factor_company', ['1', '2', '3'])->comment('1: Denim, 2: RTS, 3: Triumph')->nullable()->after('factor_invoices');
            $table->enum('factor_charge_type', ['1', '2', '3'])->comment('1: Flat, 2: Percentage, 3: Percentage w/ min. charge')->nullable()->after('factor_company');
            $table->double('factor_rate', 10, 2)->nullable()->after('factor_charge_type');
            $table->double('minimum_charge', 10, 2)->nullable()->after('factor_rate');
            $table->enum('term_start_date', ['1', '2', '3'])->comment('1: Pickup Date, 2: Delivery Date, 3: Invoice Date')->nullable()->after('minimum_charge');
            $table->string('terms', 10)->nullable()->after('term_start_date');
            $table->enum('freight_bill_format', ['1', '2'])->comment('1: Itemized, 2: Basic')->nullable()->after('terms');
            $table->date('first_credit_check')->nullable()->after('freight_bill_format');
            $table->date('last_credit_check')->nullable()->after('first_credit_check');
            $table->integer('credit_score')->unsigned()->nullable()->after('last_credit_check');
            $table->double('credit_limit', 10, 2)->nullable()->after('credit_score');
            $table->double('credit_warning', 10, 2)->nullable()->after('credit_limit');
            $table->double('credit_balance', 10, 2)->nullable()->after('credit_warning');
            $table->date('credit_app_completed')->nullable()->after('credit_balance');
            $table->date('credit_app_expiration')->nullable()->after('credit_app_completed');

            $table->enum('required_safer_status', ['1', '2'])->comment('1: Satisfactory, 2: Conditional')->nullable()->after('credit_app_expiration');
            $table->string('required_authority_tenure', 15)->nullable()->after('required_safer_status');
            $table->string('required_fleet_size', 10)->nullable()->after('required_authority_tenure');
            $table->string('required_loads_last_30_days', 10)->nullable()->after('required_fleet_size');
            $table->string('required_loads_per_month', 10)->nullable()->after('required_loads_last_30_days');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::table('brokers', function (Blueprint $table) {
            $table->dropColumn(['billing_id', 'payment_method', 'billing_afterhours', 'customer_assignment', 'terminal_assignment', 'customer_service_representative', 'customer_development_representative', 'customer_sales_representative', 'billing_representative', 'claims_representative', 'collection_representative', 'factor_invoices', 'factor_company', 'factor_charge_type', 'factor_rate', 'minimum_charge', 'term_start_date', 'terms', 'freight_bill_format', 'first_credit_check', 'last_credit_check', 'credit_score', 'credit_limit', 'credit_warning', 'credit_balance', 'credit_app_completed', 'credit_app_expiration', 'required_safer_status', 'required_authority_tenure', 'required_fleet_size', 'required_loads_last_30_days', 'required_loads_per_month']);
        });
    }
}
