<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class AddColumnsForContactsAt27122023OnBrokersTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::table('brokers', function (Blueprint $table) {
            $table->string('contact_first_name', 50)->nullable()->after('required_loads_per_month');
            $table->string('contact_middle_name', 50)->nullable()->after('contact_first_name');
            $table->string('contact_last_name', 50)->nullable()->after('contact_middle_name');
            $table->enum('contact_department', ['1', '2', '3', '4'])->nullable()->comment('1: Safety, 2: Collections, 3: Transport, 4: Dispatch')->after('contact_last_name');
            $table->string('contact_phone_number', 20)->nullable()->after('contact_department');
            $table->string('contact_ext', 10)->nullable()->after('contact_phone_number');
            $table->string('contact_fax', 20)->nullable()->after('contact_ext');
            $table->string('contact_cell_phone', 20)->nullable()->after('contact_fax');
            $table->enum('preferred_contact_method', ['1', '2', '3', '4', '5'])->nullable()->comment('1: Cell Phone, 2: Email, 3: Office Phone, 4: ProChat, 5: SMS')->after('contact_cell_phone');
            $table->date('contact_date_of_birth')->nullable()->after('preferred_contact_method');
            $table->date('contact_start_date_workiversary')->nullable()->after('contact_date_of_birth');
            $table->enum('contact_gender', ['1', '2'])->nullable()->comment('1: Male, 2: Female')->after('contact_start_date_workiversary');
            $table->enum('contact_suffix', ['1', '2', '3', '4', '5'])->nullable()->comment('1: Mr., 2: Mrs., 3: Ms., 4: Miss, 5: Dr.')->after('contact_gender');
            $table->string('contact_email', 50)->nullable()->after('contact_suffix');
            $table->enum('invite_pro_chat_messenger', ['0', '1'])->default('0')->comment('0: No, 1: Yes')->after('contact_email');
            $table->string('contact_prochat_id', 50)->nullable()->after('invite_pro_chat_messenger');
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::table('brokers', function (Blueprint $table) {
            $table->dropColumn(['contact_first_name', 'contact_middle_name', 'contact_last_name', 'contact_department', 'contact_phone_number', 'contact_ext', 'contact_fax', 'contact_cell_phone', 'preferred_contact_method', 'contact_date_of_birth', 'contact_start_date_workiversary', 'contact_gender', 'contact_suffix', 'contact_email', 'invite_pro_chat_messenger', 'contact_prochat_id']);
        });
    }
}
