<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class CreateLoadBillingInvoiceTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('load_billing_invoice', function (Blueprint $table) {
            $table->id();
            $table->integer('load_id')->unsigned();
            $table->integer('load_detail_id')->unsigned();
            $table->string('invoice_number');
            $table->enum('payments_terms_date', ['1', '2'])->comment('1: Invoice Date, 2: Delivery Date');
            $table->enum('payments_terms_days', ['10_days', '30_days', '60_days', '90_days']);
            $table->enum('invoice_template', ['1', '2'])->comment('1: Itemized, 2: Basic');
            $table->date('invoice_date');
            $table->date('due_date');
            $table->datetime('paid_date')->nullable();
            $table->enum('billing_method', ['1', '2', '3'])->comment('1: EDI, 2: Email, 3: Mail')->nullable();
            $table->string('invoice_email', 50)->nullable();
            $table->string('invoice_email_cc', 50)->nullable();
            $table->string('invoice_email_bcc', 50)->nullable();

            $table->integer('customer')->unsigned()->nullable();
            $table->string('bill_to_name')->nullable();
            $table->string('bill_to')->nullable();
            $table->string('po_number')->nullable();
            $table->string('bol_number')->nullable();
            $table->string('pickup_number')->nullable();
            $table->string('delivery_number')->nullable();
            $table->string('notes')->nullable();

            $table->enum('add_qr_code', ['0', '1'])->default('0')->comment('0: No, 1: Yes');
            $table->enum('send_payment_reminder', ['0', '1'])->default('0')->comment('0: No, 1: Yes');
            $table->enum('payment_reminder_days', ['3_days', '7_days', '14_days', '30_days', '45_days'])->nullable();

            $table->integer('salesman')->unsigned()->nullable();
            $table->integer('carrier')->unsigned()->nullable();
            $table->integer('dispatcher')->unsigned()->nullable();
            $table->integer('truck')->unsigned()->nullable();
            $table->integer('trailer')->unsigned()->nullable();
            $table->integer('driver')->unsigned()->nullable();

            $table->double('invoice_amount', 10, 2);
            $table->integer('invoice_created_by')->unsigned()->nullable();
            $table->dateTime('invoice_created_on')->nullable();
            $table->string('invoice_history')->nullable();
            $table->timestamps();
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('load_billing_invoice');
    }
}
