<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class AddTrailerUniqueIdToTrailersTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::table('trailers', function (Blueprint $table) {
            $table->string('trailer_unique_id', 100)->nullable()->after('id');
            $table->unsignedBigInteger('truck_type')->nullable()->comment('company_truck_types')->after('vin');
            $table->string('required_license', 50)->nullable()->after('color');
            $table->unsignedBigInteger('trailer_group')->nullable()->comment('company_trailer_groups')->after('current_assigned_driver');
            $table->enum('assigned_customer', ['0', '1', '2'])->nullable()->comment('0: ABC wood company, 1: Dragon Plastics, 2: Georgia Pacific')->after('trailer_group');
            $table->string('tire_qty', 50)->nullable()->comment('1 to 100')->after('tire_size');
            $table->tinyInteger('doors_type')->nullable()->comment('1=Roll Up,2=Swing')->after('floor_type');
            $table->tinyInteger('plated_trailer')->nullable()->comment('1=Yes,0=No')->after('vented');
            $table->string('inside_width', 50)->nullable()->comment('102 to 50')->after('plated_trailer');
            $table->tinyInteger('paper_spec')->nullable()->comment('1=Yes,0=No')->after('inside_width');
            $table->tinyInteger('pallet_jack')->nullable()->comment('1=Yes,0=No')->after('paper_spec');
            $table->tinyInteger('liftgate')->nullable()->comment('1=Yes,0=No')->after('pallet_jack');
            $table->tinyInteger('high_cube')->nullable()->comment('1=Yes,0=No')->after('liftgate');
            $table->tinyInteger('securement')->nullable()->comment('1=LOGISTIC POSTS,2=ETRACK')->after('spread_axie');
            $table->tinyInteger('tire_auto_inflation')->nullable()->comment('1=Yes,0=No')->after('securement');
            $table->string('air_scale_provider', 100)->nullable()->after('tracker_id');
            $table->string('air_scale_id', 100)->nullable()->after('air_scale_provider');
            $table->string('tire_sensor_provider', 100)->nullable()->after('air_scale_id');
            $table->string('tire_sensor_id', 100)->nullable()->after('tire_sensor_provider');
            $table->string('max_load_weight', 100)->nullable()->after('unladen_weight');
            $table->string('gross_weight', 100)->nullable()->after('max_load_weight');
            $table->date('carb_expiration')->nullable()->after('carb_id');
            $table->tinyInteger('carb_compliant')->nullable()->comment('1=Yes,0=No')->after('carb_expiration');

        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::table('trailers', function (Blueprint $table) {
            $table->dropColumn(['trailer_unique_id', 'truck_type', 'required_license', 'trailer_group', 'assigned_customer', 'tire_qty', 'doors_type', 'plated_trailer', 'inside_width', 'paper_spec', 'pallet_jack', 'liftgate', 'high_cube', 'securement', 'tire_auto_inflation', 'air_scale_provider', 'air_scale_id', 'tire_sensor_provider', 'tire_sensor_id', 'max_load_weight', 'gross_weight', 'carb_expiration', 'carb_compliant']);
        });
    }
}
