@extends($adminTheme)

@section('title')
    Appointment Types
@endsection

@section('content')
<div class="table-control">
    <button class="green-btn create-btn" data-toggle="modal" data-target="#createModal">Add New</button>
</div>
<div class="white-box table-view">
    <h3 class="table-title">Appointment Types</h3>
    <div class="add-responsive">
        <table class="table data-table">
            <thead>
                <tr>
                    <th>Id</th>
                    <th>Name</th>
                    <th>Action</th>
                </tr>
            </thead>
            <tbody>
            </tbody>
        </table>
    </div>
</div>

<div class="modal fade" id="createModal" tabindex="-1" aria-labelledby="createModalLabel" aria-hidden="true">
    <div class="modal-dialog modal-dialog-centered">
        <div class="modal-content">
            <div class="modal-header">
                <h5 class="modal-title" id="createModalLabel">Add Appointment Type</h5>
                <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                    <span aria-hidden="true">&times;</span>
                </button>
            </div>
            {!! Form::open(array('route' => 'admin.appointment-types.store','method'=>'POST','files'=>'true','class'=>'storeForm')) !!}
                <div class="modal-body">
                    <div class="row">
                        <div class="col-md-12">
                            <div class="form-group">
                                <label>Name<span>*</span></label>
                                {!! Form::text('name', old('name'), array('class' => 'form-control', 'autocomplete' => 'new','required' => 'required')) !!}
                            </div>
                        </div>
                    </div>
                </div>
                <div class="modal-footer">
                    <button type="button" class="btn cancel-btn" data-dismiss="modal">Cancel</button>
                    <button class="btn blue-btn">Add New</button>
                </div>
            {!! Form::close() !!}
        </div>
    </div>
</div>

<div class="modal fade" id="editModal" tabindex="-1" aria-labelledby="editModalLabel" aria-hidden="true">
    <div class="modal-dialog modal-dialog-centered">
        <div class="modal-content">
            <div class="modal-header">
                <h5 class="modal-title" id="editModalLabel">Edit Appointment Type</h5>
                <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                    <span aria-hidden="true">&times;</span>
                </button>
            </div>
            <div class="append-form">
                
            </div>
        </div>
    </div>
</div>

@endsection

@section('script')
    <script src="https://unpkg.com/bootstrap-datepicker@1.9.0/dist/js/bootstrap-datepicker.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery.form/4.3.0/jquery.form.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.18.1/moment.min.js"></script> 
    <script src="https://code.jquery.com/ui/1.13.1/jquery-ui.js"></script>
    <script type="text/javascript">
        $('.storeForm').parsley();
        $('.updateForm').parsley();
        
        var table = $('.data-table').DataTable({
            stateSave: true,
            language: {
                search: "",
                searchPlaceholder: "Search...",
            },
            processing: true,
            serverSide: true,
            bJQueryUI: true,  
            order: [[1, "desc"]],
            ajax: "{{ route('admin.appointment-types.index') }}",
            columns: [
                {data: 'id', name:'name'},
                { data: 'name', name: 'name' },
                { data: 'action', name: 'action', orderable: false, searchable: false }
            ]
        });

        $('#createModal').on('shown.bs.modal', function() {
            $('#weeklyDate').datepicker({
                format: dateLayout,
                calendarWeeks: true,
                todayHighlight: true,
                container: '#createModal .modal-body'
            }).on("change", function(e) {
                $('#createModal .modal-body .date').val(e.target.value);
            });
        });

        $("body").on("click", ".create-btn", function () {
            var today   = new Date();
            var dd      = String(today.getDate()).padStart(2, '0');
            var mm      = String(today.getMonth() + 1).padStart(2, '0'); //January is 0!
            var yyyy    = today.getFullYear();

            today       = mm + '/' + dd + '/' + yyyy;

            $(".storeForm").trigger("reset");
            $(".storeForm").find(".date").val(today);
            $(".storeForm").find(".error").text('');
        });

        $("body").on("click", ".edit-btn", function () {
            var token   = "{{ csrf_token() }}";
            var id      = $(this).data('id');
            $.ajax({
                url: "/admin/appointment-types/edit/form/view",
                method: "POST",
                data: { _token : token , id : id },
                success: function(response) {
                    $('#editModal').find('.append-form').html(response.view);

                    $('#weeklyDateEdit').datepicker({
                        format: dateLayout,
                        calendarWeeks: true,
                        todayHighlight: true,
                        container: '#editModal .modal-body'
                    }).on("change", function(e) {
                        $('#editModal .modal-body .date').val(e.target.value);
                    });

                    var date        = new Date(response.data.date);
                    $("#weeklyDateEdit").datepicker("setDate", date);

                    $(".updateForm").ajaxForm({
                        complete: function (response) {
                            if (response.responseJSON.success) {
                                window.location.href = window.location.href;
                            } else {
                                $('.error').text('');
                                $.each( response.responseJSON.errors, function( key, value ) {
                                    var key = key.replace(/\./g, '_');
                                    $.each( value , function( skey, svalue ) {
                                        $('.'+key+'_error').text(svalue);
                                    });
                                });
                            }
                        },
                    });
                }
            });
        });

        $(".storeForm").ajaxForm({
            complete: function (response) {
                if (response.responseJSON.success) {
                    window.location.href = window.location.href;
                } else {
                    $('.error').text('');
                    $.each( response.responseJSON.errors, function( key, value ) {
                        var key = key.replace(/\./g, '_');
                        $.each( value , function( skey, svalue ) {
                            $('.'+key+'_error').text(svalue);
                        });
                    });
                }
            },
        });
    </script>
@endsection