@extends($adminTheme)

@section('title')
    Chart Of Account
@endsection

@section('style')
    <link rel="stylesheet" href="{{ asset('theme/css/select2.min.css') }}">
    <style type="text/css">
        .select2-container{
            width: 100% !important;
        }
        .select2-container .select2-selection--single{
            height: 40px;
        }
        .select2-container--default .select2-selection--single, .select2-container--default .select2-selection--multiple{
            border: 1px solid #707070;
            border-radius: 0;
            color: #000000 !important;
            overflow: hidden !important;
            min-height: 40px;
            height: auto !important;
        }
        .select2-container--default .select2-selection--single .select2-selection__rendered{
            line-height: 40px;
            font-size: 14px;
        }
        .select2-container--default .select2-selection--single .select2-selection__arrow{
            height: 40px;
            margin-right: 5px;
        }
        .select2-container--default .select2-selection--single .select2-selection__arrow b{
            border-color: #000 transparent transparent transparent;
        }
        .drop-search .select2-selection__arrow b{
            display:none !important;
        }
        .custom-error-style {
            color: #dc3545;
            font-weight: bolder;
            font-size: 14px;
            line-height: 1.5;
        }
    </style>
@endsection

@section('content')
    <div class="table-control">
        <button class="green-btn create-btn" data-toggle="modal" data-target="#createModal">Add New</button>
    </div>
    <div class="white-box table-view">
        <h3 class="table-title">Chart Of Account</h3>
        <div class="add-responsive">
            <table class="table data-table">
                <thead>
                <tr>
                    <th width="5%">Account Number</th>
                    <th width="80%">Name</th>
                    <th width="8%">Type</th>
                    <th width="10%">Action</th>
                </tr>
                </thead>
                <tbody>
                </tbody>
            </table>
        </div>
    </div>

    <div class="modal fade" id="createModal" aria-labelledby="createModalLabel" aria-hidden="true">
        <div class="modal-dialog modal-dialog-centered">
            <div class="modal-content">
                <div class="modal-header">
                    <h5 class="modal-title" id="createModalLabel">Add Account Information</h5>
                    <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
                {!! Form::open(array('route' => 'admin.chart-of-accounts.store','method'=>'POST','files'=>'true','class'=>'storeForm')) !!}
                <div class="modal-body">
                    <div class="row">
                        <div class="col-md-12">
                            <div class="form-group drop-search">
                                <label>Account Type<span>*</span></label>
                                <div class="position-relative">
                                    {!! Form::select('account_type', ['' => 'Select Type']+getChartOfAccountsTypes(null, 'listing'), null, ['class' => 'form-control accountType select2', 'required' => 'required', 'data-parsley-errors-container' => '#account_type-errors']) !!}
                                    <img src="{{asset('theme/images/search-icon-black.png')}}" alt="search-icon">
                                </div>
                                <div id="account_type-errors"></div>
                                <div class="mt-1 custom-error-style account_type_error"></div>
                            </div>
                        </div>
                        <div class="col-md-12">
                            <div class="form-group drop-search">
                                <label>Parent Account</label>
                                <div class="position-relative">
                                    {!! Form::select('parent_account', ['' => 'Select Parent'], null, ['class' => 'form-control select2']) !!}
                                    <img src="{{asset('theme/images/search-icon-black.png')}}" alt="search-icon">
                                </div>
                            </div>
                        </div>
                        <div class="col-md-12">
                            <div class="form-group">
                                <label>Account Name<span>*</span></label>
                                {!! Form::text('account_name', old('account_name'), array('class' => 'form-control', 'autocomplete' => 'new','required' => 'required')) !!}
                                <div class="mt-1 custom-error-style account_name_error"></div>
                            </div>
                        </div>
                        <div class="col-md-12">
                            <div class="form-group">
                                <label>Account Number<span>*</span></label>
                                {!! Form::text('account_number', old('account_number'), array('class' => 'form-control', 'autocomplete' => 'new','required' => 'required')) !!}
                                <div class="mt-1 custom-error-style account_number_error"></div>
                            </div>
                        </div>
                    </div>
                </div>
                <div class="modal-footer">
                    <button type="button" class="btn cancel-btn" data-dismiss="modal">Cancel</button>
                    <button class="btn blue-btn">Add New</button>
                </div>
                {!! Form::close() !!}
            </div>
        </div>
    </div>

    <div class="modal fade" id="editModal" aria-labelledby="editModalLabel" aria-hidden="true">
        <div class="modal-dialog modal-dialog-centered">
            <div class="modal-content">
                <div class="modal-header">
                    <h5 class="modal-title" id="editModalLabel">Edit Account Information</h5>
                    <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
                <div class="append-form">

                </div>
            </div>
        </div>
    </div>

@endsection

@section('script')
    <script src="https://unpkg.com/bootstrap-datepicker@1.9.0/dist/js/bootstrap-datepicker.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery.form/4.3.0/jquery.form.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.18.1/moment.min.js"></script>
    <script src="https://code.jquery.com/ui/1.13.1/jquery-ui.js"></script>
    <script src="{{ asset('theme/js/select2.full.min.js') }}"></script>
    <script type="text/javascript">
        $('.storeForm').parsley();
        $('.updateForm').parsley();
        $('.select2').select2();

        var table = $('.data-table').DataTable({
            stateSave: false,
            language: {
                search: "",
                searchPlaceholder: "Search...",
            },
            processing: true,
            serverSide: true,
            bJQueryUI: true,
            pageLength: "{{ perPageView() }}",
            order: [],
            ajax: "{{ route('admin.chart-of-accounts.index') }}",
            columns: [
                { data: 'account_number', name: 'account_number' },
                { data: 'account_name', name: 'account_name', orderable: false },
                { data: 'account_type', name: 'account_type', orderable: false },
                { data: 'action', name: 'action', orderable: false, searchable: false }
            ]
        });

        $("body").on("click", ".edit-btn", function () {
            var token   = "{{ csrf_token() }}";
            var id      = $(this).data('id');
            $.ajax({
                url: "/admin/chart-of-accounts/edit/form/view",
                method: "POST",
                data: { _token : token , id : id },
                success: function(response) {
                    $('#editModal').find('.append-form').html(response.view);
                    $('.select2').select2();

                    $(".updateForm").ajaxForm({
                        complete: function (response) {
                            if (response.responseJSON.success) {
                                window.location.href = window.location.href;
                            } else {
                                $('.error').text('');
                                $.each( response.responseJSON.errors, function( key, value ) {
                                    var key = key.replace(/\./g, '_');
                                    $.each( value , function( skey, svalue ) {
                                        $('.'+key+'_error').text(svalue);
                                    });
                                });
                            }
                        },
                    });
                }
            });
        });

        $(".storeForm").ajaxForm({
            complete: function (response) {
                if (response.responseJSON.success) {
                    window.location.href = window.location.href;
                } else {
                    $('.error').text('');
                    $.each( response.responseJSON.errors, function( key, value ) {
                        var key = key.replace(/\./g, '_');
                        $.each( value , function( skey, svalue ) {
                            console.log('.'+key+'_error');
                            $('.'+key+'_error').text(svalue);
                        });
                    });
                }
            },
        });

        $(function(){
            $(document).on("change", ".accountType", function(){
                var token   = "{{ csrf_token() }}";
                var accountType      = $(this).val();

                $.ajax({
                    url: "/admin/chart-of-accounts/parent-list",
                    method: "POST",
                    data: { _token : token , accountType : accountType },
                    success: function(response) {
                        $('[name="parent_account"]').html("").html(response.dropDownHtml);
                    }
                });
            });
        });
    </script>
@endsection
