@extends($adminTheme)

@section('title')
    Pdf Template
@endsection

@section('content')
    @if(isset($pdf_template))
        {!! Form::model($pdf_template, array(
                'route'         => ['admin.pdf-templates.update', $pdf_template->id],
                'method'        => 'PATCH',
                'files'         => 'true',
                'class'         => 'post-form'
            )
        ) !!}
    @else
        {!! Form::open(array(
                'route'         => 'admin.pdf-templates.store',
                'method'        => 'POST',
                'files'         => 'true',
                'autocomplete'  => 'off',
                'class'         => 'post-form'
            )
        ) !!}
    @endif
        <div class="row">
            <div class="col-lg-12">
                <div class="white-box mb-4 generalAccount">
                    <h4 class="box-title">Pdf Template Information</h4>
                    <div class="box-padding">
                        <div class="row">
                            <div class="col-md-12">
                                <div class="form-group">
                                    <label>Name <span>*</span></label>
                                    {!! Form::text('name', old('name'), array(
                                            'class'         => 'form-control',
                                            'id'            => 'name',
                                            'placeholder'   => 'Enter Name', 
                                            'autocomplete'  => 'off',
                                            'required'      => 'required'
                                        )
                                    ) !!}
                                    @error('name')
                                        <span class="text-danger" role="alert">
                                            <strong>{{ $message }}</strong>
                                        </span>
                                    @enderror
                                </div>
                            </div>
                            <div class="col-md-12">
                                <div class="form-group">
                                    <label>Content <span>*</span></label>
                                    {!! Form::textarea('content', old('content'), array(
                                            'class'                 => 'form-control ck-editor',
                                            'id'                    => 'content',
                                            'required'              => 'required',
                                            'data-parsley-trigger'  => 'change'
                                        )
                                    ) !!}
                                    @error('content')
                                        <span class="text-danger" role="alert">
                                            <strong>{{ $message }}</strong>
                                        </span>
                                    @enderror
                                </div>
                            </div>
                            <div class="col-md-12">
                                <div class="form-group">
                                    <label>Tags</label>
                                    {!! Form::textarea('tags', old('tags'), array(
                                            'class'         => 'form-control',
                                            'id'            => 'tags',
                                            'rows'          => '4',
                                            'placeholder'   => 'e.g. {%LOAD_NUMBER%},{%LOAD_TYPE%},{%LOAD_URL%}',
                                            'autocomplete'  => 'off'
                                        )
                                    ) !!}
                                    <p class="mt-1 font-weight-bold text-danger">NOTE: PLEASE PROVIDE TAGS LIKE {%YOUR_TAG%} WITH COMMA SEPARATED. e.g. {%LOAD_NUMBER%},{%LOAD_TYPE%},{%LOAD_URL%}</p>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
        <div class="button-bottom">
            <a href="{{ route('admin.pdf-templates.index') }}" class="cancel-btn">Cancel</a>
            <button class="submit-btn">Submit</button>
        </div>
    {!! Form::close() !!}
@endsection

@section('script')
    <script src="{{ asset('theme/js/ckeditor/full/ckeditor.js') }}"></script>
    <script type="text/javascript">
        $(document).ready(function() {
            // Initialize Parsley on the form
            $('.post-form').parsley({
                errorsContainer: function(parsleyField) {
                    return parsleyField.$element.hasClass('ck-editor') ? parsleyField.$element.parent("div") : parsleyField;
                }
            });

            // Initialize CKEditor 
            var editor          = CKEDITOR.replace('content', {
                height: '400px' // Set the height here
            });

            // Listen for CKEditor instance ready event
            editor.on('instanceReady', function() {
                // Make CKEditor content field required
                $('.post-form #content').attr('required', '');

                // Listen for change event on CKEditor instances
                editor.on("change", function() {
                    // Update textarea value and trigger Parsley validation
                    editor.updateElement();
                    $('.post-form').parsley().validate();
                });
            });
        });
    </script>
@endsection