@extends($companyAdminTheme)

@section('title')
    Compliance Dashboard Report
@endsection

@section('style')
    <link rel="stylesheet" href="{{ asset('theme/css/select2.min.css') }}">
    <style>
        .select2-container .select2-dropdown .select2-results .select2-results__options::-webkit-scrollbar {
            width: 0 !important;
        }
        .select2-container--default .select2-selection--single .select2-selection__arrow {
            height: 26px !important;
        }
    </style>
@endsection

@section('content')
    <div id="contentDiv">
        <div class="compliance-dashboard-report-hedding" id="report-hedding">
            <div class="row">
                <div class="col-xl-5 col-lg-6">
                    <h2>Compliance Dashboard Report</h2>
                    <span>Date Created: {{ $createdDate ?? '' }}</span>
                </div>
                <div class="col-xl-7 col-lg-6 btnDiv">
                    <ul class="nav justify-content-lg-end">
                        <li class="nav-item">
                            <a class=" btn btn-blue disabled" href="javascript:void(0);" id="btnEmail">Email <img src="{{ asset('theme/images/mail-send.svg') }}" class="img-fluid" alt=""></a>
                        </li>
                        <li class="nav-item">
                            @isset(request()->index)
                                <a class=" btn btn-blue" href="{{ route('company.admin.compliance.report.generate.pdf', ['index' => request()->index]) }}">PDF <img src="{{ asset('theme/images/download-pdf.svg') }}" class="img-fluid" alt=""></a>
                            @else
                                <a class=" btn btn-blue" href="{{ route('company.admin.compliance.report.generate.pdf') }}">PDF <img src="{{ asset('theme/images/download-pdf.svg') }}" class="img-fluid" alt=""></a>
                            @endisset
                        </li>
                        <li class="nav-item">
                            <a class=" btn btn-blue" href="javascript:void(0);" id="btnPrint">Print <img src="{{ asset('theme/images/printer.svg') }}" class="img-fluid" alt=""></a>
                        </li>
                        <li class="nav-item">
                            <a class=" btn btn-blue disabled" href="javascript:void(0);" id="btnExport">Export <img src="{{ asset('theme/images/export.svg') }}" class="img-fluid" alt=""></a>
                        </li>
                    </ul>
                </div>
            </div>
        </div>
        <div id="listingDiv">
            {!! $listViews ?? '' !!}
        </div>
    </div>
@endsection

@section('script')
    <script src="{{ asset('theme/js/select2.full.min.js') }}"></script>
    <script type="text/javascript">
        $(function (){
            $(".select-range-dd").select2({placeholder: "", allowClear: false, minimumResultsForSearch: -1, width: 'auto'});
            $(".select-range-dd~.select2").click(function() {
                $(".select2-dropdown--below").addClass("select2-dd-range");
            });

            $(document).on("change", ".listRange", function (){
                let dataArr = [];

                $('#listingDiv .list-item').each(function (key, val){
                    let that = $(this);
                    let id = that.attr('data-id');
                    let index = that.attr('data-index');
                    let listName = that.attr('data-name');
                    let range = that.find('.form-select').find(':selected').val();
                    let isDraggable = that.attr('data-is-draggable');
                    let isActive = that.attr('data-is-active');
                    dataArr.push({'id': id, 'displayNo': index, 'listName': listName, 'range': range, 'isDraggable': isDraggable, 'isActive': isActive});
                });

                $.ajax({
                    type: 'POST',
                    url: '{!! (isset(request()->index)) ? route("company.admin.compliance.list.data", ["index" => request()->index]) : route("company.admin.compliance.list.data") !!}',
                    data: { isViewAllPage: true, displayListNo: '{!! $displayListNo !!}', dataArray: JSON.stringify(dataArr), filterArr: '{!! json_encode($filters) !!}' },
                    dataType: 'json',
                    headers: {
                        'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                    },
                }).done(function(response) {
                    if(response.success){
                        let listData = response.listData;
                        $("#listingDiv").html("").html(listData);

                        $(".select-range-dd").select2({placeholder: "", allowClear: false, minimumResultsForSearch: -1, width: 'auto'});

                        toastr.success(response.message, 'Success Alert', { timeOut: 5000 });
                    }else{
                        toastr.error(response.message, {
                            timeOut: 5000
                        });
                    }
                });
            });

            $('#btnPrint').on("click", function () {
                let contentDiv = document.getElementById('contentDiv');
                let htmlToPrint = '' +
                    '<link rel="stylesheet" href="{!! asset("/theme/css/bootstrap.min.css") !!}">' +
                    '<link rel="stylesheet" href="{!! asset("/theme/css/dataTables.bootstrap4.min.css") !!}">';
                htmlToPrint += '' +
                    '<style type="text/css">' +
                    '@page { size: A4; margin-top: 0.3in !important; margin-bottom: 0.3in !important }' +
                    '#contentDiv{ padding: 50px 80px 0px 80px }' +
                    '.btnDiv{display: none}' +
                    '.select2-selection__arrow{display: none}' +
                    '.woot--bubble-holder{display: none}' +
                    '.border-none select~.select2-container--default .select2-selection--single{min-width: 85px}' +
                    '.select2-container .select2-selection--single .select2-selection__rendered{padding-right: 0px}' +
                    '.compliance-dashboard-report-hedding { margin-bottom: 30px; }' +
                    '.compliance-dashboard-report-hedding h2{font-size: 33px;font-weight: 800;line-height: 45px;letter-spacing: -0.92px;color: #342828;margin-bottom: 2px}' +
                    '.compliance-dashboard-report-hedding h2~span{font-size: 14px;font-weight: 600;line-height: 19px;letter-spacing: -0.39px;color: #4A5159}' +
                    '.compliance-dashboard-report-tbox {margin-bottom: 20px}' +
                    '.compliance-dashboard-report-tbox .rangeDiv .form-group {display: none;}' +
                    '.compliance-dashboard-report-tbox .rangeDiv .range-label {display: block !important; padding-left: 0; line-height: 20px !important; font-size: 14px; color: #42556C; font-weight: bold; margin-bottom: 1px !important; }' +
                    '.compliance-dashboard-report-tbox h3{ font-size: 22px;font-weight: bold;line-height: 28px;letter-spacing: -0.44px;color: #42556C;margin-bottom: 1px !important; }' +
                    '.compliance-dashboard-report-tbox select{font-size: 14px;font-weight: bold;line-height: 19px;letter-spacing: -0.29px;color: #42556C;border: none;-webkit-appearance: none;-moz-appearance: none;padding: 0 15px}' +
                    '.compliance-dashboard-report-tbox .form-group{position: relative}' +
                    '.compliance-dashboard-report-tbox table thead{background-color: white;color: #342828;border-bottom: 2px solid #305F9C}' +
                    '.compliance-dashboard-report-tbox table thead th{font-size: 16px;font-weight: bold;line-height: 19px;color: #342828;padding: 5px 0 !important;text-align: center}'+
                    '.compliance-dashboard-report-tbox .table tbody tr:first-child td{border-bottom: none !important}' +
                    '.compliance-dashboard-report-tbox .table tbody tr td{font-size: 14px;font-weight: 600;line-height: 19px;color: #342828;padding: 5px 0 !important;text-align: center}' +
                    '.compliance-dashboard-report-tbox table thead th:last-child,.compliance-dashboard-report-tbox .table tbody tr td:last-child{text-align: right}' +
                    '.compliance-dashboard-report-tbox table thead th:first-child,.compliance-dashboard-report-tbox .table tbody tr td:first-child{text-align: left}' +
                    '</style>';
                htmlToPrint += contentDiv.outerHTML;

                let w = window.open();
                w.document.write(htmlToPrint);
                w.document.title = "dashboard-report";
                setTimeout(function(){
                    w.print();
                    w.close();
                }, 50);
            });
        });
    </script>
@endsection
