@extends($companyAdminTheme)

@section('title')
    Mechanical Issue
@endsection

@section('style')
<link rel="stylesheet" type="text/css" href="{{ asset('theme/css/datepicker3.min.css') }}">
<link rel="stylesheet" href="{{ asset('theme/css/select2.min.css') }}">
<link rel="stylesheet" href="{{ asset('theme/css/media3.css') }}">
<style type="text/css">
    .modal .form-group h5{
        font-size: 16px;
    }
    .select2-container{
        width: 100% !important;
    }
    .select2-container .select2-selection--single{
        height: 35px;
    }
    .select2-container--default .select2-selection--single{
        border: 1px solid #C5CEE0;
    }
    .select2-container--default .select2-selection--single .select2-selection__rendered{
        line-height: 34px;
    }
    .select2-container--default .select2-selection--single .select2-selection__arrow{
        height: 31px;
    }
    .select2-container--open .select2-dropdown{
        left: -1px;
    }
</style>
@endsection

@section('content')
<div class="table-control">
    <a href="javascript:void(0);" data-driver-id="{{ $id }}" class="create-mechanical-issues-btn btn green-btn green-link-btn" data-toggle="modal" data-target="#createMechanicalIssuesModal">Add New</a>
</div>
<div class="white-box table-view">
    <h3 class="table-title">Mechanical Issue</h3>
    <div class="add-responsive">
        <table id="driversTable" class="table data-table">
            <thead>
                <tr>
                    <th width="10%">#</th>
                    <th width="15%">Date</th>
                    <th>Title</th>
                    <th style="display: none;">Created At</th>
                    <th width="15%">Action</th>
                </tr>
            </thead>
            <tbody>
            </tbody>
        </table>
    </div>
</div>
@include('companyAdmin.driver.modal.createMechanicalIssue')
@include('companyAdmin.driver.modal.editMechanicalIssue')
@include('companyAdmin.driver.modal.showMechanicalIssue')
@endsection

@section('script')
<script src="{{ asset('theme/js/bootstrap-datepicker.js') }}"></script>
<script src="{{ asset('theme/js/jquery.form.min.js') }}"></script>
<script src="{{ asset('theme/js/select2.full.min.js') }}"></script>
<script>
var table = $('.data-table').DataTable({
    stateSave: true,
    language: {
        search: "",
        searchPlaceholder: "Search...",
    },
    processing: true,
    serverSide: true,
    bJQueryUI: true,
    // pageLength: 1,
    ordering: false,
    order: [[3, "desc"]],
    ajax: "{{ route('company.admin.mechanical.issue.index', $id) }}",
    columns: [
    	{ data: 'DT_RowIndex', name: 'DT_RowIndex', orderable:false, searchable:false },
        { data: 'date', name: 'date' },
        { data: 'title', name: 'title' },
        { data: 'created_at', name: 'created_at', visible: false, searchable:false },
        { data: 'action', name: 'action', orderable:false, searchable:false }
    ]
});

$("body").on("click", ".create-mechanical-issues-btn", function () {
    var token = $('meta[name="csrf-token"]').attr('content');
    var driver_id = $(this).data('driver-id');

    $.ajax({
        url: "/company-admin/mechanical-issue/create/form/view",
        method: "POST",
        data: { _token : token , driver_id : driver_id },
        success: function(response) {
            $('.blank-modal-body').find('.append-form').html('');
            $('#createMechanicalIssuesModal').find('.append-form').html(response.view);

            $('.date').datepicker({
                autoclose: true,
                todayHighlight: true,
            });

            $('.select2').each(function() {
                var $this = $(this);
                var placeholderText = $this.find('option:first').text();
                $this.select2({
                    dropdownParent: $('.storeMechanicalIssueForm'),
                    allowClear: true,
                    width : '100%',
                    placeholder: placeholderText
                });
                $this.find('option:first').hide();
            });

            imagePreview();
            storeMechanicalIssueFormAjax();
        }
    });
});

function storeMechanicalIssueFormAjax() {
    $(".storeMechanicalIssueForm").ajaxForm({
        complete: function (response) {
            if (response.responseJSON.success) {
                window.location.href = window.location.href;
            } else {
                $('.error').text('');
                $.each( response.responseJSON.errors, function( key, value ) {
                    var key = key.replace(/\./g, '_');
                    $.each( value , function( skey, svalue ) {
                        $('.'+key+'_error').text(svalue);
                    });
                });
                toastr.error('Please check the form below for errors', 'Error Alert', {timeOut: 5000})
            }
        },
    });
}

$("body").on("change", ".mechanical-issue-type", function () {
    var type = $(this).val();

    if (type == 1) {
        $(".mechanical-issue-modal").find(".other-div").css('display','block');
    } else {
        $(".mechanical-issue-modal").find(".other-div").css('display','none');
    }
});

$("body").on("click", ".edit-mechanical-issues-btn", function () {
    var token = $('meta[name="csrf-token"]').attr('content');
    var id = $(this).data('id');

    $.ajax({
        url: "/company-admin/mechanical-issue/edit/form/view",
        method: "POST",
        data: { _token : token , id : id },
        success: function(response) {
            $('.blank-modal-body').find('.append-form').html('');
            $('#editMechanicalIssuesModal').find('.append-form').html(response.view);

            $('.date').datepicker({
                autoclose: true,
                todayHighlight: true,
            });

            $('.select2').each(function() {
                var $this = $(this);
                var placeholderText = $this.find('option:first').text();
                $this.select2({
                    dropdownParent: $('.updateMechanicalIssueForm'),
                    allowClear: true,
                    width : '100%',
                    placeholder: placeholderText
                });
                $this.find('option:first').hide();
            });

            imagePreview();
            updateMechanicalIssueFormAjax();
        }
    });
});

function updateMechanicalIssueFormAjax() {
    $(".updateMechanicalIssueForm").ajaxForm({
        complete: function (response) {
            if (response.responseJSON.success) {
                window.location.href = window.location.href;
            } else {
                $('.error').text('');
                $.each( response.responseJSON.errors, function( key, value ) {
                    var key = key.replace(/\./g, '_');
                    $.each( value , function( skey, svalue ) {
                        $('.'+key+'_error').text(svalue);
                    });
                });
                toastr.error('Please check the form below for errors', 'Error Alert', {timeOut: 5000})
            }
        },
    });
}

$("body").on("click", ".show-mechanical-issues-btn", function () {
    var token = $('meta[name="csrf-token"]').attr('content');
    var id = $(this).data('id');

    $.ajax({
        url: "/company-admin/mechanical-issue/show/view",
        method: "POST",
        data: { _token : token , id : id },
        success: function(response) {
            $('#showMechanicalIssuesModal').find('.append-form').html(response.view);
        }
    });
});

function imagePreview() {
    var container = $('.image-wrapper'), inputFile = $('#file'), img, btn, txt = 'Choose file', txtAfter = 'Changes file';
    if(!container.find('#upload').length){
        container.find('.input').append('<input type="button" value="'+txt+'" id="upload">');
        btn = $('#upload');
        container.prepend('<img src="" class="hidden" alt="Uploaded file" id="uploadImg" width="100">');
        img = $('#uploadImg');
    } else {
        btn = $('#upload');
        img = $('#uploadImg');
    }
    btn.on('click', function(){
        img.animate({opacity: 0}, 300);
        inputFile.click();
    });
    inputFile.on('change', function(e){
        container.find('label').html( inputFile.val() );
        var i = 0;
        for(i; i < e.originalEvent.srcElement.files.length; i++) {
            var file = e.originalEvent.srcElement.files[i],
                reader = new FileReader();

            reader.onloadend = function(){
                img.attr('src', reader.result).animate({opacity: 1}, 700);
            }
            reader.readAsDataURL(file);
            img.removeClass('hidden');
        }
        btn.val( txtAfter );
        if(btn.val( txtAfter )){
            $('#upload').addClass('changeImg');
        }
    });
}
</script>
@endsection
