@extends($companyAdminTheme)

@section('title')
    Non Violations
@endsection

@section('style')
<link rel="stylesheet" type="text/css" href="{{ asset('theme/css/datepicker3.min.css') }}">
<link rel="stylesheet" href="{{ asset('theme/css/select2.min.css') }}">
<link rel="stylesheet" href="{{ asset('theme/css/media3.css') }}">
<style type="text/css">
    .modal .form-group h5{
        font-size: 16px;
    }
    .select2-container{
        width: 100% !important;
    }
    .select2-container .select2-selection--single{
        height: 35px;
    }
    .select2-container--default .select2-selection--single{
        border: 1px solid #C5CEE0;
    }
    .select2-container--default .select2-selection--single .select2-selection__rendered{
        line-height: 34px;
    }
    .select2-container--default .select2-selection--single .select2-selection__arrow{
        height: 31px;
    }
    .select2-container--open .select2-dropdown{
        left: -1px;
    }
</style>
@endsection

@section('content')
<div class="table-control">
    <a href="javascript:void(0);" data-driver-id="{{ $id }}" class="create-non-violations-btn btn green-btn green-link-btn" data-toggle="modal" data-target="#createNonViolationsModal">Add New</a>
</div>
<div class="white-box table-view">
    <h3 class="table-title">Non Violations</h3>
    <div class="add-responsive">
        <table id="driversTable" class="table data-table">
            <thead>
                <tr>
                    <th width="10%">#</th>
                    <th width="15%">Date</th>
                    <th>Title</th>
                    <th style="display: none;">Created At</th>
                    <th width="15%">Action</th>
                </tr>
            </thead>
            <tbody>
            </tbody>
        </table>
    </div>
</div>
@include('companyAdmin.driver.modal.createNonViolation')
@include('companyAdmin.driver.modal.editNonViolation')
@include('companyAdmin.driver.modal.showNonViolation')
@endsection

@section('script')
<script src="{{ asset('theme/js/bootstrap-datepicker.js') }}"></script>
<script src="{{ asset('theme/js/jquery.form.min.js') }}"></script>
<script src="{{ asset('theme/js/select2.full.min.js') }}"></script>
<script>
var table = $('.data-table').DataTable({
    stateSave: true,
    language: {
        search: "",
        searchPlaceholder: "Search...",
    },
    processing: true,
    serverSide: true,
    bJQueryUI: true,
    // pageLength: 1,
    ordering: false,
    order: [[3, "desc"]],
    ajax: "{{ route('company.admin.non.violation.index', $id) }}",
    columns: [
    	{ data: 'DT_RowIndex', name: 'DT_RowIndex', orderable:false, searchable:false },
        { data: 'date', name: 'date' },
        { data: 'title', name: 'title' },
        { data: 'created_at', name: 'created_at', visible: false, searchable:false },
        { data: 'action', name: 'action', orderable:false, searchable:false }
    ]
});

// create non violation modal
$("body").on("click",".create-non-violations-btn",function() {
    var token           = $('meta[name="csrf-token"]').attr('content');
    var driver_id       = $(this).data('driver-id');
    $.ajax({
        url: "/company-admin/non-violation/create/form/view",
        method: "POST",
        data: {_token: token, driver_id: driver_id},
        success: function (response) {
            $('.blank-modal-body').find('.append-form').html('');
            $('#createNonViolationsModal').find('.append-form').html(response.view);

            $('.date').datepicker({
                autoclose: true,
                todayHighlight: true,
            });

            $('.cutomtime').datetimepicker({
                format: 'HH:mm',
                useCurrent: true,
            });

            $('.select2').each(function() {
                var $this = $(this);
                var placeholderText = $this.find('option:first').text();
                $this.select2({
                    dropdownParent: $('.storeHOSViolationForm'),
                    allowClear: true,
                    width : '100%',
                    placeholder: placeholderText
                });
                $this.find('option:first').hide();
            });

            imagePreview();
            storeHOSViolationFormAjax();
            AddMoreHOS();

            $('.unit1').change();
            $('.unit2').change();
        }
    });

});
function storeHOSViolationFormAjax() {
    $(".storeHOSViolationForm").ajaxForm({
        complete: function (response) {
            if (response.responseJSON.success) {
                window.location.href = window.location.href;
            } else {
                $('.error').text('');
                $.each( response.responseJSON.errors, function( key, value ) {
                    var key = key.replace(/\./g, '_');
                    $.each( value , function( skey, svalue ) {
                        $('.'+key+'_error').text(svalue);
                    });
                });
                toastr.error('Please check the form below for errors', 'Error Alert', {timeOut: 5000})
            }
        },
    });
}

$("body").on("click", ".edit-non-violations-btn", function () {
    var token = $('meta[name="csrf-token"]').attr('content');
    var id = $(this).data('id');

    $.ajax({
        url: "/company-admin/non-violation/edit/form/view",
        method: "POST",
        data: {_token: token, id: id},
        success: function (response) {
            $('.blank-modal-body').find('.append-form').html('');
            $('#editNONViolationsModal').find('.append-form').html(response.view);

            $('.date').datepicker({
                autoclose: true,
                todayHighlight: true,
            });

            $('.cutomtime').datetimepicker({
                format: 'HH:mm',
                useCurrent: true,
            });

            $('.select2').each(function() {
                var $this = $(this);
                var placeholderText = $this.find('option:first').text();
                $this.select2({
                    dropdownParent: $('.updateHOSViolationForm'),
                    allowClear: true,
                    width : '100%',
                    placeholder: placeholderText
                });
                $this.find('option:first').hide();
            });

            $('.updateHOSViolationForm').parsley();

            imagePreview();
            updateHOSViolationFormAjax();
            AddMoreHOS();

            $('.unit1').change();
            $('.unit2').change();

        }
    });
});

function updateHOSViolationFormAjax() {
    $(".updateHOSViolationForm").ajaxForm({
        complete: function (response) {
            if (response.responseJSON.success) {
                window.location.href = window.location.href;
            } else {
                $('.error').text('');
                $.each( response.responseJSON.errors, function( key, value ) {
                    var key = key.replace(/\./g, '_');
                    $.each( value , function( skey, svalue ) {
                        $('.'+key+'_error').text(svalue);
                    });
                });
                toastr.error('Please check the form below for errors', 'Error Alert', {timeOut: 5000})
            }
        },
    });
}

$("body").on("click", ".show-non-violations-btn", function () {
    var token = $('meta[name="csrf-token"]').attr('content');
    var id = $(this).data('id');

    $.ajax({
        url: "/company-admin/non-violation/show/view",
        method: "POST",
        data: { _token : token , id : id },
        success: function(response) {
            $('#showNONViolationsModal').find('.append-form').html(response.view);
        }
    });
});

function imagePreview() {
    var container = $('.image-wrapper'), inputFile = $('#file'), img, btn, txt = 'Choose file', txtAfter = 'Changes file';
    if(!container.find('#upload').length){
        container.find('.input').append('<input type="button" value="'+txt+'" id="upload">');
        btn = $('#upload');
        container.prepend('<img src="" class="hidden" alt="Uploaded file" id="uploadImg" width="100">');
        img = $('#uploadImg');
    } else {
        btn = $('#upload');
        img = $('#uploadImg');
    }
    btn.on('click', function(){
        img.animate({opacity: 0}, 300);
        inputFile.click();
    });
    inputFile.on('change', function(e){
        container.find('label').html( inputFile.val() );
        var i = 0;
        for(i; i < e.originalEvent.srcElement.files.length; i++) {
            var file = e.originalEvent.srcElement.files[i],
                reader = new FileReader();

            reader.onloadend = function(){
                img.attr('src', reader.result).animate({opacity: 1}, 700);
            }
            reader.readAsDataURL(file);
            img.removeClass('hidden');
        }
        btn.val( txtAfter );
        if(btn.val( txtAfter )){
            $('#upload').addClass('changeImg');
        }
    });
}

function AddMoreHOS(){
    $("#HosaddRow").click(function () {
        var html = '';
        html += '<div class="row" id="hosvolation">';
        html += '<div class="col-md-4">';
        html += '<div class="form-group">';
        html += '<label>Unit</label>';
        html += '<select name="unit_type[]" class="form-control unittype" required>';
        html += '<option value="">Select Unit</option>';
        html += '<option value="1">Driver</option>';
        html += '<option value="2">Truck</option>';
        html += '<option value="3">Trailer</option>';
        html += '</select>';
        html += '</div>';
        html += '</div>';
        html += '<div class="col-md-2 make">';
        html += '<div class="form-group">';
        html += '<label>Make</label>';
        html += '<input type="text" name="make[]" value="" class="form-control" required>';
        html += '</div>';
        html += '</div>';
        html += '<div class="col-md-2 year">';
        html += '<div class="form-group">';
        html += '<label>Year</label>';
        html += '<input type="text" name="year[]" value="" class="form-control" required>';
        html += '</div>';
        html += '</div>';
        html += '<div class="col-md-4 vin">';
        html += '<div class="form-group">';
        html += '<label>VIN</label>';
        html += '<input type="text" name="vin[]" value="" class="form-control" required>';
        html += '</div>';
        html += '</div>';
        html += '<div class="col-md-3">';
        html += '<div class="form-group">';
        html += '<label>OOS</label>';
        html += '<select name="oos[]" class="form-control" required>';
        html += '<option value="">Select OOS</option>';
        html += '<option value="1">Yes</option>';
        html += '<option value="0">No</option>';
        html += '</select>';
        html += '</div>';
        html += '</div>';
        html += '<div class="col-md-3 violation_code">';
        html += '<div class="form-group">';
        html += '<label>Violation Code</label>';
        html += '<select name="violation_code[]" class="form-control" required>';
        html += '<option value="">Violation Code</option>';
        html += '</select>';
        html += '</div>';
        html += '</div>';
        html += '<div class="col-md-4 description">';
        html += '<div class="form-group">';
        html += '<label>Description </label>';
        html += '<input type="text" name="description[]" value="" class="form-control" required>';
        html += '</div>';
        html += '</div>';
        html += '<div class="col-md-2">';
        html += '<div class="form-group">';
        html += '<label class="d-block" style="visibility: hidden;">Action</label>';
        html += '<a href="javascript:void(0)" class="btn btn-danger" id="HosremoveRow"><i class="fa fa-trash"></i></a>';
        html += '</div>';
        html += '</div>';
        html += '</div>';

        $('#HosnewRow').append(html);



    });




}

// remove row
$(document).on('click', '#HosremoveRow', function () {
    $(this).closest('#hosvolation').remove();
});




$(document).on('change', '.unittype', function () {
    var value = this.value;
    console.log(value);

    if(value == '1'){
        $(this).closest('#hosvolation').find('.violation_code select').remove();
        $(this).closest('#hosvolation').find('.violation_code span').remove();
        $(".hosvolationdata.firsttime").clone().appendTo($(this).closest('#hosvolation').find('.violation_code .form-group'));
        $(this).closest('#hosvolation').find('.violation_code select').removeClass('firsttime');

        $(this).closest('#hosvolation').find('.violation_code').removeClass('col-md-3');
        $(this).closest('#hosvolation').find('.violation_code').addClass('col-md-5');

        $(this).closest('#hosvolation').find('.description').removeClass('col-md-4');
        $(this).closest('#hosvolation').find('.description').addClass('col-md-10');

        $(this).closest('#hosvolation').find('.make').hide(300);
        $(this).closest('#hosvolation').find('.year').hide(300);
        $(this).closest('#hosvolation').find('.vin').hide(300);

        $(this).closest('#hosvolation').find('.make input').prop('required',false);
        $(this).closest('#hosvolation').find('.year input').prop('required',false);
        $(this).closest('#hosvolation').find('.vin input').prop('required',false);

    }

    if(value == '2' || value == '3'){
        $(this).closest('#hosvolation').find('.violation_code select').remove();
        $(this).closest('#hosvolation').find('.violation_code span').remove();
        $(".hoscitationsdata.firsttime").clone().appendTo($(this).closest('#hosvolation').find('.violation_code .form-group'));
        $(this).closest('#hosvolation').find('.violation_code select').removeClass('firsttime');

        $(this).closest('#hosvolation').find('.violation_code').addClass('col-md-3');
        $(this).closest('#hosvolation').find('.violation_code').removeClass('col-md-5');

        $(this).closest('#hosvolation').find('.description').addClass('col-md-4');
        $(this).closest('#hosvolation').find('.description').removeClass('col-md-10');

        $(this).closest('#hosvolation').find('.make').show(300);
        $(this).closest('#hosvolation').find('.year').show(300);
        $(this).closest('#hosvolation').find('.vin').show(300);

        $(this).closest('#hosvolation').find('.make input').prop('required',true);
        $(this).closest('#hosvolation').find('.year input').prop('required',true);
        $(this).closest('#hosvolation').find('.vin input').prop('required',true);

        if(value == '2'){
            var truck_make = $('.noneusedata input[name="truck_make"]').val();
            var truck_year = $('.noneusedata input[name="truck_year"]').val();
            var truck_vin = $('.noneusedata input[name="truck_vin"]').val();

            $(this).closest('#hosvolation').find('.make input').val(truck_make);
            $(this).closest('#hosvolation').find('.year input').val(truck_year);
            $(this).closest('#hosvolation').find('.vin input').val(truck_vin);
        }

        if(value == '3'){
            var trailer_make = $('.noneusedata input[name="trailer_make"]').val();
            var trailer_year = $('.noneusedata input[name="trailer_year"]').val();
            var trailer_vin = $('.noneusedata input[name="truck_vin"]').val();

            $(this).closest('#hosvolation').find('.make input').val(trailer_make);
            $(this).closest('#hosvolation').find('.year input').val(trailer_year);
            $(this).closest('#hosvolation').find('.vin input').val(trailer_vin);
        }

        setTimeout( function(){
            if($('#editHOSViolationsModal').hasClass('show') || $('#editNONViolationsModal').hasClass('show')){
                $('.select2').each(function() {
                    var $this = $(this);
                    var placeholderText = $this.find('option:first').text();
                    $this.select2({
                        dropdownParent: $('.updateHOSViolationForm'),
                        allowClear: true,
                        width : '100%',
                        placeholder: placeholderText
                    });
                    $this.find('option:first').hide();
                });
            }else{
                $('.select2').each(function() {
                    var $this = $(this);
                    var placeholderText = $this.find('option:first').text();
                    $this.select2({
                        dropdownParent: $('.storeHOSViolationForm'),
                        allowClear: true,
                        width : '100%',
                        placeholder: placeholderText
                    });
                    $this.find('option:first').hide();
                });
            }
        }, 500);

    }

    $('.select2').each(function() {
        var $this = $(this);
        var placeholderText = $this.find('option:first').text();
        $this.select2({
            dropdownParent: $('.updateHOSViolationForm'),
            allowClear: true,
            width : '100%',
            placeholder: placeholderText
        });
        $this.find('option:first').hide();
    });
    <?php if(!empty($dirver->id)): ?>
        $('.select2').each(function() {
            var $this = $(this);
            var placeholderText = $this.find('option:first').text();
            $this.select2({
                dropdownParent: $('.updateHOSViolationForm'),
                allowClear: true,
                width : '100%',
                placeholder: placeholderText
            });
            $this.find('option:first').hide();
        });
    <?php else: ?>
        $('.select2').each(function() {
            var $this = $(this);
            var placeholderText = $this.find('option:first').text();
            $this.select2({
                dropdownParent: $('.storeHOSViolationForm'),
                allowClear: true,
                width : '100%',
                placeholder: placeholderText
            });
            $this.find('option:first').hide();
        });
    <?php endif; ?>

});

$(document).on('change', '.unit1', function () {
    var truck_id = this.value;
    var token = $('meta[name="csrf-token"]').attr('content');

    $.ajax({
        url: "{{route('company.admin.truck.get.details')}}",
        method: "POST",
        data: { _token : token , truck_id : truck_id},
        success: function(response) {

            $('.noneusedata input[name="truck_make"]').val(response.make);
            $('.noneusedata input[name="truck_year"]').val(response.year);
            $('.noneusedata input[name="truck_vin"]').val(response.vin_number);

        }
    });
});

$(document).on('change', '.unit2', function () {
    var trailer_id = this.value;
    var token = $('meta[name="csrf-token"]').attr('content');

    $.ajax({
        url: "{{route('company.admin.trailer.get.details')}}",
        method: "POST",
        data: { _token : token , trailer_id : trailer_id},
        success: function(response) {

            $('.noneusedata input[name="trailer_make"]').val(response.make);
            $('.noneusedata input[name="trailer_year"]').val(response.year);
            $('.noneusedata input[name="trailer_vin"]').val(response.vin);

        }
    });
});


$(document).on('change', '.hosvolationdata', function () {
    var title = $(this).find(":selected").data("title");
    $(this).closest('#hosvolation').find('.description input').val(title);
});

$(document).on('change', '.hoscitationsdata', function () {
    var title = $(this).find(":selected").data("title");
    $(this).closest('#hosvolation').find('.description input').val(title);
});



</script>
@endsection
