@extends($companyAdminTheme)

@section('title')
    Driver Onboarding Setting
@endsection

@section('style')
    <style type="text/css">
        .table {
            border-radius: 10px 10px 0px 0px;
            border-spacing: 0;
            border-collapse: separate;
            overflow: hidden;
            font-weight: 600;
            border: 1px solid #CCCCCC;
        }
        .table thead th {
            background: transparent linear-gradient(180deg, #417CCB 0%, #294E7C 100%) 0% 0% no-repeat padding-box;
            font: normal normal bold 15px Inter, Sans-Serif;
            letter-spacing: 0px;
            color: #FFFFFF !important;
        }
        .ngd-new .load-requirements-table .table td, .ngd-new .table tbody tr:first-child td {
            border-bottom: 0 !important;
        }
        .addStageBtn {
            border-radius: 10px !important;
        }
        .card {
            border-radius: 10px !important;
        }
        .rounded-corners {
            border-radius: 10px;
            border-spacing: 0;
            border-collapse: separate;
            overflow: hidden;
            font-weight: 600;
        }
        .rounded-corners th:not(:last-child),
        .rounded-corners td:not(:last-child) {
            border-right: var(--border);
        }
        .rounded-corners>thead>tr:not(:last-child)>th,
        .rounded-corners>thead>tr:not(:last-child)>td,
        .rounded-corners>tbody>tr:not(:last-child)>th,
        .rounded-corners>tbody>tr:not(:last-child)>td,
        .rounded-corners>tfoot>tr:not(:last-child)>th,
        .rounded-corners>tfoot>tr:not(:last-child)>td,
        .rounded-corners>tr:not(:last-child)>td,
        .rounded-corners>tr:not(:last-child)>th,
        .rounded-corners>thead:not(:last-child),
        .rounded-corners>tbody:not(:last-child),
        .rounded-corners>tfoot:not(:last-child) {
            border-bottom: var(--border);
        }
    </style>
@endsection

@section('content')
    <div class="card px-3 py-3">
        <div class="white-box">
            <div class="ttl-box">
                <div class="row align-items-center">
                    <div class="col-md-6 p-0">
                        <h3 class="table-title mb-2">Onboarding Setting</h3>
                    </div>
                    <div class="col-md-6 p-0 text-right">
                        <a href="{{ route('company.admin.driver.onboarding.setting.create', ['driver_id' => request()->driver_id]) }}" class="btn btn-blue addStageBtn">Add New Stage</a>
                    </div>
                </div>
            </div>
            <div class="table-responsive">
                <table class="table rounded-corners">
                    <thead>
                    <tr>
                        <th width="10%" class="text-center">Order</th>
                        <th width="75%">Title</th>
                        <th width="15%" class="text-center">Action</th>
                    </tr>
                    </thead>
                    <tbody id="sortable" data-group="pages">
                    @if (count($data))
                        @foreach ($data as $value)
                            <tr class="list-onboard" data-id="{{ $value->id }}">
                                {{--<td class="text-center">{{ $value->order }}</td>--}}
                                <td class="text-center">{{ $loop->iteration }}</td>
                                <td>{{ $value->title }}</td>
                                <td class="text-center">
                                    <a href="{{ route('company.admin.driver.onboarding.setting.edit', ['driver_id' => request()->driver_id, 'id' => $value->id]) }}" class="mr-4"><img src="{{ asset('theme/img/icon-material-edit.svg') }}" alt=""></a>
                                    <a href="javascript:void(0);" class="destroy" data-action="{{ route('company.admin.driver.onboarding.setting.destroy', ['driver_id' => request()->driver_id, 'id' => $value->id]) }}" data-id="{{ $value->id }}"><img src="{{ asset('theme/img/icon-material-delete.svg') }}" alt=""></a>
                                </td>
                            </tr>
                        @endforeach
                    @else
                        <tr class="text-center">
                            <td colspan="3">No data</td>
                        </tr>
                    @endif
                    </tbody>
                </table>
            </div>
        </div>
    </div>
@endsection

@section('script')
<script src="https://cdnjs.cloudflare.com/ajax/libs/Sortable/1.14.0/Sortable.min.js"></script>
<script>
new Sortable(sortable, {
    animation: 150,
    onUpdate: function (/**Event*/ evt) {
        let order = {};
        var i = 1;
        $(".list-onboard").each(function () {
            order[i++] = {
                id: $(this).data("id"),
                display_order: $(this).index() + 1,
            };
        });

        var token = $("input[name='_token']").val();

        $.ajax({
            url: "{{ route('company.admin.driver.onboarding.setting.sort', ['driver_id' => request()->driver_id]) }}",
            data: { _token : token , sort : order },
            method : "POST",
            success: function(response) {
                toastr.success(response.success, "Success Alert", {
                    timeOut: 5000,
                });
            }
        });
    },
});
</script>
@endsection
