@extends($companyAdminTheme)

@section('title')
    Map
@endsection

@section('style')
    <link rel="stylesheet" href="{{ asset('theme/css/select2.min.css') }}">
    <style type="text/css">
        .map-box {
            height: calc(100vh - 164px);
            min-height: 470px;
            background: #dce4ee;
            border: 1px solid #d8d8dd;
            border-radius: 10px;
        }
        #map {
            position: absolute;
            top: 0;
            bottom: 0;
            width: 100%;
        }
        .filter-menu {
            position: absolute;
            width: 240px;
            top: 100px;
            left: 10px;
            padding: 8px;
            background-color: #fff;
            box-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
            font-size: .85em;
        }
        .filter-ctrl {
            position: absolute;
            width: 180px;
            top: 10px;
            right: 10px;
            z-index: 1;
        }
        .info-panel {
            position: absolute;
            width: 240px;
            top: 10px;
            left: 10px;
            padding: 8px;
            background-color: #fff;
            box-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
            font-size: .85em;
        }
        .info-panel span {
            color: #576571;
            line-height: 32px;
            display: inline-block;
            vertical-align: bottom;
        }
        .group .button {
            display: inline-block;
            height: 22px !important;
            width: 22px !important;
            border: 1px solid transparent;
            background-repeat: no-repeat;
            background-position: center;
        }
        .group .button.truck-outline-green {
            background-image: url('{{ asset("theme/images/truck-outline-green.png") }}');
            background-size: cover;
        }
        .group .button.truck-outline-red {
            background-image: url('{{ asset("theme/images/truck-outline-red.png") }}');
            background-size: cover;
        }
    </style>
    <style type="text/css">
        .select2-container--default .select2-selection--single, .select2-container--default .select2-selection--multiple {
            border: 1px solid #D8D8DD !important;
            border-radius: 3px !important;
            color: #000000 !important;
            overflow: hidden !important;
            min-height: 40px;
            height: auto !important;
        }
        .select2-container--default .select2-selection--single .select2-selection__arrow {
            height: 40px !important;
            margin-right: 5px;
            background-image: url("{{ asset('theme/images/select-dropdown.svg') }}");
            background-size: auto;
            background-position: center right 5px;
            background-repeat: no-repeat;
        }
        .select2-container--default .select2-selection--single .select2-selection__arrow b {
            border-color: #000 transparent transparent transparent;
            display: none;
        }
    </style>
@endsection

@section('content')
    <div class="map-box">
        <div id="map" data-map-instance="null"></div>
        <div class="filter-menu d-none">
            <input id="all-trucks" type="radio" name="rtoggle" value="All Trucks" checked="checked" autocomplete="off" />
            <label for="all-trucks">All Trucks</label>
            <input id="active-trucks" type="radio" name="rtoggle" value="Active Truck" autocomplete="off" />
            <label for="active-trucks">Active Truck</label>
            <input id="inactive-trucks" type="radio" name="rtoggle" value="Inactive Truck" autocomplete="off" />
            <label for="inactive-trucks">Inactive Truck</label>
        </div>
        <div class="filter-ctrl">
            <select class="form-control" name="filter">
                <option value="">Select</option>
            </select>
        </div>
        <div class="info-panel">
            <div class="group">
                <div class="button truck-outline-green"></div>
                <span>Active Truck</span>
            </div>
            <div class="group">
                <div class="button truck-outline-red"></div>
                <span>Inactive Truck</span>
            </div>
        </div>
    </div>
@endsection

@section('script')
    <script src="{{ asset('theme/js/select2.full.min.js') }}"></script>
    <script type="text/javascript">
        if (trimbleMapsAPIKey) {
            var toggleFilteredArray = [];
            var center              = [-98.583, 39.833];
            var zoomLevel           = 3.5;
            var minzoom             = 9;

            var map                 = new TrimbleMaps.Map({
                container: 'map', // container id
                style: TrimbleMaps.Common.Style.TRANSPORTATION, //hosted style id
                center: center, // starting position
                zoom: zoomLevel, // starting zoom
                scrollZoom: false
            });

            // Store map instance in data attribute
            $('#map').data('map-instance', map);

            // Disable scroll zoom controls
            // map.scrollZoom.disable();

            // Wait for the map to load before adding layers and data sources
            map.on('load', async () => {
                try {
                    const response  = await new Promise((resolve, reject) => {
                        $.ajax({
                            url: '/company-admin/map/cluster',
                            type: 'POST',
                            data: {},
                            dataType: 'json',
                            headers: {
                                'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                            },
                            beforeSend: function () {
                                // console.log("Fetching coordinates...");
                            },
                            success: function (data) {
                                resolve(data);
                            },
                            error: function (jqXHR, status, exception) {
                                reject({ jqXHR, status, exception });
                            }
                        });
                    });
                    if(response.success){
                        // First AJAX request was successful
                        var data                    = response.data;

                        var filteredArray           = [];
                        // Check if data is defined, not null, an array, and has items
                        if (typeof data !== 'undefined' && data !== null && Array.isArray(data) && data.length > 0) {
                            // Filter out items with valid lat/lng
                            filteredArray           = data.filter(item => {
                                return item.lat !== undefined && item.lat.trim() !== "" && item.lng !== undefined && item.lng.trim() !== "";
                            });
                            // Log the initial filtered array for lat/lng validity
                            // console.log("Filtered Array (Lat/Lng valid):", filteredArray);

                            // Update toggleFilteredArray with filtered data
                            toggleFilteredArray      = filteredArray;
                            // Log the updated toggleFilteredArray
                            // console.log("toggleFilteredArray updated:", toggleFilteredArray);
                        } else {
                            // console.log("No valid data to process.");
                        }
                        
                        // Initialize an empty array for features
                        var features                = [];

                        // Check if there are items in filteredArray
                        if (filteredArray.length > 0) {
                            // Perform the action here if filteredArray is not empty
                            // console.log("filteredArray is not empty:", filteredArray);

                            // Check if any radio button with name "rtoggle" is checked
                            if ($('.filter-menu input[name="rtoggle"]:checked').length > 0) {
                                // Get the value of the checked radio button
                                var rtoggle         = $('.filter-menu input[name="rtoggle"]:checked').val();

                                // Log the state of filteredArray before further filtering
                                // console.log("filteredArray before 'rtoggle' filtering:", filteredArray);

                                // Depending on the rtoggle value, filter the data array
                                if (rtoggle === "Inactive Truck") {
                                    // Filter for "Inactive Truck"
                                    filteredArray   = filteredArray.filter(item => item.category === "Inactive Truck");
                                    // console.log("Filtered Inactive Trucks:", filteredArray);
                                } else if (rtoggle === "Active Truck") {
                                    // Filter for "Active Truck"
                                    filteredArray   = filteredArray.filter(item => item.category === "Active Truck");
                                    // console.log("Filtered Active Trucks:", filteredArray);
                                } else {
                                    // Handle other cases or do nothing
                                }

                                // Log the final filtered array after 'rtoggle' filtering
                                // console.log("Final Filtered Array:", filteredArray);
                            }

                            // Convert filteredArray items to GeoJSON features
                            $.each(filteredArray, function(index, obj) {
                                features.push({
                                    type: 'Feature',
                                    properties: {
                                        name: obj.text,
                                        category: obj.category,
                                        load_number: (obj.load_number !== undefined && obj.load_number !== null && $.trim(obj.load_number) !== '') ? obj.load_number : '',
                                        url: (obj.url !== undefined && obj.url !== null && $.trim(obj.url) !== '') ? obj.url : ''
                                    },
                                    geometry: {
                                        type: 'Point',
                                        coordinates: [obj.lng, obj.lat]
                                    }
                                });
                            });
                        } else {
                            // Log a message indicating that filteredArray is empty
                            // console.log("filteredArray is empty. No items to process.");
                        }

                        // Log features array after populating it
                        // console.log("Features:", features);

                        // Use cluster properties on datasource to group data
                        map.addSource('randomPoints', {
                            type: 'geojson',
                            cluster: true,
                            clusterRadius: 40,
                            clusterMaxZoom: 14,
                            data: {
                                type: 'FeatureCollection',
                                features: features
                            }
                        });

                        // Use filter to shot clustered points
                        map.addLayer({
                            id: 'clusteredPoints',
                            type: 'circle',
                            source: 'randomPoints',
                            filter: ['has', 'point_count'],
                            paint: {
                                'circle-radius': [
                                    'step',
                                    ['get', 'point_count'],
                                    15, 10,
                                    20, 20,
                                    25
                                ],
                                'circle-color': '#000000',
                                'circle-stroke-color': '#FFF',
                                'circle-stroke-width': 3
                            }
                        });

                        // Show count for clustered points
                        map.addLayer({
                            id: 'clusterCount',
                            type: 'symbol',
                            source: 'randomPoints',
                            filter: ['has', 'point_count'],
                            layout: {
                                'text-field': '{point_count}',
                                'text-font': ['Roboto Regular'],
                                'text-size': 12
                            },
                            paint: {
                                'text-color': '#FFF'
                            }
                        });

                        // Use filter to show unclustered points
                        map.addLayer({
                            id: 'unclusteredPoints',
                            type: 'symbol',
                            source: 'randomPoints',
                            filter: ['!', ['has', 'point_count']],
                            paint: {
                                'icon-halo-color': 'rgba(255, 255, 255, 1)',
                                'icon-halo-width': 5,
                                'icon-translate': [0, -1],
                                'text-halo-color': 'rgba(255, 255, 255, 0.8)',
                                'text-halo-width': 1.5,
                                'text-color': 'rgba(5, 57, 150, 1)'
                            },
                            layout: {
                                'icon-image': [
                                    'match',
                                    ['get', 'category'],
                                    'Active Truck',
                                    'truck-outline-green',
                                    'Inactive Truck',
                                    'truck-outline-red',
                                    'circle-fill-black'
                                ],
                                'icon-size':  ['interpolate', ['linear'], ['zoom'],
                                    9, 0.7,
                                    12, 1
                                ],
                                'text-field': ['step', ['zoom'],
                                    '',
                                    12, ['get', 'name']
                                ],
                                'text-font': ['Roboto Regular'],
                                'text-anchor': 'top',
                                'text-size': 11,
                                'text-offset': [0, 1]
                            },
                            // minzoom: minzoom
                        });

                        setTimeout(function() {
                            if($('.filter-menu').hasClass('d-none')){
                                $('.filter-menu').removeClass('d-none');
                            }
                        }, 1500);

                        // draw gps track
                        var i                       = 0;
                        var timer                   = setInterval(async () => {
                            // Your code block to be executed every 5 minutes goes here
                            // console.log("This code runs every 5 minutes.");
                            try {
                                const response      = await new Promise((resolve, reject) => {
                                    $.ajax({
                                        url: '/company-admin/map/cluster',
                                        type: 'POST',
                                        data: {},
                                        dataType: 'json',
                                        headers: {
                                            'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                                        },
                                        beforeSend: function () {
                                            // console.log("Fetching coordinates...");
                                        },
                                        success: function (data) {
                                            resolve(data);
                                        },
                                        error: function (jqXHR, status, exception) {
                                            reject({ jqXHR, status, exception });
                                        }
                                    });
                                });
                                if(response.success){
                                    // First AJAX request was successful
                                    var data                    = response.data;

                                    var filteredArray           = [];
                                    // Check if data is defined, not null, an array, and has items
                                    if (typeof data !== 'undefined' && data !== null && Array.isArray(data) && data.length > 0) {
                                        // Filter out items with valid lat/lng
                                        filteredArray           = data.filter(item => {
                                            return item.lat !== undefined && item.lat.trim() !== "" && item.lng !== undefined && item.lng.trim() !== "";
                                        });
                                        // Log the initial filtered array for lat/lng validity
                                        // console.log("Filtered Array (Lat/Lng valid):", filteredArray);

                                        // Update toggleFilteredArray with filtered data
                                        toggleFilteredArray      = filteredArray;
                                        // Log the updated toggleFilteredArray
                                        // console.log("toggleFilteredArray updated:", toggleFilteredArray);
                                    } else {
                                        // console.log("No valid data to process.");
                                    }

                                    // Initialize an empty array for features
                                    var features                = [];

                                    // Check if there are items in filteredArray
                                    if (filteredArray.length > 0) {
                                        // Perform the action here if filteredArray is not empty
                                        // console.log("filteredArray is not empty:", filteredArray);

                                        // Check if any radio button with name "rtoggle" is checked
                                        if ($('.filter-menu input[name="rtoggle"]:checked').length > 0) {
                                            // Get the value of the checked radio button
                                            var rtoggle         = $('.filter-menu input[name="rtoggle"]:checked').val();

                                            // Log the state of filteredArray before further filtering
                                            // console.log("filteredArray before 'rtoggle' filtering:", filteredArray);

                                            // Depending on the rtoggle value, filter the data array
                                            if (rtoggle === "Inactive Truck") {
                                                // Filter for "Inactive Truck"
                                                filteredArray   = filteredArray.filter(item => item.category === "Inactive Truck");
                                                // console.log("Filtered Inactive Trucks:", filteredArray);
                                            } else if (rtoggle === "Active Truck") {
                                                // Filter for "Active Truck"
                                                filteredArray   = filteredArray.filter(item => item.category === "Active Truck");
                                                // console.log("Filtered Active Trucks:", filteredArray);
                                            } else {
                                                // Handle other cases or do nothing
                                            }

                                            // Log the final filtered array after 'rtoggle' filtering
                                            // console.log("Final Filtered Array:", filteredArray);
                                        }

                                        // Convert filteredArray items to GeoJSON features
                                        $.each(filteredArray, function(index, obj) {
                                            features.push({
                                                type: 'Feature',
                                                properties: {
                                                    name: obj.text,
                                                    category: obj.category,
                                                    load_number: (obj.load_number !== undefined && obj.load_number !== null && $.trim(obj.load_number) !== '') ? obj.load_number : '',
                                                    url: (obj.url !== undefined && obj.url !== null && $.trim(obj.url) !== '') ? obj.url : ''
                                                },
                                                geometry: {
                                                    type: 'Point',
                                                    coordinates: [obj.lng, obj.lat]
                                                }
                                            });
                                        });
                                    } else {
                                        // Log a message indicating that filteredArray is empty
                                        // console.log("filteredArray is empty. No items to process.");
                                    }

                                    // Log features array after populating it
                                    // console.log("Features:", features);

                                    map.getSource('randomPoints').setData({
                                        type: "FeatureCollection",
                                        features: features
                                    });

                                    i++;
                                }else{
                                    // console.log('AJAX request failed: ' + response.message);
                                }
                            } catch (error) {
                                let errorMsg;
                                if (error.jqXHR.status === 0) {
                                    errorMsg = 'Not connected. Please verify your network connection.';
                                } else if (error.jqXHR.status === 404) {
                                    errorMsg = 'The requested page not found. [404]';
                                } else if (error.jqXHR.status === 500) {
                                    errorMsg = 'Internal Server Error [500].';
                                } else if (error.exception === 'parsererror') {
                                    errorMsg = 'Requested JSON parse failed.';
                                } else if (error.exception === 'timeout') {
                                    errorMsg = 'Time out error.';
                                } else if (error.exception === 'abort') {
                                    errorMsg = 'Ajax request aborted.';
                                } else {
                                    errorMsg = 'Uncaught Error. ' + error.jqXHR.responseText;
                                }
                                // console.log('AJAX request failed: ' + errorMsg);
                            }
                        }, 300000); // 300,000 milliseconds = 5 minutes
                    }else{
                        // console.log('AJAX request failed: ' + response.message);
                    }
                } catch (error) {
                    let errorMsg;
                    if (error.jqXHR.status === 0) {
                        errorMsg = 'Not connected. Please verify your network connection.';
                    } else if (error.jqXHR.status === 404) {
                        errorMsg = 'The requested page not found. [404]';
                    } else if (error.jqXHR.status === 500) {
                        errorMsg = 'Internal Server Error [500].';
                    } else if (error.exception === 'parsererror') {
                        errorMsg = 'Requested JSON parse failed.';
                    } else if (error.exception === 'timeout') {
                        errorMsg = 'Time out error.';
                    } else if (error.exception === 'abort') {
                        errorMsg = 'Ajax request aborted.';
                    } else {
                        errorMsg = 'Uncaught Error. ' + error.jqXHR.responseText;
                    }
                    // console.log('AJAX request failed: ' + errorMsg);
                }
            });

            let popup;
            // Listen for clicks on the truck stops layer
            map.on('click', 'unclusteredPoints', function (evt) {
                // Access properties using jQuery
                let properties      = evt.features[0].properties;

                // Initialize popup content
                let popupContent    = `Name: ${properties.name}<br />`;
                // Check if a certain property exists, and if so, add it to the popup content
                if (properties.load_number) {
                    popupContent    += `<a href="${properties.url}" target="_blank">${properties.load_number}</a>`;
                }

                var popupLocation   = evt.features[0].geometry.coordinates.slice();

                popup               = new TrimbleMaps.Popup()
                    .setLngLat(popupLocation)
                    .setHTML(popupContent)
                    .addTo(map);
            });

            map.on('zoomend', (e) => {
                let zoom            = map.getZoom();
                if(zoom <= minzoom && popup && popup.isOpen()) {
                    popup.remove(); // Remove popup from the map.
                }
            });

            $(document).on('click', '.filter-menu input[name="rtoggle"]', function(event) {
                var that    = $(this);

                // Get the value of the checked radio button
                var rtoggle = that.val();

                // Reset the select dropdown if it's a select2 element
                if ($('select[name="filter"]').hasClass('select2-hidden-accessible')) {
                    $('select[name="filter"]').val(null).trigger('change').trigger('select2:select');
                } else {
                    // Reset to the first option if not select2
                    $('select[name="filter"]').find('option:first').prop('selected', true);
                }

                // Get the 'randomPoints' source from the map
                var source  = map.getSource('randomPoints');
                if (source) {
                    // Source exists, proceed with updating it
                    // console.log('Source "randomPoints" exists. Updating data...');

                    // Initialize an empty array for features
                    var features                = [];

                    // Check if there are items in toggleFilteredArray
                    if (toggleFilteredArray.length > 0) {
                        // Perform the action here if toggleFilteredArray is not empty
                        // console.log("toggleFilteredArray is not empty:", toggleFilteredArray);

                        // Get the initial filtered array (assuming toggleFilteredArray is already defined)
                        var filteredArray       = toggleFilteredArray;
                        // Log the initial filtered array for lat/lng validity
                        // console.log("Filtered Array (Lat/Lng valid):", filteredArray);

                        // Check if there are items in filteredArray
                        if (filteredArray.length > 0) {
                            // Perform the action here if filteredArray is not empty
                            // console.log("filteredArray is not empty:", filteredArray);

                            // Log the state of filteredArray before further filtering
                            // console.log("filteredArray before 'rtoggle' filtering:", filteredArray);
                            
                            // Depending on the rtoggle value, filter the data array
                            if(rtoggle == 'Inactive Truck') {
                                // Filter for "Inactive Truck"
                                filteredArray   = filteredArray.filter(item => item.category === "Inactive Truck");
                                // console.log("Filtered Inactive Trucks:", filteredArray);
                            } else if (rtoggle == 'Active Truck') {
                                // Filter for "Active Truck"
                                filteredArray   = filteredArray.filter(item => item.category === "Active Truck");
                                // console.log("Filtered Active Trucks:", filteredArray);
                            } else {
                                // Handle other cases or do nothing
                            }

                            // Log the final filtered array after 'rtoggle' filtering
                            // console.log("Final Filtered Array:", filteredArray);

                            // Convert filteredArray items to GeoJSON features
                            $.each(filteredArray, function(index, obj) {
                                features.push({
                                    type: 'Feature',
                                    properties: {
                                        name: obj.text,
                                        category: obj.category,
                                        load_number: (obj.load_number !== undefined && obj.load_number !== null && $.trim(obj.load_number) !== '') ? obj.load_number : '',
                                        url: (obj.url !== undefined && obj.url !== null && $.trim(obj.url) !== '') ? obj.url : ''
                                    },
                                    geometry: {
                                        type: 'Point',
                                        coordinates: [obj.lng, obj.lat]
                                    }
                                });
                            });
                        } else {
                            // Log a message indicating that filteredArray is empty
                            // console.log("filteredArray is empty. No items to process.");
                        }
                    } else {
                        // Log a message indicating that toggleFilteredArray is empty
                        // console.log("toggleFilteredArray is empty. No items to process.");
                    }

                    // Update the data in the 'randomPoints' source
                    source.setData({
                        type: "FeatureCollection",
                        features: features
                    });

                    // Optionally, adjust map view to the updated data
                    map.jumpTo({ 'center': center, 'zoom': zoomLevel });
                } else {
                    // Source does not exist
                    // console.log('Source "randomPoints" does not exist. Cannot update data.');
                }
            });
        } else {
            toastr.error('The Trimble MAPS API key does not exist.', 'Error Alert', { 
                timeOut: 5000 
            });
        }
    </script>
    <script type="text/javascript">
        // Encapsulate the event handler within a closure
        (function() {
            var scrollTimeout; // Initialize a variable to hold the timeout ID

            $("#map").on('wheel', function (e) {
                // Prevent the event from bubbling up the DOM tree
                e.stopPropagation();
                
                // Log that the scroll event is detected in #map
                // console.log("Scroll event detected in #map");

                // Check if the Ctrl key is pressed
                if (e.originalEvent.ctrlKey) {
                    // Prevent the default scrolling behavior
                    e.preventDefault();
                    
                    // Log that the Ctrl key is pressed
                    // console.log("Ctrl key is pressed.");
                    
                    // Check if the map is drawn and ready for interaction
                    if (isMapDrawn('#map')) {
                        // Retrieve map instance from data attribute
                        var map             = $(this).data('map-instance');
                        // console.log('map:', map);
                        // console.log('map instanceof TrimbleMaps.Map:', map instanceof TrimbleMaps.Map);
                        // Check if map is already initialized and is an instance of TrimbleMaps.Map
                        if (map && (map instanceof TrimbleMaps.Map)) {
                            // Remove the 'map-scroll' class from #map
                            $("#map").removeClass('map-scroll');

                            // Adjust zoom level based on scroll direction
                            var direction   = e.originalEvent.deltaY > 0 ? 'out' : 'in';
                            if (direction === 'in') {
                                map.zoomIn();
                            } else {
                                map.zoomOut();
                            }
                        }
                    }
                } else {
                    // Log that the Ctrl key is not pressed
                    // console.log("Ctrl key is not pressed.");
                    
                    // Check if the map is drawn and ready for interaction
                    if (isMapDrawn('#map')) {
                        // Add the 'map-scroll' class to #map
                        $("#map").addClass('map-scroll');

                        // Clear previous timeout if exists
                        clearTimeout(scrollTimeout);
                        
                        // Set timeout to remove class after 5000 milliseconds
                        scrollTimeout       = setTimeout(function() {
                            $("#map").removeClass('map-scroll');
                        }, 5000);
                    }
                }
            });
        })();
        // Remove the 'map-scroll' class from #map after 1500 milliseconds
        $("#map").on('mouseleave', function () {
            setTimeout(function(){
                $("#map").removeClass('map-scroll');
            }, 1500);
        });
    </script>
    <script type="text/javascript">
        $(document).ready(function() {
            var filterSelect        = $('select[name="filter"]');
            filterSelect.find('option:first').prop('selected', true);
            filterSelect.select2({
                allowClear: true,
                placeholder: 'Select',
                width:'100%',
                ajax:{
                    type: 'POST',
                    url: '{{ route("company.admin.map.cluster.filter.list") }}',
                    dataType: 'json',
                    headers: {
                        'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                    },
                    delay: 250,
                    data: function(params) {
                        return {
                            rtoggle:$('.filter-menu input[name="rtoggle"]:checked').val(),
                            search:params.term,
                            page: params.page || 1, // Add pagination parameters
                            perPage: 10 // Set your preferred number of items per page
                        };
                    },
                    processResults: function(data, params) {
                        params.page = params.page || 1;
                        var results = $.map(data.items, function(item) {
                            return {
                                text: item.text,
                                id: item.id,
                                lat: item.lat,
                                lng: item.lng,
                                category: item.category,
                                disabled: !item.lat || !item.lng
                            };
                        });
                        return {
                            results: results,
                            pagination: {
                                more: (params.page * 10) < data.total_count // Adjust according to your data structure
                            }
                        };
                    }
                },
                templateResult: function(option) {
                    if (!option.id) {
                        return option.text;
                    }
                    var $option = $('<span></span>').text(option.text);

                    // Apply color only to specific categories and if not disabled
                    if (!option.disabled) {
                        if (option.category === 'Inactive Truck') {
                            $option.css('color', '#FF0000'); // Red
                        } else if (option.category === 'Active Truck') {
                            $option.css('color', '#459d6f'); // Green
                        }
                    }
                    return $option;
                },
                templateSelection: function(option) {
                    if (!option.id) {
                        return option.text;
                    }
                    var $option = $('<span></span>').text(option.text);

                    // Apply color only to specific categories and if not disabled
                    if (!option.disabled) {
                        if (option.category === 'Inactive Truck') {
                            $option.css('color', '#FF0000'); // Red
                        } else if (option.category === 'Active Truck') {
                            $option.css('color', '#459d6f'); // Green
                        }
                    }

                    // Check if lat and lng exist
                    if (option.lat !== undefined && option.lng !== undefined) {
                        // Set data attributes on the selected option
                        $(option.element).attr('data-lat', option.lat);
                        $(option.element).attr('data-lng', option.lng);
                    }
                    return $option;
                }
            }).on('select2:select', function (e) {
                // Get the selected option
                var selectedOption  = $(this).find('option:selected');
                // Get the value of the selected option
                var val             = selectedOption.val();

                // Check if data attributes exist using .attr()
                var lat             = selectedOption.attr('data-lat');
                var lng             = selectedOption.attr('data-lng');
                if (lat !== undefined && lng !== undefined) {
                    // Output the values (for demonstration)
                    // console.log("Latitude: " + lat + ", Longitude: " + lng);

                    // Check if the map is drawn and ready for interaction
                    if (isMapDrawn('#map')) {
                        // Retrieve map instance from data attribute
                        var map     = $('#map').data('map-instance');
                        // console.log('map:', map);
                        // console.log('map instanceof TrimbleMaps.Map:', map instanceof TrimbleMaps.Map);
                        // Check if map is already initialized and is an instance of TrimbleMaps.Map
                        if (map && (map instanceof TrimbleMaps.Map)) {
                            map.flyTo({
                                center: [lng, lat],
                                zoom: 18,
                                speed: 1,
                                curve: 1,
                                easing(t) {
                                    return t;
                                }
                            });
                        }
                    }
                } else {
                    // No latitude and longitude values exist
                    // console.log("No latitude and longitude values exist for the selected option.");
                }
            });
        });
    </script>
@endsection