@extends($companyAdminTheme)

@section('title')
    Pdf Template
@endsection

@section('style')
    <style type="text/css">
        .ava-template-heading {
          color: #fff;
          background-color: #2f70bb;
          border-color: #2f70bb;
          border-top-left-radius: .25rem;
          border-top-right-radius: .25rem;
          font-size: 14px;
          padding: 10px 12px;
          font-weight: bold;
        }
        .list-group-item {
            font-size: 14px;
            word-break: break-all;
            padding: 10px 34px 10px 12px;
            position: relative;
        }
        .list-group-item i {
            position: absolute;
            top: 12px;
            right: 12px;
        }
        .list-group-item.active{
            color: #fff;
            background-color: #2f70bb;
            border-color: #2f70bb;
        }
        .clipboard a > i.fa-check {
            color: #2f70bb;
        }
    </style>
@endsection

@section('content')
    {!! Form::model($pdf_template, array(
            'route'     => ['company.admin.pdf-templates.update',$pdf_template->id],
            'method'    => 'PATCH',
            'files'     => 'true',
            'class'     => 'store-form'
        )
    ) !!}
        <div class="row">
            <div class="col-lg-12">
                <div class="white-box mb-4 generalAccount">
                    <h4 class="box-title">Pdf Template Information</h4>
                    <div class="box-padding">
                        <div class="row">
                            <div class="col-md-9 pl-0">
                                <div class="row">
                                    <div class="col-md-12">
                                        <div class="form-group">
                                            <label>Name <span>*</span></label>
                                            {!! Form::text('name', old('name'), array(
                                                    'class'         => 'form-control',
                                                    'id'            => 'name',
                                                    'placeholder'   => 'Enter Name',
                                                    'readonly'      => 'readonly'
                                                )
                                            ) !!}
                                            @error('name')
                                                <span class="text-danger" role="alert">
                                                    <strong>{{ $message }}</strong>
                                                </span>
                                            @enderror
                                        </div>
                                    </div>
                                    <div class="col-md-12">
                                        <div class="form-group">
                                            <label>Content <span>*</span></label>
                                            {!! Form::textarea('content', old('content'), array(
                                                    'class'                 => 'form-control ck-editor',
                                                    'id'                    => 'content',
                                                    'required'              => 'required',
                                                    'data-parsley-trigger'  => 'change'
                                                )
                                            ) !!}
                                            @error('content')
                                                <span class="text-danger" role="alert">
                                                    <strong>{{ $message }}</strong>
                                                </span>
                                            @enderror
                                        </div>
                                    </div>
                                </div>
                            </div>
                            <div class="col-md-3 pl-0 pr-0">
                                <div class="list-group clipboard mt-3 pt-1">
                                    <h2 class="ava-template-heading">Available Template Tags</h2>
                                    @php
                                        $tagsArray = array_unique(
                                            array_filter(
                                                explode(',', $pdf_template->tags),
                                                function($value) {
                                                    // Remove null and blank values
                                                    return $value !== '' && !is_null($value);
                                                }
                                            )
                                        );
                                    @endphp
                                    @foreach($tagsArray as $tag)
                                        <a href="javascript:void(0);" class="list-group-item list-group-item-action font-weight-bold" title="Click to Copy">{{ $tag }}<i class="fa fa-copy font-16 float-right" aria-hidden="true"></i></a>
                                    @endforeach
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
        <div class="button-bottom">
            <a href="{{ route('company.admin.pdf-templates.index') }}" class="cancel-btn">Cancel</a>
            <button class="submit-btn">Submit</button>
        </div>
    {!! Form::close() !!}
@endsection

@section('script')
    <script src="{{ asset('theme/js/ckeditor/full/ckeditor.js') }}"></script>
    <script type="text/javascript">
        $(document).ready(function() {
            // Initialize Parsley on the form
            $('.store-form').parsley({
                errorsContainer: function(parsleyField) {
                    return parsleyField.$element.hasClass('ck-editor') ? parsleyField.$element.parent("div") : parsleyField;
                }
            });

            // Initialize CKEditor 
            var editor          = CKEDITOR.replace('content', {
                height: '400px' // Set the height here
            });

            // Listen for CKEditor instance ready event
            editor.on('instanceReady', function() {
                // Make CKEditor content field required
                $('.store-form #content').attr('required', '');

                // Listen for change event on CKEditor instances
                editor.on("change", function() {
                    // Update textarea value and trigger Parsley validation
                    editor.updateElement();
                    $('.store-form').parsley().validate();
                });
            });
        });
        $(document).on('click', '.clipboard .list-group-item', function (){
            let element     = $(this);
            let copyText    = element.text();
            navigator.clipboard.writeText(copyText);
            element.find('i').removeClass('fa-copy').addClass('fa-check');
            toastr.success('Copied Successfully.', 'Success Alert', {timeOut: 2000});
            setTimeout(function(){
                element.find('i').removeClass('fa-check').addClass('fa-copy');
            }, 2000);
        });
    </script>
@endsection

