@extends($companyAdminTheme)

@section('title')
    2 Factor Settings
@endsection

@section('style')
    <link rel="stylesheet" href="{{ asset('theme/css/select2.min.css') }}">
    <style type="text/css">
        label {
            font-size: 14px;
        }
        .disabled {
            color: #999;
            cursor: not-allowed;
            text-decoration: line-through;
            opacity: 0.5 !important;
            pointer-events: none !important;
        }
    </style>
@endsection

@section('content')
    <div class="white-box table-view mt-4">
        <div class="row">
            <div class="col-sm-12 col-md-12">
                <h3 class="table-title mb-0">2 Factor Settings</h3>
            </div>
            <div class="col-sm-12 col-md-12">
                <div class="d-flex align-items-center mt-1">
                    <div class="p-2">
                        <div class="d-inline-block">
                            <div class="custom-control custom-switch-blue">
                                <input type="checkbox" class="custom-control-input" name="enable_two_factor" id="enable-two-factor" value="1" {{ (isset($twoFactorSettings) && $twoFactorSettings->enable_two_factor == '1') ? 'checked' : '' }}>
                                <label class="custom-control-label" for="enable-two-factor"></label>
                            </div>
                        </div>
                    </div>
                    <div class="p-2">
                        <label class="mt-2 font-weight-bold">Enable 2-Factor Authentication</label>
                    </div>
                </div>
            </div>
            <div class="col-sm-12 col-md-12 mt-4">
                <h3 class="table-title mb-0">Authentication Settings</h3>
            </div>
            <div class="col-sm-12 col-md-12">
                <div class="d-flex align-items-center mt-1">
                    <div class="p-2">
                        <div class="d-inline-block">
                            <div class="custom-control custom-switch-blue">
                                <input type="checkbox" class="custom-control-input" name="enable_email_verify" id="enable-email-verify" value="1" {{ (isset($twoFactorSettings) && $twoFactorSettings->enable_email_verify == '1') ? 'checked' : '' }}>
                                <label class="custom-control-label enable-email-verify disabled" for="enable-email-verify"></label>
                            </div>
                        </div>
                    </div>
                    <div class="p-2">
                        <label class="mt-2 font-weight-bold">OTP Over Email {!! '['.((!is_null(auth()->guard('company-admin')->user()->email)) ? auth()->guard('company-admin')->user()->email : 'No Added').']' ?? '' !!}</label>
                    </div>
                </div>
            </div>
            <div class="col-sm-12 col-md-12">
                <div class="d-flex align-items-center">
                    <div class="p-2">
                        <div class="d-inline-block">
                            <div class="custom-control custom-switch-blue">
                                <input type="checkbox" class="custom-control-input" name="enable_sms_verify" id="enable-sms-verify" value="1" {{ (isset($twoFactorSettings) && $twoFactorSettings->enable_sms_verify == '1') ? 'checked' : '' }}>
                                <label class="custom-control-label enable-sms-verify disabled" for="enable-sms-verify"></label>
                            </div>
                        </div>
                    </div>
                    <div class="p-2">
                        <label class="mt-2 font-weight-bold">OTP Over SMS {!! '['.((!is_null(auth()->guard('company-admin')->user()->cell_phone)) ? auth()->guard('company-admin')->user()->cell_phone : 'No Added').']' ?? '' !!}</label>
                    </div>
                </div>
            </div>
            <div class="col-sm-12 col-md-12">
                <div class="button-bottom">
                    <a href="{{ route('company.admin.home') }}" class="cancel-btn">Cancel</a>
                    <button class="submit-btn">Save</button>
                </div>
            </div>
        </div>
    </div>
@endsection

@section('script')
    <script src="{{ asset('theme/js/select2.full.min.js') }}"></script>
    <script type="text/javascript">
        function loadPage(){
            if($('#enable-two-factor').is(':checked')){
                $('.enable-email-verify').removeClass('disabled');
                $('.enable-sms-verify').removeClass('disabled');
            }else{
                $('#enable-email-verify').prop('checked', false);
                $('#enable-sms-verify').prop('checked', false);
                $('.enable-email-verify').addClass('disabled');
                $('.enable-sms-verify').addClass('disabled');
            }
        }
        $(function (){
            loadPage();

           $(document).on('change', '#enable-two-factor', function (){
               loadPage();
           });

           $(document).on('click', '.submit-btn', function (){
               let enableTwoFactor = ($('#enable-two-factor').is(':checked')) ? '1' : '0';
               let enableEmailVerify = ($('#enable-email-verify').is(':checked')) ? '1' : '0';
               let enableSmsVerify = ($('#enable-sms-verify').is(':checked')) ? '1' : '0';

               if(enableTwoFactor == '1' && enableEmailVerify == '0' && enableSmsVerify == '0'){
                   toastr.error('Please enable OTP Over Email/SMS', 'Error Alert', { timeOut: 5000 });
               }else{
                   $.ajax({
                       url: '{!! route('company.admin.two-factor.update.setting') !!}',
                       method: "POST",
                       headers: { 'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content') },
                       data: {
                           enableTwoFactor: enableTwoFactor,
                           enableEmailVerify: enableEmailVerify,
                           enableSmsVerify: enableSmsVerify,
                       },
                       dataType: 'json',
                       success: function(response){
                           if(response.success == true){
                               toastr.success(response.msg, 'Success Alert', { timeOut: 5000 });
                           }else{
                               toastr.error(response.msg, 'Error Alert', { timeOut: 5000 });
                           }
                       },
                       error: function(response){
                           toastr.error('Something went wrong!', 'Error Alert', { timeOut: 5000 });
                       },
                   });
               }
           });
        });
    </script>
@endsection
