<!DOCTYPE html>
<html>
    <head>
        <meta charset="utf-8" />
        <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.6.3/jquery.min.js"></script>
        <link rel="stylesheet" href="https://maps-sdk.trimblemaps.com/v3/trimblemaps-3.8.0.css" />
        <script src="https://maps-sdk.trimblemaps.com/v3/trimblemaps-3.8.0.js"></script>
        <style>
            body { margin: 0; padding: 0; }

            #map {
                position: absolute;
                top: 0;
                bottom: 0;
                width: 100%;
            }

            .filter-ctrl {
                position: absolute;
                top: 10px;
                right: 10px;
                z-index: 1;
            }

            .filter-ctrl input[type='text'] {
                font: 12px/20px 'Helvetica Neue', Arial, Helvetica, sans-serif;
                width: 100%;
                border: 0;
                background-color: #fff;
                margin: 0;
                color: rgba(0, 0, 0, 0.5);
                padding: 10px;
                box-shadow: 0 0 0 2px rgba(0, 0, 0, 0.1);
                border-radius: 3px;
                width: 180px;
            }
        </style>
    </head>
    <body>
        <div id="map"></div>
        <div class="filter-ctrl">
          <input
          id="filter-input"
          type="text"
          name="filter"
          placeholder="Filter by name"
          autocomplete="off"
          />
        </div>

        <script>
            TrimbleMaps.APIKey          = '{!! getTrimbleApiKey() !!}'; //'83FF948F08EB3C4BB97E3AED42BEF772';
            const map                   = new TrimbleMaps.Map({
                container: 'map',
                style: TrimbleMaps.Common.Style.TRANSPORTATION, //hosted style id
                center: new TrimbleMaps.LngLat(-78.566234, 40.499440),
                zoom: 8
            });

            const bounds                = map.getBounds();
            const north                 = bounds.getNorth();
            const south                 = bounds.getSouth();
            const east                  = bounds.getEast();
            const west                  = bounds.getWest();

            // Functions for generating random points and features
            function getRandomPoint() {
                const lng               = west + ((east - west) * Math.random());
                const lat               = south + ((north - south) * Math.random());
                return [lng, lat];
            }

            const categories            = ['Active Truck', 'Inactive Truck', 'Active Trailer', 'Inactive Trailer'];

            function getRandomCategory() {
                var name                = categories[Math.floor(Math.random() * categories.length)];
                return name;
            }

            function getGeoJsonFeatures() {
                const features          = [];
                for (var i = 0; i < 50; i++) {
                    var category        = getRandomCategory();
                    if($.inArray(category, ['Active Truck', 'Inactive Truck']) !== -1){
                        var name        = 'Truck';
                        if(category==='Active Truck'){
                            var icon    = 'truck-fill-green';
                        }else{
                            var icon    = 'truck-fill-red';
                        }
                    }else{
                        var name        = 'Trailer';
                        if(category==='Active Trailer'){
                            var icon    = 'truck-outline-orange';
                        }else{
                            var icon    = 'truck-outline-purple';
                        }
                    }
                    features.push({
                        type: 'Feature',
                        properties: {
                            category: category,
                            name: name,
                            icon: icon
                        },
                        geometry: {
                            type: 'Point',
                            coordinates: getRandomPoint()
                        }
                    });
                }
                return features;
            }

            //Create getGeoJsonFeatures
            var getGeoJsonFeatures      = getGeoJsonFeatures();

            var layerIDs                = [];
            // Looking at textbox and storing input into a variable.
            var filterInput             = document.getElementById('filter-input');

            map.on('load', function() {
                map.addSource('places', {
                    type: 'geojson',
                    data: {
                        type: 'FeatureCollection',
                        features: getGeoJsonFeatures
                    }
                });

                getGeoJsonFeatures.forEach(function(feature) {
                    var category        = feature.properties['category'];
                    var name            = feature.properties['name'];
                    var icon            = feature.properties['icon'];
                    var layerID         = name.toLowerCase().replace(/ /g,"_")+'Point';
                    if (!map.getLayer(layerID)) {
                        //Add layer to render icon
                        map.addLayer({
                            id: layerID,
                            type: 'symbol',
                            source: 'places',
                            layout:{
                                'icon-image': "{icon}",
                                "icon-size": 1,
                                "text-field": "{name}",
                                "icon-allow-overlap" : true,
                                "text-allow-overlap": true,
                                "text-font": ["Roboto Regular"],
                                "text-anchor": "top",
                                "text-offset": [0,1],
                                "text-padding": 20,
                                "text-size": 10
                            },
                            "filter": ["==", ["get", "name"], name ]
                        });

                        //Push the icon to an empty array.
                        layerIDs.push(layerID);
                    }

                    // Monitor the textbox. If the input value matches a layerID,
                    // the visibility will be set to 'visible.'
                    filterInput.addEventListener('keyup', function(e){
                        var value       = e.target.value.trim().toLowerCase();
                        layerIDs.forEach(function(layerID){
                            map.setLayoutProperty(
                                layerID,
                                'visibility',
                                layerID.indexOf(value) > -1 ? 'visible' : 'none'
                            );
                        });
                    });
                });
            });
        </script>
    </body>
</html>
