@extends($companyAdminTheme)

@section('title')
    Map
@endsection

@section('style')
    <link rel="stylesheet" href="{{ asset('theme/css/select2.min.css') }}">
    <style type="text/css">
        .map-box {
            height: calc(100vh - 164px);
            min-height: 470px;
            background: #dce4ee;
            border: 1px solid #d8d8dd;
            border-radius: 10px;
        }
        #map {
            position: absolute;
            top: 0;
            bottom: 0;
            width: 100%;
        }
        .map-panel {
            position: absolute;
            width: 225px;
            top: 10px;
            left: 10px;
            padding: 8px;
            background-color: #fff;
            box-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
            font-size: .85em;
        }
        .group {
            padding: 3px 0;
        }
        .group .label {
            display: inline-block;
            width: 65px;
            font-style: italic;
            color: #888;
        }
        .group .value {
            display: inline-block;
        }
        .icon-panel {
            position: absolute;
            width: 225px;
            top: 10px;
            left: 10px;
            padding: 8px;
            background-color: #fff;
            box-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
            font-size: .85em;
        }
        .icon-panel span {
            color: #576571;
            line-height: 32px;
            display: inline-block;
            vertical-align: bottom;
        }
        .icon-div .button {
            display: inline-block;
            height: 22px !important;
            width: 22px !important;
            border: 1px solid transparent;
            background-repeat: no-repeat;
            background-position: center;
        }
        .icon-div #activeTruckIcon {
            background-image: url('{{ asset("theme/images/triangle-fill-green.png") }}');
        }
        .icon-div #inactiveTruckIcon {
            background-image: url('{{ asset("theme/images/traingle-fill-red.png") }}');
        }
        .icon-div #activeTrailerIcon {
            background-image: url('{{ asset("theme/images/square-fill-green.png") }}');
        }
        .icon-div #inactiveTrailerIcon {
            background-image: url('{{ asset("theme/images/square-fill-red.png") }}');
        }
        .filter-ctrl {
            position: absolute;
            top: 10px;
            right: 10px;
            z-index: 1;
            width: 180px;
        }
    </style>
    <style type="text/css">
        .select2-container--default .select2-selection--single, .select2-container--default .select2-selection--multiple {
            border: 1px solid #D8D8DD !important;
            border-radius: 3px !important;
            color: #000000 !important;
            overflow: hidden !important;
            min-height: 40px;
            height: auto !important;
        }
        .select2-container--default .select2-selection--single .select2-selection__arrow {
            height: 40px !important;
            margin-right: 5px;
            background-image: url("{{ asset('theme/images/select-dropdown.svg') }}");
            background-size: auto;
            background-position: center right 5px;
            background-repeat: no-repeat;
        }
        .select2-container--default .select2-selection--single .select2-selection__arrow b {
            border-color: #000 transparent transparent transparent;
            display: none;
        }
    </style>
@endsection

@section('content')
    <div class="map-box">
        <div id="map" data-map-instance="null"></div>
        <div class="map-panel d-none">
            <div class="group">
                <span class="label">Longitude:</span>
                <span id="longitude" class="value"></span>
            </div>
            <div class="group">
                <span class="label">Latitude:</span>
                <span id="latitude" class="value"></span>
            </div>
            <div class="group">
                <span class="label">Zoom:</span>
                <span id="zoom" class="value"></span>
            </div>
        </div>
        <div class="icon-panel">
            <div class="icon-div">
                <div class="button" id="activeTruckIcon"></div>
                <span>Active Truck</span>
            </div>
            <div class="icon-div">
                <div class="button" id="inactiveTruckIcon"></div>
                <span>Inactive Truck</span>
            </div>
            <div class="icon-div">
                <div class="button" id="activeTrailerIcon"></div>
                <span>Active Trailer</span>
            </div>
            <div class="icon-div">
                <div class="button" id="inactiveTrailerIcon"></div>
                <span>Inactive Trailer</span>
            </div>
        </div>
        <div class="filter-ctrl">
            <select class="form-control" name="filter">
                <option value="">Select</option>
                <optgroup label="Truck">
                    <option value="2" data-lat="lat_value_1" data-lng="lng_value_1">Point #2</option>
                    <option value="3" data-lat="lat_value_1" data-lng="lng_value_1">Point #3</option>
                    <option value="4" data-lat="lat_value_1" data-lng="lng_value_1">Point #4</option>
                    <option value="5" data-lat="lat_value_1" data-lng="lng_value_1">Point #5</option>
                    <option value="6" data-lat="lat_value_1" data-lng="lng_value_1">Point #6</option>
                    <option value="7" data-lat="lat_value_1" data-lng="lng_value_1">Point #7</option>
                </optgroup>
                <optgroup label="Trailer">
                    <option value="1" data-lat="lat_value_1" data-lng="lng_value_1">Point #1</option>
                    <option value="8" data-lat="lat_value_1" data-lng="lng_value_1">Point #8</option>
                    <option value="9" data-lat="lat_value_1" data-lng="lng_value_1">Point #9</option>
                    <option value="10" data-lat="lat_value_1" data-lng="lng_value_1">Point #10</option>
                </optgroup>
            </select>
        </div>
    </div>
@endsection

@section('script')
    <script src="{{ asset('theme/js/select2.full.min.js') }}"></script>
    <script type="text/javascript">
        if (trimbleMapsAPIKey) {
            var center              = [-78.566234, 40.499440];
            var zoomLevel           = 10;
            var minzoom             = 9;

            var map                 = new TrimbleMaps.Map({
                container: 'map', // container id
                style: TrimbleMaps.Common.Style.TRANSPORTATION, //hosted style id
                center: center, // starting position
                zoom: zoomLevel, // starting zoom
                scrollZoom: false
            });

            // Store map instance in data attribute
            $('#map').data('map-instance', map);

            // Disable scroll zoom controls
            // map.scrollZoom.disable();

            // Save the panel elements
            var lngElem             = document.getElementById('longitude');
            var latElem             = document.getElementById('latitude');
            var zoomElem            = document.getElementById('zoom');
            lngElem.innerHTML       = center[0].toFixed(6);
            latElem.innerHTML       = center[1].toFixed(6);
            zoomElem.innerHTML      = zoomLevel.toFixed(6);

            var bounds              = map.getBounds();
            var north               = bounds.getNorth();
            var south               = bounds.getSouth();
            var east                = bounds.getEast();
            var west                = bounds.getWest();

            // Functions for generating random points and features
            function getRandomPoint() {
                var lng             = west + ((east - west) * Math.random());
                var lat             = south + ((north - south) * Math.random());
                return [lng, lat];
            }

            var categories          = ['Active Truck', 'Inactive Truck', 'Active Trailer', 'Inactive Trailer'];

            function getRandomCategory() {
                var name            = categories[Math.floor(Math.random() * categories.length)];
                return name;
            }

            function getGeoJsonFeatures() {
                var features        = [];

                for (var i = 1; i <= 250; i++) {
                    var category    = getRandomCategory();
                    features.push({
                        type: 'Feature',
                        properties: {
                            name: 'Point #' + i,
                            category: category
                        },
                        geometry: {
                            type: 'Point',
                            coordinates: getRandomPoint()
                        }
                    });
                }
                return features;
            }

            // Wait for the map to load before adding layers and data sources
            map.on('load', function() {
                // Use cluster properties on datasource to group data
                map.addSource('randomPoints', {
                    type: 'geojson',
                    cluster: true,
                    clusterRadius: 40,
                    clusterMaxZoom: 14,
                    data: {
                        type: 'FeatureCollection',
                        features: getStaticGeoJsonFeatures() // getGeoJsonFeatures()
                    }
                });

                // Use filter to shot clustered points
                map.addLayer({
                    id: 'clusteredPoints',
                    type: 'circle',
                    source: 'randomPoints',
                    filter: ['has', 'point_count'],
                    paint: {
                        'circle-radius': [
                            'step',
                            ['get', 'point_count'],
                            15, 10,
                            20, 20,
                            25
                        ],
                        'circle-color': '#000000',
                        'circle-stroke-color': '#FFF',
                        'circle-stroke-width': 3
                    }
                });

                // Show count for clustered points
                map.addLayer({
                    id: 'clusterCount',
                    type: 'symbol',
                    source: 'randomPoints',
                    filter: ['has', 'point_count'],
                    layout: {
                        'text-field': '{point_count}',
                        'text-font': ['Roboto Regular'],
                        'text-size': 12
                    },
                    paint: {
                        'text-color': '#FFF'
                    }
                });

                // Use filter to show unclustered points
                map.addLayer({
                    id: 'unclusteredPoints',
                    type: 'symbol',
                    source: 'randomPoints',
                    filter: ['!', ['has', 'point_count']],
                    paint: {
                        'icon-halo-color': 'rgba(255, 255, 255, 1)',
                        'icon-halo-width': 5,
                        'icon-translate': [0, -1],
                        'text-halo-color': 'rgba(255, 255, 255, 0.8)',
                        'text-halo-width': 1.5,
                        'text-color': 'rgba(5, 57, 150, 1)'
                    },
                    layout: {
                        'icon-image': [
                            'match',
                            ['get', 'category'],
                            'Active Truck',
                            'triangle-fill-green',
                            'Inactive Truck',
                            'traingle-fill-red',
                            'Active Trailer',
                            'square-fill-green',
                            'Inactive Trailer',
                            'square-fill-red',
                            'circle-fill-black',
                        ],
                        'icon-size':  ['interpolate', ['linear'], ['zoom'],
                            9, 0.7,
                            12, 1
                        ],
                        'text-field': ['step', ['zoom'],
                            '',
                            12, ['get', 'name']
                        ],
                        'text-font': ['Roboto Regular'],
                        'text-anchor': 'top',
                        'text-size': 11,
                        'text-offset': [0, 1]
                    },
                    minzoom: minzoom
                });

                // draw gps track
                var i               = 0;
                var timer           = setInterval(() => {
                    // Your code block to be executed every 5 minutes goes here
                    console.log("This code runs every 5 minutes.");
                    // https://developer.trimblemaps.com/maps-sdk/api/#geojsonsource
                    // map.getSource('randomPoints').setData({
                    //     type: "FeatureCollection",
                    //     features: getGeoJsonFeatures()
                    // });
                    i++;
                }, 300000); // 300,000 milliseconds = 5 minutes
            });

            let popup;
            // Listen for clicks on the truck stops layer
            map.on('click', 'unclusteredPoints', function (evt) {
                var popupLocation   = evt.features[0].geometry.coordinates.slice();
                var popupContent    = `Name: ${evt.features[0].properties.name}<br />
                    Category: ${evt.features[0].properties.category}<br />
                    <a href="/company-admin/loads/Q7tXNqjDznmwy8pAkgQSMQQ8YCK2XeDA" target="_blank">DTL-10001</a>`;

                popup               = new TrimbleMaps.Popup()
                    .setLngLat(popupLocation)
                    .setHTML(popupContent)
                    .addTo(map);
            });

            map.on('zoomend', (e) => {
                let zoom            = map.getZoom();
                zoomElem.innerHTML  = zoom.toFixed(6);
                if(zoom <= minzoom && popup && popup.isOpen()) {
                    popup.remove(); // Remove popup from the map.
                }
            });

            // Mouse move on the map, mouse position gets updated.
            map.on('mousemove', (e) => {
                // Update the values in the panel
                lngElem.innerHTML   = e.lngLat.lng.toFixed(6);
                latElem.innerHTML   = e.lngLat.lat.toFixed(6);
            });
        } else {
            toastr.error('The Trimble MAPS API key does not exist.', 'Error Alert', { 
                timeOut: 5000 
            });
        }
    </script>
    <script type="text/javascript">
        // Encapsulate the event handler within a closure
        (function() {
            var scrollTimeout; // Initialize a variable to hold the timeout ID

            $("#map").on('wheel', function (e) {
                // Prevent the event from bubbling up the DOM tree
                e.stopPropagation();
                
                // Log that the scroll event is detected in #map
                // console.log("Scroll event detected in #map");

                // Check if the Ctrl key is pressed
                if (e.originalEvent.ctrlKey) {
                    // Prevent the default scrolling behavior
                    e.preventDefault();
                    
                    // Log that the Ctrl key is pressed
                    // console.log("Ctrl key is pressed.");
                    
                    // Check if the map is drawn and ready for interaction
                    if (isMapDrawn('#map')) {
                        // Retrieve map instance from data attribute
                        var map             = $(this).data('map-instance');
                        // console.log('map:', map);
                        // console.log('map instanceof TrimbleMaps.Map:', map instanceof TrimbleMaps.Map);
                        // Check if map is already initialized and is an instance of TrimbleMaps.Map
                        if (map && (map instanceof TrimbleMaps.Map)) {
                            // Remove the 'map-scroll' class from #map
                            $("#map").removeClass('map-scroll');

                            // Adjust zoom level based on scroll direction
                            var direction   = e.originalEvent.deltaY > 0 ? 'out' : 'in';
                            if (direction === 'in') {
                                map.zoomIn();
                            } else {
                                map.zoomOut();
                            }
                        }
                    }
                } else {
                    // Log that the Ctrl key is not pressed
                    // console.log("Ctrl key is not pressed.");
                    
                    // Check if the map is drawn and ready for interaction
                    if (isMapDrawn('#map')) {
                        // Add the 'map-scroll' class to #map
                        $("#map").addClass('map-scroll');

                        // Clear previous timeout if exists
                        clearTimeout(scrollTimeout);
                        
                        // Set timeout to remove class after 5000 milliseconds
                        scrollTimeout = setTimeout(function() {
                            $("#map").removeClass('map-scroll');
                        }, 5000);
                    }
                }
            });
        })();
        // Remove the 'map-scroll' class from #map after 1500 milliseconds
        $("#map").on('mouseleave', function () {
            setTimeout(function(){
                $("#map").removeClass('map-scroll');
            }, 1500);
        });
    </script>
    <script type="text/javascript">
        $(document).ready(function() {
            $('select[name="filter"] option:first').prop('selected', true);
            $('select[name="filter"]').select2({
                width:'100%',
            }).on('select2:select', function (e) {
                // Get the selected option
                var selectedOption  = $(this).find('option:selected');
                // Get the value of the selected option
                var val             = selectedOption.val();
                            
                // Check if data attributes exist using .attr()
                var lat             = selectedOption.attr('data-lat');
                var lng             = selectedOption.attr('data-lng');
                // Check if both lat and lng are not undefined or null
                if (lat !== undefined && lng !== undefined) {
                    // Output the values (for demonstration)
                    // console.log("Latitude: " + lat + ", Longitude: " + lng);

                    // Check if the map is drawn and ready for interaction
                    if (isMapDrawn('#map')) {
                        // Retrieve map instance from data attribute
                        var map     = $('#map').data('map-instance');
                        // console.log('map:', map);
                        // console.log('map instanceof TrimbleMaps.Map:', map instanceof TrimbleMaps.Map);
                        // Check if map is already initialized and is an instance of TrimbleMaps.Map
                        if (map && (map instanceof TrimbleMaps.Map)) {
                            var sitesGeojson = getStaticGeoJsonFeatures();
                            map.flyTo({
                                center: sitesGeojson[val-1].geometry.coordinates,
                                zoom: 14,
                                speed: 0.2,
                                curve: 1,
                                easing(t) {
                                    return t;
                                }
                            });
                        }
                    }
                } else {
                    // No latitude and longitude values exist
                    // console.log("No latitude and longitude values exist for the selected option.");
                }
            });
        });
        function getStaticGeoJsonFeatures(){
            return [
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #1",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.92296120745772,
                    40.590247686415864
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #2",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.84492294725673,
                    40.49689682617418
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #3",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.80847213569209,
                    40.541907031229336
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #4",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.91678855231815,
                    40.431286893350475
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #5",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.50322075841336,
                    40.38239570033871
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #6",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.65714254875907,
                    40.505747583355166
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #7",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.8590320579479,
                    40.58674842131229
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #8",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.82910681803658,
                    40.588391781406244
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #9",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.35256200778221,
                    40.44864287708266
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #10",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.83738676349313,
                    40.5036475452724
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #11",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.33072550124528,
                    40.46168198408898
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #12",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.36364695967683,
                    40.58269482552355
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #13",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.77878098210789,
                    40.57911861637952
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #14",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.78372680686921,
                    40.59170745658908
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #15",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.87619170365974,
                    40.47418412353878
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #16",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.91177963369788,
                    40.478670091110565
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #17",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.85572476053986,
                    40.417058618687584
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #18",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.70112205925979,
                    40.429963907646666
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #19",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.79507552807384,
                    40.40633804403086
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #20",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.18596200744084,
                    40.378511067500554
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #21",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.53501126923379,
                    40.546015690611675
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #22",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.25114484970324,
                    40.47606434339047
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #23",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.87422770424045,
                    40.42382563185358
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #24",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.32459120158235,
                    40.59069393311882
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #25",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.58472688499775,
                    40.42187199943065
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #26",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.33184415570064,
                    40.57137722968339
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #27",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.77955161796287,
                    40.568256625548116
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #28",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.17764914547011,
                    40.40735900879107
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #29",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.4027859950654,
                    40.59868126448945
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #30",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.3109214170376,
                    40.53923346237702
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #31",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.73421186752216,
                    40.49025833465402
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #32",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.88406372994584,
                    40.44186339572701
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #33",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.86020683370148,
                    40.476631642903634
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #34",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.56251168584443,
                    40.48470378372311
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #35",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.90885952253423,
                    40.607132291961435
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #36",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.90889915001657,
                    40.48272955934483
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #37",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.18926703694518,
                    40.59442475305876
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #38",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.39519256704395,
                    40.437983251381475
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #39",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.7379644823888,
                    40.39841536196918
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #40",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.17711711856502,
                    40.61524459096132
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #41",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.45234065653325,
                    40.56193551766331
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #42",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.43866582288042,
                    40.45743027727082
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #43",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.81043850250929,
                    40.45721223081989
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #44",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.70019551515244,
                    40.39498347469266
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #45",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.78592959281158,
                    40.45780703057032
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #46",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.50738784254531,
                    40.42547594855709
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #47",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.36528365385945,
                    40.46168108629338
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #48",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.30793878333384,
                    40.391515219562585
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #49",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.78845531831548,
                    40.43149932051229
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #50",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.40958745253886,
                    40.46543927273621
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #51",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.28620220204324,
                    40.51089330415649
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #52",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.46581386764517,
                    40.50566789609016
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #53",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.25251241392868,
                    40.431606137996766
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #54",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.44875172405607,
                    40.38895808474053
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #55",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.79952850781677,
                    40.464193250774535
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #56",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.60856537233744,
                    40.5549088928619
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #57",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.85958642278432,
                    40.60911560705591
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #58",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.78374528605339,
                    40.421746820364206
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #59",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.41393305639923,
                    40.55065806948499
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #60",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.79366169868636,
                    40.620496193345744
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #61",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.22471882567278,
                    40.57410868386666
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #62",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.53878680256905,
                    40.56537129912253
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #63",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.899961023702,
                    40.527639623714855
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #64",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.42369028184645,
                    40.41218343942692
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #65",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.89931596804233,
                    40.43479811925183
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #66",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.32916427962078,
                    40.601375632724356
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #67",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.71432260235653,
                    40.59521775800447
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #68",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.92940637887243,
                    40.45772450975662
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #69",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.53434952610628,
                    40.54879039905206
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #70",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.56823191523853,
                    40.39776790264479
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #71",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.43594728899302,
                    40.492102382903404
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #72",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.84463641454175,
                    40.388604272809125
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #73",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.61985602018532,
                    40.501359741781485
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #74",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.88506014830104,
                    40.53544076410305
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #75",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.87963710019756,
                    40.437732064639384
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #76",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.36939622652139,
                    40.569431520433916
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #77",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.50569120495263,
                    40.56226007620132
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #78",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.35189686546418,
                    40.496721365941
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #79",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.42940796011463,
                    40.51290395365781
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #80",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.18140361295838,
                    40.51938512369646
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #81",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.50140982476377,
                    40.463510327540604
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #82",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.48360193884857,
                    40.566134480958446
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #83",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.38489159404266,
                    40.39497165169047
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #84",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.56341205989679,
                    40.54968326822444
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #85",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.5966798237637,
                    40.55603446683593
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #86",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.34821642871944,
                    40.38650969396193
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #87",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.6915842562662,
                    40.51194866778755
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #88",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.84240642975782,
                    40.605597388498495
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #89",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.94406488673837,
                    40.53778313631784
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #90",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.383932109515,
                    40.5791981963007
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #91",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.71326909556116,
                    40.61191150361126
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #92",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.87404654805094,
                    40.5340643265934
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #93",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.74554797072068,
                    40.44055839836742
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #94",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.20800344740177,
                    40.42808665196707
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #95",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.77830015421063,
                    40.59671451081717
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #96",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.74698359820756,
                    40.57894310362464
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #97",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.85374787957917,
                    40.48335221540689
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #98",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.8418477869345,
                    40.43868625566546
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #99",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.89566037818435,
                    40.55790247812283
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #100",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.77913259622724,
                    40.400030411266115
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #101",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.33420273670882,
                    40.53992573477901
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #102",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.9086363455139,
                    40.536326887282705
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #103",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.90452648089314,
                    40.591635468543295
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #104",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.9458203073424,
                    40.591396918167625
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #105",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.73657726503279,
                    40.589947640898195
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #106",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.54447314334459,
                    40.45969488329096
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #107",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.61758405835386,
                    40.45012521453329
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #108",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.51904344350442,
                    40.46831526507362
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #109",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.49700234844677,
                    40.559543336242626
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #110",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.40537901614002,
                    40.46728350982746
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #111",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.16017497834403,
                    40.41092177839454
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #112",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.7462680966252,
                    40.55683308295597
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #113",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.65698633323697,
                    40.578100038435956
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #114",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.94246600143794,
                    40.50586074592695
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #115",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.85314304251771,
                    40.50243817768779
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #116",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.86818398215925,
                    40.37982604079221
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #117",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.51881118769649,
                    40.471206843176255
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #118",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.82870342293855,
                    40.5238078309011
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #119",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.7103462810942,
                    40.60063055385034
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #120",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.25306808913939,
                    40.39165793452894
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #121",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.44742568164746,
                    40.48120215775535
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #122",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.3424512915863,
                    40.59867968625656
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #123",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.7994416993054,
                    40.54206736746534
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #124",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.92246842088473,
                    40.43238574126061
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #125",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.39519119055356,
                    40.458309437918786
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #126",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.37063588184958,
                    40.5421878669773
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #127",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.33692809225879,
                    40.42562020926843
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #128",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.92817131708279,
                    40.472225273900484
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #129",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.67828835862068,
                    40.39850069876688
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #130",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.22671061219721,
                    40.578261447251116
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #131",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.72036935872096,
                    40.51895268116096
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #132",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.78678797256367,
                    40.46982572310667
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #133",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.34890578343577,
                    40.43045561976556
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #134",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.33954130220873,
                    40.48845844648213
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #135",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.70718491375239,
                    40.52676473364257
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #136",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.56283359061865,
                    40.46262702108981
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #137",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.28841019222291,
                    40.56982146740333
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #138",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.83327531597023,
                    40.6005041428711
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #139",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.2677923161431,
                    40.44567720066682
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #140",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.7569710724416,
                    40.409637282095936
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #141",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.62770461441288,
                    40.51891754017766
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #142",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.62756248755672,
                    40.61292068878049
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #143",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.42963413961947,
                    40.51700400408203
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #144",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.25239695483815,
                    40.46832468397809
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #145",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.29087197053387,
                    40.59029685760174
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #146",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.52416309762927,
                    40.50350709308795
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #147",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.57084217188367,
                    40.521951339891885
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #148",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.75510387174413,
                    40.404191005579776
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #149",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.25545803021362,
                    40.515255529073194
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #150",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.36471838548987,
                    40.57935479805907
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #151",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.4333541044195,
                    40.55724545879263
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #152",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.92823702592271,
                    40.44754785293126
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #153",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.39798152836282,
                    40.43043450760289
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #154",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.36613254880474,
                    40.38993028717171
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #155",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.7216938560156,
                    40.39103942752438
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #156",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.79927956630266,
                    40.41364884328956
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #157",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.63925806726397,
                    40.45915506335791
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #158",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.88196369978003,
                    40.46309995518631
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #159",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.50252046032038,
                    40.5663848052295
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #160",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.54429791911733,
                    40.42093281170319
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #161",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.70099243559719,
                    40.44065171484007
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #162",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.94120598684826,
                    40.608871351502444
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #163",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.36146294883294,
                    40.525615762454585
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #164",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.97312534733196,
                    40.60229267987548
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #165",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.34097053718412,
                    40.533458066848425
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #166",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.18818701288686,
                    40.60138001988275
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #167",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.30366954745041,
                    40.56746258460793
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #168",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.77116786255014,
                    40.61691567158064
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #169",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.81264805980075,
                    40.53280140917657
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #170",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.54793230412686,
                    40.45196714541825
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #171",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.72461609087357,
                    40.46284498040733
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #172",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.51729653425515,
                    40.52002513839739
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #173",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.45224494752829,
                    40.40498600534224
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #174",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.33482499666403,
                    40.486795570148594
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #175",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.94699533800097,
                    40.58286514210216
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #176",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.72305848269166,
                    40.59941619766419
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #177",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.35915014428879,
                    40.57255216631334
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #178",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.34750848268997,
                    40.398963321598565
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #179",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.70081166869761,
                    40.61268176768773
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #180",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.76133552768042,
                    40.510290214793855
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #181",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.86442406211886,
                    40.519403040301704
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #182",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.32525152115605,
                    40.4252228640624
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #183",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.33684008951798,
                    40.381638225837335
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #184",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.24670977320356,
                    40.455057165444366
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #185",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.26135601239622,
                    40.54799382643357
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #186",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.5761203286438,
                    40.49589971844303
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #187",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.24922320671106,
                    40.48908123428816
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #188",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.51351903230183,
                    40.42295947536951
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #189",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.22662848200082,
                    40.567156676993186
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #190",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.67866060901709,
                    40.59267703509744
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #191",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.63525865673735,
                    40.618380757479144
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #192",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.92096643007328,
                    40.48147423893731
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #193",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.93629946936657,
                    40.4745878505902
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #194",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.293571872922,
                    40.51192189673138
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #195",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.43707228272564,
                    40.44102759540733
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #196",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.77206554803,
                    40.57216211952365
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #197",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.67077913876093,
                    40.431382824342634
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #198",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.76002902071215,
                    40.5698521832168
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #199",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.45503643626122,
                    40.515948218578224
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #200",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.40729568646756,
                    40.48681100733715
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #201",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.94300388119079,
                    40.50127134251462
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #202",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.4033292258922,
                    40.54700234998188
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #203",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.95653780096,
                    40.52872704852528
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #204",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.61095832988022,
                    40.40231855582908
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #205",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.22359632210417,
                    40.554539063973564
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #206",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.92187542064556,
                    40.54437187914519
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #207",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.94962826051616,
                    40.50973055297677
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #208",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.80298122982214,
                    40.431466161477005
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #209",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.52737200583316,
                    40.39014236274736
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #210",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.61861524737658,
                    40.51489575887777
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #211",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.75876203259853,
                    40.56158125999269
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #212",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.80518945771101,
                    40.60765056249887
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #213",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.49191526373976,
                    40.60990641714942
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #214",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.84166042489612,
                    40.38421467999125
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #215",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.90304015782469,
                    40.38206910459034
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #216",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.91860251899571,
                    40.44175370071388
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #217",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.791136238877,
                    40.491476588309546
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #218",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.24066465005997,
                    40.45487000165601
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #219",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.68643433401492,
                    40.588276819734915
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #220",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.42443467376482,
                    40.50535528533383
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #221",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.23374409641184,
                    40.54769853195756
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #222",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.92650489435829,
                    40.61085485474004
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #223",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.40325499444542,
                    40.39698741835283
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #224",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.95640813902335,
                    40.40315738455516
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #225",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.43859588220121,
                    40.52845741027609
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #226",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.9610666330192,
                    40.615198901059046
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #227",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.41075963776184,
                    40.419164241517386
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #228",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.23217993868458,
                    40.48569338119138
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #229",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.7138095552881,
                    40.39474350873974
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #230",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.40879918520172,
                    40.492850010273216
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #231",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.88042305121928,
                    40.616703843759964
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #232",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.96226567636633,
                    40.500700543911464
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #233",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.56197423606258,
                    40.48357708711247
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #234",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.79854729236946,
                    40.54693456412181
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #235",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.65632924563886,
                    40.5702578896958
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #236",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.58902392152389,
                    40.54789233535222
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #237",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.74718937080208,
                    40.59576948849427
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #238",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.86310488213097,
                    40.582022645606074
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #239",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.50961623885448,
                    40.4827248235153
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #240",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.87393609236594,
                    40.38386622917661
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #241",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.73487352579227,
                    40.514126997775854
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #242",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.31365354190126,
                    40.57369436119848
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #243",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.91560039248252,
                    40.3789608517304
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #244",
                  "category": "Active Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.34825942763706,
                    40.44489880276725
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #245",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.34323282562816,
                    40.394256519896835
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #246",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.31132275670235,
                    40.594042367582894
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #247",
                  "category": "Inactive Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.66711764970292,
                    40.49790629206409
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #248",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.75079819168752,
                    40.5467783829325
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #249",
                  "category": "Inactive Truck"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.3356677934845,
                    40.56268794425506
                  ]
                }
              },
              {
                "type": "Feature",
                "properties": {
                  "name": "Point #250",
                  "category": "Active Trailer"
                },
                "geometry": {
                  "type": "Point",
                  "coordinates": [
                    -78.6668256493604,
                    40.39162686617401
                  ]
                }
              }
            ];
        }
    </script>
@endsection