<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="utf-8" />
        <link rel="stylesheet" href="https://maps-sdk.trimblemaps.com/v3/trimblemaps-3.16.0.css" />
        <script src="https://maps-sdk.trimblemaps.com/v3/trimblemaps-3.16.0.js"></script>
        <style>
            body { margin: 0; padding: 0; }

            #map {
                position: absolute;
                top: 0;
                bottom: 0;
                width: 100%;
            }

            #icon-panel {
                position: absolute;
                top: 10px;
                left: 10px;
                padding: 8px;
                background-color: #fff;
                box-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
                font-family: 'Open Sans', sans-serif;
            }

            #icon-panel span {
                color: #576571;
                line-height: 32px;
                display: inline-block;
                vertical-align: bottom;
            }

            .icon-div .button {
                display: inline-block;
                height: 22px !important;
                width: 22px !important;
                border: 1px solid transparent;
                background-repeat: no-repeat;
                background-position: center;
            }

            .icon-div #activeTruckIcon {
                background-image: url('{{ asset("theme/images/triangle-fill-green.png") }}');
            }

            .icon-div #inactiveTruckIcon {
                background-image: url('{{ asset("theme/images/traingle-fill-red.png") }}');
            }

            .icon-div #activeTrailerIcon {
                background-image: url('{{ asset("theme/images/square-fill-green.png") }}');
            }

            .icon-div #inactiveTrailerIcon {
                background-image: url('{{ asset("theme/images/square-fill-red.png") }}');
            }
        </style>
    </head>
    <body>
        <div id="map"></div>
        <div id="icon-panel">
            <div class="icon-div">
                <div class="button" id="activeTruckIcon"></div>
                <span>Active Truck</span>
            </div>
            <div class="icon-div">
                <div class="button" id="inactiveTruckIcon"></div>
                <span>Inactive Truck</span>
            </div>
            <div class="icon-div">
                <div class="button" id="activeTrailerIcon"></div>
                <span>Active Trailer</span>
            </div>
            <div class="icon-div">
                <div class="button" id="inactiveTrailerIcon"></div>
                <span>Inactive Trailer</span>
            </div>
        </div>

        <script>
            TrimbleMaps.APIKey      = '83FF948F08EB3C4BB97E3AED42BEF772';

            const map               = new TrimbleMaps.Map({
                container: 'map', // container id
                style: TrimbleMaps.Common.Style.TRANSPORTATION, // hosted style id
                center: [-78.566234, 40.499440], // starting position
                zoom: 10 // starting zoom
            });

            const minzoom           = 9;

            const bounds            = map.getBounds();
            const north             = bounds.getNorth();
            const south             = bounds.getSouth();
            const east              = bounds.getEast();
            const west              = bounds.getWest();

            // Functions for generating random points and features
            function getRandomPoint() {
                const lng           = west + ((east - west) * Math.random());
                const lat           = south + ((north - south) * Math.random());
                return [lng, lat];
            }

            const categories        = ['Active Truck', 'Inactive Truck', 'Active Trailer', 'Inactive Trailer'];

            function getRandomCategory() {
                var name            = categories[Math.floor(Math.random() * categories.length)];
                return name;
            }

            function getGeoJsonFeatures() {
                const features      = [];

                for (var i = 0; i < 250; i++) {
                    var category    = getRandomCategory();
                    features.push({
                        type: 'Feature',
                        properties: {
                            name: 'Point #' + i,
                            category: category
                        },
                        geometry: {
                            type: 'Point',
                            coordinates: getRandomPoint()
                        }
                    });
                }
                return features;
            }

            map.on('load', function () {
                // Use cluster properties on datasource to group data
                map.addSource('randomPoints', {
                    type: 'geojson',
                    data: {
                        type: 'FeatureCollection',
                        features: getGeoJsonFeatures()
                    }
                });

                // Add a layer to draw circles for each point in the data source
                map.addLayer({
                    id: 'styledPoints',
                    type: 'symbol',
                    source: 'randomPoints',
                    paint: {
                        'icon-halo-color': 'rgba(255, 255, 255, 1)',
                        'icon-halo-width': 5,
                        'icon-translate': [0, -1],
                        'text-halo-color': 'rgba(255, 255, 255, 0.8)',
                        'text-halo-width': 1.5,
                        'text-color': 'rgba(5, 57, 150, 1)'
                    },
                    layout: {
                        'icon-image': [
                            'match',
                            ['get', 'category'],
                            'Active Truck',
                            'triangle-fill-green',
                            'Inactive Truck',
                            'traingle-fill-red',
                            'Active Trailer',
                            'square-fill-green',
                            'Inactive Trailer',
                            'square-fill-red',
                            'circle-fill-black',
                        ],
                        'icon-size':  ['interpolate', ['linear'], ['zoom'],
                            9, 0.7,
                            12, 1
                        ],
                        'text-field': ['step', ['zoom'],
                            '',
                            12, ['get', 'name']
                        ],
                        'text-font': ['Roboto Regular'],
                        'text-anchor': 'top',
                        'text-size': 11,
                        'text-offset': [0, 1]
                    },
                    minzoom: minzoom
                });
            });

            let popup;
            // Listen for clicks on the layer
            map.on('click', 'styledPoints', function (evt) {
                const popupLocation = evt.features[0].geometry.coordinates.slice();
                const popupContent  = `
                        Name: ${evt.features[0].properties.name}<br />
                        Category: ${evt.features[0].properties.category}
                    `;

                popup               = new TrimbleMaps.Popup()
                    .setLngLat(popupLocation)
                    .setHTML(popupContent)
                    .addTo(map);
            });

            map.on('zoomend', (e) => {
                let zoom            = map.getZoom();
                if(zoom <= minzoom && popup && popup.isOpen()) {
                    popup.remove(); // Remove popup from the map.
                }
            });

        </script>
    </body>
</html>
