<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Storage;

use App\Models\Bookmark;
use App\Models\LoadBillingInvoice;
use App\Models\LoadBillingHistory;
use App\Models\LoadBillingPayInvoice;
use App\Models\LoadBillingPaymentAccount;
use App\Models\LoadBillingReminderHistory;
use App\Models\LoadBillingCollectionsHistory;

use App\Models\LoadStop;
use App\Models\LoadFile;
use App\Models\LoadDetail;
use App\Models\LoadAccessorial;
use App\Models\LoadDocumentRequest;
use App\Models\LoadDocumentRequestItem;

use App\Models\Broker;
use App\Models\Driver;
use App\Models\CompanyAdmin;
use App\Models\CompanyPdfTemplate;

use App\Models\Gallery;

use App\Mail\CompanyAdmin\SendLoadInvoice;

use App\Traits\AuditLoagTrait;

use App\Helpers\Common;

use PDF;
use QrCode;
use DataTables;
use Carbon\Carbon;

class BillingController extends CompanyAdminController
{
    use AuditLoagTrait;

    public $timeZone;
    public $timeZoneId;
    public $offset;

    public function __construct()
    {
        parent::__construct();

        $this->middleware(function ($request, $next) {
            $this->timeZone         = getTimeZone();
            $this->timeZoneId       = getTimeZoneId();
            $this->offset           = getTimeZoneOffset();

            $this->getGeoTimeZone      = getGeoTimeZone();

            return $next($request);
        });
    }

    public function childContentArray($rowData)
    {
        $childDataArr = array(
            'load_id' => $rowData->load_id,
            'load_detail_id' => $rowData->id,
        );

        return json_encode($childDataArr);
    }

    public function index(Request $request)
    {
        try {
            $getCompanyAdmin = auth()->guard('company-admin')->user();

            $getField = '';
            $isCheck = Bookmark::where('company_id', getUserCompnayId())->where('user_id', $getCompanyAdmin->id)->where('user_type', $getCompanyAdmin->type)->where('table_id', 'billing_loads')->first();
            if (!empty($isCheck->id)) {
                $getField = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_billing_loads_first'));
            } else {
                $getField = config('settings.table_billing_loads_first');
            }

            if ($request->ajax()) {
                /*
                $data = LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
                        ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
                        ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
                        ->whereIn('load_details.load_status', [5, 6, 7, 8, 10])
                        ->whereHas('loads', function($q){
                            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                        })
                        ->where('load_details.is_disable', 0);
                */
                $data = LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices', 'load_billing_invoice.id as invoice_id', 'load_billing_invoice.invoice_number')
                    ->leftJoin('load_billing_invoice', 'load_billing_invoice.load_detail_id', '=', 'load_details.id')
                    ->leftJoin('load_billing_activities', 'load_billing_activities.load_detail_id', '=', 'load_details.id')
                    ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
                    ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
                    ->where(function ($query) {
                        $query->where('load_details.load_status', 5) // DELIVERED: without Document
                            ->orWhere(function ($que) {
                                $que->whereIn('load_details.load_status', [5, 6]) // DELIVERED & READY TO BILL: With Document Missing Or Uploaded
                                    ->where('load_details.is_disable', 0)
                                    ->where(function ($q) {
                                        $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                                    })
                                    ->whereIn('load_billing_activities.paperwork_received', ['0', '1'])
                                    ->whereNotNull('load_details.driver_id')
                                    ->where('load_details.driver_id', '!=', 0);
                            })->orWhere(function ($que) {
                                $que->where(function ($q) {
                                    $q->where('load_details.load_status', 7) // ON HOLD
                                        ->orWhere('load_details.is_on_hold', 1);
                                })->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                            })->orWhere(function ($que) {
                                $que->whereIn('load_details.load_status', [8, 10]) // INVOICED & PAID
                                    ->where(function ($q) {
                                        $q->where('load_billing_activities.invoiced', '1')->orWhere('load_billing_activities.paid_invoice', '1');
                                    });
                            });
                    })
                    ->whereHas('loads', function ($q) {
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    });

                // General filter code - start
                foreach ($getField as $key => $value) {
                    $conditionReq       = 'search_string_condition_' . $key;
                    $sortReq            = 'search_string_sort_' . $key;
                    $conditionDateReq   = 'search_date_condition_' . $key;
                    $sortDateReq        = 'search_date_sort_' . $key;
                    $conditionStatusReq = 'search_status_condition_' . $key;
                    $sortStatusReq      = 'search_status_sort_' . $key;

                    // String filter
                    if (isset(request()->$conditionReq)) {
                        $typeReq        = 'search_string_type_' . $key;
                        $type           = request()->$typeReq;

                        if ($type == 'string') {
                            Common::getStringFilterResult($data, $key);
                        }
                    }

                    // Status filter
                    if (isset(request()->$conditionStatusReq) && request()->$conditionStatusReq == 1) {
                        Common::getStatusFilterResult($data, $key);
                    }

                    // Date filter
                    if (isset(request()->$conditionDateReq)) {
                        $typeReq        = 'search_date_type_' . $key;
                        $type           = request()->$typeReq;

                        if ($type == 'date') {
                            Common::getDateFilterResult($data, $key);
                        }
                    }

                    // String sort only
                    if (isset(request()->$sortReq)) {
                        Common::sortStringFilter($data, $key);
                    }

                    // Date sort only
                    if (isset(request()->$sortDateReq)) {
                        Common::sortDateFilter($data, $key);
                    }

                    // Status sort only
                    if (isset(request()->$sortStatusReq)) {
                        Common::sortStatusFilter($data, $key);
                    }
                }
                // General filter code - end

                return DataTables::of($data)
                    ->addIndexColumn()
                    ->setRowClass(function ($row) {
                        $class          = "";
                        if ($row->billing_load_is_audit == '1' && $row->billing_load_audit_by_id != auth()->guard('company-admin')->user()->id) {
                            $class = 'other-user-audit-lock';
                        }
                        return $class;
                    })
                    ->addColumn('audit', function ($row) {
                        if ($row->billing_load_is_audit == '1' && $row->billing_load_audit_by_id != auth()->guard('company-admin')->user()->id) {
                            $auditHtml = '<a href="javascript:void(0);" class="text-theme-color audit-lock" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '" title="locked by ' . $row->billing_load_audit_by_name . '"><i class="fa fa-lock" aria-hidden="true"></i></a>';
                            $auditHtml .= '<a href="javascript:void(0);" class="text-theme-color text-underline d-none audit-load" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '">Audit</a>';
                        } else {
                            $auditHtml = '<a href="javascript:void(0);" class="text-theme-color d-none audit-lock" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '" title="locked"><i class="fa fa-lock" aria-hidden="true"></i></a>';
                            $auditHtml .= '<a href="javascript:void(0);" class="text-theme-color text-underline audit-load" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '">Audit</a>';
                        }

                        return $auditHtml;
                    })
                    ->editColumn('load_number', function ($row) {
                        $html = '<div class="status-block">';
                        $html .= '<div class="round-icon details-control"><i class="fa fa-caret-down" aria-hidden="true"></i></div>';

                        if ($row->is_disable == 0) {
                            if (in_array($row->load_type, ['0', '6'])) {
                                if ($row->load_type == 0) {
                                    $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . $row->load_number . '</a>';
                                } else {
                                    $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                                }
                            } else {
                                $html .= '<a href="' . route('company.admin.loads.edit', base64Encode($row->load_id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                            }
                        } else {
                            if ($row->load_type == 0) {
                                $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . $row->load_number . '</a>';
                            } else {
                                $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                            }
                        }

                        return $html;
                    })
                    ->addColumn('invoice_number', function ($row) {
                        if ($row->invoice_number != '') {
                            return '<a href="' . route('company.admin.billing.invoice.show', ['id' => base64Encode($row->invoice_id)]) . '" target="_blank">' . $row->invoice_number . '</a>';
                        }
                        return '';
                    })
                    ->filterColumn('broker_name', function ($query, $keyword) {
                        $query->whereHas('broker', function ($q) use ($keyword) {
                            $q->where("brokers.name", 'LIKE', "%" . $keyword . "%");
                        });
                    })
                    ->editColumn('shipper_location', function ($row) {
                        $html = [];
                        if ($row->shipper_address_one) {
                            $html[0]    = $row->shipper_address_one;
                        }
                        if ($row->shipper_address_two) {
                            $html[1]    = $row->shipper_address_two;
                        }
                        if ($row->shipper_city) {
                            $html[2]    = $row->shipper_city;
                        }
                        if ($row->shipper_state) {
                            $html[3]    = $row->shipper_state;
                        }
                        if ($row->shipper_zip) {
                            $html[4]    = $row->shipper_zip;
                        }
                        if (count($html) == 0) {
                            return $row->shipper_location;
                        } else {
                            return implode(', ', $html);
                        }
                    })
                    ->editColumn('actual_shipper_date', function ($row) {
                        return !is_null($row->actual_shipper_date) ? $row->actual_shipper_date . " " . $row->shipper_timezone_abbr : '';
                    })
                    ->editColumn('delivery_location', function ($row) {
                        $html = [];
                        if ($row->delivery_address_one) {
                            $html[0]    = $row->delivery_address_one;
                        }
                        if ($row->delivery_address_two) {
                            $html[1]    = $row->delivery_address_two;
                        }
                        if ($row->delivery_city) {
                            $html[2]    = $row->delivery_city;
                        }
                        if ($row->delivery_state) {
                            $html[3]    = $row->delivery_state;
                        }
                        if ($row->delivery_zip) {
                            $html[4]    = $row->delivery_zip;
                        }
                        if (count($html) == 0) {
                            return $row->delivery_location;
                        } else {
                            return implode(', ', $html);
                        }
                    })
                    ->editColumn('actual_delivery_date', function ($row) {
                        return !is_null($row->actual_delivery_date) ? $row->actual_delivery_date . " " . $row->delivery_timezone_abbr : '';
                    })
                    ->addColumn('status', function ($row) {
                        return getLoadStatus($row->load_status, $row->is_on_hold);
                    })
                    ->filterColumn('status', function ($query, $keyword) {
                        if (Str::contains('pending', Str::lower($keyword))) {
                            $query = $query->where('load_status', 0);
                        }
                        if (Str::contains('accepted', Str::lower($keyword))) {
                            $query = $query->where('load_status', 1);
                        }
                        if (Str::contains('preplaned', Str::lower($keyword))) {
                            $query = $query->where('load_status', 2);
                        }
                        if (Str::contains('confirmed', Str::lower($keyword))) {
                            $query = $query->where('load_status', 3);
                        }
                        if (Str::contains('in transit', Str::lower($keyword))) {
                            $query = $query->where('load_status', 4);
                        }
                        if (Str::contains('delivered', Str::lower($keyword))) {
                            $query = $query->where('load_status', 5);
                        }
                        if (Str::contains('ready to bill', Str::lower($keyword))) {
                            $query = $query->where('load_status', 6);
                        }
                        if (Str::contains('on hold', Str::lower($keyword))) {
                            $query = $query->where('load_status', 7);
                        }
                        if (Str::contains('complete', Str::lower($keyword))) {
                            $query = $query->where('load_status', 8);
                        }
                        if (Str::contains('cancelled', Str::lower($keyword))) {
                            $query = $query->where('load_status', 9);
                        }
                        return $query;
                    })
                    ->editColumn('total_bill', function ($row) {
                        return getPrice($row->total_bill);
                    })
                    ->addColumn('factor_invoices', function ($row) {
                        return (!is_null($row->broker_factor_invoices) && $row->broker_factor_invoices == '1') ? 'Factored' : 'Direct Bill';
                    })
                    ->addColumn('documents', function ($row) {
                        if(getLoadDocumentsCount($row) == 0){
                            return '<i class="fa fa-file-text-o document-complete document-icon" aria-hidden="true"></i>';
                        }else{
                            if (!is_null($row->load_billing_activities) && $row->load_billing_activities->paperwork_received == '1') {
                                return '<i class="fa fa-file-text-o document-complete document-icon" aria-hidden="true"></i>';
                            }
                            return '<i class="fa fa-file-text-o document-in-review document-icon" aria-hidden="true"></i>';
                        }
                    })
                    ->addColumn('review', function ($row) {
                        if (!is_null($row->load_billing_activities) && $row->load_billing_activities->audit_complete == '1') {
                            return 'Complete';
                        }
                        return 'In Review';
                    })
                    ->addColumn('invoice_now', function ($row) {
                        /*
                        if(!is_null($row->load_billing_activities) && $row->load_billing_activities->paperwork_received == '1' && $row->load_billing_activities->audit_complete == '1'){
                            if ($row->load_billing_activities->paid_invoice == '1'){
                                return '<span class="text-green">Paid</span>';
                            }else if($row->load_billing_activities->invoiced == '1'){
                                return '<span class="text-green">Invoiced <i class="fa fa-reply" aria-hidden="true"></i></span>';
                            }
                            return '<a href="'.route('company.admin.billing.invoice.create', ['id' => base64Encode($row->id)]).'" class="btn btn-success">Invoice Now</a>';
                        }
                        return '<a href="javascript:void(0);" class="btn btn-liteGray">Invoice Now</a>';
                        */
                        if (!is_null($row->load_billing_activities)) {
                            if ($row->load_billing_activities->paperwork_received == '1' && $row->load_billing_activities->audit_complete == '1') {
                                if ($row->load_billing_activities->paid_invoice == '1') {
                                    return '<span class="text-green">Paid</span>';
                                } else if ($row->load_billing_activities->invoiced == '1') {
                                    return '<span class="text-green">Invoiced <i class="fa fa-reply" aria-hidden="true"></i></span>';
                                } else {
                                    return '<a href="' . route('company.admin.billing.invoice.create', ['id' => base64Encode($row->id)]) . '" class="btn btn-success">Invoice Now</a>';
                                }
                            } else {
                                if (count(getLoadDocuments($row)['merged_documents']) > 0) {
                                    return '<a href="javascript:void(0);" class="btn btn-liteGray">Invoice Now</a>';
                                } else {
                                    return '<a href="' . route('company.admin.billing.invoice.create', ['id' => base64Encode($row->id)]) . '" class="btn btn-success">Invoice Now</a>';
                                }
                            }
                        } else {
                            return '<a href="' . route('company.admin.billing.invoice.create', ['id' => base64Encode($row->id)]) . '" class="btn btn-success">Invoice Now</a>';
                        }
                    })
                    ->addColumn('child_content', function ($row) {
                        return $this->childContentArray($row);
                    })
                    ->rawColumns(['audit', 'load_number', 'invoice_number', 'type', 'documents', 'review', 'invoice_now', 'child_content'])
                    ->make(true);
            }

            $tableName = 'settings.table_billing_loads';
            $tables = Common::getStaticBillingLoadTables();
            $isMulitpleTable = 1;

            if ($request->page == 'new') {
                return view('companyAdmin.billing.index-html');
            }
            return view('companyAdmin.billing.index', compact('getField', 'tableName', 'tables', 'isMulitpleTable'));
        } catch (\Exception $e) {
            Log::emergency("File: " . $e->getFile());
            Log::emergency("Line: " . $e->getLine());
            Log::emergency("Message: " . $e->getMessage());
        }
    }

    public function readyToBillLoadsIndex(Request $request)
    {
        try {
            $getCompanyAdmin = auth()->guard('company-admin')->user();

            $getField = '';
            $isCheck = Bookmark::where('company_id', getUserCompnayId())->where('user_id', $getCompanyAdmin->id)->where('user_type', $getCompanyAdmin->type)->where('table_id', 'billing_loads')->first();
            if (!empty($isCheck->id)) {
                $getField = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_billing_loads_first'));
            } else {
                $getField = config('settings.table_billing_loads_first');
            }

            if ($request->ajax()) {
                /*
                $data = LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
                        ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
                        ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
                        ->where('load_details.is_disable', 0)
                        ->where(function($query) {
                            $query->where('load_details.load_status', 6)->where(function($q) {
                                    $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                                });
                        })
                        ->where('load_details.load_status', 6)
                        ->where(function($q) {
                            $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                        })
                        ->whereHas('loads', function($q){
                            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                        });
                */
                $data = LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
                    ->leftJoin('load_billing_activities', 'load_billing_activities.load_detail_id', '=', 'load_details.id')
                    ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
                    ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
                    ->where('load_details.load_status', 6)
                    ->where('load_billing_activities.paperwork_received', '1')
                    ->where('load_details.is_disable', 0)
                    ->where(function ($query) {
                        $query->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                    })
                    ->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0)
                    ->whereHas('loads', function ($q) {
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    });

                // General filter code - start
                foreach ($getField as $key => $value) {
                    $conditionReq       = 'search_string_condition_' . $key;
                    $sortReq            = 'search_string_sort_' . $key;
                    $conditionDateReq   = 'search_date_condition_' . $key;
                    $sortDateReq        = 'search_date_sort_' . $key;
                    $conditionStatusReq = 'search_status_condition_' . $key;
                    $sortStatusReq      = 'search_status_sort_' . $key;

                    // String filter
                    if (isset(request()->$conditionReq)) {
                        $typeReq        = 'search_string_type_' . $key;
                        $type           = request()->$typeReq;

                        if ($type == 'string') {
                            Common::getStringFilterResult($data, $key);
                        }
                    }

                    // Status filter
                    if (isset(request()->$conditionStatusReq) && request()->$conditionStatusReq == 1) {
                        Common::getStatusFilterResult($data, $key);
                    }

                    // Date filter
                    if (isset(request()->$conditionDateReq)) {
                        $typeReq        = 'search_date_type_' . $key;
                        $type           = request()->$typeReq;

                        if ($type == 'date') {
                            Common::getDateFilterResult($data, $key);
                        }
                    }

                    // String sort only
                    if (isset(request()->$sortReq)) {
                        Common::sortStringFilter($data, $key);
                    }

                    // Date sort only
                    if (isset(request()->$sortDateReq)) {
                        Common::sortDateFilter($data, $key);
                    }

                    // Status sort only
                    if (isset(request()->$sortStatusReq)) {
                        Common::sortStatusFilter($data, $key);
                    }
                }
                // General filter code - end

                return DataTables::of($data)
                    ->addIndexColumn()
                    ->setRowClass(function ($row) {
                        $class          = "";
                        if ($row->billing_load_is_audit == '1' && $row->billing_load_audit_by_id != auth()->guard('company-admin')->user()->id) {
                            $class = 'other-user-audit-lock';
                        }
                        return $class;
                    })
                    ->addColumn('audit', function ($row) {
                        if ($row->billing_load_is_audit == '1' && $row->billing_load_audit_by_id != auth()->guard('company-admin')->user()->id) {
                            $auditHtml = '<a href="javascript:void(0);" class="text-theme-color audit-lock" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '" title="locked by ' . $row->billing_load_audit_by_name . '"><i class="fa fa-lock" aria-hidden="true"></i></a>';
                            $auditHtml .= '<a href="javascript:void(0);" class="text-theme-color text-underline d-none audit-load" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '">Audit</a>';
                        } else {
                            $auditHtml = '<a href="javascript:void(0);" class="text-theme-color d-none audit-lock" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '" title="locked"><i class="fa fa-lock" aria-hidden="true"></i></a>';
                            $auditHtml .= '<a href="javascript:void(0);" class="text-theme-color text-underline audit-load" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '">Audit</a>';
                        }

                        return $auditHtml;
                    })
                    ->editColumn('load_number', function ($row) {
                        $html = '<div class="status-block">';
                        $html .= '<div class="round-icon details-control"><i class="fa fa-caret-down" aria-hidden="true"></i></div>';

                        if ($row->is_disable == 0) {
                            if (in_array($row->load_type, ['0', '6'])) {
                                if ($row->load_type == 0) {
                                    $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . $row->load_number . '</a>';
                                } else {
                                    $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                                }
                            } else {
                                $html .= '<a href="' . route('company.admin.loads.edit', base64Encode($row->load_id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                            }
                        } else {
                            if ($row->load_type == 0) {
                                $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . $row->load_number . '</a>';
                            } else {
                                $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                            }
                        }

                        return $html;
                    })
                    ->addColumn('invoice_number', function ($row) {
                        return '';
                    })
                    ->filterColumn('broker_name', function ($query, $keyword) {
                        $query->whereHas('broker', function ($q) use ($keyword) {
                            $q->where("brokers.name", 'LIKE', "%" . $keyword . "%");
                        });
                    })
                    ->editColumn('shipper_location', function ($row) {
                        $html = [];
                        if ($row->shipper_address_one) {
                            $html[0]    = $row->shipper_address_one;
                        }
                        if ($row->shipper_address_two) {
                            $html[1]    = $row->shipper_address_two;
                        }
                        if ($row->shipper_city) {
                            $html[2]    = $row->shipper_city;
                        }
                        if ($row->shipper_state) {
                            $html[3]    = $row->shipper_state;
                        }
                        if ($row->shipper_zip) {
                            $html[4]    = $row->shipper_zip;
                        }
                        if (count($html) == 0) {
                            return $row->shipper_location;
                        } else {
                            return implode(', ', $html);
                        }
                    })
                    ->editColumn('actual_shipper_date', function ($row) {
                        return !is_null($row->actual_shipper_date) ? $row->actual_shipper_date . " " . $row->shipper_timezone_abbr : '';
                    })
                    ->editColumn('delivery_location', function ($row) {
                        $html = [];
                        if ($row->delivery_address_one) {
                            $html[0]    = $row->delivery_address_one;
                        }
                        if ($row->delivery_address_two) {
                            $html[1]    = $row->delivery_address_two;
                        }
                        if ($row->delivery_city) {
                            $html[2]    = $row->delivery_city;
                        }
                        if ($row->delivery_state) {
                            $html[3]    = $row->delivery_state;
                        }
                        if ($row->delivery_zip) {
                            $html[4]    = $row->delivery_zip;
                        }
                        if (count($html) == 0) {
                            return $row->delivery_location;
                        } else {
                            return implode(', ', $html);
                        }
                    })
                    ->editColumn('actual_delivery_date', function ($row) {
                        return !is_null($row->actual_delivery_date) ? $row->actual_delivery_date . " " . $row->delivery_timezone_abbr : '';
                    })
                    ->addColumn('status', function ($row) {
                        return getLoadStatus($row->load_status, $row->is_on_hold);
                    })
                    ->filterColumn('status', function ($query, $keyword) {
                        if (Str::contains('pending', Str::lower($keyword))) {
                            $query = $query->where('load_status', 0);
                        }
                        if (Str::contains('accepted', Str::lower($keyword))) {
                            $query = $query->where('load_status', 1);
                        }
                        if (Str::contains('preplaned', Str::lower($keyword))) {
                            $query = $query->where('load_status', 2);
                        }
                        if (Str::contains('confirmed', Str::lower($keyword))) {
                            $query = $query->where('load_status', 3);
                        }
                        if (Str::contains('in transit', Str::lower($keyword))) {
                            $query = $query->where('load_status', 4);
                        }
                        if (Str::contains('delivered', Str::lower($keyword))) {
                            $query = $query->where('load_status', 5);
                        }
                        if (Str::contains('ready to bill', Str::lower($keyword))) {
                            $query = $query->where('load_status', 6);
                        }
                        if (Str::contains('on hold', Str::lower($keyword))) {
                            $query = $query->where('load_status', 7);
                        }
                        if (Str::contains('complete', Str::lower($keyword))) {
                            $query = $query->where('load_status', 8);
                        }
                        if (Str::contains('cancelled', Str::lower($keyword))) {
                            $query = $query->where('load_status', 9);
                        }
                        return $query;
                    })
                    ->editColumn('total_bill', function ($row) {
                        return getPrice($row->total_bill);
                    })
                    ->addColumn('factor_invoices', function ($row) {
                        return (!is_null($row->broker_factor_invoices) && $row->broker_factor_invoices == '1') ? 'Factored' : 'Direct Bill';
                    })
                    ->addColumn('documents', function ($row) {
                        if (!is_null($row->load_billing_activities) && $row->load_billing_activities->paperwork_received == '1') {
                            return '<i class="fa fa-file-text-o document-complete document-icon" aria-hidden="true"></i>';
                        }
                        return '<i class="fa fa-file-text-o document-in-review document-icon" aria-hidden="true"></i>';
                    })
                    ->addColumn('review', function ($row) {
                        if (!is_null($row->load_billing_activities) && $row->load_billing_activities->audit_complete == '1') {
                            return 'Complete';
                        }
                        return 'In Review';
                    })
                    ->addColumn('invoice_now', function ($row) {
                        if (!is_null($row->load_billing_activities) && $row->load_billing_activities->paperwork_received == '1' && $row->load_billing_activities->audit_complete == '1') {
                            if ($row->load_billing_activities->paid_invoice == '1') {
                                return '<span class="text-green">Paid</span>';
                            } else if ($row->load_billing_activities->invoiced == '1') {
                                return '<span class="text-green">Invoiced <i class="fa fa-reply" aria-hidden="true"></i></span>';
                            }
                            return '<a href="' . route('company.admin.billing.invoice.create', ['id' => base64Encode($row->id)]) . '" class="btn btn-success">Invoice Now</a>';
                        }
                        return '<a href="javascript:void(0);" class="btn btn-liteGray">Invoice Now</a>';
                    })
                    ->addColumn('child_content', function ($row) {
                        return $this->childContentArray($row);
                    })
                    ->rawColumns(['audit', 'load_number', 'invoice_number', 'type', 'documents', 'review', 'invoice_now', 'child_content'])
                    ->make(true);
            }
        } catch (\Exception $e) {
            \Log::emergency("File: " . $e->getFile());
            \Log::emergency("Line: " . $e->getLine());
            \Log::emergency("Message: " . $e->getMessage());
        }
    }

    public function missingDocsLoadsIndex(Request $request)
    {
        try {
            $getCompanyAdmin = auth()->guard('company-admin')->user();

            $getField = '';
            $isCheck = Bookmark::where('company_id', getUserCompnayId())->where('user_id', $getCompanyAdmin->id)->where('user_type', $getCompanyAdmin->type)->where('table_id', 'billing_loads')->first();
            if (!empty($isCheck->id)) {
                $getField = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_billing_loads_first'));
            } else {
                $getField = config('settings.table_billing_loads_first');
            }

            if ($request->ajax()) {
                $data = LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
                    ->leftJoin('load_billing_activities', 'load_billing_activities.load_detail_id', '=', 'load_details.id')
                    ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
                    ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
                    ->where('load_details.load_status', 5)
                    ->where('load_billing_activities.paperwork_received', '0')
                    ->where('load_details.is_disable', 0)
                    ->where(function ($query) {
                        $query->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                    })
                    ->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0)
                    ->whereHas('loads', function ($q) {
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    });

                // General filter code - start
                foreach ($getField as $key => $value) {
                    $conditionReq       = 'search_string_condition_' . $key;
                    $sortReq            = 'search_string_sort_' . $key;
                    $conditionDateReq   = 'search_date_condition_' . $key;
                    $sortDateReq        = 'search_date_sort_' . $key;
                    $conditionStatusReq = 'search_status_condition_' . $key;
                    $sortStatusReq      = 'search_status_sort_' . $key;

                    // String filter
                    if (isset(request()->$conditionReq)) {
                        $typeReq        = 'search_string_type_' . $key;
                        $type           = request()->$typeReq;

                        if ($type == 'string') {
                            Common::getStringFilterResult($data, $key);
                        }
                    }

                    // Status filter
                    if (isset(request()->$conditionStatusReq) && request()->$conditionStatusReq == 1) {
                        Common::getStatusFilterResult($data, $key);
                    }

                    // Date filter
                    if (isset(request()->$conditionDateReq)) {
                        $typeReq        = 'search_date_type_' . $key;
                        $type           = request()->$typeReq;

                        if ($type == 'date') {
                            Common::getDateFilterResult($data, $key);
                        }
                    }

                    // String sort only
                    if (isset(request()->$sortReq)) {
                        Common::sortStringFilter($data, $key);
                    }

                    // Date sort only
                    if (isset(request()->$sortDateReq)) {
                        Common::sortDateFilter($data, $key);
                    }

                    // Status sort only
                    if (isset(request()->$sortStatusReq)) {
                        Common::sortStatusFilter($data, $key);
                    }
                }
                // General filter code - end

                $data->get();
                return DataTables::of($data)
                    ->addIndexColumn()
                    ->setRowClass(function ($row) {
                        $class          = "";
                        if ($row->billing_load_is_audit == '1' && $row->billing_load_audit_by_id != auth()->guard('company-admin')->user()->id) {
                            $class = 'other-user-audit-lock';
                        }
                        return $class;
                    })
                    ->addColumn('audit', function ($row) {
                        if ($row->billing_load_is_audit == '1' && $row->billing_load_audit_by_id != auth()->guard('company-admin')->user()->id) {
                            $auditHtml = '<a href="javascript:void(0);" class="text-theme-color audit-lock" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '" title="locked by ' . $row->billing_load_audit_by_name . '"><i class="fa fa-lock" aria-hidden="true"></i></a>';
                            $auditHtml .= '<a href="javascript:void(0);" class="text-theme-color text-underline d-none audit-load" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '">Audit</a>';
                        } else {
                            $auditHtml = '<a href="javascript:void(0);" class="text-theme-color d-none audit-lock" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '" title="locked"><i class="fa fa-lock" aria-hidden="true"></i></a>';
                            $auditHtml .= '<a href="javascript:void(0);" class="text-theme-color text-underline audit-load" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '">Audit</a>';
                        }
                        return $auditHtml;
                    })
                    ->editColumn('load_number', function ($row) {
                        $html = '<div class="status-block">';
                        $html .= '<div class="round-icon details-control"><i class="fa fa-caret-down" aria-hidden="true"></i></div>';

                        if ($row->is_disable == 0) {
                            if (in_array($row->load_type, ['0', '6'])) {
                                if ($row->load_type == 0) {
                                    $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . $row->load_number . '</a>';
                                } else {
                                    $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                                }
                            } else {
                                $html .= '<a href="' . route('company.admin.loads.edit', base64Encode($row->load_id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                            }
                        } else {
                            if ($row->load_type == 0) {
                                $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . $row->load_number . '</a>';
                            } else {
                                $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                            }
                        }

                        return $html;
                    })
                    ->addColumn('invoice_number', function ($row) {
                        return '';
                    })
                    ->filterColumn('broker_name', function ($query, $keyword) {
                        $query->whereHas('broker', function ($q) use ($keyword) {
                            $q->where("brokers.name", 'LIKE', "%" . $keyword . "%");
                        });
                    })
                    ->editColumn('shipper_location', function ($row) {
                        $html = [];
                        if ($row->shipper_address_one) {
                            $html[0]    = $row->shipper_address_one;
                        }
                        if ($row->shipper_address_two) {
                            $html[1]    = $row->shipper_address_two;
                        }
                        if ($row->shipper_city) {
                            $html[2]    = $row->shipper_city;
                        }
                        if ($row->shipper_state) {
                            $html[3]    = $row->shipper_state;
                        }
                        if ($row->shipper_zip) {
                            $html[4]    = $row->shipper_zip;
                        }
                        if (count($html) == 0) {
                            return $row->shipper_location;
                        } else {
                            return implode(', ', $html);
                        }
                    })
                    ->editColumn('actual_shipper_date', function ($row) {
                        return !is_null($row->actual_shipper_date) ? $row->actual_shipper_date . " " . $row->shipper_timezone_abbr : '';
                    })
                    ->editColumn('delivery_location', function ($row) {
                        $html = [];
                        if ($row->delivery_address_one) {
                            $html[0]    = $row->delivery_address_one;
                        }
                        if ($row->delivery_address_two) {
                            $html[1]    = $row->delivery_address_two;
                        }
                        if ($row->delivery_city) {
                            $html[2]    = $row->delivery_city;
                        }
                        if ($row->delivery_state) {
                            $html[3]    = $row->delivery_state;
                        }
                        if ($row->delivery_zip) {
                            $html[4]    = $row->delivery_zip;
                        }
                        if (count($html) == 0) {
                            return $row->delivery_location;
                        } else {
                            return implode(', ', $html);
                        }
                    })
                    ->editColumn('actual_delivery_date', function ($row) {
                        return !is_null($row->actual_delivery_date) ? $row->actual_delivery_date . " " . $row->delivery_timezone_abbr : '';
                    })
                    ->addColumn('status', function ($row) {
                        return getLoadStatus($row->load_status, $row->is_on_hold);
                    })
                    ->filterColumn('status', function ($query, $keyword) {
                        if (Str::contains('pending', Str::lower($keyword))) {
                            $query = $query->where('load_status', 0);
                        }
                        if (Str::contains('accepted', Str::lower($keyword))) {
                            $query = $query->where('load_status', 1);
                        }
                        if (Str::contains('preplaned', Str::lower($keyword))) {
                            $query = $query->where('load_status', 2);
                        }
                        if (Str::contains('confirmed', Str::lower($keyword))) {
                            $query = $query->where('load_status', 3);
                        }
                        if (Str::contains('in transit', Str::lower($keyword))) {
                            $query = $query->where('load_status', 4);
                        }
                        if (Str::contains('delivered', Str::lower($keyword))) {
                            $query = $query->where('load_status', 5);
                        }
                        if (Str::contains('ready to bill', Str::lower($keyword))) {
                            $query = $query->where('load_status', 6);
                        }
                        if (Str::contains('on hold', Str::lower($keyword))) {
                            $query = $query->where('load_status', 7);
                        }
                        if (Str::contains('complete', Str::lower($keyword))) {
                            $query = $query->where('load_status', 8);
                        }
                        if (Str::contains('cancelled', Str::lower($keyword))) {
                            $query = $query->where('load_status', 9);
                        }
                        return $query;
                    })
                    ->editColumn('total_bill', function ($row) {
                        return getPrice($row->total_bill);
                    })
                    ->addColumn('factor_invoices', function ($row) {
                        return (!is_null($row->broker_factor_invoices) && $row->broker_factor_invoices == '1') ? 'Factored' : 'Direct Bill';
                    })
                    ->addColumn('documents', function ($row) {
                        if (!is_null($row->load_billing_activities) && $row->load_billing_activities->paperwork_received == '1') {
                            return '<i class="fa fa-file-text-o document-complete document-icon" aria-hidden="true"></i>';
                        }
                        return '<i class="fa fa-file-text-o document-in-review document-icon" aria-hidden="true"></i>';
                    })
                    ->addColumn('review', function ($row) {
                        if (!is_null($row->load_billing_activities) && $row->load_billing_activities->audit_complete == '1') {
                            return 'Complete';
                        }
                        return 'In Review';
                    })
                    ->addColumn('invoice_now', function ($row) {
                        if (!is_null($row->load_billing_activities) && $row->load_billing_activities->paperwork_received == '1' && $row->load_billing_activities->audit_complete == '1') {
                            if ($row->load_billing_activities->paid_invoice == '1') {
                                return '<span class="text-green">Paid</span>';
                            } else if ($row->load_billing_activities->invoiced == '1') {
                                return '<span class="text-green">Invoiced <i class="fa fa-reply" aria-hidden="true"></i></span>';
                            }
                            return '<a href="' . route('company.admin.billing.invoice.create', ['id' => base64Encode($row->id)]) . '" class="btn btn-success">Invoice Now</a>';
                        }
                        return '<a href="javascript:void(0);" class="btn btn-liteGray">Invoice Now</a>';
                    })
                    ->addColumn('child_content', function ($row) {
                        return $this->childContentArray($row);
                    })
                    ->rawColumns(['audit', 'load_number', 'invoice_number', 'type', 'documents', 'review', 'invoice_now', 'child_content'])
                    ->make(true);
            }
        } catch (\Exception $e) {
            \Log::emergency("File: " . $e->getFile());
            \Log::emergency("Line: " . $e->getLine());
            \Log::emergency("Message: " . $e->getMessage());
        }
    }

    public function onHoldLoadsIndex(Request $request)
    {
        try {
            $getCompanyAdmin = auth()->guard('company-admin')->user();

            $getField = '';
            $isCheck = Bookmark::where('company_id', getUserCompnayId())->where('user_id', $getCompanyAdmin->id)->where('user_type', $getCompanyAdmin->type)->where('table_id', 'billing_loads')->first();
            if (!empty($isCheck->id)) {
                $getField = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_billing_loads_first'));
            } else {
                $getField = config('settings.table_billing_loads_first');
            }

            if ($request->ajax()) {
                /*
                $data = LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
                        ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
                        ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
                        ->where(function($query) {
                            $query->where(function($q) {
                                $q->where('load_details.load_status', 7)->orWhere('load_details.is_on_hold', 1);
                            })
                                ->where(function($q) {
                                    $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                                });
                        })
                        ->whereHas('loads', function($q){
                            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                        });
                */
                $data = LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
                    ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
                    ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
                    ->where(function ($query) {
                        $query->where(function ($q) {
                            $q->where('load_details.load_status', 7)->orWhere('load_details.is_on_hold', 1);
                        })->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                    })
                    ->whereHas('loads', function ($q) {
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    });

                // General filter code - start
                foreach ($getField as $key => $value) {
                    $conditionReq       = 'search_string_condition_' . $key;
                    $sortReq            = 'search_string_sort_' . $key;
                    $conditionDateReq   = 'search_date_condition_' . $key;
                    $sortDateReq        = 'search_date_sort_' . $key;
                    $conditionStatusReq = 'search_status_condition_' . $key;
                    $sortStatusReq      = 'search_status_sort_' . $key;

                    // String filter
                    if (isset(request()->$conditionReq)) {
                        $typeReq        = 'search_string_type_' . $key;
                        $type           = request()->$typeReq;

                        if ($type == 'string') {
                            Common::getStringFilterResult($data, $key);
                        }
                    }

                    // Status filter
                    if (isset(request()->$conditionStatusReq) && request()->$conditionStatusReq == 1) {
                        Common::getStatusFilterResult($data, $key);
                    }

                    // Date filter
                    if (isset(request()->$conditionDateReq)) {
                        $typeReq        = 'search_date_type_' . $key;
                        $type           = request()->$typeReq;

                        if ($type == 'date') {
                            Common::getDateFilterResult($data, $key);
                        }
                    }

                    // String sort only
                    if (isset(request()->$sortReq)) {
                        Common::sortStringFilter($data, $key);
                    }

                    // Date sort only
                    if (isset(request()->$sortDateReq)) {
                        Common::sortDateFilter($data, $key);
                    }

                    // Status sort only
                    if (isset(request()->$sortStatusReq)) {
                        Common::sortStatusFilter($data, $key);
                    }
                }
                // General filter code - end

                $data->get();
                return DataTables::of($data)
                    ->addIndexColumn()
                    ->setRowClass(function ($row) {
                        $class          = "";
                        if ($row->billing_load_is_audit == '1' && $row->billing_load_audit_by_id != auth()->guard('company-admin')->user()->id) {
                            $class = 'other-user-audit-lock';
                        }
                        return $class;
                    })
                    ->addColumn('audit', function ($row) {
                        if ($row->billing_load_is_audit == '1' && $row->billing_load_audit_by_id != auth()->guard('company-admin')->user()->id) {
                            $auditHtml = '<a href="javascript:void(0);" class="text-theme-color audit-lock" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '" title="locked by ' . $row->billing_load_audit_by_name . '"><i class="fa fa-lock" aria-hidden="true"></i></a>';
                            $auditHtml .= '<a href="javascript:void(0);" class="text-theme-color text-underline d-none audit-load" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '">Audit</a>';
                        } else {
                            $auditHtml = '<a href="javascript:void(0);" class="text-theme-color d-none audit-lock" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '" title="locked"><i class="fa fa-lock" aria-hidden="true"></i></a>';
                            $auditHtml .= '<a href="javascript:void(0);" class="text-theme-color text-underline audit-load" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '">Audit</a>';
                        }

                        return $auditHtml;
                    })
                    ->editColumn('load_number', function ($row) {
                        $html = '<div class="status-block">';
                        $html .= '<div class="round-icon details-control"><i class="fa fa-caret-down" aria-hidden="true"></i></div>';

                        if ($row->is_disable == 0) {
                            if (in_array($row->load_type, ['0', '6'])) {
                                if ($row->load_type == 0) {
                                    $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . $row->load_number . '</a>';
                                } else {
                                    $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                                }
                            } else {
                                $html .= '<a href="' . route('company.admin.loads.edit', base64Encode($row->load_id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                            }
                        } else {
                            if ($row->load_type == 0) {
                                $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . $row->load_number . '</a>';
                            } else {
                                $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                            }
                        }

                        return $html;
                    })
                    ->addColumn('invoice_number', function ($row) {
                        return '';
                    })
                    ->filterColumn('broker_name', function ($query, $keyword) {
                        $query->whereHas('broker', function ($q) use ($keyword) {
                            $q->where("brokers.name", 'LIKE', "%" . $keyword . "%");
                        });
                    })
                    ->editColumn('shipper_location', function ($row) {
                        $html = [];
                        if ($row->shipper_address_one) {
                            $html[0]    = $row->shipper_address_one;
                        }
                        if ($row->shipper_address_two) {
                            $html[1]    = $row->shipper_address_two;
                        }
                        if ($row->shipper_city) {
                            $html[2]    = $row->shipper_city;
                        }
                        if ($row->shipper_state) {
                            $html[3]    = $row->shipper_state;
                        }
                        if ($row->shipper_zip) {
                            $html[4]    = $row->shipper_zip;
                        }
                        if (count($html) == 0) {
                            return $row->shipper_location;
                        } else {
                            return implode(', ', $html);
                        }
                    })
                    ->editColumn('actual_shipper_date', function ($row) {
                        return !is_null($row->actual_shipper_date) ? $row->actual_shipper_date . " " . $row->shipper_timezone_abbr : '';
                    })
                    ->editColumn('delivery_location', function ($row) {
                        $html = [];
                        if ($row->delivery_address_one) {
                            $html[0]    = $row->delivery_address_one;
                        }
                        if ($row->delivery_address_two) {
                            $html[1]    = $row->delivery_address_two;
                        }
                        if ($row->delivery_city) {
                            $html[2]    = $row->delivery_city;
                        }
                        if ($row->delivery_state) {
                            $html[3]    = $row->delivery_state;
                        }
                        if ($row->delivery_zip) {
                            $html[4]    = $row->delivery_zip;
                        }
                        if (count($html) == 0) {
                            return $row->delivery_location;
                        } else {
                            return implode(', ', $html);
                        }
                    })
                    ->editColumn('actual_delivery_date', function ($row) {
                        return !is_null($row->actual_delivery_date) ? $row->actual_delivery_date . " " . $row->delivery_timezone_abbr : '';
                    })
                    ->addColumn('status', function ($row) {
                        return getLoadStatus($row->load_status, $row->is_on_hold);
                    })
                    ->filterColumn('status', function ($query, $keyword) {
                        if (Str::contains('pending', Str::lower($keyword))) {
                            $query = $query->where('load_status', 0);
                        }
                        if (Str::contains('accepted', Str::lower($keyword))) {
                            $query = $query->where('load_status', 1);
                        }
                        if (Str::contains('preplaned', Str::lower($keyword))) {
                            $query = $query->where('load_status', 2);
                        }
                        if (Str::contains('confirmed', Str::lower($keyword))) {
                            $query = $query->where('load_status', 3);
                        }
                        if (Str::contains('in transit', Str::lower($keyword))) {
                            $query = $query->where('load_status', 4);
                        }
                        if (Str::contains('delivered', Str::lower($keyword))) {
                            $query = $query->where('load_status', 5);
                        }
                        if (Str::contains('ready to bill', Str::lower($keyword))) {
                            $query = $query->where('load_status', 6);
                        }
                        if (Str::contains('on hold', Str::lower($keyword))) {
                            $query = $query->where('load_status', 7);
                        }
                        if (Str::contains('complete', Str::lower($keyword))) {
                            $query = $query->where('load_status', 8);
                        }
                        if (Str::contains('cancelled', Str::lower($keyword))) {
                            $query = $query->where('load_status', 9);
                        }
                        return $query;
                    })
                    ->editColumn('total_bill', function ($row) {
                        return getPrice($row->total_bill);
                    })
                    ->addColumn('factor_invoices', function ($row) {
                        return (!is_null($row->broker_factor_invoices) && $row->broker_factor_invoices == '1') ? 'Factored' : 'Direct Bill';
                    })
                    ->addColumn('documents', function ($row) {
                        if (!is_null($row->load_billing_activities) && $row->load_billing_activities->paperwork_received == '1') {
                            return '<i class="fa fa-file-text-o document-complete document-icon" aria-hidden="true"></i>';
                        }
                        return '<i class="fa fa-file-text-o document-in-review document-icon" aria-hidden="true"></i>';
                    })
                    ->addColumn('review', function ($row) {
                        if (!is_null($row->load_billing_activities) && $row->load_billing_activities->audit_complete == '1') {
                            return 'Complete';
                        }
                        return 'In Review';
                    })
                    ->addColumn('invoice_now', function ($row) {
                        if (!is_null($row->load_billing_activities) && $row->load_billing_activities->paperwork_received == '1' && $row->load_billing_activities->audit_complete == '1') {
                            if ($row->load_billing_activities->paid_invoice == '1') {
                                return '<span class="text-green">Paid</span>';
                            } else if ($row->load_billing_activities->invoiced == '1') {
                                return '<span class="text-green">Invoiced <i class="fa fa-reply" aria-hidden="true"></i></span>';
                            }
                            return '<a href="' . route('company.admin.billing.invoice.create', ['id' => base64Encode($row->id)]) . '" class="btn btn-success">Invoice Now</a>';
                        }
                        return '<a href="javascript:void(0);" class="btn btn-liteGray">Invoice Now</a>';
                    })
                    ->addColumn('child_content', function ($row) {
                        return $this->childContentArray($row);
                    })
                    ->rawColumns(['audit', 'load_number', 'invoice_number', 'type', 'documents', 'review', 'invoice_now', 'child_content'])
                    ->make(true);
            }
        } catch (\Exception $e) {
            \Log::emergency("File: " . $e->getFile());
            \Log::emergency("Line: " . $e->getLine());
            \Log::emergency("Message: " . $e->getMessage());
        }
    }

    public function invoicedLoadsIndex(Request $request)
    {
        try {
            $getCompanyAdmin = auth()->guard('company-admin')->user();

            $getField = '';
            $isCheck = Bookmark::where('company_id', getUserCompnayId())->where('user_id', $getCompanyAdmin->id)->where('user_type', $getCompanyAdmin->type)->where('table_id', 'billing_loads')->first();
            if (!empty($isCheck->id)) {
                $getField = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_billing_loads_first'));
            } else {
                $getField = config('settings.table_billing_loads_first');
            }

            if ($request->ajax()) {
                $data = LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices', 'load_billing_invoice.id as invoice_id', 'load_billing_invoice.invoice_number')
                    ->leftJoin('load_billing_invoice', 'load_billing_invoice.load_detail_id', '=', 'load_details.id')
                    ->leftJoin('load_billing_activities', 'load_billing_activities.load_detail_id', '=', 'load_details.id')
                    ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
                    ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
                    ->where('load_details.load_status', 10)
                    ->where('load_billing_activities.invoiced', '1')
                    ->whereHas('loads', function ($q) {
                        $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                    });

                // General filter code - start
                foreach ($getField as $key => $value) {
                    $conditionReq       = 'search_string_condition_' . $key;
                    $sortReq            = 'search_string_sort_' . $key;
                    $conditionDateReq   = 'search_date_condition_' . $key;
                    $sortDateReq        = 'search_date_sort_' . $key;
                    $conditionStatusReq = 'search_status_condition_' . $key;
                    $sortStatusReq      = 'search_status_sort_' . $key;

                    // String filter
                    if (isset(request()->$conditionReq)) {
                        $typeReq        = 'search_string_type_' . $key;
                        $type           = request()->$typeReq;

                        if ($type == 'string') {
                            Common::getStringFilterResult($data, $key);
                        }
                    }

                    // Status filter
                    if (isset(request()->$conditionStatusReq) && request()->$conditionStatusReq == 1) {
                        Common::getStatusFilterResult($data, $key);
                    }

                    // Date filter
                    if (isset(request()->$conditionDateReq)) {
                        $typeReq        = 'search_date_type_' . $key;
                        $type           = request()->$typeReq;

                        if ($type == 'date') {
                            Common::getDateFilterResult($data, $key);
                        }
                    }

                    // String sort only
                    if (isset(request()->$sortReq)) {
                        Common::sortStringFilter($data, $key);
                    }

                    // Date sort only
                    if (isset(request()->$sortDateReq)) {
                        Common::sortDateFilter($data, $key);
                    }

                    // Status sort only
                    if (isset(request()->$sortStatusReq)) {
                        Common::sortStatusFilter($data, $key);
                    }
                }
                // General filter code - end

                $data->get();
                return DataTables::of($data)
                    ->addIndexColumn()
                    ->setRowClass(function ($row) {
                        $class          = "";
                        if ($row->billing_load_is_audit == '1' && $row->billing_load_audit_by_id != auth()->guard('company-admin')->user()->id) {
                            $class = 'other-user-audit-lock';
                        }
                        return $class;
                    })
                    ->addColumn('audit', function ($row) {
                        if ($row->billing_load_is_audit == '1' && $row->billing_load_audit_by_id != auth()->guard('company-admin')->user()->id) {
                            $auditHtml = '<a href="javascript:void(0);" class="text-theme-color audit-lock" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '" title="locked by ' . $row->billing_load_audit_by_name . '"><i class="fa fa-lock" aria-hidden="true"></i></a>';
                            $auditHtml .= '<a href="javascript:void(0);" class="text-theme-color text-underline d-none audit-load" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '">Audit</a>';
                        } else {
                            $auditHtml = '<a href="javascript:void(0);" class="text-theme-color d-none audit-lock" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '" title="locked"><i class="fa fa-lock" aria-hidden="true"></i></a>';
                            $auditHtml .= '<a href="javascript:void(0);" class="text-theme-color text-underline audit-load" data-load="' . $row->load_id . '" data-load-details="' . $row->id . '">Audit</a>';
                        }
                        return $auditHtml;
                    })
                    ->editColumn('load_number', function ($row) {
                        $html = '<div class="status-block">';
                        $html .= '<div class="round-icon details-control"><i class="fa fa-caret-down" aria-hidden="true"></i></div>';

                        if ($row->is_disable == 0) {
                            if (in_array($row->load_type, ['0', '6'])) {
                                if ($row->load_type == 0) {
                                    $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . $row->load_number . '</a>';
                                } else {
                                    $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                                }
                            } else {
                                $html .= '<a href="' . route('company.admin.loads.edit', base64Encode($row->load_id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                            }
                        } else {
                            if ($row->load_type == 0) {
                                $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . $row->load_number . '</a>';
                            } else {
                                $html .= '<a href="' . route('company.admin.loads.show', base64Encode($row->id)) . '" class="mr-1 load-edit-link" target="_blank">' . getLoadTypeList()[$row->load_type] . '</a>';
                            }
                        }

                        return $html;
                    })
                    ->addColumn('invoice_number', function ($row) {
                        if ($row->invoice_number != '') {
                            return '<a href="' . route('company.admin.billing.invoice.show', ['id' => base64Encode($row->invoice_id)]) . '" target="_blank">' . $row->invoice_number . '</a>';
                        }
                        return '';
                    })
                    ->filterColumn('broker_name', function ($query, $keyword) {
                        $query->whereHas('broker', function ($q) use ($keyword) {
                            $q->where("brokers.name", 'LIKE', "%" . $keyword . "%");
                        });
                    })
                    ->editColumn('shipper_location', function ($row) {
                        $html = [];
                        if ($row->shipper_address_one) {
                            $html[0]    = $row->shipper_address_one;
                        }
                        if ($row->shipper_address_two) {
                            $html[1]    = $row->shipper_address_two;
                        }
                        if ($row->shipper_city) {
                            $html[2]    = $row->shipper_city;
                        }
                        if ($row->shipper_state) {
                            $html[3]    = $row->shipper_state;
                        }
                        if ($row->shipper_zip) {
                            $html[4]    = $row->shipper_zip;
                        }
                        if (count($html) == 0) {
                            return $row->shipper_location;
                        } else {
                            return implode(', ', $html);
                        }
                    })
                    ->editColumn('actual_shipper_date', function ($row) {
                        return !is_null($row->actual_shipper_date) ? $row->actual_shipper_date . " " . $row->shipper_timezone_abbr : '';
                    })
                    ->editColumn('delivery_location', function ($row) {
                        $html = [];
                        if ($row->delivery_address_one) {
                            $html[0]    = $row->delivery_address_one;
                        }
                        if ($row->delivery_address_two) {
                            $html[1]    = $row->delivery_address_two;
                        }
                        if ($row->delivery_city) {
                            $html[2]    = $row->delivery_city;
                        }
                        if ($row->delivery_state) {
                            $html[3]    = $row->delivery_state;
                        }
                        if ($row->delivery_zip) {
                            $html[4]    = $row->delivery_zip;
                        }
                        if (count($html) == 0) {
                            return $row->delivery_location;
                        } else {
                            return implode(', ', $html);
                        }
                    })
                    ->editColumn('actual_delivery_date', function ($row) {
                        return !is_null($row->actual_delivery_date) ? $row->actual_delivery_date . " " . $row->delivery_timezone_abbr : '';
                    })
                    ->addColumn('status', function ($row) {
                        return getLoadStatus($row->load_status, $row->is_on_hold);
                    })
                    ->filterColumn('status', function ($query, $keyword) {
                        if (Str::contains('pending', Str::lower($keyword))) {
                            $query = $query->where('load_status', 0);
                        }
                        if (Str::contains('accepted', Str::lower($keyword))) {
                            $query = $query->where('load_status', 1);
                        }
                        if (Str::contains('preplaned', Str::lower($keyword))) {
                            $query = $query->where('load_status', 2);
                        }
                        if (Str::contains('confirmed', Str::lower($keyword))) {
                            $query = $query->where('load_status', 3);
                        }
                        if (Str::contains('in transit', Str::lower($keyword))) {
                            $query = $query->where('load_status', 4);
                        }
                        if (Str::contains('delivered', Str::lower($keyword))) {
                            $query = $query->where('load_status', 5);
                        }
                        if (Str::contains('ready to bill', Str::lower($keyword))) {
                            $query = $query->where('load_status', 6);
                        }
                        if (Str::contains('on hold', Str::lower($keyword))) {
                            $query = $query->where('load_status', 7);
                        }
                        if (Str::contains('complete', Str::lower($keyword))) {
                            $query = $query->where('load_status', 8);
                        }
                        if (Str::contains('cancelled', Str::lower($keyword))) {
                            $query = $query->where('load_status', 9);
                        }
                        return $query;
                    })
                    ->editColumn('total_bill', function ($row) {
                        return getPrice($row->total_bill);
                    })
                    ->addColumn('factor_invoices', function ($row) {
                        return (!is_null($row->broker_factor_invoices) && $row->broker_factor_invoices == '1') ? 'Factored' : 'Direct Bill';
                    })
                    ->addColumn('documents', function ($row) {
                        if (!is_null($row->load_billing_activities) && $row->load_billing_activities->paperwork_received == '1') {
                            return '<i class="fa fa-file-text-o document-complete document-icon" aria-hidden="true"></i>';
                        }
                        return '<i class="fa fa-file-text-o document-in-review document-icon" aria-hidden="true"></i>';
                    })
                    ->addColumn('review', function ($row) {
                        if (!is_null($row->load_billing_activities) && $row->load_billing_activities->audit_complete == '1') {
                            return 'Complete';
                        }
                        return 'In Review';
                    })
                    ->addColumn('invoice_now', function ($row) {
                        if (!is_null($row->load_billing_activities) && $row->load_billing_activities->paperwork_received == '1' && $row->load_billing_activities->audit_complete == '1') {
                            if ($row->load_billing_activities->paid_invoice == '1') {
                                return '<span class="text-green">Paid</span>';
                            } else if ($row->load_billing_activities->invoiced == '1') {
                                return '<span class="text-green">Invoiced <i class="fa fa-reply" aria-hidden="true"></i></span>';
                            }
                            return '<a href="' . route('company.admin.billing.invoice.create', ['id' => base64Encode($row->id)]) . '" class="btn btn-success">Invoice Now</a>';
                        }
                        return '<a href="javascript:void(0);" class="btn btn-liteGray">Invoice Now</a>';
                    })
                    ->addColumn('child_content', function ($row) {
                        return $this->childContentArray($row);
                    })
                    ->rawColumns(['audit', 'load_number', 'invoice_number', 'type', 'documents', 'review', 'invoice_now', 'child_content'])
                    ->make(true);
            }
        } catch (\Exception $e) {
            \Log::emergency("File: " . $e->getFile());
            \Log::emergency("Line: " . $e->getLine());
            \Log::emergency("Message: " . $e->getMessage());
        }
    }

    public function updateForReceivePayment($loadDetails)
    {
        $paidInvoiceAmount = 0;

        if (isset($loadDetails->load_billing_pay_invoices)) {
            $paidInvoiceTotal = $loadDetails->load_billing_pay_invoices->sum('total_payment') ?? 0;
            $chargebackAgainstBalance = $loadDetails->load_billing_pay_invoices->pluck('chargeback_against_balance');
            $chargebackAmountsData = $loadDetails->load_billing_pay_invoices->pluck('chargeback_amount');
            $chargebackAmounts = array_sum($chargebackAmountsData->map(function ($values, $key) use ($chargebackAgainstBalance) {
                if (count($values) > 0) {
                    $amounts = [];
                    foreach ($values as $k => $v) {
                        if (isset($chargebackAgainstBalance[$key][$k]) && $chargebackAgainstBalance[$key][$k] == '1') {
                            $amounts[] = $v;
                        }
                    }
                    return array_sum($amounts);
                }
                return 0;
            })->toArray());

            $paidInvoiceAmount = round(($paidInvoiceTotal + $chargebackAmounts), 2);

            /* Update Invoice Paid Status - START */
            if (isset($loadDetails->load_billing_activities) && $loadDetails->load_billing_activities->paid_invoice == '0') {
                $loadInvoice = $loadDetails->load_billing_invoice;
                if (isset($loadInvoice)) {
                    $invoiceAmount = $loadInvoice->invoice_amount ?? 0.00;

                    if ($invoiceAmount === $paidInvoiceAmount) {
                        $loadInvoice->paid_date = Carbon::now()->setTimezone(getTimeZone())->format('m/d/Y H:i:s');
                        $loadInvoice->save();

                        // Load move to Paid Status
                        $oldData = $loadDetails;
                        $loadDetails->load_status = 8;
                        $loadDetails->load_status_order = Common::getLoadStatusOrderById(8);
                        $loadDetails->updated_at = date('Y-m-d H:i:s');
                        $loadDetails->save();

                        $this->setLoadAudit($loadDetails->load_id, 'update', 'Load detail updated', $loadDetails->getChanges(), $oldData);

                        // Update load_billing_activities
                        $loadDetails->load_billing_activities->update([
                            'paid_invoice' => '1',
                            'paid_invoice_date_time' => Carbon::now()->setTimezone(getTimeZone()),
                            'paid_invoice_user_type' => auth()->guard('company-admin')->user()->type,
                            'paid_invoice_user_by' => auth()->guard('company-admin')->user()->id,
                        ]);
                    }
                }
            }
            /* Update Invoice Paid Status - End */
        }

        return $paidInvoiceAmount;
    }

    public function getRowChildView($rowData)
    {
        $rowData = json_decode($rowData);

        $loadDetails = LoadDetail::where('id', $rowData->load_detail_id)->first();

        if ($loadDetails) {
            $paidInvoiceAmount = $this->updateForReceivePayment($loadDetails);

            $rowData->delivery_date_time = ($loadDetails->delivery_date_time != '') ? Carbon::parse($loadDetails->delivery_date_time)->format('m/d/Y') : null;
            $rowData->paperwork_received_date_time = (isset($loadDetails->load_billing_activities) && $loadDetails->load_billing_activities->paperwork_received_date_time != '') ? Carbon::parse($loadDetails->load_billing_activities->paperwork_received_date_time)->format('m/d/Y') : null;
            $rowData->audit_complete_date_time = (isset($loadDetails->load_billing_activities) && $loadDetails->load_billing_activities->audit_complete_date_time != '') ? Carbon::parse($loadDetails->load_billing_activities->audit_complete_date_time)->format('m/d/Y') : null;
            $rowData->invoiced_date_time = (isset($loadDetails->load_billing_activities) && $loadDetails->load_billing_activities->invoiced_date_time != '') ? Carbon::parse($loadDetails->load_billing_activities->invoiced_date_time)->format('m/d/Y') : null;
            $rowData->paid_invoice_date_time = (isset($loadDetails->load_billing_activities) && $loadDetails->load_billing_activities->paid_invoice_date_time != '') ? Carbon::parse($loadDetails->load_billing_activities->paid_invoice_date_time)->format('m/d/Y') : null;
            $rowData->factor_invoices = (isset($loadDetails->broker) && $loadDetails->broker->factor_invoices != '') ? $loadDetails->broker->factor_invoices : null;

            $loadDocuments = getLoadDocuments($loadDetails);

            return view('companyAdmin.billing.loadChild', compact('rowData', 'loadDetails', 'loadDocuments', 'paidInvoiceAmount'))->render();
        }
        return '';
    }

    public function loadChild(Request $request)
    {
        $view = $this->getRowChildView($request->rowData);

        if ($view != '') {
            return response()->json(['success' => true, 'childHtml' => $view]);
        } else {
            return response()->json(['success' => false, 'childHtml' => '']);
        }
    }

    public function auditLoadUnlock()
    {
        // Reset All Audit Loads
        LoadDetail::where('billing_load_audit_by_id', auth()->guard('company-admin')->user()->id)
            ->update([
                'billing_load_is_audit' => '0',
                'billing_load_audit_by_id' => null,
                'billing_load_audit_by_name' => null
            ]);

        return response()->json(['success' => true]);
    }

    public function auditLoadView(Request $request)
    {
        $input = $request->all();

        $load_detail = LoadDetail::where('load_id', $input['load'])->where('id', $input['load_details'])->first();

        if (isset($load_detail)) {
            if ($load_detail->billing_load_is_audit == '1' && $load_detail->billing_load_audit_by_id != auth()->guard('company-admin')->user()->id) {
                return response()->json(['success' => false, 'message' => 'This load is locked by ' . $load_detail->billing_load_audit_by_name . ' for audit few seconds ago.!!', 'reload' => 1]);
            } else {
                // Reset All Audit Loads
                LoadDetail::where('billing_load_audit_by_id', auth()->guard('company-admin')->user()->id)->update(['billing_load_is_audit' => '0', 'billing_load_audit_by_id' => null, 'billing_load_audit_by_name' => null]);

                $load_detail->billing_load_is_audit = '1';
                $load_detail->billing_load_audit_by_id = auth()->guard('company-admin')->user()->id;
                $load_detail->billing_load_audit_by_name = auth()->guard('company-admin')->user()->full_name;
                $load_detail->save();

                $loadDocuments = getLoadDocuments($load_detail);

                $view = view('companyAdmin.billing.auditLoadView', compact('load_detail', 'loadDocuments'))->render();

                return response()->json(['success' => true, 'auditLoadView' => $view]);
            }
        } else {
            return response()->json(['success' => false, 'message' => 'Load Details not found.']);
        }
    }

    public function loadDocumentsDataUpdate(Request $request)
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $formData = $request->all();

                $fileVerifyType = '';
                if (isset($formData['file_verify']) && $formData['file_verify'] == '1') {
                    $fileVerifyType = 'Approved';
                } else if (isset($formData['file_verify']) && $formData['file_verify'] == '2') {
                    $fileVerifyType = 'Rejected';
                }

                if (isset($formData['jsonData']) && $formData['jsonData'] !== '') {
                    $jsonData = json_decode($formData['jsonData'], true);
                    $oldJsonData = $jsonData;

                    if (isset($jsonData['source'])) {
                        if (in_array($jsonData['source'], ['load_details', 'load_stops'])) {
                            if ($jsonData['source'] == 'load_details') {
                                $item = LoadDetail::whereId($jsonData['id'])->first();
                                if (!is_null($item)) {
                                    if ($jsonData['field_name'] == 'shipper_files_upload') {
                                        $getArray = isset($item->shipper_files_upload) && is_serialized($item->shipper_files_upload) && !empty(unserialize($item->shipper_files_upload)) ? unserialize($item->shipper_files_upload) : [];
                                    } else if ($jsonData['field_name'] == 'consignee_files_upload') {
                                        $getArray = isset($item->consignee_files_upload) && is_serialized($item->consignee_files_upload) && !empty(unserialize($item->consignee_files_upload)) ? unserialize($item->consignee_files_upload) : [];
                                    }
                                }
                            } else {
                                $item = LoadStop::whereId($jsonData['id'])->first();
                                if (!is_null($item)) {
                                    if ($jsonData['field_name'] == 'stop_shipper_files_upload') {
                                        $getArray = isset($item->stop_shipper_files_upload) && is_serialized($item->stop_shipper_files_upload) && !empty(unserialize($item->stop_shipper_files_upload)) ? unserialize($item->stop_shipper_files_upload) : [];
                                    } else if ($jsonData['field_name'] == 'stop_consignee_files_upload') {
                                        $getArray = isset($item->stop_consignee_files_upload) && is_serialized($item->stop_consignee_files_upload) && !empty(unserialize($item->stop_consignee_files_upload)) ? unserialize($item->stop_consignee_files_upload) : [];
                                    }
                                }
                            }

                            if (!empty($getArray)) {
                                $saveArray = [];
                                foreach ($getArray as $data) {
                                    if ((isset($data['name_id']) && $data['name_id'] == $jsonData['filename_id']) && (isset($data['name']) && $data['name'] == $jsonData['filename']) && (isset($data['type']) && $data['type'] == $jsonData['type'])) {
                                        if (isset($formData['is_received'])) {
                                            if ($formData['is_received'] == 'false') {
                                                $data['is_received']                    = 'off';
                                                $data['file_not_provided']              = 'off';
                                                $data['history']                        = null;

                                                if (isset($data['upload_user_type'])) {
                                                    unset($data['upload_user_type']);
                                                }
                                                if (isset($data['upload_by'])) {
                                                    unset($data['upload_by']);
                                                }
                                                if (isset($data['uploaded_at'])) {
                                                    unset($data['uploaded_at']);
                                                }
                                            } else {
                                                $data['is_received']                    = 'on';
                                                $data['file_not_provided']              = 'off';

                                                $history                                = isset($data['history']) && is_array($data['history']) && !empty($data['history']) ? $data['history'] : [];
                                                $hv                                     = [];
                                                $hv['uploaded_at']                      = \Carbon\Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                                $hv['upload_by']                        = auth()->guard('company-admin')->user()->id;
                                                $hv['upload_user_type']                 = auth()->guard('company-admin')->user()->type; // Helper::getNotiUserTypeList();
                                                array_push($history, $hv);
                                                $data['history']                        = $history;

                                                $data['upload_user_type']               = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                                $data['upload_by']                      = auth()->guard('company-admin')->user()->id;
                                                $data['uploaded_at']                    = Carbon::now();

                                                if (in_array($data['file_verify'], ['1', '2'])) {
                                                    $data['file_verify'] = '0';
                                                    if (isset($data['file_verify_user_type'])) {
                                                        unset($data['file_verify_user_type']);
                                                    }
                                                    if (isset($data['file_verify_by'])) {
                                                        unset($data['file_verify_by']);
                                                    }
                                                    if (isset($data['file_verified_at'])) {
                                                        unset($data['file_verified_at']);
                                                    }
                                                }
                                            }
                                        } else if (isset($formData['files']) && !empty($formData['files'])) {
                                            $files = isset($data['file']) && !empty($data['file']) ? $data['file'] : null;
                                            if (isset($files) && !empty($files)) {
                                                foreach ($files as $key => $file) {
                                                    $path = 'storage/companyAdmin/load_detail_document/' . $file;
                                                    if (File::exists($path)) {
                                                        File::delete($path);
                                                    }
                                                }
                                            }
                                            $data['is_received']                    = 'on';
                                            $data['file_not_provided']              = 'off';

                                            $history                                = isset($data['history']) && is_array($data['history']) && !empty($data['history']) ? $data['history'] : [];
                                            $hv                                     = [];
                                            $hv['uploaded_at']                      = \Carbon\Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                            $hv['upload_by']                        = auth()->guard('company-admin')->user()->id;
                                            $hv['upload_user_type']                 = auth()->guard('company-admin')->user()->type; // Helper::getNotiUserTypeList();
                                            array_push($history, $hv);
                                            $data['include_w_billing']              = isset($formData['include_w_billing']) && $formData['include_w_billing'] == 'yes' ? 'yes' : 'no';
                                            $data['history']                        = $history;

                                            $data['file']                           = UploadImagePdf($formData['files'], 'companyAdmin/load_detail_document');
                                            $data['upload_user_type']               = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                            $data['upload_by']                      = auth()->guard('company-admin')->user()->id;
                                            $data['uploaded_at']                    = Carbon::now();

                                            if (in_array($data['file_verify'], ['1', '2'])) {
                                                $data['file_verify'] = '0';
                                                if (isset($data['file_verify_user_type'])) {
                                                    unset($data['file_verify_user_type']);
                                                }
                                                if (isset($data['file_verify_by'])) {
                                                    unset($data['file_verify_by']);
                                                }
                                                if (isset($data['file_verified_at'])) {
                                                    unset($data['file_verified_at']);
                                                }
                                            }
                                        } else if (isset($formData['file_verify']) && $formData['file_verify'] != '0') {
                                            $data['file_verify']                 = $formData['file_verify'];
                                            $data['file_verify_user_type']       = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                            $data['file_verify_by']              = auth()->guard('company-admin')->user()->id;
                                            $data['file_verified_at']            = Carbon::now();
                                        } else if (isset($formData['revert_approval_or_rejection'])) {
                                            $data['file_verify']                 = '0';
                                            if (isset($data['file_verify_user_type'])) {
                                                unset($data['file_verify_user_type']);
                                            }
                                            if (isset($data['file_verify_by'])) {
                                                unset($data['file_verify_by']);
                                            }
                                            if (isset($data['file_verified_at'])) {
                                                unset($data['file_verified_at']);
                                            }
                                        }
                                    }
                                    $saveArray[] = $data;
                                }

                                if (isset($item)) {
                                    if ($jsonData['field_name'] == 'shipper_files_upload') {
                                        $oldData = $item->toArray();
                                        $item->update(['shipper_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load detail updated', $item->getChanges(), $oldData);
                                    }
                                    if ($jsonData['field_name'] == 'consignee_files_upload') {
                                        $oldData = $item->toArray();
                                        $item->update(['consignee_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load detail updated', $item->getChanges(), $oldData);
                                    }
                                    if ($jsonData['field_name'] == 'stop_shipper_files_upload') {
                                        $oldData = $item->toArray();
                                        $item->update(['stop_shipper_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load stop updated', $item->getChanges(), $oldData);
                                    }
                                    if ($jsonData['field_name'] == 'stop_consignee_files_upload') {
                                        $oldData = $item->toArray();
                                        $item->update(['stop_consignee_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load stop updated', $item->getChanges(), $oldData);
                                    }
                                }
                            }
                        } elseif ($jsonData['source'] == 'load_document_request_items') {
                            $load_document_request_item = LoadDocumentRequestItem::where('id', $jsonData['id'])->first();
                            if (!is_null($load_document_request_item)) {
                                $oldData = $load_document_request_item->toArray();

                                $data = [];
                                if (isset($load_document_request_item->files) && is_serialized($load_document_request_item->files) && !empty(unserialize($load_document_request_item->files))) {
                                    $data = unserialize($load_document_request_item->files);
                                }

                                if (!empty($data)) {
                                    if (isset($formData['is_received'])) {
                                        if ($formData['is_received'] == 'false') {
                                            $data['is_received']                    = 'off';
                                            $data['file_not_provided']              = 'off';
                                            $data['history']                        = null;

                                            if (isset($data['upload_user_type'])) {
                                                unset($data['upload_user_type']);
                                            }
                                            if (isset($data['upload_by'])) {
                                                unset($data['upload_by']);
                                            }
                                            if (isset($data['uploaded_at'])) {
                                                unset($data['uploaded_at']);
                                            }
                                        } else {
                                            $data['is_received']                        = 'on';
                                            $data['file_not_provided']                  = 'off';

                                            $history                                    = isset($data['history']) && is_array($data['history']) && !empty($data['history']) ? $data['history'] : [];
                                            $hv                                         = [];
                                            $hv['uploaded_at']                          = \Carbon\Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                            $hv['upload_by']                            = auth()->guard('company-admin')->user()->id;
                                            $hv['upload_user_type']                     = auth()->guard('company-admin')->user()->type; // Helper::getNotiUserTypeList();
                                            array_push($history, $hv);
                                            $data['history']                            = $history;

                                            $data['upload_user_type']                   = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                            $data['upload_by']                          = auth()->guard('company-admin')->user()->id;
                                            $data['uploaded_at']                        = Carbon::now();

                                            if (in_array($data['file_verify'], ['1', '2'])) {
                                                $data['file_verify'] = '0';
                                                if (isset($data['file_verify_user_type'])) {
                                                    unset($data['file_verify_user_type']);
                                                }
                                                if (isset($data['file_verify_by'])) {
                                                    unset($data['file_verify_by']);
                                                }
                                                if (isset($data['file_verified_at'])) {
                                                    unset($data['file_verified_at']);
                                                }
                                            }
                                        }
                                    } else if (isset($formData['files']) && !empty($formData['files'])) {
                                        $files = isset($data['file']) && !empty($data['file']) ? $data['file'] : null;
                                        if (isset($files) && !empty($files)) {
                                            foreach ($files as $key => $file) {
                                                $path = 'storage/companyAdmin/load_detail_document/' . $file;
                                                if (File::exists($path)) {
                                                    File::delete($path);
                                                }
                                            }
                                        }
                                        $data['is_received']                        = 'on';
                                        $data['file_not_provided']                  = 'off';

                                        $history                                    = isset($data['history']) && is_array($data['history']) && !empty($data['history']) ? $data['history'] : [];
                                        $hv                                         = [];
                                        $hv['uploaded_at']                          = \Carbon\Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                        $hv['upload_by']                            = auth()->guard('company-admin')->user()->id;
                                        $hv['upload_user_type']                     = auth()->guard('company-admin')->user()->type; // Helper::getNotiUserTypeList();
                                        array_push($history, $hv);
                                        $data['include_w_billing']                  = isset($formData['include_w_billing']) && $formData['include_w_billing'] == 'yes' ? 'yes' : 'no';
                                        $data['history']                            = $history;

                                        $data['file']                               = UploadImagePdf($formData['files'], 'companyAdmin/load_detail_document');
                                        $data['upload_user_type']                   = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                        $data['upload_by']                          = auth()->guard('company-admin')->user()->id;
                                        $data['uploaded_at']                        = Carbon::now();

                                        if (in_array($data['file_verify'], ['1', '2'])) {
                                            $data['file_verify'] = '0';
                                            if (isset($data['file_verify_user_type'])) {
                                                unset($data['file_verify_user_type']);
                                            }
                                            if (isset($data['file_verify_by'])) {
                                                unset($data['file_verify_by']);
                                            }
                                            if (isset($data['file_verified_at'])) {
                                                unset($data['file_verified_at']);
                                            }
                                        }
                                    } else if (isset($formData['file_verify']) && $formData['file_verify'] != '0') {
                                        $data['file_verify'] = $formData['file_verify'];
                                        $data['file_verify_user_type']       = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                        $data['file_verify_by']              = auth()->guard('company-admin')->user()->id;
                                        $data['file_verified_at']            = Carbon::now();
                                    } else if (isset($formData['revert_approval_or_rejection'])) {
                                        $data['file_verify']                 = '0';
                                        if (isset($data['file_verify_user_type'])) {
                                            unset($data['file_verify_user_type']);
                                        }
                                        if (isset($data['file_verify_by'])) {
                                            unset($data['file_verify_by']);
                                        }
                                        if (isset($data['file_verified_at'])) {
                                            unset($data['file_verified_at']);
                                        }
                                    }
                                }

                                $load_document_request_item->files                  = !empty($data) ? serialize($data) : null;
                                $load_document_request_item->updated_at             = date('Y-m-d H:i:s');
                                $load_document_request_item->updated_by             = auth()->guard('company-admin')->user()->id;
                                $load_document_request_item->updated_by_type        = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                if ($load_document_request_item->save()) {
                                    $this->setLoadAudit($load_document_request_item->load_id, 'update', 'Load document request item updated', $load_document_request_item->getChanges(), $oldData);
                                }
                            }
                        } elseif ($jsonData['source'] == 'load_accessorials') {
                            if (isset($formData['file_verify']) && $formData['file_verify'] != '0') {
                                $load_accessorial = LoadAccessorial::where('id', $jsonData['id'])->first();
                                if (!is_null($load_accessorial)) {
                                    $oldData = $load_accessorial->toArray();

                                    $load_accessorial->file_verify = $formData['file_verify'];
                                    $load_accessorial->file_verify_user_type = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                    $load_accessorial->file_verify_by = auth()->guard('company-admin')->user()->id;
                                    $load_accessorial->file_verified_at = Carbon::now();

                                    if ($load_accessorial->save()) {
                                        $this->setLoadAudit($load_accessorial->load_id, 'update', 'Load accessorial updated', $load_accessorial->getChanges(), $oldData);
                                    }
                                }
                            } else if (isset($formData['revert_approval_or_rejection'])) {
                                $load_accessorial = LoadAccessorial::where('id', $jsonData['id'])->first();
                                if (!is_null($load_accessorial)) {
                                    $oldData = $load_accessorial->toArray();

                                    $load_accessorial->file_verify = '0';
                                    $load_accessorial->file_verify_user_type = null; // For Company Admin User Type
                                    $load_accessorial->file_verify_by = null;
                                    $load_accessorial->file_verified_at = null;

                                    if ($load_accessorial->save()) {
                                        $this->setLoadAudit($load_accessorial->load_id, 'update', 'Load accessorial updated', $load_accessorial->getChanges(), $oldData);
                                    }
                                }
                            } else {
                                $load_accessorial = LoadAccessorial::where('id', $jsonData['id'])->first();
                                if (!is_null($load_accessorial)) {
                                    $oldData = $load_accessorial->toArray();

                                    $gallery = Gallery::where('company_id', auth()->guard('company-admin')->user()->company_id)
                                        ->where('action_id', $load_accessorial->id)
                                        ->where('source', 'load_accessorials')
                                        ->where('field_name', 'accessorial_files')
                                        ->first();

                                    $is_upload = 0;

                                    $imageName = null;
                                    // Invalid file type! Please select only jpg, jpeg, png, gif, or pdf files. - only Accessorial
                                    $fileArray = UploadImagePdf($formData['files'], 'companyAdmin/gallery');
                                    // Get the first element
                                    $firstElement = reset($fileArray);
                                    // Check if $firstElement exists
                                    if (isset($firstElement)) {
                                        // Extract filename and extension
                                        $filenameWithExtension                          = pathinfo($firstElement, PATHINFO_EXTENSION);
                                        if ($filenameWithExtension == 'pdf') {
                                            $imageName                                  = $firstElement;
                                        }
                                    }
                                    if (isset($imageName) && $imageName != '') {
                                        if (!is_null($gallery)) {
                                            $path                                       = 'storage/companyAdmin/gallery/' . $gallery->image;
                                            if (File::exists($path)) {
                                                File::delete($path);
                                            }

                                            $gallery->image                             = $imageName;
                                            $gallery->created_by                        = auth()->guard('company-admin')->user()->id;
                                            $gallery->created_by_type                   = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                            $gallery->uploaded_date                     = Carbon::now()->format('m/d/Y H:i');
                                            if ($gallery->save()) {
                                                $is_upload                              = 1;
                                            }
                                        } else {
                                            $input                                      = [];
                                            $input['company_id']                        = auth()->guard('company-admin')->user()->company_id;
                                            $input['action_id']                         = $load_accessorial->id;
                                            $input['source']                            = 'load_accessorials';
                                            $input['field_name']                        = 'accessorial_files';
                                            $input['file_name']                         = null;
                                            $input['file_type']                         = getRequestDocumentsList('load_accessorials', 'source')['file_type'];
                                            $input['image']                             = $imageName;
                                            //$input['include_w_billing']                 = 'no';
                                            //$input['hide_from_driver']                  = 'no';
                                            $input['status']                            = 'approved';
                                            $input['created_by']                        = auth()->guard('company-admin')->user()->id;
                                            $input['created_by_type']                   = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                            $input['uploaded_date']                     = Carbon::now()->format('m/d/Y H:i');
                                            $gallery                                    = Gallery::create($input);

                                            $is_upload                                  = 1;
                                        }
                                    }

                                    if ($is_upload == 1) {
                                        $history                                        = isset($load_accessorial->history) && is_serialized($load_accessorial->history) && !empty(unserialize($load_accessorial->history)) ? unserialize($load_accessorial->history) : [];
                                        $hv                                             = [];
                                        $hv['uploaded_at']                              = isset($gallery->created_at) ? \Carbon\Carbon::parse($gallery->created_at)->format('Y-m-d H:i:s') : null; // $gallery->getRawOriginal('uploaded_date')
                                        $hv['upload_by']                                = isset($gallery->created_by) ? $gallery->created_by : null;
                                        $hv['upload_user_type']                         = isset($gallery->created_by_type) ? $gallery->created_by_type : null;
                                        array_push($history, $hv);

                                        $load_accessorial->history                      = serialize($history);
                                        $load_accessorial->include_w_billing            = isset($formData['include_w_billing']) && $formData['include_w_billing'] == 'yes' ? 'yes' : 'no';

                                        if (in_array($load_accessorial->file_verify, ['1', '2'])) {
                                            $load_accessorial->file_verify = '0';
                                            $load_accessorial->file_verify_user_type = null;
                                            $load_accessorial->file_verify_by = null;
                                            $load_accessorial->file_verified_at = null;
                                        }

                                        // Load accessorial
                                        $load_accessorial->is_received                  = 1;
                                        $load_accessorial->file_not_provided            = 'off';

                                        //status
                                        $arry                                           = [];
                                        $paid_type                                      = $load_accessorial->paid_type;
                                        $reference                                      = $load_accessorial->reference;
                                        $total                                          = $load_accessorial->total;

                                        $owner_liable                                   = $load_accessorial->owner_liable;
                                        $owner_total                                    = $load_accessorial->owner_total;

                                        $driver_liable                                  = $load_accessorial->driver_liable;
                                        $driver_total                                   = $load_accessorial->driver_total;

                                        $doc_required                                   = $load_accessorial->doc_required;

                                        array_push($arry, $paid_type != '' ? 1 : 0);
                                        array_push($arry, $reference != '' ? 1 : 0);
                                        array_push($arry, $total != '' ? 1 : 0);
                                        if ($owner_liable == '1') {
                                            array_push($arry, $owner_total != '' ? 1 : 0);
                                        }
                                        if ($driver_liable == '1') {
                                            array_push($arry, $driver_total != '' ? 1 : 0);
                                        }

                                        if (in_array(0, $arry)) {
                                            $load_accessorial->status                   = 0; // Pending
                                        } else {
                                            if ($doc_required == '1') {
                                                if ($load_accessorial->is_received == 1) {
                                                    $load_accessorial->status           = 1; // Completed
                                                } else {
                                                    $load_accessorial->status           = 0; // Pending
                                                }
                                            } else {
                                                $load_accessorial->status               = 1; // Completed
                                            }
                                        }
                                        if ($driver_liable == '1' && $load_accessorial->status != 1 && isset($load_accessorial->file_not_provided) && $load_accessorial->file_not_provided == 'on') {
                                            $load_accessorial->status                   = 2; // Driver Bypass
                                        }
                                        //status

                                        if ($load_accessorial->save()) {
                                            $this->setLoadAudit($load_accessorial->load_id, 'update', 'Load accessorial updated', $load_accessorial->getChanges(), $oldData);
                                        }
                                    }
                                }
                            }
                        } elseif ($jsonData['source'] == 'load_files') {
                            if (isset($formData['file_verify']) && $formData['file_verify'] != '0') {
                                $load_file = LoadFile::where('id', $jsonData['id'])->first();
                                if (!is_null($load_file)) {
                                    $oldData = $load_file->toArray();

                                    $load_file->file_verify = $formData['file_verify'];
                                    $load_file->file_verify_user_type = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                    $load_file->file_verify_by = auth()->guard('company-admin')->user()->id;
                                    $load_file->file_verified_at = Carbon::now();

                                    if ($load_file->save()) {
                                        $this->setLoadAudit($load_file->load_id, 'update', 'Load accessorial updated', $load_file->getChanges(), $oldData);
                                    }
                                }
                            } else if (isset($formData['revert_approval_or_rejection'])) {
                                $load_file = LoadFile::where('id', $jsonData['id'])->first();
                                if (!is_null($load_file)) {
                                    $oldData = $load_file->toArray();

                                    $load_file->file_verify = '0';
                                    $load_file->file_verify_user_type = null; // For Company Admin User Type
                                    $load_file->file_verify_by = null;
                                    $load_file->file_verified_at = null;

                                    if ($load_file->save()) {
                                        $this->setLoadAudit($load_file->load_id, 'update', 'Load accessorial updated', $load_file->getChanges(), $oldData);
                                    }
                                }
                            } else {
                                $load_file = LoadFile::where('id', $jsonData['id'])->first();
                                if (!is_null($load_file)) {
                                    $oldData                                            = $load_file->toArray();

                                    $files                                              = [];
                                    if (isset($load_file->document) && is_serialized($load_file->document) && !empty(unserialize($load_file->document))) {
                                        $files                                          = unserialize($load_file->document);
                                    }
                                    if (isset($files) && !empty($files)) {
                                        foreach ($files as $key => $file) {
                                            $path                                       = 'storage/companyAdmin/loadfile/' . $file;
                                            if (File::exists($path)) {
                                                File::delete($path);
                                            }
                                        }
                                    }
                                    $data['file']                                       = UploadImagePdf($formData['files'], 'companyAdmin/loadfile');
                                    $data['upload_user_type']                           = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                    $data['upload_by']                                  = auth()->guard('company-admin')->user()->id;
                                    $data['uploaded_at']                                = \Carbon\Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');

                                    $history                                            = isset($load_file->history) && is_serialized($load_file->history) && !empty(unserialize($load_file->history)) ? unserialize($load_file->history) : [];

                                    $hv                                                 = [];
                                    $hv['upload_user_type']                             = $data['upload_user_type']; // Helper::getNotiUserTypeList();
                                    $hv['upload_by']                                    = $data['upload_by'];
                                    $hv['uploaded_at']                                  = $data['uploaded_at']; // date('Y-m-d H:i:s');
                                    array_push($history, $hv);

                                    $load_file->document                                = serialize($data['file']);
                                    $load_file->history                                 = !empty($history) ? serialize($history) : null;

                                    if (in_array($load_file->file_verify, ['1', '2'])) {
                                        $load_file->file_verify = '0';
                                        $load_file->file_verify_user_type = null;
                                        $load_file->file_verify_by = null;
                                        $load_file->file_verified_at = null;
                                    }

                                    $load_file->upload_by                               = $data['upload_by'];
                                    $load_file->upload_user_type                        = 'Company Admin';
                                    if ($load_file->save()) {
                                        $this->setLoadAudit($load_file->load_id, 'update', 'Load document summary updated', $load_file->getChanges(), $oldData);
                                    }
                                }
                            }
                        }
                    }
                } else if (isset($formData['revert_invoicing']) && $formData['revert_invoicing'] == 'true') {
                    $loadDetail = LoadDetail::whereId($formData['loadDetailsId'])->first();

                    $oldData = $loadDetail->toArray();

                    if (count(getLoadDocuments($loadDetail)['merged_documents']) == 0) {
                        $loadDetail->load_status = 5;
                        $loadDetail->load_status_order = Common::getLoadStatusOrderById(5);
                        $loadDetail->updated_at = date('Y-m-d H:i:s');

                        $loadDetail->load_billing_activities->update([
                            'paperwork_received' => '0',
                            'paperwork_received_date_time' => null,
                            'audit_complete' => '0',
                            'audit_complete_date_time' => null,
                            'audit_complete_user_type' => null,
                            'audit_complete_user_by' => null,
                            'invoiced' => '0',
                            'invoiced_date_time' => null,
                            'invoiced_user_type' => null,
                            'invoiced_user_by' => null,
                            'paid_invoice' => '0',
                            'paid_invoice_date_time' => null,
                            'paid_invoice_user_type' => null,
                            'paid_invoice_user_by' => null,
                        ]);
                    } else {
                        $loadDetail->load_status = 6;
                        $loadDetail->load_status_order = Common::getLoadStatusOrderById(6);
                        $loadDetail->updated_at = date('Y-m-d H:i:s');

                        $loadDetail->load_billing_activities->update([
                            'invoiced' => '0',
                            'invoiced_date_time' => null,
                            'invoiced_user_type' => null,
                            'invoiced_user_by' => null,
                            'paid_invoice' => '0',
                            'paid_invoice_date_time' => null,
                            'paid_invoice_user_type' => null,
                            'paid_invoice_user_by' => null,
                        ]);
                    }

                    $loadDetail->load_billing_invoice->delete();
                    foreach ($loadDetail->load_billing_pay_invoices as $loadBillingPayInvoices) {
                        $loadBillingPayInvoices->delete();
                    }
                    foreach ($loadDetail->load_billing_histories as $loadBillingHistories) {
                        $loadBillingHistories->delete();
                    }

                    if ($loadDetail->save()) {
                        $this->setLoadAudit($loadDetail->load_id, 'update', 'Load detail updated', $loadDetail->getChanges(), $oldData);
                    }
                }

                $load_detail = LoadDetail::whereId($formData['loadDetailsId'])->first();
                $loadDocuments = getLoadDocuments($load_detail);

                if (isset($formData['is_child_row']) && $formData['is_child_row'] == 'true') {
                    $response['rowChildView'] = $this->getRowChildView(json_encode(['load_id' => $load_detail->load_id, 'load_detail_id' => $load_detail->id, 'rowIndex' => $formData['parentRowIndex']])) ?? '';

                    if (isset($load_detail->load_billing_activities) && $load_detail->load_billing_activities->audit_complete == '1') {
                        $response['billStatus'] = 'audit-completed';
                    } else if (isset($load_detail->load_billing_activities) && $load_detail->load_billing_activities->invoiced == '1') {
                        $response['billStatus'] = 'invoiced';
                    } else {
                        $response['billStatus'] = '';
                    }
                } else if (isset($formData['is_from_review']) && $formData['is_from_review'] == 'true') {
                    /* Billing Activity View */
                    $loadDetail = $load_detail;
                    $billingActivityView = view('companyAdmin.billing.activity', compact('loadDetail'))->render();
                    /* Billing Activity View */

                    /* Load Documents View */
                    $loadDocumentsView = view('companyAdmin.billing.load-documents-list', compact('loadDocuments'))->render();
                    /* Load Documents View */

                    $response['billingActivityView'] = $billingActivityView ?? '';
                    $response['loadDocumentsView'] = $loadDocumentsView ?? '';
                } else {
                    $view = view('companyAdmin.billing.auditLoadView', compact('load_detail', 'loadDocuments'))->render();
                    $response['auditLoadView'] = $view ?? '';
                }

                $response['success'] = true;
                if ($fileVerifyType != '') {
                    $response['message'] = "File " . $fileVerifyType;
                } else {
                    $response['message'] = "Save successfully.";
                }
            } catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }

        return response()->json($response);
    }

    public function loadRequestDocument(Request $request)
    {
        $response['success']                                                        = false;
        $response['message']                                                        = "Oops! Something went wrong..";
        if ($request->ajax()) {
            try {
                $formData                                                           = $request->all();

                if (isset($formData['jsonData']) && $formData['jsonData'] !== '') {
                    $jsonData                                                       = json_decode($formData['jsonData'], true);

                    $response['html']                                               = view('companyAdmin.billing.requestDocument', compact('formData', 'jsonData'))->render();
                    $response['success']                                            = true;
                    $response['message']                                            = "Request Document view retrieved successfully.";
                }
            } catch (\Exception $e) {
                $response['success']                                                = false;
                $response['message']                                                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function loadRequestDocumentSave(Request $request)
    {
        $response['success']                                                        = false;
        $response['message']                                                        = "Oops! Something went wrong..";
        if ($request->ajax()) {
            try {
                $input                                                              = $request->all();

                if (isset($input['is_from_review']) && $input['is_from_review'] == 'true') {
                    $inputJsonData = json_decode($input['jsonData'], true);

                    $input['stop_no'] = ($inputJsonData['stop']['name'] . ': ' . $inputJsonData['stop']['addresses']) ?? '';
                    $input['driver_id'] = $inputJsonData['stop']['driver_id'] ?? '';
                    $input['name'] = $inputJsonData['filename'] ?? '';
                    $input['request_via'] = '1';
                    $input['instructions'] = $input['notes'] ?? '';
                    $input['include_w_billing'] = $inputJsonData['include_w_billing'] ?? 'yes';
                    $formData = $input;
                } else {
                    $formData = json_decode($input['params'], true);
                }

                if (!empty($formData) && isset($formData['jsonData']) && $formData['jsonData'] !== '') {
                    $jsonData                                                       = json_decode($formData['jsonData'], true);
                    $oldJsonData                                                    = $jsonData;

                    if (isset($jsonData['source'])) {
                        if (in_array($jsonData['source'], ['load_details', 'load_stops'])) {
                            $getArray                                               = [];
                            if ($jsonData['source'] == 'load_details') {
                                $item                                               = LoadDetail::whereId($jsonData['id'])->first();
                                if (!is_null($item)) {
                                    if ($jsonData['field_name'] == 'shipper_files_upload') {
                                        $getArray                                   = isset($item->shipper_files_upload) && is_serialized($item->shipper_files_upload) && !empty(unserialize($item->shipper_files_upload)) ? unserialize($item->shipper_files_upload) : [];
                                    } else if ($jsonData['field_name'] == 'consignee_files_upload') {
                                        $getArray                                       = isset($item->consignee_files_upload) && is_serialized($item->consignee_files_upload) && !empty(unserialize($item->consignee_files_upload)) ? unserialize($item->consignee_files_upload) : [];
                                    }
                                }
                            } else {
                                $item                                               = LoadStop::whereId($jsonData['id'])->first();
                                if (!is_null($item)) {
                                    if ($jsonData['field_name'] == 'stop_shipper_files_upload') {
                                        $getArray                                   = isset($item->stop_shipper_files_upload) && is_serialized($item->stop_shipper_files_upload) && !empty(unserialize($item->stop_shipper_files_upload)) ? unserialize($item->stop_shipper_files_upload) : [];
                                    } else if ($jsonData['field_name'] == 'stop_consignee_files_upload') {
                                        $getArray                                   = isset($item->stop_consignee_files_upload) && is_serialized($item->stop_consignee_files_upload) && !empty(unserialize($item->stop_consignee_files_upload)) ? unserialize($item->stop_consignee_files_upload) : [];
                                    }
                                }
                            }

                            if (!empty($getArray)) {
                                $saveArray                                          = [];
                                foreach ($getArray as $data) {
                                    if (
                                        (isset($data['name_id']) && $data['name_id'] == $jsonData['filename_id']) &&
                                        (isset($data['name']) && $data['name'] == $jsonData['filename']) &&
                                        (isset($data['type']) && $data['type'] == $jsonData['type'])
                                    ) {
                                        $history                                    = isset($data['history']) && is_array($data['history']) && !empty($data['history']) ? $data['history'] : [];

                                        $hv                                         = [];
                                        $hv['requested_at']                         = \Carbon\Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                        $hv['requested_by']                         = auth()->guard('company-admin')->user()->id;
                                        $hv['requested_by_type']                    = auth()->guard('company-admin')->user()->type; // Helper::getNotiUserTypeList();
                                        $hv['requested_to']                         = $input['driver_id'];
                                        $hv['request_via']                          = $input['request_via']; // 1 = Email, 2 = SMS
                                        $hv['instructions']                         = $input['instructions'];
                                        array_push($history, $hv);

                                        $data['include_w_billing']                  = isset($input['include_w_billing']) && $input['include_w_billing'] == 'yes' ? 'yes' : 'no';
                                        $data['history']                            = $history;

                                        // Update jsonData
                                        $jsonData['include_w_billing']              = $data['include_w_billing'];
                                        $jsonData['history']                        = $data['history'];
                                    }
                                    $saveArray[]                                    = $data;
                                }

                                if (isset($item)) {
                                    if ($jsonData['field_name'] == 'shipper_files_upload') {
                                        $oldData                                    = $item->toArray();
                                        $item->update(['shipper_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load detail updated', $item->getChanges(), $oldData);
                                    }
                                    if ($jsonData['field_name'] == 'consignee_files_upload') {
                                        $oldData                                    = $item->toArray();
                                        $item->update(['consignee_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load detail updated', $item->getChanges(), $oldData);
                                    }
                                    if ($jsonData['field_name'] == 'stop_shipper_files_upload') {
                                        $oldData                                    = $item->toArray();
                                        $item->update(['stop_shipper_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load stop updated', $item->getChanges(), $oldData);
                                    }
                                    if ($jsonData['field_name'] == 'stop_consignee_files_upload') {
                                        $oldData                                    = $item->toArray();
                                        $item->update(['stop_consignee_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load stop updated', $item->getChanges(), $oldData);
                                    }
                                }
                            }
                        } elseif ($jsonData['source'] == 'load_document_request_items') {
                            $load_document_request_item                             = LoadDocumentRequestItem::where('id', $jsonData['id'])->first();
                            if (!is_null($load_document_request_item)) {
                                $oldData                                            = $load_document_request_item->toArray();

                                $data                                               = [];
                                if (isset($load_document_request_item->files) && is_serialized($load_document_request_item->files) && !empty(unserialize($load_document_request_item->files))) {
                                    $data                                           = unserialize($load_document_request_item->files);
                                }

                                if (!empty($data)) {
                                    $history                                        = isset($data['history']) && is_array($data['history']) && !empty($data['history']) ? $data['history'] : [];

                                    $hv                                             = [];
                                    $hv['requested_at']                             = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                    $hv['requested_by']                             = auth()->guard('company-admin')->user()->id;
                                    $hv['requested_by_type']                        = auth()->guard('company-admin')->user()->type; // Helper::getNotiUserTypeList();
                                    $hv['requested_to']                             = $input['driver_id'];
                                    $hv['request_via']                              = $input['request_via']; // 1 = Email, 2 = SMS
                                    $hv['instructions']                             = $input['instructions'];
                                    array_push($history, $hv);

                                    $data['include_w_billing']                      = isset($input['include_w_billing']) && $input['include_w_billing'] == 'yes' ? 'yes' : 'no';
                                    $data['history']                                = $history;

                                    // Update jsonData
                                    $jsonData['include_w_billing']                  = $data['include_w_billing'];
                                    $jsonData['history']                            = $data['history'];
                                }

                                $load_document_request_item->files                  = !empty($data) ? serialize($data) : null;
                                $load_document_request_item->updated_at             = date('Y-m-d H:i:s');
                                $load_document_request_item->updated_by             = auth()->guard('company-admin')->user()->id;
                                $load_document_request_item->updated_by_type        = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                if ($load_document_request_item->save()) {
                                    $this->setLoadAudit($load_document_request_item->load_id, 'update', 'Load document request item updated', $load_document_request_item->getChanges(), $oldData);
                                }
                            }
                        } elseif ($jsonData['source'] == 'load_accessorials') {
                            $load_accessorial                                       = LoadAccessorial::where('id', $jsonData['id'])->first();
                            if (!is_null($load_accessorial)) {
                                $oldData                                            = $load_accessorial->toArray();

                                $history                                            = isset($load_accessorial->history) && is_serialized($load_accessorial->history) && !empty(unserialize($load_accessorial->history)) ? unserialize($load_accessorial->history) : [];

                                $hv                                                 = [];
                                $hv['requested_at']                                 = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                $hv['requested_by']                                 = auth()->guard('company-admin')->user()->id;
                                $hv['requested_by_type']                            = auth()->guard('company-admin')->user()->type; // Helper::getNotiUserTypeList();
                                $hv['requested_to']                                 = $input['driver_id'];
                                $hv['request_via']                                  = $input['request_via']; // 1 = Email, 2 = SMS
                                $hv['instructions']                                 = $input['instructions'];
                                array_push($history, $hv);

                                $load_accessorial->include_w_billing                = isset($input['include_w_billing']) && $input['include_w_billing'] == 'yes' ? 'yes' : 'no';
                                $load_accessorial->history                          = serialize($history);

                                // Update jsonData
                                $jsonData['include_w_billing']                      = $load_accessorial->include_w_billing;
                                $jsonData['history']                                = $history;

                                if ($load_accessorial->save()) {
                                    $this->setLoadAudit($load_accessorial->load_id, 'update', 'Load accessorial updated', $load_accessorial->getChanges(), $oldData);
                                }
                            }
                        } elseif ($jsonData['source'] == 'load_files') {
                            $load_file                                              = LoadFile::where('id', $jsonData['id'])->first();
                            if (!is_null($load_file)) {
                                $oldData                                            = $load_file->toArray();

                                $history                                            = isset($load_file->history) && is_serialized($load_file->history) && !empty(unserialize($load_file->history)) ? unserialize($load_file->history) : [];

                                $hv                                                 = [];
                                $hv['requested_at']                                 = \Carbon\Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                $hv['requested_by']                                 = auth()->guard('company-admin')->user()->id;
                                $hv['requested_by_type']                            = auth()->guard('company-admin')->user()->type; // Helper::getNotiUserTypeList();
                                $hv['requested_to']                                 = $input['driver_id'];
                                $hv['request_via']                                  = $input['request_via']; // 1 = Email, 2 = SMS
                                $hv['instructions']                                 = $input['instructions'];
                                array_push($history, $hv);

                                $load_file->include_w_billing                       = isset($input['include_w_billing']) && $input['include_w_billing'] == 'yes' ? 'yes' : 'no';
                                $load_file->history                                 = serialize($history);

                                // Update jsonData
                                $jsonData['include_w_billing']                      = $load_file->include_w_billing;
                                $jsonData['history']                                = $history;

                                if ($load_file->save()) {
                                    $this->setLoadAudit($load_file->load_id, 'update', 'Load document summary updated', $load_file->getChanges(), $oldData);
                                }
                            }
                        }
                    }

                    $load_detail = LoadDetail::whereId($formData['loadDetailsId'])->first();

                    // Send Email & SMS
                    $driver_id                                                      = $input['driver_id'];
                    $driver                                                         = Driver::whereId($driver_id)->first();
                    if (!is_null($load_detail) && !is_null($driver)) {
                        $driver_full_name                                           = isset($driver->full_name) ? $driver->full_name : '';

                        $details                                                    = [];
                        $details['company_id']                                      = auth()->guard('company-admin')->user()->company_id;
                        $details['full_name']                                       = $driver_full_name;
                        $details['load_id']                                         = $load_detail->load_id;
                        $details['load_detail_id']                                  = $load_detail->id;
                        $details['load_number']                                     = $load_detail->load_type == 0 || $load_detail->load_type == 6 ? $load_detail->load_number : getLoadTypeList()[$load_detail->load_type];

                        $stop_name                                                  = explode(':', $input['stop_no'])[0] ?? '';
                        $stop_name                                                  .= $input['name'] ? ' - ' . $input['name'] : '';
                        $documents[]                                                = $stop_name;
                        $details['documents']                                       = $documents;

                        if (isset($input['request_via'])) {
                            if ($input['request_via'] == 1 && isset($driver->email) && $driver->email != '') {
                                // 1 = Email
                                sendEmail($driver->email, new \App\Mail\LoadDocumentRequestDriver($details));
                            } elseif ($input['request_via'] == 2 && isset($driver->cell_phone) && $driver->cell_phone != '') {
                                // 2 = SMS
                                // Twilio's credentials are fetched from company table
                                $getCompanyTwilioData = getCompanyTwilioData();
                                if (isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes') {
                                    sendSMS($driver->cell_phone, 'A document request has been added to load number: ' . $details['load_number'] . '. Documents requested are: ' . implode(', ', $details['documents']) . '. Please upload these documents to the load. ' . route('driver.admin.loads.show', base64Encode($details['load_detail_id'])), $getCompanyTwilioData);
                                }
                            }
                        }
                    }
                    // Send Email & SMS

                    $loadDocuments = getLoadDocuments($load_detail);

                    if (isset($formData['is_child_row']) && $formData['is_child_row'] == 'true') {
                        $response['rowChildView'] = $this->getRowChildView(json_encode(['load_id' => $load_detail->load_id, 'load_detail_id' => $load_detail->id, 'rowIndex' => $formData['parentRowIndex']])) ?? '';
                    } else if (isset($formData['is_from_review']) && $formData['is_from_review'] == 'true') {
                        /* Billing Activity View */
                        $loadDetail = $load_detail;
                        $billingActivityView = view('companyAdmin.billing.activity', compact('loadDetail'))->render();
                        /* Billing Activity View */

                        /* Load Documents View */
                        $loadDocumentsView = view('companyAdmin.billing.load-documents-list', compact('loadDocuments'))->render();
                        /* Load Documents View */

                        $response['billingActivityView'] = $billingActivityView ?? '';
                        $response['loadDocumentsView'] = $loadDocumentsView ?? '';
                    } else {
                        $view = view('companyAdmin.billing.auditLoadView', compact('load_detail', 'loadDocuments'))->render();
                        $response['auditLoadView'] = $view;
                    }

                    $response['success']                                            = true;
                    $response['message']                                            = "Save successfully.";
                }
            } catch (\Exception $e) {
                $response['success']                                                = false;
                $response['message']                                                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function loadRequestDocumentHistory(Request $request)
    {
        $response['success']                                                        = false;
        $response['message']                                                        = "Oops! Something went wrong..";
        if ($request->ajax()) {
            try {
                $formData                                                           = $request->all();
                if (isset($formData['jsonData']) && $formData['jsonData'] !== '') {
                    $jsonData                                                       = json_decode($formData['jsonData'], true);

                    $history                                                        = isset($jsonData['history']) && is_array($jsonData['history']) && !empty($jsonData['history']) ? $jsonData['history'] : [];

                    $html                                                           = '';
                    $i                                                              = 1;
                    if (!empty($history)) {
                        foreach ($history as $key => $hvalue) {
                            if (isset($hvalue['uploaded_at'])) {
                                $uploaded_at                                                = $jsonData['uploaded_at'] ?? '------';
                                $uploaded_by                                                = $jsonData['uploaded_by'] ?? '------';
                                $upload_type                                                = $jsonData['type'] == '1' ? 'Upload' : 'Marked Received';

                                $html                                                       .= '<tr>';
                                $html                                                       .= '<td>' . $i . '</td>';
                                $html                                                       .= '<td>------</td>';
                                $html                                                       .= '<td>' . $uploaded_at . '</td>';
                                $html                                                       .= '<td>------</td>';
                                $html                                                       .= '<td>------</td>';
                                $html                                                       .= '<td>' . $uploaded_by . '</td>';
                                $html                                                       .= '<td>' . $upload_type . '</td>';
                                $html                                                       .= '</tr>';
                            } else if (isset($hvalue['requested_at'])) {
                                $requested_at                                           = isset($hvalue['requested_at']) ? \Carbon\Carbon::createFromFormat('Y-m-d H:i:s', $hvalue['requested_at'])->setTimezone($this->timeZone)->format(getTimeLayout()) . " " . $this->timeZoneId : '------';

                                $requested_by                                           = CompanyAdmin::whereId($hvalue['requested_by'])->first();
                                $requested_by_full_name                                 = '------';
                                if (!is_null($requested_by)) {
                                    $requested_by_full_name                             = $requested_by->full_name;
                                }

                                $requested_to                                           = Driver::whereId($hvalue['requested_to'])->first();
                                $requested_to_full_name                                 = [];
                                if (!is_null($requested_to)) {
                                    if (isset($requested_to->last_name) && $requested_to->last_name != '') {
                                        $requested_to_full_name[]                       = $requested_to->last_name;
                                    }
                                    if (isset($requested_to->first_name) && $requested_to->first_name != '') {
                                        $requested_to_full_name[]                       = $requested_to->first_name;
                                    }
                                }
                                $requested_to_full_name                                 = !empty($requested_to_full_name) ? implode(', ', $requested_to_full_name) : '------';

                                $html                                                   .= '<tr>';
                                $html                                                   .= '<td>' . $i . '</td>';
                                $html                                                   .= '<td>' . $requested_at . '</td>';
                                $html                                                   .= '<td>------</td>';
                                $html                                                   .= '<td>' . $requested_by_full_name . '</td>';
                                $html                                                   .= '<td>' . $requested_to_full_name . '</td>';
                                $html                                                   .= '<td>------</td>';
                                $html                                                   .= '<td>------</td>';
                                $html                                                   .= '</tr>';
                            }
                            $i++;
                        }
                    }
                    if ($html == '') {
                        $html                                                       .= '<tr><td colspan="7">No data available in table</td></tr>';
                    }

                    $response['html']                                               = $html;
                    $response['success']                                            = true;
                    $response['message']                                            = "History Retrieved successfully.";
                }
            } catch (\Exception $e) {
                $response['success']                                                = false;
                $response['message']                                                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function loadAddDocumentation(Request $request)
    {
        $response['success']                                                        = false;
        $response['message']                                                        = "Oops! Something went wrong..";
        if ($request->ajax()) {
            try {
                $formData                                                           = $request->all();

                $load_detail                                                        = LoadDetail::whereId($formData['loadDetailsId'])->first();

                if (!is_null($load_detail)) {
                    // Please compare with load view
                    $params                                                         = [
                        'load_id'                                                   => $load_detail->load_id,
                        'load_detail_id'                                            => $load_detail->id,
                        'broker_id'                                                 => $load_detail->brokerage_firm
                    ];

                    // Load stop list
                    $stopNo                                                         = 1;
                    $stopsArry                                                      = [];
                    $stopsArry[]                                                    = [
                        'stop_no'                                                   => $stopNo,
                        'load_id'                                                   => $load_detail->load_id,
                        'load_detail_id'                                            => $load_detail->id,
                        'load_stop_id'                                              => null,
                        'type'                                                      => 1, // pickup
                        'is_stop'                                                   => 0,
                        'customer_name'                                             => $load_detail->shipper->name ?? '',
                        'address_one'                                               => $load_detail->shipper_address_one ?? null,
                        'address_one'                                               => $load_detail->shipper_address_one ?? null,
                        'state'                                                     => $load_detail->shipper_state ?? null,
                        'city'                                                      => $load_detail->shipper_city ?? null,
                        'zip'                                                       => $load_detail->shipper_zip ?? null
                    ];
                    if (isset($load_detail->loads->load_stops) && $load_detail->loads->load_stops->isNotEmpty()) {
                        foreach ($load_detail->loads->load_stops as $key => $stop) {
                            ++$stopNo;
                            $stopsArry[]                                            = [
                                'stop_no'                                           => $stopNo,
                                'load_id'                                           => $load_detail->load_id,
                                'load_detail_id'                                    => $load_detail->id,
                                'load_stop_id'                                      => $stop->id,
                                'type'                                              => $stop->customer_type, // stop
                                'is_stop'                                           => 1,
                                'customer_name'                                     => $stop->customer->name ?? '',
                                'address_one'                                       => $stop->address_one ?? null,
                                'state'                                             => $stop->state ?? null,
                                'city'                                              => $stop->city ?? null,
                                'zip'                                               => $stop->zip ?? null
                            ];
                        }
                    }
                    $stopsArry[]                                                    = [
                        'stop_no'                                                   => 99,
                        'load_id'                                                   => $load_detail->load_id,
                        'load_detail_id'                                            => $load_detail->id,
                        'load_stop_id'                                              => null,
                        'type'                                                      => 2, // delivery
                        'is_stop'                                                   => 0,
                        'customer_name'                                             => $load_detail->receiver->name ?? '',
                        'address_one'                                               => $load_detail->delivery_address_one ?? null,
                        'state'                                                     => $load_detail->delivery_state ?? null,
                        'city'                                                      => $load_detail->delivery_city ?? null,
                        'zip'                                                       => $load_detail->delivery_zip ?? null
                    ];
                    $params['load_stops_array']                                     = $stopsArry;
                    // Load stop list

                    // Load driver list
                    $load_drivers_array                                             = [];
                    if (isset($load_detail->driver)) {
                        $driver                                                     = $load_detail->driver;
                        $driver_full_name                                           = [];
                        if (isset($driver->last_name) && $driver->last_name != '') {
                            $driver_full_name[]                                     = $driver->last_name;
                        }
                        if (isset($driver->first_name) && $driver->first_name != '') {
                            $driver_full_name[]                                     = $driver->first_name;
                        }
                        $driver_full_name                                           = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';
                        $load_drivers_array[$driver->id]                            = [
                            'name'                  => $driver_full_name,
                            'email'                 => $driver->email,
                            'cell_phone'            => $driver->cell_phone,
                        ];
                    }
                    if (isset($load_detail->co_driver)) {
                        $driver                                                     = $load_detail->co_driver;
                        $driver_full_name                                           = [];
                        if (isset($driver->last_name) && $driver->last_name != '') {
                            $driver_full_name[]                                     = $driver->last_name;
                        }
                        if (isset($driver->first_name) && $driver->first_name != '') {
                            $driver_full_name[]                                     = $driver->first_name;
                        }
                        $driver_full_name                                           = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';
                        $load_drivers_array[$driver->id]                            = [
                            'name'                  => $driver_full_name,
                            'email'                 => $driver->email,
                            'cell_phone'            => $driver->cell_phone,
                        ];
                    }
                    if (isset($load_detail->load_drivers)) {
                        $load_drivers                                               = $load_detail->load_drivers;
                        foreach ($load_drivers as $key => $load_driver) {
                            if (isset($load_driver->driver)) {
                                $driver                                             = $load_driver->driver;
                                $driver_full_name                                   = [];
                                if (isset($driver->last_name) && $driver->last_name != '') {
                                    $driver_full_name[]                             = $driver->last_name;
                                }
                                if (isset($driver->first_name) && $driver->first_name != '') {
                                    $driver_full_name[]                             = $driver->first_name;
                                }
                                $driver_full_name                                   = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';
                                $load_drivers_array[$driver->id]                    = [
                                    'name'                  => $driver_full_name,
                                    'email'                 => $driver->email,
                                    'cell_phone'            => $driver->cell_phone,
                                ];
                            }
                        }
                    }
                    // Load driver list

                    $load_documents                                                 = getCompanyLoadDocument();

                    $response['html']                                               = view('companyAdmin.billing.addDocument', compact('params', 'load_drivers_array', 'load_documents'))->render();
                    $response['success']                                            = true;
                    $response['message']                                            = "Add Documentation view retrieved successfully.";
                }
            } catch (\Exception $e) {
                $response['success']                                                = false;
                $response['message']                                                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function loadAddDocumentationSave(Request $request)
    {
        $response['success']                                                        = false;
        $response['message']                                                        = "Oops! Something went wrong..";
        if ($request->ajax()) {
            try {
                $formData                                                           = $request->all();

                $params                                                             = json_decode($formData['params'], true);

                $load_stops_array                                                   = $params['load_stops_array'];
                $searchkey                                                          = array_search($formData['doc_req_stop_no'], array_column($params['load_stops_array'], 'stop_no'));

                $filearry                                                           = [];
                $filearry['name_id']                                                = $formData['doc_req_name_id'];
                $filearry['name']                                                   = isset($formData['doc_req_name']) ? $formData['doc_req_name'] : '';
                $filearry['is_required']                                            = 'on';
                $filearry['type']                                                   = $formData['doc_req_type'];
                $filearry['fixed']                                                  = 'no';
                $filearry['include_w_billing']                                      = isset($formData['doc_req_include_w_billing']) && $formData['doc_req_include_w_billing'] == 'yes' ? 'yes' : 'no';
                $filearry['custom']                                                 = 'no';
                $filearry['is_received']                                            = 'off';
                $filearry['file_verify']                                            = '0'; // 0: Pending, 1: Approved, 2: Rejected
                $filearry['hide_from_driver']                                       = 'no';
                $filearry['file_not_provided']                                      = 'off';
                $filearry['file']                                                   = null;

                $request_via                                                        = [];
                if (isset($formData['doc_req_email']) && $formData['doc_req_email'] == 'on') {
                    $request_via[]                                                  = 1;
                }
                if (isset($formData['doc_req_sms']) && $formData['doc_req_sms'] == 'on') {
                    $request_via[]                                                  = 2;
                }
                $contacts                                                           = [];
                if (isset($formData['doc_req_send_driver_id']) && $formData['doc_req_send_driver_id'] != '') {
                    array_push($contacts, [
                        'driver_id' => $formData['doc_req_send_driver_id'],
                        'email'     => $formData['doc_req_send_email'] ?? '',
                        'phone'     => $formData['doc_req_send_phone'] ?? ''
                    ]);
                }

                $add                                                                = 0;
                $load_document_request                                              = LoadDocumentRequest::query()
                    ->where('load_id', $params['load_id'])
                    ->where('load_detail_id', $params['load_detail_id'])
                    ->where('broker_id', $params['broker_id'])
                    ->first();
                if (!is_null($load_document_request)) {
                    $item                                                           = new LoadDocumentRequestItem();
                    $item->load_document_request_id                                 = $load_document_request->id;
                    $item->load_id                                                  = $load_document_request->load_id;
                    $item->load_detail_id                                           = $load_document_request->load_detail_id;
                    $item->driver_id                                                = $formData['doc_req_driver_id'];
                    $item->load_stop_id                                             = $load_stops_array[$searchkey]['load_stop_id'];
                    $item->type                                                     = $load_stops_array[$searchkey]['type'];
                    $item->is_stop                                                  = $load_stops_array[$searchkey]['is_stop'];
                    $item->files                                                    = serialize($filearry);
                    $item->created_at                                               = \Carbon\Carbon::now()->format('Y-m-d H:i:s');
                    $item->created_by                                               = auth()->guard('company-admin')->user()->id;
                    $item->created_by_type                                          = auth()->guard('company-admin')->user()->type;
                    if ($item->save()) {
                        $add                                                        = 1;
                    }
                } else {
                    $load_document_request                                          = new LoadDocumentRequest();
                    $load_document_request->load_id                                 = $params['load_id'];
                    $load_document_request->load_detail_id                          = $params['load_detail_id'] ?? null;
                    $load_document_request->broker_id                               = $params['broker_id'] ?? null;
                    $load_document_request->created_at                              = Carbon::now()->format('Y-m-d H:i:s');
                    $load_document_request->created_by                              = auth()->guard('company-admin')->user()->id;
                    $load_document_request->created_by_type                         = auth()->guard('company-admin')->user()->type;
                    if ($load_document_request->save()) {
                        $item                                                       = new LoadDocumentRequestItem();
                        $item->load_document_request_id                             = $load_document_request->id;
                        $item->load_id                                              = $load_document_request->load_id;
                        $item->load_detail_id                                       = $load_document_request->load_detail_id;
                        $item->driver_id                                            = $formData['doc_req_driver_id'];
                        $item->load_stop_id                                         = $load_stops_array[$searchkey]['load_stop_id'];
                        $item->type                                                 = $load_stops_array[$searchkey]['type'];
                        $item->is_stop                                              = $load_stops_array[$searchkey]['is_stop'];
                        $item->files                                                = serialize($filearry);
                        $item->created_at                                           = Carbon::now()->format('Y-m-d H:i:s');
                        $item->created_by                                           = auth()->guard('company-admin')->user()->id;
                        $item->created_by_type                                      = auth()->guard('company-admin')->user()->type;
                        if ($item->save()) {
                            $add                                                    = 1;
                        }
                    }
                }

                if ($add == 1) {
                    // Send Email & SMS
                    if (isset($item) && !is_null($item) && !empty($contacts)) {
                        // item = Load Document Request Item
                        $load_detail                                                = $item->load_detail ?? null;
                        $driver                                                     = $item->driver ?? null;
                        if (!is_null($load_detail) && !is_null($driver)) {
                            $driver_full_name                                       = isset($driver->full_name) ? $driver->full_name : '';

                            $details                                                = [];
                            $details['company_id']                                  = auth()->guard('company-admin')->user()->company_id;
                            $details['full_name']                                   = $driver_full_name;
                            $details['load_id']                                     = $load_detail->load_id;
                            $details['load_detail_id']                              = $load_detail->id;
                            $details['load_number']                                 = $load_detail->load_type == 0 || $load_detail->load_type == 6 ? $load_detail->load_number : getLoadTypeList()[$load_detail->load_type];

                            $documents                                              = [];
                            //
                            $estop_no                                               = '';
                            $searchResults                                          = array_filter($params['load_stops_array'], function ($var) use ($item) {
                                return (
                                    $var['type'] == $item->type &&
                                    $var['is_stop'] == $item->is_stop &&
                                    $var['load_stop_id'] == $item->load_stop_id
                                );
                            });
                            if (!empty($searchResults)) {
                                $firstMatchingItem                                  = reset($searchResults); // Get the first element of the filtered array
                                $estop_no                                           = $firstMatchingItem['stop_no'] ?? '';
                            }
                            $files                                                  = [];
                            if (isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))) {
                                $files                                              = unserialize($item->files);
                            }
                            $stop_name                                              = 'Stop ' . $estop_no;
                            if (isset($files['name_id']) && $files['name_id'] != '') {
                                if ($files['name_id'] == 0) {
                                    if (isset($files['name']) && $files['name'] != '') {
                                        $stop_name                                  .= ' - ' . $files['name'] . ' ( Other )';
                                    } else {
                                        $stop_name                                  .= 'Other';
                                    }
                                } else {
                                    $stop_name                                      .= $files['name'] ? ' - ' . $files['name'] : '';
                                }
                            }
                            $documents[]                                            = $stop_name;
                            //
                            $details['documents']                                   = $documents;

                            //
                            foreach ($contacts as $key => $contact) {
                                // send Mail
                                if (isset($contact['email']) && $contact['email'] != '') {
                                    sendEmail($contact['email'], new \App\Mail\LoadDocumentRequestDriver($details));
                                }

                                // send Sms
                                if (isset($contact['phone']) && $contact['phone'] != '') {
                                    // Twilio's credentials are fetched from company table
                                    $getCompanyTwilioData                           = getCompanyTwilioData();
                                    if (isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes') {
                                        sendSMS($contact['phone'], 'A document request has been added to load number: ' . $details['load_number'] . '. Documents requested are: ' . implode(', ', $details['documents']) . '. Please upload these documents to the load. ' . route('driver.admin.loads.show', base64Encode($details['load_detail_id'])), $getCompanyTwilioData);
                                    }
                                }
                            }
                        }
                    }

                    $response['success']                                            = true;
                    $response['message']                                            = "Document request submitted successfully.";
                } else {
                    $response['success']                                            = false;
                    $response['message']                                            = "Oops! Something went wrong..";
                }

                $load_detail = LoadDetail::whereId($params['load_detail_id'])->first();
                $loadDocuments = getLoadDocuments($load_detail);

                $view = view('companyAdmin.billing.auditLoadView', compact('load_detail', 'loadDocuments'))->render();

                $response['auditLoadView'] = $view ?? '';
            } catch (\Exception $e) {
                $response['success']                                                = false;
                $response['message']                                                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function generateInvoice($loadDetail)
    {
        $invoiceData = [];

        $invoiceData['company_name'] = $loadDetail->loads->company->name ?? '';
        $invoiceData['company_address'] = $loadDetail->loads->company->address ?? '';
        $invoiceData['company_address_2'] = $loadDetail->loads->company->address_2 ?? '';
        $invoiceData['company_city'] = $loadDetail->loads->company->city ?? '';
        $invoiceData['company_state'] = $loadDetail->loads->company->state ?? '';
        $invoiceData['company_zip'] = $loadDetail->loads->company->zip ?? '';
        $invoiceData['company_phone'] = $loadDetail->loads->company->phone ?? '';

        $invoiceData['company_logo'] = $loadDetail->loads->company->logo ?? '';
        if ($invoiceData['company_logo'] != '' && file_exists(public_path('storage\\companyAdmin\\company_logo\\' . $invoiceData['company_logo']))) {
            $invoiceData['company_logo'] = public_path('storage\\companyAdmin\\company_logo\\' . $invoiceData['company_logo']);
        } else {
            $invoiceData['company_logo'] = '';
        }

        $invoiceData['customer_logo'] = '<img src="' . asset('theme/img/avtar.png') . '" alt="Customer Logo" style="width: 100%;height: 100%;object-fit: cover;object-position: center center;" />';
        $broker_logo = $loadDetail->broker->customer_logo ?? '';
        if ($broker_logo != '') {
            if (Storage::disk('public')->exists('companyAdmin/broker/customer_logo/' . $broker_logo)) {
                $invoiceData['customer_logo'] = '<img src="' . asset('storage/companyAdmin/broker/customer_logo/'.$broker_logo) . '" alt="Customer Logo" style="width: 100%;height: 100%;object-fit: cover;object-position: center center;" />';
            }
        }

        if(!empty($loadDetail->load_billing_invoice)){
            $invoiceDetail = $loadDetail->load_billing_invoice;

            $invoiceData['invoice_number'] = $invoiceDetail->invoice_number ?? getCompIdentifier() . 'Auto Generated';
            $invoiceData['payments_terms_date'] = $invoiceDetail->payments_terms_date ?? '';
            $invoiceData['payments_terms_days'] = $invoiceDetail->payments_terms_days ?? '30_days';
            $invoiceData['invoice_template'] = $invoiceDetail->invoice_template ?? '';
            $invoiceData['invoice_date'] = $invoiceDetail->invoice_date ?? '';
            $invoiceData['due_date'] = $invoiceDetail->due_date ?? '';
            $invoiceData['paid_date'] = $invoiceDetail->paid_date ?? '';
            $invoiceData['billing_method'] = $invoiceDetail->billing_method ?? '';
            $invoiceData['invoice_email'] = $invoiceDetail->invoice_email ?? '';
            $invoiceData['invoice_email_cc'] = $invoiceDetail->invoice_email_cc ?? '';
            $invoiceData['invoice_email_bcc'] = $invoiceDetail->invoice_email_bcc ?? '';

            $invoiceData['bill_to_name'] = $invoiceDetail->bill_to_name ?? '';
            $invoiceData['billing_address'] = $invoiceDetail->billing_address ?? '';
            $invoiceData['billing_address_2'] = $invoiceDetail->billing_address_2 ?? '';
            $invoiceData['billing_city'] = $invoiceDetail->billing_city ?? '';
            $invoiceData['billing_state'] = $invoiceDetail->billing_state ?? '';
            $invoiceData['billing_zip'] = $invoiceDetail->billing_zip ?? '';

            $invoiceData['po_number'] = $invoiceDetail->po_number ?? '';
            $invoiceData['bol_number'] = $invoiceDetail->bol_number ?? '';
            $invoiceData['pickup_number'] = $invoiceDetail->pickup_number ?? '';
            $invoiceData['delivery_number'] = $invoiceDetail->delivery_number ?? '';
            $invoiceData['notes'] = $invoiceDetail->notes ?? '';

            $invoiceData['add_qr_code'] = $invoiceDetail->add_qr_code ?? '';
            $invoiceData['send_payment_reminder'] = $invoiceDetail->send_payment_reminder ?? '';
            $invoiceData['payment_reminder_days'] = $invoiceDetail->payment_reminder_days ?? '';

            $invoiceData['salesman'] = $invoiceDetail->salesman ?? '';
            $invoiceData['carrier'] = $invoiceDetail->carrier ?? '';
            $invoiceData['dispatcher'] = $invoiceDetail->dispatcher ?? '';
            $invoiceData['truck'] = $invoiceDetail->truck ?? '';
            $invoiceData['trailer'] = $invoiceDetail->trailer ?? '';
            $invoiceData['driver'] = $invoiceDetail->driver ?? '';
        }else{
            $invoiceData['invoice_number'] = getCompIdentifier() . 'Auto Generated';
            $invoiceData['payments_terms_date'] = '';
            $invoiceData['payments_terms_days'] = '30_days';
            $invoiceData['invoice_template'] = '';
            $invoiceData['invoice_date'] = '';
            $invoiceData['due_date'] = '';
            $invoiceData['paid_date'] = '';
            $invoiceData['billing_method'] = $loadDetail->broker->billing_method ?? '';
            $invoiceData['invoice_email'] = $loadDetail->broker->billing_email ?? '';
            $invoiceData['invoice_email_cc'] = '';
            $invoiceData['invoice_email_bcc'] = '';

            $invoiceData['bill_to_name'] = $loadDetail->broker->bill_to_name;
            $invoiceData['billing_address'] = $loadDetail->broker->billing_address ?? '';
            $invoiceData['billing_address_2'] = $loadDetail->broker->billing_address_2 ?? '';
            $invoiceData['billing_city'] = $loadDetail->broker->billing_city ?? '';
            $invoiceData['billing_state'] = $loadDetail->broker->billing_state ?? '';
            $invoiceData['billing_zip'] = $loadDetail->broker->billing_zip ?? '';

            $invoiceData['po_number'] = $loadDetail->po_Number ?? '';
            $invoiceData['bol_number'] = $loadDetail->bol_number ?? '';
            $invoiceData['pickup_number'] = $loadDetail->pickup_number ?? '';
            $invoiceData['delivery_number'] = $loadDetail->delivery_number;
            $invoiceData['notes'] = '';

            $invoiceData['add_qr_code'] = '';
            $invoiceData['send_payment_reminder'] = '';
            $invoiceData['payment_reminder_days'] = '';

            $invoiceData['salesman'] = '';
            $invoiceData['carrier'] = $loadDetail->carrier_id ?? '';
            $invoiceData['dispatcher'] = $loadDetail->booked_by ?? '';
            $invoiceData['truck'] = $loadDetail->truck_number ?? '';
            $invoiceData['trailer'] = $loadDetail->trailer_number ?? '';
            $invoiceData['driver'] = $loadDetail->driver->id ?? '';
        }

        $invoiceData['origin'] = $loadDetail->shipper_city . ' ' . $loadDetail->shipper_state . '<br>' . $loadDetail->shipper_date_time;
        $invoiceData['destination'] = $loadDetail->delivery_city . ' ' . $loadDetail->delivery_state . '<br>' . $loadDetail->delivery_date_time;
        $invoiceData['commodity'] = getCompanyAdminCommodityList($loadDetail->loads->company_id)[$loadDetail->commodity] ?? '';
        $invoiceData['weight'] = number_format($loadDetail->weight) ?? '';

        if ($loadDetail->load_type_2 == '0') {
            $invoiceData['load_type'] = 'Full Truck';
        } else if ($loadDetail->load_type_2 == '1') {
            $invoiceData['load_type'] = 'LTL';
        }

        // Array to hold all row HTML
        $rowHtmlArray = [];

        // First Row
        $line_haul  = $loadDetail->line_haul ?? 0;
        $invoiceData['line_haul_formatted'] = !empty($line_haul) ? '$' . formatNumberWithoutZeros($line_haul, ['removeZeros' => false]) : '$0.00';

        // Second Row
        $invoiceData['loaded_miles'] = $loadDetail->loaded_miles ?? 0;
        $invoiceData['loaded_miles_formatted'] = !empty($invoiceData['loaded_miles']) ? formatNumberWithoutZeros($invoiceData['loaded_miles']) : 0;

        $invoiceData['agreed_upon_rate'] = $loadDetail->agreed_upon_rate ?? 0;
        $invoiceData['agreed_upon_rate_formatted'] = !empty($invoiceData['agreed_upon_rate']) ? '$' . formatNumberWithoutZeros($invoiceData['agreed_upon_rate']) : '$0.00';

        $fuel_surcharge = $loadDetail->fuel_surcharge ?? 0;
        $invoiceData['fuel_surcharge_formatted'] = !empty($fuel_surcharge) ? '$' . formatNumberWithoutZeros($fuel_surcharge, ['removeZeros' => false]) : '$0.00';

        $invoiceData['fuel_surcharge_rate_type'] = isset($loadDetail->fuel_surcharge_rate_type) && $loadDetail->fuel_surcharge_rate_type == 1 ? 1 : 0; // 0=Per Mile, 1=Percentage
        if ($invoiceData['fuel_surcharge_rate_type'] == 1) {
            // 1=Percentage
            $invoiceData['fuel_surcharge_cost_per_mile'] = isset($loadDetail->fuel_surcharge_cost_per_mile) ? '%' . ($loadDetail->fuel_surcharge_cost_per_mile / 100) : '%0';
        } else {
            // 0=Per Mile
            $invoiceData['fuel_surcharge_cost_per_mile'] = isset($loadDetail->fuel_surcharge_cost_per_mile) ? '$' . formatNumberWithoutZeros($loadDetail->fuel_surcharge_cost_per_mile, ['removeZeros' => false]) : '$0.00';
        }

        // Array to hold all row HTML
        $rowHtmlArray = [];

        // Array to hold all stop HTML
        $stopHtmlArray = [];

        // Total Accessorial Amount
        $accessorial_amount = 0;
        // Total Detention Amount
        $detention_amount = 0;

        // Accessorial
        if (isset($loadDetail->shipper_accessorials) && $loadDetail->shipper_accessorials->isNotEmpty()) {
            foreach ($loadDetail->shipper_accessorials as $key => $item) {
                if ($item->charge_deduct != 0) {
                    $name = isset($item->company_accessorial->name) ? $item->company_accessorial->name : '';
                    $qty = isset($item->qty) ? $item->qty : 0;
                    $qty_formatted = !empty($qty) ? formatNumberWithoutZeros($qty) : 0;

                    $rate = isset($item->rate) ? $item->rate : 0;
                    $rate_formatted = !empty($rate) ? '$' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$0.00';

                    $total = 0;
                    if ($item->charge_deduct == 1) {
                        $total += $item->total;
                    } else if ($item->charge_deduct == 2) {
                        $total -= $item->total;
                    }
                    if ($total >= 0) {
                        $total_formatted = '$' . formatNumberWithoutZeros($total, ['removeZeros' => false]);
                        $accessorial_amount += $total;
                    } else {
                        $rate_formatted = !empty($rate) ? '-$' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$0.00';
                        $total_formatted = '-$' . formatNumberWithoutZeros(abs($total), ['removeZeros' => false]);
                        $accessorial_amount -= abs($total);
                    }

                    // Add Row HTML to array
                    $stopHtmlArray['Stop 1'][] = '
                        <tr>
                            <td style="padding: 2px 2px 2px 15px" colspan="2">' . htmlspecialchars($name) . '</td>
                            <td style="padding: 2px 2px 2px 2px;text-align: center;">' . htmlspecialchars($qty_formatted) . '</td>
                            <td style="padding: 2px 2px 2px 2px;text-align: right;">' . htmlspecialchars($rate_formatted) . '</td>
                            <td style="padding: 2px 15px 2px 2px; text-align: right;" colspan="2">' . htmlspecialchars($total_formatted) . '</td>
                        </tr>
                    ';
                }
            }
        }
        // Detention
        $shipper_detention = isset($loadDetail->shipper_detention) ? $loadDetail->shipper_detention : null;
        if (!is_null($shipper_detention)) {
            if (in_array($shipper_detention->detention_status, [2, 3]) && $shipper_detention->review_status != 'Denied') {
                $billed_time = $shipper_detention->billed_time ?? '00:00';

                $rate = isset($shipper_detention->rate) ? $shipper_detention->rate : 0;
                $rate_formatted = !empty($rate) ? '$' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$0.00';

                $detention_due = isset($shipper_detention->detention_due) ? $shipper_detention->detention_due : 0;
                $detention_due_formatted = !empty($detention_due) ? '$' . formatNumberWithoutZeros($detention_due, ['removeZeros' => false]) : '$0.00';

                $detention_amount += (float)$detention_due;

                // Add Row HTML to array
                $stopHtmlArray['Stop 1'][] = '
                    <tr>
                        <td style="padding: 2px 2px 2px 15px" colspan="2">' . htmlspecialchars('Detention') . '</td>
                        <td style="padding: 2px 2px 2px 2px;text-align: center;">' . htmlspecialchars($billed_time) . '</td>
                        <td style="padding: 2px 2px 2px 2px;text-align: right;">' . htmlspecialchars($rate_formatted) . '</td>
                        <td style="padding: 2px 15px 2px 2px; text-align: right;" colspan="2">' . htmlspecialchars($detention_due_formatted) . '</td>
                    </tr>
                ';
            }
        }

        $stopNo = 2;
        // Extra Stop
        if (isset($loadDetail->loads->load_stops) && $loadDetail->loads->load_stops->isNotEmpty()) {
            foreach ($loadDetail->loads->load_stops as $key => $stop) {
                // Accessorial
                if (isset($stop->accessorials) && $stop->accessorials->isNotEmpty()) {
                    foreach ($stop->accessorials as $key => $item) {
                        if ($item->charge_deduct != 0) {
                            $name = isset($item->company_accessorial->name) ? $item->company_accessorial->name : '';

                            $qty = isset($item->qty) ? $item->qty : 0;
                            $qty_formatted = !empty($qty) ? formatNumberWithoutZeros($qty) : 0;

                            $rate = isset($item->rate) ? $item->rate : 0;
                            $rate_formatted = !empty($rate) ? '$' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$0.00';

                            $total = 0;
                            if ($item->charge_deduct == 1) {
                                $total += $item->total;
                            } else if ($item->charge_deduct == 2) {
                                $total -= $item->total;
                            }
                            if ($total >= 0) {
                                $total_formatted = '$' . formatNumberWithoutZeros($total, ['removeZeros' => false]);

                                $accessorial_amount += $total;
                            } else {
                                $rate_formatted = !empty($rate) ? '-$' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$0.00';
                                $total_formatted = '-$' . formatNumberWithoutZeros(abs($total), ['removeZeros' => false]);

                                $accessorial_amount -= abs($total);
                            }
                            // Add Row HTML to array
                            $stopHtmlArray['Stop ' . $stopNo][] = '
                                <tr>
                                    <td style="padding: 2px 2px 2px 15px" colspan="2">' . htmlspecialchars($name) . '</td>
                                    <td style="padding: 2px 2px 2px 2px;text-align: center;">' . htmlspecialchars($qty_formatted) . '</td>
                                    <td style="padding: 2px 2px 2px 2px;text-align: right;">' . htmlspecialchars($rate_formatted) . '</td>
                                    <td style="padding: 2px 15px 2px 2px; text-align: right;" colspan="2">' . htmlspecialchars($total_formatted) . '</td>
                                </tr>
                            ';
                        }
                    }
                }
                // Detention
                $stop_detention = isset($stop->detention) ? $stop->detention : null;
                if (!is_null($stop_detention)) {
                    if (in_array($stop_detention->detention_status, [2, 3]) && $stop_detention->review_status != 'Denied') {
                        $billed_time = $stop_detention->billed_time ?? '00:00';

                        $rate = isset($stop_detention->rate) ? $stop_detention->rate : 0;
                        $rate_formatted = !empty($rate) ? '$' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$0.00';

                        $detention_due = isset($stop_detention->detention_due) ? $stop_detention->detention_due : 0;
                        $detention_due_formatted = !empty($detention_due) ? '$' . formatNumberWithoutZeros($detention_due, ['removeZeros' => false]) : '$0.00';

                        $detention_amount += (float)$detention_due;

                        // Add Row HTML to array
                        $stopHtmlArray['Stop ' . $stopNo][] = '
                            <tr>
                                <td style="padding: 2px 2px 2px 15px" colspan="2">' . htmlspecialchars('Detention') . '</td>
                                <td style="padding: 2px 2px 2px 2px;text-align: center;">' . htmlspecialchars($billed_time) . '</td>
                                <td style="padding: 2px 2px 2px 2px;text-align: right;">' . htmlspecialchars($rate_formatted) . '</td>
                                <td style="padding: 2px 15px 2px 2px; text-align: right;" colspan="2">' . htmlspecialchars($detention_due_formatted) . '</td>
                            </tr>
                        ';
                    }
                }
                ++$stopNo;
            }
        }

        // Accessorial
        if (isset($loadDetail->receiver_accessorials) && $loadDetail->receiver_accessorials->isNotEmpty()) {
            foreach ($loadDetail->receiver_accessorials as $key => $item) {
                if ($item->charge_deduct != 0) {
                    $name = isset($item->company_accessorial->name) ? $item->company_accessorial->name : '';

                    $qty = isset($item->qty) ? $item->qty : 0;
                    $qty_formatted = !empty($qty) ? formatNumberWithoutZeros($qty) : 0;

                    $rate = isset($item->rate) ? $item->rate : 0;
                    $rate_formatted = !empty($rate) ? '$' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$0.00';

                    $total = 0;
                    if ($item->charge_deduct == 1) {
                        $total += $item->total;
                    } else if ($item->charge_deduct == 2) {
                        $total -= $item->total;
                    }
                    if ($total >= 0) {
                        $total_formatted = '$' . formatNumberWithoutZeros($total, ['removeZeros' => false]);

                        $accessorial_amount += $total;
                    } else {
                        $rate_formatted = !empty($rate) ? '-$' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$0.00';
                        $total_formatted = '-$' . formatNumberWithoutZeros(abs($total), ['removeZeros' => false]);

                        $accessorial_amount -= abs($total);
                    }

                    // Add Row HTML to array
                    $stopHtmlArray['Stop 99'][] = '
                        <tr>
                            <td style="padding: 2px 2px 2px 15px" colspan="2">' . htmlspecialchars($name) . '</td>
                            <td style="padding: 2px 2px 2px 2px;text-align: center;">' . htmlspecialchars($qty_formatted) . '</td>
                            <td style="padding: 2px 2px 2px 2px;text-align: right;">' . htmlspecialchars($rate_formatted) . '</td>
                            <td style="padding: 2px 15px 2px 2px; text-align: right;" colspan="2">' . htmlspecialchars($total_formatted) . '</td>
                        </tr>
                    ';
                }
            }
        }
        // Detention
        $delivery_detention = isset($loadDetail->receiver_detention) ? $loadDetail->receiver_detention : null;
        if (!is_null($delivery_detention)) {
            if (in_array($delivery_detention->detention_status, [2, 3]) && $delivery_detention->review_status != 'Denied') {
                $billed_time = $delivery_detention->billed_time ?? '00:00';

                $rate = isset($delivery_detention->rate) ? $delivery_detention->rate : 0;
                $rate_formatted = !empty($rate) ? '$' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$0.00';

                $detention_due = isset($delivery_detention->detention_due) ? $delivery_detention->detention_due : 0;
                $detention_due_formatted = !empty($detention_due) ? '$' . formatNumberWithoutZeros($detention_due, ['removeZeros' => false]) : '$0.00';

                $detention_amount += (float)$detention_due;

                // Add Row HTML to array
                $stopHtmlArray['Stop 99'][] = '
                    <tr>
                        <td style="padding: 2px 2px 2px 15px" colspan="2">' . htmlspecialchars('Detention') . '</td>
                        <td style="padding: 2px 2px 2px 2px;text-align: center;">' . htmlspecialchars($billed_time) . '</td>
                        <td style="padding: 2px 2px 2px 2px;text-align: right;">' . htmlspecialchars($rate_formatted) . '</td>
                        <td style="padding: 2px 15px 2px 2px; text-align: right;" colspan="2">' . htmlspecialchars($detention_due_formatted) . '</td>
                    </tr>
                ';
            }
        }

        // Accessorial
        if (isset($loadDetail->billing_accessorials) && $loadDetail->billing_accessorials->isNotEmpty()) {
            foreach ($loadDetail->billing_accessorials as $key => $item) {
                if ($item->charge_deduct != 0) {
                    $name = isset($item->company_accessorial->name) ? $item->company_accessorial->name : '';

                    $qty = isset($item->qty) ? $item->qty : 0;
                    $qty_formatted = !empty($qty) ? formatNumberWithoutZeros($qty) : 0;

                    $rate = isset($item->rate) ? $item->rate : 0;
                    $rate_formatted = !empty($rate) ? '$' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$0.00';

                    $total = 0;
                    if ($item->charge_deduct == 1) {
                        $total += $item->total;
                    } else if ($item->charge_deduct == 2) {
                        $total -= $item->total;
                    }
                    if ($total >= 0) {
                        $total_formatted = '$' . formatNumberWithoutZeros($total, ['removeZeros' => false]);

                        $accessorial_amount += $total;
                    } else {
                        $rate_formatted = !empty($rate) ? '-$' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$0.00';
                        $total_formatted = '-$' . formatNumberWithoutZeros(abs($total), ['removeZeros' => false]);

                        $accessorial_amount -= abs($total);
                    }

                    // Add Row HTML to array
                    $stopHtmlArray['Load'][] = '
                        <tr>
                            <td style="padding: 2px 2px 2px 15px" colspan="2">' . htmlspecialchars($name) . '</td>
                            <td style="padding: 2px 2px 2px 2px;text-align: center;">' . htmlspecialchars($qty_formatted) . '</td>
                            <td style="padding: 2px 2px 2px 2px;text-align: right;">' . htmlspecialchars($rate_formatted) . '</td>
                            <td style="padding: 2px 15px 2px 2px; text-align: right;" colspan="2">' . htmlspecialchars($total_formatted) . '</td>
                        </tr>
                    ';
                }
            }
        }

        // Append all stop to rows
        if (!empty($stopHtmlArray)) {
            foreach ($stopHtmlArray as $stopName => $stopHtmls) {
                if (!empty($stopHtmls)) {
                    foreach ($stopHtmls as $iteration => $stopHtml) {
                        // Check if the current iteration number is 0 (indicating the first iteration)
                        if ($iteration === 0) {
                            // Add Row HTML to array
                            $rowHtmlArray[] = '
                                    <tr>
                                        <td style="padding: 2px 2px 2px 15px; font-weight: bold" colspan="6">' . htmlspecialchars($stopName) . '</td>
                                    </tr>
                                ';
                        }
                        // Add Row HTML to array
                        $rowHtmlArray[] = $stopHtml;
                    }
                }
            }
        }

        $invoiceData['accessorial_and_detention'] = $rowHtmlArray;

        if (count($loadDetail->detentions) > 0) {
            if (!empty(array_filter($loadDetail->detentions->pluck('approved_by')->toArray()))) {
                $approvedBy = $loadDetail->detentions->pluck('approved_by')->toArray()[0];
                $approvedByFullName = CompanyAdmin::find($approvedBy)->full_name;
            }
        }
        $invoiceData['invoice_notes'] = $loadDetail->load_billing_invoice->notes ?? '';

        // Total Accessorial Amount
        $invoiceData['accessorial_amount_formatted'] = !empty($accessorial_amount) ? '$' . formatNumberWithoutZeros($accessorial_amount, ['removeZeros' => false]) : '$0.00';
        // Total Detention Amount
        $invoiceData['detention_amount_formatted'] = !empty($detention_amount) ? '$' . formatNumberWithoutZeros($detention_amount, ['removeZeros' => false]) : '$0.00';

        $invoiceData['invoice_total'] = $line_haul + $fuel_surcharge + $accessorial_amount + $detention_amount;
        $invoiceData['invoice_total_formatted'] = !empty($invoiceData['invoice_total']) ? '$' . formatNumberWithoutZeros($invoiceData['invoice_total'], ['removeZeros' => false]) : '$0.00';

        $amount_paid = 0;
        $pay_invoices = $loadDetail->load_billing_pay_invoices ?? null;
        if (!is_null($pay_invoices)) {
            $paidInvoiceTotal = $pay_invoices->sum('total_payment') ?? 0;
            $chargebackAgainstBalance = $pay_invoices->pluck('chargeback_against_balance');
            $chargebackAmountsData = $pay_invoices->pluck('chargeback_amount');
            $chargebackAmounts = array_sum($chargebackAmountsData->map(function ($values, $key) use ($chargebackAgainstBalance) {
                if (count($values) > 0) {
                    $amounts                            = [];
                    foreach ($values as $k => $v) {
                        if (isset($chargebackAgainstBalance[$key][$k]) && $chargebackAgainstBalance[$key][$k] == '1') {
                            $amounts[]                  = $v;
                        }
                    }
                    return array_sum($amounts);
                }
                return 0;
            })->toArray());

            $amount_paid = round(($paidInvoiceTotal + $chargebackAmounts), 2);
        }
        $invoiceData['balance_due'] = $invoiceData['invoice_total'] - $amount_paid;
        $invoiceData['balance_due_formatted'] = !empty($invoiceData['balance_due']) ? '$' . formatNumberWithoutZeros($invoiceData['balance_due'], ['removeZeros' => false]) : '$0.00';

        $invoiceData['factor_invoices'] = ($loadDetail->broker->factor_invoices == '1') ? brokerFactoringCompany()[$loadDetail->broker->factor_company] : '';

        if (!empty($loadDetail->load_billing_invoice)) {
            $invoiceData['invoice_date'] = $loadDetail->load_billing_invoice->invoice_date;
            $invoiceData['due_date'] = $loadDetail->load_billing_invoice->due_date;
        }

        return $invoiceData;
    }

    public function invoiceCreate(Request $request)
    {
        $loadDetail = LoadDetail::findOrFail(base64Decode($request->id));
        if (!empty($loadDetail->load_billing_invoice)) {
            return redirect()->route('company.admin.billing.invoice.show', ['id' => base64Encode($loadDetail->load_billing_invoice->id)]);
        }
        $loadDetail->billing_full_address = $loadDetail->broker->billingFullAddress();

        $invoiceTopNotes = [];
        foreach ($loadDetail->broker->internal_notes as $internal_notes) {
            if (in_array('3', $internal_notes->add_to)) {
                $invoiceTopNotes[] = $internal_notes->notes;
            }
        }

        if (isset($loadDetail->delivery_date_time)) {
            LoadBillingHistory::updateOrCreate(['load_detail_id' => $loadDetail->id, 'type' => 'delivered'], [
                'date_time' => Carbon::parse($loadDetail->delivery_date_time)->format('m/d/Y H:i:s'),
                'action_by' => $loadDetail->company_admin->full_name,
            ]);
        }
        if (isset($loadDetail->load_billing_activities) && $loadDetail->load_billing_activities->paperwork_received == '1') {
            LoadBillingHistory::updateOrCreate(['load_detail_id' => $loadDetail->id, 'type' => 'paperwork'], [
                'date_time' => Carbon::parse($loadDetail->load_billing_activities->paperwork_received_date_time)->format('m/d/Y H:i:s'),
                'action_by' => auth()->guard('company-admin')->user()->full_name ?? '',
            ]);
        }
        if (isset($loadDetail->load_billing_activities) && $loadDetail->load_billing_activities->audit_complete == '1') {
            LoadBillingHistory::updateOrCreate(['load_detail_id' => $loadDetail->id, 'type' => 'audited'], [
                'date_time' => Carbon::parse($loadDetail->load_billing_activities->audit_complete_date_time)->format('m/d/Y H:i:s'),
                'action_by' => CompanyAdmin::find($loadDetail->load_billing_activities->audit_complete_user_by)->full_name ?? '',
            ]);
        }

        $invoiceData = $this->generateInvoice($loadDetail);

        return view('companyAdmin.billing.create-invoice', compact('loadDetail', 'invoiceTopNotes', 'invoiceData'));
    }

    public function invoiceSave(Request $request)
    {
        try {
            $input = $request->all();
            $input['add_qr_code'] = $input['add_qr_code'] ?? '0';
            $input['send_payment_reminder'] = $input['send_payment_reminder'] ?? '0';
            $input['invoice_created_by'] = auth()->guard('company-admin')->user()->id;
            $input['invoice_created_on'] = Carbon::now();

            $invoiceCreated = LoadBillingInvoice::create($input);

            if ($invoiceCreated) {
                // Load move to Invoiced Status
                $loadDetail = LoadDetail::findOrFail($invoiceCreated->load_detail_id);
                $oldData = $loadDetail;

                $loadDetail->load_status = 10;
                $loadDetail->load_status_order = Common::getLoadStatusOrderById(10);
                $loadDetail->updated_at = date('Y-m-d H:i:s');
                $loadDetail->save();

                $this->setLoadAudit($loadDetail->load_id, 'update', 'Load detail updated', $loadDetail->getChanges(), $oldData);

                // Update load_billing_activities
                if (is_null($loadDetail->load_billing_activities)) {
                    $loadDetail->load_billing_activities = $loadDetail->load_billing_activities()->create([
                        'load_id'                       => $loadDetail->load_id,
                        'load_detail_id'                => $loadDetail->id,
                        'paperwork_received'            => '1',
                        'paperwork_received_date_time'  => Carbon::now()->setTimezone(getTimeZone()),
                        'audit_complete'                => '1',
                        'audit_complete_date_time'      => Carbon::now()->setTimezone(getTimeZone()),
                        'audit_complete_user_type'      => auth()->guard('company-admin')->user()->type,
                        'audit_complete_user_by'        => auth()->guard('company-admin')->user()->id,
                        'invoiced'                      => '1',
                        'invoiced_date_time'            => Carbon::now()->setTimezone(getTimeZone()),
                        'invoiced_user_type'            => auth()->guard('company-admin')->user()->type,
                        'invoiced_user_by'              => auth()->guard('company-admin')->user()->id,
                    ]);
                } else if (count(getLoadDocuments($loadDetail)['merged_documents']) == 0) {
                    $loadDetail->load_billing_activities->update([
                        'paperwork_received'            => '1',
                        'paperwork_received_date_time'  => Carbon::now()->setTimezone(getTimeZone()),
                        'audit_complete'                => '1',
                        'audit_complete_date_time'      => Carbon::now()->setTimezone(getTimeZone()),
                        'audit_complete_user_type'      => auth()->guard('company-admin')->user()->type,
                        'audit_complete_user_by'        => auth()->guard('company-admin')->user()->id,
                        'invoiced'                      => '1',
                        'invoiced_date_time'            => Carbon::now()->setTimezone(getTimeZone()),
                        'invoiced_user_type'            => auth()->guard('company-admin')->user()->type,
                        'invoiced_user_by'              => auth()->guard('company-admin')->user()->id,
                    ]);
                } else {
                    $loadDetail->load_billing_activities->update([
                        'invoiced'                      => '1',
                        'invoiced_date_time'            => Carbon::now()->setTimezone(getTimeZone()),
                        'invoiced_user_type'            => auth()->guard('company-admin')->user()->type,
                        'invoiced_user_by'              => auth()->guard('company-admin')->user()->id,
                    ]);
                }

                //Update billing & payment history
                if (isset($loadDetail->load_billing_activities) && $loadDetail->load_billing_activities->audit_complete == '1') {
                    LoadBillingHistory::updateOrCreate(['load_detail_id' => $loadDetail->id, 'type' => 'audited'], [
                        'date_time' => Carbon::parse($loadDetail->load_billing_activities->audit_complete_date_time)->format('m/d/Y H:i:s'),
                        'action_by' => CompanyAdmin::find($loadDetail->load_billing_activities->audit_complete_user_by)->full_name ?? '',
                        'extra_params' => array(
                            'amount' => round($invoiceCreated->invoice_amount, 2),
                            'currency' => 'USD',
                        )
                    ]);
                }
                if (isset($loadDetail->load_billing_activities) && $loadDetail->load_billing_activities->invoiced == '1') {
                    LoadBillingHistory::updateOrCreate(['load_detail_id' => $loadDetail->id, 'type' => 'invoiced'], [
                        'date_time' => Carbon::parse($loadDetail->load_billing_activities->invoiced_date_time)->format('m/d/Y H:i:s'),
                        'action_by' => CompanyAdmin::find($loadDetail->load_billing_activities->invoiced_user_by)->full_name ?? '',
                        'extra_params' => array(
                            'amount' => round($invoiceCreated->invoice_amount, 2),
                            'currency' => 'USD',
                        )
                    ]);
                }

                //IF Preferred Billing Method Not Mail then we send Email & SMS
                if($invoiceCreated->billing_method != '3'){
                    // Send Email & SMS
                    $customer_id = $input['customer'];
                    $customer = Broker::whereId($customer_id)->first();

                    if (!empty($customer)) {
                        $contactDetails = [];
                        $contactDetails['email'] = str_replace(' ', '', $invoiceCreated->invoice_email) ?? null;
                        if (isset($invoiceCreated->invoice_email_cc) && $invoiceCreated->invoice_email_cc != '') {
                            $contactDetails['cc'] = explode(',', str_replace(' ', '', $invoiceCreated->invoice_email_cc));
                        }
                        if (isset($invoiceCreated->invoice_email_bcc) && $invoiceCreated->invoice_email_bcc != '') {
                            $contactDetails['bcc'] = explode(',', str_replace(' ', '', $invoiceCreated->invoice_email_bcc));
                        }
                        $contactDetails['phone'] = $customer->billing_phone ?? null;

                        if (isset($contactDetails['email']) && $contactDetails['email'] != '') {
                            $mailData = [];
                            $mailData['company_id'] = auth()->guard('company-admin')->user()->company_id;
                            $mailData['name'] = $invoiceCreated->bill_to_name;
                            $mailData['load_number'] = $loadDetail->load_number;
                            $mailData['invoice_id'] = $invoiceCreated->id;
                            $mailData['invoice_number'] = $invoiceCreated->invoice_number;
                            $mailData['invoice_due_date'] = $invoiceCreated->due_date;
                            $mailData['invoice_amount'] = '$ ' . number_format($invoiceCreated->invoice_amount, 2);

                            sendEmail($contactDetails, new SendLoadInvoice($mailData));

                            LoadBillingHistory::updateOrCreate(['load_detail_id' => $loadDetail->id, 'type' => 'email_sent'], [
                                'date_time' => Carbon::parse($loadDetail->load_billing_activities->invoiced_date_time)->format('m/d/Y H:i:s'),
                                'action_by' => auth()->guard('company-admin')->user()->full_name ?? '',
                                'extra_params' => array(
                                    'type' => 'Email',
                                    'to' => $invoiceCreated->billingFullAddress() ?? '',
                                    'email' => $contactDetails['email'] ?? '',
                                )
                            ]);
                        }

                        if (isset($contactDetails['phone']) && $contactDetails['phone'] != '') {
                            // Twilio's credentials are fetched from company table
                            $getCompanyTwilioData = getCompanyTwilioData();
                            if (isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes') {
                                sendSMS($contactDetails['phone'], 'Invoice Created for load number: ' . $loadDetail->load_number . '. Here\'s your invoice! We appreciate your prompt payment. Invoice Due Date: ' . $invoiceCreated->due_date . '. Invoice Amount: $ ' . $invoiceCreated->invoice_amount . '. Print & Save: ' . route(
                                    'company.admin.broker.invoice.view',
                                    [
                                        'type'  => 2, // Type => // 0 = None, 1 = Email, 2 = SMS, 3 = Scan QR, 4 = Shared Link
                                        'id'    => base64Encode($invoiceCreated->id),
                                        'format' => 1 // Format => // 0 = Stream, 1 = Save means Download, 2 = Print
                                    ]
                                ), $getCompanyTwilioData);

                                LoadBillingHistory::updateOrCreate(['load_detail_id' => $loadDetail->id, 'type' => 'sms_sent'], [
                                    'date_time' => Carbon::parse($loadDetail->load_billing_activities->invoiced_date_time)->format('m/d/Y H:i:s'),
                                    'action_by' => auth()->guard('company-admin')->user()->full_name ?? '',
                                    'extra_params' => array(
                                        'type' => 'SMS',
                                        'to' => $contactDetails['phone'],
                                    )
                                ]);
                            }
                        }
                    }
                    // Send Email & SMS
                }

                notification('success', 'Invoice created successfully.');
            } else {
                notification('error', 'Oops! Something went wrong!');
            }
        } catch (\Exception $e) {
            notification('error', $e->getMessage());
        }

        if (isset($input['prev']) && $input['prev'] != '') {
            return redirect()->to($input['prev']);
        } else {
            return redirect()->route('company.admin.billing.index');
        }
    }

    public function viewInvoice(Request $request)
    {
        // Template
        $loadInvoice                                            = LoadBillingInvoice::findOrFail(base64Decode($request->id));

        $load                                                   = isset($loadInvoice->loads) ? $loadInvoice->loads : null;
        $loadDetail                                             = isset($loadInvoice->load_detail) ? $loadInvoice->load_detail : null;
        $company                                                = null;
        if (isset($load->company)) {
            $company                                            = $load->company;
        }
        $companyId                                              = $company->id ?? null;
        if (is_null($load) || is_null($loadDetail) || is_null($companyId)) {
            return abort(404);
        }

        $pdf_template                                           = CompanyPdfTemplate::where('company_id', $companyId)
            ->where('name', 'Load Invoice')
            ->first();
        if (!is_null($pdf_template)) {
            // Add Billing History
            if ($request->type == '1') {
                $emailViewed                                    = LoadBillingHistory::where('load_detail_id', $loadInvoice->load_detail_id)
                    ->where('type', 'viewed_email')
                    ->count();
                if ($emailViewed == 0) {
                    LoadBillingHistory::create([
                        'load_detail_id'                        => $loadInvoice->load_detail_id,
                        'type'                                  => 'viewed_email',
                        'date_time'                             => Carbon::now()->format('m/d/Y H:i:s'),
                        'action_by'                             => 'User',
                        'extra_params'                          => array(
                            'type'                              => 'Email',
                            'ip_address'                        => $request->ip() ?? '',
                        )
                    ]);
                }
            } else if ($request->type == '2') {
                $smsViewed                                      = LoadBillingHistory::where('load_detail_id', $loadInvoice->load_detail_id)
                    ->where('type', 'viewed_sms')
                    ->count();
                if ($smsViewed == 0) {
                    LoadBillingHistory::create([
                        'load_detail_id'                        => $loadInvoice->load_detail_id,
                        'type'                                  => 'viewed_sms',
                        'date_time'                             => Carbon::now()->format('m/d/Y H:i:s'),
                        'action_by'                             => 'User',
                        'extra_params'                          => array(
                            'type'                              => 'SMS',
                            'ip_address'                        => $request->ip() ?? '',
                        )
                    ]);
                }
            }else if ($request->type == '3') {
                $scanQrViewed = LoadBillingHistory::where('load_detail_id', $loadInvoice->load_detail_id)
                    ->where('type', 'viewed_scan_qr')
                    ->count();
                if ($scanQrViewed == 0) {
                    LoadBillingHistory::create([
                        'load_detail_id'                        => $loadInvoice->load_detail_id,
                        'type'                                  => 'viewed_scan_qr',
                        'date_time'                             => Carbon::now()->format('m/d/Y H:i:s'),
                        'action_by'                             => 'User',
                        'extra_params'                          => array(
                            'type'                              => 'Scan QR',
                            'ip_address'                        => $request->ip() ?? '',
                        )
                    ]);
                }
            }else if ($request->type == '4') {
                $sharedLinkViewed = LoadBillingHistory::where('load_detail_id', $loadInvoice->load_detail_id)
                    ->where('type', 'viewed_shared_link')
                    ->count();
                if ($sharedLinkViewed == 0) {
                    LoadBillingHistory::create([
                        'load_detail_id'                        => $loadInvoice->load_detail_id,
                        'type'                                  => 'viewed_shared_link',
                        'date_time'                             => Carbon::now()->format('m/d/Y H:i:s'),
                        'action_by'                             => 'User',
                        'extra_params'                          => array(
                            'type'                              => 'Shared Link',
                            'ip_address'                        => $request->ip() ?? '',
                        )
                    ]);
                }
            }
            // Add Billing History

            // Replace the placeholder with the desired value
            $content                                            = $pdf_template->content;

            // Inject CSS for margins and padding directly into the HTML content
            $css                                                = '<title>'.$loadInvoice->invoice_number.'</title><style>
                @page { margin: 5mm; }
                .pdf-container { font-family: "Inter", sans-serif; } // Apply the font to .pdf-container
                // .address-table { background-color: red; }
                #main-table > tbody > tr:nth-of-type(1) > td {
                   padding:12px 12px 25px;
                }
                #main-table > tbody > tr:nth-of-type(2) > td,
                #main-table > tbody > tr:nth-of-type(3) > td,
                #main-table > tbody > tr:nth-of-type(4) > td,
                #main-table > tbody > tr:nth-of-type(5) > td {
                    padding: 20px 12px;
                }
                #main-table > tbody > tr:nth-of-type(6) > td table td {
                    padding: 25px 12px;
                }
                #main-table > tbody > tr:last-of-type > td {
                   padding:12px 6px;
                }
                #address-table td { line-height:1.2 }
                #address-table tr:nth-of-type(4) td {
                   padding-top:25px;
                }
                #invoice-no td { padding:0 }
                #item-table tr td:nth-of-type(1) {
                   min-width: 60%;
                }
                #item-table thead tr th,
                #item-table tbody tr td{
                   padding:4px;
                }
                #invoice-total-table tbody tr td:nth-of-type(1){
                   padding:4px 4px 4px 0;
                }
                #invoice-total-table tbody tr td:nth-of-type(2){
                   padding:4px 0px 4px 4px;
                }
                #notes-table>tbody>tr>td:nth-of-type(1){
                   padding-right:12px
                }
            </style>';

            // Specify the font files for Dompdf
            $fontPath                                           = public_path('theme/fonts/Inter/static/');
            $css                                                .= '<style>
                @font-face {
                    font-family: "Inter", sans-serif;
                    font-style: normal;
                    font-weight: 400;
                    font-display: swap;
                    src: url('.$fontPath.'Inter-Regular.ttf) format("truetype");
                }
                @font-face {
                    font-family: "Inter", sans-serif;
                    font-style: normal;
                    font-weight: 500;
                    font-display: swap;
                    src: url('.$fontPath.'Inter-Medium.ttf) format("truetype");
                }
                @font-face {
                    font-family: "Inter", sans-serif;
                    font-style: normal;
                    font-weight: 600;
                    font-display: swap;
                    src: url('.$fontPath.'Inter-SemiBold.ttf) format("truetype");
                }
                @font-face {
                    font-family: "Inter", sans-serif;
                    font-style: normal;
                    font-weight: 700;
                    font-display: swap;
                    src: url('.$fontPath.'Inter-Bold.ttf) format("truetype");
                }
                /* Add more @font-face rules for different weights and styles if needed */
            </style>';

            $content                                            = $css . '<div class="pdf-container">' . $content . '</div>';

            // Array to hold all row HTML
            $rowHtmlArray                                       = [];

            // First Row
            $line_haul                                          = isset($loadDetail->line_haul) ? $loadDetail->line_haul : 0;
            $line_haul_formatted                                = !empty($line_haul) ? '$ ' . formatNumberWithoutZeros($line_haul, ['removeZeros' => false]) : '$ 0.00';

            $line_haul_tr                                       = '<tr>
                <td style="vertical-align:top; width:40%">' . htmlspecialchars('Linehaul') . '</td>
                <td style="vertical-align:top; width:120px">' . htmlspecialchars('1') . '</td>
                <td style="vertical-align:top; width:120px">' . htmlspecialchars($line_haul_formatted) . '</td>
                <td style="vertical-align:top; width:130px">' . htmlspecialchars($line_haul_formatted) . '</td>
            </tr>';

            // Add First Row HTML to array
            $rowHtmlArray[]                                     = $line_haul_tr;

            // Second Row
            $loaded_miles                                       = isset($loadDetail->loaded_miles) ? $loadDetail->loaded_miles : 0;
            $loaded_miles_formatted                             = !empty($loaded_miles) ? formatNumberWithoutZeros($loaded_miles) : 0;

            $agreed_upon_rate                                   = isset($loadDetail->agreed_upon_rate) ? $loadDetail->agreed_upon_rate : 0;
            $agreed_upon_rate_formatted                         = !empty($agreed_upon_rate) ? '$ ' . formatNumberWithoutZeros($agreed_upon_rate, ['removeZeros' => false]) : '$ 0.00';

            $fuel_surcharge                                     = isset($loadDetail->fuel_surcharge) ? $loadDetail->fuel_surcharge : 0;
            $fuel_surcharge_formatted                           = !empty($fuel_surcharge) ? '$ ' . formatNumberWithoutZeros($fuel_surcharge, ['removeZeros' => false]) : '$ 0.00';

            $fuel_surcharge_tr                                  = '';
            $fuel_surcharge_rate_type                           = isset($loadDetail->fuel_surcharge_rate_type) && $loadDetail->fuel_surcharge_rate_type == 1 ? 1 : 0; // 0=Per Mile, 1=Percentage
            if ($fuel_surcharge_rate_type == 1) {
                // 1=Percentage
                $fuel_surcharge_cost_per_mile                   = isset($loadDetail->fuel_surcharge_cost_per_mile) ? ($loadDetail->fuel_surcharge_cost_per_mile / 100) : '0';
                $fuel_surcharge_tr                              = '
                    <tr>
                        <td style="vertical-align:top; width:40%">' . htmlspecialchars('Fuel Surcharge') . '</td>
                        <td style="vertical-align:top; width:120px">' . htmlspecialchars($fuel_surcharge_cost_per_mile) . '</td>
                        <td style="vertical-align:top; width:120px">' . htmlspecialchars($agreed_upon_rate_formatted) . '</td>
                        <td style="vertical-align:top; width:130px">' . htmlspecialchars($fuel_surcharge_formatted) . '</td>
                    </tr>
                ';
            } else {
                // 0=Per Mile
                $fuel_surcharge_cost_per_mile                   = isset($loadDetail->fuel_surcharge_cost_per_mile) ? '$ ' . formatNumberWithoutZeros($loadDetail->fuel_surcharge_cost_per_mile, ['removeZeros' => false]) : '$ 0.00';
                $fuel_surcharge_tr                              = '
                    <tr>
                        <td style="vertical-align:top; width:40%">' . htmlspecialchars('Fuel Surcharge') . '</td>
                        <td style="vertical-align:top; width:120px">' . htmlspecialchars($loaded_miles_formatted) . '</td>
                        <td style="vertical-align:top; width:120px">' . htmlspecialchars($fuel_surcharge_cost_per_mile) . '</td>
                        <td style="vertical-align:top; width:130px">' . htmlspecialchars($fuel_surcharge_formatted) . '</td>
                    </tr>
                ';
            }

            // Add Second Row HTML to array
            $rowHtmlArray[]                                     = $fuel_surcharge_tr;

            // Array to hold all stop HTML
            $stopHtmlArray                                      = [];

            // Total Accessorial Amount
            $accessorial_amount                                 = 0;
            // Total Detention Amount
            $detention_amount                                   = 0;

            // Accessorial
            if (isset($loadDetail->shipper_accessorials) && $loadDetail->shipper_accessorials->isNotEmpty()) {
                $shipper_accessorials                           = $loadDetail->shipper_accessorials->whereIn('charge_deduct', [1, 2]);
                foreach ($shipper_accessorials as $key => $item) {
                    $name                                       = isset($item->company_accessorial->name) ? $item->company_accessorial->name : '';

                    $qty                                        = isset($item->qty) ? $item->qty : 0;
                    $qty_formatted                              = !empty($qty) ? formatNumberWithoutZeros($qty) : 0;

                    $rate                                       = isset($item->rate) ? $item->rate : 0;
                    $rate_formatted                             = !empty($rate) ? '$ ' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$ 0.00';

                    $total                                      = 0;
                    if ($item->charge_deduct == 1) {
                        $total                                  += $item->total;
                    } else if ($item->charge_deduct == 2) {
                        $total                                  -= $item->total;
                    }
                    if ($total >= 0) {
                        $total_formatted                        = '$ ' . formatNumberWithoutZeros($total, ['removeZeros' => false]);

                        $accessorial_amount                     += $total;
                    } else {
                        $rate_formatted                         = !empty($rate) ? '$ -' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$ 0.00';
                        $total_formatted                        = '$ -' . formatNumberWithoutZeros(abs($total), ['removeZeros' => false]);

                        $accessorial_amount                     -= abs($total);
                    }

                    // Add Stop HTML to array
                    $stopHtmlArray['Stop 1'][]                  = '
                        <tr>
                            <td style="vertical-align:top; width:40%">' . htmlspecialchars($name) . '</td>
                            <td style="vertical-align:top; width:120px">' . htmlspecialchars($qty_formatted) . '</td>
                            <td style="vertical-align:top; width:120px">' . htmlspecialchars($rate_formatted) . '</td>
                            <td style="vertical-align:top; width:130px">' . htmlspecialchars($total_formatted) . '</td>
                        </tr>
                    ';
                }
            }
            // Detention
            $shipper_detention                                  = isset($loadDetail->shipper_detention) ? $loadDetail->shipper_detention : null;
            if (!is_null($shipper_detention)) {
                if (in_array($shipper_detention->detention_status, [2, 3]) && $shipper_detention->review_status != 'Denied') {
                    $billed_time                                = $shipper_detention->billed_time ?? '00:00';

                    $rate                                       = isset($shipper_detention->rate) ? $shipper_detention->rate : 0;
                    $rate_formatted                             = !empty($rate) ? '$ ' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$ 0.00';

                    $detention_due                              = isset($shipper_detention->detention_due) ? $shipper_detention->detention_due : 0;
                    $detention_due_formatted                    = !empty($detention_due) ? '$ ' . formatNumberWithoutZeros($detention_due, ['removeZeros' => false]) : '$ 0.00';

                    $detention_amount                           += (float)$detention_due;

                    // Add Stop HTML to array
                    $stopHtmlArray['Stop 1'][]                  = '
                        <tr>
                            <td style="vertical-align:top; width:40%">' . htmlspecialchars('Detention') . '</td>
                            <td style="vertical-align:top; width:120px">' . htmlspecialchars($billed_time) . '</td>
                            <td style="vertical-align:top; width:120px">' . htmlspecialchars($rate_formatted) . '</td>
                            <td style="vertical-align:top; width:130px">' . htmlspecialchars($detention_due_formatted) . '</td>
                        </tr>
                    ';
                }
            }

            $stopNo                                             = 2;
            // Extra Stop
            if (isset($loadDetail->loads->load_stops) && $loadDetail->loads->load_stops->isNotEmpty()) {
                foreach ($loadDetail->loads->load_stops as $key => $stop) {
                    // Accessorial
                    if (isset($stop->accessorials) && $stop->accessorials->isNotEmpty()) {
                        $stop_accessorials                      = $stop->accessorials->whereIn('charge_deduct', [1, 2]);
                        foreach ($stop_accessorials as $key => $item) {
                            $name                               = isset($item->company_accessorial->name) ? $item->company_accessorial->name : '';

                            $qty                                = isset($item->qty) ? $item->qty : 0;
                            $qty_formatted                      = !empty($qty) ? formatNumberWithoutZeros($qty) : 0;

                            $rate                               = isset($item->rate) ? $item->rate : 0;
                            $rate_formatted                     = !empty($rate) ? '$ ' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$ 0.00';

                            $total                              = 0;
                            if ($item->charge_deduct == 1) {
                                $total                          += $item->total;
                            } else if ($item->charge_deduct == 2) {
                                $total                          -= $item->total;
                            }
                            if ($total >= 0) {
                                $total_formatted                = '$ ' . formatNumberWithoutZeros($total, ['removeZeros' => false]);

                                $accessorial_amount             += $total;
                            } else {
                                $rate_formatted                 = !empty($rate) ? '$ -' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$ 0.00';
                                $total_formatted                = '$ -' . formatNumberWithoutZeros(abs($total), ['removeZeros' => false]);

                                $accessorial_amount             -= abs($total);
                            }

                            // Add Stop HTML to array
                            $stopHtmlArray['Stop ' . $stopNo][]   = '
                                <tr>
                                    <td style="vertical-align:top; width:40%">' . htmlspecialchars($name) . '</td>
                                    <td style="vertical-align:top; width:120px">' . htmlspecialchars($qty_formatted) . '</td>
                                    <td style="vertical-align:top; width:120px">' . htmlspecialchars($rate_formatted) . '</td>
                                    <td style="vertical-align:top; width:130px">' . htmlspecialchars($total_formatted) . '</td>
                                </tr>
                            ';
                        }
                    }
                    // Detention
                    $stop_detention                             = isset($stop->detention) ? $stop->detention : null;
                    if (!is_null($stop_detention)) {
                        if (in_array($stop_detention->detention_status, [2, 3]) && $stop_detention->review_status != 'Denied') {
                            $billed_time                        = $stop_detention->billed_time ?? '00:00';

                            $rate                               = isset($stop_detention->rate) ? $stop_detention->rate : 0;
                            $rate_formatted                     = !empty($rate) ? '$ ' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$ 0.00';

                            $detention_due                      = isset($stop_detention->detention_due) ? $stop_detention->detention_due : 0;
                            $detention_due_formatted            = !empty($detention_due) ? '$ ' . formatNumberWithoutZeros($detention_due, ['removeZeros' => false]) : '$ 0.00';

                            $detention_amount                   += (float)$detention_due;

                            // Add Stop HTML to array
                            $stopHtmlArray['Stop ' . $stopNo][]   = '
                                <tr>
                                    <td style="vertical-align:top; width:40%">' . htmlspecialchars('Detention') . '</td>
                                    <td style="vertical-align:top; width:120px">' . htmlspecialchars($billed_time) . '</td>
                                    <td style="vertical-align:top; width:120px">' . htmlspecialchars($rate_formatted) . '</td>
                                    <td style="vertical-align:top; width:130px">' . htmlspecialchars($detention_due_formatted) . '</td>
                                </tr>
                            ';
                        }
                    }
                    ++$stopNo;
                }
            }

            // Accessorial
            if (isset($loadDetail->receiver_accessorials) && $loadDetail->receiver_accessorials->isNotEmpty()) {
                $receiver_accessorials                          = $loadDetail->receiver_accessorials->whereIn('charge_deduct', [1, 2]);
                foreach ($receiver_accessorials as $key => $item) {
                    $name                                       = isset($item->company_accessorial->name) ? $item->company_accessorial->name : '';

                    $qty                                        = isset($item->qty) ? $item->qty : 0;
                    $qty_formatted                              = !empty($qty) ? formatNumberWithoutZeros($qty) : 0;

                    $rate                                       = isset($item->rate) ? $item->rate : 0;
                    $rate_formatted                             = !empty($rate) ? '$ ' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$ 0.00';

                    $total                                      = 0;
                    if ($item->charge_deduct == 1) {
                        $total                                  += $item->total;
                    } else if ($item->charge_deduct == 2) {
                        $total                                  -= $item->total;
                    }
                    if ($total >= 0) {
                        $total_formatted                        = '$ ' . formatNumberWithoutZeros($total, ['removeZeros' => false]);

                        $accessorial_amount                     += $total;
                    } else {
                        $rate_formatted                         = !empty($rate) ? '$ -' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$ 0.00';
                        $total_formatted                        = '$ -' . formatNumberWithoutZeros(abs($total), ['removeZeros' => false]);

                        $accessorial_amount                     -= abs($total);
                    }

                    // Add Stop HTML to array
                    $stopHtmlArray['Stop 99'][]                 = '
                        <tr>
                            <td style="vertical-align:top; width:40%">' . htmlspecialchars($name) . '</td>
                            <td style="vertical-align:top; width:120px">' . htmlspecialchars($qty_formatted) . '</td>
                            <td style="vertical-align:top; width:120px">' . htmlspecialchars($rate_formatted) . '</td>
                            <td style="vertical-align:top; width:130px">' . htmlspecialchars($total_formatted) . '</td>
                        </tr>
                    ';
                }
            }
            // Detention
            $delivery_detention                                 = isset($loadDetail->receiver_detention) ? $loadDetail->receiver_detention : null;
            if (!is_null($delivery_detention)) {
                if (in_array($delivery_detention->detention_status, [2, 3]) && $delivery_detention->review_status != 'Denied') {
                    $billed_time                                = $delivery_detention->billed_time ?? '00:00';

                    $rate                                       = isset($delivery_detention->rate) ? $delivery_detention->rate : 0;
                    $rate_formatted                             = !empty($rate) ? '$ ' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$ 0.00';

                    $detention_due                              = isset($delivery_detention->detention_due) ? $delivery_detention->detention_due : 0;
                    $detention_due_formatted                    = !empty($detention_due) ? '$ ' . formatNumberWithoutZeros($detention_due, ['removeZeros' => false]) : '$ 0.00';

                    $detention_amount                           += (float)$detention_due;

                    // Add Stop HTML to array
                    $stopHtmlArray['Stop 99'][]                 = '
                        <tr>
                            <td style="vertical-align:top; width:40%">' . htmlspecialchars('Detention') . '</td>
                            <td style="vertical-align:top; width:120px">' . htmlspecialchars($billed_time) . '</td>
                            <td style="vertical-align:top; width:120px">' . htmlspecialchars($rate_formatted) . '</td>
                            <td style="vertical-align:top; width:130px">' . htmlspecialchars($detention_due_formatted) . '</td>
                        </tr>
                    ';
                }
            }

            // Accessorial
            if (isset($loadDetail->billing_accessorials) && $loadDetail->billing_accessorials->isNotEmpty()) {
                $billing_accessorials                           = $loadDetail->billing_accessorials->whereIn('charge_deduct', [1, 2]);
                foreach ($billing_accessorials as $key => $item) {
                    $name                                       = isset($item->company_accessorial->name) ? $item->company_accessorial->name : '';

                    $qty                                        = isset($item->qty) ? $item->qty : 0;
                    $qty_formatted                              = !empty($qty) ? formatNumberWithoutZeros($qty) : 0;

                    $rate                                       = isset($item->rate) ? $item->rate : 0;
                    $rate_formatted                             = !empty($rate) ? '$ ' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$ 0.00';

                    $total                                      = 0;
                    if ($item->charge_deduct == 1) {
                        $total                                  += $item->total;
                    } else if ($item->charge_deduct == 2) {
                        $total                                  -= $item->total;
                    }
                    if ($total >= 0) {
                        $total_formatted                        = '$ ' . formatNumberWithoutZeros($total, ['removeZeros' => false]);

                        $accessorial_amount                     += $total;
                    } else {
                        $rate_formatted                         = !empty($rate) ? '$ -' . formatNumberWithoutZeros($rate, ['removeZeros' => false]) : '$ 0.00';
                        $total_formatted                        = '$ -' . formatNumberWithoutZeros(abs($total), ['removeZeros' => false]);

                        $accessorial_amount                     -= abs($total);
                    }

                    // Add Stop HTML to array
                    $stopHtmlArray['Load'][]                    = '
                        <tr>
                            <td style="vertical-align:top; width:40%">' . htmlspecialchars($name) . '</td>
                            <td style="vertical-align:top; width:120px">' . htmlspecialchars($qty_formatted) . '</td>
                            <td style="vertical-align:top; width:120px">' . htmlspecialchars($rate_formatted) . '</td>
                            <td style="vertical-align:top; width:130px">' . htmlspecialchars($total_formatted) . '</td>
                        </tr>
                    ';
                }
            }

            // Append all stop to rows
            if (!empty($stopHtmlArray)) {
                foreach ($stopHtmlArray as $stopName => $stopHtmls) {
                    if (!empty($stopHtmls)) {
                        foreach ($stopHtmls as $iteration => $stopHtml) {
                            // Check if the current iteration number is 0 (indicating the first iteration)
                            if ($iteration === 0) {
                                // Add Row HTML to array
                                $rowHtmlArray[]                 = '
                                    <tr>
                                        <td colspan="4" style="vertical-align:top"><strong>' . htmlspecialchars($stopName) . '</strong></td>
                                    </tr>
                                ';
                            }
                            // Add Row HTML to array
                            $rowHtmlArray[]                     = $stopHtml;
                        }
                    }
                }
            }

            // Use DOMDocument to manipulate HTML
            $dom                                                = new \DOMDocument();
            $dom->loadHTML($content);
            // Find the table with the id 'item-table'
            $table                                              = $dom->getElementById('item-table');
            if ($table) {
                // Calculate the number of columns
                $num_columns                                    = 0;
                $header_row                                     = $table->getElementsByTagName('thead')->item(0)->getElementsByTagName('tr')->item(0);
                if ($header_row) {
                    $num_columns                                = $header_row->getElementsByTagName('th')->length;
                }

                if ($num_columns == 4) {
                    // Find the tbody element
                    $tbody                                      = $table->getElementsByTagName('tbody')->item(0);

                    if (!$tbody) {
                        // If tbody doesn't exist, create it
                        $tbody                                  = $dom->createElement('tbody');
                        $table->appendChild($tbody);
                    } else {
                        // Clear existing tbody content
                        while ($tbody->hasChildNodes()) {
                            $tbody->removeChild($tbody->firstChild);
                        }
                    }

                    // Append all rows to tbody
                    foreach ($rowHtmlArray as $rowHtml) {
                        $fragment                               = $dom->createDocumentFragment();
                        $fragment->appendXML($rowHtml); // Ensure each row HTML is properly parsed
                        $tbody->appendChild($fragment);
                    }
                } else {
                    // If the number of columns is not 4, remove any existing tbody content
                    $existing_tbody                             = $table->getElementsByTagName('tbody')->item(0);
                    if ($existing_tbody) {
                        $existing_tbody->parentNode->removeChild($existing_tbody);
                    }
                }
            }
            // Get the updated HTML content
            $content                                            = $dom->saveHTML();
            // Template

            // Dynamic Pdf
            $company_logo                                       = '<img src="' . public_path('theme/img/DispatchPro-Logo.svg') . '" alt="Company Logo" style="width:65px; margin:0" />'; // '<img src="' . asset('theme/img/DispatchPro-Logo.svg') . '" alt="Company Logo" style="width:65px; margin:0" />';
            $company_name                                       = $company->name ?? '';
            $company_street                                     = $company->address ?? '';
            $company_city                                       = $company->city ?? '';
            $company_state                                      = $company->state ?? '';
            $company_zip                                        = $company->zip ?? '';
            $company_phone                                      = $company->phone ?? '';
            $company_fax                                        = '';

            $invoice_number                                     = $loadInvoice->invoice_number ?? '';
            $qr_code                                            = ''; // '<img src="' . public_path('theme/img/qr-code.png') . '" alt="QR Code" style="max-width:130px; width:130px" />';
            if($loadInvoice->add_qr_code == 1){
                if(isset($company->logo) && $company->logo != '' && Storage::disk('public')->exists('companyAdmin/company_logo/' . $company->logo)){
                    $clogo                                      = Storage::disk('public')->path('companyAdmin/company_logo/' . $company->logo);
                }else{
                    $clogo                                      = '';
                }
                if(!empty($clogo)){
                    $qrSrc                                      = 'data:image/png;base64, '.base64_encode(QrCode::format('png')->style('dot')->eye('square')->size(300)->merge($clogo, 0.90, true)->errorCorrection('H')->generate(route('company.admin.broker.invoice.view', ['type' => 3, 'id' => base64Encode($loadInvoice->id), 'format' => 0])));
                }else{
                    $qrSrc                                      = 'data:image/png;base64, '.base64_encode(QrCode::format('png')->style('dot')->eye('square')->size(300)->errorCorrection('H')->generate(route('company.admin.broker.invoice.view', ['type' => 3, 'id' => base64Encode($loadInvoice->id), 'format' => 0])));
                }
                $qr_code                                        = '<img src="' . $qrSrc . '" alt="QR Code" style="max-width:130px; width:130px" />';
            }

            $customer_logo                                      = '<img src="' . public_path('theme/img/avtar.png') . '" alt="Customer Logo" style="width:55px" />'; // '<img src="' . asset('theme/img/avtar.png') . '" alt="Customer Logo" style="width:55px" />';
            $broker_logo                                        = $loadDetail->broker->customer_logo ?? '';
            if ($broker_logo != '') {
                if (Storage::disk('public')->exists('companyAdmin/broker/customer_logo/' . $broker_logo)) {
                    $broker_logo                                = Storage::disk('public')->path('companyAdmin/broker/customer_logo/' . $broker_logo); // Storage::disk('public')->url('companyAdmin/broker/customer_logo/'.$broker_logo);
                    if ($broker_logo != '') {
                        $customer_logo                          = '<img src="' . $broker_logo . '" alt="Customer Logo" style="width:55px" />';
                    }
                }
            }
            $bill_to_name                                       = $loadInvoice->bill_to_name ?? '';
            // $bill_to                                         = $loadInvoice->bill_to ?? '';
            $billing_address                                    = $loadInvoice->billing_address ?? '';
            $billing_city                                       = $loadInvoice->billing_city ?? '';
            $billing_state                                      = $loadInvoice->billing_state ?? '';
            $billing_zip                                        = $loadInvoice->billing_zip ?? '';

            $origin                                             = [];
            if (isset($loadDetail->shipper_city) && $loadDetail->shipper_city != '') {
                $origin[]                                       = $loadDetail->shipper_city;
            }
            if (isset($loadDetail->shipper_state) && $loadDetail->shipper_state != '') {
                $origin[]                                       = $loadDetail->shipper_state;
            }
            $origin                                             = implode(', ', $origin);
            $origin_date                                        = $loadDetail->shipper_date_time ?? '';

            $destination                                        = [];
            if (isset($loadDetail->delivery_city) && $loadDetail->delivery_city != '') {
                $destination[]                                  = $loadDetail->delivery_city;
            }
            if (isset($loadDetail->delivery_state) && $loadDetail->delivery_state != '') {
                $destination[]                                  = $loadDetail->delivery_state;
            }
            $destination                                        = implode(', ', $destination);
            $destination_date                                   = $loadDetail->delivery_date_time ?? '';

            $commodity_name                                     = '';
            $weight                                             = '';

            if (isset($loadDetail->itemize) && $loadDetail->itemize != 1) {
                $commodity_name                                 = $loadDetail->load_commodity->name ?? '';
                $weight                                         = isset($loadDetail->weight) ? formatNumberWithoutZeros($loadDetail->weight) . ' lbs' : '';
            }

            $load_type                                          = '';
            if (isset($loadDetail->load_type_2)) {
                if ($loadDetail->load_type_2 == 1) {
                    $load_type                                  = 'LTL';
                } else {
                    $load_type                                  = 'Full Truck';
                }
            }

            $notes                                              = $loadInvoice->notes ?? '';

            // Total Accessorial Amount
            $accessorial_amount_formatted                       = !empty($accessorial_amount) ? '$ ' . formatNumberWithoutZeros($accessorial_amount, ['removeZeros' => false]) : '$ 0.00';
            // Total Detention Amount
            $detention_amount_formatted                         = !empty($detention_amount) ? '$ ' . formatNumberWithoutZeros($detention_amount, ['removeZeros' => false]) : '$ 0.00';

            $invoice_total                                      = $line_haul + $fuel_surcharge + $accessorial_amount + $detention_amount;
            $invoice_total_formatted                            = !empty($invoice_total) ? '$ ' . formatNumberWithoutZeros($invoice_total, ['removeZeros' => false]) : '$ 0.00';

            $amount_paid                                        = 0;
            $pay_invoices                                       = $loadDetail->load_billing_pay_invoices ?? null;
            if (!is_null($pay_invoices)) {
                $paidInvoiceTotal                               = $pay_invoices->sum('total_payment') ?? 0;
                $chargebackAgainstBalance                       = $pay_invoices->pluck('chargeback_against_balance');
                $chargebackAmountsData                          = $pay_invoices->pluck('chargeback_amount');
                $chargebackAmounts                              = array_sum($chargebackAmountsData->map(function ($values, $key) use ($chargebackAgainstBalance) {
                    if (count($values) > 0) {
                        $amounts                                = [];
                        foreach ($values as $k => $v) {
                            if (isset($chargebackAgainstBalance[$key][$k]) && $chargebackAgainstBalance[$key][$k] == '1') {
                                $amounts[]                      = $v;
                            }
                        }
                        return array_sum($amounts);
                    }
                    return 0;
                })->toArray());

                $amount_paid                                    = round(($paidInvoiceTotal + $chargebackAmounts), 2);
            }
            $balance_due                                        = $invoice_total - $amount_paid;
            $balance_due_formatted                              = !empty($balance_due) ? '$ ' . formatNumberWithoutZeros($balance_due, ['removeZeros' => false]) : '$ 0.00';

            $factor_invoices                                    = isset($loadDetail->broker) && $loadDetail->broker->factor_invoices == '1' ? brokerFactoringCompany()[$loadDetail->broker->factor_company] : '';

            $terms                                              = '';
            if (isset($loadInvoice->payments_terms_days) && $loadInvoice->payments_terms_days != '' && isset(getPaymentsTermsDays()[$loadInvoice->payments_terms_days])) {
                $terms                                          = 'Net ' . getPaymentsTermsDays()[$loadInvoice->payments_terms_days];
            }
            $invoice_date                                       = $loadInvoice->invoice_date ?? '';
            $due_date                                           = $loadInvoice->due_date ?? '';

            // Define the strings to be replaced
            $find                                               = [
                '{%company_logo%}',
                '{%company_name%}',
                '{%company_street%}',
                '{%company_city%}',
                '{%company_state%}',
                '{%company_zip%}',
                '{%company_phone%}',
                '{%company_fax%}',
                '{%invoice_number%}',
                '{%qr_code%}',
                '{%customer_logo%}',
                '{%customer_name%}',
                '{%customer_address%}',
                '{%customer_city%}',
                '{%customer_state%}',
                '{%customer_zip%}',
                '{%origin%}',
                '{%origin_date%}',
                '{%destination%}',
                '{%destination_date%}',
                '{%commodity_name%}',
                '{%weight%}',
                '{%load_type%}',
                '{%notes%}',
                '{%linehaul_amount%}',
                '{%fuel_surcharge%}',
                '{%accessorial_amount%}',
                '{%detention_amount%}',
                '{%invoice_total%}',
                '{%factor_invoices%}',
                '{%terms%}',
                '{%invoice_date%}',
                '{%due_date%}',
                '{%balance_due%}'
            ];
            // Define the replacement strings
            $replace                                            = [
                $company_logo,
                $company_name,
                $company_street,
                $company_city,
                $company_state,
                $company_zip,
                $company_phone,
                $company_fax,
                $invoice_number,
                $qr_code,
                $customer_logo,
                $bill_to_name,
                $billing_address,
                $billing_city,
                $billing_state,
                $billing_zip,
                $origin,
                $origin_date,
                $destination,
                $destination_date,
                $commodity_name,
                $weight,
                $load_type,
                $notes,
                $line_haul_formatted,
                $fuel_surcharge_formatted,
                $accessorial_amount_formatted,
                $detention_amount_formatted,
                $invoice_total_formatted,
                $factor_invoices,
                $terms,
                $invoice_date,
                $due_date,
                $balance_due_formatted
            ];
            // Perform case-insensitive replacement
            $content                                            = Str::replace($find, $replace, $content);

            // Render Laravel view to HTML
            // $html = view('pdf_template', ['content' => $content])->render();

            // Echo the HTML for inspection
            // echo $html;

            // Stop further execution
            // die();

            $pdf                                                = PDF::loadHTML($content)->setPaper('a4');

            // Set options for the PDF (optional)
            $pdf->setOptions([
                // 'dpi'                => 150,
                'isHtml5ParserEnabled'  => true,
                'fontDir'               => public_path('theme/fonts/Inter/static/') // Add your font directory here
            ]);

            if (isset($request->format)) {
                if ($request->format == 1) {
                    // Return PDF with headers for download
                    return $pdf->download($invoice_number . '.pdf');
                }
            }
            return $pdf->stream($invoice_number . '.pdf');
        } else {
            return abort(404);
        }
        /*
        $loadInvoice                                            = LoadBillingInvoice::findOrFail(base64Decode($request->id));
        if($loadInvoice){
            if($request->type == '1'){
                $emailViewed                                    = LoadBillingHistory::where('load_detail_id', $loadInvoice->load_detail_id)->where('type', 'viewed_email')->count();
                if($emailViewed == 0){
                    LoadBillingHistory::create([
                        'load_detail_id'                        => $loadInvoice->load_detail_id,
                        'type'                                  => 'viewed_email',
                        'date_time'                             => Carbon::now()->format('m/d/Y H:i:s'),
                        'action_by'                             => 'User',
                        'extra_params'                          => array(
                            'type'                              => 'Email',
                            'ip_address'                        => $request->ip() ?? '',
                        )
                    ]);
                }
            }else if($request->type == '2'){
                $smsViewed                                      = LoadBillingHistory::where('load_detail_id', $loadInvoice->load_detail_id)->where('type', 'viewed_sms')->count();
                if($smsViewed == 0){
                    LoadBillingHistory::create([
                        'load_detail_id'                        => $loadInvoice->load_detail_id,
                        'type'                                  => 'viewed_sms',
                        'date_time'                             => Carbon::now()->format('m/d/Y H:i:s'),
                        'action_by'                             => 'User',
                        'extra_params'                          => array(
                            'type'                              => 'SMS',
                            'ip_address'                        => $request->ip() ?? '',
                        )
                    ]);
                }
            }

            $loadDetail                                         = LoadDetail::findOrFail($loadInvoice->load_detail_id);

            $invoiceData                                        = [];
            $invoiceData['company_name']                        = $loadDetail->loads->company->name ?? '';

            $compAddress                                        = '';
            if(isset($loadDetail->loads->company->address) && $loadDetail->loads->company->address != ''){
                $compAddress                                    .= $loadDetail->loads->company->address.',<br>';
            }
            if(isset($loadDetail->loads->company->address) && $loadDetail->loads->company->city != ''){
                $compAddress                                    .= $loadDetail->loads->company->city.', ';
            }
            if(isset($loadDetail->loads->company->address) && $loadDetail->loads->company->state != ''){
                $compAddress                                    .= $loadDetail->loads->company->state.', ';
            }
            if(isset($loadDetail->loads->company->address) && $loadDetail->loads->company->zip != ''){
                $compAddress                                    .= $loadDetail->loads->company->zip;
            }
            $invoiceData['company_address']                     = $compAddress ?? '';
            $invoiceData['company_phone']                       = $loadDetail->loads->company->phone ?? '';

            $invoiceData['invoice_number']                      = $loadInvoice->invoice_number;

            $billingAddress                                     = '';
            if(isset($loadDetail->broker->billing_address) && $loadDetail->broker->billing_address != ''){
                $billingAddress                                 .= $loadDetail->broker->billing_address.',<br>';
            }
            if(isset($loadDetail->broker->billing_address) && $loadDetail->broker->city != ''){
                $billingAddress                                 .= $loadDetail->broker->city.', ';
            }
            if(isset($loadDetail->broker->billing_address) && $loadDetail->broker->state != ''){
                $billingAddress                                 .= $loadDetail->broker->state;
            }
            $invoiceData['billing_address']                     = $billingAddress ?? '';
            $invoiceData['customer_logo']                       = $loadDetail->broker->customer_logo ?? '';

            $invoiceData['origin']                              = $loadDetail->shipper_city.', '.$loadDetail->shipper_state.'<br>'.$loadDetail->shipper_date_time;
            $invoiceData['destination']                         = $loadDetail->delivery_city.', '.$loadDetail->delivery_state.'<br>'.$loadDetail->delivery_date_time;
            $invoiceData['commodity']                           = getCompanyAdminCommodityList($loadDetail->loads->company_id)[$loadDetail->commodity] ?? '';
            $invoiceData['weight']                              = number_format($loadDetail->weight);
            if($loadDetail->load_type_2 == '0') {
                $invoiceData['load_type']                       = 'Full Truck';
            } else if($loadDetail->load_type_2 == '1'){
                $invoiceData['load_type']                       = 'LTL';
            }

            $invoiceData['line_haul']                           = $loadDetail->line_haul ?? '0.00';

            $invoiceData['loaded_miles']                        = $loadDetail->loaded_miles ?? '0.00';
            $invoiceData['fuel_surcharge_cost_per_mile']        = $loadDetail->fuel_surcharge_cost_per_mile ?? '0.00';
            $invoiceData['fuel_surcharge']                      = $loadDetail->fuel_surcharge ?? '0.00';

            $invoiceData['accessorialsTotal']                   = 0.00;

            $invoiceData['accessorials']                        = '';
            foreach($loadDetail->load_accessorials as $loadAccessorial){
                $invoiceData['accessorialsTotal']               = $invoiceData['accessorialsTotal'] + $loadAccessorial->total;
                $invoiceData['accessorials']                    .= '<tr>';
                $invoiceData['accessorials']                    .= '<td style=" padding: 2px 2px 2px 15px" colspan="2">'.getAccessorials($loadDetail->loads->company_id)[$loadAccessorial->company_accessorial_id].'</td>';
                $invoiceData['accessorials']                    .= '<td style="padding: 2px 2px 2px 2px;">'.$loadAccessorial->qty.'</td>';
                $invoiceData['accessorials']                    .= '<td style="padding: 2px 2px 2px 2px;">$ '.$loadAccessorial->rate.'</td>';
                $invoiceData['accessorials']                    .= '<td style="padding: 2px 15px 2px 2px" colspan="2">$ '.$loadAccessorial->total.'</td>';
                $invoiceData['accessorials']                    .= '</tr>';
            }

            $invoiceData['loadDetention']                       = '';
            foreach($loadDetail->detentions as $loadDetention){
                $invoiceData['accessorialsTotal']               = $invoiceData['accessorialsTotal'] + $loadDetention->detention_due;

                if($loadDetention->is_stop == 0 && $loadDetention->type == 1){
                    $invoiceData['loadDetention']               .= '<tr>';
                    $invoiceData['loadDetention']               .= '<td style=" padding: 2px 2px 2px 15px" colspan="2">Detention - Stop 1</td>';
                    $invoiceData['loadDetention']               .= '<td style="padding: 2px 2px 2px 2px;">'.$loadDetention->billed_time.'</td>';
                    $invoiceData['loadDetention']               .= '<td style="padding: 2px 2px 2px 2px;">$ '.$loadDetention->rate.'</td>';
                    $invoiceData['loadDetention']               .= '<td style="padding: 2px 15px 2px 2px" colspan="2">$ '.$loadDetention->detention_due.'</td>';
                    $invoiceData['loadDetention']               .= '</tr>';
                }else if($loadDetention->is_stop == 0 && $loadDetention->type == 2){
                    $invoiceData['loadDetention']               .= '<tr>';
                    $invoiceData['loadDetention']               .= '<td style=" padding: 2px 2px 2px 15px" colspan="2">Detention - Stop 99</td>';
                    $invoiceData['loadDetention']               .= '<td style="padding: 2px 2px 2px 2px;">'.$loadDetention->billed_time.'</td>';
                    $invoiceData['loadDetention']               .= '<td style="padding: 2px 2px 2px 2px;">$ '.$loadDetention->rate.'</td>';
                    $invoiceData['loadDetention']               .= '<td style="padding: 2px 15px 2px 2px" colspan="2">$ '.$loadDetention->detention_due.'</td>';
                    $invoiceData['loadDetention']               .= '</tr>';
                }else{
                    $invoiceData['loadDetention']               .= '<tr>';
                    $invoiceData['loadDetention']               .= '<td style=" padding: 2px 2px 2px 15px" colspan="2">Detention - Stop '.($loadDetention->stop->stop_no + 1).'</td>';
                    $invoiceData['loadDetention']               .= '<td style="padding: 2px 2px 2px 2px;">'.$loadDetention->billed_time.'</td>';
                    $invoiceData['loadDetention']               .= '<td style="padding: 2px 2px 2px 2px;">$ '.$loadDetention->rate.'</td>';
                    $invoiceData['loadDetention']               .= '<td style="padding: 2px 15px 2px 2px" colspan="2">$ '.$loadDetention->detention_due.'</td>';
                    $invoiceData['loadDetention']               .= '</tr>';
                }
            }

            $invoiceData['detention_approved_by']               = (count($loadDetail->detentions) > 0) ? CompanyAdmin::find($loadDetail->detentions->pluck('approved_by')[0])->full_name : '';
            $invoiceData['totalInvoice']                        = round($invoiceData['line_haul'] + $invoiceData['fuel_surcharge'] + $invoiceData['accessorialsTotal'], 2);
            $invoiceData['factor_invoices']                     = ($loadDetail->broker->factor_invoices == '1') ? brokerFactoringCompany()[$loadDetail->broker->factor_company] : '';
            $invoiceData['invoice_date']                        = $loadInvoice->invoice_date;
            $invoiceData['due_date']                            = $loadInvoice->due_date;

            return view('companyAdmin.billing.invoice', ['invoiceData' => $invoiceData]); // View Pdf HTMl page
        }else{
            abort(404);
        }
        */
    }

    public function invoiceShow($id)
    {
        $loadInvoice = LoadBillingInvoice::findOrFail(base64Decode($id));
        $loadDetail = LoadDetail::findOrFail($loadInvoice->load_detail_id);

        $invoiceTopNotes = [];
        foreach ($loadDetail->broker->internal_notes as $internal_notes) {
            if (in_array('3', $internal_notes->add_to)) {
                $invoiceTopNotes[] = $internal_notes->notes;
            }
        }

        $invoiceData = $this->generateInvoice($loadDetail);

        $paidInvoiceAmount = $this->updateForReceivePayment($loadDetail);

        return view('companyAdmin.billing.show-invoice', compact('loadInvoice', 'loadDetail', 'invoiceTopNotes', 'invoiceData', 'paidInvoiceAmount'));
    }

    public function invoiceResend(Request $request)
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $input = $request->all();

                $loadInvoice = LoadBillingInvoice::findOrFail(base64Decode($input['invoice']));
                $loadDetail = LoadDetail::findOrFail($loadInvoice->load_detail_id);
                $customer = Broker::whereId($loadInvoice->customer)->first();

                if (!empty($customer)) {
                    $contactDetails = [];

                    if ($input['send_by'] == '0' || $input['send_by'] == '1') {
                        if($input['send_by'] == '1'){
                            $contactDetails['email'] = $input['billing_email'] ?? null;
                        }else{
                            $contactDetails['email'] = str_replace(' ', '', $loadInvoice->invoice_email) ?? null;
                            if (isset($loadInvoice->invoice_email_cc) && $loadInvoice->invoice_email_cc != '') {
                                $contactDetails['cc'] = explode(',', str_replace(' ', '', $loadInvoice->invoice_email_cc));
                            }
                            if (isset($loadInvoice->invoice_email_bcc) && $loadInvoice->invoice_email_bcc != '') {
                                $contactDetails['bcc'] = explode(',', str_replace(' ', '', $loadInvoice->invoice_email_bcc));
                            }
                        }

                        if (isset($contactDetails['email']) && $contactDetails['email'] != '') {
                            $mailData = [];
                            $mailData['company_id'] = auth()->guard('company-admin')->user()->company_id;
                            $mailData['name'] = $loadInvoice->bill_to_name;
                            $mailData['load_number'] = $loadDetail->load_number;
                            $mailData['invoice_id'] = $loadInvoice->id;
                            $mailData['invoice_number'] = $loadInvoice->invoice_number;
                            $mailData['invoice_due_date'] = $loadInvoice->due_date;
                            $mailData['invoice_amount'] = '$ ' . number_format($loadInvoice->invoice_amount, 2);

                            sendEmail($contactDetails, new SendLoadInvoice($mailData));

                            LoadBillingHistory::create([
                                'load_detail_id' => $loadDetail->id,
                                'type' => 'email_re_sent',
                                'date_time' => Carbon::now()->format('m/d/Y H:i:s'),
                                'action_by' => auth()->guard('company-admin')->user()->full_name ?? '',
                                'extra_params' => array(
                                    'type' => 'Email',
                                    'to' => $loadInvoice->billingFullAddress() ?? '',
                                    'email' => $contactDetails['email'],
                                )
                            ]);
                        }
                    }

                    if ($input['send_by'] == '0' || $input['send_by'] == '2') {
                        if($input['send_by'] == '2'){
                            $contactDetails['phone'] = $input['billing_phone'] ?? null;
                        }else{
                            $contactDetails['phone'] = $customer->billing_phone ?? null;
                        }

                        if (isset($contactDetails['phone']) && $contactDetails['phone'] != '') {
                            // Twilio's credentials are fetched from company table
                            $getCompanyTwilioData = getCompanyTwilioData();
                            if (isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes') {
                                sendSMS($contactDetails['phone'], 'Invoice Created for load number: ' . $loadDetail->load_number . '. Here\'s your invoice! We appreciate your prompt payment. Invoice Due Date: ' . $loadInvoice->due_date . '. Invoice Amount: $ ' . $loadInvoice->invoice_amount . '. Print & Save: ' . route(
                                    'company.admin.broker.invoice.view',
                                    [
                                        'type'  => 2, // Type => // 0 = None, 1 = Email, 2 = SMS, 3 = Scan QR, 4 = Shared Link
                                        'id'    => base64Encode($loadInvoice->id),
                                        'format' => 1 // Format => // 0 = Stream, 1 = Save means Download, 2 = Print
                                    ]
                                ), $getCompanyTwilioData);

                                LoadBillingHistory::create([
                                    'load_detail_id' => $loadDetail->id,
                                    'type' => 'sms_re_sent',
                                    'date_time' => Carbon::now()->format('m/d/Y H:i:s'),
                                    'action_by' => auth()->guard('company-admin')->user()->full_name ?? '',
                                    'extra_params' => array(
                                        'type' => 'SMS',
                                        'to' => $contactDetails['phone'],
                                    )
                                ]);
                            }
                        }
                    }

                    $response['billingPaymentHistory'] = view('companyAdmin.billing.billing-payment-history', compact('loadDetail'))->render();
                    $response['success'] = true;
                    $response['message'] = "Invoice Send Successfully.";
                } else {
                    $response['success'] = false;
                    $response['message'] = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }

        return response()->json($response);
    }

    public function editInvoice(Request $request)
    {
        $loadInvoice = LoadBillingInvoice::findOrFail(base64Decode($request->id));
        $loadDetail = LoadDetail::findOrFail($loadInvoice->load_detail_id);
        $invoiceTopNotes = [];
        foreach ($loadDetail->broker->internal_notes as $internal_notes) {
            if (in_array('3', $internal_notes->add_to)) {
                $invoiceTopNotes[] = $internal_notes->notes;
            }
        }
        $invoiceData = $this->generateInvoice($loadDetail);

        $paidInvoiceAmount = $this->updateForReceivePayment($loadDetail);

        return view('companyAdmin.billing.edit-invoice', compact('loadInvoice', 'invoiceTopNotes', 'invoiceData', 'loadDetail', 'paidInvoiceAmount'));
    }

    public function updateInvoice(Request $request, $id)
    {
        $input = $request->except(['_method', '_token', 'invoice_number']);
        $input['add_qr_code'] = $input['add_qr_code'] ?? '0';
        $input['send_payment_reminder'] = $input['send_payment_reminder'] ?? '0';

        $invoiceUpdated = LoadBillingInvoice::updateOrCreate(['id' => base64Decode($id)], $input);

        if ($invoiceUpdated) {
            LoadBillingHistory::create([
                'load_detail_id' => $invoiceUpdated->load_detail_id,
                'type' => 'updated_invoice',
                'date_time' => Carbon::now()->format('m/d/Y H:i:s'),
                'action_by' => auth()->guard('company-admin')->user()->full_name ?? ''
            ]);

            notification('success', 'Invoice updated successfully.');
        } else {
            notification('error', 'Oops! Something went wrong!');
        }

        return redirect()->route('company.admin.billing.invoice.show', base64Encode($invoiceUpdated->id));
    }

    public function loadNoteSave(Request $request)
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $input = $request->except(['_token']);

                $create = LoadBillingCollectionsHistory::create($input);

                if ($create) {
                    $format = (getTimeFormat() == '12') ? config('settings.time_format.12_hour') : config('settings.time_format.24_hour');

                    $loadBillingCollectionsHistories = LoadBillingCollectionsHistory::where('load_detail_id', $input['load_detail_id'])->get();

                    $tableHtml = '';
                    foreach ($loadBillingCollectionsHistories as $loadBillingCollectionsHistory) {
                        $loadBillingCollectionsHistory->time = ($loadBillingCollectionsHistory->time != '') ? Carbon::parse($loadBillingCollectionsHistory->time)->format($format) : '';

                        $tableHtml .= '<tr>';
                        $tableHtml .= '<td class="">' . $loadBillingCollectionsHistory->type . '</td>';
                        $tableHtml .= '<td class="">' . $loadBillingCollectionsHistory->date . '</td>';
                        $tableHtml .= '<td class="">' . $loadBillingCollectionsHistory->time . '</td>';
                        $tableHtml .= '<td class="">' . $loadBillingCollectionsHistory->contact . '</td>';
                        $tableHtml .= '<td class="">' . $loadBillingCollectionsHistory->last_contact . '</td>';
                        $tableHtml .= '<td class="">' . $loadBillingCollectionsHistory->description . '</td>';
                        $tableHtml .= '</tr>';
                    }

                    $response['tableHtml'] = $tableHtml;
                    $response['success'] = true;
                    $response['message'] = "Collection note added successfully.";
                } else {
                    $response['success'] = false;
                    $response['message'] = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }

        return response()->json($response);
    }

    public function loadReminderView(Request $request)
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $viewType = $request->viewType ?? '';
                $loadDetailId = $request->loadDetailId ?? '';

                $loadDetail = LoadDetail::findOrFail($loadDetailId);

                $reminderId = $request->reminder ?? '';
                $reminderDetails = ($reminderId != '') ? LoadBillingReminderHistory::findOrFail($reminderId) : null;

                $view = view('companyAdmin.billing.reminder-modal-view', compact('viewType', 'loadDetail', 'reminderDetails'))->render();

                $response['html'] = $view ?? '';
                $response['success'] = true;
                $response['message'] = "Reminder added successfully.";
            } catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }

        return response()->json($response);
    }

    public function loadReminderSave(Request $request)
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $input = $request->except(['_token']);

                $data = [];
                $data['load_detail_id'] = $input['load_detail_id'] ?? '';
                $data['date'] = $input['reminder_date'] ?? '';
                $data['time'] = $input['reminder_time'] ?? '';
                $data['cc'] = $input['reminder_cc'] ?? null;
                $data['repeat'] = $input['reminder_repeat'] ?? '0';

                if ($data['repeat'] != '0') {
                    $data['reminder_repeat_every'] = $input['reminder_repeat_every'] ?? null;
                    if (!is_null($data['reminder_repeat_every']) && $data['reminder_repeat_every'] == 'custom') {
                        $data['reminder_repeat_every'] = $input['reminder_repeat_custom'];
                    }
                } else {
                    $data['reminder_repeat_every'] = null;
                }
                $data['next_reminder_repeat_at'] = Carbon::parse($data['date'] . ' ' . $data['time']);

                $data['instruction'] = $input['reminder_instructions'] ?? '';
                $data['created_on'] = Carbon::now()->format('m/d/Y');
                $data['created_by'] = auth()->guard('company-admin')->user()->id;

                if (isset($input['reminder_id']) && $input['reminder_id'] != '') {
                    unset($data['load_detail_id']);
                    //unset($data['next_reminder_repeat_at']);
                    unset($data['created_on']);
                    unset($data['created_by']);

                    $createOrUpdate = LoadBillingReminderHistory::updateOrCreate(['id' => $input['reminder_id'], 'load_detail_id' => $input['load_detail_id']], $data);
                } else {
                    $createOrUpdate = LoadBillingReminderHistory::create($data);
                }

                if ($createOrUpdate) {
                    $format = (getTimeFormat() == '12') ? config('settings.time_format.12_hour') : config('settings.time_format.24_hour');

                    $loadDetail = LoadDetail::findOrFail($input['load_detail_id']);
                    $loadBillingReminderHistories = LoadBillingReminderHistory::where('load_detail_id', $input['load_detail_id'])->get();

                    $tableHtml = '';
                    foreach ($loadBillingReminderHistories as $loadBillingReminderHistory) {
                        $loadBillingReminderHistory->time = ($loadBillingReminderHistory->time != '') ? Carbon::parse($loadBillingReminderHistory->time)->format($format) : '';

                        $tableHtml .= '<tr>';
                        $tableHtml .= '<td class="">' . $loadBillingReminderHistory->date . '</td>';
                        $tableHtml .= '<td class="">' . $loadBillingReminderHistory->time . '</td>';
                        if ($loadBillingReminderHistory->repeat == '1') {
                            $tableHtml .= '<td class="">On</td>';
                            $tableHtml .= '<td class="">' . (reminderRepeatFrequency()[$loadBillingReminderHistory->reminder_repeat_every] ?? '') . '</td>';
                        } else {
                            $tableHtml .= '<td class="">Off</td>';
                            $tableHtml .= '<td class="">-</td>';
                        }
                        if (!empty($loadBillingReminderHistory->cc)) {
                            $tableHtml .= '<td class="">';
                            foreach ($loadBillingReminderHistory->cc as $cc) {
                                $tableHtml .= '<span class="textBdadge">' . (getCompanyAdminListByCompany()[$cc] ?? '') . '<button type="button" class="close" aria-label="Close"><i class="fa fa-close"></i></button></span>';
                            }
                            $tableHtml .= '</td>';
                        } else {
                            $tableHtml .= '<td class="">-</td>';
                        }
                        $tableHtml .= '<td class="" style="text-wrap: pretty !important;">' . $loadBillingReminderHistory->instruction . '</td>';
                        $tableHtml .= '<td class="">' . $loadBillingReminderHistory->created_on . '</td>';
                        $tableHtml .= '<td class="">' . getCompanyAdminListByCompany()[$loadBillingReminderHistory->created_by] . '</td>';
                        $tableHtml .= '<td class="">';
                        $tableHtml .= '<div class="d-flex">';
                        $tableHtml .= '<a href="javascript:void(0);" class="mx-1 text-theme-color view-reminder" data-load-number="' . $loadDetail->load_number . '" data-load-detail="' . $loadDetail->id . '" data-reminder="' . $loadBillingReminderHistory->id . '"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                        $tableHtml .= '<a href="javascript:void(0);" class="mx-1 text-theme-color editReminderModal" data-load-number="' . $loadDetail->load_number . '" data-load-detail="' . $loadDetail->id . '" data-reminder="' . $loadBillingReminderHistory->id . '"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                        //$tableHtml .= '<a href="javascript:void(0);" class="mx-1 text-theme-color"><i class="fa fa-refresh" aria-hidden="true"></i></a>';
                        $tableHtml .= '<a href="javascript:void(0);" class="mx-1 text-theme-color deleteReminder" data-load-detail="' . $loadDetail->id . '" data-reminder="' . $loadBillingReminderHistory->id . '"><i class="fa fa-trash" aria-hidden="true"></i></a>';
                        $tableHtml .= '</div>';
                        $tableHtml .= '</td>';
                        $tableHtml .= '</tr>';
                    }

                    $response['tableHtml'] = $tableHtml;
                    $response['success'] = true;
                    $response['message'] = "Reminder saved successfully.";
                } else {
                    $response['success'] = false;
                    $response['message'] = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }

        return response()->json($response);
    }

    public function loadReminderDelete(Request $request)
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $input = $request->except(['_token']);

                $delete = LoadBillingReminderHistory::where('id', $input['reminder'])->where('load_detail_id', $input['loadDetailId'])->delete();

                if ($delete) {
                    $format = (getTimeFormat() == '12') ? config('settings.time_format.12_hour') : config('settings.time_format.24_hour');

                    $loadDetail = LoadDetail::findOrFail($input['loadDetailId']);
                    $loadBillingReminderHistories = LoadBillingReminderHistory::where('load_detail_id', $input['loadDetailId'])->get();

                    $tableHtml = '';
                    foreach ($loadBillingReminderHistories as $loadBillingReminderHistory) {
                        $loadBillingReminderHistory->time = ($loadBillingReminderHistory->time != '') ? Carbon::parse($loadBillingReminderHistory->time)->format($format) : '';

                        $tableHtml .= '<tr>';
                        $tableHtml .= '<td class="">' . $loadBillingReminderHistory->date . '</td>';
                        $tableHtml .= '<td class="">' . $loadBillingReminderHistory->time . '</td>';
                        if ($loadBillingReminderHistory->repeat == '1') {
                            $tableHtml .= '<td class="">On</td>';
                            $tableHtml .= '<td class="">' . (reminderRepeatFrequency()[$loadBillingReminderHistory->reminder_repeat_every] ?? '') . '</td>';
                        } else {
                            $tableHtml .= '<td class="">Off</td>';
                            $tableHtml .= '<td class="">-</td>';
                        }
                        if (!empty($loadBillingReminderHistory->cc)) {
                            $tableHtml .= '<td class="">';
                            foreach ($loadBillingReminderHistory->cc as $cc) {
                                $tableHtml .= '<span class="textBdadge">' . (getCompanyAdminListByCompany()[$cc] ?? '') . '<button type="button" class="close" aria-label="Close"><i class="fa fa-close"></i></button></span>';
                            }
                            $tableHtml .= '</td>';
                        } else {
                            $tableHtml .= '<td class="">-</td>';
                        }
                        $tableHtml .= '<td class="" style="text-wrap: pretty !important;">' . $loadBillingReminderHistory->instruction . '</td>';
                        $tableHtml .= '<td class="">' . $loadBillingReminderHistory->created_on . '</td>';
                        $tableHtml .= '<td class="">' . getCompanyAdminListByCompany()[$loadBillingReminderHistory->created_by] . '</td>';
                        $tableHtml .= '<td class="">';
                        $tableHtml .= '<div class="d-flex">';
                        $tableHtml .= '<a href="javascript:void(0);" class="mx-1 text-theme-color view-reminder" data-load-number="' . $loadDetail->load_number . '" data-load-detail="' . $loadDetail->id . '" data-reminder="' . $loadBillingReminderHistory->id . '"><i class="fa fa-eye" aria-hidden="true"></i></a>';
                        $tableHtml .= '<a href="javascript:void(0);" class="mx-1 text-theme-color editReminderModal" data-load-number="' . $loadDetail->load_number . '" data-load-detail="' . $loadDetail->id . '" data-reminder="' . $loadBillingReminderHistory->id . '"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                        //$tableHtml .= '<a href="javascript:void(0);" class="mx-1 text-theme-color"><i class="fa fa-refresh" aria-hidden="true"></i></a>';
                        $tableHtml .= '<a href="javascript:void(0);" class="mx-1 text-theme-color deleteReminder" data-load-detail="' . $loadDetail->id . '" data-reminder="' . $loadBillingReminderHistory->id . '"><i class="fa fa-trash" aria-hidden="true"></i></a>';
                        $tableHtml .= '</div>';
                        $tableHtml .= '</td>';
                        $tableHtml .= '</tr>';
                    }

                    $response['tableHtml'] = $tableHtml;
                    $response['success'] = true;
                    $response['message'] = "Reminder deleted successfully.";
                } else {
                    $response['success'] = false;
                    $response['message'] = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }

        return response()->json($response);
    }

    public function reviewLoadDocuments()
    {
        return view('companyAdmin.billing.review-document');
    }

    public function reviewLoadDocumentsView(Request $request)
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $input = $request->except(['_token']);

                $loadDetail = LoadDetail::findOrFail($input['loadDetailId']);

                if ($loadDetail) {
                    /* Billing Activity View */
                    $billingActivityView = view('companyAdmin.billing.activity', compact('loadDetail'))->render();
                    /* Billing Activity View */

                    /* Load Documents View */
                    $loadDocuments = getLoadDocuments($loadDetail);
                    $loadDocumentsView = view('companyAdmin.billing.load-documents-list', compact('loadDocuments'))->render();
                    /* Load Documents View */

                    $response['billingActivityView'] = $billingActivityView ?? '';
                    $response['loadDocumentsView'] = $loadDocumentsView ?? '';
                    $response['success'] = true;
                    $response['message'] = "Review load documents view get successfully.";
                } else {
                    $response['success'] = false;
                    $response['message'] = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }

        return response()->json($response);
    }

    public function getPayInvoiceView(Request $request)
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $input = $request->except(['_token']);
                $parentRowIndex = $input['parentRowIndex'] ?? '';

                $loadDetail = LoadDetail::findOrFail(base64Decode($input['loadDetailId']));

                if ($loadDetail) {
                    $invoiceAmount = $loadDetail->load_billing_invoice->invoice_amount ?? 0.00;

                    $paidInvoiceTotal = $loadDetail->load_billing_pay_invoices->sum('total_payment') ?? 0.00;
                    $chargebackAgainstBalance = $loadDetail->load_billing_pay_invoices->pluck('chargeback_against_balance');
                    $chargebackAmountsData = $loadDetail->load_billing_pay_invoices->pluck('chargeback_amount');
                    $chargebackAmounts = array_sum($chargebackAmountsData->map(function ($values, $key) use ($chargebackAgainstBalance) {
                        if (count($values) > 0) {
                            $amounts = [];
                            foreach ($values as $k => $v) {
                                if (isset($chargebackAgainstBalance[$key][$k]) && $chargebackAgainstBalance[$key][$k] == '1') {
                                    $amounts[] = $v;
                                }
                            }
                            return array_sum($amounts);
                        }
                        return 0;
                    })->toArray());

                    $invoiceAmount = round(($invoiceAmount - ($paidInvoiceTotal + $chargebackAmounts)), 2);

                    $payInvoiceView = view('companyAdmin.billing.pay-invoice', compact('loadDetail', 'invoiceAmount', 'parentRowIndex'))->render();

                    $response['html'] = $payInvoiceView ?? '';
                    $response['success'] = true;
                    $response['message'] = "Pay Invoice view get successfully.";
                } else {
                    $response['success'] = false;
                    $response['message'] = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }

        return response()->json($response);
    }

    public function getPaymentAccountsView(Request $request)
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $input = $request->except(['_token']);

                $loadDetail = LoadDetail::findOrFail(base64Decode($input['loadDetailId']));

                if ($loadDetail) {
                    $paymentAccountsView = view('companyAdmin.billing.payment-accounts', compact('loadDetail'))->render();

                    $response['html'] = $paymentAccountsView ?? '';
                    $response['success'] = true;
                    $response['message'] = "Payment Accounts view get successfully.";
                } else {
                    $response['success'] = false;
                    $response['message'] = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }

        return response()->json($response);
    }

    public function paymentAccountsSave(Request $request)
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $input = $request->except(['_token']);
                $input['account_status'] = $input['set_active'] ?? '0';

                if (isset($input['load_detail_id'])) {
                    $input['load_detail_id'] = base64Decode($input['load_detail_id']);

                    if ($input['account_status'] == '1') {
                        LoadBillingPaymentAccount::where('load_detail_id', $input['load_detail_id'])->update(['account_status' => '0']);
                    }

                    $save = LoadBillingPaymentAccount::create($input);
                } else if (isset($input['payment_account_id'])) {
                    $input['payment_account_id'] = base64Decode($input['payment_account_id']);

                    if (isset($input['remove_account']) && $input['remove_account'] == '1') {
                        $deleted = LoadBillingPaymentAccount::findOrFail($input['payment_account_id'])->delete();
                    } else {
                        if ($input['account_status'] == '1') {
                            $paymentAccount = LoadBillingPaymentAccount::findOrFail($input['payment_account_id']);
                            if ($paymentAccount) {
                                LoadBillingPaymentAccount::where('load_detail_id', $paymentAccount->load_detail_id)->update(['account_status' => '0']);
                            }
                        }

                        $save = LoadBillingPaymentAccount::updateOrCreate(['id' => $input['payment_account_id']], $input);
                    }
                }

                if (isset($deleted)) {
                    $response['success'] = true;
                    $response['message'] = "Payment Account deleted successfully.";
                } else if (!empty($save)) {
                    if (isset($input['set_as_default']) && $input['set_as_default'] == 'true') {
                        $htmlBody = '';

                        $paymentAccount = LoadBillingPaymentAccount::findOrFail($input['payment_account_id']);
                        $paymentAccounts = LoadBillingPaymentAccount::where('load_detail_id', $paymentAccount->load_detail_id)->get();

                        foreach ($paymentAccounts as $key => $paymentAccount) {
                            $htmlBody .= '<tr>';
                            $htmlBody .= '<td><a href="javascript:void(0);" class="btn-link" data-id="' . base64Encode($paymentAccount->id) . '" data-name="' . $paymentAccount->account_name . '" data-type="' . $paymentAccount->account_payment_type . '" data-number="' . $paymentAccount->account_number . '" data-exp="' . $paymentAccount->account_expiration . '" data-status="' . $paymentAccount->account_status . '" id="editPaymentAccount" data-dismiss="modal">Edit</a></td>';
                            $htmlBody .= '<td>' . ($paymentAccount->account_name ?? '') . '</td>';
                            $htmlBody .= '<td>' . (getPaymentTypes()[$paymentAccount->account_payment_type] ?? '') . '</td>';
                            $htmlBody .= '<td class="text-lowercase">' . ('xxxx' . $paymentAccount->account_number ?? '') . '</td>';
                            $htmlBody .= '<td>' . ($paymentAccount->account_expiration ?? '') . '</td>';
                            $htmlBody .= '<td>';
                            $htmlBody .= '<div class="form-check">';
                            $htmlBody .= '<input class="form-check-input" id="account-status-' . $key . '" data-id="' . base64Encode($paymentAccount->id) . '" name="account_status" type="checkbox" value="1" ' . (($paymentAccount->account_status == '1') ? "checked" : "") . '>';
                            $htmlBody .= '<label class="form-check-label pl-1" for="account-status-' . $key . '"></label>';
                            $htmlBody .= '</div>';
                            $htmlBody .= '</td>';
                            $htmlBody .= '<td>';
                            $htmlBody .= '<a href="javascript:void(0);">';
                            $htmlBody .= '<i class="fa fa-trash text-danger"></i>';
                            $htmlBody .= '</a>';
                            $htmlBody .= '</td>';
                            $htmlBody .= '</tr>';
                        }

                        $response['html'] = $htmlBody ?? '';
                    }

                    $response['success'] = true;
                    $response['message'] = "Payment Account save successfully.";
                } else {
                    $response['success'] = false;
                    $response['message'] = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }

        return response()->json($response);
    }

    public function paymentAccountsDelete(Request $request, $id)
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $paymentAccount = LoadBillingPaymentAccount::findOrFail(base64Decode($id));

                if ($paymentAccount) {
                    $loadDetailId = $paymentAccount->load_detail_id;
                    $paymentAccount->delete();

                    $paymentAccounts = LoadBillingPaymentAccount::where('load_detail_id', $loadDetailId)->get();

                    $htmlBody = '';
                    if (count($paymentAccounts) > 0) {
                        foreach ($paymentAccounts as $key => $paymentAccount) {
                            $htmlBody .= '<tr>';
                            $htmlBody .= '<td><a href="javascript:void(0);" class="btn-link" data-id="' . base64Encode($paymentAccount->id) . '" data-name="' . $paymentAccount->account_name . '" data-type="' . $paymentAccount->account_payment_type . '" data-number="' . $paymentAccount->account_number . '" data-exp="' . $paymentAccount->account_expiration . '" data-status="' . $paymentAccount->account_status . '" id="editPaymentAccount" data-dismiss="modal">Edit</a></td>';
                            $htmlBody .= '<td>' . ($paymentAccount->account_name ?? '') . '</td>';
                            $htmlBody .= '<td>' . (getPaymentTypes()[$paymentAccount->account_payment_type] ?? '') . '</td>';
                            $htmlBody .= '<td class="text-lowercase">' . ('xxxx' . $paymentAccount->account_number ?? '') . '</td>';
                            $htmlBody .= '<td>' . ($paymentAccount->account_expiration ?? '') . '</td>';
                            $htmlBody .= '<td>';
                            $htmlBody .= '<div class="form-check">';
                            $htmlBody .= '<input class="form-check-input" id="account-status-' . $key . '" data-id="' . base64Encode($paymentAccount->id) . '" name="account_status" type="checkbox" value="1" ' . (($paymentAccount->account_status == '1') ? "checked" : "") . '>';
                            $htmlBody .= '<label class="form-check-label pl-1" for="account-status-' . $key . '"></label>';
                            $htmlBody .= '</div>';
                            $htmlBody .= '</td>';
                            $htmlBody .= '<td>';
                            $htmlBody .= '<a href="javascript:void(0);">';
                            $htmlBody .= '<i class="fa fa-trash text-danger"></i>';
                            $htmlBody .= '</a>';
                            $htmlBody .= '</td>';
                            $htmlBody .= '</tr>';
                        }
                    } else {
                        $htmlBody .= '<tr>';
                        $htmlBody .= '<td colspan="7" class="text-center">No account found</td>';
                        $htmlBody .= '</tr>';
                    }

                    $response['html'] = $htmlBody ?? '';
                    $response['success'] = true;
                    $response['message'] = "Payment Account deleted successfully.";
                } else {
                    $response['success'] = false;
                    $response['message'] = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }

        return response()->json($response);
    }

    public function loadPayInvoice(Request $request)
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $input = $request->except(['_token']);
                $input['factor'] = $input['factor'] ?? 0;
                $input['chargeback'] = $input['chargeback'] ?? 0;

                if ($input['chargeback'] == 0) {
                    $input['chargeback_type'] = null;
                    $input['chargeback_user'] = null;
                    $input['chargeback_amount'] = null;
                    $input['chargeback_against_balance'] = null;
                }

                $payInvoice = LoadBillingPayInvoice::create($input);

                if ($payInvoice) {
                    if (isset($input['parent_row_index']) && $input['parent_row_index'] != '') {
                        $response['rowChildView'] = $this->getRowChildView(json_encode(['load_id' => $input['load_id'], 'load_detail_id' => $input['load_detail_id'], 'rowIndex' => $input['parent_row_index']])) ?? '';
                    } else {
                        $loadDetail = $payInvoice->load_detail;
                        $this->updateForReceivePayment($loadDetail);

                        $response['billingPaymentHistory'] = view('companyAdmin.billing.billing-payment-history', compact('loadDetail'))->render();
                    }

                    if (isset($payInvoice->load_detail->load_billing_activities) && $payInvoice->load_detail->load_billing_activities->paid_invoice == '1') {
                        $response['billStatus'] = 'paid-invoice';
                    } else {
                        $response['billStatus'] = 'unpaid-invoice';
                    }

                    $response['success'] = true;
                    $response['message'] = "Invoice paid successfully.";
                } else {
                    $response['success'] = false;
                    $response['message'] = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }

        return response()->json($response);
    }

    public function invoiceCreateLink(Request $request, $id)
    {
        $response['success'] = false;
        $response['message'] = "Oops! Something went wrong..";

        if ($request->ajax()) {
            try {
                $loadBillingInvoice = LoadBillingInvoice::findOrFail(base64Decode($id));

                if ($loadBillingInvoice) {
                    $loadDetail = LoadDetail::findOrFail($loadBillingInvoice->load_detail_id);

                    LoadBillingHistory::create([
                        'load_detail_id' => $loadDetail->id,
                        'type' => 'create_link',
                        'date_time' => Carbon::now()->format('m/d/Y H:i:s'),
                        'action_by' => auth()->guard('company-admin')->user()->full_name ?? ''
                    ]);

                    $invoiceLink = '<a href="javascript:void(0);" class="list-group-item list-group-item-action font-weight-bold" title="Click to Copy">' . route(
                        'company.admin.broker.invoice.view',
                        [
                            'type'  => 4, // Type => // 0 = None, 1 = Email, 2 = SMS, 3 = Scan QR, 4 = Shared Link
                            'id'    => base64Encode($loadBillingInvoice->id),
                            'format' => 0 // Format => // 0 = Stream, 1 = Save means Download, 2 = Print
                        ]
                    ) . '<i class="fa fa-copy font-16 float-right" aria-hidden="true"></i></a>';

                    $response['invoiceLink'] = $invoiceLink ?? '';
                    $response['billingPaymentHistory'] = view('companyAdmin.billing.billing-payment-history', compact('loadDetail'))->render();
                    $response['success'] = true;
                    $response['message'] = "Invoice Link created successfully.";
                } else {
                    $response['success'] = false;
                    $response['message'] = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success'] = false;
                $response['message'] = $e->getMessage();
            }
        }

        return response()->json($response);
    }
}
