<?php

namespace App\Http\Controllers\CompanyAdmin;

use App\Http\Controllers\CompanyAdmin\CompanyAdminController;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Database\Eloquent\Collection;

use App\Models\Load;
use App\Models\LoadStop;
use App\Models\LoadFile;
use App\Models\LoadDetail;
use App\Models\LoadAccessorial;
use App\Models\LoadDocumentRequest;
use App\Models\LoadDocumentRequestItem;

use App\Models\Gallery;

use App\Models\Offer;
use App\Models\OfferStop;
use App\Models\OfferHistory;

use App\Models\BrokerContact;

use App\Models\DispatchHubWidget;
use App\Models\DispatchHubSetting;
use App\Models\IntegrationSetting;

use App\Models\Driver;
use App\Models\Trailer;
use App\Models\Carrier;
use App\Models\Terminal;
use App\Models\Bookmark;

use App\Models\CompanyAdmin;

use App\Helpers\Common;
use App\Traits\AuditLoagTrait;

use DB;
use File;
use DataTables;

use Carbon\Carbon;
use Carbon\CarbonInterval;

use Imagick;
use Intervention\Image\Facades\Image;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ServerException;

class DispatchHubController extends CompanyAdminController{

    use AuditLoagTrait;

    public $company_admin;
    public $timeZone;
    public $timeZoneId;
    public $offset;
    public $dispatchHubSetting;

    // if change file upload logic than must check getLoadDocuments & historicDispatchDetail & historicFileSave & historicFileRemove & carousel_content & currentFileSave & currentFileRemove & additionalFileSave & departDetail - means file related all file check

    public function __construct(){
        parent::__construct();

        // Set the maximum execution time to 3600 seconds
        ini_set('max_execution_time', 0);

        // Set the maximum memory limit to 2048 megabytes
        ini_set('memory_limit', '2048M');

        $this->middleware(function ($request, $next) {
            $this->company_admin        = auth()->guard('company-admin')->user();
            $this->timeZone             = getTimeZone();
            $this->timeZoneId           = getTimeZoneId();
            $this->offset               = getTimeZoneOffset();
            $this->dispatchHubSetting   = DispatchHubSetting::IsCompanyAdmin()->first();

            return $next($request);
        });
    }

    /*
     * New Functionality - START
     */
    public function counterWidget($pendingAcceptance, $totalAvailableDrivers, $totalAvailableLoads){
        return [
            'Pending Acceptance'        => [
                'icon'                  => 'theme/images/dispatch-hub-file.svg',
                'count'                 => sprintf("%02d", $pendingAcceptance),
            ],
            'Available Drivers'         => [
                'icon'                  => 'theme/images/dispatch-hub-driver.png',
                'count'                 => sprintf("%02d", $totalAvailableDrivers),
            ],
            'Available Loads'           => [
                'icon'                  => 'theme/images/dispatch-hub-boxes.svg',
                'count'                 => sprintf("%02d", $totalAvailableLoads),
            ],
            'Service Failure'           => [
                'icon'                  => 'theme/images/dispatch-hub-delivery-time.svg',
                'count'                 => sprintf("%02d", 0),
            ],
            'Missed Checkcalls'         => [
                'icon'                  => 'theme/images/dispatch-hub-call.svg',
                'count'                 => sprintf("%02d", 0),
            ],
        ];
    }

    public function widgetData(Request $request){
        try {
            $user                       = $this->company_admin;

            $dataArray                  = json_decode($request->input('dataArr'), true);

            $insertArray                = [];
            foreach ($dataArray as $data){
                $isExist                = DispatchHubWidget::IsCompanyAdmin()->where('tab', 'counter')->count();
                if($isExist > 0){
                    DispatchHubWidget::IsCompanyAdmin()->where('tab', 'counter')->delete();
                }

                $insertArray[]          = [
                    'company_id'        => $user->company_id,
                    'company_admins_id' => $user->id,
                    'display_no'        => $data['displayNo'],
                    'widget_name'       => $data['itemName'],
                    'isActive'          => ($data['isActive'] == '1') ? '1' : '0',
                    'tab'               => 'counter',
                    'guard_name'        => 'company-admin',
                    'created_at'        => Carbon::now()->timezone($this->timeZone),
                ];
            }

            DispatchHubWidget::Insert($insertArray);

            $recordsCounts              = $this->getListRecordsCounts();
            $counterWidget              = $this->counterWidget($recordsCounts['pendingAcceptance'], $recordsCounts['totalAvailableDrivers'], $recordsCounts['totalAvailableLoads']);
            $activeComplianceWidgets    = DispatchHubWidget::IsCompanyAdmin()->where('tab', 'counter')->where('isActive', '1')->orderBy('display_no', 'ASC')->get()->toArray();

            $updatedContent             = '';
            foreach($activeComplianceWidgets as $widgets){
                $updatedContent        .= '<div class="col-xl-3 col-md-4 col-sm-6">';
                $updatedContent        .= '<div class="dispatchHubNew-box">';
                $updatedContent        .= '<div>';
                $updatedContent        .= '<h2>'.$counterWidget[$widgets['widget_name']]['count'].'</h2>';
                $updatedContent        .= '<p>'.$widgets['widget_name'].'</p>';
                $updatedContent        .= '</div>';
                $updatedContent        .= '<div><img src="'.asset($counterWidget[$widgets['widget_name']]['icon']).'" alt="Icon"></div>';
                $updatedContent        .= '</div>';
                $updatedContent        .= '</div>';
            }

            if(!empty($updatedContent)){
                $response['widgetData'] = $updatedContent;
                $response['success']    = true;
                $response['message']    = "Widget updated successfully.";
            }else{
                $response['success']    = false;
                $response['message']    = "Widget record not found.";
            }
        } catch (\Exception $e) {
            $response['success']        = false;
            $response['message']        = $e->getMessage();
        }
        return response()->json($response);
    }

    public function filterData(Request $request){
        try {
            $user                       = $this->company_admin;
            $isMerge                    = $request->input('isMerge');
            $teamArr                    = (isset($request->teamArr)) ? $request->input('teamArr') : [];
            $dispatcherArr              = (isset($request->dispatcherArr)) ? $request->input('dispatcherArr') : [];
            $equipmentArr               = (isset($request->equipmentArr)) ? $request->input('equipmentArr') : [];
            $carrierArr                 = (isset($request->carrierArr)) ? $request->input('carrierArr') : [];

            $filters                    = serialize(['teamArr' => $teamArr, 'dispatcherArr' => $dispatcherArr, 'equipmentArr' => $equipmentArr, 'carrierArr' => $carrierArr]);

            $isExist                    = DispatchHubSetting::IsCompanyAdmin()->count();

            if($isExist > 0){
                $getSettings            = $this->dispatchHubSetting;
                $getSettings->filters   = $filters;
                $getSettings->merge_compact_view = $isMerge;
                $getSettings->save();
            }else{
                DispatchHubSetting::create([
                    'company_id'        => $user->company_id,
                    'company_admins_id' => $user->id,
                    'filters'           => $filters,
                    'merge_compact_view'=> $isMerge,
                    'guard_name'        => 'company-admin',
                    'created_at'        => Carbon::now()->timezone($this->timeZone),
                ]);
            }

            $response['success']        = true;
            $response['message']        = "List updated successfully.";
        } catch (\Exception $e) {
            $response['success']        = false;
            $response['message']        = $e->getMessage();
        }
        return response()->json($response);
    }

    public function listingOrder(Request $request){
        try {
            $user                       = $this->company_admin;

            $dataArray                  = json_decode($request->input('dataArr'), true);

            $insertArray                = [];
            foreach ($dataArray as $data){
                $isExist                = DispatchHubWidget::IsCompanyAdmin()->where('tab', 'list')->count();
                if($isExist > 0){
                    DispatchHubWidget::IsCompanyAdmin()->where('tab', 'list')->delete();
                }

                $insertArray[]          = [
                    'company_id'        => $user->company_id,
                    'company_admins_id' => $user->id,
                    'display_no'        => $data['displayNo'],
                    'widget_name'       => $data['itemName'],
                    'isActive'          => '1',
                    'tab'               => 'list',
                    'guard_name'        => 'company-admin',
                    'created_at'        => Carbon::now()->timezone($this->timeZone),
                ];
            }

            DispatchHubWidget::Insert($insertArray);

            $response['success']        = true;
            $response['message']        = "Listing updated successfully.";
        } catch (\Exception $e) {
            $response['success']        = false;
            $response['message']        = $e->getMessage();
        }
        return response()->json($response);
    }

    public function getListingOrder(){
        $listingOrder                   = [];
        $dbListingDetails               = DispatchHubWidget::IsCompanyAdmin()
            ->where('tab', 'list')
            ->where(function($q) {
                $q->whereNotNull('widget_name')
                  ->where('widget_name', '!=', '')
                  ->whereNotNull('display_no')
                  ->where('display_no', '!=', '');
            })
            ->orderByRaw("FIELD(widget_name, 'On Hold', 'Available Drivers', 'PrePlanned Loads', 'In Transit', 'Available Loads', 'All Dispatch', 'Ready To Bill')")
            ->get(['display_no', 'widget_name as name'])
            ->toArray();

        if(count($dbListingDetails) > 0){
            $listingOrder               = $dbListingDetails;

            // List of valid names
            $validNames                 = ['On Hold', 'Available Drivers', 'PrePlanned Loads', 'In Transit', 'Available Loads', 'All Dispatch', 'Ready To Bill'];

            // Extract the names from the $dbListingDetails array
            $dbNames                    = array_map(function($detail) {
                return $detail['name'];
            }, $listingOrder);

            // Find names that are in $validNames but not in $dbNames
            $missingNames               = array_diff($validNames, $dbNames);

            // Find the highest display_no in the $dbListingDetails array
            $maxDisplayNo               = max(array_column($listingOrder, 'display_no'));

            // Add missing names to the $dbListingDetails array with incremented display_no
            foreach ($missingNames as $missingName) {
                $maxDisplayNo++;
                $listingOrder[]         = ['display_no' => $maxDisplayNo, 'name' => $missingName];
            }

            // Custom sorting function based on $validNames order
            usort($listingOrder, function($a, $b) use ($validNames) {
                $indexA                 = array_search($a['name'], $validNames);
                $indexB                 = array_search($b['name'], $validNames);
                return $indexA - $indexB;
            });

            // Reindex the array to start from 0 using array_values
            $listingOrder               = array_values($listingOrder);
        }else{
            $listingOrder               = [
                [
                    'display_no'        => 1,
                    'name'              => 'On Hold'
                ],
                [
                    'display_no'        => 2,
                    'name'              => 'Available Drivers'
                ],
                [
                    'display_no'        => 3,
                    'name'              => 'PrePlanned Loads'
                ],
                [
                    'display_no'        => 4,
                    'name'              => 'In Transit'
                ],
                [
                    'display_no'        => 5,
                    'name'              => 'Available Loads'
                ],
                [
                    'display_no'        => 6,
                    'name'              => 'All Dispatch'
                ],
                [
                    'display_no'        => 7,
                    'name'              => 'Ready To Bill'
                ]
            ];
        }
        return $listingOrder;
    }

    public function getListRecordsCounts(){
        $pendingAcceptance              = LoadDetail::join('drivers','drivers.id','=','load_details.driver_id')
            ->leftjoin('teams','teams.id','=','drivers.team_id')
            ->where('load_status', 0)
            ->where(function($q) {
                $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
            })
            ->where(function($q) {
                $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
            })
            ->whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->count();

        $totalOnHoldLoads               = LoadDetail::join('drivers','drivers.id','=','load_details.driver_id')
            ->leftjoin('teams','teams.id','=','drivers.team_id')
            ->where('is_on_hold', 1)
            ->where(function($q) {
                $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
            })
            ->whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->count();

        $totalPlannedLoads              = LoadDetail::join('drivers','drivers.id','=','load_details.driver_id')
            ->leftjoin('teams','teams.id','=','drivers.team_id')
            ->whereIn('load_status',[0,2,3])
            ->where(function($q) {
                $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
            })
            ->where(function($q) {
                $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
            })
            ->whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->count();

        $totalInTransitLoads            = LoadDetail::join('drivers','drivers.id','=','load_details.driver_id')
            ->leftjoin('teams','teams.id','=','drivers.team_id')
            ->where('load_status', 4)
            ->where(function($q) {
                $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
            })
            ->where(function($q) {
                $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
            })
            ->whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->count();

        $totalCompletedLoads            = LoadDetail::join('drivers','drivers.id','=','load_details.driver_id')
            ->leftjoin('teams','teams.id','=','drivers.team_id')
            ->where('load_status', 5)
            ->where(function($q) {
                $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
            })
            ->where(function($q) {
                $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
            })
            ->whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->count();

        /* [{ On Hold: 1 }, { PrePlanned: 0,2,3 }, { In Transit: 4 }] */
        $totalAllDispatch               = LoadDetail::join('drivers','drivers.id','=','load_details.driver_id')
            ->leftjoin('teams','teams.id','=','drivers.team_id')
            ->where(function($query) {
                $query->where('load_details.is_on_hold', 1);
            })->orWhere(function($query) {
                $query->whereIn('load_details.load_status',[0,2,3,4])->where(function($q) {
                    $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                });
            })
            ->where(function($q) {
                $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
            })
            ->whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->count();

        $totalAvailableLoads            = LoadDetail::where(function($q) {
                $q->whereNull('load_details.driver_id')->orWhere('load_details.driver_id', 0);
            })
            ->where('load_details.load_type', 0)
            ->where('load_details.is_on_hold', '!=', 2)
            ->whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->count();

        $totalAvailableDrivers          = Driver::query()
            ->leftJoin('teams', function($join) {
                $join->on('drivers.team_id', '=', 'teams.id');
            })
            ->leftJoin('driver_informations', function($join) {
                $join->on('drivers.id', '=', 'driver_informations.driver_id');
            })
            ->leftJoin('load_details', function($join) { // Not Complete
                $join->on('drivers.id', '=' , 'load_details.driver_id')
                    ->where(function($query) {
                        $query->whereNotIn('load_details.load_status', [5,6,10,8])
                            ->where(function($q) {
                                $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                            })
                            ->where('load_details.is_disable', 0)
                            ->whereNull('load_details.deleted_at');
                    });
            })
            ->leftJoin('load_details as load_details_al',function ($join) { // Complete
                $join->on('drivers.id', '=' , 'load_details_al.driver_id')
                    ->where(function($query) {
                        $query->where(function($q) {
                                $q->whereIn('load_details_al.load_status', [5,6,10,8])->orWhereIn('load_details_al.is_on_hold', [1,2]);
                            })
                            ->where('load_details_al.is_disable', 0)
                            ->whereNull('load_details_al.deleted_at')
                            ->where('load_details_al.actual_delivery_date', '=', DB::raw("(select MAX(a2.actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id AND (a2.load_status IN(5,6,10,8) OR a2.is_on_hold IN(1,2)) AND a2.is_disable = 0 AND a2.deleted_at IS NULL)"));
                    });
            })
            ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
            ->whereNull('load_details.id')
            ->groupBy('drivers.id')
            ->get()
            ->count();

        return [
            'pendingAcceptance'         => $pendingAcceptance,
            'totalOnHoldLoads'          => $totalOnHoldLoads,
            'totalPlannedLoads'         => $totalPlannedLoads,
            'totalInTransitLoads'       => $totalInTransitLoads,
            'totalCompletedLoads'       => $totalCompletedLoads,
            'totalAllDispatch'          => $totalAllDispatch,
            'totalAvailableLoads'       => $totalAvailableLoads,
            'totalAvailableDrivers'     => $totalAvailableDrivers
        ];
    }

    public function childContentArray($rowData){
        $childDataArr                   = array(
            'load_id'                   => $rowData->load_id,
            'load_status'               => $rowData->load_status,
            'broker_name'               => (!empty($rowData->broker)) ? $rowData->broker->name : null,
            'broker_phone_number'       => (!empty($rowData->broker)) ?  $rowData->broker->phone_number : null,
            'broker_contact_name'       => (!empty($rowData->broker) && isset($rowData->broker->broker_contact) &&  !is_null($rowData->broker->broker_contact->first())) ? $rowData->broker->broker_contact->first()->name : null,
            'broker_email'              => (!empty($rowData->broker)) ? $rowData->broker->email : null,
            'carrier_name'              => (!empty($rowData->driver) && isset($rowData->driver->driver_information) && isset($rowData->driver->driver_information->carrier)) ? $rowData->driver->driver_information->carrier->name : null,
            'carrier_mc_number'         => (!empty($rowData->driver) && isset($rowData->driver->driver_information) && isset($rowData->driver->driver_information->carrier)) ? $rowData->driver->driver_information->carrier->mc_number : null,
            'terminal_name'             => (!empty($rowData->terminal)) ? $rowData->terminal->name : null,
            'driver_team_name'          => (!empty($rowData->driver) && isset($rowData->driver->team)) ? $rowData->driver->team->name : null,
            'dispatcher_name'           => (!empty($rowData->dispatcher) && isset($rowData->dispatcher->full_name)) ? $rowData->dispatcher->full_name : null,
            'dispatcher_cell_phone'     => (!empty($rowData->dispatcher) && isset($rowData->dispatcher->cell_phone)) ? $rowData->dispatcher->cell_phone : null,
            'dispatcher_email'          => (!empty($rowData->dispatcher) && isset($rowData->dispatcher->email)) ? $rowData->dispatcher->email : null,
            'driver_name'               => (!empty($rowData->driver) && isset($rowData->dispatcher->full_name)) ? $rowData->driver->full_name : null,
            'co_driver_name'            => (!empty($rowData->co_driver) && isset($rowData->co_driver->full_name)) ? $rowData->co_driver->full_name : null,
            'driver_cell_phone'         => (!empty($rowData->driver) && isset($rowData->driver->cell_phone)) ? $rowData->driver->cell_phone : null,
            'driver_email'              => (!empty($rowData->driver) && isset($rowData->driver->email)) ? $rowData->driver->email : null,
            'truck_number'              => $rowData->truck_number,
            'trailer_number'            => $rowData->trailer_number,
            'driver_id'                 => (!empty($rowData->driver) && isset($rowData->driver->id)) ? $rowData->driver->id : null,
            'load_stop_count'           => (!empty($rowData->loads) && !empty($rowData->loads->load_stops)) ? count($rowData->loads->load_stops) : null,
            'total_miles'               => $rowData->total_miles,
            'agreed_upon_rate'          => $rowData->agreed_upon_rate,
            'shipper_name'              => (!empty($rowData->shipper) && !empty($rowData->shipper->name)) ? $rowData->shipper->name : null,
            'shipper_address_one'       => $rowData->shipper_address_one,
            'shipper_address_two'       => $rowData->shipper_address_two,
            'shipper_city'              => $rowData->shipper_city,
            'shipper_state'             => $rowData->shipper_state,
            'shipper_zip'               => $rowData->shipper_zip,
            'shipper_location'          => $rowData->shipper_location,
            'shipper_dispatch_contact'  => (!empty($rowData->shipper) && !empty($rowData->shipper->dispatch_contact)) ? $rowData->shipper->dispatch_contact : null,
            'shipper_date_time'         => $rowData->shipper_date_time,
            'shipper_phone'             => (!empty($rowData->shipper) && !empty($rowData->shipper->phone)) ? $rowData->shipper->phone : null,
            'shipper_email'             => (!empty($rowData->shipper) && !empty($rowData->shipper->email)) ? $rowData->shipper->email : null,
            'receiver_name'             => (!empty($rowData->receiver) && !empty($rowData->receiver->name)) ? $rowData->receiver->name : null,
            'delivery_address_one'      => $rowData->delivery_address_one,
            'delivery_address_two'      => $rowData->delivery_address_two,
            'delivery_city'             => $rowData->delivery_city,
            'delivery_state'            => $rowData->delivery_state,
            'delivery_zip'              => $rowData->delivery_zip,
            'delivery_location'         => $rowData->delivery_location,
            'receiver_dispatch_contact' => (!empty($rowData->receiver) && !empty($rowData->receiver->dispatch_contact)) ? $rowData->receiver->dispatch_contact : null,
            'delivery_date_time'        => $rowData->delivery_date_time,
            'receiver_phone'            => (!empty($rowData->receiver) && !empty($rowData->receiver->phone)) ? $rowData->receiver->phone : null,
            'receiver_email'            => (!empty($rowData->receiver) && !empty($rowData->receiver->email)) ? $rowData->receiver->email : null,
        );

        return json_encode($childDataArr);
    }
    /*
     * New Functionality - END
     */

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request){
        $getCompanyAdmin                                = auth()->guard('company-admin')->user();
        $checkAPIKey                                    = IntegrationSetting::where('company_id', $getCompanyAdmin->company_id)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->whereNotNull('api_key')->first();

        // On Hold Loads
        $getField                                       = '';
        $isCheck                                        = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub1')->first();
        if(!empty($isCheck->id)){
            $getField                                   = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_dispatchhub1_first')); // unserialize($isCheck->config);
        }else{
            $getField                                   = config('settings.table_dispatchhub1_first');
        }

        // In Transit Loads
        $getField2                                      = '';
        $isCheck2                                       = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub2')->first();
        if(!empty($isCheck2->id)){
            $getField2                                  = adjustTableColumnsSorting(unserialize($isCheck2->config), config('settings.table_dispatchhub2_first')); // unserialize($isCheck2->config);
        }else{
            $getField2                                  = config('settings.table_dispatchhub2_first');
        }

        // Available Drivers
        $getField3                                      = '';
        $getField3Hide                                  = [];
        $isCheck3                                       = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub3')->first();
        if(!empty($isCheck3->id)){
            $getField3                                  = adjustTableColumnsSorting(unserialize($isCheck3->config), config('settings.table_dispatchhub3_first')); // unserialize($isCheck3->config);
            foreach (config('settings.table_dispatchhub3_first') as $dkey => $dispatchhub) {
                if(!array_key_exists($dkey,$getField3)){
                    $getField3[$dkey]                   = 0;
                }
            }
            if(is_null($checkAPIKey)){
                $getField3['drive']                     = 0;
                $getField3['shift']                     = 0;
                $getField3['cycle']                     = 0;
                $getField3['cycle_tom']                 = 0;
                $getField3['duty_status']               = 0;
                $getField3['time_in_current_status']    = 0;
                $getField3['time_in_unit_break']        = 0;
                $getField3Hide                          = ['drive','shift','cycle','cycle_tom','duty_status','time_in_current_status','time_in_unit_break'];
            }
        }else{
            $getField3                                  = config('settings.table_dispatchhub3_first');
            if(is_null($checkAPIKey)){
                $getField3['drive']                     = 0;
                $getField3['shift']                     = 0;
                $getField3['cycle']                     = 0;
                $getField3['cycle_tom']                 = 0;
                $getField3['duty_status']               = 0;
                $getField3['time_in_current_status']    = 0;
                $getField3['time_in_unit_break']        = 0;
                $getField3Hide                          = ['drive','shift','cycle','cycle_tom','duty_status','time_in_current_status','time_in_unit_break'];
            }
        }

        // Planned Loads
        $getField4                                      = '';
        $isCheck4                                       = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub4')->first();
        if(!empty($isCheck4->id)){
            $getField4                                  = adjustTableColumnsSorting(unserialize($isCheck4->config), config('settings.table_dispatchhub4_first')); // unserialize($isCheck4->config);
        }else{
            $getField4                                  = config('settings.table_dispatchhub4_first');
        }

        // Available Loads
        $getField5                                      = '';
        $isCheck5                                       = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub5')->first();
        if(!empty($isCheck5->id)){
            $getField5                                  = adjustTableColumnsSorting(unserialize($isCheck5->config), config('settings.table_dispatchhub5_first')); // unserialize($isCheck5->config);
        }else{
            $getField5                                  = config('settings.table_dispatchhub5_first');
        }

        // All Dispatch
        $getField7                                      = '';
        $isCheck7                                       = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub7')->first();
        if(!empty($isCheck7->id)){
            $getField7                                  = adjustTableColumnsSorting(unserialize($isCheck7->config), config('settings.table_dispatchhub7_first')); // unserialize($isCheck7->config);
        }else{
            $getField7                                  = config('settings.table_dispatchhub7_first');
        }

        // Ready to Bill Loads
        $getField6                                      = '';
        $isCheck6                                       = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub6')->first();
        if(!empty($isCheck6->id)){
            $getField6                                  = adjustTableColumnsSorting(unserialize($isCheck6->config), config('settings.table_dispatchhub6_first')); // unserialize($isCheck6->config);
        }else{
            $getField6                                  = config('settings.table_dispatchhub6_first');
        }

        $recordsCounts                                  = $this->getListRecordsCounts();
        $pendingAcceptance                              = sprintf("%02d", $recordsCounts['pendingAcceptance']);
        $totalOnHoldLoads                               = sprintf("%02d", $recordsCounts['totalOnHoldLoads']);
        $totalInTransitLoads                            = sprintf("%02d", $recordsCounts['totalInTransitLoads']);
        $totalAvailableDrivers                          = sprintf("%02d", $recordsCounts['totalAvailableDrivers']);
        $totalPlannedLoads                              = sprintf("%02d", $recordsCounts['totalPlannedLoads']);
        $totalAvailableLoads                            = sprintf("%02d", $recordsCounts['totalAvailableLoads']);
        $totalAllDispatch                               = sprintf("%02d", $recordsCounts['totalAllDispatch']);
        $totalCompletedLoads                            = sprintf("%02d", $recordsCounts['totalCompletedLoads']);

        $isDispatchHubPage                              = 1;

        $isMulitpleTable                                = 1;

        // $tables                                      = Common::getStaticDispatchTables();
        $tables                                         = array(
            0 => array(
                "name"                                  => 'table1', /// on holds
                "id"                                    => 'on-hold-load-collapsable',
                "class"                                 => 'on-hold-load-data-table',
                "table"                                 => 'settings.table_dispatchhub1',
                "route"                                 => route('company.admin.on.hold.loads.list'),
                "getField"                              => $getField,
            ),
            1 => array(
                "name"                                  => 'table2', /// Transit Loads
                "id"                                    => 'collapsable5',
                "class"                                 => 'in-transit-loads-data-table',
                "table"                                 => 'settings.table_dispatchhub2',
                "route"                                 => route('company.admin.in.transit.loads.list'),
                "getField"                              => $getField2,
            ),
            2 => array(
                "name"                                  => 'table3', // Available Drivers
                "id"                                    => 'collapsable2',
                "class"                                 => 'available-drivers-data-table',
                "table"                                 => 'settings.table_dispatchhub3',
                "route"                                 => route('company.admin.available.drivers.list'),
                "getField"                              => $getField3,
            ),
            3 => array(
                "name"                                  => 'table4', /// Planned
                "id"                                    => 'collapsable4',
                "class"                                 => 'planned-load-data-table',
                "table"                                 => 'settings.table_dispatchhub4',
                "route"                                 => route('company.admin.planned.loads.list'),
                "getField"                              => $getField4,
            ),
            4 => array(
                "name"                                  => 'table5', // Available loads
                "id"                                    => 'collapsable5',
                "class"                                 => 'available-load-data-table',
                "table"                                 => 'settings.table_dispatchhub5',
                "route"                                 => route('company.admin.available.loads.list'),
                "getField"                              => $getField5,
            ),
            5 => array(
                "name"                                  => 'table7', // All Dispatch
                "id"                                    => 'collapsable7',
                "class"                                 => 'all-dispatch-data-table',
                "table"                                 => 'settings.table_dispatchhub7',
                "route"                                 => route('company.admin.all.dispatch.loads.list'),
                "getField"                              => $getField7,
            ),
            6 => array(
                "name"                                  => 'table6', // Ready to Bill
                "id"                                    => 'collapsable6',
                "class"                                 => 'completed-load-data-table',
                "table"                                 => 'settings.table_dispatchhub6',
                "route"                                 => route('company.admin.completed.loads.list'),
                "getField"                              => $getField6,
            )
        );

        $tableName1                                     = 'settings.table_dispatchhub';

        /************************** New Functionality - Start ***************************/
        $counterWidget                                  = $this->counterWidget($pendingAcceptance, $totalAvailableDrivers, $totalAvailableLoads);
        $dbCounterWidget                                = DispatchHubWidget::IsCompanyAdmin()->where('tab', 'counter')->orderBy('display_no', 'ASC')->get()->toArray();

        $counterWidgetsList                             = array();
        if(count($dbCounterWidget) > 0){
            $lastDisplayNo                              = null;
            foreach ($dbCounterWidget as $widget){
                if(isset($counterWidget[$widget['widget_name']])){
                    $counterWidgetsList[]               = array(
                        'display_no'                    => $widget['display_no'],
                        'widget_name'                   => $widget['widget_name'],
                        'icon'                          => $counterWidget[$widget['widget_name']]['icon'],
                        'isActive'                      => $widget['isActive'],
                        'count'                         => $counterWidget[$widget['widget_name']]['count'],
                    );

                    $lastDisplayNo                      = $widget['display_no'] + 1;
                }
            }

            $newWidgets                                 = array_values(array_diff(array_keys($counterWidget), array_column($counterWidgetsList, 'widget_name')));

            if(count($newWidgets) > 0){
                foreach ($newWidgets as $widgetName){
                    $counterWidgetsList[]               = array(
                        'display_no'                    => $lastDisplayNo,
                        'widget_name'                   => $widgetName,
                        'icon'                          => $counterWidget[$widgetName]['icon'],
                        'isActive'                      => 0,
                        'count'                         => $counterWidget[$widgetName]['count'],
                    );
                }
            }
        }else{
            $i = 1;
            foreach ($counterWidget as $widgetName => $widgetDetails){
                $counterWidgetsList[]                   = array(
                    'display_no'                        => $i,
                    'widget_name'                       => $widgetName,
                    'icon'                              => $widgetDetails['icon'],
                    'isActive'                          => ($i <= 5) ? '1' : '0',
                    'count'                             => $widgetDetails['count'],
                );
                $i++;
            }
        }

        $getFilters                                     = $this->dispatchHubSetting;
        $filters                                        = ['teamArr' => [], 'dispatcherArr' => [], 'equipmentArr' => [], 'carrierArr' => []];
        $mergeCompactView                               = null;
        if(!is_null($getFilters)){
            $filters                                    = unserialize($getFilters->filters);
            $mergeCompactView                           = $getFilters->merge_compact_view;
        }

        $listingOrder                                   = $this->getListingOrder();
        /************************** New Functionality - End ***************************/

        return view('companyAdmin.dispatchHub.index', compact(
            'getField',
            'totalOnHoldLoads',
            'getField2',
            'totalInTransitLoads',
            'getField3',
            'getField3Hide',
            'totalAvailableDrivers',
            'getField4',
            'totalPlannedLoads',
            'getField5',
            'totalAvailableLoads',
            'getField7',
            'totalAllDispatch',
            'getField6',
            'totalCompletedLoads',
            'isDispatchHubPage',
            'isMulitpleTable',
            'tables',
            'tableName1',
            'counterWidgetsList',
            'filters',
            'mergeCompactView',
            'listingOrder'
        ));
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function onHoldLoadsList(Request $request){
        $getCompanyAdmin                = auth()->guard('company-admin')->user();

        $getField                       = '';
        $isCheck                        = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub1')->first();
        if(!empty($isCheck->id)){
            $getField                   = adjustTableColumnsSorting(unserialize($isCheck->config), config('settings.table_dispatchhub1_first')); // unserialize($isCheck->config);
        }else{
            $getField                   = config('settings.table_dispatchhub1_first');
        }

        if ($request->ajax()) {
            $getFilters                 = $this->dispatchHubSetting;
            $filters                    = ['teamArr' => [], 'dispatcherArr' => [], 'equipmentArr' => [], 'carrierArr' => []];
            if(!is_null($getFilters)){
                $filters                = unserialize($getFilters->filters);
            }

            $data                       = LoadDetail::select('load_details.*', 'brokers.name as brokerage_name')
                ->join('drivers','drivers.id','=','load_details.driver_id')
                ->leftjoin('teams','teams.id','=','drivers.team_id')
                ->leftjoin('brokers','brokers.id','=','load_details.brokerage_firm')
                ->whereHas('loads', function($q){
                    $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                });
                //->filter($request->all())
                if(isset($filters['teamArr']) && count($filters['teamArr']) > 0){
                    $data->whereIn('teams.id', $filters['teamArr']);
                }
                if(isset($filters['dispatcherArr']) && count($filters['dispatcherArr']) > 0){
                    $data->whereIn('drivers.manager_id', $filters['dispatcherArr']);
                }
                if(isset($filters['equipmentArr']) && count($filters['equipmentArr']) > 0){
                    $data->whereIn('load_details.trailer_type', $filters['equipmentArr']);
                }
                if(isset($filters['carrierArr']) && count($filters['carrierArr']) > 0){
                    $data->leftjoin("driver_informations", "driver_informations.driver_id", "=", "drivers.id")
                        ->whereIn('driver_informations.carrier_id', $filters['carrierArr']);
                }
                $data->where('load_details.is_on_hold', 1)
                ->where(function($q) {
                    $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                });

                // General filter code - start
                foreach( $getField as $key => $value ) {
                    $conditionReq       = 'search_string_condition_'.$key;
                    $sortReq            = 'search_string_sort_'.$key;

                    $conditionDateReq   = 'search_date_condition_'.$key;
                    $sortDateReq        = 'search_date_sort_'.$key;

                    $conditionStatusReq = 'search_status_condition_'.$key;
                    $sortStatusReq      = 'search_status_sort_'.$key;

                    // String filter
                    if( isset( request()->$conditionReq ) ) {
                        $typeReq        = 'search_string_type_'.$key;
                        $type           = request()->$typeReq;

                        if( $type == 'string' ) {
                            Common::getStringFilterResult( $data, $key );
                        }
                    }

                    // Status filter
                    if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                        Common::getStatusFilterResult( $data, $key );
                    }

                    // Date filter
                    if( isset( request()->$conditionDateReq ) ) {
                        $typeReq        = 'search_date_type_'.$key;
                        $type           = request()->$typeReq;

                        if( $type == 'date' ) {
                            Common::getDateFilterResult( $data, $key );
                        }
                    }

                    // String sort only
                    if( isset( request()->$sortReq ) ) {
                        Common::sortStringFilter( $data, $key );
                    }

                    // Date sort only
                    if( isset( request()->$sortDateReq ) ) {
                        Common::sortDateFilter( $data, $key );
                    }

                    // Status sort only
                    if( isset( request()->$sortStatusReq ) ) {
                        Common::sortStatusFilter( $data, $key );
                    }
                }
                // General filter code - end
                return Datatables::of($data)
                    ->addIndexColumn()
                    ->addColumn('status', function($row){
                        //return getLoadStatus($row->load_status, $row->is_on_hold);
                        if(!is_null($row->load_status)){
                            $html               = '<div class="status-block">';
                            $html               .= '<div class="round-icon details-control"><i class="fa fa-caret-down" aria-hidden="true"></i></div>';
                            if($row->is_on_hold == 1){
                                $html           .= updateStatusDropdown($row->load_id, '7', 'on-hold-load-data-table');
                            }else{
                                $html           .= updateStatusDropdown($row->load_id, $row->load_status, 'on-hold-load-data-table');
                            }
                            $html               .= '</div>';
                            return $html;
                        }
                        return '';
                    })
                    ->addColumn('load_number', function($row){
                        $html                   = '';
                        if(isset($row->load_lock_by_users) && isset($row->load_lock_by_users->company_admin->full_name) && $row->load_lock_by_users->company_admin->full_name != ''){
                            $html               .= '<i aria-hidden="true" class="fa fa-lock text-danger" title="'.$row->load_lock_by_users->company_admin->full_name.'"></i>&nbsp;';
                        }
                        if ($row->is_disable == 0) {
                            if(in_array($row->load_type, ['0', '6'])) {
                                if ($row->load_type == 0) {
                                    $html       .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                                } else {
                                    $html       .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                                }
                            } else {
                                $html           .= '<a href="'.route('company.admin.loads.edit', base64Encode($row->load_id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        } else {
                            if ($row->load_type == 0) {
                                $html           .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                            } else {
                                $html           .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        }
                        return $html;
                    })
                    ->addColumn('full_name', function($row){
                        if(isset($row->driver->id)){
                            return '<span class="show-available-driver" data-driver-id="'.$row->driver->id.'" data-toggle="modal" data-target="#availableDriverModal">'.$row->driver->full_name.'</span>';
                        }else{
                            return '';
                        }
                    })
                    ->addColumn('brokerage_firm', function($row){
                        return isset($row->brokerage_name) ? $row->brokerage_name : '';
                    })
                    ->addColumn('dispatcher', function($row){
                        return getLoadDispatcherName($row->booked_by);
                    })
                    ->addColumn('origin', function($row){
                        return getAddressCityState($row->shipper_city, $row->shipper_state);
                    })
                    ->addColumn('destination', function($row){
                        return getAddressCityState($row->delivery_city, $row->delivery_state);
                    })
                    ->addColumn('actual_shipper_date', function($row){
                        return !is_null($row->actual_shipper_date) ?  Carbon::parse($row->actual_shipper_date)->timezone($this->timeZone)->format(getTimeLayout()): '';
                    })
                    ->addColumn('actual_delivery_date', function($row){
                        return !is_null($row->actual_delivery_date) ? Carbon::parse($row->actual_delivery_date)->timezone($this->timeZone)->format(getTimeLayout()): '';
                    })
                    ->addColumn('day_on_hold', function($row){
                        if(!is_null($row->load_on_hold_datetime)){
                            $dateOnHold         = Carbon::parse($row->load_on_hold_datetime);
                            $now                = Carbon::now();
                            return $dateOnHold->diffInDays($now);
                        }
                        return '';
                    })
                    ->addColumn('reason_on_hold', function($row){
                        if(!is_null($row->reason_on_hold)){
                            return '<i class="fa fa-info-circle viewReason" aria-hidden="true" data-reason="'.$row->reason_on_hold.'"></i>';
                        }
                        return '';
                    })
                    ->addColumn('child_content', function($row){
                        return $this->childContentArray($row);
                    })
                    ->rawColumns(['status','load_number','actual_shipper_date','actual_delivery_date','full_name','day_on_hold','reason_on_hold','child_content'])
                    ->make(true);
        }
    }

    public function onHoldLoadsChildHtml(Request $request){
        $rowData                        = json_decode($request->rowData);

        $shipperAddress                 = '';
        $shipperLocation                = [];
        if ($rowData->shipper_address_one) {
            $shipperLocation[0]         = $rowData->shipper_address_one;
        }
        if ($rowData->shipper_address_two) {
            $shipperLocation[1]         = $rowData->shipper_address_two;
        }
        if ($rowData->shipper_city) {
            $shipperLocation[2]         = $rowData->shipper_city;
        }
        if ($rowData->shipper_state) {
            $shipperLocation[3]         = $rowData->shipper_state;
        }
        if ($rowData->shipper_zip) {
            $shipperLocation[4]         = $rowData->shipper_zip;
        }
        if (count($shipperLocation) == 0) {
            $shipperAddress             = $rowData->shipper_location;
        } else {
            $shipperAddress             = implode(', ', $shipperLocation);
        }

        $receiverAddress                = '';
        $receiverLocation               = [];
        if ($rowData->delivery_address_one) {
            $receiverLocation[0]        = $rowData->delivery_address_one;
        }
        if ($rowData->delivery_address_two) {
            $receiverLocation[1]        = $rowData->delivery_address_two;
        }
        if ($rowData->delivery_city) {
            $receiverLocation[2]        = $rowData->delivery_city;
        }
        if ($rowData->delivery_state) {
            $receiverLocation[3]        = $rowData->delivery_state;
        }
        if ($rowData->delivery_zip) {
            $receiverLocation[4]        = $rowData->delivery_zip;
        }
        if (count($receiverLocation) == 0) {
            $receiverAddress            = $rowData->delivery_location;
        } else {
            $receiverAddress            = implode(', ', $receiverLocation);
        }

        $view                           = view('companyAdmin.dispatchHub.ajaxview.onHoldLoadsChild', compact('rowData', 'shipperAddress', 'receiverAddress'))->render();
        return response()->json(['success' => true, 'childHtml' => $view]);
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function plannedLoadsList(Request $request){
        $getCompanyAdmin                = auth()->guard('company-admin')->user();

        $getField4                      = '';
        $isCheck4                       = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub4')->first();
        if(!empty($isCheck4->id)){
            $getField4                  = adjustTableColumnsSorting(unserialize($isCheck4->config), config('settings.table_dispatchhub4_first')); // unserialize($isCheck4->config);
        }else{
            $getField4                  = config('settings.table_dispatchhub4_first');
        }

        if ($request->ajax()) {
            $getFilters                 = $this->dispatchHubSetting;
            $filters                    = ['teamArr' => [], 'dispatcherArr' => [], 'equipmentArr' => [], 'carrierArr' => []];
            if(!is_null($getFilters)){
                $filters                = unserialize($getFilters->filters);
            }

            $data                       = LoadDetail::select('load_details.*','load_details.actual_delivery_date as load_actual_delivery_date', 'drivers.cell_phone as phone_number')
                ->join('drivers','drivers.id','=','load_details.driver_id')
                ->leftjoin('teams','teams.id','=','drivers.team_id')
                ->whereHas('loads', function($q){
                    $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                });
                //->filter($request->all())
                if(isset($filters['teamArr']) && count($filters['teamArr']) > 0){
                    $data->whereIn('teams.id', $filters['teamArr']);
                }
                if(isset($filters['dispatcherArr']) && count($filters['dispatcherArr']) > 0){
                    $data->whereIn('drivers.manager_id', $filters['dispatcherArr']);
                }
                if(isset($filters['equipmentArr']) && count($filters['equipmentArr']) > 0){
                    $data->whereIn('load_details.trailer_type', $filters['equipmentArr']);
                }
                if(isset($filters['carrierArr']) && count($filters['carrierArr']) > 0){
                    $data->leftjoin("driver_informations", "driver_informations.driver_id", "=", "drivers.id")
                        ->whereIn('driver_informations.carrier_id', $filters['carrierArr']);
                }
                // ->orderBy('load_details.id','ASC')

                $data->whereIn('load_details.load_status',[0,2,3])
                ->where(function($q) {
                    $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                })
                ->where(function($q) {
                    $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                });

                // General filter code - start
                foreach( $getField4 as $key => $value ) {
                    $conditionReq       = 'search_string_condition_'.$key;
                    $sortReq            = 'search_string_sort_'.$key;

                    $conditionDateReq   = 'search_date_condition_'.$key;
                    $sortDateReq        = 'search_date_sort_'.$key;

                    $conditionStatusReq = 'search_status_condition_'.$key;
                    $sortStatusReq      = 'search_status_sort_'.$key;

                    // String filter
                    if( isset( request()->$conditionReq ) ) {
                        $typeReq        = 'search_string_type_'.$key;
                        $type           = request()->$typeReq;

                        if( $type == 'string' ) {
                            Common::getStringFilterResult( $data, $key );
                        }
                    }

                    // Status filter
                    if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                        Common::getStatusFilterResult( $data, $key );
                    }

                    // Date filter
                    if( isset( request()->$conditionDateReq ) ) {
                        $typeReq        = 'search_date_type_'.$key;
                        $type           = request()->$typeReq;

                        if( $type == 'date' ) {
                            Common::getDateFilterResult( $data, $key );
                        }
                    }

                    // String sort only
                    if( isset( request()->$sortReq ) ) {
                        Common::sortStringFilter( $data, $key );
                    }

                    // Date sort only
                    if( isset( request()->$sortDateReq ) ) {
                        Common::sortDateFilter( $data, $key );
                    }

                    // Status sort only
                    if( isset( request()->$sortStatusReq ) ) {
                        Common::sortStatusFilter( $data, $key );
                    }
                }
                // General filter code - end
                return Datatables::of($data)
                    ->addIndexColumn()
                    /*
                    ->setRowClass(function($row){
                        $class                  = "";
                        if ($row->load_status == 2 && $row->preplaned_driver_acknowledgement == 0) {
                            $class              = 'bg-yellow';
                        }elseif ($row->load_status == 3) {
                            $class              = 'bg-green';
                        }
                        return $class;
                    })
                    */
                    ->addColumn('status', function($row){
                        //return getLoadStatus($row->load_status,$row->is_on_hold);
                        if(!is_null($row->load_status)){
                            $html               = '<div class="status-block">';
                            $html               .= '<div class="round-icon details-control"><i class="fa fa-caret-down" aria-hidden="true"></i></div>';
                            $html               .= updateStatusDropdown($row->load_id, $row->load_status, 'planned-load-data-table');
                            $html               .= '</div>';
                            return $html;
                        }
                        return '';
                    })
                    ->addColumn('load_number', function($row){
                        $html                   = '';
                        if(isset($row->load_lock_by_users) && isset($row->load_lock_by_users->company_admin->full_name) && $row->load_lock_by_users->company_admin->full_name != ''){
                            $html               .= '<i aria-hidden="true" class="fa fa-lock text-danger" title="'.$row->load_lock_by_users->company_admin->full_name.'"></i>&nbsp;';
                        }
                        if ($row->is_disable == 0) {
                            if(in_array($row->load_type, ['0', '6'])) {
                                if ($row->load_type == 0) {
                                    $html       .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                                } else {
                                    $html       .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                                }
                            } else {
                                $html           .= '<a href="'.route('company.admin.loads.edit', base64Encode($row->load_id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        } else {
                            if ($row->load_type == 0) {
                                $html           .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                            } else {
                                $html           .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        }
                        return $html;
                    })
                    ->addColumn('full_name', function($row){
                        if(isset($row->driver->id)){
                            return '<span class="show-available-driver" data-driver-id="'.$row->driver->id.'" data-toggle="modal" data-target="#availableDriverModal">'.$row->driver->full_name.'</span>';
                        }else{
                            return '';
                        }
                    })
                    /*
                    ->addColumn('phone_number', function($row){
                        return isset($row->driver->cell_phone) ? $row->driver->cell_phone : '';
                    })
                    */
                    ->addColumn('dispatcher', function($row){
                        return getLoadDispatcherName($row->booked_by);
                    })
                    ->addColumn('origin', function($row){
                        return getAddressCityState($row->shipper_city, $row->shipper_state);
                    })
                    ->addColumn('actual_shipper_date', function($row){
                        return !is_null($row->actual_shipper_date) ?  Carbon::parse($row->actual_shipper_date)->timezone($this->timeZone)->format(getTimeLayout()): '';
                    })
                    ->addColumn('actual_delivery_date', function($row){
                        return !is_null($row->actual_delivery_date) ? Carbon::parse($row->actual_delivery_date)->timezone($this->timeZone)->format(getTimeLayout()): '';
                    })
                    ->addColumn('time_waiting', function($row){
                        if($row->load_status == 0 || $row->load_status == 2){
                           if($row->load_actual_delivery_date){
                                $now            = Carbon::now()->timezone($this->timeZone)->format('Y-m-d H:i:s');
                                $to             = \Carbon\Carbon::createFromFormat('Y-m-d H:i:s', $row->load_actual_delivery_date);
                                $from           = \Carbon\Carbon::createFromFormat('Y-m-d H:i:s', $now);
                                $time           = $to->diffInHours($from) . ':' . $to->diff($from)->format('%I:%S');
                                return "<span class='text-red'>".$time."</span>";
                            }else{
                                return "<span class='text-red'>00:00:00</span>";
                            }
                        }else{
                            return "";
                        }
                    })
                    ->addColumn('destination', function($row){
                        return getAddressCityState($row->delivery_city, $row->delivery_state);
                    })
                    ->addColumn('last_location', function($row){
                        return getAddressCityState($row->delivery_city, $row->delivery_state);
                    })
                    ->addColumn('customer', function($row){
                        return "Customer";
                    })
                    ->editColumn('total_bill', function($row){
                        return getPrice($row->total_bill);
                    })
                    ->editColumn('eta', function($row){
                        /*
                        if(!is_null($row->load_trip_id)){
                            $getTripDetails     = $this->getTripDetails($row->load_trip_id);
                            if(!empty($getTripDetails)){
                                if(count($getTripDetails->stops) > 0){
                                    $lastStop   = end($getTripDetails->stops);
                                    return  Carbon::parse($lastStop->currentETA)->timezone($this->timeZone)->format('m/d/Y H:i');
                                }
                            }
                        }
                        */
                        return '';
                    })
                    ->editColumn('miles', function($row){
                        if(!is_null($row->total_miles)){
                            return $row->total_miles;
                        }
                        return '';
                    })
                    ->addColumn('child_content', function($row){
                        return $this->childContentArray($row); //$this->plannedLoadChildHtml($row);
                    })
                    ->rawColumns(['load_number','actual_shipper_date','actual_delivery_date','full_name','time_waiting','status','child_content'])
                    ->make(true);
        }
    }

    public function plannedLoadChildHtml(Request $request){
        $rowData                        = json_decode($request->rowData);

        $shipperAddress                 = '';
        $shipperLocation                = [];
        if ($rowData->shipper_address_one) {
            $shipperLocation[0]         = $rowData->shipper_address_one;
        }
        if ($rowData->shipper_address_two) {
            $shipperLocation[1]         = $rowData->shipper_address_two;
        }
        if ($rowData->shipper_city) {
            $shipperLocation[2]         = $rowData->shipper_city;
        }
        if ($rowData->shipper_state) {
            $shipperLocation[3]         = $rowData->shipper_state;
        }
        if ($rowData->shipper_zip) {
            $shipperLocation[4]         = $rowData->shipper_zip;
        }
        if (count($shipperLocation) == 0) {
            $shipperAddress             = $rowData->shipper_location;
        } else {
            $shipperAddress             = implode(', ', $shipperLocation);
        }

        $receiverAddress                = '';
        $receiverLocation               = [];
        if ($rowData->delivery_address_one) {
            $receiverLocation[0]        = $rowData->delivery_address_one;
        }
        if ($rowData->delivery_address_two) {
            $receiverLocation[1]        = $rowData->delivery_address_two;
        }
        if ($rowData->delivery_city) {
            $receiverLocation[2]        = $rowData->delivery_city;
        }
        if ($rowData->delivery_state) {
            $receiverLocation[3]        = $rowData->delivery_state;
        }
        if ($rowData->delivery_zip) {
            $receiverLocation[4]        = $rowData->delivery_zip;
        }
        if (count($receiverLocation) == 0) {
            $receiverAddress            = $rowData->delivery_location;
        } else {
            $receiverAddress            = implode(', ', $receiverLocation);
        }

        $view                           = view('companyAdmin.dispatchHub.ajaxview.plannedLoadChild', compact('rowData', 'shipperAddress', 'receiverAddress'))->render();
        return response()->json(['success' => true, 'childHtml' => $view]);
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function inTransitLoadsList(Request $request){
        $getCompanyAdmin                = auth()->guard('company-admin')->user();

        $getField2                      = '';
        $isCheck2                       = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub2')->first();
        if(!empty($isCheck2->id)){
            $getField2                  = adjustTableColumnsSorting(unserialize($isCheck2->config), config('settings.table_dispatchhub2_first')); // unserialize($isCheck2->config);
        }else{
            $getField2                  = config('settings.table_dispatchhub2_first');
        }

        if ($request->ajax()) {
            $getFilters                 = $this->dispatchHubSetting;
            $filters                    = ['teamArr' => [], 'dispatcherArr' => [], 'equipmentArr' => [], 'carrierArr' => []];
            if(!is_null($getFilters)){
                $filters                = unserialize($getFilters->filters);
            }

            $data                       = LoadDetail::select('load_details.*', 'drivers.cell_phone as phone_number')
                ->join('drivers','drivers.id','=','load_details.driver_id')
                ->leftjoin('teams','teams.id','=','drivers.team_id')
                ->whereHas('loads', function($q){
                    $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                });
                //->filter($request->all())
                if(isset($filters['teamArr']) && count($filters['teamArr']) > 0){
                    $data->whereIn('teams.id', $filters['teamArr']);
                }
                if(isset($filters['dispatcherArr']) && count($filters['dispatcherArr']) > 0){
                    $data->whereIn('drivers.manager_id', $filters['dispatcherArr']);
                }
                if(isset($filters['equipmentArr']) && count($filters['equipmentArr']) > 0){
                    $data->whereIn('load_details.trailer_type', $filters['equipmentArr']);
                }
                if(isset($filters['carrierArr']) && count($filters['carrierArr']) > 0){
                    $data->leftjoin("driver_informations", "driver_informations.driver_id", "=", "drivers.id")
                        ->whereIn('driver_informations.carrier_id', $filters['carrierArr']);
                }

                //->orderBy('load_details.id','ASC')
                $data->where('load_details.load_status', 4)
                ->where(function($q) {
                    $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                })
                ->where(function($q) {
                    $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                });

                // General filter code - start
                foreach( $getField2 as $key => $value ) {
                    $conditionReq       = 'search_string_condition_'.$key;
                    $sortReq            = 'search_string_sort_'.$key;

                    $conditionDateReq   = 'search_date_condition_'.$key;
                    $sortDateReq        = 'search_date_sort_'.$key;

                    $conditionStatusReq = 'search_status_condition_'.$key;
                    $sortStatusReq      = 'search_status_sort_'.$key;

                    // String filter
                    if( isset( request()->$conditionReq ) ) {
                        $typeReq        = 'search_string_type_'.$key;
                        $type           = request()->$typeReq;

                        if( $type == 'string' ) {
                            Common::getStringFilterResult( $data, $key );
                        }
                    }

                    // Status filter
                    if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                        Common::getStatusFilterResult( $data, $key );
                    }

                    // Date filter
                    if( isset( request()->$conditionDateReq ) ) {
                        $typeReq        = 'search_date_type_'.$key;
                        $type           = request()->$typeReq;

                        if( $type == 'date' ) {
                            Common::getDateFilterResult( $data, $key );
                        }
                    }

                    // String sort only
                    if( isset( request()->$sortReq ) ) {
                        Common::sortStringFilter( $data, $key );
                    }

                    // Date sort only
                    if( isset( request()->$sortDateReq ) ) {
                        Common::sortDateFilter( $data, $key );
                    }

                    // Status sort only
                    if( isset( request()->$sortStatusReq ) ) {
                        Common::sortStatusFilter( $data, $key );
                    }
                }
                // General filter code - end
                return Datatables::of($data)
                    ->addIndexColumn()
                    ->addColumn('status', function($row){
                        //return getLoadStatus($row->load_status,$row->is_on_hold);
                        if(!is_null($row->load_status)){
                            $html               = '<div class="status-block">';
                            $html               .= '<div class="round-icon details-control"><i class="fa fa-caret-down" aria-hidden="true"></i></div>';
                            $html               .= updateStatusDropdown($row->load_id, $row->load_status, 'in-transit-loads-data-table');
                            $html               .= '</div>';
                            return $html;
                        }
                        return '';
                    })
                    ->addColumn('full_name', function($row){
                        if(isset($row->driver->id)){
                            return '<span class="show-available-driver" data-driver-id="'.$row->driver->id.'" data-toggle="modal" data-target="#availableDriverModal">'.$row->driver->full_name.'</span>';
                        }else{
                            return '';
                        }
                    })
                    /*
                    ->addColumn('phone_number', function($row){
                        return isset($row->driver->cell_phone) ? $row->driver->cell_phone : '';
                    })
                    */
                    ->addColumn('load_number', function($row){
                        $html                   = '';
                        if(isset($row->load_lock_by_users) && isset($row->load_lock_by_users->company_admin->full_name) && $row->load_lock_by_users->company_admin->full_name != ''){
                            $html               .= '<i aria-hidden="true" class="fa fa-lock text-danger" title="'.$row->load_lock_by_users->company_admin->full_name.'"></i>&nbsp;';
                        }
                        if ($row->is_disable == 0) {
                            if(in_array($row->load_type, ['0', '6'])) {
                                if ($row->load_type == 0) {
                                    $html       .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                                } else {
                                    $html       .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                                }
                            } else {
                                $html           .= '<a href="'.route('company.admin.loads.edit', base64Encode($row->load_id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        } else {
                            if ($row->load_type == 0) {
                                $html           .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                            } else {
                                $html           .= '<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        }
                        return $html;
                    })
                    ->addColumn('dispatcher', function($row){
                        return getLoadDispatcherName($row->booked_by);
                    })
                    ->addColumn('actual_shipper_date', function($row){
                        return !is_null($row->actual_shipper_date) ?  Carbon::parse($row->actual_shipper_date)->timezone($this->timeZone)->format(getTimeLayout()): '';
                    })
                    ->addColumn('actual_delivery_date', function($row){
                        return !is_null($row->actual_delivery_date) ? Carbon::parse($row->actual_delivery_date)->timezone($this->timeZone)->format(getTimeLayout()): '';
                    })
                    ->addColumn('origin', function($row){
                        return getAddressCityState($row->shipper_city, $row->shipper_state);
                    })
                    ->addColumn('destination', function($row){
                        return getAddressCityState($row->delivery_city, $row->delivery_state);
                    })
                    ->addColumn('last_location', function($row){
                        return getAddressCityState($row->delivery_city, $row->delivery_state);
                    })
                    ->editColumn('eta', function($row){
                        /*
                        if(!is_null($row->load_trip_id)){
                            $getTripDetails     = $this->getTripDetails($row->load_trip_id);
                            if(!empty($getTripDetails)){
                                if(count($getTripDetails->stops) > 0){
                                    $lastStop   = end($getTripDetails->stops);
                                    return  Carbon::parse($lastStop->currentETA)->timezone($this->timeZone)->format('m/d/Y H:i');
                                }
                            }
                        }
                        */
                        return '';
                    })
                    ->editColumn('miles_out', function($row){
                        if(!is_null($row->total_miles)){
                            return $row->total_miles;
                        }
                        return '';
                    })
                    ->addColumn('child_content', function($row){
                        return $this->childContentArray($row); //$this->inTransitLoadChildHtml($row);
                    })
                    ->rawColumns(['status','load_number','actual_shipper_date','actual_delivery_date','full_name','child_content'])
                    ->make(true);
        }
    }

    public function inTransitLoadChildHtml(Request $request){
        $rowData                        = json_decode($request->rowData);

        $shipperAddress                 = '';
        $shipperLocation                = [];
        if ($rowData->shipper_address_one) {
            $shipperLocation[0]         = $rowData->shipper_address_one;
        }
        if ($rowData->shipper_address_two) {
            $shipperLocation[1]         = $rowData->shipper_address_two;
        }
        if ($rowData->shipper_city) {
            $shipperLocation[2]         = $rowData->shipper_city;
        }
        if ($rowData->shipper_state) {
            $shipperLocation[3]         = $rowData->shipper_state;
        }
        if ($rowData->shipper_zip) {
            $shipperLocation[4]         = $rowData->shipper_zip;
        }
        if (count($shipperLocation) == 0) {
            $shipperAddress             = $rowData->shipper_location;
        } else {
            $shipperAddress             = implode(', ', $shipperLocation);
        }

        $receiverAddress                = '';
        $receiverLocation               = [];
        if ($rowData->delivery_address_one) {
            $receiverLocation[0]        = $rowData->delivery_address_one;
        }
        if ($rowData->delivery_address_two) {
            $receiverLocation[1]        = $rowData->delivery_address_two;
        }
        if ($rowData->delivery_city) {
            $receiverLocation[2]        = $rowData->delivery_city;
        }
        if ($rowData->delivery_state) {
            $receiverLocation[3]        = $rowData->delivery_state;
        }
        if ($rowData->delivery_zip) {
            $receiverLocation[4]        = $rowData->delivery_zip;
        }
        if (count($receiverLocation) == 0) {
            $receiverAddress            = $rowData->delivery_location;
        } else {
            $receiverAddress            = implode(', ', $receiverLocation);
        }

        $view                           = view('companyAdmin.dispatchHub.ajaxview.inTransitLoadChild', compact('rowData', 'shipperAddress', 'receiverAddress'))->render();
        return response()->json(['success' => true, 'childHtml' => $view]);
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function allDispatch(Request $request){
        $getCompanyAdmin                    = auth()->guard('company-admin')->user();

        $getField7                          = '';
        $isCheck7                           = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub7')->first();
        if(!empty($isCheck7->id)){
            $getField7                      = adjustTableColumnsSorting(unserialize($isCheck7->config), config('settings.table_dispatchhub7_first')); // unserialize($isCheck7->config);
        }else{
            $getField7                      = config('settings.table_dispatchhub7_first');
        }

        if ($request->ajax()) {
            $getFilters                     = $this->dispatchHubSetting;
            $filters                        = ['teamArr' => [], 'dispatcherArr' => [], 'equipmentArr' => [], 'carrierArr' => []];
            if(!is_null($getFilters)){
                $filters                    = unserialize($getFilters->filters);
            }

            $data                           = LoadDetail::select('load_details.*')
                ->join('drivers','drivers.id','=','load_details.driver_id')
                ->leftjoin('teams','teams.id','=','drivers.team_id');
                if(isset($filters['teamArr']) && count($filters['teamArr']) > 0){
                    $data->whereIn('teams.id', $filters['teamArr']);
                }
                if(isset($filters['dispatcherArr']) && count($filters['dispatcherArr']) > 0){
                    $data->whereIn('drivers.manager_id', $filters['dispatcherArr']);
                }
                if(isset($filters['equipmentArr']) && count($filters['equipmentArr']) > 0){
                    $data->whereIn('load_details.trailer_type', $filters['equipmentArr']);
                }
                if(isset($filters['carrierArr']) && count($filters['carrierArr']) > 0){
                    $data->leftjoin("driver_informations", "driver_informations.driver_id", "=", "drivers.id")
                        ->whereIn('driver_informations.carrier_id', $filters['carrierArr']);
                }

            $data                           = $data->whereHas('loads', function($query){
                    $query->where('company_id', auth()->guard('company-admin')->user()->company_id);
                })->where(function($query) {
                    $query->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                })->where(function($query){
                    $query->where(function ($que) {
                        $que->where('load_details.is_on_hold', 1);
                    })->orWhere(function ($que) {
                        $que->whereIn('load_details.load_status',[0,2,3,4])
                            ->where(function ($q) {
                                $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                            });
                    });
                });

                // General filter code - start
                foreach( $getField7 as $key => $value ) {
                    $conditionReq           = 'search_string_condition_'.$key;
                    $sortReq                = 'search_string_sort_'.$key;

                    $conditionDateReq       = 'search_date_condition_'.$key;
                    $sortDateReq            = 'search_date_sort_'.$key;

                    $conditionStatusReq     = 'search_status_condition_'.$key;
                    $sortStatusReq          = 'search_status_sort_'.$key;

                    // String filter
                    if( isset( request()->$conditionReq ) ) {
                        $typeReq            = 'search_string_type_'.$key;
                        $type               = request()->$typeReq;

                        if( $type == 'string' ) {
                            Common::getStringFilterResult($data, $key);
                        }
                    }

                    // Status filter
                    if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                        Common::getStatusFilterResult($data, $key);
                    }

                    // Date filter
                    if( isset( request()->$conditionDateReq ) ) {
                        $typeReq            = 'search_date_type_'.$key;
                        $type               = request()->$typeReq;

                        if( $type == 'date' ) {
                            Common::getDateFilterResult($data, $key);
                        }
                    }

                    // String sort only
                    if( isset( request()->$sortReq ) ) {
                        Common::sortStringFilter($data, $key);
                    }

                    // Date sort only
                    if( isset( request()->$sortDateReq ) ) {
                        Common::sortDateFilter($data, $key);
                    }

                    // Status sort only
                    if( isset( request()->$sortStatusReq ) ) {
                        Common::sortStatusFilter($data, $key);
                    }
                }
                // General filter code - end

                return Datatables::of($data)
                    ->addIndexColumn()
                    ->addColumn('status', function($row){
                        if(!is_null($row->load_status)){
                            $html               = '<div class="status-block">';
                            $html              .= '<div class="round-icon details-control"><i class="fa fa-caret-down" aria-hidden="true"></i></div>';
                            $html              .= updateStatusDropdown($row->load_id,  $row->is_on_hold == 1 ? '7' : $row->load_status, 'all-dispatch-data-table');
                            $html              .= '</div>';
                            return $html;
                        }
                        return '';
                    })
                    ->addColumn('full_name', function($row){
                        if(isset($row->driver->id)){
                            return '<span class="show-available-driver" data-driver-id="'.$row->driver->id.'" data-toggle="modal" data-target="#availableDriverModal">'.$row->driver->full_name.'</span>';
                        }else{
                            return '';
                        }
                    })
                    ->addColumn('phone_number', function($row){
                        return isset($row->driver->cell_phone) ? $row->driver->cell_phone : '';
                    })
                    ->addColumn('load_number', function($row){
                        $html                   = '';
                        if(isset($row->load_lock_by_users) && isset($row->load_lock_by_users->company_admin->full_name) && $row->load_lock_by_users->company_admin->full_name != ''){
                            $html               .= '<i aria-hidden="true" class="fa fa-lock text-danger" title="'.$row->load_lock_by_users->company_admin->full_name.'"></i>&nbsp;';
                        }
                        if ($row->is_disable == 0) {
                            if(in_array($row->load_type, ['0', '6'])) {
                                if ($row->load_type == 0) {
                                    $html       .='<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                                } else {
                                    $html       .='<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                                }
                            } else {
                                $html           .='<a href="'.route('company.admin.loads.edit', base64Encode($row->load_id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        } else {
                            if ($row->load_type == 0) {
                                $html           .='<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                            } else {
                                $html           .='<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        }
                        return $html;
                    })
                    ->addColumn('dispatcher', function($row){
                        return getLoadDispatcherName($row->booked_by);
                    })
                    ->addColumn('actual_shipper_date', function($row){
                        return !is_null($row->actual_shipper_date) ?  Carbon::parse($row->actual_shipper_date)->timezone($this->timeZone)->format(getTimeLayout()): '';
                    })
                    ->addColumn('actual_delivery_date', function($row){
                        return !is_null($row->actual_delivery_date) ? Carbon::parse($row->actual_delivery_date)->timezone($this->timeZone)->format(getTimeLayout()): '';
                    })
                    ->addColumn('origin', function($row){
                        return getAddressCityState($row->shipper_city, $row->shipper_state);
                    })
                    ->addColumn('destination', function($row){
                        return getAddressCityState($row->delivery_city, $row->delivery_state);
                    })
                    ->addColumn('last_location', function($row){
                        return getAddressCityState($row->delivery_city, $row->delivery_state);
                    })
                    ->editColumn('eta', function($row){
                        /*
                        if(!is_null($row->load_trip_id)){
                            $getTripDetails     = $this->getTripDetails($row->load_trip_id);
                            if(!empty($getTripDetails)){
                                if(count($getTripDetails->stops) > 0){
                                    $lastStop   = end($getTripDetails->stops);
                                    return  Carbon::parse($lastStop->currentETA)->timezone($this->timeZone)->format('m/d/Y H:i');
                                }
                            }
                        }
                        */
                        return '';
                    })
                    ->editColumn('miles', function($row){
                        if(!is_null($row->total_miles)){
                            return $row->total_miles;
                        }
                        return '';
                    })
                    ->addColumn('child_content', function($row){
                        return $this->childContentArray($row); //$this->allDispatchLoadChildHtml($row);
                    })
                    ->rawColumns(['status','load_number','actual_shipper_date','actual_delivery_date','full_name','child_content'])
                    ->make(true);
        }
    }

    public function allDispatchLoadChildHtml(Request $request){
        $rowData                            = json_decode($request->rowData);

        $shipperAddress                     = '';
        $shipperLocation                    = [];
        if ($rowData->shipper_address_one) {
            $shipperLocation[0]             = $rowData->shipper_address_one;
        }
        if ($rowData->shipper_address_two) {
            $shipperLocation[1]             = $rowData->shipper_address_two;
        }
        if ($rowData->shipper_city) {
            $shipperLocation[2]             = $rowData->shipper_city;
        }
        if ($rowData->shipper_state) {
            $shipperLocation[3]             = $rowData->shipper_state;
        }
        if ($rowData->shipper_zip) {
            $shipperLocation[4]             = $rowData->shipper_zip;
        }
        if (count($shipperLocation) == 0) {
            $shipperAddress                 = $rowData->shipper_location;
        } else {
            $shipperAddress                 = implode(', ', $shipperLocation);
        }

        $receiverAddress                    = '';
        $receiverLocation                   = [];
        if ($rowData->delivery_address_one) {
            $receiverLocation[0]            = $rowData->delivery_address_one;
        }
        if ($rowData->delivery_address_two) {
            $receiverLocation[1]            = $rowData->delivery_address_two;
        }
        if ($rowData->delivery_city) {
            $receiverLocation[2]            = $rowData->delivery_city;
        }
        if ($rowData->delivery_state) {
            $receiverLocation[3]            = $rowData->delivery_state;
        }
        if ($rowData->delivery_zip) {
            $receiverLocation[4]            = $rowData->delivery_zip;
        }
        if (count($receiverLocation) == 0) {
            $receiverAddress                = $rowData->delivery_location;
        } else {
            $receiverAddress                = implode(', ', $receiverLocation);
        }

        $view                               = view('companyAdmin.dispatchHub.ajaxview.allDispatchLoadChild', compact('rowData', 'shipperAddress', 'receiverAddress'))->render();
        return response()->json(['success' => true, 'childHtml' => $view]);
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function completedLoadsList(Request $request){
        $getCompanyAdmin                = auth()->guard('company-admin')->user();

        // Ready to Bill Loads
        $getField6                      = '';
        $isCheck6                       = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub6')->first();
        if(!empty($isCheck6->id)){
            $getField6                  = adjustTableColumnsSorting(unserialize($isCheck6->config), config('settings.table_dispatchhub6_first')); // unserialize($isCheck6->config);
        }else{
            $getField6                  = config('settings.table_dispatchhub6_first');
        }

        if ($request->ajax()) {
            $getFilters                 = $this->dispatchHubSetting;
            $filters                    = ['teamArr' => [], 'dispatcherArr' => [], 'equipmentArr' => [], 'carrierArr' => []];
            if(!is_null($getFilters)){
                $filters                = unserialize($getFilters->filters);
            }

            $data                       = LoadDetail::select(
                    'load_details.*',
                    'teams.name as team_name',
                    'brokers.name as brokerage_name',
                    'carriers.name as carrier_name',
                    'trailer_types.name as trailer_type_name'
                )
                ->join('drivers','load_details.driver_id','=','drivers.id')
                ->leftjoin('teams','drivers.team_id','=','teams.id')
                ->leftjoin('brokers','load_details.brokerage_firm','=','brokers.id')
                ->leftjoin('carriers','load_details.carrier_id','=','carriers.id')
                ->leftjoin('trailer_types','load_details.trailer_type','=','trailer_types.id')
                ->whereHas('loads', function($q){
                    $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                });
                //->filter($request->all())
                if(isset($filters['teamArr']) && count($filters['teamArr']) > 0){
                    $data->whereIn('teams.id', $filters['teamArr']);
                }
                if(isset($filters['dispatcherArr']) && count($filters['dispatcherArr']) > 0){
                    $data->whereIn('drivers.manager_id', $filters['dispatcherArr']);
                }
                if(isset($filters['equipmentArr']) && count($filters['equipmentArr']) > 0){
                    $data->whereIn('load_details.trailer_type', $filters['equipmentArr']);
                }
                if(isset($filters['carrierArr']) && count($filters['carrierArr']) > 0){
                    $data->leftjoin("driver_informations", "driver_informations.driver_id", "=", "drivers.id")
                        ->whereIn('driver_informations.carrier_id', $filters['carrierArr']);
                }

                //->orderBy('load_details.id','ASC')
                $data->where('load_details.load_status', 5)
                ->where(function($q) {
                    $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                })
                ->where(function($q) {
                    $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
                });

                // General filter code - start
                foreach( $getField6 as $key => $value ) {
                    $conditionReq       = 'search_string_condition_'.$key;
                    $sortReq            = 'search_string_sort_'.$key;

                    $conditionDateReq   = 'search_date_condition_'.$key;
                    $sortDateReq        = 'search_date_sort_'.$key;

                    $conditionStatusReq = 'search_status_condition_'.$key;
                    $sortStatusReq      = 'search_status_sort_'.$key;

                    // String filter
                    if( isset( request()->$conditionReq ) ) {
                        $typeReq        = 'search_string_type_'.$key;
                        $type           = request()->$typeReq;

                        if( $type == 'string' ) {
                            Common::getStringFilterResult( $data, $key );
                        }
                    }

                    // Status filter
                    if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                        Common::getStatusFilterResult( $data, $key );
                    }

                    // Date filter
                    if( isset( request()->$conditionDateReq ) ) {
                        $typeReq        = 'search_date_type_'.$key;
                        $type           = request()->$typeReq;

                        if( $type == 'date' ) {
                            Common::getDateFilterResult( $data, $key );
                        }
                    }

                    // String sort only
                    if( isset( request()->$sortReq ) ) {
                        Common::sortStringFilter( $data, $key );
                    }

                    // Date sort only
                    if( isset( request()->$sortDateReq ) ) {
                        Common::sortDateFilter( $data, $key );
                    }

                    // Status sort only
                    if( isset( request()->$sortStatusReq ) ) {
                        Common::sortStatusFilter( $data, $key );
                    }
                }
                // General filter code - end
                return Datatables::of($data)
                    ->addIndexColumn()
                    ->addColumn('load_number', function($row){
                        $html                   = '';
                        if(isset($row->load_lock_by_users) && isset($row->load_lock_by_users->company_admin->full_name) && $row->load_lock_by_users->company_admin->full_name != ''){
                            $html               .= '<i aria-hidden="true" class="fa fa-lock text-danger" title="'.$row->load_lock_by_users->company_admin->full_name.'"></i>&nbsp;';
                        }
                        if ($row->is_disable == 0) {
                            if(in_array($row->load_type, ['0', '6'])) {
                                if ($row->load_type == 0) {
                                    $html       .='<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                                } else {
                                    $html       .='<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                                }
                            } else {
                                $html           .='<a href="'.route('company.admin.loads.edit', base64Encode($row->load_id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        } else {
                            if ($row->load_type == 0) {
                                $html           .='<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                            } else {
                                $html           .='<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        }
                        return $html;
                    })
                    ->addColumn('billing_status', function($row){
                        return '';
                    })
                    ->addColumn('actual_shipper_date', function($row){
                        return !is_null($row->actual_shipper_date) ?  Carbon::parse($row->actual_shipper_date)->timezone($this->timeZone)->format(getTimeLayout()): '';
                    })
                    ->addColumn('actual_delivery_date', function($row){
                        return !is_null($row->actual_delivery_date) ? Carbon::parse($row->actual_delivery_date)->timezone($this->timeZone)->format(getTimeLayout()): '';
                    })
                    ->addColumn('origin', function($row){
                        return getAddressCityState($row->shipper_city, $row->shipper_state);
                    })
                    ->addColumn('destination', function($row){
                        return getAddressCityState($row->delivery_city, $row->delivery_state);
                    })
                    ->addColumn('full_name', function($row){
                        if(isset($row->driver->id)){
                            return '<span class="show-available-driver" data-driver-id="'.$row->driver->id.'" data-toggle="modal" data-target="#availableDriverModal">'.$row->driver->full_name.'</span>';
                        }else{
                            return '';
                        }
                    })
                    ->addColumn('child_content', function($row){
                        return $this->childContentArray($row);
                    })
                    ->rawColumns(['load_number','full_name','child_content'])
                    ->make(true);
        }
    }

    public function completedLoadBillingStatus(Request $request){
        $response['success']                                            = false;
        $response['message']                                            = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $formdata                                               = $request->all();
                $load_detail                                            = LoadDetail::whereId($formdata['id'])->first();
                if(!is_null($load_detail)){
                    $getLoadDocuments                                   = $this->getLoadDocuments($load_detail, 'billing_status');

                    // Merge Documents
                    $merged_documents                                   = $getLoadDocuments['merged_documents'];

                    $statusValues                                       = array_column($merged_documents, 'status');

                    $response['status']                                 = in_array('Pending', $statusValues) ? 0 : 1; // 0 = Missing Paperwork, 1 = Complete
                    $response['success']                                = true;
                    $response['message']                                = "Status retrieved successfully.";
                }
            } catch (\Exception $e) {
                $response['success']                                    = false;
                $response['message']                                    = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function completedLoadChildHtml(Request $request){
        $response['success']                                            = false;
        $response['message']                                            = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $formData                                               = $request->all();
                $load_detail                                            = LoadDetail::whereId($formData['id'])->first();
                if(!is_null($load_detail)){
                    $getLoadDocuments                                   = $this->getLoadDocuments($load_detail);

                    // Required Documents
                    $required_documents                                 = $getLoadDocuments['required_documents'];
                    // Requested Documents
                    $requested_documents                                = $getLoadDocuments['requested_documents'];
                    // Accessorial Documents
                    $accessorial_documents                              = $getLoadDocuments['accessorial_documents'];
                    // Other Documents
                    $other_documents                                    = $getLoadDocuments['other_documents'];
                    // Merge Documents
                    $merged_documents                                   = $getLoadDocuments['merged_documents'];

                    $billingStatusValues                                = array_column($merged_documents, 'status');

                    $billing_status                                     = in_array('Pending', $billingStatusValues) ? 0 : 1; // 0 = Missing Paperwork, 1 = Complete

                    $response['html']                                   = view('companyAdmin.dispatchHub.ajaxview.completedLoad.detail', compact('formData','load_detail','required_documents','requested_documents','accessorial_documents','other_documents','billing_status'))->render();
                    $response['success']                                = true;
                    $response['message']                                = "Detail retrieved successfully.";
                }
            } catch (\Exception $e) {
                $response['success']                                    = false;
                $response['message']                                    = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function completedLoadFileSave(Request $request){
        $response['success']                                                        = false;
        $response['message']                                                        = "Oops! Something went wrong..";
        if ($request->ajax()) {
            try {
                $formData                                                           = $request->all();
                if(isset($formData['jsonData']) && $formData['jsonData']!==''){
                    $jsonData                                                       = json_decode($formData['jsonData'], true);
                    $oldJsonData                                                    = $jsonData;

                    if(isset($jsonData['source'])){
                        if(in_array($jsonData['source'], ['load_details', 'load_stops'])){
                            $getArray                                               = [];
                            if($jsonData['source'] == 'load_details'){
                                $item                                               = LoadDetail::whereId($jsonData['id'])->first();
                                if(!is_null($item)){
                                    if ($jsonData['field_name'] == 'shipper_files_upload') {
                                        $getArray                                   = isset($item->shipper_files_upload) && is_serialized($item->shipper_files_upload) && !empty(unserialize($item->shipper_files_upload)) ? unserialize($item->shipper_files_upload) : [];
                                    } else if ($jsonData['field_name'] == 'consignee_files_upload') {
                                        $getArray                                       = isset($item->consignee_files_upload) && is_serialized($item->consignee_files_upload) && !empty(unserialize($item->consignee_files_upload)) ? unserialize($item->consignee_files_upload) : [];
                                    }
                                }
                            }else{
                                $item                                               = LoadStop::whereId($jsonData['id'])->first();
                                if(!is_null($item)){
                                    if($jsonData['field_name'] == 'stop_shipper_files_upload'){
                                        $getArray                                   = isset($item->stop_shipper_files_upload) && is_serialized($item->stop_shipper_files_upload) && !empty(unserialize($item->stop_shipper_files_upload)) ? unserialize($item->stop_shipper_files_upload) : [];
                                    } else if ($jsonData['field_name'] == 'stop_consignee_files_upload') {
                                        $getArray                                   = isset($item->stop_consignee_files_upload) && is_serialized($item->stop_consignee_files_upload) && !empty(unserialize($item->stop_consignee_files_upload)) ? unserialize($item->stop_consignee_files_upload) : [];
                                    }
                                }
                            }

                            if(!empty($getArray)){
                                $saveArray                                          = [];
                                foreach($getArray as $data){
                                    if(
                                        (isset($data['name_id']) && $data['name_id'] == $jsonData['filename_id']) &&
                                        (isset($data['name']) && $data['name'] == $jsonData['filename']) &&
                                        (isset($data['type']) && $data['type'] == $jsonData['type'])
                                    ){
                                        $history                                    = isset($data['history']) && is_array($data['history']) && !empty($data['history']) ? $data['history'] : [];

                                        if(isset($formData['is_received'])){
                                            $data['is_received']                    = 'on';
                                            // $data['file_not_provided']           = 'off';
                                            $data['upload_user_type']               = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                            $data['upload_by']                      = auth()->guard('company-admin')->user()->id;
                                            $data['uploaded_at']                    = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');

                                            $hv                                     = [];
                                            $hv['upload_user_type']                 = $data['upload_user_type'];
                                            $hv['upload_by']                        = $data['upload_by'];
                                            $hv['uploaded_at']                      = $data['uploaded_at'];
                                            array_push($history, $hv);

                                            $data['history']                        = $history;

                                            // Update jsonData
                                            $jsonData['history']                    = $data['history'];
                                            $jsonData['is_received']                = $data['is_received'];
                                            $jsonData['upload_user_type']           = $data['upload_user_type'];
                                            $jsonData['upload_by']                  = $data['upload_by'];
                                            $jsonData['uploaded_by']                = isset($data['upload_by']) ? getUserByType($data['upload_by'], $data['upload_user_type']) : null;
                                            $jsonData['uploaded_at']                = Carbon::createFromFormat('Y-m-d H:i:s', $data['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId;
                                            $jsonData['status']                     = 'Complete';
                                        } else if(isset($formData['files']) && !empty($formData['files'])){
                                            $files                                  = isset($data['file']) && !empty($data['file']) ? $data['file'] : null;
                                            if(isset($files) && !empty($files)){
                                                foreach ($files as $key => $file) {
                                                    $path                           = 'storage/companyAdmin/load_detail_document/'.$file;
                                                    if (File::exists($path)) {
                                                        File::delete($path);
                                                    }
                                                }
                                            }
                                            $data['is_received']                    = 'on';
                                            // $data['file_not_provided']           = 'off';
                                            $data['file']                           = UploadImagePdf($formData['files'], 'companyAdmin/load_detail_document');
                                            $data['upload_user_type']               = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                            $data['upload_by']                      = auth()->guard('company-admin')->user()->id;
                                            $data['uploaded_at']                    = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');

                                            $hv                                     = [];
                                            $hv['upload_user_type']                 = $data['upload_user_type'];
                                            $hv['upload_by']                        = $data['upload_by'];
                                            $hv['uploaded_at']                      = $data['uploaded_at'];
                                            array_push($history, $hv);

                                            $data['history']                        = $history;

                                            // Update jsonData
                                            $jsonData['history']                    = $data['history'];
                                            $jsonData['is_received']                = $data['is_received'];
                                            $jsonData['file']                       = $data['file'];
                                            $jsonData['upload_user_type']           = $data['upload_user_type'];
                                            $jsonData['upload_by']                  = $data['upload_by'];
                                            $jsonData['uploaded_by']                = isset($data['upload_by']) ? getUserByType($data['upload_by'], $data['upload_user_type']) : null;
                                            $jsonData['uploaded_at']                = Carbon::createFromFormat('Y-m-d H:i:s', $data['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId;
                                            $jsonData['status']                     = 'Complete';
                                        }
                                    }
                                    $saveArray[]                                    = $data;
                                }

                                if(isset($item)){
                                    if($jsonData['field_name'] == 'shipper_files_upload'){
                                        $oldData                                    = $item->toArray();
                                        $item->update(['shipper_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load detail updated', $item->getChanges(), $oldData);
                                    }
                                    if($jsonData['field_name'] == 'consignee_files_upload'){
                                        $oldData                                    = $item->toArray();
                                        $item->update(['consignee_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load detail updated', $item->getChanges(), $oldData);
                                    }
                                    if($jsonData['field_name'] == 'stop_shipper_files_upload'){
                                        $oldData                                    = $item->toArray();
                                        $item->update(['stop_shipper_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load stop updated', $item->getChanges(), $oldData);
                                    }
                                    if($jsonData['field_name'] == 'stop_consignee_files_upload'){
                                        $oldData                                    = $item->toArray();
                                        $item->update(['stop_consignee_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load stop updated', $item->getChanges(), $oldData);
                                    }
                                }
                            }
                        }elseif($jsonData['source'] == 'load_document_request_items'){
                            $load_document_request_item                             = LoadDocumentRequestItem::where('id', $jsonData['id'])->first();
                            if(!is_null($load_document_request_item)){
                                $oldData                                            = $load_document_request_item->toArray();

                                $data                                               = [];
                                if(isset($load_document_request_item->files) && is_serialized($load_document_request_item->files) && !empty(unserialize($load_document_request_item->files))){
                                    $data                                           = unserialize($load_document_request_item->files);
                                }

                                if(!empty($data)){
                                    $history                                        = isset($data['history']) && is_array($data['history']) && !empty($data['history']) ? $data['history'] : [];

                                    if(isset($formData['is_received'])){
                                        $data['is_received']                        = 'on';
                                        // $data['file_not_provided']               = 'off';
                                        $data['upload_user_type']                   = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                        $data['upload_by']                          = auth()->guard('company-admin')->user()->id;
                                        $data['uploaded_at']                        = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');

                                        $hv                                         = [];
                                        $hv['upload_user_type']                     = $data['upload_user_type'];
                                        $hv['upload_by']                            = $data['upload_by'];
                                        $hv['uploaded_at']                          = $data['uploaded_at'];
                                        array_push($history, $hv);

                                        $data['history']                            = $history;

                                        // Update jsonData
                                        $jsonData['history']                        = $data['history'];
                                        $jsonData['is_received']                    = $data['is_received'];
                                        $jsonData['upload_user_type']               = $data['upload_user_type'];
                                        $jsonData['upload_by']                      = $data['upload_by'];
                                        $jsonData['uploaded_by']                    = isset($data['upload_by']) ? getUserByType($data['upload_by'], $data['upload_user_type']) : null;
                                        $jsonData['uploaded_at']                    = Carbon::createFromFormat('Y-m-d H:i:s', $data['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId;
                                        $jsonData['status']                         = 'Complete';
                                    } else if(isset($formData['files']) && !empty($formData['files'])){
                                        $files                                      = isset($data['file']) && !empty($data['file']) ? $data['file'] : null;
                                        if(isset($files) && !empty($files)){
                                            foreach ($files as $key => $file) {
                                                $path                               = 'storage/companyAdmin/load_detail_document/'.$file;
                                                if (File::exists($path)) {
                                                    File::delete($path);
                                                }
                                            }
                                        }
                                        $data['is_received']                        = 'on';
                                        // $data['file_not_provided']               = 'off';
                                        $data['file']                               = UploadImagePdf($formData['files'], 'companyAdmin/load_detail_document');
                                        $data['upload_user_type']                   = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                        $data['upload_by']                          = auth()->guard('company-admin')->user()->id;
                                        $data['uploaded_at']                        = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');

                                        $hv                                         = [];
                                        $hv['upload_user_type']                     = $data['upload_user_type'];
                                        $hv['upload_by']                            = $data['upload_by'];
                                        $hv['uploaded_at']                          = $data['uploaded_at'];
                                        array_push($history, $hv);

                                        $data['history']                            = $history;

                                        // Update jsonData
                                        $jsonData['history']                        = $data['history'];
                                        $jsonData['is_received']                    = $data['is_received'];
                                        $jsonData['file']                           = $data['file'];
                                        $jsonData['upload_user_type']               = $data['upload_user_type'];
                                        $jsonData['upload_by']                      = $data['upload_by'];
                                        $jsonData['uploaded_by']                    = isset($data['upload_by']) ? getUserByType($data['upload_by'], $data['upload_user_type']) : null;
                                        $jsonData['uploaded_at']                    = Carbon::createFromFormat('Y-m-d H:i:s', $data['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId;
                                        $jsonData['status']                         = 'Complete';
                                    }
                                }

                                $load_document_request_item->files                  = !empty($data) ? serialize($data) : null;
                                $load_document_request_item->updated_at             = date('Y-m-d H:i:s');
                                $load_document_request_item->updated_by             = auth()->guard('company-admin')->user()->id;
                                $load_document_request_item->updated_by_type        = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                if($load_document_request_item->save()){
                                    $this->setLoadAudit($load_document_request_item->load_id, 'update', 'Load document request item updated', $load_document_request_item->getChanges(), $oldData);
                                }
                            }
                        }elseif($jsonData['source'] == 'load_accessorials'){
                            $load_accessorial                                       = LoadAccessorial::where('id', $jsonData['id'])->first();
                            if(!is_null($load_accessorial)){
                                $oldData                                            = $load_accessorial->toArray();

                                $gallery                                            = Gallery::where('company_id', auth()->guard('company-admin')->user()->company_id)
                                    ->where('action_id', $load_accessorial->id)
                                    ->where('source', 'load_accessorials')
                                    ->where('field_name', 'accessorial_files')
                                    ->latest()->first();

                                $is_upload                                          = 0;

                                $imageName                                          = null;
                                // Invalid file type! Please select only jpg, jpeg, png, gif, or pdf files. - only Accessorial
                                $fileArray                                          = UploadImagePdf($formData['files'], 'companyAdmin/gallery');
                                // Get the first element
                                $firstElement                                       = reset($fileArray);
                                // Check if $firstElement exists
                                if (isset($firstElement)) {
                                    // Extract filename and extension
                                    $filenameWithExtension                          = pathinfo($firstElement, PATHINFO_EXTENSION);
                                    if($filenameWithExtension == 'pdf'){
                                        $imageName                                  = $firstElement;
                                    }
                                }
                                if(isset($imageName) && $imageName != ''){
                                    if(!is_null($gallery)){
                                        $path                                       = 'storage/companyAdmin/gallery/'.$gallery->image;
                                        if (File::exists($path)) {
                                            File::delete($path);
                                        }

                                        $gallery->image                             = $imageName;
                                        $gallery->created_by                        = auth()->guard('company-admin')->user()->id;
                                        $gallery->created_by_type                   = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                        $gallery->uploaded_date                     = Carbon::now()->format('m/d/Y H:i');
                                        if($gallery->save()){
                                            $is_upload                              = 1;
                                        }
                                    }else{
                                        $input                                      = [];
                                        $input['company_id']                        = auth()->guard('company-admin')->user()->company_id;
                                        $input['action_id']                         = $load_accessorial->id;
                                        $input['source']                            = 'load_accessorials';
                                        $input['field_name']                        = 'accessorial_files';
                                        $input['file_name']                         = null;
                                        $input['file_type']                         = getRequestDocumentsList('load_accessorials','source')['file_type'];
                                        $input['image']                             = $imageName;
                                        // $input['include_w_billing']              = 'no';
                                        // $input['hide_from_driver']               = 'no';
                                        $input['status']                            = 'approved';
                                        $input['created_by']                        = auth()->guard('company-admin')->user()->id;
                                        $input['created_by_type']                   = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                        $input['uploaded_date']                     = Carbon::now()->format('m/d/Y H:i');
                                        $gallery                                    = Gallery::create($input);

                                        $is_upload                                  = 1;
                                    }
                                }

                                if($is_upload==1){
                                    $history                                        = isset($load_accessorial->history) && is_serialized($load_accessorial->history) && !empty(unserialize($load_accessorial->history)) ? unserialize($load_accessorial->history) : [];

                                    $hv                                             = [];
                                    $hv['upload_user_type']                         = isset($gallery->created_by_type) ? $gallery->created_by_type : auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                    $hv['upload_by']                                = isset($gallery->created_by) ? $gallery->created_by : auth()->guard('company-admin')->user()->id;
                                    $hv['uploaded_at']                              = isset($gallery->created_at) ? $gallery->getRawOriginal('created_at') : Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                    array_push($history, $hv);

                                    // Update jsonData
                                    $jsonData['history']                            = $history;
                                    $jsonData['is_received']                        = 'on';
                                    $jsonData['file']                               = $gallery->image ?? null;
                                    $jsonData['upload_user_type']                   = isset($gallery->created_by_type) ? $gallery->created_by_type : null;
                                    $jsonData['upload_by']                          = isset($gallery->created_by) ? $gallery->created_by : null;
                                    if(isset($gallery->created_by_type) && isset($gallery->created_by)){
                                        $jsonData['uploaded_by']                    = getUserByType($gallery->created_by, $gallery->created_by_type);
                                    }else{
                                        $jsonData['uploaded_by']                    = null;
                                    }
                                    $jsonData['uploaded_at']                        = isset($gallery->created_at) ? Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null; // $gallery->getRawOriginal('uploaded_date')
                                    $jsonData['status']                             = 'Complete';

                                    // Load accessorial
                                    $load_accessorial->is_received                  = 1;
                                    // $load_accessorial->file_not_provided         = 'off';
                                    $load_accessorial->history                      = !empty($history) ? serialize($history) : null;

                                    //status
                                    $arry                                           = [];
                                    $paid_type                                      = $load_accessorial->paid_type;
                                    $reference                                      = $load_accessorial->reference;
                                    $total                                          = $load_accessorial->total;

                                    $owner_liable                                   = $load_accessorial->owner_liable;
                                    $owner_total                                    = $load_accessorial->owner_total;

                                    $driver_liable                                  = $load_accessorial->driver_liable;
                                    $driver_total                                   = $load_accessorial->driver_total;

                                    $doc_required                                   = $load_accessorial->doc_required;

                                    array_push($arry, $paid_type!=''?1:0);
                                    array_push($arry, $reference!=''?1:0);
                                    array_push($arry, $total!=''?1:0);
                                    if($owner_liable=='1'){
                                        array_push($arry, $owner_total!=''?1:0);
                                    }
                                    if($driver_liable=='1'){
                                        array_push($arry, $driver_total!=''?1:0);
                                    }

                                    if(in_array(0, $arry)){
                                        $load_accessorial->status                   = 0; // Pending
                                    }else{
                                        if($doc_required=='1'){
                                            if($load_accessorial->is_received==1){
                                                $load_accessorial->status           = 1; // Completed
                                            }else{
                                                $load_accessorial->status           = 0; // Pending
                                            }
                                        }else{
                                            $load_accessorial->status               = 1; // Completed
                                        }
                                    }
                                    if ($driver_liable=='1' && $load_accessorial->status!=1 && isset($load_accessorial->file_not_provided) && $load_accessorial->file_not_provided=='on') {
                                        $load_accessorial->status                   = 2; // Driver Bypass
                                    }
                                    //status

                                    if($load_accessorial->save()){
                                        $this->setLoadAudit($load_accessorial->load_id, 'update', 'Load accessorial updated', $load_accessorial->getChanges(), $oldData);
                                    }
                                }
                            }
                        }elseif($jsonData['source'] == 'load_files'){
                            $load_file                                              = LoadFile::where('id', $jsonData['id'])->first();
                            if(!is_null($load_file)){
                                $oldData                                            = $load_file->toArray();

                                $files                                              = [];
                                if(isset($load_file->document) && is_serialized($load_file->document) && !empty(unserialize($load_file->document))){
                                    $files                                          = unserialize($load_file->document);
                                }
                                if(isset($files) && !empty($files)){
                                    foreach ($files as $key => $file) {
                                        $path                                       = 'storage/companyAdmin/loadfile/'.$file;
                                        if (File::exists($path)) {
                                            File::delete($path);
                                        }
                                    }
                                }
                                $data['file']                                       = UploadImagePdf($formData['files'], 'companyAdmin/loadfile');
                                $data['upload_user_type']                           = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                $data['upload_by']                                  = auth()->guard('company-admin')->user()->id;
                                $data['uploaded_at']                                = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');

                                $history                                            = isset($load_file->history) && is_serialized($load_file->history) && !empty(unserialize($load_file->history)) ? unserialize($load_file->history) : [];

                                $hv                                                 = [];
                                $hv['upload_user_type']                             = $data['upload_user_type']; // Helper::getNotiUserTypeList();
                                $hv['upload_by']                                    = $data['upload_by'];
                                $hv['uploaded_at']                                  = $data['uploaded_at']; // date('Y-m-d H:i:s');
                                array_push($history, $hv);

                                // Update jsonData
                                $jsonData['history']                                = $history;
                                $jsonData['is_received']                            = 'on';
                                $jsonData['file']                                   = $data['file'];
                                $jsonData['upload_user_type']                       = $data['upload_user_type'];
                                $jsonData['upload_by']                              = $data['upload_by'];
                                $jsonData['uploaded_by']                            = isset($data['upload_by']) ? getUserByType($data['upload_by'], $data['upload_user_type']) : null;
                                $jsonData['uploaded_at']                            = Carbon::createFromFormat('Y-m-d H:i:s', $data['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId;
                                $jsonData['status']                                 = 'Complete';

                                $load_file->document                                = serialize($data['file']);
                                $load_file->history                                 = !empty($history) ? serialize($history) : null;
                                $load_file->upload_by                               = $data['upload_by'];
                                $load_file->upload_user_type                        = 'Company Admin';
                                if($load_file->save()){
                                    $this->setLoadAudit($load_file->load_id, 'update', 'Load document summary updated', $load_file->getChanges(), $oldData);
                                }
                            }
                        }
                    }

                    $response['html']                                               = view('companyAdmin.dispatchHub.ajaxview.completedLoad.documentDetail', compact('formData','jsonData'))->render();
                    $response['success']                                            = true;
                    $response['message']                                            = "Save successfully.";
                }
            } catch (\Exception $e) {
                $response['success']                                                = false;
                $response['message']                                                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function completedLoadFileRemove(Request $request){
        $response['success']                                                        = false;
        $response['message']                                                        = "Oops! Something went wrong..";
        if ($request->ajax()) {
            try {
                $formData                                                           = $request->all();
                if(isset($formData['jsonData']) && $formData['jsonData']!==''){
                    $jsonData                                                       = json_decode($formData['jsonData'], true);
                    $oldJsonData                                                    = $jsonData;

                    if(isset($jsonData['source'])){
                        if(in_array($jsonData['source'], ['load_details', 'load_stops'])){
                            $getArray                                               = [];
                            if($jsonData['source'] == 'load_details'){
                                $item                                               = LoadDetail::whereId($jsonData['id'])->first();
                                if(!is_null($item)){
                                    if ($jsonData['field_name'] == 'shipper_files_upload') {
                                        $getArray                                   = isset($item->shipper_files_upload) && is_serialized($item->shipper_files_upload) && !empty(unserialize($item->shipper_files_upload)) ? unserialize($item->shipper_files_upload) : [];
                                    } else if ($jsonData['field_name'] == 'consignee_files_upload') {
                                        $getArray                                       = isset($item->consignee_files_upload) && is_serialized($item->consignee_files_upload) && !empty(unserialize($item->consignee_files_upload)) ? unserialize($item->consignee_files_upload) : [];
                                    }
                                }
                            }else{
                                $item                                               = LoadStop::whereId($jsonData['id'])->first();
                                if(!is_null($item)){
                                    if($jsonData['field_name'] == 'stop_shipper_files_upload'){
                                        $getArray                                   = isset($item->stop_shipper_files_upload) && is_serialized($item->stop_shipper_files_upload) && !empty(unserialize($item->stop_shipper_files_upload)) ? unserialize($item->stop_shipper_files_upload) : [];
                                    } else if ($jsonData['field_name'] == 'stop_consignee_files_upload') {
                                        $getArray                                   = isset($item->stop_consignee_files_upload) && is_serialized($item->stop_consignee_files_upload) && !empty(unserialize($item->stop_consignee_files_upload)) ? unserialize($item->stop_consignee_files_upload) : [];
                                    }
                                }
                            }

                            if(!empty($getArray)){
                                $saveArray                                          = [];
                                foreach($getArray as $data){
                                    if(
                                        (isset($data['name_id']) && $data['name_id'] == $jsonData['filename_id']) &&
                                        (isset($data['name']) && $data['name'] == $jsonData['filename']) &&
                                        (isset($data['type']) && $data['type'] == $jsonData['type'])
                                    ){
                                        if($jsonData['type'] == 2){
                                            // Original
                                            $data['is_received']                    = 'off';
                                            // $data['file_not_provided']           = 'on';
                                            if(isset($data['upload_user_type'])){
                                                unset($data['upload_user_type']);
                                            }
                                            if(isset($data['upload_by'])){
                                                unset($data['upload_by']);
                                            }
                                            if(isset($data['uploaded_at'])){
                                                unset($data['uploaded_at']);
                                            }

                                            // Update jsonData
                                            $jsonData['is_received']                = $data['is_received'];
                                            if(isset($jsonData['upload_user_type'])){
                                                unset($jsonData['upload_user_type']);
                                            }
                                            if(isset($jsonData['upload_by'])){
                                                unset($jsonData['upload_by']);
                                            }
                                            if(isset($jsonData['uploaded_by'])){
                                                unset($jsonData['uploaded_by']);
                                            }
                                            if(isset($jsonData['uploaded_at'])){
                                                unset($jsonData['uploaded_at']);
                                            }
                                            $jsonData['status']                     = 'Pending';
                                        }else if($jsonData['type'] == 1){
                                            // Copy
                                            $files                                  = isset($data['file']) && !empty($data['file']) ? $data['file'] : null;
                                            if(isset($files) && !empty($files)){
                                                foreach ($files as $key => $file) {
                                                    $path                           = 'storage/companyAdmin/load_detail_document/'.$file;
                                                    if (File::exists($path)) {
                                                        File::delete($path);
                                                    }
                                                }
                                            }
                                            $data['is_received']                    = 'off';
                                            // $data['file_not_provided']           = 'on';
                                            $data['file']                           = null;
                                            if(isset($data['upload_user_type'])){
                                                unset($data['upload_user_type']);
                                            }
                                            if(isset($data['upload_by'])){
                                                unset($data['upload_by']);
                                            }
                                            if(isset($data['uploaded_at'])){
                                                unset($data['uploaded_at']);
                                            }

                                            // Update jsonData
                                            $jsonData['is_received']                = $data['is_received'];
                                            $jsonData['file']                       = $data['file'];
                                            if(isset($jsonData['upload_user_type'])){
                                                unset($jsonData['upload_user_type']);
                                            }
                                            if(isset($jsonData['upload_by'])){
                                                unset($jsonData['upload_by']);
                                            }
                                            if(isset($jsonData['uploaded_by'])){
                                                unset($jsonData['uploaded_by']);
                                            }
                                            if(isset($jsonData['uploaded_at'])){
                                                unset($jsonData['uploaded_at']);
                                            }
                                            $jsonData['status']                     = 'Pending';
                                        }
                                    }
                                    $saveArray[]                                    = $data;
                                }

                                if(isset($item)){
                                    if($jsonData['field_name'] == 'shipper_files_upload'){
                                        $oldData                                    = $item->toArray();
                                        $item->update(['shipper_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load detail updated', $item->getChanges(), $oldData);
                                    }
                                    if($jsonData['field_name'] == 'consignee_files_upload'){
                                        $oldData                                    = $item->toArray();
                                        $item->update(['consignee_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load detail updated', $item->getChanges(), $oldData);
                                    }
                                    if($jsonData['field_name'] == 'stop_shipper_files_upload'){
                                        $oldData                                    = $item->toArray();
                                        $item->update(['stop_shipper_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load stop updated', $item->getChanges(), $oldData);
                                    }
                                    if($jsonData['field_name'] == 'stop_consignee_files_upload'){
                                        $oldData                                    = $item->toArray();
                                        $item->update(['stop_consignee_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load stop updated', $item->getChanges(), $oldData);
                                    }
                                }
                            }
                        }elseif($jsonData['source'] == 'load_document_request_items'){
                            $load_document_request_item                             = LoadDocumentRequestItem::where('id', $jsonData['id'])->whereNotNull('files')->first();
                            if(!is_null($load_document_request_item)){
                                $oldData                                            = $load_document_request_item->toArray();

                                $data                                               = [];
                                if(isset($load_document_request_item->files) && is_serialized($load_document_request_item->files) && !empty(unserialize($load_document_request_item->files))){
                                    $data                                           = unserialize($load_document_request_item->files);
                                }

                                if(!empty($data)){
                                    if($jsonData['type'] == 2){
                                        // Original
                                        $data['is_received']                        = 'off';
                                        // $data['file_not_provided']               = 'on';
                                        if(isset($data['upload_user_type'])){
                                            unset($data['upload_user_type']);
                                        }
                                        if(isset($data['upload_by'])){
                                            unset($data['upload_by']);
                                        }
                                        if(isset($data['uploaded_at'])){
                                            unset($data['uploaded_at']);
                                        }

                                        // Update jsonData
                                        $jsonData['is_received']                    = $data['is_received'];
                                        if(isset($jsonData['upload_user_type'])){
                                            unset($jsonData['upload_user_type']);
                                        }
                                        if(isset($jsonData['upload_by'])){
                                            unset($jsonData['upload_by']);
                                        }
                                        if(isset($jsonData['uploaded_by'])){
                                            unset($jsonData['uploaded_by']);
                                        }
                                        if(isset($jsonData['uploaded_at'])){
                                            unset($jsonData['uploaded_at']);
                                        }
                                        $jsonData['status']                         = 'Pending';
                                    }else if($jsonData['type'] == 1){
                                        // Copy
                                        $files                                      = isset($data['file']) && !empty($data['file']) ? $data['file'] : null;
                                        if(isset($files) && !empty($files)){
                                            foreach ($files as $key => $file) {
                                                $path                               = 'storage/companyAdmin/load_detail_document/'.$file;
                                                if (File::exists($path)) {
                                                    File::delete($path);
                                                }
                                            }
                                        }
                                        $data['is_received']                        = 'off';
                                        // $data['file_not_provided']               = 'on';
                                        $data['file']                               = null;
                                        if(isset($data['upload_user_type'])){
                                            unset($data['upload_user_type']);
                                        }
                                        if(isset($data['upload_by'])){
                                            unset($data['upload_by']);
                                        }
                                        if(isset($data['uploaded_at'])){
                                            unset($data['uploaded_at']);
                                        }

                                        // Update jsonData
                                        $jsonData['is_received']                    = $data['is_received'];
                                        $jsonData['file']                           = $data['file'];
                                        if(isset($jsonData['upload_user_type'])){
                                            unset($jsonData['upload_user_type']);
                                        }
                                        if(isset($jsonData['upload_by'])){
                                            unset($jsonData['upload_by']);
                                        }
                                        if(isset($jsonData['uploaded_by'])){
                                            unset($jsonData['uploaded_by']);
                                        }
                                        if(isset($jsonData['uploaded_at'])){
                                            unset($jsonData['uploaded_at']);
                                        }
                                        $jsonData['status']                         = 'Pending';
                                    }
                                }

                                $load_document_request_item->files                  = !empty($data) ? serialize($data) : null;
                                $load_document_request_item->updated_at             = date('Y-m-d H:i:s');
                                $load_document_request_item->updated_by             = auth()->guard('company-admin')->user()->id;
                                $load_document_request_item->updated_by_type        = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                if($load_document_request_item->save()){
                                    $this->setLoadAudit($load_document_request_item->load_id, 'update', 'Load document request item updated', $load_document_request_item->getChanges(), $oldData);
                                }
                            }
                        }elseif($jsonData['source'] == 'load_accessorials'){
                            $load_accessorial                                       = LoadAccessorial::where('id', $jsonData['id'])->where('is_received', 1)->first();
                            if(!is_null($load_accessorial)){
                                $oldData                                            = $load_accessorial->toArray();

                                $gallery                                            = Gallery::where('company_id', auth()->guard('company-admin')->user()->company_id)
                                    ->where('action_id', $load_accessorial->id)
                                    ->where('source', 'load_accessorials')
                                    ->where('field_name', 'accessorial_files')
                                    ->whereNotNull('image')
                                    ->latest()->first();
                                if(!is_null($gallery)){
                                    $path                                           = 'storage/companyAdmin/gallery/'.$gallery->image;
                                    if (File::exists($path)) {
                                        File::delete($path);
                                    }
                                    $gallery->image                                 = null;
                                    $gallery->created_by                            = auth()->guard('company-admin')->user()->id;
                                    $gallery->created_by_type                       = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                    $gallery->uploaded_date                         = Carbon::now()->format('m/d/Y H:i');
                                    if($gallery->save()){
                                        // Update jsonData
                                        $jsonData['is_received']                    = 'off';
                                        $jsonData['file']                           = $gallery->image;
                                        if(isset($jsonData['upload_user_type'])){
                                            unset($jsonData['upload_user_type']);
                                        }
                                        if(isset($jsonData['upload_by'])){
                                            unset($jsonData['upload_by']);
                                        }
                                        if(isset($jsonData['uploaded_by'])){
                                            unset($jsonData['uploaded_by']);
                                        }
                                        if(isset($jsonData['uploaded_at'])){
                                            unset($jsonData['uploaded_at']);
                                        }
                                        $jsonData['status']                         = 'Pending';

                                        // Load accessorial
                                        $load_accessorial->is_received              = 0;
                                        // $load_accessorial->file_not_provided     = 'on';

                                        //status
                                        $arry                                       = [];
                                        $paid_type                                  = $load_accessorial->paid_type;
                                        $reference                                  = $load_accessorial->reference;
                                        $total                                      = $load_accessorial->total;

                                        $owner_liable                               = $load_accessorial->owner_liable;
                                        $owner_total                                = $load_accessorial->owner_total;

                                        $driver_liable                              = $load_accessorial->driver_liable;
                                        $driver_total                               = $load_accessorial->driver_total;

                                        $doc_required                               = $load_accessorial->doc_required;

                                        array_push($arry, $paid_type!=''?1:0);
                                        array_push($arry, $reference!=''?1:0);
                                        array_push($arry, $total!=''?1:0);
                                        if($owner_liable=='1'){
                                            array_push($arry, $owner_total!=''?1:0);
                                        }
                                        if($driver_liable=='1'){
                                            array_push($arry, $driver_total!=''?1:0);
                                        }

                                        if(in_array(0, $arry)){
                                            $load_accessorial->status               = 0; // Pending
                                        }else{
                                            if($doc_required=='1'){
                                                if($load_accessorial->is_received==1){
                                                    $load_accessorial->status       = 1; // Completed
                                                }else{
                                                    $load_accessorial->status       = 0; // Pending
                                                }
                                            }else{
                                                $load_accessorial->status           = 1; // Completed
                                            }
                                        }
                                        if ($driver_liable=='1' && $load_accessorial->status!=1 && isset($load_accessorial->file_not_provided) && $load_accessorial->file_not_provided=='on') {
                                            $load_accessorial->status               = 2; // Driver Bypass
                                        }
                                        //status

                                        if($load_accessorial->save()){
                                            $this->setLoadAudit($load_accessorial->load_id, 'update', 'Load accessorial updated', $load_accessorial->getChanges(), $oldData);
                                        }
                                    }
                                }
                            }
                        }elseif($jsonData['source'] == 'load_files'){
                            $load_file                                              = LoadFile::where('id', $jsonData['id'])->whereNotNull('document')->first();
                            if(!is_null($load_file)){
                                $oldData                                            = $load_file->toArray();

                                $files                                              = [];
                                if(isset($load_file->document) && is_serialized($load_file->document) && !empty(unserialize($load_file->document))){
                                    $files                                          = unserialize($load_file->document);
                                }
                                if(isset($files) && !empty($files)){
                                    foreach ($files as $key => $file) {
                                        $path                                       = 'storage/companyAdmin/loadfile/'.$file;
                                        if (File::exists($path)) {
                                            File::delete($path);
                                        }
                                    }
                                }

                                // Update jsonData
                                $jsonData['is_received']                            = 'off';
                                $jsonData['file']                                   = null;
                                if(isset($jsonData['upload_user_type'])){
                                    unset($jsonData['upload_user_type']);
                                }
                                if(isset($jsonData['upload_by'])){
                                    unset($jsonData['upload_by']);
                                }
                                if(isset($jsonData['uploaded_by'])){
                                    unset($jsonData['uploaded_by']);
                                }
                                if(isset($jsonData['uploaded_at'])){
                                    unset($jsonData['uploaded_at']);
                                }
                                $jsonData['status']                                 = 'Pending';

                                $load_file->document                                = null;
                                $load_file->upload_by                               = auth()->guard('company-admin')->user()->id;
                                $load_file->upload_user_type                        = 'Company Admin';
                                if($load_file->save()){
                                    $this->setLoadAudit($load_file->load_id, 'update', 'Load document summary updated', $load_file->getChanges(), $oldData);
                                }
                            }
                        }
                    }

                    $response['html']                                               = view('companyAdmin.dispatchHub.ajaxview.completedLoad.documentDetail', compact('formData','jsonData'))->render();
                    $response['success']                                            = true;
                    $response['message']                                            = "Remove successfully.";
                }
            } catch (\Exception $e) {
                $response['success']                                                = false;
                $response['message']                                                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function completedLoadRequestDocument(Request $request){
        $response['success']                                                        = false;
        $response['message']                                                        = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $formData                                                           = $request->all();

                if(isset($formData['jsonData']) && $formData['jsonData']!==''){
                    $jsonData                                                       = json_decode($formData['jsonData'], true);

                    $response['html']                                               = view('companyAdmin.dispatchHub.ajaxview.completedLoad.requestDocument', compact('formData','jsonData'))->render();
                    $response['success']                                            = true;
                    $response['message']                                            = "Request Document view retrieved successfully.";
                }
            }catch (\Exception $e) {
                $response['success']                                                = false;
                $response['message']                                                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function completedLoadRequestDocumentSave(Request $request){
        $response['success']                                                        = false;
        $response['message']                                                        = "Oops! Something went wrong..";
        if ($request->ajax()) {
            try {
                $input                                                              = $request->all();
                $formData                                                           = json_decode($input['params'], true);

                if(!empty($formData) && isset($formData['jsonData']) && $formData['jsonData']!==''){
                    $jsonData                                                       = json_decode($formData['jsonData'], true);
                    $oldJsonData                                                    = $jsonData;

                    if(isset($jsonData['source'])){
                        if(in_array($jsonData['source'], ['load_details', 'load_stops'])){
                            $getArray                                               = [];
                            if($jsonData['source'] == 'load_details'){
                                $item                                               = LoadDetail::whereId($jsonData['id'])->first();
                                if(!is_null($item)){
                                    if ($jsonData['field_name'] == 'shipper_files_upload') {
                                        $getArray                                   = isset($item->shipper_files_upload) && is_serialized($item->shipper_files_upload) && !empty(unserialize($item->shipper_files_upload)) ? unserialize($item->shipper_files_upload) : [];
                                    } else if ($jsonData['field_name'] == 'consignee_files_upload') {
                                        $getArray                                       = isset($item->consignee_files_upload) && is_serialized($item->consignee_files_upload) && !empty(unserialize($item->consignee_files_upload)) ? unserialize($item->consignee_files_upload) : [];
                                    }
                                }
                            }else{
                                $item                                               = LoadStop::whereId($jsonData['id'])->first();
                                if(!is_null($item)){
                                    if($jsonData['field_name'] == 'stop_shipper_files_upload'){
                                        $getArray                                   = isset($item->stop_shipper_files_upload) && is_serialized($item->stop_shipper_files_upload) && !empty(unserialize($item->stop_shipper_files_upload)) ? unserialize($item->stop_shipper_files_upload) : [];
                                    } else if ($jsonData['field_name'] == 'stop_consignee_files_upload') {
                                        $getArray                                   = isset($item->stop_consignee_files_upload) && is_serialized($item->stop_consignee_files_upload) && !empty(unserialize($item->stop_consignee_files_upload)) ? unserialize($item->stop_consignee_files_upload) : [];
                                    }
                                }
                            }

                            if(!empty($getArray)){
                                $saveArray                                          = [];
                                foreach($getArray as $data){
                                    if(
                                        (isset($data['name_id']) && $data['name_id'] == $jsonData['filename_id']) &&
                                        (isset($data['name']) && $data['name'] == $jsonData['filename']) &&
                                        (isset($data['type']) && $data['type'] == $jsonData['type'])
                                    ){
                                        $history                                    = isset($data['history']) && is_array($data['history']) && !empty($data['history']) ? $data['history'] : [];

                                        $hv                                         = [];
                                        $hv['requested_at']                         = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                        $hv['requested_by']                         = auth()->guard('company-admin')->user()->id;
                                        $hv['requested_by_type']                    = auth()->guard('company-admin')->user()->type; // Helper::getNotiUserTypeList();
                                        $hv['requested_to']                         = $input['driver_id'];
                                        $hv['request_via']                          = $input['request_via']; // 1 = Email, 2 = SMS
                                        $hv['instructions']                         = $input['instructions'];
                                        array_push($history, $hv);

                                        $data['include_w_billing']                  = isset($input['include_w_billing']) && $input['include_w_billing'] == 'yes' ? 'yes' : 'no';
                                        $data['history']                            = $history;

                                        // Update jsonData
                                        $jsonData['include_w_billing']              = $data['include_w_billing'];
                                        $jsonData['history']                        = $data['history'];
                                    }
                                    $saveArray[]                                    = $data;
                                }

                                if(isset($item)){
                                    if($jsonData['field_name'] == 'shipper_files_upload'){
                                        $oldData                                    = $item->toArray();
                                        $item->update(['shipper_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load detail updated', $item->getChanges(), $oldData);
                                    }
                                    if($jsonData['field_name'] == 'consignee_files_upload'){
                                        $oldData                                    = $item->toArray();
                                        $item->update(['consignee_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load detail updated', $item->getChanges(), $oldData);
                                    }
                                    if($jsonData['field_name'] == 'stop_shipper_files_upload'){
                                        $oldData                                    = $item->toArray();
                                        $item->update(['stop_shipper_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load stop updated', $item->getChanges(), $oldData);
                                    }
                                    if($jsonData['field_name'] == 'stop_consignee_files_upload'){
                                        $oldData                                    = $item->toArray();
                                        $item->update(['stop_consignee_files_upload' => serialize($saveArray)]);
                                        $this->setLoadAudit($item->load_id, 'update', 'Load stop updated', $item->getChanges(), $oldData);
                                    }
                                }
                            }
                        }elseif($jsonData['source'] == 'load_document_request_items'){
                            $load_document_request_item                             = LoadDocumentRequestItem::where('id', $jsonData['id'])->first();
                            if(!is_null($load_document_request_item)){
                                $oldData                                            = $load_document_request_item->toArray();

                                $data                                               = [];
                                if(isset($load_document_request_item->files) && is_serialized($load_document_request_item->files) && !empty(unserialize($load_document_request_item->files))){
                                    $data                                           = unserialize($load_document_request_item->files);
                                }

                                if(!empty($data)){
                                    $history                                        = isset($data['history']) && is_array($data['history']) && !empty($data['history']) ? $data['history'] : [];

                                    $hv                                             = [];
                                    $hv['requested_at']                             = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                    $hv['requested_by']                             = auth()->guard('company-admin')->user()->id;
                                    $hv['requested_by_type']                        = auth()->guard('company-admin')->user()->type; // Helper::getNotiUserTypeList();
                                    $hv['requested_to']                             = $input['driver_id'];
                                    $hv['request_via']                              = $input['request_via']; // 1 = Email, 2 = SMS
                                    $hv['instructions']                             = $input['instructions'];
                                    array_push($history, $hv);

                                    $data['include_w_billing']                      = isset($input['include_w_billing']) && $input['include_w_billing'] == 'yes' ? 'yes' : 'no';
                                    $data['history']                                = $history;

                                    // Update jsonData
                                    $jsonData['include_w_billing']                  = $data['include_w_billing'];
                                    $jsonData['history']                            = $data['history'];
                                }

                                $load_document_request_item->files                  = !empty($data) ? serialize($data) : null;
                                $load_document_request_item->updated_at             = date('Y-m-d H:i:s');
                                $load_document_request_item->updated_by             = auth()->guard('company-admin')->user()->id;
                                $load_document_request_item->updated_by_type        = auth()->guard('company-admin')->user()->type; // For Company Admin User Type
                                if($load_document_request_item->save()){
                                    $this->setLoadAudit($load_document_request_item->load_id, 'update', 'Load document request item updated', $load_document_request_item->getChanges(), $oldData);
                                }
                            }
                        }elseif($jsonData['source'] == 'load_accessorials'){
                            $load_accessorial                                       = LoadAccessorial::where('id', $jsonData['id'])->first();
                            if(!is_null($load_accessorial)){
                                $oldData                                            = $load_accessorial->toArray();

                                $history                                            = isset($load_accessorial->history) && is_serialized($load_accessorial->history) && !empty(unserialize($load_accessorial->history)) ? unserialize($load_accessorial->history) : [];

                                $hv                                                 = [];
                                $hv['requested_at']                                 = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                $hv['requested_by']                                 = auth()->guard('company-admin')->user()->id;
                                $hv['requested_by_type']                            = auth()->guard('company-admin')->user()->type; // Helper::getNotiUserTypeList();
                                $hv['requested_to']                                 = $input['driver_id'];
                                $hv['request_via']                                  = $input['request_via']; // 1 = Email, 2 = SMS
                                $hv['instructions']                                 = $input['instructions'];
                                array_push($history, $hv);

                                $load_accessorial->include_w_billing                = isset($input['include_w_billing']) && $input['include_w_billing'] == 'yes' ? 'yes' : 'no';
                                $load_accessorial->history                          = serialize($history);

                                // Update jsonData
                                $jsonData['include_w_billing']                      = $load_accessorial->include_w_billing;
                                $jsonData['history']                                = $history;

                                if($load_accessorial->save()){
                                    $this->setLoadAudit($load_accessorial->load_id, 'update', 'Load accessorial updated', $load_accessorial->getChanges(), $oldData);
                                }
                            }
                        }elseif($jsonData['source'] == 'load_files'){
                            $load_file                                              = LoadFile::where('id', $jsonData['id'])->first();
                            if(!is_null($load_file)){
                                $oldData                                            = $load_file->toArray();

                                $history                                            = isset($load_file->history) && is_serialized($load_file->history) && !empty(unserialize($load_file->history)) ? unserialize($load_file->history) : [];

                                $hv                                                 = [];
                                $hv['requested_at']                                 = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                $hv['requested_by']                                 = auth()->guard('company-admin')->user()->id;
                                $hv['requested_by_type']                            = auth()->guard('company-admin')->user()->type; // Helper::getNotiUserTypeList();
                                $hv['requested_to']                                 = $input['driver_id'];
                                $hv['request_via']                                  = $input['request_via']; // 1 = Email, 2 = SMS
                                $hv['instructions']                                 = $input['instructions'];
                                array_push($history, $hv);

                                $load_file->include_w_billing                       = isset($input['include_w_billing']) && $input['include_w_billing'] == 'yes' ? 'yes' : 'no';
                                $load_file->history                                 = serialize($history);

                                // Update jsonData
                                $jsonData['include_w_billing']                      = $load_file->include_w_billing;
                                $jsonData['history']                                = $history;

                                if($load_file->save()){
                                    $this->setLoadAudit($load_file->load_id, 'update', 'Load document summary updated', $load_file->getChanges(), $oldData);
                                }
                            }
                        }
                    }

                    // Send Email & SMS
                    $random_id                                                      = $formData['random_id'];

                    $id                                                             = explode('_', $random_id)[0] ?? ''; // Load Detail Id
                    $load_detail                                                    = LoadDetail::whereId($id)->first();

                    $driver_id                                                      = $input['driver_id'];
                    $driver                                                         = Driver::whereId($driver_id)->first();

                    if(!is_null($load_detail) && !is_null($driver)){
                        $driver_full_name                                           = isset($driver->full_name) ? $driver->full_name : '';

                        $details                                                    = [];
                        $details['company_id']                                      = auth()->guard('company-admin')->user()->company_id;
                        $details['full_name']                                       = $driver_full_name;
                        $details['load_id']                                         = $load_detail->load_id;
                        $details['load_detail_id']                                  = $load_detail->id;
                        $details['load_number']                                     = $load_detail->load_type == 0 || $load_detail->load_type == 6 ? $load_detail->load_number : getLoadTypeList()[$load_detail->load_type];

                        $stop_name                                                  = explode(':',$input['stop_no'])[0] ?? '';
                        $stop_name                                                  .= $input['name'] ? ' - '.$input['name']: '';
                        $documents[]                                                = $stop_name;
                        $details['documents']                                       = $documents;

                        if(isset($input['request_via'])){
                            if($input['request_via'] == 1 && isset($driver->email) && $driver->email !=''){
                                // 1 = Email
                                sendEmail($driver->email, new \App\Mail\LoadDocumentRequestDriver($details));
                            } elseif ($input['request_via'] == 2 && isset($driver->cell_phone) && $driver->cell_phone !='') {
                                // 2 = SMS
                                // Twilio's credentials are fetched from company table
                                $getCompanyTwilioData                               = getCompanyTwilioData();
                                if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                                    sendSMS($driver->cell_phone, 'A document request has been added to load number: '.$details['load_number'].'. Documents requested are: '.implode(', ', $details['documents']).'. Please upload these documents to the load. '.route('driver.admin.mytrips.index'), $getCompanyTwilioData);
                                }
                            }
                        }
                    }
                    // Send Email & SMS

                    $response['html']                                               = view('companyAdmin.dispatchHub.ajaxview.completedLoad.documentDetail', compact('formData','jsonData'))->render();
                    $response['success']                                            = true;
                    $response['message']                                            = "Save successfully.";
                }
            } catch (\Exception $e) {
                $response['success']                                                = false;
                $response['message']                                                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function completedLoadRequestDocumentHistory(Request $request){
        $response['success']                                                        = false;
        $response['message']                                                        = "Oops! Something went wrong..";
        if ($request->ajax()) {
            try {
                $formData                                                           = $request->all();
                if(isset($formData['jsonData']) && $formData['jsonData']!==''){
                    $jsonData                                                       = json_decode($formData['jsonData'], true);

                    $history                                                        = isset($jsonData['history']) && is_array($jsonData['history']) && !empty($jsonData['history']) ? $jsonData['history'] : [];

                    $html                                                           = '';
                    $i                                                              = 1;
                    if(!empty($history)){
                        foreach ($history as $key => $hvalue) {
                            // Request Code
                            $requested_at                                           = isset($hvalue['requested_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $hvalue['requested_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : '------';
                            $requested_by_full_name                                 = '------';
                            if(isset($hvalue['requested_by']) && $hvalue['requested_by'] != ''){
                                $requested_by                                       = CompanyAdmin::whereId($hvalue['requested_by'])->first();
                                if(!is_null($requested_by)){
                                    $requested_by_full_name                         = $requested_by->full_name;
                                }
                            }
                            $requested_to_full_name                                 = '------';
                            if(isset($hvalue['requested_to']) && $hvalue['requested_to'] != ''){
                                $requested_to                                       = Driver::whereId($hvalue['requested_to'])->first();
                                if(!is_null($requested_to)){
                                    $requested_to_full_name                         = [];
                                    if(isset($requested_to->last_name) && $requested_to->last_name != ''){
                                        $requested_to_full_name[]                   = $requested_to->last_name;
                                    }
                                    if(isset($requested_to->first_name) && $requested_to->first_name != ''){
                                        $requested_to_full_name[]                   = $requested_to->first_name;
                                    }
                                    $requested_to_full_name                         = !empty($requested_to_full_name) ? implode(', ', $requested_to_full_name) : '------';
                                }
                            }

                            // Upload Code
                            $uploaded_at                                            = isset($hvalue['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $hvalue['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : '------';
                            $uploaded_by_full_name                                  = '------';
                            if(isset($hvalue['upload_by']) && $hvalue['upload_by'] != ''){
                                if(isset($hvalue['upload_user_type']) && $hvalue['upload_user_type'] == '3'){
                                    $uploaded_by                                    = Driver::whereId($hvalue['upload_by'])->first();
                                    if(!is_null($uploaded_by)){
                                        $uploaded_by_full_name                      = [];
                                        if(isset($uploaded_by->last_name) && $uploaded_by->last_name != ''){
                                            $uploaded_by_full_name[]                = $uploaded_by->last_name;
                                        }
                                        if(isset($uploaded_by->first_name) && $uploaded_by->first_name != ''){
                                            $uploaded_by_full_name[]                = $uploaded_by->first_name;
                                        }
                                        $uploaded_by_full_name                      = !empty($uploaded_by_full_name) ? implode(', ', $uploaded_by_full_name) : '------';
                                    }
                                }else{
                                    $uploaded_by                                    = CompanyAdmin::whereId($hvalue['upload_by'])->first();
                                    if(!is_null($uploaded_by)){
                                        $uploaded_by_full_name                      = $uploaded_by->full_name;
                                    }
                                }
                            }
                            $upload_type                                            = '------';
                            if(isset($hvalue['uploaded_at'])){
                                $upload_type                                        = $jsonData['type'] == '1' ? 'Upload' : 'Marked Received';
                            }

                            $html                                                   .= '<tr>';
                            $html                                                   .= '<td>'.$i.'</td>';
                            $html                                                   .= '<td>'.$requested_at.'</td>';
                            $html                                                   .= '<td>'.$uploaded_at.'</td>';
                            $html                                                   .= '<td>'.$requested_by_full_name.'</td>';
                            $html                                                   .= '<td>'.$requested_to_full_name.'</td>';
                            $html                                                   .= '<td>'.$uploaded_by_full_name.'</td>';
                            $html                                                   .= '<td>'.$upload_type.'</td>';
                            $html                                                   .= '</tr>';
                            $i++;
                        }
                    }
                    if($html == ''){
                        $html                                                       .= '<tr><td colspan="7">No data available in table</td></tr>';
                    }

                    $response['html']                                               = $html;
                    $response['success']                                            = true;
                    $response['message']                                            = "History Retrieved successfully.";
                }
            } catch (\Exception $e) {
                $response['success']                                                = false;
                $response['message']                                                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function completedLoadAddDocumentation(Request $request){
        $response['success']                                                        = false;
        $response['message']                                                        = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $formData                                                           = $request->all();
                $load_detail                                                        = LoadDetail::whereId($formData['id'])->first();
                if(!is_null($load_detail)){
                    // Please compare with load view

                    $params                                                         = [
                        'load_id'                                                   => $load_detail->load_id,
                        'load_detail_id'                                            => $load_detail->id,
                        'broker_id'                                                 => $load_detail->brokerage_firm
                    ];

                    // Load stop list
                    $stopNo                                                         = 1;
                    $stopsArry                                                      = [];
                    $stopsArry[]                                                    = [
                        'stop_no'                                                   => $stopNo,
                        'load_id'                                                   => $load_detail->load_id,
                        'load_detail_id'                                            => $load_detail->id,
                        'load_stop_id'                                              => null,
                        'type'                                                      => 1, // pickup
                        'is_stop'                                                   => 0,
                        'customer_name'                                             => $load_detail->shipper->name ?? '',
                        'address_one'                                               => $load_detail->shipper_address_one ?? null,
                        'address_one'                                               => $load_detail->shipper_address_one ?? null,
                        'state'                                                     => $load_detail->shipper_state ?? null,
                        'city'                                                      => $load_detail->shipper_city ?? null,
                        'zip'                                                       => $load_detail->shipper_zip ?? null
                    ];
                    if(isset($load_detail->loads->load_stops) && $load_detail->loads->load_stops->isNotEmpty()){
                        foreach ($load_detail->loads->load_stops as $key => $stop) {
                            ++$stopNo;
                            $stopsArry[]                                            = [
                                'stop_no'                                           => $stopNo,
                                'load_id'                                           => $load_detail->load_id,
                                'load_detail_id'                                    => $load_detail->id,
                                'load_stop_id'                                      => $stop->id,
                                'type'                                              => $stop->customer_type, // stop
                                'is_stop'                                           => 1,
                                'customer_name'                                     => $stop->customer->name ?? '',
                                'address_one'                                       => $stop->address_one ?? null,
                                'state'                                             => $stop->state ?? null,
                                'city'                                              => $stop->city ?? null,
                                'zip'                                               => $stop->zip ?? null
                            ];
                        }
                    }
                    $stopsArry[]                                                    = [
                        'stop_no'                                                   => 99,
                        'load_id'                                                   => $load_detail->load_id,
                        'load_detail_id'                                            => $load_detail->id,
                        'load_stop_id'                                              => null,
                        'type'                                                      => 2, // delivery
                        'is_stop'                                                   => 0,
                        'customer_name'                                             => $load_detail->receiver->name ?? '',
                        'address_one'                                               => $load_detail->delivery_address_one ?? null,
                        'state'                                                     => $load_detail->delivery_state ?? null,
                        'city'                                                      => $load_detail->delivery_city ?? null,
                        'zip'                                                       => $load_detail->delivery_zip ?? null
                    ];
                    $params['load_stops_array']                                     = $stopsArry;
                    // Load stop list

                    // Load driver list
                    $load_drivers_array                                             = [];
                    if(isset($load_detail->driver)){
                        $driver                                                     = $load_detail->driver;
                        $driver_full_name                                           = [];
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                                     = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                                     = $driver->first_name;
                        }
                        $driver_full_name                                           = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';
                        $load_drivers_array[$driver->id]                            = [
                            'name'                  => $driver_full_name,
                            'email'                 => $driver->email,
                            'cell_phone'            => $driver->cell_phone,
                        ];
                    }
                    if(isset($load_detail->co_driver)){
                        $driver                                                     = $load_detail->co_driver;
                        $driver_full_name                                           = [];
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                                     = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                                     = $driver->first_name;
                        }
                        $driver_full_name                                           = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';
                        $load_drivers_array[$driver->id]                            = [
                            'name'                  => $driver_full_name,
                            'email'                 => $driver->email,
                            'cell_phone'            => $driver->cell_phone,
                        ];
                    }
                    if(isset($load_detail->load_drivers)){
                        $load_drivers                                               = $load_detail->load_drivers;
                        foreach ($load_drivers as $key => $load_driver) {
                            if(isset($load_driver->driver)){
                                $driver                                             = $load_driver->driver;
                                $driver_full_name                                   = [];
                                if(isset($driver->last_name) && $driver->last_name != ''){
                                    $driver_full_name[]                             = $driver->last_name;
                                }
                                if(isset($driver->first_name) && $driver->first_name != ''){
                                    $driver_full_name[]                             = $driver->first_name;
                                }
                                $driver_full_name                                   = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';
                                $load_drivers_array[$driver->id]                    = [
                                    'name'                  => $driver_full_name,
                                    'email'                 => $driver->email,
                                    'cell_phone'            => $driver->cell_phone,
                                ];
                            }
                        }
                    }
                    // Load driver list

                    $load_documents                                                 = getCompanyLoadDocument();

                    $response['html']                                               = view('companyAdmin.dispatchHub.ajaxview.completedLoad.addDocument', compact('params','load_drivers_array','load_documents'))->render();
                    $response['success']                                            = true;
                    $response['message']                                            = "Add Documentation view retrieved successfully.";
                }
            }catch (\Exception $e) {
                $response['success']                                                = false;
                $response['message']                                                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function completedLoadAddDocumentationSave(Request $request){
        $response['success']                                                        = false;
        $response['message']                                                        = "Oops! Something went wrong..";
        if ($request->ajax()) {
            try {
                $formData                                                           = $request->all();

                $params                                                             = json_decode($formData['params'], true);

                $load_stops_array                                                   = $params['load_stops_array'];
                $searchkey                                                          = array_search($formData['doc_req_stop_no'], array_column($params['load_stops_array'], 'stop_no'));

                $filearry                                                           = [];
                $filearry['name_id']                                                = $formData['doc_req_name_id'];
                $filearry['name']                                                   = isset($formData['doc_req_name']) ? $formData['doc_req_name'] : '';
                $filearry['is_required']                                            = 'on';
                $filearry['type']                                                   = $formData['doc_req_type'];
                $filearry['fixed']                                                  = 'no';
                $filearry['custom']                                                 = 'no';
                $filearry['is_received']                                            = 'off';
                $filearry['include_w_billing']                                      = isset($formData['doc_req_include_w_billing']) && $formData['doc_req_include_w_billing']=='yes' ? 'yes' : 'no';
                $filearry['hide_from_driver']                                       = 'no';
                $filearry['file_not_provided']                                      = 'off';
                $filearry['file']                                                   = null;
                $filearry['history']                                                = null;

                $request_via                                                        = [];
                if(isset($formData['doc_req_email']) && $formData['doc_req_email'] == 'on'){
                    $request_via[]                                                  = 1;
                }
                if(isset($formData['doc_req_sms']) && $formData['doc_req_sms'] == 'on'){
                    $request_via[]                                                  = 2;
                }
                $contacts                                                           = [];
                if(isset($formData['doc_req_send_driver_id']) && $formData['doc_req_send_driver_id'] != ''){
                    array_push($contacts, [
                        'driver_id' => $formData['doc_req_send_driver_id'],
                        'email'     => $formData['doc_req_send_email'] ?? '',
                        'phone'     => $formData['doc_req_send_phone'] ?? ''
                    ]);
                }

                $add                                                                = 0;
                $load_document_request                                              = LoadDocumentRequest::query()
                    ->where('load_id', $params['load_id'])
                    ->where('load_detail_id', $params['load_detail_id'])
                    ->where('broker_id', $params['broker_id'])
                    ->first();
                if(!is_null($load_document_request)){
                    $item                                                           = new LoadDocumentRequestItem();
                    $item->load_document_request_id                                 = $load_document_request->id;
                    $item->load_id                                                  = $load_document_request->load_id;
                    $item->load_detail_id                                           = $load_document_request->load_detail_id;
                    $item->driver_id                                                = $formData['doc_req_driver_id'];
                    $item->load_stop_id                                             = $load_stops_array[$searchkey]['load_stop_id'];
                    $item->type                                                     = $load_stops_array[$searchkey]['type'];
                    $item->is_stop                                                  = $load_stops_array[$searchkey]['is_stop'];
                    $item->files                                                    = serialize($filearry);
                    $item->created_at                                               = Carbon::now()->format('Y-m-d H:i:s');
                    $item->created_by                                               = auth()->guard('company-admin')->user()->id;
                    $item->created_by_type                                          = auth()->guard('company-admin')->user()->type;
                    if($item->save()){
                        $add                                                        = 1;
                    }
                }else{
                    $load_document_request                                          = new LoadDocumentRequest();
                    $load_document_request->load_id                                 = $params['load_id'];
                    $load_document_request->load_detail_id                          = $params['load_detail_id'] ?? null;
                    $load_document_request->broker_id                               = $params['broker_id'] ?? null;
                    $load_document_request->created_at                              = Carbon::now()->format('Y-m-d H:i:s');
                    $load_document_request->created_by                              = auth()->guard('company-admin')->user()->id;
                    $load_document_request->created_by_type                         = auth()->guard('company-admin')->user()->type;
                    if($load_document_request->save()){
                        $item                                                       = new LoadDocumentRequestItem();
                        $item->load_document_request_id                             = $load_document_request->id;
                        $item->load_id                                              = $load_document_request->load_id;
                        $item->load_detail_id                                       = $load_document_request->load_detail_id;
                        $item->driver_id                                            = $formData['doc_req_driver_id'];
                        $item->load_stop_id                                         = $load_stops_array[$searchkey]['load_stop_id'];
                        $item->type                                                 = $load_stops_array[$searchkey]['type'];
                        $item->is_stop                                              = $load_stops_array[$searchkey]['is_stop'];
                        $item->files                                                = serialize($filearry);
                        $item->created_at                                           = Carbon::now()->format('Y-m-d H:i:s');
                        $item->created_by                                           = auth()->guard('company-admin')->user()->id;
                        $item->created_by_type                                      = auth()->guard('company-admin')->user()->type;
                        if($item->save()){
                            $add                                                    = 1;
                        }
                    }
                }

                if($add == 1){
                    // Send Email & SMS
                    if(isset($item) && !is_null($item) && !empty($contacts)){
                        // item = Load Document Request Item
                        $load_detail                                                = $item->load_detail ?? null;
                        $driver                                                     = $item->driver ?? null;
                        if(!is_null($load_detail) && !is_null($driver)){
                            $driver_full_name                                       = isset($driver->full_name) ? $driver->full_name : '';

                            $details                                                = [];
                            $details['company_id']                                  = auth()->guard('company-admin')->user()->company_id;
                            $details['full_name']                                   = $driver_full_name;
                            $details['load_id']                                     = $load_detail->load_id;
                            $details['load_detail_id']                              = $load_detail->id;
                            $details['load_number']                                 = $load_detail->load_type == 0 || $load_detail->load_type == 6 ? $load_detail->load_number : getLoadTypeList()[$load_detail->load_type];

                            $documents                                              = [];
                            //
                            $estop_no                                               = '';
                            $searchResults                                          = array_filter($params['load_stops_array'], function($var) use ($item){
                                return (
                                    $var['type'] == $item->type &&
                                    $var['is_stop'] == $item->is_stop &&
                                    $var['load_stop_id'] == $item->load_stop_id
                                );
                            });
                            if(!empty($searchResults)){
                                $firstMatchingItem                                  = reset($searchResults); // Get the first element of the filtered array
                                $estop_no                                           = $firstMatchingItem['stop_no'] ?? '';
                            }
                            $files                                                  = [];
                            if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                                $files                                              = unserialize($item->files);
                            }
                            $stop_name                                              = 'Stop '.$estop_no;
                            if(isset($files['name_id']) && $files['name_id']!=''){
                                if($files['name_id'] == 0){
                                    if(isset($files['name']) && $files['name']!=''){
                                        $stop_name                                  .= ' - '.$files['name']. ' ( Other )';
                                    }else{
                                        $stop_name                                  .= 'Other';
                                    }
                                }else{
                                    $stop_name                                      .= $files['name'] ? ' - '.$files['name']: '';
                                }
                            }
                            $documents[]                                            = $stop_name;
                            //
                            $details['documents']                                   = $documents;

                            //
                            foreach ($contacts as $key => $contact) {
                                // send Mail
                                if(isset($contact['email']) && $contact['email']!=''){
                                    sendEmail($contact['email'], new \App\Mail\LoadDocumentRequestDriver($details));
                                }

                                // send Sms
                                if(isset($contact['phone']) && $contact['phone']!=''){
                                    // Twilio's credentials are fetched from company table
                                    $getCompanyTwilioData                           = getCompanyTwilioData();
                                    if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                                        sendSMS($contact['phone'], 'A document request has been added to load number: '.$details['load_number'].'. Documents requested are: '.implode(', ', $details['documents']).'. Please upload these documents to the load. '.route('driver.admin.mytrips.index'), $getCompanyTwilioData);
                                    }
                                }
                            }
                        }
                    }

                    $response['success']                                            = true;
                    $response['message']                                            = "Document request submitted successfully.";
                }else{
                    $response['success']                                            = false;
                    $response['message']                                            = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success']                                                = false;
                $response['message']                                                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function completedLoadChangeStatus(Request $request){
        $response['success']                                                        = false;
        $response['message']                                                        = "Oops! Something went wrong..";
        if ($request->ajax()) {
            try {
                $item                                                               = LoadDetail::where('id', $request->id)->first();
                if(!is_null($item)){
                    $oldData                                                        = $item->toArray();

                    // Change status not load stop only load detail for functionality
                    // 1 = Preplaned Driver Acknowledgement, 2 = Confirmed Driver Acknowledgement, 3 = Intransit Driver Acknowledgement, 4 = Delivered Driver Acknowledgement
                    // 0 = Pending', 1 = Accepted, 2 = Preplaned, 11 = Send To Driver, 3 = Confirmed, 4 = In Transit, 5 = Delivered, 6 = Ready to Bill, 10 = Invoiced, 8 = Paid
                    // 1 = On Hold, 2 = Cancelled, 3 = Drooped
                    $item->load_status                                              = 6;
                    $item->load_status_order                                        = Common::getLoadStatusOrderById( 6 );
                    $item->updated_at                                               = date('Y-m-d H:i:s');
                    if($item->save()){
                        $this->setLoadAudit($item->load_id, 'update', 'Load detail updated', $item->getChanges(), $oldData);
                        createLoadLog($item->getChanges(), $item, 'edit');

                        $response['success']                                        = true;
                        $response['message']                                        = "Status has been changed successfully.";
                    }else{
                        $response['success']                                        = false;
                        $response['message']                                        = "Status has been changed unsuccessfully.";
                    }
                }else{
                    $response['success']                                            = false;
                    $response['message']                                            = "Oops! Something went wrong..";
                }
            } catch (\Exception $e) {
                $response['success']                                                = false;
                $response['message']                                                = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function getLoadDocuments($load_detail, $billing_status = null){
        // Billing status null than require relationship but not null than no require relationship
        // Required Documents
        $required_documents                                             = [];
        // Requested Documents
        $requested_documents                                            = [];
        // Accessorial Documents
        $accessorial_documents                                          = [];
        // Other Documents
        $other_documents                                                = [];

        // Required Documents
        if(isset($load_detail->shipper_files_upload) && is_serialized($load_detail->shipper_files_upload) && !empty(unserialize($load_detail->shipper_files_upload))){
            $uploads                                                    = unserialize($load_detail->shipper_files_upload);
            $uploads                                                    = array_filter($uploads, function ($var) {
                return (
                    isset($var['name_id']) && $var['name_id'] !== null && $var['is_required'] == 'on' && in_array($var['type'], [1, 2])
                );
            });
            foreach ($uploads as $key => $value) {
                // Type 1 = Copy, 2 = Original
                $array                                                  = [];
                $array['id']                                            = $load_detail->id;
                $array['source']                                        = 'load_details';
                $array['field_name']                                    = 'shipper_files_upload';

                $array['filename']                                      = $value['name'] ?? '';
                $array['filename_id']                                   = $value['name_id'] ?? '';

                $array['type']                                          = $value['type'];

                $array['is_received']                                   = isset($value['is_received']) && $value['is_received'] == 'on' ? 'on' : 'off';

                $status                                                 = 'Pending';
                if($array['type'] == 1){
                    $array['file']                                      = $value['file'] ?? null;
                    if(isset($value['file']) && !empty($value['file'])){
                        $status                                         = 'Complete';
                        // billing_status
                        if(is_null($billing_status)){
                            $array['upload_user_type']                  = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                            $array['upload_by']                         = isset($value['upload_by']) ? $value['upload_by'] : null;
                            if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                $array['uploaded_by']                   = getUserByType($value['upload_by'], $value['upload_user_type']);
                            }else{
                                $array['uploaded_by']                   = null;
                            }
                            $array['uploaded_at']                       = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null;
                        }
                    }
                }else{
                    if($array['is_received'] == 'on'){
                        $status                                         = 'Complete';
                        // billing_status
                        if(is_null($billing_status)){
                            $array['upload_user_type']                  = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                            $array['upload_by']                         = isset($value['upload_by']) ? $value['upload_by'] : null;
                            if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                $array['uploaded_by']                   = getUserByType($value['upload_by'], $value['upload_user_type']);
                            }else{
                                $array['uploaded_by']                   = null;
                            }
                            $array['uploaded_at']                       = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null;
                        }
                    }
                }

                $array['include_w_billing']                             = isset($value['include_w_billing']) && $value['include_w_billing'] == 'yes' ? 'yes' : 'no';
                $array['hide_from_driver']                              = isset($value['hide_from_driver']) && $value['hide_from_driver'] == 'yes' ? 'yes' : 'no';
                $array['history']                                       = isset($value['history']) && is_array($value['history']) && !empty($value['history']) ? $value['history'] : [];
                $array['status']                                        = $status;

                // billing_status
                if(is_null($billing_status)){
                    $addresses                                          = [];
                    if(isset($load_detail->shipper_address_one) && $load_detail->shipper_address_one!==''){
                        $addresses[]                                    = $load_detail->shipper_address_one;
                    }
                    if(isset($load_detail->shipper_address_two) && $load_detail->shipper_address_two!==''){
                        $addresses[]                                    = $load_detail->shipper_address_two;
                    }
                    if(isset($load_detail->shipper_city) && $load_detail->shipper_city!==''){
                        $addresses[]                                    = $load_detail->shipper_city;
                    }
                    if(isset($load_detail->shipper_state) && $load_detail->shipper_state!==''){
                        $addresses[]                                    = $load_detail->shipper_state;
                    }
                    if(isset($load_detail->shipper_zip) && $load_detail->shipper_zip!==''){
                        $addresses[]                                    = $load_detail->shipper_zip;
                    }
                    $addresses                                          = implode(', ', $addresses);

                    $driver                                             = null;
                    $driver_id                                          = 0;
                    $driver_full_name                                   = [];
                    if (isset($load_detail->driver)) {
                        $driver                                         = $load_detail->driver;
                        $driver_id                                      = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                         = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                         = $driver->first_name;
                        }
                    }
                    $driver_full_name                                   = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                    $stop_details                                       = [];
                    $stop_details['name']                               = 'Stop 1';
                    $stop_details['driver_id']                          = $driver_id;
                    $stop_details['driver_full_name']                   = $driver_full_name;
                    $stop_details['customer_name']                      = $load_detail->shipper->name ?? '';
                    $stop_details['addresses']                          = $addresses;
                    // Extra Info
                    $array['stop']                                      = $stop_details;
                }

                $required_documents[]                                   = $array;
            }
        }
        // Requested Documents
        if(isset($load_detail->shipper_load_document_request_items)){
            $shipper_load_document_request_items                        = $load_detail->shipper_load_document_request_items;
            foreach ($shipper_load_document_request_items as $key => $item) {
                $value                                                  = [];
                if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                    $value                                              = unserialize($item->files);
                }
                if(!empty($value)){
                    if(
                        isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1, 2])
                    ){
                        // Type 1 = Copy, 2 = Original
                        $array                                          = [];
                        $array['id']                                    = $item->id;
                        $array['source']                                = 'load_document_request_items';
                        $array['field_name']                            = 'files';

                        $array['filename']                              = $value['name'] ?? '';
                        $array['filename_id']                           = $value['name_id'] ?? '';

                        $array['type']                                  = $value['type'];

                        $array['is_received']                           = isset($value['is_received']) && $value['is_received'] == 'on' ? 'on' : 'off';

                        $status                                         = 'Pending';
                        if($array['type'] == 1){
                            $array['file']                              = $value['file'] ?? null;
                            if(isset($value['file']) && !empty($value['file'])){
                                $status                                 = 'Complete';
                                // billing_status
                                if(is_null($billing_status)){
                                    $array['upload_user_type']          = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                                    $array['upload_by']                 = isset($value['upload_by']) ? $value['upload_by'] : null;
                                    if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                        $array['uploaded_by']           = getUserByType($value['upload_by'], $value['upload_user_type']);
                                    }else{
                                        $array['uploaded_by']           = null;
                                    }
                                    $array['uploaded_at']               = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null;
                                }
                            }
                        }else{
                            if($array['is_received'] == 'on'){
                                $status                                 = 'Complete';
                                // billing_status
                                if(is_null($billing_status)){
                                    $array['upload_user_type']          = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                                    $array['upload_by']                 = isset($value['upload_by']) ? $value['upload_by'] : null;
                                    if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                        $array['uploaded_by']           = getUserByType($value['upload_by'], $value['upload_user_type']);
                                    }else{
                                        $array['uploaded_by']           = null;
                                    }
                                    $array['uploaded_at']               = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null;
                                }
                            }
                        }

                        $array['include_w_billing']                     = isset($value['include_w_billing']) && $value['include_w_billing'] == 'yes' ? 'yes' : 'no';
                        $array['hide_from_driver']                      = isset($value['hide_from_driver']) && $value['hide_from_driver'] == 'yes' ? 'yes' : 'no';
                        $array['history']                               = isset($value['history']) && is_array($value['history']) && !empty($value['history']) ? $value['history'] : [];
                        $array['status']                                = $status;

                        // billing_status
                        if(is_null($billing_status)){
                            $addresses                                  = [];
                            if(isset($load_detail->shipper_address_one) && $load_detail->shipper_address_one!==''){
                                $addresses[]                            = $load_detail->shipper_address_one;
                            }
                            if(isset($load_detail->shipper_address_two) && $load_detail->shipper_address_two!==''){
                                $addresses[]                            = $load_detail->shipper_address_two;
                            }
                            if(isset($load_detail->shipper_city) && $load_detail->shipper_city!==''){
                                $addresses[]                            = $load_detail->shipper_city;
                            }
                            if(isset($load_detail->shipper_state) && $load_detail->shipper_state!==''){
                                $addresses[]                            = $load_detail->shipper_state;
                            }
                            if(isset($load_detail->shipper_zip) && $load_detail->shipper_zip!==''){
                                $addresses[]                            = $load_detail->shipper_zip;
                            }
                            $addresses                                  = implode(', ', $addresses);

                            $driver                                     = null;
                            $driver_id                                  = 0;
                            $driver_full_name                           = [];
                            if(isset($item->driver)){
                                $driver                                 = $item->driver;
                                $driver_id                              = $driver->id;
                                if(isset($driver->last_name) && $driver->last_name != ''){
                                    $driver_full_name[]                 = $driver->last_name;
                                }
                                if(isset($driver->first_name) && $driver->first_name != ''){
                                    $driver_full_name[]                 = $driver->first_name;
                                }
                            } elseif (isset($load_detail->driver)) {
                                $driver                                 = $load_detail->driver;
                                $driver_id                              = $driver->id;
                                if(isset($driver->last_name) && $driver->last_name != ''){
                                    $driver_full_name[]                 = $driver->last_name;
                                }
                                if(isset($driver->first_name) && $driver->first_name != ''){
                                    $driver_full_name[]                 = $driver->first_name;
                                }
                            }
                            $driver_full_name                           = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                            $stop_details                               = [];
                            $stop_details['name']                       = 'Stop 1';
                            $stop_details['driver_id']                  = $driver_id;
                            $stop_details['driver_full_name']           = $driver_full_name;
                            $stop_details['customer_name']              = $load_detail->shipper->name ?? '';
                            $stop_details['addresses']                  = $addresses;
                            // Extra Info
                            $array['stop']                              = $stop_details;
                        }

                        $requested_documents[]                          = $array;
                    }
                }
            }
        }
        // Accessorial Documents
        if(isset($load_detail->shipper_accessorials)){
            $shipper_accessorials                                       = $load_detail->shipper_accessorials->where('doc_required', '!=', 0);
            foreach ($shipper_accessorials as $key => $item) {
                $gallery                                                = Gallery::where('company_id', auth()->guard('company-admin')->user()->company_id)
                    ->where('action_id', $item->id)
                    ->where('source', 'load_accessorials')
                    ->where('field_name', 'accessorial_files')
                    ->latest()->first();
                $filename                                               = $item->company_accessorial->name;
                if(!is_null($gallery)){
                    $filename                                          .= isset($gallery->file_name) ? ' - '.$gallery->file_name : '';
                }
                // Type 1 = Copy, 2 = Original
                $array                                                  = [];
                $array['id']                                            = $item->id;
                $array['source']                                        = 'load_accessorials';
                $array['field_name']                                    = 'image';

                $array['filename']                                      = $filename;
                // $array['filename_id']                                = 0;

                $array['type']                                          = 1;

                $array['is_received']                                   = 'off';

                $status                                                 = 'Pending';
                $array['file']                                          = $gallery->image ?? null;
                if(isset($gallery->image) && !empty($gallery->image)){
                    $status                                             = 'Complete';
                    $array['is_received']                               = 'on';
                    // billing_status
                    if(is_null($billing_status)){
                        $array['upload_user_type']                      = isset($gallery->created_by_type) ? $gallery->created_by_type : null;
                        $array['upload_by']                             = isset($gallery->created_by) ? $gallery->created_by : null;
                        if(isset($gallery->created_by_type) && isset($gallery->created_by)){
                            $array['uploaded_by']                       = getUserByType($gallery->created_by, $gallery->created_by_type);
                        }else{
                            $array['uploaded_by']                       = null;
                        }
                        $array['uploaded_at']                           = isset($gallery->created_at) ? Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null; // $gallery->getRawOriginal('uploaded_date')
                    }
                }

                $array['include_w_billing']                             = isset($item->include_w_billing) && $item->include_w_billing == 'yes' ? 'yes' : 'no';
                $array['hide_from_driver']                              = isset($item->hide_from_driver) && $item->hide_from_driver == 'yes' ? 'yes' : 'no';
                $array['history']                                       = isset($item->history) && is_serialized($item->history) && !empty(unserialize($item->history)) ? unserialize($item->history) : [];
                $array['status']                                        = $status;

                // billing_status
                if(is_null($billing_status)){
                    $addresses                                          = [];
                    if(isset($load_detail->shipper_address_one) && $load_detail->shipper_address_one!==''){
                        $addresses[]                                    = $load_detail->shipper_address_one;
                    }
                    if(isset($load_detail->shipper_address_two) && $load_detail->shipper_address_two!==''){
                        $addresses[]                                    = $load_detail->shipper_address_two;
                    }
                    if(isset($load_detail->shipper_city) && $load_detail->shipper_city!==''){
                        $addresses[]                                    = $load_detail->shipper_city;
                    }
                    if(isset($load_detail->shipper_state) && $load_detail->shipper_state!==''){
                        $addresses[]                                    = $load_detail->shipper_state;
                    }
                    if(isset($load_detail->shipper_zip) && $load_detail->shipper_zip!==''){
                        $addresses[]                                    = $load_detail->shipper_zip;
                    }
                    $addresses                                          = implode(', ', $addresses);

                    $driver                                             = null;
                    $driver_id                                          = 0;
                    $driver_full_name                                   = [];
                    if(isset($item->driver)){
                        $driver                                         = $item->driver;
                        $driver_id                                      = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                         = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                         = $driver->first_name;
                        }
                    } elseif (isset($load_detail->driver)) {
                        $driver                                         = $load_detail->driver;
                        $driver_id                                      = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                         = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                         = $driver->first_name;
                        }
                    }
                    $driver_full_name                                   = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                    $stop_details                                       = [];
                    $stop_details['name']                               = 'Stop 1';
                    $stop_details['driver_id']                          = $driver_id;
                    $stop_details['driver_full_name']                   = $driver_full_name;
                    $stop_details['customer_name']                      = $load_detail->shipper->name ?? '';
                    $stop_details['addresses']                          = $addresses;
                    // Extra Info
                    $array['stop']                                      = $stop_details;
                }

                $accessorial_documents[]                                = $array;
            }
        }

        $stop_no                                                        = 2;
        if(isset($load_detail->loads->load_stops) && $load_detail->loads->load_stops->isNotEmpty()){
            foreach ($load_detail->loads->load_stops as $key => $stop) {
                $customer_type                                          = $stop->customer_type;
                $field_name                                             = $customer_type=='1'?'stop_shipper_files_upload':'stop_consignee_files_upload';
                // Required Documents
                if(isset($stop->$field_name) && is_serialized($stop->$field_name) && !empty(unserialize($stop->$field_name))){
                    $uploads                                            = unserialize($stop->$field_name);
                    $uploads                                            = array_filter($uploads, function ($var) {
                        return (
                            isset($var['name_id']) && $var['name_id'] !== null && $var['is_required'] == 'on' && in_array($var['type'], [1, 2])
                        );
                    });
                    foreach ($uploads as $key => $value) {
                        // Type 1 = Copy, 2 = Original
                        $array                                          = [];
                        $array['id']                                    = $stop->id;
                        $array['source']                                = 'load_stops';
                        $array['field_name']                            = $field_name;

                        $array['filename']                              = $value['name'] ?? '';
                        $array['filename_id']                           = $value['name_id'] ?? '';

                        $array['type']                                  = $value['type'];

                        $array['is_received']                           = isset($value['is_received']) && $value['is_received'] == 'on' ? 'on' : 'off';

                        $status                                         = 'Pending';
                        if($array['type'] == 1){
                            $array['file']                              = $value['file'] ?? null;
                            if(isset($value['file']) && !empty($value['file'])){
                                $status                                 = 'Complete';
                                // billing_status
                                if(is_null($billing_status)){
                                    $array['upload_user_type']          = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                                    $array['upload_by']                 = isset($value['upload_by']) ? $value['upload_by'] : null;
                                    if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                        $array['uploaded_by']           = getUserByType($value['upload_by'], $value['upload_user_type']);
                                    }else{
                                        $array['uploaded_by']           = null;
                                    }
                                    $array['uploaded_at']               = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null;
                                }
                            }
                        }else{
                            if($array['is_received'] == 'on'){
                                $status                                 = 'Complete';
                                // billing_status
                                if(is_null($billing_status)){
                                    $array['upload_user_type']          = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                                    $array['upload_by']                 = isset($value['upload_by']) ? $value['upload_by'] : null;
                                    if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                        $array['uploaded_by']           = getUserByType($value['upload_by'], $value['upload_user_type']);
                                    }else{
                                        $array['uploaded_by']           = null;
                                    }
                                    $array['uploaded_at']               = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null;
                                }
                            }
                        }

                        $array['include_w_billing']                     = isset($value['include_w_billing']) && $value['include_w_billing'] == 'yes' ? 'yes' : 'no';
                        $array['hide_from_driver']                      = isset($value['hide_from_driver']) && $value['hide_from_driver'] == 'yes' ? 'yes' : 'no';
                        $array['history']                               = isset($value['history']) && is_array($value['history']) && !empty($value['history']) ? $value['history'] : [];
                        $array['status']                                = $status;

                        // billing_status
                        if(is_null($billing_status)){
                            $addresses                                  = [];
                            if(isset($stop->address_one) && $stop->address_one!==''){
                                $addresses[]                            = $stop->address_one;
                            }
                            if(isset($stop->address_two) && $stop->address_two!==''){
                                $addresses[]                            = $stop->address_two;
                            }
                            if(isset($stop->city) && $stop->city!==''){
                                $addresses[]                            = $stop->city;
                            }
                            if(isset($stop->state) && $stop->state!==''){
                                $addresses[]                            = $stop->state;
                            }
                            if(isset($stop->zip) && $stop->zip!==''){
                                $addresses[]                            = $stop->zip;
                            }
                            $addresses                                  = implode(', ', $addresses);

                            $driver                                     = null;
                            $driver_id                                  = 0;
                            $driver_full_name                           = [];
                            if (isset($load_detail->driver)) {
                                $driver                                 = $load_detail->driver;
                                $driver_id                              = $driver->id;
                                if(isset($driver->last_name) && $driver->last_name != ''){
                                    $driver_full_name[]                 = $driver->last_name;
                                }
                                if(isset($driver->first_name) && $driver->first_name != ''){
                                    $driver_full_name[]                 = $driver->first_name;
                                }
                            }
                            $driver_full_name                           = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                            $stop_details                               = [];
                            $stop_details['name']                       = 'Stop '.$stop_no;
                            $stop_details['driver_id']                  = $driver_id;
                            $stop_details['driver_full_name']           = $driver_full_name;
                            $stop_details['customer_name']              = $stop->customer->name ?? '';
                            $stop_details['addresses']                  = $addresses;
                            // Extra Info
                            $array['stop']                              = $stop_details;
                        }

                        $required_documents[]                           = $array;
                    }
                }
                // Requested Documents
                if(isset($stop->load_document_request_items)){
                    $stop_load_document_request_items                   = $stop->load_document_request_items;
                    foreach ($stop_load_document_request_items as $key => $item) {
                        $value                                          = [];
                        if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                            $value                                      = unserialize($item->files);
                        }
                        if(!empty($value)){
                            if(
                                isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1, 2])
                            ){
                                // Type 1 = Copy, 2 = Original
                                $array                                  = [];
                                $array['id']                            = $item->id;
                                $array['source']                        = 'load_document_request_items';
                                $array['field_name']                    = 'files';

                                $array['filename']                      = $value['name'] ?? '';
                                $array['filename_id']                   = $value['name_id'] ?? '';

                                $array['type']                          = $value['type'];

                                $array['is_received']                   = isset($value['is_received']) && $value['is_received'] == 'on' ? 'on' : 'off';

                                $status                                 = 'Pending';
                                if($array['type'] == 1){
                                    $array['file']                      = $value['file'] ?? null;
                                    if(isset($value['file']) && !empty($value['file'])){
                                        $status                         = 'Complete';
                                        // billing_status
                                        if(is_null($billing_status)){
                                            $array['upload_user_type']  = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                                            $array['upload_by']         = isset($value['upload_by']) ? $value['upload_by'] : null;
                                            if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                                $array['uploaded_by']   = getUserByType($value['upload_by'], $value['upload_user_type']);
                                            }else{
                                                $array['uploaded_by']   = null;
                                            }
                                            $array['uploaded_at']       = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null;
                                        }
                                    }
                                }else{
                                    if($array['is_received'] == 'on'){
                                        $status                         = 'Complete';
                                        // billing_status
                                        if(is_null($billing_status)){
                                            $array['upload_user_type']  = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                                            $array['upload_by']         = isset($value['upload_by']) ? $value['upload_by'] : null;
                                            if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                                $array['uploaded_by']   = getUserByType($value['upload_by'], $value['upload_user_type']);
                                            }else{
                                                $array['uploaded_by']   = null;
                                            }
                                            $array['uploaded_at']       = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null;
                                        }
                                    }
                                }

                                $array['include_w_billing']             = isset($value['include_w_billing']) && $value['include_w_billing'] == 'yes' ? 'yes' : 'no';
                                $array['hide_from_driver']              = isset($value['hide_from_driver']) && $value['hide_from_driver'] == 'yes' ? 'yes' : 'no';
                                $array['history']                       = isset($value['history']) && is_array($value['history']) && !empty($value['history']) ? $value['history'] : [];
                                $array['status']                        = $status;

                                // billing_status
                                if(is_null($billing_status)){
                                    $addresses                          = [];
                                    if(isset($stop->address_one) && $stop->address_one!==''){
                                        $addresses[]                    = $stop->address_one;
                                    }
                                    if(isset($stop->address_two) && $stop->address_two!==''){
                                        $addresses[]                    = $stop->address_two;
                                    }
                                    if(isset($stop->city) && $stop->city!==''){
                                        $addresses[]                    = $stop->city;
                                    }
                                    if(isset($stop->state) && $stop->state!==''){
                                        $addresses[]                    = $stop->state;
                                    }
                                    if(isset($stop->zip) && $stop->zip!==''){
                                        $addresses[]                    = $stop->zip;
                                    }
                                    $addresses                          = implode(', ', $addresses);

                                    $driver                             = null;
                                    $driver_id                          = 0;
                                    $driver_full_name                   = [];
                                    if(isset($item->driver)){
                                        $driver                         = $item->driver;
                                        $driver_id                      = $driver->id;
                                        if(isset($driver->last_name) && $driver->last_name != ''){
                                            $driver_full_name[]         = $driver->last_name;
                                        }
                                        if(isset($driver->first_name) && $driver->first_name != ''){
                                            $driver_full_name[]         = $driver->first_name;
                                        }
                                    } elseif (isset($load_detail->driver)) {
                                        $driver                         = $load_detail->driver;
                                        $driver_id                      = $driver->id;
                                        if(isset($driver->last_name) && $driver->last_name != ''){
                                            $driver_full_name[]         = $driver->last_name;
                                        }
                                        if(isset($driver->first_name) && $driver->first_name != ''){
                                            $driver_full_name[]         = $driver->first_name;
                                        }
                                    }
                                    $driver_full_name                   = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                                    $stop_details                       = [];
                                    $stop_details['name']               = 'Stop '.$stop_no;
                                    $stop_details['driver_id']          = $driver_id;
                                    $stop_details['driver_full_name']   = $driver_full_name;
                                    $stop_details['customer_name']      = $stop->customer->name ?? '';
                                    $stop_details['addresses']          = $addresses;
                                    // Extra Info
                                    $array['stop']                      = $stop_details;
                                }

                                $requested_documents[]                  = $array;
                            }
                        }
                    }
                }
                // Accessorial Documents
                if(isset($stop->accessorials)){
                    $stop_accessorials                                  = $stop->accessorials->where('doc_required', '!=', 0);
                    foreach ($stop_accessorials as $key => $item) {
                        $gallery                                        = Gallery::where('company_id', auth()->guard('company-admin')->user()->company_id)
                            ->where('action_id', $item->id)
                            ->where('source', 'load_accessorials')
                            ->where('field_name', 'accessorial_files')
                            ->latest()->first();
                        $filename                                       = $item->company_accessorial->name;
                        if(!is_null($gallery)){
                            $filename                                  .= isset($gallery->file_name) ? ' - '.$gallery->file_name : '';
                        }
                        // Type 1 = Copy, 2 = Original
                        $array                                          = [];
                        $array['id']                                    = $item->id;
                        $array['source']                                = 'load_accessorials';
                        $array['field_name']                            = 'image';

                        $array['filename']                              = $filename;
                        // $array['filename_id']                        = 0;

                        $array['type']                                  = 1;

                        $array['is_received']                           = 'off';

                        $status                                         = 'Pending';
                        $array['file']                                  = $gallery->image ?? null;
                        if(isset($gallery->image) && !empty($gallery->image)){
                            $status                                     = 'Complete';
                            $array['is_received']                       = 'on';
                            // billing_status
                            if(is_null($billing_status)){
                                $array['upload_user_type']              = isset($gallery->created_by_type) ? $gallery->created_by_type : null;
                                $array['upload_by']                     = isset($gallery->created_by) ? $gallery->created_by : null;
                                if(isset($gallery->created_by_type) && isset($gallery->created_by)){
                                    $array['uploaded_by']               = getUserByType($gallery->created_by, $gallery->created_by_type);
                                }else{
                                    $array['uploaded_by']               = null;
                                }
                                $array['uploaded_at']                   = isset($gallery->created_at) ? Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null; // $gallery->getRawOriginal('uploaded_date')
                            }
                        }

                        $array['include_w_billing']                     = isset($item->include_w_billing) && $item->include_w_billing == 'yes' ? 'yes' : 'no';
                        $array['hide_from_driver']                      = isset($item->hide_from_driver) && $item->hide_from_driver == 'yes' ? 'yes' : 'no';
                        $array['history']                               = isset($item->history) && is_serialized($item->history) && !empty(unserialize($item->history)) ? unserialize($item->history) : [];
                        $array['status']                                = $status;

                        // billing_status
                        if(is_null($billing_status)){
                            $addresses                                  = [];
                            if(isset($stop->address_one) && $stop->address_one!==''){
                                $addresses[]                            = $stop->address_one;
                            }
                            if(isset($stop->address_two) && $stop->address_two!==''){
                                $addresses[]                            = $stop->address_two;
                            }
                            if(isset($stop->city) && $stop->city!==''){
                                $addresses[]                            = $stop->city;
                            }
                            if(isset($stop->state) && $stop->state!==''){
                                $addresses[]                            = $stop->state;
                            }
                            if(isset($stop->zip) && $stop->zip!==''){
                                $addresses[]                            = $stop->zip;
                            }
                            $addresses                                  = implode(', ', $addresses);

                            $driver                                     = null;
                            $driver_id                                  = 0;
                            $driver_full_name                           = [];
                            if(isset($item->driver)){
                                $driver                                 = $item->driver;
                                $driver_id                              = $driver->id;
                                if(isset($driver->last_name) && $driver->last_name != ''){
                                    $driver_full_name[]                 = $driver->last_name;
                                }
                                if(isset($driver->first_name) && $driver->first_name != ''){
                                    $driver_full_name[]                 = $driver->first_name;
                                }
                            } elseif (isset($load_detail->driver)) {
                                $driver                                 = $load_detail->driver;
                                $driver_id                              = $driver->id;
                                if(isset($driver->last_name) && $driver->last_name != ''){
                                    $driver_full_name[]                 = $driver->last_name;
                                }
                                if(isset($driver->first_name) && $driver->first_name != ''){
                                    $driver_full_name[]                 = $driver->first_name;
                                }
                            }
                            $driver_full_name                           = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                            $stop_details                               = [];
                            $stop_details['name']                       = 'Stop '.$stop_no;
                            $stop_details['driver_id']                  = $driver_id;
                            $stop_details['driver_full_name']           = $driver_full_name;
                            $stop_details['customer_name']              = $stop->customer->name ?? '';
                            $stop_details['addresses']                  = $addresses;
                            // Extra Info
                            $array['stop']                              = $stop_details;
                        }

                        $accessorial_documents[]                        = $array;
                    }
                }
                //
                ++$stop_no;
            }
        }

        // Required Documents
        if(isset($load_detail->consignee_files_upload) && is_serialized($load_detail->consignee_files_upload) && !empty(unserialize($load_detail->consignee_files_upload))){
            $uploads                                                    = unserialize($load_detail->consignee_files_upload);
            $uploads                                                    = array_filter($uploads, function ($var) {
                return (
                    isset($var['name_id']) && $var['name_id'] !== null && $var['is_required'] == 'on' && in_array($var['type'], [1, 2])
                );
            });
            foreach ($uploads as $key => $value) {
                // Type 1 = Copy, 2 = Original
                $array                                                  = [];
                $array['id']                                            = $load_detail->id;
                $array['source']                                        = 'load_details';
                $array['field_name']                                    = 'consignee_files_upload';

                $array['filename']                                      = $value['name'] ?? '';
                $array['filename_id']                                   = $value['name_id'] ?? '';

                $array['type']                                          = $value['type'];

                $array['is_received']                                   = isset($value['is_received']) && $value['is_received'] == 'on' ? 'on' : 'off';

                $status                                                 = 'Pending';
                if($array['type'] == 1){
                    $array['file']                                      = $value['file'] ?? null;
                    if(isset($value['file']) && !empty($value['file'])){
                        $status                                         = 'Complete';
                        // billing_status
                        if(is_null($billing_status)){
                            $array['upload_user_type']                  = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                            $array['upload_by']                         = isset($value['upload_by']) ? $value['upload_by'] : null;
                            if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                $array['uploaded_by']                   = getUserByType($value['upload_by'], $value['upload_user_type']);
                            }else{
                                $array['uploaded_by']                   = null;
                            }
                            $array['uploaded_at']                       = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null;
                        }
                    }
                }else{
                    if($array['is_received'] == 'on'){
                        $status                                         = 'Complete';
                        // billing_status
                        if(is_null($billing_status)){
                            $array['upload_user_type']                  = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                            $array['upload_by']                         = isset($value['upload_by']) ? $value['upload_by'] : null;
                            if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                $array['uploaded_by']                   = getUserByType($value['upload_by'], $value['upload_user_type']);
                            }else{
                                $array['uploaded_by']                   = null;
                            }
                            $array['uploaded_at']                       = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null;
                        }
                    }
                }

                $array['include_w_billing']                             = isset($value['include_w_billing']) && $value['include_w_billing'] == 'yes' ? 'yes' : 'no';
                $array['hide_from_driver']                              = isset($value['hide_from_driver']) && $value['hide_from_driver'] == 'yes' ? 'yes' : 'no';
                $array['history']                                       = isset($value['history']) && is_array($value['history']) && !empty($value['history']) ? $value['history'] : [];
                $array['status']                                        = $status;

                // billing_status
                if(is_null($billing_status)){
                    $addresses                                          = [];
                    if(isset($load_detail->delivery_address_one) && $load_detail->delivery_address_one!==''){
                        $addresses[]                                    = $load_detail->delivery_address_one;
                    }
                    if(isset($load_detail->delivery_address_two) && $load_detail->delivery_address_two!==''){
                        $addresses[]                                    = $load_detail->delivery_address_two;
                    }
                    if(isset($load_detail->delivery_state) && $load_detail->delivery_state!==''){
                        $addresses[]                                    = $load_detail->delivery_state;
                    }
                    if(isset($load_detail->delivery_city) && $load_detail->delivery_city!==''){
                        $addresses[]                                    = $load_detail->delivery_city;
                    }
                    if(isset($load_detail->delivery_zip) && $load_detail->delivery_zip!==''){
                        $addresses[]                                    = $load_detail->delivery_zip;
                    }
                    $addresses                                          = implode(', ', $addresses);

                    $driver                                             = null;
                    $driver_id                                          = 0;
                    $driver_full_name                                   = [];
                    if (isset($load_detail->driver)) {
                        $driver                                         = $load_detail->driver;
                        $driver_id                                      = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                         = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                         = $driver->first_name;
                        }
                    }
                    $driver_full_name                                   = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                    $stop_details                                       = [];
                    $stop_details['name']                               = 'Stop 99';
                    $stop_details['driver_id']                          = $driver_id;
                    $stop_details['driver_full_name']                   = $driver_full_name;
                    $stop_details['customer_name']                      = $load_detail->receiver->name ?? '';
                    $stop_details['addresses']                          = $addresses;
                    // Extra Info
                    $array['stop']                                      = $stop_details;
                }

                $required_documents[]                                   = $array;
            }
        }
        // Requested Documents
        if(isset($load_detail->receiver_load_document_request_items)){
            $receiver_load_document_request_items                       = $load_detail->receiver_load_document_request_items;
            foreach ($receiver_load_document_request_items as $key => $item) {
                $value                                                  = [];
                if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                    $value                                              = unserialize($item->files);
                }
                if(!empty($value)){
                    if(
                        isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1, 2])
                    ){
                        // Type 1 = Copy, 2 = Original
                        $array                                          = [];
                        $array['id']                                    = $item->id;
                        $array['source']                                = 'load_document_request_items';
                        $array['field_name']                            = 'files';

                        $array['filename']                              = $value['name'] ?? '';
                        $array['filename_id']                           = $value['name_id'] ?? '';

                        $array['type']                                  = $value['type'];

                        $array['is_received']                           = isset($value['is_received']) && $value['is_received'] == 'on' ? 'on' : 'off';

                        $status                                         = 'Pending';
                        if($array['type'] == 1){
                            $array['file']                              = $value['file'] ?? null;
                            if(isset($value['file']) && !empty($value['file'])){
                                $status                                 = 'Complete';
                                // billing_status
                                if(is_null($billing_status)){
                                    $array['upload_user_type']          = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                                    $array['upload_by']                 = isset($value['upload_by']) ? $value['upload_by'] : null;
                                    if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                        $array['uploaded_by']           = getUserByType($value['upload_by'], $value['upload_user_type']);
                                    }else{
                                        $array['uploaded_by']           = null;
                                    }
                                    $array['uploaded_at']               = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null;
                                }
                            }
                        }else{
                            if($array['is_received'] == 'on'){
                                $status                                 = 'Complete';
                                // billing_status
                                if(is_null($billing_status)){
                                    $array['upload_user_type']          = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                                    $array['upload_by']                 = isset($value['upload_by']) ? $value['upload_by'] : null;
                                    if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                        $array['uploaded_by']           = getUserByType($value['upload_by'], $value['upload_user_type']);
                                    }else{
                                        $array['uploaded_by']           = null;
                                    }
                                    $array['uploaded_at']               = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null;
                                }
                            }
                        }

                        $array['include_w_billing']                     = isset($value['include_w_billing']) && $value['include_w_billing'] == 'yes' ? 'yes' : 'no';
                        $array['hide_from_driver']                      = isset($value['hide_from_driver']) && $value['hide_from_driver'] == 'yes' ? 'yes' : 'no';
                        $array['history']                               = isset($value['history']) && is_array($value['history']) && !empty($value['history']) ? $value['history'] : [];
                        $array['status']                                = $status;

                        // billing_status
                        if(is_null($billing_status)){
                            $addresses                                  = [];
                            if(isset($load_detail->shipper_address_one) && $load_detail->shipper_address_one!==''){
                                $addresses[]                            = $load_detail->shipper_address_one;
                            }
                            if(isset($load_detail->shipper_address_two) && $load_detail->shipper_address_two!==''){
                                $addresses[]                            = $load_detail->shipper_address_two;
                            }
                            if(isset($load_detail->shipper_city) && $load_detail->shipper_city!==''){
                                $addresses[]                            = $load_detail->shipper_city;
                            }
                            if(isset($load_detail->shipper_state) && $load_detail->shipper_state!==''){
                                $addresses[]                            = $load_detail->shipper_state;
                            }
                            if(isset($load_detail->shipper_zip) && $load_detail->shipper_zip!==''){
                                $addresses[]                            = $load_detail->shipper_zip;
                            }
                            $addresses                                  = implode(', ', $addresses);

                            $driver                                     = null;
                            $driver_id                                  = 0;
                            $driver_full_name                           = [];
                            if(isset($item->driver)){
                                $driver                                 = $item->driver;
                                $driver_id                              = $driver->id;
                                if(isset($driver->last_name) && $driver->last_name != ''){
                                    $driver_full_name[]                 = $driver->last_name;
                                }
                                if(isset($driver->first_name) && $driver->first_name != ''){
                                    $driver_full_name[]                 = $driver->first_name;
                                }
                            } elseif (isset($load_detail->driver)) {
                                $driver                                 = $load_detail->driver;
                                $driver_id                              = $driver->id;
                                if(isset($driver->last_name) && $driver->last_name != ''){
                                    $driver_full_name[]                 = $driver->last_name;
                                }
                                if(isset($driver->first_name) && $driver->first_name != ''){
                                    $driver_full_name[]                 = $driver->first_name;
                                }
                            }
                            $driver_full_name                           = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                            $stop_details                               = [];
                            $stop_details['name']                       = 'Stop 99';
                            $stop_details['driver_id']                  = $driver_id;
                            $stop_details['driver_full_name']           = $driver_full_name;
                            $stop_details['customer_name']              = $load_detail->shipper->name ?? '';
                            $stop_details['addresses']                  = $addresses;
                            // Extra Info
                            $array['stop']                              = $stop_details;
                        }

                        $requested_documents[]                          = $array;
                    }
                }
            }
        }
        // Accessorial Documents
        if(isset($load_detail->receiver_accessorials)){
            $receiver_accessorials                                      = $load_detail->receiver_accessorials->where('doc_required', '!=', 0);
            foreach ($receiver_accessorials as $key => $item) {
                $gallery                                                = Gallery::where('company_id', auth()->guard('company-admin')->user()->company_id)
                    ->where('action_id', $item->id)
                    ->where('source', 'load_accessorials')
                    ->where('field_name', 'accessorial_files')
                    ->latest()->first();
                $filename                                               = $item->company_accessorial->name;
                if(!is_null($gallery)){
                    $filename                                          .= isset($gallery->file_name) ? ' - '.$gallery->file_name : '';
                }
                // Type 1 = Copy, 2 = Original
                $array                                                  = [];
                $array['id']                                            = $item->id;
                $array['source']                                        = 'load_accessorials';
                $array['field_name']                                    = 'image';

                $array['filename']                                      = $filename;
                // $array['filename_id']                                = 0;

                $array['type']                                          = 1;

                $array['is_received']                                   = 'off';

                $status                                                 = 'Pending';
                $array['file']                                          = $gallery->image ?? null;
                if(isset($gallery->image) && !empty($gallery->image)){
                    $status                                             = 'Complete';
                    $array['is_received']                               = 'on';
                    // billing_status
                    if(is_null($billing_status)){
                        $array['upload_user_type']                      = isset($gallery->created_by_type) ? $gallery->created_by_type : null;
                        $array['upload_by']                             = isset($gallery->created_by) ? $gallery->created_by : null;
                        if(isset($gallery->created_by_type) && isset($gallery->created_by)){
                            $array['uploaded_by']                       = getUserByType($gallery->created_by, $gallery->created_by_type);
                        }else{
                            $array['uploaded_by']                       = null;
                        }
                        $array['uploaded_at']                           = isset($gallery->created_at) ? Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null; // $gallery->getRawOriginal('uploaded_date')
                    }
                }

                $array['include_w_billing']                             = isset($item->include_w_billing) && $item->include_w_billing == 'yes' ? 'yes' : 'no';
                $array['hide_from_driver']                              = isset($item->hide_from_driver) && $item->hide_from_driver == 'yes' ? 'yes' : 'no';
                $array['history']                                       = isset($item->history) && is_serialized($item->history) && !empty(unserialize($item->history)) ? unserialize($item->history) : [];
                $array['status']                                        = $status;

                // billing_status
                if(is_null($billing_status)){
                    $addresses                                          = [];
                    if(isset($load_detail->delivery_address_one) && $load_detail->delivery_address_one!==''){
                        $addresses[]                                    = $load_detail->delivery_address_one;
                    }
                    if(isset($load_detail->delivery_address_two) && $load_detail->delivery_address_two!==''){
                        $addresses[]                                    = $load_detail->delivery_address_two;
                    }
                    if(isset($load_detail->delivery_state) && $load_detail->delivery_state!==''){
                        $addresses[]                                    = $load_detail->delivery_state;
                    }
                    if(isset($load_detail->delivery_city) && $load_detail->delivery_city!==''){
                        $addresses[]                                    = $load_detail->delivery_city;
                    }
                    if(isset($load_detail->delivery_zip) && $load_detail->delivery_zip!==''){
                        $addresses[]                                    = $load_detail->delivery_zip;
                    }
                    $addresses                                          = implode(', ', $addresses);

                    $driver                                             = null;
                    $driver_id                                          = 0;
                    $driver_full_name                                   = [];
                    if(isset($item->driver)){
                        $driver                                         = $item->driver;
                        $driver_id                                      = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                         = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                         = $driver->first_name;
                        }
                    } elseif (isset($load_detail->driver)) {
                        $driver                                         = $load_detail->driver;
                        $driver_id                                      = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                         = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                         = $driver->first_name;
                        }
                    }
                    $driver_full_name                                   = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                    $stop_details                                       = [];
                    $stop_details['name']                               = 'Stop 99';
                    $stop_details['driver_id']                          = $driver_id;
                    $stop_details['driver_full_name']                   = $driver_full_name;
                    $stop_details['customer_name']                      = $load_detail->receiver->name ?? '';
                    $stop_details['addresses']                          = $addresses;
                    // Extra Info
                    $array['stop']                                      = $stop_details;
                }

                $accessorial_documents[]                                = $array;
            }
        }

        // Accessorial Documents
        if(isset($load_detail->billing_accessorials)){
            $billing_accessorials                                       = $load_detail->billing_accessorials->where('doc_required', '!=', 0);
            foreach ($billing_accessorials as $key => $item) {
                $gallery                                                = Gallery::where('company_id', auth()->guard('company-admin')->user()->company_id)
                    ->where('action_id', $item->id)
                    ->where('source', 'load_accessorials')
                    ->where('field_name', 'accessorial_files')
                    ->latest()->first();
                $filename                                               = $item->company_accessorial->name;
                if(!is_null($gallery)){
                    $filename                                          .= isset($gallery->file_name) ? ' - '.$gallery->file_name : '';
                }
                // Type 1 = Copy, 2 = Original
                $array                                                  = [];
                $array['id']                                            = $item->id;
                $array['source']                                        = 'load_accessorials';
                $array['field_name']                                    = 'image';

                $array['filename']                                      = $filename;
                // $array['filename_id']                                = 0;

                $array['type']                                          = 1;

                $array['is_received']                                   = 'off';

                $status                                                 = 'Pending';
                $array['file']                                          = $gallery->image ?? null;
                if(isset($gallery->image) && !empty($gallery->image)){
                    $status                                             = 'Complete';
                    $array['is_received']                               = 'on';
                    // billing_status
                    if(is_null($billing_status)){
                        $array['upload_user_type']                      = isset($gallery->created_by_type) ? $gallery->created_by_type : null;
                        $array['upload_by']                             = isset($gallery->created_by) ? $gallery->created_by : null;
                        if(isset($gallery->created_by_type) && isset($gallery->created_by)){
                            $array['uploaded_by']                       = getUserByType($gallery->created_by, $gallery->created_by_type);
                        }else{
                            $array['uploaded_by']                       = null;
                        }
                        $array['uploaded_at']                           = isset($gallery->created_at) ? Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null; // $gallery->getRawOriginal('uploaded_date')
                    }
                }

                $array['include_w_billing']                             = isset($item->include_w_billing) && $item->include_w_billing == 'yes' ? 'yes' : 'no';
                $array['hide_from_driver']                              = isset($item->hide_from_driver) && $item->hide_from_driver == 'yes' ? 'yes' : 'no';
                $array['history']                                       = isset($item->history) && is_serialized($item->history) && !empty(unserialize($item->history)) ? unserialize($item->history) : [];
                $array['status']                                        = $status;

                // billing_status
                if(is_null($billing_status)){
                    $driver                                             = null;
                    $driver_id                                          = 0;
                    $driver_full_name                                   = [];
                    if(isset($item->driver)){
                        $driver                                         = $item->driver;
                        $driver_id                                      = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                         = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                         = $driver->first_name;
                        }
                    } elseif (isset($load_detail->driver)) {
                        $driver                                         = $load_detail->driver;
                        $driver_id                                      = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                         = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                         = $driver->first_name;
                        }
                    }
                    $driver_full_name                                   = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                    // Extra Info
                    $array['stop']                                      = [
                        'name'                                          => 'Load',
                        'driver_id'                                     => $driver_id,
                        'driver_full_name'                              => $driver_full_name
                    ];
                }

                $accessorial_documents[]                                = $array;
            }
        }

        // Other Documents
        $get_other_documents                                            = LoadFile::where('load_id', $load_detail->load_id)
            ->orderBy('id','desc')->get();
        if($get_other_documents->isNotEmpty()){
            foreach ($get_other_documents as $key => $other_document) {
                // Type 1 = Copy, 2 = Original
                $array                                                  = [];
                $array['id']                                            = $other_document->id;
                $array['source']                                        = 'load_files';
                $array['field_name']                                    = 'document';

                $array['filename']                                      = $other_document->company_load_document->name ?? $other_document->filename;
                // $array['filename_id']                                = 0;

                $array['type']                                          = 1;
                $array['is_received']                                   = 'off';

                $status                                                 = 'Pending';
                $array['file']                                          = isset($other_document->document) && is_serialized($other_document->document) && !empty(unserialize($other_document->document)) ? unserialize($other_document->document) : null;
                if(isset($array['file']) && !empty($array['file'])){
                    $upload_user_type                                   = null;
                    if(isset($other_document->upload_user_type) && $other_document->upload_user_type == 'Company Admin'){
                        $upload_user_type                               = $other_document->company_admin->type;
                    }elseif(isset($other_document->upload_user_type) && $other_document->upload_user_type == 'Driver'){
                        $upload_user_type                               = 3;
                    }

                    $status                                             = 'Complete';
                    $array['is_received']                               = 'on';
                    // billing_status
                    if(is_null($billing_status)){
                        $array['upload_user_type']                      = $upload_user_type;
                        $array['upload_by']                             = isset($other_document->upload_by) ? $other_document->upload_by : null;
                        if(isset($upload_user_type) && isset($other_document->upload_by)){
                            $array['uploaded_by']                       = getUserByType($other_document->upload_by, $upload_user_type);
                        }else{
                            $array['uploaded_by']                       = null;
                        }
                        $array['uploaded_at']                           = isset($other_document->updated_at) ? Carbon::createFromFormat('Y-m-d H:i:s', $other_document->updated_at)->setTimezone($this->timeZone)->format(getTimeLayout())." ".$this->timeZoneId : null;
                    }
                }

                $array['include_w_billing']                             = isset($other_document->include_w_billing) && $other_document->include_w_billing == 'yes' ? 'yes' : 'no';
                $array['hide_from_driver']                              = isset($other_document->hide_from_driver) && $other_document->hide_from_driver == 'yes' ? 'yes' : 'no';
                $array['history']                                       = isset($other_document->history) && is_serialized($other_document->history) && !empty(unserialize($other_document->history)) ? unserialize($other_document->history) : [];
                $array['status']                                        = $status;

                // billing_status
                if(is_null($billing_status)){
                    $driver                                             = null;
                    $driver_id                                          = 0;
                    $driver_full_name                                   = [];
                    if (isset($load_detail->driver)) {
                        $driver                                         = $load_detail->driver;
                        $driver_id                                      = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                         = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                         = $driver->first_name;
                        }
                    }
                    $driver_full_name                                   = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                    // Extra Info
                    $array['stop']                                      = [
                        'name'                                          => 'Load',
                        'driver_id'                                     => $driver_id,
                        'driver_full_name'                              => $driver_full_name
                    ];
                }

                $other_documents[]                                      = $array;
            }
        }

        $merged_documents                                               = [];
        // Check if $required_documents is not empty before merging
        if (!empty($required_documents)) {
            $merged_documents                                           = array_merge($merged_documents, $required_documents);
        }
        // Check if $requested_documents is not empty before merging
        if (!empty($requested_documents)) {
            $merged_documents                                           = array_merge($merged_documents, $requested_documents);
        }
        // Check if $accessorial_documents is not empty before merging
        if (!empty($accessorial_documents)) {
            $merged_documents                                           = array_merge($merged_documents, $accessorial_documents);
        }
        // Check if $other_documents is not empty before merging
        if (!empty($other_documents)) {
            $merged_documents                                           = array_merge($merged_documents, $other_documents);
        }
        // If you want to reindex the merged array
        $merged_documents                                               = array_values($merged_documents);

        return [
            'merged_documents'      => $merged_documents,
            'required_documents'    => $required_documents,
            'requested_documents'   => $requested_documents,
            'accessorial_documents' => $accessorial_documents,
            'other_documents'       => $other_documents
        ];
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function availableLoadsList(Request $request){
        $getCompanyAdmin                = auth()->guard('company-admin')->user();

        // Available Loads
        $getField5                      = '';
        $isCheck5                       = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub5')->first();
        if(!empty($isCheck5->id)){
            $getField5                  = adjustTableColumnsSorting(unserialize($isCheck5->config), config('settings.table_dispatchhub5_first')); // unserialize($isCheck5->config);
        }else{
            $getField5                  = config('settings.table_dispatchhub5_first');
        }

        if ($request->ajax()) {
            $data                       = LoadDetail::select('load_details.*', 'brokers.name as brokerage_name')
                ->leftjoin('brokers','brokers.id','=','load_details.brokerage_firm');
                //->filter($request->all())
                if((isset($filters['teamArr']) && count($filters['teamArr']) > 0) || (isset($filters['dispatcherArr']) && count($filters['dispatcherArr']) > 0) || (isset($filters['equipmentArr']) && count($filters['equipmentArr']) > 0) || isset($filters['carrierArr']) && count($filters['carrierArr']) > 0){
                    $data->leftjoin("drivers","drivers.id","=","load_details.driver_id")
                        ->leftjoin('teams','teams.id','=','drivers.team_id');

                    if(isset($filters['teamArr']) && count($filters['teamArr']) > 0){
                        $data->whereIn('teams.id', $filters['teamArr']);
                    }
                    if(isset($filters['dispatcherArr']) && count($filters['dispatcherArr']) > 0){
                        $data->whereIn('drivers.manager_id', $filters['dispatcherArr']);
                    }
                    if(isset($filters['equipmentArr']) && count($filters['equipmentArr']) > 0){
                        $data->whereIn('load_details.trailer_type', $filters['equipmentArr']);
                    }
                    if(isset($filters['carrierArr']) && count($filters['carrierArr']) > 0){
                        $data->leftjoin("driver_informations", "driver_informations.driver_id", "=", "drivers.id")->whereIn('driver_informations.carrier_id', $filters['carrierArr']);
                    }
                }

                //->orderBy('load_details.id','ASC')
                $data->where(function($q) {
                    $q->whereNull('load_details.driver_id')->orWhere('load_details.driver_id', 0);
                })
                ->where('load_details.load_type', 0)
                ->where('load_details.is_on_hold', '!=', 2)
                ->whereHas('loads', function($q){
                    $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
                });

               // General filter code - start
               foreach( $getField5 as $key => $value ) {
                    $conditionReq       = 'search_string_condition_'.$key;
                    $sortReq            = 'search_string_sort_'.$key;

                    $conditionDateReq   = 'search_date_condition_'.$key;
                    $sortDateReq        = 'search_date_sort_'.$key;

                    $conditionStatusReq = 'search_status_condition_'.$key;
                    $sortStatusReq      = 'search_status_sort_'.$key;

                    // String filter
                    if( isset( request()->$conditionReq ) ) {
                        $typeReq        = 'search_string_type_'.$key;
                        $type           = request()->$typeReq;

                        if( $type == 'string' ) {
                            Common::getStringFilterResult( $data, $key );
                        }
                    }

                    // Status filter
                    if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                        Common::getStatusFilterResult( $data, $key );
                    }

                    // Date filter
                    if( isset( request()->$conditionDateReq ) ) {
                        $typeReq        = 'search_date_type_'.$key;
                        $type           = request()->$typeReq;

                        if( $type == 'date' ) {
                            Common::getDateFilterResult( $data, $key );
                        }
                    }

                    // String sort only
                    if( isset( request()->$sortReq ) ) {
                        Common::sortStringFilter( $data, $key );
                    }

                    // Date sort only
                    if( isset( request()->$sortDateReq ) ) {
                        Common::sortDateFilter( $data, $key );
                    }

                    // Status sort only
                    if( isset( request()->$sortStatusReq ) ) {
                        Common::sortStatusFilter( $data, $key );
                    }
                }
                // General filter code - end

                $data = $data->get();
                return Datatables::of($data)
                    ->addIndexColumn()
                    ->addColumn('load_status', function($row){
                        return getLoadStatus($row->load_status, $row->is_on_hold);
                    })
                    ->addColumn('load_number', function($row){
                        $html                   = '';
                        if(isset($row->load_lock_by_users) && isset($row->load_lock_by_users->company_admin->full_name) && $row->load_lock_by_users->company_admin->full_name != ''){
                            $html               .= '<i aria-hidden="true" class="fa fa-lock text-danger" title="'.$row->load_lock_by_users->company_admin->full_name.'"></i>&nbsp;';
                        }
                        if ($row->is_disable == 0) {
                            if(in_array($row->load_type, ['0', '6'])) {
                                if ($row->load_type == 0) {
                                    $html       .='<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                                } else {
                                    $html       .='<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                                }
                            } else {
                                $html           .='<a href="'.route('company.admin.loads.edit', base64Encode($row->load_id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        } else {
                            if ($row->load_type == 0) {
                                $html           .='<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.$row->load_number.'</a>';
                            } else {
                                $html           .='<a href="'.route('company.admin.loads.show', base64Encode($row->id)).'" class="mr-1 load-edit-link" target="_blank">'.getLoadTypeList()[$row->load_type].'</a>';
                            }
                        }
                        return $html;
                    })
                    ->addColumn('brokerage_firm', function($row){
                        return isset($row->brokerage_name) ? $row->brokerage_name : '';
                    })
                    ->addColumn('actual_shipper_date', function($row){
                        return !is_null($row->actual_shipper_date) ?  Carbon::parse($row->actual_shipper_date)->timezone($this->timeZone)->format(getTimeLayout()): '';
                    })
                    ->addColumn('actual_delivery_date', function($row){
                        return !is_null($row->actual_delivery_date) ? Carbon::parse($row->actual_delivery_date)->timezone($this->timeZone)->format(getTimeLayout()): '';
                    })
                    ->addColumn('origin', function($row){
                        return getAddressCityState($row->shipper_city, $row->shipper_state);
                    })
                    ->addColumn('destination', function($row){
                        return getAddressCityState($row->delivery_city, $row->delivery_state);
                    })
                    ->addColumn('miles', function($row){
                        if(!is_null($row->total_miles)){
                            return $row->total_miles;
                        }
                        return '';
                    })
                    ->addColumn('rate', function($row){
                        return "$".formatNumberWithoutZeros($row->total_bill);
                    })
                    ->addColumn('cpm', function($row){
                        $finalRate              = $row->total_bill;
                        $loadedMiles            = $row->loaded_miles;

                        if(($finalRate != '' && $finalRate > 0) && ($loadedMiles != '' && $loadedMiles> 0)) {
                            $cpm                = formatNumberWithoutZeros($finalRate/$loadedMiles);
                        }else{
                            $cpm                = 0;
                        }
                        return '$'.$cpm;
                    })
                    ->addColumn('cust_rep', function($row){
                        if(!is_null($row->rep)){
                            return $row->rep;
                        }
                        return '';
                    })
                    ->addColumn('carrier_rep', function($row){
                        return '-';
                    })
                    ->addColumn('time_left', function($row){
                        if(!is_null($row->actual_shipper_date)){
                            $startDate          = Carbon::createFromFormat('Y-m-d H:i:s', Carbon::parse($row->actual_shipper_date)->timezone(getTimeZone())->format('Y-m-d H:i:s'));
                            $endDate            = Carbon::createFromFormat('Y-m-d H:i:s', Carbon::now()->timezone(getTimeZone())->format('Y-m-d H:i:s'));

                            $days               = $startDate->diffInDays($endDate);
                            $hours              = $endDate->hour;
                            $minutes            = $endDate->minute;
                            $seconds            = $endDate->second;
                            return $days.' days '.$hours.' hr '.$minutes.' min '.$seconds.' sec';
                        }else{
                            return "";
                        }
                    })
                    ->addColumn('book', function($row){
                        return "Start";
                    })
                    ->addColumn('nearest_driver', function($row){
                        return "-";
                    })
                    ->addColumn('last_location', function($row){
                        return getAddressCityState($row->delivery_city, $row->delivery_state);
                    })
                    ->rawColumns(['load_number','time_left','action'])
                    ->make(true);
        }
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function availableDriversList(Request $request){
        $getCompanyAdmin                = auth()->guard('company-admin')->user();

        $checkAPIKey                    = IntegrationSetting::where('company_id', $getCompanyAdmin->company_id)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->whereNotNull('api_key')->first();

        $getField3                      = '';
        $getField3Hide                  = [];
        $isCheck3                       = Bookmark::where('company_id',getUserCompnayId())->where('user_id',$getCompanyAdmin->id)->where('user_type',$getCompanyAdmin->type)->where('table_id','dispatchhub3')->first();
        if(!empty($isCheck3->id)){
            $getField3                  = adjustTableColumnsSorting(unserialize($isCheck3->config), config('settings.table_dispatchhub3_first')); // unserialize($isCheck3->config);
            foreach (config('settings.table_dispatchhub3_first') as $dkey => $dispatchhub) {
                if(!array_key_exists($dkey,$getField3)){
                    $getField3[$dkey]   = 0;
                }
            }
            if(is_null($checkAPIKey)){
                $getField3['drive']                     = 0;
                $getField3['shift']                     = 0;
                $getField3['cycle']                     = 0;
                $getField3['cycle_tom']                 = 0;
                $getField3['duty_status']               = 0;
                $getField3['time_in_current_status']    = 0;
                $getField3['time_in_unit_break']        = 0;
                $getField3Hide                          = ['drive','shift','cycle','cycle_tom','duty_status','time_in_current_status','time_in_unit_break'];
            }
        }else{
            $getField3                  = config('settings.table_dispatchhub3_first');
            if(is_null($checkAPIKey)){
                $getField3['drive']                     = 0;
                $getField3['shift']                     = 0;
                $getField3['cycle']                     = 0;
                $getField3['cycle_tom']                 = 0;
                $getField3['duty_status']               = 0;
                $getField3['time_in_current_status']    = 0;
                $getField3['time_in_unit_break']        = 0;
                $getField3Hide                          = ['drive','shift','cycle','cycle_tom','duty_status','time_in_current_status','time_in_unit_break'];
            }
        }

        if ($request->ajax()) {
            $getFilters                 = $this->dispatchHubSetting;
            $filters                    = ['teamArr' => [], 'dispatcherArr' => [], 'equipmentArr' => [], 'carrierArr' => []];
            if(!is_null($getFilters)){
                $filters                = unserialize($getFilters->filters);
            }

            $data                       = Driver::select(
                    'drivers.*',

                    'teams.id as teamId',

                    //cuurent
                    'load_details.id as load_detail_id',
                    'load_details.load_id as load_id',

                    'load_details.load_number',
                    'load_details.trailer_number',
                    'load_details.trailer_type',
                    'load_details.truck_number',

                    'load_details.is_disable',
                    'load_details.load_type',
                    'load_details.load_status',

                    'load_details.booked_by',

                    'load_details.delivery_location',
                    'load_details.delivery_city',
                    'load_details.delivery_state',
                    'load_details.actual_delivery_date',

                    //previous
                    'load_details_al.id as previous_load_detail_id',
                    'load_details_al.load_id as previous_load_id',

                    'load_details_al.load_number as previous_load_number',
                    'load_details_al.trailer_number as previous_trailer_number',
                    'load_details_al.trailer_type as previous_trailer_type',
                    'load_details_al.truck_number as previous_truck_number',

                    'load_details_al.is_disable as previous_load_is_disable',
                    'load_details_al.load_type as previous_load_load_type',
                    'load_details_al.load_status as previous_load_status',

                    'load_details_al.booked_by as previous_booked_by',

                    'load_details_al.delivery_location as load_delivery_location',
                    'load_details_al.delivery_city as previous_load_delivery_city',
                    'load_details_al.delivery_state as previous_load_delivery_state',
                    'load_details_al.actual_delivery_date as load_actual_delivery_date',

                    DB::raw('TIMESTAMPDIFF(hour, load_details_al.actual_delivery_date, NOW()) as duration')
                )
                ->leftJoin('teams', function($join) {
                    $join->on('drivers.team_id', '=', 'teams.id');
                })
                ->leftJoin('driver_informations', function($join) {
                    $join->on('drivers.id', '=', 'driver_informations.driver_id');
                })
                ->leftJoin('load_details', function($join) { // Not Complete
                    $join->on('drivers.id', '=' , 'load_details.driver_id')
                        //->on('load_details.actual_delivery_date', '=', DB::raw("(SELECT max(actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id)"))
                        ->where(function($query) {
                            $query->whereNotIn('load_details.load_status', [5,6,10,8])
                                ->where(function($q) {
                                    $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                                })
                                ->where('load_details.is_disable', 0)
                                ->whereNull('load_details.deleted_at');
                                //->where('load_details.actual_delivery_date', '=', DB::raw("(select MAX(a2.actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id AND a2.load_status NOT IN(6,7,8,9) AND a2.is_disable = 0 AND a2.deleted_at IS NULL)"));
                        });
                })
                ->leftJoin('load_details as load_details_al',function ($join) { // Complete
                    $join->on('drivers.id', '=' , 'load_details_al.driver_id')
                        //->on('load_details_al.actual_delivery_date', '=', DB::raw("(SELECT max(actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id)"))
                        ->where(function($query) {
                            $query->where(function($q) {
                                    $q->whereIn('load_details_al.load_status', [5,6,10,8])->orWhereIn('load_details_al.is_on_hold', [1,2]);
                                })
                                ->where('load_details_al.is_disable', 0)
                                ->whereNull('load_details_al.deleted_at')
                                ->where('load_details_al.actual_delivery_date', '=', DB::raw("(select MAX(a2.actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id AND (a2.load_status IN(5,6,10,8) OR a2.is_on_hold IN(1,2)) AND a2.is_disable = 0 AND a2.deleted_at IS NULL)"));
                        });
                })
                ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
                ->whereNull('load_details.id');

            /*
            if ($request->has('team_id') && $request->team_id!='') {
                $data->where('teams.id', $request->team_id);
            }
            if ($request->has('dispatcher') && $request->dispatcher!='') {
                $data->where('drivers.manager_id', $request->dispatcher);
            }
            */

            if(isset($filters['teamArr']) && count($filters['teamArr']) > 0){
                $data->whereIn('teams.id', $filters['teamArr']);
            }
            if(isset($filters['dispatcherArr']) && count($filters['dispatcherArr']) > 0){
                $data->whereIn('drivers.manager_id', $filters['dispatcherArr']);
            }
            if(isset($filters['equipmentArr']) && count($filters['equipmentArr']) > 0){
                $data->whereIn('load_details.trailer_type', $filters['equipmentArr']);
            }
            if(isset($filters['carrierArr']) && count($filters['carrierArr']) > 0){
                $data->whereIn('driver_informations.carrier_id', $filters['carrierArr']);
            }
            // General filter code - start
            foreach( $getField3 as $key => $value ) {
                $conditionReq           = 'search_string_condition_'.$key;
                $sortReq                = 'search_string_sort_'.$key;

                $conditionDateReq       = 'search_date_condition_'.$key;
                $sortDateReq            = 'search_date_sort_'.$key;

                $conditionStatusReq     = 'search_status_condition_'.$key;
                $sortStatusReq          = 'search_status_sort_'.$key;

                // String filter
                if( isset( request()->$conditionReq ) ) {
                    $typeReq            = 'search_string_type_'.$key;
                    $type               = request()->$typeReq;

                    if( $type == 'string' ) {
                        Common::getStringFilterResult( $data, $key );
                    }
                }

                // Status filter
                if( isset( request()->$conditionStatusReq ) && request()->$conditionStatusReq == 1 ) {
                    Common::getStatusFilterResult( $data, $key );
                }

                // Date filter
                if( isset( request()->$conditionDateReq ) ) {
                    $typeReq            = 'search_date_type_'.$key;
                    $type               = request()->$typeReq;

                    if( $type == 'date' ) {
                        Common::getDateFilterResult( $data, $key );
                    }
                }

                // String sort only
                if( isset( request()->$sortReq ) ) {
                    Common::sortStringFilter( $data, $key );
                }

                // Date sort only
                if( isset( request()->$sortDateReq ) ) {
                    Common::sortDateFilter( $data, $key );
                }

                // Status sort only
                if( isset( request()->$sortStatusReq ) ) {
                    Common::sortStatusFilter( $data, $key );
                }
            }
            // General filter code - end
            $data                       = $data->groupBy('drivers.id');
            $data                       = $data->get();

            $i                          = 0;
            foreach ($data as $row){
                $driverSamsaraInfo                  = $this->getDriverSamsaraInfo($row);
                $data[$i]['driverSamsaraInfo']      = null;
                if(!empty($driverSamsaraInfo)){
                    $data[$i]['driverSamsaraInfo']  = $driverSamsaraInfo;
                }
                $i++;
            }

            // $data =$data->where('load_details_al.truck_number','0264')->get();

            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('reserve_status', function($row){
                    $action             = '';
                    $action            .= '<a href="'.route("company.admin.book.offer").'?driver_id='.base64Encode($row->id).'" class="btn btn-outline-info" target="_blank">Book</a>';
                    $action            .= '<a href="javascript:void(0);" class="tag-img ml-3 details-control"><div class="img-holder"><img src="'.asset('theme/images/tag.svg').'" alt="offer"><p style="'.(($row->driver_offers->count() > 99) ? 'font-size: 8px;' : '').'">'.$row->driver_offers->count().'</p></div></span>';
                    return $action;
                })
                ->addColumn('time_waiting', function($row){
                    if (!is_null($row->load_actual_delivery_date)) {
                        $startDate      = Carbon::createFromFormat('Y-m-d H:i:s', Carbon::parse($row->load_actual_delivery_date)->timezone(getTimeZone())->format('Y-m-d H:i:s'));
                        $endDate        = Carbon::createFromFormat('Y-m-d H:i:s', Carbon::now()->timezone(getTimeZone())->format('Y-m-d H:i:s'));

                        $days           = $startDate->diffInDays($endDate);
                        $hours          = $endDate->hour;
                        $minutes        = $endDate->minute;
                        $seconds        = $endDate->second;

                        $time           = $days.' days '.$hours.' hr '.$minutes.' min '.$seconds.' sec';
                        return "<span class='text-red'>".$time."</span>";
                    } else {
                        return "";
                    }
                })
                ->addColumn('full_name', function($row){
                    if(isset($row->id)){
                        return '<span class="show-available-driver" data-driver-id="'.$row->id.'" data-toggle="modal" data-target="#availableDriverModal">'.$row->full_name.'</span>';
                    }else{
                        return '';
                    }
                })
                ->addColumn('truck_number', function($row){
                    //previous_truck_number
                    return isset($row->previous_truck_number) ? $row->previous_truck_number : '';
                })
                ->addColumn('trailer_number', function($row){
                    //previous_trailer_number
                    return isset($row->previous_trailer_number) ? $row->previous_trailer_number : '';
                })
                ->addColumn('driver_manager', function($row){
                    if (!is_null($row->dispatcher)) {
                        $manager        = isset($row->dispatcher->full_name) ? $row->dispatcher->full_name : '';
                    } else {
                        $manager        =  "";
                    }
                    return $manager;
                })
                ->addColumn('last_location', function($row){
                    return getAddressCityState($row->previous_load_delivery_city, $row->previous_load_delivery_state);
                })
                ->addColumn('duty_status', function($row){
                    if(!is_null($row['driverSamsaraInfo'])){
                        if($row['driverSamsaraInfo']['duty_status'] == "sleeperBed"){
                            return '<span class="badge badge-theme">SB</span> '.$row['driverSamsaraInfo']['duty_status'];
                        }else if($row['driverSamsaraInfo']['duty_status'] == "offDuty"){
                            return '<span class="badge badge-danger">OF</span> '.$row['driverSamsaraInfo']['duty_status'];
                        }else if($row['driverSamsaraInfo']['duty_status'] == "onDuty"){
                            return '<span class="badge badge-themeOrange">on</span> '.$row['driverSamsaraInfo']['duty_status'];
                        }else if($row['driverSamsaraInfo']['duty_status'] == "driving"){
                            return '<span class="badge badge-success">D</span> '.$row['driverSamsaraInfo']['duty_status'];
                        }else{
                            return $row['driverSamsaraInfo']['duty_status'];
                        }
                    }
                    return '';
                })
                ->addColumn('drive', function($row){
                    if(!is_null($row['driverSamsaraInfo'])){
                        return $row['driverSamsaraInfo']['drive'];
                    }
                    return '';
                })
                ->addColumn('shift', function($row){
                    if(!is_null($row['driverSamsaraInfo'])){
                        return $row['driverSamsaraInfo']['shift'];
                    }
                    return '';
                })
                ->addColumn('cycle', function($row){
                    if(!is_null($row['driverSamsaraInfo'])){
                        return $row['driverSamsaraInfo']['cycle'];
                    }
                    return '';
                })
                ->addColumn('cycle_tom', function($row){
                    if(!is_null($row['driverSamsaraInfo'])){
                        return $row['driverSamsaraInfo']['cycle_tom'];
                    }
                    return '';
                })
                ->addColumn('time_until_break', function($row){
                    if(!is_null($row['driverSamsaraInfo'])){
                        return $row['driverSamsaraInfo']['time_in_unit_break'];
                    }
                    return '';
                })
                ->addColumn('updated_at', function($row){
                    $htmlCode           = '<div class="d-flex justify-content-between bd-highlight">';
                    $htmlCode          .= !is_null($row->updated_at) ? Carbon::parse($row->updated_at)->timezone($this->timeZone)->format(getTimeLayout()) : '';
                    $htmlCode          .= '<a href="javascript:void(0);" class="refreshDriverSamsaraInfo ml-3" data-driver="'.$row->id.'">Refresh</a>';
                    $htmlCode          .= '</div>';
                    return $htmlCode;
                })
                ->addColumn('previous_load', function($row){
                    if (!is_null($row->previous_load_id)) {
                        if ($row->previous_load_is_disable == 0) {
                            if(in_array($row->previous_load_load_type, ['0', '6'])) {
                                if ($row->previous_load_load_type == 0) {
                                    return '<a href="'.route('company.admin.loads.show', base64Encode($row->previous_load_detail_id)).'" class="ml-3 load-edit-link" target="_blank">'.$row->previous_load_number.'</a>';
                                } else {
                                    return '<a href="'.route('company.admin.loads.show', base64Encode($row->previous_load_detail_id)).'" class="ml-3 load-edit-link" target="_blank">'.getLoadTypeList()[$row->previous_load_load_type].'</a>';
                                }
                            } else {
                                return '<a href="'.route('company.admin.loads.edit', base64Encode($row->previous_load_id)).'" class="ml-3 load-edit-link" target="_blank">'.getLoadTypeList()[$row->previous_load_load_type].'</a>';
                            }
                        } else {
                            if ($row->previous_load_load_type == 0) {
                                return '<a href="'.route('company.admin.loads.show', base64Encode($row->previous_load_detail_id)).'" class="ml-3 load-edit-link" target="_blank">'.$row->previous_load_number.'</a>';
                            } else {
                                return '<a href="'.route('company.admin.loads.show', base64Encode($row->previous_load_detail_id)).'" class="ml-3 load-edit-link" target="_blank">'.getLoadTypeList()[$row->previous_load_load_type].'</a>';
                            }
                        }
                    }else{
                        return '';
                    }
                })
                ->addColumn('child_content', function($row){
                    $dataArr                            = [];
                    if($row->driver_offers->count()){
                        foreach ($row->driver_offers as $offerDetails){
                            $dataArr[]                  = [
                                'driver_id'             => $row->id,
                                'rate'                  => (isset($offerDetails->histories) && isset($offerDetails->histories->where('type', 2)->first()->rate)) ? $offerDetails->histories->where('type', 2)->first()->rate : null,
                                'loaded_miles'          => $offerDetails->loaded_miles,
                                'shipper_city'          => $offerDetails->shipper_city,
                                'shipper_state'         => $offerDetails->shipper_state,
                                'delivery_city'         => $offerDetails->delivery_city,
                                'delivery_state'        => $offerDetails->delivery_state,
                                'id'                    => $offerDetails->id,
                                'score_ratting'         => $offerDetails->score_ratting,
                                'brokerage_mc'          => $offerDetails->brokerage_mc,
                                'broker_name'           => (isset($offerDetails->broker)) ? $offerDetails->broker->name : null,
                                'point_of_contact'      => $offerDetails->point_of_contact,
                                'phone'                 => $offerDetails->phone,
                                'shipper_date_time'     => $offerDetails->shipper_date_time,
                                'delivery_date_time'    => $offerDetails->delivery_date_time,
                                'stops_count'           => (isset($offerDetails->stops)) ? $offerDetails->stops->count() : 0,
                                'empty_miles'           => $offerDetails->empty_miles,
                                'total_miles'           => $offerDetails->total_miles,
                                'rate_offered'          => $offerDetails->rate_offered,
                                'trailer_type'          => (isset($offerDetails->mile_trailer_type)) ? $offerDetails->mile_trailer_type->name : null,
                            ];
                        }
                    }
                    return json_encode($dataArr); //$this->availableDriversChildHtml($row);
                })
                ->rawColumns(['reserve_status','time_waiting','full_name','previous_load','updated_at','child_content','duty_status','driverSamsaraInfo'])
                ->make(true);
        }
    }

    public function availableDriversChildHtml(Request $request){
        $offers                             = json_decode($request->rowData);
        $view                               = view('companyAdmin.dispatchHub.ajaxview.availableDriversChild', compact('offers'))->render();
        return response()->json(['success' => true, 'childHtml' => $view]);
    }

    public function availableDriverView(Request $request){
        $driver                                 = Driver::find($request->driver_id);

        $load_detail                            = LoadDetail::whereNotNull('trailer_number')->where('driver_id',$driver->id)->latest()->first();
        if(isset($load_detail->trailer_number) && $load_detail->trailer_number!=''){
            $trailer_number                     = $load_detail->trailer_number ?? null;
        }else{
            $trailer_number                     = $driver->driver_information->trailer_number ?? null;
        }
        $trailer_type                           = null;
        if(!is_null($trailer_number)){
            $trailer                            = Trailer::where('trailer_id', $trailer_number)->first();
            if(isset($trailer->trailer_type->name)){
                $trailer_type                   = $trailer->trailer_type->name;
            }
        }
        $samsara_info                           = $this->getDriverSamsaraInfo($driver);
        $progress                               = [];

        $drive_percentage                       = 0;
        $shift_percentage                       = 0;
        $cycle_percentage                       = 0;
        $unit_break_percentage                  = 0;
        if(!empty($samsara_info)){
            $drive                              = isset($samsara_info['drive'])?explode(':', $samsara_info['drive']):explode(':', '00:00');
            $drive_hours                        = '00';
            $drive_minutes                      = '00';
            if(!empty($drive)){
                $drive_hours                    = $drive[0];
                $drive_minutes                  = $drive[1];
            }
            $drive_seconds                      = ((int)$drive_hours * 3600) + ((int)$drive_minutes * 60);
            $drive_max_seconds                  = (11 * 3600);
            if($drive_seconds >= $drive_max_seconds){
                $drive_percentage               = 100;
            }else{
                $drive_percentage               = intval(($drive_seconds/$drive_max_seconds)*100);
            }

            $shift                              = isset($samsara_info['shift'])?explode(':', $samsara_info['shift']):explode(':', '00:00');
            $shift_hours                        = '00';
            $shift_minutes                      = '00';
            if(!empty($shift)){
                $shift_hours                    = $shift[0];
                $shift_minutes                  = $shift[1];
            }
            $shift_seconds                      = ((int)$shift_hours * 3600) + ((int)$shift_minutes * 60);
            $shift_max_seconds                  = (14 * 3600);
            if($shift_seconds >= $shift_max_seconds){
                $shift_percentage               = 100;
            }else{
                $shift_percentage               = intval(($shift_seconds/$shift_max_seconds)*100);
            }

            $cycle                              = isset($samsara_info['cycle'])?explode(':', $samsara_info['cycle']):explode(':', '00:00');
            $cycle_hours                        = '00';
            $cycle_minutes                      = '00';
            if(!empty($cycle)){
                $cycle_hours                    = $cycle[0];
                $cycle_minutes                  = $cycle[1];
            }
            $cycle_seconds                      = ((int)$cycle_hours * 3600) + ((int)$cycle_minutes * 60);
            $cycle_max_seconds                  = (70 * 3600);
            if($cycle_seconds >= $cycle_max_seconds){
                $cycle_percentage               = 100;
            }else{
                $cycle_percentage               = intval(($cycle_seconds/$cycle_max_seconds)*100);
            }

            $unit_break                         = isset($samsara_info['time_in_unit_break'])?explode(':', $samsara_info['time_in_unit_break']):explode(':', '00:00');
            $unit_break_hours                   = '00';
            $unit_break_minutes                 = '00';
            if(!empty($unit_break)){
                $unit_break_hours               = $unit_break[0];
                $unit_break_minutes             = $unit_break[1];
            }
            $unit_break_seconds                 = ((int)$unit_break_hours * 3600) + ((int)$unit_break_minutes * 60);
            $unit_break_max_seconds             = (8 * 3600);
            if($unit_break_seconds >= $unit_break_max_seconds){
                $unit_break_percentage          = 100;
            }else{
                $unit_break_percentage          = intval(($unit_break_seconds/$unit_break_max_seconds)*100);
            }
        }
        $progress['drive_percentage']           = $drive_percentage;
        $progress['shift_percentage']           = $shift_percentage;
        $progress['cycle_percentage']           = $cycle_percentage;
        $progress['unit_break_percentage']      = $unit_break_percentage;
        $view                                   = view('companyAdmin.dispatchHub.view.availableDriver', compact('driver','trailer_number','trailer_type','samsara_info', 'progress'))->render();
        return response()->json(['view' => $view]);
    }

    public function refreshDriverSamsaraInfo(Request $request){
        $response['success']                    = false;
        $response['message']                    = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $driver                         = Driver::where('id', $request->id)->first();
                if(!is_null($driver)){
                    $driver->updated_at         = Carbon::now()->timezone($this->timeZone)->format('Y-m-d H:i:s');
                    $driver->save();

                    $data                       = $this->getDriverSamsaraInfo($driver);
                    $data['updated_at']         = Carbon::now()->timezone($this->timeZone)->format(getTimeLayout());

                    $response['data']           = $data;
                    $response['success']        = true;
                    $response['message']        = "Driver samsara info retrieved successfully.";
                }
            }catch (\Exception $e) {
                $response['success']            = false;
                $response['message']            = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function driverInfo(Request $request){
        $response['success']                    = false;
        $response['message']                    = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $driver                         = Driver::where('id', $request->id)->first();
                if(!is_null($driver)){
                    $response['data']           = $this->getDriverInfo($driver);
                    $response['success']        = true;
                    $response['message']        = "Driver info retrieved successfully.";
                }
            }catch (\Exception $e) {
                $response['success']            = false;
                $response['message']            = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function getDriverInfo($driver){
        $data                                   = [];

        $data['company']                        = $driver->company->name ?? null;
        $data['truck_number']                   = $driver->driver_information->truck_number ?? null;

        $load_detail                            = LoadDetail::whereNotNull('trailer_number')->where('driver_id',$driver->id)->latest()->first();
        if(isset($load_detail->trailer_number) && $load_detail->trailer_number!=''){
            $data['trailer_number']             = $load_detail->trailer_number;
        }else{
            $data['trailer_number']             = $driver->driver_information->trailer_number;
        }

        $truck                                  = $driver->driver_information->truck ?? null;
        if(!is_null($truck)){
            $data['truck_width']                = $truck->width;
            $data['truck_height']               = $truck->height;
            $data['truck_length']               = $truck->length;
        }else{
            $data['truck_width']                = null;
            $data['truck_height']               = null;
            $data['truck_length']               = null;
        }

        $carrier_option                         = '<option value="" selected="selected">Select Carrier</option>';
        $carrier_id                             = $driver->driver_information->carrier_id ?? null;
        if(!is_null($carrier_id)){
            $carrier                            = Carrier::find($carrier_id);
            if(!is_null($carrier)){
                $carrier_option                 = '<option value="'.$carrier->id.'" selected="selected">'.$carrier->name.'</option>';
            }
        }
        $data['carrier_option']                 = $carrier_option;

        $termianl_option                        = '<option value="" selected="selected">Select Terminal</option>';
        $terminal_id                            = $driver->driver_information->terminal_id ?? null;
        if(!is_null($terminal_id)){
            $terminal                           = Terminal::find($terminal_id);
            if(!is_null($terminal)){
                $termianl_option                = '<option value="'.$terminal->id.'" selected="selected">'.$terminal->name.'</option>';
            }
        }
        $data['termianl_option']                = $termianl_option;

        $data['driver_manager']                 = $driver->dispatcher->full_name ?? null;
        return $data;
    }

    public function driverSamsaraInfo(Request $request){
        $response['success']                    = false;
        $response['message']                    = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $driver                         = Driver::where('id', $request->id)->first();
                if(!is_null($driver)){
                    $response['data']           = $this->getDriverSamsaraInfo($driver);
                    $response['success']        = true;
                    $response['message']        = "Driver samsara info retrieved successfully.";
                }
            }catch (\Exception $e) {
                $response['success']            = false;
                $response['message']            = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public function getDriverSamsaraInfo($driver){
        $data                                           = [];
        $samsara_id                                     = (isset($driver->samsara_id) && $driver->samsara_id!='')?$driver->samsara_id:'';//4644765
        if($samsara_id!=''){
            $carrierId = (isset($driver->driver_information->carrier_id)) ? $driver->driver_information->carrier_id : null;
            $getAPIKey                                  = IntegrationSetting::where('company_id', $driver->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->first();
            if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                $client                                 = new \GuzzleHttp\Client(['verify' => false]);
                $api_url                                = 'https://api.samsara.com/fleet/hos/clocks?driverIds='.$samsara_id;
                $api_response                           = $client->request('GET', $api_url, [
                    'headers'                           => [
                        'Accept'                        => 'application/json',
                        'Authorization'                 => 'Bearer '.$getAPIKey->api_key,
                    ],
                ]);

                $api_data                               = json_decode($api_response->getBody());
                if(!empty($api_data)){
                    $appData                            = $api_data->data[0];
                    if(!empty($appData)){
                        $data['drive']                  = formatSeconds($appData->clocks->drive->driveRemainingDurationMs);
                        $data['shift']                  = formatSeconds($appData->clocks->shift->shiftRemainingDurationMs);
                        $data['cycle']                  = formatSeconds($appData->clocks->cycle->cycleRemainingDurationMs);
                        $data['cycle_tom']              = formatSeconds($appData->clocks->cycle->cycleTomorrowDurationMs);
                        $data['duty_status']            = $appData->currentDutyStatus->hosStatusType;
                        $data['time_in_current_status'] = (!empty($appData->currentDutyStatus->hosStatusType)) ? $appData->currentDutyStatus->hosStatusType : 'Disconnected';
                        $data['time_in_unit_break']     = formatSeconds($appData->clocks->break->timeUntilBreakDurationMs);
                    }
                }
            }
        }
        return $data;
    }

    public function driverLastLocation(Request $request){
        $response['success']                    = false;
        $response['message']                    = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $driver                         = Driver::where('id', $request->id)->first();
                if(!is_null($driver)){
                    $response['data']           = $this->getDriverLastLocation($driver);
                    $response['success']        = true;
                    $response['message']        = "Driver last location retrieved successfully.";
                }
            }catch (\Exception $e) {
                $response['success']            = false;
                $response['message']            = $e->getMessage();
            }
        }
        return response()->json($response);
    }

    public static function getDriverLastLocation($driver){
        $last_location                          = [];
        $truck                                  = $driver->driver_information->truck ?? null;
        if(!is_null($truck)){
            $vehicleIds                         = $truck->samsara_id ?? null;
            if(!is_null($vehicleIds)){
                $carrierId = (isset($driver->driver_information->carrier_id)) ? $driver->driver_information->carrier_id : null;
                $getAPIKey                      = IntegrationSetting::where('company_id', $driver->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_vehicles', '1')->first();
                if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                    try {
                        $url                    = 'https://api.samsara.com/fleet/vehicles/locations?vehicleIds='.$vehicleIds;
                        $client                 = new Client();
                        $api_response           = $client->request('GET', $url, [
                            'headers'           => [
                                'Accept'        => 'application/json',
                                'Authorization' => 'Bearer '.$getAPIKey->api_key,
                            ],
                        ]);
                        // Check if response status code is OK (200)
                        if ($api_response->getStatusCode() == 200) {
                            $body               = json_decode($api_response->getBody());
                            // Do something with the response body
                            if(isset($body)){
                                $data           = isset($body->data[0])?$body->data[0]:null;
                                if(isset($data)){
                                    if(isset($data->location)){
                                        try {
                                            $location                       = $data->location;

                                            $trimble_api_key                = getTrimbleApiKey();
                                            if(!is_null($trimble_api_key)){
                                                $array                      = [
                                                    'Coords'                => $location->longitude.','.$location->latitude,
                                                    'matchNamedRoadsOnly'   => 'true',
                                                    'maxCleanupMiles'       => '20',
                                                    'region'                => 'NA',
                                                    'dataset'               => 'Current',
                                                    'vehicle'               => 'Truck',
                                                    'timestamp'             => Carbon::now(),
                                                    'includeTrimblePlaceIds'=> 'true'
                                                ];

                                                $param                      = array();
                                                foreach($array as $k => $v){
                                                    $param[]                = $k . '=' . urlencode($v);
                                                }
                                                $param                      = implode('&', $param);

                                                $trimble_url                = 'https://pcmiler.alk.com/apis/rest/v1.0/Service.svc/locations/reverse?'.$param;
                                                $trimble_response           = $client->request('GET', $trimble_url, [
                                                    'headers'               => [
                                                        'Accept'            => 'application/json',
                                                        'Content-Type'      => 'application/json',
                                                        'Authorization'     => $trimble_api_key
                                                    ],
                                                ]);

                                                // Check if response status code is OK (200)
                                                if ($trimble_response->getStatusCode() == 200) {
                                                    $trimble_body           = json_decode($trimble_response->getBody());
                                                    if(isset($trimble_body)){
                                                        $last_location      = [
                                                            'StreetAddress' => $trimble_body->Address->StreetAddress,
                                                            'City'          => $trimble_body->Address->City,
                                                            'State'         => $trimble_body->Address->State,
                                                            'Zip'           => $trimble_body->Address->Zip,
                                                            'TimeZone'      => $trimble_body->TimeZone,
                                                            'Lat'           => $trimble_body->Coords->Lat,
                                                            'Lon'           => $trimble_body->Coords->Lon,
                                                            'time'          => Carbon::parse($location->time)->format('Y-m-d H:i:s'),
                                                        ];
                                                    }
                                                }
                                            }
                                        } catch (ClientException $e) {
                                            // Handle client-side errors (e.g. 4xx status codes)
                                            $trimble_response               = $e->getResponse();
                                            if ($trimble_response) {
                                                $statusCode                 = $trimble_response->getStatusCode();
                                                $body                       = $trimble_response->getBody()->getContents();
                                                Log::error($body);
                                            } else {
                                                Log::error($e->getMessage());
                                            }
                                        } catch (ServerException $e) {
                                            // Handle server-side errors (e.g. 5xx status codes)
                                            $trimble_response               = $e->getResponse();
                                            if ($response) {
                                                $statusCode                 = $trimble_response->getStatusCode();
                                                $body                       = $trimble_response->getBody()->getContents();
                                                Log::error($body);
                                            } else {
                                                Log::error($e->getMessage());
                                            }
                                        } catch (\Exception $e) {
                                            // Handle other exceptions
                                            Log::error($e->getMessage());
                                        }
                                    }
                                }
                            }
                        }
                    } catch (ClientException $e) {
                        // Handle server-side errors (e.g. 5xx status codes)
                        $api_response           = $e->getResponse();
                        if ($api_response) {
                            $statusCode         = $api_response->getStatusCode();
                            $body               = $api_response->getBody()->getContents();
                            Log::error($body);
                        } else {
                            Log::error($e->getMessage());
                        }
                    } catch (ServerException $e) {
                        // Handle server-side errors (e.g. 5xx status codes)
                        $api_response           = $e->getResponse();
                        if ($api_response) {
                            $statusCode         = $api_response->getStatusCode();
                            $body               = $api_response->getBody()->getContents();
                            Log::error($body);
                        } else {
                            Log::error($e->getMessage());
                        }
                    } catch (\Exception $e) {
                        // Handle other exceptions
                        Log::error($e->getMessage());
                    }
                }
            }
        }
        if(empty($last_location)){
            $load_details                       = Driver::select(
                    'drivers.id',
                    'drivers.samsara_id',
                    'drivers.first_name',
                    'drivers.last_name',
                    'drivers.middle_name',

                    //cuurent
                    'load_details.id as load_detail_id',
                    'load_details.load_id as load_id',

                    'load_details.load_number',

                    'load_details.booked_by',
                    'load_details.truck_number',
                    'load_details.trailer_number',
                    'load_details.trailer_type',

                    'load_details.load_type',
                    'load_details.load_status',
                    'load_details.is_on_hold',
                    'load_details.is_disable',

                    'load_details.delivery_address_one',
                    'load_details.delivery_address_two',
                    'load_details.delivery_city',
                    'load_details.delivery_state',
                    'load_details.delivery_zip',

                    'load_details.delivery_latitude',
                    'load_details.delivery_longitude',
                    'load_details.delivery_timezone',
                    'load_details.delivery_timezone_offset',
                    'load_details.delivery_timezone_abbr',
                    'load_details.delivery_address_verified',

                    'load_details.actual_delivery_date',

                    //previous
                    'load_details_al.id as previous_load_detail_id',
                    'load_details_al.load_id as previous_load_id',

                    'load_details_al.load_number as previous_load_number',

                    'load_details_al.booked_by as previous_booked_by',
                    'load_details_al.truck_number as previous_truck_number',
                    'load_details_al.trailer_number as previous_trailer_number',
                    'load_details_al.trailer_type as previous_trailer_type',

                    'load_details_al.load_type as previous_load_load_type',
                    'load_details_al.load_status as previous_load_status',
                    'load_details_al.is_on_hold as previous_is_on_hold',
                    'load_details_al.is_disable as previous_load_is_disable',

                    'load_details_al.delivery_address_one as previous_load_delivery_address_one',
                    'load_details_al.delivery_address_two as previous_load_delivery_address_two',
                    'load_details_al.delivery_city as previous_load_delivery_city',
                    'load_details_al.delivery_state as previous_load_delivery_state',
                    'load_details_al.delivery_zip as previous_load_delivery_zip',

                    'load_details_al.delivery_latitude as previous_load_delivery_latitude',
                    'load_details_al.delivery_longitude as previous_load_delivery_longitude',
                    'load_details_al.delivery_timezone as previous_load_delivery_timezone',
                    'load_details_al.delivery_timezone_offset as previous_load_delivery_timezone_offset',
                    'load_details_al.delivery_timezone_abbr as previous_load_delivery_timezone_abbr',
                    'load_details_al.delivery_address_verified as previous_load_delivery_address_verified',

                    'load_details_al.actual_delivery_date as previous_load_actual_delivery_date'
                )
                ->leftJoin('load_details', function($join) { // Not Complete
                    $join->on('drivers.id', '=' , 'load_details.driver_id')
                        ->where(function($query) {
                            $query->whereNotIn('load_details.load_status', [5,6,10,8])
                                ->where(function($q) {
                                    $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                                })
                                ->where('load_details.is_disable', 0)
                                ->whereNull('load_details.deleted_at');
                        });
                })
                ->leftJoin('load_details as load_details_al',function ($join) { // Complete
                    $join->on('drivers.id', '=' , 'load_details_al.driver_id')
                        ->where(function($query) {
                            $query->where(function($q) {
                                    $q->whereIn('load_details_al.load_status', [5,6,10,8])->orWhereIn('load_details_al.is_on_hold', [1,2]);
                                })
                                ->where('load_details_al.is_disable', 0)
                                ->whereNull('load_details_al.deleted_at')
                                ->where('load_details_al.actual_delivery_date', '=', DB::raw("(select MAX(a2.actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id AND (a2.load_status IN(5,6,10,8) OR a2.is_on_hold IN(1,2)) AND a2.is_disable = 0 AND a2.deleted_at IS NULL)"));
                        });
                })
                ->whereNull('load_details.id')
                ->whereNotNull('load_details_al.id')
                ->where('drivers.id', $driver->id)
                ->groupBy('drivers.id')
                ->first();
            if(!is_null($load_details)){
                $last_location                  = [
                    'StreetAddress'             => $load_details->previous_load_delivery_address_one,
                    'City'                      => $load_details->previous_load_delivery_city,
                    'State'                     => $load_details->previous_load_delivery_state,
                    'Zip'                       => $load_details->previous_load_delivery_zip,
                    'TimeZone'                  => $load_details->previous_load_delivery_timezone_abbr,
                    'Lat'                       => $load_details->previous_load_delivery_latitude,
                    'Lon'                       => $load_details->previous_load_delivery_longitude,
                    'time'                      => $load_details->previous_load_actual_delivery_date,
                ];
            }else{
                if(!is_null($truck)){
                    $last_location              = [
                        'StreetAddress'         => $truck->last_location_address,
                        'City'                  => $truck->last_location_city,
                        'State'                 => $truck->last_location_state,
                        'Zip'                   => $truck->last_location_zip,
                        'TimeZone'              => $truck->last_location_timezone_abbr,
                        'Lat'                   => $truck->last_location_latitude,
                        'Lon'                   => $truck->last_location_longitude,
                        'time'                  => null,
                    ];
                }
            }
        }
        return $last_location;
    }

    public function bookOffer(Request $request, $id=null, $useAsTemplate = null){
        if(!is_null($id)){
            if(!is_null($useAsTemplate) && $useAsTemplate != '1'){
                $useAsTemplate                          = null;
            }

            $offer                                      = Offer::findOrFail(base64Decode($id));

            $rep_contact_html                           = '';
            if(isset($offer->broker)){
                $broker                                 = $offer->broker;
                $getBrokerContact                       = BrokerContact::where('source', 'broker')->where('action_id', $broker->id)->get();
                if(!empty($broker->dispatch_contact_name)){
                    $rep_contact_html                   .= '<option value="'.$broker->dispatch_contact_name.'" data-phone="'.$broker->dispatch_contact_phone.'" data-ext="'.$broker->dispatch_contact_contact_ext.'" data-email="'.$broker->dispatch_contact_email.'">'.$broker->dispatch_contact_name.'</option>';
                }
                foreach($getBrokerContact as $cdata){
                    $rep_contact_html                   .= '<option value="'.$cdata->name.'" data-phone="'.$cdata->phone_number.'" data-ext="'.$cdata->ext.'" data-email="'.$cdata->email.'">'.$cdata->name.'</option>';
                }
            }
            return view('companyAdmin.dispatchHub.offer.create', compact('offer','rep_contact_html', 'useAsTemplate'));
        }else{
            $driver                                     = null;
            if($request->has('driver_id') && $request->driver_id!=''){
                $driver_id                              = base64Decode($request->driver_id);
                $driver                                 = Driver::where('id', $driver_id)->first();
            }
            return view('companyAdmin.dispatchHub.offer.create', compact('driver'));
        }
    }

    public function bookOfferStore(Request $request){
        $data['success']                                                = false;
        $data['message']                                                = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $time_format                                            = 'H:i';
                if(getTimeFormat()==12){
                    $time_format                                        = 'h:i A';
                }

                if($request->has('offer_id') && $request->offer_id !='' && is_null($request->use_as_template)){
                    $offer                                              = Offer::where('id', $request->offer_id)->first();
                    if($offer){
                        $offer->driver_id                               = $request->driver_id;

                        $offer->m_trailer_type                          = $request->m_trailer_type;
                        $offer->truck_number                            = $request->truck_number;
                        $offer->trailer_number                          = $request->trailer_number;
                        $offer->trailer_type                            = $request->trailer_type;
                        $offer->carrier_id                              = $request->carrier_id;

                        $offer->broker_id                               = $request->broker_id;
                        $offer->brokerage_mc                            = $request->broker_mc;
                        $offer->terms                                   = $request->broker_terms;
                        $offer->point_of_contact                        = $request->rep;
                        $offer->phone                                   = $request->broker_phone;
                        $offer->ext                                     = $request->broker_ext;
                        $offer->email                                   = $request->broker_email;

                        $offer->score_ratting                           = $request->score_ratting;
                        $offer->score_credit                            = $request->score_credit;
                        $offer->score_updated_at                        = $request->score_updated_at;

                        $offer->current_state                           = $request->current_state;
                        $offer->current_city                            = $request->current_city;
                        $offer->current_zip                             = $request->current_zip;

                        $current_address_verified                       = $request->current_address_verified ?? 0;
                        if($current_address_verified==0){
                            $offer->current_latitude                    = null;
                            $offer->current_longitude                   = null;
                            $offer->current_timezone                    = null;
                            $offer->current_timezone_offset             = null;
                            $offer->current_timezone_abbr               = null;
                        }else{
                            $current_timezone                           = null;
                            $current_timezone_offset                    = null;
                            $current_timezone_abbr                      = $request->current_timezone_abbr ?? null;
                            if(!is_null($current_timezone_abbr)){
                                $current_timezone                       = timezone_name_from_abbr($current_timezone_abbr);
                                $current_timezone_offset                = Carbon::createFromTimestamp(time(), $current_timezone)->getOffsetString();
                            }

                            $offer->current_latitude                    = $request->current_latitude;
                            $offer->current_longitude                   = $request->current_longitude;
                            $offer->current_timezone                    = $current_timezone;
                            $offer->current_timezone_offset             = $current_timezone_offset;
                            $offer->current_timezone_abbr               = $current_timezone_abbr;
                        }
                        $offer->current_address_verified                = $current_address_verified;

                        $offer->shipper_date_time                       = !is_null($request->shipper_date_time) ? Carbon::createFromFormat(getTimeLayout(), $request->shipper_date_time)->format('Y-m-d H:i:s') : null;
                        $offer->shipper_date_time_end                   = !is_null($request->shipper_date_time_end) ? Carbon::createFromFormat(getTimeLayout(), $request->shipper_date_time_end)->format('Y-m-d H:i:s') : null;
                        $offer->shipper_date                            = ($request->pickup_date)?date('Y-m-d', strtotime($request->pickup_date)):null;
                        $offer->shipper_open_time                       = !is_null($request->pickupo_time) ? Carbon::createFromFormat($time_format, $request->pickupo_time)->format('H:i') : null;
                        $offer->shipper_close_time                      = !is_null($request->pickupc_time) ? Carbon::createFromFormat($time_format, $request->pickupc_time)->format('H:i') : null;
                        $offer->shipper_state                           = $request->shipper_state;
                        $offer->shipper_city                            = $request->shipper_city;
                        $offer->shipper_zip                             = $request->shipper_zip;

                        $shipper_address_verified                       = $request->shipper_address_verified ?? 0;
                        if($shipper_address_verified==0){
                            $offer->shipper_latitude                    = null;
                            $offer->shipper_longitude                   = null;
                            $offer->shipper_timezone                    = null;
                            $offer->shipper_timezone_offset             = null;
                            $offer->shipper_timezone_abbr               = null;
                        }else{
                            $shipper_timezone                           = null;
                            $shipper_timezone_offset                    = null;
                            $shipper_timezone_abbr                      = $request->shipper_timezone_abbr ?? null;
                            if(!is_null($shipper_timezone_abbr)){
                                $shipper_timezone                       = timezone_name_from_abbr($shipper_timezone_abbr);
                                $shipper_timezone_offset                = Carbon::createFromTimestamp(time(), $shipper_timezone)->getOffsetString();
                            }

                            $offer->shipper_latitude                    = $request->shipper_latitude;
                            $offer->shipper_longitude                   = $request->shipper_longitude;
                            $offer->shipper_timezone                    = $shipper_timezone;
                            $offer->shipper_timezone_offset             = $shipper_timezone_offset;
                            $offer->shipper_timezone_abbr               = $shipper_timezone_abbr;
                        }
                        $offer->shipper_address_verified                = $shipper_address_verified;

                        $offer->delivery_date_time                      = !is_null($request->delivery_date_time) ? Carbon::createFromFormat(getTimeLayout(), $request->delivery_date_time)->format('Y-m-d H:i:s') : null;
                        $offer->delivery_date_time_end                  = !is_null($request->delivery_date_time_end) ? Carbon::createFromFormat(getTimeLayout(), $request->delivery_date_time_end)->format('Y-m-d H:i:s') : null;
                        $offer->delivery_date                           = ($request->delivery_date)?date('Y-m-d', strtotime($request->delivery_date)):null;
                        $offer->delivery_open_time                      = !is_null($request->deliveryo_time) ? Carbon::createFromFormat($time_format, $request->deliveryo_time)->format('H:i') : null;
                        $offer->delivery_close_time                     = !is_null($request->deliveryc_time) ? Carbon::createFromFormat($time_format, $request->deliveryc_time)->format('H:i') : null;
                        $offer->delivery_state                          = $request->delivery_state;
                        $offer->delivery_city                           = $request->delivery_city;
                        $offer->delivery_zip                            = $request->delivery_zip;

                        $delivery_address_verified                      = $request->delivery_address_verified ?? 0;
                        if($delivery_address_verified==0){
                            $offer->delivery_latitude                   = null;
                            $offer->delivery_longitude                  = null;
                            $offer->delivery_timezone                   = null;
                            $offer->delivery_timezone_offset            = null;
                            $offer->delivery_timezone_abbr              = null;
                        }else{
                            $delivery_timezone                          = null;
                            $delivery_timezone_offset                   = null;
                            $delivery_timezone_abbr                     = $request->delivery_timezone_abbr ?? null;
                            if(!is_null($delivery_timezone_abbr)){
                                $delivery_timezone                      = timezone_name_from_abbr($delivery_timezone_abbr);
                                $delivery_timezone_offset               = Carbon::createFromTimestamp(time(), $delivery_timezone)->getOffsetString();
                            }

                            $offer->delivery_latitude                   = $request->delivery_latitude;
                            $offer->delivery_longitude                  = $request->delivery_longitude;
                            $offer->delivery_timezone                   = $delivery_timezone;
                            $offer->delivery_timezone_offset            = $delivery_timezone_offset;
                            $offer->delivery_timezone_abbr              = $delivery_timezone_abbr;
                        }
                        $offer->delivery_address_verified               = $delivery_address_verified;

                        $offer->temp                                    = $request->temp;
                        $offer->weight                                  = $request->weight;
                        $offer->commodity                               = $request->commodity;
                        $offer->load_value                              = $request->load_value;

                        $offer->empty_miles                             = $request->empty_miles;
                        $offer->loaded_miles                            = $request->loaded_miles;
                        $offer->total_miles                             = $request->total_miles;
                        $offer->tolls                                   = $request->tolls;

                        $offer->route_type                              = $request->route_type;
                        if($request->has('open_border') && $request->open_border=='1'){
                            $offer->open_border                         = '1';
                        }else{
                            $offer->open_border                         = '0';
                        }
                        if($request->has('hazardous') && $request->hazardous=='1'){
                            $offer->hazardous                           = '1';
                            $offer->hazardous_option                    = $request->hazardous_option;
                        }else{
                            $offer->hazardous                           = '0';
                            $offer->hazardous_option                    = null;
                        }
                        if($request->has('over_dimension') && $request->over_dimension=='1'){
                            $offer->over_dimension                      = '1';
                            $offer->length                              = $request->length;
                            $offer->width                               = $request->width;
                            $offer->height                              = $request->height;
                        }else{
                            $offer->over_dimension                      = '0';
                            $offer->length                              = null;
                            $offer->width                               = null;
                            $offer->height                              = null;
                        }

                        $rate_offered                                   = $request->rate_offered;
                        $agreed_upon_rate                               = $request->agreed_upon_rate;

                        $is_rate_offered                                = '0';
                        if($request->has('is_rate_offered') && $request->is_rate_offered=='1'){
                            $is_rate_offered                            = '1';
                        }
                        if(!is_null($rate_offered)){
                            $is_rate_offered                            = '1';
                        }else{
                            $is_rate_offered                            = '0';
                        }

                        $is_final_rate                                  = '0';
                        if($request->has('is_final_rate') && $request->is_final_rate=='1'){
                            $is_final_rate                              = '1';
                        }
                        if(!is_null($agreed_upon_rate)){
                            $is_final_rate                              = '1';
                        }else{
                            $is_final_rate                              = '0';
                        }

                        $offer->rate_offered                            = $rate_offered;
                        $offer->is_rate_offered                         = $is_rate_offered;
                        $offer->agreed_upon_rate                        = $agreed_upon_rate;
                        $offer->is_final_rate                           = $is_final_rate;
                        $offer->rate_per_day                            = $request->rate_per_day;
                        $offer->rate_per_hour                           = $request->rate_per_hour;
                        $offer->notes                                   = $request->notes;

                        if(!is_null($request->is_add_note_to_load))
                            $offer->is_add_note_to_load                 = $request->is_add_note_to_load;
                        else
                            $offer->is_add_note_to_load                 = 0;
                        if(in_array($request->favorite, ['Yes','No']))
                            $offer->favorite                            = $request->favorite;

                        $offer->days                                    = $request->days;
                        $offer->hours                                   = $request->hours;
                        $offer->minutes                                 = $request->minutes;

                        if($request->has('status')){
                            $offer->status                              = $request->status;
                            $offer->status_at                           = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                        }
                        $offer->updated_at                              = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                        $offer->updated_by                              = auth()->guard('company-admin')->user()->id;
                        if($offer->save()){
                            if($request->has('rate_offered') && $request->rate_offered!=''){
                                $is_add                                 = true;
                                $history                                = $offer->histories->where('type', 0)->first();
                                if(!is_null($history)){
                                    if($history->rate==$request->rate_offered){
                                        $is_add                         = false;
                                    }
                                }
                                if($is_add){
                                    OfferHistory::create([
                                        'offer_id'                      => $offer->id,
                                        'submitted_by'                  => auth()->guard('company-admin')->user()->id,
                                        'rate'                          => $request->rate_offered,
                                        'created_at'                    => Carbon::now(),
                                        'created_by'                    => auth()->guard('company-admin')->user()->id
                                    ]);
                                }
                            }
                            if($request->has('agreed_upon_rate') && $request->agreed_upon_rate!=''){
                                $is_add                                 = true;
                                $history                                = $offer->histories->where('type', 2)->first();
                                if(!is_null($history)){
                                    if($history->rate==$request->agreed_upon_rate){
                                        $is_add                         = false;
                                    }
                                }
                                if($is_add){
                                    OfferHistory::create([
                                        'offer_id'                      => $offer->id,
                                        'submitted_by'                  => auth()->guard('company-admin')->user()->id,
                                        'rate'                          => $request->agreed_upon_rate,
                                        'type'                          => 2,
                                        'created_at'                    => Carbon::now(),
                                        'created_by'                    => auth()->guard('company-admin')->user()->id
                                    ]);
                                }
                            }

                            $stops                                      = isset($request->stop)?$request->stop:[];
                            $old_stop_row_id                            = array_column($stops, 'stop_row_id');
                            $remove_stop_query                          = OfferStop::where('offer_id', $offer->id);
                            if(!empty($old_stop_row_id)){
                                $remove_stop_query->whereNotIn('id', $old_stop_row_id);
                            }
                            $remove_stops                               = $remove_stop_query->get();
                            if(!empty($stops)){
                                foreach ($stops as $stkey => $stop) {
                                    $mode                               = '';
                                    if(isset($stop['stop_row_id']) && $stop['stop_row_id']!=''){
                                        $item                           = OfferStop::where('id', '=', $stop['stop_row_id'])->first();
                                        if(!is_null($item)){
                                            $mode                       = 'edit';
                                        }
                                    }else{
                                        $item                           = new OfferStop();
                                        $mode                           = 'add';
                                    }
                                    if($mode!=''){
                                        $item->stop_no                  = $stop['stop_no'] ?? null;
                                        $item->state                    = $stop['state'];
                                        $item->city                     = $stop['city'];
                                        $item->zip                      = $stop['zip'];
                                        $address_verified               = $stop['address_verified'] ?? 0;
                                        if($address_verified==0){
                                            $item->latitude             = null;
                                            $item->longitude            = null;
                                            $item->timezone             = null;
                                            $item->timezone_offset      = null;
                                            $item->timezone_abbr        = null;
                                            $item->address_verified     = 0;
                                        }else{
                                            $stop_timezone              = null;
                                            $stop_timezone_offset       = null;
                                            $stop_timezone_abbr         = $stop['timezone_abbr'] ?? null;
                                            if(!is_null($stop_timezone_abbr)){
                                                $stop_timezone          = timezone_name_from_abbr($stop_timezone_abbr);
                                                $stop_timezone_offset   = Carbon::createFromTimestamp(time(), $stop_timezone)->getOffsetString();
                                            }

                                            $item->latitude             = $stop['latitude'] ?? null;
                                            $item->longitude            = $stop['longitude'] ?? null;
                                            $item->timezone             = $stop_timezone;
                                            $item->timezone_offset      = $stop_timezone_offset;
                                            $item->timezone_abbr        = $stop_timezone_abbr;
                                            $item->address_verified     = 1;
                                        }
                                        if($mode=='add'){
                                            $item->offer_id             = $offer->id;
                                            $item->created_at           = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                            $item->created_by           = auth()->guard('company-admin')->user()->id;
                                        }else{
                                            $item->updated_at           = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                            $item->updated_by           = auth()->guard('company-admin')->user()->id;
                                        }
                                        $item->save();
                                    }
                                }
                            }
                            if($remove_stops->count() > 0){
                                foreach ($remove_stops as $remove) {
                                    $remove->deleted_by                 = auth()->guard('company-admin')->user()->id;
                                    $remove->save();
                                    $remove->delete();
                                }
                            }

                            // $data['offerHistory']                    = view('companyAdmin.dispatchHub.view.offerHistory',compact('offer'))->render();
                            $data['offer']                              = $offer;
                            $data['success']                            = true;
                            $data['message']                            = "Offer updated successfully.";
                            $data['backPage']                           = $request->backPage;
                        }
                    }
                }else{
                    $insertArry                                         = [];
                    $insertArry['driver_id']                            = $request->driver_id;
                    $insertArry['submitted_by']                         = auth()->guard('company-admin')->user()->id;

                    $insertArry['m_trailer_type']                       = $request->m_trailer_type;
                    $insertArry['truck_number']                         = $request->truck_number;
                    $insertArry['trailer_number']                       = $request->trailer_number;
                    $insertArry['trailer_type']                         = $request->trailer_type;
                    $insertArry['carrier_id']                           = $request->carrier_id;

                    $insertArry['broker_id']                            = $request->broker_id;
                    $insertArry['brokerage_mc']                         = $request->broker_mc;
                    $insertArry['terms']                                = $request->broker_terms;
                    $insertArry['point_of_contact']                     = $request->rep;
                    $insertArry['phone']                                = $request->broker_phone;
                    $insertArry['ext']                                  = $request->broker_ext;
                    $insertArry['email']                                = $request->broker_email;

                    $insertArry['score_ratting']                        = $request->score_ratting;
                    $insertArry['score_credit']                         = $request->score_credit;
                    $insertArry['score_updated_at']                     = $request->score_updated_at;

                    $insertArry['current_state']                        = $request->current_state;
                    $insertArry['current_city']                         = $request->current_city;
                    $insertArry['current_zip']                          = $request->current_zip;

                    $current_address_verified                           = $request->current_address_verified ?? 0;
                    if($current_address_verified==0){
                        $insertArry['current_latitude']                 = null;
                        $insertArry['current_longitude']                = null;
                        $insertArry['current_timezone']                 = null;
                        $insertArry['current_timezone_offset']          = null;
                        $insertArry['current_timezone_abbr']            = null;
                    }else{
                        $current_timezone                               = null;
                        $current_timezone_offset                        = null;
                        $current_timezone_abbr                          = $request->current_timezone_abbr ?? null;
                        if(!is_null($current_timezone_abbr)){
                            $current_timezone                           = timezone_name_from_abbr($current_timezone_abbr);
                            $current_timezone_offset                    = Carbon::createFromTimestamp(time(), $current_timezone)->getOffsetString();
                        }

                        $insertArry['current_latitude']                 = $request->current_latitude;
                        $insertArry['current_longitude']                = $request->current_longitude;
                        $insertArry['current_timezone']                 = $current_timezone;
                        $insertArry['current_timezone_offset']          = $current_timezone_offset;
                        $insertArry['current_timezone_abbr']            = $current_timezone_abbr;
                    }
                    $insertArry['current_address_verified']             = $current_address_verified;

                    $insertArry['shipper_date_time']                    = !is_null($request->shipper_date_time) ? Carbon::createFromFormat(getTimeLayout(), $request->shipper_date_time)->format('Y-m-d H:i:s') : null;
                    $insertArry['shipper_date_time_end']                = !is_null($request->shipper_date_time_end) ? Carbon::createFromFormat(getTimeLayout(), $request->shipper_date_time_end)->format('Y-m-d H:i:s') : null;
                    $insertArry['shipper_date']                         = ($request->pickup_date)?date('Y-m-d', strtotime($request->pickup_date)):null;
                    $insertArry['shipper_open_time']                    = !is_null($request->pickupo_time) ? Carbon::createFromFormat($time_format, $request->pickupo_time)->format('H:i') : null;
                    $insertArry['shipper_close_time']                   = !is_null($request->pickupc_time) ? Carbon::createFromFormat($time_format, $request->pickupc_time)->format('H:i') : null;
                    $insertArry['shipper_state']                        = $request->shipper_state;
                    $insertArry['shipper_city']                         = $request->shipper_city;
                    $insertArry['shipper_zip']                          = $request->shipper_zip;

                    $shipper_address_verified                           = $request->shipper_address_verified ?? 0;
                    if($shipper_address_verified==0){
                        $insertArry['shipper_latitude']                 = null;
                        $insertArry['shipper_longitude']                = null;
                        $insertArry['shipper_timezone']                 = null;
                        $insertArry['shipper_timezone_offset']          = null;
                        $insertArry['shipper_timezone_abbr']            = null;
                    }else{
                        $shipper_timezone                               = null;
                        $shipper_timezone_offset                        = null;
                        $shipper_timezone_abbr                          = $request->shipper_timezone_abbr ?? null;
                        if(!is_null($shipper_timezone_abbr)){
                            $shipper_timezone                           = timezone_name_from_abbr($shipper_timezone_abbr);
                            $shipper_timezone_offset                    = Carbon::createFromTimestamp(time(), $shipper_timezone)->getOffsetString();
                        }

                        $insertArry['shipper_latitude']                 = $request->shipper_latitude;
                        $insertArry['shipper_longitude']                = $request->shipper_longitude;
                        $insertArry['shipper_timezone']                 = $shipper_timezone;
                        $insertArry['shipper_timezone_offset']          = $shipper_timezone_offset;
                        $insertArry['shipper_timezone_abbr']            = $shipper_timezone_abbr;
                    }
                    $insertArry['shipper_address_verified']             = $shipper_address_verified;

                    $insertArry['delivery_date_time']                   = !is_null($request->delivery_date_time) ? Carbon::createFromFormat(getTimeLayout(), $request->delivery_date_time)->format('Y-m-d H:i:s') : null;
                    $insertArry['delivery_date_time_end']               = !is_null($request->delivery_date_time_end) ? Carbon::createFromFormat(getTimeLayout(), $request->delivery_date_time_end)->format('Y-m-d H:i:s') : null;
                    $insertArry['delivery_date']                        = ($request->delivery_date)?date('Y-m-d', strtotime($request->delivery_date)):null;
                    $insertArry['delivery_open_time']                   = !is_null($request->deliveryo_time) ? Carbon::createFromFormat($time_format, $request->deliveryo_time)->format('H:i') : null;
                    $insertArry['delivery_close_time']                  = !is_null($request->deliveryc_time) ? Carbon::createFromFormat($time_format, $request->deliveryc_time)->format('H:i') : null;
                    $insertArry['delivery_state']                       = $request->delivery_state;
                    $insertArry['delivery_city']                        = $request->delivery_city;
                    $insertArry['delivery_zip']                         = $request->delivery_zip;

                    $delivery_address_verified                          = $request->delivery_address_verified ?? 0;
                    if($delivery_address_verified==0){
                        $insertArry['delivery_latitude']                = null;
                        $insertArry['delivery_longitude']               = null;
                        $insertArry['delivery_timezone']                = null;
                        $insertArry['delivery_timezone_offset']         = null;
                        $insertArry['delivery_timezone_abbr']           = null;
                    }else{
                        $delivery_timezone                              = null;
                        $delivery_timezone_offset                       = null;
                        $delivery_timezone_abbr                         = $request->delivery_timezone_abbr ?? null;
                        if(!is_null($delivery_timezone_abbr)){
                            $delivery_timezone                          = timezone_name_from_abbr($delivery_timezone_abbr);
                            $delivery_timezone_offset                   = Carbon::createFromTimestamp(time(), $delivery_timezone)->getOffsetString();
                        }

                        $insertArry['delivery_latitude']                = $request->delivery_latitude;
                        $insertArry['delivery_longitude']               = $request->delivery_longitude;
                        $insertArry['delivery_timezone']                = $delivery_timezone;
                        $insertArry['delivery_timezone_offset']         = $delivery_timezone_offset;
                        $insertArry['delivery_timezone_abbr']           = $delivery_timezone_abbr;
                    }
                    $insertArry['delivery_address_verified']            = $delivery_address_verified;

                    $insertArry['temp']                                 = $request->temp;
                    $insertArry['weight']                               = $request->weight;
                    $insertArry['commodity']                            = $request->commodity;
                    $insertArry['load_value']                           = $request->load_value;

                    $insertArry['empty_miles']                          = $request->empty_miles;
                    $insertArry['loaded_miles']                         = $request->loaded_miles;
                    $insertArry['total_miles']                          = $request->total_miles;
                    $insertArry['tolls']                                = $request->tolls;

                    $insertArry['route_type']                           = $request->route_type;
                    if($request->has('open_border') && $request->open_border=='1'){
                        $insertArry['open_border']                      = '1';
                    }
                    if($request->has('hazardous') && $request->hazardous=='1'){
                        $insertArry['hazardous']                        = '1';
                        $insertArry['hazardous_option']                 = $request->hazardous_option;
                    }
                    if($request->has('over_dimension') && $request->over_dimension=='1'){
                        $insertArry['over_dimension']                   = '1';
                        $insertArry['length']                           = $request->length;
                        $insertArry['width']                            = $request->width;
                        $insertArry['height']                           = $request->height;
                    }

                    $rate_offered                                       = $request->rate_offered;
                    $agreed_upon_rate                                   = $request->agreed_upon_rate;

                    $is_rate_offered                                    = '0';
                    if($request->has('is_rate_offered') && $request->is_rate_offered=='1'){
                        $is_rate_offered                                = '1';
                    }
                    if(!is_null($rate_offered)){
                        $is_rate_offered                                = '1';
                    }else{
                        $is_rate_offered                                = '0';
                    }

                    $is_final_rate                                      = '0';
                    if($request->has('is_final_rate') && $request->is_final_rate=='1'){
                        $is_final_rate                                  = '1';
                    }
                    if(!is_null($agreed_upon_rate)){
                        $is_final_rate                                  = '1';
                    }else{
                        $is_final_rate                                  = '0';
                    }

                    $insertArry['rate_offered']                         = $rate_offered;
                    $insertArry['is_rate_offered']                      = $is_rate_offered;
                    $insertArry['agreed_upon_rate']                     = $agreed_upon_rate;
                    $insertArry['is_final_rate']                        = $is_final_rate;
                    $insertArry['rate_per_day']                         = $request->rate_per_day;
                    $insertArry['rate_per_hour']                        = $request->rate_per_hour;
                    $insertArry['notes']                                = $request->notes;

                    if(!is_null($request->is_add_note_to_load))
                        $insertArry['is_add_note_to_load']              = $request->is_add_note_to_load;
                    if(in_array($request->favorite, ['Yes','No']))
                        $insertArry['favorite']                         = $request->favorite;

                    $insertArry['days']                                 = $request->days;
                    $insertArry['hours']                                = $request->hours;
                    $insertArry['minutes']                              = $request->minutes;

                    if($request->has('status')){
                        $insertArry['status']                           = $request->status;
                    }else{
                        $insertArry['status']                           = 0;
                    }
                    $insertArry['status_at']                            = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');

                    $insertArry['created_at']                           = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                    $insertArry['created_by']                           = auth()->guard('company-admin')->user()->id;
                    $offer                                              = Offer::create($insertArry);
                    if($offer){
                        if($request->has('rate_offered') && $request->rate_offered!=''){
                            OfferHistory::create([
                                'offer_id'                              => $offer->id,
                                'submitted_by'                          => auth()->guard('company-admin')->user()->id,
                                'rate'                                  => $request->rate_offered,
                                'created_at'                            => Carbon::now(),
                                'created_by'                            => auth()->guard('company-admin')->user()->id
                            ]);
                        }
                        if($request->has('agreed_upon_rate') && $request->agreed_upon_rate!=''){
                            OfferHistory::create([
                                'offer_id'                              => $offer->id,
                                'submitted_by'                          => auth()->guard('company-admin')->user()->id,
                                'rate'                                  => $request->agreed_upon_rate,
                                'type'                                  => 2,
                                'created_at'                            => Carbon::now(),
                                'created_by'                            => auth()->guard('company-admin')->user()->id
                            ]);
                        }

                        $stops                                          = isset($request->stop)?$request->stop:[];
                        if(!empty($stops)){
                            foreach ($stops as $stkey => $stop) {
                                $stop_array                             = [];
                                $stop_array['stop_no']                  = $stop['stop_no'] ?? null;
                                $stop_array['offer_id']                 = $offer->id;
                                $stop_array['state']                    = $stop['state'];
                                $stop_array['city']                     = $stop['city'];
                                $stop_array['zip']                      = $stop['zip'];
                                $address_verified                       = $stop['address_verified'] ?? 0;
                                if($address_verified==0){
                                    $stop_array['latitude']             = null;
                                    $stop_array['longitude']            = null;
                                    $stop_array['timezone']             = null;
                                    $stop_array['timezone_offset']      = null;
                                    $stop_array['timezone_abbr']        = null;
                                    $stop_array['address_verified']     = 0;
                                }else{
                                    $stop_timezone                      = null;
                                    $stop_timezone_offset               = null;
                                    $stop_timezone_abbr                 = $stop['timezone_abbr'] ?? null;
                                    if(!is_null($stop_timezone_abbr)){
                                        $stop_timezone                  = timezone_name_from_abbr($stop_timezone_abbr);
                                        $stop_timezone_offset           = Carbon::createFromTimestamp(time(), $stop_timezone)->getOffsetString();
                                    }

                                    $stop_array['latitude']             = $stop['latitude'] ?? null;
                                    $stop_array['longitude']            = $stop['longitude'] ?? null;
                                    $stop_array['timezone']             = $stop_timezone;
                                    $stop_array['timezone_offset']      = $stop_timezone_offset;
                                    $stop_array['timezone_abbr']        = $stop_timezone_abbr;
                                    $stop_array['address_verified']     = 1;
                                }
                                $stop_array['created_at']               = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                                $stop_array['created_by']               = auth()->guard('company-admin')->user()->id;
                                OfferStop::create($stop_array);
                            }
                        }

                        // $data['offerHistory']                        = view('companyAdmin.dispatchHub.view.offerHistory',compact('offer'))->render();
                        $data['offer']                                  = $offer;
                        $data['success']                                = true;
                        $data['message']                                = "Offer created successfully.";
                        $data['backPage']                               = $request->backPage;
                    }
                }
            }catch (\Exception $e) {
                $data['success']                                        = false;
                $data['message']                                        = $e->getMessage();
            }
        }
        return response()->json($data);
    }

    public function counterOfferSave(Request $request){
        $data['success']                                            = false;
        $data['message']                                            = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $offer                                              = Offer::where('id', $request->offer_id)->first();
                if($offer){
                    OfferHistory::create([
                        'offer_id'                                  => $request->offer_id,
                        'submitted_by'                              => auth()->guard('company-admin')->user()->id,
                        'rate'                                      => $request->counter_offer,
                        'type'                                      => 1,
                        'created_at'                                => Carbon::now(),
                        'created_by'                                => auth()->guard('company-admin')->user()->id
                    ]);
                    $data['offerHistory']                           = view('companyAdmin.dispatchHub.view.offerHistory',compact('offer'))->render();
                    $data['success']                                = true;
                    $data['message']                                = "Counter offer created successfully.";
                }
            }catch (\Exception $e) {
                $data['success']                                    = false;
                $data['message']                                    = $e->getMessage();
            }
        }
        return response()->json($data);
    }

    public function cancelReservationOffer(Request $request){
        $data['success']                                            = false;
        $data['message']                                            = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $offer                                              = Offer::where('id', $request->offer_id)->first();
                if($offer){
                    $offer->status                                  = 0;
                    $offer->status_at                               = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                    $offer->updated_at                              = Carbon::now()->format('Y-m-d H:i:s'); // date('Y-m-d H:i:s');
                    $offer->updated_by                              = auth()->guard('company-admin')->user()->id;
                    if($offer->save()){
                        $data['success']                            = true;
                        $data['message']                            = "Cancel reservation successfully.";
                    }
                }
            }catch (\Exception $e) {
                $data['success']                                    = false;
                $data['message']                                    = $e->getMessage();
            }
        }
        return response()->json($data);
    }

    public function bookOfferDelete($id){
        $offer      = Offer::find($id);
        $driver_id  = $offer->driver_id;
        $offer->delete();
        //notification('success', $this->crudMessage('delete', 'Offer'));
        //return redirect()->route('company.admin.driver.offers',$driver_id);
        return response()->json(['success' => true, 'message' => 'Offer deleted successfully']);
    }

    public function bookOfferView(Request $request){
        $data['success']                = false;
        $data['message']                = "Oops! Something went wrong..";
        if($request->ajax()){
            try {
                $driver                 = Driver::select(
                        'drivers.*',

                        'teams.id as teamId',

                        //cuurent
                        'load_details.id as load_detail_id',
                        'load_details.load_id as load_id',

                        'load_details.load_number',
                        'load_details.trailer_number',
                        'load_details.trailer_type',
                        'load_details.truck_number',

                        'load_details.is_disable',
                        'load_details.load_type',
                        'load_details.load_status',

                        'load_details.booked_by',

                        'load_details.delivery_location',
                        'load_details.delivery_city',
                        'load_details.delivery_state',
                        'load_details.actual_delivery_date',

                        //previous
                        'load_details_al.id as previous_load_detail_id',
                        'load_details_al.load_id as previous_load_id',

                        'load_details_al.load_number as previous_load_number',
                        'load_details_al.trailer_number as previous_trailer_number',
                        'load_details_al.trailer_type as previous_trailer_type',
                        'load_details_al.truck_number as previous_truck_number',

                        'load_details_al.is_disable as previous_load_is_disable',
                        'load_details_al.load_type as previous_load_load_type',
                        'load_details_al.load_status as previous_load_status',

                        'load_details_al.booked_by as previous_booked_by',

                        'load_details_al.delivery_location as load_delivery_location',
                        'load_details_al.delivery_city as previous_load_delivery_city',
                        'load_details_al.delivery_state as previous_load_delivery_state',
                        'load_details_al.actual_delivery_date as load_actual_delivery_date',
                        'load_details_al.delivery_timezone_abbr as load_delivery_timezone_abbr',

                        DB::raw('TIMESTAMPDIFF(hour, load_details_al.actual_delivery_date, NOW()) as duration')
                    )
                    ->leftJoin('teams', function($join) {
                        $join->on('drivers.team_id', '=', 'teams.id');
                    })
                    ->leftJoin('driver_informations', function($join) {
                        $join->on('drivers.id', '=', 'driver_informations.driver_id');
                    })
                    ->leftJoin('load_details', function($join) { // Not Complete
                        $join->on('drivers.id', '=' , 'load_details.driver_id')
                            //->on('load_details.actual_delivery_date', '=', DB::raw("(SELECT max(actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id)"))
                            ->where(function($query) {
                                $query->whereNotIn('load_details.load_status', [5,6,10,8])
                                    ->where(function($q) {
                                        $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                                    })
                                    ->where('load_details.is_disable', 0)
                                    ->whereNull('load_details.deleted_at');
                                    //->where('load_details.actual_delivery_date', '=', DB::raw("(select MAX(a2.actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id AND a2.load_status NOT IN(6,7,8,9) AND a2.is_disable = 0 AND a2.deleted_at IS NULL)"));
                            });
                    })
                    ->leftJoin('load_details as load_details_al',function ($join) { // Complete
                        $join->on('drivers.id', '=' , 'load_details_al.driver_id')
                            //->on('load_details_al.actual_delivery_date', '=', DB::raw("(SELECT max(actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id)"))
                            ->where(function($query) {
                                $query->where(function($q) {
                                        $q->whereIn('load_details_al.load_status', [5,6,10,8])->orWhereIn('load_details_al.is_on_hold', [1,2]);
                                    })
                                    ->where('load_details_al.is_disable', 0)
                                    ->whereNull('load_details_al.deleted_at')
                                    ->where('load_details_al.actual_delivery_date', '=', DB::raw("(select MAX(a2.actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id AND (a2.load_status IN(5,6,10,8) OR a2.is_on_hold IN(1,2)) AND a2.is_disable = 0 AND a2.deleted_at IS NULL)"));
                            });
                    })
                    ->whereNull('load_details.id')
                    ->where('drivers.id', $request->id)
                    ->groupBy('drivers.id')
                    ->first();
                if(!is_null($driver)){
                    $getTimeZone        = getTimeZone();
                    $getTimeZoneId      = getTimeZoneId();

                    $carrierId          = (isset($driver->driver_information->carrier_id)) ? $driver->driver_information->carrier_id : null;
                    $checkAPIKey        = IntegrationSetting::where('company_id', $driver->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->first();
                    $samsara_data       = [];
                    $samsara_id         = (isset($driver->samsara_id) && $driver->samsara_id!='')?$driver->samsara_id:'';//4644765
                    if(!is_null($checkAPIKey) && $samsara_id!=''){
                        $getAPIKey      = IntegrationSetting::where('company_id', $driver->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->first();
                        if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                            $client                         = new \GuzzleHttp\Client();
                            $api_url                        = 'https://api.samsara.com/fleet/hos/clocks?driverIds='.$samsara_id;
                            $api_response                   = $client->request('GET', $api_url, [
                                'headers'                   => [
                                    'Accept'                => 'application/json',
                                    'Authorization'         => 'Bearer '.$getAPIKey->api_key,
                                ],
                            ]);

                            $api_data                       = json_decode($api_response->getBody());
                            if(!empty($api_data)){
                                $appData                    = $api_data->data[0];
                                if(!empty($appData)){
                                    $samsara_data['clocks'] = $appData;

                                    $vehicleIds             = (isset($appData->currentVehicle->id) && $appData->currentVehicle->id!='')?$appData->currentVehicle->id:'';
                                    if($vehicleIds!=''){
                                        $api_url            = 'https://api.samsara.com/fleet/vehicles/locations?vehicleIds='.$vehicleIds;
                                        $api_response       = $client->request('GET', $api_url, [
                                            'headers'                       => [
                                                'Accept'                    => 'application/json',
                                                'Authorization'             => 'Bearer '.$getAPIKey->api_key,
                                            ],
                                        ]);
                                        $api_data                           = json_decode($api_response->getBody());
                                        if(!empty($api_data)){
                                            $appData                        = $api_data->data[0];
                                            if(!empty($appData)){
                                                $samsara_data['vehicleloc'] = $appData;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    $offer              = null;
                    if($request->has('offer_id') && $request->offer_id!=''){
                        $offer          = Offer::with(['histories','histories.company_admin'])->where('id', $request->offer_id)->first();
                    }
                    $html               = view('companyAdmin.dispatchHub.view.bookOffer',compact(
                        'driver','offer','getTimeZone','getTimeZoneId','checkAPIKey','samsara_data'
                    ))->render();
                    $data['success']    = true;
                    $data['message']    = "Driver offer retrieved successfully.";
                    $data['html']       = $html;
                }else{
                    $data['success']    = false;
                    $data['message']    = "Driver not found.";
                }
            }catch (\Exception $e) {
                $data['success']        = false;
                $data['message']        = $e->getMessage();
            }
        }
        return response()->json($data);
    }

    public function driverOffers($id){
        $driver                 = Driver::select(
                'drivers.*',

                'teams.id as teamId',

                //cuurent
                'load_details.id as load_detail_id',
                'load_details.load_id as load_id',

                'load_details.load_number',
                'load_details.trailer_number',
                'load_details.trailer_type',
                'load_details.truck_number',

                'load_details.is_disable',
                'load_details.load_type',
                'load_details.load_status',

                'load_details.booked_by',

                'load_details.delivery_location',
                'load_details.delivery_city',
                'load_details.delivery_state',
                'load_details.actual_delivery_date',

                //previous
                'load_details_al.id as previous_load_detail_id',
                'load_details_al.load_id as previous_load_id',

                'load_details_al.load_number as previous_load_number',
                'load_details_al.trailer_number as previous_trailer_number',
                'load_details_al.trailer_type as previous_trailer_type',
                'load_details_al.truck_number as previous_truck_number',

                'load_details_al.is_disable as previous_load_is_disable',
                'load_details_al.load_type as previous_load_load_type',
                'load_details_al.load_status as previous_load_status',

                'load_details_al.booked_by as previous_booked_by',

                'load_details_al.delivery_location as load_delivery_location',
                'load_details_al.delivery_city as previous_load_delivery_city',
                'load_details_al.delivery_state as previous_load_delivery_state',
                'load_details_al.actual_delivery_date as load_actual_delivery_date',
                'load_details_al.delivery_timezone_abbr as load_delivery_timezone_abbr',

                DB::raw('TIMESTAMPDIFF(hour, load_details_al.actual_delivery_date, NOW()) as duration')
            )
            ->leftJoin('teams', function($join) {
                $join->on('drivers.team_id', '=', 'teams.id');
            })
            ->leftJoin('driver_informations', function($join) {
                $join->on('drivers.id', '=', 'driver_informations.driver_id');
            })
            ->leftJoin('load_details', function($join) { // Not Complete
                $join->on('drivers.id', '=' , 'load_details.driver_id')
                    //->on('load_details.actual_delivery_date', '=', DB::raw("(SELECT max(actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id)"))
                    ->where(function($query) {
                        $query->whereNotIn('load_details.load_status', [5,6,10,8])
                            ->where(function($q) {
                                $q->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
                            })
                            ->where('load_details.is_disable', 0)
                            ->whereNull('load_details.deleted_at');
                            //->where('load_details.actual_delivery_date', '=', DB::raw("(select MAX(a2.actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id AND a2.load_status NOT IN(6,7,8,9) AND a2.is_disable = 0 AND a2.deleted_at IS NULL)"));
                    });
            })
            ->leftJoin('load_details as load_details_al',function ($join) { // Complete
                $join->on('drivers.id', '=' , 'load_details_al.driver_id')
                    //->on('load_details_al.actual_delivery_date', '=', DB::raw("(SELECT max(actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id)"))
                    ->where(function($query) {
                        $query->where(function($q) {
                                $q->whereIn('load_details_al.load_status', [5,6,10,8])->orWhereIn('load_details_al.is_on_hold', [1,2]);
                            })
                            ->where('load_details_al.is_disable', 0)
                            ->whereNull('load_details_al.deleted_at')
                            ->where('load_details_al.actual_delivery_date', '=', DB::raw("(select MAX(a2.actual_delivery_date) from load_details as a2 WHERE a2.driver_id = drivers.id AND (a2.load_status IN(5,6,10,8) OR a2.is_on_hold IN(1,2)) AND a2.is_disable = 0 AND a2.deleted_at IS NULL)"));
                    });
            })
            ->whereNull('load_details.id')
            ->where('drivers.id', $id)
            ->groupBy('drivers.id')
            ->first();
        if($driver){
            $getTimeZone        = getTimeZone();
            $getTimeZoneId      = getTimeZoneId();

            $carrierId = (isset($driver->driver_information->carrier_id)) ? $driver->driver_information->carrier_id : null;
            $checkAPIKey        = IntegrationSetting::where('company_id', $driver->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->first();
            $samsara_data       = [];
            $samsara_id         = (isset($driver->samsara_id) && $driver->samsara_id!='')?$driver->samsara_id:'';//4644765
            if(!is_null($checkAPIKey) && $samsara_id!=''){
                $getAPIKey      = IntegrationSetting::where('company_id', $driver->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->first();
                if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                    $client                         = new \GuzzleHttp\Client();
                    $api_url                        = 'https://api.samsara.com/fleet/hos/clocks?driverIds='.$samsara_id;
                    $api_response                   = $client->request('GET', $api_url, [
                        'headers'                   => [
                            'Accept'                => 'application/json',
                            'Authorization'         => 'Bearer '.$getAPIKey->api_key,
                        ],
                    ]);

                    $api_data                       = json_decode($api_response->getBody());
                    if(!empty($api_data)){
                        $appData                    = $api_data->data[0];
                        if(!empty($appData)){
                            $samsara_data['clocks'] = $appData;

                            $vehicleIds             = (isset($appData->currentVehicle->id) && $appData->currentVehicle->id!='')?$appData->currentVehicle->id:'';
                            if($vehicleIds!=''){
                                $api_url            = 'https://api.samsara.com/fleet/vehicles/locations?vehicleIds='.$vehicleIds;
                                $api_response       = $client->request('GET', $api_url, [
                                    'headers'                       => [
                                        'Accept'                    => 'application/json',
                                        'Authorization'             => 'Bearer '.$getAPIKey->api_key,
                                    ],
                                ]);
                                $api_data                           = json_decode($api_response->getBody());
                                if(!empty($api_data)){
                                    $appData                        = $api_data->data[0];
                                    if(!empty($appData)){
                                        $samsara_data['vehicleloc'] = $appData;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            return view('companyAdmin.dispatchHub.view.driverOffer', compact('driver','getTimeZone','getTimeZoneId','checkAPIKey','samsara_data'));
        }else{
            return abort(404);
        }
    }

    public function driverOffersData(Request $request){
        $offers                     = Offer::where('driver_id', $request->id)->where('status', '!=', 4)
            ->orderBy('favorite', 'ASC')
            ->orderBy('created_at', 'DESC');
        return Datatables::of($offers)
                ->addIndexColumn()
                ->addColumn('tag', function($row){
                    $action         = '<img src="'.asset('theme/img/tag.png').'" alt="offer">';
                    if(isset($row->favorite) && $row->favorite=='Yes'){
                        $action    .= '<img src="'.asset('theme/img/star-1.png').'" alt="Favorite offer" class="ml-2">';
                    }
                    if(hasPermission(config('settings.company_admin_modules.load_reservations'),config('settings.permissions.delete'))){
                        $action .= '<a href="javascript:void(0)" class="destroy" data-action="'.route('company.admin.book.offer.delete',$row->id).'" data-offer_id="'.$row->id.'"><img src="'.asset('theme/img/delete-icon-dark.png').'" alt="delete offer" class="ml-3"></a>';
                    }
                    return $action;
                })
                ->addColumn('reserve_status', function($row){
                    return '';
                    /*
                    $action             = '';
                    if($row->status==2){
                        if(hasPermission(config('settings.company_admin_modules.load_reservations'),config('settings.permissions.edit'))){
                            $action    .= '<a href="javascript:void(0)" class="btn btn-outline-info offerButton" data-id="'.$row->driver_id.'" data-offer_id="'.$row->id.'" data-href="'.route("company.admin.book.offer.view").'">Reserved</a>';
                        }
                    }else{
                        if(hasPermission(config('settings.company_admin_modules.load_reservations'),config('settings.permissions.create'))){
                            $action    .= '<a href="javascript:void(0)" class="btn btn-outline-info offerButton" data-id="'.$row->driver_id.'" data-offer_id="'.$row->id.'" data-href="'.route("company.admin.book.offer.view").'">Book</a>';
                        }
                    }
                    return $action;
                    */
                })
                ->addColumn('broker_name', function ($row) {
                    return ($row->broker) ? $row->broker->name:'';
                })
                ->addColumn('final_rate', function ($row) {
                    $finaloffer     = $row->histories->where('type', 2)->first();
                    if(!is_null($finaloffer)){
                        if($finaloffer->rate > 0)
                            return '$'.formatNumberWithoutZeros($finaloffer->rate);
                    }
                    return '';
                })
                ->addColumn('cpm', function ($row) {
                    $finalrate      = '';
                    $finaloffer     = $row->histories->where('type', 2)->first();
                    if(!is_null($finaloffer)){
                        $finalrate  = $finaloffer->rate;
                    }
                    if(($finalrate!='' && $finalrate > 0) && (isset($row->loaded_miles) && $row->loaded_miles!='' && $row->loaded_miles > 0))
                        return '$'.formatNumberWithoutZeros($finalrate/$row->loaded_miles);
                    return '';
                })
                ->addColumn('notes', function($row){
                    return '';
                    /*
                    return '<span class="down-arrow offerButton" data-id="'.$row->driver_id.'" data-offer_id="'.$row->id.'" data-href="'.route("company.admin.book.offer.view").'"></span>';
                    */
                })
                ->addColumn('drive', function ($row) {
                    return '';
                })
                ->addColumn('shift', function ($row) {
                    return '';
                })
                ->addColumn('cycle', function ($row) {
                    return '';
                })
                ->addColumn('status', function ($row) {
                    return '';
                })
                ->addColumn('eta', function ($row) {
                    return '';
                })
                ->rawColumns(['tag','reserve_status','notes'])
                ->make(true);
    }

    public function availableDriverSamsaraData(Request $request){
        if($request->ajax()){
            try {
                $row                                    = Driver::where('id', $request->id)->first();
                if($row){
                    $samsara_data                       = [];
                    $samsara_id                         = (isset($row->samsara_id) && $row->samsara_id!='')?$row->samsara_id:'';//4644765
                    if($samsara_id!=''){
                        $carrierId = (isset($row->driver_information->carrier_id)) ? $row->driver_information->carrier_id : null;
                        $getAPIKey                      = IntegrationSetting::where('company_id', $row->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->first();
                        if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                            $client                     = new \GuzzleHttp\Client();
                            $api_url                    = 'https://api.samsara.com/fleet/hos/clocks?driverIds='.$samsara_id;
                            $api_response               = $client->request('GET', $api_url, [
                                'headers'               => [
                                    'Accept'            => 'application/json',
                                    'Authorization'     => 'Bearer '.$getAPIKey->api_key,
                                ],
                            ]);

                            $api_data                   = json_decode($api_response->getBody());
                            if(!empty($api_data)){
                                $appData                = $api_data->data[0];
                                if(!empty($appData)){
                                    $samsara_data['drive']                  = formatSeconds($appData->clocks->drive->driveRemainingDurationMs);
                                    $samsara_data['shift']                  = formatSeconds($appData->clocks->shift->shiftRemainingDurationMs);
                                    $samsara_data['cycle']                  = formatSeconds($appData->clocks->cycle->cycleRemainingDurationMs);
                                    $samsara_data['cycle_tom']              = formatSeconds($appData->clocks->cycle->cycleTomorrowDurationMs);
                                    $samsara_data['duty_status']            = $appData->currentDutyStatus->hosStatusType;
                                    $samsara_data['time_in_current_status'] = '';
                                    $samsara_data['time_in_unit_break']     = formatSeconds($appData->clocks->break->timeUntilBreakDurationMs);
                                }
                            }
                        }
                    }
                    if(!empty($samsara_data)){
                        $data['success']                = true;
                        $data['message']                = "Samsara data retrieved successfully.";
                        $data['data']                   = $samsara_data;
                    }else{
                        $data['success']                = false;
                        $data['message']                = "Samsara data not found.";
                    }
                }else{
                    $data['success']                    = false;
                    $data['message']                    = "Driver not found.";
                }
            }catch (\Exception $e) {
                $data['success']                        = false;
                $data['message']                        = $e->getMessage();
            }
        }
        return response()->json($data);
    }

    public function updateLoadCoDriver(Request $request){
        try {
            $loadId                         = $request->loadId;
            $coDriver                       = (isset($request->coDriver) && !is_null($request->coDriver)) ? $request->coDriver : null;

            $loadDetails                    = LoadDetail::where('load_id', $loadId)->first();

            if(!is_null($loadDetails)){
                $loadDetails->co_driver_id  = $coDriver;
                $loadDetails->save();

                $response['success']        = true;
                $response['message']        = "Record updated successfully.";
            }else{
                $response['success']        = false;
                $response['message']        = "Record not found.";
            }
        } catch (\Exception $e) {
            $response['success']            = false;
            $response['message']            = $e->getMessage();
        }
        return response()->json($response);
    }

    public function updateLoadStatus(Request $request){
        try {
            $loadId                                     = $request->loadId;
            $loadDetails                                = LoadDetail::where('load_id', $loadId)->first();

            if(!is_null($loadDetails)){
                $isOnHoldPopup                          = (isset($request->isOnHoldPopup)) ? $request->isOnHoldPopup : null;

                if(!is_null($isOnHoldPopup)){
                    $reasonOnHold                       = (isset($request->reasonOnHold)) ? $request->reasonOnHold : null;
                    $reasonOnHoldSubCategory            = (isset($request->reasonOnHoldSubCategory)) ? $request->reasonOnHoldSubCategory : null;
                    $reasonOnHoldNotes                  = (isset($request->reasonOnHoldNotes)) ? $request->reasonOnHoldNotes : null;

                    $loadDetails->is_on_hold            = '1';
                    // $loadDetails->load_status        = '7';
                    $loadDetails->notification_reason   = $reasonOnHold;
                    $loadDetails->reason_on_hold        = $reasonOnHoldNotes;
                    $loadDetails->load_on_hold_datetime = Carbon::now()->format('Y-m-d H:i:s');
                    $loadDetails->save();
                }else{
                    $loadDetails->is_on_hold            = null;
                    // $loadDetails->load_status        = $request->status;
                    $loadDetails->notification_reason   = null;
                    $loadDetails->reason_on_hold        = null;
                    $loadDetails->load_on_hold_datetime = null;
                    $loadDetails->save();
                }

                $response['success']                    = true;
                $response['message']                    = "Record updated successfully.";
            }else{
                $response['success']                    = false;
                $response['message']                    = "Record not found.";
            }
        } catch (\Exception $e) {
            $response['success']                        = false;
            $response['message']                        = $e->getMessage();
        }

        return response()->json($response);
    }

    public function loadComparisonView(Request $request){
        $driverId                           = $request->id;
        $offers                             = Offer::where('driver_id', $driverId)->where('status', '!=', 4)->orderBy('favorite', 'ASC')->orderBy('created_at', 'DESC')->get();

        $x                                  = 'A';
        $offersData                         = array();
        foreach($offers as $offer){
            $cpm                            = '0';
            if(!is_null($offer)){
                $finalRate                  = '';
                $finalOffer                 = $offer->histories->where('type', 2)->first();
                if(!is_null($finalOffer)){
                    $finalRate              = $finalOffer->rate;
                }
                if(($finalRate != '' && $finalRate > 0) && (isset($offer->loaded_miles) && $offer->loaded_miles != '' && $offer->loaded_miles > 0)) {
                    $cpm                    = formatNumberWithoutZeros($finalRate/$offer->loaded_miles);
                }
            }
            $Origin                         = null;
            if(!is_null($offer->shipper_city)){
                $Origin                    .= $offer->shipper_city.', ';
            }
            if(!is_null($offer->shipper_state)){
                $Origin                    .= $offer->shipper_state;
            }
            /*
            if(!is_null($offer->shipper_zip)){
                $Origin                    .= $offer->shipper_zip;
            }
            */

            $Destination                    = null;
            if(!is_null($offer->delivery_city)){
                $Destination               .= $offer->delivery_city.', ';
            }
            if(!is_null($offer->delivery_state)){
                $Destination               .= $offer->delivery_state;
            }
            /*
            if(!is_null($offer->delivery_zip)){
                $Destination               .= $offer->delivery_zip;
            }
            */

            $tollCosts                      = (!is_null($offer->tolls)) ? $offer->tolls  : 0;

            $offersData['offer '.$x]        = array(
                'Pickup Date'               => (!is_null($offer->shipper_date_time)) ? Carbon::parse($offer->shipper_date_time)->timezone($this->timeZone)->format('m/d/Y') : '',
                'Pickup Time'               => (!is_null($offer->delivery_date_time)) ? Carbon::parse($offer->delivery_date_time)->timezone($this->timeZone)->format('H:i') : '',
                'Origin'                    => $Origin,
                'Delivery Date'             => (!is_null($offer->delivery_date_time)) ? Carbon::parse($offer->delivery_date_time)->timezone($this->timeZone)->format('m/d/Y') : '',
                'Delivery Time'             => (!is_null($offer->delivery_date_time)) ? Carbon::parse($offer->delivery_date_time)->timezone($this->timeZone)->format('H:i') : '',
                'Destination'               => $Destination,
                'Deadhead Miles'            => (!is_null($offer->empty_miles)) ? $offer->empty_miles : '',
                'Loaded Miles'              => (!is_null($offer->loaded_miles)) ? $offer->loaded_miles : '',
                'Total Miles'               => (!is_null($offer->total_miles)) ? $offer->total_miles : '',
                'Revenue'                   => (!is_null($offer->rate_offered)) ? '$'.formatNumberWithoutZeros($offer->rate_offered) : '$0',
                'CPM'                       => '$'.$cpm,
                'Linehaul'                  => '-',
                'Fuel Surcharge'            => '-',
                'Fuel Costs'                => '-',
                'Toll Costs'                => '$'.$tollCosts,
                'Driver Pay'                => '-',
                'Net'                       => '$'.($tollCosts),
                'Rev/Day (RPD)'             => (!is_null($offer->rate_per_day)) ? '$'.$offer->rate_per_day : '',
                'Rev/HR (RPH)'              => (!is_null($offer->rate_per_hour)) ? '$'.$offer->rate_per_hour : '',
                'Days on Truck'             => (!is_null($offer->days)) ? $offer->days  : '',
                'Hour on Truck'             => (!is_null($offer->hours)) ? $offer->hours  : '',
                'Id'                        => (!is_null($offer->id)) ? $offer->id : '',
            );
            $x++;
        }

        $tableData                          = array();
        foreach($offersData as $offerName => $offerValues){
            $tableData['Details'][]         = $offerName;
            foreach ($offerValues as $rowName => $rowData){
                $tableData[$rowName][]      = $rowData;
            }
        }

        $view                               = view('companyAdmin.dispatchHub.ajaxview.loadComparisonView', compact('offersData', 'tableData', 'driverId'))->render();
        return response()->json(['success' => true, 'htmlView' => $view]);
    }

    public function getTripDetails($tripId){
        $data                   = [];
        $tripId                 = (isset($tripId) && $tripId!='') ? $tripId : '';
        if($tripId != ''){
            $getAPIKey          = getTrimbleApiKey(); //env('TRIMBLE_MAPS_API_KEY');
            if(!empty($getAPIKey)){
                $client         = new \GuzzleHttp\Client();
                $api_url        = 'https://tripmanagement.alk.com/api/trip?tripId='.$tripId;
                $api_response   = $client->request('GET', $api_url, [
                    'headers'           => [
                        'Accept'        => 'application/json',
                        'Authorization' => $getAPIKey,
                    ],
                ]);
                $appData        = json_decode($api_response->getBody());
                if(!empty($appData)){
                    return $appData;
                }
            }
        }
        return $data;
    }
}
