<?php
use Illuminate\Support\Str;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Session;

use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\ServerException;

use App\Mail\CompanyAdmin\LoadDroppedFromDropTrailer;
use App\Mail\CompanyAdmin\LoadOnHoldFromDropTrailer;

use App\Models\Company;
use App\Models\CompanyTerm;
use App\Models\CompanyHoliday;
use App\Models\CompanyCommodity;
use App\Models\CompanyLoadDocument;
use App\Models\CompanyChartOfAccount;
use App\Models\CompanyLoadRequirementsItem;
use App\Models\CompanyEmailWrapperTemplate;

use App\Models\CompanyTruckType;
use App\Models\CompanyDeductionType;

use App\Models\CompanyInsuranceType;
use App\Models\CompanyPmType;
use App\Models\CompanyPmServiceType;
use App\Models\CompanyPmWarrantyProvider;
use App\Models\CompanyEquipmentTitle;
use App\Models\CompanyDocumentTitle;

use App\Models\CompanyTrailerGroup;
use App\Models\CompanyTruckGroup;

use App\Models\CompanyDropLocation;
use App\Models\CompanyRecruitingSource;

use App\Models\CompanyDocumentProfile;
use App\Models\CompanyEquipmentProfile;
use App\Models\CompanyInsuranceProfile;
use App\Models\CompanyPmServiceProfile;
use App\Models\CompanyPmServiceWarrantyProfile;

use App\Models\CompanyAdmin;
use App\Models\CompanyAdminRole;
use App\Models\CompanyAdminModule;
use App\Models\CompanyAdminPermission;
use App\Models\CompanyAdminModulePermission;
use App\Models\CompanyAdminRoleModulePermission;

use App\Models\Announcement;
use App\Models\AnnouncementClearByUser;

use App\Models\State;
use App\Models\Timezone;

use App\Models\Team;

use App\Models\Driver;
use App\Models\DriverFinancial;
use App\Models\DriverInformation;
use App\Models\DriverComplianceSafety;
use App\Models\DriverHosViolation;
use App\Models\DriverInspectionLevel;
use App\Models\DriverHosViolationCode;
use App\Models\DriverDisciplinaryAction;
use App\Models\DriverCitationsViolationCode;

use App\Models\DriverOnboarding;
use App\Models\DriverOnboardingSetting;

use App\Models\Load;
use App\Models\LoadLog;
use App\Models\LoadFile;
use App\Models\LoadStop;
use App\Models\LoadDetail;
use App\Models\LoadDriver;

use App\Models\Broker;
use App\Models\BrokerContact;
use App\Models\BrokerRequiredDocumentProfile;

use App\Models\Customer;
use App\Models\Shipper;
use App\Models\Receiver;

use App\Models\Vendor;
use App\Models\Carrier;
use App\Models\Terminal;

use App\Models\Trailer;
use App\Models\TrailerType;
use App\Models\TrailerMaintenanceLog;

use App\Models\TrailerCompanyDocumentProfile;
use App\Models\TrailerCompanyEquipmentProfile;
use App\Models\TrailerCompanyInsuranceProfile;
use App\Models\TrailerCompanyPmServiceProfile;
use App\Models\TrailerCompanyPmServiceWarrantyProfile;

use App\Models\Truck;
use App\Models\TruckOwner;
use App\Models\TruckMaintenanceLog;

use App\Models\ChartOfAccount;
use App\Models\HazardousOption;
use App\Models\IntegrationSetting;

use App\Models\ManageType;
use App\Models\ManageTypeCategory;
use App\Models\ManageCompanyType;

use App\Models\Event;
use App\Models\EventType;

use App\Models\Leave;

use App\Models\Alert;

use App\Models\Notify;
use App\Models\Notification;
use App\Models\NotificationDaytime;

use App\Models\Gallery;
use App\Models\GalleryDocumentName;

use App\Models\FileUpload;

use App\Models\Maintenance;
use App\Models\MaintenanceCalendar;
use App\Models\MaintenanceTicketType;
use App\Models\MaintenanceAppointment;
use App\Models\MaintenanceAppointmentType;

use App\Models\ChatGeneralBoard;
use App\Models\ChatLoad;
use App\Models\ChatLoadParticipants;
use App\Models\ChatLoadInternal;
use App\Models\ChatLoadInternalParticipants;

use App\Models\SmsChatHistory; // Added at 10-04-2023

use App\Models\SmsLog;
use App\Events\MessageFailed;

use App\Models\EmailLog;

use App\Models\DefaultFuelSurcharge;
use App\Models\FuelSurchargeTemplate;

use App\Models\LoadBillingActivity;
use App\Models\LoadBillingPaymentAccount;

use App\Models\ReportDriver;

use App\Helpers\Common;

use Twilio\Rest\Client;
use Shuchkin\SimpleXLSX;
use Webklex\PDFMerger\Facades\PDFMergerFacade as PDFMerger;
use Carbon\Carbon;
use Carbon\CarbonPeriod;
use App\Models\LeftMenu;
use App\Models\LeftMenuByUser;
use App\Models\LoadNote;

include 'includeHelper.php';

if(! function_exists('getNextAutoID')){
    function getNextAutoID($tableName){
        $statement  = DB::select("show table status like '{$tableName}'");
        return $statement[0]->Auto_increment;
    }
}

if(! function_exists('generateStrUniqueId')){
    function generateStrUniqueId($table=null) {
        if(!is_null($table)){
            return getNextAutoID($table).\Str::random(4).substr(base_convert(sha1(uniqid(mt_rand())), 16, 36), 0, 12);
        }else{
            return \Str::random(4).substr(base_convert(sha1(uniqid(mt_rand())), 16, 36), 0, 12);
        }
    }
}

if(! function_exists('generateDummyId')){
    function generateDummyId($table,$field='id') {
        $number = mt_rand(1000000000, mt_getrandmax()); // better than rand()
        // call the same function if the barcode exists already
        if (checkDummyIdExists($table,$number,$field)) {
            return generateDummyId($table,$field);
        }
        // otherwise, it's valid and can be used
        return $number;
    }
}

if(! function_exists('checkDummyIdExists')){
    function checkDummyIdExists($table,$number,$field='id') {
        return DB::table($table)->where($field,$number)->exists();
    }
}

if(! function_exists('generateDummyLoadId')){
    function generateDummyLoadId() {
        $ref_no     = mt_rand(100, 999).sprintf("%02d", getNextAutoID("loads"));
        if (checkLoadIdExists($ref_no) > 0) {
            return generateDummyLoadId();
        }
        return $ref_no;
    }
}

if(! function_exists('checkLoadIdExists')){
    function checkLoadIdExists($ref_no) {
        return Load::where('id',$ref_no)->exists();
    }
}

if (! function_exists('encryptData')) {
    function encryptData($data){
        if($data){
            $key                = 'Compellabs'; // Replace with your encryption key
            $cipher             = 'AES-256-CBC';
            $ivLength           = openssl_cipher_iv_length($cipher);
            $iv                 = openssl_random_pseudo_bytes($ivLength);

            $encryptedData      = openssl_encrypt($data, $cipher, $key, OPENSSL_RAW_DATA, $iv);
            $encryptedData      = rtrim(strtr(base64_encode($iv . $encryptedData), '+/', '-_'), '=');
            return $encryptedData;
        }else{
            return '';
        }
    }
}

if (! function_exists('decryptData')) {
    function decryptData($encryptedData){
        if($encryptedData){
            $key                = 'Compellabs'; // Replace with your encryption key
            $cipher             = 'AES-256-CBC';
            $encryptedData      = base64_decode(str_pad(strtr($encryptedData, '-_', '+/'), strlen($encryptedData) % 4, '=', STR_PAD_RIGHT));
            $ivLength           = openssl_cipher_iv_length($cipher);
            $iv                 = substr($encryptedData, 0, $ivLength);
            $encrypted          = substr($encryptedData, $ivLength);

            $decryptedData      = openssl_decrypt($encrypted, $cipher, $key, OPENSSL_RAW_DATA, $iv);
            return $decryptedData;
        }else{
            return '';
        }
    }
}

if (! function_exists('base64Encode')) {
    function base64Encode($id){
        if($id){
            $ciphering          = "AES-128-CTR";
            // Use OpenSSl Encryption method
            $iv_length          = openssl_cipher_iv_length($ciphering);
            $options            = 0;
            // Non-NULL Initialization Vector for encryption
            $encryption_iv      = '1234567891011121';
            // Store the encryption key
            $encryption_key     = "Compellabs";
            $output             = openssl_encrypt($id, $ciphering,
            $encryption_key, $options, $encryption_iv);

            $base               = str_replace(array('+','/','='),array('-','_',''),$output);
            return Str::random(30).$base;
        }else{
            return '';
        }
    }
}

if (! function_exists('base64Decode')) {
    function base64Decode($string){
        if($string){
            $id                 = substr($string, 30);
            $base               = str_replace(array('-','_',''),array('+','/','='),$id);

            $ciphering          = "AES-128-CTR";
            // Use OpenSSl Encryption method
            $iv_length          = openssl_cipher_iv_length($ciphering);
            $options            = 0;
            // Non-NULL Initialization Vector for encryption
            $encryption_iv      = '1234567891011121';
            // Store the encryption key
            $encryption_key     = "Compellabs";
            $output             = openssl_decrypt($base, $ciphering,
            $encryption_key, $options, $encryption_iv);
            return $output;
        }else{
            return '';
        }
    }
}

if(! function_exists('formatNumberWithoutZeros')){
    function formatNumberWithoutZeros($number, $options = []) {
        $defaults               = [
            'removeZeros'       => true,
            'removeCommas'      => false,
        ];

        $options                = array_merge($defaults, $options);

        $formattedNumber        = number_format((float)str_replace(',', '', $number), 2, '.', ',');
        if ($options['removeZeros']) {
            $formattedNumber    = preg_replace('/(\.00)$/', '', $formattedNumber);
        }
        if ($options['removeCommas']) {
            $formattedNumber    = str_replace(',', '', $formattedNumber);
        }
        return $formattedNumber;
    }
}

if(! function_exists('removeNumberWithCommas')){
    function removeNumberWithCommas($number) {
        return str_replace(',', '', $number);
    }
}

if(! function_exists('stringDateToday')){
    function stringDateToday($string) {
        $reg    = "/\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}/";
        $date   = new \DateTime();
        preg_match($reg, $string, $matches, PREG_OFFSET_CAPTURE);
        if(isset($matches) & !empty($matches)){
            foreach ($matches as $key => $value) {
                $word       = isset($value[0])?convertTimeToUSERzone(['str' => $value[0], 'toFormat' => getTimeLayout(), 'fromFormat' => 'Y-m-d H:i:s'])." ".getTimeZoneId():'';
                $pos        = isset($value[1])?$value[1]:'';
                if($pos!=''){
                    $string = substr_replace($string, $word, $pos, 19);
                }
            }
        }
        return $string;
    }
}

if(! function_exists('dateTimeFormatter')){
    function dateTimeFormatter($str, $toFormat = 'Y-m-d H:i:s', $fromFormat = ''){
        if(empty($str)){
            return '';
        }

        if($fromFormat==''){
            $fromFormat = $toFormat;
        }

        $date           = Carbon::createFromFormat($fromFormat, $str);
        return $date->format($toFormat);
    }
}

if(! function_exists('groupByColumn')){
    function groupByColumnArray($array, $columnName) {
        $result         = [];
        foreach ($array as $item) {
            $key                = $item[$columnName];
            if (!isset($result[$key])) {
                $result[$key]   = [];
            }
            $result[$key][]     = $item;
        }

        return $result;
    }
}

if (! function_exists('getDiffInMinutes')) {
    function getDiffInMinutes($datetime){
        $to             = \Carbon\Carbon::createFromFormat('Y-m-d H:s:i', $datetime);
        $from           = Carbon::now()->toDateTimeString();
        $diff_in_hours  = $to->diffInMinutes($from);
        return $diff_in_hours;
    }
}

if (! function_exists('formatSeconds')) {
    function formatSeconds( $seconds ){
        $seconds        = $seconds / 1000;
        $hours          = 0;
        $milliseconds   = str_replace( "0.", '', $seconds - floor( $seconds ) );
        if ( $seconds > 3600 ){
            $hours      = floor( $seconds / 3600 );
        }
        $seconds        = $seconds % 3600;
        return str_pad( $hours, 2, '0', STR_PAD_LEFT ). gmdate( ':i', $seconds );
    }
}

if (! function_exists('getTimeFormat')) {
    function getTimeFormat(){
        $timeFormat             = 1;
        if (request()->is('admin') || request()->is('admin/*')) {
            $user               = auth()->guard('admin')->user();
            if(isset($user)){
                $timeFormat     = $user->time_format;
            }
        }else if (request()->is('company-admin') || request()->is('company-admin/*')) {
            if(!request()->is('company-admin/detention-request/broker/*')){
                $user           = auth()->guard('company-admin')->user();
                if(isset($user)){
                    $timeFormat = $user->time_format;
                }
            }
        }else if (request()->is('driver-admin') || request()->is('driver-admin/*')) {
            $user               = auth()->guard('driver-admin')->user();
            if(isset($user)){
                $timeFormat     = $user->time_format;
            }
        }
        return $timeFormat;
    }
}

if (! function_exists('getTimeLayout')) {
    function getTimeLayout($datetime='1'){
        $timeFormat             = getTimeFormat();
        if ($timeFormat == 12) {
            $dateFormat         = ($datetime=='1') ? config('settings.date_time_format.12_hour','m/d/Y h:i A') : config('settings.time_format.12_hour','h:i A');
        } else {
            $dateFormat         = ($datetime=='1') ? config('settings.date_time_format.24_hour','m/d/Y H:i') : config('settings.time_format.24_hour','H:i');
        }
        return $dateFormat;
    }
}

if (! function_exists('getGeoLocation')) {
    function getGeoLocation($ip =''){
        // Define a list of local IP addresses
        $localIps       = ['127.0.0.1', '::1', 'localhost'];
        // Check if the current IP address is in the list of local IPs
        if (in_array(Request::server('REMOTE_ADDR'), $localIps)) {
            // code to run on local server
            $user_ip    = '43.241.146.108';
        } else {
            // code to run on live server
            $user_ip    = $ip ? $ip: \Request::ip();
        }
        return unserialize(@file_get_contents("http://www.geoplugin.net/php.gp?ip=$user_ip"));
    }
}

if(! function_exists('getGeoTimeZone')){
    function getGeoTimeZone($type=''){
        $tzlist         = \DateTimeZone::listIdentifiers(\DateTimeZone::ALL);
        if($type=='admin'){
            $timezone   = Cache::remember('admin_geoplugin_timezone', 300, function () use ($tzlist){
                $getGeoLocation = getGeoLocation();
                $timezone       = $getGeoLocation['geoplugin_timezone'] ?? '';
                if($timezone!=='' && in_array($timezone, $tzlist)){
                    saveTimeZone($timezone);
                }
                return $timezone;
            });
        }elseif ($type=='company_admin') {
            $timezone   = Cache::remember('company_admin_geoplugin_timezone', 300, function () use ($tzlist){
                $getGeoLocation = getGeoLocation();
                $timezone       = $getGeoLocation['geoplugin_timezone'] ?? '';
                if($timezone!=='' && in_array($timezone, $tzlist)){
                    saveTimeZone($timezone);
                }
                return $timezone;
            });
        }elseif ($type=='driver_admin') {
            $timezone   = Cache::remember('driver_admin_geoplugin_timezone', 300, function () use ($tzlist){
                $getGeoLocation = getGeoLocation();
                $timezone       = $getGeoLocation['geoplugin_timezone'] ?? '';
                if($timezone!=='' && in_array($timezone, $tzlist)){
                    saveTimeZone($timezone);
                }
                return $timezone;
            });
        }else{
            $getGeoLocation = getGeoLocation();
            if(isset($getGeoLocation['geoplugin_timezone']) && $getGeoLocation['geoplugin_timezone']!=''){
                $timezone   = $getGeoLocation['geoplugin_timezone'];
                if($timezone!=='' && in_array($timezone, $tzlist)){
                    saveTimeZone($timezone);
                }
            }
        }
        if(!isset($timezone) || $timezone==''){
            $timezone       = config('app.timezone','UTC');
        }
        return $timezone;
    }
}

if(! function_exists('getTimeZone')){
    function getTimeZone(){
        $timezone                               = config('app.timezone','UTC');
        if (request()->is('admin') || request()->is('admin/*')) {
            $user                               = auth()->guard('admin')->user();
            if(isset($user)){
                if($user->timezone_type==0){
                    if(Cache::has('admin_geoplugin_timezone')){
                        $geoplugin_timezone     = Cache::get('admin_geoplugin_timezone');
                        if($geoplugin_timezone!=''){
                            $timezone           = $geoplugin_timezone;
                        }else{
                            $timezone           = getGeoTimeZone('admin');
                        }
                    }else{
                        $timezone               = getGeoTimeZone('admin');
                    }
                }else{
                    if($user->getTimezone){
                        $timezone               = $user->getTimezone->name;
                    }else{
                        $timezone               = getGeoTimeZone('admin');
                    }
                }
            }else{
                $timezone                       = getGeoTimeZone('admin');
            }
        }else if (request()->is('company-admin') || request()->is('company-admin/*')) {
            if(request()->is('company-admin/detention-request/broker/*')){
                $timezone                       = getGeoTimeZone();
            }else{
                $user                           = auth()->guard('company-admin')->user();
                if(isset($user)){
                    if($user->timezone_type==0){
                        if(Cache::has('company_admin_geoplugin_timezone')){
                            $geoplugin_timezone = Cache::get('company_admin_geoplugin_timezone');
                            if($geoplugin_timezone!=''){
                                $timezone       = $geoplugin_timezone;
                            }else{
                                $timezone       = getGeoTimeZone('company_admin');
                            }
                        }else{
                            $timezone           = getGeoTimeZone('company_admin');
                        }
                    }else{
                        if($user->getTimezone){
                            $timezone           = $user->getTimezone->name;
                        }else{
                            $timezone           = getGeoTimeZone('company_admin');
                        }
                    }
                }else{
                    $timezone                   = getGeoTimeZone('company_admin');
                }
            }
        }else if (request()->is('driver-admin') || request()->is('driver-admin/*')) {
            $user                               = auth()->guard('driver-admin')->user();
            if(isset($user)){
                if($user->timezone_type==0){
                    if(Cache::has('driver_admin_geoplugin_timezone')){
                        $geoplugin_timezone     = Cache::get('driver_admin_geoplugin_timezone');
                        if($geoplugin_timezone!=''){
                            $timezone           = $geoplugin_timezone;
                        }else{
                            $timezone           = getGeoTimeZone('driver_admin');
                        }
                    }else{
                        $timezone               = getGeoTimeZone('driver_admin');
                    }
                }else{
                    if($user->getTimezone){
                        $timezone               = $user->getTimezone->name;
                    }else{
                        $timezone               = getGeoTimeZone('driver_admin');
                    }
                }
            }else{
                $timezone                       = getGeoTimeZone('driver_admin');
            }
        }
        return $timezone;
    }
}

if(! function_exists('getTimeZoneId')){
    function getTimeZoneId($auto=0){
        if($auto==0)
            $timezone       = getTimeZone();
        else
            $timezone       = getGeoTimeZone();
        return Carbon::now()->timezone($timezone)->format('T');
    }
}

if(! function_exists('getTimeZoneOffset')){
    function getTimeZoneOffset($timezoneName=''){
        $timezone       = ($timezoneName!='') ?  $timezoneName : getTimeZone();

        $offset         = Carbon::createFromTimestamp(time(), $timezone)->getOffsetString();
        return $offset;
    }
}

if(! function_exists('saveTimeZone')){
    function saveTimeZone($timezone){
        $offset         = Carbon::createFromTimestamp(0, $timezone)->getOffsetString();
        $azones['offset']                   = $offset;
        $azones['diff_from_gtm']            = 'UTC/GMT '.$offset;
        Timezone::updateOrCreate(['name'=> $timezone], $azones);
        Cache::forget('timezones');
    }
}

if(! function_exists('convertTimeToUSERzone')){
    function convertTimeToUSERzone($arry=[]){
        if(!empty($arry) && (isset($arry['str']) && $arry['str']!='')){
            if(!(isset($arry['fromFormat']) && $arry['fromFormat']!='')){
                $arry['fromFormat'] = $arry['toFormat'];
            }

            $value              = '';
            if(isset($arry['auto']) && $arry['auto']==1){
                $timezone       = getGeoTimeZone();
            }else{
                if(isset($arry['abbr']) && $arry['abbr']!=''){
                    $timezone   = timezone_name_from_abbr($arry['abbr']);
                }else{
                    $timezone   = $arry['userTimezone'] ?? getTimeZone();
                }
            }

            $date               = Carbon::createFromFormat($arry['fromFormat'], $arry['str']);
            $date->setTimezone($timezone);
            $value              = $date->format($arry['toFormat']);
            return $value;
        }else{
            return '';
        }
    }
}

if(! function_exists('convertTimeToUTCzone')){
    function convertTimeToUTCzone($arry=[]){
        if(!empty($arry) && (isset($arry['str']) && $arry['str']!='')){
            if(!(isset($arry['fromFormat']) && $arry['fromFormat']!='')){
                $arry['fromFormat'] = $arry['toFormat'];
            }

            $value              = '';
            if(isset($arry['auto']) && $arry['auto']==1){
                $timezone       = getGeoTimeZone();
            }else{
                if(isset($arry['abbr']) && $arry['abbr']!=''){
                    $timezone   = timezone_name_from_abbr($arry['abbr']);
                }else{
                    $timezone   = $arry['userTimezone'] ?? getTimeZone();
                }
            }

            $date               = Carbon::createFromFormat($arry['fromFormat'], $arry['str'], $timezone);
            $date->setTimezone('UTC');
            $value              = $date->format($arry['toFormat']);
            return $value;
        }else{
            return '';
        }
    }
}

if (! function_exists('getTrimbleLocation')) {
    function getTrimbleLocation($array){
        $response           = array();
        $param              = array();
        foreach($array as $k => $v){
            $param[]        = $k . '=' . urlencode($v);
        }
        $param              = implode('&', $param);
        $getUrl             = "https://pcmiler.alk.com/apis/rest/v1.0/Service.svc/locations?".$param;

        $ch                 = curl_init();
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_URL, $getUrl);
        curl_setopt($ch, CURLOPT_TIMEOUT, 80);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Accept        : application/json',
            'Content-Type  : application/json',
            'Authorization : '.getTrimbleApiKey(),
        ));
        $response           = curl_exec($ch);
        if(curl_error($ch)){
            $error_msg      =  'Request Error:' . curl_error($ch);
        }
        curl_close($ch);
        if (!isset($error_msg)) {
            $curlData       = json_decode($response,true);
            if(isset($curlData[0] )&& !empty($curlData[0])){
                $response   = $curlData[0];
            }
        }
        return $response;
    }
}

function getAppTemplates(){
    $path           = resource_path().'/views/companyAdmin/integrationSettings';
    $filesInFolder  = File::files($path);
    $files          = [];
    foreach($filesInFolder as $path) {
        $file       = pathinfo($path);
        if($file['filename'] != 'index.blade'){
            $files[]=  $file['filename'] ;
        }
    }
    return $files;
}

function getAppIsActive($app_center_id){
    $company_id     = auth()->guard('company-admin')->user()->company_id;
    $integration    = IntegrationSetting::where('app_center_id', $app_center_id)->where('company_id', $company_id)->count();
    if($integration > 0){
        return true;
    }
    return false;
}

function getDriverMatchSamsara($licenseNumber){
    if($licenseNumber == 'Un458353'){
        dd($integration);
    }
    $company_id     = auth()->guard('company-admin')->user()->company_id;
    $integration    = Driver::where('license_number', base64_encode($licenseNumber))->where('company_id', $company_id)->first();
    if(!empty($integration->id)){
        return $integration->id;
    }
    return 0;
}

function getVehicleMatchSamsara($vin){
    $company_id     = auth()->guard('company-admin')->user()->company_id;
    $integration    = Truck::where('vin_number', $vin)->where('company_id', $company_id)->first();
    if(!empty($integration->id)){
        return $integration->id;
    }
    return 0;
}

function getTrailerMatchSamsara($trailer_id){
    $company_id     = auth()->guard('company-admin')->user()->company_id;
    $integration    = Trailer::where('trailer_id', $trailer_id)->where('company_id', $company_id)->first();
    if(!empty($integration->id)){
        return $integration->id;
    }
    return 0;
}

function getCompanyList(){
    return Company::get();
}

function getCompanyNameById(){
    return Company::pluck('name', 'id')->toArray();
}

function getDashboardLink(){
    if (Auth::guard('company-admin')->check() && request()->route()->getPrefix() == config('settings.route_prefix.companyadmin')) {
        $route      = route('company.admin.home');
    } else {
        $route      = "#";
    }
    return $route;
}

function hasPermission($moduleId, $permissionId, $role_id = null){
    if (Auth::guard('company-admin')->user()->role_id == 0 && Auth::guard('company-admin')->check() && request()->route()->getPrefix() == config('settings.route_prefix.companyadmin')) {
        $accessible     = true;
    } else {
        if(empty($role_id)){
            $role_id    = Auth::guard('company-admin')->user()->role_id;
        }
        $accessible     = CompanyAdminRoleModulePermission::whereRoleId($role_id)->whereHas('module_permission', function ($q) use ($moduleId, $permissionId) {
            $q->whereModuleId($moduleId)->wherePermissionId($permissionId);
        })->first();
    }
    return $accessible;
}

function hasDispatcherRevenueTracker(){
    $arry                       = [];
    $user                       = Auth::guard('company-admin')->user();
    if($user->type == 2){
        $hasPermission          = hasPermission(config('settings.company_admin_modules.dispatcher_revenue_tracker'),config('settings.permissions.view'));
        if($hasPermission){
            $success            = false;
            $view_own_team      = $hasPermission->module_permission_id=='142'?true:false;
            if($view_own_team==true && is_null($user->team)){
                $view_own_team  = false;
            }
            $view_all_teams     = $hasPermission->module_permission_id=='143'?true:false;
            if($view_own_team==true || $view_all_teams==true){
                $success        = true;
            }

            $permissions[]      = [
                'success'       => $view_own_team,
                'name'          => 'View own team',
                'slug'          => 'view_own_team',
                'href'          => route('company.admin.revenue.tracker.team.id', base64Encode($user->team_id))
            ];
            $permissions[]      = [
                'success'       => $view_all_teams,
                'name'          => 'View all teams',
                'slug'          => 'view_all_teams',
                'href'          => route('company.admin.revenue.trackers.index')
            ];
            $arry = [
                'success'       => $success,
                'type'          => $user->type,
                'permissions'   => $permissions
            ];
        }else{
            $arry               = [
                'success'       => false,
                'type'          => $user->type
            ];
        }
    }else{
        $arry                   = [
            'success'           => true,
            'type'              => $user->type
        ];
    }
    return $arry;
}

function getUserCompnayId(){
    $company_id             = null;
    if ((request()->is('company-admin') || request()->is('company-admin/*')) && auth()->guard('company-admin')->check()) {
        $company_id         = auth()->guard('company-admin')->user()->company_id;
    }else if ((request()->is('driver-admin') || request()->is('driver-admin/*')) && auth()->guard('driver-admin')->check()) {
        $company_id         = auth()->guard('driver-admin')->user()->company_id;
    }
    return $company_id;
}

function notification($type, $message){
    Session::put($type, $message);
}

function isCompnayNotification(){
    return auth()->guard('company-admin')->user()->notification;
}

function getCompanyLogo(){
    $logo                   = '';
    if (auth()->guard('company-admin')->check()) {
        $setting            = auth()->guard('company-admin')->user()->getCompany;

        if($setting){
            $logo           = asset('storage/companyAdmin/company_logo').'/'.$setting->logo;
        }
    }
    return $logo;
}

function getCampanyEmailLogo(){
    $logo                   = '';
    if (auth()->guard('company-admin')->check()) {
        $setting            = auth()->guard('company-admin')->user()->getCompany;
        if($setting){
            $logo           = asset('storage/companyAdmin/company_email') . '/' . $setting->email_logo;
        }
    }else{
        $logo = asset('theme/img/logo-new.png'); //Default logo
    }
    return $logo;
}

function getCompanyAdminByRole($moduleId, $permissionId){
    $getCompanyAdminRoles   = CompanyAdminRole::where('company_id', getUserCompnayId())->get();
    $role_id                = [];
    foreach($getCompanyAdminRoles as $roles){
        $accessible         = CompanyAdminRoleModulePermission::whereRoleId($roles->id)->whereHas('module_permission', function ($q) use ($moduleId, $permissionId) {
            $q->whereModuleId($moduleId)->wherePermissionId($permissionId);
        })->first();
        if($accessible){
            $role_id[]      = $roles->id;
        }
    }
    $getCompanyAdmin        = CompanyAdmin::whereIn('role_id',$role_id)->get();
    return $getCompanyAdmin;
}

function getCompanyTwilioData($company_id = null) {
    $data                                   = [];
    if(!is_null($company_id)){
        $getCompany                         = Company::whereId($company_id)->first();
        if(!is_null($getCompany)){
            $data                           = [
                'twilio_sid'                => $getCompany->twilio_sid,
                'twilio_auth_token'         => $getCompany->twilio_auth_token,
                'twilio_active_number'      => $getCompany->twilio_active_number,
                'active'                    => isset($getCompany->twilio_sid) && isset($getCompany->twilio_auth_token) && isset($getCompany->twilio_active_number) ? 'yes' : 'no',
                'sms_limit_type'            => $getCompany->sms_limit_type,
                'sms_limit'                 => $getCompany->sms_limit,
                'sms_last_added'            => $getCompany->sms_last_added,
            ];
        }
    }else{
        if(auth()->guard('company-admin')->check()){
            $user                           = auth()->guard('company-admin')->user();
            if(!is_null($user) && isset($user->getCompany)){
                $data                       = [
                    'twilio_sid'            => $user->getCompany->twilio_sid,
                    'twilio_auth_token'     => $user->getCompany->twilio_auth_token,
                    'twilio_active_number'  => $user->getCompany->twilio_active_number,
                    'active'                => isset($user->getCompany->twilio_sid) && isset($user->getCompany->twilio_auth_token) && isset($user->getCompany->twilio_active_number) ? 'yes' : 'no',
                    'sms_limit_type'        => $user->getCompany->sms_limit_type,
                    'sms_limit'             => $user->getCompany->sms_limit,
                    'sms_last_added'        => $user->getCompany->sms_last_added,
                ];
            }
        } else if(auth()->guard('driver-admin')->check()){
            $user                           = auth()->guard('driver-admin')->user();
            if(!is_null($user) && isset($user->company)){
                $data                       = [
                    'twilio_sid'            => $user->company->twilio_sid,
                    'twilio_auth_token'     => $user->company->twilio_auth_token,
                    'twilio_active_number'  => $user->company->twilio_active_number,
                    'active'                => isset($user->company->twilio_sid) && isset($user->company->twilio_auth_token) && isset($user->company->twilio_active_number) ? 'yes' : 'no',
                    'sms_limit_type'        => $user->company->sms_limit_type,
                    'sms_limit'             => $user->company->sms_limit,
                    'sms_last_added'        => $user->company->sms_last_added,
                ];
            }
        }
    }
    return $data;
}

function sendSMS($to, $message, $getCompanyTwilioData = [], $userDetails = []){
    try {

        $company_id = '';
        $user_id = '';
        $to_name = '';

        $getCompanyAdmin = CompanyAdmin::where('home_phone', $to)->orWhere('cell_phone', $to)->first();
        if(isset($getCompanyAdmin->full_name) && !empty($getCompanyAdmin->full_name)){
            $company_id = $getCompanyAdmin->company_id;
            $user_id = $getCompanyAdmin->id;
            $to_name = $getCompanyAdmin->full_name;
        }else{
            $getDriver = Driver::where('home_phone', $to)->orWhere('cell_phone', $to)->first();
            if(isset($getDriver->full_name) && !empty($getDriver->full_name)){
                $company_id = $getDriver->company_id;
                $user_id = $getDriver->id;
                $to_name = $getDriver->full_name;
            }

        }

        //dd($company_id, $user_id, $to_name);

        $userHasSmsLimit = false;
        if(isset($getCompanyTwilioData['sms_limit_type']) && $getCompanyTwilioData['sms_limit_type'] == '0'){
            // User has unlimited SMS
            $userHasSmsLimit = true;
        }else if(isset($getCompanyTwilioData['sms_limit_type']) && $getCompanyTwilioData['sms_limit_type'] == '1' && $getCompanyTwilioData['sms_limit'] > 0){
            $userHasSmsLimit = true;
        }

        if($userHasSmsLimit == false) {
            SmsLog::create([
                'company_id' => $userDetails['company_id'],
                'user_id' => $user_id,
                'from' => $from,
                'to' => $to,
                'to_name' => $to_name,
                'content' => 'Send SMS functionality is disabled due to plan is list is expired.',
                'sent_date' => Carbon::now()->format('Y-m-d H:i:s'),
                'status' => '1',
            ]);

            return false;
        }

        if(!empty($getCompanyTwilioData) && isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
            $sid        = $getCompanyTwilioData['twilio_sid'];
            $token      = $getCompanyTwilioData['twilio_auth_token'];
            $from       = $getCompanyTwilioData['twilio_active_number'];
        }else{
            if(auth()->guard('admin')->check()){
                $sid        = env("TWILIO_SID");
                $token      = env("TWILIO_AUTH_TOKEN");
                $from       = env("TWILIO_NUMBER");
            }
        }

        $twilio         = new Client($sid, $token);
        $Checktwilio        = $twilio->messages->create($to, [
            "body"      => $message,
            "from"      => $from
        ]);


        if($Checktwilio){
            if(!empty($company_id)){

                $company = Company::find($company_id);
                $company->sms_limit = $company->sms_limit - 1;
                $company->save();

                SmsLog::create([
                    'company_id' => $userDetails['company_id'],
                    'user_id' => $user_id,
                    'from' => $from,
                    'to' => $to,
                    'to_name' => $to_name,
                    'content' => $message,
                    'sent_date' => Carbon::now()->format('Y-m-d H:i:s'),
                    'status' => '1',
                ]);
            }
        }

    } catch (\Exception $e) {
        // Handle any other exception
        if(!empty($userDetails)){
            SmsLog::create([
                'company_id' => $userDetails['company_id'],
                'user_id' => $userDetails['user_id'],
                'from' => $from,
                'to' => $to,
                'content' => $e->getMessage(),
                'sent_date' => Carbon::now()->format('Y-m-d H:i:s'),
                'status' => '0',
            ]);
        }
        // Log the error message
        \Log::error('An error occurred while sending the email: ' . $e->getMessage());

        // Optionally, you can re-throw the exception if desired
        // throw $e;
    }
}

function getOnHoldDispatcher($horus){
    $allData = [];

    $getData = LoadDetail::Join('loads', function ($join) {
            $join->on('load_details.load_id', '=', 'loads.id');
        })
        ->Join('company_admins', function ($join) use ($horus) {
            $join->on('loads.company_id', '=', 'company_admins.company_id');
            $join->where('company_admins.type', '=', 0);
            $join->where('company_admins.notification', '=', 1);
            $join->where('company_admins.notifictiono_hours', '=', $horus);
        })
        ->where(function($q) {
            $q->where('load_details.load_status', 7)->orWhere('load_details.is_on_hold', 1);
        })
        ->where(function($q) {
            $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
        })
        ->where('load_details.booked_by', '!=', 0)
        ->whereNotNull('load_details.load_on_hold_datetime')
        ->groupBy('load_details.booked_by')
        ->select('load_details.driver_id', 'load_details.booked_by')
        ->whereNull('load_details.deleted_at')
        ->get()->toArray();

    $getData2 = LoadDetail::Join('loads', function ($join) {
            $join->on('load_details.load_id', '=', 'loads.id');
        })
        ->Join('company_admins', function ($join) use ($horus) {
            $join->on('loads.company_id', '=', 'company_admins.company_id');
            $join->where('company_admins.type', '=', 0);
            $join->where('company_admins.notification', '=', 1);
            $join->where('company_admins.notifictiono_hours', '=', $horus);
        })
        ->where(function($q) {
            $q->where('load_details.load_status', 7)->orWhere('load_details.is_on_hold', 1);
        })
        ->where(function($q) {
            $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
        })
        ->where('load_details.booked_by', '=', 0)
        ->whereNotNull('load_details.load_on_hold_datetime')
        ->groupBy('load_details.driver_id')
        ->select('load_details.driver_id', 'load_details.booked_by')
        ->whereNull('load_details.deleted_at')
        ->get()->toArray();

    $getDispatc                 = array_merge($getData, $getData2);

    $getNotificationReason      = getNotificationReason();

    if (count($getDispatc) > 0) {
        $all                    = 0;
        foreach ($getDispatc as $dispatc) {
            $mailData           = [];
            if ($dispatc['booked_by'] == 0) {
                $getDispatcher  = Driver::find($dispatc['driver_id']);
            } else {
                $getDispatcher  = CompanyAdmin::findOrFail($dispatc['booked_by']);
            }

            $getDriver          = Driver::find($dispatc['driver_id']);
            if ($dispatc['booked_by'] == 0) {
                $getData        = LoadDetail::where(function($q) {
                        $q->where('load_status', 7)->orWhere('is_on_hold', 1);
                    })
                    ->where(function($q) {
                        $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
                    })
                    ->whereNull('deleted_at')
                    ->where('driver_id', '=', $dispatc['driver_id'])
                    ->where('booked_by', '=', 0)
                    ->whereNotNull('load_on_hold_datetime')
                    ->select('id', 'load_number', 'reason_on_hold', 'notification_reason', 'load_on_hold_datetime')
                    ->get();
            } else {
                $getData        = LoadDetail::where(function($q) {
                        $q->where('load_status', 7)->orWhere('is_on_hold', 1);
                    })
                    ->where(function($q) {
                        $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
                    })
                    ->whereNull('deleted_at')
                    ->where('booked_by', '=', $dispatc['booked_by'])
                    ->where('booked_by', '!=', 0)
                    ->whereNotNull('load_on_hold_datetime')
                    ->select('id', 'load_number', 'reason_on_hold', 'notification_reason', 'load_on_hold_datetime')
                    ->get();
            }

            $mailData['company_id'] = $getDispatcher->company_id;
            $mailData['email']      = $getDispatcher->email;
            $mailData['name']       = $getDispatcher->first_name . ' ' . $getDispatcher->last_name;


            if (count($getData)) {
                $senddata       = [];
                $m              = 0;
                foreach ($getData as $loadData) {
                    $date       = Carbon::parse($loadData->load_on_hold_datetime);
                    $now        = Carbon::now();
                    $diff_hours = $date->diffForHumans($now);

                    $senddata[$m]['load_number']            = $loadData->load_number;
                    $senddata[$m]['driver_name']            = $getDriver->first_name . ' ' . $getDriver->last_name;
                    $senddata[$m]['notification_reason']    = $getNotificationReason[$loadData->notification_reason];
                    $senddata[$m]['reason_on_hold']         = $loadData->reason_on_hold;
                    $senddata[$m]['diff_hours']             = $diff_hours;

                    $m++;
                }

                $mailData['senddata'] = $senddata;
            }
            $allData[$all] = $mailData;
            $all++;
        }
    }
    return $allData;
}

function getStateList(){
    return State::orderBy('name', 'ASC')->pluck('name', 'id')->all();
}

function getLoadNumberByDriverId(){
    return LoadDetail::whereNotNull('load_number')->pluck('load_number', 'load_number')->all();
}

function getDriverInspectionLevel(){
    return DriverInspectionLevel::pluck('name', 'id')->all();
}

function getCompanyLoadDocument(){
    return CompanyLoadDocument::isCompany()->get();
}

function getDriverHosViolationCode(){
    return DriverHosViolationCode::get();
}

function getDriverCitationsViolationCode(){
    return DriverCitationsViolationCode::get();
}

function getDriverDisciplinaryAction(){
    return DriverDisciplinaryAction::pluck('name', 'id')->all();
}

function getTeamList(){
    return Team::company()->pluck('name', 'id')->all();
}

function getRoleManagerList(){
    return CompanyAdminRole::company()->where('guard_name', 'manager')->pluck('name', 'id')->all();
}

function getRoleDispatcherList(){
    return CompanyAdminRole::company()->where('guard_name', 'dispatcher')->pluck('name', 'id')->all();
}

function getRoledriverList(){
    return CompanyAdminRole::company()->where('guard_name', 'driver')->pluck('name', 'id')->all();
}

function getBrokerList(){
    return Broker::isCompany()->pluck('name', 'id')->all();
}

function getShipperList(){
    return Shipper::isCompany()->pluck('name', 'id')->all();
}

function getCustomerList(){
    return Customer::isCompany()->pluck('name', 'id')->all();
}

function getReceiverList(){
    return Receiver::isCompany()->pluck('name', 'id')->all();
}

function getBrokerListByCompany($companyId){
    return Carrier::where('company_id', $companyId)->pluck('name', 'id')->all();
}

function getEquipmentList(){
    $trailers                   = getTrailerNumberListByCompany();
    $trucks                     = Truck::isCompany()->pluck('unit', 'id')->all();

    $merge                      = array_merge($trailers, $trucks);

    $equipmentList              = [];
    foreach ($merge as $value) {
        $equipmentList[$value]  = $value;
    }

    return $equipmentList;
}

function getTruckNumberList(){
    return Truck::isCompany()->pluck('unit', 'unit')->all();
}

function getTruckByUnit($unit){
    $id             = '';
    $truck          = Truck::where('unit',$unit)->first();
    if(isset($truck)){
        $id         = $truck->id;
    }
    return $id;
}

function getTruckList(){
    return Truck::isCompany()->pluck('unit', 'id')->all();
}

function getVendorList(){
    return Vendor::isCompany()->pluck('name', 'id')->all();
}

function getMaintenanceTicketTypeList(){
    return MaintenanceTicketType::isCompany()->pluck('name', 'id')->all();
}

function getMaintenanceAppointmentTypeList(){
    return MaintenanceAppointmentType::isCompany()->pluck('name', 'id')->all();
}

function getTrailerList(){
    return Trailer::isCompany()->pluck('trailer_id', 'id')->all();
}

function getTrailerNumberListByCompany(){
    $loadTrailersNumber                 = Load::select("load_details.trailer_number")
        ->company()
        ->join("load_details", "load_details.load_id", "=", "loads.id")
        ->pluck('trailer_number', 'trailer_number')
        ->all();

    $trailersNumber                     = Trailer::isCompany()
        ->pluck('trailer_id', 'trailer_id')
        ->all();

    $merge                              = array_merge($trailersNumber, $loadTrailersNumber);

    $trailersNumberList                 = [];
    foreach ($merge as $value) {
        if(!is_null($value))
            $trailersNumberList[$value] = $value;
    }

    return $trailersNumberList;
}

function getLastDriverTrailerID($driver_id){
    $id             = '';
    $truck          = LoadDetail::where('driver_id', $driver_id)->first();
    if(isset($truck) && isset($truck->trailer_number)){
        $trailer    = Trailer::where('trailer_id', $truck->trailer_number)->first();
        if(isset($trailer)){
            $id     = $trailer->id;
        }
    }
    return $id;
}

function getLoadAssignTrailers($data = []){
    $getUserCompnayId                           = getUserCompnayId();

    $query                                      = LoadDetail::selectRaw('load_details.id,
            load_details.load_id,
            load_details.load_number,
            load_details.driver_id,
            load_details.truck_number,
            IFNULL(CONCAT(load_details.trailer_number, ",", GROUP_CONCAT(load_trailers.trailer_number SEPARATOR ",")), GROUP_CONCAT(load_trailers.trailer_number SEPARATOR ",")) as trailer_numbers,
            load_details.delivery_address_one,
            load_details.delivery_city,
            load_details.delivery_state,
            load_details.delivery_zip,
            load_details.load_type,
            load_details.load_status,
            load_details.is_on_hold,
            load_details.is_disable')
        ->leftJoin('load_trailers', 'load_details.load_id', '=', 'load_trailers.load_id');
        if(isset($data['id'])){
            $query->where('load_details.id', '!=', $data['id']);
        }
        $query->whereHas('loads', function ($q) use ($getUserCompnayId){
            $q->where('company_id', $getUserCompnayId);
        })
        ->where('load_details.is_disable', 0)
        ->where( function($q) {
            $q->whereIn('load_details.load_status', [4])
                ->where(function($q) {
                    $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
                });
        })
        ->havingRaw('trailer_numbers IS NOT NULL');
        if(isset($data['trailer_number'])){
            $query->havingRaw('FIND_IN_SET(?, trailer_numbers) > 0', [$data['trailer_number']]);
        }
        $query->groupBy('load_details.id');
    $results                                    = $query->get();

    $data                                       = [];
    if($results->isNotEmpty()){
        foreach ($results as $key => $trailer) {
            $trailer_numbers                    = isset($trailer->trailer_numbers) ? explode(',', $trailer->trailer_numbers) : [];
            if(!empty($trailer_numbers)){
                foreach ($trailer_numbers as $key => $val) {
                    $text                       = $val.' - Assigned - ';
                    if(in_array($trailer->load_type, [1, 2, 3, 4, 5])){ // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 4 = NO DISPATCH, 5 = TIME OFF
                        $text                  .= getLoadTypeList()[$trailer->load_type];
                    }else{
                        $text                   .= $trailer->load_number;
                    }
                    $address                    = [];
                    if(isset($trailer->delivery_address_one)){
                        array_push($address, $trailer->delivery_address_one);
                    }
                    if(isset($trailer->delivery_city)){
                        array_push($address, $trailer->delivery_city);
                    }
                    if(isset($trailer->delivery_state)){
                        array_push($address, $trailer->delivery_state);
                    }
                    if(isset($trailer->delivery_zip)){
                        array_push($address, $trailer->delivery_zip);
                    }
                    if(!empty($address)){
                        $text                   .= ' '.implode(', ', $address);
                    }
                    $data[$val]                 = $text;
                }
            }
        }
    }
    return $data;
}

function getLoadAvailableTrailers($load_assign_trailers = []){
    $getUserCompnayId                           = getUserCompnayId();

    $query                                      = Trailer::where('company_id', $getUserCompnayId)
        ->whereNull('current_assigned_truck');
    if(!empty($load_assign_trailers)){
        $query->whereNotIn('trailer_id', array_keys($load_assign_trailers));
    }
    $results                                    = $query->get();
    $data                                       = [];
    if($results->isNotEmpty()){
        foreach ($results as $key => $trailer) {
            $val                                = $trailer->trailer_id;
            $text                               = $val.' - Available';
            $address                            = [];
            if(isset($trailer->last_location_address)){
                array_push($address, $trailer->last_location_address);
            }
            if(isset($trailer->last_location_city)){
                array_push($address, $trailer->last_location_city);
            }
            if(isset($trailer->last_location_state)){
                array_push($address, $trailer->last_location_state);
            }
            if(isset($trailer->last_location_zip)){
                array_push($address, $trailer->last_location_zip);
            }
            if(!empty($address)){
                $text                           .= ' - '.implode(', ', $address);
            }
            $data[$val]                         = $text;
        }
    }
    return $data;
}

function getLoadPickUpTrailers($load_assign_trailers = [], $load_available_trailers = []){
    $getUserCompnayId                           = getUserCompnayId();

    $query                                      = Trailer::where('company_id', $getUserCompnayId);
    $trailers                                   = $query->get();
    $data                                       = [];
    if($trailers->isNotEmpty()){
        foreach ($trailers as $key => $trailer) {
            if(isset($load_assign_trailers[$trailer->trailer_id])){
                $data[$trailer->trailer_id]     = $load_assign_trailers[$trailer->trailer_id];
            } elseif (isset($load_available_trailers[$trailer->trailer_id])) {
                $data[$trailer->trailer_id]     = $load_available_trailers[$trailer->trailer_id];
            } else{
                $val                            = $trailer->trailer_id;
                $text                           = $val.' - Assigned';
                if(isset($trailer->truck)){
                    $text                       .= ' - '.$trailer->truck->unit;
                    $address                    = [];
                    if(isset($trailer->truck->last_location_address)){
                        array_push($address, $trailer->truck->last_location_address);
                    }
                    if(isset($trailer->truck->last_location_city)){
                        array_push($address, $trailer->truck->last_location_city);
                    }
                    if(isset($trailer->truck->last_location_state)){
                        array_push($address, $trailer->truck->last_location_state);
                    }
                    if(isset($trailer->truck->last_location_zip)){
                        array_push($address, $trailer->truck->last_location_zip);
                    }
                    if(!empty($address)){
                        $text                   .= ' - '.implode(', ', $address);
                    }
                }else{
                    $address                    = [];
                    if(isset($trailer->last_location_address)){
                        array_push($address, $trailer->last_location_address);
                    }
                    if(isset($trailer->last_location_city)){
                        array_push($address, $trailer->last_location_city);
                    }
                    if(isset($trailer->last_location_state)){
                        array_push($address, $trailer->last_location_state);
                    }
                    if(isset($trailer->last_location_zip)){
                        array_push($address, $trailer->last_location_zip);
                    }
                    if(!empty($address)){
                        $text                   .= ' - '.implode(', ', $address);
                    }
                }
                $data[$val]                     = $text;
            }
        }
    }
    return $data;
}

function getTrailerTypeList(){
    $getUserCompnayId  = getUserCompnayId();
    $trailerTypes      = TrailerType::select('id', 'name')
        ->where(function($q) use ($getUserCompnayId) {
            $q->where('company_id', $getUserCompnayId)->orWhere('company_id', 0);
        })
        ->orderBy('id','asc')
        ->pluck('name', 'id')
        ->all();
    return $trailerTypes;
    //return ['0' => 'Van', '1' => 'Reefer', '2' => 'Flatbed', '3' => 'Stepdeck', '4' => 'Power Only', '5' => 'Other'];
}

function getTrailerTypeListByCompany($type=0){
    if($type==1){
        // Trailer have trailerType only trailer type list
        $getUserCompnayId               = getUserCompnayId();
        $trailerTypeList                = Trailer::isCompany()->pluck('type', 'id')->toArray();
        $trailerTypes                   = TrailerType::where(function($q) use ($getUserCompnayId) {
                $q->where('company_id', $getUserCompnayId)->orWhere('company_id', 0);
            })
            ->whereIn('id', $trailerTypeList)
            ->orderBy('id','asc')
            ->get();
        return $trailerTypes;
    }elseif($type==2){
        // All trailer type list
        $getUserCompnayId               = getUserCompnayId();
        $trailerTypes                   = TrailerType::where(function($q) use ($getUserCompnayId) {
                $q->where('company_id', $getUserCompnayId)->orWhere('company_id', 0);
            })
            ->orderBy('id','asc')
            ->get();
        return $trailerTypes;
    }else{
        // Array trailer type list
        return getTrailerTypeList();
    }
}

function getRecruitingSourceListByCompany($companyId = null){
    if(empty($companyId)){
        $companyId = auth()->guard('company-admin')->user()->company_id;
    }
    return CompanyRecruitingSource::where('company_id', $companyId)->pluck('name', 'id')->all();
}

function getCompanyAdminListByCompany($companyId = null){
    if(empty($companyId)){
        $companyId = auth()->guard('company-admin')->user()->company_id;
    }
    return CompanyAdmin::select(DB::raw("CONCAT(first_name,' ',last_name) AS name"), 'id')
        ->where('company_id', $companyId)
        ->pluck('name', 'id')
        ->all();
}

function getUserListByCompany($companyId = null){
    if(empty($companyId)){
        $companyId  = auth()->guard('company-admin')->user()->company_id;
    }

    $data           = CompanyAdmin::select(DB::raw("CONCAT(first_name,' ',last_name) AS name"), DB::raw("CONCAT(id,'-',type) AS id"))
        ->where('company_id', $companyId)
        ->pluck('name', 'id')
        ->all();

    $data2          = Driver::select(DB::raw("CONCAT(first_name,' ',last_name) AS name"), DB::raw("CONCAT(id,'-','3') AS id"))
        ->where('company_id', $companyId)
        ->pluck('name', 'id')
        ->all();

    return array_merge($data, $data2);
}

function getManagerList(){
    return CompanyAdmin::select(DB::raw("CONCAT(first_name,' ',last_name) AS name"), 'id')
        ->where('type', 1)
        ->pluck('name', 'id')
        ->all();
}

function getTeamHeadListByCompany($companyId){
    $teamHeadIds = Team::where('company_id', $companyId)
        ->whereNotNull('head_id')
        ->pluck('head_id', 'id')
        ->all();

    return CompanyAdmin::select(DB::raw("CONCAT(first_name,' ',last_name) AS name"), 'id')
        ->whereIn('id', $teamHeadIds)
        ->pluck('name', 'id')
        ->all();
}

function getHeadTeamListByCompany($companyId){
    return Team::where('company_id', $companyId)
        ->whereNotNull('head_id')
        ->pluck('name', 'id')
        ->all();
}

function getDriverListByCompany($companyId=null, $notDriver=null){
    if(!empty($companyId)){
        return Driver::select(DB::raw("CONCAT(last_name,', ',first_name) AS name"), 'id')
        ->where('company_id', $companyId)
        ->where(function ($query) use ($notDriver){
            if(!is_null($notDriver)){
                $query->where('id', '!=', $notDriver);
            }
        })
        ->where('drivers.status', '!=', '2')
        ->orderByRaw('drivers.last_name ASC, drivers.first_name ASC')
        ->pluck('name', 'id')
        ->all();
        return null;
    }else{
        return Driver::IsCompany()
        ->select(DB::raw("CONCAT(last_name,', ',first_name) AS name"), 'id')
        ->where('drivers.status', '!=', '2')
        ->orderByRaw('drivers.last_name ASC, drivers.first_name ASC')
        ->pluck('name', 'id')
        ->all();
    }
}

function getDispatcherListByCompany($companyId){
    return CompanyAdmin::select(DB::raw("CONCAT(last_name,', ',first_name) AS name"), 'id')
        ->where('company_id', $companyId)
        ->where('type', 2)
        ->orderByRaw('last_name ASC, first_name ASC')
        ->pluck('name', 'id')
        ->all();
}

function getHowOften(){
    return ['1' => 'Does not repeat', '2' => 'Daily', '3' => 'Weekly on Wednesday', '4' => 'Monthly on the fourth Wednesday', '5' => 'Monthly on the last Wednesday', '6' => 'Annually on April 27', '7' => 'Every Weekday (Monday to Friday)', '8' => 'Custom'];
}

function getNoteType(){
    return ['1' => 'Public Note', '2' => 'Private Note'];
}

function getLoadNoteType(){
    return ['3' => 'Internal Load Note', '1' => 'External Driver Note', '2' => 'Broker Invoice Note', '4' => 'Initial Note'];
}

function getNoteRequestType(){
    return ['1' => 'Update', '2' => 'Request', '3' => 'Issue', '4' => 'Note', '5' => 'Other'];
}

function driverAccidentTypeList(){
    return ['0' => 'Accident', '1' => 'Incident', '2' => 'Claim', '3' => 'Injury', '4' => 'CItation Number', '5' => 'Other'];
}

function getNotificationReason(){
    return ['0' => 'Lumper', '1' => 'BOL', '2' => 'Detention', '3' => 'Claim : OSD', '4' => 'Credit', '5' => 'Driver Rejected'];
}

function getDays(){
    return ['0' => 'Sunday', '1' => 'Monday', '2' => 'Tuesday', '3' => 'Wednesday', '4' => 'Thursday', '5' => 'Friday', '6' => 'Saturday'];
}

function getEventTypes(){
    return ['0' => 'Seminars', '1' => 'Birthdays', '2' => 'Meeting',  '3' => 'Expirations', '4' => 'Maintenance Trucks', '5' => 'Maintenance Trailers', '6' => 'Other', '7' => 'My Leave', '8' => 'Personal Events', '9' => 'Driver Leaves', '10' => 'Dispatcher Leaves', '11' => 'My Violations', '12' => 'Maintenance Ticket'];
}

function getEventRepeat(){
    return ['0'=> 'Does Not Repeat','1' => 'All Day', '2' => 'Every Week', '3' => 'Every Month', '4' => 'Every Six Month', '5' => 'Annually'];
}

function getShareWith(){
    return ['0' => 'Private', '1' => 'Public', '2' => 'Drivers', '3' => 'Dispatchers'];
}

function driverMechanicalIssueTypeList(){
    return ['0' => '?', '1' => 'Other'];
}

function getDocumentType(){
    return ['0' => 'Copy', '1' => 'Original'];
}

function getLeaveStatus(){
    return ['0' => 'Pending Approval', '1' => 'Approved', '2' => 'Decline'];
}

function getPaidType(){
    return ['0' => 'Unpaid', '1' => 'Paid'];
}

function getLeaveDurationType(){
    return ['1' => 'Full Day', '2' => 'Half Day'];
}

function getInvoiceMethod(){
    return ['0' => 'Email', '1' => 'Print', '2' => 'Mail', '3' => 'Other'];
}

function getPaymentTerms(){
    return ['1' => 'PIA', '2' => 'Net 10', '3' => 'Net 30', '4' => 'Net 60', '5' => 'Net 90', '6' => 'Net 90', '7' => 'EOM', '8' => 'COD', '9' => 'Other'];
}

function getLoadStatusList(){
    return ['0' => 'Pending', '1' => 'Accepted', '2' => 'Preplaned', '3' => 'Confirmed', '4' => 'In Transit', '5' => 'Delivered', '6' => 'Ready To Bill', '7' => 'On Hold', '8' => 'Complete', '9' => 'Cancelled', '10' => 'Invoiced', '11' => 'Send To Driver'];
}

function getTicketStatusList(){
    return ['1' => 'New', '2' => 'Check And Advise', '3' => 'Pending Approval', '4' => 'In Progess', '5' => 'Completed
', '6' => 'Canceled'];
}

function getColorCode(){
    return ['0' => '#ff0000', '1' => '#00ffff', '2' => '#0000ff', '3' => '#0000a0', '4' => '#add8e6', '5' => '#800080', '6' => '#ffff00', '7' => '#00ff00', '8' => '#ff00ff', '9' => '#c0c0c0', '10' => '#808080', '#ffa500' => '#a52a2a', '12' => '#800000', '13' => '#008000', '14' => '#88b14b', '15' => '#ef562d', '16' => '#d1af94', '17' => '#FF00FF', '18' => '#800080'];
}

function getLoadTypeList(){
    return ['0' => 'Occupied', '1' => 'Maintenance', '2' => 'Unauthorized', '3' => 'Hos', '4' => 'No Dispatch', '5' => 'Time Off', '6' => 'Tonu'];
}

function getTrailerStatusList(){
    return ['0' => 'Available', '1' => 'Inactive', '2' => 'Hold - Safety', '3' => 'Hold - Shop','5' => 'OOS (DOT)','6' => 'OOS (Mech)', '4' => 'Other'];
}

function getUnreadDelay(){
    return ['0' => 'None', '5' => 'Every 5 minutes', '15' => 'Every 15 minutes', '30' => 'Every 30 minutes', '60' => 'Every 1 hours'];
}

function checkNofify($actionId, $fieldName, $fieldValue=1){
    $company_id = auth()->guard('company-admin')->user()->company_id;
    $user_id    = auth()->guard('company-admin')->user()->id;
    $user_type  = auth()->guard('company-admin')->user()->type;

    $data       = Notify::query()
        ->where('company_id', $company_id)
        ->where('user_id', $user_id)
        ->where('user_type', $user_type)
        ->where('action_name_id', $actionId)
        ->where($fieldName, $fieldValue)
        ->count();

    if($data > 0){
        return 1;
    }
    return 0;
}

function checkNofifyValue($actionId, $fieldName){
    $company_id = auth()->guard('company-admin')->user()->company_id;
    $user_id    = auth()->guard('company-admin')->user()->id;
    $user_type  = auth()->guard('company-admin')->user()->type;

    $data       = Notify::query()
        ->where('company_id', $company_id)
        ->where('user_id', $user_id)
        ->where('user_type', $user_type)
        ->where('action_name_id', $actionId)
        ->first();

    if(isset($data->$fieldName) && !empty($data)){
        return $data->$fieldName;
    }
    return null;
}


/*
function getNotiSourceList(){
    return ['1' => 'leaves', '2' => 'loads', '3' => 'brokers', '4' => 'drivers', '5' => 'liner-fleet-calendar', '6' => 'vendors', '7' => 'compel.announcement', '8' => 'company.announcement'];
}
*/

function getNotiSourceList($search='', $column='id') {
    $arr = array(
        array(
            "id"            => 1,
            "name"          => "Late Pickup",
            "slug"          => "loads",
            "group"         => 1,
            "status"        => 1
        ),
        array(
            "id"            => 2,
            "name"          => "Late Delivery",
            "slug"          => "loads",
            "group"         => 1,
            "status"        => 1
        ),
        array(
            "id"            => 3,
            "name"          => "Load Placed on Hold",
            "slug"          => "loads",
            "group"         => 1,
            "status"        => 1
        ),
        array(
            "id"            => 4,
            "name"          => "Load Notes",
            "slug"          => "loads",
            "group"         => 1,
            "status"        => 1
        ),
        array(
            "id"            => 5,
            "name"          => "Driver added",
            "slug"          => "drivers",
            "group"         => 2,
            "status"        => 1
        ),
        array(
            "id"            => 6,
            "name"          => "Load missing for the previous day",
            "slug"          => "liner-fleet-calendar",
            "group"         => 1,
            "status"        => 1
        ),
        array(
            "id"            => 7,
            "name"          => "Time off request submitted",
            "slug"          => "leaves",
            "group"         => 3,
            "status"        => 1
        ),
        array(
            "id"            => 8,
            "name"          => "Time Off Request updates",
            "slug"          => "leaves",
            "group"         => 3,
            "status"        => 1
        ),
        array(
            "id"            => 9,
            "name"          => "Compel Announcements",
            "slug"          => "compel.announcement",
            "group"         => 4,
            "status"        => 1
        ),
        array(
            "id"            => 10,
            "name"          => "Company Announcements",
            "slug"          => "company.announcement",
            "group"         => 4,
            "status"        => 1
        ),
        array(
            "id"            => 11,
            "name"          => "Broker Notes",
            "slug"          => "brokers",
            "group"         => 5,
            "status"        => 1
        ),
        array(
            "id"            => 12,
            "name"          => "Vendor Notes",
            "slug"          => "vendors",
            "group"         => 6,
            "status"        => 1
        ),
        array(
            "id"            => 13,
            "name"          => "Request Document",
            "slug"          => "request-documents",
            "group"         => 7,
            "status"        => 1
        ),
        array(
            "id"            => 14,
            "name"          => "Document Added",
            "slug"          => "request-documents",
            "group"         => 7,
            "status"        => 1
        ),
        array(
            "id"            => 15,
            "name"          => "OOS Approval",
            "slug"          => "loads",
            "group"         => 1,
            "status"        => 1
        ),
        array(
            "id"            => 16,
            "name"          => "Add Maintenances",
            "slug"          => "maintenances",
            "group"         => 100, // Other Group
            "status"        => 1
        ),
        array(
            "id"            => 17,
            "name"          => "Dentention Started by Driver",
            "slug"          => "loads",
            "group"         => 8, // Dentention Group
            "status"        => 1
        ),
        array(
            "id"            => 18,
            "name"          => "Dentention Stopped by Driver",
            "slug"          => "loads",
            "group"         => 8, // Dentention Group
            "status"        => 1
        ),
        array(
            "id"            => 19,
            "name"          => "Dentention Approved by Broker",
            "slug"          => "loads",
            "group"         => 8, // Dentention Group
            "status"        => 1
        ),
        array(
            "id"            => 20,
            "name"          => "Dentention Amended by Broker",
            "slug"          => "loads",
            "group"         => 8, // Dentention Group
            "status"        => 1
        ),
        array(
            "id"            => 21,
            "name"          => "Dentention Denied by Broker",
            "slug"          => "loads",
            "group"         => 8, // Dentention Group
            "status"        => 1
        ),
        array(
            "id"            => 22,
            "name"          => "Drop & Hook trailer discrepancy",
            "slug"          => "loads",
            "group"         => 1,
            "status"        => 1
        )
    );
    $new        = array_filter($arr, function ($var) {
        return ($var['status'] == 1);
    });
    if($search!=''){
        $key    = array_search($search, array_column($new, $column));
        if($key!==false){
            return $new[$key];
        }else{
            return [];
        }
    }
    return $new;
}

function getNotiUserTypeList(){
    return ['0' => 'company_admin', '1' => 'managers', '2' => 'dispatchers', '3' => 'drivers'];
}

function createNotification($source, $name, $user_id, $user_type, $type=1, $action_id=null, $description=null, $module='Notification', $properties=[], $company_id=null){
    $data                           = [];
    $data['company_id']             = is_null($company_id)?getUserCompnayId():$company_id;
    $data['module']                 = $module;
    $data['type']                   = strval($type);
    $data['source']                 = $source;
    $data['action_id']              = $action_id;
    $data['name']                   = $name;
    $data['description']            = $description;
    $data['user_id']                = $user_id;
    $data['user_type']              = $user_type;
    if(!empty($properties)){
        $data['properties']         = json_encode($properties);
    }

    $getNotify                      = Notify::where('user_id',$user_id)->where('user_type',$user_type)->where('action_name_id',$source)->where('is_unread_delay',1)->first();
    if(!empty($getNotify->id)){
        $data['is_unread_delay']    = 1;
    }else{
        $data['is_unread_delay']    = 0;
    }

    Notification::create($data);
}

function isUnreadDelay($user_id, $user_type, $action_name_id){
    $getNotify                      = Notify::select('email', 'sms')
        ->where('user_id', $user_id)
        ->where('user_type', $user_type)
        ->where('action_name_id',$action_name_id)
        ->where('is_unread_delay', 0)
        ->first();
    return $getNotify;
}

function getNotificationList($type='Notification'){
    $notifications      = null;
    if(auth()->guard('company-admin')->check()){
        $id             = auth()->guard('company-admin')->user()->id;
        $notifications  = Notification::where('user_id',$id)
            //->where('module', $type)
            ->whereIn('user_type',['0','1','2'])
            ->orderBy('id', 'desc')
            ->take(5)
            ->get();
    }else if(auth()->guard('manager')->check()){
        $id             = auth()->guard('manager')->user()->id;
        $notifications  = Notification::where('user_id',$id)
            //->where('module', $type)
            ->whereIn('user_type',['0','1','2'])
            ->orderBy('id', 'desc')
            ->take(5)
            ->get();
    }

    if(!is_null($notifications)){
        $html               = '';
        foreach($notifications as $notification){
            $href           = '';
            $html          .= '<div class="dropdown-item">';
            $html          .= '<h6>';

            if(in_array($notification->source,[7,8])){ // leaves
                $getLeave   = \App\Models\Leave::select('driver_id','company_admins_id')->where('id',$notification->action_id)->first();
                if($getLeave){
                    if(isset($getLeave->driver_id)){
                        $href               = route('company.admin.drivers.show', base64Encode($getLeave->driver_id));
                    }
                    if(isset($getLeave->company_admins_id)){
                        $actionId           = $getLeave->company_admins_id;
                        $getCompanyAdmin    = \App\Models\CompanyAdmin::find($actionId);
                        if($getCompanyAdmin->type == 1){
                            $href           = route('company.admin.managers.show', base64Encode($getCompanyAdmin->id));
                        }elseif ($getCompanyAdmin->type == 2) {
                            $href           = route('company.admin.dispatchers.show', base64Encode($getCompanyAdmin->id));
                        }
                    }
                }
            }elseif (in_array($notification->source,[1,2,3,4])) { // loads
                $loadDetail                 = \App\Models\LoadDetail::select('id')->where('load_id',$notification->action_id)->first();
                if ($loadDetail) {
                    $href                   = route('company.admin.'.getNotiSourceList($notification->source, 'id')['slug'].'.show', base64Encode($loadDetail->id));
                }
            }elseif ($notification->source==5) { // drivers
                if($notification->action_id)
                    $href                   = route('company.admin.'.getNotiSourceList($notification->source, 'id')['slug'].'.show', base64Encode($notification->action_id));
            }elseif ($notification->source==6) { // liner fleet calendar
                $href                       = route('company.admin.'.getNotiSourceList($notification->source, 'id')['slug'].'.index');
                $param                      = '';
                if(isset($notification->properties) && $notification->properties!=''){
                    $properties             = json_decode($notification->properties,true);
                    unset($properties['user_type']);
                    if(!empty($properties)){
                        $p = 0;
                        foreach ($properties as $pkey => $propertie) {
                            if($p==0){
                                $param      .= '?';
                            }
                            if($p!=0){
                                $param      .= '&';
                            }
                            if($pkey=='user_id'){
                                $param      .= 'manager='.$propertie;
                            }
                            if($pkey=='previous_date'){
                                $date       = \Carbon\Carbon::createFromFormat('Y-m-d', $propertie);
                                $month      = $date->format('m');
                                $monthName  = $date->format('F');
                                $year       = $date->format('Y');
                                $param      .= 'year='.$year.'&month='.$month;
                            }
                            $p++;
                        }
                    }
                }
                if($param!=''){
                    $href                   .= $param;
                }
            }elseif ($notification->source==9) { // dispatch-pro-announcement
                $href                       = route('company.admin.'.getNotiSourceList($notification->source, 'id')['slug'].'.show', base64Encode($notification->action_id));
            }elseif ($notification->source==10) { // company-announcements
                $href                       = route('company.admin.announcement.show', $notification->action_id);
            }elseif (in_array($notification->source, [17,18,19,20,21])) { // detentions
                $detention                  = \App\Models\LoadDetention::whereId($notification->action_id)->first();
                if($detention){
                    $href                   = route('company.admin.'.getNotiSourceList($notification->source, 'id')['slug'].'.edit', base64Encode($detention->load_detail->load_id)).'?detention_id='.$detention->id;
                }
            }else{ // etc ex-brokers
                $href                       = route('company.admin.'.getNotiSourceList($notification->source, 'id')['slug'].'.edit', base64Encode($notification->action_id));
            }

            $textType                       = 'text-primary';
            if($notification->type==2 && $notification->module=='Alert'){
                if(empty($notification->read_at)){
                    $textType               = 'text-danger';
                }
            }

            if($href!=''){
                $html                       .= '<a href="'.$href.'" class="'.$textType.'">'.$notification->name;
            }else{
                $html                       .= '<a class="'.$textType.'">'.$notification->name;
            }

            if(empty($notification->read_at)){
                $html                       .= ' <small class="text-danger">New</small>';
            }


            $html   .= '</a>';
            $html   .= '<span class="float-right mt-2">';

            $to             = \Carbon\Carbon::createFromFormat('Y-m-d H:i:s', $notification->created_at);
            $from           = \Carbon\Carbon::now()->toDateTimeString();
            $diff_in_hours  = $to->diffForHumans($from);

            $html   .= $diff_in_hours;
            $html   .= '</span>';
            $html   .= '</h6>';
            $html   .= '<p>'.Str::limit(stringDateToday(strip_tags($notification->description)), 50).'</p>';
            $html   .= '</div>';

            if(empty($notification->read_at) || $notification->read_at == ''){
                $html .= '<style type="text/css">.usernotsoi {display: block!important;}</style>';
            }
        }
    }else{
        $html = '';
    }
    return $html;
}

function clearNotification(){
    if(auth()->guard('company-admin')->check()){
        $id         = auth()->guard('company-admin')->user()->id;
        $data       = Notification::where('user_id',$id)->whereIn('user_type',['0','1','2'])->whereNull('read_at')->update(['read_at'=> Carbon::now()]);
        return 1;
    }else if(auth()->guard('manager')->check()){
        $id         = auth()->guard('manager')->user()->id;
        $data       = Notification::where('user_id',$id)->whereIn('user_type',['0','1','2'])->whereNull('read_at')->update(['read_at'=> Carbon::now()]);
        return 1;
    }else{
        return 0;
    }
}

function getNotificationCount(){
    if(auth()->guard('company-admin')->check()){
        return Notification::where('user_id', auth()->guard('company-admin')->user()->id)->whereIn('user_type', ['0','1','2'])->whereNull('read_at')->count();
    }else if(auth()->guard('manager')->check()){
        return Notification::where('user_id', auth()->guard('manager')->user()->id)->whereIn('user_type', ['0','1','2'])->whereNull('read_at')->count();
    }else{
        return 0;
    }
}

function getUnreadyNotifyUser($minutes){
    //$getNotification                              = Notification::whereNull('read_at')->groupBy('source')->get();

    $getNotification                                = Notification::whereNull('read_at')->where('is_unread_delay', 0)->groupBy('source')->get();

    $getNotification2                               = Notification::whereNull('read_at')->where('is_unread_delay', 1)->groupBy('source')->get();
    if (count($getNotification2) > 0) {
        $getNotification                            = $getNotification2->merge($getNotification);
    }

    $mailData                                       = [];
    $i                                              = 0;
    $checkSource                                    = [];

    foreach($getNotification as $notify){
        if($notify->user_type == 0 || $notify->user_type == 1 || $notify->user_type == 2){
            $getUser                                = CompanyAdmin::where('id', $notify->user_id)->first();
        }elseif($notify->user_type == 3){
            $getUser                                = Driver::where('id', $notify->user_id)->first();
        }else{
            return [];
        }

        if(isset($getUser->id)){
            if($notify->is_unread_delay == 1){
                $checkMinute                        = 'unread_delay';
                $isEnabled                          = 'is_unread_delay';
            }else{
                $checkMinute                        = 'resend_unread';
                $isEnabled                          = 'is_resend_unread';
                if(in_array($notify->source, [1,2,3,4]) && $notify->is_unread_delay == 0){
                    // 1 = Late Pickup, 2 = Late Delivery, 3 = Load Placed on Hold, 4 = Load Notes
                    continue;
                }
            }

            $getdaytime                             = Notify::where('user_id', $getUser->id)->where('user_type', $notify->user_type)->where('action_name_id', $notify->source)->where($checkMinute, $minutes)->where($isEnabled, 1)->first();
            if(isset($getdaytime->id) && !empty($getdaytime->id)){
                $day_array                          = explode(',',$getdaytime->days_id);

                $days                               = [];
                $start_time                         = '';
                $end_time                           = '';
                foreach($day_array as $day){
                    $days[]                         = $day;
                    $start_time                     = $getdaytime->start_time;
                    $end_time                       = $getdaytime->end_time;
                }

                $mytime                             = Carbon::now();
                $getday_name                        = array_keys(getDays(),$mytime->format('l'))[0];
                $getTime                            = $mytime->format('H:i:s');

                //dd($start_time, $end_time, $getTime);

                if(isset($getUser->email) && in_array($getday_name, $days) && $start_time <= $getTime && $end_time >= $getTime || $getdaytime->send_anytime == 1){
                    if(in_array($notify->source, $checkSource)){
                        continue;
                    }
                    $checkSource[]                  = $notify->source;
                    $getNotification                = Notification::whereNull('read_at')->where('source', $notify->source)->where('user_id', $getUser->id)->count();
                    $getNotification_name           = getNotiSourceList($notify->source, 'id')['name'];

                    $mailData[$i]['company_id']     = $getdaytime->company_id;

                    $mailData[$i]['name']           = $getUser->full_name;
                    $mailData[$i]['email']          = $getUser->email;
                    $mailData[$i]['phone']          = $getUser->cell_phone;
                    $mailData[$i]['source']         = $notify->source;
                    $mailData[$i]['msg']            = 'You have '.$getNotification.' "'.$getNotification_name.'" unread notifications in Portal.';
                    $mailData[$i]['url']            = route('company.admin.notifications.index');

                    $mailData[$i]['notify_sms']     = $getdaytime->sms;
                    $mailData[$i]['notify_email']   = $getdaytime->email;

                    if($notify->is_unread_delay == 1){
                        Notification::where('user_id',$getUser->id)->where('user_type', $notify->user_type)->where('source', $notify->source)->where('is_unread_delay', 1)->update(['is_unread_delay' => 0]);
                    }
                }
            }
        }
        $i++;
    }
    return $mailData;
}

function getPrice($price){
    $fmt            = new \NumberFormatter('en_US', \NumberFormatter::CURRENCY);
    $formattedPrice = $fmt->formatCurrency($price, "USD");
    // Remove trailing .00
    $formattedPrice = str_replace('.00', '', $formattedPrice);
    return $formattedPrice;
}

function getDaysPerYear(){
    $days_array = [];
    for ($x = 0; $x <= 30; $x++) {
        $days_array[$x] = $x.' days per year';
    }
    return $days_array;
}

function getLoadStatus($loadStatus,$is_on_hold=null){
    if($is_on_hold==1){
        $loadStatus = 7;
    }
    if($is_on_hold==2){
        $loadStatus = 9;
    }
    return getLoadStatusList()[$loadStatus];
}

function getLoadDispatcherName($bookedBy){
    if (isset($bookedBy)) {
        if ($bookedBy == 0) {
            $name = "Self Dispatch";
        } else {
            $name = CompanyAdmin::find($bookedBy);
            $name = isset($name->full_name) ? $name->full_name : '';
        }
        return isset($name) ? $name : '';
    }
    return '';
}

function getLoadCurrentDriver($id){
    $name       = '';
    $load       = Load::find($id);
    if(isset($load->load_detail->driver) && $load->load_detail->driver->full_name!=''){
        $name   = $load->load_detail->driver->full_name;
    }
    return $name;
}

function getLoadCurrentDispatcher($id){
    $loadDetail = Load::find($id)->load_detail;

    if ($loadDetail->booked_by == 0) {
        $name   = "Self Dispatch";
    } else {
        $name   = $loadDetail->dispatcher->full_name;
    }

    return isset($name) ? $name : '';
}

function totalLoads(){
    return LoadDetail::whereHas('loads', function ($q) {
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
        })
        ->where('is_disable', 0)
        ->count();
}

function totalActiveLoads(){
    return LoadDetail::select('*')
        ->where('is_disable', 0)
        ->where(function($query) {
            $query->whereNotIn('load_status',[5,6,7,8,9,10])
                ->orWhere(function($q) {
                    $q->whereNull('driver_id')->orWhere('driver_id', 0);
                });
        })
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
        })
        ->whereNull('deleted_at')
        ->whereHas('loads', function($q){
            $q->where('loads.company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })->count();
}

function totalCompletedLoads(){
    return LoadDetail::select('*')
        ->where('is_disable', 0)
        ->whereIn('load_type', [0,6,4]) // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
        ->where('load_status', 8)
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
        })
        ->where(function($q) {
            $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
        })->count();
}

function totalReadyToBillLoads(){
    return LoadDetail::select('*')
        ->where('is_disable', 0)
        ->whereIn('load_status', [6,10])
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
        })
        ->where(function($q) {
            $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
        })->count();
}

function totalUnCompletedLoads(){
    $loadDetailIds = Load::company()->pluck('load_detail_id', 'load_detail_id')->all();
    return LoadDetail::whereIn('id', $loadDetailIds)->where('load_status', '!=', 8)->count();
}

function getLatestLoadByCompany(){
    $loads = LoadDetail::whereHas('loads', function ($q) {
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
        })
        ->orderBy('created_at', 'DESC')
        ->where('is_disable', 0)
        ->take(3)
        ->get();
    return $loads;
}

function getLoadByDaterevenue($date, $driver_id){
    $date               = Carbon::parse($date)->format('Y-m-d');

    $data               = Driver::select(
            "load_details.*",
            "load_details.id as load_details_id",
            "load_drivers.load_detail_id",
            "load_drivers.miles")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->where('drivers.id', $driver_id)
        ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->where(function($q) {
            $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
        })
        ->whereNull('load_details.deleted_at')
        ->where(function ($query) use ($date) {
            $query->whereDate('actual_shipper_date', '<', $date)
                ->whereDate('actual_delivery_date', '>=', $date)
                ->orWhereDate('actual_delivery_date', $date);
        })
        ->groupBy('load_drivers.load_detail_id');
    $data               = @$data->get();

    $data2              = Driver::select(
            "load_details.*",
            "load_details.id as load_details_id",
            "load_drivers.load_detail_id")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->where('drivers.id', $driver_id)
        ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
        ->whereIn('load_details.load_type', [0, 4, 6]) // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
        ->where('load_details.occupied_pickup_day', 1)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->where(function($q) {
            $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
        })
        ->whereNull('load_details.deleted_at')
        ->where(function ($query) use ($date) {
            $query->whereDate('actual_shipper_date', '<=', $date)
                ->whereDate('actual_delivery_date', '>=', $date)
                ->orWhereDate('actual_delivery_date', $date);
        })
        ->groupBy('load_drivers.load_detail_id');
    $data2              = @$data2->get();

    if (count($data2) > 0) {
        $data           = @$data2->merge($data);
    }

    $unoccu_data        = Driver::select(
            "load_details.*",
            "load_details.id as load_details_id",
            "load_drivers.load_detail_id")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->where('drivers.id', $driver_id)
        ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
        ->whereIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->where(function($q) {
            $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
        })
        ->whereNull('load_details.deleted_at')
        ->where(function ($query) use ($date) {
            $query->whereDate('actual_shipper_date', '<=', $date)
                ->whereDate('actual_delivery_date', '>=', $date)
                ->orWhereDate('actual_delivery_date', $date);
        })
        ->groupBy('load_drivers.load_detail_id');

    $unoccu_data        = @$unoccu_data->get();

    $merged             = @$unoccu_data->merge($data);

    $result             = @$merged->all();

    return @$result;
}

function getLoadByDate($date, $load, $driverId, $trailer_type = null, $dispatcher = null){
    $driverId           = isset($driverId) ? $driverId : $load->driver_id;

    $date               = Carbon::parse($date)->format('Y-m-d');

    $data               = Driver::select(
            "load_details.*",
            "load_details.id as load_details_id",
            "load_drivers.load_detail_id",
            "load_drivers.miles")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->where('drivers.id', $driverId)
        ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->where(function($q) {
            $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
        })
        ->whereNull('load_details.deleted_at')
        ->where(function ($query) use ($date) {
            $query->whereDate('actual_shipper_date', '<', $date)
                ->whereDate('actual_delivery_date', '>=', $date)
                ->orWhereDate('actual_delivery_date', $date);
        })
        ->where(function ($query){
            $query->where('drivers.status', '!=', '2')->orWhere(function ($que){
                $que->where('drivers.status', '2')
                    ->where(DB::raw('DATE_FORMAT(drivers.termination_date, "%Y-%m-%d")'), '>=', DB::raw('DATE_FORMAT(load_details.actual_shipper_date, "%Y-%m-%d")'))
                    ->where(DB::raw('DATE_FORMAT(drivers.termination_date, "%Y-%m-%d")'), '>=', DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'));
            });
        })
        ->groupBy('load_drivers.load_detail_id');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $data           = $data->whereIn('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $data           = $data->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        $data           = $data->where('drivers.manager_id', $dispatcher);
    }
    $data               = $data->get();

    $data2              = Driver::select(
            "load_details.*",
            "load_details.id as load_details_id",
            "load_drivers.load_detail_id")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->where('drivers.id', $driverId)
        ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
        ->whereIn('load_details.load_type', [0, 4, 6]) // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
        ->where('load_details.occupied_pickup_day', 1)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->where(function($q) {
            $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
        })
        ->whereNull('load_details.deleted_at')
        ->where(function ($query) use ($date) {
            $query->whereDate('actual_shipper_date', '<=', $date)
                ->whereDate('actual_delivery_date', '>=', $date)
                ->orWhereDate('actual_delivery_date', $date);
        })
        ->where(function ($query){
            $query->where('drivers.status', '!=', '2')->orWhere(function ($que){
                $que->where('drivers.status', '2')
                    ->where(DB::raw('DATE_FORMAT(drivers.termination_date, "%Y-%m-%d")'), '>=', DB::raw('DATE_FORMAT(load_details.actual_shipper_date, "%Y-%m-%d")'))
                    ->where(DB::raw('DATE_FORMAT(drivers.termination_date, "%Y-%m-%d")'), '>=', DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'));
            });
        })
        ->groupBy('load_drivers.load_detail_id');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $data2          = $data2->whereIn('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $data2          = $data2->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        $data2          = $data2->where('drivers.manager_id', $dispatcher);
    }

    $data2              = $data2->get();
    if (count($data2) > 0) {
        $data           = $data2->merge($data);
    }

    $unoccu_data        =
        Driver::select(
            "load_details.*",
            "load_details.id as load_details_id",
            "load_drivers.load_detail_id")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->where('drivers.id', $load->driver_id)
        ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
        ->whereIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->where(function($q) {
            $q->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
        })
        ->whereNull('load_details.deleted_at')
        ->where(function ($query) use ($date) {
            $query->whereDate('actual_shipper_date', '<=', $date)
                ->whereDate('actual_delivery_date', '>=', $date)
                ->orWhereDate('actual_delivery_date', $date);
        })
        ->where(function ($query){
            $query->where('drivers.status', '!=', '2')->orWhere(function ($que){
                $que->where('drivers.status', '2')
                    ->where(DB::raw('DATE_FORMAT(drivers.termination_date, "%Y-%m-%d")'), '>=', DB::raw('DATE_FORMAT(load_details.actual_shipper_date, "%Y-%m-%d")'))
                    ->where(DB::raw('DATE_FORMAT(drivers.termination_date, "%Y-%m-%d")'), '>=', DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'));
            });
        })
        ->groupBy('load_drivers.load_detail_id');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $unoccu_data    = $unoccu_data->whereIn('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $unoccu_data    = $unoccu_data->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        $unoccu_data    = $unoccu_data->where('drivers.manager_id', $dispatcher);
    }
    $unoccu_data        = $unoccu_data->get();

    $merged             = $unoccu_data->merge($data);
    $result             = $merged->all();
    return $result;
}

function getLoadTitle($load){
    $load_type              = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF

    $checkid                = [];
    $i                      = 0;
    $checkIsLoad            = [];
    $m                      = 0;
    foreach ($load as $load_details) {
        if (in_array($load_details['load_type'], $load_type)) {
            $checkid[$i]    = $load_details['load_type'];
            $i++;
        } else {
            $checkIsLoad[$m]= $load_details['load_number'];
            $m++;
        }
    }

    if (count($checkid) > 0) {
        $typeLabel          = '';
        foreach ($checkid as $key => $value) {
            if($key==0){
                $typeLabel  = getLoadTypeList()[$value];
            }
        }
        $html               = '<span class="text-red">' . $typeLabel . '</span>';

        // $laod_id         = implode(', ', $checkIsLoad);
        // if (empty($laod_id)) {
        //     $html        = '<span>' . count($load) . ' Loads</span>';
        // }else{
        //     foreach ($checkid as $key => $value) {
        //         $laod_id.= ', '.getLoadTypeList()[$value];
        //     }
        //     $html        = '<span class="text-red">' . $laod_id . '</span>';
        // }
    } else {
        $html               = '<span>' . count($load) . ' Loads</span>';

        // $typeLabel          = '';
        // foreach ($checkIsLoad as $key => $value) {
        //     if($key==0){
        //         $typeLabel  = $value;
        //     }
        // }
        // $html               = '<span>' . $typeLabel . '</span>';
    }
    return $html;
}

function getLoadColor($load)
{
    $load_type = [1, 2, 3, 5]; // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
    $checkid = [];
    $i = 0;
    $checkIsLoad = [];
    $m = 0;

    if (count($load) == 1) {
        $status_type = $load[0]['load_status'];
        $color_class = getLoadColorClass($status_type);
    } else {

        /*foreach($load as $load_details){
            if(in_array($load_details['load_type'], $load_type)){
                $checkid[$i] = $load_details['load_status'];
                $i++;
            }else{
                $checkIsLoad[$m] = $load_details['load_status'];
                $m++;
            }
        }

        if(count($checkid) > 0){
            $color_class = 'red';
        }else{
            $color_class = '';
        }*/

        $color_class = '';

    }

    return $color_class;
}

function getLoadColorClass($load_status)
{
    $class_name = [0 => '', 1 => '', 2 => 'yellow', 3 => 'green', 4 => 'green', 5 => 'orange', 6 => 'blue', 7 => 'red', 8 => 'grey', 9 => ''];
    return (isset($class_name[$load_status])) ? $class_name[$load_status] : '';
}

function getLoadMiles($load)
{
    $load_type      = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
    if (!in_array($load->load_type, $load_type)) {
        if ($load->is_disable == 0) {
            return number_format($load->miles);
        } else {
            $data   = LoadDriver::where('load_id', $load->load_id)
                ->where('driver_id', $load->driver_id)
                ->orderBy('id', 'DESC')
                ->first()
                ->miles;
            return $data;
        }
    }
    return null;
}

function getLoadPay($load)
{
    $load_type  = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
    if (!in_array($load->load_type, $load_type)) {
        $data   = LoadDriver::select("load_drivers.*")
            ->join("load_details", "load_details.id", "=", "load_drivers.load_detail_id")
            ->where('load_drivers.load_id', $load->load_id)
            ->where('load_drivers.driver_id', $load->driver_id)
            ->first()
            ->driver_revenue;

        $fmt = new \NumberFormatter('en_US', \NumberFormatter::CURRENCY);
        return $fmt->formatCurrency($data, "USD");
    }
    return null;
}

function getLoadPayFloat($load)
{
    $load_type  = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
    if (!in_array($load->load_type, $load_type)) {
        $data   = LoadDriver::select("load_drivers.*")
            ->join("load_details", "load_details.id", "=", "load_drivers.load_detail_id")
            ->where('load_drivers.load_id', $load->load_id)
            ->where('load_drivers.driver_id', $load->driver_id)
            ->first()
            ->driver_revenue;

        return $data;
    }
    return null;
}

function getLoadTargetRevenue($targetRevenue)
{
    $fmt = new \NumberFormatter('en_US', \NumberFormatter::CURRENCY);
    return $fmt->formatCurrency($targetRevenue, "USD");
}

function getLoadRevenue($load)
{
    $load_type                  = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
    if (!in_array($load->load_type, $load_type)) {
        $loadRevenue            = $load->agreed_upon_rate + $load->detention;
        if(isset($load->accessorial) && is_serialized($load->accessorial) && !empty($load->accessorial)){
            $accessorial        = unserialize($load->accessorial);
            foreach ($accessorial as $key => $accessorial_field){
                $loadRevenue    += isset($accessorial_field['value'])?$accessorial_field['value']:0;
            }
        }
        $fmt                    = new \NumberFormatter('en_US', \NumberFormatter::CURRENCY);
        return $fmt->formatCurrency($loadRevenue, "USD");
    }
    return null;
}

function getLoadRevenueFloat($load)
{
    $load_type                  = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
    if (!in_array($load->load_type, $load_type)) {
        $loadRevenue            = $load->agreed_upon_rate + $load->detention;
        if(isset($load->accessorial) && is_serialized($load->accessorial) && !empty($load->accessorial)){
            $accessorial        = unserialize($load->accessorial);
            foreach ($accessorial as $key => $accessorial_field){
                $loadRevenue    += isset($accessorial_field['value'])?$accessorial_field['value']:0;
            }
        }
        return $loadRevenue;
    }
    return null;
}

function getLoadGrossRevenue($startDate, $endDate, $driverId, $load = null, $trailer_type = null, $dispatcher = null)
{
    // dd($driverId);
    $data           = Driver::select("load_details.driver_revenue")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where("load_drivers.driver_id", $driverId)
        ->groupBy('load_drivers.load_detail_id')
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNull('load_details.deleted_at');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $data       = $data->whereIn('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $data       = $data->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //  $data   = $data->where('load_details.booked_by',$dispatcher);
        $data       = $data->where('drivers.manager_id', $dispatcher);
    }

    $data           = $data->pluck("load_details.driver_revenue")->all();

    $loadIds        = LoadDetail::select("load_details.load_id")
        ->join("drivers", "drivers.id", "=", "load_details.driver_id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->where('load_details.is_disable', 1)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNull('load_details.deleted_at');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $loadIds    = $loadIds->whereIn('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $loadIds    = $loadIds->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$loadIds  = $loadIds->where('load_details.booked_by',$dispatcher);
        $loadIds    = $loadIds->where('drivers.manager_id', $dispatcher);
    }

    $loadIds        = $loadIds->pluck("load_details.load_id")->all();


    $data2          = [];
    $r              = 0;
    foreach ($loadIds as $key => $value) {
        $data2[$r]  = LoadDriver::where("load_drivers.load_id", $value)
            ->join("load_details", "load_details.id", "=", "load_drivers.load_detail_id")
            ->where("load_drivers.driver_id", $driverId)
            ->whereNull('load_details.deleted_at')
            ->first()
            ->driver_revenue;
        $r++;
    }

    $total          = array_sum($data) + array_sum($data2);
    return $total;
}

function getLoadGrossRevenue2($startDate, $endDate, $driverId)
{
    $data           = Driver::select("load_details.driver_revenue")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where("load_drivers.driver_id", $driverId)
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNull('load_details.deleted_at')
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->groupBy('load_drivers.load_detail_id');

    $data           = $data->pluck("load_details.driver_revenue")->all();

    $loadIds        = LoadDetail::select("load_details.load_id")
        ->join("drivers", "drivers.id", "=", "load_details.driver_id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where('load_details.is_disable', 1)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNull('load_details.deleted_at')
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF

    $loadIds        = $loadIds->pluck("load_details.load_id")->all();

    $data2          = [];
    $r              = 0;
    foreach ($loadIds as $key => $value) {
        $data2[$r]  = LoadDriver::where("load_drivers.load_id", $value)
            ->join("load_details", "load_details.id", "=", "load_drivers.load_detail_id")
            ->where("load_drivers.driver_id", $driverId)
            ->whereNull('load_details.deleted_at')
            ->first()
            ->driver_revenue;
        $r++;
    }

    $total          = array_sum($data) + array_sum($data2);
    return $total;
}

function getLoadSingleCPM($load)
{
    $load_type      = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
    if (!in_array($load->load_type, $load_type)) {
        $fmt        = new \NumberFormatter('en_US', \NumberFormatter::CURRENCY);
        return $fmt->formatCurrency($load->fuel_surcharge_cost_per_mile, "USD");
    }
    return null;
}

function getLoadSingleCPM2($value, $miles)
{
    $miles          = filter_var($miles, FILTER_SANITIZE_NUMBER_INT);
    $expolode       = explode('.', $value);
    if (count($expolode) > 1) {
        $price_numebr = filter_var($expolode[0], FILTER_SANITIZE_NUMBER_INT);
        $price      = $price_numebr . '.' . $expolode[1];
    } else {
        $price      = filter_var($value, FILTER_SANITIZE_NUMBER_INT);
    }
    if (is_numeric($price) && is_numeric($miles)) {
        $cal        = $price / $miles;
        $fmt        = new \NumberFormatter('en_US', \NumberFormatter::CURRENCY);
        return $fmt->formatCurrency($cal, "USD");
    }
    return null;
}

function getLoadCPM($startDate, $endDate, $driverId, $load, $trailer_type = null, $dispatcher = null)
{
    if (getLoadGrossRevenue($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher) == 0 || getLoadMileage($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher) == 0) {
        return 0;
    } else {
        return number_format(getLoadGrossRevenue($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher) / getLoadMileage($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher), 2);
    }
}

function getLoadMileage($startDate, $endDate, $driverId, $load = null, $trailer_type = null, $dispatcher = null)
{
    $data               = Driver::select("load_drivers.miles")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where("load_drivers.driver_id", $driverId)
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at')
        ->groupBy('load_drivers.load_detail_id');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $data           = $data->whereIn('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $data           = $data->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$data         = $data->where('load_details.booked_by',$dispatcher);
        $data           = $data->where('drivers.manager_id', $dispatcher);
    }

    $data               = $data->pluck("load_drivers.miles")->all();

    $loadIds            = LoadDetail::select("load_details.load_id")
        ->join("drivers", "drivers.id", "=", "load_details.driver_id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where('load_details.is_disable', 1)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $loadIds        = $loadIds->where('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $loadIds        = $loadIds->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$loadIds      = $loadIds->where('load_details.booked_by',$dispatcher);
        $loadIds        = $loadIds->where('drivers.manager_id', $dispatcher);
    }

    $loadIds            = $loadIds->pluck("load_details.load_id")->all();

    $data2              = [];
    foreach ($loadIds as $key => $value) {
        $data2[]        = LoadDriver::where("load_id", $value)
            ->where("driver_id", $driverId)
            ->orderBy("id", "DESC")
            ->first()
            ->miles;
    }

    return array_sum($data) + array_sum($data2);

    return $data;
}

function getLoadMileage2($startDate, $endDate, $driverId)
{
    $data               = Driver::select("load_drivers.miles")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where("load_drivers.driver_id", $driverId)
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at')
        ->groupBy('load_drivers.load_detail_id');

    $data               = $data->pluck("load_drivers.miles")->all();

    $loadIds            = LoadDetail::select("load_details.load_id")
        ->join("drivers", "drivers.id", "=", "load_details.driver_id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where('load_details.is_disable', 1)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at');

    $loadIds            = $loadIds->pluck("load_details.load_id")->all();

    $data2              = [];
    foreach ($loadIds as $key => $value) {
        $data2[]        = LoadDriver::where("load_id", $value)
            ->where("driver_id", $driverId)
            ->orderBy("id", "DESC")
            ->first()
            ->miles;
    }

    return array_sum($data) + array_sum($data2);
    return $data;
}

/********Create By Dhruvish***********/
function getLoadMileagerevenuetracker($startDate, $endDate, $driverId, $load = null, $trailer_type = null, $dispatcher = null)
{
    $data               = Driver::select("load_drivers.miles")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where("load_drivers.driver_id", $driverId)
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at')
        ->groupBy('load_drivers.load_detail_id');
    if ($trailer_type != null) {
        $data           = $data->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$data         = $data->where('load_details.booked_by',$dispatcher);
        $data           = $data->where('drivers.manager_id', $dispatcher);
    }

    $data               = $data->pluck("load_drivers.miles")->all();

    $loadIds            = LoadDetail::select("load_details.load_id")
        ->join("drivers", "drivers.id", "=", "load_details.driver_id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where('load_details.is_disable', 1)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at');

    if ($trailer_type != null) {
        $loadIds        = $loadIds->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$loadIds      = $loadIds->where('load_details.booked_by',$dispatcher);
        $loadIds        = $loadIds->where('drivers.manager_id', $dispatcher);
    }

    $loadIds            = $loadIds->pluck("load_details.load_id")->all();

    $data2              = [];
    foreach ($loadIds as $key => $value) {
        $data2[]        = LoadDriver::where("load_id", $value)
            ->whereNull('deleted_at')
            ->where("driver_id", $driverId)
            ->orderBy("id", "DESC")
            ->first()
            ->miles;
    }

    return array_sum($data) + array_sum($data2);

    return $data;
}

function getLoadOccupancy($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher)
{
    //$getLoadUnoccupancy   = getLoadUnoccupancy($startDate,$endDate,$driverId,$load,$trailer_type,$dispatcher);
    $getDatePickupCount     = getDatePickupCount($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher);
    $period                 = CarbonPeriod::create($startDate, $endDate);

    $totalDays              = 0;
    $bcoutn                 = 0;
    $mizcoutn               = 0;
    foreach ($period as $date) {
        $dateYMD            = $date->format('m/d/Y');
        $getload            = getLoadByDate($dateYMD, $load,$driverId, $trailer_type, $dispatcher);
        if (count($getload) > 0) {
            $load_type      = array(0, 4, 6); // 0 = OCCUPIED, 4 = NO DISPATCH, 6 = TONU
            foreach ($getload as $load_details) {
                if (in_array($load_details['load_type'], $load_type)) {
                    $totalDays++;
                    break;
                }
            }
            // $mizcoutn++;
        }
        if (count($getload) > 1) {
            $load_type      = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
            $uc             = 0;
            $oc             = 0;
            foreach ($getload as $load_details) {
                if (in_array($load_details['load_type'], $load_type)) {
                    $uc++;
                } else {
                    $oc++;
                }
            }
            if ($oc > 0 && $uc > 0) {
                $bcoutn++;
            }
            // $mizcoutn++;
        }
        if (count($getload) > 0) {
            $mizcoutn++;
        }
    }

    $totalDays2             = $totalDays + $getDatePickupCount;
    $totDay                 = $totalDays + $mizcoutn;
    $totalDays3             = $totalDays2 - $bcoutn;

    $mapeval                = 0;
    if ($totDay > 0) {
        $mapeval            = max(number_format(100 / $mizcoutn * $totalDays3, 2), 0);
    }
    return $mapeval;
}

function getDatePickupCount($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher)
{
    $data_pickup_count      = Driver::select("load_details.id")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->whereNull('load_details.deleted_at')
        ->where('drivers.id', $driverId)
        ->where('load_drivers.driver_id', $driverId)
        ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
        ->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
        ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
        ->where('load_details.occupied_pickup_day', 1)
        ->whereNotIn('load_details.load_type', [0, 4, 6]) // 0 = OCCUPIED, 4 = NO DISPATCH, 6 = TONU
        ->whereNull('load_details.deleted_at');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $data_pickup_count  = $data_pickup_count->whereIn('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $data_pickup_count  = $data_pickup_count->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$data_pickup_count = $data_pickup_count->where('load_details.booked_by',$dispatcher);
        $data_pickup_count  = $data_pickup_count->where('drivers.manager_id', $dispatcher);
    }

    $data_pickup_count      = $data_pickup_count->count();
    return $data_pickup_count;
}


function getOccupancyBasedRevenue($startDate, $endDate, $driverId, $targetRevenue, $load, $trailer_type = null, $dispatcher = null)
{

    if (getLoadGrossRevenue($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher) == 0 || getLoadOccupancy($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher) == 0) {
        $revenue        = 0.00;
    } else {
        if ((getLoadOccupancy($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher) * $targetRevenue) * 100 == 0) {

            $revenue    = 0.00;
        } else {

            $revenue    = (getLoadGrossRevenue($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher) / (getLoadOccupancy($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher) * $targetRevenue)) * 100;
        }
    }

    return number_format($revenue, 2);
}


function getLoadTargetOvnerUnder($startDate, $endDate, $driverId, $targetRevenue, $load, $trailer_type = null, $dispatcher = null)
{
    return getLoadGrossRevenue($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher) - $targetRevenue;
}

function driverOnboardingSetting($driverId = null)
{
    //return DriverOnboardingSetting::isCompany()->orderBy('order')->get();
    if(!is_null($driverId)){
        $driverOnboarding   = DriverOnboarding::where('driver_id', $driverId)->first();
        if(!is_null($driverOnboarding)){
            if($driverOnboarding->status == 2){
                $settings   = DriverOnboardingSetting::isCompany()->wherenull('apply_to_profile')->wherenull('hired_before_stage_added_date')->orderBy('order')->get();
                $settings1  = DriverOnboardingSetting::isCompany()->where('apply_to_profile', $driverId)->wherenull('hired_before_stage_added_date')->orderBy('order')->get();
                $settings2  = DriverOnboardingSetting::isCompany()->wherenull('apply_to_profile')->where('hired_before_stage_added_date', '>', Carbon::parse($driverOnboarding->application_date)->format('Y-m-d'))->orderBy('order')->get();

                $settings   = $settings->merge($settings1);
                $settings   = $settings->merge($settings2);
                return  $settings;
            }else{
                $settings   = DriverOnboardingSetting::isCompany()->wherenull('apply_to_profile')->wherenull('hired_before_stage_added_date')->orderBy('order')->get();
                $settings1  = DriverOnboardingSetting::isCompany()->where('apply_to_profile', $driverId)->wherenull('hired_before_stage_added_date')->orderBy('order')->get();

                $settings   = $settings->merge($settings1);
                return $settings;
            }
        }
    }
    return DriverOnboardingSetting::isCompany()->wherenull('apply_to_profile')->wherenull('hired_before_stage_added_date')->orderBy('order')->get();
}

function driverOnboardingStatusList()
{
    return ['0' => 'Pending', '1' => 'Onboarding', '2' => 'Hired', '3' => 'Rejected'];
}

function getAddressCityState($city, $state)
{
    if (!is_null($city) && !is_null($state)) {
        $cityState = $city . ', ' . $state;
    } elseif (!is_null($city) && is_null($state)) {
        $cityState = $city;
    } elseif (is_null($city) && !is_null($state)) {
        $cityState = $state;
    } else {
        $cityState = null;
    }

    return $cityState;
}

function perPageView()
{
    $pageView           = 10;
    if(auth()->guard('company-admin')->check()){
        $pageView       = auth()->guard('company-admin')->user()->per_page_view;
        if (empty($pageView)) {
            $pageView   = 10;
        }
    }elseif(auth()->guard('driver-admin')->check()){
        $pageView       = 10;
    }
    return $pageView;
}

function getDeliveryDateOption()
{
    $DeliveryDateOption = auth()->guard('company-admin')->user()->delivery_date_option;
    if (empty($DeliveryDateOption)) {
        $DeliveryDateOption = 1;
    }
    return $DeliveryDateOption;
}

function getColorOption()
{
    $colorOptions   = auth()->guard('company-admin')->user()->color_options;
    return $colorOptions;
}

function getManageTypeCategory()
{
    return ManageTypeCategory::pluck('name', 'id')->all();
}

function getHOSViolationsList()
{
    return ManageCompanyType::where('company_id', auth()->guard('company-admin')->user()->company_id)
        ->where('category_id', 1)
        ->pluck('name', 'id')->all();
}

function getCitationsViolationsWarningsList()
{
    return ManageCompanyType::where('company_id', auth()->guard('company-admin')->user()->company_id)
        ->where('category_id', 2)
        ->pluck('name', 'id')->all();
}

function getAccidentIncidentClaimInjuryList()
{
    return ManageCompanyType::where('company_id', auth()->guard('company-admin')->user()->company_id)
        ->where('category_id', 3)
        ->pluck('name', 'id')->all();
}

function getMechanicalIssuesList()
{
    return ManageCompanyType::where('company_id', auth()->guard('company-admin')->user()->company_id)
        ->where('category_id', 4)
        ->pluck('name', 'id')->all();
}


function getDateMDY($date)
{
    if (!empty($date) && isset($date)) {
        return Carbon::createFromFormat('m/d/Y H:i', $date)->format('m/d/Y');
    }

    return '';
}

function getTime($date)
{
    if (!empty($date) && isset($date)) {
        $timeFormat = auth()->guard('company-admin')->user()->time_format;
        if ($timeFormat == 12) {
            $dateFormat = 'g:i A';
        } else {
            $dateFormat = 'H:i';
        }
        return Carbon::createFromFormat('m/d/Y H:i', $date)->format($dateFormat);
    }

    return '';
}


function getDatePickupCountMonth($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher)
{
//    dd($startDate, $endDate);
    $data_pickup_count = Driver::select("load_details.id")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->whereNull('load_details.deleted_at')
        ->where('drivers.id', $driverId)
        ->where('load_drivers.driver_id', $driverId)
        ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
        ->where(DB::raw('DATE_FORMAT(load_details.actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
        ->where(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
        ->where('load_details.occupied_pickup_day', 1)
        ->whereNotIn('load_details.load_type', [0, 4, 6]); // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
    if ($trailer_type != null) {
        $data_pickup_count = $data_pickup_count->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$data_pickup_count = $data_pickup_count->where('load_details.booked_by',$dispatcher);
        $data_pickup_count = $data_pickup_count->where('drivers.manager_id', $dispatcher);
    }

    $data_pickup_count = $data_pickup_count->count();


    return $data_pickup_count;

}

function getLoadByDateMonth($date, $driverId, $trailer_type = null, $dispatcher = null)
{


    return getLoadByDateMonth2($date, $driverId, $trailer_type = null, $dispatcher = null);

    /*$date = Carbon::parse($date)->format('Y-m-d');

    $data =
        Driver::select("load_details.*", "load_details.id as load_details_id", "load_drivers.load_detail_id", "load_drivers.miles")
            ->join("load_details", "load_details.driver_id", "=", "drivers.id")
            ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
            ->whereNull('load_details.deleted_at')
            ->where('load_details.load_status', '!=', 9)
            ->where(function($q) {
                $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
            })
            ->where('drivers.id', $driverId)
            ->where('load_drivers.driver_id', $driverId)
            ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
            ->where(function ($query) use ($date) {
                $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<', $date)
                    ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $date)
                    ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $date);
            })
            ->groupBy('load_drivers.load_detail_id');

    if ($trailer_type != null) {
        $data = $data->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$data = $data->where('load_details.booked_by',$dispatcher);
        $data = $data->where('drivers.manager_id', $dispatcher);
    }

    $data = $data->get();


    $data2 =
        Driver::select("load_details.*", "load_details.id as load_details_id", "load_drivers.load_detail_id")
            ->join("load_details", "load_details.driver_id", "=", "drivers.id")
            ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
            ->whereNull('load_details.deleted_at')
            ->where('load_details.load_status', '!=', 9)
            ->where(function($q) {
                $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
            })
            ->where('drivers.id', $driverId)
            ->whereIn('load_details.load_type', [0, 4, 6]) // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
            ->where('load_details.occupied_pickup_day', 1)
            ->where('load_drivers.driver_id', $driverId)
            ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
            ->where(function ($query) use ($date) {
                $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $date)
                    ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $date)
                    ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $date);
            })
            ->groupBy('load_drivers.load_detail_id');

    if ($trailer_type != null) {
        $data2 = $data2->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$data2 = $data2->where('load_details.booked_by',$dispatcher);
        $data2 = $data2->where('drivers.manager_id', $dispatcher);
    }

    $data2 = $data2->get();


    if (count($data2) > 0) {
        $data = $data2->merge($data);
    }

    $unoccu_data =
        Driver::select("load_details.*", "load_details.id as load_details_id", "load_drivers.load_detail_id")
            ->join("load_details", "load_details.driver_id", "=", "drivers.id")
            ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
            ->whereNull('load_details.deleted_at')
            ->where('load_details.load_status', '!=', 9)
            ->where(function($q) {
                $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
            })
            ->where('drivers.id', $driverId)
            ->where('load_drivers.driver_id', $driverId)
            ->whereIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
            ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
            ->where(function ($query) use ($date) {
                $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $date)
                    ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $date)
                    ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $date);
            })
            ->groupBy('load_drivers.load_detail_id');
    if ($trailer_type != null) {
        $unoccu_data = $unoccu_data->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$unoccu_data = $unoccu_data->where('load_details.booked_by',$dispatcher);
        $unoccu_data = $unoccu_data->where('drivers.manager_id', $dispatcher);
    }
    $unoccu_data = $unoccu_data->get();

    $merged = $unoccu_data->merge($data);


    $result = $merged->all();

    return $result;*/
}

/* draws a calendar */

function getLoadOccupancyCalendarMonth($startDate, $endDate, $driverId, $load = null, $trailer_type = null, $dispatcher = null)
{
    //$getLoadUnoccupancy = getLoadUnoccupancy($startDate,$endDate,$driverId,$load,$trailer_type,$dispatcher);
    $getDatePickupCount = getDatePickupCountMonth($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher);
    $period = CarbonPeriod::create($startDate, $endDate);
    $totalDays = 0;
    $bcoutn = 0;
    $daycount = 0;
    foreach ($period as $date) {

        $dateYMD = $date->format('m/d/Y');
        $getload = getLoadByDateMonth($dateYMD, $driverId, $trailer_type, $dispatcher, $driverId);
        if (count($getload) > 0) {
            $load_type = array(0, 4, 6); // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
            foreach ($getload as $load_details) {
                if (in_array($load_details['load_type'], $load_type)) {
                    $totalDays++;
                    break;
                }
            }
            $daycount++;
        }
        if (count($getload) > 1) {
            $load_type = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
            $uc = 0;
            $oc = 0;
            foreach ($getload as $load_details) {
                if (in_array($load_details['load_type'], $load_type)) {
                    $uc++;
                } else {
                    $oc++;
                }
            }
            if ($oc > 0 && $uc > 0) {
                $bcoutn++;
            }
            $daycount++;
        }

    }


    $totalDays = $totalDays + $getDatePickupCount;
    $totalDays = $totalDays - $bcoutn;
    $monthnumber = \Carbon\Carbon::createFromFormat('Y-m-d', $startDate)->format('m');
    $mapeval = 0;
    if($daycount > 0){
        $mapeval = max(number_format(100 / $daycount * $totalDays, 2), 0);
    }

    return $mapeval;
}

function getLoadOccupancyCalendarMonth2($startDate, $endDate, $driverId)
{

    $loadDetails = LoadDetail::where('driver_id', $driverId)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
            ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate);
        })
        ->whereNull('deleted_at')
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->get();
        //->pluck('load_number');



    $daycount = 0;
    $totalDays = 0;
    $occupiedPickupDay = 0;
    $dateYMD = [];
    $OccupieddateYMD = [];
    $UnOccupieddateYMD = [];
    $bothUnoccuporid = 0;
    $bcoutn = 0;

    foreach($loadDetails as $load){

        $startDateLoad = \Carbon\Carbon::createFromFormat(getTimeLayout(), $load->actual_shipper_date)->format('Y-m-d');
        $endDateLoad = \Carbon\Carbon::createFromFormat(getTimeLayout(), $load->actual_delivery_date)->format('Y-m-d');

        if($startDateLoad <= $startDate){
            $startDateLoad = $startDate;
        }

        if($endDateLoad >= $endDate){
            $endDateLoad = $endDate;
        }

        $period = CarbonPeriod::create($startDateLoad, $endDateLoad);
        //dd($period, $startDate, $endDate);
        $dateae = 0;
        foreach ($period as $date) {
            if($dateae == 0){
                if($load->occupied_pickup_day == 1 || in_array($load->load_type, [1, 2, 3, 5])){ // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
                    if(!in_array($date->format('m/d/Y'), $dateYMD)){
                        $dateYMD[] = $date->format('m/d/Y');
                    }
                }
            }else{
                if(!in_array($date->format('m/d/Y'), $dateYMD)){
                    $dateYMD[] = $date->format('m/d/Y');
                }
            }

            $dateae++;
        }

        $cheie = 0;

        if (in_array($load->load_type, [0, 4, 6])) { // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
            foreach ($period as $date) {
                if($cheie == 0){
                    if($load->occupied_pickup_day == 1){
                        if(!in_array($date->format('m/d/Y'), $OccupieddateYMD)){
                            $OccupieddateYMD[] = $date->format('m/d/Y');
                        }
                    }
                }else{
                    if(!in_array($date->format('m/d/Y'), $OccupieddateYMD)){
                        $OccupieddateYMD[] = $date->format('m/d/Y');
                    }
                }
                $cheie++;
            }

        }

        if (in_array($load->load_type, [1, 2, 3, 5])) { // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
            foreach ($period as $date) {
                if(!in_array($date->format('m/d/Y'), $UnOccupieddateYMD)){
                    $UnOccupieddateYMD[] = $date->format('m/d/Y');
                }
            }
        }

    }

    foreach($UnOccupieddateYMD as $unOcuDate){
        if(in_array($unOcuDate, $OccupieddateYMD)){
            $bcoutn++;
        }
    }

    $daycount = count($dateYMD);
    $totalDays = count($OccupieddateYMD);
    $totalDays = $totalDays - $bcoutn;

    $mapeval = 0;
    if($daycount > 0){
        $mapeval = max(number_format(100 / $daycount * $totalDays, 2), 0);
    }

    //dd($driverId,$endDate, $startDate, $daycount, $totalDays, $mapeval);

    return ['getOccupancy' => $mapeval, 'getLoadGrossRevenue' => $mapeval];
}

function getLoadOccupancyCalendarMonth3($startDate, $endDate, $loadDetailsCollection = array())
{
    $loadDetails = $loadDetailsCollection;

    $daycount = 0;
    $totalDays = 0;
    $occupiedPickupDay = 0;
    $dateYMD = [];
    $OccupieddateYMD = [];
    $UnOccupieddateYMD = [];
    $bothUnoccuporid = 0;
    $bcoutn = 0;

    foreach($loadDetails as $load){

        $startDateLoad = \Carbon\Carbon::createFromFormat(getTimeLayout(), $load->actual_shipper_date)->format('Y-m-d');
        $endDateLoad = \Carbon\Carbon::createFromFormat(getTimeLayout(), $load->actual_delivery_date)->format('Y-m-d');

        if($startDateLoad <= $startDate){
            $startDateLoad = $startDate;
        }

        if($endDateLoad >= $endDate){
            $endDateLoad = $endDate;
        }

        $period = CarbonPeriod::create($startDateLoad, $endDateLoad);
        //dd($period, $startDate, $endDate);
        $dateae = 0;
        foreach ($period as $date) {
            if($dateae == 0){
                if($load->occupied_pickup_day == 1 || in_array($load->load_type, [1, 2, 3, 5])){ // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
                    if(!in_array($date->format('m/d/Y'), $dateYMD)){
                        $dateYMD[] = $date->format('m/d/Y');
                    }
                }
            }else{
                if(!in_array($date->format('m/d/Y'), $dateYMD)){
                    $dateYMD[] = $date->format('m/d/Y');
                }
            }

            $dateae++;
        }

        $cheie = 0;

        if (in_array($load->load_type, [0, 4, 6])) { // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
            foreach ($period as $date) {
                if($cheie == 0){
                    if($load->occupied_pickup_day == 1){
                        if(!in_array($date->format('m/d/Y'), $OccupieddateYMD)){
                            $OccupieddateYMD[] = $date->format('m/d/Y');
                        }
                    }
                }else{
                    if(!in_array($date->format('m/d/Y'), $OccupieddateYMD)){
                        $OccupieddateYMD[] = $date->format('m/d/Y');
                    }
                }
                $cheie++;
            }

        }

        if (in_array($load->load_type, [1, 2, 3, 5])) { // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
            foreach ($period as $date) {
                if(!in_array($date->format('m/d/Y'), $UnOccupieddateYMD)){
                    $UnOccupieddateYMD[] = $date->format('m/d/Y');
                }
            }
        }

    }

    foreach($UnOccupieddateYMD as $unOcuDate){
        if(in_array($unOcuDate, $OccupieddateYMD)){
            $bcoutn++;
        }
    }

    $daycount = count($dateYMD);
    $totalDays = count($OccupieddateYMD);
    $totalDays = $totalDays - $bcoutn;

    $mapeval = 0;
    if($daycount > 0){
        $mapeval = max(number_format(100 / $daycount * $totalDays, 2), 0);
    }

    //dd($driverId,$endDate, $startDate, $daycount, $totalDays, $mapeval);

    return ['getOccupancy' => $mapeval, 'getLoadGrossRevenue' => $mapeval];
}

function getLoadTotalDays($startDate, $endDate, $driverId, $load = null, $trailer_type = null, $dispatcher = null)
{

    //$getLoadUnoccupancy = getLoadUnoccupancy($startDate,$endDate,$driverId,$load,$trailer_type,$dispatcher);
    $getDatePickupCount = getDatePickupCountMonth($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher);
    $period = CarbonPeriod::create($startDate, $endDate);
    $totalDays = 0;
    $bcoutn = 0;
    foreach ($period as $date) {

        $dateYMD = $date->format('m/d/Y');
        $getload = getLoadByDateMonth($dateYMD, $driverId, $trailer_type, $dispatcher, $driverId);
        if (count($getload) > 0) {
            $load_type = array(0);
            foreach ($getload as $load_details) {
                if (in_array($load_details['load_type'], $load_type)) {
                    $totalDays++;
                    break;
                }
            }
        }
        if (count($getload) > 1) {
            $load_type = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
            $uc = 0;
            $oc = 0;
            foreach ($getload as $load_details) {
                if (in_array($load_details['load_type'], $load_type)) {
                    $uc++;
                } else {
                    $oc++;
                }
            }
            if ($oc > 0 && $uc > 0) {
                $bcoutn++;
            }
        }
    }
    $totalDays = $totalDays + $getDatePickupCount;
    return $totalDays = $totalDays - $bcoutn;

}

function getloadCountByDriverId($startDate, $endDate, $driver_id)
{
    $data = LoadDetail::select("load_details.load_id")
        ->join("drivers", "drivers.id", "=", "load_details.driver_id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driver_id)
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNull('load_details.deleted_at')
        ->count();

    return $data;
}

function getloadCount($startDate, $endDate, $load_type, $driver_id)
{
    $data = LoadDetail::select("load_details.load_id")
        ->join("drivers", "drivers.id", "=", "load_details.driver_id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driver_id)
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->where('load_details.load_type', $load_type)
        ->whereNull('load_details.deleted_at')
        ->count();
    return $data;
}

function getloadCountDayByStatus($startDate, $endDate, $load_type, $driver_id)
{

    $loadDetails = Driver::select("load_details.*", "load_details.id as load_details_id", "load_drivers.load_detail_id")
            ->join("load_details", "load_details.driver_id", "=", "drivers.id")
            ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
            ->whereNull('load_details.deleted_at')
            ->where('load_details.load_status', '!=', 9)
            ->where(function($q) {
                $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
            })
            ->where('drivers.id', $driver_id)
            ->where('load_details.load_type', $load_type)
            ->where('load_drivers.driver_id', $driver_id)
            ->where('drivers.company_id', auth()->guard('company-admin')->user()->company_id)
            ->where(function ($query) use ($endDate, $startDate) {
                $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate);
            })
            ->groupBy('load_drivers.load_detail_id')
            ->get();



    $dayCount = 0;
    if(count($loadDetails) > 0){
        foreach($loadDetails as $load){
            $startDateLoad = Carbon::parse($load->actual_shipper_date)->format('Y-m-d');
            $endDateLoad = Carbon::parse($load->actual_delivery_date)->format('Y-m-d');

            if($startDateLoad <= $startDate){
                $startDateLoad = $startDate;
            }

            if($endDateLoad >= $endDate){
                $endDateLoad = $endDate;
            }

            $period = CarbonPeriod::create($startDateLoad, $endDateLoad);

            $dcount = 0;
            $ky=0;
            foreach ($period as $date) {

                $load_type = array(0, 4, 6); // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
                if (in_array($load->load_type, $load_type)) {
                    if($ky != 0){
                        $dcount += 1;
                    }

                }

                $load_type = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
                if (in_array($load->load_type, $load_type)) {
                    $dcount += 1;
                }



                $ky++;
            }
            if($load->occupied_pickup_day == 1){
                $dcount += 1;
            }

            $dayCount += $dcount;
        }
    }

    //dd($dayCount);
    return $dayCount;
}

function getDriverTargetRevenue($startDate, $endDate, $driver_id)
{
    $get_start_quarter = Carbon::parse($startDate)->quarter;
    $get_start_year = Carbon::parse($startDate)->year;
    $get_end_quarter = Carbon::parse($endDate)->quarter;
    $get_end_year = Carbon::parse($endDate)->year;

    $data = DriverFinancial::where("driver_id", $driver_id)
        ->where("year", '>=', $get_start_year)
        ->where("year", '<=', $get_end_year)
        ->where("quarter", '>=', $get_start_quarter)
        ->where("quarter", '<=', $get_end_quarter)
        ->sum('revenue_target');
    return $data;
}


function getDriverTargetRevenuetracker($startDate, $endDate, $driver_id)
{
    $get_start_quarter = Carbon::parse($startDate)->quarter;
    $get_start_year = Carbon::parse($startDate)->year;
    $get_end_quarter = Carbon::parse($endDate)->quarter;
    $get_end_year = Carbon::parse($endDate)->year;

    $data = DriverFinancial::where("driver_id", $driver_id)
        ->where("year", '>=', $get_start_year)
        ->where("year", '<=', $get_end_year)
        ->where("quarter", '>=', $get_start_quarter)
        ->where("quarter", '<=', $get_end_quarter)
        ->sum('revenue_target');
    return $data;
}

/********Create By Dhruvish***********/
function getDrivertarget_milestracker($startDate, $endDate, $driver_id)
{
    //07-april-2022 $get_start_quarter = Carbon::parse($startDate)->quarter;
    // $get_start_year = Carbon::parse($startDate)->year;
    // $get_end_quarter = Carbon::parse($endDate)->quarter;
    // $get_end_year = Carbon::parse($endDate)->year;

    // $data = DriverFinancial::where("driver_id", $driver_id)
    //         ->where("year",'>=', $get_start_year)
    //         ->where("year",'<=', $get_end_year)
    //         ->where("quarter",'>=', $get_start_quarter)
    //         ->where("quarter",'<=', $get_end_quarter)
    //         ->sum('target_miles');
    // return $data;

    $get_start_quarter = Carbon::parse($startDate)->quarter;
    $get_start_year = Carbon::parse($startDate)->year;
    $get_end_quarter = Carbon::parse($endDate)->quarter;
    $get_end_year = Carbon::parse($endDate)->year;

    //exit();

    $data = DriverFinancial::where("driver_id", $driver_id)
        ->where(function ($query) use ($startDate, $endDate) {
            $query->whereBetween('start_date', [$startDate, $endDate])
                ->orWhereBetween('end_date', [$startDate, $endDate]);
        })
        ->get();

    //dd($data);

    $getval_per_quarter = [];
    $diff_in_quarterdaysdd = [];
    foreach ($data as $divData) {

        //dd($getPeriod_month,$getPeriod);

        $to = Carbon::createFromFormat('Y-m-d', $divData->start_date);
        $from = Carbon::createFromFormat('Y-m-d', $divData->end_date);
        $diff_in_quarterdays = $to->diffInDays($from);

        $diff_in_quarterdaysdd[] = $diff_in_quarterdays;

        $getval_per_days = ($divData->target_miles) / $diff_in_quarterdays;

        $to_d = Carbon::createFromFormat('Y-m-d', $startDate);
        $from_d = Carbon::createFromFormat('Y-m-d', $endDate);
        $diff_in_days = $to_d->diffInDays($from_d) + 1;


        //dd($diff_in_days,$diff_in_quarterdays);

        if (($divData->start_date >= $startDate) && ($divData->end_date >= $endDate)) {
            if (array_sum($diff_in_quarterdaysdd) > $diff_in_days) {

                //$mindays =  array_sum($diff_in_quarterdaysdd) - ($diff_in_days);

                $mindays = $diff_in_days;

            } else {

                $mindays = ($diff_in_days) - array_sum($diff_in_quarterdaysdd);

            }

            //dd($getval_per_days,$mindays);

            $getval_per_quarter[] = ($getval_per_days) * ($mindays);

        } else {

            $getval_per_quarter[] = ($getval_per_days) * ($diff_in_quarterdays);

        }

        //$i++;
    }


    //dd($getval_per_quarter,$diff_in_quarterdaysdd);

    $getTOT = array_sum($getval_per_quarter);

    // dd($getTOT);

    return $getTOT;


}


function getDrivercpm_targettracker($startDate, $endDate, $driver_id)
{
    $get_start_quarter = Carbon::parse($startDate)->quarter;
    $get_start_year = Carbon::parse($startDate)->year;
    $get_end_quarter = Carbon::parse($endDate)->quarter;
    $get_end_year = Carbon::parse($endDate)->year;

    $data = DriverFinancial::where("driver_id", $driver_id)
        ->where("year", '>=', $get_start_year)
        ->where("year", '<=', $get_end_year)
        ->where("quarter", '>=', $get_start_quarter)
        ->where("quarter", '<=', $get_end_quarter)
        ->sum('cpm_target');
    return $data;
}


function getDriveroccupancytracker($startDate, $endDate, $driver_id)
{
    $get_start_quarter = Carbon::parse($startDate)->quarter;
    $get_start_year = Carbon::parse($startDate)->year;
    $get_end_quarter = Carbon::parse($endDate)->quarter;
    $get_end_year = Carbon::parse($endDate)->year;

    $data = DriverFinancial::where("driver_id", $driver_id)
        ->where("year", '>=', $get_start_year)
        ->where("year", '<=', $get_end_year)
        ->where("quarter", '>=', $get_start_quarter)
        ->where("quarter", '<=', $get_end_quarter)
        ->sum('occupancy');
    return $data;
}


function getDriverdriver_revenueDatatracker($startDate, $endDate, $driver_id)
{
    $get_start_quarter = Carbon::parse($startDate)->quarter;
    $get_start_year = Carbon::parse($startDate)->year;
    $get_end_quarter = Carbon::parse($endDate)->quarter;
    $get_end_year = Carbon::parse($endDate)->year;

    // $data =  LoadDetail::select("load_details.load_id")
    //         ->join("drivers","drivers.id","=","load_details.driver_id")
    //         ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'),[$startDate, $endDate])
    //         ->where("drivers.id", $driver_id)
    //         ->get();


    $data = LoadDetail::whereNull('deleted_at')->where('driver_id', $driver_id)->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])->pluck('driver_revenue')->toArray();

    return $data;
}


function getDriverTargetOccupancy($startDate, $endDate, $driver_id)
{
    $get_start_quarter = Carbon::parse($startDate)->quarter;
    $get_start_year = Carbon::parse($startDate)->year;
    $get_end_quarter = Carbon::parse($endDate)->quarter;
    $get_end_year = Carbon::parse($endDate)->year;

    $data = DriverFinancial::where("driver_id", $driver_id)
        ->where("year", '>=', $get_start_year)
        ->where("year", '<=', $get_end_year)
        ->where("quarter", '>=', $get_start_quarter)
        ->where("quarter", '<=', $get_end_quarter)
        ->sum('occupancy');
    return $data;
}

function getLoadColorClassMonth($load_type)
{
    // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH, 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
    $class_name = [0 => 'green', 6 => 'warning', 4 => 'grey', 1 => 'blue', 2 => 'darkred', 3 => 'yellow', 5 => 'red'];
    return isset($class_name[$load_type]) ? $class_name[$load_type] : 'grey';
}

function todayHoliday($date)
{

    $data = CompanyHoliday::where('company_id', auth()->guard('company-admin')->user()->company_id)
        ->where('from_date', '<=', $date)
        ->where('to_date', '>=', $date)
        ->pluck('name')
        ->toArray();

    return implode(',', $data);

}

function HolidayByDate($startDate, $endDate)
{

    $data = CompanyHoliday::where('company_id', auth()->guard('company-admin')->user()->company_id)
        ->where('from_date', '==', $startDate)
        ->where('to_date', '==', $endDate)
        ->get();
    return $data;

}

function draw_calendar_year($month, $year, $driver_id, $showtype)
{

    $startDate = $year . '-' . $month . '-01';
    $endDate = $year . '-' . $month . '-31';

    $fmt = new \NumberFormatter('en_US', \NumberFormatter::CURRENCY);

    $loadDetails = LoadDetail::where('load_details.driver_id', $driver_id)
        ->join("load_drivers as ld", "ld.load_detail_id", "=", "load_details.id")
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
            ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
            ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
            ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNull('load_details.deleted_at')
        //->select('load_details.id as id','load_details.driver_id','load_details.actual_shipper_date','load_details.actual_delivery_date','load_details.load_status','load_details.load_type','load_details.occupied_pickup_day','load_details.driver_revenue','ld.miles')
        ->select('load_details.id as id','load_details.driver_id','load_details.actual_shipper_date','load_details.actual_delivery_date','load_details.load_status','load_details.load_type','load_details.occupied_pickup_day','load_details.driver_revenue','ld.miles')
        ->get();

    $holidays = HolidayByDate($startDate, $endDate);


    $period = CarbonPeriod::create($startDate, $endDate);
    $totalDays = 0;
    $bcoutn = 0;
    $daycount = 0;
    $uc = 0;
    $oc = 0;
    /*foreach($loadDetails as $load){
        if(in_array($load->load_type, [0, 4, 6]) && $load->occupied_pickup_day == 1){
                $totalDays++;

            }
    }*/


    foreach ($period as $date) {

        $date = $date->format('Y-m-d');
        $laod_array = [];
        foreach($loadDetails as $load){
            $actual_shipper_date = Carbon::createFromFormat(getTimeLayout(),$load->actual_shipper_date)->format('Y-m-d');
            $actual_delivery_date = Carbon::createFromFormat(getTimeLayout(),$load->actual_delivery_date)->format('Y-m-d');

            if(in_array($load->load_type, [0, 4, 6]) && $load->occupied_pickup_day == 0 && $actual_shipper_date < $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
                $laod_array[$load->id] = $load;
            }
            if(in_array($load->load_type, [0, 4, 6]) && $load->occupied_pickup_day == 1 && $actual_shipper_date <= $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
                $laod_array[$load->id] = $load;
            }
            if(in_array($load->load_type, [1, 2, 3, 5]) && $actual_shipper_date <= $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
                $laod_array[$load->id] = $load;
            }
        }

        $getload = [];
        $ki=0;
        foreach($laod_array as $key => $larray){
            $getload[$ki] = $larray;
            $ki++;
        }

        if (count($getload) > 0) {
            $load_type = array(0, 4, 6); // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
            foreach ($getload as $load_details) {
                if (in_array($load_details['load_type'], $load_type)) {
                    $totalDays++;
                    break;
                }
            }
            $daycount++;
        }
        if (count($getload) > 1) {
            $load_type = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
            $uc = 0;
            $oc = 0;
            foreach ($getload as $load_details) {
                if (in_array($load_details['load_type'], $load_type)) {
                    $uc++;
                } else {
                    $oc++;
                }
            }
            if ($oc > 0 && $uc > 0) {
                $bcoutn++;
            }
            $daycount++;
        }

    }

    //dd($totalDays, $uc, $oc, $bcoutn, $daycount);
    $totalDays = $totalDays - $bcoutn;
    $getLoadOccupancy = 0;
    if($daycount > 0){
        $getLoadOccupancy = round(max(number_format(100 / $daycount * $totalDays, 2), 0));
    }

    $calendar = '';
    if ($showtype == 'year' || $showtype == 'quarterly') {
        $calendar = '<div class=" col-lg-4 col-sm-6 ">';
    }

    $calendar .= '<div class="calendar-content">';

    if ($showtype == 'year' || $showtype == 'quarterly') {
        if ($getLoadOccupancy > 74) {
            $calendar .= '<h6>' . Carbon::create()->month($month)->format('F') . '<span>' . $getLoadOccupancy . '% <img src="' . asset("theme/img/arrow-up.svg") . '" alt="" class="img-fluid" /> </span></h6>';
        } else {
            $calendar .= '<h6>' . Carbon::create()->month($month)->format('F') . '<span>' . $getLoadOccupancy . '% <img src="' . asset("theme/img/arrow-down.svg") . '" alt="" class="img-fluid" /> </span></h6>';
        }

    }


    $calendar .= '<div class="calendar-body">';
    if ($showtype == 'year' || $showtype == 'quarterly') {
        $headings = array('S', 'M', 'T', 'W', 'T', 'F', 'S');
    } else {
        $headings = array('Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday');
    }
    $calendar .= '<div class="calendar-days">';
    foreach ($headings as $days) {
        $calendar .= '<div>' . $days . '</div>';
    }
    $calendar .= '</div>';


    $calendar .= '<div class="calendar-dates-no">';

    $running_day = date('w', mktime(0, 0, 0, $month, 1, $year));
    $days_in_month = date('t', mktime(0, 0, 0, $month, 1, $year));
    $days_in_this_week = 1;
    $day_counter = 0;
    $dates_array = array();


    for ($x = 0; $x < $running_day; $x++):
        if ($showtype == 'year' || $showtype == 'quarterly') {
            $calendar .= '<div class="calendar-date text-gary"></div>';
        } else {
            $calendar .= '<div class="calendar-date"></div>';
        }
        $days_in_this_week++;
    endfor;


    $getMile = [];
    $getRevenue = [];
    $mi=0;

    for ($list_day = 1; $list_day <= $days_in_month; $list_day++):

        $sdate = $year . '-' . $month . '-' . $list_day;
        $getdate = Carbon::parse($sdate)->format('Y-m-d');

        $date = $getdate;

        $data_check = [];
        foreach($loadDetails as $load){
            $actual_shipper_date = Carbon::createFromFormat(getTimeLayout(),$load->actual_shipper_date)->format('Y-m-d');
            $actual_delivery_date = Carbon::createFromFormat(getTimeLayout(),$load->actual_delivery_date)->format('Y-m-d');

            if(in_array($load->load_type, [0, 4, 6]) && $load->occupied_pickup_day == 0 && $actual_shipper_date < $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
                $data_check[$load->id] = $load;
            }
            if(in_array($load->load_type, [0, 4, 6]) && $load->occupied_pickup_day == 1 && $actual_shipper_date <= $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
                $data_check[$load->id] = $load;
            }
            if(in_array($load->load_type, [1, 2, 3, 5]) && $actual_shipper_date <= $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
                $data_check[$load->id] = $load;
            }
        }
        $ki=0;
        $data = [];
        foreach($data_check as $key => $larray){
            $data[$ki] = $larray;
            $getRevenue[$key] = $larray->driver_revenue;
            $getMile[$key] = $larray->miles;
            $ki++;
            $mi++;
        }

        $holiday_name = [];

        if(count($holidays->all()) > 0){
            $k = 0;
            foreach($holidays as $holiday){

                $holiday_start_date = Carbon::createFromFormat('m/d/Y',$holiday->from_date)->format('Y-m-d');
                $holiday_end_date = Carbon::createFromFormat('m/d/Y',$holiday->to_date)->format('Y-m-d');
                if($holiday_start_date <= $date && $holiday_end_date >= $date){
                    $holiday_name[$k] = $holiday->name;
                }

                $k++;
            }
            dd($holiday_name);
        }



        $holidaysHTML = '';
        if (count($holiday_name) > 0) {
            $holidays = implode(',',$holiday_name);
            $holidaysHTML = '<a href="javascript:void(0)" class="holid hidden" data-toggle="tooltip" data-placement="right" title="' . $holidays . '" ><i class="fa fa-glass"></i></a>';
        }

        if (count($data) == 1) {
            if ($showtype == 'year' || $showtype == 'quarterly') {
                $calendar .= '<div class="calendar-date ' . $data[0]->load_type . ' bg-' . getLoadColorClassMonth($data[0]->load_type) . '"><span class="date-text">' . $list_day . ' ' . $holidaysHTML . '</span></div>';
            } else {
                $calendar .= '<div class="calendar-date ' . $data[0]->load_type . ' bg-' . getLoadColorClassMonth($data[0]->load_type) . ' occu-type-key-text">';
                $calendar .= '<span class="date-text">' . $list_day . ' ' . $holidaysHTML . '</span>';
                //$calendar .= '<a target="_blank" href="' . route('company.admin.loads.show', base64Encode($data[0]->id)) . '"><p>' . getLoadTypeList()[$data[0]->load_type] . '</p></a>';
                $calendar .= '<a href="javascript:void(0);" class="viewLoadDetailsByDate"  data-date="'.$date.'" data-load="'.$data[0]->id.'"><p>' . getLoadTypeList()[$data[0]->load_type] . '</p></a>';
                $calendar .= '</div>';
            }
        } elseif (count($data) > 1) {
            if ($showtype == 'year' || $showtype == 'quarterly') {
                $calendar .= '<div class="calendar-date multiple-loads"><span class="date-text">' . $list_day . '</span></div>';
            } else {
                $calendar .= '<div class="calendar-date multiple-loads">';
                $calendar .= '<span class="date-text">' . $list_day . '</span>';
                $calendar .= '<div class="ecensew">';

                /*
                foreach ($data as $data) {
                    $calendar .= '<a class="cal-text-' . getLoadColorClassMonth($data->load_type) . '" target="_blank" href="' . route('company.admin.loads.show', base64Encode($data->id)) . '">' . getLoadTypeList()[$data->load_type] . '</a>';
                }
                */

                $dataLoads = []; $existLoadType = [];
                foreach ($data as $key => $values) {
                    $dataLoads[] = $values->id;
                }
                foreach ($data as $key => $values) {
                    if(!in_array(getLoadTypeList()[$values->load_type], $existLoadType)){
                        $existLoadType[] = getLoadTypeList()[$values->load_type];

                        $calendar .= '<a href="javascript:void(0);" class="cal-text-' . getLoadColorClassMonth($values->load_type) . ' viewLoadDetailsByDate" data-date="'.$date.'" data-load="'.implode(',', $dataLoads).'">' . getLoadTypeList()[$values->load_type] . '</a>';
                    }
                }
                $calendar .= '</div></div>';
            }
        } else {
            $calendar .= '<div class="calendar-date"><span class="date-text">' . $list_day . ' ' . $holidaysHTML . '</span></div>';
        }

        if ($running_day == 6):
            //$calendar.= '</tr>';
            if (($day_counter + 1) != $days_in_month):
                //$calendar.= '<tr class="calendar-row">';
            endif;
            $running_day = -1;
            $days_in_this_week = 0;
        endif;
        $days_in_this_week++;
        $running_day++;
        $day_counter++;
    endfor;




    if ($days_in_this_week < 8):
        for ($x = 1; $x <= (8 - $days_in_this_week); $x++):
            if ($showtype == 'year' || $showtype == 'quarterly') {
                $calendar .= '<div class="calendar-date text-gary"></div>';
            } else {
                $calendar .= '<div class="calendar-date"></div>';
            }
        endfor;
    endif;

    $getRevenue = array_sum($getRevenue);
    $getMile = array_sum($getMile);
    $mileprice = 0;
    if ($getRevenue > 0) {
        $mileprice = $getRevenue / $getMile;
    }

    $calendar .= '</div>';
    $calendar .= '</div>';
    if ($showtype == 'year' || $showtype == 'quarterly') {
        $calendar .= '<div class="calender-fotter"><a>Rev ' . $fmt->formatCurrency($getRevenue, "USD") . ' | ' . $fmt->formatCurrency($mileprice, "USD") . ' Mile</a></div>';
        //$calendar .= '<div class="calender-fotter"><a>Rev TTT | TTT Mile</a></div>';
    }
    $calendar .= '</div>';
    if ($showtype == 'year' || $showtype == 'quarterly') {
        $calendar .= '</div>';
    }


    return $calendar;
}


function draw_calendar_month_days_name($month, $year)
{

    $startDate = $year . '-' . $month . '-01';
    $endDate = $year . '-' . $month . '-31';

    $get_gross_revenue = getLoadGrossRevenue($startDate, $endDate, $driver_id);
    $getLoadMileage = getLoadMileage($startDate, $endDate, $driver_id);

    $getLoadOccupancy = 0;
    $getLoadOccupancy = round(getLoadOccupancyCalendarMonth($startDate, $endDate, $driver_id));

    $mileprice = 0;
    if ($get_gross_revenue > 0) {
        $mileprice = $get_gross_revenue / $getLoadMileage;
    }

    $fmt = new \NumberFormatter('en_US', \NumberFormatter::CURRENCY);

    if ($showtype == 'year' || $showtype == 'quarterly') {
        $headings = array('S', 'M', 'T', 'W', 'T', 'F', 'S');
    } else {

    }
    $headings = array('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
    $calendar .= '<div class="calendar-days">';
    foreach ($headings as $days) {
        $calendar .= '<div>' . $days . '</div>';
    }
    $calendar .= '</div>';


    /* all done, return result */
    return $calendar;
}


function draw_calendar_weekly($date, $driver_id)
{

    $list_day = Carbon::createFromFormat('Y-m-d', $date)->format('m/d/Y');
    $calendar = '';

    $data = getLoadByDaterevenue($date, $driver_id);

    $holidays = todayHoliday($date);
    $holidaysHTML = '';
    if (!empty($holidays)) {
        $holidaysHTML = '<a href="javascript:void(0)" class="holid hidden" data-toggle="tooltip" data-placement="right" title="' . $holidays . '" ><i class="fa fa-glass"></i></a>';
    }

    if (count($data) == 1) {
        $calendar .= '<div class="calendar-date ' . $data[0]->load_type . ' bg-' . getLoadColorClassMonth($data[0]->load_type) . ' occu-type-key-text"><span class="date-text">' . $list_day . ' ' . $holidaysHTML . '</span><a target="_blank" href="' . route('company.admin.loads.show', base64Encode($data[0]->load_id)) . '"><p>' . getLoadTypeList()[$data[0]->load_type] . '</p></a></div>';

    } elseif (count($data) > 1) {

        $calendar .= '<div class="calendar-date multiple-loads"><span class="date-text">' . $list_day . ' ' . $holidaysHTML . '</span><div class="ecensew">';
        foreach ($data as $data) {
            $calendar .= '<a class="cal-text-' . getLoadColorClassMonth($data->load_type) . '" target="_blank" href="' . route('company.admin.loads.show', base64Encode($data->id)) . '">' . getLoadTypeList()[$data->load_type] . '</a>';
        }
        $calendar .= '</div></div>';
    } else {
        $calendar .= '<div class="calendar-date"><span class="date-text">' . $list_day . ' ' . $holidaysHTML . '</span></div>';
    }


    return $calendar;
}


function getLoadTypeListSmall()
{
    return ['0' => 'o', '1' => 'm', '2' => 'u', '3' => 'h', '4' => 'n', '5' => 't', '6' => 'tn'];
}

function liner_fleet_calendar($monthdata, $driver_id)
{
    $startDate      = $monthdata[0]['date'];
    $lastArray      = end($monthdata);
    $endDate        = $lastArray['date'];

    $loadDetails    = LoadDetail::select('id','driver_id','actual_shipper_date','actual_delivery_date','load_status','load_type','occupied_pickup_day')
        ->where('driver_id', $driver_id)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
            ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
            ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
            ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('is_disable', 0)
        ->where('load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0,1]);
        })
        ->get();

    $calendar       = '';
    foreach ($monthdata as $mdata){
        $daynumber  = $mdata['daynumber'];
        $date       = $mdata['date'];
        $calendar2  = '';
        $laod_array = [];
        foreach($loadDetails as $load){
            $actual_shipper_date        = Carbon::createFromFormat(getTimeLayout(),$load->actual_shipper_date)->format('Y-m-d');
            $actual_delivery_date       = Carbon::createFromFormat(getTimeLayout(),$load->actual_delivery_date)->format('Y-m-d');

            if(in_array($load->load_type, [0, 4, 6]) && $load->occupied_pickup_day == 0 && $actual_shipper_date < $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
                $laod_array[$load->id]  = [$load->load_type];
            }
            if(in_array($load->load_type, [0, 4, 6]) && $load->occupied_pickup_day == 1 && $actual_shipper_date <= $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
                $laod_array[$load->id]  = [$load->load_type];
            }
            if(in_array($load->load_type, [1, 2, 3, 5]) && $actual_shipper_date <= $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
                $laod_array[$load->id]  = [$load->load_type];
            }
        }

        if(count($laod_array) > 0){
            if (count($laod_array) == 1) {
                foreach($laod_array as $key => $larray){
                    $calendar           .= '<td class="sinlod ' . getLoadTypeListSmall()[$larray[0]] . '-box dayn' . $daynumber . ' driv' . $driver_id . '">';
                    $calendar           .= '<a target="_blank" href="' . route('company.admin.loads.show', base64Encode($key)) . '">' . getLoadTypeListSmall()[$larray[0]] . '</a>';
                    $calendar           .= '</td>';
                }

            } elseif (count($laod_array) > 1) {
                $loadtype               = [];
                $ki                     = 0;
                foreach($laod_array as $key => $larray){
                    $loadtype[$ki]      = $larray[0];
                    $ki++;
                }

                if ($loadtype[0] == 0 && $loadtype[1] == 0) {
                    $calendar           .= '<td style="background-color: transparent;" class="o-box multilod dayn' . $daynumber . ' driv' . $driver_id . '">';
                } else {
                    $calendar           .= '<td class="multilod dayn' . $daynumber . ' driv' . $driver_id . '">';
                }

                foreach($laod_array as $key => $larray){
                    $calendar           .= '<a class="cal-text-' . getLoadColorClassMonth($larray[0]) . '" target="_blank" href="' . route('company.admin.loads.show', base64Encode($key)) . '">' . getLoadTypeListSmall()[$larray[0]] . '</a>';
                }
                $calendar               .= '</td>';
            }else{
                //
            }
        }
        if(count($laod_array) == 0){
            $date                       = Carbon::parse($date)->format('m/d/Y H:i');
            //$calendar                 .= '<td><a href="'.route('company.admin.loads.create').'?driver_id='.$driver_id.'&shipper_date='.$date.'"><i class="fa fa-plus-circle"></i></a></td>';
            $calendar                   .= '<td><a href="javascript:void(0);" class="addLoadPlus" data-from="liner_fleet_calendar" data-driver_id="'.$driver_id.'" data-shipper_date="'.date('m/d/Y', strtotime($date)).'" data-url="'.route('company.admin.loads.create').'?driver_id='.$driver_id.'&shipper_date='.$date.'"><i class="fa fa-plus-circle"></i></a></td>';
        }
    }
    return $calendar;
}

function leave_calendar($data_type, $data_type_id, $date, $daynumber = null)
{
    if($data_type == 'driver'){
        $getData = Leave::where('driver_id', $data_type_id)
            ->where('company_id', auth()->guard('company-admin')->user()->company_id)
            ->where(function ($query) use ($date) {
                $query->where(DB::raw('DATE_FORMAT(from_date, "%Y-%m-%d")'), '<', $date)
                    ->where(DB::raw('DATE_FORMAT(to_date, "%Y-%m-%d")'), '>=', $date)
                    ->orWhere(DB::raw('DATE_FORMAT(to_date, "%Y-%m-%d")'), $date);
            })
            ->get();

        $getData2 = Leave::where('driver_id', $data_type_id)
            ->where('company_id', auth()->guard('company-admin')->user()->company_id)
            ->where(function ($query) use ($date) {
                $query->where(DB::raw('DATE_FORMAT(from_date, "%Y-%m-%d")'), '<=', $date)
                ->where(DB::raw('DATE_FORMAT(to_date, "%Y-%m-%d")'), '>=', $date)
                ->orWhere(DB::raw('DATE_FORMAT(to_date, "%Y-%m-%d")'), $date);
            })
            ->get();

        if (count($getData2) > 0) {
            $getData = $getData2->merge($getData);
        }
    }

    if($data_type == 'dispatchers' || $data_type == 'managers'){
        $getData = Leave::where('company_admins_id', $data_type_id)
            ->where('company_id', auth()->guard('company-admin')->user()->company_id)
            ->where(function ($query) use ($date) {
                $query->where(DB::raw('DATE_FORMAT(from_date, "%Y-%m-%d")'), '<', $date)
                    ->where(DB::raw('DATE_FORMAT(to_date, "%Y-%m-%d")'), '>=', $date)
                    ->orWhere(DB::raw('DATE_FORMAT(to_date, "%Y-%m-%d")'), $date);
            })
            ->get();

        $getData2 = Leave::where('company_admins_id', $data_type_id)
            ->where('company_id', auth()->guard('company-admin')->user()->company_id)
            ->where(function ($query) use ($date) {
                $query->where(DB::raw('DATE_FORMAT(from_date, "%Y-%m-%d")'), '<=', $date)
                ->where(DB::raw('DATE_FORMAT(to_date, "%Y-%m-%d")'), '>=', $date)
                ->orWhere(DB::raw('DATE_FORMAT(to_date, "%Y-%m-%d")'), $date);
            })
            ->get();

        if (count($getData2) > 0) {
            $getData = $getData2->merge($getData);
        }
    }

    $calendar = '';
    if (count($getData) == 1) {
        $calendar .= '<td style="background-color:'.$getData[0]->leave_type->color.';color:#fff;"><a href="javascript:void(0)" style="color: #fff;" class="show-leave-details" data-leave-id="'.$getData[0]->id.'" data-toggle="modal" data-target="#leaveDetailsModal"> ' . substr($getData[0]->leave_type->name, 0, 1) . '</td>';
    } elseif (count($getData) > 1) {
        $calendar .= '<td>';
        foreach ($getData as $data) {
            $calendar .= '<a href="javascript:void(0)" class="show-leave-details" data-leave-id="'.$data->id.'" data-toggle="modal" data-target="#leaveDetailsModal" style="color:'.$data->leave_type->color.';"  >' . substr($data->leave_type->name, 0, 1) . '</a>';
        }
        $calendar .= '</td>';
    } else {
        $calendar .= '<td></td>';
    }
    return $calendar;
}


function getWeekRange($cyear, $cmonth)
{

    $today = Carbon::createFromDate($cyear, $cmonth, '01',getTimeZone());

    $date = $today->copy()->firstOfMonth()->startOfDay();
    $eom = $today->copy()->endOfMonth()->startOfDay();


    $dates = [];

    for ($i = 1; $date->lte($eom); $i++) {
        $datenumber = [];

        $datenumber[] = $date->copy()->format('d');
        while ($date->dayOfWeek != Carbon::SATURDAY && $date->lte($eom)) {
            $datenumber[] = $date->addDay()->format('d');
        }

        $dates[$i] = $datenumber;
        $date->addDay();
    }

    return $dates;
}


function getlandingpageData($driversId, $descType)
{


    $dataget_schedu_orien = DriverComplianceSafety::with(['driverData' => function ($query) {
        $query->select('id', 'first_name', 'last_name');
    }])->whereIn('driver_id', $driversId)->whereNotNull($descType)->orderBy($descType, 'DESC')->take(5)->get();

    return $dataget_schedu_orien;
}

function getEventColor($EventTypeName)
{

    $getcolor = EventType::where('name', $EventTypeName)->first();
    return isset($getcolor->color) ? $getcolor->color : '#7F8C8D';
}

function getDriverEventFullCalendar_backup($driversId,$calendarEvent=null,
                                    $birthdays=null,
                                    $expirations=null,
                                    $maintenanceTrucks=null,
                                    $maintenanceTrailers=null,
                                    $hidePersonalEvents=null,
                                    $IsArray=null,
                                    $myLeaves=null,
                                    $myViolations=null,
                                    $maintenanceTicket=null){
    $getEvent = Event::where('driver_id', $driversId)->whereNull('share')->orWhere('share',1)->get();

    $getShareEvent = Event::where('driver_id', $driversId)->where('share',2)->get();

    $driver = Driver::withTrashed()->where('id',$driversId)->first();
    $getdriverexp = $driver->driver_compliance_safety;



    $i=0;
    $event = "[";
    $event_array = [];
    if($hidePersonalEvents == 'on'){
        foreach($getEvent as $sevent) {
            $Checktime = Carbon::parse($sevent->to_date)->format('H:i');

            $from_date = $sevent->from_date;
            $to_date = $sevent->to_date;
            if($Checktime == "00:00"){
                $from_date = Carbon::parse($sevent->from_date)->format('Y-m-d');
                if($sevent->is_repeat == 1){
                    $to_date = Carbon::parse($sevent->to_date)->addYear(2)->format('Y-m-d');
                }else{
                    $to_date = Carbon::parse($sevent->to_date)->format('Y-m-d');
                }
            }else{
                $from_date = Carbon::parse($sevent->from_date)->format('Y-m-d H:i');

                if($sevent->is_repeat == 1){
                    $to_date = Carbon::parse($sevent->to_date)->addYear(2)->format('Y-m-d H:i');
                }else{
                    $to_date = Carbon::parse($sevent->to_date)->format('Y-m-d H:i');
                }

            }

            $color = getEventColor($sevent->type);
            if($sevent->share == 1){
                $id = 0;
            }else{
                $id = $sevent->id;
            }

            $event .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";

            $event_array[] = ['title' => $sevent->name,'start' => $from_date,'end' => $to_date,'icon' => $sevent->fa_icon];

            /******For Repeat*****/
            if(!empty($sevent->is_repeat) && $sevent->is_repeat != 1){

                /******Weeks*****/
                if($sevent->is_repeat == 2){
                    for ($x = 1; $x <= 24; $x++) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($sevent->from_date)->addWeeks($x)->format('Y-m-d');
                            $to_date = Carbon::parse($sevent->to_date)->addWeeks($x)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($sevent->from_date)->addWeeks($x)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($sevent->to_date)->addWeeks($x)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";
                        $event_array[] = ['title' => $sevent->name,'start' => $from_date,'end' => $to_date,'icon' => $sevent->fa_icon];
                    }
                }
                /******Month*****/
                if($sevent->is_repeat == 3){
                    for ($x = 1; $x <= 24; $x++) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($sevent->from_date)->addMonths($x)->format('Y-m-d');
                            $to_date = Carbon::parse($sevent->to_date)->addMonths($x)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($sevent->from_date)->addMonths($x)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($sevent->to_date)->addMonths($x)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";
                        $event_array[] = ['title' => $sevent->name,'start' => $from_date,'end' => $to_date,'icon' => $sevent->fa_icon];
                    }
                }
                /******Six Month*****/
                if($sevent->is_repeat == 4){
                    $six = [6,12,18,24,30,36];
                    foreach($six as $sixa) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($sevent->from_date)->addMonths($sixa)->format('Y-m-d');
                            $to_date = Carbon::parse($sevent->to_date)->addMonths($sixa)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($sevent->from_date)->addMonths($sixa)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($sevent->to_date)->addMonths($sixa)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";
                        $event_array[] = ['title' => $sevent->name,'start' => $from_date,'end' => $to_date,'icon' => $sevent->fa_icon];
                    }
                }
                /******Years*****/
                if($sevent->is_repeat == 5){
                    for ($x = 1; $x <= 6; $x++) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($sevent->from_date)->addYear($x)->format('Y-m-d');
                            $to_date = Carbon::parse($sevent->to_date)->addYear($x)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($sevent->from_date)->addYear($x)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($sevent->to_date)->addYear($x)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";
                        $event_array[] = ['title' => $sevent->name,'start' => $from_date,'end' => $to_date,'icon' => $sevent->fa_icon];
                    }
                }

            }
            /****** End Repeat*****/


            $i++;
        }
    }

    if($calendarEvent == 1){
        foreach($getShareEvent as $shareent) {
            $Checktime = Carbon::parse($shareent->to_date)->format('H:i');

            $from_date = $shareent->from_date;
            $to_date = $shareent->to_date;
            if($Checktime == "00:00"){
                $from_date = Carbon::parse($shareent->from_date)->format('Y-m-d');
                if($shareent->is_repeat == 1){
                    $to_date = Carbon::parse($shareent->to_date)->addYear(2)->format('Y-m-d');
                }else{
                    $to_date = Carbon::parse($shareent->to_date)->format('Y-m-d');
                }
            }else{
                $from_date = Carbon::parse($shareent->from_date)->format('Y-m-d H:i');

                if($shareent->is_repeat == 1){
                    $to_date = Carbon::parse($shareent->to_date)->addYear(2)->format('Y-m-d H:i');
                }else{
                    $to_date = Carbon::parse($shareent->to_date)->format('Y-m-d H:i');
                }

            }

            $color = getEventColor($shareent->type);

            $event .= "{color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
            $event_array[] = ['title' => $shareent->name,'start' => $from_date,'end' => $to_date,'icon' => $shareent->fa_icon];
            $i++;


            /******For Repeat*****/
            if(!empty($shareent->is_repeat) && $shareent->is_repeat != 1){

                /******Weeks*****/
                if($shareent->is_repeat == 2){
                    for ($x = 1; $x <= 24; $x++) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($shareent->from_date)->addWeeks($x)->format('Y-m-d');
                            $to_date = Carbon::parse($shareent->to_date)->addWeeks($x)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($shareent->from_date)->addWeeks($x)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($shareent->to_date)->addWeeks($x)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
                        $event_array[] = ['title' => $shareent->name,'start' => $from_date,'end' => $to_date,'icon' => $shareent->fa_icon];
                    }
                }
                /******Month*****/
                if($shareent->is_repeat == 3){
                    for ($x = 1; $x <= 24; $x++) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($shareent->from_date)->addMonths($x)->format('Y-m-d');
                            $to_date = Carbon::parse($shareent->to_date)->addMonths($x)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($shareent->from_date)->addMonths($x)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($shareent->to_date)->addMonths($x)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
                        $event_array[] = ['title' => $shareent->name,'start' => $from_date,'end' => $to_date,'icon' => $shareent->fa_icon];
                    }
                }
                /******Six Month*****/
                if($shareent->is_repeat == 4){
                    $six = [6,12,18,24,30,36];
                    foreach($six as $sixa) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($shareent->from_date)->addMonths($sixa)->format('Y-m-d');
                            $to_date = Carbon::parse($shareent->to_date)->addMonths($sixa)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($shareent->from_date)->addMonths($sixa)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($shareent->to_date)->addMonths($sixa)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
                        $event_array[] = ['title' => $shareent->name,'start' => $from_date,'end' => $to_date,'icon' => $shareent->fa_icon];
                    }
                }
                /******Years*****/
                if($shareent->is_repeat == 5){
                    for ($x = 1; $x <= 100; $x++) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($shareent->from_date)->addYear($x)->format('Y-m-d');
                            $to_date = Carbon::parse($shareent->to_date)->addYear($x)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($shareent->from_date)->addYear($x)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($shareent->to_date)->addYear($x)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
                        $event_array[] = ['title' => $shareent->name,'start' => $from_date,'end' => $to_date,'icon' => $shareent->fa_icon];
                    }
                }

            }
            /****** End Repeat*****/

        }
    }

    if($expirations == 'on'){

        if(!empty($driver->license_exp)){
            for ($x = -3; $x <= 6; $x++) {
                $from_date = Carbon::parse($driver->license_exp)->addYear($x)->format('Y-m-d');
                $to_date = Carbon::parse($driver->license_exp)->addYear($x)->format('Y-m-d');
                $event .= "{color:'".getEventColor(3)."',title:'Drivers License Exp',start:'".$from_date."',end:'".$to_date."'},";
                $event_array[] = ['title' => 'Drivers License Exp','start' => $from_date,'end' => $to_date];
            }
        }

        if(!empty($driver->medical_cert)){
            $from_date = Carbon::parse($driver->medical_cert)->format('Y-m-d');
            $to_date = Carbon::parse($driver->medical_cert)->format('Y-m-d');
            $event .= "{color:'".getEventColor(3)."',title:'Medical Cert',start:'".$from_date."',end:'".$to_date."'},";
            $event_array[] = ['title' => 'Medical Cert','start' => $from_date,'end' => $to_date];
        }

        if(!empty($driver->mvr_date)){
            for ($x = -3; $x <= 6; $x++) {
                $from_date = Carbon::parse($driver->mvr_date)->addYear($x)->format('Y-m-d');
                $to_date = Carbon::parse($driver->mvr_date)->addYear($x)->format('Y-m-d');
                $event .= "{color:'".getEventColor(3)."',title:'MVR Date',start:'".$from_date."',end:'".$to_date."'},";
                $event_array[] = ['title' => 'MVR Date','start' => $from_date,'end' => $to_date];
            }
        }

        if(!empty($driver->next_review)){
            for ($x = -3; $x <= 6; $x++) {
                $from_date = Carbon::parse($driver->next_review)->addYear($x)->format('Y-m-d');
                $to_date = Carbon::parse($driver->next_review)->addYear($x)->format('Y-m-d');
                $event .= "{color:'".getEventColor(3)."',title:'Next Review',start:'".$from_date."',end:'".$to_date."'},";
                $event_array[] = ['title' => 'Next Review','start' => $from_date,'end' => $to_date];
            }
        }

        if(!empty($driver->tag_exp)){
            $from_date = Carbon::parse($driver->tag_exp)->format('Y-m-d');
            $to_date = Carbon::parse($driver->tag_exp)->format('Y-m-d');
            $event .= "{color:'".getEventColor(3)."',title:'Tag Exp',start:'".$from_date."',end:'".$to_date."'},";
            $event_array[] = ['title' => 'Tag Exp','start' => $from_date,'end' => $to_date];
        }

        if(!empty($driver->passport_expiration)){
            $from_date = Carbon::parse($driver->passport_expiration)->format('Y-m-d');
            $to_date = Carbon::parse($driver->passport_expiration)->format('Y-m-d');
            $event .= "{color:'".getEventColor(3)."',title:'Passport Exp',start:'".$from_date."',end:'".$to_date."'},";
            $event_array[] = ['title' => 'Passport Exp','start' => $from_date,'end' => $to_date];
        }
    }

    if($birthdays == 'on'){
        //$getDriverList = Driver::where('company_id',auth()->guard('company-admin')->user()->company_id)->get();
        $getDriverList = Driver::where('id',$driver->id)->get();
        foreach($getDriverList as $driverl){
            try {
                if(isset($driverl->birthday)){
                    for ($x = -3; $x <= 100; $x++) {
                        $from_date = Carbon::parse($driverl->birthday)->addYear($x)->format('Y-m-d');
                        $to_date = Carbon::parse($driverl->birthday)->addYear($x)->format('Y-m-d');

                        $event .= "{color:'".getEventColor(0)."',title:'".$driverl->full_name."',start:'".$from_date."',end:'".$to_date."',icon:'fa fa-birthday-cake'},";
                        $event_array[] = ['title' => $driverl->full_name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-birthday-cake'];
                    }
                }
            }catch (\Exception $e) {
                //echo 'invalid date, enduser understands the error message';
            }
            try {

                if(isset($driverl->start_date)){
                    for ($x = -3; $x <= 10; $x++) {
                        $from_date = Carbon::parse($driverl->start_date)->addYear($x)->format('Y-m-d');
                        $to_date = Carbon::parse($driverl->start_date)->addYear($x)->format('Y-m-d');
                        $event .= "{color:'".getEventColor(0)."',title:'Work Anniversary: ".preg_replace('/&#0*39;|\'/', '', $driverl->full_name)."',start:'".$from_date."',end:'".$to_date."',icon:'fa fa-birthday-cake'},";
                        $event_array[] = ['title' => $driverl->full_name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-birthday-cake'];
                    }
                }
            }catch (\Exception $e) {
                //echo 'invalid date, enduser understands the error message';
            }
        }
    }

    if($maintenanceTrucks == 'on'){
        $getDriverInformation = DriverInformation::where('driver_id',$driver->id)->get();
        foreach($getDriverInformation as $driverInformation){
            $getTruck = Truck::where('unit',$driverInformation->truck_number)->first();
            if(!empty($getTruck->id)){
                $getTrucklog = TruckMaintenanceLog::where('truck_id',$getTruck->id)->where('is_completed',0)->get();
                if(count($getTrucklog) > 0){
                    foreach($getTrucklog as $trucklog){
                        if(!empty($trucklog->due_date)){
                            $from_date = Carbon::parse($trucklog->due_date)->format('Y-m-d');
                            $to_date = Carbon::parse($trucklog->due_date)->format('Y-m-d');
                            $event .= "{color:'".getEventColor(4)."',title:'Truck Maintenance: ".$driverInformation->truck_number."', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-truck'},";
                            $event_array[] = ['title' => 'Truck Maintenance: '.$driverInformation->truck_number,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-truck'];
                        }
                    }
                }
            }
        }
    }

    if($maintenanceTrailers == 'on'){
        $getDriverInformation = DriverInformation::where('driver_id',$driver->id)->get();
        foreach($getDriverInformation as $driverInformation){
            $getTrailer = Trailer::where('trailer_id',$driverInformation->trailer_number)->first();
            if(!empty($getTrailer->id)){
                $getTrailerlog = TrailerMaintenanceLog::where('trailer_id',$getTrailer->id)->where('is_completed',0)->get();
                if(count($getTrailerlog) > 0){
                    foreach($getTrailerlog as $trailerlog){
                        if(!empty($trailerlog->due_date)){
                            $from_date = Carbon::parse($trailerlog->due_date)->format('Y-m-d');
                            $to_date = Carbon::parse($trailerlog->due_date)->format('Y-m-d');
                            $event .= "{color:'".getEventColor(5)."',title:'Trailer Maintenance: ".$driverInformation->trailer_number."', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-truck'},";
                            $event_array[] = ['title' => 'Trailer Maintenance: '.$driverInformation->trailer_number,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-truck'];
                        }
                    }
                }
            }
        }
    }

    if($myLeaves == 'on'){
        $getLeaveDriver = Leave::where('driver_id', $driversId)->get();
        if(count($getLeaveDriver) > 0){
            foreach($getLeaveDriver as $dleave){
                $from_date = Carbon::parse($dleave->from_date)->format('Y-m-d');
                $to_date = Carbon::parse($dleave->to_date)->format('Y-m-d');

                $event .= "{color:'".$dleave->leave_type->color."',title:'Leave - ".$dleave->leave_type->name." ', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-universal-access'},";
                $event_array[] = ['title' => 'Leave - '.$dleave->leave_type->name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-universal-access'];
            }
        }
    }

    if($myViolations == 'on'){

        $getMyViolations = DriverHosViolation::where('driver_id', $driversId)->get();
        if(count($getMyViolations) > 0){
            foreach($getMyViolations as $dviolations){
                $from_date = Carbon::parse($dviolations->event_calendar_date)->format('Y-m-d');
                $to_date = Carbon::parse($dviolations->event_calendar_date)->format('Y-m-d');

                $event .= "{color:'".getEventColor(11)."',title:'Violations - ".$dviolations->title." ', start:'".$from_date."', end:'".$to_date."',dataid:'".$dviolations->id."'},";
                $event_array[] = ['title' => 'Violations - '.$dviolations->title,'start' => $from_date,'end' => $to_date,'dataid' => $dviolations->id];
            }
        }
    }

    if($maintenanceTicket == 'on'){

        $getMaintenance = Maintenance::Join('maintenance_calendars', function($join) use ($driversId) {
                $join->on('maintenances.id', '=', 'maintenance_calendars.maintenance_id');
                $join->where('maintenance_calendars.user_id', $driversId);
                $join->where('maintenance_calendars.user_type', 3);
                $join->where('maintenance_calendars.company_id', auth()->guard('company-admin')->user()->company_id);

            })
            ->where('maintenances.company_id', auth()->guard('company-admin')->user()->company_id)
            ->get();

        if(count($getMaintenance) > 0){
            foreach($getMaintenance as $dMaintenance){
                $from_date = Carbon::parse($dMaintenance->due_date)->format('Y-m-d');
                $to_date = Carbon::parse($dMaintenance->due_date)->format('Y-m-d');

                $event .= "{color:'".getEventColor(12)."',title:'Maintenance - ".$dMaintenance->ticket_number.' - '.$dMaintenance->name." ', start:'".$from_date."', end:'".$to_date."',maintenance_id:'".$dMaintenance->maintenance_id."'},";

                $event_array[] = ['title' => 'Maintenance - '.$dMaintenance->ticket_number.' - '.$dMaintenance->name,'start' => $from_date,'end' => $to_date,'maintenance_id' => $dMaintenance->maintenance_id];
            }
        }

        $getMaintenanceAppointment = MaintenanceAppointment::where('owner', $driversId.'-3')->where('company_id', auth()->guard('company-admin')->user()->company_id)->orWhere('attendees', $driversId.'-3')->get();

        if(count($getMaintenanceAppointment) > 0){
            foreach($getMaintenanceAppointment as $dMaintenancea){
                $from_date = Carbon::parse($dMaintenancea->start_time)->format('Y-m-d');
                $to_date = Carbon::parse($dMaintenancea->end_time)->format('Y-m-d');

                $event .= "{color:'".getEventColor(12)."',title:'M. Appointment - ".$dMaintenancea->summary." ', start:'".$from_date."', end:'".$to_date."'},";
                $event_array[] = ['title' => 'M. Appointment - '.$dMaintenancea->summary,'start' => $from_date,'end' => $to_date];
            }
        }

    }


    $event .= "]";

    if($IsArray == 1){
        return $event_array;
    }else{
        return $event;
    }

}

function getDriverEventFullCalendar($driversId, $calendarEvent=null,
                                    $birthdays=null,
                                    $expirations=null,
                                    $maintenanceTrucks=null,
                                    $maintenanceTrailers=null,
                                    $hidePersonalEvents=null,
                                    $IsArray=null,
                                    $myLeaves=null,
                                    $myViolations=null,
                                    $maintenanceTicket=null,
                                    $startDate=null,
                                    $endDate=null) {
    $getEvent = Event::where('driver_id', $driversId)
        ->where(function ($query) {
            $query->whereNull('share')->orWhere('share',1);
        })
        ->where(function ($query) use ($startDate, $endDate){
            if(!is_null($startDate) && !is_null($endDate)){
                $query->where('from_date', '>=', $startDate)->where('from_date', '<=', $endDate);
            }
        })->get();

    $getShareEvent = Event::where('driver_id', $driversId)
        ->where('share',2)
        ->where(function ($query) use ($startDate, $endDate){
            if(!is_null($startDate) && !is_null($endDate)){
                $query->where('from_date', '>=', $startDate)->where('from_date', '<=', $endDate);
            }
        })->get();

    $driver = Driver::withTrashed()->where('id',$driversId)->first();
    $getdriverexp = $driver->driver_compliance_safety;

    $i=0;
    $event = "[";
    $event_array = [];
    if($hidePersonalEvents == 'on'){
        foreach($getEvent as $sevent) {
            $Checktime = Carbon::parse($sevent->to_date)->format('H:i');

            $from_date = $sevent->from_date;
            $to_date = $sevent->to_date;
            if($Checktime == "00:00"){
                $from_date = Carbon::parse($sevent->from_date)->format('Y-m-d');
                if($sevent->is_repeat == 1){
                    $to_date = Carbon::parse($sevent->to_date)->addYear(2)->format('Y-m-d');
                }else{
                    $to_date = Carbon::parse($sevent->to_date)->format('Y-m-d');
                }
            }else{
                $from_date = Carbon::parse($sevent->from_date)->format('Y-m-d H:i');

                if($sevent->is_repeat == 1){
                    $to_date = Carbon::parse($sevent->to_date)->addYear(2)->format('Y-m-d H:i');
                }else{
                    $to_date = Carbon::parse($sevent->to_date)->format('Y-m-d H:i');
                }

            }

            $color = getEventColor($sevent->type);
            if($sevent->share == 1){
                $id = 0;
            }else{
                $id = $sevent->id;
            }

            $event .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";

            $event_array[] = ['title' => $sevent->name,'start' => $from_date,'end' => $to_date,'icon' => $sevent->fa_icon];

            /******For Repeat*****/
            if(!empty($sevent->is_repeat) && $sevent->is_repeat != 1){

                /******Weeks*****/
                if($sevent->is_repeat == 2){
                    for ($x = 1; $x <= 24; $x++) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($sevent->from_date)->addWeeks($x)->format('Y-m-d');
                            $to_date = Carbon::parse($sevent->to_date)->addWeeks($x)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($sevent->from_date)->addWeeks($x)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($sevent->to_date)->addWeeks($x)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";
                        $event_array[] = ['title' => $sevent->name,'start' => $from_date,'end' => $to_date,'icon' => $sevent->fa_icon];
                    }
                }
                /******Month*****/
                if($sevent->is_repeat == 3){
                    for ($x = 1; $x <= 24; $x++) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($sevent->from_date)->addMonths($x)->format('Y-m-d');
                            $to_date = Carbon::parse($sevent->to_date)->addMonths($x)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($sevent->from_date)->addMonths($x)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($sevent->to_date)->addMonths($x)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";
                        $event_array[] = ['title' => $sevent->name,'start' => $from_date,'end' => $to_date,'icon' => $sevent->fa_icon];
                    }
                }
                /******Six Month*****/
                if($sevent->is_repeat == 4){
                    $six = [6,12,18,24,30,36];
                    foreach($six as $sixa) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($sevent->from_date)->addMonths($sixa)->format('Y-m-d');
                            $to_date = Carbon::parse($sevent->to_date)->addMonths($sixa)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($sevent->from_date)->addMonths($sixa)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($sevent->to_date)->addMonths($sixa)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";
                        $event_array[] = ['title' => $sevent->name,'start' => $from_date,'end' => $to_date,'icon' => $sevent->fa_icon];
                    }
                }
                /******Years*****/
                if($sevent->is_repeat == 5){
                    for ($x = 1; $x <= 6; $x++) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($sevent->from_date)->addYear($x)->format('Y-m-d');
                            $to_date = Carbon::parse($sevent->to_date)->addYear($x)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($sevent->from_date)->addYear($x)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($sevent->to_date)->addYear($x)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";
                        $event_array[] = ['title' => $sevent->name,'start' => $from_date,'end' => $to_date,'icon' => $sevent->fa_icon];
                    }
                }

            }
            /****** End Repeat*****/


            $i++;
        }
    }

    if($calendarEvent == 1){
        foreach($getShareEvent as $shareent) {
            $Checktime = Carbon::parse($shareent->to_date)->format('H:i');

            $from_date = $shareent->from_date;
            $to_date = $shareent->to_date;
            if($Checktime == "00:00"){
                $from_date = Carbon::parse($shareent->from_date)->format('Y-m-d');
                if($shareent->is_repeat == 1){
                    $to_date = Carbon::parse($shareent->to_date)->addYear(2)->format('Y-m-d');
                }else{
                    $to_date = Carbon::parse($shareent->to_date)->format('Y-m-d');
                }
            }else{
                $from_date = Carbon::parse($shareent->from_date)->format('Y-m-d H:i');

                if($shareent->is_repeat == 1){
                    $to_date = Carbon::parse($shareent->to_date)->addYear(2)->format('Y-m-d H:i');
                }else{
                    $to_date = Carbon::parse($shareent->to_date)->format('Y-m-d H:i');
                }

            }

            $color = getEventColor($shareent->type);

            $event .= "{color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
            $event_array[] = ['title' => $shareent->name,'start' => $from_date,'end' => $to_date,'icon' => $shareent->fa_icon];
            $i++;


            /******For Repeat*****/
            if(!empty($shareent->is_repeat) && $shareent->is_repeat != 1){

                /******Weeks*****/
                if($shareent->is_repeat == 2){
                    for ($x = 1; $x <= 24; $x++) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($shareent->from_date)->addWeeks($x)->format('Y-m-d');
                            $to_date = Carbon::parse($shareent->to_date)->addWeeks($x)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($shareent->from_date)->addWeeks($x)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($shareent->to_date)->addWeeks($x)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
                        $event_array[] = ['title' => $shareent->name,'start' => $from_date,'end' => $to_date,'icon' => $shareent->fa_icon];
                    }
                }
                /******Month*****/
                if($shareent->is_repeat == 3){
                    for ($x = 1; $x <= 24; $x++) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($shareent->from_date)->addMonths($x)->format('Y-m-d');
                            $to_date = Carbon::parse($shareent->to_date)->addMonths($x)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($shareent->from_date)->addMonths($x)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($shareent->to_date)->addMonths($x)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
                        $event_array[] = ['title' => $shareent->name,'start' => $from_date,'end' => $to_date,'icon' => $shareent->fa_icon];
                    }
                }
                /******Six Month*****/
                if($shareent->is_repeat == 4){
                    $six = [6,12,18,24,30,36];
                    foreach($six as $sixa) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($shareent->from_date)->addMonths($sixa)->format('Y-m-d');
                            $to_date = Carbon::parse($shareent->to_date)->addMonths($sixa)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($shareent->from_date)->addMonths($sixa)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($shareent->to_date)->addMonths($sixa)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
                        $event_array[] = ['title' => $shareent->name,'start' => $from_date,'end' => $to_date,'icon' => $shareent->fa_icon];
                    }
                }
                /******Years*****/
                if($shareent->is_repeat == 5){
                    for ($x = 1; $x <= 100; $x++) {
                        if($Checktime == "00:00"){
                            $from_date = Carbon::parse($shareent->from_date)->addYear($x)->format('Y-m-d');
                            $to_date = Carbon::parse($shareent->to_date)->addYear($x)->format('Y-m-d');
                        }else{
                            $from_date = Carbon::parse($shareent->from_date)->addYear($x)->format('Y-m-d H:i');
                            $to_date = Carbon::parse($shareent->to_date)->addYear($x)->format('Y-m-d H:i');
                        }
                        $event .= "{id:".$id.",color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
                        $event_array[] = ['title' => $shareent->name,'start' => $from_date,'end' => $to_date,'icon' => $shareent->fa_icon];
                    }
                }

            }
            /****** End Repeat*****/

        }
    }

    if($expirations == 'on'){
        if(!empty($driver->license_exp)){
            for ($x = -3; $x <= 6; $x++) {
                $from_date = Carbon::parse($driver->license_exp)->addYear($x)->format('Y-m-d');
                $to_date = Carbon::parse($driver->license_exp)->addYear($x)->format('Y-m-d');

                if(!is_null($startDate) && !is_null($endDate)){
                    if($from_date >= $startDate && $to_date <= $endDate){
                        $event .= "{color:'".getEventColor(3)."',title:'Drivers License Exp',start:'".$from_date."',end:'".$to_date."'},";
                        $event_array[] = ['title' => 'Drivers License Exp','start' => $from_date,'end' => $to_date];
                    }
                }else{
                    $event .= "{color:'".getEventColor(3)."',title:'Drivers License Exp',start:'".$from_date."',end:'".$to_date."'},";
                    $event_array[] = ['title' => 'Drivers License Exp','start' => $from_date,'end' => $to_date];
                }
            }
        }

        if(!empty($driver->medical_cert)){
            $from_date = Carbon::parse($driver->medical_cert)->format('Y-m-d');
            $to_date = Carbon::parse($driver->medical_cert)->format('Y-m-d');

            if(!is_null($startDate) && !is_null($endDate)){
                if($from_date >= $startDate && $to_date <= $endDate){
                    $event .= "{color:'".getEventColor(3)."',title:'Medical Cert',start:'".$from_date."',end:'".$to_date."'},";
                    $event_array[] = ['title' => 'Medical Cert','start' => $from_date,'end' => $to_date];
                }
            }else{
                $event .= "{color:'".getEventColor(3)."',title:'Medical Cert',start:'".$from_date."',end:'".$to_date."'},";
                $event_array[] = ['title' => 'Medical Cert','start' => $from_date,'end' => $to_date];
            }
        }

        if(!empty($driver->mvr_date)){
            for ($x = -3; $x <= 6; $x++) {
                $from_date = Carbon::parse($driver->mvr_date)->addYear($x)->format('Y-m-d');
                $to_date = Carbon::parse($driver->mvr_date)->addYear($x)->format('Y-m-d');

                if(!is_null($startDate) && !is_null($endDate)){
                    if($from_date >= $startDate && $to_date <= $endDate){
                        $event .= "{color:'".getEventColor(3)."',title:'MVR Date',start:'".$from_date."',end:'".$to_date."'},";
                        $event_array[] = ['title' => 'MVR Date','start' => $from_date,'end' => $to_date];
                    }
                }else{
                    $event .= "{color:'".getEventColor(3)."',title:'MVR Date',start:'".$from_date."',end:'".$to_date."'},";
                    $event_array[] = ['title' => 'MVR Date','start' => $from_date,'end' => $to_date];
                }
            }
        }

        if(!empty($driver->next_review)){
            for ($x = -3; $x <= 6; $x++) {
                $from_date = Carbon::parse($driver->next_review)->addYear($x)->format('Y-m-d');
                $to_date = Carbon::parse($driver->next_review)->addYear($x)->format('Y-m-d');

                if(!is_null($startDate) && !is_null($endDate)){
                    if($from_date >= $startDate && $to_date <= $endDate){
                        $event .= "{color:'".getEventColor(3)."',title:'Next Review',start:'".$from_date."',end:'".$to_date."'},";
                        $event_array[] = ['title' => 'Next Review','start' => $from_date,'end' => $to_date];
                    }
                }else{
                    $event .= "{color:'".getEventColor(3)."',title:'Next Review',start:'".$from_date."',end:'".$to_date."'},";
                    $event_array[] = ['title' => 'Next Review','start' => $from_date,'end' => $to_date];
                }
            }
        }

        if(!empty($driver->tag_exp)){
            $from_date = Carbon::parse($driver->tag_exp)->format('Y-m-d');
            $to_date = Carbon::parse($driver->tag_exp)->format('Y-m-d');

            if(!is_null($startDate) && !is_null($endDate)){
                if($from_date >= $startDate && $to_date <= $endDate){
                    $event .= "{color:'".getEventColor(3)."',title:'Tag Exp',start:'".$from_date."',end:'".$to_date."'},";
                    $event_array[] = ['title' => 'Tag Exp','start' => $from_date,'end' => $to_date];
                }
            }else{
                $event .= "{color:'".getEventColor(3)."',title:'Tag Exp',start:'".$from_date."',end:'".$to_date."'},";
                $event_array[] = ['title' => 'Tag Exp','start' => $from_date,'end' => $to_date];
            }
        }

        if(!empty($driver->passport_expiration)){
            $from_date = Carbon::parse($driver->passport_expiration)->format('Y-m-d');
            $to_date = Carbon::parse($driver->passport_expiration)->format('Y-m-d');

            if(!is_null($startDate) && !is_null($endDate)){
                if($from_date >= $startDate && $to_date <= $endDate){
                    $event .= "{color:'".getEventColor(3)."',title:'Passport Exp',start:'".$from_date."',end:'".$to_date."'},";
                    $event_array[] = ['title' => 'Passport Exp','start' => $from_date,'end' => $to_date];
                }
            }else{
                $event .= "{color:'".getEventColor(3)."',title:'Passport Exp',start:'".$from_date."',end:'".$to_date."'},";
                $event_array[] = ['title' => 'Passport Exp','start' => $from_date,'end' => $to_date];
            }
        }
    }

    if($birthdays == 'on'){
        //$getDriverList = Driver::where('company_id',auth()->guard('company-admin')->user()->company_id)->get();
        $getDriverList = Driver::where('id',$driver->id)->get();
        foreach($getDriverList as $driverl){
            try {
                if(isset($driverl->birthday)){
                    for ($x = -3; $x <= 100; $x++) {
                        $from_date = Carbon::parse($driverl->birthday)->addYear($x)->format('Y-m-d');
                        $to_date = Carbon::parse($driverl->birthday)->addYear($x)->format('Y-m-d');

                        if(!is_null($startDate) && !is_null($endDate)){
                            if($from_date >= $startDate && $to_date <= $endDate){
                                $event .= "{color:'".getEventColor(0)."',title:'".$driverl->full_name."',start:'".$from_date."',end:'".$to_date."',icon:'fa fa-birthday-cake'},";
                                $event_array[] = ['title' => $driverl->full_name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-birthday-cake'];
                            }
                        }else{
                            $event .= "{color:'".getEventColor(0)."',title:'".$driverl->full_name."',start:'".$from_date."',end:'".$to_date."',icon:'fa fa-birthday-cake'},";
                            $event_array[] = ['title' => $driverl->full_name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-birthday-cake'];
                        }
                    }
                }
            }catch (\Exception $e) {
                //echo 'invalid date, enduser understands the error message';
            }
            try {

                if(isset($driverl->start_date)){
                    for ($x = -3; $x <= 10; $x++) {
                        $from_date = Carbon::parse($driverl->start_date)->addYear($x)->format('Y-m-d');
                        $to_date = Carbon::parse($driverl->start_date)->addYear($x)->format('Y-m-d');

                        if(!is_null($startDate) && !is_null($endDate)){
                            if($from_date >= $startDate && $to_date <= $endDate){
                                $event .= "{color:'".getEventColor(0)."',title:'Work Anniversary: ".preg_replace('/&#0*39;|\'/', '', $driverl->full_name)."',start:'".$from_date."',end:'".$to_date."',icon:'fa fa-birthday-cake'},";
                                $event_array[] = ['title' => $driverl->full_name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-birthday-cake'];
                            }
                        }else{
                            $event .= "{color:'".getEventColor(0)."',title:'Work Anniversary: ".preg_replace('/&#0*39;|\'/', '', $driverl->full_name)."',start:'".$from_date."',end:'".$to_date."',icon:'fa fa-birthday-cake'},";
                            $event_array[] = ['title' => $driverl->full_name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-birthday-cake'];
                        }
                    }
                }
            }catch (\Exception $e) {
                //echo 'invalid date, enduser understands the error message';
            }
        }
    }

    if($maintenanceTrucks == 'on'){
        $getDriverInformation = DriverInformation::where('driver_id',$driver->id)->get();
        foreach($getDriverInformation as $driverInformation){
            $getTruck = Truck::where('unit',$driverInformation->truck_number)->first();
            if(!empty($getTruck->id)){
                $getTrucklog = TruckMaintenanceLog::where('truck_id',$getTruck->id)->where('is_completed',0)->get();
                if(count($getTrucklog) > 0){
                    foreach($getTrucklog as $trucklog){
                        if(!empty($trucklog->due_date)){
                            $from_date = Carbon::parse($trucklog->due_date)->format('Y-m-d');
                            $to_date = Carbon::parse($trucklog->due_date)->format('Y-m-d');

                            if(!is_null($startDate) && !is_null($endDate)){
                                if($from_date >= $startDate && $to_date <= $endDate){
                                    $event .= "{color:'".getEventColor(4)."',title:'Truck Maintenance: ".$driverInformation->truck_number."', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-truck'},";
                                    $event_array[] = ['title' => 'Truck Maintenance: '.$driverInformation->truck_number,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-truck'];
                                }
                            }else{
                                $event .= "{color:'".getEventColor(4)."',title:'Truck Maintenance: ".$driverInformation->truck_number."', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-truck'},";
                                $event_array[] = ['title' => 'Truck Maintenance: '.$driverInformation->truck_number,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-truck'];
                            }
                        }
                    }
                }
            }
        }
    }

    if($maintenanceTrailers == 'on'){
        $getDriverInformation = DriverInformation::where('driver_id',$driver->id)->get();
        foreach($getDriverInformation as $driverInformation){
            $getTrailer = Trailer::where('trailer_id',$driverInformation->trailer_number)->first();
            if(!empty($getTrailer->id)){
                $getTrailerlog = TrailerMaintenanceLog::where('trailer_id',$getTrailer->id)->where('is_completed',0)->get();
                if(count($getTrailerlog) > 0){
                    foreach($getTrailerlog as $trailerlog){
                        if(!empty($trailerlog->due_date)){
                            $from_date = Carbon::parse($trailerlog->due_date)->format('Y-m-d');
                            $to_date = Carbon::parse($trailerlog->due_date)->format('Y-m-d');

                            if(!is_null($startDate) && !is_null($endDate)){
                                if($from_date >= $startDate && $to_date <= $endDate){
                                    $event .= "{color:'".getEventColor(5)."',title:'Trailer Maintenance: ".$driverInformation->trailer_number."', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-truck'},";
                                    $event_array[] = ['title' => 'Trailer Maintenance: '.$driverInformation->trailer_number,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-truck'];
                                }
                            }else{
                                $event .= "{color:'".getEventColor(5)."',title:'Trailer Maintenance: ".$driverInformation->trailer_number."', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-truck'},";
                                $event_array[] = ['title' => 'Trailer Maintenance: '.$driverInformation->trailer_number,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-truck'];
                            }
                        }
                    }
                }
            }
        }
    }

    if($myLeaves == 'on'){
        $getLeaveDriver = Leave::where('driver_id', $driversId)->get();
        if(count($getLeaveDriver) > 0){
            foreach($getLeaveDriver as $dleave){
                $from_date = Carbon::parse($dleave->from_date)->format('Y-m-d');
                $to_date = Carbon::parse($dleave->to_date)->format('Y-m-d');

                if(!is_null($startDate) && !is_null($endDate)){
                    if($from_date >= $startDate && $to_date <= $endDate){
                        $event .= "{color:'".$dleave->leave_type->color."',title:'Leave - ".$dleave->leave_type->name." ', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-universal-access'},";
                        $event_array[] = ['title' => 'Leave - '.$dleave->leave_type->name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-universal-access'];
                    }
                }else{
                    $event .= "{color:'".$dleave->leave_type->color."',title:'Leave - ".$dleave->leave_type->name." ', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-universal-access'},";
                    $event_array[] = ['title' => 'Leave - '.$dleave->leave_type->name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-universal-access'];
                }
            }
        }
    }

    if($myViolations == 'on'){

        $getMyViolations = DriverHosViolation::where('driver_id', $driversId)->get();
        if(count($getMyViolations) > 0){
            foreach($getMyViolations as $dviolations){
                $from_date = Carbon::parse($dviolations->event_calendar_date)->format('Y-m-d');
                $to_date = Carbon::parse($dviolations->event_calendar_date)->format('Y-m-d');

                if(!is_null($startDate) && !is_null($endDate)){
                    if($from_date >= $startDate && $to_date <= $endDate){
                        $event .= "{color:'".getEventColor(11)."',title:'Violations - ".$dviolations->title." ', start:'".$from_date."', end:'".$to_date."',dataid:'".$dviolations->id."'},";
                        $event_array[] = ['title' => 'Violations - '.$dviolations->title,'start' => $from_date,'end' => $to_date,'dataid' => $dviolations->id];
                    }
                }else{
                    $event .= "{color:'".getEventColor(11)."',title:'Violations - ".$dviolations->title." ', start:'".$from_date."', end:'".$to_date."',dataid:'".$dviolations->id."'},";
                    $event_array[] = ['title' => 'Violations - '.$dviolations->title,'start' => $from_date,'end' => $to_date,'dataid' => $dviolations->id];
                }
            }
        }
    }

    if($maintenanceTicket == 'on'){
        $getMaintenance = Maintenance::Join('maintenance_calendars', function($join) use ($driversId) {
            $join->on('maintenances.id', '=', 'maintenance_calendars.maintenance_id');
            $join->where('maintenance_calendars.user_id', $driversId);
            $join->where('maintenance_calendars.user_type', 3);
            $join->where('maintenance_calendars.company_id', auth()->guard('company-admin')->user()->company_id);

        })
            ->where('maintenances.company_id', auth()->guard('company-admin')->user()->company_id)
            ->get();

        if(count($getMaintenance) > 0){
            foreach($getMaintenance as $dMaintenance){
                $from_date = Carbon::parse($dMaintenance->due_date)->format('Y-m-d');
                $to_date = Carbon::parse($dMaintenance->due_date)->format('Y-m-d');

                if(!is_null($startDate) && !is_null($endDate)){
                    if($from_date >= $startDate && $to_date <= $endDate){
                        $event .= "{color:'".getEventColor(12)."',title:'Maintenance - ".$dMaintenance->ticket_number.' - '.$dMaintenance->name." ', start:'".$from_date."', end:'".$to_date."',maintenance_id:'".$dMaintenance->maintenance_id."'},";
                        $event_array[] = ['title' => 'Maintenance - '.$dMaintenance->ticket_number.' - '.$dMaintenance->name,'start' => $from_date,'end' => $to_date,'maintenance_id' => $dMaintenance->maintenance_id];
                    }
                }else{
                    $event .= "{color:'".getEventColor(12)."',title:'Maintenance - ".$dMaintenance->ticket_number.' - '.$dMaintenance->name." ', start:'".$from_date."', end:'".$to_date."',maintenance_id:'".$dMaintenance->maintenance_id."'},";
                    $event_array[] = ['title' => 'Maintenance - '.$dMaintenance->ticket_number.' - '.$dMaintenance->name,'start' => $from_date,'end' => $to_date,'maintenance_id' => $dMaintenance->maintenance_id];
                }
            }
        }

        $getMaintenanceAppointment = MaintenanceAppointment::where('owner', $driversId.'-3')->where('company_id', auth()->guard('company-admin')->user()->company_id)->orWhere('attendees', $driversId.'-3')->get();

        if(count($getMaintenanceAppointment) > 0){
            foreach($getMaintenanceAppointment as $dMaintenancea){
                $from_date = Carbon::parse($dMaintenancea->start_time)->format('Y-m-d');
                $to_date = Carbon::parse($dMaintenancea->end_time)->format('Y-m-d');

                if(!is_null($startDate) && !is_null($endDate)){
                    if($from_date >= $startDate && $to_date <= $endDate){
                        $event .= "{color:'".getEventColor(12)."',title:'M. Appointment - ".$dMaintenancea->summary." ', start:'".$from_date."', end:'".$to_date."'},";
                        $event_array[] = ['title' => 'M. Appointment - '.$dMaintenancea->summary,'start' => $from_date,'end' => $to_date];
                    }
                }else{
                    $event .= "{color:'".getEventColor(12)."',title:'M. Appointment - ".$dMaintenancea->summary." ', start:'".$from_date."', end:'".$to_date."'},";
                    $event_array[] = ['title' => 'M. Appointment - '.$dMaintenancea->summary,'start' => $from_date,'end' => $to_date];
                }
            }
        }
    }

    $event .= "]";

    if($IsArray == 1){
        return $event_array;
    }else{
        return $event;
    }
}


function getCompanyAdminEventFullCalendar(
    $companyAdminId,
    $calendarEvent=null,
    $birthdays=null,
    $expirations=null,
    $maintenanceTrucks=null,
    $maintenanceTrailers=null,
    $selectCompnay=null,
    $selectTeam=null,
    $hidePersonalEvents=null,
    $driverleaves=null,
    $myLeaves=null,
    $dispatcherLeaves=null,
    $maintenanceTicket=null)
{
    $getEvent                   = Event::where('company_admin_id', $companyAdminId)->orWhere('share',1)->get();
    $getShareEvent              = Event::where('driver_id', $companyAdminId)->where('share',3)->get();

    $companyId                  = auth()->guard('company-admin')->user()->company_id;
    if(!empty($selectCompnay)){
        $companyId              = $selectCompnay;
    }

    if(!empty($selectTeam)){
        $getEvent               = Event::Join('drivers', function($join) use ($selectTeam) {
                $join->on('events.driver_id', '=', 'drivers.id');
                $join->where('drivers.team_id','=', $selectTeam);
            })
            ->whereNull('events.share')
            ->orWhere('events.share',1)
            ->get();
        $getShareEvent          = Event::Join('drivers', function($join) use ($selectTeam) {
                $join->on('events.driver_id', '=', 'drivers.id');
                $join->where('drivers.team_id','=', $selectTeam);
            })
            ->where('events.share',2)
            ->get();
    }
    $id = "";
    $i                          = 0;
    $event                      = "[";

    if($hidePersonalEvents == 'on'){
        if($getEvent->isNotEmpty()){
            foreach($getEvent as $sevent) {
                $Checktime      = Carbon::parse($sevent->to_date)->format('H:i');
                if($Checktime == "00:00"){
                    $from_date = Carbon::parse($sevent->from_date)->format('Y-m-d');
                    $to_date    = Carbon::parse($sevent->to_date)->format('Y-m-d');
                }else{
                    $from_date  = Carbon::parse($sevent->from_date)->format('Y-m-d H:i');
                    $to_date    = Carbon::parse($sevent->to_date)->format('Y-m-d H:i');
                }
                $color          = getEventColor($sevent->type);

                if($sevent->share == 1){
                    if($companyAdminId == $sevent->company_admin_id){
                        $id     = $sevent->id;
                    }else{
                        $id     = 0;
                    }

                }else{
                    $id         = $sevent->id;
                }

                $event          .= "{id:".$id.",color:'".$color."',title:'".preg_replace('/&#0*39;|\'/', '', $sevent->name)."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";
                //$event          .= "{id:15,color:'#7F8C8D',title:'SSSS',start:'2022-07-15',end:'2022-07-15',icon:'fa fa-birthday-cake'},";



                /******For Repeat*****/
                if(isset($sevent->is_repeat) && $sevent->is_repeat != 1){
                    /******Weeks*****/
                    if($sevent->is_repeat == 2){
                        for ($x = 1; $x <= 24; $x++) {
                            if($Checktime == "00:00"){
                                $from_date  = Carbon::parse($sevent->from_date)->addWeeks($x)->format('Y-m-d');
                                $to_date    = Carbon::parse($sevent->to_date)->addWeeks($x)->format('Y-m-d');
                            }else{
                                $from_date  = Carbon::parse($sevent->from_date)->addWeeks($x)->format('Y-m-d H:i');
                                $to_date    = Carbon::parse($sevent->to_date)->addWeeks($x)->format('Y-m-d H:i');
                            }
                            $event          .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";
                        }
                    }
                    /******Month*****/
                    if($sevent->is_repeat == 3){
                        for ($x = 1; $x <= 24; $x++) {
                            if($Checktime == "00:00"){
                                $from_date  = Carbon::parse($sevent->from_date)->addMonths($x)->format('Y-m-d');
                                $to_date    = Carbon::parse($sevent->to_date)->addMonths($x)->format('Y-m-d');
                            }else{
                                $from_date  = Carbon::parse($sevent->from_date)->addMonths($x)->format('Y-m-d H:i');
                                $to_date    = Carbon::parse($sevent->to_date)->addMonths($x)->format('Y-m-d H:i');
                            }
                            $event          .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";
                        }
                    }
                    /******Six Month*****/
                    if($sevent->is_repeat == 4){
                        $six = [6,12,18,24,30,36];
                        foreach($six as $sixa) {
                            if($Checktime == "00:00"){
                                $from_date  = Carbon::parse($sevent->from_date)->addMonths($sixa)->format('Y-m-d');
                                $to_date    = Carbon::parse($sevent->to_date)->addMonths($sixa)->format('Y-m-d');
                            }else{
                                $from_date  = Carbon::parse($sevent->from_date)->addMonths($sixa)->format('Y-m-d H:i');
                                $to_date    = Carbon::parse($sevent->to_date)->addMonths($sixa)->format('Y-m-d H:i');
                            }
                            $event          .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";
                        }
                    }
                    /******Years*****/
                    if($sevent->is_repeat == 5){
                        for ($x = 1; $x <= 6; $x++) {
                            if($Checktime == "00:00"){
                                $from_date  = Carbon::parse($sevent->from_date)->addYear($x)->format('Y-m-d');
                                $to_date    = Carbon::parse($sevent->to_date)->addYear($x)->format('Y-m-d');
                            }else{
                                $from_date  = Carbon::parse($sevent->from_date)->addYear($x)->format('Y-m-d H:i');
                                $to_date    = Carbon::parse($sevent->to_date)->addYear($x)->format('Y-m-d H:i');
                            }
                            $event          .= "{id:".$id.",color:'".$color."',title:'".$sevent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$sevent->fa_icon."'},";
                        }
                    }
                }
                /****** End Repeat*****/
                $i++;
            }
        }
    }

    if($calendarEvent == 1){
        if($getShareEvent->isNotEmpty()){
            foreach($getShareEvent as $shareent) {
                $Checktime      = Carbon::parse($shareent->to_date)->format('H:i');
                if($Checktime == "00:00"){
                    $from_date  = Carbon::parse($shareent->from_date)->format('Y-m-d');
                    $to_date    = Carbon::parse($shareent->to_date)->format('Y-m-d');
                }else{
                    $from_date  = Carbon::parse($shareent->from_date)->format('Y-m-d H:i');
                    $to_date    = Carbon::parse($shareent->to_date)->format('Y-m-d H:i');
                }
                $color          = getEventColor($shareent->type);
                $event          .= "{color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";

                /******For Repeat*****/
                if(isset($shareent->is_repeat) && $shareent->is_repeat != 1){
                    /******Weeks*****/
                    if($shareent->is_repeat == 2){
                        for ($x = 1; $x <= 24; $x++) {
                            if($Checktime == "00:00"){
                                $from_date  = Carbon::parse($shareent->from_date)->addWeeks($x)->format('Y-m-d');
                                $to_date    = Carbon::parse($shareent->to_date)->addWeeks($x)->format('Y-m-d');
                            }else{
                                $from_date  = Carbon::parse($shareent->from_date)->addWeeks($x)->format('Y-m-d H:i');
                                $to_date    = Carbon::parse($shareent->to_date)->addWeeks($x)->format('Y-m-d H:i');
                            }
                            $event          .= "{id:".$id.",color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
                        }
                    }
                    /******Month*****/
                    if($shareent->is_repeat == 3){
                        for ($x = 1; $x <= 24; $x++) {
                            if($Checktime == "00:00"){
                                $from_date  = Carbon::parse($shareent->from_date)->addMonths($x)->format('Y-m-d');
                                $to_date    = Carbon::parse($shareent->to_date)->addMonths($x)->format('Y-m-d');
                            }else{
                                $from_date  = Carbon::parse($shareent->from_date)->addMonths($x)->format('Y-m-d H:i');
                                $to_date    = Carbon::parse($shareent->to_date)->addMonths($x)->format('Y-m-d H:i');
                            }
                            $event          .= "{id:".$id.",color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
                        }
                    }
                    /******Six Month*****/
                    if($shareent->is_repeat == 4){
                        $six = [6,12,18,24,30,36];
                        foreach($six as $sixa) {
                            if($Checktime == "00:00"){
                                $from_date  = Carbon::parse($shareent->from_date)->addMonths($sixa)->format('Y-m-d');
                                $to_date    = Carbon::parse($shareent->to_date)->addMonths($sixa)->format('Y-m-d');
                            }else{
                                $from_date  = Carbon::parse($shareent->from_date)->addMonths($sixa)->format('Y-m-d H:i');
                                $to_date    = Carbon::parse($shareent->to_date)->addMonths($sixa)->format('Y-m-d H:i');
                            }
                            $event          .= "{id:".$id.",color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
                        }
                    }
                    /******Years*****/
                    if($shareent->is_repeat == 5){
                        for ($x = 1; $x <= 6; $x++) {
                            if($Checktime == "00:00"){
                                $from_date  = Carbon::parse($shareent->from_date)->addYear($x)->format('Y-m-d');
                                $to_date    = Carbon::parse($shareent->to_date)->addYear($x)->format('Y-m-d');
                            }else{
                                $from_date  = Carbon::parse($shareent->from_date)->addYear($x)->format('Y-m-d H:i');
                                $to_date    = Carbon::parse($shareent->to_date)->addYear($x)->format('Y-m-d H:i');
                            }
                            $event          .= "{id:".$id.",color:'".$color."',title:'".$shareent->name."',start:'".$from_date."',end:'".$to_date."',icon:'".$shareent->fa_icon."'},";
                        }
                    }
                }
                /****** End Repeat*****/
                $i++;
            }
        }
    }

    if($calendarEvent != 2){
        if(!empty($selectTeam)){
            $getCompanyDriverList           = Driver::where('company_id',$companyId)->where('team_id',$selectTeam)->get();
        }else{
            $getCompanyDriverList           = Driver::where('company_id',$companyId)->get();
        }
        if($getCompanyDriverList->isNotEmpty()){
            foreach($getCompanyDriverList as $driver){
                if($birthdays == 'on'){
                   if(isset($driver->birthday) && $driver->birthday!=''){
                        for ($x = -3; $x <= 100; $x++) {
                            if(Carbon::parse($driver->birthday)->addYear($x)->format('Y') == "1969"){
                                $from_date      = Carbon::parse($driver->birthday.'00:00:00')->addYear($x)->format('Y-m-d H:i:s');
                                $to_date        = Carbon::parse($driver->birthday.'23:59:59')->addYear($x)->format('Y-m-d H:i:s');
                            }else{
                                $from_date      = Carbon::parse($driver->birthday)->addYear($x)->format('Y-m-d');
                                $to_date        = Carbon::parse($driver->birthday)->addYear($x)->format('Y-m-d');
                            }

                            $event .= "{color:'".getEventColor(0)."',title:'".$driver->full_name."',start:'".$from_date."',end:'".$to_date."',icon:'fa fa-birthday-cake'},";

                            $event_array[]  = ['title' => $driver->full_name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-birthday-cake'];
                        }
                    }
                    if(isset($driver->start_date) && $driver->start_date!=''){
                        for ($x = -3; $x <= 20; $x++) {
                            $from_date      = Carbon::parse($driver->getRawOriginal('start_date'))->addYear($x)->format('Y-m-d');
                            $to_date        = Carbon::parse($driver->getRawOriginal('start_date'))->addYear($x)->format('Y-m-d');
                            $event          .= "{color:'".getEventColor(0)."',title:'Work Anniversary: ".preg_replace('/&#0*39;|\'/', '', $driver->full_name)."',start:'".$from_date."',end:'".$to_date."',icon:'fa fa-birthday-cake'},";
                            $event_array[]  = ['title' => $driver->full_name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-birthday-cake'];
                        }
                    }
                }
                if($expirations == 'on'){
                    if(isset($driver->driver_compliance_safety->license_exp) && $driver->driver_compliance_safety->license_exp!=''){
                        $from_date          = Carbon::parse($driver->driver_compliance_safety->license_exp)->format('Y-m-d');
                        $to_date            = Carbon::parse($driver->driver_compliance_safety->license_exp)->format('Y-m-d');
                        $event              .= "{color:'".getEventColor(3)."',title:'Drivers License Exp: ".$driver->full_name."',start:'".$from_date."',end:'".$to_date."'},";
                    }
                    if(isset($driver->driver_compliance_safety->tag_exp) && $driver->driver_compliance_safety->tag_exp!=''){
                        $from_date          = Carbon::parse($driver->driver_compliance_safety->tag_exp)->format('Y-m-d');
                        $to_date            = Carbon::parse($driver->driver_compliance_safety->tag_exp)->format('Y-m-d');
                        $event              .= "{color:'".getEventColor(3)."',title:'Tag Exp: ".$driver->full_name."',start:'".$from_date."',end:'".$to_date."'},";
                    }
                    if(isset($driver->driver_compliance_safety->passport_expiration) && $driver->driver_compliance_safety->passport_expiration!=''){
                        $from_date          = Carbon::parse($driver->driver_compliance_safety->passport_expiration)->format('Y-m-d');
                        $to_date            = Carbon::parse($driver->driver_compliance_safety->passport_expiration)->format('Y-m-d');
                        $event              .= "{color:'".getEventColor(3)."',title:'Passport Exp: ".$driver->full_name."',start:'".$from_date."',end:'".$to_date."'},";
                    }
                }
                if($maintenanceTrucks == 'on'){
                    $getDriverInformation   = DriverInformation::where('driver_id',$driver->id)->get();
                    if($getDriverInformation->isNotEmpty()){
                        foreach($getDriverInformation as $driverInformation){
                            $getTruck       = Truck::where('unit',$driverInformation->truck_number)->first();
                            if(!empty($getTruck->id)){
                                $getTrucklog            = TruckMaintenanceLog::where('truck_id',$getTruck->id)->where('is_completed',0)->get();
                                if(count($getTrucklog) > 0){
                                    foreach($getTrucklog as $trucklog){
                                        if(!empty($trucklog->due_date)){
                                            $from_date  = Carbon::parse($trucklog->due_date)->format('Y-m-d');
                                            $to_date    = Carbon::parse($trucklog->due_date)->format('Y-m-d');
                                            $event      .= "{color:'".getEventColor(4)."',title:'Truck Maintenance: ".$driverInformation->truck_number."', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-truck'},";
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                if($maintenanceTrailers == 'on'){
                    $getDriverInformation   = DriverInformation::where('driver_id',$driver->id)->get();
                    if($getDriverInformation->isNotEmpty()){
                        foreach($getDriverInformation as $driverInformation){
                            $getTrailer     = Trailer::where('trailer_id',$driverInformation->trailer_number)->first();
                            if(!empty($getTrailer->id)){
                                $getTrailerlog  = TrailerMaintenanceLog::where('trailer_id',$getTrailer->id)->where('is_completed',0)->get();
                                if(count($getTrailerlog) > 0){
                                    foreach($getTrailerlog as $trailerlog){
                                        if(!empty($trailerlog->due_date)){
                                            $from_date  = Carbon::parse($trailerlog->due_date)->format('Y-m-d');
                                            $to_date    = Carbon::parse($trailerlog->due_date)->format('Y-m-d');
                                            $event      .= "{color:'".getEventColor(5)."',title:'Trailer Maintenance: ".$driverInformation->trailer_number."', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-truck'},";
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    if($driverleaves == 'on'){
        $getCompanyAdmin                    = CompanyAdmin::whereId($companyAdminId)->first();
        if(!is_null($getCompanyAdmin)){
            if($getCompanyAdmin->type == 1){
                $getDirverList              = Driver::select('id as driver_id')->where('manager_id',$companyAdminId)->get();
            }else{
                $getDirverList              = LoadDetail::select('driver_id')->where('booked_by', $companyAdminId)->groupBy('driver_id')->get();
            }
            if($getDirverList->isNotEmpty()){
                foreach($getDirverList as $driver){
                    $getDriver              = Driver::where('id',$driver->driver_id)->first();
                    if(!is_null($getDriver)){
                        $getDriverLeave     = Leave::where('driver_id', $getDriver->id)->get();
                        if($getDriverLeave->isNotEmpty()){
                            foreach($getDriverLeave as $leave){
                                $from_date  = Carbon::parse($leave->from_date)->format('Y-m-d');
                                $to_date    = Carbon::parse($leave->to_date)->format('Y-m-d');
                                $event      .= "{color:'".$leave->leave_type->color."',title:'Leave - ".$getDriver->full_name." - ".$leave->leave_type->name." ', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-universal-access'},";
                                $event_array[] = ['title' => 'Leave - '.$getDriver->full_name.' - '.$leave->leave_type->name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-universal-access'];
                            }
                        }
                    }
                }
            }
        }
    }

    if($dispatcherLeaves == 'on'){
        $getDispatchers = CompanyAdmin::Company()->where('type',2)->get();
        if($getDispatchers->isNotEmpty()){
            foreach($getDispatchers as $dispatcher){
                $getDispatchersLeave        = Leave::where('company_admins_id', $dispatcher->id)->get();
                if($getDispatchersLeave->isNotEmpty()){
                    foreach($getDispatchersLeave as $leave){
                        $from_date          = Carbon::parse($leave->from_date)->format('Y-m-d');
                        $to_date            = Carbon::parse($leave->to_date)->format('Y-m-d');
                        $event              .= "{color:'".$leave->leave_type->color."',title:'Leave - ".$dispatcher->full_name.' - '.$leave->leave_type->name." ', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-universal-access'},";
                        $event_array[]      = ['title' => 'Leave - '.$dispatcher->full_name.' - '.$leave->leave_type->name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-universal-access'];
                    }
                }
            }
        }
    }

    if($myLeaves == 'on'){
        $getMyLeaves                        = Leave::where('company_admins_id', $companyAdminId)->get();
        if($getMyLeaves->isNotEmpty()){
            foreach($getMyLeaves as $leave){
                $from_date                  = Carbon::parse($leave->from_date)->format('Y-m-d');
                $to_date                    = Carbon::parse($leave->to_date)->format('Y-m-d');
                $event                      .= "{color:'".$leave->leave_type->color."',title:'Leave - ".$leave->leave_type->name." ', start:'".$from_date."', end:'".$to_date."',icon:'fa fa-universal-access'},";
                $event_array[]              = ['title' => 'Leave - '.$leave->leave_type->name,'start' => $from_date,'end' => $to_date,'icon' => 'fa fa-universal-access'];
            }
        }
    }


    if($maintenanceTicket == 'on'){

        $getCompanyAdmin = CompanyAdmin::find($companyAdminId);

        $getMaintenance = Maintenance::Join('maintenance_calendars', function($join) use ($getCompanyAdmin) {
                $join->on('maintenances.id', '=', 'maintenance_calendars.maintenance_id');
                $join->where('maintenance_calendars.user_id', $getCompanyAdmin->id);
                $join->where('maintenance_calendars.user_type', $getCompanyAdmin->type);
                $join->where('maintenance_calendars.company_id', auth()->guard('company-admin')->user()->company_id);
            })
            ->where('maintenances.company_id', auth()->guard('company-admin')->user()->company_id)
            ->get();


        if(count($getMaintenance) > 0){
            foreach($getMaintenance as $dMaintenance){
                $from_date = Carbon::parse($dMaintenance->due_date)->format('Y-m-d');
                $to_date = Carbon::parse($dMaintenance->due_date)->format('Y-m-d');

                $event .= "{color:'".getEventColor(12)."',title:'Maintenance - ".$dMaintenance->ticket_number.' - '.$dMaintenance->name." ', start:'".$from_date."', end:'".$to_date."',maintenance_id:'".$dMaintenance->maintenance_id."'},";
                $event_array[] = ['title' => 'Violations - '.$dMaintenance->ticket_number.' - '.$dMaintenance->name,'start' => $from_date,'end' => $to_date, 'maintenance_id' => $dMaintenance->maintenance_id];
            }
        }


        $getMaintenanceAppointment = MaintenanceAppointment::where('owner', $getCompanyAdmin->id.'-'.$getCompanyAdmin->type)
                                    ->where('company_id', auth()->guard('company-admin')->user()->company_id)
                                    ->orwhereRaw("find_in_set('".$getCompanyAdmin->id.'-'.$getCompanyAdmin->type."',maintenance_appointments.attendees)")
                                    ->get();

        /*$getMaintenanceAppointment = MaintenanceAppointment::where('owner', $getCompanyAdmin->id.'-'.$getCompanyAdmin->type)
        // ->orWhere('attendees', $getCompanyAdmin->id.'-'.$getCompanyAdmin->type)
        ->orwhereRaw("find_in_set('".$getCompanyAdmin->id.'-'.$getCompanyAdmin->type."',maintenance_appointments.attendees)")
        ->get();*/

        if(count($getMaintenanceAppointment) > 0){
            foreach($getMaintenanceAppointment as $dMaintenancea){
                $from_date = Carbon::parse($dMaintenancea->start_time)->format('Y-m-d');
                $to_date = Carbon::parse($dMaintenancea->end_time)->format('Y-m-d');

                $event .= "{color:'".getEventColor(12)."',title:'M. Appointment - ".$dMaintenancea->summary." ', start:'".$from_date."', end:'".$to_date."', maintenance_id:'".$dMaintenancea->maintenance_id."'},";
                $event_array[] = ['title' => 'M. Appointment - '.$dMaintenancea->summary,'start' => $from_date,'end' => $to_date];
            }
        }

    }

    $event                                  .= "]";
    return $event;
}

function getTargetRevenueByDate($startDate, $endDate, $driver_id)
{

    $get_start_quarter = Carbon::parse($startDate)->quarter;
    $get_start_year = Carbon::parse($startDate)->year;
    $get_end_quarter = Carbon::parse($endDate)->quarter;
    $get_end_year = Carbon::parse($endDate)->year;

    $data = DriverFinancial::where("driver_id", $driver_id)
        ->where("year", '>=', $get_start_year)
        ->where("year", '<=', $get_end_year)
        ->where("quarter", '>=', $get_start_quarter)
        ->where("quarter", '<=', $get_end_quarter)
        ->orderBy('quarter', 'ASC')
        ->get();

    $to_d = Carbon::createFromFormat('Y-m-d', $startDate);
    $from_d = Carbon::createFromFormat('Y-m-d', $endDate);
    $treve = [];
    foreach ($data as $divData) {

        //dd($getPeriod_month,$getPeriod);

        $to = Carbon::createFromFormat('Y-m-d', $divData->start_date);
        $from = Carbon::createFromFormat('Y-m-d', $divData->end_date);
        $quarterdays = $to->diffInDays($from) + 1;
        //$quarterdays = $to->diffInDays($from);

        $perdaytarget = $divData->revenue_target / $quarterdays;

        $period = CarbonPeriod::create($divData->start_date, $divData->end_date);

        $daycount = 1;
        foreach ($period as $date) {
            if ($date->between($to_d, $from_d)) {
                $daycount++;
            }
        }

        if ($daycount > 91) {

            $daycount = 91;
        }

        if ($daycount > 0) {
            $treve[] = $perdaytarget * $daycount;
        }
    }

    $gettot = round(array_sum($treve));
    return $gettot;
}

/********Create By Dhruvish***********/
/*
function getDrivertarget_milestrackerByDate_backup($startDate, $endDate, $driver_id)
{
    $get_start_quarter = Carbon::parse($startDate)->quarter;
    $get_start_year = Carbon::parse($startDate)->year;
    $get_end_quarter = Carbon::parse($endDate)->quarter;
    $get_end_year = Carbon::parse($endDate)->year;

    $data = DriverFinancial::join('load_details','load_details.driver_id','driver_financials.driver_id')
        ->where("driver_financials.driver_id", $driver_id)
        ->where("driver_financials.year", '>=', $get_start_year)
        ->where("driver_financials.year", '<=', $get_end_year)
        ->where("driver_financials.quarter", '>=', $get_start_quarter)
        ->where("driver_financials.quarter", '<=', $get_end_quarter)
        ->orderBy('driver_financials.quarter', 'ASC')
        ->whereNull('deleted_at')

        ->get();

    $to_d = Carbon::createFromFormat('Y-m-d', $startDate);
    $from_d = Carbon::createFromFormat('Y-m-d', $endDate);
    $treve = [];
    foreach ($data as $divData) {
        //dd($getPeriod_month,$getPeriod);
        if(!is_null($divData->start_date) && !is_null($divData->end_date)){
            $to = Carbon::createFromFormat('Y-m-d', $divData->start_date);
            $from = Carbon::createFromFormat('Y-m-d', $divData->end_date);
            $quarterdays = $to->diffInDays($from) + 1;
            //$quarterdays = $to->diffInDays($from);

            $perdaytarget = $divData->target_miles / $quarterdays;

            $period = CarbonPeriod::create($divData->start_date, $divData->end_date);

            $daycount = 1;
            foreach ($period as $date) {
                if ($date->between($to_d, $from_d)) {
                    $daycount++;
                }
            }
            if ($daycount > 91) {
                $daycount = 91;
            }
            if ($daycount > 0) {
                $treve[] = $perdaytarget * $daycount;
            }
        }
        // dd($daycount);
    }

    //  dd($treve);

    $gettot = array_sum($treve);

    return round($gettot);
}
*/
function getDrivertarget_milestrackerByDate($startDate, $endDate, $driver_id)
{
    $get_start_quarter = Carbon::parse($startDate)->quarter;
    $get_start_year = Carbon::parse($startDate)->year;
    $get_end_quarter = Carbon::parse($endDate)->quarter;
    $get_end_year = Carbon::parse($endDate)->year;

    $driverFinancialQuarters = DriverFinancial::where("driver_financials.driver_id", $driver_id)
        ->where("driver_financials.year", '>=', $get_start_year)
        ->where("driver_financials.year", '<=', $get_end_year)
        ->where("driver_financials.quarter", '>=', $get_start_quarter)
        ->where("driver_financials.quarter", '<=', $get_end_quarter)
        ->orderBy('driver_financials.quarter', 'ASC')
        ->get();

    $reportStartDate = Carbon::parse($startDate)->format('Y-m-d');
    $reportEndDate = Carbon::parse($endDate)->format('Y-m-d');

    $treve = [];
    foreach ($driverFinancialQuarters as $key => $driverFinancialQuarter) {
        if(!is_null($driverFinancialQuarter->start_date) && !is_null($driverFinancialQuarter->end_date)){
            $to = Carbon::createFromFormat('Y-m-d', $driverFinancialQuarter->start_date);
            $from = Carbon::createFromFormat('Y-m-d', $driverFinancialQuarter->end_date);
            
            $quarterDays = $to->diffInDays($from) + 1;

            $perdaytarget = $driverFinancialQuarter->target_miles / $quarterDays;

            $periodDates = CarbonPeriod::create($driverFinancialQuarter->start_date, $driverFinancialQuarter->end_date);

            $dayCount = 0;
            foreach ($periodDates as $periodDate) {
                $periodDate = Carbon::parse($periodDate)->format('Y-m-d');
                if ($periodDate >= $reportStartDate && $periodDate <= $reportEndDate) {
                    $dayCount++; //if periodDate between report Start & End Date $dayCount increase
                }
            }
            if ($dayCount > 0) {
                $treve[] = $perdaytarget * $dayCount;
            }
        }
    }
    //dd($treve);

    $gettot = array_sum($treve);

    return round($gettot);
}
/********Create By Dhruvish***********/

function getLoadOccupancynewdata($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher)
{
    //$getLoadUnoccupancy = getLoadUnoccupancy($startDate,$endDate,$driverId,$load,$trailer_type,$dispatcher);
    $getDatePickupCount = getDatePickupCount($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher);

    $period = CarbonPeriod::create($startDate, $endDate);

    $totalDays = 0;
    $bcoutn = 0;
    foreach ($period as $date) {

        $dateYMD = $date->format('m/d/Y');
        $getload = getLoadByDate($dateYMD, $load, null, $trailer_type, $dispatcher);
        if (count($getload) > 0) {
            $load_type = array(0, 4, 6); // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
            foreach ($getload as $load_details) {
                if (in_array($load_details['load_type'], $load_type)) {
                    $totalDays++;
                    break;
                }
            }
        }
        if (count($getload) > 1) {
            $load_type = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
            $uc = 0;
            $oc = 0;
            foreach ($getload as $load_details) {
                if (in_array($load_details['load_type'], $load_type)) {
                    $uc++;
                } else {
                    $oc++;
                }
            }
            if ($oc > 0 && $uc > 0) {
                $bcoutn++;
            }
        }
    }

    $totalDays = $totalDays + $getDatePickupCount;
    $totalDays = $totalDays - $bcoutn;
    $mapeval = max(number_format(100 / 7 * $totalDays, 2), 0);

    return $mapeval;

}

function getTargetRevenueMilesByDateForRevenueTrackers($startDate, $endDate, $driver_id, $loadDetails)
{


    $get_start_quarter  = Carbon::parse($startDate)->quarter;
    $get_start_year     = Carbon::parse($startDate)->year;
    $get_end_quarter    = Carbon::parse($endDate)->quarter;
    $get_end_year       = Carbon::parse($endDate)->year;

    $data = DriverFinancial::where("driver_id", $driver_id)
        ->where("year", '>=', $get_start_year)
        ->where("year", '<=', $get_end_year)
        ->where("quarter", '>=', $get_start_quarter)
        ->where("quarter", '<=', $get_end_quarter)
        ->orderBy('quarter', 'ASC')
        ->get();

    $to_d               = Carbon::createFromFormat('Y-m-d', $startDate);
    $from_d             = Carbon::createFromFormat('Y-m-d', $endDate);
    $trevemiles         = [];
    $treverevenue       = [];
    $trevecpm           = [];
    $treveOccupancy     = [];
    $TotalQuarter     = [];

    $i=0;
    foreach ($data as $divData) {

        //dd($getPeriod_month,$getPeriod);
        $to             = Carbon::createFromFormat('Y-m-d', $divData->start_date);
        $from           = Carbon::createFromFormat('Y-m-d', $divData->end_date);
        $quarterdays    = $to->diffInDays($from) + 1;
        //$quarterdays  = $to->diffInDays($from);

        $perdayMiles    = $divData->target_miles / $quarterdays;
        $perdayRevenue  = $divData->revenue_target / $quarterdays;
        //$perdayCPM    = $divData->cpm_target / $quarterdays;
        //$perdayOccupancy = $divData->occupancy / $quarterdays;
        $perdayCPM      = $divData->cpm_target;
        $perdayOccupancy= $divData->occupancy;

        $period         = CarbonPeriod::create($divData->start_date, $divData->end_date);
        $daycount       = 1;
        foreach ($period as $date) {
            if ($date->between($to_d, $from_d)) {
                $daycount++;
            }
        }

        if ($daycount > 91) {
            $daycount = 91;
        }

        if ($daycount > 0) {
            $trevemiles[]       = $perdayMiles * $daycount;
            $treverevenue[]     = $perdayRevenue * $daycount;
            //$trevecpm[]       = $perdayCPM * $daycount;
            //$treveOccupancy[] = $perdayOccupancy * $daycount;
            $treveOccupancy[]   = $perdayOccupancy;
        }

        $date = $date->format('Y-m-d');
        $laod_array = [];
        $check_load_existe = 0;
        foreach($loadDetails as $load){
            $actual_shipper_date = \Carbon\Carbon::createFromFormat(getTimeLayout(), $load->actual_shipper_date)->format('Y-m-d');
            $actual_delivery_date = \Carbon\Carbon::createFromFormat(getTimeLayout(), $load->actual_delivery_date)->format('Y-m-d');
            if($actual_shipper_date <= $divData->end_date && $actual_delivery_date >= $divData->start_date){
                $check_load_existe++;
            }

        }

        if($check_load_existe > 0){
            $trevecpm[]         = $perdayCPM;
            $quarter = Carbon::createFromFormat('Y-m-d', $divData->end_date)->quarter;
            $year = Carbon::createFromFormat('Y-m-d', $divData->end_date)->year;
            $TotalQuarter[$i] = $quarter.$year;
            $i++;
        }


    }

    $gettotRevenue      = round(array_sum($treverevenue));
    $gettotMiles        = round(array_sum($trevemiles));
    $gettotCPM          = array_sum($trevecpm);
    $gettotOccupancy    = array_sum($treveOccupancy);

    $data = ['revenue' => $gettotRevenue,'miles' => $gettotMiles,'cpm' => $gettotCPM,'occupancy' => $gettotOccupancy,'getQuarter' => $TotalQuarter];
    return $data;
}

function getTargetRevenueMilesByDate($startDate, $endDate, $driverIds){
    $driverIds                              = array($driverIds);

    $startQuarterDate                       = Carbon::parse($startDate)->startOfQuarter()->format('Y-m-d');
    $endQuarterDate                         = Carbon::parse($endDate)->endOfQuarter()->format('Y-m-d');

    $getDriverFinancials                    = DriverFinancial::select('id','driver_id','target_miles','revenue_target','cpm_target','occupancy','year','quarter','start_date','end_date')
        ->whereIn('driver_id', $driverIds)
        ->whereDate('start_date', '>=', $startQuarterDate)
        ->whereDate('end_date', '<=', $endQuarterDate)
        ->orderBy('year', 'ASC')
        ->orderBy('quarter', 'ASC')
        ->get()->toArray();
    $arry                                   = [];
    if(!empty($getDriverFinancials)){
        $driverWiseFinancials               = [];
        $loadDriverWiseFinancials           = [];
        foreach ($getDriverFinancials as $dfkey => $value) {
            // $existsLoad                  = LoadDetail::where('driver_id', $value['driver_id'])
            //     ->where(function ($query) use ($value) {
            //         $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $value['end_date'])
            //             ->whereBetween(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), [$value['start_date'], $value['end_date']]);
            //     })
            //     ->where('load_status', '!=', 9)->where(function($q) { $q->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0,1]); })->exists();
            // if($existsLoad){
            //     $loadDriverWiseFinancials[$value['driver_id']][]     = $value;
            // }
            $driverWiseFinancials[$value['driver_id']][]                = $value;
        }

        //

        $to_d                               = Carbon::createFromFormat('Y-m-d', $startDate);
        $from_d                             = Carbon::createFromFormat('Y-m-d', $endDate);

        foreach ($driverWiseFinancials as $dwfdkey => $driverFinancials) {
            $loads                          = LoadDetail::selectRaw(
                    'YEAR(actual_shipper_date) AS actual_shipper_date_year,
                    QUARTER(DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")) AS actual_shipper_date_quarter,
                    YEAR(actual_delivery_date) AS actual_delivery_date_year,
                    QUARTER(DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")) AS actual_delivery_date_quarter
                    '
                )
                ->where('driver_id', $dwfdkey)
                ->where(function ($query) use ($driverFinancials) {
                    foreach ($driverFinancials as $key => $value) {
                        if($key==0){
                            $query->where(function ($q) use ($value) {
                                $q->where(DB::raw('YEAR(actual_shipper_date)'), $value['year'])
                                    ->where(DB::raw('QUARTER(DATE_FORMAT(actual_shipper_date, "%Y-%m-%d"))'), $value['quarter'])
                                    ->where(DB::raw('YEAR(actual_delivery_date)'), $value['year'])
                                    ->where(DB::raw('QUARTER(DATE_FORMAT(actual_delivery_date, "%Y-%m-%d"))'), $value['quarter']);
                            });
                        }else{
                            $query->orWhere(function ($q) use ($value) {
                                $q->where(DB::raw('YEAR(actual_shipper_date)'), $value['year'])
                                    ->where(DB::raw('QUARTER(DATE_FORMAT(actual_shipper_date, "%Y-%m-%d"))'), $value['quarter'])
                                    ->where(DB::raw('YEAR(actual_delivery_date)'), $value['year'])
                                    ->where(DB::raw('QUARTER(DATE_FORMAT(actual_delivery_date, "%Y-%m-%d"))'), $value['quarter']);
                            });
                        }
                    }
                })
                ->where('load_status', '!=', 9)
                ->where(function($q) {
                    $q->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0,1]);
                })
                ->groupBy(['actual_shipper_date_year', 'actual_shipper_date_quarter'])
                ->groupBy(['actual_delivery_date_year', 'actual_delivery_date_quarter'])
                ->get()
                ->toArray();
            $flattened                      = Arr::dot($loads);

            $trevemiles                     = [];
            $treverevenue                   = [];
            //$trevecpm                     = [];
            //$treveOccupancy               = [];

            foreach ($driverFinancials as $dfdkey => $driverFinancial) {
                if(in_array($driverFinancial['year'], $flattened) && in_array($driverFinancial['quarter'], $flattened)){
                    $loadDriverWiseFinancials[$dwfdkey][] = $driverFinancial;
                }

                $to                         = Carbon::createFromFormat('Y-m-d', $driverFinancial['start_date']);
                $from                       = Carbon::createFromFormat('Y-m-d', $driverFinancial['end_date']);
                $quarterdays                = $to->diffInDays($from) + 1;

                $perdayMiles                = $driverFinancial['target_miles'] / $quarterdays;
                $perdayRevenue              = $driverFinancial['revenue_target'] / $quarterdays;
                //$perdayCPM                = $driverFinancial['cpm_target'];
                //$perdayOccupancy          = $driverFinancial['occupancy'];

                $period                     = CarbonPeriod::create($driverFinancial['start_date'], $driverFinancial['end_date']);
                $daycount                   = 1;
                foreach ($period as $date) {
                    if ($date->between($to_d, $from_d)) {
                        $daycount++;
                    }
                }

                if ($daycount > 91) {
                    $daycount = 91;
                }

                if ($daycount > 0) {
                    $trevemiles[]           = $perdayMiles * $daycount;
                    $treverevenue[]         = $perdayRevenue * $daycount;
                    //$trevecpm[]           = $perdayCPM;
                    //$treveOccupancy[]     = $perdayOccupancy;
                }



            }
            $gettotRevenue                  = array_sum($treverevenue);
            $gettotMiles                    = array_sum($trevemiles);
            //$gettotCPM                    = array_sum($trevecpm);
            //$gettotOccupancy              = array_sum($treveOccupancy);
            $arry[$dwfdkey]['revenue']      = $gettotRevenue;
            $arry[$dwfdkey]['miles']        = $gettotMiles;
            //$arry[$dwfdkey]['cpm']        = $gettotCPM;
            //$arry[$dwfdkey]['occupancy']  = $gettotOccupancy;
        }

        foreach ($loadDriverWiseFinancials as $ldwfdkey => $loadDriverFinancials) {
            $driverTotalQuarter             = count($loadDriverFinancials);
            $driverCpmTarget                = 0;
            $driverOccupancyTarget          = 0;
            foreach ($loadDriverFinancials as $dfdkey => $loadDriverFinancial) {
                $driverCpmTarget            += $loadDriverFinancial['cpm_target'];
                $driverOccupancyTarget      += $loadDriverFinancial['occupancy'];
            }
            $driverGoalCpmTarget            = $driverCpmTarget/$driverTotalQuarter;
            $driverGoalOccupancyTarget      = $driverOccupancyTarget/$driverTotalQuarter;

            $arry[$ldwfdkey]['cpm']         = $driverGoalCpmTarget;
            $arry[$ldwfdkey]['occupancy']   = $driverGoalOccupancyTarget;
        }
    }
    return [
        'drivers_wise_target'               => $arry,
        'revenue'                           => array_sum(array_column($arry, 'revenue')),
        'miles'                             => array_sum(array_column($arry, 'miles')),
        'cpm'                               => array_sum(array_column($arry, 'cpm')),
        'occupancy'                         => array_sum(array_column($arry, 'occupancy')),
    ];
}

function getTargetRevenueMilesByDate2($startDate, $endDate, $driverFinancialCollection = array()){

    $getDriverFinancials = $driverFinancialCollection->toArray();

    $arry = [];
    if(!empty($getDriverFinancials)){
        $driverWiseFinancials = [];
        $loadDriverWiseFinancials = [];
        foreach ($getDriverFinancials as $dfkey => $value) {
            $driverWiseFinancials[$value['driver_id']][] = $value;
        }

        $to_d = Carbon::createFromFormat('Y-m-d', $startDate);
        $from_d = Carbon::createFromFormat('Y-m-d', $endDate);
        foreach ($driverWiseFinancials as $dwfdkey => $driverFinancials) {
            $loads = LoadDetail::selectRaw('YEAR(actual_shipper_date) AS actual_shipper_date_year,
                        QUARTER(DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")) AS actual_shipper_date_quarter,
                        YEAR(actual_delivery_date) AS actual_delivery_date_year,
                        QUARTER(DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")) AS actual_delivery_date_quarter')
                ->where('driver_id', $dwfdkey)
                ->where(function ($query) use ($driverFinancials) {
                    foreach ($driverFinancials as $key => $value) {
                        if($key==0){
                            $query->where(function ($q) use ($value) {
                                $q->where(DB::raw('YEAR(actual_shipper_date)'), $value['year'])
                                    ->where(DB::raw('QUARTER(DATE_FORMAT(actual_shipper_date, "%Y-%m-%d"))'), $value['quarter'])
                                    ->where(DB::raw('YEAR(actual_delivery_date)'), $value['year'])
                                    ->where(DB::raw('QUARTER(DATE_FORMAT(actual_delivery_date, "%Y-%m-%d"))'), $value['quarter']);
                            });
                        }else{
                            $query->orWhere(function ($q) use ($value) {
                                $q->where(DB::raw('YEAR(actual_shipper_date)'), $value['year'])
                                    ->where(DB::raw('QUARTER(DATE_FORMAT(actual_shipper_date, "%Y-%m-%d"))'), $value['quarter'])
                                    ->where(DB::raw('YEAR(actual_delivery_date)'), $value['year'])
                                    ->where(DB::raw('QUARTER(DATE_FORMAT(actual_delivery_date, "%Y-%m-%d"))'), $value['quarter']);
                            });
                        }
                    }
                })
                ->where('load_status', '!=', 9)
                ->where(function($q) {
                    $q->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0,1]);
                })
                ->groupBy(['actual_shipper_date_year', 'actual_shipper_date_quarter'])
                ->groupBy(['actual_delivery_date_year', 'actual_delivery_date_quarter'])
                ->get()->toArray();
            $flattened = Arr::dot($loads);

            $trevemiles = [];
            $treverevenue = [];

            foreach ($driverFinancials as $dfdkey => $driverFinancial) {
                if(in_array($driverFinancial['year'], $flattened) && in_array($driverFinancial['quarter'], $flattened)){
                    $loadDriverWiseFinancials[$dwfdkey][] = $driverFinancial;
                }

                $to = Carbon::createFromFormat('Y-m-d', $driverFinancial['start_date']);
                $from = Carbon::createFromFormat('Y-m-d', $driverFinancial['end_date']);
                $quarterdays = $to->diffInDays($from) + 1;

                $perdayMiles = $driverFinancial['target_miles'] / $quarterdays;
                $perdayRevenue = $driverFinancial['revenue_target'] / $quarterdays;

                $period = CarbonPeriod::create($driverFinancial['start_date'], $driverFinancial['end_date']);
                $daycount = 1;
                foreach ($period as $date) {
                    if ($date->between($to_d, $from_d)) {
                        $daycount++;
                    }
                }

                if ($daycount > 91) {
                    $daycount = 91;
                }

                if ($daycount > 0) {
                    $trevemiles[]           = $perdayMiles * $daycount;
                    $treverevenue[]         = $perdayRevenue * $daycount;
                }
            }
            $gettotRevenue                  = round(array_sum($treverevenue));
            $gettotMiles                    = round(array_sum($trevemiles));

            $arry[$dwfdkey]['revenue']      = $gettotRevenue;
            $arry[$dwfdkey]['miles']        = $gettotMiles;
        }

        foreach ($loadDriverWiseFinancials as $ldwfdkey => $loadDriverFinancials) {
            $driverTotalQuarter             = count($loadDriverFinancials);
            $driverCpmTarget                = 0;
            $driverOccupancyTarget          = 0;
            foreach ($loadDriverFinancials as $dfdkey => $loadDriverFinancial) {
                $driverCpmTarget            += $loadDriverFinancial['cpm_target'];
                $driverOccupancyTarget      += $loadDriverFinancial['occupancy'];
            }
            $driverGoalCpmTarget            = $driverCpmTarget/$driverTotalQuarter;
            $driverGoalOccupancyTarget      = $driverOccupancyTarget/$driverTotalQuarter;

            $arry[$ldwfdkey]['cpm']         = $driverGoalCpmTarget;
            $arry[$ldwfdkey]['occupancy']   = $driverGoalOccupancyTarget;
        }
    }
    return [
        'drivers_wise_target'           => $arry,
        'revenue'                       => array_sum(array_column($arry, 'revenue')),
        'miles'                         => array_sum(array_column($arry, 'miles')),
        'cpm'                           => array_sum(array_column($arry, 'cpm')),
        'occupancy'                     => array_sum(array_column($arry, 'occupancy')),
    ];
}

function getLoadByDateMonth2($date, $driverId, $trailer_type = null, $dispatcher = null)
{

    $date = Carbon::parse($date)->format('Y-m-d');

    $data = LoadDetail::select("id","load_type")
        ->where('driver_id', $driverId)
        ->where('load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0,1]);
        })
        ->where(function ($query) use ($date) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<', $date)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $date)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $date);
        })
        ->groupBy('id')
        ->get();

    $data2 = LoadDetail::select("id","load_type")
        ->where('driver_id', $driverId)
        ->where('load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0,1]);
        })
        ->whereIn('load_type', [0, 4, 6]) // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
        ->where('occupied_pickup_day', 1)
        ->where(function ($query) use ($date) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $date)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $date)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $date);
        })
        ->groupBy('id')
        ->get();

    if (count($data2) > 0) {
        $data = $data2->merge($data);
    }

    $unoccu_data = LoadDetail::select("id","load_type")
        ->where('driver_id', $driverId)
        ->where('load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhereIn('is_on_hold', [0,1]);
        })
        ->whereIn('load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->where(function ($query) use ($date) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $date)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $date)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $date);
        })
        ->groupBy('id')
        ->get();

    $merged = $unoccu_data->merge($data);
    $result = $merged->all();

    return $result;
}

function getWeekDate($day=1)
{
    $weekly_strat_day   = $day ?? 1;
    if($weekly_strat_day==0){
        $startDate      = Carbon::now()->timezone(getTimeZone())->startOfWeek(Carbon::SUNDAY)->format('Y-m-d');
        $endDate        = Carbon::now()->timezone(getTimeZone())->endOfWeek(Carbon::SATURDAY)->format('Y-m-d');
    }elseif($weekly_strat_day==1){
        $startDate      = Carbon::now()->timezone(getTimeZone())->startOfWeek(Carbon::MONDAY)->format('Y-m-d');
        $endDate        = Carbon::now()->timezone(getTimeZone())->endOfWeek(Carbon::SUNDAY)->format('Y-m-d');
    }elseif($weekly_strat_day==2){
        $startDate      = Carbon::now()->timezone(getTimeZone())->startOfWeek(Carbon::TUESDAY)->format('Y-m-d');
        $endDate        = Carbon::now()->timezone(getTimeZone())->endOfWeek(Carbon::MONDAY)->format('Y-m-d');
    }elseif($weekly_strat_day==3){
        $startDate      = Carbon::now()->timezone(getTimeZone())->startOfWeek(Carbon::WEDNESDAY)->format('Y-m-d');
        $endDate        = Carbon::now()->timezone(getTimeZone())->endOfWeek(Carbon::TUESDAY)->format('Y-m-d');
    }elseif($weekly_strat_day==4){
        $startDate      = Carbon::now()->timezone(getTimeZone())->startOfWeek(Carbon::THURSDAY)->format('Y-m-d');
        $endDate        = Carbon::now()->timezone(getTimeZone())->endOfWeek(Carbon::WEDNESDAY)->format('Y-m-d');
    }elseif($weekly_strat_day==5){
        $startDate      = Carbon::now()->timezone(getTimeZone())->startOfWeek(Carbon::FRIDAY)->format('Y-m-d');
        $endDate        = Carbon::now()->timezone(getTimeZone())->endOfWeek(Carbon::THURSDAY)->format('Y-m-d');
    }elseif($weekly_strat_day==6){
        $startDate      = Carbon::now()->timezone(getTimeZone())->startOfWeek(Carbon::SATURDAY)->format('Y-m-d');
        $endDate        = Carbon::now()->timezone(getTimeZone())->endOfWeek(Carbon::FRIDAY)->format('Y-m-d');
    }
    return [
        'startDate' => $startDate,
        'endDate'   => $endDate
    ];
}

function getAnnouncementsUserList($search='', $column='id')
{
    $arr = array(
        array(
            "id"            => 1,
            "name"          => "Manager",
        ),
        array(
            "id"            => 2,
            "name"          => "Dispatcher"
        ),
        array(
            "id"            => 3,
            "name"          => "Driver"
        )
    );
    if($search!==''){
        $key    = array_search($search, array_column($arr, $column));
        if($key!==false){
            return $arr[$key];
        }else{
            return [];
        }
    }
    return $arr;
}

function widgetArray($param=[]) {
    if(auth()->guard('company-admin')->check() && !(isset($param['type']) && $param['type']==3)){
        if(empty($param)){
            $param['type']              = auth()->guard('company-admin')->user()->type;
            $param['tabname']           = 'own';
            $param['search']            = '';
            $param['column']            = 'id';
        }
        if($param['type']==2 && $param['tabname']=='own'){
            $arr    = array(
                array(
                    "id"                => 1,
                    "name"              => "Occupancy",
                    "slug"              => "occupancy",
                    "status"            => 1,
                    "display_no"        => 1,
                    "active_image"      => asset('theme/img/widget/occupancy.png'),
                    "inactive_image"    => asset('theme/img/widget/occupancy-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 0, 'w' => 3, 'h' => 3])
                ),
                array(
                    "id"                => 2,
                    "name"              => "Revenue",
                    "slug"              => "revenue",
                    "status"            => 1,
                    "display_no"        => 2,
                    "active_image"      => asset('theme/img/widget/revenue.png'),
                    "inactive_image"    => asset('theme/img/widget/revenue-gray.png'),
                    "properties"        => json_encode(['x' => 3, 'y' => 0, 'w' => 3, 'h' => 3])
                ),
                array(
                    "id"                => 3,
                    "name"              => "CPM",
                    "slug"              => "cpm",
                    "status"            => 1,
                    "display_no"        => 3,
                    "active_image"      => asset('theme/img/widget/cpm.png'),
                    "inactive_image"    => asset('theme/img/widget/cpm-gray.png'),
                    "properties"        => json_encode(['x' => 6, 'y' => 0, 'w' => 3, 'h' => 3])
                ),
                array(
                    "id"                => 4,
                    "name"              => "Trailer Type",
                    "slug"              => "trailer_type",
                    "status"            => 1,
                    "display_no"        => 4,
                    "active_image"      => asset('theme/img/widget/trailerType.png'),
                    "inactive_image"    => asset('theme/img/widget/trailerType-gray.png'),
                    "properties"        => json_encode(['x' => 9, 'y' => 0, 'w' => 3, 'h' => 3])
                ),
                // array(
                //     "id"                => 5,
                //     "name"              => "Company Stats",
                //     "slug"              => "company_stats",
                //     "status"            => 1,
                //     "display_no"        => 13,
                //     "active_image"      => asset('theme/img/widget/faTable.png'),
                //     "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                //     "properties"        => json_encode(['x' => 0, 'y' => 3, 'w' => 3, 'h' => 3])
                // ),
                // array(
                //     "id"                => 6,
                //     "name"              => "Driver Per Team",
                //     "slug"              => "driver_per_team",
                //     "status"            => 1,
                //     "display_no"        => 5,
                //     "active_image"      => asset('theme/img/widget/driverByTeam.png'),
                //     "inactive_image"    => asset('theme/img/widget/driverByTeam-gray.png'),
                //     "properties"        => json_encode(['x' => 3, 'y' => 3, 'w' => 6, 'h' => 3])
                // ),
                array(
                    "id"                => 7,
                    "name"              => "Revenue",
                    "slug"              => "target_actual_revenue",
                    "status"            => 1,
                    "display_no"        => 8,
                    "active_image"      => asset('theme/img/widget/faArea.png'),
                    "inactive_image"    => asset('theme/img/widget/faArea-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 3, 'w' => 12, 'h' => 4])
                ),
                array(
                    "id"                => 8,
                    "name"              => "Loads In Transit",
                    "slug"              => "loads_in_transit",
                    "status"            => 1,
                    "display_no"        => 12,
                    "active_image"      => asset('theme/img/widget/faTable.png'),
                    "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 7, 'w' => 12, 'h' => 4])
                ),
                // array(
                //     "id"                => 9,
                //     "name"              => "Revenue By Team",
                //     "slug"              => "revenue_by_team",
                //     "status"            => 1,
                //     "display_no"        => 6,
                //     "active_image"      => asset('theme/img/widget/faPieChart.png'),
                //     "inactive_image"    => asset('theme/img/widget/faPieChart-gray.png'),
                //     "properties"        => json_encode(['x' => 0, 'y' => 14, 'w' => 6, 'h' => 3])
                // ),
                array(
                    "id"                => 10,
                    "name"              => "Revenue By Trailer Type",
                    "slug"              => "revenue_by_trailer_type",
                    "status"            => 1,
                    "display_no"        => 7,
                    "active_image"      => asset('theme/img/widget/faPieChart.png'),
                    "inactive_image"    => asset('theme/img/widget/faPieChart-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 11, 'w' => 12, 'h' => 3])
                ),
                array(
                    "id"                => 11,
                    "name"              => "Loads Stats",
                    "slug"              => "loads_stats",
                    "status"            => 1,
                    "display_no"        => 14,
                    "active_image"      => asset('theme/img/widget/faTable.png'),
                    "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 26, 'w' => 4, 'h' => 3])
                ),
                array(
                    "id"                => 12,
                    "name"              => "Revenue Actual Vs Goal",
                    "slug"              => "revenue_actual_vs_goal",
                    "status"            => 1,
                    "display_no"        => 10,
                    "active_image"      => asset('theme/img/widget/faArea.png'),
                    "inactive_image"    => asset('theme/img/widget/faArea-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 17, 'w' => 12, 'h' => 3])
                ),
                array(
                    "id"                => 13,
                    "name"              => "Revenue Occupancy Vs Goal",
                    "slug"              => "revenue_occupancy_vs_goal",
                    "status"            => 1,
                    "display_no"        => 11,
                    "active_image"      => asset('theme/img/widget/faArea.png'),
                    "inactive_image"    => asset('theme/img/widget/faArea-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 20, 'w' => 12, 'h' => 3])
                ),
                array(
                    "id"                => 14,
                    "name"              => "Revenue & Occupancy",
                    "slug"              => "revenue_occupancy",
                    "status"            => 1,
                    "display_no"        => 9,
                    "active_image"      => asset('theme/img/widget/faArea.png'),
                    "inactive_image"    => asset('theme/img/widget/faArea-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 14, 'w' => 12, 'h' => 3])
                ),
                array(
                    "id"                => 15,
                    "name"              => "Events",
                    "slug"              => "events",
                    "status"            => 1,
                    "display_no"        => 15,
                    "active_image"      => asset('theme/img/widget/faTable.png'),
                    "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 23, 'w' => 4, 'h' => 3])
                ),
                array(
                    "id"                => 16,
                    "name"              => "Notes",
                    "slug"              => "notes",
                    "status"            => 1,
                    "display_no"        => 16,
                    "active_image"      => asset('theme/img/widget/faTable.png'),
                    "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                    "properties"        => json_encode(['x' => 4, 'y' => 26, 'w' => 8, 'h' => 3])
                ),
                array(
                    "id"                => 17,
                    "name"              => "Compel Announcements",
                    "slug"              => "compel_announcements",
                    "status"            => 1,
                    "display_no"        => 17,
                    "active_image"      => asset('theme/img/widget/faTable.png'),
                    "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                    "properties"        => json_encode(['x' => 4, 'y' => 23, 'w' => 4, 'h' => 3])
                ),
                array(
                    "id"                => 18,
                    "name"              => "Company Announcements",
                    "slug"              => "company_announcements",
                    "status"            => 1,
                    "display_no"        => 18,
                    "active_image"      => asset('theme/img/widget/faTable.png'),
                    "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                    "properties"        => json_encode(['x' => 8, 'y' => 23, 'w' => 4, 'h' => 3])
                )
            );
        }else{
            $arr    = array(
                array(
                    "id"                => 1,
                    "name"              => "Occupancy",
                    "slug"              => "occupancy",
                    "status"            => 1,
                    "display_no"        => 1,
                    "group"             => 2,
                    "active_image"      => asset('theme/img/widget/occupancy.png'),
                    "inactive_image"    => asset('theme/img/widget/occupancy-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 0, 'w' => 3, 'h' => 3])
                ),
                array(
                    "id"                => 2,
                    "name"              => "Revenue",
                    "slug"              => "revenue",
                    "status"            => 1,
                    "display_no"        => 2,
                    "active_image"      => asset('theme/img/widget/revenue.png'),
                    "inactive_image"    => asset('theme/img/widget/revenue-gray.png'),
                    "properties"        => json_encode(['x' => 3, 'y' => 0, 'w' => 3, 'h' => 3])
                ),
                array(
                    "id"                => 3,
                    "name"              => "CPM",
                    "slug"              => "cpm",
                    "status"            => 1,
                    "display_no"        => 3,
                    "active_image"      => asset('theme/img/widget/cpm.png'),
                    "inactive_image"    => asset('theme/img/widget/cpm-gray.png'),
                    "properties"        => json_encode(['x' => 6, 'y' => 0, 'w' => 3, 'h' => 3])
                ),
                array(
                    "id"                => 4,
                    "name"              => "Trailer Type",
                    "slug"              => "trailer_type",
                    "status"            => 1,
                    "display_no"        => 4,
                    "active_image"      => asset('theme/img/widget/trailerType.png'),
                    "inactive_image"    => asset('theme/img/widget/trailerType-gray.png'),
                    "properties"        => json_encode(['x' => 9, 'y' => 0, 'w' => 3, 'h' => 6])
                ),
                array(
                    "id"                => 5,
                    "name"              => "Company Stats",
                    "slug"              => "company_stats",
                    "status"            => 1,
                    "display_no"        => 13,
                    "active_image"      => asset('theme/img/widget/faTable.png'),
                    "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 3, 'w' => 3, 'h' => 3])
                ),
                array(
                    "id"                => 6,
                    "name"              => "Driver Per Team",
                    "slug"              => "driver_per_team",
                    "status"            => 1,
                    "display_no"        => 5,
                    "active_image"      => asset('theme/img/widget/driverByTeam.png'),
                    "inactive_image"    => asset('theme/img/widget/driverByTeam-gray.png'),
                    "properties"        => json_encode(['x' => 3, 'y' => 3, 'w' => 6, 'h' => 3])
                ),
                array(
                    "id"                => 7,
                    "name"              => "Revenue",
                    "slug"              => "target_actual_revenue",
                    "status"            => 1,
                    "display_no"        => 8,
                    "active_image"      => asset('theme/img/widget/faArea.png'),
                    "inactive_image"    => asset('theme/img/widget/faArea-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 6, 'w' => 12, 'h' => 4])
                ),
                array(
                    "id"                => 8,
                    "name"              => "Loads In Transit",
                    "slug"              => "loads_in_transit",
                    "status"            => 1,
                    "display_no"        => 12,
                    "active_image"      => asset('theme/img/widget/faTable.png'),
                    "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 10, 'w' => 12, 'h' => 4])
                ),
                array(
                    "id"                => 9,
                    "name"              => "Revenue By Team",
                    "slug"              => "revenue_by_team",
                    "status"            => 1,
                    "display_no"        => 6,
                    "active_image"      => asset('theme/img/widget/faPieChart.png'),
                    "inactive_image"    => asset('theme/img/widget/faPieChart-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 14, 'w' => 6, 'h' => 3])
                ),
                array(
                    "id"                => 10,
                    "name"              => "Revenue By Trailer Type",
                    "slug"              => "revenue_by_trailer_type",
                    "status"            => 1,
                    "display_no"        => 7,
                    "active_image"      => asset('theme/img/widget/faPieChart.png'),
                    "inactive_image"    => asset('theme/img/widget/faPieChart-gray.png'),
                    "properties"        => json_encode(['x' => 6, 'y' => 14, 'w' => 6, 'h' => 3])
                ),
                array(
                    "id"                => 11,
                    "name"              => "Loads Stats",
                    "slug"              => "loads_stats",
                    "status"            => 1,
                    "display_no"        => 14,
                    "active_image"      => asset('theme/img/widget/faTable.png'),
                    "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 29, 'w' => 4, 'h' => 3])
                ),
                array(
                    "id"                => 12,
                    "name"              => "Revenue Actual Vs Goal",
                    "slug"              => "revenue_actual_vs_goal",
                    "status"            => 1,
                    "display_no"        => 10,
                    "active_image"      => asset('theme/img/widget/faArea.png'),
                    "inactive_image"    => asset('theme/img/widget/faArea-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 20, 'w' => 12, 'h' => 3])
                ),
                array(
                    "id"                => 13,
                    "name"              => "Revenue Occupancy Vs Goal",
                    "slug"              => "revenue_occupancy_vs_goal",
                    "status"            => 1,
                    "display_no"        => 11,
                    "active_image"      => asset('theme/img/widget/faArea.png'),
                    "inactive_image"    => asset('theme/img/widget/faArea-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 23, 'w' => 12, 'h' => 3])
                ),
                array(
                    "id"                => 14,
                    "name"              => "Revenue & Occupancy",
                    "slug"              => "revenue_occupancy",
                    "status"            => 1,
                    "display_no"        => 9,
                    "active_image"      => asset('theme/img/widget/faArea.png'),
                    "inactive_image"    => asset('theme/img/widget/faArea-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 17, 'w' => 12, 'h' => 3])
                ),
                array(
                    "id"                => 15,
                    "name"              => "Events",
                    "slug"              => "events",
                    "status"            => 1,
                    "display_no"        => 15,
                    "active_image"      => asset('theme/img/widget/faTable.png'),
                    "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                    "properties"        => json_encode(['x' => 0, 'y' => 26, 'w' => 4, 'h' => 3])
                ),
                array(
                    "id"                => 16,
                    "name"              => "Notes",
                    "slug"              => "notes",
                    "status"            => 1,
                    "display_no"        => 16,
                    "active_image"      => asset('theme/img/widget/faTable.png'),
                    "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                    "properties"        => json_encode(['x' => 4, 'y' => 29, 'w' => 8, 'h' => 3])
                ),
                array(
                    "id"                => 17,
                    "name"              => "Compel Announcements",
                    "slug"              => "compel_announcements",
                    "status"            => 1,
                    "display_no"        => 17,
                    "active_image"      => asset('theme/img/widget/faTable.png'),
                    "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                    "properties"        => json_encode(['x' => 4, 'y' => 26, 'w' => 4, 'h' => 3])
                ),
                array(
                    "id"                => 18,
                    "name"              => "Company Announcements",
                    "slug"              => "company_announcements",
                    "status"            => 1,
                    "display_no"        => 18,
                    "active_image"      => asset('theme/img/widget/faTable.png'),
                    "inactive_image"    => asset('theme/img/widget/faTable-gray.png'),
                    "properties"        => json_encode(['x' => 8, 'y' => 26, 'w' => 4, 'h' => 3])
                )
            );
        }
        $new        = array_values(array_filter($arr, function ($var) {
                return ($var['status'] == 1);
            }));
        if($param['search']!=''){
            $key    = array_search($param['search'], array_column($new, $param['column']));
            if($key!==false){
                return $new[$key];
            }else{
                return [];
            }
        }
        return $new;
    }elseif(auth()->guard('driver-admin')->check()){
        if(empty($param)){
            $param['search']            = '';
            $param['column']            = 'id';
        }
        $arr        = array(
            array(
                "id"                    => 1,
                "name"                  => "Occupancy",
                "slug"                  => "occupancy",
                "status"                => 1,
                "display_no"            => 1,
                "active_image"          => asset('theme/img/widget/occupancy.png'),
                "inactive_image"        => asset('theme/img/widget/occupancy-gray.png'),
                "properties"            => json_encode(['x' => 0, 'y' => 0, 'w' => 3, 'h' => 3])
            ),
            array(
                "id"                    => 2,
                "name"                  => "Revenue",
                "slug"                  => "revenue",
                "status"                => 1,
                "display_no"            => 2,
                "active_image"          => asset('theme/img/widget/revenue.png'),
                "inactive_image"        => asset('theme/img/widget/revenue-gray.png'),
                "properties"            => json_encode(['x' => 3, 'y' => 0, 'w' => 3, 'h' => 3])
            ),
            array(
                "id"                    => 3,
                "name"                  => "CPM",
                "slug"                  => "cpm",
                "status"                => 1,
                "display_no"            => 3,
                "active_image"          => asset('theme/img/widget/cpm.png'),
                "inactive_image"        => asset('theme/img/widget/cpm-gray.png'),
                "properties"            => json_encode(['x' => 6, 'y' => 0, 'w' => 3, 'h' => 3])
            ),
            array(
                "id"                    => 4,
                "name"                  => "Trailer Type",
                "slug"                  => "trailer_type",
                "status"                => 1,
                "display_no"            => 4,
                "active_image"          => asset('theme/img/widget/trailerType.png'),
                "inactive_image"        => asset('theme/img/widget/trailerType-gray.png'),
                "properties"            => json_encode(['x' => 9, 'y' => 0, 'w' => 3, 'h' => 3])
            ),
            array(
                "id"                    => 7,
                "name"                  => "Revenue",
                "slug"                  => "target_actual_revenue",
                "status"                => 1,
                "display_no"            => 8,
                "active_image"          => asset('theme/img/widget/faArea.png'),
                "inactive_image"        => asset('theme/img/widget/faArea-gray.png'),
                "properties"            => json_encode(['x' => 0, 'y' => 3, 'w' => 12, 'h' => 4])
            ),
            array(
                "id"                    => 8,
                "name"                  => "Loads In Transit",
                "slug"                  => "loads_in_transit",
                "status"                => 1,
                "display_no"            => 12,
                "active_image"          => asset('theme/img/widget/faTable.png'),
                "inactive_image"        => asset('theme/img/widget/faTable-gray.png'),
                "properties"            => json_encode(['x' => 0, 'y' => 7, 'w' => 12, 'h' => 4])
            ),
            array(
                "id"                    => 10,
                "name"                  => "Revenue By Trailer Type",
                "slug"                  => "revenue_by_trailer_type",
                "status"                => 1,
                "display_no"            => 7,
                "active_image"          => asset('theme/img/widget/faPieChart.png'),
                "inactive_image"        => asset('theme/img/widget/faPieChart-gray.png'),
                "properties"            => json_encode(['x' => 0, 'y' => 11, 'w' => 12, 'h' => 3])
            ),
            array(
                "id"                    => 11,
                "name"                  => "Loads Stats",
                "slug"                  => "loads_stats",
                "status"                => 1,
                "display_no"            => 14,
                "active_image"          => asset('theme/img/widget/faTable.png'),
                "inactive_image"        => asset('theme/img/widget/faTable-gray.png'),
                "properties"            => json_encode(['x' => 0, 'y' => 26, 'w' => 4, 'h' => 3])
            ),
            array(
                "id"                    => 12,
                "name"                  => "Revenue Actual Vs Goal",
                "slug"                  => "revenue_actual_vs_goal",
                "status"                => 1,
                "display_no"            => 10,
                "active_image"          => asset('theme/img/widget/faArea.png'),
                "inactive_image"        => asset('theme/img/widget/faArea-gray.png'),
                "properties"            => json_encode(['x' => 0, 'y' => 17, 'w' => 12, 'h' => 3])
            ),
            array(
                "id"                    => 13,
                "name"                  => "Revenue Occupancy Vs Goal",
                "slug"                  => "revenue_occupancy_vs_goal",
                "status"                => 1,
                "display_no"            => 11,
                "active_image"          => asset('theme/img/widget/faArea.png'),
                "inactive_image"        => asset('theme/img/widget/faArea-gray.png'),
                "properties"            => json_encode(['x' => 0, 'y' => 20, 'w' => 12, 'h' => 3])
            ),
            array(
                "id"                    => 14,
                "name"                  => "Revenue & Occupancy",
                "slug"                  => "revenue_occupancy",
                "status"                => 1,
                "display_no"            => 9,
                "active_image"          => asset('theme/img/widget/faArea.png'),
                "inactive_image"        => asset('theme/img/widget/faArea-gray.png'),
                "properties"            => json_encode(['x' => 0, 'y' => 14, 'w' => 12, 'h' => 3])
            ),
            array(
                "id"                    => 15,
                "name"                  => "Events",
                "slug"                  => "events",
                "status"                => 1,
                "display_no"            => 15,
                "active_image"          => asset('theme/img/widget/faTable.png'),
                "inactive_image"        => asset('theme/img/widget/faTable-gray.png'),
                "properties"            => json_encode(['x' => 0, 'y' => 23, 'w' => 4, 'h' => 3])
            ),
            array(
                "id"                    => 16,
                "name"                  => "Notes",
                "slug"                  => "notes",
                "status"                => 1,
                "display_no"            => 16,
                "active_image"          => asset('theme/img/widget/faTable.png'),
                "inactive_image"        => asset('theme/img/widget/faTable-gray.png'),
                "properties"            => json_encode(['x' => 4, 'y' => 26, 'w' => 8, 'h' => 3])
            ),
            array(
                "id"                    => 17,
                "name"                  => "Compel Announcements",
                "slug"                  => "compel_announcements",
                "status"                => 1,
                "display_no"            => 17,
                "active_image"          => asset('theme/img/widget/faTable.png'),
                "inactive_image"        => asset('theme/img/widget/faTable-gray.png'),
                "properties"            => json_encode(['x' => 4, 'y' => 23, 'w' => 4, 'h' => 3])
            ),
            array(
                "id"                    => 18,
                "name"                  => "Company Announcements",
                "slug"                  => "company_announcements",
                "status"                => 1,
                "display_no"            => 18,
                "active_image"          => asset('theme/img/widget/faTable.png'),
                "inactive_image"        => asset('theme/img/widget/faTable-gray.png'),
                "properties"            => json_encode(['x' => 8, 'y' => 23, 'w' => 4, 'h' => 3])
            )
        );
        $new        = array_values(array_filter($arr, function ($var) {
                return ($var['status'] == 1);
            }));
        if($param['search']!=''){
            $key    = array_search($param['search'], array_column($new, $param['column']));
            if($key!==false){
                return $new[$key];
            }else{
                return [];
            }
        }
        return $new;
    }else{
        return [];
    }
}

function getWeekRangeForLoadOfWeek($year,$weeklyDay){
    $weeklyStar     = 0;
    $weeklyEnd      = 6;
    if($weeklyDay == 0){
        $weeklyStar = 0;
        $weeklyEnd  = 6;
    }
    if($weeklyDay == 1){
        $weeklyStar = 1;
        $weeklyEnd  = 0;
    }
    if($weeklyDay == 2){
        $weeklyStar = 2;
        $weeklyEnd  = 1;
    }
    if($weeklyDay == 3){
        $weeklyStar = 3;
        $weeklyEnd  = 2;
    }
    if($weeklyDay == 4){
        $weeklyStar = 4;
        $weeklyEnd  = 3;
    }
    if($weeklyDay == 5){
        $weeklyStar = 5;
        $weeklyEnd  = 4;
    }
    if($weeklyDay == 6){
        $weeklyStar = 6;
        $weeklyEnd  = 5;
    }

    //format string
    $f          = 'm/d/Y';

    $allweek    = '';
    $dates      = [];
    $m          = 1;
    for ($x = 1; $x <= 1; $x++) {
        $date   = $year.'-'.$x.'-1';
        $today  = Carbon::parse($date);
        $date   = $today->copy()->firstOfMonth()->startOfDay();
        $eom    = $today->copy()->addMonth(12)->startOfDay();

        $m=1;
        for($i  = 1; $date->lte($eom); $i++){
            $sdate = clone $date->startOfWeek($weeklyStar);
            $edate = clone $date->endOfWeek($weeklyEnd);

            if($year == $sdate->format('Y')){
                $dates[$m]['name']          = $sdate->format($f).' - '.$edate->format($f);
                $dates[$m]['year']          = $sdate->format('Y');
                $dates[$m]['start_date']    = $sdate->format($f);
                $dates[$m]['end_date']      = $edate->format($f);
                $m++;
            }
            $date->addDays(1);
        }
    }
    return $dates;
}


function getTerminalList($id=null)
{
    if(!empty($id)){
        return Terminal::IsCompany()->whereNotIn('id', [$id])->pluck('name', 'id')->all();
    }else{
        return Terminal::IsCompany()->pluck('name', 'id')->all();
    }

}

function getCarrierList()
{
    return Carrier::IsCompany()->pluck('name', 'id')->all();
}

function getTruckOwnerList()
{
    return TruckOwner::IsCompany()->pluck('name', 'id')->all();
}

function getBrokerContactList($source, $action_id)
{
    return BrokerContact::where('source', $source)->where('action_id', $action_id)->orderBy('id','ASC')->get();
}

function getGalleryCount($source, $action_id, $filed_name)
{
        $getGallery = Gallery::IsCompany()->where('action_id', $action_id)->where('source', $source)->where('field_name', $filed_name)->whereNotNull('image')->get()->count();
        if($getGallery > 0){
            return 1;
        }
    return 0;
}

function getGalleryDocumentNameList($source=null)
{
    if(!empty($source)){
        return GalleryDocumentName::IsCompany()->where('field_name',$source)->pluck('name', 'id')->all();
    }else{
        return GalleryDocumentName::IsCompany()->pluck('name', 'id')->all();
    }

}

function getRequestDocumentsSource(){
    return $arr = array(
        array(
            "name"          => "Truck",
            "source"          => "truck",
        ),
        array(
            "name"          => "Trailer",
            "source"          => "trailer",
        ),
        array(
            "name"          => "Driver",
            "source"          => "driver",
        )
    );
}

function getRequestDocumentsList($search='', $column='source') {
    $arr = array(
        array(
            "name"              => "Tag Photo",
            "field_name"        => "tag_photo",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Phys Dam Attachment",
            "field_name"        => "phys_dam_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 3,
        ),
        array(
            "name"              => "Bobtail Attachment",
            "field_name"        => "bobtail_carrier_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 3,
        ),
        array(
            "name"              => "FAI Photo",
            "field_name"        => "fai_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "KYU Photo",
            "field_name"        => "kyu_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "2290 Photo",
            "field_name"        => "2290_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "NYHUT 2290 Photo",
            "field_name"        => "nyhut_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "CARB ID Photo",
            "field_name"        => "carb_id_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Other",
            "field_name"        => "other_truck",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Profile Image",
            "field_name"        => "profile_image",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "License Image",
            "field_name"        => "license_exp_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Other",
            "field_name"        => "other_driver",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Tag Upload Image",
            "field_name"        => "tag_upload_image",
            "source"            => 'trailer',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Physical Dam Upload Image",
            "field_name"        => "phys_dam_attachment",
            "source"            => 'trailer',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "FAI Upload Image",
            "field_name"        => "fai_attachment",
            "source"            => 'trailer',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Trailer Interchange Upload Image",
            "field_name"        => "trailer_interchange_image",
            "source"            => 'trailer',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Owner Attachment",
            "field_name"        => "owner_documents",
            "source"            => 'trailer',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Lease Attachment",
            "field_name"        => "lease_documents",
            "source"            => 'trailer',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Other",
            "field_name"        => "other_trailer",
            "source"            => 'trailer',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Violations Attachment",
            "field_name"        => "image",
            "source"            => 'driver_hos_violations',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Truck Gallery",
            "field_name"        => "truck_gallery",
            "source"            => 'gallery',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Trailer Gallery",
            "field_name"        => "trailer_gallery",
            "source"            => 'gallery',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "COD Up Front",
            "field_name"        => "up_front_file",
            "source"            => 'load',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "COD At Pickup",
            "field_name"        => "at_pickup_file",
            "source"            => 'load',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "COD At Delivery",
            "field_name"        => "at_delivery_file",
            "source"            => 'load',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Load Accessorial",
            "field_name"        => "accessorial_files",
            "source"            => 'load_accessorials',
            "status"            => 1,
            "file_type"         => 3,
        ),
        array(
            "name"              => "Load Accessorial Temp",
            "field_name"        => "accessorial_files",
            "source"            => 'load_accessorial_temps',
            "status"            => 1,
            "file_type"         => 3,
        ),
        array(
            "name"              => "Insurance File",
            "field_name"        => "insurance_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "TWIC Certified Image",
            "field_name"        => "twic_certified_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "TSA Certified Image",
            "field_name"        => "tsa_certified_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Passport Image",
            "field_name"        => "passport_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Medical Certificate Image",
            "field_name"        => "medical_certificate_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Last MVR Image",
            "field_name"        => "last_mvr_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Drug & Alcohol Image",
            "field_name"        => "drug_alcohol_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Next Review Image",
            "field_name"        => "next_review_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Tanker Endorsement Image",
            "field_name"        => "tanker_endorsement_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Deduction File",
            "field_name"        => "deduction_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Paid Time Off File",
            "field_name"        => "paid_time_off_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Passenger File",
            "field_name"        => "passenger_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Pet File",
            "field_name"        => "pet_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Medical File",
            "field_name"        => "medical_file",
            "source"            => 'driver',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Insurance Profile File",
            "field_name"        => "profile_insurance_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Additional Insurance File",
            "field_name"        => "additional_insurance_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Purchase Agreement File",
            "field_name"        => "purchase_agreement_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Lease Agreement File",
            "field_name"        => "lease_agreement_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Rental Agreement File",
            "field_name"        => "rental_agreement_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Oregon File",
            "field_name"        => "oregon_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Document Profile File",
            "field_name"        => "document_profile_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Additional Document File",
            "field_name"        => "additional_document_file",
            "source"            => 'truck',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Customer General Info File",
            "field_name"        => "customer_gi_file",
            "source"            => 'brokers',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Customer Credit App File",
            "field_name"        => "upload_credit_app_file",
            "source"            => 'brokers',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Carrier Insurance File",
            "field_name"        => "carrier_insurance_file",
            "source"            => 'brokers',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Customer Insurance File",
            "field_name"        => "customer_insurance_file",
            "source"            => 'brokers',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Bond Information File",
            "field_name"        => "bond_information_file",
            "source"            => 'brokers',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Contact File",
            "field_name"        => "contact_file",
            "source"            => 'brokers',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Contract File",
            "field_name"        => "contract_file",
            "source"            => 'brokers',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Pricing File",
            "field_name"        => "pricing_file",
            "source"            => 'brokers',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Fuel Table File",
            "field_name"        => "fuel_table_file",
            "source"            => 'brokers',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Quick Quote File",
            "field_name"        => "quick_quote_file",
            "source"            => 'brokers',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Performance Review File",
            "field_name"        => "performance_review_file",
            "source"            => 'brokers',
            "status"            => 1,
            "file_type"         => 1,
        ),
        array(
            "name"              => "Insurance Requirement File",
            "field_name"        => "insurance_requirement_file",
            "source"            => 'brokers',
            "status"            => 1,
            "file_type"         => 1,
        ),
    );
    $new        = array_filter($arr, function ($var) {
        return ($var['status'] == 1);
    });
    if($search!=''){
        $key    = array_search($search, array_column($new, $column));
        if($key!==false){
            return $new[$key];
        }else{
            return [];
        }
    }
    return $new;
}

function getFullNameByid($id, $type){
    if($type == 3){
        $driver         = Driver::find($id);
        if(!empty($driver->id)){
            return $driver->full_name;
        }else{
            return "None";
        }

    }

    if($type == 0 || $type == 1 || $type == 2){
        $companyAdmin   = CompanyAdmin::find($id);
        if(!empty($companyAdmin->id)){
            return $companyAdmin->full_name;
        }else{
            return "None";
        }
    }

    return "None";
}

function checkOosDriver($driver_id){
    $hos        = DriverHosViolation::join('driver_hos_violations_datas as dhvd', function ($join){
        $join->on('driver_hos_violations.id', '=', 'dhvd.driver_hos_violation_id')
             ->where('dhvd.unit_type', 1)
             ->where('dhvd.oos', 1);
    })
    ->where('driver_hos_violations.driver_id', $driver_id)
    ->where('driver_hos_violations.status', 'incomplete')
    ->select('driver_hos_violations.id')
    ->first();
    if(isset($hos->id) && !empty($hos->id)){
        return true;
    }
    return false;
}

function checkOosTrailer($trailer_number){
    $trailer    = Trailer::where('trailer_id', $trailer_number)->first();
    if(!empty($trailer->id)){
       $hos     = DriverHosViolation::join('driver_hos_violations_datas as dhvd', function ($join){
            $join->on('driver_hos_violations.id', '=', 'dhvd.driver_hos_violation_id')
                 ->where('dhvd.unit_type', 3)
                 ->where('dhvd.oos', 1);
        })
        ->where('driver_hos_violations.unit2', $trailer->id)
        ->where('driver_hos_violations.status', 'incomplete')
        ->select('driver_hos_violations.id')
        ->first();
        if(isset($hos->id) && !empty($hos->id)){
            return true;
        }
    }
    return false;
}

function checkOosTruck($truck_number){
    $truck      = Truck::where('unit', $truck_number)->first();
    if(!empty($trailer->id)){
       $hos     = DriverHosViolation::join('driver_hos_violations_datas as dhvd', function ($join){
            $join->on('driver_hos_violations.id', '=', 'dhvd.driver_hos_violation_id')
                 ->where('dhvd.unit_type', 2)
                 ->where('dhvd.oos', 1);
        })
        ->where('driver_hos_violations.unit1', $truck->id)
        ->where('driver_hos_violations.status', 'incomplete')
        ->select('driver_hos_violations.id')
        ->first();
        if(isset($hos->id) && !empty($hos->id)){
            return true;
        }
    }
    return false;
}

function getOosTruck(){
    $hos        = DriverHosViolation::join('driver_hos_violations_datas as dhvd', function ($join){
        $join->on('driver_hos_violations.id', '=', 'dhvd.driver_hos_violation_id')
             ->where('dhvd.unit_type', 2)
             ->where('dhvd.oos', 1);
    })
    ->join("trucks", "trucks.id", "=", "driver_hos_violations.unit1")
    ->whereNotNull('driver_hos_violations.unit1')
    ->where('driver_hos_violations.status', 'incomplete')
    ->groupBy('driver_hos_violations.id')
    ->pluck('trucks.unit')
    ->all();
    return $hos;
}

function geolocation($latitude,$longitude){
    $location       = '';
    $geolocation    = $latitude.','.$longitude;
    $request        = 'https://maps.googleapis.com/maps/api/geocode/json?latlng='.$geolocation.'&language=en-EN&sensor=false&key='.env('GOOGLE_MAPS_KEY');
    $file_contents  = @file_get_contents($request);
    $json_decode    = json_decode($file_contents);
    if(isset($json_decode->results[0])) {
        $response   = array();
        foreach($json_decode->results[0]->address_components as $addressComponet) {
            if(in_array('political', $addressComponet->types)) {
                $response[] = $addressComponet->long_name;
            }
        }
        //https://stackoverflow.com/questions/29624114/get-location-from-latitude-and-longitude-in-php
        if(!empty($response)){
            $location       =  implode(' , ', $response);
        }
    }
    return $location;
}

/*
function UploadImagePdf($files, $path){
    $file_temp              = [];
    $image_array            = [];
    foreach($files as $image){
        $seme               = FileUpload::upload('companyAdmin/temp', $image);
        $stopath            = public_path('storage/companyAdmin/temp/'.$seme);
        $data               = file_get_contents($stopath);
        $type               = pathinfo($stopath, PATHINFO_EXTENSION);
        if($type == 'png' || $type == 'jpeg' || $type == 'jpg' || $type == 'gif'){
            $base64         = 'data:image/' . $type . ';base64,' . base64_encode($data);
            $file_temp[]    = $base64;
        }else{
            $image_array[]  = $seme = FileUpload::upload($path, $image);
        }
    }

    if(count($file_temp) > 0){
        $fileName           = 'load_required_'.rand(10,1000);
        $image_array[]      = FileUpload::PDFupload($path, ['data' => $file_temp], $fileName);
    }
    return $image_array;
}
*/

function UploadImagePdf($files, $path){
    $fileArray                              = [];
    $imageTemps                             = [];
    foreach($files as $file){
        $fileExtension                      = $file->extension();
        if($fileExtension == 'png' || $fileExtension == 'jpeg' || $fileExtension == 'jpg' || $fileExtension == 'gif'){
            // Image
            $fileName                       = FileUpload::upload('companyAdmin/temp', $file);
            if(isset($fileName) && $fileName != ''){
                $filePathWithName           = 'storage/companyAdmin/temp/'.$fileName;
                if (File::exists($filePathWithName)) {
                    $publicPath             = public_path($filePathWithName);
                    // Use @file_get_contents to suppress warnings/errors if the file doesn't exist
                    $fileContents           = @file_get_contents($publicPath);
                    // Check if $fileContents is not false (indicating successful read)
                    if ($fileContents !== false) {
                        // Process or display the file contents as needed
                        $base64             = 'data:image/' . $fileExtension . ';base64,' . base64_encode($fileContents);
                        $imageTemps[]       = [
                            'base64'        => $base64,
                            'path'          => $filePathWithName
                        ];
                    }
                }
            }
        }else{
            // Other Document
            $fileArray[]                    = FileUpload::upload($path, $file);
        }
    }
    if(!empty($imageTemps)){
        $imageBase64                        = array_column($imageTemps, 'base64');
        if(!empty($imageBase64)){
            $name                           = Str::random(8) .'_'. substr(Str::uuid()->toString(), 0, 8); // 'load_required_'.rand(10,1000);
            $fileName                       = FileUpload::PDFupload($path, ['data' => $imageBase64], $name);
            if(isset($fileName) && $fileName != ''){
                $fileArray[]                = $fileName;
                $imagePathWithNames         = array_column($imageTemps, 'path');
                if(!empty($imagePathWithNames)){
                    foreach ($imagePathWithNames as $key => $imagePathWithName) {
                        if (File::exists($imagePathWithName)) {
                            File::delete($imagePathWithName);
                        }
                    }
                }
            }
        }
    }

    if(!empty($fileArray)){
        // File array include file names
        $pdfFileNames                       = array_filter($fileArray, function ($pdfFileName) use ($path) {
            if(Str::startsWith($path, 'storage/')){
                $pdfPath                    = $path .'/'. $pdfFileName;
            }else{
                $pdfPath                    = 'storage/'.$path .'/'. $pdfFileName;
            }
            return pathinfo($pdfFileName, PATHINFO_EXTENSION) === 'pdf' && File::exists($pdfPath);
        });
        if(!empty($pdfFileNames) && count($pdfFileNames) > 1){
            // get only pdf file names
            $pdfFiles                       = [];
            foreach ($pdfFileNames as $key => $pdfFileName) {
                $pdfFiles[]                 = [
                    'name'                  => $pdfFileName,
                    'path'                  => Str::startsWith($path, 'storage/') ? $path .'/'. $pdfFileName : 'storage/'.$path .'/'. $pdfFileName
                ];
            }
            if(!empty($pdfFiles)){
                // get pdf path with name

                // Initialize the PDFMerger
                $pdfMerger                  = PDFMerger::init();

                // Add PDFs to be merged
                foreach ($pdfFiles as $key => $pdfFile) {
                    $pdfMerger->addPDF(public_path($pdfFile['path']));
                }

                // You can also add PDFs from a base64 encoded string
                // $pdfMerger->addBase64('base64_encoded_string');

                // Merge the PDFs
                $pdfMerger->merge();

                $mergedPdfPath              = 'storage/'.$path;
                if(!File::isDirectory($mergedPdfPath)){
                    File::makeDirectory($mergedPdfPath, 0777, true, true);
                }
                $mergedPdfName              = time().'_'.Str::random(8).'_merged.pdf'; // time().'_'.Str::random(8) .'_'. substr(Str::uuid()->toString(), 0, 8).'_merged.pdf'; // time().'_load_required_'.rand(10,1000);

                 // Save or output the merged PDF
                $mergedPdfPathWithName      = $mergedPdfPath.'/'.$mergedPdfName;
                $pdfMerger->save(public_path($mergedPdfPathWithName));

                if(File::exists($mergedPdfPathWithName)){
                    foreach ($pdfFiles as $key => $pdfFile) {
                        while (($searchkey = array_search($pdfFile['name'], $fileArray)) !== false) {
                            unset($fileArray[$searchkey]);
                            if (File::exists($pdfFile['path'])) {
                                File::delete($pdfFile['path']);
                            }
                        }
                    }
                    $fileArray[]            = $mergedPdfName;
                }
            }
        }
    }
    return array_values($fileArray);
}

function getOccupancyBasedRevenue2($GrossRevenue, $LoadOccupancy, $targetRevenue)
{

    $LoadOccupancy = $LoadOccupancy / 100;
    if ($GrossRevenue == 0 || $LoadOccupancy == 0) {
        $revenue = 0.00;
    } else {
        if (($LoadOccupancy * $targetRevenue) * 100 == 0) {
            $revenue = 0.00;
        } else {
            $setp = $LoadOccupancy * $targetRevenue;
            $revenue = $GrossRevenue / $setp;
            $revenue = $revenue * 100;
        }
    }

    return $revenue;
}



function getReportDrivers($driver_id){
    // $getload                                     = LoadDetail::where('driver_id', 10)->where('is_disable', 0)->sum('driver_revenue');
    // dd($getload);
    // $getDriver                                   = ReportDriver::where('driver_id', 10)->where('date', '>=' ,'2021-01-01')->where('date', '<=' ,'2023-12-31')->sum('actual_revenue');
    // dd($getDriver);

    $checkflag                                      = 0;
    ReportDriver::where('driver_id', $driver_id)->delete();

    $getoldestLoad                                  = LoadDetail::select('created_at')->whereNull('deleted_at')->where('driver_id', $driver_id)->oldest()->first();
    $getlatestLoad                                  = LoadDetail::select('created_at')->whereNull('deleted_at')->where('driver_id', $driver_id)->latest()->first();
    if(isset($getoldestLoad->created_at) && !empty($getoldestLoad->created_at)){
        $start_date                                 = \Carbon\Carbon::createFromFormat('Y-m-d H:s:i', $getoldestLoad->created_at)->format('Y-m-d');
        $end_date                                   = \Carbon\Carbon::createFromFormat('Y-m-d H:s:i', $getlatestLoad->created_at)->format('Y-m-d');
        $checkflag                                  = 1;
    }else{
        $start_date                                 = Carbon::now()->format('Y-m-d');
        $end_date                                   = Carbon::now()->format('Y-m-d');
    }

    $oldestdata                                     = DriverFinancial::select('start_date')->where("driver_id", $driver_id)->orderBy("start_date", 'asc')->first();
    $latestload                                     = DriverFinancial::select('end_date')->where("driver_id", $driver_id)->orderBy("end_date", 'desc')->first();

    if(isset($oldestdata->start_date) && !empty($oldestdata->start_date)){
        $start_date_finance                         = \Carbon\Carbon::createFromFormat('Y-m-d', $oldestdata->start_date)->format('Y-m-d');
        $end_date_finance                           = \Carbon\Carbon::createFromFormat('Y-m-d', $latestload->end_date)->format('Y-m-d');

        $checkflag                                  = 1;
    }else{
        $start_date_finance                         = Carbon::now()->format('Y-m-d');
        $end_date_finance                           = Carbon::now()->format('Y-m-d');
    }

    if($checkflag == 1){
        if($start_date >= $start_date_finance){
            $startDateLoad                          = $start_date_finance;
        }else{
            $startDateLoad                          = $start_date;
        }

        if($end_date <= $end_date_finance){
            $endDateLoad                            = $end_date_finance;
        }else{
            $endDateLoad                            = $end_date;
        }

        // dd($end_date_finance, $end_date ,$endDateLoad);
        // dd($start_date, $startDateLoad, $end_date, $endDateLoad, $driver_id);

        $period                                     = CarbonPeriod::create($startDateLoad, $endDateLoad);
        $dateae                                     = 0;
        $getDriver                                  = Driver::find($driver_id);
        $getDriver->update(['is_report_drivers' => 1]);
        foreach ($period as $date) {
            $date                                   = $date->format('Y-m-d');
            $getData                                = getLoadOccupancyDaysCount($date, $date, $driver_id);
            $actual_revenue                         = getLoadGrossRevenue($date, $date, $driver_id);
            // $actual_mileage                      = getLoadMileage($date, $date, $driver_id);
            $actual_mileage                         = getLoadLoadedMileage($date, $date, $driver_id); // loaded_miles
            $actual_total_mileage                   = getLoadTotalMileage($date, $date, $driver_id); // total_miles
            $getTarget                              = getTargetRevenueMilesByDate($date, $date, $driver_id);

            $load_trailer_type                      = LoadDetail::where('load_details.driver_id', $driver_id)
                ->join("load_drivers as ld", "ld.load_detail_id", "=", "load_details.id")
                ->where(function ($query) use ($date) {
                    $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $date)
                        ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $date)
                        ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $date)
                        ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $date);
                })
                ->where('load_details.load_status', '!=', 9)
                ->where(function($q) {
                    $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
                })
                ->whereNull('load_details.deleted_at')
                ->select('load_details.trailer_type')
                ->first();

            $report_data                            = [];
            $report_data['company_id']              = $getDriver->company_id;
            $report_data['team_id']                 = isset($getDriver->team_id) ? $getDriver->team_id : 0;
            $report_data['manager_id']              = isset($getDriver->manager_id) ? $getDriver->manager_id : 0;
            $report_data['trailer_type']            = isset($load_trailer_type->trailer_type) ? $load_trailer_type->trailer_type : null;
            $report_data['driver_id']               = $driver_id;
            $report_data['date']                    = $date;
            $report_data['target_cpm']              = $getTarget['cpm'];
            $report_data['target_occupancy']        = $getTarget['occupancy'];
            $report_data['target_revenue']          = $getTarget['revenue'];
            $report_data['target_miles']            = $getTarget['miles'];
            $report_data['actual_revenue']          = $actual_revenue;
            $report_data['actual_mileage']          = $actual_mileage;
            $report_data['actual_total_mileage']    = $actual_total_mileage;
            $report_data['actual_daycount']         = $getData['daycount'];
            $report_data['actual_totaldays']        = $getData['totalDays'];
            ReportDriver::create($report_data);
        }
    }
}

function isReportDriversZero($driverId)
{
    $getDriver                                      = Driver::find($driverId);
    $getDriver->update(['is_report_drivers' => 0]);
}

function getLoadOccupancyDaysCount($startDate, $endDate, $driverId)
{
    $loadDetails                                    = LoadDetail::where('load_details.driver_id', $driverId)
        ->join("load_drivers as ld", "ld.load_detail_id", "=", "load_details.id")
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
            ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
            ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
            ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNull('load_details.deleted_at')
        //->select('load_details.id as id','load_details.driver_id','load_details.actual_shipper_date','load_details.actual_delivery_date','load_details.load_status','load_details.load_type','load_details.occupied_pickup_day','load_details.driver_revenue','ld.miles')
        ->select('load_details.load_id as id','load_details.driver_id','load_details.actual_shipper_date','load_details.actual_delivery_date','load_details.load_status','load_details.load_type','load_details.occupied_pickup_day','load_details.driver_revenue','ld.miles')
        ->get();

    $period                                         = CarbonPeriod::create($startDate, $endDate);
    $totalDays                                      = 0;
    $bcoutn                                         = 0;
    $daycount                                       = 0;
    $uc                                             = 0;
    $oc                                             = 0;
    /*
    foreach($loadDetails as $load){
        if(in_array($load->load_type, [0, 4, 6]) && $load->occupied_pickup_day == 1){
            $totalDays++;
        }
    }
    */

    foreach ($period as $date) {
        $date                                       = $date->format('Y-m-d');
        $laod_array                                 = [];
        foreach($loadDetails as $load){
            $actual_shipper_date                    = Carbon::createFromFormat(getTimeLayout(),$load->actual_shipper_date)->format('Y-m-d');
            $actual_delivery_date                   = Carbon::createFromFormat(getTimeLayout(),$load->actual_delivery_date)->format('Y-m-d');
            if(in_array($load->load_type, [0, 4, 6]) && $load->occupied_pickup_day == 0 && $actual_shipper_date < $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
                $laod_array[$load->id]              = $load;
            }
            if(in_array($load->load_type, [0, 4, 6]) && $load->occupied_pickup_day == 1 && $actual_shipper_date <= $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
                $laod_array[$load->id]              = $load;
            }
            if(in_array($load->load_type, [1, 2, 3, 5]) && $actual_shipper_date <= $date && $actual_delivery_date >= $date || $actual_delivery_date == $date){
                // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
                $laod_array[$load->id]              = $load;
            }
        }

        $getload                                    = [];
        $ki                                         = 0;
        foreach($laod_array as $key => $larray){
            $getload[$ki]                           = $larray;
            $ki++;
        }

        if (count($getload) > 0) {
            $load_type                              = array(0, 4, 6); // 0 = OCCUPIED, 4 = NO DISPATCH, 6 = TONU
            foreach ($getload as $load_details) {
                if (in_array($load_details['load_type'], $load_type)) {
                    $totalDays++;
                    break;
                }
            }
            $daycount++;
        }

        if (count($getload) > 1) {
            $load_type                              = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
            $uc                                     = 0;
            $oc                                     = 0;
            foreach ($getload as $load_details) {
                if (in_array($load_details['load_type'], $load_type)) {
                    $uc++;
                } else {
                    $oc++;
                }
            }
            if ($oc > 0 && $uc > 0) {
                $bcoutn++;
            }
            $daycount++;
        }
    }

    // dd($totalDays, $uc, $oc, $bcoutn, $daycount);
    $totalDays                                      = $totalDays - $bcoutn;
    return ['daycount' => $daycount, 'totalDays' => $totalDays];
}

function createLoadLog($chagnes, $data, $type=null){
    // return true;
    if (request()->is('company-admin') || request()->is('company-admin/*')) {
        $user                                   = auth()->guard('company-admin')->user();
    }else if (request()->is('driver-admin') || request()->is('driver-admin/*')) {
        $user                                   = auth()->guard('driver-admin')->user();
    }
    // $user                                    = auth()->guard('company-admin')->user();
    $text                                       = [];
    if($type == 'create'){
        $text[]                                 = 'Status: '.getLoadStatus($data->load_status).' logs:';
        $text[]                                 = 'Load created at: '.convertDateFormate(Carbon::now());
        $text[]                                 = 'Load Created By: '.$user->first_name.' '.$user->last_name;

    }else if($type == 'edit'){
        if(array_key_exists("load_status",$chagnes)){
            if($data->load_status == '2'){
                LoadLog::where('load_id', $data->load_id)->whereIn('load_status',[2,11,3,4,5,6,10,8])->delete();
                $text[]                         = 'Status: '.getLoadStatus($data->load_status).' logs:';
                $text[]                         = 'Load updated at: '.convertDateFormate(Carbon::now());
                $text[]                         = 'Load Created By: '.$user->first_name.' '.$user->last_name;
            }
            if($data->load_status == '11'){
                LoadLog::where('load_id', $data->load_id)->whereIn('load_status',[3,4,5,6,10,8])->delete();
                $text[]                         = 'Status: '.getLoadStatus($data->load_status).' logs:';
                $text[]                         = 'Sent to Driver: Yes';
                $text[]                         = 'Sent to Driver at: '.convertDateFormate(Carbon::now());
            }
            if($data->load_status == '3'){
                LoadLog::where('load_id', $data->load_id)->whereIn('load_status',[3,4,5,6,10,8])->delete();
                $text[]                         = 'Status: '.getLoadStatus($data->load_status).' logs:';
                $text[]                         = 'Driver accepted: Yes';
                $text[]                         = 'Driver accepted load at: '.convertDateFormate(Carbon::now());
                $text[]                         = 'User that accepted the load: Yes';
            }
            if($data->load_status == '4'){
                LoadLog::where('load_id', $data->load_id)->whereIn('load_status',[4,5,6,10,8])->delete();
                $addresses                      = [];
                if(isset($data->shipper_address_one) && $data->shipper_address_one!==''){
                    $addresses[]                = $data->shipper_address_one;
                }
                if(isset($data->shipper_address_two) && $data->shipper_address_two!==''){
                    $addresses[]                = $data->shipper_address_two;
                }
                if(isset($data->shipper_city) && $data->shipper_city!==''){
                    $addresses[]                = $data->shipper_city;
                }
                if(isset($data->shipper_state) && $data->shipper_state!==''){
                    $addresses[]                = $data->shipper_state;
                }
                if(isset($data->shipper_zip) && $data->shipper_zip!==''){
                    $addresses[]                = $data->shipper_zip;
                }
                $text[]                         = 'Status IN TRANSIT and DELIVERED logs:';
                $text[]                         = 'PICKUP:';
                $text[]                         = 'Location: '.implode(', ', $addresses);
                $text[]                         = 'Location Geo Coordinates: '.$data->shipper_latitude.','.$data->shipper_longitude;
                $text[]                         = 'Location Time Zone: '.$data->shipper_timezone;
                $text[]                         = 'Stop miles from load start:'.$data->loaded_miles;
                $text[]                         = 'Stop miles from last stop:'.$data->loaded_miles;
                //$text[]                       = 'Stop % of entire load:';
                if($data->shipper_is_arrived == 1){
                    $text[]                     = 'Driver arrived at location: Yes';
                    $text[]                     = 'Driver arrived at: '.convertDateFormate($data->shipper_arrived_at);
                }
                //$text[]                       = 'User that set at stop Arrived status:';
                /*
                if($data->status == 1){
                    $text[]                     = 'Driver completed stop at: '.convertDateFormate($data->status_at);
                    $text[]                     = 'User that set completed status: Yes';
                }
                if($data->status == 2){
                    $text[]                     = 'Driver left stop: Yes';
                    $text[]                     = 'Driver completed stop at: '.convertDateFormate($data->status_at);
                }
                */
                //$text[]                       = 'Detention at stop: Yes';
                $text[]                         = '<br/>';
            }
            if($data->load_status == '5'){
                LoadLog::where('load_id', $data->load_id)->whereIn('load_status',[5,6,10,8])->delete();
                $getStop                        = LoadStop::where('load_id', $data->load_id)->orderBy('stop_no', 'asc')->get();
                if(!empty($getStop) && count($getStop) > 0){
                    foreach($getStop as $key => $stop){
                        $stopAddress            = [];
                        if(isset($stop->address_one) && $stop->address_one!==''){
                            $stopAddress[]      = $stop->address_one;
                        }
                        if(isset($stop->address_two) && $stop->address_two!==''){
                            $stopAddress[]      = $stop->address_two;
                        }
                        if(isset($stop->city) && $stop->city!==''){
                            $stopAddress[]      = $stop->city;
                        }
                        if(isset($stop->state) && $stop->state!==''){
                            $stopAddress[]      = $stop->state;
                        }
                        if(isset($stop->zip) && $stop->zip!==''){
                            $stopAddress[]      = $stop->zip;
                        }
                        $nume                   = $key+1;
                        $text[]                 = 'Stop :'.$nume;
                        $text[]                 = 'Location: '.implode(', ', $stopAddress);
                        $text[]                 = 'Location Geo Coordinates: '.$stop->latitude.','.$stop->longitude;
                        $text[]                 = 'Location Time Zone: '.$stop->timezone;
                        $text[]                 = 'Stop miles from load start:'.$data->loaded_miles;
                        $text[]                 = 'Stop miles from last stop:'.$data->loaded_miles;
                        //$text[]               = 'Stop % of entire load:';
                        if($stop->is_arrived == 1){
                            $text[]             = 'Driver arrived at location: Yes';
                            $text[]             = 'Driver arrived at: '.convertDateFormate($stop->arrived_at);
                        }
                        //$text[]               = 'User that set at stop Arrived status:';
                        if($stop->status == 1){
                            $text[]             = 'Driver completed stop at: '.convertDateFormate($stop->status_at);
                            $text[]             = 'User that set completed status: Yes';
                        }
                        if($stop->status == 2){
                            $text[]             = 'Driver left stop: Yes';
                            $text[]             = 'Driver completed stop at: '.convertDateFormate($stop->status_at);
                        }
                        //$text[]               = 'Detention at stop: Yes';
                        $text[]                 = '<br/>';
                    }
                }

                $addresses2                     = [];
                if(isset($data->delivery_address_one) && $data->delivery_address_one!==''){
                    $addresses2[]               = $data->delivery_address_one;
                }
                if(isset($data->delivery_address_two) && $data->delivery_address_two!==''){
                    $addresses2[]               = $data->delivery_address_two;
                }
                if(isset($data->delivery_city) && $data->delivery_city!==''){
                    $addresses2[]               = $data->delivery_city;
                }
                if(isset($data->delivery_state) && $data->delivery_state!==''){
                    $addresses2[]               = $data->delivery_state;
                }
                if(isset($data->delivery_zip) && $data->delivery_zip!==''){
                    $addresses2[]               = $data->delivery_zip;
                }

                $text[]                         = 'DELIVERY:';
                $text[]                         = 'Location: '.implode(', ', $addresses2);
                $text[]                         = 'Location Geo Coordinates: '.$data->delivery_latitude.','.$data->delivery_longitude;
                $text[]                         = 'Location Time Zone: '.$data->delivery_timezone;
                $text[]                         = 'Stop miles from load start:'.$data->loaded_miles;
                $text[]                         = 'Stop miles from last stop:'.$data->loaded_miles;
                if($data->delivery_is_arrived == 1){
                    $text[]                     = 'Driver arrived at location: Yes';
                    $text[]                     = 'Driver arrived at: '.convertDateFormate($data->delivery_arrived_at);
                }
                /*
                $text[]                         = 'Stop % of entire load:';
                $text[]                         = 'Driver arrived at location: Yes';
                $text[]                         = 'Driver arrived at: Date and Time';
                $text[]                         = 'User that set at stop Arrived status:';
                $text[]                         = 'Driver left stop: Yes';
                $text[]                         = 'Driver completed stop at: '.convertDateFormate(Carbon::now());
                $text[]                         = 'User that set completed status:';
                $text[]                         = 'Detention at stop: Yes';
                */
            }
            if($data->load_status == '6'){
                LoadLog::where('load_id', $data->load_id)->whereIn('load_status',[10,8])->delete();
                $text[]                         = 'Status READY TO BILL logs';
                $text[]                         = 'Load went to ready to bill status at: '.convertDateFormate(Carbon::now());
                $text[]                         = 'Load was placed in ready to bill status by: '.$user->first_name.' '.$user->last_name;
            }
            if($data->load_status == '8'){
                LoadLog::where('load_id', $data->load_id)->whereIn('load_status',[8])->delete();
                $text[]                         = 'Status PAID logs';
                $text[]                         = 'Load went to completed status at: '.convertDateFormate(Carbon::now());
                $text[]                         = 'Load was placed in completed status by: '.$user->first_name.' '.$user->last_name;
            }
        }

        if(array_key_exists("is_on_hold",$chagnes)){
            LoadLog::where('load_id', $data->load_id)->where('description', 'like', '%ON HOLD%')->orWhere('description', 'like', '%CANCELED%')->delete();
            if($data->is_on_hold == '1'){
                $text[]                         = 'Status ON HOLD logs';
                $text[]                         = 'Load went to on hold status: Yes';
                $text[]                         = ' User that set load to on hold status:';
                $text[]                         = 'Status prior to on hold:';
                $text[]                         = 'Load went on hold at:';
                $text[]                         = 'Load was removed from on hold at:';
                $text[]                         = ' Load was remove from on hold by:';
            }else if($data->is_on_hold == '2'){
                $text[]                         = 'Status CANCELED logs';
                $text[]                         = 'Load Canceled at: '.convertDateFormate(Carbon::now());
                $text[]                         = 'Load Canceled by: '.$user->first_name.' '.$user->last_name;
            }else{
                //
            }
        }

        if(array_key_exists("drop_trailer",$chagnes)){
            $address = [];
            if($data->add1 != ''){ $address[] = $data->add1; }
            if($data->add2 != ''){ $address[] = $data->add2; }
            if($data->city != ''){ $address[] = $data->city; }
            if($data->state != ''){ $address[] = $data->state; }
            if($data->zip != ''){ $address[] = $data->zip; }

            if($data->type == '1'){
                $text[] = 'Drop Trailer logs';
                $text[] = 'Dropped Stop Number: '.$data->stop_number;
                $text[] = 'Dropped Trailer Number: '.$data->trailer_number;
                $text[] = 'Dropped Status: '.(($data->empty_loaded == '1') ? 'Empty' : 'Loaded');
                $text[] = 'Dropped Location: '.((count($address) > 0) ? implode(', ', $address) : '');
                $text[] = 'Dropped at: '.convertDateFormate(Carbon::parse($data->drop_date));
                $text[] = 'Dropped By: '.$user->first_name.' '.$user->last_name;
            }else if($data->type == '2'){
                $text[] = 'Hook Trailer logs';
                $text[] = 'Hooked Stop Number: '.$data->stop_number;
                $text[] = 'Hooked Trailer Number: '.$data->trailer_number;
                $text[] = 'Hooked Status: '.(($data->empty_loaded == '1') ? 'Empty' : 'Loaded');
                $text[] = 'Hooked Location: '.((count($address) > 0) ? implode(', ', $address) : '');
                $text[] = 'Hooked at: '.convertDateFormate(Carbon::parse($data->drop_date));
                $text[] = 'Hooked By: '.$user->first_name.' '.$user->last_name;
            }
        }
    }else{
        //
    }
    if(!empty($text)){
        $input                                  = [];
        $input['company_id']                    = $user->company_id;
        $input['load_id']                       = $data->load_id;
        $input['load_stop_id']                  = (array_key_exists("drop_trailer",$chagnes)) ? $data->id  : null;
        $input['load_status']                   = $data->load_status;
        $input['load_type']                     = $data->load_type;
        $input['description']                   = serialize($text);
        LoadLog::create($input);
    }
}

function convertDateFormate($value){
    return Carbon::parse($value)->format(getTimeLayout());
}

function getDriverFullName($driver_id){
    $getDriver = Driver::findOrFail($driver_id);
    return $getDriver->full_name;
}

function is_serialized( $data, $strict = true ) {
    // If it isn't a string, it isn't serialized.
    if ( ! is_string( $data ) ) {
        return false;
    }
    $data = trim( $data );
    if ( 'N;' === $data ) {
        return true;
    }
    if ( strlen( $data ) < 4 ) {
        return false;
    }
    if ( ':' !== $data[1] ) {
        return false;
    }
    if ( $strict ) {
        $lastc = substr( $data, -1 );
        if ( ';' !== $lastc && '}' !== $lastc ) {
            return false;
        }
    } else {
        $semicolon = strpos( $data, ';' );
        $brace     = strpos( $data, '}' );
        // Either ; or } must exist.
        if ( false === $semicolon && false === $brace ) {
            return false;
        }
        // But neither must be in the first X characters.
        if ( false !== $semicolon && $semicolon < 3 ) {
            return false;
        }
        if ( false !== $brace && $brace < 4 ) {
            return false;
        }
    }
    $token = $data[0];
    switch ( $token ) {
        case 's':
            if ( $strict ) {
                if ( '"' !== substr( $data, -2, 1 ) ) {
                    return false;
                }
            } elseif ( false === strpos( $data, '"' ) ) {
                return false;
            }
            // Or else fall through.
        case 'a':
        case 'O':
            return (bool) preg_match( "/^{$token}:[0-9]+:/s", $data );
        case 'b':
        case 'i':
        case 'd':
            $end = $strict ? '$' : '';
            return (bool) preg_match( "/^{$token}:[0-9.E+-]+;$end/", $data );
    }
    return false;
}

function getUserByType($user_id, $type){
    if($type == 3){
        $getName = Driver::findOrFail($user_id)->full_name;
    }else{
        $getName = CompanyAdmin::findOrFail($user_id)->full_name;
    }
    return $getName;
}

function getCompanyAdminCommodityList($companyId = null)
{
    if(empty($companyId)){
        $companyId = auth()->guard('company-admin')->user()->company_id;
    }
    return CompanyCommodity::select('name', 'id')
        ->where('company_id', $companyId)
        ->pluck('name', 'id')
        ->all();
}

function getHazardousOptions()
{
    return HazardousOption::select('name', 'id')->pluck('name', 'id')->all();
}

function getCompanyTerms()
{
    return CompanyTerm::IsCompany()->select('name', 'id')->pluck('name', 'id')->all();
}

function getCompanyLoadRequirementsItemList()
{
    return CompanyLoadRequirementsItem::IsCompany()->select('name')->pluck('name', 'name')->all();
}

function getChartOfAccountsTypes($type, $useFor = '')
{
    $accountTypesArr = array(
        0 => 'Bank',
        1 => 'Income',
        2 => 'Expense',
        3 => 'Equity',
        4 => 'Current Asset',
        5 => 'Current Liability',
        6 => 'Long Term Liability',
        7 => 'Cost Of Sales',
        8 => 'Accounts Receivable',
        9 => 'Accounts Payable',
        10 => 'Other Income',
        11 => 'Other Expense'
    );

    if($useFor === 'listing'){
        return  $accountTypesArr;
    }else{
        return (isset($accountTypesArr[$type])) ? $accountTypesArr[$type] : null;
    }
}

function csvToArray($filePath = '', $delimiter = ',')
{
    if (!file_exists($filePath) || !is_readable($filePath))
        return false;

    $header = null; $data = array();
    if (($handle = fopen($filePath, 'r')) !== false) {
        while (($row = fgetcsv($handle, 1000, $delimiter)) !== false) {
            if (!$header) {
                $header = $row;
            } else {
                $data[] = array_combine($header, $row);
            }
        }
        fclose($handle);
    }

    return $data;
}

function xlsxToArray($filePath = '')
{
    if (!file_exists($filePath) || !is_readable($filePath))
        return false;

    $data = array();

    if ($xlsx = SimpleXLSX::parse($filePath)) {
        $rows = $xlsx->rows();

        if(count($rows) > 0){
            $headers = $rows[0];
            $withoutHeaderRows = array_slice($rows, 1);

            foreach ($withoutHeaderRows as $key => $value){
                foreach ($headers as $k => $v){
                    $data[$key][$v] = $value[$k];
                }
            }
        }
    }

    return $data;
}

function dispatchHubStatus()
{
    return ['0' => 'Pending', '2' => 'Preplaned', '3' => 'Confirmed', '4' => 'In Transit', '7' => 'On Hold'];
}

function getTrimbleApiKey(){
    $trimble_api_key        = null;
    if ((request()->is('company-admin') || request()->is('company-admin/*')) && auth()->guard('company-admin')->check()) {
        $trimble_api_key    = auth()->guard('company-admin')->user()->getCompany->trimble_api_key;
    }else if ((request()->is('driver-admin') || request()->is('driver-admin/*')) && auth()->guard('driver-admin')->check()) {
        $trimble_api_key    = auth()->guard('driver-admin')->user()->getCompany->trimble_api_key;
    }else if ((request()->is('admin') || request()->is('admin/*')) && auth()->guard('admin')->check()) {
        $trimble_api_key    = env('TRIMBLE_MAPS_API_KEY');
    }
    return $trimble_api_key;
}

function getStatusCircleIcon($status)
{
    if(in_array($status, ["Pending", "Preplaned"])){
        return asset('theme/images/yellow-circle-icon.svg');
    }else if(in_array($status, ["Confirmed", "In Transit"])){
        return asset('theme/images/green-circle-icon.svg');
    }else if($status == "On Hold"){
        return asset('theme/images/red-circle-icon.svg');
    }
}

function updateStatusDropdown($loadId, $loadStatus, $table)
{
    $html = '<select class="form-control updateStatus">';
    foreach (dispatchHubStatus() as $key => $value){
        $html .= '<option value="'.$key.'" data-class="avatar" data-style="background-image: url('.getStatusCircleIcon($value).');" data-load="'.$loadId.'" data-tabel="'.$table.'" '.(($loadStatus == $key) ? "selected" : "" ).'>'.$value.'</option>';
    }
    $html .= '</select>';

    return $html;
}

function deadheadLocation($driver, $formData=[]){
    // 1 = previous_load, 2 = app samsara
    $type                           = isset($formData['type']) ? $formData['type'] : null;

    $lists                          = [];
    if(is_null($type) || $type==2){
        $truck                      = $driver->driver_information->truck ?? null;
        if(!is_null($truck)){
            $location               = truckCurrentLocation($truck);
            if(!empty($location)){
                try {
                    $trimble_api_key                = getTrimbleApiKey();
                    if(!is_null($trimble_api_key)){
                        $array                      = [
                            'Coords'                => $location['longitude'].','.$location['latitude'],
                            'matchNamedRoadsOnly'   => 'true',
                            'maxCleanupMiles'       => '20',
                            'region'                => 'NA',
                            'dataset'               => 'Current',
                            'vehicle'               => 'Truck',
                            'timestamp'             => Carbon::now(),
                            'includeTrimblePlaceIds'=> 'true'
                        ];

                        $param                      = array();
                        foreach($array as $k => $v){
                            $param[]                = $k . '=' . urlencode($v);
                        }
                        $param                      = implode('&', $param);

                        $url                        = 'https://pcmiler.alk.com/apis/rest/v1.0/Service.svc/locations/reverse?'.$param;
                        $client                     = new \GuzzleHttp\Client();
                        $api_response               = $client->request('GET', $url, [
                            'headers'               => [
                                'Accept'            => 'application/json',
                                'Content-Type'      => 'application/json',
                                'Authorization'     => $trimble_api_key
                            ],
                        ]);

                        // Check if response status code is OK (200)
                        if ($api_response->getStatusCode() == 200) {
                            $body                   = json_decode($api_response->getBody());
                            if(isset($body)){
                                $lists['app']       = [
                                    'StreetAddress' => $body->Address->StreetAddress,
                                    'City'          => $body->Address->City,
                                    'State'         => $body->Address->State,
                                    'Zip'           => $body->Address->Zip,
                                    'TimeZone'      => $body->TimeZone,
                                    'Lat'           => $body->Coords->Lat,
                                    'Lon'           => $body->Coords->Lon,
                                    'time'          => Carbon::parse($location['time'])->format('Y-m-d H:i:s'),
                                ];
                            }
                        }
                    }
                } catch (\GuzzleHttp\Exception\ClientException $e) {
                    // Handle client-side errors (e.g. 4xx status codes)
                    $api_response                   = $e->getResponse();
                    if ($api_response) {
                        $statusCode                 = $api_response->getStatusCode();
                        $body                       = $api_response->getBody()->getContents();
                        Log::error($body);
                    } else {
                        Log::error($e->getMessage());
                    }
                } catch (\GuzzleHttp\Exception\ServerException $e) {
                    // Handle server-side errors (e.g. 5xx status codes)
                    $api_response                   = $e->getResponse();
                    if ($api_response) {
                        $statusCode                 = $api_response->getStatusCode();
                        $body                       = $api_response->getBody()->getContents();
                        Log::error($body);
                    } else {
                        Log::error($e->getMessage());
                    }
                } catch (\Exception $e) {
                    // Handle other exceptions
                    Log::error($e->getMessage());
                }
            }
        }
    }

    if(is_null($type) || $type==1){
        $records                    = LoadDetail::select(
                //cuurent
                'load_details.id',
                'load_details.load_number',
                'load_details.load_id',

                'load_details.driver_id',
                'load_details.booked_by',
                'load_details.truck_number',
                'load_details.trailer_number',
                'load_details.trailer_type',

                'load_details.load_type',
                'load_details.load_status',
                'load_details.is_on_hold',
                'load_details.is_disable',

                'load_details.delivery_address_one',
                'load_details.delivery_address_two',
                'load_details.delivery_city',
                'load_details.delivery_state',
                'load_details.delivery_zip',

                'load_details.delivery_latitude',
                'load_details.delivery_longitude',
                'load_details.delivery_timezone',
                'load_details.delivery_timezone_offset',
                'load_details.delivery_timezone_abbr',
                'load_details.delivery_address_verified',

                'load_details.actual_delivery_date',

                //previous
                'previous.id as previous_id',
                'previous.load_number as previous_load_number',
                'previous.load_id as previous_load_id',

                'previous.driver_id as previous_driver_id',
                'previous.booked_by as previous_booked_by',
                'previous.truck_number as previous_truck_number',
                'previous.trailer_number as previous_trailer_number',
                'previous.trailer_type as previous_trailer_type',

                'previous.load_type as previous_load_type',
                'previous.load_status as previous_load_status',
                'previous.is_on_hold as previous_is_on_hold',
                'previous.is_disable as previous_is_disable',

                'previous.delivery_address_one as previous_delivery_address_one',
                'previous.delivery_address_two as previous_delivery_address_two',
                'previous.delivery_city as previous_delivery_city',
                'previous.delivery_state as previous_delivery_state',
                'previous.delivery_zip as previous_delivery_zip',

                'previous.delivery_latitude as previous_delivery_latitude',
                'previous.delivery_longitude as previous_delivery_longitude',
                'previous.delivery_timezone as previous_delivery_timezone',
                'previous.delivery_timezone_offset as previous_delivery_timezone_offset',
                'previous.delivery_timezone_abbr as previous_delivery_timezone_abbr',
                'previous.delivery_address_verified as previous_delivery_address_verified',

                'previous.actual_delivery_date as previous_actual_delivery_date'
            )
            ->leftJoin('load_details as previous', function ($join) use ($driver) {
                $join->on('previous.actual_delivery_date', '=', DB::raw('(SELECT MAX(r2.actual_delivery_date) FROM load_details as r2 WHERE r2.actual_delivery_date < load_details.actual_delivery_date AND(r2.driver_id = "'.$driver->id.'" AND r2.load_type IN(0, 6) AND(r2.is_on_hold IS NULL OR r2.is_on_hold IN(0, 1)) AND r2.is_disable = 0 AND r2.deleted_at IS NULL))'))
                    ->where(function($q) use ($driver) {
                        $q->where('previous.driver_id', $driver->id)
                            ->whereIn('previous.load_type', [0, 6]) // 0 = OCCUPIED, 6 = TONU
                            ->where(function($sub) {
                                $sub->whereNull('previous.is_on_hold')->orWhereIn('previous.is_on_hold', [0, 1]);
                            })
                            ->where('previous.is_disable', 0)
                            ->whereNull('previous.deleted_at');
                    });
            })->where(function($q) use ($driver, $formData) {
                if((isset($formData['shipper_date']) && $formData['shipper_date']!='')){
                    $q->whereRaw("DATE_FORMAT(load_details.delivery_date_time, '%Y-%m-%d %H:%i') <= ?", [$formData['shipper_date']]);
                }
                $q->where('load_details.driver_id', $driver->id)
                    ->whereIn('load_details.load_type', [0, 6]) // 0 = OCCUPIED, 6 = TONU
                    ->where(function($sub) {
                        $sub->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0, 1]);
                    })
                    ->where('load_details.is_disable', 0);
            })
            ->orderBy('load_details.actual_delivery_date', 'desc')
            ->limit(1)
            ->first();
        if(!is_null($records)){
            if(isset($records->delivery_latitude)){
                $lists['previous_load']         = [
                    'StreetAddress'             => $records->delivery_address_one,
                    'City'                      => $records->delivery_city,
                    'State'                     => $records->delivery_state,
                    'Zip'                       => $records->delivery_zip,
                    'TimeZone'                  => $records->delivery_timezone_abbr,
                    'Lat'                       => $records->delivery_latitude,
                    'Lon'                       => $records->delivery_longitude,
                    'time'                      => $records->actual_delivery_date
                ];
            }
        }
    }
    return $lists;
}

function truckCurrentLocation($truck){
    $arry                   = [];
    if(isset($truck)){
        $vehicleIds         = $truck->samsara_id ?? null;
        if(!is_null($vehicleIds)){
            $getAPIKey      = IntegrationSetting::where('company_id', $truck->company_id)->where('carrier_id', $truck->carrier_id)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_vehicles', '1')->first();
            if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                try {
                    $url                    = 'https://api.samsara.com/fleet/vehicles/locations?vehicleIds='.$vehicleIds;
                    $client                 = new \GuzzleHttp\Client();
                    $api_response           = $client->request('GET', $url, [
                        'headers'           => [
                            'Accept'        => 'application/json',
                            'Authorization' => 'Bearer '.$getAPIKey->api_key,
                        ],
                    ]);
                    // Check if response status code is OK (200)
                    if ($api_response->getStatusCode() == 200) {
                        $body               = json_decode($api_response->getBody());
                        // Do something with the response body
                        if(isset($body)){
                            $data           = isset($body->data[0])?$body->data[0]:null;
                            if(isset($data)){
                                if(isset($data->location)){
                                    $location       = $data->location;
                                    $arry           = [
                                        'time'      => Carbon::parse($location->time)->format('Y-m-d H:i:s'),
                                        'latitude'  => $location->latitude,
                                        'longitude' => $location->longitude,
                                        'heading'   => $location->heading,
                                        'speed'     => $location->speed,
                                        'location'  => $location->reverseGeo->formattedLocation,
                                        'response'  => $location
                                    ];
                                }
                            }
                        }
                    }
                } catch (\GuzzleHttp\Exception\ClientException $e) {
                    // Handle server-side errors (e.g. 5xx status codes)
                    $api_response           = $e->getResponse();
                    if ($api_response) {
                        $statusCode         = $api_response->getStatusCode();
                        $body               = $api_response->getBody()->getContents();
                        Log::error($body);
                    } else {
                        Log::error($e->getMessage());
                    }
                } catch (\GuzzleHttp\Exception\ServerException $e) {
                    // Handle server-side errors (e.g. 5xx status codes)
                    $api_response           = $e->getResponse();
                    if ($api_response) {
                        $statusCode         = $api_response->getStatusCode();
                        $body               = $api_response->getBody()->getContents();
                        Log::error($body);
                    } else {
                        Log::error($e->getMessage());
                    }
                } catch (\Exception $e) {
                    // Handle other exceptions
                    Log::error($e->getMessage());
                }
            }
        }
    }
    return $arry;
}

function getAllSafetyEvents($truck, $startTime, $endTime){
    $arry                   = [];
    if(isset($truck)){
        $vehicleIds         = $truck->samsara_id ?? null;
        if(!is_null($vehicleIds)){
            $getAPIKey      = IntegrationSetting::where('company_id', $truck->company_id)->where('carrier_id', $truck->carrier_id)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_vehicles', '1')->first();
            if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                $startTime  = Carbon::createFromFormat('Y-m-d H:i:s', $startTime)->format('Y-m-d\TH:i:s\Z'); // '2023-01-22T09:13:00Z';
                // Carbon::createFromFormat('Y-m-d H:i:s', $startTime)->format('Y-m-d\TH:i:sP');
                $endTime    = Carbon::createFromFormat('Y-m-d H:i:s', $endTime)->format('Y-m-d\TH:i:s\Z'); // '2023-06-23T12:00:00Z';
                // Carbon::createFromFormat('Y-m-d H:i:s', $endTime)->format('Y-m-d\TH:i:sP');
                try {
                    $url                    = 'https://api.samsara.com/fleet/safety-events?startTime='.$startTime.'&endTime='.$endTime.'&vehicleIds='.$vehicleIds;
                    $client                 = new \GuzzleHttp\Client();
                    $api_response           = $client->request('GET', $url, [
                        'headers'           => [
                            'Accept'        => 'application/json',
                            'Authorization' => 'Bearer '.$getAPIKey->api_key,
                        ],
                    ]);
                    // Check if response status code is OK (200)
                    if ($api_response->getStatusCode() == 200) {
                        $body               = json_decode($api_response->getBody());
                        // Do something with the response body
                        if(isset($body)){
                            if(isset($body->data) && !empty($body->data)){
                                $arry       = $body->data;
                            }
                        }
                    }
                } catch (\GuzzleHttp\Exception\ClientException $e) {
                    // Handle server-side errors (e.g. 5xx status codes)
                    $api_response           = $e->getResponse();
                    if ($api_response) {
                        $statusCode         = $api_response->getStatusCode();
                        $body               = $api_response->getBody()->getContents();
                        Log::error($body);
                    } else {
                        Log::error($e->getMessage());
                    }
                } catch (\GuzzleHttp\Exception\ServerException $e) {
                    // Handle server-side errors (e.g. 5xx status codes)
                    $api_response           = $e->getResponse();
                    if ($api_response) {
                        $statusCode         = $api_response->getStatusCode();
                        $body               = $api_response->getBody()->getContents();
                        Log::error($body);
                    } else {
                        Log::error($e->getMessage());
                    }
                } catch (\Exception $e) {
                    // Handle other exceptions
                    Log::error($e->getMessage());
                }
            }
        }
    }
    return $arry;
}

function appointmentRequestSend($appointment){
    if(isset($appointment->broker) && isset($appointment->customer)){
        $appointment_email_contacts                             = [];
        $appointment_phone_contacts                             = [];

        if(isset($appointment->broker->email)){
            $appointment_email_contacts[]                       = [
                'table'                                         => 'brokers',
                'field_name'                                    => 'email',
                'id'                                            => $appointment->broker->id,
                'name'                                          => $appointment->broker->name,
                'email'                                         => $appointment->broker->email,
                'type'                                          => 0 // brokers
            ];
        }
        if(isset($appointment->broker->phone)){
            $appointment_phone_contacts[]                       = [
                'table'                                         => 'brokers',
                'field_name'                                    => 'phone',
                'id'                                            => $appointment->broker->id,
                'name'                                          => $appointment->broker->name,
                'phone'                                         => $appointment->broker->phone,
                'type'                                          => 0 // brokers
            ];
        }

        if(isset($appointment->broker->billing_email)){
            $appointment_email_contacts[]                       = [
                'table'                                         => 'brokers',
                'field_name'                                    => 'billing_email',
                'id'                                            => $appointment->broker->id,
                'name'                                          => $appointment->broker->name,
                'email'                                         => $appointment->broker->billing_email,
                'type'                                          => 0 // brokers
            ];
        }
        if(isset($appointment->broker->billing_phone)){
            $appointment_phone_contacts[]                       = [
                'table'                                         => 'brokers',
                'field_name'                                    => 'billing_phone',
                'id'                                            => $appointment->broker->id,
                'name'                                          => $appointment->broker->name,
                'phone'                                         => $appointment->broker->billing_phone,
                'type'                                          => 0 // brokers
            ];
        }

        if(isset($appointment->broker->broker_contact) && $appointment->broker->broker_contact->isNotEmpty()){
            foreach ($appointment->broker->broker_contact as $key => $cdata) {
                if(isset($cdata->email)){
                    $appointment_email_contacts[]               = [
                        'table'                                 => 'broker_contacts',
                        'field_name'                            => 'email',
                        'id'                                    => $cdata->id,
                        'name'                                  => $cdata->name,
                        'email'                                 => $cdata->email,
                        'type'                                  => 1 // broker_contacts
                    ];
                }
                if(isset($cdata->cell_number)){
                    $appointment_phone_contacts[]               = [
                        'table'                                 => 'broker_contacts',
                        'field_name'                            => 'cell_number',
                        'id'                                    => $cdata->id,
                        'name'                                  => $cdata->name,
                        'phone'                                 => $cdata->cell_number,
                        'type'                                  => 1 // broker_contacts
                    ];
                }
            }
        }

        if(isset($appointment->customer->email)){
            $appointment_email_contacts[]                       = [
                'table'                                         => 'customers',
                'field_name'                                    => 'email',
                'id'                                            => $appointment->customer->id,
                'name'                                          => $appointment->customer->name,
                'email'                                         => $appointment->customer->email,
                'type'                                          => 2 // customers
            ];
        }
        if(isset($appointment->customer->phone)){
            $appointment_phone_contacts[]                       = [
                'table'                                         => 'customers',
                'field_name'                                    => 'phone',
                'id'                                            => $appointment->customer->id,
                'name'                                          => $appointment->customer->name,
                'phone'                                         => $appointment->customer->phone,
                'type'                                          => 2 // customers
            ];
        }
        if(!empty($appointment_email_contacts)){
            if(isset($appointment->emails) && is_serialized($appointment->emails) && !empty(unserialize($appointment->emails))){
                $emails                                         = unserialize($appointment->emails);
                foreach ($emails as $key => $item) {
                    $filteredItems                              = array_filter($appointment_email_contacts, function ($var) use ($item) {
                        return $var['table'] === $item['table'] && $var['field_name'] === $item['field_name'] && $var['email'] === $item['email'];
                    });
                    // Get the first matching key
                    $matchingKeys                               = array_keys($filteredItems);
                    $firstMatchingKey                           = reset($matchingKeys);
                    if($firstMatchingKey!==false){
                        $appointment_contact                    = $appointment_email_contacts[$firstMatchingKey] ?? [];
                        if(!empty($appointment_contact)){
                            $text                               = $appointment_contact['table'].'|'.$appointment_contact['field_name'].'|'.$appointment_contact['email'];
                            $longUrl                            = route('company.admin.appointment.request.view', [
                                'id'                            => encryptData($appointment->id),
                                'email'                         => encryptData($text), // text val
                                'source'                        => encryptData('1'), // 1 = Email, 2 = Phone For Sms
                            ]);
                            // Store the long URL and the generated short code in the database
                            $short_url                          = \App\Models\ShortUrl::create([
                                'long_url'                      => $longUrl,
                                'short_code'                    => generateStrUniqueId('short_urls'),
                            ]);
                            if(!is_null($short_url)){
                                $details                        = [];
                                $details['short_code']          = $short_url->short_code;
                                $details['appointment']         = $appointment;
                                $details['appointment_contact'] = $appointment_contact;
                                $details['name']                = $appointment_contact['name'];
                                $details['email']               = $appointment_contact['email'];
                                $details['broker_name']         = isset($appointment->broker->name) ? $appointment->broker->name : '';
                                $details['customer_name']       = isset($appointment->customer->name) ? $appointment->customer->name : '';

                                if(auth()->guard('company-admin')->check()){
                                    sendEmail($appointment_contact['email'],new \App\Mail\AppointmentRequestBroker($details));
                                }else{
                                    try {
                                        Mail::to($appointment_contact['email'])->send(new \App\Mail\AppointmentRequestBroker($details));
                                    } catch (\Swift_TransportException $e) {
                                        // Handle transport exception (e.g., connection failure)

                                        // Log the error message
                                        \Log::error('Email sending failed: ' . $e->getMessage());

                                        // Optionally, you can re-throw the exception if desired
                                        // throw $e;
                                    } catch (\Exception $e) {
                                        // Handle any other exception

                                        // Log the error message
                                        \Log::error('An error occurred while sending the email: ' . $e->getMessage());

                                        // Optionally, you can re-throw the exception if desired
                                        // throw $e;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        if(!empty($appointment_phone_contacts)){
            if(isset($appointment->phones) && is_serialized($appointment->phones) && !empty(unserialize($appointment->phones))){
                $phones                                         = unserialize($appointment->phones);
                foreach ($phones as $key => $item) {
                    $filteredItems                              = array_filter($appointment_phone_contacts, function ($var) use ($item) {
                        return $var['table'] === $item['table'] && $var['field_name'] === $item['field_name'] && $var['phone'] === $item['phone'];
                    });
                    // Get the first matching key
                    $matchingKeys                               = array_keys($filteredItems);
                    $firstMatchingKey                           = reset($matchingKeys);
                    if($firstMatchingKey!==false){
                        $appointment_contact                    = $appointment_phone_contacts[$firstMatchingKey] ?? [];
                        if(!empty($appointment_contact)){
                            $text                               = $appointment_contact['table'].'|'.$appointment_contact['field_name'].'|'.$appointment_contact['phone'];
                            $longUrl                            = route('company.admin.appointment.request.view', [
                                'id'                            => encryptData($appointment->id),
                                'email'                         => encryptData($text), // text val
                                'source'                        => encryptData('2'), // 1 = Email, 2 = Phone For Sms
                            ]);
                            // Store the long URL and the generated short code in the database
                            $short_url                          = \App\Models\ShortUrl::create([
                                'long_url'                      => $longUrl,
                                'short_code'                    => generateStrUniqueId('short_urls'),
                            ]);
                            if(!is_null($short_url)){
                                // Twilio's credentials are fetched from company table
                                $getCompanyTwilioData           = getCompanyTwilioData($appointment->broker->company_id ?? 0);
                                if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                                    $load_number                = $appointment->load_detail->load_number ?? '#########';
                                    $load_number                = str_replace(' ', '', $load_number);

                                    $messageBody                = "Dispatch Pro- Appointment Request for load # ".$load_number."\n".route('company.admin.appointment.request.redirect', ['code'  => $short_url->short_code]);

                                    sendSMS($appointment_contact['phone'], $messageBody, $getCompanyTwilioData);
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return true;
}

function convertToSqlQuery($query)
{
    return vsprintf(str_replace('?', '%s', $query->toSql()), collect($query->getBindings())->map(function ($binding) {
        return is_numeric($binding) ? $binding : "'{$binding}'";
    })->toArray());
}

function getLoadTotalMiles($load)
{
    $load_type      = array(1, 2, 3, 5); // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
    if (!in_array($load->load_type, $load_type)) {
        if ($load->is_disable == 0) {
            return number_format($load->total_miles);
        } else {
            $data   = LoadDriver::where('load_id', $load->load_id)
                ->where('driver_id', $load->driver_id)
                ->orderBy('id', 'DESC')
                ->first()
                ->total_miles;
            return $data;
        }

    }
    return null;
}

function getLoadDeadheadMileage($startDate, $endDate, $driverId, $load = null, $trailer_type = null, $dispatcher = null)
{
    $data           = Driver::select("load_drivers.empty_miles")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where("load_drivers.driver_id", $driverId)
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at')
        ->groupBy('load_drivers.load_detail_id');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $data       = $data->whereIn('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $data       = $data->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$data     = $data->where('load_details.booked_by',$dispatcher);
        $data       = $data->where('drivers.manager_id', $dispatcher);
    }

    $data           = $data->pluck("load_drivers.empty_miles")->all();

    $loadIds        = LoadDetail::select("load_details.load_id")
        ->join("drivers", "drivers.id", "=", "load_details.driver_id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where('load_details.is_disable', 1)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $loadIds    = $loadIds->where('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $loadIds    = $loadIds->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$loadIds  = $loadIds->where('load_details.booked_by',$dispatcher);
        $loadIds    = $loadIds->where('drivers.manager_id', $dispatcher);
    }

    $loadIds        = $loadIds->pluck("load_details.load_id")->all();

    $data2          = [];
    foreach ($loadIds as $key => $value) {
        $data2[]    = LoadDriver::where("load_id", $value)
            ->where("driver_id", $driverId)
            ->orderBy("id", "DESC")
            ->first()
            ->empty_miles;
    }
    return array_sum($data) + array_sum($data2);
}

function getLoadLoadedMileage($startDate, $endDate, $driverId, $load = null, $trailer_type = null, $dispatcher = null)
{
    $data           = Driver::select("load_drivers.loaded_miles")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where("load_drivers.driver_id", $driverId)
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at')
        ->groupBy('load_drivers.load_detail_id');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $data       = $data->whereIn('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $data       = $data->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$data     = $data->where('load_details.booked_by',$dispatcher);
        $data       = $data->where('drivers.manager_id', $dispatcher);
    }

    $data           = $data->pluck("load_drivers.loaded_miles")->all();

    $loadIds        = LoadDetail::select("load_details.load_id")
        ->join("drivers", "drivers.id", "=", "load_details.driver_id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where('load_details.is_disable', 1)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $loadIds    = $loadIds->where('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $loadIds    = $loadIds->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$loadIds = $loadIds->where('load_details.booked_by',$dispatcher);
        $loadIds    = $loadIds->where('drivers.manager_id', $dispatcher);
    }

    $loadIds        = $loadIds->pluck("load_details.load_id")->all();

    $data2          = [];
    foreach ($loadIds as $key => $value) {
        $data2[]    = LoadDriver::where("load_id", $value)
            ->where("driver_id", $driverId)
            ->orderBy("id", "DESC")
            ->first()
            ->loaded_miles;
    }
    return array_sum($data) + array_sum($data2);
}

function getLoadTotalMileage($startDate, $endDate, $driverId, $load = null, $trailer_type = null, $dispatcher = null)
{
    $data           = Driver::select("load_drivers.total_miles")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where("load_drivers.driver_id", $driverId)
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at')
        ->groupBy('load_drivers.load_detail_id');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $data       = $data->whereIn('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $data       = $data->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$data     = $data->where('load_details.booked_by',$dispatcher);
        $data       = $data->where('drivers.manager_id', $dispatcher);
    }

    $data           = $data->pluck("load_drivers.total_miles")->all();

    $loadIds        = LoadDetail::select("load_details.load_id")
        ->join("drivers", "drivers.id", "=", "load_details.driver_id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where('load_details.is_disable', 1)
        ->where('load_details.load_status', '!=', 9) // 9 = Cancelled
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $loadIds    = $loadIds->where('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $loadIds    = $loadIds->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$loadIds  = $loadIds->where('load_details.booked_by',$dispatcher);
        $loadIds    = $loadIds->where('drivers.manager_id', $dispatcher);
    }

    $loadIds        = $loadIds->pluck("load_details.load_id")->all();

    $data2          = [];
    foreach ($loadIds as $key => $value) {
        $data2[]    = LoadDriver::where("load_id", $value)
            ->where("driver_id", $driverId)
            ->orderBy("id", "DESC")
            ->first()
            ->total_miles;
    }

    return array_sum($data) + array_sum($data2);

    return $data;
}

function getLoadTotalMileCPM($startDate, $endDate, $driverId, $load, $trailer_type = null, $dispatcher = null)
{
    if (getLoadGrossRevenue($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher) == 0 || getLoadTotalMileage($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher) == 0) {
        return 0;
    } else {
        return number_format(getLoadGrossRevenue($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher) / getLoadTotalMileage($startDate, $endDate, $driverId, $load, $trailer_type, $dispatcher), 2);
    }
}

function getCarrierWiseTerminalList($carrierId)
{
    return Terminal::where("carrier_id", $carrierId)->pluck('name', 'id')->all();
}

function getCompIdentifier()
{
    return strtoupper(substr(auth()->guard('company-admin')->user()->getCompany->name, 0, 3)).'-';
}

function checkeGalleryFileUpload($actionId, $field)
{
    $isUploaded = Gallery::where('action_id', $actionId)->where('field_name', $field)->whereNotNull('uploaded_date')->count();

    if($isUploaded > 0){
        return 'uploaded';
    }else{
        return null;
    }
}

function getZoneWiseStateList()
{
    $stateList = State::pluck('name', 'id')->all();

    $statesArr = [];
    foreach ($stateList as $key => $stateName){
        if(in_array($stateName, ['Arizona', 'New Mexico', 'Oklahoma', 'Texas'])){
            $statesArr['Southwest'][$key] = $stateName;
        }else if(in_array($stateName, ['California', 'Colorado', 'Idaho', 'Montana', 'Nevada', 'Oregon', 'Washington', 'Wyoming', 'Hawaii', 'Alaska', 'Utah'])){
            $statesArr['West'][$key] = $stateName;
        }else if(in_array($stateName, ['Illinois', 'Indiana', 'Iowa', 'Kansas', 'Michigan', 'Minnesota', 'Nebraska', 'North Dakota', 'Ohio', 'South Dakota', 'Wisconsin', 'Missouri', 'Minnesota'])){
            $statesArr['Midwest'][$key] = $stateName;
        }else if(in_array($stateName, ['Alabama', 'Arkansas', 'Florida', 'Georgia', 'Kentucky', 'Louisiana', 'Mississippi', 'North Carolina', 'South Carolina', 'Tennessee', 'Virginia', 'West Virginia'])){
            $statesArr['Southeast'][$key] = $stateName;
        }else if(in_array($stateName, ['Delaware', 'Pennsylvania', 'New York', 'Maine', 'Maryland', 'New Hampshire', 'New Jersey', 'Rhode Island', 'Vermont', 'Massachusetts', 'Connecticut'])){
            $statesArr['Northeast'][$key] = $stateName;
        }
    }
    ksort($statesArr['Southwest']);
    ksort($statesArr['West']);
    ksort($statesArr['Midwest']);
    ksort($statesArr['Southeast']);
    ksort($statesArr['Northeast']);

    return $statesArr;
}

if(! function_exists('chatGeneralBoardCreate')){
    function chatGeneralBoardCreate($user) {
        if($user->getTable() == "drivers"){
            $unique_id              = 'driver';
        }else{
            if($user->type==0){
                $unique_id          = 'company_admin';
            }else if ($user->type==1) {
                $unique_id          = 'manager';
            }else if ($user->type==2) {
                $unique_id          = 'dispatcher';
            }else{
                return null;
            }
        }

        $arry                   = [];
        $arry['company_id']     = $user->company_id;
        $arry['unique_id']      = $unique_id.'_'.$user->id;
        $arry['friendly_name']  = $user->first_name.' '.$user->last_name;
        $item                   = ChatGeneralBoard::firstOrCreate($arry);
        if(!is_null($item)){
            try {
                $jsonData = [
                    'participant_details' => [
                        [
                            'username'      => $item->unique_id,
                            'friendlyName'  => $item->friendly_name,
                            'profilePic'    => $user->profile_image ?? '',
                        ],
                    ]
                ];
                $url                    = env('NODE_CHAT_API_BASE_URL','http://localhost:8000').'/api/CreateGenelarBoardConversation';
                $client                 = new \GuzzleHttp\Client();
                $response               = $client->request('POST', $url, [
                    'headers'           => [
                        'Accept'        => 'application/json',
                        'Content-Type'  => 'application/json',
                    ],
                    'json'              => $jsonData
                ]);
                // Check if response status code is OK (200)
                if ($response->getStatusCode() == 200) {
                    $body               = json_decode($response->getBody());
                    // Do something with the response body
                    if(isset($body)){
                        $item->sid      = $body->conversation->sid;
                        $item->save();
                        // Log::info('sid : '.$body->conversation->sid);
                    }else{
                        // Log::error('Create General Board Conversation : body not set.');
                    }
                }
            } catch (\GuzzleHttp\Exception\RequestException $e) {
                // Handle other request-related errors
                if ($e->hasResponse()) {
                    $response           = $e->getResponse();
                    $statusCode         = $response->getStatusCode();
                    $reason             = $response->getReasonPhrase();
                    $body               = $response->getBody()->getContents();
                    // Log::error('Create General Board Conversation : '.$body);
                } else {
                    $errorMessage       = $e->getMessage();
                    // Log::error('Create General Board Conversation : '.$errorMessage);
                }
            } catch (\Exception $e) {
                // Handle other exceptions
                $errorMessage           = $e->getMessage();
                // Log::error('Create General Board Conversation : '.$errorMessage);
            }
        }
        return $item;
    }
}

if(! function_exists('chatLoadCreate')){
    function chatLoadCreate($loadDetail) {
        $attributes                     = $loadDetail->getAttributes();

        // $pickupDate                     = Carbon::parse(date('Y-m-d', strtotime($attributes['actual_shipper_date'])));
        $pickupDate                     = Carbon::parse(date('Y-m-d', strtotime($attributes['actual_delivery_date'])));
        $fromDate                       = Carbon::now()->format('Y-m-d');
        $fromDate                       = Carbon::parse($fromDate);
        // $toDate                         = $pickupDate->addDays(2);
        $toDate                         = $pickupDate->addDays(4);
        $diffInDays                     = $fromDate->diffInDays($toDate);

        $arry                           = [];
        $company_id                     = $loadDetail->loads->company_id;
        $arry['company_id']             = $company_id;
        $arry['unique_id']              = 'load_'.$loadDetail->load_id;
        $arry['friendly_name']          = $loadDetail->load_number;
        $arry['ttl']                    = $diffInDays;
        $item                           = ChatLoad::firstOrCreate($arry);
        if(!is_null($item)){
            try {
                $url                    = env('NODE_CHAT_API_BASE_URL','http://localhost:8000').'/api/CreateLoadConversation';
                $client                 = new \GuzzleHttp\Client();
                $response               = $client->request('POST', $url, [
                    'headers'           => [
                        'Accept'        => 'application/json',
                        'Content-Type'  => 'application/x-www-form-urlencoded',
                    ],
                    'form_params'       => [
                        'load_number'   => $item->friendly_name,
                        'ttl'           => $diffInDays,
                        'load_id'       => $loadDetail->load_id
                    ]
                ]);
                // Check if response status code is OK (200)
                if ($response->getStatusCode() == 200) {
                    $body               = json_decode($response->getBody());
                    // Do something with the response body
                    if(isset($body)){
                        Log::info('load sid : '.$body->conversation->sid);
                        $load_sid       = $body->conversation->sid;
                        $item->sid      = $load_sid;
                        if($item->save()){
                            $participants                   = [];
                            if(isset($loadDetail->dispatcher)){
                                $arry                       = [];
                                $user                       = $loadDetail->dispatcher;
                                $type                       = 'dispatcher';
                                if($user->type==0){
                                    $type                   = 'company_admin';
                                }else if ($user->type==1) {
                                    $type                   = 'manager';
                                }else if ($user->type==2) {
                                    $type                   = 'dispatcher';
                                }
                                $unique_id                  = $type.'_'.$user->id;
                                $arry['chat_load_id']       = $item->id;
                                $arry['company_id']         = $company_id;
                                $arry['unique_id']          = $unique_id;
                                $arry['friendly_name']      = $user->full_name;
                                if(isset($user->profile_image) && $user->profile_image!='' && file_exists(public_path('storage/companyAdmin/profile/'.$user->profile_image))){
                                    $arry['profile_image']  = asset('storage/companyAdmin/profile').'/'.$user->profile_image;
                                }
                                $participants[$unique_id]   = $arry;
                            }
                            if(isset($loadDetail->company_admin)){
                                $arry                       = [];
                                $user                       = $loadDetail->company_admin;
                                $type                       = 'dispatcher';
                                if($user->type==0){
                                    $type                   = 'company_admin';
                                }else if ($user->type==1) {
                                    $type                   = 'manager';
                                }else if ($user->type==2) {
                                    $type                   = 'dispatcher';
                                }
                                $unique_id                  = $type.'_'.$user->id;
                                $arry['chat_load_id']       = $item->id;
                                $arry['company_id']         = $company_id;
                                $arry['unique_id']          = $unique_id;
                                $arry['friendly_name']      = $user->full_name;
                                if(isset($user->profile_image) && $user->profile_image!='' && file_exists(public_path('storage/companyAdmin/profile/'.$user->profile_image))){
                                    $arry['profile_image']  = asset('storage/companyAdmin/profile').'/'.$user->profile_image;
                                }
                                $participants[$unique_id]   = $arry;
                            }
                            if(isset($loadDetail->driver)){
                                $arry                       = [];
                                $user                       = $loadDetail->driver;
                                $unique_id                  = 'driver_'.$user->id;
                                $arry['chat_load_id']       = $item->id;
                                $arry['company_id']         = $company_id;
                                $arry['unique_id']          = $unique_id;
                                $arry['friendly_name']      = $user->full_name;
                                if(isset($user->profile_image) && $user->profile_image!='' && file_exists(public_path('storage/companyAdmin/profile/'.$user->profile_image))){
                                    $arry['profile_image']  = asset('storage/companyAdmin/profile').'/'.$user->profile_image;
                                }
                                $participants[$unique_id]   = $arry;
                            }
                            if(isset($loadDetail->co_driver)){
                                $arry                       = [];
                                $user                       = $loadDetail->co_driver;
                                $unique_id                  = 'driver_'.$user->id;
                                $arry['chat_load_id']       = $item->id;
                                $arry['company_id']         = $company_id;
                                $arry['unique_id']          = $unique_id;
                                $arry['friendly_name']      = $user->full_name;
                                if(isset($user->profile_image) && $user->profile_image!='' && file_exists(public_path('storage/companyAdmin/profile/'.$user->profile_image))){
                                    $arry['profile_image']  = asset('storage/companyAdmin/profile').'/'.$user->profile_image;
                                }
                                $participants[$unique_id]   = $arry;
                            }
                            if(!empty($participants)){
                                foreach ($participants as $key => $participantData) {
                                    try {
                                        $jsonData               = [
                                            'load_sid'              => $load_sid,
                                            'participant_details'   => [
                                                [
                                                    'username'      => $participantData['unique_id'],
                                                    'friendlyName'  => $participantData['friendly_name'],
                                                    'profilePic'    => $participantData['profile_image'] ?? '',
                                                ],
                                            ]
                                        ];
                                        $url                    = env('NODE_CHAT_API_BASE_URL','http://localhost:8000').'/api/addLoadParticipant';
                                        $client                 = new \GuzzleHttp\Client();
                                        $response               = $client->request('POST', $url, [
                                            'headers'           => [
                                                'Accept'        => 'application/json',
                                                'Content-Type'  => 'application/json',
                                            ],
                                            'json'              => $jsonData
                                        ]);
                                        // Check if response status code is OK (200)
                                        if ($response->getStatusCode() == 200) {
                                            $body               = json_decode($response->getBody());
                                            // Do something with the response body
                                            if(isset($body)){
                                                Log::info('loadParticipant sid : '.$body->loadParticipant->sid);
                                                if(isset($participantData['profile_image'])){
                                                    unset($participantData['profile_image']);
                                                }
                                                $participantData['sid'] = $body->loadParticipant->sid;
                                                ChatLoadParticipants::firstOrCreate($participantData);
                                            }else{
                                                Log::error('Create Participant Conversation : body not set.');
                                            }
                                        }
                                    } catch (\GuzzleHttp\Exception\RequestException $e) {
                                        // Handle other request-related errors
                                        if ($e->hasResponse()) {
                                            $response           = $e->getResponse();
                                            $statusCode         = $response->getStatusCode();
                                            $reason             = $response->getReasonPhrase();
                                            $body               = $response->getBody()->getContents();
                                            Log::error('Create Load Participant Conversation : '.$body);
                                        } else {
                                            $errorMessage       = $e->getMessage();
                                            Log::error('Create Load Participant Conversation : '.$errorMessage);
                                        }
                                    } catch (\Exception $e) {
                                        // Handle other exceptions
                                        $errorMessage           = $e->getMessage();
                                        Log::error('Create Load Participant Conversation : '.$errorMessage);
                                    }
                                }
                            }
                        }
                    }else{
                        Log::error('Create Load Conversation : body not set.');
                    }
                }
            } catch (\GuzzleHttp\Exception\RequestException $e) {
                // Handle other request-related errors
                if ($e->hasResponse()) {
                    $response           = $e->getResponse();
                    $statusCode         = $response->getStatusCode();
                    $reason             = $response->getReasonPhrase();
                    $body               = $response->getBody()->getContents();
                    Log::error('Create Load Conversation : '.$body);
                } else {
                    $errorMessage       = $e->getMessage();
                    Log::error('Create Load Conversation : '.$errorMessage);
                }
            } catch (\Exception $e) {
                // Handle other exceptions
                $errorMessage           = $e->getMessage();
                Log::error('Create Load Conversation : '.$errorMessage);
            }
        }
        return $item;
    }
}

// NEW CODE 28-03-2023
if(! function_exists('chatLoadttlUpdate')){
    function chatLoadttlUpdate($load_id) {
        $unique_id  = 'load_'.$load_id;

        $fromDate   = Carbon::now()->format('Y-m-d');
        $fromDate   = Carbon::parse($fromDate);
        $toDate     = Carbon::now()->addDays(5);
        $diffInDays = $fromDate->diffInDays($toDate);

        $chatLoad   = ChatLoad::where([
            ['unique_id', $unique_id],
            ['status', 'active']
        ])->update(['ttl'=> $diffInDays]);
    }
}

function getAllSMSHistory()
{
    $sid        = env("TWILIO_SID");
    $token      = env("TWILIO_AUTH_TOKEN");
    $twilio     = new Client($sid, $token);

    $messages   = $twilio->messages->read([]);

    $data       = array();

    foreach ($messages as $message) {
        $data[] = [
            'from'      => $message->from,
            'to'        => $message->to,
            'message'   => $message->body,
            'date'      => $message->dateSent,
            'status'    => $message->status,
            'sid'       => $message->sid
        ];
    }
    return $data;
}

function getFilterSMSHistory( $from = '', $to = '', $num = '' )
{
    if( auth()->guard('company-admin')->check() ) {
        $logged_in_guard    = 'company-admin';
    } else if( auth()->guard('driver-admin')->check() ) {
        $logged_in_guard    = 'driver-admin';
    } else {
        return [];
    }

    $twilio_sid             = auth()->guard($logged_in_guard)->user()->twilio_sid;
    $twilio_auth_token      = auth()->guard($logged_in_guard)->user()->twilio_auth_token;
    $twilio_active_number   = auth()->guard($logged_in_guard)->user()->twilio_active_number;

    if( $twilio_active_number == '' ) {
        return [];
    }

    $twilio                 = new Client($twilio_sid, $twilio_auth_token);

    $filter                 = [];
    if( $from != '' ) {
        $filter             = ["from" => $from];
    }
    if( $to != '' ) {
        $filter             = ["to" => $to];
    }
    if( $from != '' && $to != '' ) {
        $filter             = ["from" => $from, "to" => $to];
    }

    if( $num != '' ) {
        $messages           = $twilio->messages->read($filter, $num);
    } else {
        $messages           = $twilio->messages->read($filter);
    }

    $data                   = array();

    foreach ($messages as $message) {
        if( $message->direction == 'inbound' ) {
            $msg_align_class    = 'chat-mianLeft';
            $msg_body           = $message->body;
        } else {
            $msg_align_class    = 'chat-mianRight';
            $msg_body           = auth()->guard($logged_in_guard)->user()->first_name.' '.auth()->guard($logged_in_guard)->user()->last_name.' - '.$message->body;
        }

        $data[]             = [
            'from'              => $message->from,
            'to'                => $message->to,
            'message'           => $msg_body,
            'date'              => $message->dateSent->format('d-m-Y H:i:s'),
            'status'            => $message->status,
            'sid'               => $message->sid,
            'direction'         => $message->direction,
            'msg_align_class'   => $msg_align_class
        ];
    }

    return $data;
}

function getFilterSMSHistoryByCompany( $company_id, $skip, $take, $logged_in_guard, $search = '' ,$tophone = '' )
{
    if( $search != '' ) {
        $messages   = SmsChatHistory::where( 'company_id', $company_id )
        ->where(function ($query) use ( $tophone  ) {
            $query->where('from_number','LIKE','%'.$tophone.'%')
            ->orWhere('to_number', 'LIKE','%'. $tophone.'%');
        })
        ->where('msg_body','LIKE','%'.$search.'%')
        ->skip($skip)->take($take)
        ->orderBy('id', 'DESC')
        ->get();
    } else {
        $messages   = SmsChatHistory::where( 'company_id', $company_id )
        ->where(function ($query) use ( $tophone  ) {
            $query->where('from_number','LIKE','%'.$tophone.'%')
            ->orWhere('to_number', 'LIKE','%'. $tophone.'%');
        })
        ->skip($skip)->take($take)
        ->orderBy('id', 'DESC')
        ->get();
    }

    $user_id        = auth()->guard( $logged_in_guard )->user()->id;
    $sender_id      = $logged_in_guard.'_'.$user_id;

    $data           = array();
    foreach ($messages as $message) {
        // Added by Jaypal Raj 18-04-2023
        $message->is_sms_read   = 1;
        $message->save();

        if( $sender_id == $message->unique_id_sender ) {
            $msg_align_class    = 'chat-mianRight';
        } else {
            $msg_align_class    = 'chat-mianLeft';
        }

        $sender                 = $message->from_number;

        if( $message->table_name != '' ) {
            if( $message->table_name == 'company_admins' ) {
                $user_id        = explode("_", $message->unique_id_sender )[1];
                $user           = CompanyAdmin::select('id','first_name','last_name')->where('id', $user_id)->first();
                $sender         = ucwords( $user->first_name.' '.$user->last_name );
            } else if( $message->table_name == 'drivers' ) {
                $user_id        = explode("_", $message->unique_id_sender )[1];
                $user           = Driver::select('id','first_name','last_name')->where('id', $user_id)->first();
                $sender         = ucwords( $user->first_name.' '.$user->last_name );
            } else {
                $sender         = $message->from_number;
            }
        }

        $data[]                 = [
            'from'              => $sender,
            'to'                => $message->to_number,
            'message'           => $message->msg_body,
            'date'              => date('d-m-Y H:i:s', strtotime( $message->sms_date )),
            'status'            => $message->status,
            'sid'               => $message->msg_id,
            'direction'         => $message->type,
            'msg_align_class'   => $msg_align_class,
            'time'              => date("h:i A", strtotime( $message->sms_date ))
        ];
    }
    return $data;
}

// Added at 18-04-2023
function getunReadSMSByCompany( $company_id, $skip, $take, $logged_in_guard ,$tophone )
{
    $messages   = SmsChatHistory::where( 'company_id', $company_id )
    ->where(function ($query) use ($tophone ) {
        $query->where('from_number','LIKE','%'.$tophone.'%')
        ->orWhere('to_number', 'LIKE','%'. $tophone.'%');
    })
    ->where( 'is_sms_read', 0 )
    ->where( 'type', 'received' )
    ->skip($skip)->take($take)
    ->orderBy('id', 'DESC')
    ->get();

    $user_id    = auth()->guard( $logged_in_guard )->user()->id;
    $sender_id  = $logged_in_guard.'_'.$user_id;

    $data       = array();
    foreach ($messages as $message) {
        $message->is_sms_read   = 1;
        $message->save();

        if( $sender_id == $message->unique_id_sender ) {
            $msg_align_class    = 'chat-mianRight';
        } else {
            $msg_align_class    = 'chat-mianLeft';
        }

        $sender                 = $message->from_number;

        if( $message->table_name != '' ) {
            if( $message->table_name == 'company_admins' ) {
                $user_id        = explode("_", $message->unique_id_sender )[1];
                $user           = CompanyAdmin::select('id','first_name','last_name')->where('id', $user_id)->first();
                $sender         = ucwords( $user->first_name.' '.$user->last_name );
            } else if( $message->table_name == 'drivers' ) {
                $user_id        = explode("_", $message->unique_id_sender )[1];
                $user           = Driver::select('id','first_name','last_name')->where('id', $user_id)->first();
                $sender         = ucwords( $user->first_name.' '.$user->last_name );
            } else {
                $sender         = $message->from_number;
            }
        }

        $data[]                 = [
            'from'              => $sender,
            'to'                => $message->to_number,
            'message'           => $message->msg_body,
            'date'              => date('d-m-Y H:i:s', strtotime( $message->sms_date )),
            'status'            => $message->status,
            'sid'               => $message->msg_id,
            'direction'         => $message->type,
            'msg_align_class'   => $msg_align_class,
            'time'              => date("h:i A", strtotime( $message->sms_date ))
        ];
    }
    return $data;
}

function getTotalSmsCountCompany( $company_id )
{
    $smsCount = SmsChatHistory::where( 'company_id', $company_id )->count();
    return $smsCount;
}

function startConversation()
{
    $sid            = env("TWILIO_SID");
    $token          = env("TWILIO_AUTH_TOKEN");
    $twilio         = new Client($sid, $token);

    $conversation   = $twilio->conversations->v1->conversations->create(["friendlyName" => "My First Conversation"]);
    return $conversation;
}

function fetchConversation( $cid )
{
    $sid            = env("TWILIO_SID");
    $token          = env("TWILIO_AUTH_TOKEN");
    $twilio         = new Client($sid, $token);

    $conversation   = $twilio->conversations->v1->conversations($cid)->fetch();
    return $conversation;
}

function addParticipant( $cid, $participant_num )
{
    $sid            = env("TWILIO_SID");
    $token          = env("TWILIO_AUTH_TOKEN");
    $my_number      = env("TWILIO_NUMBER");

    $twilio         = new Client($sid, $token);

    $participant    = $twilio->conversations->v1->conversations($cid)
        ->participants
        ->create([
            "messagingBindingAddress"       => $my_number,
            "messagingBindingProxyAddress"  => $participant_num
        ]);
    return $conversation;
}

// ADDED AT 11-04-2023
function sendSMSByCompany($company_id, $to, $from, $message)
{
    $company        = Company::where('id', $company_id)->first();

    $sid            = $company->twilio_sid;
    $token          = $company->twilio_auth_token;
    $twilio         = new Client($sid, $token);

    $message        = $twilio->messages->create($to, [
        "body" => $message,
        "from" => $from
    ]);

    \Log::emergency( $message );
    return $message->sid;
}

// DEVELOPED AT 13-04-2023
if(! function_exists('closeCurrentLoadForNewLoad')){
    function closeCurrentLoadForNewLoad($loadDetail) {
        $load_id                = $loadDetail->load_id;
        $load_number            = $loadDetail->load_number;

        $unique_id              = 'load_'.$load_id;

        $getOldChatLoad         = ChatLoad::where([
            ['unique_id', $unique_id],
            ['status', 'active']
        ])->first();
        if(!is_null($getOldChatLoad)){
            $fromDate           = Carbon::now()->format('Y-m-d');
            $fromDate           = Carbon::parse($fromDate);
            $toDate             = date("Y-m-d", strtotime($getOldChatLoad->created_at));
            $toDate             = Carbon::parse($toDate);
            $diffInDays         = $fromDate->diffInDays($toDate);
            $diffInDays         = $getOldChatLoad->ttl - $diffInDays;

            $conversationSid    = $getOldChatLoad->sid;
            $load_number        = $load_number;
            $ttl                = $diffInDays;

            // API TO CALL HERE, Change only if get SID
            try {
                $url            = env('NODE_CHAT_API_BASE_URL','http://localhost:8000').'/api/closeCurrentLoadForNewLoad';
                $client         = new \GuzzleHttp\Client();
                $response       = $client->request('POST', $url, [
                    'headers'                   => [
                        'Accept'                => 'application/json',
                        'Content-Type'          => 'application/x-www-form-urlencoded'
                    ],
                    'form_params'               => [
                        'conversationSid'       => $conversationSid,
                        'load_number'           => $load_number,
                        'ttl'                   => $ttl
                    ]
                ]);

                // Check if response status code is OK (200)
                if ($response->getStatusCode() == 200) {
                    $body                       = json_decode($response->getBody());

                    if(isset($body) && $body->conversation->sid != '' ) {
                        Log::info('load sid : '.$body->conversation->sid);

                        $getOldChatLoad->status = 'closed';
                        $getOldChatLoad->save();

                        // Close old chat participants
                        ChatLoadParticipants::where('chat_load_id', $getOldChatLoad->id)->update(['status'=> 'closed']);

                        $arry                   = [];
                        $company_id             = $getOldChatLoad->company_id;
                        $arry['company_id']     = $company_id;
                        $arry['unique_id']      = $getOldChatLoad->unique_id;
                        $arry['friendly_name']  = $load_number;
                        $arry['ttl']            = $ttl;
                        $load_sid               = $body->conversation->sid;
                        $arry['sid']            = $load_sid;

                        $item                   = ChatLoad::firstOrCreate($arry);

                        if($item->save()){
                            $participants                               = [];
                            if(isset($loadDetail->dispatcher)){
                                $arry                                   = [];
                                $user                                   = $loadDetail->dispatcher;
                                $type                                   = 'dispatcher';
                                if($user->type==0){
                                    $type                               = 'company_admin';
                                }else if ($user->type==1) {
                                    $type                               = 'manager';
                                }else if ($user->type==2) {
                                    $type                               = 'dispatcher';
                                }
                                $unique_id                              = $type.'_'.$user->id;
                                $arry['chat_load_id']                   = $item->id;
                                $arry['company_id']                     = $company_id;
                                $arry['unique_id']                      = $unique_id;
                                $arry['friendly_name']                  = $user->full_name;
                                if(isset($user->profile_image) && $user->profile_image!='' && file_exists(public_path('storage/companyAdmin/profile/'.$user->profile_image))){
                                    $arry['profile_image']              = asset('storage/companyAdmin/profile').'/'.$user->profile_image;
                                }
                                $participants[$unique_id]               = $arry;
                            }
                            if(isset($loadDetail->company_admin)){
                                $arry                                   = [];
                                $user                                   = $loadDetail->company_admin;
                                $type                                   = 'dispatcher';
                                if($user->type==0){
                                    $type                               = 'company_admin';
                                }else if ($user->type==1) {
                                    $type                               = 'manager';
                                }else if ($user->type==2) {
                                    $type                               = 'dispatcher';
                                }
                                $unique_id                              = $type.'_'.$user->id;
                                $arry['chat_load_id']                   = $item->id;
                                $arry['company_id']                     = $company_id;
                                $arry['unique_id']                      = $unique_id;
                                $arry['friendly_name']                  = $user->full_name;
                                if(isset($user->profile_image) && $user->profile_image!='' && file_exists(public_path('storage/companyAdmin/profile/'.$user->profile_image))){
                                    $arry['profile_image']              = asset('storage/companyAdmin/profile').'/'.$user->profile_image;
                                }
                                $participants[$unique_id]               = $arry;
                            }
                            if(isset($loadDetail->driver)){
                                $arry                                   = [];
                                $user                                   = $loadDetail->driver;
                                $unique_id                              = 'driver_'.$user->id;
                                $arry['chat_load_id']                   = $item->id;
                                $arry['company_id']                     = $company_id;
                                $arry['unique_id']                      = $unique_id;
                                $arry['friendly_name']                  = $user->full_name;
                                if(isset($user->profile_image) && $user->profile_image!='' && file_exists(public_path('storage/companyAdmin/profile/'.$user->profile_image))){
                                    $arry['profile_image']              = asset('storage/companyAdmin/profile').'/'.$user->profile_image;
                                }
                                $participants[$unique_id]               = $arry;
                            }
                            if(isset($loadDetail->co_driver)){
                                $arry                                   = [];
                                $user                                   = $loadDetail->co_driver;
                                $unique_id                              = 'driver_'.$user->id;
                                $arry['chat_load_id']                   = $item->id;
                                $arry['company_id']                     = $company_id;
                                $arry['unique_id']                      = $unique_id;
                                $arry['friendly_name']                  = $user->full_name;
                                if(isset($user->profile_image) && $user->profile_image!='' && file_exists(public_path('storage/companyAdmin/profile/'.$user->profile_image))){
                                    $arry['profile_image']              = asset('storage/companyAdmin/profile').'/'.$user->profile_image;
                                }
                                $participants[$unique_id]               = $arry;
                            }
                            if(!empty($participants)){
                                foreach ($participants as $key => $participantData) {
                                    try {
                                        $jsonData                       = [
                                            'load_sid'                  => $load_sid,
                                            'participant_details'       => [
                                                [
                                                    'username'          => $participantData['unique_id'],
                                                    'friendlyName'      => $participantData['friendly_name'],
                                                    'profilePic'        => $participantData['profile_image'] ?? ''
                                                ],
                                            ]
                                        ];
                                        $url                            = env('NODE_CHAT_API_BASE_URL','http://localhost:8000').'/api/addLoadParticipant';
                                        $client                         = new \GuzzleHttp\Client();
                                        $response                       = $client->request('POST', $url, [
                                            'headers'                   => [
                                                'Accept'                => 'application/json',
                                                'Content-Type'          => 'application/json'
                                            ],
                                            'json'                      => $jsonData
                                        ]);
                                            // Check if response status code is OK (200)
                                        if ($response->getStatusCode() == 200) {
                                            $body                       = json_decode($response->getBody());
                                                // Do something with the response body
                                            if(isset($body)){
                                                Log::info('loadParticipant sid : '.$body->loadParticipant->sid);
                                                if(isset($participantData['profile_image'])){
                                                    unset($participantData['profile_image']);
                                                }
                                                $participantData['sid'] = $body->loadParticipant->sid;
                                                ChatLoadParticipants::firstOrCreate($participantData);
                                            }else{
                                                Log::error('Create Participant Conversation : body not set.');
                                            }
                                        }
                                    } catch (\GuzzleHttp\Exception\RequestException $e) {
                                            // Handle other request-related errors
                                        if ($e->hasResponse()) {
                                            $response                   = $e->getResponse();
                                            $statusCode                 = $response->getStatusCode();
                                            $reason                     = $response->getReasonPhrase();
                                            $body                       = $response->getBody()->getContents();
                                            Log::error('Create Load Participant Conversation : '.$body);
                                        } else {
                                            $errorMessage               = $e->getMessage();
                                            Log::error('Create Load Participant Conversation : '.$errorMessage);
                                        }
                                    } catch (\Exception $e) {
                                            // Handle other exceptions
                                        $errorMessage                   = $e->getMessage();
                                        Log::error('Create Load Participant Conversation : '.$errorMessage);
                                    }
                                }
                            }
                        }
                    }else{
                        Log::error('Create Load Conversation : body not set.');
                    }
                }
            } catch (\GuzzleHttp\Exception\RequestException $e) {
                // Handle other request-related errors
                if ($e->hasResponse()) {
                    $response                   = $e->getResponse();
                    $statusCode                 = $response->getStatusCode();
                    $reason                     = $response->getReasonPhrase();
                    $body                       = $response->getBody()->getContents();
                    Log::error('Create Load Conversation : '.$body);
                } else {
                    $errorMessage               = $e->getMessage();
                    Log::error('Create Load Conversation : '.$errorMessage);
                }
            } catch (\Exception $e) {
                $errorMessage                   = $e->getMessage();
                Log::error('Create Load Conversation : '.$errorMessage);
            }
        }
    }
}

// DEVELOPED AT 13-04-2023
if(! function_exists('removeConversationParticipant')){
    function removeConversationParticipant($loadDetail) {
        $oldSid                                 = [];
        $newids                                 = [];

        if( $loadDetail->isDirty('driver_id') ) {
            $oldSid['driver_id']                = 'driver_'.$loadDetail->getOriginal('driver_id');

            $newids[]                           = array('type' => 'driver', 'id' => $loadDetail->driver_id, 'user' => $loadDetail->driver);
        }

        if( $loadDetail->isDirty('co_driver_id') ) {
            $oldSid['co_driver_id']             = 'driver_'.$loadDetail->getOriginal('co_driver_id');

            $newids[]                           = array('type' => 'driver', 'id' => $loadDetail->co_driver_id, 'user' => $loadDetail->co_driver);
        }

        if( $loadDetail->isDirty('booked_by') ) {
            $oldBookedBy                        = CompanyAdmin::select('id','type')->where('id', $loadDetail->getOriginal('booked_by'))->first();

            if( $oldBookedBy->type == 0 ) {
                $oldSid['booked_by']            = 'company_admin_'.$loadDetail->getOriginal('booked_by');
            } else if( $oldBookedBy->type == 1 ) {
                $oldSid['booked_by']            = 'manager_'.$loadDetail->getOriginal('booked_by');
            } else if( $oldBookedBy->type == 2 ) {
                $oldSid['booked_by']            = 'dispatcher_'.$loadDetail->getOriginal('booked_by');
            }

            $user                               = $loadDetail->dispatcher;
            if($user->type==0){
                $type                           = 'company_admin';
            }else if ($user->type==1) {
                $type                           = 'manager';
            }else if ($user->type==2) {
                $type                           = 'dispatcher';
            }

            $newids[]                           = array('type' => $type, 'id' => $loadDetail->booked_by);
        }

        if( count( $oldSid ) > 0 ) {
            // $load_number                     = $loadDetail->load_number;
            $load_id                            = $loadDetail->load_id;
            $unique_id                          = 'load_'.$load_id;
            $chatLoad                           = ChatLoad::where([
                ['unique_id', $unique_id],
                ['status', 'active']
            ])->first();
            if(!is_null($chatLoad)){
                $oldParticipants                = ChatLoadParticipants::select('sid')
                    ->where('chat_load_id',$chatLoad->id)
                    ->whereIn('unique_id', $oldSid)
                    ->pluck('sid')
                    ->toArray();
                if(!empty($oldParticipants)){
                    try {
                        $url                    = env('NODE_CHAT_API_BASE_URL','http://localhost:8000').'/api/removeConversationParticipant';
                        $client                 = new \GuzzleHttp\Client();

                        $company_id             = $chatLoad->company_id;
                        $conversationSid        = $chatLoad->sid;
                        $load_sid               = $conversationSid;

                        $jsonData               = [
                            'conversationSid'   => $conversationSid,
                            'old_participants'  => $oldParticipants
                        ];
                        $response               = $client->request('POST', $url, [
                            'headers'           => [
                                'Accept'        => 'application/json',
                                'Content-Type'  => 'application/json',
                            ],
                            'json'              => $jsonData
                        ]);

                        // Check if response status code is OK (200)
                        if ($response->getStatusCode() == 200) {
                            $body               = json_decode($response->getBody());

                            if(isset($body) && $body->success == true ) {
                                Log::info('Participant remove api success');

                                ChatLoadParticipants::where('chat_load_id',$chatLoad->id)
                                    ->whereIn('sid', $oldParticipants)
                                    ->update(['status' => 'closed']);

                                if(!empty($newids)){
                                    foreach ($newids as $key => $value) {
                                        if( $value['type'] == 'driver' ) {
                                            //$user                             = $loadDetail->driver;
                                            $user                               = $value['user'];
                                        } else {
                                            $user                               = $loadDetail->dispatcher;
                                        }

                                        $unique_id                              = $value['type'].'_'.$value['id'];
                                        $participantData['chat_load_id']        = $chatLoad->id;
                                        $participantData['company_id']          = $company_id;
                                        $participantData['unique_id']           = $unique_id;
                                        $participantData['friendly_name']       = $user->full_name;

                                        if(isset($user->profile_image) && $user->profile_image!='' && file_exists(public_path('storage/companyAdmin/profile/'.$user->profile_image))) {
                                            $participantData['profile_image']   = asset('storage/companyAdmin/profile').'/'.$user->profile_image;
                                        }

                                        try {
                                            $jsonData                           = [
                                                'load_sid'                      => $load_sid,
                                                'participant_details'           => [
                                                    [
                                                        'username'              => $participantData['unique_id'],
                                                        'friendlyName'          => $participantData['friendly_name'],
                                                        'profilePic'            => $participantData['profile_image'] ?? ''
                                                    ],
                                                ]
                                            ];
                                            $url                                = env('NODE_CHAT_API_BASE_URL','http://localhost:8000').'/api/addLoadParticipant';
                                            $client                             = new \GuzzleHttp\Client();
                                            $response                           = $client->request('POST', $url, [
                                                'headers'                       => [
                                                    'Accept'                    => 'application/json',
                                                    'Content-Type'              => 'application/json'
                                                ],
                                                'json'                          => $jsonData
                                            ]);

                                            // Check if response status code is OK (200)
                                            if ($response->getStatusCode() == 200) {
                                                $body                           = json_decode($response->getBody());

                                                // Do something with the response body
                                                if(isset($body)){
                                                    Log::info('loadParticipant sid : '.$body->loadParticipant->sid);
                                                    if(isset($participantData['profile_image'])){
                                                        unset($participantData['profile_image']);
                                                    }
                                                    $participantData['sid']     = $body->loadParticipant->sid;
                                                    ChatLoadParticipants::firstOrCreate($participantData);
                                                }else{
                                                    Log::error('Create Participant Conversation : body not set.');
                                                }
                                            }
                                        } catch (\GuzzleHttp\Exception\RequestException $e) {
                                                // Handle other request-related errors
                                            if ($e->hasResponse()) {
                                                $response                       = $e->getResponse();
                                                $statusCode                     = $response->getStatusCode();
                                                $reason                         = $response->getReasonPhrase();
                                                $body                           = $response->getBody()->getContents();
                                                Log::error('Create Load Participant Conversation : '.$body);
                                            } else {
                                                $errorMessage                   = $e->getMessage();
                                                Log::error('Create Load Participant Conversation : '.$errorMessage);
                                            }
                                        } catch (\Exception $e) {
                                                // Handle other exceptions
                                            $errorMessage                       = $e->getMessage();
                                            Log::error('Create Load Participant Conversation : '.$errorMessage);
                                        }
                                    }
                                }
                            } else {
                                Log::error('Create Load Conversation : body not set.');
                            }
                        }
                    } catch (\GuzzleHttp\Exception\RequestException $e) {
                        // Handle other request-related errors
                        if ($e->hasResponse()) {
                            $response       = $e->getResponse();
                            $statusCode     = $response->getStatusCode();
                            $reason         = $response->getReasonPhrase();
                            $body           = $response->getBody()->getContents();
                            Log::error('Create Load Conversation : '.$body);
                        } else {
                            $errorMessage   = $e->getMessage();
                            Log::error('Create Load Conversation : '.$errorMessage);
                        }
                    } catch (\Exception $e) {
                        $errorMessage       = $e->getMessage();
                        Log::error('Create Load Conversation : '.$errorMessage);
                    }
                }
            }
        }
    }
}

if(! function_exists('chatLoadInternalParticipantsUpdate')){
    function chatLoadInternalParticipantsUpdate($data_arry) {
        $company_id                         = $data_arry['company_id'];
        $participant_details                = [];
        $remove_partisipants                = [];
        $oldSid                             = [];
        $newids                             = [];
        if (array_key_exists('participant_details', $data_arry)) {
            $participant_details            = $data_arry['participant_details'];
        }
        if (array_key_exists('remove_partisipants', $data_arry)) {
            $remove_partisipants            = $data_arry['remove_partisipants'];
        }
        $ChatLoadInternal                   = ChatLoadInternal::where([
            ['sid', $data_arry['loadInternalSid']],
            ['status', 'active']
        ])->first();
        if($ChatLoadInternal){
            if(count($remove_partisipants) > 0){
                $oldParticipants            = ChatLoadInternalParticipants::select('sid')->where('chat_load_internal_id',$ChatLoadInternal->id)->whereIn('unique_id', $remove_partisipants)->where('status', 'active')->pluck('sid')->toArray();
                // dd($oldParticipants);
                // dd($ChatLoadInternal->id);
                if( count( $oldParticipants ) > 0 ) {
                    try {
                        $url                = env('NODE_CHAT_API_BASE_URL','http://localhost:8000').'/api/removeConversationParticipant';
                        $client             = new \GuzzleHttp\Client();

                        $jsonData           = [
                            'conversationSid'   => $data_arry['loadInternalSid'],
                            'old_participants'  => $oldParticipants
                        ];

                        $response           = $client->request('POST', $url, [
                            'headers'           => [
                                'Accept'        => 'application/json',
                                'Content-Type'  => 'application/json',
                            ],
                            'json'              => $jsonData
                        ]);

                        // Check if response status code is OK (200)
                        if ($response->getStatusCode() == 200) {
                            $body           = json_decode($response->getBody());

                            if(isset($body) && $body->success == true ) {
                                Log::info('Participant remove api success');

                                ChatLoadInternalParticipants::whereIn('sid', $oldParticipants)->update(['status' => 'closed']);
                            } else {
                                Log::error('Create Load Conversation : body not set.');
                            }
                        }
                    } catch (\GuzzleHttp\Exception\RequestException $e) {
                        // Handle other request-related errors
                        if ($e->hasResponse()) {
                            $response       = $e->getResponse();
                            $statusCode     = $response->getStatusCode();
                            $reason         = $response->getReasonPhrase();
                            $body           = $response->getBody()->getContents();
                            Log::error('chatLoadInternalParticipantsUpdate remove old internal participant : '.$body);
                        } else {
                            $errorMessage   = $e->getMessage();
                            Log::error('chatLoadInternalParticipantsUpdate remove old internal participant : '.$errorMessage);
                        }
                    } catch (\Exception $e) {
                        $errorMessage       = $e->getMessage();
                        Log::error('chatLoadInternalParticipantsUpdate remove old internal participant : '.$errorMessage);
                    }
                }
            }
            else{
                Log::error('No old participants modifications found');
            }

            if( count( $participant_details ) > 0 ) {
                try {
                    foreach ($participant_details as $key => $value) {
                        $participantData['chat_load_internal_id'] = $ChatLoadInternal->id;
                        $participantData['company_id'] = $company_id;
                        $participantData['unique_id'] = $value['username'];
                        $participantData['friendly_name'] = $value['friendlyName'];

                        try {
                            $jsonData               = [
                                'load_sid' => $data_arry['loadInternalSid'],
                                'participant_details' => [
                                    [
                                        'username' => $participantData['unique_id'],
                                        'friendlyName' => $participantData['friendly_name'],
                                        'profilePic' => $value['profilePic'] ?? '',
                                    ],
                                ]
                            ];
                            $url                    = env('NODE_CHAT_API_BASE_URL','http://localhost:8000').'/api/addLoadParticipant';
                            $client                 = new \GuzzleHttp\Client();
                            $response               = $client->request('POST', $url, [
                                'headers'           => [
                                    'Accept'        => 'application/json',
                                    'Content-Type'  => 'application/json',
                                ],
                                'json'              => $jsonData
                            ]);

                            // Check if response status code is OK (200)
                            if ($response->getStatusCode() == 200) {
                                $body               = json_decode($response->getBody());

                                // Do something with the response body
                                if(isset($body)){
                                    Log::info('loadParticipant sid : '.$body->loadParticipant->sid);

                                    $participantData['sid'] = $body->loadParticipant->sid;
                                    ChatLoadInternalParticipants::firstOrCreate($participantData);
                                }else{
                                    Log::error('Create internal Participant Conversation : body not set.');
                                }
                            }
                        } catch (\GuzzleHttp\Exception\RequestException $e) {
                                // Handle other request-related errors
                            if ($e->hasResponse()) {
                                $response           = $e->getResponse();
                                $statusCode         = $response->getStatusCode();
                                $reason             = $response->getReasonPhrase();
                                $body               = $response->getBody()->getContents();
                                Log::error('Create Load Participant Conversation : '.$body);
                            } else {
                                $errorMessage       = $e->getMessage();
                                Log::error('Create Load Participant Conversation : '.$errorMessage);
                            }
                        } catch (\Exception $e) {
                                // Handle other exceptions
                            $errorMessage           = $e->getMessage();
                            Log::error('Create Load Participant Conversation : '.$errorMessage);
                        }
                    }
                } catch (\GuzzleHttp\Exception\RequestException $e) {
                    // Handle other request-related errors
                    if ($e->hasResponse()) {
                        $response           = $e->getResponse();
                        $statusCode         = $response->getStatusCode();
                        $reason             = $response->getReasonPhrase();
                        $body               = $response->getBody()->getContents();
                        Log::error('Create Load Conversation : '.$body);
                    } else {
                        $errorMessage       = $e->getMessage();
                        Log::error('Create Load Conversation : '.$errorMessage);
                    }
                } catch (\Exception $e) {
                    $errorMessage           = $e->getMessage();
                    Log::error('Create Load Conversation : '.$errorMessage);
                }
            }
        }
        else{
            Log::error('Invalid Chat Internal Load sid');
        }

    }
}

if(! function_exists('chatLoadInternalCreate')){
    function chatLoadInternalCreate($data_arry) {
        // $actual_delivery_date        = Carbon::parse(date('Y-m-d', strtotime($data_arry['actual_delivery_date'])));
        // $fromDate                    = Carbon::now()->format('Y-m-d');
        // $fromDate                    = Carbon::parse($fromDate);
        // $toDate                      = $actual_delivery_date->addDays(4);
        // $diffInDays                  = $fromDate->diffInDays($toDate);

        $arry                           = [];
        $company_id                     = $data_arry['company_id'];
        $arry['company_id']             = $company_id;
        $arry['unique_id']              = 'load_'.$data_arry['load_id'];
        $arry['friendly_name']          = 'internal_'.$data_arry['load_id'];
        $arry['ttl']                    = $data_arry['ttl'];
        // $arry['ttl']                    = $diffInDays;
        $item                           = ChatLoadInternal::firstOrCreate($arry);
        if(!is_null($item)){
            try {
                $url                    = env('NODE_CHAT_API_BASE_URL','http://localhost:8000').'/api/CreateLoadInternalConversation';
                $client                 = new \GuzzleHttp\Client();
                $response               = $client->request('POST', $url, [
                    'headers'           => [
                        'Accept'        => 'application/json',
                        'Content-Type'  => 'application/x-www-form-urlencoded',
                    ],
                    'form_params'       => [
                        'load_number'   => $item->friendly_name,
                        'ttl'           => $data_arry['ttl']
                    ]
                ]);
                // Check if response status code is OK (200)
                if ($response->getStatusCode() == 200) {
                    $body               = json_decode($response->getBody());
                    // Do something with the response body
                    if(isset($body)){
                        Log::info('load internal sid : '.$body->conversation->sid);
                        $load_sid       = $body->conversation->sid;
                        $item->sid      = $load_sid;
                        if($item->save()){
                            $participants                   = $data_arry['participant_details'];
                            // $getIds                      = [];
                            // if($data_arry['notify_private_company_admin_id'] != ""){
                            //     $getIds                                 = explode(',',$data_arry['notify_private_company_admin_id']);
                            //     array_push($getIds,$data_arry['created_by']);
                            // }
                            // else{
                            //     array_push($getIds,$data_arry['created_by']);
                            // }
                            // if(count($getIds) > 0){
                            //     foreach($getIds as $getId){
                            //         // if(isset($getLoad->id)){
                            //             $userdata                       = CompanyAdmin::find($getId);
                            //             $arry                           = [];
                            //             $type                           = 'dispatcher';
                            //             if($userdata->type==0){
                            //                 $type                       = 'company_admin';
                            //             }else if ($userdata->type==1) {
                            //                 $type                       = 'manager';
                            //             }else if ($userdata->type==2) {
                            //                 $type                       = 'dispatcher';
                            //             }
                            //             $unique_id                      = $type.'_'.$userdata->id;
                            //             // $arry['chat_load_internal_id']  = 1;
                            //             $arry['chat_load_internal_id']  = $item->id;
                            //             $arry['company_id']             = $data_arry['company_id'];
                            //             $arry['unique_id']              = $unique_id;
                            //             $arry['friendly_name']          = $userdata->full_name;
                            //             if(isset($userdata->profile_image) && $userdata->profile_image!='' && file_exists(public_path('storage/companyAdmin/profile/'.$userdata->profile_image))){
                            //                 $arry['profile_image']      = asset('storage/companyAdmin/profile').'/'.$userdata->profile_image;
                            //             }
                            //             $participants[$unique_id]       = $arry;
                            //         // }
                            //     }
                            // }
                            if(!empty($participants)){
                                foreach ($participants as $key => $participantData) {
                                    try {
                                        $jsonData               = [
                                            'load_sid'              => $load_sid,
                                            'participant_details'   => [
                                                [
                                                    'username'      => $participantData['username'],
                                                    'friendlyName'  => $participantData['friendlyName'],
                                                    'profilePic'    => $participantData['profilePic'] ?? '',
                                                ],
                                            ]
                                        ];
                                        $url                    = env('NODE_CHAT_API_BASE_URL','http://localhost:8000').'/api/addLoadParticipant';
                                        $client                 = new \GuzzleHttp\Client();
                                        $response               = $client->request('POST', $url, [
                                            'headers'           => [
                                                'Accept'        => 'application/json',
                                                'Content-Type'  => 'application/json',
                                            ],
                                            'json'              => $jsonData
                                        ]);
                                        // Check if response status code is OK (200)
                                        if ($response->getStatusCode() == 200) {
                                            $body               = json_decode($response->getBody());
                                            // Do something with the response body
                                            if(isset($body)){
                                                Log::info('loadParticipant sid : '.$body->loadParticipant->sid);
                                                // if(isset($participantData['profilePic'])){
                                                //     unset($participantData['profilePic']);
                                                // }
                                                $partcipant_data_array                     = [];
                                                $partcipant_data_array['chat_load_internal_id']  = $item->id;
                                                $arry['chat_load_internal_id']  = $item->id;
                                                $partcipant_data_array['company_id']             = $data_arry['company_id'];
                                                $partcipant_data_array['unique_id']              = $participantData['username'];
                                                $partcipant_data_array['sid'] = $body->loadParticipant->sid;
                                                $p = ChatLoadInternalParticipants::firstOrCreate($partcipant_data_array);
                                                // dd($p);
                                            }else{
                                                Log::error('Create Participant Conversation : body not set.');
                                            }
                                        }
                                    } catch (\GuzzleHttp\Exception\RequestException $e) {
                                        // Handle other request-related errors
                                        if ($e->hasResponse()) {
                                            $response           = $e->getResponse();
                                            $statusCode         = $response->getStatusCode();
                                            $reason             = $response->getReasonPhrase();
                                            $body               = $response->getBody()->getContents();
                                            Log::error('Create Load Participant Conversation : '.$body);
                                        } else {
                                            $errorMessage       = $e->getMessage();
                                            Log::error('Create Load Participant Conversation : '.$errorMessage);
                                        }
                                    } catch (\Exception $e) {
                                        // Handle other exceptions
                                        $errorMessage           = $e->getMessage();
                                        Log::error('Create Load Participant Conversation : '.$errorMessage);
                                    }
                                }
                            }
                        }
                    }else{
                        Log::error('Create Load Conversation : body not set.');
                    }
                }
            } catch (\GuzzleHttp\Exception\RequestException $e) {
                // Handle other request-related errors
                if ($e->hasResponse()) {
                    $response           = $e->getResponse();
                    $statusCode         = $response->getStatusCode();
                    $reason             = $response->getReasonPhrase();
                    $body               = $response->getBody()->getContents();
                    Log::error('Create Load Conversation : '.$body);
                } else {
                    $errorMessage       = $e->getMessage();
                    Log::error('Create Load Conversation : '.$errorMessage);
                }
            } catch (\Exception $e) {
                // Handle other exceptions
                $errorMessage           = $e->getMessage();
                Log::error('Create Load Conversation : '.$errorMessage);
            }
        }
        return $item;
    }
}
if(! function_exists('addMessageTochatLoadInternal')) {
    function addMessageTochatLoadInternal($data_arry)
    {
        try {
            $url        = env('NODE_CHAT_API_BASE_URL', 'http://localhost:8000') . '/api/addMessageTochatLoadInternal';
            $client     = new \GuzzleHttp\Client();
            $response   = $client->request('POST', $url, [
                'headers' => [
                    'Accept'        => 'application/json',
                    'Content-Type'  => 'application/x-www-form-urlencoded',
                ],
                // 'form_params'    => [
                //     'note'       => 'Hello note 1',
                //     'files'      => $data_arry
                // ]
                'multipart'         => [
                    'name'          => 'files[]',
                    'contents'      => fopen($data_arry[0]->getPathname(), 'r'),
                    'filename'      => $data_arry[0]->getClientOriginalName()
                ]
            ]);
            // Check if response status code is OK (200)
            if ($response->getStatusCode() == 200) {
                dd($response);
            }
        } catch (\GuzzleHttp\Exception\RequestException $e) {
            // Handle other request-related errors
            if ($e->hasResponse()) {
                $response           = $e->getResponse();
                $statusCode         = $response->getStatusCode();
                $reason             = $response->getReasonPhrase();
                $body               = $response->getBody()->getContents();
                Log::error('Create Load Conversation : ' . $body);
            } else {
                $errorMessage       = $e->getMessage();
                Log::error('Create Load Conversation : ' . $errorMessage);
            }
        }
        dd("after api call");
    }
}

/*
function getFilterSMSHistoryByCompany( $company_id, $skip, $take, $logged_in_guard, $search = '' )
{
     if( $search != '' ) {
         $messages = SmsChatHistory::where( 'company_id', $company_id )
         ->where('msg_body','LIKE','%'.$search.'%')
         ->skip($skip)->take($take)
         ->orderBy('id', 'DESC')
         ->get();
     } else {
         $messages = SmsChatHistory::where( 'company_id', $company_id )
         ->skip($skip)->take($take)
         ->orderBy('id', 'DESC')
         ->get();
     }
     $user_id = auth()->guard( $logged_in_guard )->user()->id;
     $sender_id = $logged_in_guard.'_'.$user_id;

     $data = array();

     foreach ($messages as $message) {
         if( $sender_id == $message->unique_id_sender ) {
             $msg_align_class = 'chat-mianRight';
         } else {
             $msg_align_class = 'chat-mianLeft';
         }

         if( $message->table_name != '' ) {
             if( $message->table_name == 'company_admins' ) {
                 $user_id = explode("_", $message->unique_id_sender )[1];
                 $user = CompanyAdmin::select('id','first_name','last_name')->where('id', $user_id)->first();
                 $sender = ucwords( $user->first_name.' '.$user->last_name );
             } else if( $message->table_name == 'drivers' ) {
                 $user_id = explode("_", $message->unique_id_sender )[1];
                 $user = Driver::select('id','first_name','last_name')->where('id', $user_id)->first();
                 $sender = ucwords( $user->first_name.' '.$user->last_name );
             } else {
                 $sender = $message->from_number;
             }
         }

         $data[] = [
             'from' => $sender,
             'to' => $message->to_number,
             'message' => $message->msg_body,
             'date' => date('d-m-Y H:i:s', strtotime( $message->sms_date )),
             'status' => $message->status,
             'sid' => $message->msg_id,
             'direction' => $message->type,
             'msg_align_class' => $msg_align_class,
             'time' => date("h:i A", strtotime( $message->sms_date ))
         ];
     }

     return $data;
 }
*/

/*
function getFilterSMSHistoryByCompany( $company_id, $skip, $take, $logged_in_guard, $search = '' )
{
     if( $search != '' ) {
         $messages = SmsChatHistory::where( 'company_id', $company_id )
         ->where('msg_body','LIKE','%'.$search.'%')
         ->skip($skip)->take($take)
         ->orderBy('id', 'DESC')
         ->get();
     } else {
         $messages = SmsChatHistory::where( 'company_id', $company_id )
         ->skip($skip)->take($take)
         ->orderBy('id', 'DESC')
         ->get();
     }

     $user_id = auth()->guard( $logged_in_guard )->user()->id;
     $sender_id = $logged_in_guard.'_'.$user_id;

     $data = array();

     foreach ($messages as $message) {
         if( $sender_id == $message->unique_id_sender ) {
             $msg_align_class = 'chat-mianRight';
         } else {
             $msg_align_class = 'chat-mianLeft';
         }

         $sender = $message->from_number;

         if( $message->table_name != '' ) {
             if( $message->table_name == 'company_admins' ) {
                 $user_id = explode("_", $message->unique_id_sender )[1];
                 $user = CompanyAdmin::select('id','first_name','last_name')->where('id', $user_id)->first();
                 $sender = ucwords( $user->first_name.' '.$user->last_name );
             } else if( $message->table_name == 'drivers' ) {
                 $user_id = explode("_", $message->unique_id_sender )[1];
                 $user = Driver::select('id','first_name','last_name')->where('id', $user_id)->first();
                 $sender = ucwords( $user->first_name.' '.$user->last_name );
             } else {
                 $sender = $message->from_number;
             }
         }

         $data[] = [
             'from' => $sender,
             'to' => $message->to_number,
             'message' => $message->msg_body,
             'date' => date('d-m-Y H:i:s', strtotime( $message->sms_date )),
             'status' => $message->status,
             'sid' => $message->msg_id,
             'direction' => $message->type,
             'msg_align_class' => $msg_align_class,
             'time' => date("h:i A", strtotime( $message->sms_date ))
         ];
     }

     return $data;
 }
*/

function getLoadFuelSurcharge($startDate, $endDate, $driverId, $load = null, $trailer_type = null, $dispatcher = null)
{
    $data = Driver::select("load_details.fuel_surcharge")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where("load_drivers.driver_id", $driverId)
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at')
        ->groupBy('load_drivers.load_detail_id');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $data       = $data->whereIn('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $data       = $data->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$data     = $data->where('load_details.booked_by',$dispatcher);
        $data       = $data->where('drivers.manager_id', $dispatcher);
    }

    $data = $data->pluck("load_details.fuel_surcharge")->all();

    return array_sum($data);
}

function getLoadLinehaul($startDate, $endDate, $driverId, $load = null, $trailer_type = null, $dispatcher = null)
{
    $data = Driver::select("load_details.line_haul")
        ->join("load_details", "load_details.driver_id", "=", "drivers.id")
        ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
        ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
        ->where("drivers.id", $driverId)
        ->where("load_drivers.driver_id", $driverId)
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at')
        ->groupBy('load_drivers.load_detail_id');

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $data       = $data->whereIn('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $data       = $data->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        //$data     = $data->where('load_details.booked_by',$dispatcher);
        $data       = $data->where('drivers.manager_id', $dispatcher);
    }

    $data = $data->pluck("load_details.line_haul")->all();

    return array_sum($data);
}

function getLoadAccessorials($startDate, $endDate, $driverId, $load = null, $trailer_type = null, $dispatcher = null)
{
    $data = Driver::select("load_accessorials.driver_total")
            ->join("load_details", "load_details.driver_id", "=", "drivers.id")
            ->join("load_drivers", "load_drivers.load_detail_id", "=", "load_details.id")
            ->join("load_accessorials", "load_accessorials.load_detail_id", "=", "load_details.id")
            ->whereBetween(DB::raw('DATE_FORMAT(load_details.actual_delivery_date, "%Y-%m-%d")'), [$startDate, $endDate])
            ->where("drivers.id", $driverId)
            ->where("load_drivers.driver_id", $driverId)
            ->where("load_accessorials.driver_id", $driverId)
            ->where('load_details.is_disable', 0)
            ->where('load_details.load_status', '!=', 9)
            ->where(function($q) {
                $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
            })
            ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
            ->whereNull('load_details.deleted_at');
    $data = $data->pluck("load_accessorials.driver_total")->all();

    if(is_array($trailer_type) && count($trailer_type) > 0){
        $data       = $data->whereIn('load_details.trailer_type', $trailer_type);
    }else if ($trailer_type != null) {
        $data       = $data->where('load_details.trailer_type', $trailer_type);
    }

    if ($dispatcher != null) {
        $data       = $data->where('drivers.manager_id', $dispatcher);
    }

    return array_sum($data);
}

function documentRequestReminder($gallery){
    if(!is_null($gallery->reminder_on)){
        if($gallery->created_by_type == 3){
            $driverDetails                          = Driver::findOrFail($gallery->created_by);
            $driverEmail                            = $driverDetails->email ?? null;
            $driverPhone                            = $driverDetails->cell_phone ?? null;
            $driverName                             = $driverDetails->full_name ?? null;
            $requestedByName                        = CompanyAdmin::findOrFail($gallery->requested_by)->full_name ?? null;

            if(!is_null($driverEmail) || !is_null($driverPhone)){
                $reminder_on                        = explode(',', $gallery->reminder_on);

                foreach ($reminder_on as $source) {
                    if($source == '1' && !is_null($driverEmail)){
                        $details                    = [];
                        $details['company_id']      = $gallery->company_id;
                        $details['file_name']       = str_replace('_', ' ', Str::title($gallery->field_name));
                        $details['driver_name']     = $driverName;
                        $details['requested_by']    = $requestedByName;
                        $details['request_note']    = $gallery->notes;

                        if(auth()->guard('company-admin')->check()){
                            sendEmail($driverEmail,new \App\Mail\DocumentRequestDriver($details));
                        }else{
                            try {
                                Mail::to($driverEmail)->send(new \App\Mail\DocumentRequestDriver($details));
                            } catch (\Swift_TransportException $e) {
                                // Handle transport exception (e.g., connection failure)

                                // Log the error message
                                Log::error('Email sending failed: ' . $e->getMessage());

                                // Optionally, you can re-throw the exception if desired
                                // throw $e;
                            } catch (\Exception $e) {
                                // Handle any other exception

                                // Log the error message
                                Log::error('An error occurred while sending the email: ' . $e->getMessage());

                                // Optionally, you can re-throw the exception if desired
                                // throw $e;
                            }
                        }
                    }else if($source == '2' && !is_null($driverPhone)){
                        if(is_null($gallery->notes)) {
                            // Twilio's credentials are fetched from company table
                            $getCompanyTwilioData   = getCompanyTwilioData($driverDetails->company_id);
                            if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                                sendSMS($driverPhone, 'Dispatch Pro - Please upload '.str_replace('_', ' ', Str::title($gallery->field_name)).' as soon as possible. Document Requested by '.$requestedByName, $getCompanyTwilioData);
                            }
                        }else{
                            // Twilio's credentials are fetched from company table
                            $getCompanyTwilioData   = getCompanyTwilioData($driverDetails->company_id);
                            if(isset($getCompanyTwilioData['active']) && $getCompanyTwilioData['active'] == 'yes'){
                                sendSMS($driverPhone, 'Dispatch Pro - Document Request for '.str_replace('_', ' ', Str::title($gallery->field_name)).'. '.$gallery->notes.' Document Requested by '.$requestedByName, $getCompanyTwilioData);
                            }
                        }
                    }
                }
            }
        }
    }
    return true;
}

// Trailer

function getTrailerYears()
{
    $years      = [];
    // Start from the current year and go back to 1940
    for ($year  = date('Y') + 1; $year >= 1940; $year--) {
        $years[$year] = $year;
    }
    return $years;
}

function getTrailerAxles()
{
    $truckAxles = [];
    foreach (range(1, 30) as $range){
        $truckAxles[$range] = $range;
    }
    return $truckAxles;
}

function getTrailerTerms(){
    return ['30_days' => '30 Days', '60_days' => '60 Days', '90_days' => '90 Days', '120_days' => '120 Days', '180_days' => '180 Days', '1_year' => '1 year', '2_year' => '2 year', '3_year' => '3 year', '4_year' => '4 year', '5_year' => '5 year'];
}

function getTrailerSchedules(){
    return ['1_year' => 'Annual', '30_days' => '30 Days', '60_days' => '60 Days', '90_days' => '90 Days', '120_days' => '120 Days', '180_days' => '180 Days'];
}

function getTrailerEldProvider()
{
    return ['1' => 'GeoTab', '2' => 'OmniTracs', '3' => 'Samsara', '4' => 'TransFlo'];
}

// Trailer Profiles

function getTrailerCompanyInsuranceProfiles()
{
    return TrailerCompanyInsuranceProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->get();
}

function getTrailerCompanyInsuranceProfilesName()
{
    return TrailerCompanyInsuranceProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('profile_name', 'id')->all();
}

function getTrailerCompanyPmServiceProfiles()
{
    return TrailerCompanyPmServiceProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->get();
}

function getTrailerCompanyPmServiceProfilesName()
{
    return TrailerCompanyPmServiceProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('profile_name', 'id')->all();
}

function getTrailerCompanyPmServiceWarrantyProfiles()
{
    return TrailerCompanyPmServiceWarrantyProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->get();
}

function getTrailerCompanyPmServiceWarrantyProfilesName()
{
    return TrailerCompanyPmServiceWarrantyProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('profile_name', 'id')->all();
}

function getTrailerCompanyEquipmentProfiles()
{
    return TrailerCompanyEquipmentProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->get();
}

function getTrailerCompanyEquipmentProfilesName()
{
    return TrailerCompanyEquipmentProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('profile_name', 'id')->all();
}

function getTrailerCompanyDocumentProfiles()
{
    return TrailerCompanyDocumentProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->get();
}

function getTrailerCompanyDocumentProfilesName()
{
    return TrailerCompanyDocumentProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('profile_name', 'id')->all();
}

// Truck

function getTruckYears()
{
    $truckYears = [];
    // Start from the current year and go back to 1940
    for ($year  = date('Y') + 1; $year >= 1940; $year--) {
        $truckYears[$year] = $year;
    }
    return $truckYears;
}

function getTruckAxles()
{
    $truckAxles = [];
    foreach (range(1, 10) as $range){
        $truckAxles[$range] = $range;
    }
    return $truckAxles;
}

function getTruckTerms(){
    return ['30_days' => '30 Days', '60_days' => '60 Days', '90_days' => '90 Days', '120_days' => '120 Days', '180_days' => '180 Days', '1_year' => '1 year', '2_year' => '2 year', '3_year' => '3 year', '4_year' => '4 year', '5_year' => '5 year'];
}

function getTruckSchedules(){
    return ['1_year' => 'Annual', '30_days' => '30 Days', '60_days' => '60 Days', '90_days' => '90 Days', '120_days' => '120 Days', '180_days' => '180 Days'];
}

// Truck Profiles

function getCompanyInsuranceProfiles()
{
    return CompanyInsuranceProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->get();
}

function getCompanyInsuranceProfilesName()
{
    return CompanyInsuranceProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('profile_name', 'id')->all();
}

function getCompanyPmServiceProfiles()
{
    return CompanyPmServiceProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->get();
}

function getCompanyPmServiceProfilesName()
{
    return CompanyPmServiceProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('profile_name', 'id')->all();
}

function getCompanyPmServiceWarrantyProfiles()
{
    return CompanyPmServiceWarrantyProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->get();
}

function getCompanyPmServiceWarrantyProfilesName()
{
    return CompanyPmServiceWarrantyProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('profile_name', 'id')->all();
}

function getCompanyEquipmentProfiles()
{
    return CompanyEquipmentProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->get();
}

function getCompanyEquipmentProfilesName()
{
    return CompanyEquipmentProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('profile_name', 'id')->all();
}

function getCompanyDocumentProfiles()
{
    return CompanyDocumentProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->get();
}

function getCompanyDocumentProfilesName()
{
    return CompanyDocumentProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('profile_name', 'id')->all();
}

//

function getCompanyPmServiceIntervalType()
{
    return ['1' => 'Mileage', '2' => 'Hours', '3' => 'Date'];
}

function getCompanyTruckTypes()
{
    return CompanyTruckType::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('name', 'id')->all();
}

function getDeductionNameList()
{
    return CompanyDeductionType::IsCompany()->pluck('name', 'id')->all();
}

function getCompanyInsuranceTypes()
{
    return CompanyInsuranceType::where('company_id', auth()->guard('company-admin')->user()->company_id)->pluck('name', 'id')->all();
}

function getCompanyPmType()
{
    return CompanyPmType::IsCompany()->pluck('name', 'id')->all();
}

function getCompanyPmServiceType()
{
    return CompanyPmServiceType::IsCompany()->pluck('name', 'id')->all();
}

function getCompanyPmServiceWarrantyProviders()
{
    return CompanyPmWarrantyProvider::IsCompany()->pluck('name', 'id')->all();
}

function getEquipmentTitles()
{
    return CompanyEquipmentTitle::IsCompany()->pluck('name', 'id')->all();
}

function getDocumentTitles()
{
    return CompanyDocumentTitle::IsCompany()->pluck('name', 'id')->all();
}

function getTrailerGroup()
{
    return CompanyTrailerGroup::IsCompany()->pluck('name', 'id')->all(); // ['0' => 'Dedicated', '1' => 'DOD', '2' => 'Government', '3' => 'Oil & Energy', '4' => 'Produce'];
}

function getTruckGroup()
{
    return CompanyTruckGroup::IsCompany()->pluck('name', 'id')->all(); // ['0' => 'Dedicated', '1' => 'DOD', '2' => 'Government', '3' => 'Oil & Energy', '4' => 'Produce'];
}

//

function getAssignedCustomer()
{
    return ['0' => 'ABC wood company', '1' => 'Dragon Plastics', '2' => 'Georgia Pacific'];
}

function getRequiredClass()
{
    return ['CDL_CA' => 'CDL Class A', 'CDL_CB' => 'CDL Class B', 'CDL_CC' => 'CDL Class C', 'CDL_CD' => 'CDL Class D', 'CDL_CM' => 'CDL Class M'];
}

function getInsideWidth(){
    $array = [];
    for ($i = 102; $i >= 50; $i--) {
        $array[$i] = $i;
    }
    return $array;
}

function getSecurement()
{
    return ['1' => 'LOGISTIC POSTS', '2' => 'ETRACK'];
}

function getTruckTireQty()
{
    $truckTireQty               = [];
    foreach (range(1, 100) as $range){
        $truckTireQty[$range]   = $range;
    }
    return $truckTireQty;
}

function getTruckFuelType()
{
    return ['0' => 'Diesel', '1' => 'Regular Gas', '2' => 'Electric'];
}

function getTruckOwnershipType()
{
    return ['0' => 'Owned By Carrier', '1' => 'Leased By Carrier', '2' => 'Rented By Carrier', '3' => 'Owned By Other', '4' => 'Leased By Other', '5' => 'Rented By Other'];
}

function getTruckLastLocation($truck){
    $last_location      = [];
    if(!is_null($truck)){
        $vehicleIds     = $truck->samsara_id ?? null;
        if(!is_null($vehicleIds)){
            $carrierId  = (isset($truck->carrier_id)) ? $truck->carrier_id : null;
            $getAPIKey                      = IntegrationSetting::where('company_id', $truck->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_vehicles', '1')->first();
            if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
                try {
                    $url                    = 'https://api.samsara.com/fleet/vehicles/locations?vehicleIds='.$vehicleIds;
                    $client                 = new \GuzzleHttp\Client(['verify' => false]);
                    $api_response           = $client->request('GET', $url, [
                        'headers'           => [
                            'Accept'        => 'application/json',
                            'Authorization' => 'Bearer '.$getAPIKey->api_key,
                        ],
                    ]);
                    // Check if response status code is OK (200)
                    if ($api_response->getStatusCode() == 200) {
                        $body               = json_decode($api_response->getBody());
                        // Do something with the response body
                        if(isset($body)){
                            $data           = isset($body->data[0])?$body->data[0]:null;
                            if(isset($data)){
                                if(isset($data->location)){
                                    try {
                                        $location                       = $data->location;
                                        $trimble_api_key                = getTrimbleApiKey();
                                        if(!is_null($trimble_api_key)){
                                            $array                      = [
                                                'Coords'                => $location->longitude.','.$location->latitude,
                                                'matchNamedRoadsOnly'   => 'true',
                                                'maxCleanupMiles'       => '20',
                                                'region'                => 'NA',
                                                'dataset'               => 'Current',
                                                'vehicle'               => 'Truck',
                                                'timestamp'             => Carbon::now(),
                                                'includeTrimblePlaceIds'=> 'true'
                                            ];

                                            $param                      = array();
                                            foreach($array as $k => $v){
                                                $param[]                = $k . '=' . urlencode($v);
                                            }
                                            $param                      = implode('&', $param);

                                            $trimble_url                = 'https://pcmiler.alk.com/apis/rest/v1.0/Service.svc/locations/reverse?'.$param;
                                            $trimble_response           = $client->request('GET', $trimble_url, [
                                                'headers'               => [
                                                    'Accept'            => 'application/json',
                                                    'Content-Type'      => 'application/json',
                                                    'Authorization'     => $trimble_api_key
                                                ],
                                            ]);

                                            // Check if response status code is OK (200)
                                            if ($trimble_response->getStatusCode() == 200) {
                                                $trimble_body           = json_decode($trimble_response->getBody());
                                                if(isset($trimble_body)){
                                                    $last_location      = [
                                                        'StreetAddress' => $trimble_body->Address->StreetAddress,
                                                        'City'          => $trimble_body->Address->City,
                                                        'State'         => $trimble_body->Address->State,
                                                        'Zip'           => $trimble_body->Address->Zip,
                                                        'TimeZone'      => $trimble_body->TimeZone,
                                                        'Lat'           => $trimble_body->Coords->Lat,
                                                        'Lon'           => $trimble_body->Coords->Lon,
                                                        'time'          => Carbon::parse($location->time)->format('Y-m-d H:i:s'),
                                                    ];
                                                }
                                            }
                                        }
                                    } catch (ClientException $e) {
                                        // Handle client-side errors (e.g. 4xx status codes)
                                        $trimble_response               = $e->getResponse();
                                        if ($trimble_response) {
                                            $statusCode                 = $trimble_response->getStatusCode();
                                            $body                       = $trimble_response->getBody()->getContents();
                                            Log::error($body);
                                        } else {
                                            Log::error($e->getMessage());
                                        }
                                    } catch (ServerException $e) {
                                        // Handle server-side errors (e.g. 5xx status codes)
                                        $trimble_response               = $e->getResponse();
                                        if ($response) {
                                            $statusCode                 = $trimble_response->getStatusCode();
                                            $body                       = $trimble_response->getBody()->getContents();
                                            Log::error($body);
                                        } else {
                                            Log::error($e->getMessage());
                                        }
                                    } catch (\Exception $e) {
                                        // Handle other exceptions
                                        Log::error($e->getMessage());
                                    }
                                }
                            }
                        }
                    }
                } catch (ClientException $e) {
                    // Handle server-side errors (e.g. 5xx status codes)
                    $api_response           = $e->getResponse();
                    if ($api_response) {
                        $statusCode         = $api_response->getStatusCode();
                        $body               = $api_response->getBody()->getContents();
                        Log::error($body);
                    } else {
                        Log::error($e->getMessage());
                    }
                } catch (ServerException $e) {
                    // Handle server-side errors (e.g. 5xx status codes)
                    $api_response           = $e->getResponse();
                    if ($api_response) {
                        $statusCode         = $api_response->getStatusCode();
                        $body               = $api_response->getBody()->getContents();
                        Log::error($body);
                    } else {
                        Log::error($e->getMessage());
                    }
                } catch (\Exception $e) {
                    // Handle other exceptions
                    Log::error($e->getMessage());
                }
            }
        }
    }

    if(empty($last_location)){
        if(!is_null($truck)){
            $last_location              = [
                'StreetAddress'         => $truck->last_location_address,
                'City'                  => $truck->last_location_city,
                'State'                 => $truck->last_location_state,
                'Zip'                   => $truck->last_location_zip,
                'TimeZone'              => $truck->last_location_timezone_abbr,
                'Lat'                   => $truck->last_location_latitude,
                'Lon'                   => $truck->last_location_longitude,
                'time'                  => null,
            ];
        }
    }

    return $last_location;
}

function getDriverSamsaraInfo($driver){
    $data                                   = [];
    $samsara_id                             = (isset($driver->samsara_id) && $driver->samsara_id!='')?$driver->samsara_id:'';//4644765
    if($samsara_id!=''){
        $carrierId = (isset($driver->driver_information->carrier_id)) ? $driver->driver_information->carrier_id : null;
        $getAPIKey                          = IntegrationSetting::where('company_id', $driver->company_id)->where('carrier_id', $carrierId)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_drivers', '1')->first();
        if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
            $client                         = new \GuzzleHttp\Client(['verify' => false]);
            $api_url                        = 'https://api.samsara.com/fleet/hos/clocks?driverIds='.$samsara_id;
            $api_response                   = $client->request('GET', $api_url, [
                'headers'                   => [
                    'Accept'                => 'application/json',
                    'Authorization'         => 'Bearer '.$getAPIKey->api_key,
                ],
            ]);

            $api_data                       = json_decode($api_response->getBody());
            if(!empty($api_data)){
                $appData                    = $api_data->data[0];
                if(!empty($appData)){
                    $data['drive']                  = formatSeconds($appData->clocks->drive->driveRemainingDurationMs);
                    $data['shift']                  = formatSeconds($appData->clocks->shift->shiftRemainingDurationMs);
                    $data['cycle']                  = formatSeconds($appData->clocks->cycle->cycleRemainingDurationMs);
                    $data['cycle_tom']              = formatSeconds($appData->clocks->cycle->cycleTomorrowDurationMs);
                    $data['duty_status']            = $appData->currentDutyStatus->hosStatusType;
                    $data['time_in_current_status'] = '';
                    $data['time_in_unit_break']     = formatSeconds($appData->clocks->break->timeUntilBreakDurationMs);
                }
            }
        }
    }
    return $data;
}

function getAttachTrailerList(){
    return Trailer::select('trailer_id', 'current_assigned_truck', DB::raw("CONCAT(last_location_address,' ',last_location_city,', ',last_location_state,' ',last_location_zip) AS last_location"))->isCompany()->get()->toArray();
}

function getCompanyDropLocations(){
    return CompanyDropLocation::select(DB::raw("CONCAT(address,' ',city,', ',state,' ',zip) AS drop_location"), 'id')->IsCompany()->pluck('drop_location', 'id')->all();
}

function getDriversByCompany(){
    return Driver::IsCompany()->join('driver_informations', 'driver_informations.driver_id', '=', 'drivers.id')
        ->select('drivers.id', DB::raw("CONCAT(drivers.last_name,', ',drivers.first_name) AS name"), 'driver_informations.truck_number')
        ->orderByRaw('drivers.last_name ASC, drivers.first_name ASC')
        ->get();
}

function truckUpcomingEvents($truckId) {
    $truck  = Truck::IsCompany()->withTrashed()->where('id', $truckId)->first();
    $events = [];
    if(count($truck->truck_profile_insurances) > 0){
        foreach ($truck->truck_profile_insurances as $truckProfileInsurance) {
            $from_date      = Carbon::parse($truckProfileInsurance->expiration_date)->format('Y-m-d');
            $to_date        = Carbon::parse($truckProfileInsurance->expiration_date)->format('Y-m-d');
            $events[]       = [
                'icon'      => '',
                'title'     => getCompanyInsuranceTypes()[$truckProfileInsurance->insurance_type] ?? '',
                'sub_title' => getCompanyInsuranceProfiles()->pluck('profile_name', 'id')[$truckProfileInsurance->insurance_profile] ?? '',
                'start'     => $from_date,
                'end'       => $to_date
            ];
        }
    }

    if(count($truck->truck_additional_insurances) > 0){
        foreach ($truck->truck_additional_insurances as $truckAdditionalInsurance) {
            $from_date      = Carbon::parse($truckAdditionalInsurance->expiration_date)->format('Y-m-d');
            $to_date        = Carbon::parse($truckAdditionalInsurance->expiration_date)->format('Y-m-d');
            $events[]       = [
                'icon'      => '',
                'title'     => getCompanyInsuranceTypes()[$truckAdditionalInsurance->insurance_type] ?? '',
                'sub_title' => 'Additional Insurance',
                'start'     => $from_date,
                'end'       => $to_date
            ];
        }
    }

    if($truck->tag_exp != ""){
        $from_date          = Carbon::parse($truck->tag_exp)->format('Y-m-d');
        $to_date            = Carbon::parse($truck->tag_exp)->format('Y-m-d');
        $events[]           = [
            'icon'          => '',
            'title'         => 'Tag Exp',
            'sub_title'     => '',
            'start'         => $from_date,
            'end'           => $to_date
        ];
    }

    if($truck->oregon_date != ""){
        $from_date          = Carbon::parse($truck->oregon_date)->format('Y-m-d');
        $to_date            = Carbon::parse($truck->oregon_date)->format('Y-m-d');
        $events[]           = [
            'icon'          => '',
            'title'         => 'Oregon',
            'sub_title'     => '',
            'start'         => $from_date,
            'end'           => $to_date
        ];
    }

    if($truck->exp_date_2290 != ""){
        $from_date          = Carbon::parse($truck->exp_date_2290)->format('Y-m-d');
        $to_date            = Carbon::parse($truck->exp_date_2290)->format('Y-m-d');
        $events[]           = [
            'icon'          => '',
            'title'         => '2290 Exp',
            'sub_title'     => '',
            'start'         => $from_date,
            'end'           => $to_date
        ];
    }

    if($truck->nyhut_date != ""){
        $from_date          = Carbon::parse($truck->nyhut_date)->format('Y-m-d');
        $to_date            = Carbon::parse($truck->nyhut_date)->format('Y-m-d');
        $events[]           = [
            'icon'          => '',
            'title'         => 'NYHUT Exp',
            'sub_title'     => '',
            'start'         => $from_date,
            'end'           => $to_date
        ];
    }

    if($truck->kyu_due_date != ""){
        $from_date          = Carbon::parse($truck->kyu_due_date)->format('Y-m-d');
        $to_date            = Carbon::parse($truck->kyu_due_date)->format('Y-m-d');
        $events[]           = [
            'icon'          => '',
            'title'         => 'KYU Exp',
            'sub_title'     => '',
            'start'         => $from_date,
            'end'           => $to_date
        ];
    }

    if($truck->carb_id_date != ""){
        $from_date          = Carbon::parse($truck->carb_id_date)->format('Y-m-d');
        $to_date            = Carbon::parse($truck->carb_id_date)->format('Y-m-d');
        $events[]           = [
            'icon'          => '',
            'title'         => 'Carb Exp',
            'sub_title'     => '',
            'start'         => $from_date,
            'end'           => $to_date
        ];
    }

    if(count($truck->truck_profile_pm_services) > 0){
        foreach ($truck->truck_profile_pm_services as $truckProfilePmService) {
            if(DateTime::createFromFormat('Y-m-d H:i:s', $truckProfilePmService->due_date) !== false){
                $from_date  = Carbon::parse($truckProfilePmService->due_date)->format('Y-m-d');
                $to_date    = Carbon::parse($truckProfilePmService->due_date)->format('Y-m-d');
                $events[]   = [
                    'icon'  => '',
                    'title' => getCompanyPmServiceType()[$truckProfilePmService->service_type] ?? '',
                    'sub_title' => getCompanyPmServiceProfiles()->pluck('profile_name', 'id')[$truckProfilePmService->pm_service_profile] ?? '',
                    'start' => $from_date,
                    'end'   => $to_date
                ];
            }
        }
    }

    if(count($truck->truck_additional_pm_services) > 0){
        foreach ($truck->truck_additional_pm_services as $truckAdditionalPmService) {
            if(DateTime::createFromFormat('Y-m-d H:i:s', $truckAdditionalPmService->due_date) !== false) {
                $from_date  = Carbon::parse($truckAdditionalPmService->due_date)->format('Y-m-d');
                $to_date    = Carbon::parse($truckAdditionalPmService->due_date)->format('Y-m-d');
                $events[]   = [
                    'icon'  => '',
                    'title' => getCompanyPmServiceType()[$truckAdditionalPmService->service_type] ?? '',
                    'sub_title' => 'Additional PM Maintenance & Service',
                    'start' => $from_date,
                    'end'   => $to_date
                ];
            }
        }
    }

    if(count($truck->truck_warranty_pm_services) > 0){
        foreach ($truck->truck_warranty_pm_services as $truckWarrantyPmService) {
            if(DateTime::createFromFormat('Y-m-d H:i:s', $truckWarrantyPmService->warranty_exp) !== false) {
                $from_date  = Carbon::parse($truckWarrantyPmService->warranty_exp)->format('Y-m-d');
                $to_date    = Carbon::parse($truckWarrantyPmService->warranty_exp)->format('Y-m-d');
                $events[]   = [
                    'icon'  => '',
                    'title' => getCompanyPmServiceWarrantyProviders()[$truckWarrantyPmService->warranty_provider] ?? '',
                    'sub_title' => 'Warranty PM Maintenance & Service',
                    'start' => $from_date,
                    'end'   => $to_date
                ];
            }
        }
    }

    if(count($truck->truck_profile_documents) > 0){
        foreach ($truck->truck_profile_documents as $truckProfileDocument) {
            $from_date      = Carbon::parse($truckProfileDocument->exp_date)->format('Y-m-d');
            $to_date        = Carbon::parse($truckProfileDocument->exp_date)->format('Y-m-d');
            $events[]       = [
                'icon'      => '',
                'title'     => getDocumentTitles()[$truckProfileDocument->title] ?? '',
                'sub_title' => getCompanyDocumentProfiles()->pluck('profile_name', 'id')[$truckProfileDocument->document_profile] ?? '',
                'start'     => $from_date,
                'end'       => $to_date
            ];
        }
    }

    if(count($truck->truck_additional_documents) > 0){
        foreach ($truck->truck_additional_documents as $truckAdditionalDocument) {
            $from_date      = Carbon::parse($truckAdditionalDocument->exp_date)->format('Y-m-d');
            $to_date        = Carbon::parse($truckAdditionalDocument->exp_date)->format('Y-m-d');
            $events[]       = [
                'icon'      => '',
                'title'     => getDocumentTitles()[$truckAdditionalDocument->title] ?? '',
                'sub_title' => 'Additional On Board Documents',
                'start'     => $from_date,
                'end'       => $to_date
            ];
        }
    }

    return $events;
}

function concateAddress($Obj) {
    $address        = [];
    if($Obj->address){
        $address[]  = $Obj->address;
    }
    if($Obj->address_2){
        $address[]  = $Obj->address_2;
    }
    if($Obj->city){
        $address[]  = $Obj->city;
    }
    if($Obj->state){
        $address[]  = $Obj->state;
    }
    if($Obj->zip){
        $address[]  = $Obj->zip;
    }
    if(count($address) > 0){
        return implode(',', $address);
    }else{
        return '';
    }
}

function getTruckTagType(){
    return ['1' => 'Interstate', '2' => 'Intrastate'];
}

function truckTotalActiveLoads($startDate, $endDate, $truckNumber){
    return LoadDetail::select('*')
        ->where('truck_number', $truckNumber)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('is_disable', 0)
        ->where(function($query) {
            $query->whereNotIn('load_status',[5,6,7,8,9,10])
                    ->orWhere(function($q) {
                        $q->whereNull('driver_id')->orWhere('driver_id', 0);
                    });
        })
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
        })
        ->whereNull('deleted_at')
        ->whereHas('loads', function($q){
            $q->where('loads.company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })->count();
}

function truckTotalReadyToBillLoads($startDate, $endDate, $truckNumber){
    return LoadDetail::select('*')
        ->where('truck_number', $truckNumber)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('is_disable', 0)
        ->whereIn('load_status', [6,10])
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
        })
        ->where(function($q) {
            $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })->count();
}

function truckTotalDeliveredLoads($startDate, $endDate, $truckNumber) {
    return LoadDetail::select('*')
        ->where('truck_number', $truckNumber)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('is_disable', 0)
        ->where('load_details.load_status', 5)
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
        })
        ->where(function($q) {
            $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })->count();
}

function truckTotalCompletedLoads($startDate, $endDate, $truckNumber){
    return LoadDetail::select('*')
        ->where('truck_number', $truckNumber)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('is_disable', 0)
        ->where('load_details.load_type', 0) //->whereIn('load_type', [0,6,4]) // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
        ->where('load_status', 8)
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
        })
        ->where(function($q) {
            $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })->count();
}

function truckTotalOnHoldLoads($startDate, $endDate, $truckNumber) {
    return LoadDetail::select('*')
        ->where('truck_number', $truckNumber)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('is_disable', 0)
        ->where(function($q) {
            $q->where('load_details.load_status', 7)->orWhere('load_details.is_on_hold', 1);
        })
        ->where(function($q) {
            $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })->count();
}

function truckTotalCancelledLoads($startDate, $endDate, $truckNumber) {
    return LoadDetail::select('*')
        ->where('truck_number', $truckNumber)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('is_disable', 0)
        ->where('load_details.load_type', 0) // 0 = OCCUPIED
        ->where(function($q) {
            $q->where('load_details.load_status', 9)->orWhere('load_details.is_on_hold', 2);
        })
        ->where(function($q) {
            $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })->count();
}

function truckTotalLoads($startDate, $endDate, $truckNumber){
    return LoadDetail::whereHas('loads', function ($q) {
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
            })
            ->where('truck_number', $truckNumber)
            ->where(function ($query) use ($endDate, $startDate) {
                $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                    ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                    ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                    ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
            })
            ->where('load_details.is_disable', 0)
            ->where('load_details.load_status', '!=', 9)
            ->where(function($q) {
                $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
            })
            ->whereNull('load_details.deleted_at')
            ->count();
}

function truckCargoClaimsLoads($startDate, $endDate, $truckNumber){
    return 0;
}

function getLoadCountByTruckId($startDate, $endDate, $truckNumber){
    $data = LoadDetail::select("load_details.load_id")
        ->join("trucks", "trucks.unit", "=", "load_details.truck_number")
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where("load_details.truck_number", $truckNumber)
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNull('load_details.deleted_at')
        ->count();

    return $data;
}

function getLoadTotalMileageByTruck($startDate, $endDate, $truckNumber){
    $totalMiles = Truck::join("load_details", "load_details.truck_number", "=", "trucks.unit")
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where("trucks.unit", $truckNumber)
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->whereNull('load_details.deleted_at')
        ->sum("load_details.total_miles");

    return $totalMiles;
}

function getLoadGrossRevenueBYTruck($startDate, $endDate, $truckNumber){
    $totalRevenue = Truck::join("load_details", "load_details.truck_number", "=", "trucks.unit")
        ->where(function ($query) use ($endDate, $startDate) {
            if($startDate != "" && $endDate != ""){
                $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                    ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                    ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                    ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
            }
        })
        ->where("trucks.unit", $truckNumber)
        ->whereNotIn('load_details.load_type', [1, 2, 3, 5]) // 1 = MAINTENANCE, 2 = UNAUTHORIZED, 3 = HOS, 5 = TIME OFF
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNull('load_details.deleted_at')
        ->sum("load_details.driver_revenue");
    return $totalRevenue;
}

function getTruckDiagnosticsData($truck){
    $diagnosticsData                    = [];
    $responseData                       = [];

    $diagnosticTypes                    = [
        'gps', 'engineStates', 'fuelPercents',
        'defLevelMilliPercent', 'batteryMilliVolts', 'obdOdometerMeters',
        'obdEngineSeconds', 'engineCoolantTemperatureMilliC', 'engineOilPressureKPa',
        'ecuSpeedMph', 'engineLoadPercent', 'barometricPressurePa',
        'ambientAirTemperatureMilliC', 'intakeManifoldTemperatureMilliC', 'engineImmobilizer',
        'seatbeltDriver'
    ];

    $getAPIKey                          = IntegrationSetting::where('company_id', $truck->company_id)->where('carrier_id', $truck->carrier_id)->where('app_center_id', config('settings.app_id.samsara'))->where('sync_vehicles', '1')->first();

    if(isset($getAPIKey->api_key) && !empty($getAPIKey->api_key)){
        foreach (array_chunk($diagnosticTypes, 3) as $diagnosticType){
            try {
                $url                    = 'https://api.samsara.com/fleet/vehicles/stats?vehicleIds='.$truck->samsara_id.'&types='.implode(',', $diagnosticType);
                $client                 = new \GuzzleHttp\Client(['verify' => false]);
                $response               = $client->request('GET', $url, [
                    'headers' => [
                        'Accept' => 'application/json',
                        'Authorization' => 'Bearer '.$getAPIKey->api_key,
                    ],
                ]);
                // Check if response status code is OK (200)
                if ($response->getStatusCode() == 200) {
                    $body               = json_decode($response->getBody());
                    // Do something with the response body
                    if(isset($body)){
                        $data           = isset($body->data[0]) ? $body->data[0] : null;

                        if(isset($data)){
                            if(isset($data->gps)){
                                $responseData['speedMilesPerHour']              = array(
                                    'time'  => Carbon::parse($data->gps->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => round($data->gps->speedMilesPerHour).' mph' ?? ''
                                );
                            }
                            if(isset($data->engineState)){
                                $responseData['engineStates']                   = array(
                                    'time'  => Carbon::parse($data->engineState->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => $data->engineState->value ?? ''
                                );
                            }
                            if(isset($data->fuelPercent)){
                                $responseData['fuelPercents']                   = array(
                                    'time'  => Carbon::parse($data->fuelPercent->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => round($data->fuelPercent->value) ?? ''
                                );
                            }
                            if(isset($data->defLevelMilliPercent)){
                                $responseData['defLevelMilliPercent']           = array(
                                    'time'  => Carbon::parse($data->defLevelMilliPercent->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => round(($data->defLevelMilliPercent->value * 0.001)) ?? ''
                                );
                            }
                            if(isset($data->batteryMilliVolts)){
                                $millivoltsTovolts                              = ($data->batteryMilliVolts->value > 0) ? round(($data->batteryMilliVolts->value / 1000), 1) : 0;
                                $responseData['batteryMilliVolts']              = array(
                                    'time'  => Carbon::parse($data->batteryMilliVolts->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => $millivoltsTovolts.' V' ?? ''
                                );
                            }
                            if(isset($data->obdOdometerMeters)){
                                $responseData['obdOdometerMeters']              = array(
                                    'time'              => Carbon::parse($data->obdOdometerMeters->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value'             => number_format(round($data->obdOdometerMeters->value * 0.000621371)).' mi' ?? '',
                                    'original_value'    => round(($data->obdOdometerMeters->value * 0.000621371))
                                );
                            }
                            if(isset($data->obdEngineSeconds)){
                                $engineHours                                    = ($data->obdEngineSeconds->value > 0) ? number_format(($data->obdEngineSeconds->value / 3600)) : 0;
                                $responseData['obdEngineSeconds']               = array(
                                    'time'              => Carbon::parse($data->obdEngineSeconds->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value'             => $engineHours.' hours' ?? '',
                                    'original_value'    => ($data->obdEngineSeconds->value > 0) ? round(($data->obdEngineSeconds->value / 3600)) : 0
                                );
                            }
                            if(isset($data->engineCoolantTemperatureMilliC)){
                                $coolantTemperature                             = ($data->engineCoolantTemperatureMilliC->value > 0) ? round(($data->engineCoolantTemperatureMilliC->value * 0.0018), 1) : 0;
                                $responseData['engineCoolantTemperatureMilliC'] = array(
                                    'time'  => Carbon::parse($data->engineCoolantTemperatureMilliC->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => $coolantTemperature.'°F' ?? ''
                                );
                            }
                            if(isset($data->engineOilPressureKPa)){
                                $engineOilPressure                              = ($data->engineOilPressureKPa->value > 0) ? round(($data->engineOilPressureKPa->value / 6.895), 2) : 0;
                                $responseData['engineOilPressureKPa']           = array(
                                    'time'  => Carbon::parse($data->engineOilPressureKPa->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => $engineOilPressure.' PSI' ?? ''
                                );
                            }
                            if(isset($data->ecuSpeedMph)){
                                $responseData['ecuSpeedMph']                    = array(
                                    'time'  => Carbon::parse($data->ecuSpeedMph->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => round($data->ecuSpeedMph->value).' mph' ?? ''
                                );
                            }
                            if(isset($data->engineLoadPercent)){
                                $responseData['engineLoadPercent']              = array(
                                    'time'  => Carbon::parse($data->engineLoadPercent->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => $data->engineLoadPercent->value.'%' ?? ''
                                );
                            }
                            if(isset($data->barometricPressurePa)){
                                $responseData['barometricPressurePa']           = array(
                                    'time'  => Carbon::parse($data->barometricPressurePa->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => round(($data->barometricPressurePa->value * 0.000145038), 2).' PSI' ?? ''
                                );
                            }
                            if(isset($data->seatbeltDriver)){
                                $responseData['seatbeltDriver']                 = array(
                                    'time'  => Carbon::parse($data->seatbeltDriver->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => $data->seatbeltDriver->value ?? ''
                                );
                            }
                            if(isset($data->ambientAirTemperatureMilliC)){
                                $ambientAirTemperature = ($data->ambientAirTemperatureMilliC->value > 0) ? round(($data->ambientAirTemperatureMilliC->value * 0.0018), 1) : 0;
                                $responseData['ambientAirTemperatureMilliC']    = array(
                                    'time'  => Carbon::parse($data->ambientAirTemperatureMilliC->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => $ambientAirTemperature.'°F' ?? ''
                                );
                            }
                            if(isset($data->intakeManifoldTemperatureMillic)){
                                $intakeManifoldTemperature                      = ($data->intakeManifoldTemperatureMillic->value > 0) ? round(($data->intakeManifoldTemperatureMillic->value * 0.0018), 1) : 0;
                                $responseData['intakeManifoldTemperatureMilliC'] = array(
                                    'time'  => Carbon::parse($data->intakeManifoldTemperatureMillic->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => $intakeManifoldTemperature.'°F' ?? ''
                                );
                            }
                            if(isset($data->engineImmobilizer)){
                                $engineImmobilizer      = '';
                                if($data->engineImmobilizer->state == "ignition_enabled"){
                                    $engineImmobilizer  = 'Enabled';
                                }else if($data->engineImmobilizer->state == "ignition_disabled"){
                                    $engineImmobilizer  = 'Disabled';
                                }

                                $responseData['engineImmobilizer']              = array(
                                    'time' => Carbon::parse($data->engineImmobilizer->time)->diffForHumans(['parts' => 2]) ?? '',
                                    'value' => $engineImmobilizer,
                                );
                            }
                        }else{
                            Log::error('Truck vin number ('.$truck->vin_number.') : Samsara response data not found.');
                        }
                    }
                }
            } catch (\GuzzleHttp\Exception\ClientException $e) {
                // Handle client-side errors (e.g. 4xx status codes)
                $response               = $e->getResponse();
                if ($response) {
                    $statusCode         = $response->getStatusCode();
                    $body               = $response->getBody()->getContents();
                    Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : client exception '.$body);
                } else {
                    $errorMessage       = $e->getMessage();
                    Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : client exception '.$errorMessage);
                }
            } catch (\GuzzleHttp\Exception\ServerException $e) {
                // Handle server-side errors (e.g. 5xx status codes)
                $response               = $e->getResponse();
                if ($response) {
                    $statusCode         = $response->getStatusCode();
                    $body               = $response->getBody()->getContents();
                    Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : server exception '.$body);
                } else {
                    $errorMessage       = $e->getMessage();
                    Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : server exception '.$errorMessage);
                }
            } catch (\Exception $e) {
                // Handle other exceptions
                $errorMessage           = $e->getMessage();
                Log::error('Truck vin number ('.$truck->vin_number ?? '-'.') : other exception '.$errorMessage);
            }
        }
    }

    foreach ($diagnosticTypes as $diagnosticType){
        if($diagnosticType == 'gps'){
            $diagnosticsData['speedMilesPerHour']   = $responseData['speedMilesPerHour'] ?? '';
        }else{
            $diagnosticsData[$diagnosticType]       = $responseData[$diagnosticType] ?? '';
        }
    }

    return $diagnosticsData;
}

function getCompanyAdminNameById($id){
    return CompanyAdmin::Company()->find($id)->fullname ?? null;
}

function getDispatcherAndLoadDriver($driver = null, $coDriver = null){
    $loadDrivers = [];
    if(!is_null($driver) && $driver != 0){
        //$driver = Driver::IsCompany()->findOrFail($driver)->first();
        $driver = Driver::IsCompany()->find($driver);

        if($driver){
            $loadDrivers[$driver->id] = $driver->first_name.' '.$driver->last_name;
        }
    }
    if(!is_null($coDriver) && $coDriver != 0){
        //$coDriver = Driver::IsCompany()->findOrFail($coDriver)->first();
        $coDriver = Driver::IsCompany()->find($coDriver);

        if($coDriver){
            $loadDrivers[$coDriver->id] = $coDriver->first_name.' '.$coDriver->last_name;
        }
    }

    $dispatchers = CompanyAdmin::select(DB::raw("CONCAT(first_name,' ',last_name) AS name"), 'id')
                    ->where('company_id', auth()->guard('company-admin')->user()->company_id)
                    ->pluck('name', 'id')
                    ->all();

    $notifyUserList = [];
    foreach ($dispatchers as $key => $val){
        $notifyUserList['disp-'.$key] = $val;
    }
    foreach ($loadDrivers as $key => $val){
        $notifyUserList['driv-'.$key] = $val;
    }

    return $notifyUserList;
}

function sendOnHoldDropTrailerNotification($notifyUsers, $data)
{
    foreach($notifyUsers as $notifyUser){
        if(Str::beforeLast($notifyUser, '-') === "disp"){
            $dispatcherId = Str::afterLast($notifyUser, '-');

            $dispatcherData = CompanyAdmin::find($dispatcherId);

            if($dispatcherData->email != ""){
                $mailData = [];
                $mailData['company_id'] = $dispatcherData->company_id ?? '';
                $mailData['email_to'] = $dispatcherData->email ?? '';
                $mailData['name'] = $dispatcherData->full_name ?? '';
                $mailData['load_number'] = $data['load_number'] ?? '';

                if(auth()->guard('company-admin')->check()){
                    sendEmail($mailData['email_to'],new LoadOnHoldFromDropTrailer($mailData));
                }else{
                    Mail::to($mailData['email_to'])->send(new LoadOnHoldFromDropTrailer($mailData));
                }
            }
        }else if(Str::beforeLast($notifyUser, '-') === "driv"){
            $driverId = Str::afterLast($notifyUser, '-');

            $driverData = Driver::find($driverId);

            if($driverData->email != ""){
                $mailData = [];
                $mailData['company_id'] = $driverData->company_id ?? '';
                $mailData['email_to'] = $driverData->email ?? '';
                $mailData['name'] = $driverData->full_name ?? '';
                $mailData['load_number'] = $data['load_number'] ?? '';

                if(auth()->guard('company-admin')->check()){
                    sendEmail($mailData['email_to'],new LoadOnHoldFromDropTrailer($mailData));
                }else{
                    Mail::to($mailData['email_to'])->send(new LoadOnHoldFromDropTrailer($mailData));
                }
            }
        }
    }
}

function sendDroppedDropTrailerNotification($notifyUsers, $data)
{
    foreach($notifyUsers as $notifyUser){
        if(Str::beforeLast($notifyUser, '-') === "disp"){
            $dispatcherId = Str::afterLast($notifyUser, '-');

            $dispatcherData = CompanyAdmin::find($dispatcherId);

            if($dispatcherData->email != ""){
                $mailData = [];
                $mailData['company_id'] = $dispatcherData->company_id ?? '';
                $mailData['email_to'] = $dispatcherData->email ?? '';
                $mailData['name'] = $dispatcherData->full_name ?? '';
                $mailData['load_number'] = $data['load_number'] ?? '';

                if(auth()->guard('company-admin')->check()){
                    sendEmail($mailData['email_to'],new LoadDroppedFromDropTrailer($mailData));
                }else{
                    Mail::to($mailData['email_to'])->send(new LoadDroppedFromDropTrailer($mailData));
                }
            }
        }
    }
}

function truckStatus($page = ''){
    if($page == 'add'){
        return ['0' => 'Active', '1' => 'OOS', '3' => 'Rehire'];
    }else{
        return ['0' => 'Active', '1' => 'OOS', '2' => 'Terminate', '3' => 'Rehire'];
    }
}

function getCustomizedTruckNumberList(){
    $truckDetails = Truck::isCompany()->select('unit', 'driver', 'status')->get();

    $truckListArray = [];
    foreach($truckDetails as $truckDetail){
        $truckListArray[$truckDetail->unit] = array(
            'driver' => ($truckDetail->driver_details) ? $truckDetail->driver_details->last_name.' '.$truckDetail->driver_details->first_name : 'Unassigned',
            'status' => truckStatus()[$truckDetail->status] ?? ''
        );
    }

    return $truckListArray;
}

function getCustomizedTrailerList(){
    $trailerDetails = Trailer::isCompany()->select('trailer_id', 'current_assigned_driver', 'status')->get();

    $trailerListArray = [];
    foreach($trailerDetails as $trailerDetail){
        $trailerListArray[$trailerDetail->trailer_id] = array(
            'driver' => ($trailerDetail->driver) ? $trailerDetail->driver->last_name.' '.$trailerDetail->driver->first_name : 'Unassigned',
            'status' => getTrailerStatusList()[$trailerDetail->status] ?? ''
        );
    }

    return $trailerListArray;
}

function adjustTableColumnsSorting($dbFields, $configFields): array
{
    /*
     * This function use for Table Bookmark(DB Columns) & New added columns merge and view on Adjust Table Columns popup
     * Fixes sorting issue & DP-100 - Add newly created columns in columns settings for each table.
     */
    $dbFieldsKeys = array_keys($dbFields);
    $configFieldsKeys = array_keys($configFields);

    foreach ($configFieldsKeys as $key =>$value){
        if(!in_array($value, $dbFieldsKeys)){
            array_splice( $dbFieldsKeys, $key, 0, $value);
        }
    }

    $updatedDbColumnSorting = [];
    foreach ($dbFieldsKeys as $elementKey){
        if(array_key_exists($elementKey, $dbFields)){
            $updatedDbColumnSorting[$elementKey] = $dbFields[$elementKey];
        }else{
            $updatedDbColumnSorting[$elementKey] = $configFields[$elementKey];
        }
    }

    return $updatedDbColumnSorting;
}

function sendEmail($emailTo, $mailable)
{
    try {
        if(is_array($emailTo) && count($emailTo) > 0){
            $mail = Mail::to($emailTo['email']);
            if(isset($emailTo['cc'])){
                $mail->cc($emailTo['cc']);
            }
            if(isset($emailTo['bcc'])){
                $mail->bcc($emailTo['bcc']);
            }
            $mail->send($mailable);
        }else{
            Mail::to($emailTo)->send($mailable);
        }
    } catch (\Swift_TransportException $e) {
        // Log the error message
        Log::error('Email sending failed: ' . $e->getMessage());

        // Usage listing page database email Log
        EmailLog::create([
            'company_id' => auth()->guard('company-admin')->user()->company_id ?? null,
            'user_id' => auth()->guard('company-admin')->user()->id ?? null,
            'from' => env('MAIL_FROM_ADDRESS') ?? null,
            'to' => $emailTo ?? null,
            'subject' => $mailable->subject ?? null,
            'content' => $e->getMessage() ?? null,
            'sent_date' => Carbon::now()->format('Y-m-d H:i:s'),
            'status' => '0',
        ]);
    } catch (\Exception $e) {
        // Log the error message
        Log::error('An error occurred while sending the email: ' . $e->getMessage());

        // Usage listing page database email Log
        EmailLog::create([
            'company_id' => auth()->guard('company-admin')->user()->company_id ?? null,
            'user_id' => auth()->guard('company-admin')->user()->id ?? null,
            'from' => env('MAIL_FROM_ADDRESS') ?? null,
            'to' => $emailTo ?? null,
            'subject' => $mailable->subject ?? null,
            'content' => $e->getMessage() ?? null,
            'sent_date' => Carbon::now()->format('Y-m-d H:i:s'),
            'status' => '0',
        ]);
    }
}

function getCompanyAdminAnnouncements()
{
    $alerts = [];
    $timeZone     = getTimeZone();
    $offset       = getTimeZoneOffset();
    $currentDateTime = Carbon::now()->timezone($timeZone);

    $announcementClear = AnnouncementClearByUser::where('company_id', auth()->guard('company-admin')->user()->company_id)
        ->where('user_type', auth()->guard('company-admin')->user()->type)
        ->where('user_id', auth()->guard('company-admin')->user()->id)
        ->pluck('announcement_id')->toArray();

    $announcements = Announcement::query()
        ->select('*', DB::raw('CONVERT_TZ(announcements.created_at,"+00:00","'.$offset.'") as created_at'))
        ->where('module','Compel')
        ->where('show_on_top','1')
        ->where(function($query) {
            if(auth()->guard('company-admin')->user()->type == 0 || auth()->guard('company-admin')->user()->type == 1){
                $query->where(function($q) {
                    $q->whereNull('company_id')->whereRaw('FIND_IN_SET(1,show_up)');
                })->orWhere(function($q) {
                    $q->whereNotNUll('company_id')
                        ->whereRaw('FIND_IN_SET(1,show_up)')
                        ->whereRaw('FIND_IN_SET("'.auth()->guard('company-admin')->user()->company_id.'",company_id)');
                });
            }else if(auth()->guard('company-admin')->user()->type == 2) {
                $query->where(function($q) {
                    $q->whereNull('company_id')->whereRaw('FIND_IN_SET(2,show_up)');
                })->orWhere(function($q) {
                    $q->whereNotNUll('company_id')
                        ->whereRaw('FIND_IN_SET(2,show_up)')
                        ->whereRaw('FIND_IN_SET("'.auth()->guard('company-admin')->user()->company_id.'",company_id)');
                });
            }
        })->orderBy('created_at', 'DESC')->get();

    foreach ($announcements as $announcement){
        if($announcement->sticky == '1'){
            if($currentDateTime->greaterThanOrEqualTo($announcement->start_view) && $currentDateTime->lessThanOrEqualTo($announcement->end_view)){
                $alerts[] = $announcement;
            }
        }else{
            if($currentDateTime->greaterThanOrEqualTo($announcement->start_view) && !in_array($announcement->id, $announcementClear)){
                $alerts[] = $announcement;
            }
        }
    }

    return $alerts;
}

function getDriverAdminAnnouncements()
{
    $timeZone     = getTimeZone();
    $offset       = getTimeZoneOffset();
    $currentDateTime = Carbon::now()->timezone($timeZone);

    $alerts = [];

    $announcementClear = AnnouncementClearByUser::where('company_id', auth()->guard('driver-admin')->user()->company_id)
        ->where('user_type', auth()->guard('driver-admin')->user()->type)
        ->where('user_id', auth()->guard('driver-admin')->user()->id)
        ->pluck('announcement_id')->toArray();

    $announcements = Announcement::query()
        ->select('*', DB::raw('CONVERT_TZ(announcements.created_at,"+00:00","'.$offset.'") as created_at'))
        ->where('module','Compel')
        ->where('show_on_top','1')
        ->where(function($query) {
            $query->where(function($q) {
                $q->whereNull('company_id')->whereRaw('FIND_IN_SET(3,show_up)');
            })->orWhere(function($q) {
                $q->whereNotNUll('company_id')
                    ->whereRaw('FIND_IN_SET(3,show_up)')
                    ->whereRaw('FIND_IN_SET("'.auth()->guard('driver-admin')->user()->company_id.'",company_id)');
            });
        })->orderBy('created_at', 'DESC')->get();

    foreach ($announcements as $announcement){
        if($announcement->sticky == '1'){
            if($currentDateTime->greaterThanOrEqualTo($announcement->start_view) && $currentDateTime->lessThanOrEqualTo($announcement->end_view)){
                $alerts[] = $announcement;
            }
        }else{
            if($currentDateTime->greaterThanOrEqualTo($announcement->start_view) && !in_array($announcement->id, $announcementClear)){
                $alerts[] = $announcement;
            }
        }
    }

    return $alerts;
}

function getCompanyEmailTemplate($companyEmailTemplateDetails)
{
    $companyId = $companyEmailTemplateDetails->company_id;

    $companyDetails = Company::findOrFail($companyId);

    $companyAddress = [];
    if($companyDetails->address != ""){
        $companyAddress[] = $companyDetails->address;
    }
    if($companyDetails->address_2 != ""){
        $companyAddress[] = $companyDetails->address_2;
    }
    if($companyDetails->city != ""){
        $companyAddress[] = $companyDetails->city;
    }
    if($companyDetails->state != ""){
        $companyAddress[] = $companyDetails->state;
    }
    if($companyDetails->zip != ""){
        $companyAddress[] = $companyDetails->zip;
    }

    $companyEmailLogo = '<a href="'.route('company.admin.login').'" style="display: inline-block;" target="_blank"><img src="'.asset('theme/img/logo-new.png').'" class="logo" alt="DispatchPro Logo"></a>';
    if($companyDetails->email_logo != ''){
        $logo = asset('storage/companyAdmin/company_email').'/'.$companyDetails->email_logo;
        $companyEmailLogo = '<a href="'.route('company.admin.login').'" style="display: inline-block;" target="_blank"><img src="'.$logo.'" class="logo" alt="DispatchPro Logo"></a>';
    }

    $emailBodyContent = $companyEmailTemplateDetails->content ?? '';
    $socialLink = '';
    $companyName = $companyDetails->name ?? '';
    $companyAddress = (count($companyAddress) > 0) ? $companyName.' - '.implode(', ', $companyAddress) : '';

    $companyEmailWrapperTemplate = CompanyEmailWrapperTemplate::where('company_id', $companyId)->first();

    if(!is_null($companyEmailWrapperTemplate)){
        $companyEmailWrapperTemplate = $companyEmailWrapperTemplate->content;
    }else{
        $companyEmailWrapperTemplate = view('companyAdmin.emailTemplate.email-wrapper')->render();
    }

    return str_replace(
        ['{%COMPANY_LOGO%}', '{%EMAIL_BODY%}', '{%SOCIAL_LINKS%}', '{%COMPANY_NAME%}', '{%COMPANY_ADDRESS%}'],
        [$companyEmailLogo, $emailBodyContent, $socialLink, $companyName, $companyAddress],
        $companyEmailWrapperTemplate
    );
}

function getDefaultFuelSurcharges(){
    $record                                                     = null;
    if (auth()->guard('company-admin')->check()) {
        $record                                                 = DefaultFuelSurcharge::IsCompany()->first();
        if(!is_null($record)){
            if(isset($record->default_fuel_surcharge_diesel) && !is_null($record->default_fuel_surcharge_diesel)){
                $record->default_fuel_surcharge_diesel          = number_format($record->default_fuel_surcharge_diesel, '2', '.', ',');
            }
            if(isset($record->default_fuel_surcharge_gas) && !is_null($record->default_fuel_surcharge_gas)){
                $record->default_fuel_surcharge_gas             = number_format($record->default_fuel_surcharge_gas, '2', '.', ',');
            }
            $dieselTemplate                                     = FuelSurchargeTemplate::IsCompany()->where('customer', 'FSD')->first();
            if(!is_null($dieselTemplate)){
                $record->default_fuel_surcharge_diesel_id       = $dieselTemplate->id;
            }
            $gasTemplate                                        = FuelSurchargeTemplate::IsCompany()->where('customer', 'FSG')->first();
            if(!is_null($gasTemplate)){
                $record->default_fuel_surcharge_gas_id          = $gasTemplate->id;
            }
        }
    }
    return $record;
}

function getRegionWiseCurrentRate($fuelType, $regionName, $currentWeekMonday){
    $eiaApiKey                              = env('EIA_API_KEY');
    $fuelSurchargeRate                      = '';
    if(isset($eiaApiKey) && $eiaApiKey != ''){
        $regionName                         = strtoupper(Str::between($regionName, '(', ')'));
        try {
            $url                            = 'https://api.eia.gov/v2/petroleum/pri/gnd/data/?api_key='.$eiaApiKey.'&frequency=weekly&data[0]=value&facets[product][]='.$fuelType.'&start='.$currentWeekMonday.'&end='.$currentWeekMonday;
            $client                         = new \GuzzleHttp\Client(['verify' => false]);
            $api_response                   = $client->request('GET', $url, [
                'headers'                   => [
                    'Accept'                => 'application/json'
                ]
            ]);
            // Check if response status code is OK (200)
            if ($api_response->getStatusCode() == 200) {
                $body                       = json_decode($api_response->getBody());
                // Do something with the response body
                if(isset($body) && isset($body->response->data) && !empty($body->response->data)){
                    $data                   = $body->response->data;
                    $record                 = $data[array_keys(array_column($data, 'area-name'),$regionName)[0]] ?? '';
                    if(isset($record)){
                        $fuelSurchargeRate  = round($record->value, 2) ?? '';
                    }
                }
            }
        } catch (\GuzzleHttp\Exception\ClientException $e) {
            // Handle server-side errors (e.g. 5xx status codes)
            $api_response                   = $e->getResponse();
            if ($api_response) {
                $statusCode                 = $api_response->getStatusCode();
                $body                       = $api_response->getBody()->getContents();
                Log::error($body);
            } else {
                Log::error($e->getMessage());
            }
        } catch (\GuzzleHttp\Exception\ServerException $e) {
            // Handle server-side errors (e.g. 5xx status codes)
            $api_response                   = $e->getResponse();
            if ($api_response) {
                $statusCode                 = $api_response->getStatusCode();
                $body                       = $api_response->getBody()->getContents();
                Log::error($body);
            } else {
                Log::error($e->getMessage());
            }
        } catch (\Exception $e) {
            // Handle other exceptions
            Log::error($e->getMessage());
        }
    }
    return $fuelSurchargeRate;
}

function saveDefaultFuelSurcharges($template){
    $currentWeekMonday                      = Carbon::now()->startOfWeek()->format('Y-m-d');
    $currentRegionRate                      = getRegionWiseCurrentRate($template->fuel_type, $template->region, $currentWeekMonday);
    if($currentRegionRate != ''){
        $startPrice                         = round($template->start_price, 2);
        $endingPrice                        = round($template->ending_price, 2);
        $priceChange                        = round($template->price_change, 2);

        $fuelArray                          = array();
        if($template->rate_type == "1"){
            $basePercentage                 = round($template->base_percentage, 2);
            $percentageChange               = round($template->percentage_change, 2);
            for ($i = $startPrice, $l = $endingPrice; $i < $l; $i+= $priceChange){
                $i                          = round($i, 2);
                if($i == $startPrice){
                    $fuelArray[]            = array(
                        'fuel_price_start'  => '',
                        'fuel_price_end'    => round($i - 0.01, 2),
                        'fuel_surcharge'    => 'No Surcharge',
                    );

                    $fuelArray[]            = array(
                        'fuel_price_start'  => round($i, 2),
                        'fuel_price_end'    => round($i + ($priceChange - 0.01), 2),
                        'fuel_surcharge'    => $basePercentage,
                    );
                }else{
                    $basePercentage         += $percentageChange;

                    $fuelArray[]            = array(
                        'fuel_price_start'  => round($i, 2),
                        'fuel_price_end'    => round($i + ($priceChange - 0.01), 2),
                        'fuel_surcharge'    => $basePercentage,
                    );
                }
            }
        }else{
            $perMileStart                   = round($template->per_mile_start, 2);
            $perMileChange                  = round($template->per_mile_change, 2);
            for ($i = $startPrice, $l = $endingPrice; $i < $l; $i+= $priceChange){
                $i                          = round($i, 2);
                if($i == $startPrice){
                    $fuelArray[]            = array(
                        'fuel_price_start'  => '',
                        'fuel_price_end'    => round($i - 0.01, 2),
                        'fuel_surcharge'    => 'No Surcharge',
                    );

                    $fuelArray[]            = array(
                        'fuel_price_start'  => round($i, 2),
                        'fuel_price_end'    => round($i + ($priceChange - 0.01), 2),
                        'fuel_surcharge'    => $perMileStart,
                    );
                }else{
                    $perMileStart           += $perMileChange;

                    $fuelArray[]            = array(
                        'fuel_price_start'  => round($i, 2),
                        'fuel_price_end'    => round($i + ($priceChange - 0.01), 2),
                        'fuel_surcharge'    => $perMileStart,
                    );
                }
            }
        }
        foreach ($fuelArray as $fuelDetail){
            if(($fuelDetail['fuel_price_start'] <= $currentRegionRate) && ($fuelDetail['fuel_price_end'] >= $currentRegionRate)){
                if($template->fuel_type == "EPMR"){
                    DefaultFuelSurcharge::updateOrCreate(['company_id' => $template->company_id], ['default_fuel_surcharge_gas' => $fuelDetail['fuel_surcharge'], 'rate_type_gas' => $template->rate_type, 'last_updated_gas' => $currentWeekMonday]);

                    if($template->rate_type == "1"){
                        return [
                            'value' => number_format($fuelDetail['fuel_surcharge'], 2, '.', '').'%',
                            'last_updated' => $currentWeekMonday
                        ];
                    }else{
                        return [
                            'value' => '$'.number_format($fuelDetail['fuel_surcharge'], 2, '.', ''),
                            'last_updated' => $currentWeekMonday
                        ];
                    }
                }else if($template->fuel_type == "EPD2D"){
                    DefaultFuelSurcharge::updateOrCreate(['company_id' => $template->company_id], ['default_fuel_surcharge_diesel' => $fuelDetail['fuel_surcharge'], 'rate_type_diesel' => $template->rate_type, 'last_updated_diesel' => $currentWeekMonday]);

                    if($template->rate_type == "1"){
                        return [
                            'value' => number_format($fuelDetail['fuel_surcharge'], 2, '.', '').'%',
                            'last_updated' => $currentWeekMonday
                        ];
                    }else{
                        return [
                            'value' => '$'.number_format($fuelDetail['fuel_surcharge'], 2, '.', ''),
                            'last_updated' => $currentWeekMonday
                        ];
                    }
                }
            }
        }
    }
}

function getTemplateFuelSurchargeArray($template, $startOfWeek = null){
    // Retrive fuel surcharge data
    $resultArray                                = [];
    if(is_null($startOfWeek)){
        $startOfWeek                            = Carbon::now()->startOfWeek()->format('Y-m-d');
    }
    $startOfWeekRegionRate                      = getRegionWiseCurrentRate($template->fuel_type, $template->region, $startOfWeek);
    if($startOfWeekRegionRate != ''){
        $startPrice                             = round($template->start_price, 2);
        $endingPrice                            = round($template->ending_price, 2);
        $priceChange                            = round($template->price_change, 2);

        $fuelArray                              = array();
        if($template->rate_type == '1'){
            // 0: Per Mile, 1: Percentage, 2: Both
            $basePercentage                     = round($template->base_percentage, 2);
            $percentageChange                   = round($template->percentage_change, 2);
            for ($i = $startPrice, $l = $endingPrice; $i < $l; $i+= $priceChange){
                $i                              = round($i, 2);
                if($i == $startPrice){
                    $fuelArray[]                = array(
                        'fuel_price_start'      => '',
                        'fuel_price_end'        => round($i - 0.01, 2),
                        'fuel_surcharge'        => 'No Surcharge',
                    );

                    $fuelArray[]                = array(
                        'fuel_price_start'      => round($i, 2),
                        'fuel_price_end'        => round($i + ($priceChange - 0.01), 2),
                        'fuel_surcharge'        => $basePercentage,
                    );
                }else{
                    $basePercentage             += $percentageChange;

                    $fuelArray[]                = array(
                        'fuel_price_start'      => round($i, 2),
                        'fuel_price_end'        => round($i + ($priceChange - 0.01), 2),
                        'fuel_surcharge'        => $basePercentage,
                    );
                }
            }
        }else{
            // 0: Per Mile, 1: Percentage, 2: Both
            $perMileStart                       = round($template->per_mile_start, 2);
            $perMileChange                      = round($template->per_mile_change, 2);
            for ($i = $startPrice, $l = $endingPrice; $i < $l; $i+= $priceChange){
                $i                              = round($i, 2);
                if($i == $startPrice){
                    $fuelArray[]                = array(
                        'fuel_price_start'      => '',
                        'fuel_price_end'        => round($i - 0.01, 2),
                        'fuel_surcharge'        => 'No Surcharge',
                    );

                    $fuelArray[]                = array(
                        'fuel_price_start'      => round($i, 2),
                        'fuel_price_end'        => round($i + ($priceChange - 0.01), 2),
                        'fuel_surcharge'        => $perMileStart,
                    );
                }else{
                    $perMileStart               += $perMileChange;

                    $fuelArray[]                = array(
                        'fuel_price_start'      => round($i, 2),
                        'fuel_price_end'        => round($i + ($priceChange - 0.01), 2),
                        'fuel_surcharge'        => $perMileStart,
                    );
                }
            }
        }

        $filteredArray                          = array_values(array_filter($fuelArray, function ($array) use ($startOfWeekRegionRate) {
            return $startOfWeekRegionRate >= $array["fuel_price_start"] && $startOfWeekRegionRate <= $array["fuel_price_end"];
        }));
        if (!empty($filteredArray)) {
            $resultArray                        = $filteredArray[0];
            // $key                             = array_search($resultArray, $fuelArray);
            // echo "Key: $key\n";
        } else {
            // echo "No match found for the given rate.\n";
        }
    }
    return $resultArray;
}

function brokerStatus(){
    return ['1' => ['text' => 'Active', 'color' => '#2F9840'], '2' => ['text' => 'Inactive', 'color' => '#ED1C24'], '3' => ['text' => 'Credit Hold', 'color' => '#ED1C24'], '4' => ['text' => 'Compliance Hold', 'color' => '#ED1C24']];
}

function brokerType(){
    return ['1' => '3PL', '2' => '4PL', '3' => 'Broker', '4' => 'Direct Customer', '5' => 'Freight Forwarder'];
}

function brokerOfficeType(){
    return ['1' => 'Headquarters', '2' => 'Satellite Office'];
}

function brokerSourcing(){
    return ['1' => 'Internal Sourcing', '2' => 'External Load Boarde', '3' => 'Both'];
}

function brokerPerformanceReviewFrequency(){
    return ['weekly' => 'Weekly', 'monthly' => 'Monthly', 'quarterly' => 'Quarterly', 'annually' => 'Annually'];
}

function getCustomerFuelSurchargeTemplates($type, $brokerId){
    if($type == 'DIESEL'){
        //return FuelSurchargeTemplate::IsCompany()->where('fuel_type', 'EPD2D')->where('customer', '!=', 'FSD')->pluck('customer_file_name', 'id')->toArray();
        return FuelSurchargeTemplate::IsCompany()->where('fuel_type', 'EPD2D')->where('customer', $brokerId)->pluck('customer_file_name', 'id')->toArray();
    }else if($type == 'GAS'){
        //return FuelSurchargeTemplate::IsCompany()->where('fuel_type', 'EPMR')->where('customer', '!=', 'FSG')->pluck('customer_file_name', 'id')->toArray();
        return FuelSurchargeTemplate::IsCompany()->where('fuel_type', 'EPMR')->where('customer', $brokerId)->pluck('customer_file_name', 'id')->toArray();
    }else {
        return [];
    }
}

function brokerBillingMethod(){
    return ['1' => 'EDI', '2' => 'Email', '3' => 'Mail'];
}

function brokerPaymentMethod(){
    return ['1' => ['text' => 'ACH/Wire', 'icon' => asset('theme/images/ach-wire.svg')], '2' => ['text' => 'Cash', 'icon' => asset('theme/images/cash.svg')], '3' => ['text' => 'Certified Check', 'icon' => asset('theme/images/certified-check.svg')], '4' => ['text' => 'Credit/Debit Card', 'icon' => asset('theme/images/credit-debit-card.svg')], '5' => ['text' => 'Standard Check', 'icon' => asset('theme/images/standard-check.svg')]];
}

function brokerCustomerAssignment(){
    return ['1' => 'Company' , '2' => 'Agent'];
}

function brokerFactoringCompany(){
    return ['1' => 'Denim', '2' => 'RTS', '3' => 'Triumph'];
}

function brokerFactorChargeType(){
    return ['1' => 'Flat', '2' => 'Percentage', '3' => 'Percentage w/ min. charge'];
}

function brokerTermStartDate(){
    return ['1' => 'Pickup Date', '2' => 'Delivery Date', '3' => 'Invoice Date'];
}

function brokerTerms(){
    return ['cod' => 'C.O.D', '15_days' => '15 Days', '30_days' => '30 Days', '45_days' => '45 Days', '60_days' => '60 Days', '90_days' => '90 Days', '1_days' => '1 Days', '2_days' => '2 Days', '3_days' => '3 Days', '4_days' => '4 Days', '5_days' => '5 Days', '6_days' => '6 Days', '7_days' => '7 Days'];
}

function brokerFreightBillFormat(){
    return ['1' => 'Itemized', '2' => 'Basic'];
}

function brokerRequiredSaferStatus(){
    return ['1' => 'Satisfactory', '2' => 'Conditional'];
}

function brokerRequiredAuthorityTenure(){
    return ['30_days' => '30 Days', '60_days' => '60 Days', '90_days' => '90 Days', '120_days' => '120 Days', '6_months' => '6 Months', '1_years' => '1 Years', '2_years' => '2 Years', '3_years' => '3 Years', '4_years' => '4 Years', '5_years' => '5 Years', '6_years' => '6 Years', '7_years' => '7 Years', '8_years' => '8 Years', '9_years' => '9 Years', '10_years_plus' => '10 Years+'];
}

function brokerRequiredFleetSize(){
    return ['1_truck' => '1 Truck', '2_trucks' => '2 Trucks', '3_trucks' => '3 Trucks', '4_trucks' => '4 Trucks', '5_trucks' => '5 Trucks', '6_trucks' => '6 Trucks', '7_trucks' => '7 Trucks', '8_trucks' => '8 Trucks', '9_trucks' => '9 Trucks', '10_trucks' => '10 Trucks', '11_trucks' => '11 Trucks', '12_trucks' => '12 Trucks'];
}

function brokerRequiredLoadsLast30Days(){
    return ['1_load' => '1 Load', '2_loads' => '2 Loads', '3_loads' => '3 Loads', '4_loads' => '4 Loads', '5_loads' => '5 Loads', '6_loads' => '6 Loads', '7_loads' => '7 Loads', '8_loads' => '8 Loads', '9_loads' => '9 Loads'];
}

function brokerRequiredLoadsPerMonth(){
    return ['1_load' => '1 Load', '2_loads' => '2 Loads', '3_loads' => '3 Loads', '4_loads' => '4 Loads', '5_loads' => '5 Loads', '6_loads' => '6 Loads', '7_loads' => '7 Loads', '8_loads' => '8 Loads', '9_loads' => '9 Loads'];
}

function brokerPCMilerVersion(){
    return ['1' => 'PC*Miler 37'];
}

function brokerMileageType(){
    return ['1' => 'Fastest', '2' => 'Shortest', '3' => 'Practical'];
}

function brokerFuelSurchargeIndex(){
    return ['1' => 'Jan-Jun 2023 RFP Surcharge'];
}

function getAccessorials($companyId = null){
    if(empty($companyId)){
        $companyId = auth()->guard('company-admin')->user()->company_id;
    }
    return DB::table('company_accessorials')->where('company_id', $companyId)->pluck('name', 'id')->toArray();
}

function getBrokerRequiredDocumentProfiles()
{
    return BrokerRequiredDocumentProfile::where('company_id', auth()->guard('company-admin')->user()->company_id)->get();
}

function brokerDepartment()
{
    return [
        'accounting' => 'Accounting',
        'after-hours' => 'After Hours',
        'asset-management' => 'Asset Management',
        'billing' => 'Billing',
        'board-of-directors' => 'Board of Directors',
        'brokerage' => 'Brokerage',
        'business-development' => 'Business Development',
        'claims' => 'Claims',
        'collections' => 'Collections',
        'corporate-communications' => 'Corporate Communications',
        'creative-services' => 'Creative Services',
        'customer-development' => 'Customer Development',
        'customer-experience' => 'Customer Experience',
        'customer-service' => 'Customer Service',
        'dispatch' => 'Dispatch',
        'engineering' => 'Engineering',
        'finance' => 'Finance',
        'general-management' => 'General Management',
        'human-resources' => 'Human Resources',
        'information-technology' => 'Information Technology',
        'insurance' => 'Insurance',
        'investor-relations' => 'Investor Relations',
        'legal' => 'Legal',
        'marketing' => 'Marketing',
        'onboarding-team' => 'Onboarding Team',
        'product-management' => 'Product Management',
        'production' => 'Production',
        'project-management-office' => 'Project Management Office',
        'purchasing' => 'Purchasing',
        'quality-assurance' => 'Quality Assurance',
        'recruiting' => 'Recruiting',
        'risk-management' => 'Risk Management',
        'safety' => 'Safety',
        'sales' => 'Sales',
        'shop' => 'Shop',
        'sourcing' => 'Sourcing',
        'strategic-initiatives-Programs' => 'Strategic Initiatives & Programs',
        'technology' => 'Technology',
        'transport' => 'Transport'
    ];
    //return ['Safety' => 'Safety', 'Collections' => 'Collections', 'Transport' => 'Transport', 'Dispatch' => 'Dispatch'];
}

function brokerPreferredContactMethod()
{
    return ['1' => 'Cell Phone', '2' => 'Email', '3' => 'Office Phone', '4' => 'ProChat', '5' => 'SMS'];
}

function brokerSuffix()
{
    return ['1' => 'Mr.', '2' => 'Mrs.', '3' => 'Ms.', '4' => 'Miss', '5' => 'Dr.'];
}

function brokerContactStatus(){
    return ['1' => 'Active', '2' => 'Inactive'];
}

function brokerStandardNotificationTypes(){
    return ['1' => 'Arrival', '2' => 'Loaded'];
}
function brokerStandardNotificationCommunicationTypes(){
    return ['1' => 'SMS', '2' => 'Prochat', '3' => 'Email', '4' => 'DispatchPro Notification Center'];
}

function brokerGeofenseNotificationLocation(){
    return ['1' => 'Shipper', '2' => 'Customer', '3' => 'Consignee'];
}

function brokerGeofenseNotificationTypes(){
    return ['1' => 'Arrival', '2' => 'Loaded'];
}

function brokerGeofenseNotificationNotify(){
    return ['1' => 'Shipper', '2' => 'Customer', '3' => 'Consignee'];
}

function brokerGeofenseNotificationCommunicationTypes(){
    return ['1' => 'SMS', '2' => 'Prochat', '3' => 'Email', '4' => 'DispatchPro'];
}

function brokerIntegrationNotificationPartners(){
    return ['1' => 'FourKites', '2' => 'MacroPoint', '3' => 'Trucker Tools', '4' => 'Samsara', '5' => 'Mi Fleet'];
}

function brokerIntegrationNotificationTypes(){
    return ['1' => 'All', '2' => 'Last Location'];
}

function getBrokerContactsOptions($source, $action_id)
{
    return BrokerContact::select('id', DB::raw('CONCAT(first_name," ",last_name) as full_name'))->where('source', $source)->where('action_id', $action_id)->where('contact_status', '1')->orderBy('id','ASC')->pluck('full_name', 'id')->toArray();
}

function getBrokerCustomerNotesAddTo()
{
    return ['1' => 'Billing Page (Top of Page)', '2' => 'Customer Profile (Top of Page)', '3' => 'Invoice Page (Top of Page)', '4' => 'Load View/Edit (Top of Page)'];
}

function getBrokerInternalNotesAddTo()
{
    return ['1' => 'Billing Page (Top of Page)', '2' => 'Customer Profile (Top of Page)', '3' => 'Invoice Page (Top of Page)', '4' => 'Load View/Edit (Top of Page)'];
}

function brokerCustomerFileStatus(){
    return ['1' => 'Active', '2' => 'Inactive', '3' => 'Pending'];
}

function getBrokerParentCompany($brokerId) {
    return Broker::IsCompany()->select('id', DB::raw('CONCAT(name, " - ", (select name from companies where id = company_id), " - ", city) as parent'))->where('office_type', '1')->where('id', '!=', $brokerId)->pluck('parent', 'id')->toArray();
}

function numberToWords($number)
{
    $fmt = new \NumberFormatter('en_US', \NumberFormatter::SPELLOUT);
    if($number){
        return ucwords($fmt->format($number).' dollars');
    }
    return null;
}

function customerFileTypes(){
    return ['1' => 'Contracts', '2' => 'Pricing', '3' => 'Fuel Tables', '4' => 'Quick Quotes', '5' => 'Performance Reviews', '6' => 'Insurance Requirements'];
}

function customerFileKeys(){
    return ['1' => 'contract_file_', '2' => 'pricing_file_', '3' => 'fuel_table_file_', '4' => 'quick_quote_file_', '5' => 'performance_review_file_', '6' => 'insurance_requirement_file_'];
}

function totalOnHoldLoads(){
    return LoadDetail::select('*')
        //->where('is_disable', 0)
        ->where(function($q) {
            $q->where('load_details.load_status', 7)->orWhere('load_details.is_on_hold', 1);
        })
        ->where(function($q) {
            $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
        })->count();
}

function updateLoadBillingActivity($loadDetailId)
{
    $load_detail = LoadDetail::whereId($loadDetailId)->first();

    if(!is_null($load_detail))
    {
        $getLoadDocuments = getLoadDocuments($load_detail);

        // Merge Documents
        $merged_documents = $getLoadDocuments['merged_documents'];

        if(!empty($merged_documents)){
            $billingActivity = LoadBillingActivity::firstOrCreate(['load_id' => $load_detail->load_id, 'load_detail_id' => $load_detail->id]);
            if($billingActivity->wasRecentlyCreated) {
                $billingActivity->refresh();
            }

            //1: Check Missing Paperwork Complete
            $paperworkReceivedStatus = in_array('Pending', array_column($merged_documents, 'status')) ? '0' : '1'; // 0 = Missing Paperwork, 1 = Complete

            if($billingActivity->paperwork_received == '0' && $paperworkReceivedStatus == '1'){
                $billingActivity->paperwork_received = '1';
                $billingActivity->paperwork_received_date_time = Carbon::now()->setTimezone(getTimeZone());

                //Load Move to ready to bill
                $load_detail->load_status = 6;
                $load_detail->load_status_order = Common::getLoadStatusOrderById( 6 );
                $load_detail->updated_at = date('Y-m-d H:i:s');
                $load_detail->save();
            }else if($billingActivity->paperwork_received == '1' && $paperworkReceivedStatus == '0'){
                $billingActivity->paperwork_received = '0';
                $billingActivity->paperwork_received_date_time = null;

                //Load Move to delivery status
                $load_detail->load_status = 5;
                $load_detail->load_status_order = Common::getLoadStatusOrderById( 5 );
                $load_detail->updated_at = date('Y-m-d H:i:s');
                $load_detail->save();
            }

            //2: Check Audit Complete
            if($billingActivity->paperwork_received == '1' && $paperworkReceivedStatus == '1'){
                $auditComplete = (count(array_intersect(['0', '2'], array_column($merged_documents, 'file_verify'))) != '0') ? '0' : '1'; // 0 = Audit Pending, 1 = Audit Complete

                if($billingActivity->audit_complete == '0' && $auditComplete == '1'){
                    $billingActivity->audit_complete = '1';
                    $billingActivity->audit_complete_date_time = Carbon::now()->setTimezone(getTimeZone());
                    $billingActivity->audit_complete_user_type = auth()->guard('company-admin')->user()->type;
                    $billingActivity->audit_complete_user_by = auth()->guard('company-admin')->user()->id;
                }else if($billingActivity->audit_complete == '1' && $auditComplete == '0'){
                    $billingActivity->audit_complete = '0';
                    $billingActivity->audit_complete_date_time = null;
                    $billingActivity->audit_complete_user_type = null;
                    $billingActivity->audit_complete_user_by = null;
                }
            }else{
                $billingActivity->audit_complete = '0';
                $billingActivity->audit_complete_date_time = null;
                $billingActivity->audit_complete_user_type = null;
                $billingActivity->audit_complete_user_by = null;
            }

            $billingActivity->save();
        }else{
            $billingActivity = LoadBillingActivity::whereLoadId($load_detail->load_id)->whereLoadDetailId($load_detail->id)->first();
            if(!is_null($billingActivity)){
                $billingActivity->paperwork_received = '0';
                $billingActivity->paperwork_received_date_time = null;
                $billingActivity->audit_complete = '0';
                $billingActivity->audit_complete_date_time = null;
                $billingActivity->audit_complete_user_type = null;
                $billingActivity->audit_complete_user_by = null;
                $billingActivity->invoiced = '0';
                $billingActivity->invoiced_date_time = null;
                $billingActivity->invoiced_user_type = null;
                $billingActivity->invoiced_user_by = null;
                $billingActivity->paid_invoice = '0';
                $billingActivity->paid_invoice_date_time = null;
                $billingActivity->paid_invoice_user_type = null;
                $billingActivity->paid_invoice_user_by = null;
                $billingActivity->save();
            }
        }
    }
}

function getLoadDocuments($load_detail)
{
    // Required Documents
    $required_documents = [];
    // Requested Documents
    $requested_documents = [];
    // Accessorial Documents
    $accessorial_documents = [];
    // Other Documents
    $other_documents = [];

    // Required Documents
    if(isset($load_detail->shipper_files_upload) && is_serialized($load_detail->shipper_files_upload) && !empty(unserialize($load_detail->shipper_files_upload))){
        $uploads = unserialize($load_detail->shipper_files_upload);
        $uploads = array_filter($uploads, function ($var) {
            return (
                isset($var['name_id']) && $var['name_id'] !== null && $var['is_required'] == 'on' && in_array($var['type'], [1, 2]) && $var['include_w_billing'] == 'yes'
            );
        });
        foreach ($uploads as $key => $value) {
            // Type 1 = Copy, 2 = Original
            $array  = [];
            $array['id'] = $load_detail->id;
            $array['source'] = 'load_details';
            $array['field_name'] = 'shipper_files_upload';

            $array['filename'] = $value['name'] ?? '';
            $array['filename_id'] = $value['name_id'] ?? '';

            $array['type'] = $value['type'];

            $array['is_received'] = isset($value['is_received']) && $value['is_received'] == 'on' ? 'on' : 'off';

            $status = 'Pending';
            if($array['type'] == 1){
                $array['file'] = $value['file'] ?? null;
                if(isset($value['file']) && !empty($value['file'])){
                    $status = 'Complete';
                    $array['upload_user_type'] = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                    $array['upload_by'] = isset($value['upload_by']) ? $value['upload_by'] : null;
                    if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                        $array['uploaded_by'] = getUserByType($value['upload_by'], $value['upload_user_type']);
                    }else{
                        $array['uploaded_by'] = null;
                    }
                    $array['uploaded_at'] = isset($value['uploaded_at']) ? \Illuminate\Support\Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                }
            }else{
                if($array['is_received'] == 'on'){
                    $status = 'Complete';
                    $array['upload_user_type'] = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                    $array['upload_by'] = isset($value['upload_by']) ? $value['upload_by'] : null;
                    if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                        $array['uploaded_by'] = getUserByType($value['upload_by'], $value['upload_user_type']);
                    }else{
                        $array['uploaded_by'] = null;
                    }
                    $array['uploaded_at'] = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                }
            }

            $array['include_w_billing']                             = isset($value['include_w_billing']) && $value['include_w_billing'] == 'yes' ? 'yes' : 'no';
            $array['hide_from_driver']                              = isset($value['hide_from_driver']) && $value['hide_from_driver'] == 'yes' ? 'yes' : 'no';
            $array['history']                                       = isset($value['history']) && is_array($value['history']) && !empty($value['history']) ? $value['history'] : [];
            $array['status'] = $status;

            $addresses = [];
            if(isset($load_detail->shipper_address_one) && $load_detail->shipper_address_one!==''){
                $addresses[] = $load_detail->shipper_address_one;
            }
            if(isset($load_detail->shipper_address_two) && $load_detail->shipper_address_two!==''){
                $addresses[] = $load_detail->shipper_address_two;
            }
            if(isset($load_detail->shipper_city) && $load_detail->shipper_city!==''){
                $addresses[] = $load_detail->shipper_city;
            }
            if(isset($load_detail->shipper_state) && $load_detail->shipper_state!==''){
                $addresses[] = $load_detail->shipper_state;
            }
            if(isset($load_detail->shipper_zip) && $load_detail->shipper_zip!==''){
                $addresses[] = $load_detail->shipper_zip;
            }
            $addresses = implode(', ', $addresses);

            $driver                                                 = null;
            $driver_id                                              = 0;
            $driver_full_name                                       = [];
            if (isset($load_detail->driver)) {
                $driver                                             = $load_detail->driver;
                $driver_id                                          = $driver->id;
                if(isset($driver->last_name) && $driver->last_name != ''){
                    $driver_full_name[]                             = $driver->last_name;
                }
                if(isset($driver->first_name) && $driver->first_name != ''){
                    $driver_full_name[]                             = $driver->first_name;
                }
            }
            $driver_full_name                                       = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

            $stop_details = [];
            $stop_details['name'] = 'Stop 1';
            $stop_details['driver_id']                              = $driver_id;
            $stop_details['driver_full_name']                       = $driver_full_name;
            $stop_details['customer_name'] = $load_detail->shipper->name ?? '';
            $stop_details['addresses'] = $addresses;
            // Extra Info
            $array['stop'] = $stop_details;
            if($status == 'Complete') {
                $array['file_verify'] = (isset($value['file_verify']) && in_array($value['file_verify'], ['1', '2'])) ? $value['file_verify'] : '0'; // 0: Pending, 1: Approved, 2: Rejected
                if ($array['file_verify'] != '0') {
                    $array['file_verify_user_type'] = isset($value['file_verify_user_type']) ? $value['file_verify_user_type'] : null;
                    $array['file_verify_by'] = isset($value['file_verify_by']) ? $value['file_verify_by'] : null;
                    if (isset($array['file_verify_user_type']) && isset($array['file_verify_by'])) {
                        $array['file_verified_by'] = getUserByType($value['file_verify_by'], $value['file_verify_user_type']);
                    } else {
                        $array['file_verified_by'] = null;
                    }
                    $array['file_verified_at'] = isset($value['file_verified_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['file_verified_at'])->setTimezone(getTimeZone())->format(getTimeLayout()) . " " . getTimeZoneId() : null;
                }
            }

            $required_documents[] = $array;
        }
    }
    // Requested Documents
    if(isset($load_detail->shipper_load_document_request_items)){
        $shipper_load_document_request_items = $load_detail->shipper_load_document_request_items;
        foreach ($shipper_load_document_request_items as $key => $item) {
            $value = [];
            if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                $value = unserialize($item->files);
            }
            if(!empty($value)){
                if(
                    isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1, 2]) && $value['include_w_billing'] == 'yes'
                ){
                    // Type 1 = Copy, 2 = Original
                    $array                                          = [];
                    $array['id']                                    = $item->id;
                    $array['source']                                = 'load_document_request_items';
                    $array['field_name']                            = 'files';

                    $array['filename']                              = $value['name'] ?? '';
                    $array['filename_id']                           = $value['name_id'] ?? '';

                    $array['type']                                  = $value['type'];

                    $array['is_received']                           = isset($value['is_received']) && $value['is_received'] == 'on' ? 'on' : 'off';

                    $status                                         = 'Pending';
                    if($array['type'] == 1){
                        $array['file']                              = $value['file'] ?? null;
                        if(isset($value['file']) && !empty($value['file'])){
                            $status                                 = 'Complete';
                            $array['upload_user_type']              = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                            $array['upload_by']                     = isset($value['upload_by']) ? $value['upload_by'] : null;
                            if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                $array['uploaded_by']               = getUserByType($value['upload_by'], $value['upload_user_type']);
                            }else{
                                $array['uploaded_by']               = null;
                            }
                            $array['uploaded_at']                   = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                        }
                    }else{
                        if($array['is_received'] == 'on'){
                            $status                                 = 'Complete';
                            $array['upload_user_type']              = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                            $array['upload_by']                     = isset($value['upload_by']) ? $value['upload_by'] : null;
                            if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                $array['uploaded_by']               = getUserByType($value['upload_by'], $value['upload_user_type']);
                            }else{
                                $array['uploaded_by']               = null;
                            }
                            $array['uploaded_at']                   = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                        }
                    }

                    $array['include_w_billing']                     = isset($value['include_w_billing']) && $value['include_w_billing'] == 'yes' ? 'yes' : 'no';
                    $array['hide_from_driver']                      = isset($value['hide_from_driver']) && $value['hide_from_driver'] == 'yes' ? 'yes' : 'no';
                    $array['history']                               = isset($value['history']) && is_array($value['history']) && !empty($value['history']) ? $value['history'] : [];
                    $array['status']                                = $status;

                    $addresses                                      = [];
                    if(isset($load_detail->shipper_address_one) && $load_detail->shipper_address_one!==''){
                        $addresses[]                                = $load_detail->shipper_address_one;
                    }
                    if(isset($load_detail->shipper_address_two) && $load_detail->shipper_address_two!==''){
                        $addresses[]                                = $load_detail->shipper_address_two;
                    }
                    if(isset($load_detail->shipper_city) && $load_detail->shipper_city!==''){
                        $addresses[]                                = $load_detail->shipper_city;
                    }
                    if(isset($load_detail->shipper_state) && $load_detail->shipper_state!==''){
                        $addresses[]                                = $load_detail->shipper_state;
                    }
                    if(isset($load_detail->shipper_zip) && $load_detail->shipper_zip!==''){
                        $addresses[]                                = $load_detail->shipper_zip;
                    }
                    $addresses                                      = implode(', ', $addresses);

                    $driver                                         = null;
                    $driver_id                                      = 0;
                    $driver_full_name                               = [];
                    if(isset($item->driver)){
                        $driver                                     = $item->driver;
                        $driver_id                                  = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                     = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                     = $driver->first_name;
                        }
                    } elseif (isset($load_detail->driver)) {
                        $driver                                     = $load_detail->driver;
                        $driver_id                                  = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                     = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                     = $driver->first_name;
                        }
                    }
                    $driver_full_name                               = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                    $stop_details                                   = [];
                    $stop_details['name']                           = 'Stop 1';
                    $stop_details['driver_id']                      = $driver_id;
                    $stop_details['driver_full_name']               = $driver_full_name;
                    $stop_details['customer_name']                  = $load_detail->shipper->name ?? '';
                    $stop_details['addresses']                      = $addresses;
                    // Extra Info
                    $array['stop']                                  = $stop_details;
                    if($status == 'Complete'){
                        $array['file_verify']                          = (isset($value['file_verify']) && in_array($value['file_verify'], ['1', '2'])) ? $value['file_verify'] : '0'; // 0: Pending, 1: Approved, 2: Rejected
                        if($array['file_verify'] != '0'){
                            $array['file_verify_user_type'] = isset($value['file_verify_user_type']) ? $value['file_verify_user_type'] : null;
                            $array['file_verify_by'] = isset($value['file_verify_by']) ? $value['file_verify_by'] : null;
                            if(isset($array['file_verify_user_type']) && isset($array['file_verify_by'])){
                                $array['file_verified_by'] = getUserByType($value['file_verify_by'], $value['file_verify_user_type']);
                            }else{
                                $array['file_verified_by'] = null;
                            }
                            $array['file_verified_at'] = isset($value['file_verified_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['file_verified_at'])->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                        }
                    }

                    $requested_documents[]                          = $array;
                }
            }
        }
    }
    // Accessorial Documents
    if(isset($load_detail->shipper_accessorials)){
        $shipper_accessorials = $load_detail->shipper_accessorials->where('doc_required', '!=', 0)->where('include_w_billing', 'yes');
        foreach ($shipper_accessorials as $key => $item) {
            $gallery = Gallery::where('company_id', auth()->guard('company-admin')->user()->company_id)
                ->where('action_id', $item->id)
                ->where('source', 'load_accessorials')
                ->where('field_name', 'accessorial_files')
                ->first();
            $filename                                               = $item->company_accessorial->name;
            if(!is_null($gallery)){
                if(isset($gallery->hide_from_driver) && $gallery->hide_from_driver == 'yes'){
                    $hide_from_driver                               = 'yes';
                }
                $filename                                          .= isset($gallery->file_name) ? ' - '.$gallery->file_name : '';
            }
            // Type 1 = Copy, 2 = Original
            $array                                                  = [];
            $array['id']                                            = $item->id;
            $array['source']                                        = 'load_accessorials';
            $array['field_name']                                    = 'image';

            $array['filename']                                      = $filename;
            // $array['filename_id']                                = 0;

            $array['type']                                          = 1;

            $array['is_received']                                   = 'off';

            $status                                                 = 'Pending';
            $array['file']                                          = $gallery->image ?? null;
            if(isset($gallery->image) && !empty($gallery->image)){
                $status                                             = 'Complete';
                $array['is_received']                               = 'on';
                $array['upload_user_type']                          = isset($gallery->created_by_type) ? $gallery->created_by_type : null;
                $array['upload_by']                                 = isset($gallery->created_by) ? $gallery->created_by : null;
                if(isset($gallery->created_by_type) && isset($gallery->created_by)){
                    $array['uploaded_by']                           = getUserByType($gallery->created_by, $gallery->created_by_type);
                }else{
                    $array['uploaded_by']                           = null;
                }
                $array['uploaded_at']                               = isset($gallery->created_at) ? Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null; // $gallery->getRawOriginal('uploaded_date')
            }

            $array['include_w_billing']                             = isset($item->include_w_billing) && $item->include_w_billing == 'yes' ? 'yes' : 'no';
            $array['hide_from_driver']                              = isset($item->hide_from_driver) && $item->hide_from_driver == 'yes' ? 'yes' : 'no';
            $array['history']                                       = isset($item->history) && is_serialized($item->history) && !empty(unserialize($item->history)) ? unserialize($item->history) : [];
            $array['status']                                        = $status;

            $addresses                                              = [];
            if(isset($load_detail->shipper_address_one) && $load_detail->shipper_address_one!==''){
                $addresses[]                                        = $load_detail->shipper_address_one;
            }
            if(isset($load_detail->shipper_address_two) && $load_detail->shipper_address_two!==''){
                $addresses[]                                        = $load_detail->shipper_address_two;
            }
            if(isset($load_detail->shipper_city) && $load_detail->shipper_city!==''){
                $addresses[]                                        = $load_detail->shipper_city;
            }
            if(isset($load_detail->shipper_state) && $load_detail->shipper_state!==''){
                $addresses[]                                        = $load_detail->shipper_state;
            }
            if(isset($load_detail->shipper_zip) && $load_detail->shipper_zip!==''){
                $addresses[]                                        = $load_detail->shipper_zip;
            }
            $addresses                                              = implode(', ', $addresses);

            $driver                                                 = null;
            $driver_id                                              = 0;
            $driver_full_name                                       = [];
            if(isset($item->driver)){
                $driver                                             = $item->driver;
                $driver_id                                          = $driver->id;
                if(isset($driver->last_name) && $driver->last_name != ''){
                    $driver_full_name[]                             = $driver->last_name;
                }
                if(isset($driver->first_name) && $driver->first_name != ''){
                    $driver_full_name[]                             = $driver->first_name;
                }
            } elseif (isset($load_detail->driver)) {
                $driver                                             = $load_detail->driver;
                $driver_id                                          = $driver->id;
                if(isset($driver->last_name) && $driver->last_name != ''){
                    $driver_full_name[]                             = $driver->last_name;
                }
                if(isset($driver->first_name) && $driver->first_name != ''){
                    $driver_full_name[]                             = $driver->first_name;
                }
            }
            $driver_full_name                                       = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

            $stop_details                                           = [];
            $stop_details['name']                                   = 'Stop 1';
            $stop_details['driver_id']                              = $driver_id;
            $stop_details['driver_full_name']                       = $driver_full_name;
            $stop_details['customer_name']                          = $load_detail->shipper->name ?? '';
            $stop_details['addresses']                              = $addresses;
            // Extra Info
            $array['stop']                                          = $stop_details;
            if($status == 'Complete'){
                $array['file_verify'] = (isset($item->file_verify) && in_array($item->file_verify, ['1', '2'])) ? $item->file_verify : '0'; // 0: Pending, 1: Approved, 2: Rejected
                if($array['file_verify'] != '0'){
                    $array['file_verify_user_type'] = $item->file_verify_user_type ?? null;
                    $array['file_verify_by'] = $item->file_verify_by ?? null;
                    if(isset($array['file_verify_user_type']) && isset($array['file_verify_by'])){
                        $array['file_verified_by'] = getUserByType($item->file_verify_by, $item->file_verify_user_type);
                    }else{
                        $array['file_verified_by'] = null;
                    }
                    $array['file_verified_at'] = isset($value['file_verified_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $item->file_verified_at)->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                }
            }

            $accessorial_documents[]                                = $array;
        }
    }

    $stop_no = 2;
    if(isset($load_detail->loads->load_stops) && $load_detail->loads->load_stops->isNotEmpty()) {
        foreach ($load_detail->loads->load_stops as $key => $stop) {
            $customer_type = $stop->customer_type;
            $field_name = $customer_type=='1' ? 'stop_shipper_files_upload' : 'stop_consignee_files_upload';

            // Required Documents
            if(isset($stop->$field_name) && is_serialized($stop->$field_name) && !empty(unserialize($stop->$field_name))){
                $uploads = unserialize($stop->$field_name);
                $uploads = array_filter($uploads, function ($var) {
                    return (
                        isset($var['name_id']) && $var['name_id'] !== null && $var['is_required'] == 'on' && in_array($var['type'], [1, 2]) && $var['include_w_billing'] == 'yes'
                    );
                });
                foreach ($uploads as $key => $value) {
                    // Type 1 = Copy, 2 = Original
                    $array = [];
                    $array['id'] = $stop->id;
                    $array['source'] = 'load_stops';
                    $array['field_name'] = $field_name;

                    $array['filename'] = $value['name'] ?? '';
                    $array['filename_id'] = $value['name_id'] ?? '';

                    $array['type'] = $value['type'];

                    $array['is_received'] = isset($value['is_received']) && $value['is_received'] == 'on' ? 'on' : 'off';

                    $status = 'Pending';
                    if($array['type'] == 1){
                        $array['file'] = $value['file'] ?? null;
                        if(isset($value['file']) && !empty($value['file'])){
                            $status = 'Complete';
                            $array['upload_user_type'] = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                            $array['upload_by'] = isset($value['upload_by']) ? $value['upload_by'] : null;
                            if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                $array['uploaded_by'] = getUserByType($value['upload_by'], $value['upload_user_type']);
                            }else{
                                $array['uploaded_by'] = null;
                            }
                            $array['uploaded_at'] = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                        }
                    }else{
                        if($array['is_received'] == 'on'){
                            $status = 'Complete';
                            $array['upload_user_type'] = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                            $array['upload_by'] = isset($value['upload_by']) ? $value['upload_by'] : null;
                            if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                $array['uploaded_by'] = getUserByType($value['upload_by'], $value['upload_user_type']);
                            }else{
                                $array['uploaded_by'] = null;
                            }
                            $array['uploaded_at'] = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                        }
                    }

                    $array['include_w_billing']                     = isset($value['include_w_billing']) && $value['include_w_billing'] == 'yes' ? 'yes' : 'no';
                    $array['hide_from_driver']                      = isset($value['hide_from_driver']) && $value['hide_from_driver'] == 'yes' ? 'yes' : 'no';
                    $array['history']                               = isset($value['history']) && is_array($value['history']) && !empty($value['history']) ? $value['history'] : [];
                    $array['status'] = $status;

                    $addresses = [];
                    if(isset($stop->address_one) && $stop->address_one!==''){
                        $addresses[] = $stop->address_one;
                    }
                    if(isset($stop->address_two) && $stop->address_two!==''){
                        $addresses[] = $stop->address_two;
                    }
                    if(isset($stop->city) && $stop->city!==''){
                        $addresses[] = $stop->city;
                    }
                    if(isset($stop->state) && $stop->state!==''){
                        $addresses[] = $stop->state;
                    }
                    if(isset($stop->zip) && $stop->zip!==''){
                        $addresses[] = $stop->zip;
                    }
                    $addresses = implode(', ', $addresses);

                    $driver                                         = null;
                    $driver_id                                      = 0;
                    $driver_full_name                               = [];
                    if (isset($load_detail->driver)) {
                        $driver                                     = $load_detail->driver;
                        $driver_id                                  = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                     = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                     = $driver->first_name;
                        }
                    }
                    $driver_full_name                               = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                    $stop_details = [];
                    $stop_details['name'] = 'Stop '.$stop_no;
                    $stop_details['driver_id']                      = $driver_id;
                    $stop_details['driver_full_name']               = $driver_full_name;
                    $stop_details['customer_name'] = $stop->customer->name ?? '';
                    $stop_details['addresses'] = $addresses;
                    // Extra Info
                    $array['stop'] = $stop_details;
                    if($status == 'Complete') {
                        $array['file_verify'] = (isset($value['file_verify']) && in_array($value['file_verify'], ['1', '2'])) ? $value['file_verify'] : '0'; // 0: Pending, 1: Approved, 2: Rejected
                        if ($array['file_verify'] != '0') {
                            $array['file_verify_user_type'] = isset($value['file_verify_user_type']) ? $value['file_verify_user_type'] : null;
                            $array['file_verify_by'] = isset($value['file_verify_by']) ? $value['file_verify_by'] : null;
                            if (isset($array['file_verify_user_type']) && isset($array['file_verify_by'])) {
                                $array['file_verified_by'] = getUserByType($value['file_verify_by'], $value['file_verify_user_type']);
                            } else {
                                $array['file_verified_by'] = null;
                            }
                            $array['file_verified_at'] = isset($value['file_verified_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['file_verified_at'])->setTimezone(getTimeZone())->format(getTimeLayout()) . " " . getTimeZoneId() : null;
                        }
                    }

                    $required_documents[] = $array;
                }
            }
            // Requested Documents
            if(isset($stop->load_document_request_items)){
                $stop_load_document_request_items                   = $stop->load_document_request_items;
                foreach ($stop_load_document_request_items as $key => $item) {
                    $value                                          = [];
                    if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                        $value                                      = unserialize($item->files);
                    }
                    if(!empty($value)){
                        if(
                            isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1, 2]) && $value['include_w_billing'] == 'yes'
                        ){
                            // Type 1 = Copy, 2 = Original
                            $array                                  = [];
                            $array['id']                            = $item->id;
                            $array['source']                        = 'load_document_request_items';
                            $array['field_name']                    = 'files';

                            $array['filename']                      = $value['name'] ?? '';
                            $array['filename_id']                   = $value['name_id'] ?? '';

                            $array['type']                          = $value['type'];

                            $array['is_received']                   = isset($value['is_received']) && $value['is_received'] == 'on' ? 'on' : 'off';

                            $status                                 = 'Pending';
                            if($array['type'] == 1){
                                $array['file']                      = $value['file'] ?? null;
                                if(isset($value['file']) && !empty($value['file'])){
                                    $status                         = 'Complete';
                                    $array['upload_user_type']      = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                                    $array['upload_by']             = isset($value['upload_by']) ? $value['upload_by'] : null;
                                    if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                        $array['uploaded_by']       = getUserByType($value['upload_by'], $value['upload_user_type']);
                                    }else{
                                        $array['uploaded_by']       = null;
                                    }
                                    $array['uploaded_at']           = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                                }
                            }else{
                                if($array['is_received'] == 'on'){
                                    $status                         = 'Complete';
                                    $array['upload_user_type']      = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                                    $array['upload_by']             = isset($value['upload_by']) ? $value['upload_by'] : null;
                                    if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                        $array['uploaded_by']       = getUserByType($value['upload_by'], $value['upload_user_type']);
                                    }else{
                                        $array['uploaded_by']       = null;
                                    }
                                    $array['uploaded_at']           = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                                }
                            }

                            $array['include_w_billing']             = isset($value['include_w_billing']) && $value['include_w_billing'] == 'yes' ? 'yes' : 'no';
                            $array['hide_from_driver']              = isset($value['hide_from_driver']) && $value['hide_from_driver'] == 'yes' ? 'yes' : 'no';
                            $array['history']                       = isset($value['history']) && is_array($value['history']) && !empty($value['history']) ? $value['history'] : [];
                            $array['status']                        = $status;

                            $addresses                              = [];
                            if(isset($stop->address_one) && $stop->address_one!==''){
                                $addresses[]                        = $stop->address_one;
                            }
                            if(isset($stop->address_two) && $stop->address_two!==''){
                                $addresses[]                        = $stop->address_two;
                            }
                            if(isset($stop->city) && $stop->city!==''){
                                $addresses[]                        = $stop->city;
                            }
                            if(isset($stop->state) && $stop->state!==''){
                                $addresses[]                        = $stop->state;
                            }
                            if(isset($stop->zip) && $stop->zip!==''){
                                $addresses[]                        = $stop->zip;
                            }
                            $addresses                              = implode(', ', $addresses);

                            $driver                                 = null;
                            $driver_id                              = 0;
                            $driver_full_name                       = [];
                            if(isset($item->driver)){
                                $driver                             = $item->driver;
                                $driver_id                          = $driver->id;
                                if(isset($driver->last_name) && $driver->last_name != ''){
                                    $driver_full_name[]             = $driver->last_name;
                                }
                                if(isset($driver->first_name) && $driver->first_name != ''){
                                    $driver_full_name[]             = $driver->first_name;
                                }
                            } elseif (isset($load_detail->driver)) {
                                $driver                             = $load_detail->driver;
                                $driver_id                          = $driver->id;
                                if(isset($driver->last_name) && $driver->last_name != ''){
                                    $driver_full_name[]             = $driver->last_name;
                                }
                                if(isset($driver->first_name) && $driver->first_name != ''){
                                    $driver_full_name[]             = $driver->first_name;
                                }
                            }
                            $driver_full_name                       = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                            $stop_details                           = [];
                            $stop_details['name']                   = 'Stop '.$stop_no;
                            $stop_details['driver_id']              = $driver_id;
                            $stop_details['driver_full_name']       = $driver_full_name;
                            $stop_details['customer_name']          = $stop->customer->name ?? '';
                            $stop_details['addresses']              = $addresses;
                            // Extra Info
                            $array['stop']                           = $stop_details;
                            if($status == 'Complete') {
                                $array['file_verify'] = (isset($value['file_verify']) && in_array($value['file_verify'], ['1', '2'])) ? $value['file_verify'] : '0'; // 0: Pending, 1: Approved, 2: Rejected
                                if ($array['file_verify'] != '0') {
                                    $array['file_verify_user_type'] = isset($value['file_verify_user_type']) ? $value['file_verify_user_type'] : null;
                                    $array['file_verify_by'] = isset($value['file_verify_by']) ? $value['file_verify_by'] : null;
                                    if (isset($array['file_verify_user_type']) && isset($array['file_verify_by'])) {
                                        $array['file_verified_by'] = getUserByType($value['file_verify_by'], $value['file_verify_user_type']);
                                    } else {
                                        $array['file_verified_by'] = null;
                                    }
                                    $array['file_verified_at'] = isset($value['file_verified_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['file_verified_at'])->setTimezone(getTimeZone())->format(getTimeLayout()) . " " . getTimeZoneId() : null;
                                }
                            }

                            $requested_documents[]                  = $array;
                        }
                    }
                }
            }
            // Accessorial Documents
            if(isset($stop->accessorials)){
                $stop_accessorials = $stop->accessorials->where('doc_required', '!=', 0)->where('include_w_billing', 'yes');
                foreach ($stop_accessorials as $key => $item) {
                    $gallery = Gallery::where('company_id', auth()->guard('company-admin')->user()->company_id)
                        ->where('action_id', $item->id)
                        ->where('source', 'load_accessorials')
                        ->where('field_name', 'accessorial_files')
                        ->first();
                    $filename                                       = $item->company_accessorial->name;
                    if(!is_null($gallery)){
                        if(isset($gallery->hide_from_driver) && $gallery->hide_from_driver == 'yes'){
                            $hide_from_driver                       = 'yes';
                        }
                        $filename                                  .= isset($gallery->file_name) ? ' - '.$gallery->file_name : '';
                    }
                    // Type 1 = Copy, 2 = Original
                    $array                                          = [];
                    $array['id']                                    = $item->id;
                    $array['source']                                = 'load_accessorials';
                    $array['field_name']                            = 'image';

                    $array['filename']                              = $filename;
                    // $array['filename_id']                        = 0;

                    $array['type']                                  = 1;

                    $array['is_received']                           = 'off';

                    $status                                         = 'Pending';
                    $array['file']                                  = $gallery->image ?? null;
                    if(isset($gallery->image) && !empty($gallery->image)){
                        $status                                     = 'Complete';
                        $array['is_received']                       = 'on';
                        $array['upload_user_type']                  = isset($gallery->created_by_type) ? $gallery->created_by_type : null;
                        $array['upload_by']                         = isset($gallery->created_by) ? $gallery->created_by : null;
                        if(isset($gallery->created_by_type) && isset($gallery->created_by)){
                            $array['uploaded_by']                   = getUserByType($gallery->created_by, $gallery->created_by_type);
                        }else{
                            $array['uploaded_by']                   = null;
                        }
                        $array['uploaded_at']                       = isset($gallery->created_at) ? Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null; // $gallery->getRawOriginal('uploaded_date')
                    }

                    $array['include_w_billing']                     = isset($item->include_w_billing) && $item->include_w_billing == 'yes' ? 'yes' : 'no';
                    $array['hide_from_driver']                      = isset($item->hide_from_driver) && $item->hide_from_driver == 'yes' ? 'yes' : 'no';
                    $array['history']                               = isset($item->history) && is_serialized($item->history) && !empty(unserialize($item->history)) ? unserialize($item->history) : [];
                    $array['status']                                = $status;

                    $addresses                                      = [];
                    if(isset($stop->address_one) && $stop->address_one!==''){
                        $addresses[]                                = $stop->address_one;
                    }
                    if(isset($stop->address_two) && $stop->address_two!==''){
                        $addresses[]                                = $stop->address_two;
                    }
                    if(isset($stop->city) && $stop->city!==''){
                        $addresses[]                                = $stop->city;
                    }
                    if(isset($stop->state) && $stop->state!==''){
                        $addresses[]                                = $stop->state;
                    }
                    if(isset($stop->zip) && $stop->zip!==''){
                        $addresses[]                                = $stop->zip;
                    }
                    $addresses                                      = implode(', ', $addresses);

                    $driver                                         = null;
                    $driver_id                                      = 0;
                    $driver_full_name                               = [];
                    if(isset($item->driver)){
                        $driver                                     = $item->driver;
                        $driver_id                                  = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                     = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                     = $driver->first_name;
                        }
                    } elseif (isset($load_detail->driver)) {
                        $driver                                     = $load_detail->driver;
                        $driver_id                                  = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                     = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                     = $driver->first_name;
                        }
                    }
                    $driver_full_name                               = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                    $stop_details                                   = [];
                    $stop_details['name']                           = 'Stop '.$stop_no;
                    $stop_details['driver_id']                      = $driver_id;
                    $stop_details['driver_full_name']               = $driver_full_name;
                    $stop_details['customer_name']                  = $stop->customer->name ?? '';
                    $stop_details['addresses']                      = $addresses;
                    // Extra Info
                    $array['stop']                                  = $stop_details;
                    if($status == 'Complete'){
                        $array['file_verify'] =  (isset($item->file_verify) && in_array($item->file_verify, ['1', '2'])) ? $item->file_verify : '0'; // 0: Pending, 1: Approved, 2: Rejected
                        if($array['file_verify'] != '0'){
                            $array['file_verify_user_type'] = $item->file_verify_user_type ?? null;
                            $array['file_verify_by'] = $item->file_verify_by ?? null;
                            if(isset($array['file_verify_user_type']) && isset($array['file_verify_by'])){
                                $array['file_verified_by'] = getUserByType($item->file_verify_by, $item->file_verify_user_type);
                            }else{
                                $array['file_verified_by'] = null;
                            }
                            $array['file_verified_at'] = isset($value['file_verified_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $item->file_verified_at)->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                        }
                    }

                    $accessorial_documents[]                        = $array;
                }
            }

            ++$stop_no;
        }
    }

    // Required Documents
    if(isset($load_detail->consignee_files_upload) && is_serialized($load_detail->consignee_files_upload) && !empty(unserialize($load_detail->consignee_files_upload))){
        $uploads = unserialize($load_detail->consignee_files_upload);
        $uploads = array_filter($uploads, function ($var) {
            return (
                isset($var['name_id']) && $var['name_id'] !== null && $var['is_required'] == 'on' && in_array($var['type'], [1, 2]) && $var['include_w_billing'] == 'yes'
            );
        });
        foreach ($uploads as $key => $value) {
            // Type 1 = Copy, 2 = Original
            $array = [];
            $array['id'] = $load_detail->id;
            $array['source'] = 'load_details';
            $array['field_name'] = 'consignee_files_upload';

            $array['filename'] = $value['name'] ?? '';
            $array['filename_id'] = $value['name_id'] ?? '';

            $array['type'] = $value['type'];

            $array['is_received'] = isset($value['is_received']) && $value['is_received'] == 'on' ? 'on' : 'off';

            $status = 'Pending';
            if($array['type'] == 1){
                $array['file'] = $value['file'] ?? null;
                if(isset($value['file']) && !empty($value['file'])){
                    $status = 'Complete';
                    $array['upload_user_type'] = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                    $array['upload_by'] = isset($value['upload_by']) ? $value['upload_by'] : null;
                    if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                        $array['uploaded_by'] = getUserByType($value['upload_by'], $value['upload_user_type']);
                    }else{
                        $array['uploaded_by'] = null;
                    }
                    $array['uploaded_at'] = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                }
            }else{
                if($array['is_received'] == 'on'){
                    $status = 'Complete';
                    $array['upload_user_type'] = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                    $array['upload_by'] = isset($value['upload_by']) ? $value['upload_by'] : null;
                    if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                        $array['uploaded_by'] = getUserByType($value['upload_by'], $value['upload_user_type']);
                    }else{
                        $array['uploaded_by'] = null;
                    }
                    $array['uploaded_at'] = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                }
            }

            $array['include_w_billing']                             = isset($value['include_w_billing']) && $value['include_w_billing'] == 'yes' ? 'yes' : 'no';
            $array['hide_from_driver']                              = isset($value['hide_from_driver']) && $value['hide_from_driver'] == 'yes' ? 'yes' : 'no';
            $array['history']                                       = isset($value['history']) && is_array($value['history']) && !empty($value['history']) ? $value['history'] : [];
            $array['status'] = $status;

            $addresses = [];
            if(isset($load_detail->delivery_address_one) && $load_detail->delivery_address_one!==''){
                $addresses[] = $load_detail->delivery_address_one;
            }
            if(isset($load_detail->delivery_address_two) && $load_detail->delivery_address_two!==''){
                $addresses[] = $load_detail->delivery_address_two;
            }
            if(isset($load_detail->delivery_state) && $load_detail->delivery_state!==''){
                $addresses[] = $load_detail->delivery_state;
            }
            if(isset($load_detail->delivery_city) && $load_detail->delivery_city!==''){
                $addresses[] = $load_detail->delivery_city;
            }
            if(isset($load_detail->delivery_zip) && $load_detail->delivery_zip!==''){
                $addresses[] = $load_detail->delivery_zip;
            }
            $addresses = implode(', ', $addresses);

            $driver                                                 = null;
            $driver_id                                              = 0;
            $driver_full_name                                       = [];
            if (isset($load_detail->driver)) {
                $driver                                             = $load_detail->driver;
                $driver_id                                          = $driver->id;
                if(isset($driver->last_name) && $driver->last_name != ''){
                    $driver_full_name[]                             = $driver->last_name;
                }
                if(isset($driver->first_name) && $driver->first_name != ''){
                    $driver_full_name[]                             = $driver->first_name;
                }
            }
            $driver_full_name                                       = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

            $stop_details = [];
            $stop_details['name'] = 'Stop 99';
            $stop_details['driver_id']                              = $driver_id;
            $stop_details['driver_full_name']                       = $driver_full_name;
            $stop_details['customer_name'] = $load_detail->receiver->name ?? '';
            $stop_details['addresses'] = $addresses;
            // Extra Info
            $array['stop'] = $stop_details;
            if($status == 'Complete') {
                $array['file_verify'] = (isset($value['file_verify']) && in_array($value['file_verify'], ['1', '2'])) ? $value['file_verify'] : '0'; // 0: Pending, 1: Approved, 2: Rejected
                if ($array['file_verify'] != '0') {
                    $array['file_verify_user_type'] = isset($value['file_verify_user_type']) ? $value['file_verify_user_type'] : null;
                    $array['file_verify_by'] = isset($value['file_verify_by']) ? $value['file_verify_by'] : null;
                    if (isset($array['file_verify_user_type']) && isset($array['file_verify_by'])) {
                        $array['file_verified_by'] = getUserByType($value['file_verify_by'], $value['file_verify_user_type']);
                    } else {
                        $array['file_verified_by'] = null;
                    }
                    $array['file_verified_at'] = isset($value['file_verified_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['file_verified_at'])->setTimezone(getTimeZone())->format(getTimeLayout()) . " " . getTimeZoneId() : null;
                }
            }

            $required_documents[] = $array;
        }
    }
    // Requested Documents
    if(isset($load_detail->receiver_load_document_request_items)){
        $receiver_load_document_request_items                       = $load_detail->receiver_load_document_request_items;
        foreach ($receiver_load_document_request_items as $key => $item) {
            $value                                                  = [];
            if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                $value                                              = unserialize($item->files);
            }
            if(!empty($value)){
                if(
                    isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1, 2]) && $value['include_w_billing'] == 'yes'
                ){
                    // Type 1 = Copy, 2 = Original
                    $array                                          = [];
                    $array['id']                                    = $item->id;
                    $array['source']                                = 'load_document_request_items';
                    $array['field_name']                            = 'files';

                    $array['filename']                              = $value['name'] ?? '';
                    $array['filename_id']                           = $value['name_id'] ?? '';

                    $array['type']                                  = $value['type'];

                    $array['is_received']                           = isset($value['is_received']) && $value['is_received'] == 'on' ? 'on' : 'off';

                    $status                                         = 'Pending';
                    if($array['type'] == 1){
                        $array['file']                              = $value['file'] ?? null;
                        if(isset($value['file']) && !empty($value['file'])){
                            $status                                 = 'Complete';
                            $array['upload_user_type']              = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                            $array['upload_by']                     = isset($value['upload_by']) ? $value['upload_by'] : null;
                            if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                $array['uploaded_by']               = getUserByType($value['upload_by'], $value['upload_user_type']);
                            }else{
                                $array['uploaded_by']               = null;
                            }
                            $array['uploaded_at']                   = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                        }
                    }else{
                        if($array['is_received'] == 'on'){
                            $status                                 = 'Complete';
                            $array['upload_user_type']              = isset($value['upload_user_type']) ? $value['upload_user_type'] : null;
                            $array['upload_by']                     = isset($value['upload_by']) ? $value['upload_by'] : null;
                            if(isset($array['upload_user_type']) && isset($array['upload_by'])){
                                $array['uploaded_by']               = getUserByType($value['upload_by'], $value['upload_user_type']);
                            }else{
                                $array['uploaded_by']               = null;
                            }
                            $array['uploaded_at']                   = isset($value['uploaded_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['uploaded_at'])->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                        }
                    }

                    $array['include_w_billing']                     = isset($value['include_w_billing']) && $value['include_w_billing'] == 'yes' ? 'yes' : 'no';
                    $array['hide_from_driver']                      = isset($value['hide_from_driver']) && $value['hide_from_driver'] == 'yes' ? 'yes' : 'no';
                    $array['history']                               = isset($value['history']) && is_array($value['history']) && !empty($value['history']) ? $value['history'] : [];
                    $array['status']                                = $status;

                    $addresses                                      = [];
                    if(isset($load_detail->shipper_address_one) && $load_detail->shipper_address_one!==''){
                        $addresses[]                                = $load_detail->shipper_address_one;
                    }
                    if(isset($load_detail->shipper_address_two) && $load_detail->shipper_address_two!==''){
                        $addresses[]                                = $load_detail->shipper_address_two;
                    }
                    if(isset($load_detail->shipper_city) && $load_detail->shipper_city!==''){
                        $addresses[]                                = $load_detail->shipper_city;
                    }
                    if(isset($load_detail->shipper_state) && $load_detail->shipper_state!==''){
                        $addresses[]                                = $load_detail->shipper_state;
                    }
                    if(isset($load_detail->shipper_zip) && $load_detail->shipper_zip!==''){
                        $addresses[]                                = $load_detail->shipper_zip;
                    }
                    $addresses                                      = implode(', ', $addresses);

                    $driver                                         = null;
                    $driver_id                                      = 0;
                    $driver_full_name                               = [];
                    if(isset($item->driver)){
                        $driver                                     = $item->driver;
                        $driver_id                                  = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                     = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                     = $driver->first_name;
                        }
                    } elseif (isset($load_detail->driver)) {
                        $driver                                     = $load_detail->driver;
                        $driver_id                                  = $driver->id;
                        if(isset($driver->last_name) && $driver->last_name != ''){
                            $driver_full_name[]                     = $driver->last_name;
                        }
                        if(isset($driver->first_name) && $driver->first_name != ''){
                            $driver_full_name[]                     = $driver->first_name;
                        }
                    }
                    $driver_full_name                               = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

                    $stop_details                                   = [];
                    $stop_details['name']                           = 'Stop 99';
                    $stop_details['driver_id']                      = $driver_id;
                    $stop_details['driver_full_name']               = $driver_full_name;
                    $stop_details['customer_name']                  = $load_detail->shipper->name ?? '';
                    $stop_details['addresses']                      = $addresses;
                    // Extra Info
                    $array['stop']                                  = $stop_details;
                    if($status == 'Complete') {
                        $array['file_verify'] = (isset($value['file_verify']) && in_array($value['file_verify'], ['1', '2'])) ? $value['file_verify'] : '0'; // 0: Pending, 1: Approved, 2: Rejected
                        if ($array['file_verify'] != '0') {
                            $array['file_verify_user_type'] = isset($value['file_verify_user_type']) ? $value['file_verify_user_type'] : null;
                            $array['file_verify_by'] = isset($value['file_verify_by']) ? $value['file_verify_by'] : null;
                            if (isset($array['file_verify_user_type']) && isset($array['file_verify_by'])) {
                                $array['file_verified_by'] = getUserByType($value['file_verify_by'], $value['file_verify_user_type']);
                            } else {
                                $array['file_verified_by'] = null;
                            }
                            $array['file_verified_at'] = isset($value['file_verified_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $value['file_verified_at'])->setTimezone(getTimeZone())->format(getTimeLayout()) . " " . getTimeZoneId() : null;
                        }
                    }

                    $requested_documents[]                          = $array;
                }
            }
        }
    }
    // Accessorial Documents
    if(isset($load_detail->receiver_accessorials)){
        $receiver_accessorials = $load_detail->receiver_accessorials->where('doc_required', '!=', 0)->where('include_w_billing', 'yes');
        foreach ($receiver_accessorials as $key => $item) {
            $gallery = Gallery::where('company_id', auth()->guard('company-admin')->user()->company_id)
                ->where('action_id', $item->id)
                ->where('source', 'load_accessorials')
                ->where('field_name', 'accessorial_files')
                ->first();
            $filename                                               = $item->company_accessorial->name;
            $hide_from_driver                                       = 'no';
            if(!is_null($gallery)){
                if(isset($gallery->hide_from_driver) && $gallery->hide_from_driver == 'yes'){
                    $hide_from_driver                               = 'yes';
                }
                $filename                                          .= isset($gallery->file_name) ? ' - '.$gallery->file_name : '';
            }
            // Type 1 = Copy, 2 = Original
            $array                                                  = [];
            $array['id']                                            = $item->id;
            $array['source']                                        = 'load_accessorials';
            $array['field_name']                                    = 'image';

            $array['filename']                                      = $filename;
            // $array['filename_id']                                = 0;

            $array['type']                                          = 1;

            $array['is_received']                                   = 'off';

            $status                                                 = 'Pending';
            $array['file']                                          = $gallery->image ?? null;
            if(isset($gallery->image) && !empty($gallery->image)){
                $status                                             = 'Complete';
                $array['is_received']                               = 'on';
                $array['upload_user_type']                          = isset($gallery->created_by_type) ? $gallery->created_by_type : null;
                $array['upload_by']                                 = isset($gallery->created_by) ? $gallery->created_by : null;
                if(isset($gallery->created_by_type) && isset($gallery->created_by)){
                    $array['uploaded_by']                           = getUserByType($gallery->created_by, $gallery->created_by_type);
                }else{
                    $array['uploaded_by']                           = null;
                }
                $array['uploaded_at']                               = isset($gallery->created_at) ? Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null; // $gallery->getRawOriginal('uploaded_date')
            }

            $array['include_w_billing']                             = isset($item->include_w_billing) && $item->include_w_billing == 'yes' ? 'yes' : 'no';
            $array['hide_from_driver']                              = isset($item->hide_from_driver) && $item->hide_from_driver == 'yes' ? 'yes' : 'no';
            $array['history']                                       = isset($item->history) && is_serialized($item->history) && !empty(unserialize($item->history)) ? unserialize($item->history) : [];
            $array['status']                                        = $status;

            $addresses                                              = [];
            if(isset($load_detail->delivery_address_one) && $load_detail->delivery_address_one!==''){
                $addresses[]                                        = $load_detail->delivery_address_one;
            }
            if(isset($load_detail->delivery_address_two) && $load_detail->delivery_address_two!==''){
                $addresses[]                                        = $load_detail->delivery_address_two;
            }
            if(isset($load_detail->delivery_state) && $load_detail->delivery_state!==''){
                $addresses[]                                        = $load_detail->delivery_state;
            }
            if(isset($load_detail->delivery_city) && $load_detail->delivery_city!==''){
                $addresses[]                                        = $load_detail->delivery_city;
            }
            if(isset($load_detail->delivery_zip) && $load_detail->delivery_zip!==''){
                $addresses[]                                        = $load_detail->delivery_zip;
            }
            $addresses                                              = implode(', ', $addresses);

            $driver                                                 = null;
            $driver_id                                              = 0;
            $driver_full_name                                       = [];
            if(isset($item->driver)){
                $driver                                             = $item->driver;
                $driver_id                                          = $driver->id;
                if(isset($driver->last_name) && $driver->last_name != ''){
                    $driver_full_name[]                             = $driver->last_name;
                }
                if(isset($driver->first_name) && $driver->first_name != ''){
                    $driver_full_name[]                             = $driver->first_name;
                }
            } elseif (isset($load_detail->driver)) {
                $driver                                             = $load_detail->driver;
                $driver_id                                          = $driver->id;
                if(isset($driver->last_name) && $driver->last_name != ''){
                    $driver_full_name[]                             = $driver->last_name;
                }
                if(isset($driver->first_name) && $driver->first_name != ''){
                    $driver_full_name[]                             = $driver->first_name;
                }
            }
            $driver_full_name                                       = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

            $stop_details                                           = [];
            $stop_details['name']                                   = 'Stop 99';
            $stop_details['driver_id']                              = $driver_id;
            $stop_details['driver_full_name']                       = $driver_full_name;
            $stop_details['customer_name']                          = $load_detail->receiver->name ?? '';
            $stop_details['addresses']                              = $addresses;
            // Extra Info
            $array['stop']                                          = $stop_details;
            if($status == 'Complete'){
                $array['file_verify'] = (isset($item->file_verify) && in_array($item->file_verify, ['1', '2'])) ? $item->file_verify : '0'; // 0: Pending, 1: Approved, 2: Rejected
                if($array['file_verify'] != '0'){
                    $array['file_verify_user_type'] = $item->file_verify_user_type ?? null;
                    $array['file_verify_by'] = $item->file_verify_by ?? null;
                    if(isset($array['file_verify_user_type']) && isset($array['file_verify_by'])){
                        $array['file_verified_by'] = getUserByType($item->file_verify_by, $item->file_verify_user_type);
                    }else{
                        $array['file_verified_by'] = null;
                    }
                    $array['file_verified_at'] = isset($value['file_verified_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $item->file_verified_at)->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                }
            }

            $accessorial_documents[]                                = $array;
        }
    }

    // Accessorial Documents
    if(isset($load_detail->billing_accessorials)){
        $billing_accessorials = $load_detail->billing_accessorials->where('doc_required', '!=', 0)->where('include_w_billing', 'yes');
        foreach ($billing_accessorials as $key => $item) {
            $gallery = Gallery::where('company_id', auth()->guard('company-admin')->user()->company_id)
                ->where('action_id', $item->id)
                ->where('source', 'load_accessorials')
                ->where('field_name', 'accessorial_files')
                ->first();
            $filename                                               = $item->company_accessorial->name;
            if(!is_null($gallery)){
                if(isset($gallery->hide_from_driver) && $gallery->hide_from_driver == 'yes'){
                    $hide_from_driver                               = 'yes';
                }
                $filename                                          .= isset($gallery->file_name) ? ' - '.$gallery->file_name : '';
            }
            // Type 1 = Copy, 2 = Original
            $array                                                  = [];
            $array['id']                                            = $item->id;
            $array['source']                                        = 'load_accessorials';
            $array['field_name']                                    = 'image';

            $array['filename']                                      = $filename;
            // $array['filename_id']                                = 0;

            $array['type']                                          = 1;

            $array['is_received']                                   = 'off';

            $status                                                 = 'Pending';
            $array['file']                                          = $gallery->image ?? null;
            if(isset($gallery->image) && !empty($gallery->image)){
                $status                                             = 'Complete';
                $array['is_received']                               = 'on';
                $array['upload_user_type']                          = isset($gallery->created_by_type) ? $gallery->created_by_type : null;
                $array['upload_by']                                 = isset($gallery->created_by) ? $gallery->created_by : null;
                if(isset($gallery->created_by_type) && isset($gallery->created_by)){
                    $array['uploaded_by']                           = getUserByType($gallery->created_by, $gallery->created_by_type);
                }else{
                    $array['uploaded_by']                           = null;
                }
                $array['uploaded_at']                               = isset($gallery->created_at) ? Carbon::createFromFormat('Y-m-d H:i:s', $gallery->created_at)->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null; // $gallery->getRawOriginal('uploaded_date')
            }

            $array['include_w_billing']                             = isset($item->include_w_billing) && $item->include_w_billing == 'yes' ? 'yes' : 'no';
            $array['hide_from_driver']                              = isset($item->hide_from_driver) && $item->hide_from_driver == 'yes' ? 'yes' : 'no';
            $array['history']                                       = isset($item->history) && is_serialized($item->history) && !empty(unserialize($item->history)) ? unserialize($item->history) : [];
            $array['status']                                        = $status;

            $driver                                                 = null;
            $driver_id                                              = 0;
            $driver_full_name                                       = [];
            if(isset($item->driver)){
                $driver                                             = $item->driver;
                $driver_id                                          = $driver->id;
                if(isset($driver->last_name) && $driver->last_name != ''){
                    $driver_full_name[]                             = $driver->last_name;
                }
                if(isset($driver->first_name) && $driver->first_name != ''){
                    $driver_full_name[]                             = $driver->first_name;
                }
            } elseif (isset($load_detail->driver)) {
                $driver                                             = $load_detail->driver;
                $driver_id                                          = $driver->id;
                if(isset($driver->last_name) && $driver->last_name != ''){
                    $driver_full_name[]                             = $driver->last_name;
                }
                if(isset($driver->first_name) && $driver->first_name != ''){
                    $driver_full_name[]                             = $driver->first_name;
                }
            }
            $driver_full_name                                       = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

            // Extra Info
            $array['stop']                                          = [
                'name'                                              => 'Load',
                'driver_id'                                         => $driver_id,
                'driver_full_name'                                  => $driver_full_name
            ];
            if($status == 'Complete'){
                $array['file_verify'] = (isset($item->file_verify) && in_array($item->file_verify, ['1', '2'])) ? $item->file_verify : '0'; // 0: Pending, 1: Approved, 2: Rejected
                if($array['file_verify'] != '0'){
                    $array['file_verify_user_type'] = $item->file_verify_user_type ?? null;
                    $array['file_verify_by'] = $item->file_verify_by ?? null;
                    if(isset($array['file_verify_user_type']) && isset($array['file_verify_by'])){
                        $array['file_verified_by'] = getUserByType($item->file_verify_by, $item->file_verify_user_type);
                    }else{
                        $array['file_verified_by'] = null;
                    }
                    $array['file_verified_at'] = isset($value['file_verified_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $item->file_verified_at)->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                }
            }

            $accessorial_documents[]                                = $array;
        }
    }

    // Other Documents
    $get_other_documents = LoadFile::where('load_id', $load_detail->load_id)->where('include_w_billing', 'yes')->orderBy('id','desc')->get();
    if($get_other_documents->isNotEmpty()){
        foreach ($get_other_documents as $key => $other_document) {
            // Type 1 = Copy, 2 = Original
            $array                                                  = [];
            $array['id']                                            = $other_document->id;
            $array['source']                                        = 'load_files';
            $array['field_name']                                    = 'document';

            $array['filename']                                      = $other_document->company_load_document->name ?? $other_document->filename;
            // $array['filename_id']                                = 0;

            $array['type']                                          = 1;
            $array['is_received']                                   = 'off';

            $status                                                 = 'Pending';
            $array['file']                                          = isset($other_document->document) && is_serialized($other_document->document) && !empty(unserialize($other_document->document)) ? unserialize($other_document->document) : null;
            if(isset($array['file']) && !empty($array['file'])){
                $upload_user_type                                   = null;
                if(isset($other_document->upload_user_type) && $other_document->upload_user_type == 'Company Admin'){
                    $upload_user_type                               = $other_document->company_admin->type;
                }elseif(isset($other_document->upload_user_type) && $other_document->upload_user_type == 'Driver'){
                    $upload_user_type                               = 3;
                }

                $status                                             = 'Complete';
                $array['is_received']                               = 'on';
                $array['upload_user_type']                          = $upload_user_type;
                $array['upload_by']                                 = isset($other_document->upload_by) ? $other_document->upload_by : null;
                if(isset($upload_user_type) && isset($other_document->upload_by)){
                    $array['uploaded_by']                           = getUserByType($other_document->upload_by, $upload_user_type);
                }else{
                    $array['uploaded_by']                           = null;
                }
                $array['uploaded_at']                               = isset($other_document->updated_at) ? Carbon::createFromFormat('Y-m-d H:i:s', $other_document->updated_at)->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
            }

            $array['include_w_billing']                             = isset($other_document->include_w_billing) && $other_document->include_w_billing == 'yes' ? 'yes' : 'no';
            $array['hide_from_driver']                              = isset($other_document->hide_from_driver) && $other_document->hide_from_driver == 'yes' ? 'yes' : 'no';
            $array['history']                                       = isset($other_document->history) && is_serialized($other_document->history) && !empty(unserialize($other_document->history)) ? unserialize($other_document->history) : [];
            $array['status']                                        = $status;

            $driver                                             = null;
            $driver_id                                          = 0;
            $driver_full_name                                   = [];
            if (isset($load_detail->driver)) {
                $driver                                         = $load_detail->driver;
                $driver_id                                      = $driver->id;
                if(isset($driver->last_name) && $driver->last_name != ''){
                    $driver_full_name[]                         = $driver->last_name;
                }
                if(isset($driver->first_name) && $driver->first_name != ''){
                    $driver_full_name[]                         = $driver->first_name;
                }
            }
            $driver_full_name                                   = !empty($driver_full_name) ? implode(', ', $driver_full_name) : '';

            // Extra Info
            $array['stop']                                      = [
                'name'                                          => 'Load',
                'driver_id'                                     => $driver_id,
                'driver_full_name'                              => $driver_full_name
            ];

            if($status == 'Complete'){
                $array['file_verify'] = (isset($other_document->file_verify) && in_array($other_document->file_verify, ['1', '2'])) ? $other_document->file_verify : '0'; // 0: Pending, 1: Approved, 2: Rejected
                if($array['file_verify'] != '0'){
                    $array['file_verify_user_type'] = $other_document->file_verify_user_type ?? null;
                    $array['file_verify_by'] = $other_document->file_verify_by ?? null;
                    if(isset($array['file_verify_user_type']) && isset($array['file_verify_by'])){
                        $array['file_verified_by'] = getUserByType($other_document->file_verify_by, $other_document->file_verify_user_type);
                    }else{
                        $array['file_verified_by'] = null;
                    }
                    $array['file_verified_at'] = isset($value['file_verified_at']) ? Carbon::createFromFormat('Y-m-d H:i:s', $other_document->file_verified_at)->setTimezone(getTimeZone())->format(getTimeLayout())." ".getTimeZoneId() : null;
                }
            }

            $other_documents[]                                  = $array;
        }
    }

    $merged_documents = [];
    // Check if $required_documents is not empty before merging
    if (!empty($required_documents)) {
        $merged_documents = array_merge($merged_documents, $required_documents);
    }
    // Check if $requested_documents is not empty before merging
    if (!empty($requested_documents)) {
        $merged_documents = array_merge($merged_documents, $requested_documents);
    }
    // Check if $accessorial_documents is not empty before merging
    if (!empty($accessorial_documents)) {
        $merged_documents = array_merge($merged_documents, $accessorial_documents);
    }
    // Check if $other_documents is not empty before merging
    if (!empty($other_documents)) {
        $merged_documents                                           = array_merge($merged_documents, $other_documents);
    }

    // If you want to reindex the merged array
    $merged_documents = array_values($merged_documents);

    return [
        'merged_documents'      => $merged_documents,
        'required_documents'    => $required_documents,
        'requested_documents'   => $requested_documents,
        'accessorial_documents' => $accessorial_documents,
        'other_documents'       => $other_documents
    ];
}

function reminderRepeatFrequency()
{
    return ['1_day' => '1 Day', '1_week' => '1 Week', '15_day' => '15 Day', '1_month' => '1 Month', '2_month' => '2 Month', '3_month' => '3 Month', '4_month' => '4 Month', '5_month' => '5 Month', '6_month' => '6 Month'];
}

function getDeliveredLoads($companyId = null)
{
    if(empty($companyId)){
        $companyId = auth()->guard('company-admin')->user()->company_id;
    }

    $loadNumbers = LoadDetail::leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
                    ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
                    //->whereIn('load_details.load_status', [5, 6, 7, 8, 10])
                    ->whereIn('load_details.load_status', [5, 6])
                    ->whereHas('loads', function($q) use ($companyId){
                        $q->where('company_id', $companyId);
                    })
                    ->where('load_details.is_disable', 0)
                    ->whereNotNull('load_details.load_number')
                    ->pluck('load_details.load_number', 'load_details.id')
                    ->toArray();

    return $loadNumbers;
}

function getPdfPageCount($path)
{
    if($path != ''){
        $pdf = file_get_contents($path);
        return preg_match_all("/\/Page\W/", $pdf, $dummy) ?? 0;
    }
    return 0;
}

function getPayInvoicePaidBy()
{
    return ['1' => 'Denim Financial', '2' => 'RTS Financial', '3' => 'Triumph Pay'];
}

function getPaymentTypes()
{
    return ['1' => 'ACH/Wire', '2' => 'Cash', '3' => 'Certified Check', '4' => 'Credit/Debit Card', '5' => 'Standard Check'];
}

function getChargebackType()
{
    return ['1' => 'Customer', '2' => 'Driver', '3' => 'Truck Owner'];
}

function getPaymentAccount($loadDetailId)
{
    return LoadBillingPaymentAccount::select('id', DB::raw('CONCAT(account_name," XXXX", account_number) as name'))->where('load_detail_id', $loadDetailId)->pluck('name', 'id')->toArray();
}

function getPaymentAccountNumber($loadDetailId)
{
    return LoadBillingPaymentAccount::where('load_detail_id', $loadDetailId)->pluck('account_number', 'id')->toArray();
}

function getGlAccounts()
{
    return CompanyChartOfAccount::IsCompany()->select('id', DB::raw('CONCAT(account_number," - ", account_name) as name'))->pluck('name', 'id')->toArray();
}

function billingTotalReadyToBillLoads(){
    return LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
        ->leftJoin('load_billing_activities', 'load_billing_activities.load_detail_id', '=', 'load_details.id')
        ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
        ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
        ->where('load_details.load_status', 6)
        ->where('load_billing_activities.paperwork_received', '1')
        ->where('load_details.is_disable', 0)
        ->where(function($query) {
            $query->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
        })
        ->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0)
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
        })->count();
}

function billingTotalOnHoldLoads(){
    return LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
        ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
        ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
        ->where(function($query) {
            $query->where(function($q) {
                $q->where('load_details.load_status', 7)->orWhere('load_details.is_on_hold', 1);
            })->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
        })->count();
}

function billingTotalMissingDocumentLoads(){
    return LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
        ->leftJoin('load_billing_activities', 'load_billing_activities.load_detail_id', '=', 'load_details.id')
        ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
        ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
        ->where('load_details.load_status', 5)
        ->where('load_billing_activities.paperwork_received', '0')
        ->where('load_details.is_disable', 0)
        ->where(function($query) {
            $query->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
        })
        ->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0)
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
        })->count();
}

function billingTotalUnbilledLoads(){
    return LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
        ->leftJoin('load_billing_activities', 'load_billing_activities.load_detail_id', '=', 'load_details.id')
        ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
        ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
        ->where('load_details.load_status', 6)
        ->where('load_billing_activities.paperwork_received', '1')
        ->where('load_billing_activities.audit_complete', '1')
        ->where('load_details.is_disable', 0)
        ->where(function($query) {
            $query->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
        })
        ->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0)
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
        })->count();
}

function billingTotalInvoicedLoads(){
    return LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
            ->leftJoin('load_billing_activities', 'load_billing_activities.load_detail_id', '=', 'load_details.id')
            ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
            ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
            ->where('load_details.load_status', 10)
            ->where('load_billing_activities.invoiced', '1')
            ->whereHas('loads', function($q){
                $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
            })->count();
}


function customerBillingTotalReadyToBillLoads($brokerId){
    return LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
        ->leftJoin('load_billing_activities', 'load_billing_activities.load_detail_id', '=', 'load_details.id')
        ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
        ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
        ->where('load_details.brokerage_firm', $brokerId)
        ->where('load_details.load_status', 6)
        ->where('load_billing_activities.paperwork_received', '1')
        ->where('load_billing_activities.audit_complete', '0')
        ->where('load_billing_activities.paperwork_received_date_time', '>=', Carbon::create(date('Y'))->startOfYear()->format('Y-m-d'))
        ->where('load_billing_activities.paperwork_received_date_time', '<=', Carbon::create(date('Y'))->endOfYear()->format('Y-m-d'))
        ->where('load_details.is_disable', 0)
        ->where(function($query) {
            $query->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
        })
        ->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0)
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
        })->sum('load_details.total_bill');
}

function customerUnbilledLoads($brokerId){
    return LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
        ->leftJoin('load_billing_activities', 'load_billing_activities.load_detail_id', '=', 'load_details.id')
        ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
        ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
        ->where('load_details.brokerage_firm', $brokerId)
        ->where('load_details.load_status', 6)
        ->where('load_billing_activities.paperwork_received', '1')
        ->where('load_billing_activities.audit_complete', '1')
        ->where('load_billing_activities.audit_complete_date_time', '>=', Carbon::create(date('Y'))->startOfYear()->format('Y-m-d'))
        ->where('load_billing_activities.audit_complete_date_time', '<=', Carbon::create(date('Y'))->endOfYear()->format('Y-m-d'))
        ->where('load_details.is_disable', 0)
        ->where(function($query) {
            $query->whereNull('load_details.is_on_hold')->orWhere('load_details.is_on_hold', 0);
        })
        ->whereNotNull('load_details.driver_id')->where('load_details.driver_id', '!=', 0)
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
        })->sum('load_details.total_bill');
}

function customerOverdueLoads($brokerId){
    return LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
        ->leftJoin('load_billing_activities', 'load_billing_activities.load_detail_id', '=', 'load_details.id')
        ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
        ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
        ->where('load_details.brokerage_firm', $brokerId)
        ->where('load_details.load_status', 10)
        ->where('load_billing_activities.invoiced', '1')
        ->where('load_billing_activities.invoiced_date_time', '>=', Carbon::create(date('Y'))->startOfYear()->format('Y-m-d'))
        ->where('load_billing_activities.invoiced_date_time', '<=', Carbon::create(date('Y'))->endOfYear()->format('Y-m-d'))
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
        })->sum('load_details.total_bill');
}

function customerTotalBilledLoads($brokerId){
    return LoadDetail::select('load_details.*', 'brokers.name as broker_name', 'brokers.factor_invoices as broker_factor_invoices')
        ->leftJoin('load_billing_activities', 'load_billing_activities.load_detail_id', '=', 'load_details.id')
        ->leftJoin('drivers', 'load_details.driver_id', '=', 'drivers.id')
        ->leftJoin('brokers', 'load_details.brokerage_firm', '=', 'brokers.id')
        ->where('load_details.brokerage_firm', $brokerId)
        ->whereIn('load_details.load_status', [10, 8])
        ->where('load_billing_activities.invoiced', '1')
        ->where('load_billing_activities.paid_invoice', '1')
        ->where('load_billing_activities.paid_invoice_date_time', '>=', Carbon::create(date('Y'))->startOfYear()->format('Y-m-d'))
        ->where('load_billing_activities.paid_invoice_date_time', '<=', Carbon::create(date('Y'))->endOfYear()->format('Y-m-d'))
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id);
        })->sum('load_details.total_bill');
}

function brokerTotalActiveLoads($startDate, $endDate, $brokerId){
    return LoadDetail::select('*')
        ->where('brokerage_firm', $brokerId)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('is_disable', 0)
        ->where(function($query) {
            $query->whereNotIn('load_status',[5,6,7,8,9,10])
                ->orWhere(function($q) {
                    $q->whereNull('driver_id')->orWhere('driver_id', 0);
                });
        })
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
        })
        ->whereNull('deleted_at')
        ->whereHas('loads', function($q){
            $q->where('loads.company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })->count();
}

function brokerTotalReadyToBillLoads($startDate, $endDate, $brokerId){
    return LoadDetail::select('*')
        ->where('brokerage_firm', $brokerId)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('is_disable', 0)
        ->whereIn('load_status', [6,10])
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
        })
        ->where(function($q) {
            $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })->count();
}

function brokerTotalDeliveredLoads($startDate, $endDate, $brokerId) {
    return LoadDetail::select('*')
        ->where('brokerage_firm', $brokerId)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('is_disable', 0)
        ->where('load_details.load_status', 5)
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
        })
        ->where(function($q) {
            $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })->count();
}

function brokerTotalCompletedLoads($startDate, $endDate, $brokerId){
    return LoadDetail::select('*')
        ->where('brokerage_firm', $brokerId)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('is_disable', 0)
        ->where('load_details.load_type', 0) //->whereIn('load_type', [0,6,4]) // 0 = OCCUPIED, 6 = TONU, 4 = NO DISPATCH
        ->where('load_status', 8)
        ->where(function($q) {
            $q->whereNull('is_on_hold')->orWhere('is_on_hold', 0);
        })
        ->where(function($q) {
            $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })->count();
}

function brokerTotalOnHoldLoads($startDate, $endDate, $brokerId) {
    return LoadDetail::select('*')
        ->where('brokerage_firm', $brokerId)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('is_disable', 0)
        ->where(function($q) {
            $q->where('load_details.load_status', 7)->orWhere('load_details.is_on_hold', 1);
        })
        ->where(function($q) {
            $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })->count();
}

function brokerTotalCancelledLoads($startDate, $endDate, $brokerId) {
    return LoadDetail::select('*')
        ->where('brokerage_firm', $brokerId)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('is_disable', 0)
        ->where('load_details.load_type', 0) // 0 = OCCUPIED
        ->where(function($q) {
            $q->where('load_details.load_status', 9)->orWhere('load_details.is_on_hold', 2);
        })
        ->where(function($q) {
            $q->whereNotNull('driver_id')->where('driver_id', '!=', 0);
        })
        ->whereHas('loads', function($q){
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })->count();
}

function brokerTotalLoads($startDate, $endDate, $brokerId){
    return LoadDetail::whereHas('loads', function ($q) {
            $q->where('company_id', auth()->guard('company-admin')->user()->company_id)->whereNull('loads.deleted_at');
        })
        ->where('brokerage_firm', $brokerId)
        ->where(function ($query) use ($endDate, $startDate) {
            $query->where(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), '<=', $endDate)
                ->where(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), '>=', $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_delivery_date, "%Y-%m-%d")'), $startDate)
                ->orWhere(DB::raw('DATE_FORMAT(actual_shipper_date, "%Y-%m-%d")'), $endDate);
        })
        ->where('load_details.is_disable', 0)
        ->where('load_details.load_status', '!=', 9)
        ->where(function($q) {
            $q->whereNull('load_details.is_on_hold')->orWhereIn('load_details.is_on_hold', [0,1]);
        })
        ->whereNull('load_details.deleted_at')
        ->count();
}

function brokerCargoClaimsLoads($startDate, $endDate, $brokerId){
    return 0;
}

function getDispatcherListByCompanyWithTeam($companyId = null){
    if(empty($companyId)){
        $companyId = auth()->guard('company-admin')->user()->company_id;
    }
    return CompanyAdmin::select('id', DB::raw("CONCAT(first_name,' ',last_name) AS name"), 'team_id')
        ->where('company_id', $companyId)
        ->where('type', 2)
        ->orderByRaw('first_name ASC, last_name ASC')
        ->get();
}


function getPaymentsTermsDays(){
    return ['10_days' => '10 Days', '30_days' => '30 Days', '60_days' => '60 Days', '90_days' => '90 Days'];
}

function getTruckEldProvider()
{
    return ['geo-tab' => 'GeoTab', 'omni-tracs' => 'OmniTracs', 'samsara' => 'Samsara', 'trans-flo' => 'TransFlo'];
}

function getTruckTransponderProvider()
{
    return ['ez-pass' => 'EZPass', 'pre-pass' => 'PrePass', 'best-pass' => 'BestPass'];
}

function convertTimeFormate($value){
    if($value != ''){
        $timeFormat = getTimeFormat();
        if ($timeFormat == 12) {
            return Carbon::parse($value)->format('H:i A');
        }else{
            return Carbon::parse($value)->format('H:i');
        }
    }
    return '';
}

function getLeftSideBarMenus()
{
    $menus = [];
    $moduleAccess = [
        'list' => (
            hasPermission(config('settings.company_admin_modules.drivers'),config('settings.permissions.view'))
            || hasPermission(config('settings.company_admin_modules.trucks'),config('settings.permissions.view'))
            || hasPermission(config('settings.company_admin_modules.trailers'),config('settings.permissions.view'))
        ),
        'compliance' => hasPermission(config('settings.company_admin_modules.compliance'), config('settings.permissions.view')),
        'maintenance' => hasPermission(config('settings.company_admin_modules.maintenances'),config('settings.permissions.view')),
        'payroll' => true,
        'fuel-tolls' => true,
        'ifta' => true,
        'safety' => true,
        'fmcsa' => true,
        'reports' => true,
        'calender' => true,
        'messenger' => true,
        'prorouter' => hasPermission(config('settings.company_admin_modules.pro_routes'),config('settings.permissions.view')),
        'weekly-view' => hasPermission(config('settings.company_admin_modules.reports'),config('settings.permissions.view')),
        'loads' => hasPermission(config('settings.company_admin_modules.loads'),config('settings.permissions.view')),
    ];

    $leftMenus = LeftMenu::get();

    $userCompanyId = auth()->guard('company-admin')->user()->company_id;
    $userId = auth()->guard('company-admin')->user()->id;
    $userType = auth()->guard('company-admin')->user()->type;

    $leftMenuByUser = LeftMenuByUser::where('company_id', $userCompanyId)
        ->where('user_id', $userId)
        ->where('user_type', $userType)
        ->get();

    if($leftMenuByUser->count() > 0){
        foreach ($leftMenus as $index => $leftMenu) {
            if(isset($moduleAccess[Str::slug($leftMenu->name)]) && $moduleAccess[Str::slug($leftMenu->name)]){
                $leftMenuByUserDetail = $leftMenuByUser->where('menu_id', $leftMenu->id)->first();

                if(!empty($leftMenuByUserDetail)){
                    $menus[] = [
                        'id' => base64Encode($leftMenuByUserDetail->id),
                        'menu_id' => base64Encode($leftMenu->id),
                        'name' => $leftMenu->name,
                        'display_no' => $leftMenuByUserDetail->display_no,
                        'active_status' => $leftMenuByUserDetail->active_status,
                        'url' => (!empty($leftMenu->url)) ? route($leftMenu->url) : '',
                        'icon' => (!empty($leftMenu->icon)) ? asset('theme/images/'.$leftMenu->icon) : ''
                    ];
                }else{
                    $menus[] = [
                        'menu_id' => base64Encode($leftMenu->id),
                        'name' => $leftMenu->name,
                        'display_no' => ++$index,
                        'active_status' => 0,
                        'url' => (!empty($leftMenu->url)) ? route($leftMenu->url) : '',
                        'icon' => (!empty($leftMenu->icon)) ? asset('theme/images/'.$leftMenu->icon) : ''
                    ];
                }
            }
        }
    }else{
        foreach ($leftMenus as $index => $leftMenu) {
            if(isset($moduleAccess[Str::slug($leftMenu->name)]) && $moduleAccess[Str::slug($leftMenu->name)]){
                $menus[] = [
                    'menu_id' => base64Encode($leftMenu->id),
                    'name' => $leftMenu->name,
                    'display_no' => ++$index,
                    'active_status' => 1,
                    'url' => (!empty($leftMenu->url)) ? route($leftMenu->url) : '',
                    'icon' => (!empty($leftMenu->icon)) ? asset('theme/images/'.$leftMenu->icon) : ''
                ];
            }
        }
    }

    return collect($menus)->sortBy('display_no')->toArray();
}

function getLoadDocumentsCount($load_detail)
{
    // Required Documents
    $required_documents = [];
    // Requested Documents
    $requested_documents = [];
    // Accessorial Documents
    $accessorial_documents = [];
    // Other Documents
    $other_documents = [];

    // Required Documents
    if(isset($load_detail->shipper_files_upload) && is_serialized($load_detail->shipper_files_upload) && !empty(unserialize($load_detail->shipper_files_upload))){
        $uploads = unserialize($load_detail->shipper_files_upload);
        $uploads = array_filter($uploads, function ($var) {
            return (isset($var['name_id']) && $var['name_id'] !== null && $var['is_required'] == 'on' && in_array($var['type'], [1, 2]) && $var['include_w_billing'] == 'yes');
        });
        $required_documents[] = count($uploads);
    }
    // Requested Documents
    if(isset($load_detail->shipper_load_document_request_items)){
        $shipper_load_document_request_items = $load_detail->shipper_load_document_request_items;
        foreach ($shipper_load_document_request_items as $key => $item) {
            $value = [];
            if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                $value = unserialize($item->files);
            }
            if(!empty($value)){
                if(isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1, 2]) && $value['include_w_billing'] == 'yes'){
                    $requested_documents[] = 1;
                }
            }
        }
    }
    // Accessorial Documents
    if(isset($load_detail->shipper_accessorials)){
        $shipper_accessorials = $load_detail->shipper_accessorials->where('doc_required', '!=', 0)->where('include_w_billing', 'yes');
        $accessorial_documents[] = count($shipper_accessorials);
    }

    $stop_no = 2;
    if(isset($load_detail->loads->load_stops) && $load_detail->loads->load_stops->isNotEmpty()) {
        foreach ($load_detail->loads->load_stops as $key => $stop) {
            $customer_type = $stop->customer_type;
            $field_name = $customer_type=='1' ? 'stop_shipper_files_upload' : 'stop_consignee_files_upload';

            // Required Documents
            if(isset($stop->$field_name) && is_serialized($stop->$field_name) && !empty(unserialize($stop->$field_name))){
                $uploads = unserialize($stop->$field_name);
                $uploads = array_filter($uploads, function ($var) {
                    return (isset($var['name_id']) && $var['name_id'] !== null && $var['is_required'] == 'on' && in_array($var['type'], [1, 2]) && $var['include_w_billing'] == 'yes');
                });
                $required_documents[] = count($uploads);
            }
            // Requested Documents
            if(isset($stop->load_document_request_items)){
                $stop_load_document_request_items                   = $stop->load_document_request_items;
                foreach ($stop_load_document_request_items as $key => $item) {
                    $value                                          = [];
                    if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                        $value                                      = unserialize($item->files);
                    }
                    if(!empty($value)){
                        if(isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1, 2]) && $value['include_w_billing'] == 'yes'){
                            $requested_documents[] = 1;
                        }
                    }
                }
            }
            // Accessorial Documents
            if(isset($stop->accessorials)){
                $stop_accessorials = $stop->accessorials->where('doc_required', '!=', 0)->where('include_w_billing', 'yes');
                $accessorial_documents[] = count($stop_accessorials);
            }

            ++$stop_no;
        }
    }

    // Required Documents
    if(isset($load_detail->consignee_files_upload) && is_serialized($load_detail->consignee_files_upload) && !empty(unserialize($load_detail->consignee_files_upload))){
        $uploads = unserialize($load_detail->consignee_files_upload);
        $uploads = array_filter($uploads, function ($var) {
            return (isset($var['name_id']) && $var['name_id'] !== null && $var['is_required'] == 'on' && in_array($var['type'], [1, 2]) && $var['include_w_billing'] == 'yes');
        });
        $required_documents[] = count($uploads);
    }
    // Requested Documents
    if(isset($load_detail->receiver_load_document_request_items)){
        $receiver_load_document_request_items                       = $load_detail->receiver_load_document_request_items;
        foreach ($receiver_load_document_request_items as $key => $item) {
            $value                                                  = [];
            if(isset($item->files) && is_serialized($item->files) && !empty(unserialize($item->files))){
                $value                                              = unserialize($item->files);
            }
            if(!empty($value)){
                if(isset($value['name_id']) && $value['name_id'] !== null && $value['is_required'] == 'on' && in_array($value['type'], [1, 2]) && $value['include_w_billing'] == 'yes'){
                    $requested_documents[] = 1;
                }
            }
        }
    }
    // Accessorial Documents
    if(isset($load_detail->receiver_accessorials)){
        $receiver_accessorials = $load_detail->receiver_accessorials->where('doc_required', '!=', 0)->where('include_w_billing', 'yes');
        $accessorial_documents[] = count($receiver_accessorials);
    }

    // Accessorial Documents
    if(isset($load_detail->billing_accessorials)){
        $billing_accessorials = $load_detail->billing_accessorials->where('doc_required', '!=', 0)->where('include_w_billing', 'yes');
        $accessorial_documents[] = count($billing_accessorials);
    }

    // Other Documents
    $get_other_documents = LoadFile::where('load_id', $load_detail->load_id)->where('include_w_billing', 'yes')->orderBy('id','desc')->get();
    if($get_other_documents->isNotEmpty()){
        $other_documents[] = count($get_other_documents);
    }

    return (array_sum($required_documents) + array_sum($requested_documents) + array_sum($accessorial_documents) + array_sum($other_documents)) ?? 0;
}

function getCompanyAdminListByCompanyWithNameEmail($companyId = null){
    if(empty($companyId)){
        $companyId = auth()->guard('company-admin')->user()->company_id;
    }
    return CompanyAdmin::select(DB::raw("CONCAT(first_name,' ',last_name, ' - ', email) AS name"), 'id')
        ->where('company_id', $companyId)
        ->pluck('name', 'id')
        ->all();
}

function getLoadInternalOrInitialNotes($loadId){
    return LoadNote::whereIn('type', ['3', '4'])->where('load_id', $loadId)->orderBy('id', 'ASC')->pluck('note')->toArray();
}

function getEventTypesWithIcon(){
    return [
        '0' => ['text' => 'Seminars', 'icon' => asset('theme/images/event-seminars.png')], 
        '1' => ['text' => 'Birthdays', 'icon' => asset('theme/images/event-birthdays.png')], 
        '2' => ['text' => 'Meeting', 'icon' => asset('theme/images/event-meeting.png')],  
        '3' => ['text' => 'Expirations', 'icon' => asset('theme/images/event-expirations.png')], 
        '4' => ['text' => 'Maintenance Trucks', 'icon' => asset('theme/images/event-maintenance-trucks.png')], 
        '5' => ['text' => 'Maintenance Trailers', 'icon' => asset('theme/images/event-maintenance-trailers.png')], 
        '6' => ['text' => 'Other', 'icon' => asset('theme/images/event-other.png')], 
        '7' => ['text' => 'My Leave', 'icon' => asset('theme/images/event-my-leave.png')], 
        '8' => ['text' => 'Personal Events', 'icon' => asset('theme/images/event-personal-events.png')], 
        '9' => ['text' => 'Driver Leaves', 'icon' => asset('theme/images/event-driver-leaves.png')], 
        '10' => ['text' => 'Dispatcher Leaves', 'icon' => asset('theme/images/event-dispatcher-leaves.png')], 
        '11' => ['text' => 'My Violations', 'icon' => asset('theme/images/event-my-violations.png')], 
        '12' => ['text' => 'Maintenance Ticket', 'icon' => asset('theme/images/event-maintenance-ticket.png')]
    ];
}
